/*
 * ccent.c
 *
 * Derived from testch.c.  Correct centromere finding.
 *
 * Modifications
 *
 *	14 Sep 1986		CAS		Includes
 */
#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>

#define TERM "/t5"


main()
{
	struct object *spobj, *mpol, *cvh, *rect, *chromosomeaxis();
	struct chromosome *obj, *readobj();
	struct object *convhull(), *spin(), *minwrect();
	struct pframe f;
	int i, j, number;
	int x,y;
	double theta;

	dinit();
	lutfrig();
	number = 0;
	while ((obj = readobj(stdin)) != NULL) {
		number++;
		if (area(obj) < 60)
			fprintf(stderr,"tiny object ignored %d\n",number);
		else if (obj->plist == NULL || obj->plist->otype <= 1) {
		f.type = 60;
		f.scale = 16;
		f.ox = obj->idom->kol1;
		f.oy = obj->idom->line1;
		f.ix = f.iy = 1;
		f.dx = 2048;
		f.dy = 2048;
		cvh = convhull(obj);
		rect = minwrect(cvh);

		/*
		 * rotate to vertical
		 */
		spobj = spin(obj,((struct frect *)rect->idom)->rangle);
		mpol = chromosomeaxis(spobj);
	
		colour(254);
		clear();
		f.ox = (spobj->idom->kol1 + spobj->idom->lastkl)/2;
		f.oy = (spobj->idom->line1 + spobj->idom->lastln)/2;
		picframe(spobj,&f);
		f.ox *= 8;
		f.oy *= 8;
		f.scale /= 8;
		picframe(mpol,&f);
		f.ox = spobj->idom->kol1;
		f.oy = spobj->idom->line1;
		f.scale *= 8;
		
		ccentpos(&x,&y,&theta,f.ox,f.oy,0.0,&f);
		fprintf(stderr,"\nX %3d Y %3d THETA %f\n",x,y,theta);

		freeobj(spobj);
		freeobj(mpol);
		freeobj(cvh);
		freeobj(rect);
		}
		freeobj(obj);
	}
}


/*
 * return correct centromere position.
 * start with an original suggested position (which may e.g. come
 * from automatic measurement).
 */
ccentpos(x,y,theta,origx,origy,origtheta,f)
int *x,*y;
double *theta, origtheta;
struct pframe *f;
{
	FILE *in;
	int c;
	in = fopen(TERM,"r");
	setbuf(in,NULL);
	/*
	 * display original position
	 */
	intens(1);
	colour(1);
	cvdisp(origx,origy,origtheta,f);
	*x = origx;
	*y = origy;
	*theta = origtheta;
	/*
	 * drive around
	 */
	while ((c = (getc(in) & 127)) != '0') {
		/*
		 * delete preceding point if end point has moved
		 */
		if ((c >= '1' && c <= '9' && c != '5') || (c=='p' || c=='q')) {
			intens(0);
			cvdisp(*x,*y,*theta,f);
		}
		switch (c) {
		default :
			putc('\007',stderr);
			break;

		case 'E':
			fclose(in);
			return;

		case '1':
			(*x)--;
		case '2':
			(*y)--;
			break;

		case '3':
			(*y)--;
		case '6':
			(*x)++;
			break;

		case '7':
			(*y)++;
		case '4':
			(*x)--;
			break;

		case '9':
			(*x)++;
		case '8':
			(*y)++;
			break;

		case 'p':
			*theta -= 0.05;
			break;

		case 'q':
			*theta += 0.05;
			break;
		}
		intens(1);
		cvdisp(*x,*y,*theta,f);
	}
}

/*
 * display vector and center cross for centromere
 */
cvdisp(x,y,t,f)
double t;
struct pframe *f;
{
	double s,c,sin(),cos();
	struct fvector v;
	v.type = 31;
	s = 4.0 * sin(t);
	c = 4.0 * cos(t);
	v.k1 = x+s;
	v.k2 = x-s;
	v.l1 = y-c;
	v.l2 = y+c;
	picframe(&v,f);
	s *= 4.0;
	c *= 4.0;
	v.k1 = x-c;
	v.k2 = x+c;
	v.l1 = y-s;
	v.l2 = y+s;
	picframe(&v,f);
}
