/*
 * ccount.c		17/1/84		Jim Piper
 *
 * interactively count a cell file
 *
 * Modifications
 *
 *	05 Mar 1990		dcb		Remove overlay defns, and include display.h
 *	30 Sep 1987		CAS		Make kcont a pointer (and global - yuk)
 *	 3 Sep 1987		CAS		New ddgc open/dev calls
 *	29 Jan 1987		CAS		Kcont->FIP symbolic
 *	20 Nov 1986		CAS		Added onoff param to countdisp
 *	19 Nov 1986		CAS		Added onoff param to dispotype call
 *							dispblobs-> dispexpblobs (as in karylib)
 *	14 Sep 1986		CAS		Includes changed activelist from int to SMALL
 */

#include <stdio.h>
#include <wstruct.h>
#include <interact.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <fileheaders.h>
#include <display.h>

#define MAX(i,j)	(i>j? i: j)
#define MIN(i,j)	(i<j? i: j)
#define WIDTH		8

int IDEVICE;
int numchroms;
int explode = 0;
char termit[20];
struct kcontrol *kcont;

extern DDGS *ddgsopen();

main(argc,argv)
char **argv;
{
	struct chromosome *objlist[MAXOBJS];
	struct object *marklist[MAXOBJS];
	SMALL activelist[MAXOBJS];
	struct pframe f;

	setbuf(stderr,NULL);		/* UNBUFFERED */
	setmemalloc(16384);
	setpr(getpid(),255);
	kcont = (struct kcontrol *) Malloc (sizeof (struct kcontrol));

	/*
	 * kcont is the overall control structure for everything
	 * and is passed to slave processes and slave processors
	 */
	kcont->ool = objlist;
	kcont->rol = NULL;
	kcont->mol = NULL;
	kcont->bol = marklist;
	kcont->f = &f;
	kcont->maxnumber = 0;
	kcont->mstatus = 0;
	kcont->acl = activelist;
	kcont->FIP = DIGFIP;
	kcont->lock = 0;
	sprintf(termit,"/term");

	/* default interaction device - keyboard or mouse */
	IDEVICE = KEYBOARD;

	/* change defaults ? */
	while (argc > 1 && argv[1][0] == '-') {
		switch (argv[1][1]) {
			case 'f':
				kcont->FIP = DIGTV;
				break;
			case 'm':
				IDEVICE = MOUSE;
				break;
			case 'k':
				IDEVICE = KEYBOARD;
				break;
			case 't':
				sprintf(termit,"/t%s",&argv[1][2]);
				break;
		} 
		argv++;
		argc--;
	}

	/* set picture output */
	kcont->ddgschannel = ddgsopen(FS_VME512,1);
	dinit();
	fast();
	clear();
	slow();
	overlaycol();
	colour(GREYCOLS);
	allread(&kcont,stdin);
	alldisplay(&kcont);
	colour(OVERLAY2);
	dispexpblobs(&kcont);
	countdisp(numchroms,1);
	while (intwith(&kcont))
		;
	finish();
}


/*
 * main interaction routine.
 */
intwith(kcont)
struct kcontrol *kcont;
{
	FILE *in;
	register int c;
	int com,ser,onecol;
	struct chromosome *tobj;
	struct ipoint curs;
	struct pframe *f;
	static dbound=0;
	static dbackg=0;
	static dhull=0;
	static drect=0;

	f = kcont->f;
	in = fopen(termit,"r");
	setbuf(in,NULL);
	if (in == NULL)
		fprintf(stderr,"cannot open terminal\n");
	for (;;) {
		fprintf(stderr,"\n: ");
		c = getc(in);
		switch (c) {
			case 'x':		/* test identification by cursor */
				initident(kcont,&curs);
				do {
					keyobjident(kcont,&ser,&com,&curs,in);
					if (ser >= 0) {
						tobj = kcont->ool[ser];
						switch(com) {
							case 'c':
								modcount(tobj,1,f,&numchroms);
								break;
							case 'd':			/* 'd' */
								modcount(tobj,-1,f,&numchroms);
								break;
							default:
								break;
						}
					}
				} while (com != 'e');
				endident(kcont,&curs);
				fclose(in);
				return(1);

			case 'y':		/* count sequentially */
				countall(kcont,in);
				fclose(in);
				return(1);
				
								
			case 'b':		/* display boundaries */
				if (dbound == 0)
					dbound = 1;
				else
					dbound = 0;
				intens(dbound);
				allbound(kcont);
				intens(1);
				fclose(in);
				return(1);
			
			case 'h':		/* display hull */
				if (dhull == 0)
					dhull = 1;
				else
					dhull = 0;
				intens(dhull);
				allhull(kcont);
				intens(1);
				fclose(in);
				return(1);

			case 'r':		/* display rectangle */
				if (drect == 0)
					drect = 1;
				else
					drect = 0;
				intens(drect);
				allrect(kcont);
				intens(1);
				fclose(in);
				return(1);
			
			case 'c':		/* alter background colour */
				if (dbackg < 3)
					dbackg++;
				else
					dbackg = 0;
				setrgb(0,255-dbackg*64,255-dbackg*64,255-dbackg*64);
				if (dbackg <= 1)
					onecol = 0;
				else
					onecol = 255;
				setrgb(1,onecol,onecol,onecol);	/* contrasting lut(1) */
				fclose(in);
				return(1);

			case 's':		/* explode display */
				colour(255);
				fast();
				clear();
				slow();
				colour(GREYCOLS);
				intens(1);
				if (explode)
					explode = 0;
				else
					explode = 1;
				alldisplay(kcont);
				colour(OVERLAY2);
				dispexpblobs(kcont);
				countdisp(numchroms,1);
				fclose(in);
				return(1);

			case 'e':		/* exit */
				fclose(in);
				return(0);
		}
	}
}


/*
 * interactively adjust count of all objects:
 *	<return>	leave this object
 *		-		back up to previous object
 *		c		increase count (alter type noise->chromosme->composite)
 *		d		decrease count (alter type composite->chromosome->noise)
 */
countall(kcont,in)
register struct kcontrol *kcont;
FILE*in;
{
	int c, objnum;
	struct chromosome *tobj;
	struct chromplist *plist;
	struct pframe *f;
	
	f = kcont->f;
	for (objnum=0; objnum<kcont->maxnumber; objnum++) {
		if (kcont->acl[objnum]) {
			tobj = kcont->ool[objnum];
			plist = tobj->plist;
			colour(OVERLAY1);
			intens(1);
			typedisp(tobj);
			dispotype(tobj,f,1);
			disprect(tobj,f);
			do {
				c = getc(in);
				switch(c) {
					case '\n':
						break;
					case '-':
						if (objnum > 0)
							objnum-=2;
						break;
					default :
						bleep();
						break;
					case 'c':
						modcount(tobj,1,f,&numchroms);
						break;
					case 'd':			/* 'd' */
						modcount(tobj,-1,f,&numchroms);
						break;
				}
			} while (c != '\n' && c!= '-');
			intens(0);
			typedisp(tobj);
			disprect(tobj,f);
		}
	}
}

static struct pframe *f;
static int current;


/*
 * identify the next object and applicable command
 */
keyobjident(kcont,serial,command,cursor,ifile)
struct kcontrol *kcont;
int *serial, *command;
struct ipoint *cursor;
FILE *ifile;
{
	register int new, com;
	f = kcont->f;
	for (;;) {
		while ((com = movetty(cursor,ifile)) == 0) {
			new = locate(kcont,cursor);
			if (new != current) {
				markident(current,0,kcont);
				current = new;
				markident(current,1,kcont);
			}
		}
		if (current < 0 && com != 'e')
			bleep();
		else {
			*serial = current;
			*command = com;
			if (com == 'e')
				markident(current,0,kcont);
			return;
		}
	}
}
	

movetty(curs,ifile)
struct ipoint *curs;
FILE *ifile;
{
	register int c,mult;
	mult = 1;
	c = getc(ifile) & 127;
	colour(OVERLAY1);
	/*
	 * delete preceding point if end point has moved
	 */
	if (c >= '1' && c <= '9' && c != '5') {
		intens(0);
		picframe(curs,f);
		intens(1);
	}
	switch (c) {
	case 'e':
		intens(0);
		picframe(curs,f);
		intens(1);
		return(c);

	default :
		return(c);

	case '1':
		curs->k -= mult * f->ix;
	case '2':
		curs->l -= mult * f->iy;
		break;

	case '3':
		curs->l -= mult * f->iy;
	case '6':
		curs->k += mult * f->ix;
		break;

	case '7':
		curs->l += mult * f->iy;
	case '4':
		curs->k -= mult * f->ix;
		break;

	case '9':
		curs->k += mult * f->ix;
	case '8':
		curs->l += mult * f->iy;
		break;
	}
	picframe(curs,f);
	return(0);
}


bleep()
{
	putc('\007',stderr);
}

