/*
 * chromclass.c		Jim Piper	27/3/84
 ****************************************
 * Converted to integer arithmetic 3/1/86
 ****************************************
 *
 * Normalise feature measurements, then
 * classify chromosomes using Mahalanobis distance from class means.
 * Also classify using re-arrangement from over-subscribed classes.
 *
 * Optionally either write entire classified cell for karyogramming,
 * or just write summary output in format suitable for constructing
 * confusion matrix (if flag -c specified).
 *
 * This program should be run on individual cells.
 *
 * NOTE THIS PROGRAM WORKS WITH CLASS NUMBERS 0 - MAXCLASS-1.
 *
 **********************************************************************
 * NORMALISATION REQUIRES ALL CHROMOSOMES, COMPOSITES AND OVERLAPS !!!!
 **********************************************************************
 *
 * Modifications
 *
 *	14 Sep 1986		CAS		Includes
 */


#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>


/* logs of prior probabilities */
double lpriorp[MAXCLASS] = {
0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 
0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 
0.0, 0.0, 0.0, -0.5};

int clwn[MAXCLASS] = {
2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
2, 2, 2, 1};


main(argc,argv)
char **argv;
{
	FILE *fclass;
	struct chromosome *obj, *readobj(), *objlist[MAXOBJS], *chrlist[MAXOBJS];
	struct chromosome *otype1_3_list[MAXOBJS];
	struct classcont *classcontp;
	int i,j, nclass,numobj,numchr,ntype1_3,class,liksort();
	int ndim,sfv[MAXDIM],nsfv, chrindex[100], sex;
	int fvec[MAXDIM],svec[MAXDIM];
	struct chromcl *chp,chrcl[MAXOBJS];
	struct classcont clcontents[MAXCLASS];
	struct normaliser vn[MAXDIM];
	int confusion = 0;


	if (argc > 1 && argv[1][0] == '-' && argv[1][1] == 'c') {
		confusion++;
		argc--;
		argv++;
	}

	/* read class mean vectors and variance-covariance matrix */
	fclass = fopen(argv[1],"r");
	if (fclass == NULL) {
		fprintf(stderr,"Cannot open classifier parameter file\n");
		exit(2);
	}
fprintf(stderr,"reading classifier\n");
	readclassparams(fclass,&nclass,&ndim,sfv,&nsfv);
#ifdef DEBUG
fprintf(stderr,"nclass %d ndim %d nsfv %d\n",nclass,ndim,nsfv);
for (i=0; i<nsfv; i++)
	fprintf(stderr,"%3d",sfv[i]);
fprintf(stderr,"\n");
#endif DEBUG
	
	/*
	 * clear class contents structure
	 */
	for (i=0; i<nclass; i++)
		clcontents[i].n = 0;

	/* 
	 * Read objects, classify by likelihood, order the likelihoods.
	 */
	starttimer();
	numobj = 0;
	numchr = 0;
	ntype1_3 = 0;
	sex = 0;
fprintf(stderr,"reading objects\n");
	while ((obj = readobj(stdin)) != NULL) {
#ifdef DEBUG
fprintf(stderr,"object %d type %d  ",numobj,obj->type);
#endif DEBUG
		objlist[numobj] = obj;
		switch (obj->type) {
		case 1:
			/* is object to be included in normalisation ?? */
#ifdef DEBUG
fprintf(stderr," otype %2d",obj->plist->otype);
#endif DEBUG
			if (obj->plist->otype <= 3)
				otype1_3_list[ntype1_3++] = obj;
			/* is object a chromosome ? */
			if (obj->plist->otype <= 1) {
				chrlist[numchr] = obj;
				chrindex[numchr] = numobj;
				numchr++;
			}
			break;

		case 110:		/* cell i.d. - use for sex */
			break;

		default:
			break;
		}
		numobj++;
#ifdef DEBUG
fprintf(stderr,"\n");
#endif DEBUG
	}
	elapsedtime();
	/*
	 * compute cell normaliser
	 */
#ifdef DEBUG
fprintf(stderr,"%d objects %d chromosomes\n",numobj,numchr);
#endif DEBUG
fprintf(stderr,"constructing normaliser\n");
	normvec(otype1_3_list,ntype1_3,vn);
	elapsedtime();
#ifdef DEBUG
fprintf(stderr,"normalised OK\n");
#endif DEBUG
	/*
	 * classify using the normalised chromosome features
	 */
fprintf(stderr,"distance classifier\n");
	chp = chrcl;
	for (j=0; j<numchr; j++) {
		obj = chrlist[j];
		/* feature vector */
		getvec(obj->plist,fvec);
		/* normalised feature vector */
		normivec(fvec,svec,sfv,vn,ndim,nsfv);
		/* likelihood classifier */
		class = iclassdist(svec,lpriorp,&(chp->cllika[0]),nclass,nsfv) - 1;
		chp->trueclass = obj->plist->Cpgroup - 1;
		chp->rawclass = class;
		chp->newclass = class;
		classcontp = &clcontents[class];
		classcontp->chcl[classcontp->n] = chp;
		classcontp->n++;
#ifdef DEBUG
fprintf(stderr,"chromosome %d\n",j);
for (i=0; i<nclass; i++) {
fprintf(stderr,"%3d %10e\n",chp->cllika[i].class,chp->cllika[i].lval);
}
#endif DEBUG
#ifndef MIZAR
		/* sort the class likelihods of this chromosome into order */
		qsort(&(chp->cllika[0]),nclass,sizeof(struct cllik),liksort);
#endif MIZAR
#ifdef DEBUG
fprintf(stderr,"chromosome %d\n",j);
for (i=0; i<nclass; i++) {
fprintf(stderr,"%3d %f\n",chp->cllika[i].class,chp->cllika[i].lval);
}
#endif DEBUG
		chp++;
	}
	elapsedtime();
	/*
	 * re-arrange so as to move objects from over-subscribed
	 * classes to under-subscribed as far as plausibility allows
	 */
fprintf(stderr,"rearrangement classifier\n");
	rearrange(chrcl,clcontents,numchr,clwn,nclass);
	elapsedtime();

	/*
	 * write the data
	 */
fprintf(stderr,"writing classified objects\n");
	for (i=0; i<numchr; i++) {
		if (confusion > 0 && chrcl[i].trueclass < MAXCLASS)
			printf("%2d %2d %2d\n",chrcl[i].trueclass+1,chrcl[i].rawclass+1,chrcl[i].newclass+1);
		chrlist[i]->plist->pgroup = chrcl[i].newclass+1;
	}
	if (confusion == 0)
		for (i=0; i<numobj; i++)
			writeobj(stdout,objlist[i]);
	exit(0);
}
