/*
 * chromnorm.c		Jim Piper	6/9/84
 *
 * Normalise feature measurements, then write to file with correct class.
 * If flag -u, then construct same output, but unnormalised.
 * If flag -d, output derived as well as original chromosomes.  In this
 * case, indicate derived object by adding 100 to correct class (so that
 * it will be ignored by program "vcv") and ALSO WRITE A CELL SEPARATOR.
 *
 **********************************************************************
 * NORMALISATION REQUIRES ALL CHROMOSOMES, COMPOSITES AND OVERLAPS !!!!
 **********************************************************************
 *       MIZAR-OS9 VERSION (INTEGER ARITHMETIC WIDELY USED)
 **********************************************************************
 *
 * Modifications
 *
 *	14 Sep 1986		CAS		Includes
 */

#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>

extern int _memmins;

main(argc,argv)
char **argv;
{
	struct chromosome *obj, *readobj(), *otype1_3_list[100];
	int i,j, class, ntype1_3, ndim;
	int ivec[MAXDIM];
	int fvec[MAXDIM];
	struct normaliser vn[MAXDIM];
	int sfv[MAXDIM];
	int normalise = 1;
	int derived = 0;

	_memmins = 65536;
	while (argc > 1 && argv[1][0] == '-') {
		switch (argv[1][1]) {
			case 'u' :
				normalise = 0;
				break;
			case 'd' :
				derived++;
				break;
		}
		argv++;
		argc--;
	}
	ndim = getdim(0);
	for (i=0; i<MAXDIM; i++)
		sfv[i] = i;
	/* 
	 * Read objects
	 */
	ntype1_3 = 0;
	while ((obj = readobj(stdin)) != NULL) {
		switch (obj->type) {
		case 1:
			if (obj->plist->otype <= 3) {
				otype1_3_list[ntype1_3] = obj;
				ntype1_3++;
			}
			break;

		default:
			break;
		}
	}
	/*
	 * compute cell normalisers
	 */
	if (normalise)
		normvec(otype1_3_list,ntype1_3,vn);
	/*
	 * normalise chromosome features, output non-derived objects only
	 */
	for (j=0; j<ntype1_3; j++) {
		obj = otype1_3_list[j];
		if (obj->plist->otype <= 1 &&
			(derived != 0 || obj->plist->history[0] == 0) ) {
			/* write correct class */
			class = obj->plist->Cpgroup;
			if (class == 0)
				class = obj->plist->pgroup;
			if (obj->plist->history[0] != 0)
				class += 100;
			printf("%3d ",class);
			/* feature vector */
			getvec(obj->plist,ivec);
			/* compute and write normalised feature vector */
			if (normalise) {
				normivec(ivec,fvec,sfv,vn,ndim,ndim);
				for (i=0; i<ndim; i++) {
					printf("%6d ",fvec[i]);
				}
				printf("\n");
			} else {
				for (i=0; i<ndim; i++) {
					printf("%6d ",ivec[i]);
				}
				printf("\n");
			}
		}
	}
	/* cell separator if derived object data */
	if (derived != 0) {
		printf("%3d ",-1);
		for (i=0; i<ndim; i++)
			printf("%6d ",0);
		printf("\n");
	}
}
