/*
 *
 *      D E M O P R O F . C  --  profiles demonstrator
 *
 *
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *  Written: Jim Piper
 *           MRC CAPCU
 *	
 * Derived from testch.c.  Test bed for centromere finding.
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *
 *  Modifications
 *
 *	24 Mar 1988		CAS			Clean up - add header etc
 *
 */
#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <ddgs.h>

int wh[2] = {-1,2};


double xscale = 1.0;
double yscale = 0.75;

main(argc,argv)
char **argv;
{
	char	*swptr, **targv;
	int		targc, doneone;
	int		store = FS_VME512, devno = 0;
	DDGS	*dg, *ddgsopen();
	struct object *spobj, *mpol, *cvh, *rect;
	struct chromosome *obj, *readobj();
	struct object *fipconvhull(), *spinsqueeze(), *minwrect();
	struct object *poly, *centpoly(), *chromaxis2();
	struct object *prof, *p[3], *nprofile();
	struct object *polyprof(), *polyp;
	struct histogramdomain *h;
	struct polygondomain *pdom, *uspaxis8();
	struct pframe f;
	int i, j, l, number, cpos1, cpos2, ca, cl, ydiff;
	double os9frig;
	char s[100];
	int VERT = 0;

	targc = argc; targv = argv;

	argc--; argv++;			/* dump process name */
	while (argc > 0) {
		if (*argv[0] == '-') {
			swptr = *argv;
			*argv = NULL;
			while (*(++swptr)) {
				switch (*swptr) {
				  case '?':
				  	printf("Syntax:   demoprof <options>\n");
				  	printf("Function: \n");
				  	printf("Options:  v - Show version number\n");
				  	printf("          r - rotate to vertical\n");
					printf("          1 - Use seat 1 framestore\n");
					printf("          2 - Use seat 2 framestore\n");
					printf("                  also 3 and 4\n");
					printf("          V - Virtuoso framestore (Default CRS 512)\n");
				  	printf("          ? - Help\n");
				  	exit(0);
				  case 'v':
						printf("Testch - Version 0.1\n");
						exit(0);
					case 'r':
						VERT = 1;
						break;	
					case '1':
						devno = 0;
						break;
					case '2':
						devno = 1;
						break;
					case '3':
						devno = 2;
						break;
					case '4':
						devno = 3;
						break;
					case 'V':
						store = FS_VIRTUOSO_INVERT;
						break;
				  default:
				 	printf("demoprof - unknown option\n");
				}
			}
		}
		argv++; argc--;
	}

	argc = targc; argv = targv;

	dg = ddgsopen(store,devno);
	dinit();
	llut();
	tsize(50,50);
	number = 0;
	while ((obj = readobj(stdin)) != NULL) {
		number++;
		if (area(obj) < 60)
			fprintf(stderr,"tiny object ignored %d\n",number);
		else if (obj->plist == NULL || obj->plist->otype <= 1) {
		f.type = 60;
		f.scale = 4;
		f.ix = 8;
		f.iy = 8;
		f.dx = 2560;
		f.dy = 1024;
		if (VERT)
			spobj = (struct object *) obj;
		else {
			cvh = fipconvhull(obj);
			rect = minwrect(cvh);

			/*
			 * rotate to vertical
			 */
			os9frig = ((struct frect *)rect->idom)->rangle;
			spobj = spinsqueeze(obj,&os9frig,&xscale,&yscale);
		}
/*		normalise(spobj);		*/
		f.ox = spobj->idom->kol1;
		f.oy = spobj->idom->line1;
		mpol = chromaxis2(spobj);
	
		/*
		 * moment profile computation and smoothing
		 */
		multiprofiles(spobj,mpol,p,wh,2);
		shorten(mpol,p[0],p[1]);
		prof = p[0];
/*		histosmooth(prof, 3);	*/
		histonormalise(prof);
		clear();
/*		dispnum(number,128,128);	*/
		picframe(spobj,&f);
		f.dx -= 768;
		h = (struct histogramdomain *) prof->idom;
		ydiff = h->npoints - (spobj->idom->lastln-spobj->idom->line1+1);
		ydiff *= f.scale * f.iy / 2;
		f.dy -= ydiff;
		moveto(f.dx,f.dy);
		moveby(0,-100);
		text("mean");
		moveby(0,-70);
		text("density");
		moveby(0,-70);
		text("profile");
		moveby(0,240);
		lineby(0,h->npoints*f.scale*f.iy);
		f.ix = 2;
		picframe(prof,&f);
		prof = p[1];

		/*
		 * locate centromere in mirrored profile with
		 * quantum parameter set to be 10% of peak value.
		 */
		histosmooth(prof, 3);
		cpos1 = cmerelocation(prof,10,1);
		f.dx -= 769;
		cpos1++;
		moveto(f.dx,f.dy+cpos1*f.scale*f.iy);
		moveby(-600,0);
		text("centromere");
		moveby(600,0);
		lineby(1450,0);

		histonormalise(prof);
		h = (struct histogramdomain *) prof->idom;
		moveto(f.dx,f.dy);
		moveby(0,-100);
		text("shape");
		moveby(0,-70);
		text("profile");
		moveby(0,170);
		lineby(0,h->npoints*f.scale*f.iy);
		f.ix = 2;
		picframe(prof,&f);

		f.dy += ydiff;
		f.dx += 1536;
		f.ix = f.iy = 1;
		f.ox *= 8;
		f.oy *= 8;
		f.dx += 4*f.scale;
		picframe(mpol,&f);

		if (cpos1 >= 0) {
			poly = centpoly(mpol,cpos1);
			pdom = (struct polygondomain *) poly->idom;
			poly->idom = (struct intervaldomain *) uspaxis8(pdom);
			picframe(poly,&f);
			free(pdom);
		}

		freeobj(prof);
		freeobj(cvh);
		freeobj(mpol);
		freeobj(rect);
		
		tsleep(25);
		}
		freeobj(obj);
	}
}
