/*
 * erodesplit.c		Jim Piper @ AUC		04-04-86
 *
 * Attempt composite split by erosion, return list
 * of resultant objects.  Their evaluation should
 * be carried out elsewhere.  The procedure is as follows:
 *		(1) Erode object required number of times
 *		(2) Label object
 *		(3) Exit if only one labelled segment.
 *		(4) otherwise dilate each labelled segment same number of
 *			times as was eroded.
 *		(5) Return resultant objects.
 *
 * Parameters are essentially the same as for label:
 *	obj		input object
 *	*n		number of objects found (also returned by function)
 *	objl	pointer to list for output objects
 *	maxn	maximum size of the output list
 *	ignlns	objects less than this high or wide to be ignored
 *	nerode	number of erosion passes required
 *
 * Modifications
 *
 *	14 Sep 1986		CAS		Includes
 */
#include <stdio.h>
#include <wstruct.h>


erodesplit(obj,n,objl,maxn,ignlns,nerode)
struct object *obj, **objl;
int *n;
{
	struct object *erosion(), *dilation(), *o1, *o2;
	int i, j;
	
	/*
	 * erosion pass
	 */
	o1 = obj;
	for (i=0; i<nerode; i++) {
		o2 = erosion(o1);
		if (i != 0) 
			freeobj(o1);
		o1 = o2;
	}
	/*
	 * label
	 */
	label(o1,n,objl,maxn,ignlns);
	/*
	 * return if no segmentation achieved
	 */
	if (*n <= 1)
		return(*n);
	/*
	 * dilation pass - dilate each object
	 */
	for (j=0; j < *n; j++) {
		o1 = objl[j];
		for (i=0; i<nerode; i++) {
			o2 = dilation(o1);
			freeobj(o1);
			o1 = o2;
		}
		/* give final objects a grey-table */
		o2->vdom = obj->vdom;
		objl[j] = o2;
	}
	return(*n);
}
