/*
 * interseg.c		17/1/84		Jim Piper
 *
 * interactively segment a cell file
 *
 * Modifications
 *
 *	 3 Sep 1987		CAS		New ddgs open/dev calls
 *	29 Jan 1987		CAS		Kcont->FIP symbolic
 *	14 Sep 1986		CAS		Includes
 */

#include <stdio.h>
#include <wstruct.h>
#include <interact.h>
#include <chromanal.h>
#include <ddgs.h>
#include <fileheaders.h>

#define MAX(i,j)	(i>j? i: j)
#define MIN(i,j)	(i<j? i: j)
#define WIDTH		8

int number,scale,cellw,celll;
int FIP, IDEVICE;
int numchroms,numchwas;
struct pframe f;
char TERM[20];
char MTERM[20];
char *ITERM;

#define STO	"stobw"

main(argc,argv)
char **argv;
{
	struct chromosome *objlist[200];
	struct object *marklist[200];
	int activelist[200];
	char *sto;

	setbuf(stderr,NULL);		/* UNBUFFERED */
	setpr(getpid(),255);
	/* set aspect ratio for FIP digitisations */
	FIP = DIGFIP;
	/* default interaction device - mouse */
	IDEVICE = MOUSE;
	sprintf(TERM,"/term");
	sprintf(MTERM,"/t4");
	ITERM = MTERM;
	/* change defaults ? */
	while (argc > 1 && argv[1][0] == '-') {
		switch (argv[1][1]) {
			case 'f':
				FIP = DIGTV;
				break;
			case 'k':
				IDEVICE = KEYBOARD;
				ITERM = TERM;
				break;
			case 't':
				sprintf(&TERM[1],"%s",&argv[1][1]);
				break;
		} 
		argv++;
		argc--;
	}
	fprintf(stderr,"Using terminal %s\n",TERM);

	/* set picture output */
	sto = STO;
	if (argc>1)
		sto = argv[1];
	ddgsopen(FS_VME512,1);
	dinit();
	clear();
	slow();
	lutfrig();
	colour(254);
	tsize(50,50);
	readanddisplay(objlist,marklist,activelist);
	colour(1);
	dispbound(objlist,marklist,activelist);
		tsize(130,150);
		dispnum(numchroms,300,250);
		tsize(50,50);
		update();
		numchwas = numchroms;
	while (intwith(objlist,marklist,activelist)) {
		tsize(130,150);
		intens(0);
		dispnum(numchwas,300,250);
		intens(1);
		dispnum(numchroms,300,250);
		tsize(50,50);
		update();
		numchwas = numchroms;
	}
	putaway(objlist,activelist);
	finish();
}



readanddisplay(objlist,marklist,activelist)
struct chromosome **objlist;
struct object **marklist;
SMALL *activelist;
{
	struct chromosome *obj, *readobj();
	struct object *blobmark();
	struct chromplist *makechromplist();
	int objnum,cellkol1,cellline1,celllastkl,celllastln,celldiam;
	while((obj = readobj(stdin)) != NULL) {
		if (obj->type == 1) {
			if (obj->plist == NULL) {
				obj->plist = makechromplist();
				obj->plist->number = number+1;
			}
			objlist[number] = obj;
			marklist[number] = blobmark(obj);
			activelist[number++] = 1;
			if (number == 1) {
				cellline1 = obj->idom->line1;
				cellkol1 = obj->idom->kol1;
				celllastln = obj->idom->lastln;
				celllastkl = obj->idom->lastkl;
			} else {
				cellline1 = MIN(cellline1,obj->idom->line1);
				cellkol1 = MIN(cellkol1,obj->idom->kol1);
				celllastln = MAX(celllastln,obj->idom->lastln);
				celllastkl = MAX(celllastkl,obj->idom->lastkl);
			}
		}
	}
	numchroms = number;

	scale = 8;
	celldiam = celllastln - cellline1;
	if (FIP == DIGFIP)
		celldiam = 3*celldiam/4;
	if (celldiam < celllastkl - cellkol1)
		celldiam = celllastkl - cellkol1;
	if (celldiam > 500) {
		scale = 8 * 480 / celldiam;
		fprintf(stderr,"default scale changed to %d\n",scale);
		fprintf(stderr,"aspect ratio may be slightly wrong\n");
	}
	f.dx = 2080;
	f.dy = 2080;
	f.scale = 1;
	f.ix = scale;
	f.iy = (FIP!=DIGTV? 3*scale/4: scale);
	f.ox = (cellkol1+celllastkl)/2;
	f.oy = (cellline1+celllastln)/2;
	intens(1);

	for (objnum=0; objnum<number; objnum++) {
		picframe(objlist[objnum],&f);
	}
}


dispbound(objlist,marklist,activelist)
struct chromosome **objlist;
struct object **marklist;
SMALL *activelist;
{
	int objnum;
	
	for (objnum=0; objnum<number; objnum++) {
		if (activelist[objnum]) {
			picframe(marklist[objnum],&f);
			frameobjnum(objnum+1,objlist[objnum],&f);
		}
	}
}

intwith(objlist,marklist,activelist)
struct object **marklist;
struct chromosome **objlist;
SMALL *activelist;
{
	int c,i,j,k,parent,objnum,jn[2];
	FILE *in;
	struct chromosome *extractov(), *unionn(), *uobjl[2];
	struct object *polyobj, *interactpoly(), *blobmark();
	struct object *bound, *ibound(), *boundlist[20];
	struct intervaldomain *idom;
	struct chromplist *makechromplist();
	struct chromplist *plist, *qlist;

	in = fopen(TERM,"r");
	if (in == NULL)
		fprintf(stderr,"cannot open terminal\n");
	for (;;) {
		write(fileno(stderr),": ",2);
		c = getc(in);
		switch (c) {
		case 'c' :	/* COMPOSITE - SPLIT */
			objnum = getonum(in,activelist);
			if (objnum < 0)
				goto exitswitch;
			objnum--;
			fclose(in);
			idom = objlist[objnum]->idom;
			intens(0);
			picframe(marklist[objnum],&f);
			frameobjnum(objnum+1,objlist[objnum],&f);
			intens(1);
			fprintf(stderr,"split with cursor (? for more help)\n");
			polyobj = interactpoly(IDEVICE,&f,(idom->kol1+idom->lastkl)/2,
			(idom->line1+idom->lastln)/2,ITERM);
			if (polyobj == NULL) {
				picframe(marklist[objnum],&f);
				frameobjnum(objnum+1,objlist[objnum],&f);
				return(1);
			}
			/* split original and picture results */
			polysplit(objlist[objnum],polyobj,&objlist[number],&k);
			activelist[objnum] = 0;
			qlist = (struct chromplist *)objlist[objnum]->plist;
			qlist->otype = 2;	/* composite */
			qlist->Cotype = 2;	/* composite */
			qlist->otconf = 100;
			qlist->history[1] = 1;	/* manually split */
			for (i=number; i<number+k; i++) {
				marklist[i] = blobmark(objlist[i]);
				picframe(marklist[i],&f);
				frameobjnum(i+1,objlist[i],&f);
				objlist[i]->plist = makechromplist();
				plist = (struct chromplist *)objlist[i]->plist;
				plist->number = i+1;
				plist->pnumber = objnum+1;
				plist->otype = 1; 	/* chromosome */
				plist->otconf = 100;
				plist->history[0] = 1;	/* result of manual split */
				activelist[i] = 1;
			}
			number += k;
			numchroms += (k-1);
			return(1);

		case 'o':	/* OVERLAP - INCREMENT COUNT */
			objnum = getonum(in,activelist);
			if (objnum < 0)
				goto exitswitch;
			objnum--;
			activelist[objnum] = 0;
			fprintf(stderr,"how many chromosomes?\n");
			fscanf(in,"%d",&j);
			numchroms += j-1;
			getc(in);	/* past newline */
			fclose(in);
			qlist = objlist[objnum]->plist;
			qlist->Cotype = 3;	/* overlap */
			qlist->otype = 3;	/* overlap */
			qlist->otconf = 100;
			qlist->history[1] = 2;	/* manually segmented */
			idom = objlist[objnum]->idom;
			intens(0);
			picframe(marklist[objnum],&f);
			frameobjnum(objnum+1,objlist[objnum],&f);
			intens(1);
			for (i=0; i<j; i++) {
				fprintf(stderr,"Draw axis of chromosome %d using cursor (? for more help)\n",i+1);
				polyobj = interactpoly(IDEVICE,&f,(idom->kol1+idom->lastkl)/2
				,(idom->line1+idom->lastln)/2,ITERM);
				if (polyobj == NULL)
					break;
				/* extract object and picture results */
				objlist[number] = extractov(objlist[objnum],polyobj,WIDTH);
				objlist[number]->plist = makechromplist();
				plist = objlist[number]->plist;
				plist->number = number+1;
				plist->pnumber = objnum+1;
				plist->Cotype = 1; 	/* chromosome */
				plist->otype = 1; 	/* chromosome */
				plist->history[0] = 2;	/* result of manual extraction from overlap */
				boundlist[i] = ibound(objlist[number],1);
				picframe(boundlist[i],&f);
				number++;
			}
			for (i=number-j-1; i<number; i++) {
				activelist[i] = 1;
				marklist[i] = blobmark(objlist[i]);
				picframe(marklist[i],&f);
				frameobjnum(i+1,objlist[i],&f);
			}
			intens(0);
			for (i=0; i<j; i++) {
				picframe(boundlist[i],&f);
				freeobj(boundlist[i]);
			}
			intens(1);
			return(1);

		case 'n' :	/* NOISE OR NUCLEUS */
			c = getc(in);
			if (c != 'o' && c != 'u') {
				putc('\007',stderr);
				while (getc(in) != '\n')
					;
			} else {
				objnum = getonum(in,activelist);
				if (objnum < 0)
					goto exitswitch;
				objnum--;
				activelist[objnum] = 0;
				qlist = objlist[objnum]->plist;
				qlist->Cotype = (c=='u'? 5: 6);
				qlist->otype = qlist->Cotype;
				qlist->otconf = 100;
				numchroms--;
			}
exitswitch:
			getc(in);		/* get past the newline */
			fclose(in);
			return(1);

		case 'N':	/* RENORMALISE DISPLAY */
			renormalisedisplay(objlist,activelist);
			dispbound(objlist,marklist,activelist);
			goto exitswitch;

		case 'j':	/* JOIN TWO OBJECTS */
			jn[0] = getonum(in,activelist);
			if (jn[0] < 0)
				goto exitswitch;
			jn[1] = getonum(in,activelist);
			if (jn[1] < 0)
				goto exitswitch;
			intens(0);
			for (j=0; j<=1; j++) {
				i = jn[j] - 1;
				picframe(marklist[i],&f);
				frameobjnum(i+1,objlist[i],&f);
				activelist[i] = 0;
				qlist = objlist[i]->plist;
				qlist->otype = 4;	/* piece */
				qlist->Cotype = 4;	/* piece */
				qlist->otconf = 100;
				qlist->history[1] = 4;	/* manually joined */
				uobjl[j] = objlist[i];
			}
			intens(1);
			objlist[number] = unionn(uobjl, 2, 1);
			marklist[number] = blobmark(objlist[number]);
			picframe(marklist[number],&f);
			frameobjnum(number+1,objlist[number],&f);
			objlist[number]->plist = makechromplist();
			plist = (struct chromplist *)objlist[number]->plist;
			plist->number = number+1;
			plist->pnumber = jn[0];		/* 1st parent */
			plist->history[2] = jn[1];	/* 2nd parent */
			plist->otype = 1; 	/* chromosome */
			plist->otconf = 100;
			plist->history[0] = 4;	/* result of manual join */
			activelist[number] = 1;
			number++;
			numchroms--;
			goto exitswitch;

		case 's':	/* SHOW BOUNDARY */
			i = getonum(in,activelist);
			if (i < 0)
				goto exitswitch;
			bound = ibound(objlist[i-1],1);
			picframe(bound,&f);
			getc(in);	/* past newline */
			fprintf(stderr,"type newline to continue:\n");
			getc(in);
			intens(0);
			picframe(bound,&f);
			intens(1);
			freeobj(bound);
			fclose(in);
			return(1);

		case 'u':	/* UNDO AN INTERACTION */
			fscanf(in,"%d",&objnum);
			if (objnum <= 0 || objnum > number || activelist[objnum-1] < 0) {
				fprintf(stderr,"\007illegal object number\n");
				goto exitswitch;
			}
			objnum--;
			plist = objlist[objnum]->plist;
			switch (plist->otype) {
			case 1:
				/*
			 	* find if derived object, find parent
			 	*/
				parent = plist->pnumber;
				if (parent <= 0) {
					fprintf(stderr,"NOT a valid derived object\n");
					goto exitswitch;
				}
				/*
			 	* find other derived objects and permanently de-activate
			 	*/
				intens(0);
				for (i=0; i<number; i++) {
					if (objlist[i]->plist->pnumber == parent) {
						picframe(marklist[i],&f);
						frameobjnum(i+1,objlist[i],&f);
						activelist[i] = -1;
						if (objlist[i]->plist->otype <= 1)
							numchroms--;
					}
				}
				/*
			 	* re-activate and re-display parent
			 	*/
				intens(1);
				parent--;
				qlist = objlist[parent]->plist;
				qlist->history[1] = 0;
				qlist->otype = qlist->Cotype = 1;
				picframe(marklist[parent],&f);
				frameobjnum(parent+1,objlist[parent],&f);
				activelist[parent] = 1;
				numchroms++;
				/*
				 * if a join, get other parent
				 */
				if (plist->history[0] == 4) {
					parent = plist->history[2] - 1;
					qlist = objlist[parent]->plist;
					qlist->history[1] = 0;
					qlist->otype = qlist->Cotype = 1;
					picframe(marklist[parent],&f);
					frameobjnum(parent+1,objlist[parent],&f);
					activelist[parent] = 1;
					numchroms++;
				}
				goto exitswitch;
			case 0:
			case 2:
			case 3:
			case 4:
			case 5:
			default:
				fprintf(stderr,"Object %d now assumed to be a chromosome\n",objnum+1);
				plist->otype = 1;
				numchroms++;
				goto exitswitch;
			}

		case 'e':	/* EXIT */
			getc(in);	/* past newline */
			fclose(in);
			return(0);

		case '?':		/* HELP */
			helpmess();
			goto exitswitch;

		default :
			putc('\007',stderr);
fprintf(stderr,"%03o\n",(int)c);
			while (getc(in) != '\n')
				;
			fclose(in);
			return(1);
		}
	}
}


/*
 * return a valid object number of a displayed object, or -1
 */
getonum(in,activelist)
FILE *in;
SMALL *activelist;
{
	int i;
	fscanf(in,"%d",&i);
	if (i <= 0 || i > number || activelist[i-1] <= 0) {
		fprintf(stderr,"\007illegal object number\n");
		return(-1);
	} else
		return(i);
}

static helpmess()
{
	fprintf(stderr,"AVAILABLE COMMANDS ARE:\n");
	fprintf(stderr,"c n       object n is a composite - split\n");
	fprintf(stderr,"e         exit interaction\n");
	fprintf(stderr,"j n1 n2   join object n1 to object n2\n");
	fprintf(stderr,"no n      object n is noise - ignore\n");
	fprintf(stderr,"nu n      object n is nucleus - ignore\n");
	fprintf(stderr,"o n       object n is an overlap - resolve components\n");
	fprintf(stderr,"u n       undo action previously taken to obtain object n\n");
	fprintf(stderr,"N         renormalise display, ignoring noise and nuclei\n");
}

/*
 * put away chromosomes, composites and overlaps only,
 * and ignore false-start objects left around by the "u" command.
 */
putaway(olist,activelist)
struct chromosome **olist;
SMALL *activelist;
{
	int i;
	for (i=0; i<number; i++)
		if (activelist[i] >= 0 && olist[i]->plist->otype <= 3)
			writeobj(stdout,olist[i]);
}



renormalisedisplay(objlist,activelist)
struct chromosome **objlist;
SMALL *activelist;
{
	int i,unum;
	struct chromosome *objl[100];
	unum = 0;
	for (i=0; i<number; i++)
		if (activelist[i])
			objl[unum++] = objlist[i];
	normaliseset(objl,unum);
	colour(254);
	for (i=0; i<unum; i++)
		picframe(objl[i],&f);
	colour(1);
}


/*
 * normalise grey-tables of set of objects jointly to lie in range 0-255
 */
normaliseset(objl,unum)
struct object **objl;
{
	register GREY v,min,max,range, *g;
	int i,n;
	struct object *obj;
	struct iwspace iwsp;
	struct gwspace gwsp;
	min = 1;
	max = 0;
	for (n=0; n<unum; n++) {
		obj = objl[n];
		initgreyscan(obj,&iwsp,&gwsp);
		while (nextgreyinterval(&iwsp) == 0) {
			g = gwsp.grintptr;
			for (i=0; i<iwsp.colrmn; i++) {
				v = *g++;
				if (min > max)
					min = max = v;
				else if (v > max)
					max = v;
				else if (v < min)
					min = v;
			}
		}
	}
	range = max - min + 1;
	for (n=0; n<unum; n++) {
		obj = objl[n];
		initgreyscan(obj,&iwsp,&gwsp);
		while (nextgreyinterval(&iwsp) == 0) {
			g = gwsp.grintptr;
#ifdef MIZAR
			gstretch(g,iwsp.colrmn,255,min,range);
#else MIZAR
			for (i=0; i<iwsp.colrmn; i++) {
				*g = (256 * (*g - min)) / range;
				g++;
			}
#endif MIZAR
		}
	}
}


/*
 * find the darkest point of an object,
 * mark it with a blob.
 */
struct object *blobmark(obj)
struct object *obj;
{
	struct ipoint *blobpoint;
	struct iwspace iwsp;
	struct gwspace gwsp;
	COORD linmax, colmax;
	register GREY *g, gmax;
	register int k;

	gmax = 0;
	initgreyscan(obj,&iwsp,&gwsp);
	while (nextgreyinterval(&iwsp) == 0) {
		g = gwsp.grintptr;
		for (k=0; k< iwsp.colrmn; k++) {
			if (*g > gmax) {
				gmax = *g;
				linmax = iwsp.linpos;
				colmax = iwsp.lftpos + k;
			}
			g++;
		}
	}
	blobpoint = (struct ipoint *) Calloc(sizeof(struct ipoint),1);
	blobpoint->type = 40;
	blobpoint->k = colmax;
	blobpoint->l = linmax;
	blobpoint->style = 2;
	return((struct object *) blobpoint);
}
