/*
 * jmkcontrol.c		Jim Piper		13-11-86
 *
 * from MRC version of mkcontrol.c 13-11-86
 *
 * Control karyotyping in a multi-process, multi-processor environment
 *
 *****************************************************************************
 * MODIFICATION HISTORY
 * --------------------
 * Who	When		What
 * ---	----		----
 *	18 Jul 1991		ih		Add NULL to displayall call (linebuf not available)
 *	23 Oct 1989		dcb		Pass initslave the slave process priority
 *  28 Feb 1989		SEAJ	Changed call to kcopyback to take extra parameter.
 *	21 Sep 1987		CAS		Make kcont an externally avail structure pointer
 *	 2 Sep 1987		CAS		Add dgsopen + ddgsdev calls for new ddgslib
 *	29 Jan 1987		CAS		Kcont->FIP symbolic
 * JP	13-11-86	Call to setinterrupt(&kcont) added.
 * JP	04-11-86	"overlaycol()" replaced by "overlaycol2()"
 * JP	31-10-86	Object type measuring stage added on compile-time switch.
 * JP	31-10-86	Mods to structure of Boundary/hull list 
 * JP	29-10-86	Boundary and hull list added to correspond to change
 *					to "kcontrol.h"
 * JP	28-10-86	"probotype()" called on each original object before
 *					displaying metaphase
 * JP	17-10-86	"lutfrig()" replaced by "overlaycol()"
 * JP	20-08-86	Demonstration of straightening included
 *****************************************************************************
 */
#include <stdio.h>
#include <multip.h>
#include <wstruct.h>
#include <chromanal.h>
#include <kcontrol.h>
#define YES 1
#define NO 0



/*   externals that were defined in kary   			*/

/* THESE EXTERNALS WILL BE STUCK IN A STRUCTURE SOON */
struct text *textobj;
struct cellid *cellinformation;
int SHANDON = 0;
int VIDEORECORD = 0;
int aspectr = 0;
int DOWN = 0;
int thereiscellinfo = 0;

struct kcontrol *kcont;


main(argc,argv)
char **argv;
{
	struct chromosome *oolist[MAXOBJS], *rolist[MAXOBJS];
	struct chromosome *molist[MAXOBJS], *mpcopyobj();
	struct object *bolist[MAXOBJS];
	struct othings bhl[MAXOBJS];
	SMALL activelist[MAXOBJS];
	struct pframe f, framelist[MAXOBJS];
	register int i, slave, maxsla;
	int interrupt();
	char termit[20];
	int interact=YES;
	int karyflag=NO;
		
	setmemalloc(16384);
	setpr(getpid(),MPRIOR);
	setbuf(stderr,NULL);
	intercept(interrupt);
	starttimer();

	initkcontrol(kcont);
	setinterrupt(kcont);

	/* ARGUMENT SWITCHES */
	maxsla = 0;
	while (argc > 1 && argv[1][0] == '-') {
		switch (argv[1][1]) {
			case 'i': /* interaction off flag */
				interact = NO ;
				break;
			case 's': /* max slave processor number */
				maxsla = atoi(&argv[1][2]);
				if (maxsla > NSLAVES)
					maxsla = NSLAVES;
				if (maxsla < 1)
					maxsla = 1;
				maxsla--;
				break;
			case 't': /* set "seat" (terminal) for interaction */
				sprintf(termit,"/t%s",&argv[1][2]);
				kcont->seat = 1 ;
				kcont->terminal = termit;
				fprintf(stderr,"Using terminal %s\n",kcont->terminal);
				break;
			case 'f':	/* Set FIP to 3 for larger chromosomes after rotation */
				kcont->FIP = atoi(&argv[1][2]);
				break;
			case 'k':	/* Karyotype only */
				karyflag++;
				break;
		}
		argc--;
		argv++;
	}
	ddgsopen(FS_VME512,kcont->seat);
	kcont->ool = oolist;
	kcont->rol = rolist;
	kcont->bol = bolist;
	kcont->mol = molist;
	kcont->oth = bhl;
	kcont->f = &f;
	kcont->ivf = framelist;
	kcont->acl = activelist;
	kcont->explode = 0;
	
	for (i=0; i<MAXOBJS; i++) {
		oolist[i] = NULL;
		rolist[i] = NULL;
		bolist[i] = NULL;
		molist[i] = NULL;
		bhl[i].bound = bhl[i].hull = NULL;
		bhl[i].procb = bhl[i].proch = 0;
	}

	/*
	 * start the slaves, one per processor including in master
	 */
	for (slave=0; slave<=maxsla; slave++)
		initslave(&kcont,slave,slave%NPROCESSORS, SPRIOR);
	/*
	 * start display
	 */	
	dinit();
	overlaycol2();

	/*
	 * If karyflag is true then data must be read into molist
	 * with copies of pointers in oolist, and all the segmentation
	 * and feature measurement can be circumvented.
	 */
	if (karyflag) {
		readkary(stdin,&kcont);
		initclass(&kcont);
		cellframe(&kcont);
	}
	else {
		/*
		 * clear screen
		 */
		fast();
		mpclear(&kcont);
		slow();
	
#ifdef OTMEAS
		/*
		 * start object-type feature measurement in slave processors
		 * which can then get on while data is read from disc
		 */
		kcont->mstatus |= MAXNCHANGE;
		otmeasstart(&kcont);
#else OTMEAS
		/*
		 * start chromosome feature measurement in slave processors
		 */
		fmeasstart(&kcont);
#endif OTMEAS
	
		/* read data from disc (bump up priority for speed) */
		setpr(getpid(),MPRIOR+10);
		readcell(stdin,&kcont);
		kcont->mstatus &= ~MAXNCHANGE;
		setpr(getpid(),MPRIOR);

		/* do a quick fix on probable object types before displaying */
		for (i=0; i<kcont->maxnumber; i++)
			probotype(kcont->ool[i]);
	
		/* execute segmentation in this process */
		segdisplay(&kcont);
	
#ifdef OTMEAS
		/*
		 * wait for object type measurement to finish, clean
		 * up MSTART, MEND bits in activity list
		 */
		waitotmeas(&kcont);
		for (i=0; i<kcont->maxnumber; i++)
			activelist[i] &= ~(MSTART|MEND);
	
		/*
		 * start chromosome feature measurement in slave processors
		 */
		fmeasstart(&kcont);
#endif OTMEAS
	
		/* Read classifier params while operator thinks about what to segment. */
		wormcast(&kcont,MASTER,"mkcontrol");
		initclass(&kcont);
		wormcast(&kcont,MASTER,"mkcontrol");

		/*
		 * do segmentation interaction
		 */	
		localseg(&kcont,interact);
		fprintf(stderr,"SEGMENTATION FINISHED\n");

		/* wait for feature measurement to finish */
		waitfmeas(&kcont);
		
		/* copy back objects processed on another processor */
		kcopyback(&kcont,0, &kcont->maxnumber);
		wormcast(&kcont,MASTER,"mkcontrol after kcopyback");
		fprintf(stderr,"MEASUREMENT FINISHED\n");
		elapsedtime();

		/* classify */
		class(&kcont);
		elapsedtime();
		wormcast(&kcont,MASTER,"mkcontrol after classifier");
	}

	/* karyotype in this process */
	localkary(&kcont,interact);
	wormcast(&kcont,MASTER,"mkcontrol after karyotype");
	
	/* write the data to file */
	Cplist(&kcont);
	putaway(stdout,&kcont);

	/* kill the other processes in this and other processors */
	fprintf(stderr,"KCONTROL: free stack %d\n",freemem());
	elapsedtime();
	interrupt(2);
}

/*
 * Ensure that the corrected chromosome feature values
 * and classification results are inserted in the property list.
 * Currently only applied to active objects (i.e. chromosomes)
 * but SHOULD be used before writing any object (e.g. split
 * composites etc.) that may be required subsequently.
 */
Cplist(kc)
register struct kcontrol *kc;
{
	register struct chromplist *plist;
	register int i;

	for (i=0; i<kc->maxnumber; i++) {
		if (kc->acl[i] & ACTIVE) {
			plist = kc->mol[i]->plist;
			plist->Cpgroup = plist->pgroup;
			plist->Cangle = plist->angle;
			plist->Cotype = plist->otype;
		}
	}
}


localkary(kc,interact)
struct kcontrol *kc;
int interact;
{
	struct chromosome *obj;
	struct chromplist *plist;
	int i,ac;

	/*
	 * read objects
	 */
	for (i=0; i<kc->maxnumber; i++) {
		ac = kc->acl[i];
		ac &= ACTIVE;
		if (ac != 0) {
			obj = kc->mol[i];	
#ifdef DEBUG
fprintf(stderr,"obj %d active, type %d\n",i,obj->type);
#endif DEBUG
			plist = obj->plist;
			if (obj->type != 1 || plist == NULL) {
				fprintf(stderr,"%d: wrong type or no property list\n",i);
				kc->acl[i] &= ~ACTIVE;
			} else
				initialorientation(i,kc);
		}
	}

	/* check classification and display */
	classify(kc);
	karyogram(kc);
	displayall(kc,NULL);
	if (interact==YES)
		kinteract(kc);
}

localseg(kcont,interact)
struct kcontrol *kcont;
int interact;
{
	/*
	 * interact until "end" command accepted
	 */
	if (interact==YES)
		intwith(kcont);
	kcont->mstatus |= SEGEND;
}
