/*
 * mkcontrol.c
 *
 * Control karyotyping in a multi-process,
 * multi-processor environment
 *
 * Modifications
 *
 *	23 Oct 1989		dcb		Pass initslave the slave process priority
 *	 1 Oct 1987		CAS		kcont a ptr (+ global)
 *	 3 Sep 1987		CAS		New ddgs calls
 *	14 Sep 1986		CAS		Includes
 */

#include <stdio.h>
#include <multip.h>
#include <wstruct.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <files.h>

#define YES = 0
#define NO = 1

struct kcontrol *kcont;


char extensions [][EXTL] = { RAW_S , SEG_S , KAR_S } ;

char *get_to(find,s)
char find;
char *s;
{
	while(*s)
		if (*s++==find)
			return(s)
		;
	;
	return(NULL);
}




main(argc,argv)
char **argv;
{
	struct chromosome *oolist[MAXOBJS], *rolist[MAXOBJS];
	struct chromosome *molist[MAXOBJS], *mpcopyobj();
	struct object *bolist[MAXOBJS];
	SMALL activelist[MAXOBJS];
	struct pframe f, framelist[MAXOBJS];
	register int i, slave, maxsla;
	int interrupt();
	char *nptr;
	char fileroot[25];
	int which_files;
	struct filestate fp[sizeof(extensions)];	
	setmemalloc(16384);
	setpr(getpid(),MPRIOR);
	setbuf(stderr,NULL);
	intercept(interrupt);
	starttimer();

	kcont = (struct kcontrol *) Malloc(sizeof(struct kcontrol));
	initkcontrol(kcont);

	/* ARGUMENT SWITCHES */
	maxsla = 0;
	while (argc > 1) {
		switch (argv[1][0]) {
			case 's': /* max slave processor number */
				maxsla = atoi(&argv[1][1]);
				if (maxsla > NSLAVES)
					maxsla = NSLAVES;
				if (maxsla < 1)
					maxsla = 1;
				maxsla--;
				break;
			case 't': /* set "seat" (terminal) for interaction */
				kcont->seat = atoi(&argv[1][1]);
				if (kcont->seat < 0 || kcont->seat > 1) {
					fprintf(stderr,"illegal terminal %d\n",kcont->seat);
					exit(1);
				}
				if (kcont->seat == 1) { /* Use higher crs display */
					kcont->terminal = "/t1";
				}
				break;
			case 'f':	/* get intput from these files  */
				if ((nptr=get_to('=',&argv[1][1]))!=NULL)
					if (strlen(nptr)<22)
						strcpy(fileroot,nptr);
					else
						{
						fprintf(stderr,"File name too long\n");
						exit(1);
						}
				else
					{
					fprintf(stderr,"Invalid file specifier\n");
					exit(1);
					}
				break;
			}
		argc--;
		argv++;
	}
	ddgsopen(FS_VME512,kcont->seat);

	kcont->ool = oolist;
	kcont->rol = rolist;
	kcont->bol = bolist;
	kcont->mol = molist;
	kcont->f = &f;
	kcont->ivf = framelist;
	kcont->acl = activelist;
	
	for (i=0; i<MAXOBJS; i++) {
		oolist[i] = NULL;
		rolist[i] = NULL;
		bolist[i] = NULL;
		molist[i] = NULL;
		activelist[i] = 0;
	}

	/*
	 * start the slaves, one per processor including in master
	 */
	for (slave=0; slave<=maxsla; slave++)
		initslave(&kcont,slave,slave%NPROCESSORS, SPRIOR);

	/*
	 * start display
	 */	
	dinit();
	lutfrig();
	fast();
	mpclear(&kcont);
	slow();
	/*
	 * check what files we have and the action we should perform
	 *
	 */
	 
	 set_up_list(fileroot,extensions,fp,sizeof(extensions));
	 check_for_files(".",fp,sizeof(extensions));
	 which_files=check_order(fp,sizeof(extensions));
	 
	 switch(which_files)
	 	{
	 	case RAW_B|KAR_B:
	 	case RAW_B:
	 			do_raw(&kcont,fp);
	 			break;
	 	case RAW_B|SEG_B:
	 			do_seg(&kcont,fp);
	 			break;
	 	case RAW_B|SEG_B|KAR_B:
	 			do_kar(&kcont,fp);
	 			break;
		}
}


do_raw(kc,fp,n)
struct kcontrol *kc;
struct filestate *fp;
{
FILE *fptr;
	/*
	 * start feature measurement in slave processors
	 * which can then get on while data is read from disc
	 */
	fmeasstart(kc);

	/* read data from disc (bump up priority for speed) */
	setpr(getpid(),MPRIOR+10);
	fptr=fopen(fp[RAW_P].f_name,fp[RAW_P].f_open);	
	read_raw(fptr,kc);
	fclose(fptr);
	setpr(getpid(),MPRIOR);

	/* execute segmentation in this process */
	segdisplay(kc);
	/* Read classifier params while operator thinks about what to segment. */
	wormcast(kc,MASTER,"mkcrol");
	initclass(kc);
	wormcast(kc,MASTER,"mkcrol");
	seg(kc);
	fprintf(stderr,"SEGMENTATION FINISHED\n");

	/* wait for feature measurement to finish */
	waitfmeas(kc);
	
	/* copy back objects processed on another processor */
	kcopyback(kc,0);
	wormcast(kc,MASTER,"mkcrol after kcopyback");
	fprintf(stderr,"MEASUREMENT FINISHED\n");
	elapsedtime();

	/* classify in this process */
	class(kc);
	elapsedtime();
	wormcast(kc,MASTER,"mkcrol after classifier");

	/* karyotype in this process */
	kary(kc);
	wormcast(kc,MASTER,"mkcrol after karyotype");
	
	/* write the data to file */
	fptr=fopen(fp[KAR_P].f_name,fp[KAR_P].f_open);	
	write_kar(fptr,kc);
	fclose(fptr);

	/* kill the other processes in this and other processors */
	fprintf(stderr,"KCONTROL: free stack %d\n",freemem());
	elapsedtime();
	interrupt(2);
}

do_seg(kc,fp,n)
struct kcrol *kc;
struct filestate *fp;
{
	fprintf(stderr,"LATER\nMuch LATER\n");
}
do_kar(kc,fp,n)
struct kcrol *kc;
struct filestate *fp;
{
	fprintf(stderr,"LATER\n");
}
/*
 *
 *  Check the files that are open and set up appropiate opening flags
 * in the control structure
 *
 */
 
check_order(fp,n)
struct filestate fp[];
int n;
{
int which_files=0;

									/* now go through the list setting bits
									 * in which_file to say files are there
									 */
	if (fp[RAW_P].f_state==FOUND)
		{
		strcpy(fp[RAW_P].f_open,"r");
		which_files |= RAW_B ;
		if (fp[SEG_P].f_state==FOUND)
			{
			strcpy(fp[SEG_P].f_open,"r+");
									/* open for read and write */
			which_files |= SEG_B;
			if (fp[KAR_P].f_state==FOUND)
				{
				strcpy(fp[KAR_P].f_open,"r+");
				which_files |= KAR_B ;
				}
			else
				strcpy(fp[KAR_P].f_open,"w");
			;
			}
		else
			{
			strcpy(fp[SEG_P].f_open,"w");
			strcpy(fp[KAR_P].f_open,"w");
			}
		;
		}
	else
		{
		fprintf(stderr,"Cannot find %s \n",fp[RAW_P].f_name);
		exit(1);
		}
	;
	return(which_files);
}
