/*
 * mkcontrol.c
 *
 * Control karyotyping in a multi-process,
 * multi-processor environment
 *
 * Modifications
 *
 *	18 Jul 1991		ih		Add NULL to displayall call (linebuf not available)
 *	23 Oct 1989		dcb		Pass initslave the slave process priority
 *	 3 sep 1987		CAS		New ddgs calls (open/dev)
 *	14 Sep 1986		CAS		Includes
 */
#include <stdio.h>
#include <multip.h>
#include <wstruct.h>
#include <chromanal.h>
#include <kcontrol.h>

#define YES 1
#define NO 0

struct kcontrol *kcont;

/*   externals that were defined in kary   			*/

/* THESE EXTERNALS WILL BE STUCK IN A STRUCTURE SOON */
struct text *textobj;
struct cellid *cellinformation;
int SHANDON = 0;
int VIDEORECORD = 0;
int aspectr = 0;
int DOWN = 0;
int thereiscellinfo = 0;


main(argc,argv)
char **argv;
{
	struct chromosome *oolist[MAXOBJS], *rolist[MAXOBJS];
	struct chromosome *molist[MAXOBJS], *mpcopyobj();
	struct object *bolist[MAXOBJS];
	SMALL activelist[MAXOBJS];
	struct pframe f, framelist[MAXOBJS];
	register int i, slave, maxsla;
	int interrupt();
	char termit[20];
	int interact=YES;
		
	setmemalloc(16384);
	setpr(getpid(),MPRIOR);
	setbuf(stderr,NULL);
	intercept(interrupt);
	starttimer();
	
	kcont = (struct kcontrol *) Calloc(sizeof(struct kcontrol), 1);
	
	initkcontrol(kcont);

	/* ARGUMENT SWITCHES */
	maxsla = 0;
	while (argc > 1) {
		switch (argv[1][0]) {
			case 'i': /* interaction off flag */
				interact = NO ;
				break;
			case 's': /* max slave processor number */
				maxsla = atoi(&argv[1][1]);
				if (maxsla > NSLAVES)
					maxsla = NSLAVES;
				if (maxsla < 1)
					maxsla = 1;
				maxsla--;
				break;
			case 't': /* set "seat" (terminal) for interaction */
				sprintf(termit,"/t%s",&argv[1][1]);
				kcont->seat = 1 ;
				kcont->terminal = termit;
				fprintf(stderr,"Using terminal %s\n",kcont->terminal);
				break;
			case 'f':	/* Set FIP flag */
				kcont->FIP = atoi(&argv[1][1]);
				break;
		}
		argc--;
		argv++;
	}

	ddgsopen(FS_VME512,kcont->seat);
	kcont->rol = rolist;
	kcont->bol = bolist;
	kcont->mol = molist;
	kcont->f = &f;
	kcont->ivf = framelist;
	kcont->acl = activelist;
	
	for (i=0; i<MAXOBJS; i++) {
		oolist[i] = NULL;
		rolist[i] = NULL;
		bolist[i] = NULL;
		molist[i] = NULL;
		activelist[i] = 0;
	}

	/*
	 * start the slaves, one per processor including in master
	 */
	for (slave=0; slave<=maxsla; slave++)
		initslave(&kcont,slave,slave%NPROCESSORS, SPRIOR);
	/*
	 * start display
	 */	
	dinit();
	lutfrig();
	fast();
	mpclear(&kcont);
	slow();

	/*
	 * start feature measurement in slave processors
	 * which can then get on while data is read from disc
	 */
	fmeasstart(&kcont);

	/* read data from disc (bump up priority for speed) */
	setpr(getpid(),MPRIOR+10);
	readcell(stdin,&kcont);
	setpr(getpid(),MPRIOR);

	/* execute segmentation in this process */
	segdisplay(&kcont);
	/* Read classifier params while operator thinks about what to segment. */
	wormcast(&kcont,MASTER,"mkcontrol");
	initclass(&kcont);
	wormcast(&kcont,MASTER,"mkcontrol");
	localseg(&kcont,interact);
	fprintf(stderr,"SEGMENTATION FINISHED\n");

	/* wait for feature measurement to finish */
	waitfmeas(&kcont);
	
	/* copy back objects processed on another processor */
	kcopyback(&kcont,0);
	wormcast(&kcont,MASTER,"mkcontrol after kcopyback");
	fprintf(stderr,"MEASUREMENT FINISHED\n");
	elapsedtime();

	/* classify in this process */
	class(&kcont);
	elapsedtime();
	wormcast(&kcont,MASTER,"mkcontrol after classifier");

	/* karyotype in this process */
	localkary(&kcont,interact);
	wormcast(&kcont,MASTER,"mkcontrol after karyotype");
	
	/* write the data to file */
	Cplist(&kcont);
	putaway(stdout,&kcont);

	/* kill the other processes in this and other processors */
	fprintf(stderr,"KCONTROL: free stack %d\n",freemem());
	elapsedtime();
	interrupt(2);
}

/*
 * Ensure that the corrected chromosome feature values
 * and classification results are inserted in the property list.
 * Currently only applied to active objects (i.e. chromosomes)
 * but SHOULD be used before writing any object (e.g. split
 * composites etc.) that may be required subsequently.
 */
Cplist(kc)
register struct kcontrol *kc;
{
	register struct chromplist *plist;
	register int i;

	for (i=0; i<kc->maxnumber; i++) {
		if (kc->acl[i] & ACTIVE) {
			plist = kc->mol[i]->plist;
			plist->Cpgroup = plist->pgroup;
			plist->Cangle = plist->angle;
			plist->Cotype = plist->otype;
		}
	}
}


localkary(kc,interact)
struct kcontrol *kc;
int interact;
{
	struct chromosome *obj;
	struct chromplist *plist;
	int i,ac;

	/*
	 * read objects
	 */
	for (i=0; i<kc->maxnumber; i++) {
		ac = kc->acl[i];
		ac &= ACTIVE;
		if (ac != 0) {
			obj = kc->mol[i];	
#ifdef DEBUG
fprintf(stderr,"obj %d active, type %d\n",i,obj->type);
#endif DEBUG
			plist = obj->plist;
			if (obj->type != 1 || plist == NULL) {
				fprintf(stderr,"%d: wrong type or no property list\n",i);
				exit(2);
			}
			initialorientation(i,kc);
		}
	}

	/* check classification and display */
	classify(kc);
	karyogram(kc);
	displayall(kc,NULL);
	if (interact==YES)
		kinteract(kc);
}

localseg(kcont,interact)
struct kcontrol *kcont;
int interact;
{
	/*
	 * interact until "end" command accepted
	 */
	if (interact==YES)
		intwith(kcont);
	kcont->mstatus |= SEGEND;
}
