/*
 *
 *		P A R I S . C   --   Display paris chromosome ideograms
 *
 *	Written: Smaragda Georgiou
 *			 Image Recognition Systems
 *			 720 Birchwood Boulevard
 *			 Birchwood Science Park
 *			 Warrington
 *			 Cheshire
 *
 *	Date:	 Nov. 1, 1988
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *
 *	Modifications :
 *
 *	3 Feb 1992	Joanne		Give filename of ideogram data file a
 *					suffi
 */
#include <stdio.h>
#include <ideogram.h>
#include <ddgs.h>

#define IDEOGRAMDATA "/dd/params/human.col"
#define VALID 1
#define INVALID 0

int 	x, y;
int		band_type;
int		scale;
char	chromosome[3];

/*
 * forward declarations of functions
 */
char *skipword(), *skipband();
struct chrom_data *get_ideogram();

/*
 *	External function definitions
 */
DDGS	*ddgsopen();


main(argc,argv)
int		argc;
char	**argv;
{
	char	*swptr, **targv, **sargv;
	int		targc;
	int		store = FS_VME512, devno = 0;
	char	ideogramdata[100], *spec, *getspec();
	DDGS	*dg;
	               
/*
 *	strcpy(ideogramdata,IDEOGRAMDATA); 
 * 	give filename a suffix so that ideograms of different species
 *	can be used
 */
 	spec = getspec();
sprintf(ideogramdata, "%s.%s", IDEOGRAMDATA, spec);
	targc = argc; targv = argv;
	argc--; argv++;			/* dump process name */
	while (argc > 0) {
		if (*argv[0] == '-') {
			swptr = *argv;
			*argv = NULL;
			while (*(++swptr)) {
				switch (*swptr) {
					case '?':
						printf("Syntax:   Paris <options>\n");
						printf("Function: \n");
						printf("Options:  v - Show version number\n");
						printf("          f - Specify data file\n");
						printf("          1 - Use seat 1 framestore\n");
						printf("          2 - Use seat 2 framestore\n");
						printf("                  also 3 and 4\n");
						printf("          V - Virtuoso framestore (Default CRS 512)\n");
						printf("          ? - Help\n");
						exit(0);
					case 'v':
						printf("Paris - Version 1.0\n");
						exit(0);
					case 'f':
						strcpy(ideogramdata,*++argv);
						*argv = NULL; argc--;
						break;
					case '1':
					case '2':
					case '3':
					case '4':
						devno = *swptr - '1';
						break;
					case 'V':
						store = FS_VIRTUOSO_INVERT;
						break;
					case 'I':
						store = FS_VIRTUOSO;
						break;
					default:
					 	printf("paris - unknown option\n");
				}
			}
		}
		argv++; argc--;
	}
#ifdef PARAMS
/*
 * Compact params to fill holes left by switches
 */
	argc = --targc; argv = ++targv;				/* dump process name */
	sargv = targv;
	while (targc-- > 0) {						/* Squeez + fill the holes */
		if (*targv)
			*sargv++ = *targv;
		else
			argc--;
		targv++;
	}
/*
 *	Use params 
 */
	if (argc) {
		while (argc--) {
			if ((infile = fopen(*argv,"r")) == NULL) {
				fprintf(stderr,"Can't open file %s\n",*argv);
			}
			dofile(infile);
			fclose(infile);
			argv++;
		}
	} else {
		dofile(stdin);
	}
#endif PARAMS
	dg = ddgsopen(store, devno);
	dinit();	
	llut();
	ideogram(ideogramdata);
}

/*
 * I D I O G R A M ---
 */
ideogram (ideogramdata)
char	*ideogramdata;
{ 

	char pre_chrom[3];
		
	FILE *fp, *fopen();		
	int j;
	char answer[2];
	char band[4];
	char x_pos[5],y_pos[5];
	struct chrom_data *chrom_ptr;

  /* open the data file */
    if ((fp = fopen (ideogramdata,"r")) == NULL) {
		fprintf(stdout,"Cannot open file '%s' \n",ideogramdata);
		exit(99);
	}
	pre_chrom[0] = '\0';
  
  /* start an infinite loop */	
    while (1) {
	 /* user requests and validation */

        fprintf (stdout,"Enter chromosome number  : ");
        fscanf (stdin,"%s",chromosome);

        if (check_chrom(chromosome) == VALID) {

    	    fprintf(stdout,"Enter band type : ");
    	    fscanf (stdin,"%d",&band_type);

			if ( check_band(band_type)== VALID) {
				if ( ((strcmp(chromosome,'Y')) || (strcmp(chromosome,'X') ))
					&& (atoi(pre_chrom) > 1) )
				if ( atoi(chromosome) <=  atoi(pre_chrom) )
					 /* go to the beginning of the data file */	
					fseek (fp,0,0);

				chrom_ptr = get_ideogram(fp,chromosome,band_type);
				if (chrom_ptr != NULL){

					fprintf (stdout,"Enter scale : ");
					fscanf (stdin,"%d",&scale);
					
					fprintf (stdout,"Clear the screen ? " );
					fscanf (stdin,"%s",answer);
					if (answer[0] == 'y') {
						clear ();
						x = 800;
						y = 2048;
					}
					else {
						fprintf (stdout,"x_position : ");
						fscanf (stdin,"%d",&x);
						fprintf (stdout,"y_position : ");
						fscanf (stdin,"%d",&y);
						if (x==0)
							x = 800;
						if (y==0) 
							y = 2048;
					}

					draw_ideogram (chrom_ptr,x,y,scale);
					label_ideogram (chrom_ptr,x,y,scale);
					free_ideogram(chrom_ptr);
				}
			}
		}
		strcpy (pre_chrom,chromosome);
    }
    fclose (fp);
}

/*
 *	C H E C K _ C H R O M ---
 */
 check_chrom (chrom)
 char chrom[5];
 {
  /* transform lower case entries of "x" or "y" into upper, the only form
   * existing in the database */   
	if ( strcmp(chrom,"x")==0 )
		strcpy (chrom,"X\0");
	if	( strcmp(chrom,"y")==0) 
		strcpy (chrom,"Y\0");

 	if (((atoi(chrom) >= 1) && (atoi(chrom) <= 22))
 		||(strcmp(chrom,"X")== 0) || (strcmp(chrom,"Y")==0) )
 		return (VALID);
 	else {
 		fprintf(stdout,"Please re-enter; \n");
 		return (INVALID);
 	}
 }
 
/*
 *	C H E C K _ B A N D ---
 */
check_band (band)
int band;
{
	if ( (band == P900) || (band == P600) || (band == P300)) 
		return (VALID);
	else {
		fprintf(stdout,"Please re-enter \n");
		return (INVALID);
	}
}
