/*
 * slave.c		1/11/85		Jim Piper
 *
 ************************************************
 * SLAVE PROCESS RUNNING IN MASTER OR SLAVE		*
 * PROCESSOR PROVIDING SERVICES TO MASTER		*
 *												*
 * In the master processor, this program is     *
 * forked by "mkcontrol" with an argument.		*
 * In a slave processor,						*
 * this program is kicked from "slaveshell".    *
 * It obtains the general control structure,    *
 * defined in "../h/kcontrol.h", which it then  *
 * proceeds to process.                         *
 ************************************************
 */
/*	Modifications:
 *
 *	 4 Dec 1988		CAS		Added a wee bit more debug
 *	02 dec 1988		dcb		Added new command SLAVE_DEBUG, which turns on
 *							as much debugging as possible to stderr channel
 *	11 Feb 1988		CAS		use proc num not slave num for convaddr
 *	23 Sep 1987		CAS		Integrate jimp's mods (17/3, 30/3, 31/3)
 *	 4 Sep 1987		CAS		Realise that convaddr is needed...
 *	 2 Sep 1987		CAS		Tidy + sort ddgs open/dev calls
 *	19 Aug 1987		BDP		Took out a CRS specific crsaddr call for seat 2
 **	31 Mar 1987		jimp	More karyotype server functions (ENLARGE, STR_POL)
 **	30 Mar 1987		jimp	Another karyotype server function (ROTATE)
 **	17 Mar 1987		jimp	New karyotype server functions
 *
 */

#include <stdio.h>
#include <wstruct.h>
#include <kcontrol.h>
#include <mp.h>
main(argc,argv)
char **argv;
{
	struct kcontrol *kcont;
	int		slave, processor, com, *c_addr, *c_proc;
	DDGS	*mddgschannel;

	setmemalloc(12800);
	setbuf(stderr,NULL);
			
	/*
	 * set up control structure
	 */
	if (argc < 3) {
		fprintf(stderr,"SLAVE: to few args\n");
		exit(51);
	}
	kcont = (struct kcontrol *) atoi(argv[1]);
	slave = atoi(argv[2]);
#ifdef DEBUG
	fprintf(stderr,"SLAVE %d forked, kcontrol 0x%x seat %d\n",
		slave,kcont,kcont->seat);
#endif DEBUG

/*
 *	set up display stuff
 */
 	processor = kcont->processor[slave] >> P_OR;
	mddgschannel = (DDGS *)convaddr(kcont->ddgschannel,MASTER,processor);
	ddgsopen(mddgschannel->type,mddgschannel->devno);
		
	setup();

	/*
	 * loop, executing slave commands
	 */
	for (;;) {
		com = kcont->command[slave];
#ifdef DEBUG
		if (com)
			fprintf(stderr,"Slave recd command %d\n",com);
#endif DEBUG
		switch (com) {
			case 0:
				break;
	
			default:
				kcont->command[slave] = 0;	/* acceptance */
				break;
	
			case SCREEN_CLEAR:
				kcont->command[slave] = 0;	/* acceptance */
				sl_clear(slave,kcont);
				break;

			case INDV_DISPLAY_LIST:
				kcont->command[slave] = 0;	/* acceptance */
				sl_ipic(slave,kcont);
				break;

			case COM_DISPLAY_LIST:
				kcont->command[slave] = 0;	/* acceptance */
				sl_cpic(slave,kcont);
				break;

			case CHROM_FEATURE_MEASURE:
				kcont->command[slave] = 0;	/* acceptance */
				kmeas(slave,kcont);
				break;

			case CHROM_LIKL_MEASURE:
				kcont->command[slave] = 0;	/* acceptance */
				sl_likl(slave,kcont);
				break;
			case K_STRAIGHTEN:
			case K_ENHANCE:
			case K_NORMALISE:
			case K_ROTATE:
			case K_ENLARGE:
			case K_STR_POL:
				kcont->command[slave] = 0;	/* acceptance */
				sl_k_serv(slave,kcont,com);
				break;
			case SLAVE_DEBUG: /* Turn on debugging for slave */
				kcont->command[slave] = 0;	/* acceptance */
				sl_debug(slave, kcont);
				break;
		}
		tsleep(10);	/* wait for next command */
	}
}
