/*
 *      I D D G S L I N . C  --  Line stuff
 *
 *
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *	Based on DDGS
 *			 Jim Piper
 *			 MRC CAPCU Edinburgh
 *			 9th April 1985
 *
 * Device dependent graphics system with approximately same 
 * functionality as DIGS.
 * Window is assumed to be 4096 square so there is scaling everywhere.
 *
 *  Modifications
 *  -------------
 *	 1 Feb 1991	jimp	Modified to write into large store buffer
 *	13 Jun 1990		CAS		2.3 compiler needs void if return with no param
 */
#include <stdio.h>
#include <ddgs.h>
#define EXTERN extern
#include <iddgsdefs.h>

/*static int WIDTH = 2; /*GJP*/
static int WIDTH = 1;
void	d_ddgsline(); 

/*
 *	L I N E B Y  --
 *
 */
lineby(x,y)
register int x,y;
{
	d_ddgsline(vstatus.curx>>XSHIFT,vstatus.cury>>YSHIFT,
		(vstatus.curx+x)>>XSHIFT,(vstatus.cury+y)>>YSHIFT);
	moveby(x,y);
}

/*
 *	L I N E T O  --
 *
 */
lineto(x,y)
register int x,y;
{
	d_ddgsline(vstatus.curx>>XSHIFT,vstatus.cury>>YSHIFT,
	(x + CENTERXOFFSET) >>XSHIFT,
	(y + CENTERYOFFSET) >>YSHIFT);
	moveto(x,y);
}

/*
 *	D _ D D G S L I N E  --
 *
 * line-drawing routine.
 * has a very primitive (but cheap) variable width capability
 */
void d_ddgsline(x0, y0, x1, y1)
register int x0, y0, x1;
int y1;
{
	register int i;
	PIXEL *a;
	int lineln;
	int res1, minx, maxx, miny, maxy, dx, dy;
	int xinc, yinc, dysigned, h1,h2,res2;

	h2 = WIDTH>>1;
	h1 = WIDTH - 1 - h2;
	/* check if horizontal line */
	if (y1 == y0) {
		y0 -= h1;
		y1 += h2;
		if (x1 < x0) {
			i=x1; x1=x0; x0=i;
		}
		d_ddgsrect(x0,y0,x1,y1);
		return;
	}
	/* check if vertical line */
	if ((dx = x1-x0) == 0) {
		x0 -= h1;
		x1 += h2;
		if (y1 < y0) {
			i=y1; y1=y0; y0=i;
		}
		d_ddgsrect(x0,y0,x1,y1);
		return;
	}
	/* compute direction and increments, ensure x0 <= x1 */
	xinc = 1;
	yinc = 1;
	if (dx < 0) {
		dx = -dx;
		i = x0; x0 = x1; x1 = i;
		i = y0; y0 = y1; y1 = i;
	}
	dysigned = dy = y1-y0;
	if (dy < 0) {
		yinc = -1;
		dy = -dy;
	}
	
	/*
	 * compute bounds of picture, taking line width into account
	 * and modify end points accordingly
	 */
	minx = vstatus.WINDMINX+h2; maxx = vstatus.WINDMAXX-h2;
	miny = vstatus.WINDMINY+h2; maxy = vstatus.WINDMAXY-h2;
	if (x1 < minx || x0 > maxx)
		return;
	if (yinc > 0 && (y1 < miny || y0 > maxy))
		return;
	if (yinc < 0 && (y1 > maxy || y0 < miny))
		return;
	if (x0 < minx) {
		if (dx != 0)
			y0 += (dysigned*(minx-x0)/dx);
		x0 = minx;
	}
	if (x1 > maxx) {
		if (dx != 0)
			y1 += (dysigned*(maxx-x1)/dx);
		x1 = maxx;
	}
	if (y0 < miny) {
		if (dy != 0)
			x0 += (dx*(miny-y0)/dysigned);
		y0 = miny;
	} else if (y0 > maxy) {
		if (dy != 0)
			x0 += (dx*(maxy-y0)/dysigned);
		y0 = maxy;
	}
	if (y1 > maxy) {
		if (dy != 0)
			x1 += (dx*(maxy-y1)/dysigned);
		y1 = maxy;
	} else if (y1 < miny) {
		if (dy != 0)
			x1 += (dx*(miny-y1)/dysigned);
		y1 = miny;
	}
	
	res1 = 0;
	res2 = 0;
	/*
	 * Crude attempt at "lwidth" function.
	 * This is implemented by drawing a 
	 * horizontal or vertical line, length "lwidth", at
	 * each point of the line segment.  This should produce
	 * a correctly filled line segment, but width is wrong unless
	 * it is a vertical or horizontal line.
	 * For better version, see "rasterp.c" on Vax
	 */
	lineln = dgb->lineln;
			if (dx >= dy) {
				while (x0 != x1) {
					ADDRESS(a,x0,y0-h1);
					for (i = WIDTH; i>0; i--,a-=lineln)
						POINT(a, vstatus.colourbits);
					res1 += dy;
					x0 += xinc;
					if (res1 > res2) {
						res2 += dx - res1;
						res1 = 0;
						y0 += yinc;
					}
				}
				ADDRESS(a,x1,y1-h1);
				for (i = WIDTH; i>0; i--,a-=lineln)
					POINT(a, vstatus.colourbits);
			} else {
				while (y0 != y1) {
					ADDRESS(a,x0-h1,y0);
					for (i = WIDTH; i>0; i--,a++)
						POINT(a, vstatus.colourbits);
					res1 += dx;
					y0 += yinc;
					if (res1 > res2) {
						res2 += dy - res1;
						res1 = 0;
						x0 += xinc;
					}
				}
				ADDRESS(a,x1-h1,y1);
				for (i = WIDTH; i>0; i--,a++)
					POINT(a, vstatus.colourbits);
			}
}

/*
 *	L W I D T H  --
 *
 */
lwidth(w)
{
	if (w>0 && w<16) {
#ifdef GJP
		if (w<2)
			WIDTH=2;
		else
#endif GJP
			WIDTH = w;
	}
}

/*
 *	D _ D D G S R E C T  --
 *
 * Fill a rectangle from (x0,y0) to (x1,y1),
 * with x0<=x1 and y0<=y1 guaranteed.
 */
d_ddgsrect(x0,y0,x1,y1)
register int x0,x1,y0;
int	y1;
{
	register int colour,otherplanes;
	PIXEL *a, *atop;
	register	int	lineln;
	colour = vstatus.colourbits;
	otherplanes = vstatus.otherplanes;
	if (x0 < vstatus.WINDMINX) {
		x0 = vstatus.WINDMINX;
		if (x1 < vstatus.WINDMINX)
			x1 = vstatus.WINDMINX;
	}
	if (x1 > vstatus.WINDMAXX) {
		x1 = vstatus.WINDMAXX;
		if (x0 > vstatus.WINDMAXX)
			x0 = vstatus.WINDMAXX;
	}
	if (y0 < vstatus.WINDMINY) {
		y0 = vstatus.WINDMINY;
		if (y1 < vstatus.WINDMINY)
			y1 = vstatus.WINDMINY;
	}
	if (y1 > vstatus.WINDMAXY) {
		y1 = vstatus.WINDMAXY;
		if (y0 > vstatus.WINDMAXY)
			y0 = vstatus.WINDMAXY;
	}
/*
 * if vertical line width 1, use special code for speed,
 * including using register variables .....
 */
 	lineln = dgb->lineln;
			if (x0 == x1) {
				ADDRESS(a,x0,y0);
				atop = a - (y1-y0)*lineln;
				if (colour == 0)
					for (; a >= atop; a-=lineln)
						CLEARPOINT2(a,otherplanes);
				else
					for (; a >= atop; a-=lineln)
						POINT3(a,otherplanes,colour);
			} else {
				for (; y0 <= y1; y0++) {
					ADDRESS(a,x0,y0);
					atop = a + (x1-x0);
					if (colour == 0)
						for (; a<=atop; a++)
							CLEARPOINT2(a,otherplanes);
					else
						for (; a<=atop; a++)
							POINT3(a,otherplanes,colour);
				}
			}
}
