/*
 *      I D D G S P I X . C  --  Pixel plotting stuff
 *
 *
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *	Based on DDGS
 *			 Jim Piper
 *			 MRC CAPCU Edinburgh
 *			 9th April 1985
 *
 * Device dependent graphics system with approximately same 
 * functionality as DIGS.
 * Window is assumed to be 4096 square so there is scaling everywhere.
 *
 *  Modifications
 *  -------------
 *	 1 Feb 1991	jimp	Modified to write into large store buffer
 *  30 Jan 1989		SEAJ	If PBSIZE not big enough than allocate a bigger buffer.
 *	 8 Aug 1988		CAS		pixelmode - return old mode
 *
 */
#include <stdio.h>
#include <machdef.h>
#include <ddgs.h>
#define EXTERN extern
#include <iddgsdefs.h>

#define PBSIZE 128

static GREY pbuf[PBSIZE];
static int pixlines = 0;
static int pixcols = 0;
static int mpixels = 0;
static int npixels = 0;
static int pixy = 0;
static int pixytop = 0;
static int pixtotal = 0;
static int pixxscale = 1;
static int pixyscale = 1;
static int pixxinc = 0;
static int pixyinc = 0;
static int PIXELMODE = 1;


/*
 *	P I X B E G I N --
 *
 */
pixbegin(bpp,ppl,lpp,xsize,ysize)
{
	pixlines = lpp;
	pixcols = ppl;
	pixxscale = xsize;
	pixyscale = ysize;
	pixxinc = (pixxscale-1)>>XSHIFT;
	if (pixxinc < 0)
		pixxinc = 0;
	pixyinc = (pixyscale-1)>>YSHIFT;
	if (pixyinc < 0)
		pixyinc = 0;
	mpixels = 0;
	npixels = 0;
	pixtotal = pixlines * pixcols;
	pixy = vstatus.cury>>YSHIFT;
	pixytop = pixy + pixyinc;
	if (pixy < vstatus.WINDMINY)
		pixy = vstatus.WINDMINY;
	if (pixytop > vstatus.WINDMAXY)
		pixytop = vstatus.WINDMAXY;
}

/*
 *	P I X E L --
 *
 */
pixel(p)
register int p;
{
	PIXEL *a,*b;
	register int xtop,x,j;
	int v;
	if (++npixels > pixtotal)
		fprintf(stderr,"Too many pixels\n");

	if (vstatus.intensity == 0)
		p = 0;
	else
		p &=vstatus.colourbits;
	x = vstatus.curx>>XSHIFT;
	xtop = x + pixxinc;
	if (x < vstatus.WINDMINX)
		x = vstatus.WINDMINX;
	if (xtop > vstatus.WINDMAXX)
		xtop = vstatus.WINDMAXX;
			ADDRESS(a,x,pixy);
			xtop -= x;
			for (j=pixy; j<=pixytop; j++) {
				b = a;
				switch (PIXELMODE) {
					default:
					case 1:
						for (x=0; x<=xtop; x++)
							*b++ = p;
						break;
					case 2:
						for (x=0; x<=xtop; x++) {
							v = *b & vstatus.otherplanes;
							*b++ = v | p;
						}
						break;
					case 3:
						for (x=0; x<=xtop; x++) {
							if ((*b & vstatus.otherplanes) == 0)
								*b = p;
							b++;
						}
						break;
				}
				a -= dgb->lineln;
			}
	vstatus.curx += pixxscale;
	if (++mpixels == pixcols && npixels < pixtotal) {
		mpixels = 0;
		moveby(-pixcols*pixxscale, pixyscale);
		pixy = vstatus.cury>>YSHIFT;
		pixytop = pixy + pixyinc;
		if (pixy < vstatus.WINDMINY)
			pixy = vstatus.WINDMINY;
		if (pixytop > vstatus.WINDMAXY)
			pixytop = vstatus.WINDMAXY;
	}
}

/*
 *	P I X E N D  --
 *
 */
pixend()
{
	if (npixels < pixtotal)
		fprintf(stderr,"Too few pixels\n");
}

/*
 *	P I X L I N E  --
 *
 * Draw pixels in a horizontal line direct from a pixel buffer.
 * If xsize is 8 then this can be an interval in a grey-table.
 * If xsize is not 8 then convert to xscale 8 by filling a local
 * buffer and calling pixline recursively.  If ysize is more
 * than 8 then iteratively call pixline recursively with
 * ysize 8 and appropriate origin movement.  If ysize is less than
 * 8 only display a line if its screen position is different from
 * a line ysize DIGS units further up, i.e. suppress multiple
 * drawing of the same screen line (this may occasionally lead
 * to a single line Woolz object not being displayed).
 *
 *	Note register union doesn't work - so the code will be slower
 */
pixline(ppl,xsize,ysize,g,dir)
register GREY *g;
register int ppl;
{
	PIXEL *a, *atop;
	register int x,y,colourbits;
	static GREY *pbt, *pbts;
	int np, npt, v, curxwas, curywas, bigbuf = 0;

	if (ysize < 8 && (vstatus.cury>>YSHIFT) == ((vstatus.cury+ysize)>>YSHIFT))
		return;
	if (xsize != 8) {
		/*
		 * Fill buffer
		 */
		np = (ppl*xsize+5)/8;
		y = 0;
		curxwas = vstatus.curx;
		while (np > 0) {
			npt = np;
			if (npt > PBSIZE) {
				pbts = pbt = (GREY *) Calloc(npt, sizeof(GREY));
				bigbuf = 1;
			} else
				pbts = pbt = pbuf;
			for (x=npt; x>0; x--) {
				*pbt++ = *g;
				y += 8;
				while (y > xsize) {
					y -= xsize;
					g++;
				}
			}
			/*
			 * Call pixline recursively
			 */
			pixline(npt,8,ysize,pbts,dir);
			vstatus.curx += 8*npt;
			np -= npt;
		}
		vstatus.curx = curxwas;
		if (bigbuf) {
			Free(pbts);
			bigbuf = 0;
		}
	}
	else if (ysize > 8) {
		/*
		 * Iterate recursive calls to pixline
		 */
		curywas = vstatus.cury;
		for (; ysize>0; ysize-=8) {
			pixline(ppl,8,8,g,dir);
			vstatus.cury += 8;
		}
		vstatus.cury = curywas;
	} else {
		colourbits = vstatus.colourbits;
		ppl--;
		x = vstatus.curx>>XSHIFT;
		y = vstatus.cury>>YSHIFT;
		/* check bounds */
		if (y < vstatus.WINDMINY || y > vstatus.WINDMAXY)
			return;
				if (dir > 0) {
					if (x < vstatus.WINDMINX) {
						g += (vstatus.WINDMINX-x);
						ppl -= (vstatus.WINDMINX-x);
						x = vstatus.WINDMINX;
					}
					if (x+ppl > vstatus.WINDMAXX)
						ppl -= (x+ppl-vstatus.WINDMAXX);
					ADDRESS(a,x,y);
					atop = a + ppl;
					if (colourbits != 0) {
						switch (PIXELMODE) {
							default:
							case 1:
								while (a <= atop)
									*a++ = *g++ & colourbits;
								break;
							case 2:
								while (a <= atop) {
									v = *a & vstatus.otherplanes;
									*a++ = v | (*g++ & colourbits);
								}
								break;
							case 3:
								while (a <= atop) {
									if ((*a & vstatus.otherplanes) == 0)
										*a = *g & colourbits;
									a++;
									g++;
								}
								break;
						}
					} else {
						switch (PIXELMODE) {
							default:
							case 1:
								while (a <= atop)
									*a++ = 0;
								break;
							case 2:
								while (a <= atop)
									*a++ &= vstatus.otherplanes;
								break;
							case 3:
								while (a <= atop) {
									if ((*a & vstatus.otherplanes) == 0)
										*a = 0;
									a++;
								}
								break;
						}
					}
				} else {
					if (x >vstatus.WINDMAXX) {
						g += (x-vstatus.WINDMAXX);
						ppl -= (x-vstatus.WINDMAXX);
						x = vstatus.WINDMAXX;
					}
					if (x-ppl < vstatus.WINDMINX)
						ppl -= (vstatus.WINDMINX-x+ppl);
					ADDRESS(a,x,y);
					atop = a - ppl;
					if (colourbits != 0) {
						switch (PIXELMODE) {
							default:
							case 1:
								while (a >= atop)
									*a-- = *g++ & colourbits;
								break;
							case 2:
								while (a >= atop) {
									v = *a & vstatus.otherplanes;
									*a-- = v | (*g++ & colourbits);
								}
								break;
							case 3:
								while (a >= atop) {
									if ((*a & vstatus.otherplanes) == 0)
										*a = *g & colourbits;
									a--;
									g++;
								}
								break;
						}
					} else {
						switch (PIXELMODE) {
							default:
							case 1:
								while (a >= atop)
									*a-- = 0;
								break;
							case 2:
								while (a >= atop)
									*a-- &= vstatus.otherplanes;
								break;
							case 3:
								while (a >= atop) {
									if ((*a & vstatus.otherplanes) == 0)
										*a = 0;
									a--;
								}
								break;
						}
					}
				}
	}
}


/*
 *	P I X E L M O D E  --
 *
 * set the pixel-drawing mode as follows (assuming ~INVERT mode):
 *
 * (1) Value inserted in all 8 planes = greyvalue & vstatus.colourbits
 * (2) Value inserted in selected planes only = greyvalue & vstatus.colourbits
 * (3) Greyvalue & vstatus.colourbits inserted ONLY IF existing
 *			value&~vstatus.colourbits == 0
 */
pixelmode(m)
{
	int		old;
	old = PIXELMODE;
	PIXELMODE = m;
	return(old);
}
