/*
 * ident.c		Jim Piper		21-8-85
 *
 * Interactively identify displayed object by pointing, and
 * return a command.  Use the objectlist, marklist, activelist
 * type of data structure used in multi-processor karyotyping.
 *
 *	Modification History
 *	--------------------
 *	05 Mar 1990		dcb		Remove overlay definitions, and include display.h
 *	12 Nov 1986		jimp	Use <display.h> for symbolic colours
 *	11 Sep 1986		CAS		Includes
 *	24/12/85 jimp			take out device-dependent stuff (i.e. mouse)
 *							and insert elsewhere (kycount.c in this case)
 *	17/9/85	denis/mark		dont close mouse channel in end-ident
 */

#include <stdio.h>
#ifndef VAX
#include <wstruct.h>
#include <interact.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <display.h>
#else VAX
#include "/mnt1/woolz/mizar/WOOLZ/h/wstruct.h"
#include "/mnt1/woolz/mizar/WOOLZ/h/interact.h"
#include "/mnt1/woolz/mizar/WOOLZ/h/chromanal.h"
#include "/mnt1/woolz/mizar/WOOLZ/h/kcontrol.h"
#endif

#define ABS(x) (x>=0?x:-x)

static int current;

/*
 * start this identify/command session
 */
initident(kcont,cursor)
struct kcontrol *kcont;
struct ipoint *cursor;
{
	current = -1;
	cursor->type = 40;
	cursor->style = 6;
	cursor->k = kcont->f->ox;
	cursor->l = kcont->f->oy;
	colour(OVERLAY1);
	intens(1);
	picframe(cursor,kcont->f);
}

/*
 * end this identify/command session
 */
endident(kcont,cursor)
struct kcontrol *kcont;
struct ipoint *cursor;
{
	intens(0);
	picframe(cursor,kcont->f);
	markident(current,0,kcont);
}

/*
 * identify the next object
 */
objident(kcont,cursor)
struct kcontrol *kcont;
struct ipoint *cursor;
{
	register int new;
	new = locate(kcont,cursor);
	if (new != current) {
		markident(current,0,kcont);
		current = new;
		markident(current,1,kcont);
	}
	return(new);
}


/*
 * display rectangle round identified object.
 * display object type and count (if composite) at bottom of screen.
 */
markident(n,onoff,kcont)
register struct kcontrol *kcont;
{
	register struct chromosome *obj;
	if (n>=0) {
		obj = kcont->ool[n];
		intens(onoff);
		colour(OVERLAY1);
		disprect(obj,kcont->f);
		typedisp(obj);
	}
}

static char *tag[7] = {
	"UNKNOWN",
	"CHROM",
	"COMPO",
	"OVLAP",
	"NOISE",
	"NUCLEUS",
	"PIECE"
};

/*
 * display object type and composite count at bottom right of screen
 */
typedisp(obj)
struct chromosome *obj;
{
	struct chromplist *plist;
	plist = obj->plist;
	tsize(130,150);
	dispstring(tag[plist->otype],3000,32);
	if (plist->otype == 2 || plist->otype == 3)
		dispnum(plist->history[2],3800,32);
}

/*
 * find if (curs->k,curs->l) uniquely identifies an object.
 * If so, return object serial number, otherwise -1.
 * Assumption: objects DO NOT OVERLAP.
 *
 * Technique: first see if point lies within right rectangle
 * surrounding any object.  Second, for each such object, see
 * which rectangle is smallest.  Third, if ambiguous, see which
 * is nearest centre.
 */
locate(kcont,curs)
struct kcontrol *kcont;
register struct ipoint *curs;
{
	register int i,j,k,l;
	int dk,dl,d,k1,kl,l1,ll,hit,area,minarea,mindist;
	register struct intervaldomain *idom;
	
	minarea = 10000000;
	mindist = 1000;
	k = curs->k;
	l = curs->l;
	hit = -1;
	for (i=0; i<kcont->maxnumber; i++) {
		if (kcont->acl[i] != 0) {
			idom = kcont->ool[i]->idom;
			k1 = idom->kol1;
			kl = idom->lastkl;
			if (kl-k1 < 8) {
				kl += 4;
				k1 -= 4;
			}
			l1 = idom->line1;
			ll = idom->lastln;
			if (ll-l1 < 8) {
				ll += 4;
				l1 -= 4;
			}
			if (k>=k1 && k<=kl && l>=l1 && l<=ll) {
				area = (kl-k1) * (ll-l1);
				dk = 2*k-k1-kl;
				dl = 2*l-l1-ll;
				d = ABS(dk)+ABS(dl);
				if (area < minarea || (area==minarea && d<mindist)) {
					hit = i;
					minarea = area;
					mindist = d;
				}
			}
		}
	}
	return(hit);
}
