/*
 *	K A R Y S U B S 1 . C  --  
 *
 * All routines in this (rather large) module refer to one
 * or other of the static data structures defined below.
 *
 *  Written: Jim Piper
 *           Medical Research Council
 *           Clinical and Population Cytogenetics Unit
 *           Edinburgh
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *  Date:    23rd January 1984
 *
 *	Modifications
 *
 *	 7 Feb 1991		CAS			remove Mizar defs -> osk, voids
 *	 2 Dec 1988		CAS			displayall, kdisplay, unkdisplay, remove_class
 *								to subs2 in an effort to reduce duplication
 *	 2 Dec 1988		CAS/SEAJ	Moved newclass to karysubs2 + changed params
 *								to firstclass/newclass
 *	 3 Oct 1988		CAS/GJP		Set up frame type
 *	 9 Feb 1987		Jimp		Remove "NSTACKPOS" to chromanal.h
 *	28 Oct 1986		Jimp		All active objects passed to "firstclass"
 *	17 Oct 1986		Jimp		display.h used
 *	11 Sep 1986		CAs			New jimp aalborg version - includes changed
 *	20 Aug 1986		Jimp		"ndentify()" added
 *
 */

#include <stdio.h>
#include <wstruct.h>
#include <interact.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <display.h>

#define NORMALISE	02
#define TEMPHIGHLIGHT	04

extern struct text *textobj;
extern struct cellid *cellinformation;
extern int SHANDON;
extern int VIDEORECORD;
extern int aspectr;
extern int DOWN;
extern int thereiscellinfo;


struct kclasscont kcc[MAXCLASS];

/*
 * structure which describes placement of chromosomes on screen
 * one entry per class.  the parameters are
 *	- DIGS column number of center of area
 *	- DIGS line number of base of area
 *	- default number of positions for class members
 *	- current number of positions for class members
 *	- DIGS width available
 *	- character string which names this class
 */
struct kdisppos {
	int col;
	int lin;
	int defaultpos;
	int npos;
	int space;
	char *tag;
} disppos[MAXCLASS] = {
	{ 400,3000,2,2,800,"1"},
	{1150,3000,2,2,800,"2"},
	{1900,3000,2,2,800,"3"},
	{2850,3000,2,2,800,"4"},
	{3600,3000,2,2,800,"5"},
	{ 350,2200,2,2,550,"6"},
	{ 900,2200,2,2,550,"7"},
	{1450,2200,2,2,550,"8"},
	{2000,2200,2,2,550,"9"},
	{2550,2200,2,2,550,"10"},
	{3100,2200,2,2,550,"11"},
	{3650,2200,2,2,550,"12"},
	{ 350,1600,2,2,500,"13"},
	{ 850,1600,2,2,500,"14"},
	{1350,1600,2,2,500,"15"},
	{2100,1600,2,2,500,"16"},
	{2600,1600,2,2,500,"17"},
	{3100,1600,2,2,500,"18"},
	{ 350,1150,2,2,500,"19"},
	{ 850,1150,2,2,500,"20"},
	{1700,1150,2,2,500,"21"},
	{2200,1150,2,2,500,"22"},
	{3650,1150,2,2,550,"X"},
	{3100,1150,2,2,550,"Y"},
	{-1,-1,0,0,600,"A"}
};

/*
 * The bottom row is a 24-position stack, filling from the left.
 * The first 12 locations are reasonably well spaced, but then things
 * get a bit tighter.
 * We need a stack occupancy and a column-origin table.
 */
int stackcol[NSTACKPOS+1] = {-1,
	 250, 500, 750,1000,1250,1500,
	1750,2000,2250,2500,2750,3000,
	3100,3200,3300,3400,3500,3600,
	3650,3700,3750,3800,3850,3900
};
#define STACKLIN	200

int kscale = 8;


/*
 * construct initial karyogram from classified objects
 */
karyogram(kcont)
struct kcontrol *kcont;
{
	int i, j, stack;
	struct chromplist *plist;
	struct kclasscont *kccp;
	/*
	 * clear various things
	 */
	kccp = kcc;
	for (i=0; i<MAXCLASS; i++) {
		disppos[i].npos = disppos[i].defaultpos;
		kccp->n = kccp->maxn = 0;
		for (j=0; j<MAXCHILD; j++)
			kccp->kno[j] = -1;
		kccp++;
	}
	for (i=0; i<=NSTACKPOS; i++)
		unstack(i);

	/*
	 * classified objects placed in notional positions
	 * i.e. not actually displayed yet (see "displayall")
	 */
	for (i=0; i<kcont->maxnumber; i++) {
		if ((ACTIVE & kcont->acl[i]) != 0) {
			firstclass(i,kcont,disppos);
		}
	}
}


/*
 * tidy up screen.  In particular, move remaining abnormals to left
 * of screen and put trisomies in correct group positions.
 */
tidyscreen(kcont)
struct kcontrol *kcont;
{
	register int i,j,l;
	struct kident k;
	register struct kclasscont *kc = kcc;
	struct kdisppos *dp = disppos;
	for (i=0; i<UNDEFCLASS; i++) {
		if (kc->n > dp->npos) {
			k.kclass = i;
			/*
			 * delete class contents from present positions
			 */
			for (j=0; j<kc->maxn; j++) {
				k.pos = j+1;
				k.no = kc->kno[j];
				if (k.no >= 0) {
					k.stackpos = kc->sp[j];
					unkdisplay(&k,kcont);
					kc->sp[j] = -1;
				}
			}
			/*
			 * pack down class contents structure
			 * at end, kc->n and kc->maxn should be same.
			 */
			for (j=0; j<kc->maxn; j++) {
				if (kc->kno[j] < 0) {
					kc->maxn--;
					for (l=j; l<kc->maxn; l++) {
						kc->kno[l] = kc->kno[l+1];
					}
				}
			}
			/*
			 * increment the number permitted in class display
			 * and display the class contents
			 */
			dp->npos = kc->n;
			for (j=0; j<kc->n; j++) {
				k.pos = j+1;
				k.no = kc->kno[j];
				k.stackpos = kc->sp[j];
				kdisplay(&k,kcont);
			}
		}
		kc++;
		dp++;
	}
	disptags();
}

karyscale(s)
{
	kscale = s;
}



kdsetup(k,kcont)
struct kident *k;
struct kcontrol *kcont;
{
	struct chromosome *obj, *newgrey();
	int normit,i,l;
	char tag[10];
	struct pframe *f = kcont->ivf+k->no;

	/*
	 * set the picture frame
	 */
	setframe(f,k,kcont);

	/*
	 * construct and display the object's tag,
	 * but only if in the stack area.
	 */
	if (k->pos > disppos[k->kclass].npos) {
		l = strlen(disppos[k->kclass].tag);
		for (i=0; i<l; i++)
			tag[i] = disppos[k->kclass].tag[i];
		tag[i++] = 'a' + k->pos - 1;
		tag[i] = '\0';
		colour(OVERLAY1);
		dispstring(tag,f->dx,f->dy-96);
	}

	/*
	 * normalise if required - can't imagine why we want to do this.
	 */
	if (kcont->mol&&(obj=kcont->mol[k->no])) {
		normit = obj->plist->dispopts & NORMALISE;
		if (normit)
			normalise(obj);
		/*
	 	 * temporary high-light
	 	 */
		if (obj->plist->dispopts & TEMPHIGHLIGHT) {
			obj->plist->dispopts &= ~TEMPHIGHLIGHT;
			colour(OVERLAY1);
			bdisprect(obj,f);
		}

		/*
		 * mark the object if so specified
		 */
		if (obj->plist->dispmark != 0)
			dispmark(f);
	}
}


/*
 * set the display frame "f" to suitable for display of chromosome
 * described in identification structure "k".  The following are
 * relevant:
 *	- Chromosome class
 *	- ordinal within the class, and whether the "disppos" structure
 *	  currently expects this many chromosomes.  If not, then a
 *	  "stack" location is provided.
 *	- whether chromosome should be inverted
 *	- modifications for aspect ratio and video recording
 */
setframe(f,k,kcont)
register struct pframe *f;
register struct kident *k;
struct kcontrol *kcont;
{
	struct chromosome *obj;
	register struct intervaldomain *idom;
	register struct kdisppos *dispp;
	int width;

	if (kcont->mol&&(obj=kcont->mol[k->no])) {
		f->type = 60;
		idom = obj->idom;
		width = idom->lastkl - idom->kol1;
		dispp = disppos + k->kclass;
		f->scale = kscale;
		f->ox = (idom->kol1 + idom->lastkl)/2;
		f->dy = dispp->lin - DOWN;
		if (f->dy < 100)
			f->dy = 100;

	/*
 	 * 3 cases to be considered:
	 *	1)	k->pos > dispp->npos
	 *	2)	dispp->npos == 2    &&    k->pos <= 2
	 *	3)	k->pos <= dispp->npos > 2
	 */
		if (k->pos > dispp->npos) {
			f->dx = stackcol[k->stackpos];
			f->dy = STACKLIN;
		} else if (dispp->npos == 2) {
			switch (k->pos) {
			case 1:
				f->dx = dispp->col - 24 - kscale*width/2;
				break;
			case 2:
			default:
				f->dx = dispp->col + 24 + kscale*width/2;
				break;
			}
		} else {
		/* this gives the left hand edge of box - wrong */
			f->dx = dispp->col - dispp->space/2 + (2*k->pos-1)*dispp->space/(2*dispp->npos);
		}

	/*
	 * to be inverted ?
	 */
		if (obj->plist->dispor == -1) {
			f->ix = -1;
			f->iy = -1;
			f->oy = idom->lastln;
		} else {
			f->ix = 1;
			f->iy = 1;
			f->oy = idom->line1;
		}

	/*
	 * Various funny display modes
	 */
		if (VIDEORECORD)
		f->dx = 9*(400+f->dx)/10;
		if (aspectr != 0) { /* aspect ratio suitable for hardcopy */
			f->ix *= 2;
			f->iy *= 3;
			f->scale /= 2;
		}
	}
}


#ifndef OSK
/*
 * send to versatec or printer, and stop
 */
verskary(v,in)
FILE *in;
{
	int i;
	finish();
	endout();
	switch(v) {
		case 0:
			setout("stovers",'p');
			dinit();
			break;
		case 1:
			setout("stolpr",'p');
			dinit();
			break;
		case 2:
			setout("stolas",'p');
			dinit();
			fprintf(stderr,"how many copies ? ");
			fscanf(in,"%d",&i);
			user(i,0);		/* no of copies laser printer only */
			break;
	}
	fprintf(stderr,"which dither matrix (1-8) ? ");
	fscanf(in,"%d",&i);
	setdither(i);
	displayall();
	moveto(2048,2048);
	lwidth(4);
	box(4092,4092);
	moveto(0,878);
	pixbegin(8,1,1,4096,3218);
	pixel(0);
	pixend();
	moveto(0,0);
	pixbegin(8,1,1,1492,878);
	pixel(0);
	pixend();
	finish();
	endout();
	putaway(objlist);
	exit(0);
}
#endif OSK




/*
 * identify which chromosome user is referring to when input is typed.
 * fill details into the identification structure
 */
identify(k,in)
struct kident *k;
FILE *in;
{
	register int class, pos;
	register struct kclasscont *kccp;
	class = getclass(in);
	kccp = kcc+class;
	pos = getpos(in);
	if (class>=0 && class<MAXCLASS && pos >= 0 && pos < MAXCHILD) {
		k->no = kccp->kno[pos];
		if (k->no >= 0) {
			k->kclass = class;
			k->pos = pos+1;
			k->stackpos = kccp->sp[pos];
			return(1);
		}
	}
	fprintf(stderr,"\007?? - try again\n");
	return(0);
}


/*
 * identify whether there is a chromosome at position "pos", class "class",
 * and if so fill details into the identification structure "k"
 */
ndentify(k,class,pos)
struct kident *k;
{
	register struct kclasscont *kccp;
	kccp = kcc+class;
	if (class>=0 && class<MAXCLASS && pos >= 0 && pos < MAXCHILD) {
		k->no = kccp->kno[pos];
		if (k->no >= 0) {
			k->kclass = class;
			k->pos = pos+1;
			k->stackpos = kccp->sp[pos];
			return(1);
		}
	}
	return(0);
}



/*
 * re-use a position just emptied to get an object from
 * bottom line into correct position where appropriate.
 * The position and class are specified in identifier
 * structure "k".
 */
re_use_if_poss (k,kcont)
register struct kident *k;
struct kcontrol *kcont;
{
	struct kident kn;
	register int i;
	register struct kclasscont *kc = kcc + k->kclass;
	/*
	 * do nothing in various circumstances:
	 *	- spare position non-empty (an error ?)
	 *	- spare position is a stack position
	 */
	if (kc->n == kc->maxn)
		return(0);
	if (k->pos > disppos[k->kclass].npos)
		return(0);
	if (kc->kno[k->pos-1] >= 0)
		return(0);
	/*
	 * position seems (i) not stack, (ii) empty.
	 * see if there is anything to fill it
	 */
	for (i=kc->maxn-1; i>=2; i--) {
		if (kc->kno[i] >= 0) {
			/* delete from first position */
			kn.no = kc->kno[i];
			kn.pos = i+1;
			kn.stackpos = kc->sp[i];
			kn.kclass = k->kclass;
			unkdisplay(&kn,kcont);
			remove_class(&kn);
			/* add to better position */
			newclass(&kn,k->kclass,kcont,disppos);
			kdisplay(&kn,kcont);
			return(0);
		}
	}
}

/*
 * display the standard class tags on screen
 */
disptags()
{
	register int i;
	register struct kdisppos *dp = disppos;

	colour(OVERLAY1);
	for (i=0; i<UNDEFCLASS; i++) {
		dispstring(dp->tag,dp->col-(i<9||i>=22? 32: 48),dp->lin-96);
		dp++;
	}
	update();
}
