/*
 * kclass.c		Jim Piper	27/3/84
 *
 * Normalise feature measurements, then
 * classify chromosomes using maximum likelihood.
 * Also classify using re-arrangement from over-subscribed classes.
 *
 * NOTE THIS PROGRAM WORKS WITH CLASS NUMBERS 0 - MAXCLASS-1.
 *
 **********************************************************************
 * NORMALISATION REQUIRES ALL CHROMOSOMES, COMPOSITES AND OVERLAPS !!!!
 *
 * BUT - we don't measure features except on objects believed to be
 * chromosomes, so can only use them for now.
 **********************************************************************
 *
 *
 * Modifications
 *
 *   24 Feb 1992	Joanne		clcontents initialised in class() !!
 *   17 Jan 1992	Joanne		initclass only needs one parameter and
 *					calls readparams() rather than setparams()
 *					since the data for the classifier layout 
 *					is immediately read from file now.
 *	 9 Apr 1991		ih				debug print of CC->ntype1_3 before normvec
 *	 3 May 1989		dcb				VCVHEAD changes to VCVTYPE for new class.h
 *	 7 Oct 1988		CAS/DCB/SEAJ	Don't like the way that the classifier
 *									struct eats up most of our static storage space
 *									SO.. Malloc the bloody thing as it should
 *									have been in the first place...
 *	19 Aug 1988		SCG		set classifier from rankcf structure (setparams)
 *	03 Aug 1988		SCG		rearrange chromsomes in the class array so that
 *							most probable ones come first
 *	29 Jul 1988		SCG		Allow user defined classification to override 
 *							classifier decisions (fixclass)
 *  05 Mar 1987		GJP		Use kary_exit not exit
 *	20 Feb 1987		CAS		Removed bug where we closed class file twice !
 *							Modified for karylib compatability so we can
 *							stuff it back where it belongs !!!!!!!!!!
 *	10 Feb 1987		jimp	Clobjsetup() brought up to standard of the
 *							karylib version - necessary for classification
 *							after resegmentation.
 *	05 Dec 1986		jimp	Classifier parameters read from buffered file
 *	13 Nov 1986		jimp	Comment above about only using chromosomes was
 *							not implemented - has now been.
 *	12 Nov 1986		jimp	->gool1 replaced by ->C
 *	 1 Oct 1986		CAS			Remove local file buffer for class train
 *								file
 *	30 Sep 1986		CAS			Close param file, make classifier static
 *								and define file name in #define
 * 23 Sep 1986		GJP			Change the printing of debugs to debug_chan 
 *	jimp	21-08-86	parameter file now set by #define
 *  7 May 1986		GJP			Debugs removed
 * Modified for using "kcontrol" structure for potential
 * re-classification and multi-processing classification 29/1/85
 ***********************************************************************
 */

#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <class.h>
#include <multip.h>

#define MAXPROB 2000000000.0

static FILE *debug_channel = NULL;

extern char *getcurclass();
extern char *getspec();


/*
 *-------------------------------------------------------------------------
 * Classifier structure.
 *
 * BEWARE - this structure contains some arrays and some pointers.
 * When converting addesses for multi-processing, the compiler will
 * access the arrays correctly by offsetting from the structure address,
 * which itself nust of course be converted in a slave processor.
 * But the POINTER VALUES in the structure must be explicitly converted.
 *-------------------------------------------------------------------------
 */
static struct classifier *C = NULL;

/*
 * activity list for classification
 */
SMALL clactive[MAXOBJS];

/* logs of prior probabilities */
double tlpriorp[MAXCLASS] = {
0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 
0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 
0.0, 0.0, 0.0, -0.5};

int tclwn[MAXCLASS] = {
2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
2, 2, 2, 1};


Classifier_debug(chan)
FILE	*chan;
{
	debug_channel = chan;
}

/*
 * Set up classifier data structure
 */
initclass (kc)
register struct kcontrol *kc;
{
	register int i;

	if (C != NULL) Free(C);
	C = (struct classifier *) Calloc(sizeof(struct classifier),1);

/*
	if (C == NULL)
		C = (struct classifier *) Calloc(sizeof(struct classifier),1);
*/
	for (i=0; i<MAXCLASS; i++)
		C->clcontents[i].n = 0;
	C->numchr = 0;
	C->ntype1_3 = 0;
	C->phenosex = 0;
	C->lpriorp = tlpriorp;
	C->clwn = tclwn;
	kc->C = C;
	if (debug_channel)
		fprintf(debug_channel,"CLASS - Classifier structure position %x size %x\n",
			C,sizeof(struct classifier));
	kc->gacl1 = clactive;
	readparams(kc);
}


/*
 * classify all chromosomes
 */
class(kc)
register struct kcontrol *kc;
{
	register int i;
	struct classifier *CC;
	
	CC = kc->C;
	
/*
 *  Initialise clcontents before every classification
 */
 	for (i=0; i<MAXCLASS; i++)
		CC->clcontents[i].n = 0;

/*
 * set up lists of chromosomes and objects
 */
	clobjsetup(kc,0,kc->maxnumber);	
/*
 * compute cell normaliser
 */
	if (debug_channel) {
		fprintf(debug_channel,"CLASS - computing normalisers: ");
		fprintf(debug_channel,"CLASS - CC 0x%x CC->otype1_3_list 0x%x CC->ntype1_3 %d\n",CC,CC->otype1_3_list,CC->ntype1_3);
	}
	normvec(CC->otype1_3_list,CC->ntype1_3,CC->vn);

/*	elapsedtime();
*/

	/*
	 * Compute likelihoods using the normalised chromosome features.
	 * First clear function bits in activity list
	 */
	for (i=0; i<CC->numchr; i++)
		kc->gacl1[i] = ACTIVE;
	if (debug_channel) {
		fprintf(debug_channel, "CLASS - numchr = %d\n", CC->numchr);
		fprintf(debug_channel, "CLASS - chromosome features before mplikl\n");
		debug_features(kc);
	}

	mplikl(kc);
	if (debug_channel) {
		fprintf(debug_channel, "CLASS - after call to mplikl features and clcontents\n");
		debug_features(kc);
/*		debug_clcontents(CC); */
	}
/*
	elapsedtime();
*/
	fixclass(CC);

	/*
	 * re-arrange so as to move objects from over-subscribed
	 * classes to under-subscribed as far as plausibility allows
	 */
	rearrange(CC->chrocl,CC->clcontents,CC->numchr,CC->clwn,CC->nclass);
	if (debug_channel)
		fprintf(debug_channel,"CLASS - rearrangement classifier: ");
/*
	elapsedtime();
*/
	
	/*
	 * insert class assignments in property lists
	 */
	for (i=0; i<CC->numchr; i++) 
		CC->chrlist[i]->plist->pgroup = CC->chrocl[i].newclass+1;
}
/*
 * D E B U G _ F E A T U R E S
 *
 */
debug_features(kc)
register struct kcontrol *kc;
{
	struct chromplist *plist;
	int fvec[MAXDIM];
	int i, j;
	
	for (i = 0; i < kc->C->numchr; i++) {
		fprintf(debug_channel, "CHROMOSOME %d :\n", i);
		plist = kc->C->chrlist[i]->plist;
		getvec(plist, fvec);
		for(j = 0; j < MAXDIM; j++) {
			if (j % 5 == 0 && j != 0)
				fprintf(debug_channel, "\n");
			fprintf(debug_channel, "%d : %d   ", j, fvec[j]);
		}
		fprintf(debug_channel, "\n");
	}
}
/*
 * D E B U G _ C L C O N T E N T S
 *
 */
debug_clcontents(CC)
struct classifier *CC;
{
	int i, j, k;
	struct chromcl *cl;
	struct chchain *chain;
	
	for (i = 0; i < MAXCLASS; i++) {
		fprintf(debug_channel, "CLCONTENTS[%d] : n %d\n", i, CC->clcontents[i].n);
		for (j = 0; j < MAXCHILD; j++) {
			cl = CC->clcontents[i].chcl[j];
			while (cl != NULL) {
				fprintf(debug_channel, "CHCL : raw %d true %d new %d best %d moveval %f\n",
					cl->rawclass, cl->trueclass, cl->newclass, cl->bestclass,cl->moveval);
				for (k = 0; k < CUTOFF; k++) {
					fprintf(debug_channel, "CLLIKA : class %d lval %f\n",
						cl->cllika[k].class, cl->cllika[k].lval);
				}
				chain = cl->movechain;
				for (k = 0; (k < MAXCHILD && chain != NULL); k++) {
					fprintf(debug_channel, "CHCHAIN : accum_cost %f nchild %d class %d bestchild %d\n",
						chain->accum_cost, chain->nchildren, chain->class, 
						chain->bestchild);
					chain = chain->child[k];
				}
				chain = cl->movechain;
				if (chain == NULL)
					cl = NULL;
				else
					cl = chain->chrcl;
			}
		}
	}
}

/*
 *	C L O B J S E T U P
 *
 * Extract list of chromosomes.
 * Also extract what in the original design was the combined list
 * of chromosomes, composites, and overlaps for the normaliser.
 * However, since only chromosomes have had features measured,
 * this is a problem and we in fact only insert chromosomes into
 * this list as well.		-		jimp	13-11-86
 */
clobjsetup(kc,minnumber,maxnumber)
register struct kcontrol *kc;
{
	register struct classifier *C;
	register int numobj, ac;
	register struct chromosome *obj;

	C = kc->C;
	C->numchr = 0;
	C->ntype1_3 = 0;
	for (numobj = minnumber; numobj < maxnumber; numobj++) {
		ac = kc->acl[numobj];
		ac &= ACTIVE;
		if (ac == ACTIVE) {
			if (kc->mol&&(obj=kc->mol[numobj])) {
				if (debug_channel)
					fprintf(debug_channel,"CLOBJSETUP - object %d type %d addr %d",
							numobj,obj->type,obj);
				switch (obj->type) {
				case 1:
				/* is object to be included in normalisation ?? */
					if (debug_channel)
						fprintf(debug_channel,"CLOBJSETUP - plist length %d otype %2d",
								obj->plist->size,obj->plist->otype);
				/* is object a chromosome ? */
					if (obj->plist->otype <= 1) {
						C->chrlist[C->numchr++] = obj;
						C->otype1_3_list[C->ntype1_3++] = obj;
					}
					break;

				case 110:		/* cell i.d. - use for sex */
					break;

				default:
					break;
				}
			}
			else
				if (debug_channel) fprintf(debug_channel,"Null object %d (address 0x%x) or no mol list",numobj,kc->mol[numobj]);
			if (debug_channel)
				fprintf(debug_channel,"\n");
		}
	}
	if (debug_channel)
		fprintf(debug_channel,"CLOBJSETUP - %d objects %d chromosomes\n",kc->maxnumber,
				C->numchr);
}


/*
 * read class mean vectors and variance-covariance matrix
 */
static char filename[100];
readparams(kc)
struct kcontrol *kc;
{
	FILE *fclass, *fbopen();
	struct classifier *C;
	int i;
	char *spec, *class, *getspec(), *getcurclass();

	spec = getspec();
	class = getcurclass();
	sprintf(filename, "/dd/params/class.train.%s.%s", spec, class);
	C = kc->C;
	fclass = fbopen(filename,"r",BUFSIZ);
	if (fclass == NULL) {
		fprintf(stderr,"Cannot open classifier parameter file\n");
		kary_exit(2);
	}
	if (debug_channel)
		fprintf(debug_channel,"READPARAMS - Fclass iob 0x%x (af open) path %d base 0x%x bufsize %d\n",
			fclass,fclass->_fd,fclass->_base, fclass->_bufsiz);
	readclassparams(fclass,C);
	fbclose(fclass);
	if (debug_channel) {
		fprintf(debug_channel,"READPARAMS - Fclass iob (af read) path %d base 0x%x bufsize %d\n",
			fclass->_fd,fclass->_base, fclass->_bufsiz);
		fprintf(debug_channel,"READPARAMS - nclass %d ndim %d nsfv %d\n",C->nclass,C->ndim,C->nsfv);
		for (i=0; i<C->nsfv; i++)
			fprintf(debug_channel,"READPARAMS - %3d",C->sfv[i]);
		fprintf(debug_channel,"\n");
	}
}



/*
 *	R E A D C L A S S P A R A M S
 *
 */
readclassparams(c,C)
FILE *c;
register struct classifier *C;
{
	register int i,j;
	double d;
	
	/* dimensions and selected feature serial numbers */
	fscanf(c,"%d %d %d",&C->nclass,&C->ndim,&C->nsfv);
	for (i=0; i<C->nsfv; i++)
		fscanf(c,"%d",&C->sfv[i]);
/*
 * per class inverted variances. Add log of prior probability
 * for computational efficiency
 */
	for (i=0; i<C->nclass; i++) {
		fscanf(c,"%E",&C->clinvvarsqrtdet[i]);
		C->clinvvarsqrtdet[i] += C->lpriorp[i];
	}
	
/*
 * class integer means
 */
	for (i=0; i<C->nclass; i++)
		for (j=0; j<C->nsfv; j++)
			fscanf(c,"%d",&C->iclassmeans[i][j]);
	
/*
 * per class integer inverted variances
 */
	for (i=0; i<C->nclass; i++) {
		fscanf(c,"%E",&d);
		C->conversionf[i] = 1.0 / d;
		for (j=0; j<C->nsfv; j++)
			fscanf(c,"%d",&C->iclassinvvars[i][j]);
	}
}

/*
 * F I X C L A S S --- Allow user assigned classification to override classifier results
 *
 */
fixclass(CC)
register struct classifier *CC;
{
int i, j;

	if (debug_channel)
		fprintf(debug_channel, "FIXCLASS\n");
	/*
	 * for all chromosomes
	 */
	for (i = 0; i < CC->numchr; i++) {
		/*
		 * check whether a class has been given by the user
		 */
		if((CC->chrocl[i].trueclass > 0) && (CC->chrocl[i].trueclass <= MAXCLASS)) {
			/*
			 * put in all structs user chosen class and a large probability
			 */	
			for(j = 0; j < CUTOFF; j++) {
				CC->chrocl[i].cllika[j].class = CC->chrocl[i].trueclass - 1;
				CC->chrocl[i].cllika[j].lval = MAXPROB;
			}
		}
	}
	if (debug_channel)
		fprintf(debug_channel, "FIXCLASS - end\n");
}

/*
 * set class mean vectors and variance matrix
 */
setparams(kc, vhd)
struct kcontrol *kc;
VCVTYPE *vhd;
{
	struct classifier *C;
	int i, j;

	C = kc->C;
	
	/* 
	 * dimensions and selected feature serial numbers 
	 */
	C->nclass = vhd->nclasses;
	C->ndim = MAXVCVFEATS;
	C->nsfv = vhd->nfeats;
	for (i=0; i<C->nsfv; i++)
		C->sfv[i] = vhd->vfd[i]->feat;

/*
 * per class inverted variances. Add log of prior probability
 * for computational efficiency
 */
	for (i = 0; i < C->nclass; i++)
		C->clinvvarsqrtdet[i] = C->lpriorp[i] + vhd->logsqrtdet[i];
	
/*
 * class integer means
 */
	for (i = 0; i < C->nclass; i++)
		for (j = 0; j < C->nsfv; j++)
			C->iclassmeans[i][j] = vhd->vfd[j]->featmean[i];
	
/*
 * per class integer inverted variances
 */
	for (i=0; i<C->nclass; i++) {
		C->conversionf[i] = 1.0 / vhd->scalef[i];
		for (j=0; j<C->nsfv; j++)
			C->iclassinvvars[i][j] = vhd->vfd[j]->featsd[i];
	}
}
			
