/*
 * jkinteract.c		23/1/84		Jim Piper
 ***********************************************************************
 *	Modifications
 *
 *  11 Jun 1991		ih		Check for null obj anf obj[kid.no] in kinteract
 *	 7 Feb 1991		CAS			remove Mizar defs -> osk, voids
 *	 4 Jul 1989		SEAJ		Added calls to newpgroup and newCpgroup.
 *  28 Feb 1989		SEAJ		Changed call to kcopyback to add new parameter.
 *	 2 Dec 1988		CAS/SEAJ	Changed params to newclass, firstclass, swapclass
 *								and added a extern disppos struct
 *	11 Nov 1986		jimp	->gool1 replaced by ->C,
 *							display.h added and colours coded symbolically
 *	11 Sep 1986		CAS		Converted to vax long names & includes fixed
 *	22-08-86		jimp	Straightening, interpolation and filtering
 *							options added.
 ***********************************************************************
 */

#include <stdio.h>
#include <wstruct.h>
#include <interact.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <multip.h>
#include <display.h>

#define RADIANS 	57.296
#define WIDTH		8
#define NORMALISE	02
#define TEMPHIGHLIGHT	04

extern int aspectr;
extern int DOWN;
extern struct kdisppos disppos[];

/*
 * re-arrange chromosomes on screen
 * Interactive protocol :
 *	type two strings separated by space, then return, as follows:
 *
 *	i nx		invert chromosome at position nx
 *	c nx		object at position nx is a composite; split
 *	d nx		delete object at position nx
 *	o nx		object at position nx is an overlap; treat suitably
 *	m nx m		move object at nx to class m
 *	no nx		object nx is noise of some sort - deleted
 *	nu nx		object nx is nucleus - deleted
 *	s nx my		swap objects at positions nx and my
 *	r nx		redraw object in position nx
 *	e		finish interaction, tidy screen, check comments
 *	t		tidy screen - (e.g. place trisomies correctly)
 *	p		print current karyotype on printer
 *	v		print current karyotype on versatec
 *	A		alter aspect ratio to 3 to 2 vertical to horizontal and
 *			re-display, for better reproduction in the Honeywell
 *			video hard copy unit
 *	R nx d		rotate object at position nx by d degrees anticlockwise
 *			(cumulative)
 *	S n		re-display at scale n
 *	M n		mark object n as interesting
 *	N nx		re-display object at nx normalised or unnormalised
 *			(toggle)
 *	B		display in 4 big quarters of screen
 *	D n		move display down by "n" (DIGS) from default.
 *************************************************************************
 *	MODIFICATIONS 20-08-86	JIMP @ I.R.S.
 *	------------------------------------
 *	The "keeplist" array is now an explicit copy of the original pointers in
 *	"kcont->mol".  When an object is rotated/straightened the pointer
 *	to the NEW object is stored in "kcont->mol".  This has the side-effect
 *	that this copying must also be done when composites/overlaps are split.
 *
 *	NEW COMMANDS
 *	------------
 *	H n		Expand all chromosomes of type n by interpolation,
 *			display on a clear screen.
 *	J		Enhance everything
 *  L nx	Enhance object by applying some sort of Laplacian filter
 *	W nx	Recover original object in cases where rotation/straightening
 *			has been performed.
 *	V		Straighten everything.
 *	X nx	Automatically straighten object at position nx and redisplay
 *	Y nx	Manually straighten object at position nx and redisplay
 *************************************************************************
 */

kinteract(kc)
struct kcontrol *kc;
{
	struct chromosome *polyobj, *interactpoly();
	struct chromosome *extractov(), *spin(), *squeeze(), *autostraight();
	struct chromosome *manualstraight(), *enhance();
	struct chromosome *nobj, *mobj, *chromfeatures();
	struct chromosome **objlist, *keeplist[MAXOBJS];
	struct intervaldomain *idom;
	struct chromplist *makechromplist();
	struct chromplist *plist, *qlist;
	struct kident kid1, kid2;
	char c, c2[50];
	int i2, i, j1, j2, k, class;
	struct pframe f;
	FILE *in;
	char inbuffer[BUFSIZ];
	struct classifier *CC;
	
	CC = (struct classifier *) kc->C;
	for (i=0; i<kc->maxnumber; i++)
		keeplist[i] = kc->mol[i];
	if (!(objlist=kc->mol)) return(0);
	in = fopen(kc->terminal,"r");
	in->_bufsiz = BUFSIZ;
	setbuf(in,inbuffer);
	for (;;) {
		wormcast(kc,MASTER,"kinteract");
		write(fileno(stderr),": ",2);
		c = getc(in);
		c &= 127;		/* mask parity bit */
		if (c == '\n')
			continue;
		switch (c) {

#ifndef OSK
		case 'l' :
			verskary(2,in);
			break;
		case 'p' :
			verskary(1,in);
			break;
		case 'v' :
			verskary(0,in);
			break;
#endif OSK

		case 't' :	/* tidy screen */
			tidyscreen(kc);
			getnl(in);		/* get past the newline */
			break;

		case 'e' :
			tidyscreen(kc);
			getnl(in);		/* get past the newline */
			fclose(in);
			return(0);

		case 'n' :		/* noise */
			c = getc(in);
			if (c == 'o' || c=='u') {
				if (identify(&kid1,in)) {
					unkdisplay(&kid1,kc);
					remove_class(&kid1);
					kc->acl[kid1.no] &= ~ACTIVE;
					if (objlist[kid1.no]&&(qlist=objlist[kid1.no]->plist)) {
						qlist->Cotype = (c=='u'? 5: 6);
						qlist->otype = qlist->Cotype;
						qlist->otconf = 100;
						re_use_if_poss(&kid1,kc);
					}
				}
			} else
				fprintf(stderr,"try again\n");
			getnl(in);		/* get past the newline */
			break;

		case 'd' :	/* delete, make noise if formerly chromosome */
			if (identify(&kid1,in)) {
				unkdisplay(&kid1,kc);
				remove_class(&kid1);
				kc->acl[kid1.no] &= ~ACTIVE;
				if (objlist[kid1.no]&&(qlist=objlist[kid1.no]->plist)) {
					if (qlist->otype == 1) {
						qlist->otype = 6;	/* noise */
						qlist->Cotype = 6;
						qlist->otconf = 100;
					}
					unkdisplay(&kid1,kc);
					remove_class(&kid1);
					re_use_if_poss(&kid1,kc);
				}
			}
			getnl(in);		/* get past the newline */
			break;

		case 'z' :	/* show in metaphase spread */
			if (identify(&kid1,in)) {
				/*
				 * clear screen, display original objects in original frame
				 */
				fast();
				mpclear(kc);
				slow();
				mpcpic(kc,GREYCOLS,1);
				colour(OVERLAY1);
				bdisprect(kc->ool[kid1.no],kc->f);
				getnl(in);		/* past newline */
				fprintf(stderr,"type newline to continue\n");
				getnl(in);		/* past newline */
				displayall(kc);
			}
			break;
				
		case 'R' :	/* rotate by specified degrees */
			if (identify(&kid1,in)) {
				fscanf(in,"%d",&i2);
				unkdisplay(&kid1,kc);
				if (nobj=objlist[kid1.no]) {
					objlist[kid1.no] = spin(nobj,i2/RADIANS);
					objlist[kid1.no]->plist = nobj->plist;
				/*
				 * remove any intermediate rotated/straightened objects
				 */
					if (nobj != keeplist[kid1.no]) {
						if (nobj->idom == keeplist[kid1.no]->idom)
							nobj->idom = NULL;
						nobj->plist = NULL;
						freeobj(nobj);
					}
					kdisplay(&kid1,kc);
				}
			}
			getnl(in);		/* get past the newline */
			break;

		case 'H' :	/* interpolated display */
			k = getclass(in);
			if (k >= 0 && k <= MAXCLASS) {
				fast();
				mpclear(kc);
				slow();
				colour(GREYCOLS);
				intens(1);
				f.scale = 8;
				f.dx = f.dy = 1024;
				for (i=0; i<MAXCHILD; i++)
					if (ndentify(&kid1,k,i)) {
						if (nobj=objlist[kid1.no]) {
							if (nobj->plist->dispor > 0) {
								f.ix = f.iy = 1;
								f.ox = nobj->idom->kol1;
								f.oy = nobj->idom->line1;
							} else {
								f.ix = f.iy = -1;
								f.ox = nobj->idom->lastkl;
								f.oy = nobj->idom->lastln;
							}
							nobj = squeeze(nobj,1.9,1.9);
							picframe(nobj,&f);
							f.dx += 128 + f.scale*(nobj->idom->lastkl - nobj->idom->kol1);
							nobj->plist = NULL;
							freeobj(nobj);
						}
					}
				fprintf(stderr,"Type return to continue\n");
				getnl(in);		/* get past the newline */
				getnl(in);		/* get past the newline */
				displayall(kc);
			} else
				getnl(in);		/* get past the newline */
			break;
				
		case 'J' :	/* enhance everything by Laplacian */
			for (k=0; k<MAXCLASS; k++)
				for (i=0; i<MAXCHILD; i++)
					if (ndentify(&kid1,k,i)) {
						if (nobj=objlist[kid1.no]) {
							objlist[kid1.no] = enhance(nobj);
							objlist[kid1.no]->plist = nobj->plist;
						/*
						 * remove any intermediate rotated/straightened objects
						 */
							if (nobj != keeplist[kid1.no]) {
								nobj->idom = NULL;
								nobj->plist = NULL;
								freeobj(nobj);
							}
							kdisplay(&kid1,kc);
						}
					}
			getnl(in);		/* get past the newline */
			break;

		case 'L' :	/* enhance by Laplacian */
			if (identify(&kid1,in)) {
				if (nobj=objlist[kid1.no]) {
					objlist[kid1.no] = enhance(nobj);
					objlist[kid1.no]->plist = nobj->plist;
				/*
				 * remove any intermediate rotated/straightened objects
				 */
					if (nobj != keeplist[kid1.no]) {
						nobj->idom = NULL;
						nobj->plist = NULL;
						freeobj(nobj);
					}
					kdisplay(&kid1,kc);
				}
			}
			getnl(in);		/* get past the newline */
			break;

		case 'M' :			/* display or unkdisplay a mark (toggle) */
			if (identify(&kid1,in)) {
				setframe(&f,&kid1,kc);
				if (objlist[kid1.no]&&(plist=objlist[kid1.no]->plist)) {
					if (plist->dispmark == 0) {
						plist->dispmark = 1;
						dispmark(&f);
					} else {
						plist->dispmark = 0;
						intens(0);
						dispmark(&f);
						intens(1);
					}
				}
			}
			getnl(in);		/* get past the newline */
			break;

		case 'N' :	/* grey-scale normalisation toggle */
			if (identify(&kid1,in)&&objlist[kid1.no]&&objlist[kid1.no]->plist) {
				objlist[kid1.no]->plist->dispopts = ((objlist[kid1.no]->plist->dispopts & NORMALISE)?
					objlist[kid1.no]->plist->dispopts & ~NORMALISE:
					objlist[kid1.no]->plist->dispopts | NORMALISE);
				kdisplay(&kid1,kc);
			}
			getnl(in);		/* get past the newline */
			break;

		case 'i' :	/* invert */
			if (identify(&kid1,in)) {
				unkdisplay(&kid1,kc);
				if (objlist[kid1.no]&&objlist[kid1.no]->plist) {
					objlist[kid1.no]->plist->dispor *= -1;
					kdisplay(&kid1,kc);
				}
			}
			getnl(in);		/* get past the newline */
			break;

		case 'r' :	/* redraw */
			if (identify(&kid1,in))
				kdisplay(&kid1,kc);
			getnl(in);		/* get past the newline */
			break;

		case 'c' :	/* composite: split */
			if (identify(&kid1,in)) {
				/*
				 * clear screen, display original objects in original frame
				 */
				fast();
				mpclear(kc);
				slow();
				mpcpic(kc,GREYCOLS,1);
				colour(OVERLAY1);
				bdisprect(kc->ool[kid1.no],kc->f);
				/*
				 * kick feature measurement in slaves,
				 * and call segmentation correction stuff
				 */
				i2 = kc->maxnumber;
				fmeasstart(kc);
				k = segcomp(kc,kid1.no);
				kc->mstatus |= SEGEND;
				waitfmeas(kc);
				/*
				 * if fail to segment then do nothing
				 */
				if (kc->maxnumber > i2) {
					/*
					 * copy back measured objects to master,
					 * call them "unknown" class
					 */
					kcopyback(kc,i2, kc->maxnumber);
					/*
					 * compute likelihoods of new objects
					 */
					clobjsetup(kc,i2,kc->maxnumber);
					for (j2=0; j2<CC->numchr; j2++)
						kc->gacl1[j2] = ACTIVE;
					mplikl(kc);
					remove_class(&kid1);
					re_use_if_poss(&kid1,kc);
					for (j2=i2; j2<kc->maxnumber; j2++) {
						/* insert class assignments in property lists */
						class = CC->chrlist[j2-i2]->plist->pgroup = CC->chrocl[j2-i2].newclass+1;
						CC->chrlist[j2-i2]->plist->dispopts |= TEMPHIGHLIGHT;
						initialorientation(j2,kc);
						firstclass(j2,kc,disppos);
						keeplist[j2] = kc->mol[j2];
					}
				}
				getnl(in);		/* get past the newline */
				displayall(kc);
			}
			break;

		case 'o' :	/* overlap: extract one chromosome */
			if (identify(&kid1,in)) {
				/*
				 * clear screen, display original objects in original frame
				 */
				fast();
				mpclear(kc);
				slow();
				mpcpic(kc,GREYCOLS,1);
				colour(OVERLAY1);
				bdisprect(kc->ool[kid1.no],kc->f);
				/*
				 * kick feature measurement in slaves,
				 * and call segmentation correction stuff
				 */
				fprintf(stderr,"how many chromosomes?\n");
				fscanf(in,"%d",&j1);
				fmeasstart(kc);
				i2 = kc->maxnumber;
				segovlp(kc,kid1.no,j1);
				kc->mstatus |= SEGEND;
				waitfmeas(kc);
				/*
				 * if fail to segment then do nothing
				 */
				if (kc->maxnumber > i2) {
					/*
					 * copy back measured objects to master,
					 * call them "uknown" class
					 */
					kcopyback(kc,i2, kc->maxnumber);
					for (j2=i2; j2<kc->maxnumber; j2++) {
						initialorientation(j2,kc);
						kid2.no = j2;
						newpgroup(&kid2, UNDEFCLASS, kc);
						newclass(&kid2,UNDEFCLASS,kc,disppos);
						keeplist[j2] = kc->mol[j2];
					}
					remove_class(&kid1);
					re_use_if_poss(&kid1,kc);
				}
				getnl(in);		/* get past the newline */
				displayall(kc);
			}
			break;

		case 'm' :	/* move object to new class */
			if (identify(&kid1,in)) {
					/* ----------------------------- */
					/* What if X or Y ?????? */
					/* ----------------------------- */
				class = getclass(in);
				unkdisplay(&kid1,kc);
				remove_class(&kid1);
				re_use_if_poss(&kid1,kc);
				newCpgroup(&kid1, class, kc);
				newclass(&kid1,class,kc,disppos);
				kdisplay(&kid1,kc);
			}
			getnl(in);		/* get past the newline */
			break;

		case 's' :	/* swap two objects */
			if (identify(&kid1,in))
				if (identify(&kid2,in)) {
			/*
			 * move objects to desired target positions
			 */
					unkdisplay(&kid1,kc);
					unkdisplay(&kid2,kc);
					swapclass(&kid1,&kid2,kc,disppos);
					kdisplay(&kid1,kc);
					kdisplay(&kid2,kc);
				}
			getnl(in);		/* get past the newline */
			break;

		case 'S' :			/* display at different scale */
			fscanf(in,"%d",&i2);
			karyscale(i2);
			displayall(kc);
			getnl(in);		/* get past the newline */
			break;

		case 'W' :	/* recover original */
			if (identify(&kid1,in)) {
				unkdisplay(&kid1,kc);
				nobj=objlist[kid1.no];
				if (objlist[kid1.no]=keeplist[kid1.no]) {
					if (nobj&&(nobj!=keeplist[kid1.no])) {
						if (nobj->idom == keeplist[kid1.no]->idom)
							nobj->idom = NULL;
						nobj->plist = NULL;
						freeobj(nobj);
					}
					kdisplay(&kid1,kc);
				}
				else if (nobj) {
					nobj->plist=NULL;
					freeobj(nobj);
				}
			}
			getnl(in);		/* get past the newline */
			break;

		case 'V' :	/* straighten everything automatically */
			for (k=0; k<MAXCLASS; k++)
				for (i=0; i<MAXCHILD; i++)
					if (ndentify(&kid1,k,i)) {
						unkdisplay(&kid1,kc);
						if (nobj=objlist[kid1.no]) {
							objlist[kid1.no] = autostraight(nobj);
							objlist[kid1.no]->plist = nobj->plist;
						/*
						 * remove any intermediate rotated/straightened objects
						 */
							if (nobj != keeplist[kid1.no]) {
								if (nobj->idom == keeplist[kid1.no]->idom)
									nobj->idom = NULL;
								nobj->plist = NULL;
								freeobj(nobj);
							}
							kdisplay(&kid1,kc);
						}
					}
			getnl(in);		/* get past the newline */
			break;

		case 'X' :	/* straighten automatically */
			if (identify(&kid1,in)) {
				unkdisplay(&kid1,kc);
				if (nobj=objlist[kid1.no]) {
					objlist[kid1.no] = autostraight(nobj);
					objlist[kid1.no]->plist = nobj->plist;
				/*
				 * remove any intermediate rotated/straightened objects
				 */
					if (nobj != keeplist[kid1.no]) {
						if (nobj->idom == keeplist[kid1.no]->idom)
							nobj->idom = NULL;
						nobj->plist = NULL;
						freeobj(nobj);
					}
					kdisplay(&kid1,kc);
				}
			}
			getnl(in);		/* get past the newline */
			break;

		case 'Y' :	/* straighten manually */
			if (identify(&kid1,in)) {
				fast();
				mpclear(kc);
				slow();
				if (nobj=objlist[kid1.no]) {
					mobj = manualstraight(kc,nobj);
					if (mobj != NULL) {
						objlist[kid1.no] = mobj;
						mobj->plist = nobj->plist;
					/*
					 * remove any intermediate rotated/straightened objects
					 */
						if (nobj != keeplist[kid1.no]) {
							if (nobj->idom == keeplist[kid1.no]->idom)
								nobj->idom = NULL;
							nobj->plist = NULL;
							freeobj(nobj);
						}
					}
				}
				displayall(kc);
			}
			getnl(in);		/* get past the newline */
			break;

		case 'D' :
			fscanf(in,"%d",&DOWN);
			displayall(kc);
			getnl(in);		/* get past the newline */
			break;

		case 'A' :
			aspectr = 1;
			displayall(kc);
			getnl(in);		/* get past the newline */
			break;

		default :
			fprintf(stderr,"try again\n");
			getnl(in);		/* get past the newline */
			break;
		}
	}
}


getnl(in)
FILE *in;
{
	while (getc(in) != '\n')
		;
}
