/*
 * segfuncs1.c		7/11/85		Jim Piper
 *
 * Modifications
 *
 *	 7 Feb 1991		CAS			remove Mizar defs -> osk
 *	10 Mar 1988		CAS			2 vewrsions of this - try to reduce to 1
 *	25 Sep 1987		CAS			Moved legal obj no checker here (want MF accss)
 *	 2 Sep 1987		jimp		Colour(OVERLAY1) forced in dispblobs(), plus
 *								proper detection of ACTIVE objects.
 *	20 Aug 1987		jimp		SI transferred to kcont->SI
 *	 3 Aug 1987		jimp		Include <seg_control.h>, suitable mods...
 *	30 Jul 1987		jimp		Numbers_toggle stuff
 *	03 Mar 1987		jimp		General tidy of renormalisedisplay,
 *								defines.
 *	20 Nov 1986		CAS			Removed cellframe (now in karylib)
 *	 1 Oct 1986		CAS			Removed blobmark (also in countsubs)
 *								General hackery
 *	15 Sep 1986		CAS			Casts
 *	27 Aug 1986		CAS			Removed mkcontrol help message !!
 */

#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <display.h>
#include <seg_control.h>

#define MAX(i,j)	(i>j? i: j)
#define MIN(i,j)	(i<j? i: j)


#ifdef GGG
/*
 * display current number of chromosomes at bottom of screen
 */
dispnchrom(numchroms)
{
	static int numchwas;
	tsize(130,150);
	intens(0);
	dispnum(numchwas,100,100);
	intens(1);
	dispnum(numchroms,100,100);
	tsize(50,50);
	numchwas = numchroms;
}
#endif GGG


/*
 * display the marks of active objects
 * and serial numbers of objects
 */
dispblobs(kcont)
struct kcontrol *kcont;
{
	int objnum;
	
	colour(OVERLAY1);
	for (objnum=0; objnum<kcont->maxnumber; objnum++) {
		if (kcont->acl[objnum] & ACTIVE) {
			intens(kcont->SI->blobs_toggle);
			picframe(kcont->bol[objnum],kcont->f);
			intens(kcont->SI->numbers_toggle);
			frameobjnum(objnum+1,kcont->ool[objnum],kcont->f);
		}
	}
}


/*
 * display active object from master process
 */
mastdisplay(kcont)
struct kcontrol *kcont;
{
	struct object **objlist;
	SMALL *acl;
	int ac,objnum;
	
	objlist = (struct object **) kcont->ool;
	acl = kcont->acl;
	for (objnum = 0; objnum < kcont->maxnumber; objnum++) {
		ac = *acl++;
		ac &= ACTIVE;
		if (ac == ACTIVE)
			picframe(*objlist,kcont->f);
		objlist++;
	}
}


/*
 * make the marks at the center of each object
 */
makemarks(kcont)
struct kcontrol *kcont;
{
	struct object **objlist, **bol, *blobmark();
	int objnum;
	
	objlist = (struct object **) kcont->ool;
	bol = kcont->bol;
	for (objnum = 0; objnum < kcont->maxnumber; objnum++)
		*bol++ = blobmark(*objlist++);
}


#ifdef	GGG
/*
 * return a valid object number of a displayed object, or -1
 */
getonum(in,kcont)
FILE *in;
struct kcontrol *kcont;
{
	int i;
	SMALL *activelist = kcont->acl;
	fscanf(in,"%d",&i);
	if (i <= 0 || i > kcont->maxnumber || activelist[i-1] <= 0) {
		fprintf(stderr,"\007illegal object number\n");
		return(-1);
	} else
		return(i);
}

#endif	GGG

/*
 * renormalise the set of active objects (e.g. after removing
 * a particularly dense nucleus or stain particle)
 */
renormalisedisplay(kcont)
register struct kcontrol *kcont;
{
	register int i,unum;
	register struct chromosome **objlist = kcont->ool;
	register SMALL *activelist = kcont->acl;
	struct chromosome *objl[MAXOBJS];
	for (unum=i=0; i<kcont->maxnumber; i++)
		if (activelist[i] & ACTIVE)
			objl[unum++] = objlist[i];
	normaliseset(objl,unum);
	colour(GREYCOLS);
	for (i=0; i<unum; i++)
		picframe(objl[i],kcont->f);
}


/*
 * normalise grey-tables of set of objects jointly to lie in range 0-255
 */
normaliseset(objl,unum)
register struct object **objl;
{
	register GREY v,min,max,range, *g;
	int i,n;
	struct object *obj;
	struct iwspace iwsp;
	struct gwspace gwsp;
	min = 1;
	max = 0;
	for (n=0; n<unum; n++) {
		obj = objl[n];
		initgreyscan(obj,&iwsp,&gwsp);
		while (nextgreyinterval(&iwsp) == 0) {
			g = gwsp.grintptr;
			for (i=0; i<iwsp.colrmn; i++) {
				v = *g++;
				if (min > max)
					min = max = v;
				else if (v > max)
					max = v;
				else if (v < min)
					min = v;
			}
		}
	}
	range = max - min + 1;
	for (n=0; n<unum; n++) {
		obj = objl[n];
		initgreyscan(obj,&iwsp,&gwsp);
		while (nextgreyinterval(&iwsp) == 0) {
			g = gwsp.grintptr;
#ifdef OSK
			gstretch(g,iwsp.colrmn,255,min,range);
#else OSK
			for (i=0; i<iwsp.colrmn; i++) {
				*g = (256 * (*g - min)) / range;
				g++;
			}
#endif OSK
		}
	}
}
