/*
 * seginteract.c		7/11/85		Jim Piper
 *
 *	Modifications
 *
 *	10 Mar 1988		CAS			moved helpmess, dispnchroms, getonum, kgetc to 
 *								here from segfuncs1 as a tidy exercise
 *	19 Nov 1986		CAS			dispblobs->dispexpblobs to follow displaysubs.c
 * JP	29-10-86	Use "checkattention()" to delete attention marks before
 *					type changes.  Watch out for colour problem !
 * JP	29-10-86	Alterations to correspond to altered "getonum" (see
 *					segfuncs1.c), and to use "drawattention" to show
 *					boundary (command 's').
 * JP	29-10-86	New command 't' to force type to chromosome, and
 *					new function call to check if an object has attention
 *					drawn and if so remove that attention signal.
 * JP	17-10-86	MSTART, ~MEND checked before interacting with object.
 */

#include <stdio.h>
#include <wstruct.h>
#include <interact.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <display.h>
#include <multip.h>

static dispnchroms();

intwith(kcont)
struct kcontrol *kcont;
{
	int c,i,j,k,objnum,j0,j1,numchroms;
	FILE *in;
	struct chromplist *plist, *qlist;
	struct object **marklist = kcont->bol;
	struct chromosome **objlist = kcont->ool;
	SMALL *activelist = kcont->acl;
	struct pframe *f = kcont->f;
	char inbuffer[BUFSIZ];

	numchroms = kcont->maxnumber;
	in = fopen(kcont->terminal,"r");
	if (in == NULL)
		fprintf(stderr,"cannot open terminal\n");
	in->_bufsiz = BUFSIZ;
	setbuf(in,inbuffer);
	for (;;) {
		dispnchroms(numchroms);
		fprintf(stderr,"\n: ");
		c = kgetc(in);
		switch (c) {
		
		case 't' :	/* TRUE CHROMOSOME ACTUALLY - FIX PROPERTY LIST */
			objnum = getonum(in,kcont);
			if (objnum < 0)
				break;
			checkattention(0,objnum,kcont);
			colour(OVERLAY1);
			plist = objlist[objnum]->plist;
			plist->Cotype = plist->otype = CHROMOSOME;
			kgetc(in);
			break;

		case 'c' :	/* COMPOSITE - SPLIT */
			objnum = getonum(in,kcont);
			if (objnum < 0)
				break;
			inset(kcont,objnum);
			checkattention(0,objnum,kcont);
			colour(OVERLAY1);
			j = segcomp(kcont,objnum);
			numchroms += (j-1);
			inclear(kcont,objnum);
			break;

		case 'o':	/* OVERLAP - INCREMENT COUNT */
			objnum = getonum(in,kcont);
			if (objnum < 0)
				break;
			fprintf(stderr,"how many chromosomes?\n");
			fscanf(in,"%d",&j);
			inset(kcont,objnum);
			checkattention(0,objnum,kcont);
			colour(OVERLAY1);
			j = segovlp(kcont,objnum,j);
			numchroms += (j-1);
			inclear(kcont,objnum);
			break;

		case 'n' :	/* NOISE OR NUCLEUS */
			c = kgetc(in);
			if (c != 'o' && c != 'u') {
				putc('\007',stderr);
				while (kgetc(in) != '\n')
					;
			} else {
				objnum = getonum(in,kcont);
				if (objnum < 0)
					break;
				inset(kcont,objnum);
				checkattention(0,objnum,kcont);
				colour(OVERLAY1);
				activelist[objnum] = 0;
				qlist = objlist[objnum]->plist;
				qlist->Cotype = (c=='u'? 5: 6);
				qlist->otype = qlist->Cotype;
				qlist->otconf = 100;
				inclear(kcont,objnum);
				numchroms--;
			}
			break;

		case 'N':	/* RENORMALISE DISPLAY */
			renormalisedisplay(kcont);
			dispexpblobs(kcont);
			dispserials(kcont);
			kgetc(in);		/* get past the newline */
			break;

		case 'j':	/* JOIN TWO OBJECTS */
			j0 = getonum(in,kcont);
			if (j0 < 0)
				break;
			j1 = getonum(in,kcont);
			if (j1 < 0)
				break;
			inset(kcont,j0);
			inset(kcont,j1);
			checkattention(0,j0,kcont);
			checkattention(0,j1,kcont);
			colour(OVERLAY1);
			segjoin(kcont,j0,j1);
			inclear(kcont,j0);
			inclear(kcont,j1);
			numchroms--;
			break;

		case 's':	/* SHOW BOUNDARY */
			i = getonum(in,kcont);
			if (i < 0)
				break;
			drawattention(1,kcont,i,MASTER,OVERLAY1,1);
			fprintf(stderr,"type newline to continue:\n");
			kgetc(in);
			drawattention(1,kcont,i,MASTER,OVERLAY1,0);
			break;

		case 'u':	/* UNDO AN INTERACTION */
			objnum = getonum(in,kcont);
			if (objnum < 0) {
				fprintf(stderr,"\007illegal object number\n");
				break;
			}
			inset(kcont,objnum);
			k = segundo(kcont,objnum);
			inclear(kcont,objnum);
			numchroms += k;
			break;

		case 'e':	/* EXIT */
			kgetc(in);	/* past newline */
			fclose(in);
			return(0);

		case '?':		/* HELP */
			helpmess();
			kgetc(in);		/* get past the newline */
			break;

		default :
			putc('\007',stderr);
			while (kgetc(in) != '\n')
				;
			break;
		}
	}
}


/*
 * return a valid object number of a displayed object, or -1
 */
getonum(in,kcont)
FILE *in;
struct kcontrol *kcont;
{
	int i;
	SMALL *activelist = kcont->acl;
	fscanf(in,"%d",&i);
	if (i <= 0 || i > kcont->maxnumber || activelist[i-1] <= 0) {
		fprintf(stderr,"\007illegal object number\n");
		return(-1);
	} else
		return(i-1);
}


helpmess()
{
	fprintf(stderr,"AVAILABLE COMMANDS ARE:\n");
	fprintf(stderr,"c n       object n is a composite - split\n");
	fprintf(stderr,"e         exit interaction\n");
	fprintf(stderr,"j n1 n2   join object n1 to object n2\n");
	fprintf(stderr,"no n      object n is noise - ignore\n");
	fprintf(stderr,"nu n      object n is nucleus - ignore\n");
	fprintf(stderr,"o n       object n is an overlap - resolve components\n");
	fprintf(stderr,"t n       object n is a true chromosome\n");
	fprintf(stderr,"u n       undo action previously taken to obtain object n\n");
	fprintf(stderr,"N         renormalise display, ignoring noise and nuclei\n");
}

/*
 * display current number of chromosomes at bottom of screen
 */
static dispnchroms(numchroms)
{
	static int numchwas;
	intens(0);
	countdisp(numchwas,1);
	intens(1);
	countdisp(numchroms,1);
	numchwas = numchroms;
}


kgetc(in)
FILE *in;
{
	char c;
	read(fileno(in),&c,1);
	return((int)c);
}


