/*
 *		A B E . C   --   
 *
 *  Written: David Burgess
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Date: 6th June 1989
 *	Modifications
 *
 *	0.5.5
 *	15 Jan 1991		dcb		Version 0.5.1 (Functions added) + init_scoresheet
 *							(Allows re-initialisation)
 *	14 Nov 1990		ih		Use alcalloc to Calloc alc
 *	26 Oct 1990		dcb		Version 0.5 (New allcontr, + error checks)
 *	10 Apr 1990		dcb		Version 0.4 (Recompile for new allcontr)
 *	02 Jan 1990		dcb		Global score renamed scores & 0.3 remove init_environ
 *	12 Dec 1989		dcb		errno for V2.2
 *  29 Jun 1989		SEAJ	Removed calls to setcolour and makecellhead, and 
 *							initialise environment.
 *	28 Jun 1989		SEAJ	Added call to aberr_debug.
 */
#define INIFILE
#include <cmdsys.h>
#include <module.h>
#include <procid.h>
#include <cfipp.h>
#include <allcontr.h>
#include <kcontrol.h>
#include <kyy.h>
#include <slides.h>
#include <aberrations.h>
#include <sgstat.h>

#define TRUE (1)
#define FALSE (0)

/*
 *	System data structures etc
 */
struct kcontrol *kcont = NULL;

/*
 *	Externals
 */
extern struct entry *Freeentrylist();
extern struct score_sheet *read_score();
extern struct entry *elist, *estart;
extern char *Freeptr();
extern print_score();
extern DDGS *ddgsopen();
extern int (*setexception())();
extern int memexitfn();
extern int errno;

/*
 *	Files
 */
FILE *debug_channel = NULL;				/* system debug file */
FILE *debugfp = NULL;					/* Miproc comms Debug filepointer */

/*
 *	Globals
 */
int	(*quit_fn)() = NULL;		/* Call this when we get ^c */
int dispbanner(), aberr_score(), aberr_init();
struct formdata *Freebayformdata(), *Freebayformstruct();
int quit=0;
int slave_mode;					/* Forked from another process */
int first=TRUE;					/* Controls certain first time initialisation */
char *abeid = "0.5.5";			/* The version number */
struct sgbuf savestdin;
struct score_sheet *orig_score, *scores=NULL; /* Original, and current scoresheet */
struct score_sheet *bayscoresheets[MAXBAYS];	/* One per bay */
struct bayinfo bay_info[MAXBAYS];				/* One per bay */
int bay = 0;

/*
 *	Statics
 */
static struct pframe mainframe;

/*
 *	I N T E R R U P T  -- interrupt catcher
 */
interrupt(sig)
int sig;
{
	char c;

	if (quit) return;
	quit = 1;
	if (alc->debug)
		printf("ABE - seen sig %d\n",sig);
	if (quit_fn) quit_fn();
	else def_quit_fn();
	quit = 0;
	return;
}

/*
 *	D E F _ Q U I T _ F N -- Default quit function, call this when
 *	we get ctrl-C if no other quit function set up
 */
def_quit_fn()
{
	char c;

	while (_gs_rdy(fileno(stdin)) > 0)
		read(fileno(stdin),&c,1);	/* Flush input */
	screengo(ENDLINE,0);
	printf("Quit seen - Do you wish to stop ABE ? ");
	while (_gs_rdy(fileno(stdin)) <= 0); /* Wait for input */
	read(fileno(stdin),&c,1);
	if ((c == 'y') || (c == 'Y')) {
		shutdown();
	}
} /* end def_quit_fn */

/*
 *	S H U T D O W N  --  Shut all down
 */
static
shutdown()
{
	tidyworld();
	fprintf(stderr, "ABE: shutdown\n");
	alldone(1);
}

/*
 *	T I D Y W O R L D  --
 */
static
tidyworld()
{
	cook(fileno(stdin), &savestdin);
	system("tmode echo eof=4");
}

/*
 *	E X C E P T I O N  --  Exception traps come here
 *
 */
exception(exc,addr)
int		exc, addr;
{
	procid	pd;
	int		fd, time, date, day, tick;
	char	buffer[100];

	gprdsc(getpid(),&pd);
	sprintf(buffer,"(V%s) Exception %d, addr 0x%x, base 0x%x (offset 0x%x)\n",
		abeid, exc, addr, pd._pmodul, addr - (int)pd._pmodul);
	errorlog(buffer);
	shutdown();
}

/*
 *	M A I N  --  Abe main loop
 */
main(argc,argv, argp)
int argc; char **argv, **argp;
{
	char	*swptr, **targv, buf[80];
	int		i, targc;
	struct association *sf;
	struct mult_slide *bp;

	/*sleep(1) ;*/ /* allow caller to settle down */
	_gs_opt(fileno(stdin), &savestdin);	/* Keep a copy for cleanup later */
	intercept(interrupt);
	setexception(exception);
	Alloc_exit(memexitfn);
	targc = argc; targv = argv;

	slave_mode = FALSE;
	argc--; argv++;			/* dump process name */
	while (argc > 0) {
		if (*argv[0] == '-') {
			swptr = *argv;
			*argv = NULL;
			while (*(++swptr)) {
				switch (*swptr) {
				case '?':	print_options();
							exit(0);
				case 'v':	printf("abe: version %s", abeid);
							exit(0);
				case 's':	slave_mode = TRUE;
							alc = (struct allkontr *)atoi(*++argv);
							if (alc->debug) printf("abe: alc = 0x%x\n");
							*argv = NULL; argc--;
							break;
				default:	printf("abe: - unknown option\n");
				}
			}
		}
		argv++; argc--;
	}

	if (alc->debug) {
		argc = targc; argv = targv;
		while(--argc) printf("%s\n", *argv++);
	}
	argc = targc; argv = targv;
	argc--; argv++;			/* dump process name */
	
/*
 *	Init global structures
 */
 	setmemalloc(32*1024); /* 32K */
	if (!slave_mode) {
		alc=alcalloc();
		alc->cfipp = (struct cfipparams *) Calloc(1, sizeof(struct cfipparams));
		sysinit(alc);
		getwd(alc->dir, 40);	/* Set current dir (sysinit ?) */
/*
 *	Start up menu handler
 */
		mouseport(alc->mouseterm);	/* Where we want the mouse to be */
		muini(stdout);				/* Init cmdsys + mouse */
		alc->m2s = mukont->mouss;	/* Get mouse struct from cmdsys */
	}

	set_header_fn(dispbanner);	/* Display routine instead of up menu line */
	pastmu = 0;				/* do recall up-menus on command string */
	uplow = 0;				/* do not identify upper-lower case */
	bay=1;
/*	dojob = myjobs;	*/		/* may be called from 'lead' at request of slave */

/*
 *	Start up debugging (debugfile.abe)
 *	2 different channels - debugfp for miproc
 *						 - debug_channel for system
 */
	screengo(ENDLINE, 0);
	sprintf(buf, "%s.abe", alc->debugfile);
	if (alc->mipdebug) {
		fprintf(stderr, "abe: mipdebug on. Opening %s\n",buf);
		if (debugfp = fopen(buf, "w") ) {
			fprintf(debugfp, "abe: miprog debugging information\n");
		} else {
			fprintf (stderr, "Failed to open debug file error %d\n" ,errno);
		}
		fflush(debugfp);
	}
 	if (alc->debug) {
 		if (debugfp)
 			debug_channel = debugfp;
 		else {
			fprintf(stderr, "abe: normal debug on. Opening %s\n",buf);
			if ((debug_channel = fopen(buf, "w")) ){
				fprintf(debug_channel,"abe: General debug information\n");
 			} else {
				fprintf (stderr,"Failed to open debug file error %d\n" ,errno);
			}
		}
		fflush(debug_channel);
		Alloc_debug(debug_channel);
		aberr_debug(debug_channel);
	}
	if (alc->debug) {
		fprintf(debug_channel, "Combifip version = %s\n", alc->cfipp->program);
	}
	if (slave_mode) {
		slavemain(targv[1], aberr_score, aberr_init);
	} else {
	 	bp = alc->frsbay;
 		for(i = 0; i < alc->maxbays; i++) {
			strcpy(bp->nameids.case,"UNSET");
			*bp->nameids.slide = 0;
 			bp++;
 		}
		aberr_init();
		aberr_score();
		alldone(0);
	}
}

/*
 *	A B E R R _ S C O R E -- Call up the form menu for scoring
 */
aberr_score()
{
	if (slave_mode) printf("Aberration scoring\n");
	while (form());
}

/*
 *	A B E R R _ I N I T --	Initialise for scoring, using this function has the
 *							side effect that the original scoresheet must now
 *							be a global (call it orig_score)
 */
aberr_init()
{
	int i;

	init_scoresheet(SCORESHEET);
	if (bay == 0) {
		for (i=0; i<MAXBAYS; i++) {
			bay_info[i].printsave = TRUE;	/* List has been printed/saved*/
			bay_info[i].scored = FALSE;		/* List has been scored */
			bay_info[i].elist = NULL;		/* No list created yet */
			bay_info[i].fd = NULL;			/* No form data yet */
		}
	}
	bay = baynn(alc->curbay)-1;	/* Find out what bay we're on */
}

/*
 *	I N I T _ S C O R E S H E E T -- Initialise the scoresheet
 */
init_scoresheet(fn)
char *fn;
{
	struct mult_slide *cb;
	struct score_sheet *ss;
	int i, ok=FALSE;

	printf("Reading scoresheet file %s\n", fn);
	if ((ss = read_score(fn)) == NULL) {	/* Initialisation */
		fprintf(stderr,"Error reading scoresheet file");
		if (scores) fprintf(stderr,", old scoresheet retained\n");
		else fprintf("\n");
		return(-1);
	}
	if (scores) { /* Re-initialisation */
		if (alc->debug)	fprintf(debug_channel,"Re-init, freeing scoresheets\n");
		printf("Removing old scoresheet\n");
/*
 *	Check scores is one of orig_score, or bayscoresheets[0..3]
 */
		if (scores == orig_score) ok = TRUE;
		for (i=0;i<MAXBAYS;i++) {
			if (scores == bayscoresheets[i]) ok = TRUE;
		}
		if (!ok) errorprint("Current scoresheet is lost\n");
		elist = estart = bay_info[bay].elist = Freeentrylist(elist);
		Freebayinfo(bayscoresheets[bay]);	/* Free current */
		Freebayinfo(orig_score);			/* and default */
		initentrylist();					/* Initialise for new entries */
	}
	scores = orig_score = ss;				/* Assign scoresheet just read */
	cb = alc->frsbay;
	for (i=0; i<MAXBAYS; i++) {
#ifdef GGG
		cb->newlist = TRUE; /* Ho hum... */
		cb++;
#endif
/*
 *	Reset those bays that now have NULL scoresheets
 */
		if (bayscoresheets[i] == NULL) bayscoresheets[i] = orig_score;
	}
}

#ifdef GGG
/*
 *	A S S I G N F O R M --	Assign space for form structure, and the assoc,
 *							and menutxt strings
 */
struct association *
assignform(assoc, menutxt)
char *assoc, *menutxt;
{
	struct association *form;
	char *strsave();

	form = (struct association *)Calloc(1, sizeof(struct association));
	form->assoc = strsave(assoc);
	form->menutxt = strsave(menutxt);
	form->data = NULL;
	form->next = NULL;
	return(form);
}
#endif

/*
 *	D I S P B A N N E R -- Display case/slide id. for banner like kmr
 */
dispbanner()
{
char	fmt[40], strg[41];
struct	cyto_file *cfp;

	cfp = &alc->curbay->nameids;
	sprintf(fmt, "Case:%%-%dsSlide:%%-%dsCell:%%-4d",
		CASELENGTH, SLIDELENGTH);
	sprintf(strg, fmt, cfp->case, cfp->slide, cfp->cellno);
	bigmessage(0, 0, strg);	/* Display enlarged message at the top. */
}

#ifdef GGG
/*
 *	U P L I S T
 *
 */
static struct chromosome ** 
uplist() {
	return ( (struct chromosome **) 
				Calloc ( MAXOBJS,sizeof(struct chromosome *)));
}

/*
 *	U P C O N T R O L S  --  Generate and initialise kcontrol structure
 *
 */
static
upcontrols() 
{
	kcont = (struct kcontrol *)Calloc(1, sizeof(struct kcontrol));
#ifdef NOTYET
	subinitkc(kcont);			/* Initialise slaves etc */	
	initkcontrol(kcont);
#endif
	alc->kcn = kcont;			/* make kcontrol structure available through
										   'allcontrol' system */
/*
 *	Now overide defaults with sensible values
 */
	kcont->ool = uplist();
	kcont->rol = uplist();
	kcont->bol = (struct object **) uplist();
	kcont->mol = uplist();
	kcont->kol = uplist();
/*	kcont->grol1 = NULL;		/ * Becomes keeplist in karyotyping. */
	kcont->f = &mainframe;
	kcont->ivf = (struct pframe *) Calloc ( MAXOBJS,sizeof(struct pframe));
	kcont->acl = (SMALL *)Calloc (MAXOBJS,sizeof(SMALL));
	kcont->seat = alc->seat - 1;
	kcont->terminal = alc->ourterm;
	kcont->mouse = alc->mouseterm;
	kcont->ddgschannel = alc->ddgschannel;
	switch (alc->config) {
		case CONF_FIP:
			kcont->FIP = DIGFIP;
			break;
		case CONF_TV:
			kcont->FIP = DIGTV;
			break;
		default:
			kcont->FIP = DIGFIP;
	}
#ifdef DEBUG
	if (alc->debug) {
		printf("kcont - 0x%x seat %d\n",kcont, kcont->seat);
		printf("kcont - mouse %s terminal %s\n",kcont->mouse, kcont->terminal);
	}
#endif
}

#endif

/*
 *	P R I N T _ O P T I O N S -- Print options switches & syntax
 */
print_options()
{
	printf("%s%s%s%s%s%s%s",
	"Syntax:\tabe <-v?> (stand-alone mode)\n",
	"\t\t<cmdsys ptr> -s <alc ptr> (slave mode)\n",
	"Function:\taberration scoring\n",
	"Options:\n",
	"\t-v - version number\n",
	"\t-s - start in slave mode\n",
	"\t-? - print this help list\n");
}

/*
 *	F R E E B A Y I N F O --  Free all relevant bay info
 *	1) Free scoresheet having regard to other bays that may use same scoresheet.
 *	NOTE: Root scoresheet also freed, allowing new one to be read by read_score.
 *	For those bayscoresheet entries that also pointed to the now freed scoresheet
 *	1)	Scoresheet pointers are NULLed
 *	2)	Scored, and printed flags are reset
 *	3)	The entrylist is freed off
 *	4)	The form data, and data structures are removed
 */
Freebayinfo(ss)
struct score_sheet *ss;
{
	struct score_sheet *tmp;
	int i;

	if (ss == NULL) return(NULL);
	tmp = ss;
	Freescoresheet(ss);
	Freeptr(ss);
	if (orig_score == tmp) orig_score = NULL;
	for (i=0; i<MAXBAYS; i++) {
		if (bayscoresheets[i] == ss) {
			bayscoresheets[i] = NULL;
			bay_info[i].scored = FALSE;
			bay_info[i].printsave = TRUE;
			bay_info[i].elist = Freeentrylist(bay_info[i].elist);
			Freebayformdata(bay_info[i].fd); /* Remove form data */
			bay_info[i].fd = Freebayformstruct(bay_info[i].fd); /* Remove form structures - data freed by Freebayformdata  */
		}
	}
}

/*
 *	F R E E B A Y F O R M D A T A --	Free the form data bits held
 *	in the bay_info structure.
 */
struct formdata *
Freebayformdata(fd)
struct formdata *fd;
{
	for (;fd; fd = fd->next) {
		if (alc->debug) fprintf(debug_channel, "Freebayformdata: Free at 0x%x\n",
			fd->data);
		fd->data = Freeptr(fd->data);
	}
	return(NULL);
}

/*
 *	F R E E B A Y F O R M S T R U C T  --	Free the bay info form data structures
 */
struct formdata *
Freebayformstruct(fd)
struct formdata *fd;
{
struct formdata *free_ptr;

	while (fd) {
		if (alc->debug)
			fprintf(debug_channel, "Freebayformstruct: Free at 0x%x\n", fd);
		free_ptr = fd;
		fd = fd->next;
		Freeptr(free_ptr);
	}
	return(NULL);
}


