/*
 *      A B E R R A T I O N . H  -- Header file for aberration package.
 *
 *
 *  Written: Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    2nd June 1989.
 *
 *  Modifications
 *
 *	18 Jan 1991		dcb		Add bayinfo structure
 *	10 Jan 1991		dcb		Move MAXOPTS to read_score.c and define properly
 *	09 Jan 1991		dcb		Add FUNCTION reserved word, and KEYFUNC keyword
 *							Define function stucture and add to scoresheet.
 *	26 Jul 1989		dcb		SCOREFILE -> SCORESHEET
 *	 5 Jul 1989		dcb		Default scorefile to /dd/params
 *  28 Jun 1989		SEAJ	Added new fields abmenu, acmenu and opt.
 *	26 Jun 1989		SEAJ	Made NONE into NOSEP
 *
 */
#define VERSION 1
#define SCORESHEET "/dd/params/scoresheet" /* Default scoresheet */
#define SCOREDIR   "/dd/params"				/* Default directory */
#define SCOREFN    "scoresheet"				/* Default file name */
#define NOSEP 0
#define SINGLE 1
#define DOUBLE 2
#define MAXCOLS 25
#define NCOLS 4
#define LINELENGTH 130
#define MAXBAYS 4	/* Do proper later */

/*
 *	Keyword definitions
 */
#define NAME	   	0
#define TITLE  		1
#define HEADER 		2
#define LEVEL1 		3
#define LEVEL2		4
#define FORM   		5
#define ABERR 		6
#define ADD   		7
#define ALLBOX 		8
#define BOXED  		9
#define VER			10
#define NEWLINE 	11
#define SEP			12
#define MENUTITLE 	13
#define INFO		14
#define ACTION		15
#define ABMENU		16
#define ACMENU		17
#define KEYFUNC		18

/*
 *	Reserved word definitions - Match to definition in form.c
 */
#define CASE		0
#define	SLIDE		1
#define	CELLNO		2
#define	PATNAME		3
#define	OPID		4
#define	DATE		5
#define	LAB			6
#define	POSN		7
#define	XPOS		8
#define	YPOS		9
#define	BLANK		10
#define	NONE		11
#define	TOTAL		12
#define	NEXT		13
#define	PREV		14
#define	COMMENT		15
#define MINUS		16
#define RESET		17
#define REFRESH		18
#define	ZERO		19
#define	ADJUST		20
#define	FUNCTION	21
#define	EXIT		22

/*
 * S C O R E _ S H E E T
 */
struct score_sheet {
	short				version;		/* version number */
	char				*name;			/* overall title for score sheet */
	char				*menuname;		/* Title banner for scoring menu */
	short 				boxed;			/* 0, 1, 2 - no box; box, allbox */
	struct scoremenu	*abmenu;		/* aberration menu layout information */
	struct scoremenu	*acmenu;		/* actions menu layout information */
	struct banner		*title;			/* title pointer */
	struct banner		*header;		/* header pointer */
	struct association	*form;			/* association pointer */
	struct level		*level1;		/* level 1 pointer */
	struct level		*level2;		/* level 2 pointer */
	struct aberration	*aberr;			/* aberration pointer */
	struct information	*info;			/* information pointer */
	struct operation	*action;		/* actions pointer */
	struct function		*func;			/* Function definitions */
};

/*
 * B A N N E R
 */
struct banner {
	int 				width;			/* total field width */
	short				newline;		/* newline following ? */
	short				vert;			/* following vertical separator */
	short 				horiz;			/* following horizontal separator */
	char				*assoc;			/* associated abbreviation for 
										   metaphase list text */
	char				*txt;			/* text to print */
	char				*data;			/* real information */
	struct banner		*next;			/* pointer to next header */
};
/*
 * A S S O C I A T I O N
 */
struct association {
	short				opt;			/* Data is optional (0) or not (1) */
	char				*assoc;			/* associated abbreviation for 
										   metaphase list text */
	char				*data;			/* Form data, from form menu, or */
										/* metaphase list */
	char 				*menutxt;		/* menu line text */
	struct association	*next;			/* pointer to next association */
};
/*
 * L E V E L
 */
struct level {
	int					col;			/* column number */
	short				vert;			/* following vertical separator */
	short				horiz;			/* following horizontal separator */
	short				total;			/* total for column? */
	int					width;			/* total field width */
	char				*assoc;			/* associated abbreviation for metaphase list text */
	char				*txt;			/* text to print */
	struct level		*next;			/* pointer to next level */
};
/*
 * A B E R R A T I O N
 */
struct aberration {
	char				*key;			/* key press */
	char				*shortstr;		/* short text for column printing */
	int					col;			/* associated column number */
	char				*longstr;			/* full aberration name for use in menu */
	struct aberration	*next;			/* pointer to next aberration */
};
/*
 * E N T R Y
 */
struct entry {
	short				col;			/* column position */
	short				printed;		/* has it been printed ? */
	short				num;			/* Number of this type of aberration */
	char				*txt;			/* text to be put into column */
	struct entry		*next;			/* next entry for this cell */
	struct entry		*foll;			/* following cell's first entry */
	struct entry		*prev;			/* Previous cell's first entry */
};

/*
 *	I N F O R M A T I O N
 */
struct information {
	char	*assoc;					/* Associated abbreviation for info */
	char	*screentxt;				/* Screen text to show */
	int		x, y;					/* x, y position on screen */
	struct information *next;		/* Next in list */
};

/*
 *	O P E R A T I O N
 */
struct operation {
	char			*key;		/* Keypress(es) for this action */
	char			*assoc;		/* Associated abbreviation for this action */
	char			*menutxt;	/* Menu line text */
	struct operation *next;		/* Next in list */
};

/*
 * S C O R E M E N U
 */
struct scoremenu {
	short			row,		/* menu row position */
					nrows,		/* number of rows */
					col[NCOLS];	/* column positions */
};

/*
 *	F U N C T I O N -- Define a function between columns
 */
struct function {
	short	col;		/* Column number this function refers to */
	char	*def;		/* Function definition (arithmetic/logical expr) */
	char	*true;		/* Text to print if function is TRUE */
	char	*false;		/* Text to print if function is FALSE */
	struct function *next;	/* Next in list */
};

/*
 *	B A Y I N F O -- Bay information structure
 */
struct bayinfo {
	int printsave;
	int scored;
	struct entry *elist;
	struct formdata {
		char *data;
		struct formdata *next;
	} *fd;		/* Form data list */
};

