/*
 *		F O R M . C   --	Enter information for aberration scoring
 *
 *  Written: D C Burgess
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Date: 6th June 1989
 *	Modifications
 *
 *	18 Jan 1991	dcb		Add bay_info structure array
 *	16 Jan 1991	dcb		Move Freeformdata to free_structs.c
 *	15 Jan 1991	dcb		Introduce select scoresheet option
 *	09 Jan 1991 dcb		Add FUNCTION case to getrestext, and Function reserved word
 *	22 Nov 1990 ih		In form() check alc->debug before fprintf(debug_channel
 *	26 Oct 1990 dcb		IRS -> Cytoscan
 *	02 Jan 1990	dcb		Global score renamed scores
 */
#include <cmdsys.h>
#include <allcontr.h>
#include <slides.h>
#include <aberrations.h>

/*
 *	Defines
 *	NOTE: The version id is defined in abe.c as string "abeid"
 */
#define HOME_MENU_NAME "Cytoscan Aberration Scoring   Version "
#define MAXFORM 10		/* Max no. of form entries */
#define MAXSTR	80		/* Max input string length */
#define MAXBUF	80		/* Max. length of form menu info */
#define SET		(1)
#define NOTSET	(0)
#define TRUE	(1)
#define FALSE	(0)
#define LEFT_ADJ	0	/* Left adjustment of numeric fields in getrestext */
#define RIGHT_ADJ	1	/* Right adjustment of numeric fields in getrestext */

/*
 *	Reserved words - Order is important and matches
 *	reserved word definitions in aberrations.h
 */
static reserved[] = {
	"Case", "Slide", "Cellno", "Patname", "Opid", "Date", "Lab",
	"Posn", "Xpos", "Ypos", "Blank", "None", "Total", "Next", "Prev",
	"Comment", "Minus", "Reset", "Refresh",	"Zero", "Adjust", "Function", "Exit"
};
#define RESTOT sizeof(reserved)/sizeof(char *)

/*
 *	Externals
 */
extern struct score_sheet *read_score();
extern print_score();
extern struct formdata *Freebayformdata(), *Freebayformstruct();
extern FILE *debug_channel;				/* system debug file */
extern FILE *debugfp;					/* Miproc comms Debug filepointer */
extern int rec_no, rank_no, tot_score;
extern struct allkontr *alc;
extern struct entry *elist, *estart;
extern char *abeid;
extern int slave_mode;					/* Forked from another process */
extern struct score_sheet *orig_score, *scores, *bayscoresheets[MAXBAYS];
extern struct bayinfo bay_info[];
extern int bay;

/*
 *	Statics
 */
static int setformdata(), ab_score(), pr_score(), rd_metlist();
static int mic_control(), wr_scorelist(), rd_scorelist();
static select_scoresheet();

/*
 *	Globals
 */
/* int shonum(); */
int fipops();
char *strsave(), *setformmenu(), *getrestext(), *Freeptr(), *home_menu_id();

/*
 *	F O R M  --  Form filling menu
 */
form()
{
	int	(*linkerbug) ();
	int i;
	int not_exit = 1;
	char locstr[MAXFORM][MAXSTR];

	if (alc->debug) fprintf(debug_channel, "form: in\n");
	set_bay_info(locstr);
	pr_scoresheetptrs();	/* A debug function */
	if (alc->curbay->newlist) {
		if (slave_mode) printf("New metaphase list has been read/created\n");
		setnewscore();					/* Set appropriate scoresheet */
#ifdef GGG
		for(i=0; i<MAXFORM; i++) locstr[i][0]='\0';
		setformentry(scores, locstr);	/* Set any entries so far defined */
#endif
		initglobals(scores);				/* Initialise scoring globals */
		bay_info[bay].printsave = TRUE;
		bay_info[bay].scored = FALSE;
	}

	while (not_exit) {
		for(i=0; i<MAXFORM; i++) locstr[i][0]='\0';
		setformentry(scores, locstr);	/* Set any entries so far defined */
		downmenu();
		not_exit = formmenu(locstr);
		if (alc->debug)
			fprintf(debug_channel, "Returned from formmenu with %d\n", not_exit);
		upmenu();
		if (not_exit == 0)
			if (checkbayssaved() == FALSE) not_exit = 1; /* Don't exit */
	}
	if (alc->debug) fprintf(debug_channel, "form: out\n");
	return(0);
}

/*
 *	F O R M M E N U -- Set up the form menu
 */
formmenu(locstr)
char locstr[][MAXSTR];
{
 	int i, n, refresh0, r0, refresh1, refresh2, refresh3, exit;
	char *cmdstr[MAXFORM];
 	struct association *sf;

	newmenu (home_menu_id(HOME_MENU_NAME, abeid));
	refresh0 = cmmd ("select scoresheet;%-+d%f", &r0, select_scoresheet);
				mouse(LEFTBT);
	for(i=0, sf = scores->form; ((i<MAXFORM) && sf); i++, sf = sf->next) {
		if (alc->debug)
			fprintf(debug_channel, "Form %d, string addr 0x%x\n", i, locstr[i]);
		cmdstr[i] = setformmenu(sf, locstr[i]); /* Set menu text */
		cmmd (cmdstr[i], sf, locstr[i], setformdata);
	}
	cmmd ("score list;%-+d%f", scores, ab_score); mouse(MIDBT);
	cmmd ("print scores;%-+d%f", scores, pr_score); mouse(RIGHTBT);
	cmmd ("file scored list;%-+d%f", scores, wr_scorelist); mouse(LONGRB);
	if (!slave_mode)
		refresh1 = cmmd ("read metaphase list;%-+d%f", locstr, rd_metlist);
	refresh2 = cmmd ("retrieve scored list;%-+d%f", locstr, rd_scorelist);
	refresh3 = cmmd ("microscope control;%-+d%f", locstr, mic_control); mouse(LONGMB);
	exit = cmmd ("exit"); mouse(LONGLB);

	while ((n = thismu()) != exit) {
		if ((n == refresh0) && r0) break;
		if (n == refresh1) break;
		if (n == refresh2) break;
		if (n == refresh3) break;
	}
	while(i) {
		if (alc->debug) fprintf(debug_channel, "Free cmd line %d @ 0x%x\n",
			i, cmdstr[i-1]);
		Free(cmdstr[--i]);
	}
	if (n == exit) return(0);
	else {
		return(1);
	}
} /* end formmenu */

/*
 *	R D _ M E T L I S T -- Local function to call getlist
 */
static int
rd_metlist(locstr)
char locstr[][MAXSTR]; /* or (*locstr)[MAXSTR] */
{
	struct association *sf;
	struct banner *ban;
	int i;

	if (!bay_info[bay].printsave) {
		errorprint("Current scoresheet not printed/saved\n");
		if (!go_on()) return(-1);
	}
	if (!getlist()) return(-1);
	setnewscore();			/* Set up the scoresheet */
	for(i=0; i<MAXFORM; i++) locstr[i][0]='\0';
	for (ban = scores->title; ban; ban = ban->next) ban->data = NULL;
	for (ban = scores->header; ban; ban = ban->next) ban->data = NULL;
	setformentry(scores, locstr);
	initglobals(scores);		/* Initialise scoring globals */
	bay_info[bay].printsave = TRUE;
	bay_info[bay].scored = FALSE;
	setdisplay(NOTBUST); /* Since it will be refreshed anyway */
	return(0);
}

/*
 *	S E L E C T _ S C O R E S H E E T -- Allow user to change scoresheet file
 */
static int
select_scoresheet(rflag)
int *rflag;
{
	static char ss_dir[40]=SCOREDIR, ss_file[40]=SCOREFN;
	char orig_dir[40], orig_file[40], fn[80], *Malloc();
	int exit, cmd, mod=FALSE;

	if (alc->debug) {
		fprintf(debug_channel, "Select scoresheet\n");
		fprintf(debug_channel, "Scoresheet directory=%s, file=%s\n",
			ss_dir, ss_file);
	}
	if (checkbayssaved() == FALSE) {
		*rflag = FALSE;
		return(0);
	}

	strcpy(orig_dir, ss_dir);
	strcpy(orig_file, ss_file);

	newmenu ("Select scoresheet");
	cmmd ("set scoresheet directory;:k40%s", ss_dir);
	cmmd("set scoresheet file;:k40%s", ss_file);
	exit = cmmd ("exit"); mouse(LONGLB);

	while (TRUE) {
		cmd = thismu();
		if (strcmp(orig_file, ss_file) ||
			((cmd == exit) && strcmp(orig_dir, ss_dir))) {
/* Changed file or directory changed, and exit - Read new scoresheet */
			strcpy(fn, ss_dir); strcat(fn, "/"); strcat(fn, ss_file);
			if (init_scoresheet(fn) < 0) {	/* Re-Initialisation */
				strcpy(ss_dir, orig_dir);
				strcpy(ss_file, orig_file);
				setdisplay(BUST);
			} else {
				strcpy(orig_dir, ss_dir);
				strcpy(orig_file, ss_file);
				mod = TRUE;
				setdisplay(NOTBUST); /* Since it will be refreshed anyway */
				putprompt();
			}
		}
		if (cmd == exit) break;
	}
	if (mod == TRUE) setdisplay(NOTBUST);
	*rflag = mod;	/* Redraw display if file was modified */
	return(0);
}

/*
 *	W R _ S C O R E L I S T -- Local function to call file_score
 *								for filing a scored list
 */
wr_scorelist(score)
struct score_sheet *score;
{
	if (!bay_info[bay].scored) {
		errorprint("Current list has not been scored\n");
		return;
	}
	file_score();
	bay_info[bay].printsave = TRUE;
}

/*
 *	R D _ S C O R E L I S T -- Local function to call retrieve_score
 */
static int
rd_scorelist(locstr)
char locstr[][MAXSTR]; /* or (*locstr)[MAXSTR] */
{
	int i;

	if (!bay_info[bay].printsave) {
		errorprint("Current scoresheet not printed/saved\n");
		if (!go_on()) return(-1);
	}
	if (retrieve_score() == 0) { /* 0 is OK return */
		printf("Scored list read OK\n");
		for(i=0; i<MAXFORM; i++) locstr[i][0]='\0';
		setformentry(scores, locstr);
		bay_info[bay].printsave = TRUE;
		bay_info[bay].scored = TRUE;		/* Must have been to be saved !! */
	} else {
		printf("Error in reading scored list\n");
	}
	setdisplay(NOTBUST); /* Since it will be refreshed anyway */
}

/*
 *	M I C _ C O N T R O L -- Local function to call fipops
 */
static int
mic_control(locstr)
char locstr[][MAXSTR]; /* or (*locstr)[MAXSTR] */
{
	int i, bay_out;

	if (alc->debug) {
		for (i=0; i<MAXBAYS;i++) {
			fprintf(debug_channel, "bay_info[%d] = {%d, %d, 0x%x, 0x%x}\n",
			i, bay_info[i].printsave, bay_info[i].scored,
			bay_info[i].elist, bay_info[i].fd);
		}
		fprintf(debug_channel, "Old bay = %d\n", bay+1);
	}

	downmenu();
	fipops();
	upmenu();
	set_bay_info(locstr);
	if (alc->debug) {
		fprintf(debug_channel, "mic_control: out, scores = 0x%x\n", scores);
		fprintf(debug_channel, "New bay = %d\n", bay+1);
	}
	setdisplay(NOTBUST); /* Since it will be refreshed anyway */
}

/*
 *	A B _ S C O R E -- Local function to call scoring menu
 */
static int
ab_score(score)
struct score_sheet *score;
{
	struct association *sf;
	int data_set = SET;
/*
 *	Check for unfilled forms
 */
	for (sf = score->form; sf; sf = sf->next) {
		if ((sf->data == NULL) && (sf->opt)) { 
			fprintf(stderr, "Error: %s not set\n", sf->menutxt);
			data_set = NOTSET;
		}
	}
	if (alc->curbay->startptr == NULL) {
		errorprint("Error: No metaphase list read\n");
		data_set = NOTSET;
	}
	if (data_set == SET) {
		setcprcd(&rec_no, &rank_no); /* Drive to position */
		aberrationscore(score);
		setdisplay(BUST);
		bay_info[bay].printsave = FALSE;
		bay_info[bay].scored = TRUE;
	}
}

/*
 *	P R _ S C O R E -- Local function for calling print_score
 */
static int
pr_score(score)
struct score_sheet *score;
{
	if (alc->curbay->startptr == NULL) {
		errorprint("Please read metaphase list first\n");
		return;
	}
	if (!bay_info[bay].scored) {
		errorprint("Current list has not been scored\n");
		return;
	}
/*
 *	Get associated metaphase list information, or form information
 *	and copy to	banner struct
 */
	printscoreform(score->form); /* A debug function */
 	makebanner(score->title, score);
 	makebanner(score->header, score);
	print_score(score, elist);
	bay_info[bay].printsave = TRUE;
}

/*
 *	S E T F O R M D A T A -- Set form data field
 */
static int
setformdata(form, str)
struct association *form;
char *str;
{
	if (alc->curbay->startptr == NULL) {
		errorprint("Please read metaphase list first\n");
		*str = '\0';	/* Make it appear as if data not set */
		return;
	}
	form->data = strsave(str);
	bay_info[bay].printsave = FALSE;	/* New data not saved */
}

/*
 *	S E T F O R M M E N U -- Set form menu line, including cmdsys format info
 */
char *
setformmenu(sf, str)
struct association *sf;
char *str;
{
	char buf[MAXBUF+15];

	if (strlen(sf->menutxt) > MAXBUF) {
		errorprint("Form string too long - ignored\n");
		return(NULL);
	}
	if (sf->menutxt == NULL) return(NULL); /* Skip null entries */
	strcpy(buf, sf->menutxt);
	if (findresindex(sf->assoc) > 0) {
		strcat(buf, ";:k40%-+d%-S%f\000"); /* Reserved, display but don't set */
	} else {
		strcat(buf, ";:k40%-+d%S%f\000"); /* Can set */
	}
	if (alc->debug)
		fprintf(debug_channel, "form: format string is %s\n", buf);
	return(strsave(buf));
} /* end setformmenu */

/*
 *	S E T F O R M E N T R Y -- Set form entry from known data
 */
setformentry(score, str)
struct score_sheet *score;
char str[][MAXSTR];
{
	int i;
	char *txt;
	struct association *sf;

	for(i=0, sf = score->form; (i<MAXFORM && sf); i++, sf = sf->next) {
		txt = getrestext(findresindex(sf->assoc), LEFT_ADJ, 0);
		if (txt) {
			sf->data = strsave(txt); /* Because it gets freed later */
			strcpy(str[i], txt);
			if (alc->debug)
				fprintf(debug_channel, "setformentry: Form %d, data %s, to addr 0x%x\n",
					i, sf->data, str[i]);
		} else if (sf->data) { /* Possibly read from score file */
			strcpy(str[i], sf->data);
			if (alc->debug)
				fprintf(debug_channel, "setformentry: Form %d, data %s, to addr 0x%x\n",
					i, sf->data, str[i]);
		}
	}
} /* end setformentry */

/*
 *	F I N D R E S I N D E X -- Find index for reserved word
 *	Returns -1, if word not found, or passed null pointer.
 */
int
findresindex(str)
char *str;
{
	int i;

	if (str == NULL) return(-1);
	i = findindex(str, reserved, RESTOT);
	if (alc->debug)
		fprintf(debug_channel, "findresindex: %s as %d\n", str, i);
	if (i < RESTOT)	return(i);
	else return(-1);
}

/*
 *	M A K E B A N N E R --	Assign form info or met list info to
 *							the banner titles/headers
 */
makebanner(ban, score)
struct banner *ban;
struct score_sheet *score;
{
	struct association *sf;
	int sw;

	while (ban) {
		sw = findresindex(ban->assoc);
		switch (sw) {
			case CASE:
			case SLIDE:
			case CELLNO:
			case PATNAME:
			case OPID:
			case DATE:
			case LAB:
			case BLANK:
				ban->data = getrestext(sw, RIGHT_ADJ, 0);
				break;
			default:
				/*
				 *	Scan form list as well
				 */
	 			for (sf = score->form; sf; sf = sf->next)
 					if (strcmp(ban->assoc, sf->assoc) == 0)
 						ban->data = sf->data;
 				break;
		} /* end switch */
		ban = ban->next;
	}
}

/*
 *	G E T R E S T E X T --	Return pointer to reserved text string
 *							Returns NULL if string not found
 *	res = Reserved word to look for
 *	adj = 0 for left adjust, 1, for right adjust
 *	col = current column number (for FUNCTION only)
 */
char *
getrestext(res, adj, col)
int res, adj, col;
{
	struct mult_slide *cb;
	char *fmt;
	struct function *fun;

	cb = alc->curbay;
	switch(res) {
		case CASE:
			return(cb->nameids.case);
			break;
		case SLIDE:
			return(cb->nameids.slide);
			break;
		case CELLNO:
			fmt = (adj) ? "%4d" : "%-4d";
			return(itoa(fmt, cb->nameids.cellno));
			break;
		case PATNAME:
			return(cb->nameids.patnam);
			break;
		case OPID:
			return(alc->user);
			break;
		case DATE:
			return(alc->mydate);
			break;
		case LAB:
			return(alc->sitename);
			break;
		case POSN:
			fmt = (adj) ? "%6d,%6d" : "%-6d,%-6d";
			if (cb->currptr && cb->currptr->mlrec)
				return(itoa(fmt,
					cb->currptr->mlrec->xval, cb->currptr->mlrec->yval));
			break;
		case XPOS:
			fmt = (adj) ? "%6d" : "%-6d";
			if (cb->currptr && cb->currptr->mlrec)
				return(itoa(fmt, cb->currptr->mlrec->xval));
			break;
		case YPOS:
			fmt = (adj) ? "%6d" : "%-6d";
			if (cb->currptr && cb->currptr->mlrec)
				return(itoa(fmt, cb->currptr->mlrec->yval));
			break;
		case FUNCTION:
			fprintf(stderr, "getrestext called illegally\n");
			return("");
			break;
		case BLANK:
			return("");
			break;
		case TOTAL:
			fmt = (adj) ? "%4d" : "%-4d";
			return(itoa(fmt, tot_score));
			break;
		case COMMENT:
			return(docomment(estart));
			break;
		default:
			return(NULL);
			break;
	} /* end switch */
	return(NULL);
} /* end getrestext */

/*
 *	P R I N T S C O R E F O R M -- Print linked list of forms
 */
printscoreform(sf)
struct association *sf;
{
	if (alc->debug) {
		fprintf(debug_channel, "Score form\n----------\n");
		while(sf) {
			fprintf(debug_channel, "assoc=%s, ",sf->assoc);
			fprintf(debug_channel, "data=%s, ",sf->data);
			fprintf(debug_channel, "menutxt=%s, ",sf->menutxt);
			fprintf(debug_channel, "next=0x%x\n",sf->next);
			sf = sf->next;
		}
	}
}

/*
 *	E R R O R P R I N T -- Print an error message at ENDLINE
 */
errorprint(message, a1, a2, a3)
char *message;
int a1, a2, a3;
{
	int nargs=0;
	char *s;

	screengo(ENDLINE, 1);
	for(s=message; *s; s++) if (*s == '\%') nargs++;
	switch(nargs) {
		case 0:	printf( message);
				break;
		case 1:	fprintf(stderr, message, a1);
				break;
		case 2:	fprintf(stderr, message, a1, a2);
				break;
		default: fprintf(stderr, message, a1, a2, a3);
				break;
	}
}

/*
 *	H O M E _ M E N U _ I D -- Form home menu id from name, and version id
 */
char *
home_menu_id(name, vid)
char *name, *vid;
{
	int size;
	char *rs, *Malloc();

	size = strlen(name) + strlen(vid) + 1;
	rs = Malloc(size);
	strcpy(rs, name);
	strcat(rs, vid);
	return(rs);
}

/*
 *	S E T N E W S C O R E -- Set new scoresheet if required
 */
setnewscore()
{
	if (alc->curbay->newlist) {
		if (bayscoresheets[bay] != orig_score) {	/* From scored list */
			Freescoresheet(bayscoresheets[bay]);	/* Free old one */
			Free(bayscoresheets[bay]);				/* and root scoresheet */
			Freebayformdata(bay_info[bay].fd);
			bay_info[bay].fd = Freebayformstruct(bay_info[bay].fd);
		} else {
/*			Freeformdata(scores->form);*/				/* Free old form data */
			Freebayformdata(bay_info[bay].fd);		/* Bay copy */
		}
		scores = bayscoresheets[bay] = orig_score;
		alc->curbay->newlist = 0;					/* Now not considered new */
	}
	pr_scoresheetptrs();
}

/*
 *	P R _ S C O R E S H E E T P T R S -- Print score sheet pointers
 */
pr_scoresheetptrs()
{
	int i;
	if (alc->debug) {
		fprintf(debug_channel, "Scoresheets:\norig_score 0x%x\nscores 0x%x\n",
			orig_score, scores);
		for(i=0; i<alc->maxbays; i++)
			fprintf(debug_channel, "bayscoresheets[%d] 0x%x\n",
				i, bayscoresheets[i]);
	}
}

/*
 *	C H E C K B A Y S S A V E D -- Check scoring info in all bays has been printed or saved
 *	Returns TRUE if yes, or OK to go on, else returns FALSE
 */
int
checkbayssaved()
{
	int i;

	for (i=0; i<MAXBAYS; i++) {
		if ((bay_info[i].scored == TRUE) &&
			(bay_info[i].printsave == FALSE)) {
			errorprint("Scores on bay %d not printed or saved\n", i+1);
			if (!go_on()) return(FALSE);	/* Don't change */
		}
	}
}

/*
 *	S E T _ B A Y _ I N F O -- Set estart, and bay if bay has changed
 */
set_bay_info(locstr)
char locstr[][MAXSTR]; /* or (*locstr)[MAXSTR] */
{
	int newbay, i;

	newbay = baynn(alc->curbay) - 1;	/* Possibly new bay */
	if (bay != newbay) { /* User has changed bays */
		if (alc->debug) fprintf(debug_channel, "Bay changed to %d\n", newbay+1);
		formtobay(scores, bay);					/* Save old form info */
		bay = newbay;
		elist = estart = bay_info[newbay].elist;/* Set current score entries */
		scores = bayscoresheets[newbay];		/* Set current scoresheet */
		baytoform(scores, newbay);				/* New form info */
		for(i=0; i<MAXFORM; i++) locstr[i][0]='\0';
		setformentry(scores, locstr);
	}
}

/*
 *	F O R M T O B A Y -- Move the data items of form to the bay_info structure
 */
formtobay(ss, bayno)
struct score_sheet *ss;
int bayno;
{
	struct association *sf;
	struct formdata **bfd, *fd;

	bfd = &(bay_info[bayno].fd);
	fd = *bfd;
	if (alc->debug)
		fprintf(debug_channel, "formtobay: bay %d, (fd=0x%x)\n", bayno+1, fd);
	for (sf = ss->form; sf; sf = sf->next, fd = fd->next) {
		if (fd == NULL)
			fd = *bfd = (struct formdata *)Calloc(sizeof(struct formdata), 1);
		fd->data = sf->data;
		if (alc->debug)
			fprintf(debug_channel, "formtobay: data \"%s\" saved\n", sf->data);
		bfd = &(fd->next);
	}
}

/*
 *	B A Y T O F O R M -- Move the data items from bay_info back to score->form
 */
baytoform(ss, bayno)
struct score_sheet *ss;
int bayno;
{
	struct association *sf;
	struct formdata *fd;

	fd = bay_info[bayno].fd;
	if (alc->debug)
		fprintf(debug_channel, "formtobay: bay %d, (fd=0x%x)\n", bayno+1, fd);
	
	for (sf = ss->form; sf; sf = sf->next) {
		if (fd != NULL) {
			sf->data = fd->data;
			fd = fd->next;
		} else {
			sf->data = NULL;	/* Remove the old entry */
		}
	}
}
