/*
 *	F R E E _ S T R U C T S  -- Frre scoring/scoresheet data structures
 *
 *  Written: D C Burgess
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1989)
 *
 *  Date:    16th January 1991 (Separated into separate module)
 *
 *	Modifications
 */
#include <stdio.h>
#include <allcontr.h>
#include <aberrations.h>
#ifdef GGG
#include <slides.h>
#include <ctype.h>
#include <fileheaders.h>
#include <preps.h>
#include <sgstat.h>
#include <errno.h>
#endif

/*
 *	External references
 */
extern struct allkontr *alc;
extern FILE *debug_channel;

/*
 *	Globals
 */
char *Freeptr();
struct score_sheet *Freescoresheet();
struct banner *Freebanner();
struct association *Freeassociation();
struct level *Freelevel();
struct aberration *Freeaberration();
struct information *Freeinformation();
struct operation *Freeoperation();

/*
 *	F R E E P T R --	Free pointer, taking account of NULL input
 */
char *
Freeptr(ptr)
{
	if (ptr != NULL) Free(ptr);
	return(NULL);
}

/*
 *	F R E E S C O R E S H E E T --	Free all scoresheet memory,
 *									except scoresheet structure
 */
struct score_sheet *
Freescoresheet(score)
struct score_sheet *score;
{
	if (alc->debug) fprintf(debug_channel, "Freescoresheet\n");
	if (score == NULL) return(NULL);
	score->name = Freeptr(score->name);
	score->menuname = Freeptr(score->menuname);
	score->abmenu = (struct scoremenu *)Freeptr(score->abmenu);
	score->acmenu = (struct scoremenu *)Freeptr(score->acmenu);
/*
 *	The header, and title may point to the same memory area,
 *	therefore we can't free them twice 
 */
	if (score->title == score->header) {	
		score->title = score->header = Freebanner(score->title);
	} else {
		score->title = Freebanner(score->title);
		score->header = Freebanner(score->header);
	}
	score->form = Freeassociation(score->form);
	score->level1 = Freelevel(score->level1);
	score->level2 = Freelevel(score->level2);
	score->aberr = Freeaberration(score->aberr);
	score->info = Freeinformation(score->info);
	score->action = Freeoperation(score->action);
	score->version = 0;
	score->boxed = 0;
	if (alc->debug) fprintf(debug_channel, "Scoresheet freed\n");
	return(NULL);
} /* end Freescoresheet */

/*
 *	F R E E B A N N E R -- Free off banner list
 */
struct banner *
Freebanner(ban)
struct banner *ban;
{
	struct banner *free_ptr;

	if (alc->debug) fprintf(debug_channel, "Freebanner\n");
	while (ban) {
		free_ptr = ban;
		ban = ban->next;
		Freeptr(free_ptr->assoc);
		Freeptr(free_ptr->txt);
/*
 *	No need to free data field, since either static store,
 *	or from score->form, freed by Freeassociation
 *	(Freebayformdata())
 */
/*		Freeptr(free_ptr->data);*/
		Freeptr(free_ptr);
	}
	return(NULL);
}

/*
 *	F R E E A S S O C I A T I O N -- Free off association list
 */
struct association *
Freeassociation(ass)
struct association *ass;
{
	struct association *free_ptr;

	if (alc->debug) fprintf(debug_channel, "Freeassociation\n");
	while (ass) {
		free_ptr = ass;
		ass = ass->next;
		Freeptr(free_ptr->assoc);
/*
 *	Don't free data field here, use Freebayformdata() separately
 *	since data for each bay will be different.
 *	This is a kludge, the data field doesn't really
 *	belong in this structure
 */
 */*		Freeptr(free_ptr->data); */
		Freeptr(free_ptr->menutxt);
		Freeptr(free_ptr);
	}
	return(NULL);
}

/*
 *	F R E E L E V E L -- Free off level linked list.
 */
struct level *
Freelevel(lev)
struct level *lev;
{
	struct level *free_ptr;

	if (alc->debug) fprintf(debug_channel, "Freelevel\n");
	while (lev) {
		free_ptr = lev;
		lev = lev->next;
		Freeptr(free_ptr->assoc);
		Freeptr(free_ptr->txt);
		Freeptr(free_ptr);
	}
	return(NULL);
}

/*
 *	F R E E A B E R R A T I O N -- Free off aberration linked list.
 */
struct aberration *
Freeaberration(ab)
struct aberration *ab;
{
	struct aberration *free_ptr;

	if (alc->debug) fprintf(debug_channel, "Freeaberration\n");
	while (ab) {
		free_ptr = ab;
		ab = ab->next;
		Freeptr(free_ptr->key);
		Freeptr(free_ptr->shortstr);
		Freeptr(free_ptr->longstr);
		Freeptr(free_ptr);
	}
	return(NULL);
}

/*
 *	F R E E I N F O R M A T I O N -- Free off information linked list.
 */
struct information *
Freeinformation(inf)
struct information *inf;
{
	struct information *free_ptr;

	if (alc->debug) fprintf(debug_channel, "Freeinformation\n");
	while (inf) {
		free_ptr = inf;
		inf = inf->next;
		Freeptr(free_ptr->assoc);
		Freeptr(free_ptr->screentxt);
		Freeptr(free_ptr);
	}
	return(NULL);
}

/*
 *	F R E E O P E R A T I O N -- Free off operation linked list.
 */
struct operation *
Freeoperation(oper)
struct operation *oper;
{
	struct operation *free_ptr;

	if (alc->debug) fprintf(debug_channel, "Freeoperation\n");
	while (oper) {
		free_ptr = oper;
		oper = oper->next;
		Freeptr(free_ptr->key);
		Freeptr(free_ptr->assoc);
		Freeptr(free_ptr->menutxt);
		Freeptr(free_ptr);
	}
	return(NULL);
}

/*
 *	F R E E F O R M D A T A --	Free the form data bits (We assume they're
 *								malloc'd/calloc'd
 */
Freeformdata(sf)
struct association *sf;
{
	for (;sf; sf = sf->next) {
		if (alc->debug) fprintf(debug_channel, "Freeformdata: Free at 0x%x\n",
			sf->data);
		sf->data = Freeptr(sf->data);
	}
}

