/*
 *	M E T R E V I E W -- Read met lists and cope with moving about them
 *
 *  Written: D C Burgess
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1989)
 *
 *  Date:    6th June 1989
 *
 *	Modifications
 *	 1 Aug 1991	CAS		Remove unused conversionstr def
 *	03 Jan 1990	dcb		Remove moveabout, shonum and movrecord & all other
 *						ifdef'd out bits
 *	10 Jul 89	dcb		Ifdef shonum out, and moveabout
 *	20 Jun 89	dcb		Unstatic shonum & setcprcd
 *	11 Aug 89	dcb		Try just using mpt->zval in setoffsets
 */
#include <cmdsys.h>
#include <slides.h>
#include <kyy.h>
#include <cfipp.h>
#include <fipcomm.h>
#include <kcontrol.h>
/*
 *	Defines
 */
#define FORW 1			/* Nextrec stuff */
#define BACKW -1		/* More nextrec stuff */

/*
 *	Externals
 */
extern struct kcontrol *kcont;
extern char	*fiperr[];
extern	FILE *debug_channel;
/* extern char *conversionstr[]; */
extern	int bay_move(), baynn(), changeobj(), mic_conv(), fipops();
extern int getlist();

/*
 *	Globals
 */
int next_met(), prev_met(), rec_met(), rank_met();

/*
 *	Globals
 */
char *Calloc();
int rec_no, rank_no;

/*
 *		S E T O F F S E T S
 *	Defined here, used globally
 *
 */
int
setoffsets(mpt)
struct	mlclust *mpt;
{
struct	mult_slide *bp;
	bp = alc->curbay;
	bp->xcpoff = xs - mpt->xval;
	bp->ycpoff = ys - mpt->yval;
	bp->zcpoff = zs - mpt->zval; /*(mpt->zval+bp->focslc);*/
	
}

/*
 *  I N I T _ O F F S E T.
 *
 *	This routine initialises the static offsets to zero.
 *	It is called from getlist (metlist.c) and getmph (findmet.c).
 * 
 */
init_offset()
{
	alc->curbay->xcpoff = alc->curbay->ycpoff = alc->curbay->zcpoff = 0;
}

/*
 *
 *  G C _ M I P _ T I D Y.
 *
 *			Leaves the miproc in a sensible state when leaving getcell.
 *
 */
static
gc_mip_tidy(xdoff, ydoff)
int	xdoff, ydoff;
{
	int	stat = 0;

	if (tomip1("x", xs-xdoff) != FS_OK)  return(FS_FAIL); /* Return x, y posn */
	if (tomip1("y", ys-ydoff) != FS_OK)  return(FS_FAIL);
	if (highres(OFF) != FS_OK)  return(FS_FAIL);
	getpos(2);
	return(movespeed());											
}


/* 
 *	S E T C P R C D ---  Set record and rank
 */
setcprcd(rec_no, rank_no)
int	*rec_no, *rank_no;
{
struct	mult_slide *bp;
struct	mlclust *mpt;

	bp = alc->curbay;
	if(bp->startptr == NULL)
		*rec_no = *rank_no = 0;
	else {
		mpt = bp->currptr->mlrec;
		*rec_no = bp->nameids.cellno = mpt->recnr;
		*rank_no = mpt->qrank;
		recall(mpt->xval+bp->xcpoff, mpt->yval+bp->ycpoff,
			   mpt->zval+bp->focslc+bp->zcpoff);
	}
}

/*
 *	P R I N T _ C Y T O -- Debug, print contents of cyto structure
 */
print_cyto(cyto)
struct cyto_file *cyto;
{
	if (cyto == NULL) {
		fprintf(debug_channel, "print_cyto: Passed NULL pointer\n");
		return(-1);
	}
	fprintf(debug_channel, "type=%d\ncase=%s\npatient=%s\nslide=%s\ncellno=%d\n",
		cyto->type, cyto->case, cyto->patnam, cyto->slide, cyto->cellno);
	return(0);
} /* end print_cyto */

/*
 *	P R I N T _ C E L L _ H -- Print the cell_h structure contents
 */
print_cell_h(ch)
struct cell_header *ch;
{
	if (ch == NULL) {
		fprintf(debug_channel, "print_cell_h: Passed NULL pointer\n");
		return(-1);
	}
	fprintf(debug_channel, "size=%d, version=%d, file_type=%d\ndate=%s, lab_name=%s\n",
		(int)ch->size, ch->version, ch->file_type, ch->date, ch->lab_name);
	fprintf(debug_channel, "opid=%s, patid=%s, patnam=%s\n",
		ch->opid, ch->patid, ch->patnam);
	fprintf(debug_channel, "slideid=%s, cellid=%d, dig_type=%d\n",
		ch->slideid, ch->cellid, ch->dig_type);
	fprintf(debug_channel, "x_coord=%d, y_coord=%d, z_coord=%d\n",
		ch->x_coord, ch->y_coord, ch->z_coord);
	fprintf(debug_channel, "x_size=%d, y_size=%d\n",
		ch->x_size, ch->y_size);
	fprintf(debug_channel, "comments[1]=%s, comments[2]=%s, krtype=%s\n",
		ch->comments[1], ch->comments[2], ch->krtype);
	fprintf(debug_channel, "supid=%s, supdate=%s\n",
		ch->supid, ch->supdate);
	return(0);

} /* end print_cell_h */

/*
 *	N E X T _ M E T
 */
int
next_met()
{
	int rec_no, rank_no;

	if(!nextrec(FORW)) {
		fprintf(output,"End of subset list\007\n");
		return(-1);
	}
	setcprcd(&rec_no, &rank_no);			/* Get record and display */
	return(0);
}

/*
 *	P R E V _ M E T
 */
int
prev_met()
{
	int rec_no, rank_no;

	if(!nextrec(BACKW)) {
		fprintf(output,"Beginning of subset list\007\n");
		return(-1);
	}
	setcprcd(&rec_no, &rank_no);			/* Get record and display */
	return(0);
}

/*
 *	R E C _ M E T
 */
int
rec_met(record)
{
	int rec_no, rank_no;

	if(!findrec(record)) {
		fprintf(output,"Request outside list bounds\007\n");
		return(-1);
	}
	setcprcd(&rec_no, &rank_no);			/* Get record and display */
	return(0);
}

/*
 *	R A N K _ M E T
 */
int
rank_met(rank)
{
	int rec_no, rank_no;

	if(!findrank(rank)) {
		fprintf(output,"Request outside list bounds\007\n");
		return(-1);
	}
	setcprcd(&rec_no, &rank_no);			/* Get record and display */
	return(0);
}

