/*
 *      P R I N T _ S C O R E .  C -- Read score sheet info from file. 
 *
 *		This routine and it's sister (read_score.c) are independent of
 *		the application, there is therfore a local debug_channel ab_debug.
 *
 *  Written: Sarah E. A. Jones
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    6th June 1989.
 *
 *  Modifications
 *
 *	12 Feb 1991		ih			Added a page feed at end of print scores
 *	28 Aug 1989		dcb			Get centring right in banners, so no overruns
 *								set hold outside print loop in print_entries,
 *								only print separator if there are entries.
 *  28 Jun 1989		SEAJ		Changed do_banner to use ban->data.
 *  28 Jun 1989		SEAJ		Added local debugging.
 *	27 Jun 1989		SEAJ		Cope with NULL entry text.
 *  26 Jun 1989		SEAJ		Ignore zero entries.
 *  19 Jun 1989		SEAJ		Made score into sc to stop name clashes
 */
#include <stdio.h>
#include <fbio.h>
#include <spool.h>
#include <aberrations.h>

#define PAGELENGTH  64
#define ESC			'\033'
#define COND_ON		'\017'
#define COND_OFF	'\022'
#define SINGVERT 	'\263'
#define DOUBVERT 	'\272'
#define SINGHORIZ 	'\304'
#define DOUBHORIZ 	'\315'
#define PAGEFEED	"\014\0"
/*
 * Global varaibles
 */
FILE *ab_debug = NULL;;

/*
 * local structures
 */
struct column {
	int pos;			/* buffer position */
	int width;			/* column width */
	short sep;			/* following column separator */
	short totflag;		/* does this column have total ? */
	int total;			/* total for this column */
};
struct column cols[MAXCOLS];
SFILE *sfp;
struct score_sheet *sc;
static int lineno;	/* initialised to 1 for spooler's initial linefeed */
static int pageno;
int num_cols;	/* store the number of columns */

/*
 * A B E R R _ D E B U G -- Make debug channel into ab_debug
 *
 */
aberr_debug(fp)
FILE *fp;
{
	ab_debug = fp;
}

/*
 * P R I N T _ S C O R E
 *
 */
print_score(score_in, ent)
struct score_sheet *score_in;
struct entry *ent;
{
	char *buffer;
	
	if (ab_debug)
		fprintf(ab_debug, "PRINT_SCORE\n");
	lineno = 1;
	pageno = 1;
	buffer = (char *) Calloc(sizeof(char), LINELENGTH);
	setup();
	sc = score_in;
	if (sc != NULL) {
		do_name(sc->name, buffer);
		do_page(buffer);
		if (sc->boxed)
			topline(sc->title);
		do_banner(sc->title, buffer); 
		if (sc->level1)
			print_level(1, sc->level1, buffer);
		if (sc->level2)
			print_level(2, sc->level2, buffer);
		print_entries(ent, sc, buffer);
	}
	if (ab_debug)
		fprintf(ab_debug, "END PRINT_SCORE\n");
	sprintf(buffer, PAGEFEED);
	writebuf(0, buffer, OFF);
	fflush(sfp->fp);
	closedown(buffer);
	Free(buffer);
}

/*
 *  S E T U P -- open spool file, set condensed mode and margins
 *
 */
static
setup(buffer)
char *buffer;
{
	char *buf;
	
	buf = buffer;
	if ((sfp = spoolopen()) == NULL) {
		fprintf(stderr, "Unable to open print file\n");
		exit(0);
	}
	*buf++ = COND_ON;
	*buf = '\0';
	writebuf(0, buffer, OFF);
	buf = buffer;
	*buf++ = ESC;
	*buf++ = 'X';
	*buf++ = 7;
	*buf++ = 142;
	*buf = '\0';
	writebuf(0, buffer, OFF);
}
/*
 * C L O S E D O W N -- put printer back to normal
 *
 */
closedown(buffer)
char *buffer;
{
	char *buf;
		
	buf = buffer;
	*buf++ = ESC;
	*buf++ = 'X';
	*buf++ = 1;
	*buf++ = 142;
	*buf = '\0';
	writebuf(0, buffer, OFF);
	buf = buffer;
	*buf++ = COND_OFF;
	*buf = '\0';
	writebuf(0, buffer, OFF);
	spoolclose(sfp);
}
/*
 * T O P L I N E
 *
 */
topline(ban)
struct banner *ban;
{
	char *buffer, *buf;
	int i;
	
	if (ban == NULL) return;
	buffer = (char *) Calloc(sizeof(char), LINELENGTH+2);
	buf = buffer;
	buf = buf + LINELENGTH - 1;
	if (sc->boxed == SINGLE) {
		*buffer = '\332';
		*buf++ = '\277';
	} else {
		*buffer = '\311';
		*buf++ = '\273';
	}
	*buf++ = '\n';
	*buf++ = '\0';
	buf = buffer;
	buf++;						/* offset for initail character */
	while (ban->newline == 0) {
		buf += ban->width - 1;
		line_down(ban->vert, sc->boxed, buf);
		buf++;
		ban = ban->next;
	}
	buf = buffer;
	for (i = 0; i < LINELENGTH; i++) {
		if (*buf == '\0') {
			if (sc->boxed == SINGLE)
				*buf = SINGHORIZ;
			else
				*buf = DOUBHORIZ;
		}
		buf++;
	}
	buf = buffer;
	writebuf(1, buffer, OFF);
}
/*
 * B O T T O M L I N E
 *
 */
bottomline()
{
	char *buffer, *buf;
	int i;
	
	buffer = (char *) Calloc(sizeof(char), LINELENGTH+2);
	buf = buffer;
	buf = buf + LINELENGTH - 1;
	if (sc->boxed == SINGLE) {
		*buffer = '\300';
		*buf++ = '\331';
	} else {
		*buffer = '\310';
		*buf++ = '\274';
	}
	*buf++ = '\n';
	*buf++ = '\0';
	buf = buffer;
	buf++;				/* skip first character */
	for (i = 0; i < num_cols; i++) {
		buf += cols[i].width - 1;
		line_up(cols[i].sep, sc->boxed, buf);
		buf++;
	}			
	buf = buffer;
	buf++;						/* offset for initail character */
	for (i = 0; i < LINELENGTH; i++) {
		if (*buf == '\0') {
			if (sc->boxed == SINGLE)
				*buf = SINGHORIZ;
			else
				*buf = DOUBHORIZ;
		}
		buf++;
	}
	buf = buffer;
	writebuf(1, buffer, OFF);
}
/*
 * W R I T E B U F
 *
 */
writebuf(num, buffer, box)
int num, box;
char *buffer;
{
	int i;
	char *buf, line[LINELENGTH+2];		/* 2 is for boxing, newline and null */

	if (box) {
		buf = line;
		strcpy(buf+1, buffer);
		if (sc->boxed == SINGLE) {
			if (*buffer == SINGHORIZ) {
				*buf = '\303';
				line[LINELENGTH-1] = '\264';
			} else if (*buffer == DOUBHORIZ) {
				*buf = '\306';
				line[LINELENGTH-1] = '\265';
			} else {
				*buf = SINGVERT;
				line[LINELENGTH-1] = SINGVERT;
			}
		} else if (sc->boxed == DOUBLE) {
			if (*buffer == SINGHORIZ) {
				*buf = '\307';
				line[LINELENGTH-1] = '\266';
			} else if (*buffer == DOUBHORIZ) {
				*buf = '\314';
				line[LINELENGTH-1] = '\271';
			} else {
				*buf = DOUBVERT;
				line[LINELENGTH-1] = DOUBVERT;
			}
		} else
			*buf = ' ';
		line[LINELENGTH] = '\n';			/* newline */
		line[LINELENGTH+1] = '\0';			/* null */
		fbwrite(line, strlen(line), 1, sfp->fp);
	} else {
		fbwrite(buffer, strlen(buffer), 1, sfp->fp);
	}
	for (i = 0; i < LINELENGTH; i++)		/* clear buffer */
		*buffer++ = ' ';
	if (num > 0)
		lineno += num;
}
/*
 * N E W P A G E
 *
 */
newpage(buffer)
char *buffer;
{
	sprintf(buffer, PAGEFEED);
	writebuf(0, buffer, OFF);
	do_name(sc->name, buffer);
	do_page(buffer);
	if (sc->boxed)
		topline(sc->header);
	do_banner(sc->header, buffer); 
	if (sc->level1)
		print_level(1, sc->level1, buffer);
	if (sc->level2)
		print_level(2, sc->level2, buffer);
}
/*
 * D O _ P A G E
 *
 */
do_page(buffer)
char *buffer;
{
	char string[10], *buf;
	
	sprintf(string, "Page %d", pageno++);
	buf = buffer + LINELENGTH - strlen(string) - 3;
	sprintf(buf, "%s\n\0", string);
	writebuf(1, buffer, OFF);
}
/*
 * D O _ N A M E
 *
 */
do_name(name, buffer)
char *name, *buffer;
{
	int len;
	char *buf;

	if (*name == NULL)
		return;	
	if (ab_debug)
		fprintf(ab_debug, "DO_NAME %s\n", name);
	buf = buffer;
	*buf = '\n';
	writebuf(1, buffer, OFF);
	buf = buffer;
	len = strlen(name);
	len = (LINELENGTH - len) / 4;
	buf += len;
	sprintf(buf, "\016%s\024\n", name);
	writebuf(1, buffer, OFF);
}	
/*
 * D O _ B A N N E R -- Print either titles or headers
 *
 */
do_banner(ban, buffer)
struct banner *ban;
char *buffer;
{
	char *buf;
	int j, txtlen, datalen, left, copylen;
	struct banner *hold;
	struct level *lev;

	if (ban == NULL)
		return;
	buf = buffer;
	hold = ban;
	while (ban != NULL) {
		if (ab_debug)
			fprintf(ab_debug, "DO_BANNER txt %s\n", ban->txt);
		strncpy(buf, ban->txt, strlen(ban->txt));
		if (ban->data != NULL) {
			txtlen = strlen(ban->txt);
			datalen = strlen(ban->data);
			left = ban->width - txtlen - datalen;
			left = (left > 0) ? left/2 : 0;
			copylen = ban->width - txtlen - left;
 			copylen = (copylen > datalen) ? datalen : copylen;
 			strncpy((buf+txtlen+left), ban->data, copylen);
			if (ab_debug) {
				fprintf(ab_debug, "DO_BANNER data %s\n", ban->data);
				fprintf(ab_debug,"txtlen=%d, datalen=%d, left=%d, copylen=%d\n",
					txtlen, datalen, left, copylen);
			}
		}
		buf += ban->width-1;
		if (ban->vert == SINGLE)
			*buf = SINGVERT;			/* single bar */
		else if (ban->vert == DOUBLE)
			*buf = DOUBVERT;			/* double bar */
		buf++;
		if (ban->horiz || ban->newline) {
			writebuf(1, buffer, ON);
			buf = buffer;
			if (ban->horiz) {			/* if there is a horizontal separator */
				while (hold != ban->next) {		/* do upward connections */
					buf += hold->width-1;
					line_up(hold->vert, hold->horiz, buf);
					buf++;
					hold = hold->next;
				}
				buf = buffer;
				if (ban->next != NULL) {				/* banner following */
					while ((hold->newline == 0) && (hold != NULL)) {
						buf += hold->width-1;
						if (*buf != ' ')				/* been here before */
							line_cross(hold->vert, hold->horiz, buf);
						else							/* not been here before */
							line_down(hold->vert, hold->horiz, buf);
						buf++;
						hold = hold->next;
					}
					hold = ban->next;
				} else {								/* level heading follows */
					if ((sc->level1 || sc->level2) && (ban->newline < 2)) {
						if (sc->level1)
							lev = sc->level1;
						else if (sc->level2)
							lev = sc->level2;
						while (lev != NULL) {
							buf += lev->width-1;
							if (*buf != ' ')	/* been here before */
								line_cross(lev->vert, ban->horiz, buf);
							else				/* not been here before */
								line_down(lev->vert, ban->horiz, buf);
							buf++;
							lev = lev->next;
						}
					}
				}
				buf = buffer;
				for (j = 0; j < LINELENGTH-2; j++) {
					if (*buf == ' ') {
						if (ban->horiz == SINGLE)
							*buf = SINGHORIZ;
						else
							*buf = DOUBHORIZ;
					}
					buf++;
				}
				writebuf(1, buffer, ON);
				buf = buffer;
			}
		}
		ban = ban->next;
	}
}
/*
 *  P R I N T _ L E V E L
 *
 */
print_level(level, lev, buffer)
int level;
struct level *lev;
char *buffer;
{
	char *buf;
	int j, len, horiz;
	struct level *hold;
	
	hold = lev;
	buf = buffer;
	while (lev != NULL) {
		if (ab_debug)
			fprintf(ab_debug, "PRINT_LEVEL txt %s\n", lev->txt);
		if (strncmp(lev->txt, "Blank", 5)) {
			len = strlen(lev->txt);
			len = lev->width - len - 1;
			if (len > 0)
				len /= 2;
			else
				len = 0;
			strncpy(buf+len, lev->txt, strlen(lev->txt));
		}
		buf += lev->width-1;
		if (lev->vert == SINGLE)
			*buf = SINGVERT;			/* single bar */
		else if (lev->vert == DOUBLE)
			*buf = DOUBVERT;			/* double bar */
		buf++;
		horiz = lev->horiz;
		lev = lev->next;
	}
	writebuf(1, buffer, ON);
	buf = buffer;
	if (horiz) {
		while (hold != NULL) {
			buf += hold->width-1;
			line_up(hold->vert, horiz, buf);
			buf++;
			hold = hold->next;
		}
		buf = buffer;
		if (level == 1) {
			if (sc->level2)
				hold = sc->level2;
			else
				hold = sc->level1;
		} else 
			hold = sc->level2;
		while (hold != NULL) {
			buf += hold->width-1;
			if (*buf != ' ')		/* been here before */
				line_cross(hold->vert, horiz, buf);
			else
				line_down(hold->vert, horiz, buf);
			buf++;
			hold = hold->next;
		}
		buf = buffer;
		for (j = 0; j < LINELENGTH; j++) {
			if (*buf == ' ') {
				if (horiz == SINGLE)
					*buf = SINGHORIZ;
				else
					*buf = DOUBHORIZ;
			}
			buf++;
		}
		writebuf(1, buffer, ON);
		buf = buffer;
	}
}
/*
 * L I N E _ U P 
 *
 */
line_up(vert, horiz, buf)
int vert, horiz;
char *buf;
{
	if (vert == SINGLE) {
		if (horiz == SINGLE)
			*buf = '\301';
		else 
			*buf = '\317';
	} else if (vert == DOUBLE) {
		if (horiz == SINGLE)
			*buf = '\320';
		else
			*buf = '\312';
	}
}
/*
 * L I N E _ D O W N
 *
 */
line_down(vert, horiz, buf)
int vert, horiz;
char *buf;
{
	if (vert == SINGLE) {
		if (horiz == SINGLE)
			*buf = '\302';
		else
			*buf = '\321';
	} else if (vert == DOUBLE) {
		if (horiz == SINGLE)
			*buf = '\322';
		else
			*buf = '\313';
	}
}
/*
 * L I N E _ C R O S S
 *
 */
line_cross(vert, horiz, buf)
int vert, horiz;
char *buf;
{
	if (vert == SINGLE) {
		if (horiz == SINGLE)
			*buf = '\305';
		else
			*buf = '\330';
	} else if (vert == DOUBLE) {
		if (horiz == SINGLE)
			*buf = '\327';
		else
			*buf = '\316';
 	}
}
/*
 * C A L C _ C O L S
 *
 */
calc_cols()
{
	struct level *lev = NULL;
	int count = 0;
	
	num_cols = 0;
	if (sc->level1) {
		if (sc->level2)
			lev = sc->level2;
		else
			lev = sc->level1;
	}
	while (lev != NULL) {
		cols[lev->col].pos = count;
		cols[lev->col].width = lev->width;
		cols[lev->col].sep = lev->vert;
		cols[lev->col].total = 0;
		cols[lev->col].totflag = lev->total;
		if (lev->col > num_cols)
			num_cols = lev->col;
		count += lev->width;
		lev = lev->next;
	}
}
/*
 * P R I N T _ E N T R I E S -- Print the entries list
 *
 */
print_entries(first, score_in, buffer)
struct entry *first;
struct score_sheet *score_in;
char *buffer;
{
	struct entry *ent, *top, *hold;
	int done = 0, notodo = 0, numberoflines = 0, page = 0, firstpage = 1;
	int i, full[MAXCOLS], len;
	char *buf, tmpstr[50];

	if (ab_debug)
		fprintf(ab_debug, "PRINT_ENTRIES\n");
	sc = score_in;
	calc_cols();	
	for (i = 0; i < MAXCOLS; i++)
		full[i] = 0;
	ent = first;
	while (ent != NULL) {
		top = ent;
		while (ent != NULL) {
			if (cols[ent->col].totflag)
				cols[ent->col].total += ent->num;	/* keep total up to date */
			ent->printed = 0;					/* mark entry as not yet printed */
			if (ent->num > 0)			/* don't count zero entries */
				notodo++;
			full[ent->col]++;					/* count number of lines this column */
			ent = ent->next;
		}
		for (i = 0; i <= num_cols; i++) {
			if (full[i] > numberoflines)
				numberoflines = full[i]; 	/* find number of lines this entry */
			full[i] = 0;
		}
		numberoflines += 2;					/* add on line before and bottom line */
		if (lineno + numberoflines >= PAGELENGTH) { /* throw page if necessary */
			if (sc->boxed)
				bottomline();
			lineno = 0;
			newpage(buffer);
			page = 1;
		}
		if (!page && !firstpage && notodo) {			/* do separator */
			buf = buffer;
			for (i = 1; i <= num_cols; i++) {
				buf += cols[i].width - 1;
				line_cross(cols[i].sep, SINGLE, buf);
				buf++;	
			}
			buf = buffer;
			for (i = 0; i < LINELENGTH; i++) {
				if (*buf == ' ')
					*buf = SINGHORIZ;
				buf++;
			}
			writebuf(1, buffer, ON);
		}
		hold = top;
		while (done < notodo) {				/* while there are still some to print */
			ent = top;
			while (ent != NULL) {
				hold = ent;
				if (ent->num > 0) {				/* ignore zero entries */
					if (ent->printed == 0) { 		/* not yet been printed */
						if (full[ent->col] == 0) {	/* column empty */
							if (ab_debug)
								fprintf(ab_debug, "entry num %d col %d printed %d txt %s\n", 
									ent->num, ent->col, ent->printed, ent->txt);
							buf = buffer;
							buf += cols[ent->col].pos;
							if (ent->txt) {
								if (ent->num > 1)
									sprintf(tmpstr, "%d%s\0", ent->num, ent->txt);
								else
									sprintf(tmpstr, "%s\0", ent->txt);
							} else
								sprintf(tmpstr, "%d\0", ent->num);
							len = strlen(tmpstr);
							len = cols[ent->col].width - len - 1;
							if (len > 0)			/* center text */
								len /= 2;
							else
								len = 0;
							strncpy(buf+len, tmpstr, strlen(tmpstr));
							ent->printed = 1;		/* mark printed */
							full[ent->col] = 1;		/* column now full */
							done++;
						}
					}
				} else {
					done++;		/* mark as printed even though num is zero */
					ent->printed = 1;
				}
				ent = ent->next;
			}
			buf = buffer;
			for (i = 1; i <= num_cols; i++) { /* put single vertical lines */
				buf += cols[i].width - 1;
				if (cols[i].sep == SINGLE)
					*buf = SINGVERT;
				else if (cols[i].sep == DOUBLE)
					*buf = DOUBVERT;
				buf++;
			}
			writebuf(1, buffer, ON);
			for (i = 0; i < MAXCOLS; i++)	/* mark all columns empty */
				full[i] = 0;
		}
		notodo = 0;							/* reset all parameters */
		done = 0;
		numberoflines = 0;
		page = 0;
		firstpage = 0;
		if (ab_debug)
			fprintf(ab_debug, "\n");
		ent = hold->foll;
	}
	do_totals(buffer);
}
/*
 * D O _ T O T A L S
 *
 */
do_totals(buffer)
char *buffer;
{
	char *buf, tmpstr[10];
	int i, len;

	if (ab_debug)
		fprintf(ab_debug, "DO_TOTALS\n");
	if (lineno + 3 >= PAGELENGTH) {
		if (sc->boxed)
			bottomline();
		lineno = 0;
		newpage(buffer);
	} else {
		buf = buffer;
		for (i = 1; i <= num_cols; i++) {			/* double crosses */
			buf += cols[i].width - 1;
			line_cross(cols[i].sep, DOUBLE, buf);
			buf++;	
		}
		buf = buffer;
		for (i = 0; i < LINELENGTH; i++) {			/* double line */
			if (*buf == ' ')
				*buf = DOUBHORIZ;
			buf++;
		}
		writebuf(1, buffer, ON);
	}
	buf = buffer;
	for (i = 1; i <= num_cols; i++) {
		if (cols[i].totflag > 0) {
			sprintf(tmpstr, "%d", cols[i].total);
			len = strlen(tmpstr);
			len = cols[i].width - len - 1;
			if (len > 0)
				len /= 2;
			else
				len = 0;
			strncpy(buf+len, tmpstr, strlen(tmpstr));
		}
		buf += cols[i].width - 1;
		if (cols[i].sep == SINGLE)
			*buf = SINGVERT;
		else if (cols[i].sep == DOUBLE)
			*buf = DOUBVERT;
		buf++;
	}
	writebuf(1, buffer, ON);				/* write buffer */
	if (sc->boxed)
		bottomline();
}
