/*
 *      R E A D _ S C O R E . C -- Read score sheet info from file.
 *
 *		This routine and it's sister (print_score.c) are independent of
 *		the application, there is therfore a local debug_channel ab_debug.
 *
 *
 *  Written: Sarah E. A. Jones
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    2nd June 1989.
 *
 *  Modifications
 *
 *	22 Jan 1991		dcb		read_score returns error codes on error, and
 *							doesn't call exit directly.
 *	09 Jan 1991		dcb		Add Function keyword, add KEYFUNC case for reading
 *							function definition, and getfunc for reading it
 *							Print function input in show_score
 *	26 Oct 1990		dcb		Check for NULL banner in check_bannerwidth, also
 *							use title setups for header if header not defined
 *	12 Dec 1989		dcb		errno for V2.2
 *	28 Aug 1989		dcb		Add check_bannerwidth function
 *  28 Jun 1989		SEAJ	Added new fields abmenu, acmenu and opt.
 *  28 Jun 1989		SEAJ	Added local debugging.
 *  27 Jun 1989		SEAJ	Cope with None in Aberration string.
 *  20 Jun 1989		SEAJ	Added in reading struct information and operation.
 *	19 Jun 1989		SEAJ	Modified to bring up to date with DCB's.
 */
#include <stdio.h>
#include <aberrations.h>

static char *options[] = {
	"Name",			/* 0 */
	"Title",		/* 1 */
	"Header",		/* 2 */
	"Level1",		/* 3 */
	"Level2",		/* 4 */
	"Form",			/* 5 */
	"Aberr",		/* 6 */
	"Add",			/* 7 */
	"Allbox",		/* 8 */
	"Box",			/* 9 */
	"Version",		/* 10 */
	"Newline",		/* 11 */
	"Sep",			/* 12 */
	"Menutitle",	/* 13 */
	"Info",			/* 14 */
	"Action",		/* 15 */
	"Abmenu",		/* 16 */
	"Acmenu",		/* 17 */
	"Function"		/* 18 */
};
#define MAXOPTS (sizeof(options)/sizeof(char *))

extern char *skipword(), *skipwhite(), *next_word();
extern FILE *ab_debug;
extern int errno;

struct banner *getbanner();
struct level *getlevel();
struct association *getassoc();
struct aberration *getaberr();
struct information *getinfo();
struct operation *getaction();
struct scoremenu *getscoremenu();
struct function *getfunc();

/*
 * R E A D _ S C O R E
 *
 */
struct score_sheet *
read_score(infile)
char *infile;
{
	struct score_sheet *score;
	int fd;
	
	if (ab_debug)
		fprintf(ab_debug, "READ_SCORE\n");
	if ((fd = open(infile, 1)) == -1) {
		fprintf(stderr,"Can't open file %s\n", infile);
		return(NULL);	/* Error return - error code in errno */
	}
	score = (struct score_sheet *) Calloc(sizeof(struct score_sheet), 1);
	score->version = VERSION;
	score->name = NULL;
	score->menuname = NULL;
	score->boxed = 0;
	score->abmenu = NULL;
	score->acmenu = NULL;
	score->title = NULL;
	score->header = NULL;
	score->form = NULL;
	score->level1 = NULL;
	score->level2 = NULL;
	score->aberr = NULL;
	score->info = NULL;
	score->action = NULL;
	score->func = NULL;

	readtxtfile(fd, score);
	close(fd);
	sort_level(score->level1, score->level2);
	if (check_bannerwidth(score->title) < 0) return(NULL);
	if (score->header == NULL)
		score->header = score->title;
	if (check_bannerwidth(score->header) < 0) return(NULL);
	return(score);
}

/*
 *	R E A D T X T F I L E --
 *
 */
static
readtxtfile(in, score)
int in;
struct score_sheet *score;
{
	int		i, type = -1;
	char	buffer[200], *buf;
	struct banner *ban;
	struct level *lev;
	struct association *frm;
	struct aberration *abb;
	struct information *inf;
	struct operation *act;
	struct function *fun;

	while (next_line(in, buffer, sizeof(buffer))) {
		buf = buffer;
		i = findindex(buf, options, MAXOPTS);
		if (i < MAXOPTS) {
			buf = skipword(buf);
			switch(i) {
				case NAME :
					score->name = (char *) Calloc(sizeof(char), strlen(buf)+1);
					strncpy(score->name, buf, strlen(buf)+1);
					break;
				case TITLE :
					type = TITLE;
					if (score->title == NULL)
						score->title = getbanner(buf);
					else {
						ban = score->title;
						while (ban->next != NULL)
							ban = ban->next;
						ban->next = getbanner(buf);
					}
					break;
				case HEADER :
					type = HEADER;
					if (score->header == NULL)
						score->header = getbanner(buf);
					else {
						ban = score->header;
						while (ban->next != NULL)
							ban = ban->next;
						ban->next = getbanner(buf);
					}
					break;
				case LEVEL1 :
					type = LEVEL1;
					if (score->level1 == NULL)
						score->level1 = getlevel(buf);
					else {
						lev = score->level1;
						while (lev->next != NULL)
							lev = lev->next;
						lev->next = getlevel(buf);
					}
					break;
				case LEVEL2 :
					type = LEVEL2;
					if (score->level2 == NULL)
						score->level2 = getlevel(buf);
					else {
						lev = score->level2;
						while (lev->next != NULL)
							lev = lev->next;
						lev->next = getlevel(buf);
					}
					break;
				case FORM :
					if (score->form == NULL)
						score->form = getassoc(buf);
					else {
						frm = score->form;
						while (frm->next != NULL)
							frm = frm->next;
						frm->next = getassoc(buf);
					}
					break;
				case ABERR :
					if (score->aberr == NULL)
						score->aberr = getaberr(buf);
					else {
						abb = score->aberr;
						while (abb->next != NULL)
							abb = abb->next;
						abb->next = getaberr(buf);
					}
					break;
				case ADD :
					break;
				case ALLBOX :
					score->boxed = DOUBLE;
					break;
				case BOXED :
					score->boxed = SINGLE;
					break;
				case VER :
					score->version = atoi(buf);
					break;
				case NEWLINE :
					setnewline(score, type, buf);
					break;
				case SEP :
					setsep(score, type, buf);
					break;
				case MENUTITLE :
					score->menuname = (char *) Calloc(sizeof(char), strlen(buf)+1);
					strncpy(score->menuname, buf, strlen(buf)+1);
					break;
				case INFO :
					if (score->info == NULL)
						score->info = getinfo(buf);
					else {
						inf = score->info;
						while (inf->next != NULL)
							inf = inf->next;
						inf->next = getinfo(buf);
					}
					break;
				case ACTION :
					if (score->action == NULL)
						score->action = getaction(buf);
					else {
						act = score->action;
						while (act->next != NULL)
							act = act->next;
						act->next = getaction(buf);
					}
					break;
				case ABMENU :
					if (score->abmenu != NULL)
						Free(score->abmenu);
					score->abmenu = getscoremenu(buf);
					break;
				case ACMENU :
					if (score->acmenu != NULL)
						Free(score->acmenu);
					score->acmenu = getscoremenu(buf);
					break;
				case KEYFUNC:
					if (score->func == NULL)
						score->func = getfunc(buf);
					else {
						fun = score->func;
						while (fun->next != NULL)
							fun = fun->next;
						fun->next = getfunc(buf);
					}
					break;
						
			}
		}
	}
	if (ab_debug)
		show_score(score);
}
/*
 * G E T B A N N E R
 *
 */
struct banner *
getbanner(buf)
char *buf;
{
	struct banner *ptr;

	ptr = (struct banner *) Calloc (sizeof(struct banner), 1);
	ptr->width = atoi(buf);
	buf = skipword(buf);
	ptr->vert = NOSEP;
	if (*buf == '|') {
		ptr->vert = SINGLE;
		if (*(buf+1) == '|')
			ptr->vert = DOUBLE;
	}
	buf = skipword(buf);
	ptr->assoc = next_word(buf);
	buf = skipword(buf);
	ptr->txt = (char *) Calloc(sizeof(char), strlen(buf)+1);
	strncpy(ptr->txt, buf, strlen(buf)+1);
	ptr->next = NULL;
	return(ptr);
}
/*
 * G E T L E V E L
 *
 */
struct level *
getlevel(buf)
char *buf;
{
	struct level *ptr;
	
	ptr = (struct level *) Calloc(sizeof(struct level), 1);
	ptr->col = atoi(buf);
	buf = skipword(buf);
	ptr->vert = NOSEP;
	if (*buf == '|') {
		ptr->vert = SINGLE;
		if (*(buf+1) == '|')
			ptr->vert = DOUBLE;
	}
	buf = skipword(buf);
	ptr->width = atoi(buf);
	buf = skipword(buf);
	ptr->total = atoi(buf);
	buf = skipword(buf);
	ptr->assoc = next_word(buf);
	buf = skipword(buf);
	ptr->txt = (char *) Calloc(sizeof(char), strlen(buf)+1);
	strncpy(ptr->txt, buf, strlen(buf)+1);
	ptr->next = NULL;
	return(ptr);
}
/*
 * G E T A S S O C
 *
 */
struct association *
getassoc(buf)
char *buf;
{
	struct association *ptr;
	
	ptr = (struct association *) Calloc(sizeof(struct association), 1);
	ptr->opt = atoi(buf);
	buf = skipword(buf);
	ptr->assoc = next_word(buf);
	buf = skipword(buf);
	ptr->menutxt = (char *) Calloc(sizeof(char), strlen(buf)+1);
	strncpy(ptr->menutxt, buf, strlen(buf)+1);
	ptr->next = NULL;
	return(ptr);
}
/*
 * G E T A B E R R
 *
 */
struct aberration *
getaberr(buf)
char *buf;
{
	struct aberration *ptr;
	
	ptr = (struct aberration *) Calloc(sizeof(struct aberration), 1);
	ptr->key = next_word(buf);
	buf = skipword(buf);
	ptr->shortstr = next_word(buf);
	if (!strcmp(ptr->shortstr, "None")) {
		Free(ptr->shortstr);
		ptr->shortstr = NULL;
	}
	buf = skipword(buf);
	ptr->col = atoi(buf);
	buf = skipword(buf);
	ptr->longstr = (char *) Calloc(sizeof(char), strlen(buf)+1);
	strncpy(ptr->longstr, buf, strlen(buf)+1);
	ptr->next = NULL;
	return(ptr);
}
/*
 * G E T I N F O
 *
 */
struct information *
getinfo(buf)
char *buf;
{
	struct information *info;
	
	info = (struct information *) Calloc(sizeof(struct information), 1);
	info->assoc = next_word(buf);
	buf = skipword(buf);
	info->x = atoi(buf);
	buf = skipword(buf);
	info->y = atoi(buf);
	buf = skipword(buf);
	info->screentxt = (char *) Calloc(sizeof(char), strlen(buf)+1);
	strncpy(info->screentxt, buf, strlen(buf)+1);
	info->next = NULL;
	return(info);
}
/*
 * G E T A C T I O N
 *
 */
struct operation *
getaction(buf)
char *buf;
{
	struct operation *act;
	
	act = (struct operation *)Calloc(sizeof(struct operation), 1);
	act->key = next_word(buf);
	buf = skipword(buf);
	act->assoc = next_word(buf);
	buf = skipword(buf);
	act->menutxt = (char *) Calloc(sizeof(char), strlen(buf)+1);
	strncpy(act->menutxt, buf, strlen(buf)+1);
	act->next = NULL;
	return(act);
}

/*
 * G E T S C O R E S H E E T
 *
 */
struct scoremenu *
getscoremenu(buf)
char *buf;
{
	struct scoremenu *ptr;
	int i = 0;
	
	ptr = (struct scoremenu *) Calloc(sizeof(struct scoremenu), 1);
	ptr->row = atoi(buf);
	buf = skipword(buf);
	ptr->nrows = atoi(buf);
	buf = skipword(buf);
	while (*buf != '\0') {
		ptr->col[i++] = atoi(buf);
		buf = skipword(buf);
	}
	return(ptr);
}

/*
 *	G E T F U N C -- Get the function definition
 */
struct function *
getfunc(buf)
char *buf;
{
	struct function *fun;
	
	fun = (struct function *) Calloc(sizeof(struct function), 1);
	fun->col = atoi(buf);
	buf = skipword(buf);
	fun->def = next_word(buf);
	buf = skipword(buf);
	fun->true = next_word(buf);
	buf = skipword(buf);
	fun->false = next_word(buf);
	fun->next = NULL;
	return(fun);
}

/*
 * S E T N E W L I N E
 *
 */
setnewline(score, type, buf)
struct score_sheet *score;
int type;
char *buf;
{
	struct banner *ban;
	
	if (type == HEADER)
		ban = score->header;
	else
		ban = score->title;
	while (ban->next != NULL)
		ban = ban->next;
	ban->newline = 1;
}
/*
 * S E T S E P
 *
 */
setsep(score, type, buf)
struct score_sheet *score;
int type;
char *buf;
{
	struct banner *ban;
	struct level *lev;
	int separator = NOSEP;
	
	if (*buf == '-')
		separator = SINGLE;
	else if (*buf == '=')
		separator = DOUBLE;
	switch(type) {
		case HEADER :
		case TITLE :
			if (type == HEADER)
				ban = score->header;
			else
				ban = score->title;
			while (ban->next != NULL)
				ban = ban->next;
			ban->horiz = separator;
			ban->newline = 1;
			break;
		case LEVEL1 :
		case LEVEL2 :
			if (type == LEVEL1)
				lev = score->level1;
			else
				lev = score->level2;
			while (lev->next != NULL)
				lev = lev->next;
			lev->horiz = separator;
			break;
	}
}
/*
 * S H O W _ S C O R E
 *
 */
show_score(score)
struct score_sheet *score;
{
	struct banner *ban;
	struct level *lev;
	struct association *frm;
	struct aberration *abb;
	struct information *info;
	struct operation *act;
	struct scoremenu *scptr;
	struct function *fun;

	fprintf(ab_debug, "SHOW_SCORE\n");
	fprintf(ab_debug, "version %d\n", score->version);
	fprintf(ab_debug, "name %s\n", score->name);
	fprintf(ab_debug, "menuname %s\n", score->menuname);
	fprintf(ab_debug, "boxed %d\n", score->boxed);
	scptr = score->abmenu;
	fprintf(ab_debug, "abmenu row %d nrows %d col %d %d %d %d\n", scptr->row, 
		scptr->nrows, scptr->col[0], scptr->col[1], scptr->col[2], scptr->col[3]);
	scptr = score->acmenu;
	fprintf(ab_debug, "acmenu row %d nrows %d col %d %d %d %d\n", scptr->row, 
		scptr->nrows, scptr->col[0], scptr->col[1], scptr->col[2], scptr->col[3]);
	for (fun = score->func; fun; fun = fun->next) {
		fprintf(ab_debug, "function col %d def \"%s\" TRUE=%s FALSE=%s\n",
			fun->col, fun->def, fun->true, fun->false);
	}
	ban = score->title;
	while (ban != NULL) {
		fprintf(ab_debug, "title width %d newline %d vert %d  horiz %d assoc %s txt %s\n", 
			ban->width, ban->newline, ban->vert, ban->horiz, 
			ban->assoc, ban->txt);
		ban = ban->next;
	}
	ban = score->header;
	while (ban != NULL) {
		fprintf(ab_debug, "header width %d newline %d vert %d  horiz %d assoc %s txt %s\n", 
			ban->width, ban->newline, ban->vert, ban->horiz, 
			ban->assoc, ban->txt);
		ban = ban->next;
	}
	lev = score->level1;
	while (lev != NULL) {
		fprintf(ab_debug, "level1 col %d vert %d horiz %d width %d total %d assoc %s txt %s\n",
			lev->col, lev->vert, lev->horiz, lev->width, lev->total, 
			lev->assoc, lev->txt);
		lev = lev->next;
	}
	lev = score->level2;
	while (lev != NULL) {
		fprintf(ab_debug, "level2 col %d vert %d horiz %d width %d assoc %s txt %s\n",
			lev->col, lev->vert, lev->horiz, lev->width, lev->assoc, lev->txt);
		lev = lev->next;
	}
	frm = score->form;
	while (frm != NULL) {
		fprintf(ab_debug, "form opt %d assoc %s txt %s\n", frm->opt, frm->assoc, frm->menutxt);
		frm = frm->next;
	}
	abb = score->aberr;
	while (abb != NULL) {
		fprintf(ab_debug, "aberr key %s short %s col %d long %s\n", abb->key, 
			abb->shortstr, abb->col, abb->longstr);
		abb = abb->next;
	}
	info = score->info;
	while (info != NULL) {
		fprintf(ab_debug, "info assoc %s x %d y %d text %s\n", info->assoc, 
			info->x, info->y, info->screentxt);
		info = info->next;
	}
	act = score->action;
	while (act != NULL) {
		fprintf(ab_debug, "action key %s assoc %s text %s\n", act->key, 
			act->assoc, act->menutxt);
		act = act->next;
	}
}
/*
 * S O R T _ L E V E L
 *
 */
sort_level(lev1, lev2)
struct level *lev1, *lev2;
{
	struct level *start, *finish;
	int col = 1, count = 0;
	
	if (lev1 && lev2) {
		while (lev1 != NULL) {
			start = lev2;
			while ((lev1->col == lev2->col) && (lev2 != NULL)) {
				count += lev2->width;
				lev2 = lev2->next;
			}
			finish = lev2;
			lev1->width = count;
			count = 0;
			while (start != finish) {
				start->col = col;
				col++;
				start = start->next;
			}
			lev1 = lev1->next;
		}
	}
}

/*
 *	C H E C K _ B A N N E R W I D T H -- Check total banner widths do not
 *	exceed the available line length. Not strictly required according to the
 *	specification, since scoresheet should be automatically produced, but this
 *	function will probably save time until that is the case. There are bound
 *	to be errors in file definition.
 *	If a banner width would mean the total line length being exceeded, then it
 *	is reduced, and a warning message printed. If the width is not big enough
 *	even for the txt, never mind the data, then an error message is printed,
 *	and the function returns with error code -1
 */
static int
check_bannerwidth(ban)
struct banner *ban;
{
	int tot_width=0;

	if (ban == NULL) {
		fprintf(stderr, "Error: banner text (Title) not defined\n");
		exit(1); /* This should never happen */
	}
	for( ; ban; ban = ban->next) {
		if (ban->width > (LINELENGTH - tot_width)) { /* Not enough space */
			ban->width = LINELENGTH - tot_width;
			fprintf(stderr, "Warning: %s field width reduced to %d\n",
				ban->txt, ban->width);
		}
		tot_width += ban->width;
		if (ban->width < strlen(ban->txt)) {
			fprintf(stderr, "Error: not enough space for banner text %s\n",
				ban->txt);
			return(-1);
		}
		if (ban->newline) tot_width = 0;
	}
	return(0);	/* OK return */
} /* end check_bannerwidth */
