/*
 *	S C O R I N G -- Aberration scoring
 *
 *	Put up menu and do scoring/move about
 *
 *  Written: D C Burgess
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1989)
 *
 *  Date:    6th June 1989
 *
 *	Modifications
 *	31 Jan 1991 dcb/ih	Increase max specials to 10 from 4
 *	18 Jan 1991	dcb		Add bay_info structure array
 *	09 Jan 1991	dcb		Add FUNCTION special to initspecials, and addspecials
 *	04 Jan 1990	dcb		Freetext using ent, not elp in Frreentry
 *	03 Jan 1990	dcb		Flush text output after confirm message. Make printf's
 *						fprintf(output, ...); Remove moveabout references.
 *	28 Aug 1989	dcb		fix makeentry ? (at 19:55 pm!)
 *	11 Aug 1989	dcb		Unstatic makeentry bring infosection out of setupmenu
 *						and put in line in aberrationscore, this was due to the
 *						comment being cleared by updatenumbers
 */
#include <stdio.h>
#include <allcontr.h>
#include <slides.h>
#include <aberrations.h>
#include <curses.h>
#include <ctype.h>

#define CTRLD		4
#define TROW		0	/* Title row */
#define NWIDTH		3	/* Field width of preceeding aberration number */
#ifdef GGG
#define DIV_LINE	"--------------------------------------------------------------------------------"
#endif
#define DIV_LINE	"\033(0qqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqq\033(B"
#define BLANKLINE	"                                                                                 "
#define PROMPT		"Key: "
#define PRLEN		5
#define ERROR_LINE	23	/* Screen position */
#define INPUT_LINE	23	/* Screen position */
#define PROMPT_LINE	23	/* curscr position */
#define MAXCOMMENT	160	/* Max length of comment */
#define MAXSPECIALS	10	/* Max number of special columns */
#define MAXRANGE	999	/* Max number for an aberration */

/*
 *	Other structures
 */
struct special {
	int col;			/* It's column number */
	int	resindex;		/* Associated abbr. index */
	char *assoc;		/* The associated abbr */
};

struct comm {			/* For the per-cell comment */
	int	col;			/* It's column number */
	int width;			/* It's field width */
	int	totlen;			/* Total comment length */
	int	x;				/* X posn. on screen */
	int	y;				/* Y posn. on screen */
};

/*
 *	External references
 */
extern int xs, ys, bay;
extern struct allkontr *alc;
extern FILE *debug_channel, *output;
extern SGTTY savestdin;
extern struct bayinfo bay_info[];

/*
 *	Statics
 */
static struct special specials[MAXSPECIALS];	/* Special columns on scoresheet */
static int rowpos[128], colpos[128];
static int getcol=PRLEN;
static struct comm *comment=NULL;
static struct entry *elp, *endentrylist();
static struct entry *Freeentryline(), *Freeentrylist();
static char *Freetext();
static int scored = FALSE;	/* Cell has been scored */

/*
 *	Globals
 */
char *findkey(), getsingle(), getkey(), *strnsave(), *docomment();
char *itoa(), *Freeptr(), *getrestext();
struct entry *elist=NULL, *estart, *eprev, *makeentry();
int tot_score = 0;

/*
 *	A B E R R A T I O N S C O R E -- Main aberration score menu
 */
aberrationscore(score)
struct score_sheet *score;
{
	struct aberration *ab;
	int done=FALSE, n, number=0, resetcount=FALSE, minus=FALSE;
	int clearcmd=FALSE;
	char c, comstr[MAXCOMMENT];

	initscr();
	setupmenu(score);
	initspecials(score);
	updatenumbers(score, estart);
	infosection(score);
	clearcmdline();
	refresh();

	while (!done) {
		c = getkey(&clearcmd);
		if ((c >= '0') && (c <= '9')) {	/* Numeric input */
			number = number*10 + (c - '0');
			if (c != '0') continue;
		}
		switch (findresindex(findkey(score, c))) { /* Look for an action */
			case EXIT:
				if (!confirm("Exit, are you sure (y/n) ? ")) break;
				done = TRUE;
				if (scored) addspecials(score);
				break;
			case NEXT:
				seterrors();
				clearcmd = TRUE;
				/* Add specials BEFORE moving to next met */
				if (scored) addspecials(score);
				if (next_met() == -1) break; /* next_met failed */
				elp = endentrylist(estart);
				if (!elp->foll) { /* No following entry */
					elp->foll = makeentry(0, 0, NULL); /* so make one */
					elp->prev = eprev;
				}
				elp = elp->foll;
				eprev = estart;
				estart = elp;
				updatenumbers(score, estart);
				break;
			case PREV:
				clearcmd = TRUE;
				seterrors();
				if (prev_met() == -1) break; /* prev_met failed */
				if (eprev) {
					estart = eprev;
				} else {
					fprintf(stderr, "Reached start of list\n");
				}
				elp = endentrylist(estart);
				eprev = elp->prev;
				elp = estart;
				updatenumbers(score, estart);
				break;
			case COMMENT:
				scrn_pos(INPUT_LINE, 1);
				fprintf(output, "Comment ? ");
				scrn_clreol();
				getstring(comstr, MAXCOMMENT);
				clearoldcomment();
				addcomment(estart, comstr);
				scrn_pos(INPUT_LINE, 1);
				scrn_clreol();
				clearcmd = TRUE;
				break;
			case MINUS:
				minus = TRUE;
				break;
			case RESET:
				if (number == 0) resetcount = TRUE;
				break;
			case REFRESH:
				touchwin(stdscr);
				clearcmdline();
				break;
			case ADJUST:
				seterrors();
				centrecell();
				clearcmd = TRUE;
				break;
			case ZERO:
				if (scored) tot_score--;
				clearcmd = TRUE;
				Freeentryline(estart);
				updatenumbers(score, estart);
				break;
			default:
				/*
				 *	Scan aberration list for key
				 */
				for(ab = score->aberr; ab; ab = ab->next) {
					if (*(ab->key) == c) { /* Found it */
						if (!number && !resetcount) number = 1; /* At least 1 */
						if (minus) number = -number;
						if ((n = addentry(ab, number, resetcount)) < 0) break;
						updatescreen(ab, n);
						number = 0;
						resetcount = FALSE;
						minus = FALSE;
						clearcmd = TRUE;
						if (!scored) tot_score++;
						scored = TRUE;
						break; /* Look no further */
					}
				}
		} /* end switch */
		infosection(score);
		if (clearcmd) move(PROMPT_LINE, PRLEN);
		refresh();
	}
	printentries(elist);
	restoremenu();
}

/*
 *	A D D E N T R Y -- Add to entry list
 */
addentry(ab, num, set)
struct aberration *ab;
int num, set;
{
	char buf[20];
	struct entry *last;

	if (!estart) {
		aberror("Fatal: No list created");
		return(-1);
	}
	for (elp = estart; elp; elp = elp->next) {
		last = elp;
		if ((elp->col == ab->col) && (strcmp(elp->txt, ab->shortstr)==0)) {
			if (set) elp->num = num;	/* Direct set */
			else elp->num += num;		/* Update number */
			checknum(elp);
			return(elp->num);
		}
	} /* end for */
/*
 *	Add entry to end of list, or use unused entry
 */
	if (set) return(0);	/* No entry to reset */
 	elp = last;
	if (elp->col == 0)	{		/* Unused entry */
		elp->col = ab->col;
		elp->txt = ab->shortstr;
		elp->num = num;
		checknum(elp);
	} else {
		elp->next = makeentry(ab->col, num, ab->shortstr);
		elp = elp->next;
		elp->prev = last->prev;	/* Copy in case of info */
		last->prev = NULL;
		elp->foll = last->foll;
		last->foll = NULL;
	}
	return(elp->num);
} /* end addentry */

/*
 *	A D D C O M M E N T -- Add comment entry into list
 */
addcomment(elst, comstr)
struct entry *elst;
char *comstr;
{
	struct entry *elp, *elast;
	int clength, chtosave;
	char *cptr, *ctemp;

	if (comment == NULL) return(0);	/* No comment col specd, do nothing */
	clength = strlen(comstr);		/* Total length of comment */
	comment->totlen = clength;		/* Store in static struct */
	cptr = comstr;					/* Pointer to current bit */

	for (elp=elst; elp; elp = elp->next) {
		elast = elp;
		if (elp->col == comment->col)	{	/* This is the one !! */
			chtosave = (clength > comment->width) ? comment->width : clength;
			elp->num = 0;					/* Erase last entry */
			elp->txt = Freeptr(elp->txt);	/* Free old text */
			if (clength > 0) {
				elp->num = 1;
				elp->txt = strnsave(cptr, chtosave);
				cptr += chtosave;
				clength -= chtosave;
			}
		}
	}
	elp = elast;
	while (clength > 0) {
		elast = elp;
		chtosave = clength > comment->width ? comment->width : clength;
		elp->next = makeentry(comment->col, 1, strnsave(cptr, chtosave));
		elp->prev = elast->prev;	/* Copy in case of info */
		elast->prev = NULL;
		elp->foll = elast->foll;
		elast->foll = NULL;
		elp = elp->next;
		cptr += chtosave;
		clength -= chtosave;
	}
	return(0);
}

/*
 *	D O C O M M E N T -- Return All comments as concatenated string
 */
char *
docomment(elst)
struct entry *elst;
{
	static char commentstr[MAXCOMMENT];
	struct entry *elp;
	int i;

	for (i=0; i<MAXCOMMENT; i++) commentstr[i] = '\0';
	if (comment == NULL) return(commentstr);

	for (elp = elst; elp; elp = elp->next) {
		if ((elp->col == comment->col) && elp->num) {
			strcat(commentstr, elp->txt);
		}
	}
	return(commentstr);
}

/*
 *	U P D A T E S C R E E N -- Update number next to aberration on screen
 */
updatescreen(ab, num)
struct aberration *ab;
int num;
{
	int r, c;
	r = rowpos[*ab->key];
	c = colpos[*ab->key];
	if (num) {
		move(r, c);
		wprintw(stdscr, "%-3d", num); /* Note: 3 is NWIDTH */
	} else {
		repeatch(r, c, ' ', NWIDTH);
	}
} /* end updatescreen */

/*
 *	R E P E A T C H - Repeat number of characters from rol/col position given
 */
repeatch(r, c, ch, rptcount)
char ch;
int r, c, rptcount;
{
	int i;

	for(i=0; i<rptcount; i++) {
		move(r, c + i);
		addch(ch);
	}
}

/*
 *	U P D A T E N U M B E R S -- Update aberration scores on screen
 */
updatenumbers(score, e)
struct score_sheet *score;
struct entry *e;
{
	int i, j;
	struct aberration *ab;
	struct scoremenu *menu;

	menu = score->abmenu;

	for (i=menu->row; i < (menu->row + menu->nrows); i++) {
		for (j=0; j<NCOLS; j++)
			if (menu->col[j] >= 0)
				repeatch(i, menu->col[j], ' ', NWIDTH);
	}
	scored = FALSE;
	for(; e; e = e->next) {
		for(ab = score->aberr; ab; ab = ab->next) {
			if ((ab->col == e->col) && (strcmp(ab->shortstr, e->txt) == 0)) {
				scored = TRUE;
				updatescreen(ab, e->num);
			}
		}
	}
	clearoldcomment();
} /* end updatenumbers */

/*
 *	S E T U P M E N U -- Set up aberration score menu
 */
setupmenu(score)
struct score_sheet *score;
{
	int crow, ccol, endrow, colno;
	struct aberration *abe;
	struct scoremenu *abmenu, *acmenu;

	abmenu = score->abmenu;
	acmenu = score->acmenu;

	wclearw(stdscr);
	title(score->menuname);
	comment->totlen = MAXCOMMENT;	 /* Initialise */
	div_line(acmenu->row - 1);
	actionmenu(score);
	div_line(acmenu->row + acmenu->nrows);
/*
 *	The aberration menu
 */
	div_line(abmenu->row - 1);
	abe = score->aberr;
	crow = abmenu->row;
	colno = 0;
	ccol = abmenu->col[colno] + NWIDTH;
	endrow = abmenu->row + abmenu->nrows;
	while(abe) {
		rowpos[*abe->key] = crow;
		colpos[*abe->key] = ccol - NWIDTH;
		move(crow, ccol);
		addch((*(abe->key)));
		move(crow, ccol+NWIDTH-1);
		addstr((abe->longstr));
		abe = abe->next;
		if (++crow >= endrow) {
			if (++colno < NCOLS) {
				crow = abmenu->row;
				ccol = abmenu->col[colno] + NWIDTH;
			} else {
				aberror("Too many aberration menu items");
				break;
			}
		}
	}
	div_line(endrow);
	seterrors();
	clearcmdline();
	return;
}

/*
 *	R E S T O R E M E N U -- Set things back for cmdsys menu
 */
restoremenu()
{
	scrn_scroll(21, 24);
	scrn_pos(20, 0);
	scrn_clreos();
	scrn_pos(23, 0);
	return;
}

/*
 *	I N I T G L O B A L S -- Initialise some global arrays and pointers
 *	Mainly to prevent endless list searching
 */
initglobals(score)
struct score_sheet *score;
{
	tot_score = 0;
	initspecials(score);
	initentrylist();	/* Free old one (if any), and make new */
	return;
} /* end initglobals */

/*
 *	I N I T S P E C I A L S --	Initialise special array/comment
 */
initspecials(score)
struct score_sheet *score;
{
	int i, as;
	struct level *level, *startlevel;
	struct information *ip;
/*
 *	Scan level N header for specials
 */
	i=0;
	if (score->level2) startlevel = score->level2;
	else startlevel = score->level1;

 	for (level = startlevel; level; level = level->next) {
 		as = findresindex(level->assoc);
 		switch (as) {
 			case COMMENT:
				if (!comment)
					comment = (struct comm *)Calloc(1, sizeof(struct comm));
				comment->col = level->col;
				comment->width = level->width;
				break;	/* NOTE, x, y filled later from info list */
			case CELLNO:
			case POSN:
			case XPOS:
			case YPOS:
			case FUNCTION:
				specials[i].col = level->col;	/* Fill specials array */
				specials[i].resindex = as;
				specials[i].assoc = level->assoc;
				i++;
				break;
 			default:
 				break;
 		}
 		if (i > MAXSPECIALS) {
 			fprintf(stderr, "Too many special columns, rest ignored\n");
			break;
		}
 	}
 	if (i <= MAXSPECIALS) specials[i].col = -1;	/* End of array */
/*
 *	Find comment x, y position on screen
 */
	for(ip = score->info; ip; ip = ip->next) {
		switch (findresindex(ip->assoc)) {
			case COMMENT:
				comment->y = ip->y;
				comment->x = ip->x + strlen(ip->screentxt);
				break;
			default:
				break;
		}
	}
} /* end initspecials */

/*
 *	I N I T E N T R Y L I S T -- Initialise the entry list pointers
 */
initentrylist()
{
	elist = Freeentrylist(elist);
	elp = estart = elist = bay_info[bay].elist = makeentry(0, 0, NULL);
	eprev = NULL;
}

/*
 *	F R E E E N T R Y L I N E -- Free a line of entries (Not entire list)
 *	line is the start point in the line from which freeing is to occur
 *	1)	The first entry is zeroed, and kept
 *	2)	Freeing occurs until the last entry in the line
 *	3)	The pointers from the last entry are copied to the first entry,
 *		to link in the first entry.
 */
static struct entry *
Freeentryline(line)
struct entry *line;
{
	struct entry *free_entry, *linkptr, *pptr, *fptr;

	line->txt = Freetext(line);
	line->col = 0;
	line->printed = 0;
	line->num = 0;

	pptr = line->prev;
	fptr = line->foll;
	linkptr = line->next;

	while (linkptr) {
		free_entry = linkptr;
		pptr = linkptr->prev;
		fptr = linkptr->foll;
		linkptr = linkptr->next;
		Freeentry(free_entry);		/* Free this entry */
	}
/*
 *	Now link back in
 */
	line->prev  = pptr;
	line->next  = NULL;
	line ->foll = fptr;

	return(line);
}

/*
 *	F R E E E N T R Y L I S T -- Free of an entire list of entries
 */
struct entry *
Freeentrylist(list)
struct entry *list;
{
	struct entry *free_entry, *linkptr;

	if (alc->debug)
		fprintf(debug_channel, "Freeentrylist: in, free at 0x%x\n", list);
	linkptr = list;
	while (linkptr) {
		free_entry = linkptr;
		linkptr = (linkptr->next) ? linkptr->next : linkptr->foll;
		Freeentry(free_entry);
	}
	if (alc->debug) fprintf(debug_channel, "Freeentrylist: out\n");
	return(NULL);
}

/*
 *	F R E E E N T R Y -- Free off an entry with the following rules
 *
 *	1) If the entry is a comment, or special, free the text pointer
 *	2) The next/foll/prev pointers are assumed to be taken care of elsewhere
 *	3) The entry is then freed
 */
Freeentry(ent)
struct entry *ent;
{
	Freetext(ent);	/* Free text if Malloc'd */ /* ent NOT elp dcb 4/1/90 */
	Freeptr(ent);	/* Now free entry */
}

/*
 *	F R E E T E X T --	Free off the entry text pointer if this space was
 *						originally grabbed using Malloc. The rule is:
 *	If the entry text is a comment, or a special, then the space will have
 *	been grabbed using Malloc, otherwise it will have been assigned by
 *	a straight copy of a static text pointer. Checking is done by matching
 *	column numbers.
 */
static char *
Freetext(ent)
struct entry *ent;
{
	struct special *spec;
	if (ent->col) {
		if ((comment) && (ent->col == comment->col)) Freeptr(ent->txt);
		else {
			for (spec = specials; spec->col != -1; spec++)
				if (ent->col == spec->col) {
					Freeptr(ent->txt);
					break;
				}
		}
	}
}

/*
 *	T I T L E -- Put double width title on screen
 */
title(str)
char *str;
{
	int tcol;

	tcol = (40 - strlen(str)) / 2;
	move(TROW, tcol);
	addch(DBLWIDTH);				/* translated on output */
	move(TROW, tcol+1);
	addstr(str);
	return;
}

/*
 *	I N F O S E C T I O N -- Draw the information section of the screen
 */
infosection(score)
struct score_sheet *score;
{
	struct information *ip;
	struct association *sf;
	int sw;

	for(ip = score->info; ip; ip = ip->next) {
		move(ip->y, ip->x);
		addstr(ip->screentxt);
		move(ip->y, ip->x + strlen(ip->screentxt));
		if ((sw = findresindex(ip->assoc)) < 0) { /* Not reserved */
			/*
			 *	Scan form list for an associated abbr
			 */
 			for (sf = score->form; sf; sf = sf->next)
 				if (strcmp(ip->assoc, sf->assoc) == 0)
 					addstr(sf->data);
		} else {
			addstr(getrestext(sw, 0, 0));
		}
	} /* end for ip ... */
	move(PROMPT_LINE, getcol);
	return;
}

/*
 *	A C T I O N M E N U -- Put up action menu section of the screen
 */
actionmenu(score)
struct score_sheet *score;
{
	struct operation *act;
	struct scoremenu *acmenu;
	int row, col, endrow, colno;

	acmenu = score->acmenu;
	colno = 0;
	row = acmenu->row;
	col = acmenu->col[colno];
	endrow = acmenu->row + acmenu->nrows;

	for (act = score->action; act; act = act->next) {
		move(row, col);
		addch(*act->key);
		move(row, col+NWIDTH-1);
		addstr(act->menutxt);
		if (++row >= endrow) {
			if (++colno < NCOLS) {
				row = acmenu->row;
				col = acmenu->col[colno] + NWIDTH;
			} else {
				aberror("Too many action menu items");
				break;
			}
		}
	}
	return;
}

/*
 *	M A K E E N T R Y -- Make an aberration entry
 */
struct entry *
makeentry(col, num, txt)
int col, num;
char *txt;
{
	struct entry *scoreentry;

	scoreentry = (struct entry *)Calloc(1, sizeof(struct entry));
	scoreentry->col = col;
	scoreentry->num = num;
	checknum(scoreentry);
	scoreentry->printed = FALSE;
	if (num) scoreentry->txt = txt; /* Not (elp->num) */
	scoreentry->next = NULL;
	scoreentry->foll = NULL;
	return(scoreentry);
}

/*
 *	F I N D K E Y -- Scan action list, and find assoc. abbreviation for
 *	given key or return NULL if key not found
 */
char *
findkey(score, key)
struct score_sheet *score;
char key;
{
	struct operation *act;
	for(act = score->action; act; act = act->next) {
		if (*(act->key) == key) {
			if (alc->debug)
				fprintf(debug_channel, "findkey: Key %c, assoc %s\n",
					key, act->assoc);
			return(act->assoc);
		}
	}
	return(NULL);

}

/*
 *	A D D S P E C I A L S --	Add special information into entry list
 *								If not already done
 */
addspecials(score)
struct score_sheet *score;
{
	int i, funcval;
	char *txt;
	struct mlclust *ml;
	struct entry *e, *elast;
	struct function *fun;

	ml = alc->curbay->currptr->mlrec;
	for(i=0; i<MAXSPECIALS; i++) {
		if (alc->debug) {
			fprintf(debug_channel,
				"addspecials: specials[%d].col=%d\n", i, specials[i].col);
			fprintf(debug_channel,
				"addspecials: specials[%d].resindex=%d\n", i, specials[i].resindex);
			fprintf(debug_channel,
				"addspecials: specials[%d].assoc=%s\n", i, specials[i].assoc);
		}
		if (specials[i].col == -1) break; /* Done list */
		switch (specials[i].resindex) {
			case CELLNO:
			case POSN:
			case XPOS:
			case YPOS:
				txt = getrestext(specials[i].resindex, 1, specials[i].col);
				break;
			case FUNCTION:
				txt = NULL;	/* Default string is NULL */
				for (fun = score->func; fun; fun = fun->next) {
					if (fun->col == specials[i].col) { /* Found the function */
						funcval = dofunc(fun->def);
						if (alc->debug)
							fprintf(debug_channel, "Value %d\n", funcval);
						if (*(fun->true) != '\0')
							txt = (funcval) ? fun->true : fun->false;
						else
							txt = itoa("%d",funcval);
					}
				}
				break;
			default:
				fprintf(stderr, "Illegal special found\n");
				txt = "";
				break;
		} /* end switch */
		if (alc->debug)
			fprintf(debug_channel, "addspecials: txt = \"%s\"\n", txt);
		/*
		 *	Now put in list (If not already there)
		 */
		for (e = estart; e; e = e->next) {
			elast = e;
			if (e->col == specials[i].col) break;	/* Entry already there */
		}
		if (e == NULL) { /* Run off end of list, so no entry yet created */
			e = elast->next = makeentry(specials[i].col, 1, strsave(txt));
			e->prev = elast->prev;	/* Don't lose rest of entries */
			elast->prev = NULL;
			e->foll = elast->foll;
			elast->foll = NULL;
		}
	} /* end for i... */
} /* end addspecials */

/*
 *	E N D E N T R Y L I S T -- Scan through entry list to the end
 */
static struct entry *
endentrylist(ep)
struct entry *ep;
{
	struct entry *last;
	for(; ep; ep = ep->next) last = ep;
	return(last);
}

/*
 *	F I N D E N T R Y L I S T -- Scan through entry list until col matches
 *								 or returns last entry.
 */
static struct entry *
findentrylist(ep, col)
struct entry *ep;
{
	struct entry *last;
	for(; ep; ep = ep->next) {
		last = ep;
		if (ep->col == col) return(ep);
	}
	return(last);
}

/*
 *	P R I N T E N T R I E S -- Print entire entrylist - a debug function
 */
printentries(el)
struct entry *el;
{
	int line=0, epl;
	struct entry *e;

	if (alc->debug) {
		fprintf(debug_channel, "Entry list (0x%x)\n----------\n", el);
		fprintf(debug_channel, "Line (col,num,\"text\") (....) ... el=0xhhhh\n");
		do {
			fprintf(debug_channel, "%d\t", line);
			epl = 1;
			for (e=el; e; e = e->next) {
				fprintf(debug_channel, "(%d,%d,\"%s\")\t",
					e->col, e->num, e->txt);
				el = e->foll;
				if (epl++ % 5 == 0) fprintf(debug_channel, "\n\t");
			}
			fprintf(debug_channel, "el=0x%x\n", el);
			line++;
		} while(el);
	}
}

/*
 *	G E T K E Y -- Get keypress from stdin with echo
 *	clearcmd determines whether the command line, and error line should be
 *	cleared	before the key is echoed, the flag is reset if the command line is
 *	cleared. The key is only echoed if it a printable character
 */
char
getkey(clearcmd)
int *clearcmd;
{
	char c;

	c = getsingle();
	if (*clearcmd) {
		clearcmdline();
		clearerrline();
		*clearcmd = FALSE;
	}
	if (isprint(c)) {
		move(PROMPT_LINE, getcol++);
		addch(c);
	}
	move(PROMPT_LINE, getcol);
	refresh();
	return(c);
}

/*
 *	G E T S I N G L E -- Get single character from stdin
 */
char
getsingle()
{
	int n;
	char c;

	raw(fileno(stdin), &savestdin);
	while (_gs_rdy(fileno(stdin)) <= 0) tsleep(50);
	n = read(fileno(stdin), &c, 1) & 0377; 
	cook(fileno(stdin), &savestdin);
	if (alc->debug) fprintf(debug_channel, "getsingle: %c\n", c);
	return(c);
}

/*
 *	G E T S T R I N G -- Get string (Terminated by newline) from stdin
 */
getstring(str, max)
char *str;
int max;
{
	int n=0;
	char c;

	while ((c = getchar()) != '\n') {
		*str++ = c;
		if (++n == max) { fprintf(output, "\007"); fflush(stdout); str--; n--; }
	}
	*str = '\0';
}

/*
 *	A B E R R O R -- Simple print of error on aberration menu screen
 */
aberror(text)
char *text;
{
	move(ERROR_LINE-1, 1);	/* curscr pos is one less */
	addstr(text);
	refresh();
}

/*
 *	C H E C K N U M -- Check number within range, and adjust if necessary
 */
checknum(elp)
struct entry *elp;
{
	if (elp->num <= 0) {
		elp->num = 0;
	} else if (elp->num > MAXRANGE) {
		elp->num = MAXRANGE;
	}
}

/*
 *	C L E A R O L D C O M M E N T
 */
clearoldcomment()
{
	if (comment != NULL) repeatch(comment->y, comment->x, ' ', comment->totlen);
}

/*
 *	C L E A R C M D L I N E -- Clear old commands from PROMPT_LINE
 */
clearcmdline()
{
	getcol = PRLEN;
	move(PROMPT_LINE, 0); addstr(BLANKLINE);
	move(PROMPT_LINE, 0); addstr(PROMPT);
	move(PROMPT_LINE, PRLEN);
	addch(DBLWIDTH);
}

/*
 *	C L E A R E R R L I N E -- Clear the error line(s) (On screen function only)
 */
clearerrline()
{
	scrn_pos(ERROR_LINE-1, 1);
	scrn_clreol();
	scrn_pos(ERROR_LINE, 1);
	scrn_clreol();
}

/*
 *	C O N F I R M -- Output confirm prompt, and get reply
 */
confirm(txt)
{
	char c;

	scrn_pos(INPUT_LINE, 1);
	fprintf(output, txt);; fflush(output);
	scrn_clreol();
	if (((c = getsingle()) == 'y') || (c == 'Y')) return(1);
	else {
		clearerrline();
		clearcmdline();
		return(0);
	}
}

/*
 *	D I V _ L I N E -- Add dividing line on screen
 */
div_line(row)
{
	move(row, 0);
	addstr(DIV_LINE);
}

/*
 *	S E T E R R O R S --	Set screen for any error messages that may be
 *							written to stdout/stderr, to prevent them from
 *							trashing the screen
 */
seterrors()
{
	scrn_scroll(ERROR_LINE-1, ERROR_LINE);	/* Use line above as well */
	scrn_pos(ERROR_LINE, 1);	/* Here's where error messages go */
}

/*
 *	C E N T R E C E L L -- Prompt for cell to be centred, and adjust offests
 *	Returns:	0	- success
 *				-1	- fail
 *	This is a bit like fn. centrit, but copes with scoring screen
 */
centrecell()
{
struct	mlclust *mpt;
struct	mult_slide *bp;

	bp = alc->curbay;
	
	fprintf(output, "Please check cell centred and in focus\n");

	if (!confirm("Proceed (y/n) ? ")) {
		return(-1);
	}
	if (bp->startptr != NULL) {
		if (getpos(2)) return(-1);
		setoffsets(bp->currptr->mlrec);
	}
	return(0) ;
}

/*
 *	G E T S P E C I A L --	Given column number, return special code,
 *							or -1 if not found.
 */
getspecial(col)
int col;
{
	struct special *spec;
#ifdef GGG
	if (alc->debug) {
		fprintf(debug_channel, "getspecial: col %d\n", col);
		if (comment) fprintf(debug_channel,
			"getspecial: Comment present, col = %d\n", comment->col);
		fprintf(debug_channel, "getspecial: special cols are: ");
		for (spec = specials; spec->col != -1; spec++)
			fprintf(debug_channel, "%d ", spec->col);
		fprintf(debug_channel, "\n");
	}
#endif
	if (comment && (col == comment->col)) {
		if (alc->debug)
			fprintf(debug_channel, "getspecial: Col %d is comment, index=%d\n",
				col, COMMENT);
		return(COMMENT);
	}
	for (spec = specials; spec->col != -1; spec++)
		if (col == spec->col) {
			if (alc->debug)
				fprintf(debug_channel,
					"getspecial: Col %d is special, index=%d\n",
					col, spec->resindex);
			return(spec->resindex);
		}
	if (alc->debug) fprintf(debug_channel, "getspecial: col %d is NOT special\n", col);
	return(-1);
}

/*
 *	D O F U N C -- Evaluate a special column function
 */
dofunc(fstr)
char *fstr;
{
	int rv;
	char *sbuf, *sbufstart;

	if (fstr == NULL) return(0);
	sbufstart = sbuf = (char *)Malloc(strlen(fstr)+1);
	strcpy(sbuf, fstr);
	if (arith(&sbuf, &rv) < 0)
		fprintf(stderr, "Function evaluation error\n");
	Free(sbufstart);
	return(rv);
}

/*
 *	A R I T H -- Arithmetical evaluation
 */
arith(exp, rval)
char **exp;
int *rval;
{
	int ops[2], operands=0, err;
	char operator;

	while(TRUE) {
		switch(**exp) {
			case '(':	(*exp)++;	/* Skip it */
						if ((err = arith(exp, &ops[operands])) != 0)
							return(-1);
						operands++;
						break;
			case '\0':
			case '\n':
			case ')':	(*exp)++;		/* Skip it */
						*rval = ops[0];	/* Returned value */
						return(0);
			case ' ':	(*exp)++;		/* Skip white space */
						break;
			case '+':
			case '-':
			case '*':
			case '/':
			case '|':
			case '&':	if (operands == 0) ops[operands++] = 0;	/* Unary */
						operator = **exp;
						(*exp)++;		/* Skip it */
						break;
			default:	if ((err = eval(exp, &ops[operands])) != 0)
							return(err);
						operands++;
						while ( ((**exp >= '0') && (**exp <= '9')) ||
							((**exp|0x20) == 'x') || (**exp == '%') ||
							(((**exp|0x20) >= 'a') && ((**exp|0x20) <= 'f'))
						  )
				 	(*exp)++;	/* skip the characters */
		} /* end switch **exp... */
		if (operands == 2) { /* do we operate ? */
			switch (operator) {
			case '+':	ops[0] += ops[1];
						break;
			case '-':	ops[0] -= ops[1];
						break;
			case '*':	ops[0] *= ops[1];
						break;
			case '/':	if (ops[1] == 0) { /* divide by zero error */
							return(-1);
						}
						ops[0] /= ops[1];
						break;
			case '|':	ops[0] |= ops[1];
						break;
			case '&':	ops[0] &= ops[1];
						break;
			} /* end switch operator... */
			operands = 1;	/* Only one operand now */
		} /* end if operands... */
	} /* end while (TRUE) */
} /* end arith() */

/*
 * EVAL - Evaluate value of column number
 *		  If no error, return the value as val, and function value 0 (OK)
 *		  If error return -ve error code (Parameter error)
 *                val is undefined.
 */
eval(exp, val)
char **exp;
int *val;
{
	struct entry *e;
	int fcol;

	fcol = atoi(*exp);
	*val = 0;
	for (e=estart; e; e = e->next) {
		if (e->col == fcol) {
			*val = e->num;
			break;
		}
	}
	if (alc->debug)
		fprintf(debug_channel, "Col %d has value %d\n", fcol, *val);
	return(0);

}/* end eval */

						

