/*
 * allfree.c
 *
 * Free out all the objects in a "kcontrol" structure.
 * This version only works in a single processor,
 * multi-processor versions will need more thought.
 *
 * CAVEATS (1) as above (no multiprocessing)
 *         (2) watch out for shared property lists between
 *            ool, rol, mol objects - free independently ?
 *
 * Modifications
 *
 *	 2 Mar 1989		CAS			Remove old MIZAR/VAX defines
 *	23 Dec 1986		CAS			Convert bugfile to debug_channel
 *	 8 Oct 1986		CAS			Disabled allfree - since it would cause
 *								chaos if called
 *	17 Sep 1986		CAS			Renamed initkcont -> cleankcont
 *	15 Sep 1986		CAS			Equate bol and mol/ool to NULL seperatly
 *	 7 Aug 1986		GJP			Remove the cmdspace includes
 */
#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <allcontr.h>

extern struct allkontr *alc;
extern FILE *debug_channel;

static
allfree(kcont)
register struct kcontrol *kcont;
{
	register int i;
	if (alc->debug)
		fprintf(debug_channel,"In allfree : kcont->maxnumber %d\n",kcont->maxnumber);
	sleep(4);
	for (i=0; i<kcont->maxnumber; i++) {
		if (kcont->ool != NULL && kcont->ool[i] != NULL)
			freeobj(kcont->ool[i]);
		if (kcont->bol != NULL && kcont->bol[i] != NULL)
			freeobj(kcont->bol[i]);
		if (kcont->rol != NULL && kcont->rol[i] != NULL)
			freeobj(kcont->rol[i]);
		if (kcont->mol != NULL && kcont->mol[i] != NULL)
			freeobj(kcont->mol[i]);
	}
	cleankcont(kcont);
}

/*
 * this clears the kcont structure, BUT YOU MUST FIRST
 * HAVE DETTACHED THE ACTUAL OBJECT POINTER ARRAYS.
 */
cleankcont(kcont)
register struct kcontrol *kcont;
{
	register int i;
	
	kcont->status[0] = 0;
	kcont->maxnumber = 0;
	kcont->current_obj = 0;
	kcont->lock = 0;
	for (i=0; i<MAXOBJS; i++) {
		kcont->ool[i] = kcont->rol[i] = kcont->mol[i] = NULL;
		kcont->bol[i] = NULL;
		kcont->acl[i] = 0;
	}
}
