/*
 *      C O L O U R . C  --  Routines to set up tables for colour monitor
 *							display
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *
 *  Modifications
 *
 *       1 May 1992	Joanne	made green=red, since green was invisable on 
 *				monochrome screen
 *	20 Jul 1990		dcb			User LUTS straight grey scale
 *	19 Jul 1990		dcb			adjustlut: Only colour bottom 2 bit planes
 *	21 Aug 1989		CAs			Removed unused names defs
 *								Symbolic defs of colours
 *	07 Feb 1989		dcb			display/modify colours to devel pending investigation
 *	21 Sep 1988		CAS			Contrasting LUT 2 for black background
 *	16 Aug 1988		CAS			Add a reverse lut + black background
 *	26 Jul 1988		CAS			Removed 'adjustlut' debug message
 *	15 Jul 1988		CAS			Undo mod to clear background - done properly elsewhere
 *	30 Jun 1988		CAS			Remove all overlays for clear background
 *	18 May 1988		CAS			Major rewrite for user adjustable lut
 */
#include <cmdsys.h>
#include <allcontr.h>
#include <wstruct.h>
#include <display.h>
#include <colour.h>

#define ABS(x) (((x)>0) ? (x) : -(x))
/*
 *	Sizes for overlayed lut fiddling bits
 */
#define XORIG	1024
#define YORIG	1024
#define SIZE	2048
/*
 *	Cursor styles
 */
#define CURS_BOX	4
#define CURS_ARROW	6

extern struct allkontr *alc;

/*
 *	Forward function definitions
 */
int	latchred(), latchgreen(), latchblue(), latchall();
int		adjustlut(), lutmouse();

#ifdef OLD
/*
 *	Table of colour names
 */
char *colours[] = { " ", "grey", "fire", "grass", "clear", "shaded", "reverse"};
#endif
/*
 *	Table of default lut set points
 */
#define RED 0
#define GREEN 0
#define BLUE 2
#define POINTS 3
#define ALL 4
/*
 *	Lut's
 */
#define GREY	0
#define FIRE	1
#define GRASS	2
#define REVERSE 3
#define USER	4
#define MAXPTS 6		/* Max points along a lut */
static short cluts[6][4][MAXPTS] = {{ { 255,  0 },				/* Grey R */
									{ 255,  0 },				/* Grey G */
									{ 255,  0 },				/* Grey B */
									{   0,256 }},				/* Grey Points */
								{	{ 250,224,132,  0 },		/* Fire R */
									{ 240,160,  0,  0 },		/* Fire G */
									{ 176,  0,  0,  0 },		/* Fire B */
									{   0, 52,188,256 }},		/* Fire Points */
								{	{ 220,  0,  0,  0,  0 },	/* Grass R */
									{ 220,220, 80,  0,  0 },	/* Grass G */
									{   0,  0,  0, 40,  0 },	/* Grass B */
									{   0, 32,180,200,256 }},	/* Grass points */
								{	{   0,255 },				/* Reverse R */
									{   0,255 },				/* Reverse G */
									{   0,255 },				/* Reverse B */
									{   0,256 }},				/* Reverse Points */
								{	{ 255, 0 },	/* User R */
									{ 255, 0 },	/* User G */
									{ 255, 0 },	/* User B */
									{ 0, 256 }},/* User Points */
								{	{ 255, 0 },	/* User R */
									{ 255, 0 },	/* User G */
									{ 255, 0 },	/* User B */
									{ 0, 256 }},/* User Points */
							};

#ifdef OLDUSERPOINTS
								{	{   0,  0,190,  0,255 },	/* User R */
									{ 128, 64,  0,  0,255 },	/* User G */
									{ 255,  0,  0,  0,255 },	/* User B */
									{   0, 40, 80,120,256 }}	/* User Points */
#endif
/*

 *	Local static storage
 */
static int fixedcolours = 1;		/* Grey/Fire/Grass don't change */
static int cursorcolour;
static struct ipoint cursor;
static struct pframe frame;
static int table, tablecolour, tablepoint, tablefixed;

#ifdef UNUSED
/*
 *	C H A N G E C O L O U R  --  Change colour / LUT's of display.
 *
 */
changecolour(foreground,background,phase)
int		*foreground, *background,phase;
{
	int		col, Back_cmd, Rev_cmd;
	if (alc->debug)
		fprintf(stderr,"Changecol alc 0x%x fore 0x%x * %d, back 0x%x * %d\n",
			alc, foreground, *foreground, background, *background);
	newmenu("Colour scale and background");
	cmd(1,"grey");mouse(LEFTBT);
	cmd(2,"fire");mouse(MIDBT);
	cmd(3,"grass");mouse(RIGHTBT);
	cmd(4,"reverse");
	Back_cmd = cmd(5,"clear background");
	cmd(6,"shaded");
	Rev_cmd = cmd(7,"black");
	cmmd("display / modify current;%-+d%-+d%-+d%f",foreground,background,phase,adjustlut);mouse(LONGMB);

	while (col = thismu()) {
		if (col < Back_cmd) {
			*foreground = col;
			*background = 0;
		} else if (col <= Rev_cmd)
			*background = col - Back_cmd + 1;
		setcolour(*foreground,*background,phase);
	}
}

/*
 *	S E T C O L O U R
 *
 */
setcolour(foreground,background,phase)
int		foreground, background,phase;
{
	int		i;
	if (alc->debug)
		fprintf(stderr,"Colours - fore %d back %d \n",foreground,background);
	switch(foreground) {
		case CL_GREY:
			colourscale(GREY,2,phase); 
			setrgb(1,255,0,0);
			setrgb(2,0,0,255);
			setrgb(3,0,255,0);
			break;
		case CL_FIRE:
			colourscale(FIRE,2,phase); 
			setrgb(1,0,0,0);
			setrgb(2,0,0,255);
			setrgb(3,0,255,0);
			break;
		case CL_GRASS:
			colourscale(GRASS,2,phase); 
			setrgb(1,255,0,0);
			setrgb(2,0,0,255);
			setrgb(3,0,0,0);
			break;
		case CL_REVERSE:
			colourscale(REVERSE,2,phase);
			setrgb(1,255,255,255);
			setrgb(2,0,0,255);
			setrgb(3,0,255,0);
			break;
		case CL_SPEC:
			colourscale(USER+phase,2,phase);
			setrgb(1,255,0,0);
			setrgb(2,0,0,255);
			setrgb(3,0,255,0);
			break;
	}
/*
 *	Note: Overlay 2 must contrast (in green) with background
 */
	switch (background) {
		case 0: setrgb(0,255,255,255);
				break;
		case CL_CLEAR:
			setrgb(0,255,255,255);	/* Clear Background */
			setrgb(1,255,255,255);	/* OVERLAY1 disapears into background */
			break;
		case CL_SHADE:
			setrgb(0,197,197,197);	/* Shaded */
			setrgb(1,0,0,0);		/* Black is a good contrasting lut(1) */
			break;
		case CL_BLACK:
			setrgb(0,0,0,0);		/* Black*/
			setrgb(1,255,255,255);	/* White is a good contrasting lut(1) */
			setrgb(2,0,255,255);
			break;
	}			
	/* set a value in 3rd overlay plane */
	for (i = 4; i < 255; i += 8)
		setrgb(i,255,0,196);
}
#endif

/*
 *	A D J U S T L U T  --
 *
 */
adjustlut(fore,back,phase)
int		*fore, *back, phase;
{
	int		i, j;
	int		rlat,glat,blat;
	int		(*oldmouse)(), (*setmsv())();
	rlat = glat = blat = 0;
	for (i=0; i<256; i+=4) { /* Step on 4 not 8 - dcb */
/*
 *	Old code set bottom 3 bits in LUT
 *	remove for now. -  dcb
 		for (j=i+1; j<i+7; j++) {
			setrgb(j,255*(j&1),255*(j&2),255*(j&4));
		}
		setrgb(i+7,0,0,0);
 */
  		setrgb(i+1, 255, 0, 0);
  		setrgb(i+2, 0, 0, 255);
  		setrgb(i+3, 0, 255, 0);
	}
	table = *fore - 1;
	if (fixedcolours)
		table = USER;
	if (table == USER) table += phase;
	if (table < 0) table = 0;			/* Colours start from 0 not 1 */
	intens(1);
	plotall(table);
	simplehue(&cluts[table][POINTS][0], &cluts[table][RED][0],
			&cluts[table][GREEN][0], &cluts[table][BLUE][0], 2);
	frame.oy = frame.dx = 2048 ;
	frame.ox = frame.dy = 2048;
	frame.func = 0;
	frame.ix =  frame.iy = frame.scale = 1;
	cursor.type = 40;
	cursor.style = CURS_ARROW;
	cursor.k = XORIG + SIZE/2;
	cursor.l = YORIG + SIZE/2;
	cursorcolour = OVERLAY1;  
	drawlutcursor();
	oldmouse = setmsv(lutmouse);
	newmenu("Colour / Brightness adjustment");
	if (alc->devel) {
		cmmd("latch/unlatch red;%-+d%-+d%-+d%-+d%f",&rlat,&glat,&blat,&table,latchred); mouse(LONGLB);
		cmmd("latch/unlatch green;%-+d%-+d%-+d%-+d%f",&rlat,&glat,&blat,&table,latchgreen); mouse(LONGMB);
		cmmd("latch/unlatch blue;%-+d%-+d%-+d%-+d%f",&rlat,&glat,&blat,&table,latchblue); mouse(LONGRB);
	}
	cmmd("latch/unlatch all;%-+d%-+d%-+d%-+d%f",&rlat,&glat,&blat,&table,latchall); mouse(MIDBT);
	while(thismu())
		;
	intens(0);
	setmsv(oldmouse);
	plotall(table);
	eraselutcursor();
	*fore = table + 1 - phase;
}

/*
 *	P L O T A L L  --
 *
 */
plotall(table)
int		table;
{
	colour(1);  /*   was 7     joanne */
	moveto(XORIG,YORIG);
	lineby(SIZE,0);
	moveto(XORIG,YORIG);
	lineby(0,SIZE);
	colour(1<<RED);
	plotlut(&cluts[table][POINTS][0],&cluts[table][RED][0]);
	colour(1<<GREEN);
	plotlut(&cluts[table][POINTS][0],&cluts[table][GREEN][0]);
	colour(1<<BLUE);
	plotlut(&cluts[table][POINTS][0],&cluts[table][BLUE][0]);
}

/*
 *	L A T C H A L L  --  Align + drag RGB together
 *
 */
latchall(rlat,glat,blat,fore)
int		*rlat,*glat,*blat,*fore;
{
	eraselutcursor();
	if (*rlat && *glat && *blat) {
		*rlat = *glat = *blat = 0;
		cursor.style = CURS_ARROW;
		cursorcolour = RED; 		/* was OVERLAY2	    Joanne */
	} else {
		*rlat = *glat = *blat = 1;
		cursor.style = CURS_BOX;
		cursorcolour = GREEN;
		tablecolour = ALL;
		latchon(&cluts[*fore][0][0],ALL);
	}
	drawlutcursor();
}
/*
 *	L A T C H R E D  --  Latch on/off red lut
 *
 */
latchred(rlat,glat,blat,fore)
int		*rlat,*glat,*blat,*fore;
{
	*glat = 0; *blat = 0;
	eraselutcursor();
	if (*rlat) {
		*rlat = 0;
		cursor.style = CURS_ARROW;
		cursorcolour = OVERLAY1;
	} else {
		*rlat = 1;
		cursor.style = CURS_BOX;
		cursorcolour = 1<<RED;
		tablecolour = RED;
		latchon(&cluts[*fore][0][0],RED);
	}
	drawlutcursor();
}

/*
 *	L A T C H G R E E N  --  Latch on/off green lut
 *
 */
latchgreen(rlat,glat,blat,fore)
int		*rlat,*glat,*blat,*fore;
{
	*rlat = 0; *blat = 0;
	eraselutcursor();
	if (*glat) {
		*glat = 0;
		cursor.style = CURS_ARROW;
		cursorcolour = OVERLAY1;
	} else {
		*glat = 1;
		cursor.style = CURS_BOX;
		cursorcolour = 1<<GREEN;
		tablecolour = GREEN;
		latchon(&cluts[*fore][0][0],GREEN);
	}
	drawlutcursor();
}

/*
 *	L A T C H B L U E  --  Latch on/off blue lut line
 *
 */
latchblue(rlat,glat,blat,fore)
int		*rlat,*glat,*blat,*fore;
{
	*rlat = 0; *glat = 0;
	eraselutcursor();
	if (*blat) {
		*blat = 0;
		cursor.style = CURS_ARROW;
		cursorcolour = OVERLAY1;
	} else {
		*blat = 1;
		cursor.style = CURS_BOX;
		cursorcolour = 1<<BLUE;
		tablecolour = BLUE;
		latchon(&cluts[*fore][0][0],BLUE);
	}
	drawlutcursor();
}

#define SFACT 256	/* Scale factor */
/*
 *	L A T C H O N  --  Get hooked on to nearest point
 *
 *	Given the lut, hook on to it, update the cursor position
 *	and set up stuf for mouse tracking routine
 */
latchon(lut,col)
short		lut[4][MAXPTS], col;
{
	int		i, space, x,y,dist, nearx, neary, neardist, nearpoint, all = 0;
	int		distnext, distprev, intermed;
	space = 0;
	neardist = 6553600; nearpoint = 0;
	if (col == ALL) {
		all = 1;
		col = GREEN;
	}
	nearx = lut[POINTS][0]; neary = lut[col][0];
	for (i=0; i<MAXPTS-1; i++) {
		if (lut[POINTS][i] == 256) {
			space = 1;
			break;
		}
		dist = lindist(cursor.k,cursor.l,XORIG+(lut[POINTS][i]*SIZE)/256,YORIG+(lut[col][i]*SIZE)/256,
										XORIG+(lut[POINTS][i+1]*SIZE)/256,YORIG+(lut[col][i+1]*SIZE)/256,&x,&y);
		if (dist < neardist) {
			neardist = dist;
			nearx = x; neary = y;
			nearpoint = i;
		}
	}
	x = XORIG+(lut[POINTS][nearpoint]*SIZE)/256 - cursor.k;
	y = YORIG+(lut[col][nearpoint]*SIZE)/256 - cursor.l;
	distprev = x * x + y * y;
	x = XORIG+(lut[POINTS][nearpoint+1]*SIZE)/256 - cursor.k;
	y = YORIG+(lut[col][nearpoint+1]*SIZE)/256 - cursor.l;
	distnext = x * x + y * y;
	if (distnext < 2048)
		nearpoint++;
	else if ((distprev > 2048) && space) {					/* Insert a point */
		intens(0);
		colour(1<<RED);
		plotlut(&lut[POINTS][0],&lut[RED][0]);
		colour(1<<GREEN);
		plotlut(&lut[POINTS][0],&lut[GREEN][0]);
		colour(1<<BLUE);
		plotlut(&lut[POINTS][0],&lut[BLUE][0]);
		for (i=MAXPTS-1; i>nearpoint+1; i--) {
			lut[POINTS][i] = lut[POINTS][i-1];
			lut[RED][i]    = lut[RED][i-1];
			lut[GREEN][i]  = lut[GREEN][i-1];
			lut[BLUE][i]   = lut[BLUE][i-1];
		}
		nearpoint++;
		lut[POINTS][nearpoint] = ((nearx-XORIG)*256)/SIZE;
		if (all) {
			lut[RED][nearpoint] = lut[GREEN][nearpoint] = lut[BLUE][nearpoint] = ((neary-YORIG)*256)/SIZE;
		} else {
			intermed = (SFACT * (lut[POINTS][nearpoint] - lut[POINTS][nearpoint-1])) / (lut[POINTS][nearpoint+1] - lut[POINTS][nearpoint-1]);
			lut[RED][nearpoint] = lut[RED][nearpoint-1] + (intermed * (lut[RED][nearpoint+1] - lut[RED][nearpoint-1])) / SFACT;
			lut[GREEN][nearpoint] = lut[GREEN][nearpoint-1] + (intermed * (lut[GREEN][nearpoint+1] - lut[GREEN][nearpoint-1])) / SFACT;
			lut[BLUE][nearpoint] = lut[BLUE][nearpoint-1] + (intermed * (lut[BLUE][nearpoint+1] - lut[BLUE][nearpoint-1])) / SFACT;
		}
		intens(1);
		colour(1<<RED);
		plotlut(&lut[POINTS][0],&lut[RED][0]);
		colour(1<<GREEN);
		plotlut(&lut[POINTS][0],&lut[GREEN][0]);
		colour(1<<BLUE);
		plotlut(&lut[POINTS][0],&lut[BLUE][0]);
	} else if (!space) {
			if (distprev > distnext) nearpoint++;
	}
	if ((nearpoint == 0) || (lut[POINTS][nearpoint] == 256))
		tablefixed = 1;
	else
		tablefixed = 0;
	cursor.k = XORIG+(lut[POINTS][nearpoint]*SIZE)/256;
	cursor.l = YORIG+(lut[col][nearpoint]*SIZE)/256;
	tablepoint = nearpoint;
	simplehue(&lut[POINTS][0],&lut[RED][0],&lut[GREEN][0],&lut[BLUE][0],2);
}

/*
 *	L I N D I S T  --  Calc dist between pt + line
 *
 *	Point at (x,y)
 *	Line from (xa,ya) to (xb,yb)
 *	Nearest pt returned in (xn,yn)
 */
lindist(x,y,xa,ya,xb,yb,xn,yn)
int		x,y,xa,ya,xb,yb,*xn,*yn;
{
	int		grad, c, xc, yc;
/*	Line is y = grad x + c */
	grad = ((yb - ya) * SFACT) / (xb - xa);
	c = ya*SFACT - grad*xa;
/* Calc cross point of perpend line */
	xc = (grad * (y * SFACT - c) + SFACT * SFACT * x) / (grad * grad + SFACT * SFACT);
	yc = (grad * xc + c) / SFACT;
	if (xc > xb) {
		*xn = xb;
		*yn = yb;
	} else if (xc < xa) {
		*xn = xa;
		*yn = ya;
	} else {
		*xn = xc;
 		*yn = yc;
	}
	return((*xn - x) * (*xn - x) + (*yn - y) * (*yn - y));
}

/*
 *	P L O T L U T  --  Plot - given array of x points + array of y points
 *
 */
plotlut(points,col)
short	*points,*col;
{
	moveto(XORIG,YORIG+(*col * SIZE)/256);
	do {
		lineto(XORIG+(*(++points) * SIZE)/256, YORIG+(*(++col) * SIZE)/256);
	} while (*points != 256);
}

#ifdef UNUSED
/*
 *	C O L O U R S C A L E  --  Set up a colour scale from defaults
 *
 */
static int colourscale(col,m,phase)
int		col, m, phase;
{
	linearhue(&cluts[col][POINTS][0],&cluts[col][RED][0],&cluts[col][GREEN][0],&cluts[col][BLUE][0],m);
	if (fixedcolours && (col < USER)) {
		lutcopy(&cluts[USER+phase][0][0],&cluts[col][0][0]);
	}
}
#endif

/*
 *	L U T C O P Y  --  Copy a lut
 *
 */
static int lutcopy(to,from)
short	to[4][MAXPTS], from[4][MAXPTS];
{
	int		i, j;
	for (i=0; i<4; i++) {
		for (j=0; j<MAXPTS; j++) {
			to[i][j] = from[i][j];
		}
	}
}

/*
 *	L U T M O U S E  --  Mouse handler for lut fiddling
 *
 */
static int lutmouse()
{
	short		*points, *red, *green, *blue, *current;

	eraselutcursor();
	if (cursor.style == CURS_BOX) {
		intens(0);
		points = &cluts[table][POINTS][0];
		red = &cluts[table][RED][0];
		green = &cluts[table][GREEN][0];
		blue = &cluts[table][BLUE][0];
		if (tablecolour != ALL)
			current = &cluts[table][tablecolour][0];
		else
			current = green;
		if (!tablefixed || (tablecolour == ALL)) {
			colour(1<<RED);
			plotlut(points,red);
			colour(1<<GREEN);
			plotlut(points,green);
			colour(1<<BLUE);
			plotlut(points,blue);
		} else {
			colour(1<<tablecolour);
			plotlut(points,current);
		}
		current[tablepoint] += (alc->m2s->YINC*4*alc->mousescale*256)/SIZE;
		if (current[tablepoint] > 255)
			current[tablepoint] = 255;
		else if (current[tablepoint] < 0)
			current[tablepoint] = 0;
		cursor.l = YORIG + (current[tablepoint]*SIZE)/256;
		if (tablecolour == ALL) {
			red[tablepoint] = blue[tablepoint] = green[tablepoint];
		}
		if (!tablefixed) {
			points[tablepoint] += (alc->m2s->XINC*256*4*alc->mousescale)/SIZE;
			if (points[tablepoint] >= points[tablepoint+1]) {
				if (ABS(current[tablepoint]-current[tablepoint+1]) <= 2)
					closelut(&cluts[table][0][0],tablepoint);
				else
					points[tablepoint] = points[tablepoint+1] - 1;
				if (points[tablepoint] == 256) tablefixed = 1;
			} else if (points[tablepoint] <= points[tablepoint-1]) {
				if (ABS(current[tablepoint]-current[tablepoint-1]) <= 2) {
					closelut(&cluts[table][0][0],tablepoint);
					tablepoint--;
				} else
					points[tablepoint] = points[tablepoint-1] + 1;
				if (tablepoint == 0) tablefixed = 1;
			}
			cursor.k = XORIG + (points[tablepoint]*SIZE)/256;
		}
		intens(1);
		if (!tablefixed || (tablecolour == ALL)) {
			colour(1<<RED);
			plotlut(points,red);
			colour(1<<GREEN);
			plotlut(points,green);
			colour(1<<BLUE);
			plotlut(points,blue);
		} else {
			colour(1<<tablecolour);
			plotlut(points,current);
		}
		simplehue(points,red,green,blue,2);
	} else
		movelutcursor(alc->m2s->XINC*4*alc->mousescale,alc->m2s->YINC*4*alc->mousescale);
	drawlutcursor();
}

/*
 *	C L O S E L U T  --
 *
 */
closelut(lut,point)
short	lut[4][MAXPTS];
int		point;
{
	int		i;
	if (lut[POINTS][point] == 256) return;
	for (i=point; i<(MAXPTS-1); i++) {
		lut[RED][i] = lut[RED][i+1];
		lut[GREEN][i] = lut[GREEN][i+1];
		lut[BLUE][i] = lut[BLUE][i+1];
		lut[POINTS][i] = lut[POINTS][i+1];
	}
}

/*
 *	E R A S E L U T C U R S O R  --
 *
 */
eraselutcursor()
{
	intens(0);
	colour(cursorcolour);
	picframe(&cursor,&frame);
	intens(1);
}

/*
 *	D R A W L U T C U R S O R  --
 *
 */
drawlutcursor()
{
	colour(cursorcolour);
	picframe(&cursor,&frame);
}

/*
 *	M O V E L U T C U R S O R  --
 *
 */
movelutcursor(xinc,yinc)
int		xinc, yinc;
{
	cursor.k += xinc;
	cursor.l += yinc;
}
