/*
 * countsubs.c		17/1/84		Jim Piper
 *
 * subroutines to assist with interactively counting a cell file
 *
 * Modifications
 *
 *  17 Jun 1991		ih			Check for null obj in allstruct and addplist
 *   5 Jun 1991		ih			Free off non-null bol in allstruct (?)
 *	18 Nov 1988		CAS			Fix checkobj (precedence..)
 *	13 Jun 1988		CAS			split allstruct -> sep addplist fn
 *	11 Mar 1988		CAS			Moved checkobj here from segfuncs1
 *	21 Sep 1987		CAS			remove unused ref to explode
 *	29 Jan 1987		CAS			Cockup with kcont->FIP
 *	22 DEc 1986		CAS			Removed another aspect ratio message...
 *	18 Dec 1986		CAS			kcont->FIP symbolic values
 *	19 Nov 1986		CAS			Removed all bits duplicated in karylib
 *								displaysubs.c, moved frameobjnnum to woolz/basic
 *	28 Oct 1986		CAS			Debug
 *								Added a min size param to labeldc
 *	17 Oct 1986		CAs			Change local MAXOBJ to MAXOBJS - as it
 *								is in the rest of the system
 *	16 Oct 1986		CAS			Anothr bug in dispotype erase (tsize not set)
 *	13 Oct 1986		CAS			Bug in dispotype erase
 *	 8 Oct 1986		CAS			Countdisp -> ident.c
 *								modcount -> ident.c
 *	 1 Oct 1986		CAS			Changed dispblobs to dispexpblobs
 *	30 Sep 1986		CAS			Changed calloc to Calloc
 *	29 Sep 1986		CAS			Moved 'allread' to fileio.c
 *	10 Sep 1986		CAS			Removed def of MAXOBJ (its defined in kyy.h)
 *	20 Mar 1986		Denis		load alc->lkount in labeldc
 *
 */
#define P2N(a,b,c)
#define P2(a,b,c)	fprintf(stderr,"a b %d c %d\n",b,c); sleep(1)
#include <stdio.h>
#include <kyy.h>
#include <interact.h>
#include <chromanal.h>
#include <kcontrol.h>

#define MAX(i,j)	(i>j? i: j)
#define MIN(i,j)	(i<j? i: j)
#define WIDTH		8

extern FILE *debug_channel;
extern struct kcontrol *kcont;

/*
 * read objects, insert in control structure,
 * compute cell column and line centre and extent,
 * set suitable scales in frame.
 */
labeldc(kcont,minsiz) 
struct kcontrol *kcont;
int		minsiz;
{
		/* segment, ignore objects <= 2 pixels wide or tall */
		int lkount;
		label (alc->fipdig,&lkount,kcont->ool,MAXOBJS,minsiz);
		alc->lkount = kcont->maxnumber = lkount;
		alc->lbelld = 1;
		if (alc->debug)
			fprintf(stderr,"done label %d objs\n",lkount);
		allstruct(kcont);
		return(lkount);
}
/*
 *	A L L S T R U C T
 *
 */
allstruct(kcont)
struct kcontrol *kcont;
{
	struct chromosome *obj;
	struct object *blobmark();
	struct chromplist *makechromplist();
	int scale,objnum,cellkol1,cellline1,celllastkl,celllastln,celldiam,jo;
	struct pframe *f;
P2N(allstruct begin,kcont);	
	f = kcont->f;
	addplist(kcont);
	for ( jo = 0; jo < kcont->maxnumber; ++jo ) {
		if (alc->debug) {
			fprintf(debug_channel,"allstruct obj %d\n",jo);
		}
		obj = kcont->ool[jo] ;
		if (obj&&(obj->type == 1)) {
			if (kcont->bol[jo]) freeobj(kcont->bol[jo]);
			kcont->bol[jo] = blobmark(obj);
			kcont->acl[jo] = 1;
			if (! jo ) {
				cellline1 = obj->idom->line1;
				cellkol1 = obj->idom->kol1;
				celllastln = obj->idom->lastln;
				celllastkl = obj->idom->lastkl;
			} else {
				cellline1 = MIN(cellline1,obj->idom->line1);
				cellkol1 = MIN(cellkol1,obj->idom->kol1);
				celllastln = MAX(celllastln, obj->idom->lastln);
				celllastkl = MAX(celllastkl,obj->idom->lastkl);
			}
		}
	}
	alc->numchroms = kcont->maxnumber;

	scale = 8;
	celldiam = celllastln - cellline1;
	if (kcont->FIP == DIGFIP)
		celldiam = 3*celldiam/4;
	if (celldiam < celllastkl - cellkol1)
		celldiam = celllastkl - cellkol1;
	if (celldiam > 500) {
		scale = 8 * 480 / celldiam;
		fprintf(stderr,"default scale changed to %d\n",scale);
	}
	f->dx = 2048;
	f->dy = 2048;
	f->scale = 1;
	f->ix = scale;
	f->iy = ((kcont->FIP == DIGFIP) ? 3*scale/4 : scale);
	f->ox = (cellkol1+celllastkl)/2;
	f->oy = (cellline1+celllastln)/2;
}

/*
 *	A D D P L I S T   --   Add and initialise chromosome plist
 *
 */
addplist(kcont)
struct kcontrol *kcont;
{
	struct chromosome *obj;
	struct chromplist *makechromplist();
	int		jo;
	for ( jo = 0; jo < kcont->maxnumber; ++jo ) {
		if (alc->debug) {
			fprintf(debug_channel,"addplist obj %d\n",jo);
		}
		obj = kcont->ool[jo] ;
		if (obj&&(obj->type == 1)) {
			if (obj->plist == NULL) {
				obj->plist = makechromplist();
				obj->plist->number = jo+1;
			}
			/*
			 * if not already typed, make a chromosome
			 */
			if (obj->plist->otype == 0) {
				obj->plist->otype = 1;
				obj->plist->history[2] = 1;
			}
		}
	}
}


/*
 *	C H E C K O B J  --  Check valid obj num + is active
 *
 */
checkobj(onum)
int		onum;
{
	if ((onum < 1) || (onum > kcont->maxnumber) || ((kcont->acl[onum-1] & ACTIVE) != ACTIVE)) {
		printf("Invalid chromosome selection !\007\n");
		return(0);
	}
	return(1);
}


