/*
 *		C U R S M O V E . C
 *
 *	Routines associated witn cursor moving.
 *
 *  Written: 
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *  Date:    18th Sept 1986
 *
 *	From identbis.c which was loosely based on Jim Pipers ident.c
 *
 * Interactively identify displayed object by pointing, and
 * return a command.  Use the objectlist, marklist, activelist
 * type of data structure used in multi-processor karyotyping.
 *
 * Modifications
 *
 *	14 Sep 1988		CAS			Removed unused MOUSESCALE define
 *	 5 Aug 1988		CAS			Use alc->mousescale in tmousecurs
 */
#include <cmdsys.h>
#include <wstruct.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <allcontr.h>
#include <display.h>

/*
 * globals defined here, used elsewhere (WHY ???)
 */
struct pframe *idf;

/*
 * external references to other data
 */
extern struct ipoint cursor;
extern struct allkontr *alc;
extern struct kcontrol *kcont;

/*
 * static variables
 */
static cursorcolour = OVERLAY1;
static struct pframe idframe;


/*
 *	S E T K Y F R A M E
 *
 *	set up frame for use in cursor display during karyotype phase
 */
static
setkyframe()
{
	idframe.scale = 8;
	idframe.dx = alc->ddgschannel->xcenter;
	idframe.dy = alc->ddgschannel->ycenter;
	idframe.ix = 1;
	idframe.iy = 1;
	idframe.ox = 500;
	idframe.oy = 500;
}

/*
 *	C U R S F R A M E  -
 *
 */
cursframe(phase) {
	if (phase) {
		setkyframe();
		idf = &idframe ;
	} else
		idf = kcont->f;
}


/*
 *	N O C U R S O R  --
 *
 */
nocursor() {
	colour(OVERLAY1 | OVERLAY2);
	intens(0);
	picframe(&cursor,idf);
}

/*
 *	C U R S C O L  --  Change the cursor arrow colour
 *
 */
curscol(hue) {
	if (cursorcolour != hue) erasecursor();
	cursorcolour = hue;
	showcursor(cursor.l,cursor.k);
}

/*
 *	E R A S E C U R S O R
 *
 */
erasecursor()
{
	colour(cursorcolour);
	intens(0);
	picframe(&cursor,idf);
	intens(1);
}

short frinvxdig(),frinvydig();
/*
 *	A B S C U R S O R  --
 *
 */
abscursor(ll,kk) {
	showcursor( frinvydig(ll,idf), frinvxdig(kk,idf));
}

/*
 *	S H O W C U R S O R  --
 *
 */
showcursor(ll,kk)
{
	erasecursor();   /* rely on erase to set current cursor colour ! */
	cursor.k = kk;
	cursor.l = ll;
	intens(1);
	picframe(&cursor,idf);
}
/*
 *	Temp fix until I can work out how to unscramble ident/cursmove
 *									and segfix/karyfix
 */
tmousecurs()
{
	mousecurs(&cursor,idf,alc->m2s,alc->mousescale,cursorcolour);
}

/*
 *	T R A C K 1  -  
 *
 */
track1(overlay)
{
	mousecurs(&cursor,idf,alc->m2s,1,cursorcolour);
}

/*
 *	T R A C K X  -  
 *
 */
trackx(scale)
{
	mousecurs(&cursor,idf,alc->m2s,scale,cursorcolour);
}

/*
 *	F A R C U R S O R
 *
 */
farcursor (jdom) 
register struct intervaldomain *jdom;
{
	if (cursor.l < jdom->line1 || cursor.l > jdom->lastln ||
		cursor.k < jdom->kol1 || cursor.k > jdom->lastkl) return(1);
	return(0);
}

