/* Copyright MRC CAPCU March 1986
 * denshisto.h
 *
 * read and display density histogram from MIPROC via PIA
 *
 * Modifications
 *
 *  20 Aug 1900		ih		Added a piabase parameter to readpia etc
 *	10 Feb 1989		dcb		Use top 0.1 % to find top val.
 *	20 Jan 1989		dcb		Added findtop to give 'top value' in histogram
 *							gettop returns it.
 *	12 May 1987		SCG		Write hvals in debug channel.
 *	01 May 1987		SCG		Write raw histo data to a file in debug mode.
 *	 5 Mar 1987		CAS		interrupt(2) -> (102) and removed another
 *							goto in line with GJP on SCG's machine
 *	23 Dec 1986		CAS		Changed bugfile into debug_channel
 *	13 Oct 1986		CAS		NULL histo after freeing
 *	 6 Oct 1986		GJP		Added a freobj to erashisto
 *	30 Sep 1986		CAS		Convert malloc to Malloc
 *	15 Sep 1986		CAS		Manual conv to long names (style -> r)
 *	25 Jul 1986		CAS		Added timeout to openpia call
 *	21 Mar 1986		Denis	erasehisto added
 *	20 Mar 1986		Denis	Problem:
 * 		consistent failure on repeat digitisation + histo, but unable
 * 		identify actual cause. Split histo/display (showhisto) and read
 * 		Miproc before display has fixed - for the time.
 */
#include <stdio.h>
#include <wstruct.h>
#include <allcontr.h>

#define HLENG 256
#define HLENG2 512
#define HHEIGHT	3500	/*histogram max displayed height*/
#define APERCENT 0.1	/* Percentage of values allowed above 'max. value' */
#define X0	512
#define Y0	512
#define XSCALE 12


extern struct allkontr *alc;
extern FILE *debug_channel;

static oldt1=-1,oldt2;
struct object *makemain();
static struct object *h = NULL;
static struct histogramdomain *hdomp = NULL;
static int tv;	/* Histogram top value */

/*
 *
 *	D E N S H I S T O     -   density histogram stuff
 *
 */
denshisto() 
{
struct pframe f;
register int i, j, *hvp;
short hbuf[HLENG];
int hmax, maxbin;

	if (openpia(0,8,alc->piabase) != 0) {
		fprintf(stderr,"DENSHISTO: failed to open PIA\n");
		return;
	}
	else if (alc->debug)
		fprintf(debug_channel,"DENSHISTO opened PIA\n");
	/* make histogram */
	if (h == NULL) {
		hdomp = (struct histogramdomain *) Malloc(sizeof(struct histogramdomain) +
		       HLENG * sizeof(int));
		hdomp->type = 1;
		hdomp->r = 1;	/* display with horizontal base */
		hdomp->l =  hdomp->k = 0;
		hdomp->npoints= HLENG;
		hdomp->hv = (int *) (hdomp + 1);
		h = makemain(13,hdomp,NULL,NULL,NULL);
	}
	hvp = hdomp->hv;
	/* read data */
	i = readpia(hbuf,HLENG2,alc->piabase);
	if (i != 512) {
		if (alc->debug)
			fprintf(debug_channel,"DENSHISTO: read %d bytes\n",i);
		interrupt(102);
	}
	closepia(alc->piabase);
	if (alc->debug) {
		fprintf(debug_channel,"DENSHISTO After pia\n");
	}
	/* copy into histogram */
	hmax = 0;
	maxbin = 0;
	for (i=0; i<HLENG; i++) {
		if(alc->debug == 3) {
			fprintf(debug_channel, "%4d ", hbuf[i]);
			if (i && (i % 14) == 0) fprintf(debug_channel, "\n");
		}
		*hvp = hbuf[i];
		if (hmax < *hvp) {
			hmax = *hvp;
			maxbin = i;
		}
		++hvp;
	}
	if (alc->debug == 3) fprintf(debug_channel, "\n");
	if ((tv = findtop(h)) < 0) {	/* Find top value in this histogram */
		fprintf(stderr, "\007Can't find top value in histogram (error %d)\n",
			tv);
	}
	/*
	 * Scale to give max value = HHEIGHT
	 */
	if (hmax > HHEIGHT) {
		hvp = hdomp->hv;
		for (i=0; i<HLENG; i++)  *hvp++ = (*hvp * HHEIGHT ) /hmax ;
	}
	/* smooth not very brutally */
	histosmooth(h,2); 
	return(h);
}

/*
 *
 *	E R A S E H I S T O  -   erase the histogram from the screen
 *							 and delete the histogram if it existed
 *
 */
 erasehisto() {
	if (oldt1 < 0 || !h ) return;
	intens(0);
	showhisto(oldt1,oldt2);
	intens(1);
							/* we have deleted the object on the scren also 
							   delete the object in memory */
	freeobj(h);
	h = NULL;
	oldt1 = -1;
}

/*
 *
 *  S H O W H I S T O -  display histogram generated as above, with 
 *						 graticule and threshold marks
 *
 */
showhisto(t1,t2) {
	struct pframe f;
	register int i, j, fix5;
						/* record t1,t2 values for erasure */
	oldt1 = t1; oldt2 = t2;
						/* set frame and display */
	f.scale = 1;
	f.ix = XSCALE;
	fix5 = 5*f.ix;
	f.iy = 1;
	f.dx = X0;
	f.dy = Y0;
	f.ox = f.oy = 0;
	colour(2);
	picframe(h,&f); 
	colour(1);
	moveto(f.dx,f.dy-64);
	lineby(f.ix*HLENG,0);
	for (i=0; i<HLENG; i+=10) {
		moveto(f.dx+i*f.ix,f.dy-64);
		lineby(0,-32);
		moveby(fix5,32);
		lineby(0,-16);
	}
	for (i=0; i<HLENG; i+=50) {
		j = f.dx+i*f.ix;
		moveto(j,f.dy-64);
		lineby(0,-64);
		dispnum(i,j-32,f.dy-224);
	}
	lwidth(2);
	moveto(f.dx+XSCALE*t1,f.dy);
	lineby(0,200);
	moveto(f.dx+XSCALE*t2,f.dy);
	lineby(0,200);
	lwidth(1);
}

/*
 * F I N D T O P - Find top value in histogram, this is defined
 * as being the point above which there are a certain percentage of
 * pixel values. Currently fixed at compile time as APERCENT
 */
static int
findtop(histo)
struct object *histo;
{
	struct histogramdomain *histogram;
	register int i, *idv, total, topvals;

	if (histo->type != 13) return(-1); /* Wrong type */

	histogram = (struct histogramdomain *)histo->idom;
	if (histogram->type != 1) return(-2); /* Don't cope yet */

	/*
	 *	Accumulate total number of pixels
	 */
	idv = histogram->hv;
	total = 0;
	for (i=0; i<HLENG; i++) total += *idv++;
	if (alc->debug) fprintf(debug_channel, "Total pixel values = %d\n", total);
	if(total == 0) return(-3);
	/*
	 * Work out percentage value
	 */
	topvals = (int)((APERCENT * total)/100);
	if (alc->debug) fprintf(debug_channel, "topvals = %d\n", topvals);
	/*
	 * Now step back until this is exceeded
	 */
	total = 0; /* Now accumulate total from the other end */
	for (i=HLENG; i>=0; i--)
		if (total < topvals) total += *(--idv);
		else break;
	i++; /* One step in this direction means we are UNDER the % limit */
	if (alc->debug) fprintf(debug_channel, "Top histogram value = %d\n", i);
	return(i); /* Return this value */
} /* end findtop */

/*
 * G E T T O P -- Get top histogram value.
 * This function is in place of putting tv in global store
 */
gettop()
{
	return(tv);
} /* end gettop */


