/*
 *      D I R O P S . C  --  Directory display operations
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    16th March 1987
 *
 *  Modifications
 *
 *	28 Jan 1991	jimp	Modifications for flexible screen.  Arrange to
 *				insert "screens" into cellist[MAXCELL] with flag
 *				bitno(FLEXFILE) set.  Print BELOW the list of
 *				metaphase/karyotype files.  Fixed a bug in
 *				addcell() - should call newcell(bitno(typ),cell).
 *	15 Dec 1989		CAS			Get one place where CELLCOLUMN wasn't used
 *	25 Jul 1989		dcb			Add scorefile type, move CELLCOLUMN to 45
 *	 1 Apr 1987		CAS			Handle screen break properly - ie display
 *								Correct case/slide at top of next screen
 *	30 Mar 1987		CAS			Procede - Act on 'No' response
 *
 */
#include <cmdsys.h>
#include <kyy.h>

#define STARTLINE 5
#define MAXLINE 20
#define MAXCELL	100		/* Max number of cells we can handle per case */

#define METCOLUMN 36
#define DATCOLUMN 39
#define SCORECOLUMN 42
#define CELLCOLUMN 45

extern struct allkontr *alc;
extern FILE *debug_channel;

static	int	linenumber;
static	struct cyto_file last;

/*
 *	C Y T O _ D I R  --  Display in a pretty sort of way an array of
 *							cyto type files
 *
 *	Uses lines STARTLINE onwards on screen
 *	continu - used to indicate still working on same dir - 
 *		0 - start from scratch
 *		1 - New case etc - but screen not moved
 *		2 - Same case - carry on as if no break
 */
cyto_dir(cfiles,nfiles,continu)
struct cyto_file *cfiles;
int		nfiles, continu;
{
	struct cyto_file *cfp;
	int		i;

	switch (continu) {
		case 0:
			clearlns(STARTLINE,ENDLINE);
			screengo(STARTLINE,1);
			linenumber = STARTLINE;
		case 1:
			last.case[0] = '\0';
			last.slide[0] = '\0';
			clrcells();
		case 2:
			break;
	}
	for (cfp=cfiles; cfp < cfiles + nfiles; cfp++) {
		if (alc->debug) {
			fprintf(debug_channel,"type %d pat %s sli %s cell %d\n",cfp->type,
				cfp->case,cfp->slide,cfp->cellno);
		}
/*
 *	If change in case - flush last case cells
 */
		if (strcasecmp(last.case,cfp->case)) {
			if (showcells(CELLCOLUMN,4,&last) == 0)
				return;
			clrcells();
			if (newcase(cfp) == 0)
				return;
		}
/*
 *	If change in slide - flush last slide cells 
 */
		if (strcasecmp(last.slide,cfp->slide)) {
			if (showcells(CELLCOLUMN,4,&last) == 0)
				return;
			clrcells();
			if ((i = nextline(cfp,1)) == 1)
				newslide(cfp);
			else if (i == -1)
				return;
		}
/*
 *	Accumulate this case/slide
 */
		switch (cfp->type) {
			case METFILE:
				scrn_column(METCOLUMN);
				printf("ML");
				fflush(stdout);
				break;
			case GDATFILE:
				scrn_column(DATCOLUMN);
				printf("DF");
				fflush(stdout);
				break;
			case SCOREFILE:
				scrn_column(SCORECOLUMN);
				printf("SC");
				fflush(stdout);
				break;
			case CELLFILE:
			case KARYFILE:
			case FLEXFILE:
				addcell(cfp->type,cfp->cellno);
				break;
		}
	}
/*
 *	Flush final case/slide cells
 */
	if (showcells(CELLCOLUMN,4,&last) == 0)
		return;
	clrcells();
	printf("\n");
    if (go_on());
}

/*
 *	N E W C A S E  --
 *
 * Returns:		1 if OK
 *				0 if nextline saw 'No' rewsponse to procede qn
 */
newcase(cfp)
struct cyto_file *cfp;
{
	int		i;
	if ((i = nextline(cfp,2)) == 1) {
		printf("Case: ");
		scrn_rev();
		printf("%16s",cfp->case);
		scrn_norm();
		strcpy(last.case,cfp->case);
		newslide(cfp);
		return(1);
	} else if (i == -1)
		return(0);
	return(1);
}

/*
 *	N E W S L I D E  --
 *
 */
newslide(cfp)
struct cyto_file *cfp;
{
	scrn_column(23);
	printf("slide %s",cfp->slide);
	strcpy(last.slide,cfp->slide);
	fflush(stdout);
}

/*
 *	Cell list sort + accumulate routines and static storage
 *	=======================================================
 */

struct clist {	int	cell;
				int	flag;
			};
static struct clist cellist[MAXCELL];

/*
 *	C L R C E L L S  ---
 *
 */
static
clrcells()
{
	struct clist *ptr;
	int		i;

	ptr = cellist;
	for (i=0; i<MAXCELL; i++) {
		ptr->cell = 0;
		ptr->flag = 0;
		ptr++;
	}
}

/*
 *	F I N D C E L L  ---
 *
 */
static
findcell(cell)
int		cell;
{
	struct clist *ptr;
	int		i;

	ptr = cellist;
	for (i=0; i<MAXCELL; i++) {
		if (ptr->cell == cell)
			return(ptr);
		if (!ptr->flag)
			break;
		ptr++;
	}
	return(NULL);
}

/*
 *	N E W C E L L  --
 *
 */
static
newcell(typ,cell)
int	typ,cell;
{
	struct clist *ptr;
	int		i;

	ptr = cellist;
	for (i=0; i<MAXCELL; i++) {
		if (!ptr->flag) {
			ptr->cell = cell;
			ptr->flag = typ;
			return(ptr);
		}
		ptr++;
	}
	return(NULL);
}

/*
 *	A D D C E L L  --
 *
 */
static
addcell(typ,cell)
int	typ,cell;
{
	struct	clist *ptr;

	if ((typ != CELLFILE) && (typ != KARYFILE) && (typ != FLEXFILE)) return(0);
	if (ptr = (struct clist *) findcell(cell))
		ptr->flag |= bitno(typ);
	else 
		newcell(bitno(typ),cell);
}

/*
 *	C S O R T  --
 *
 */
static
csort(c1,c2)
struct clist *c1, *c2;
{
	return (c1->cell-c2->cell);
}

/*
 *	S H O W C E L L S  --
 *
 *	return:
 *			1	OK
 *			0	'No' response to procede
 */
static
showcells(skol,columns,cfp)
int	skol, columns;
struct cyto_file *cfp;
{
	struct clist *ptr;
	int		ncells,i,k;

	ptr = cellist;
	for (ncells=0; ncells<MAXCELL; ncells++) {
		if (!ptr->flag)
			break;
		ptr++;
	}
	if (!ncells)
		return(1);

	if (alc->debug)
		fprintf(debug_channel,"qsort %d\n",ncells);

	if (ncells > 1)
		qsort(cellist,ncells,sizeof(struct clist),csort);

	k = 0;
	ptr = cellist;
	scrn_column(skol);
	for (i = 0; i<ncells; i++) {
		if (k == columns) {
			if (nextline(cfp,1) == -1)
				return(0);
			scrn_column(skol);
			k = 0;
		}
		if (ptr->flag & (bitno(CELLFILE) | bitno(KARYFILE))) {
			printf(" %4d",ptr->cell);
			if (ptr->flag & bitno(KARYFILE))
				printf("K");
			else
				printf(" ");
			k++;
		}
		fflush(stdout);
		ptr++;
	}
	k = columns;	/* force a new-line */
	ptr = cellist;
	scrn_column(skol);
	for (i = 0; i<ncells; i++,ptr++) {
		if (ptr->flag & bitno(FLEXFILE)) {
			if (k == columns) {
				if (nextline(cfp,1) == -1)
					return(0);
				scrn_column(skol);
				k = 0;
			}
			printf(" %4dF",ptr->cell);
			k++;
		fflush(stdout);
		}
	}
	return(1);
}


/*
 *	N E X T L I N E  --  Bump display line
 *
 *	Returns:
 *		-1	'No' response to procede 
 *		0  if increment caused loopround
 *		1  otherwise
 */
nextline(cfp,count)
struct cyto_file *cfp;
int		count;
{
	int	i;
	linenumber += count;
	if (linenumber > MAXLINE) {
		screengo(ENDLINE,0);
		if (!go_on())
			return(-1);
		clearlns(STARTLINE,ENDLINE);
		screengo(STARTLINE,1);
		linenumber = STARTLINE;
		if (newcase(cfp) == 0)
			return(-1);
		return(0);
	} else
		for (i=0; i<count; i++)
			printf("\n");
		return(1);
}
