/*
 *      F I E L D F U S E . C  --  Field fusion stuff
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *  Date:    12th August 1988
 *
 *  Modifications
 *
 *	29 Mar 1990		dcb			Fixed origin offset within quadrant
 *	12 Mar 1990		dcb			Use lin_colour_lut in fieldfuse, and initfuse
 *	07 Mar 1990		dcb			Remove OVERLAY3 use, in initfuse
 *	28 Feb 1990		dcb			Apply current LUT instead of using llut
 *	14 Dec 1989		dcb			fieldfuse: Don't return if global command typed
 *	31 Aug 1989		dcb			fieldfuse: Only call fgfreeze if a virtuoso f/s
 *	 2 Jun 1989		dcb			Clear screen & reset lut on exit
 *	19 May 1989		dcb			Don't set main frame if FIP digitised. Keep
 *								fused frames under devel option
 *	 8 May 1989		dcb			Allow selection of objects in main field (again)
 *	26 Apr 1989		dcb			Don't set frames if FIP digitised (already done)
 *								Change mouse, and add quit option.
 *  24 Jan 1989		SEAJ		Changed order of mouse buttons in fieldfuse.
 */
#include <cmdsys.h>
#include <kyy.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <ddgs.h>
#define LOCATEADD	8	/* Locate proximity control */
#define ABS(x) (((x) >= 0) ? (x) : (-x))
#define ORIGIN_OFFSET -20 /*Posn in obj space to be located at quadrant origin*/

/*
 *	External things to import
 */
extern FILE *debug_channel;
extern struct kcontrol *kcont;
extern struct ipoint cursor;

/*
 *	Fusion related data
 */
#define FUSEMAX 3	/* Max number of fuse images */

struct object *fuseobj[FUSEMAX][MAXOBJS];	/* Fuse object lists */
struct pframe fuseframe[FUSEMAX+1];			/* Fuse frames */
int fusecount[FUSEMAX];						/* Number of objects in fuseobj lists */
int fuseframes = 0;							/* Number of frames grabbed for fusing */
static int xsplit, ysplit;					/* Positions of fuse windows */
static int quad, objnum = -1;				/* Current field + object */
static selectfield, selectobj = -1;			/* Object selected for transfer */
static struct object *selectboundary = NULL;/* Boundary around selected object */

/*
 *	Internal function forward declarations
 */
static int trackfuse();					/* Cursor track routine */
static int includeobj();
static int transferobj();

/*
 *	F I E L D F U S E  --  Fuse all the bits we got together
 *
 */
fieldfuse()
{
	int	Ex_cmd, Qu_cmd = -1, stat;

	if ((alc->ddgschannel->type == FS_VIRTUOSO) ||
		(alc->ddgschannel->type == FS_VIRTUOSO_INVERT))
			fgfreeze();
	initfuse();
	cursor.type = 40;
	cursor.style = 6;
	cursor.k = kcont->f->ox;
	cursor.l = kcont->f->oy;

	cursframe(0);
	curscol(OVERLAY2);
	setmsv(trackfuse);
	newmenu("Field fuse");
	cmmd("select object ;%f",includeobj);mouse(MIDBT);
	cmmd("transfer object ;%f",transferobj);mouse(RIGHTBT);
	Ex_cmd = cmmd("accept and exit ");mouse(LONGMB);
	if (alc->devel) {
		Qu_cmd = cmmd("accept/exit - keep fuse frames "); mouse(LONGLB);
	}
	while (((stat = thismu()) != Ex_cmd) && (stat != Qu_cmd)) ;

	setmsv(NULL);
	if (nncmd == Ex_cmd) freefuse();
	kcont->maxnumber = alc->lkount;		/* Keep these the same */
	qclear();
	lin_colour_lut();			/* Apply linear LUT, and overlay colours */
}

/*
 *	I N C L U D E O B J  -- 
 *
 */
static int includeobj()
{
	struct object *bobj, *obj, *ibound();
	struct pframe *fr;
	if (objnum >= 0) {
		switch (quad) {
			case 0:
/*
 *	This case was commented out
 */
				obj = (struct object *) kcont->ool[objnum];
				fr = &fuseframe[FUSEMAX];
				break;
			case 1:
			case 2:
			case 3:
				obj = fuseobj[quad-1][objnum];
				fr = &fuseframe[quad-1];
				break;
			default:
				obj = NULL;
				fr = NULL;
		}
		if (obj) {
			colour(OVERLAY1);
			if (selectboundary) {
				intens(0);
				picframe(selectboundary,fr);
				freeobj(selectboundary);
			}
			selectboundary = ibound(obj,1);
			intens(1);
			picframe(selectboundary,fr);
			selectfield = quad - 1;
			selectobj = objnum;
		}
	}
}

/*
 *	T R A N S F E R O B J  -- Menu callable transfer object function
 *
 */
static int
transferobj()
{
	if (selectobj == -1) return;	/* No object selected */

	if (selectfield == -1) {		/* Selected main field */
		if (quad == 0) {
			printf("Can't transfer to here\n");
			return;
		}
		transfer(selectobj, &fuseframe[FUSEMAX], &fuseframe[quad-1],
			&alc->lkount, &fusecount[quad-1], kcont->ool, fuseobj[quad-1]);
	} else {						/* Selected other field */
		if (quad != 0) {
			printf("Can't transfer to here\n");
			return;
		}
		transfer(selectobj, &fuseframe[selectfield], &fuseframe[FUSEMAX],
			&fusecount[selectfield], &alc->lkount,
			fuseobj[selectfield], kcont->ool);
	}
	selectobj = -1;
} /* end transferobj */

/*
 *	T R A N S F E R -- Do the actual business of object transferral
 *
 *	In:	Selected object no,
 *		from frame, to frame,
 *		from obj count, to obj count
 *		from obj list, to obj list
 *	Out: from obj count, and to obj count altered, as are the object lists
 */
static int
transfer(onum, ffr, tfr, foc, toc, fol, tol)
int onum, *foc, *toc;
struct pframe *ffr, *tfr;
struct object **fol, **tol;
{
	int		i, prefx, prefy, oldpmode;
	struct object *bobj, *ibound();
/*
 *	Un-display old
 */
	intens(0);
	oldpmode = pixelmode(2);
	colour(GREYCOLS);
	picframe(fol[onum], ffr);
	pixelmode(oldpmode);
/*
 *	Convert cursor to ddgs
 */
	prefx = cursor.k * (kcont->f->ix * kcont->f->scale);
	prefy = cursor.l * (kcont->f->iy * kcont->f->scale);
/*
 *	Now localise to real position in our frame
 */
	prefx = ((prefx - tfr->dx) / (tfr->scale * tfr->ix)) + tfr->ox;
	prefy = ((prefy - tfr->dy) / (tfr->scale * tfr->iy)) + tfr->oy;
/*
 *	Then see if we can fit new object in here
 */		
	fitobj(fol[onum], prefx, prefy, tol, *toc);
/*
 *	Transfer from one list to the other + do relevant screen stuff
 */
	tol[*toc] = fol[onum]; /* Add object to end of 'to' list */
	(*toc)++;
	intens(1);
	colour(GREYCOLS);
	picframe(fol[onum], tfr);
	(*foc)--;
	for (i=onum; i < *foc; i++)
		fol[i] = fol[i+1];			/* Shuffle objects down */
	fol[i] = NULL;
	if (selectboundary) {
		freeobj(selectboundary);
		selectboundary = 0;
	}
} /* end transfer */

/*
 *	F I T O B J  --  Work out where we can put this object so as not to
 *						overlap one in the supplied list
 *						Prefered position in x/y
 */
fitobj(obj,x,y,olist,count)
struct object	*obj, **olist;
int		x, y, count;
{
	struct intervaldomain *idom;
	struct valuetable *vdom, *copyvaluetb();
	int		width,height,xmove,ymove;
	
	idom = obj->idom;
	width = idom->lastkl - idom->kol1;
	height = idom->lastln - idom->line1;
	xmove = x - idom->kol1 - width/2;
	ymove = y - idom->line1 - height/2;
	vdom = copyvaluetb(obj,1,obj->vdom->bckgrnd);
	freevaluetable(obj->vdom);
	obj->vdom = vdom;
	moveidom(idom,xmove,ymove);
	movevdom(obj->vdom,xmove,ymove);
}

/*
 *	O V E R L A P I N G  --  Check whether ix overlaps jx
 *
 */
overlaping(ix1,ix2,jx1,jx2)
int		ix1,ix2,jx1,jx2;
{
	return(((jx1 <= ix1) && (ABS(jx1-ix1) < (jx2-jx1))) || 
			((jx1 > ix1) && (ABS(jx1-ix1) < (ix2-ix1))));
}

/*
 *	T R A C K F U S E  --	Cursor tracking + object identifying for fuse
 *
 */
static int trackfuse()
{
	int		curskol, curslin;
	if (alc->m2s->moving)
		tmousecurs();
	else {
		curskol = cursor.k * (kcont->f->ix * kcont->f->scale);
		curslin = cursor.l * (kcont->f->iy * kcont->f->scale);
		quad = 0;
		if (curskol > xsplit) quad++;
		if (curslin < ysplit) quad += 2;
		switch (quad) {
			case 0:
/*
 *	This case was commented out
 */
				objnum = locateobj(kcont->ool,alc->lkount,&fuseframe[FUSEMAX],curskol,curslin);
				if (objnum > -1) {
					unbox();
					boxobj(kcont->ool[objnum],&fuseframe[FUSEMAX],OVERLAY2);
				}
/*				objnum = -1;*/
				break;
			case 1:
			case 2:
			case 3:
				objnum = locateobj(fuseobj[quad-1],fusecount[quad-1],&fuseframe[quad-1],curskol,curslin);
				if (objnum > -1) {
					unbox();
					boxobj(fuseobj[quad-1][objnum],&fuseframe[quad-1],OVERLAY2);
				}
				break;
		}
	}
}

/*
 *	L O C A T E O B J  --
 *
 *	
 * find if x, y (in ddgs coordinates) uniquely identifies an object.
 * If so, return object serial number, otherwise -1.
 * Assumption: objects DO NOT OVERLAP.
 *
 * Technique: first see if point lies within right rectangle
 * surrounding any object, and slightly bigger.  Second, for each such
 * object, see which rectangle is smallest.  Third, if ambiguous,
 * see which is nearest centre.
 */
locateobj(olist,count,frame,x,y)
struct object **olist;
int	count;
struct pframe *frame;
int		x,y;
{
	register int i,k,l,nn;
	int dk,dl,d,k1,kl,l1,ll,hit,area,minarea,mindist;
	register struct intervaldomain *idom;
	
	minarea = 10000000;
	mindist = 1000;
/*
 *	Move position of interest into object frame
 */
	x = ((x - frame->dx) / (frame->scale * frame->ix)) + frame->ox;
	y = ((y - frame->dy) / (frame->scale * frame->iy)) + frame->oy;
	hit = -1;
	for (i=0; i<count; i++) {
		if (olist[i] && olist[i]->idom) {
			idom = olist[i]->idom;
			k1 = idom->kol1 - LOCATEADD;
			if (x >= k1) {
				kl = idom->lastkl + LOCATEADD;
				if (x <= kl) {
					l1 = idom->line1 - LOCATEADD;
					if (y >= l1) {
						ll = idom->lastln + LOCATEADD;
						if (y <= ll) {
							area = (kl-k1) * (ll-l1);
							if (area <= minarea) {
								dk = 2*k-k1-kl;
								dl = 2*l-l1-ll;
								d = ABS(dk)+ABS(dl);
								if (area < minarea || d<mindist) {
									hit = i;
									minarea = area;
									mindist = d;
								}
							}
						}
					}
				}
			}
		}
	}
	return(hit);
}


/*
 *	I N I T F U S E   --
 *
 */
initfuse()
{
	int		i, xmax, ymax, xmin, ymin;

	selectobj = -1;
	setframe(kcont->f,8);		/* Cursor frame */
	xmax = alc->ddgschannel->lineln << alc->ddgschannel->xshift;
	ymax = alc->ddgschannel->lineno << alc->ddgschannel->yshift;
	xsplit = xmax/2;
	ysplit = ymax/2;
	cursor.l = xsplit;
	cursor.k = ysplit;
	fast();
	clear();
	slow();
	lwidth(3);
	colour(255 & ~(OVERLAY1|OVERLAY2)); /* dcb */
	moveto(0,ysplit); lineto(xmax,ysplit);
	moveto(xsplit,0); lineto(xsplit,ymax);
	lin_colour_lut();					/* Set linear LUT + overlay colours */
	lwidth(1);
	colour(GREYCOLS);
 /*
  *	arraydigitise sets main frame
  */
 	if (kcont->FIP != DIGFIP) setframe(&fuseframe[FUSEMAX], 3);
	cellsize(kcont->ool,alc->lkount,&xmin,&xmax,&ymin,&ymax);
	fuseframe[FUSEMAX].ox = ORIGIN_OFFSET; /* xmin - xsplit/24;	/* scale (3) * (1/8 xsplit) */
	fuseframe[FUSEMAX].oy = ORIGIN_OFFSET; /* ymin - ysplit/24; */
	fuseframe[FUSEMAX].dy = ysplit;
	displayobjlist(kcont->ool,alc->lkount,&fuseframe[FUSEMAX]);
	for (i=0; i<FUSEMAX; i++) {
		if (kcont->FIP != DIGFIP) setframe(&fuseframe[i],4); /* arraydigitise sets them */
		switch (i) {
			case 0:
				fuseframe[i].dy = ysplit;
			case 2:
				fuseframe[i].dx = xsplit;
			case 1:
				break;
		}
	}
	for (i=0; i<fuseframes; i++) {
		if (kcont->FIP != DIGFIP) {
			cellsize(fuseobj[i],fusecount[i],&xmin,&xmax,&ymin,&ymax);
			fuseframe[i].ox = ORIGIN_OFFSET; /* xmin; */
			fuseframe[i].oy = ORIGIN_OFFSET; /* ymin; */
		}
		if (alc->debug) printf("(ox, oy) = (%d, %d)\n",
			fuseframe[i].ox, fuseframe[i].oy);
		displayobjlist(fuseobj[i],fusecount[i],&fuseframe[i]);
	}
}

/*
 *	D I S P L A Y O B J L I S T  --
 *
 */
static int displayobjlist(olist,count,fr)
struct object	**olist;
int		count;
struct pframe *fr;
{
	while (count--) {
		if (*olist != NULL) {
			picframe(*olist,fr);
		}
		olist++;
	}
}

/*
 *	F R E E F U S E  --  Free fuse field images
 *
 */
freefuse()
{
	int		field,obj;
	for (field=0; field<fuseframes; field++) {
		for (obj=0; obj<MAXOBJS; obj++) {
			if (fuseobj[field][obj] != NULL) {
				freeobj(fuseobj[field][obj]);
				fuseobj[field][obj] = NULL;
			}
		}
	}
	fuseframes = 0;
	if (selectboundary) {
		freeobj(selectboundary);
		selectboundary = NULL;
	}
}


