/*
 *	F I L E _ S C O R E -- File/retrieve scoring information
 *
 *  Written: D C Burgess
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1989)
 *
 *  Date:    6th June 1989
 *
 *	Modifications
 *  16 Jan 1991 ih		Moved here from abe and moded to cope with comb. met/score lists
 *	05 Jan 1990	dcb		Don't free data field in Freebanner (no need)
 *	02 Jan 1990	dcb		Global score renamed scores
 */
#include <stdio.h>
#include <allcontr.h>
#include <slides.h>
#include <aberrations.h>
#include <ctype.h>
#include <fileheaders.h>
#include <preps.h>
#include <sgstat.h>
#include <errno.h>

#define SCOREVERSION	1
#define SCOREBLKFIX		(2 + sizeof(int))
#define TITLEFIX		(5 + sizeof(short))
#define HEADFIX			(5 + sizeof(short))
#define SFFIX			3
#define LEVELFIX		(5 + (2 * sizeof(short)))
#define ABFIX			(2 + sizeof(short))
#define INFOFIX			4
#define ACTFIX			3
#define ENDFIX			2
#define ENTRYBLKFIX		(2 + sizeof(int))
#define CELLHDRFIX		(2 + (5 * sizeof(short)))
#define ENTRYTEXTFIX	(2 + (2 * sizeof(short)))
#define ENTRYNOTEXTFIX	(2 + (2 * sizeof(short)))
#define ENDFILEFIX		2

#define SCOREBLKCODE	1
#define GLOBALCODE		2
#define ABMENUCODE		3
#define ACMENUCODE		4
#define TITLECODE		5
#define HEADERCODE		6
#define ASSOCCODE		7
#define LEV1CODE		8
#define LEV2CODE		9
#define ABLISTCODE		10
#define INFOCODE		11
#define ACTIONCODE		12
#define ENDSCORECODE	13
#define ENTRYBLKCODE	14
#define CELLHDRCODE		15
#define ENTRYTEXT		16
#define ENTRYNOTEXT		17
#define ENDFILECODE		18

#define BYTE char
#define TRUE (1)
#define FALSE (0)
#define ERRORFILE "/dd/sys/Errmsg"
#define FORW 1 /* Also defined in metreview.c */

/*
 *	External references
 */
extern int xs, ys;
extern struct allkontr *alc;
extern FILE *debug_channel;
extern struct sgbuf savestdin;
extern struct entry *elist, *estart, *eprev;
extern int tot_score;
extern struct entry *Freeentrylist(), *makeentry();
extern struct score_sheet *orig_score, *scores; /* Original/current scoresheet */
extern struct score_sheet *bayscoresheets[MAXBAYS];	/* One per bay */

/*
 *	Globals
 */
char *findkey(), getsingle(), getkey(), *strsave(), *strnsave(), *docomment();
char *itoa(), *Freeptr(), *getrestext(), *Calloc();
struct banner *Freebanner(), *assign_ban();
struct association *Freeassociation(), *assign_assoc();
struct level *Freelevel(), *assign_lev();
struct aberration *Freeaberration(), *assign_aberr();
struct information *Freeinformation(), *assign_info();
struct operation *Freeoperation(), *assign_action();
struct entry *assign_etxt(), *assign_enotxt();

/*
 *	Statics
 */
static int getint();
static short getshort();

/*
 *	Structures
 */
struct scoreheader {
	SMALL	size;			/* Size of structure (incl size) */
	int		version;		/* Header version number */
	int		file_type;		/* Type of file this header is attached to */
/*
 * General header data
 */
	char	date[DATELENGTH];		/* Date of last change */
	char	lab_name[LABLENGTH];
	char	opid[OPERLENGTH];		/* last operator to change data */
	char	patid[CASELENGTH];		/* Patient/case id */
	char	patnam[PATLENGTH];		/* Patient name */
	char	slideid[SLIDELENGTH];	/* slide */
/*
 *	Metaphase list dependant information
 */
	short	tot_score;				/* Total number scored */
	short 	scantop, scanbot,
			scanleft, scanright;	/* Box size */
	int		slwidth;				/* slide width; if measured */
	int		xoff, yoff, zoff;		/* Centring offsets */
	short	nrecs;					/* Number of records */
	int		tissue;					/* tissue culture */
	int		stain;					/* stain technique */
};

/*
 *	F I L E _ S C O R E -- File scores
 */
file_score()
{
	FILE *scorefile, *get_scorefile();

	if ((scorefile = get_scorefile("w")) == NULL) {
		errorprint("Can't write to scorefile\n");
	} else
		write_scorefile(scores, scorefile);
}

/*
 *	R E T R I E V E _ S C O R E -- Retrieve scores
 */
retrieve_score()
{
	FILE *scorefile, *get_scorefile();
	struct mult_slide *bp;
	int bay;

	bp = alc->curbay;
	if(bp->startptr != NULL) {			/* If a list exists remove it. */
		printf("Read in new scores; abandon previous ? \n");
		if(yesno(1))  					/* Confirm the removal of old list. */
			return(-1);
		freelist();						/* Free old list */
	}
/*
 *	Free scoresheet if not original, and entry lists.
 *	If we are already using the original scoresheet, then create a root
 *	new scoresheet. If we go back to using the original scoresheet,
 *	then this will be because a new metaphase list has been read
 *	or created, in which case the new scoresheet will be completely freed
 *	NOTE: The root scoresheet structure is never freed
 */
	bay = baynn(bp) - 1; /* Find our bay */
	if (bayscoresheets[bay] == orig_score) {
		scores = (struct score_sheet *) Calloc(sizeof(struct score_sheet), 1);
		scores->version = VERSION;
		bayscoresheets[bay] = scores;
	} else {
		Freescoresheet(scores);
	}
	elist = Freeentrylist(elist);
	if ((scorefile = get_scorefile("r")) == NULL) {
		errorprint("Scorefile not read\n");
		return(-1);
	} else
		if (read_scorefile(scores, scorefile) < 0) {
			errorprint("Error on reading scorefile\n");
			return(-1);
		}
	return(0);
}

/*
 *	W R I T E _ S C O R E F I L E -- Write header information, scoresheet info
 *									 and scores.
 */
write_scorefile(score, scorefile)
struct score_sheet *score;
FILE *scorefile;
{
	struct scoreheader *header, *makeheader();
	char *ss, *sd, *blk, scblkcode[SCOREBLKFIX], enblkcode[ENTRYBLKFIX];
	int ss_size, sd_size;

	if (alc->debug) {
		fprintf(debug_channel,
			"write_scorefile: scoresheet, and entry list before filing\n");
		show_score(score);
		printentries(elist);
		fprintf(debug_channel, "write_scorefile: done debug, about to file\n");
	}
	windmill("Writing scorefile ");
	header = makeheader();
	write(fileno(scorefile), header, sizeof(struct scoreheader));
	Free(header);
	windmill(NULL);

	if ((ss_size = makescoresheet(score, &ss)) < 0) return(-1);
	blk = scblkcode;
	*blk++ = SCOREBLKFIX;
	*blk++ = SCOREBLKCODE;
	putint(&blk, ss_size + SCOREBLKFIX);
	write(fileno(scorefile), scblkcode, SCOREBLKFIX);	/* Total score size */
	write(fileno(scorefile), ss, ss_size);
	Free(ss);
	windmill(NULL);

	if ((sd_size = makescoredata(&sd)) < 0) return(-1);
	blk = enblkcode;
	*blk++ = ENTRYBLKFIX;
	*blk++ = ENTRYBLKCODE;
	putint(&blk, sd_size + ENTRYBLKFIX);
	write(fileno(scorefile), enblkcode, ENTRYBLKFIX); /* Total entry size */
	write(fileno(scorefile), sd, sd_size);
	Free(sd);
	windmill(NULL);
	
	writeeof(scorefile);
	windmillend("done\n");
}

/*
 *	M A K E H E A D E R -- Construct scoring file specific header information
 */
struct scoreheader *
makeheader()
{
	struct scoreheader *hd;
	struct mult_slide *cb;

	cb = alc->curbay;

	hd = (struct scoreheader *)Calloc(1, sizeof(struct scoreheader));
	hd->size = sizeof(struct scoreheader);
	hd->version = SCOREVERSION;
	hd->file_type = 110;	/* ??? */
	strcpy(hd->date, alc->mydate);
	strcpy(hd->lab_name, alc->sitename);
	strcpy(hd->opid, alc->user);
	strcpy(hd->patid, cb->nameids.case);
	strcpy(hd->patnam, cb->nameids.patnam);
	strcpy(hd->slideid, cb->nameids.slide);
	hd->tot_score = tot_score;
	hd->scantop = cb->mbox.btop;
	hd->scanbot = cb->mbox.bbot;
	hd->scanleft = cb->mbox.bleft;
	hd->scanright = cb->mbox.bright;
	hd->slwidth = cb->slwidth;
	hd->xoff = cb->xcpoff;
	hd->yoff = cb->ycpoff;
	hd->zoff = cb->zcpoff;
	hd->nrecs = cb->totin;
	hd->tissue = cb->typeids.ktistype;
	hd->stain= cb->typeids.kfstype;
	return(hd);
} /* end makeheader */

/*
 *	M A K E S C O R E S H E E T
 */
makescoresheet(score, ss)
struct score_sheet *score;
char **ss;
{
	char *op;
	short totalsize=0, globalsize=0, abmenusize=0, acmenusize=0, titlesize=0;
	short headersize=0, assocsize=0, lev1size=0, lev2size=0, ablistsize=0;
	short infosize=0, actionsize=0, endsize=0;
	register short fixsize;
	int i, s1, s2, s3;
	struct banner *ban;
	struct association *sf;
	struct level *lev;
	struct aberration *ab;
	struct information *inf;
	struct operation *ac;
/*
 *	Skip through scoresheet to find total size
 */
	globalsize = 3 + strlen(score->name) + strlen(score->menuname) + 2;
	abmenusize = 8;
	acmenusize = 8;
	fixsize = TITLEFIX + 2; /* 2 text strings */
	for( ban = score->title; ban; ban = ban->next)
		titlesize += fixsize + strlen(ban->assoc) + strlen(ban->txt);
	for( ban = score->header; ban; ban = ban->next)
		headersize += fixsize + strlen(ban->assoc) + strlen(ban->txt);
	fixsize = SFFIX + 3;
	for( sf = score->form; sf; sf = sf->next) 
		assocsize += fixsize + strlen(sf->assoc) + strlen(sf->data)
			+ strlen(sf->menutxt);
	fixsize = LEVELFIX + 2;
	for(lev = score->level1; lev; lev = lev->next)
		lev1size += fixsize + strlen(lev->assoc) + strlen(lev->txt);
	for(lev = score->level2; lev; lev = lev->next)
		lev2size += fixsize + strlen(lev->assoc) + strlen(lev->txt);
	fixsize = ABFIX + 3;
	for(ab = score->aberr; ab; ab = ab->next)
		ablistsize += fixsize + strlen(ab->key)
			+ strlen(ab->shortstr) + strlen(ab->longstr);
	fixsize = INFOFIX + 2;
	for(inf = score->info; inf; inf = inf->next)
		infosize += fixsize + strlen(inf->assoc) + strlen(inf->screentxt);
	fixsize = ACTFIX + 3;
	for(ac = score->action; ac; ac = ac->next)
		actionsize += fixsize + strlen(ac->key) +
			strlen(ac->assoc) + strlen(ac->menutxt);
	endsize = ENDFIX;
	if (alc->debug) {
		fprintf(debug_channel, "Scoresheet sizes\n\
globalsize=%d, abmenusize=%d, acmenusize=%d, titlesize=%d\n\
headersize=%d, assocsize=%d, lev1size=%d, lev2size=%d\n\
ablistsize=%d, infosize=%d, actionsize=%d, endsize=%d\n",
		globalsize, abmenusize, acmenusize, titlesize, headersize,
		assocsize, lev1size, lev2size, ablistsize, infosize,
		actionsize, endsize);
	}
	totalsize = globalsize + abmenusize + acmenusize + titlesize + headersize
				+ assocsize + lev1size + lev2size + ablistsize + infosize
				+ actionsize + endsize;

	if (alc->debug) fprintf(debug_channel, "Total size = %d\n", totalsize);
	*ss = (char *)Calloc(1, totalsize);	/* Now grab the space */

/*
 *	Write relevant information to this memory area
 */
	op = *ss;

	*op++ = globalsize;
	*op++ = GLOBALCODE;
	*op++ = score->boxed;
	s1 = strlen(score->name) + 1;
	s2 = strlen(score->menuname) + 1;
	strncpy(op, score->name, s1); op += s1;
	strncpy(op, score->menuname, s2); op += s2;
	*op++ = abmenusize;
	*op++ = ABMENUCODE;
	*op++ = score->abmenu->row;
	*op++ = score->abmenu->nrows;
	for (i=0; i<NCOLS; i++)	*op++ = score->abmenu->col[i];
	*op++ = acmenusize;
	*op++ = ACMENUCODE;
	*op++ = score->acmenu->row;
	*op++ = score->acmenu->nrows;
	for (i=0; i<NCOLS; i++)	*op++ = score->acmenu->col[i];

	for( ban = score->title; ban; ban = ban->next) {
		s1 = strlen(ban->assoc) + 1;
		s2 = strlen(ban->txt) + 1;
		*op++ = TITLEFIX + s1 + s2;
		*op++ = TITLECODE;
		putshort(&op, (short)ban->width);
		*op++ = ban->newline;
		*op++ = ban->vert;
		*op++ = ban->horiz;
		strncpy(op, ban->assoc, s1); op += s1;
		strncpy(op, ban->txt, s2); op += s2;
	}
	for( ban = score->header; ban; ban = ban->next) {
		s1 = strlen(ban->assoc) + 1;
		s2 = strlen(ban->txt) + 1;
		*op++ = HEADFIX + s1 + s2;
		*op++ = HEADERCODE;
		putshort(&op, (short)ban->width);
		*op++ = ban->newline;
		*op++ = ban->vert;
		*op++ = ban->horiz;
		strncpy(op, ban->assoc, s1); op += s1;
		strncpy(op, ban->txt, s2); op += s2;
	}
	for(sf = score->form; sf; sf = sf->next) {
		s1 = strlen(sf->assoc) + 1;
		s2 = strlen(sf->data) + 1;
		s3 = strlen(sf->menutxt) + 1;
		*op++ = SFFIX + s1 + s2 + s3;
		*op++ = ASSOCCODE;
		*op++ = sf->opt;
		strncpy(op, sf->assoc, s1); op += s1;
		strncpy(op, sf->data, s2); op += s2;
		strncpy(op, sf->menutxt, s3); op += s3;
	}
	for(lev = score->level1; lev; lev = lev->next) {
		s1 = strlen(lev->assoc) + 1;
		s2 = strlen(lev->txt) + 1;
		*op++ = LEVELFIX + s1 + s2;
		*op++ = LEV1CODE;
		putshort(&op, (short)lev->col);
		*op++ = lev->vert;
		*op++ = lev->horiz;
		*op++ = lev->total;
		putshort(&op, (short)lev->width);
		strncpy(op, lev->assoc, s1); op += s1;
		strncpy(op, lev->txt, s2); op += s2;
	}
	for(lev = score->level2; lev; lev = lev->next) {
		s1 = strlen(lev->assoc) + 1;
		s2 = strlen(lev->txt) + 1;
		*op++ = LEVELFIX + s1 + s2;
		*op++ = LEV2CODE;
		putshort(&op, (short)lev->col);
		*op++ = lev->vert;
		*op++ = lev->horiz;
		*op++ = lev->total;
		putshort(&op, (short)lev->width);
		strncpy(op, lev->assoc, s1); op += s1;
		strncpy(op, lev->txt, s2); op += s2;
	}
	for(ab = score->aberr; ab; ab = ab->next) {
		s1 = strlen(ab->key) + 1;
		s2 = strlen(ab->shortstr) + 1;
		s3 = strlen(ab->longstr) + 1;
		*op++ = ABFIX + s1 + s2 + s3;
		*op++ = ABLISTCODE;
		putshort(&op, (short)ab->col);
		strncpy(op, ab->key, s1); op += s1;
		strncpy(op, ab->shortstr, s2); op += s2;
		strncpy(op, ab->longstr, s3); op += s3;
	}
	for(inf = score->info; inf; inf = inf->next) {
		s1 = strlen(inf->assoc) + 1;
		s2 = strlen(inf->screentxt) + 1;
		*op++ = INFOFIX + s1 + s2;
		*op++ = INFOCODE;
		*op++ = inf->x;
		*op++ = inf->y;
		strncpy(op, inf->assoc, s1); op += s1;
		strncpy(op, inf->screentxt, s2); op += s2;
	}
	for(ac = score->action; ac; ac = ac->next) {
		s1 = strlen(ac->key) + 1;
		s2 = strlen(ac->assoc) + 1;
		s3 = strlen(ac->menutxt) + 1;
		*op++ = ACTFIX + s1 + s2 + s3;
		*op++ = ACTIONCODE;
		*op++ = 0; /*ac->confirm;*/
		strncpy(op, ac->key, s1); op += s1;
		strncpy(op, ac->assoc, s2); op += s2;
		strncpy(op, ac->menutxt, s3); op += s3;
	}
	*op++ = ENDFIX;
	*op++ = ENDSCORECODE;
	return(totalsize);
} /* end makescoresheet */

/*
 *	M A K E S C O R E D A T A -- Make a memory block of score data
 */
makescoredata(sd)
char **sd;
{
	struct entry *e1, *e2, *elast;
	int totalsize = 0, nentries, s1, nrecs;
	char *op, *nentryptr;
	struct mult_slide *bp;
	struct mlclust *ml;
	struct metlink *keeppos;	/* Keep original scored posn */
/*
 *	Skip through to find the size
 */
	nrecs = alc->curbay->totin;
	totalsize = CELLHDRFIX * nrecs;
	if (alc->debug)
		fprintf(debug_channel, "Nrecs=%d\nHdr fixed size=%d\n", nrecs, totalsize);
	for (e1 = elist; e1; e1 = e1->foll) {
		for (e2 = e1; e2; e2 = e2->next) {
			switch (getspecial(e2->col)) {
				case COMMENT:				/* Save comment text */
					if (e2->num)
						totalsize += ENTRYTEXTFIX + strlen(e2->txt) + 1;
					break;
				case CELLNO:
				case XPOS:
				case YPOS:
				case POSN:
					break; 				/* Don't save these at all */
				default:				/* Save no text */
					totalsize += ENTRYNOTEXTFIX;
					break;
			} /* end switch */
			elast = e2;
		} /* end for e2 ... */
		e1 = elast;
		if (alc->debug) fprintf(debug_channel, "Score size = %d\n", totalsize);
		if (alc->debug)
			fprintf(debug_channel, "sizing - e1=0x%x, foll=0x%x\n", e1, e1->foll);
	} /* end for e1... */

	*sd = (char *)Calloc(1, totalsize);
	op = *sd;
/*
 *	Now make the data block
 */
	bp = alc->curbay; /* !!! */
	if (bp->startptr == NULL) {
		fprintf(stderr, "Fatal: No metaphase list in\n"); /* This should never occur */
		Free(sd);
		return(-1);
	}
	keeppos = bp->currptr;
	if (bp->currptr != bp->csubptr) {
		printf("\007Bay currptr NOT EQUAL bay subptr !\n");
		if (alc->debug)
			fprintf(debug_channel, "Bay currptr NOT EQUAL bay subptr !\n");
	}
	bp->currptr = bp->csubptr = bp->startptr; /* Init subset ptr too !! */
	for (e1 = elist; e1; e1 = e1->foll) {
/*		*op++ = CELLHDRFIX;		 Fixed size
		*op++ = CELLHDRCODE;
		ml = bp->currptr->mlrec;
		if (alc->debug)
			fprintf(debug_channel, "recno %d, x %d, y %d, z %d\n",
				ml->recnr, ml->xval, ml->yval, ml->zval);
		putshort(&op, ml->recnr);	/* Cell number 
		putshort(&op, ml->xval);	/* X position 
		putshort(&op, ml->yval);	/* Y position 
		putshort(&op, ml->zval);	/* Z position */
		nrecs--;					/* Stored one more record */
		nentries = 0;
		nentryptr = op;				/* Save to fill in later */
		op += sizeof(short);		/* Skip its place */
		if (alc->debug) fprintf(debug_channel, "Aberrations:");
		for (e2 = e1; e2; e2 = e2->next) {
			switch (getspecial(e2->col)) {
				case COMMENT:				/* Save comment text */
					if (e2->num) {
						if (alc->debug)
							fprintf(debug_channel,
								"Saving comment %s\n", e2->txt);
						s1 = strlen(e2->txt) + 1;
						*op++ = ENTRYTEXTFIX + s1;
						*op++ = ENTRYTEXT;
						putshort(&op, (short)e2->col);
						putshort(&op, (short)e2->num);
						strncpy(op, e2->txt, s1); op += s1;
						nentries++;
					}
					break;
				case CELLNO:
				case XPOS:
				case YPOS:
				case POSN:
					break; 				/* Don't save these at all */
				default:				/* Save no text */
 					if (alc->debug)	fprintf(debug_channel, " (%d @ col %d)",
 						e2->num, e2->col);
					*op++ = ENTRYNOTEXTFIX;
					*op++ = ENTRYNOTEXT;
					putshort(&op, (short)e2->col);
					putshort(&op, (short)e2->num);
					nentries++;
					break;
			} /* end switch */
			elast = e2;
		} /* end for e2 ... */
		e1 = elast;
		if (alc->debug) fprintf(debug_channel, "\nnentries = %d\n", nentries);
		putshort(&nentryptr, (short)nentries);	/* Now fill in number of entries */
		nextrec(FORW);					/* Advance to next record */
	} /* end for e1... */
/*
 *	Now store those records which have not been scored
 */
 	while(nrecs) {
		*op++ = CELLHDRFIX;		/* Fixed size */
		*op++ = CELLHDRCODE;
		ml = bp->currptr->mlrec;
		if (alc->debug)
			fprintf(debug_channel, "Not scored, recno %d, x %d, y %d, z %d\n",
				ml->recnr, ml->xval, ml->yval, ml->zval);
		putshort(&op, ml->recnr);	/* Cell number */
		putshort(&op, ml->xval);	/* X position */
		putshort(&op, ml->yval);	/* Y position */
		putshort(&op, ml->zval);	/* Z position */
		nrecs--;					/* Stored one more record */
		putshort(&op, 0);			/* Not scored, then no entries */
		nextrec(FORW);				/* Advance to next record (no checking) */
	} /* end while nrecs ... */
/*
 *	Restore current scoring position
 */
	bp->currptr = bp->csubptr = keeppos;
	bp->nameids.cellno = bp->startptr->mlrec->recnr; /* Re-init cell no. */
	return(totalsize);
} /* end makescoredata */

/*
 *	R E A D _ S C O R E F I L E -- Read in the score file info.
 *	We assume that the main scoresheet structure has already been allocated
 *	N.B. This function has been modified from the abe version to expect
 *	scorefile not to be at the start of the file, hence the modified seek(s)
 *        --- ih
 */
read_scorefile(score, scorefile)
struct score_sheet *score;
FILE *scorefile;
{
	struct scoreheader *hd;
	struct mult_slide *cb;
	char *rdata, *rfree;
	SMALL size;
	int rtn=0, end, stat, version, cell_entry, *rtb, *rtp, j, nentries=0, mill=0;
	int cellrankno;	/* Cell rank number - Assume they are read in rank order */
	BYTE len;
	struct banner *ban;
	struct level *lev;
	struct association *sf;
	struct aberration *ab;
	struct information *inf;
	struct operation *ac;
/*	struct mlclust *mptbuf, *grabmlbuf();
	struct metlink *linkptr, *rec_in, *grabnlnk();*/
	struct entry *e, **eptr; /* Gen. entry ptr, & current entry->next address */

/*	cb = alc->curbay;*/
/*
 *	Stage 1 - Read the header
 */
	read(fileno(scorefile), &size, sizeof(SMALL));
	read(fileno(scorefile), &version, sizeof(int));
	if (alc->debug)
		fprintf(debug_channel, "Hdr size=%d, version %d\n", size, version);
	switch (version) {
		case 1:
			hd = (struct scoreheader *)Calloc(1, sizeof(struct scoreheader));
			/* do a relative rewind to get to the start of the scoring info -ih*/
			lseek(scorefile,-(sizeof(SMALL)+sizeof(int)),SEEK_CUR);
			stat = read(fileno(scorefile), hd, size);
			if (stat != size) {
				Free(hd);
				return(readerr(stat));
			}
			assign_v1(hd, score);
			break;
		default:
			fprintf(stderr, "Illegal file version\n");
			Free(hd);
			return(-1);
			break;			/* There are no other versions */
	}
	Free(hd);
/*
 *	Stage 2 -- Read scoresheet and scoring info
 */
/*	mptbuf = grabmlbuf(cb->totin);					 grab records buffer */
/*	rec_in = cb->startptr = grabnlnk(cb->totin);	 Link space */
	end = FALSE;
	cell_entry = TRUE;								/* Fake entry */
	cellrankno = 1;
	windmill("Reading scoring information ");
	while(!end) {
		stat = read(fileno(scorefile), &len, 1);
		if (stat != 1) {
			rtn = readerr(stat);
			break;
		}
		if (len <= 1) {
			fprintf(stderr, "Read abandoned due to file format error\n");
			rtn = -1;
			break;
		}
		rfree = rdata = Calloc(1, len-1);
		stat = read(fileno(scorefile), rdata, len-1);
		if (stat != len-1) {
			rtn = readerr(stat);
			break;
		}
		if (alc->debug)
			fprintf(debug_channel, "Item code %d, size %d\n", *rdata, len);
		if (mill++ % 4 == 0) windmill(NULL);
		switch(*rdata++) {
		case SCOREBLKCODE:
			if (alc->debug)
				fprintf(debug_channel, "Scoresheet block size = 0x%x\n", getint(&rdata));
			break;
		case GLOBALCODE:
			score->boxed = (char)*rdata++;
			score->name = strsave(rdata); rdata += strlen(score->name) + 1;
			score->menuname = strsave(rdata);	
			break;
		case ABMENUCODE:
			assign_sm(&(score->abmenu), rdata);
			break;
		case ACMENUCODE:
			assign_sm(&(score->acmenu), rdata);
			break;
		case TITLECODE:
			if (score->title == NULL)
				score->title = assign_ban(rdata);
			else {
				for (ban = score->title; ban->next; ban = ban->next);
				ban->next = assign_ban(rdata);
			}
			break;
		case HEADERCODE:
			if (score->header == NULL)
				score->header = assign_ban(rdata);
			else {
				for (ban = score->header; ban->next; ban = ban->next);
				ban->next = assign_ban(rdata);
			}
			break;
		case ASSOCCODE:
			if (score->form == NULL)
				score->form = assign_assoc(rdata);
			else {
				for (sf = score->form; sf->next; sf = sf->next);
				sf->next = assign_assoc(rdata);
			}
			break;
		case LEV1CODE:
			if (score->level1 == NULL)
				score->level1 = assign_lev(rdata);
			else {
				for (lev = score->level1; lev->next; lev = lev->next);
				lev->next = assign_lev(rdata);
			}
			break;
		case LEV2CODE:
			if (score->level2 == NULL)
				score->level2 = assign_lev(rdata);
			else {
				for (lev = score->level2; lev->next; lev = lev->next);
				lev->next = assign_lev(rdata);
			}
			break;
		case ABLISTCODE:
			if (score->aberr == NULL)
				score->aberr = assign_aberr(rdata);
			else {
				for (ab = score->aberr; ab->next; ab = ab->next);
				ab->next = assign_aberr(rdata);
			}
			break;
		case INFOCODE:
			if (score->info == NULL)
				score->info = assign_info(rdata);
			else {
				for (inf = score->info; inf->next; inf = inf->next);
				inf->next = assign_info(rdata);
			}
			break;
		case ACTIONCODE:
			if (score->action == NULL)
				score->action = assign_action(rdata);
			else {
				for (ac = score->action; ac->next; ac = ac->next);
				ac->next = assign_action(rdata);
			}
			break;
		case ENDSCORECODE:
			initspecials(score);	/* Init special arrays */
			break;
		case ENTRYBLKCODE:
			if (alc->debug)
				fprintf(debug_channel, "Score entries block size = 0x%x\n", getint(&rdata));
			break;
/*		case CELLHDRCODE:
			if (nentries != 0) fprintf(stderr,
					"File format error, entries specified but not found\n");
			if ((elist) && cell_entry)
				addspecials(score);			/* Add back the special info 
			rec_in->mlrec = mptbuf++;		/* Point at mlrec space 
			assign_mlrec(rdata, rec_in->mlrec, &nentries);
			rec_in->mlrec->qrank = cellrankno++;
			cb->currptr = rec_in;			/* The current info 
			if (!cell_entry) {				/* No entries made
				e = makeentry(0, 0, NULL);	/* So make a dummy entry 
				if (!elist) {				/* No list yet started 
					elist = estart = e;		/* Set up list 
					eptr = &(e->foll);		/* Next links to foll 
				} else {					/* List already started 
					*eptr = e;				/* Link to foll of prev entry 
					e->prev = eprev;		/* Link to prev line 
					eprev = e;				/* Advance prev ptr to this line 
					eptr = &(e->foll);		/* So next links to foll 
					estart = NULL;			/* No starting entry yet 
				}
			} else {						/* Entries have been scored 
				if (elist) {
					for (e = estart; e->next; e = e->next) ; /* Find last entry 
					eptr = &(e->foll);		/* Link to foll 
					e->prev = eprev;		/* Link to prev line 
					eprev = estart;			/* Adv ptr to this line 
					estart = NULL;			/* Ground estart 
				}
			}
			rec_in++;						/* set-up for next record 
			cell_entry = FALSE;				/* No entries so far 
			break;*/
		case ENTRYTEXT:
			if (checkentries(nentries) < 0) break;
			if (elist == NULL) {
				elist = estart = assign_etxt(rdata);
				eptr = &(elist->next);
				eprev = NULL;
			} else {
				*eptr = assign_etxt(rdata);
				if (estart == NULL) estart = *eptr;
				eptr = &((*eptr)->next);
			}
			nentries--;			/* One less entry */
			cell_entry = TRUE;	/* an entry has been made */
			break;
		case ENTRYNOTEXT:
			if (checkentries(nentries) < 0) break;
			if (elist == NULL) {
				elist = estart = assign_enotxt(rdata, score);
				eptr = &(elist->next);
				eprev = NULL;
			} else {
				*eptr = assign_enotxt(rdata, score);
				if (estart == NULL) estart = *eptr;
				eptr = &((*eptr)->next);
			}
			nentries--;			/* One less entry */
			cell_entry = TRUE;	/* an entry has been made */
			break;
		case ENDFILECODE:
			end = TRUE;
			break;
		} /* end switch */
		Free(rfree);
	} /* end while !end... */
	windmillend("done\n");
/*
 *	Now link ranks - Here we assume they have been read in in rank order
 *	i.e. The qrank value, not being stored cannot be read in. There is no
 *	qsort phase, and thus the rtb array just contains numbers
 *	0 to (no. of records - 1).
 *	NOTE: the qrank value is set up on 'read' to be the record number.
 *
	rtp = rtb = (int *) Malloc(cb->totin * sizeof (int));
	for (j = 0 ; j < cb->totin; ++j) *rtp++ = j;
	linkranks(rtb);							/* Link in rank order
	Free (rtb);								/* Free space
 *
 *	Now link records belonging to current subset
 *	Set byg = 0
 *	assume cb->listlow, cb->listhigh runs from 1 ... ->totin
 *	assume cb->minrank, cb->maxrank runs from 1 ... ->totin
 *	These are not currently stored in the scoring file, but are stored in the
 *	metaphase header.
 *
	if (alc->debug)
		fprintf(debug_channel, "bygroup: %d\n", cb->byg);
	cb->byg = 1;
	cb->ngrps = 10;
	cb->listlow  = 1;
	cb->listhigh = cb->totin;
	cb->minrank  = 1;
	cb->maxrank  = cb->totin;
	linksub();
 *
 *	Init entry list ptrs correctly
 */
	estart = elist;
	eprev = NULL;
	cb->nameids.cellno = cb->startptr->mlrec->recnr; /* Init cell no. */
/*
 *	Set current & subset ptrs
 */
	cb->currptr = cb->csubptr = cb->fsubptr = cb->startptr;
	if (alc->debug) {
		fprintf(debug_channel,
			"read_scorefile: done read, now show scoresheet and entry info\n");
		show_score(score);
		printentries(elist);
	}
	return(rtn);
} /* end read_scorefile */

/*
 * 	A S S I G N _ V 1 -- Assign version 1 header info
 *	Assign score file version to scoresheet version
 */
assign_v1(hd, score)
struct scoreheader *hd;
struct score_sheet *score;
{
	struct mult_slide *cb;

	cb = alc->curbay;

	score->version = hd->version;
	strcpy(alc->mydate, hd->date);
	strcpy(alc->sitename, hd->lab_name);
	strcpy(alc->user, hd->opid);
	strcpy(cb->nameids.case, hd->patid);
	strcpy(cb->nameids.patnam, hd->patnam);
	strcpy(cb->nameids.slide, hd->slideid);
	tot_score = hd->tot_score;
	cb->mbox.btop = hd->scantop;
	cb->mbox.bbot = hd->scanbot;
	cb->mbox.bleft = hd->scanleft;
	cb->mbox.bright = hd->scanright;
	cb->slwidth = hd->slwidth;
	cb->xcpoff = hd->xoff;
	cb->ycpoff = hd->yoff;
	cb->zcpoff = hd->zoff;
	cb->totin = hd->nrecs;
	cb->typeids.ktistype = hd->tissue;
	cb->typeids.kfstype = hd->stain;
} /* end assign_v1 */

/*
 *	A S S I G N _ S M -- Assign score menu fields
 */
static
assign_sm(menu, data)
struct scoremenu **menu;
char *data;
{
	struct scoremenu *sm;
	int i;

	if (*menu)
		fprintf(stderr, "Warning: menu already allocated\n");
	else
		*menu = (struct scoremenu *)Calloc(1, sizeof(struct scoremenu));
	sm = *menu;
	sm->row = (BYTE)*data++;
	sm->nrows = (BYTE)*data++;
	for (i=0; i<NCOLS; i++) sm->col[i] = (BYTE)*data++;
}

/*
 *	A S S I G N _ B A N -- Assign banner information
 */
struct banner *
assign_ban(data)
char *data;
{
	struct banner *ban;

	ban = (struct banner *)Calloc(1, sizeof(struct banner));
	ban->width = (int)getshort(&data);
	ban->newline = (BYTE)*data++;
	ban->vert = (BYTE)*data++;
	ban->horiz = (BYTE)*data++;
	ban->assoc = strsave(data); data += strlen(data) + 1;
	ban->txt = strsave(data);
	return(ban);
} /* end assign_ban */

/*
 *	A S S I G N _ A S S O C -- Assign association info.
 */
struct association *
assign_assoc(data)
char *data;
{
	struct association *sf;

	sf = (struct association *)Calloc(1, sizeof(struct association));
	sf->opt = (BYTE)*data++;
	sf->assoc = strsave(data); data += strlen(data) + 1;
	sf->data = strsave(data); data += strlen(data) + 1;
	sf->menutxt = strsave(data);
	if (alc->debug)
		fprintf(debug_channel, "assign_assoc: data = %s\n", sf->data);
	return(sf);
} /* end assign_assoc */

/*
 *	A S S I G N _ L E V -- Assign level information
 */
struct level *
assign_lev(data)
char *data;
{
	struct level *lev;

	lev = (struct level *)Calloc(1, sizeof(struct level));
	lev->col = (int)getshort(&data);
	lev->vert = (BYTE)*data++;
	lev->horiz = (BYTE)*data++;
	lev->total = (BYTE)*data++;
	lev->width = (int)getshort(&data);
	lev->assoc = strsave(data); data += strlen(data) + 1;
	lev->txt = strsave(data);
	return(lev);
} /* end assign_lev */

/*
 *	A S S I G N _ A B E R R -- Assign aberration info
 */
struct aberration *
assign_aberr(data)
char *data;
{
	struct aberration *ab;

	ab = (struct aberration *)Calloc(1, sizeof(struct aberration));
	ab->col = (int)getshort(&data);
	ab->key = strsave(data); data += strlen(data) + 1;
	ab->shortstr = strsave(data); data += strlen(data) + 1;
	ab->longstr = strsave(data);
	return(ab);
} /* end assign_aberr */

/*
 *	A S S I G N _ I N F O -- Assign information info
 */
struct information *
assign_info(data)
char *data;
{
	struct information *info;

	info = (struct information *)Calloc(1, sizeof(struct information));
	info->x = (BYTE)*data++;
	info->y = (BYTE)*data++;
	info->assoc = strsave(data); data += strlen(data) + 1;
	info->screentxt = strsave(data);
	return(info);
} /* end assign_info */

/*
 *	A S S I G N _ A C T I O N -- Assign actions info
 */
struct operation *
assign_action(data)
char *data;
{
	struct operation *ac;
	char dum;
	
	ac = (struct operation *)Calloc(1, sizeof(struct operation));
	/*ac->confirm*/dum = (BYTE)*data++;
	ac->key = strsave(data); data += strlen(data) + 1;
	ac->assoc = strsave(data); data += strlen(data) + 1;
	ac->menutxt = strsave(data);
	return(ac);
} /* end assign_action */

/*
 *	A S S I G N _ M L R E C -- Assign info to metaphase record
 */
assign_mlrec(data, ml, nent)
char *data;
struct mlclust *ml;
int *nent;
{
	ml->recnr = getshort(&data);
	ml->xval = getshort(&data);
	ml->yval = getshort(&data);
	ml->zval = getshort(&data);
	*nent = getshort(&data);
	alc->curbay->nameids.cellno = ml->recnr; /* Accessible for getrestext */
}

/*
 *	A S S I G N _ E T X T -- Assign entry information, with text
 */
struct entry *
assign_etxt(data)
char *data;
{
	struct entry *e;
	e = (struct entry *)Calloc(1, sizeof(struct entry));
	e->col = (int)getshort(&data);
	e->num = (int)getshort(&data);
	e->txt = strsave(data);
	return(e);
} /* end assign_etxt */

/* 
 * A S S I G N _ E N O T X T -- Assign entry information, no text is used
 */
struct entry *
assign_enotxt(data, score)
char *data;
struct score_sheet *score;
{
	struct entry *e;
	struct aberration *ab;

	e = (struct entry *)Calloc(1, sizeof(struct entry));
	e->col = (int)getshort(&data);
	e->num = (int)getshort(&data);
/*
 *	Scan aberration list for col number, then insert this text
 *	NOTE:
 *  -----
 *	THERE IS NO WAY OF ACCURATELY RESTORING THE SHORT ABERRATION TEXT
 *	IF THERE IS MORE THAN ONE ABERRATION TYPE ALLOCATED TO THE SAME
 *	COLUMN. THIS IS THEN A PROBLEM, AN INITIAL WORK-AROUND IS NOT
 *	TO ALLOW MORE THAN ONE ABERRATION IN THE SAME COLUMN. EVENTUALLY SOME
 *	ADDITIONAL CODE MUST BE USED TO UNAMBIGUOUSLY SPECIFY THE SHORT TEXT
 *	IT IS A WASTE OF SPACE TO STORE THE ASCII TEXT ITSELF. THIS COULD
 *	BE DONE ON SOME SORT OF COL,ROW BASIS. COL,1 COL,2 ... ETC
 */
	for(ab = score->aberr; ab; ab = ab->next) {
 		if (e->col == ab->col) {
 			e->txt = ab->shortstr;
 			if (alc->debug)
 				fprintf(debug_channel, "assign_enotxt: Restore \"%s\" to col %d\n",
 					ab->shortstr, e->col);
		}
	}
 	return(e);
} /* end assign_enotxt */
	
/*
 *	G E T _ S C O R E F I L E --	Call check case id. menu, and
 *									open returned file name
 */
FILE *
get_scorefile(mode)
char *mode;
{
	FILE *gf = NULL;
	int getidmode;

	getidmode = (strcmp(mode, "r") == 0) ? READ : WRITE;
	if (getid(SCOREFILE, getidmode)) { /* Put it in alc->metfile */
		if ((gf = fopen(alc->metfile, mode)) == NULL) {
			errorprint("File error - ");
			fprintf(stderr, "Can't open %s for %s (%d)\n",
				alc->metfile, mode, errno);
		}
	}
	return(gf);
} /* end get_scorefile */

/*
 *	P U T I N T -- Write integer value to buffer
 */
putint(out, val)
char **out;
int val;
{
	char *ival;
	int i;

	ival = (char *)&val;
	for (i=0; i<sizeof(int); i++) {
		**out = *ival++;
		(*out)++;
	}
}

/*
 *	P U T S H O R T -- Write short value to buffer
 */
putshort(out, val)
char **out;
short val;
{
	char *sval;
	int i;

	sval = (char *)&val;
	for (i=0; i<sizeof(short); i++) {
		**out = *sval++;
		(*out)++;
	}
}

/*
 *	G E T I N T --	Read integer value from buffer, buffer ptr is advanced
 *					over the integer
 */
static int
getint(in)
char **in;
{
	int *ival, i;
	char conv[sizeof(int)];

	for (i=0; i<sizeof(int); i++) { conv[i] = **in; (*in)++; }
	ival = (int *)conv;
	if (alc->debug > 1)	
		fprintf(debug_channel, "getint: 0x%X value %d\n", *in, *ival);
	return(*ival);
}

/*
 *	G E T S H O R T --	Read short value from buffer, buffer ptr is advanced
 *						over the short
 */
static short
getshort(in)
char **in;
{
	short *sval, i;
	char conv[sizeof(short)];

	for (i=0; i<sizeof(short); i++) { conv[i] = **in; (*in)++; }
	sval = (short *)conv;
	if (alc->debug > 1)	
		fprintf(debug_channel, "getshort: 0x%X value %d\n", *in, *sval);
	return(*sval);
}

/*
 * W R I T E E O F -- Write the end of file code
 */
writeeof(scorefile)
FILE *scorefile;
{
	char *end, *op;

	if (alc->debug) fprintf(debug_channel, "writeeof:\n");
	end = (char *)Calloc(1, 2);
	op = end;
	*op++ = ENDFILEFIX;
	*op++ = ENDFILECODE;
	write(fileno(scorefile), end, ENDFILEFIX);
	Free(end);

} /* end writeeof */

/*
 *	F R E E S C O R E S H E E T --	Free all scoresheet memory,
 *									except scoresheet structure
 */
Freescoresheet(score)
struct score_sheet *score;
{
	if (alc->debug) fprintf(debug_channel, "Freeing scoresheet\n");
	score->name = Freeptr(score->name);
	score->menuname = Freeptr(score->menuname);
	score->abmenu = (struct scoremenu *)Freeptr(score->abmenu);
	score->acmenu = (struct scoremenu *)Freeptr(score->acmenu);
	score->title = Freebanner(score->title);
	score->header = Freebanner(score->header);
	score->form = Freeassociation(score->form);
	score->level1 = Freelevel(score->level1);
	score->level2 = Freelevel(score->level2);
	score->aberr = Freeaberration(score->aberr);
	score->info = Freeinformation(score->info);
	score->action = Freeoperation(score->action);
	score->version = 0;
	score->boxed = 0;
	if (alc->debug) fprintf(debug_channel, "Scoresheet freed\n");
} /* end Freescoresheet */

/*
 *	F R E E B A N N E R -- Free off banner list
 */
struct banner *
Freebanner(ban)
struct banner *ban;
{
	struct banner *free_ptr;

	while (ban) {
		free_ptr = ban;
		ban = ban->next;
		Freeptr(free_ptr->assoc);
		Freeptr(free_ptr->txt);
/*
 *	No need to free data field, since either static store,
 *	or from score->form, freed by Freeassociation
 */
/*		Freeptr(free_ptr->data);*/
		Freeptr(free_ptr);
	}
	return(NULL);
}

/*
 *	F R E E A S S O C I A T I O N -- Free off association list
 */
struct association *
Freeassociation(ass)
struct association *ass;
{
	struct association *free_ptr;

	while (ass) {
		free_ptr = ass;
		ass = ass->next;
		Freeptr(free_ptr->assoc);
		Freeptr(free_ptr->data);
		Freeptr(free_ptr->menutxt);
		Freeptr(free_ptr);
	}
	return(NULL);
}

/*
 *	F R E E L E V E L -- Free off level linked list.
 */
struct level *
Freelevel(lev)
struct level *lev;
{
	struct level *free_ptr;

	while (lev) {
		free_ptr = lev;
		lev = lev->next;
		Freeptr(free_ptr->assoc);
		Freeptr(free_ptr->txt);
		Freeptr(free_ptr);
	}
	return(NULL);
}

/*
 *	F R E E A B E R R A T I O N -- Free off aberration linked list.
 */
struct aberration *
Freeaberration(ab)
struct aberration *ab;
{
	struct aberration *free_ptr;

	while (ab) {
		free_ptr = ab;
		ab = ab->next;
		Freeptr(free_ptr->key);
		Freeptr(free_ptr->shortstr);
		Freeptr(free_ptr->longstr);
		Freeptr(free_ptr);
	}
	return(NULL);
}

/*
 *	F R E E I N F O R M A T I O N -- Free off information linked list.
 */
struct information *
Freeinformation(inf)
struct information *inf;
{
	struct information *free_ptr;

	while (inf) {
		free_ptr = inf;
		inf = inf->next;
		Freeptr(free_ptr->assoc);
		Freeptr(free_ptr->screentxt);
		Freeptr(free_ptr);
	}
	return(NULL);
}

/*
 *	F R E E O P E R A T I O N -- Free off operation linked list.
 */
struct operation *
Freeoperation(oper)
struct operation *oper;
{
	struct operation *free_ptr;

	while (oper) {
		free_ptr = oper;
		oper = oper->next;
		Freeptr(free_ptr->key);
		Freeptr(free_ptr->assoc);
		Freeptr(free_ptr->menutxt);
		Freeptr(free_ptr);
	}
	return(NULL);
}

/*
 *	R E A D E R R -- Print an appropriate read error message
 */
readerr(stat)
int stat;
{
	switch (stat) {
		case -1:
			fprintf(stderr, "Read error detected -");
			show_error(errno);
		case 0:
			fprintf(stderr, "Premature end-of-file encountered\n");
			if (errno != E_EOF) 
				fprintf(stderr, "due to error - ");
				show_error(errno);
				fflush(stderr);
			break;
		default:
			fprintf(stderr, "Truncated data read from file\n");
		}
	return(-1); /* All are error conditions */
}

/*
 *	S H O W _ E R R O R  --  Print error message associated with error number
 *	With acknowlegements to CAS for template.
 */
show_error(ern)
int		ern;
{
	char	strbuf[8], tmpbuf[80];
	FILE	*erf;
	int		tmperr;

	sprintf(strbuf,"000:%03d",ern);
	tmperr = ern;
	if ((erf = fopen(ERRORFILE,"r")) != NULL) {
		while (fgets(tmpbuf,80,erf) != NULL) {
			if (!strncmp(strbuf,tmpbuf,7)) {
				printf("%s",tmpbuf);
				while ((fgets(tmpbuf,80,erf) != NULL) && (tmpbuf[0] == ' ')) {
					printf("%s",tmpbuf);
				}
				break;
			}
		}
		fclose(erf);
	} else {
		fprintf(stderr, "(%d)\n", tmperr);
		fprintf(stderr, "Can't open system error message file (%d)\n", errno);
	}
	return(1);
}

	
static int
checkentries(n)
int n;
{
	if (n == 0) {
		fprintf(stderr,
	"File format error, score entry found but not specified - Entry ignored\n");
		return(-1);
	}
	return(0);
}

