/*
 *      F I L E I O . C  --  File i/o routines for kmr/mf
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *  Date:    29 Sept 1986
 *
 *	Some of the routines in here would be better in libraries - Consider this
 *	the first step along the line to getting that.
 *
 *  Modifications
 *
 *       5 Feb 1992		Joanne		change patnam for NEF coord.
 *	15 Dec 1990		jimp@MRC	Added FLEXFILE type (bug-fix 19-12-90)
 *	25 Jul 1989		dcb			Add SCOREFILE type
 *	 6 Jul 1989		SEAJ		Made readcell() reset Cpgroup for header versions
 *								less than 3.
 *	 1 Jun 1989		CAS			Def errno for v2.2
 *	22 May 1989		dcb			Trivial - disp_messg display
 *  22 May 1989		SEAJ		Reset timing arrays in do_timing.
 *	19 May 1989		dcb			mouse on case id quit
 *	18 May 1989		dcb			translate filename in checkoverwrite
 *	17 May 1989		SEAJ		Put in timing information
 *	15 Feb 1989		CAS			Fileerror -> not static
 *	14 Feb 1989		CAS			Changes to file opening and error messages
 *	 2 Feb 1989		CAS			More fiddling with file writing
 *								fixed free left message (added %%)
 *	31 Jan 1989		CAS			Changed preexpand code
 *	30 Jan 1989		CAS			Added status check from writeobj
 *								ifdef'd out fswriteobj
 *  28 Nov 1988		SEAJ		Added extra checks for disc useage.
 *	25 Oct 1988		SEAJ		Set Cellcmd to -99 in getid to stop overwriting
 *								patient name
 *	 7 Jul 1988		SEAJ		Set MASTCOPY in active list if entered from 
 *								karyotype review
 *	10 Jun 1988		CAS			Limit max cell number to 9999
 *	25 Mar 1988		CAS			Tidy up cmmd's in getid
 *	24 Mar 1988		CAS			Ensure cell header correctly initialised
 */

#include <cmdsys.h>
#include <wstruct.h>
#include <kcontrol.h>
#include <chromanal.h>
#include <allcontr.h>
#include <slides.h>
#include <multip.h>
#include <flex.h>

#define MAX(i,j) (i>j ? i: j)
#define MIN(i,j) (i<j ? i: j)
#define MAXTIMS 300
/*
 *	External data references
 */
extern int errno;
extern struct kcontrol *kcont;
extern struct allkontr *alc;

extern FILE *debug_channel;
 
/*
 * Local static storage
 */
static FILE *imgin;
static int timings[MAXTIMS], curr_time = -1;
static char* strings[MAXTIMS];
/*
 * External routines
 */
extern int setdir();
extern int show_dir();
extern int iscyto();
extern int fileexists();
extern int setcase();
extern int setname();
extern int setslide();
extern struct chromplist *convcellhead();

/*
 *	O P E N C F I L E  --  Open cell file for reading
 *
 *	Attempt to open alc->dir/alc->picname for input
 *
 *	Return -1 if fail
 */
opencfile()
{
	FILE	*fbdopen();
	int start_time, local;

	if (alc->timing)
		setuptim(&local, &start_time, "opencfile");
	if (alc->debug && imgin)
		fprintf(debug_channel,"Old imgin file not closed\n");
	if (!(imgin = fbdopen(alc->picname,alc->dir,"r",BUFSIZ)))  {
		fileerror("Failed to open %s",errno,alc->picname);
		if (alc->timing) {
			timings[local] = tick() - start_time;
		}
		return(-1);
	}
	if (alc->debug)
		fprintf(debug_channel,"Tryfile opened - fp 0x%x path %d buffer 0x%x\n",
			imgin,imgin->_fd,imgin->_base);
	if (alc->timing)
		timings[local] = tick() - start_time;
	return(0);
}


/*
 *	C L O S E C F I L E
 *
 */
closecfile()
{
	int start_time, local;

	if (alc->timing)
		setuptim(&local, &start_time, "closecfile");
	if (imgin)
		fbclose(imgin);
	else if (alc->debug)
		fprintf(debug_channel,"Re-close of imgin file\n");
	imgin = NULL;
	if (alc->timing) {
		timings[local] = tick() - start_time;
		do_timings();
	}
}


/*
 *	R E A D C F I L E  --  Read cell file
 *
 */
readcfile(rotated)
{
	int		mypid;	
	int start_time, local;

	if (alc->timing)
		setuptim(&local, &start_time, "readcfile");
	mypid = getpid();
	if (alc->debug) {
		printf( "About to read file %s/%s rotated %d kcont 0x%x\n",
				alc->dir,alc->picname,rotated,kcont);
		fprintf(debug_channel,"File pointer 0x%x path %d buffer 0x%x bufsize %d\n",
			imgin,imgin->_fd,imgin->_base,imgin->_bufsiz);
	} else
		fprintf(stderr,"Reading file %s\n",alc->picname);
	if (alc->timing)
		timings[local] = tick() - start_time;
	return(readcell(imgin,kcont,rotated));
}
#ifdef UNUSED
/*
 *	F I X F I L E B U F F E R
 *
 */
fixfilebuffer(fp)
FILE *fp;
{
	int start_time, local;

	if (alc->timing)
		setuptim(&local, &start_time, "fixfilebuffer");
	if (!alc->debug) return;
	if (fp && fp->_base);
	if (alc->debug) {
		fprintf(debug_channel,"Fix file buffer fp 0x%x path %d buffer 0x%x bufsize %d mode 0x%x\n",
			fp,fp->_fd,fp->_base,fp->_bufsiz,fp->_flag);
	}
	if (fp->_fd >= 3) {
		if ((fp->_base != &fp->_save)) {
			if (fp->_bufsiz == 1)
				fp->_bufsiz = BUFSIZ;
		} else {
			if (alc->debug) fprintf(stderr,"Fileio - File buffer lost\007\n");
			fp->_flag &= ~_INIT;
			fp->_base = 0;
			fp->_ptr = 0;
			fp->_bufsiz = 0;
		}
	}
	if (alc->debug) {
		fprintf(debug_channel,"File pointer 0x%x path %d buffer 0x%x bufsize %d\n",
			fp,fp->_fd,fp->_base,fp->_bufsiz);
	}
	if (alc->timing)
		timings[local] = tick() - start_time;
}
#endif UNUSED

/*
 *	R E A D C E L L  --  read objects from file.
 *
 * Give them a property list if they lack one,
 * and set them active so that feature measurement may proceed
 *
 *	return 0 if fail 
 */
static
readcell(f,kcont,rotated)
FILE *f;
struct kcontrol *kcont;
{
	struct chromosome *obj, **ol;
	struct chromosome *readobj();
	struct chromplist *makechromplist();
	int objnum, first_time, first, start_time, local, version;

	if (alc->timing) {
		setuptim(&first, &first_time, "readcell -- total time");
		setuptim(&local, &start_time, "readcell--up to cellheadfill");
	}
	kcont->maxnumber = 0;	
	ol = rotated ? kcont->mol : kcont->ool;
	if (alc->debug) {
		fprintf(debug_channel,"Before readobj - fp 0x%x path %d buffer 0x%x size %d\n",
			f,f->_fd,f->_base,f->_bufsiz);
		fprintf(debug_channel,"Stack left %d\n",freemem());
	}
/*
 *	Read objects (typ 1 -> chrom, typ 110-> header)
 */
	cellheadfill(alc->cell_h,&alc->nameids);
 	windmill("Reading cell - ");
	if (alc->timing) {
		timings[local] = tick() - start_time;
		setuptim(&local, &start_time, "readcell--while loop");
	}
	while ((obj= readobj(f)) != NULL) {
		if (obj->type == 110) {
			if (alc->debug)
				fprintf(stdout,"File has a header record\n");
			version = ((struct cell_header *)obj->plist)->version;
			if (alc->debug)
				fprintf(debug_channel, "readcell header version %d\n", version);
			obj->plist = convcellhead(obj->plist);
			_strass(alc->cell_h,obj->plist,sizeof(struct cell_header));
			cellheadextract(alc->cell_h);
			freeobj(obj);
		} else if (obj->type == 1) {
			if (obj->plist == NULL) {
				obj->plist = makechromplist();
				obj->plist->number = kcont->maxnumber+1;
			} else if (version < 3) 		/* if old version */
				obj->plist->Cpgroup = 0;	/* reset Cpgroup, now used properly */
			objnum = rotated? obj->plist->number-1: kcont->maxnumber;
			ol[objnum] = obj;
			kcont->acl[objnum] = ACTIVE;
			if (rotated)
				kcont->acl[objnum] |= MASTCOPY;
			if (kcont->maxnumber <= objnum)
				kcont->maxnumber = objnum + 1;
		} else {
			windmillend("Illegal file object");
			if (alc->timing) {
				timings[local] = tick() - start_time;
				timings[first] = tick() - first_time;
			}
			return(0);
		}
		if (alc->debug) {
			fprintf(debug_channel,"Before readobj - fp 0x%x path %d buffer 0x%x size %d\n",
				f,f->_fd,f->_base,f->_bufsiz);
			fprintf(debug_channel,"Stack left %d\n",freemem());
		}
		windmill(NULL);
		if (alc->timing) {
			timings[local] = tick() - start_time;
			setuptim(&local, &start_time, "readcell--while loop");
		}
	}
	windmillend(NULL);
	if (alc->timing) {
		curr_time--;
		timings[first] = tick() - first_time;
	}
	return(1);
}


/*
 *	P U T F I L E  --  Write a digitised image to disc
 *
 *	where
 *		obb = orig unlabeled object (used to calc size)
 *		objl = oblist
 *		mycount = number of objects
 *
 * open file and write data to disk
 */
putfile(filename,obb,objl,mycount)
char	*filename;
struct object *obb, **objl;
int		mycount;
{
	FILE *fbdopen(), *fp;
	struct	object *head;
	int i,k,stat, start_time, local, first, first_time;

	if (alc->timing) {
		setuptim(&first, &first_time, "putfile--total time");
		setuptim(&local, &start_time, "putfile--up to seqsize");
	}
	stat = 1;
	if ((fp = fbdopen(filename,alc->dir,"w",BUFSIZ)) == NULL) {
		fileerror("Failed to open file %s",errno,filename);
		if (alc->timing) {
			timings[local] = tick() - start_time;
		}
		return(0);
	}
	windmill("Writing cell - ");
	head = (struct object *) makemain(110,NULL,NULL,alc->cell_h,NULL);
	k = seqsize(head);
	if (alc->timing) {
		timings[local] = tick() - start_time;
		setuptim(&local, &start_time, "seqsize and preexpand");
	}
/*
 *	Then size the file and write the data - either as objects or one lump
 */
	if (mycount) {
		for (i=0; i< mycount; i++ )
			k += seqsize(objl[i]);
		if (preexpand(fp,k) == 0) {
			fileerror("Failed to write file",errno);
			stat = 0;
		} else {
			writeobj(fp,head);
			for (i=0; i< mycount; i++ ) {
				if (writeobj(fp,objl[i]) == -1) {
					fileerror("Failed, can't write object",errno);
					stat = 0;
					break;
				} else {
					if (alc->debug) {
						screengo(24,60);
						printf("writing object %d\n",i);
					}
					windmill(NULL);
				}
			}
		}
	} else {
		k += seqsize(obb);
		if (preexpand(fp,k) == 0) {
			fileerror("Failed to write file",errno);
			stat = 0;
		} else {
			writeobj(fp,head);
			writeobj(fp,obb); /* if not labelled, write single */
			fprintf(stderr,"data copied to file %s\n",filename);
		}
	}
	if (alc->timing) {
		timings[local] = tick() - start_time;
		setuptim(&local, &start_time, "putfile--freeobj and close");
	}
	head->plist = NULL;
	freeobj(head);
	if (alc->debug)
		printf("putfile: Bytes  used %ld\n",ftell(fp));
	fbclose(fp);
	if (stat) {
		windmillend(NULL);
	} else {
		windmillend("");
		unlink(filename);
	}
	if (alc->timing) {
		timings[local] = tick() - start_time;
		timings[first] = tick() - first_time;
		do_timings();
	}
	return(stat);
}

#ifdef UNUSED
/*
 *	F S P U T F I L E  --  TV digitise version of putfile
 *
 * open file and write data to disk
 */
fsputfile(s,obb,objl,mycount) 
char *s; 
struct object *obb, **objl;
{
	FILE *fbopen(), *fp;
	struct intervaldomain *idom;
	struct object *head;
	register int i,k = 0, stat = 0;
	int start_time, local;

	if (alc->timing) {
		setuptim(&local, &start_time, "fsputfile");
	}
	if ((fp = fbopen(s,"w",BUFSIZ)) == NULL) {
		fprintf(stderr,"failed to open file %s\n",s);
	} else {
		head = (struct object *) makemain(110,NULL,NULL,alc->cell_h,NULL);
		k = seqsize(head);
		if (mycount) {
			for (i=0; i< mycount; i++ ) {
				idom = objl[i]->idom;
				if (idom->line1 == 0 || idom->lastln == 511)
					continue;
				if (idom->kol1 == 0 || idom->lastkl == 511)
					continue;
				k += seqsize(objl[i]);
			}
			if (stat = preexpand(fp,k)) {
				writeobj(fp,head);
				if (alc->debug)
					printf("putfile: mycount %d: Bytes taken %d\n",mycount,k-1);
				/* write segmented objects */
				for (i=0; i< mycount; i++ ) {
					idom = objl[i]->idom;
					if (idom->line1 == 0 || idom->lastln == 511)
						continue;
					if (idom->kol1 == 0 || idom->lastkl == 511)
						continue;
					fswriteobj(fp,objl[i]);
					printf("writing obj %d\n",i);
				}
				if (alc->debug)
					printf("putfile: Bytes  used %ld\n",ftell(fp));			
			}
		} else {
			k += seqsize(obb);
			if (stat = preexpand(fp,k)) {
				writeobj(fp,head);
				if (alc->debug)
					printf("putfile: mycount %d: Bytes taken %d\n",mycount,k-1);
				fswriteobj(fp,obb); /* if not labelled, write single */
			}
		}
		fprintf(stderr,"data copied to file %s\n",s);
		head->plist = NULL;
		freeobj(head);
		fbclose(fp);
		if (stat == 0)
			unlink(s);				/* decrement link count of file */
	}
	if (alc->timing) {
		timings[local] = tick() - start_time;
	}
}
#endif

/*
 *	P U T K F I L E  --  Write karyotype data to disc
 *
 * Returns 0 if fail
 *
 */
putkfile(filename,objlist)
char *filename;
struct chromosome **objlist; 
{
	FILE *fbdopen(), *fp;
	struct	object *head;
	int i,k,stat, start_time, local, first, first_time;

	if (alc->timing) {
		setuptim(&first, &first_time, "putkfile -- total time");
		setuptim(&local, &start_time, "putkfile--wormcast");
	}
	stat = 1;
	if (alc->debug)
		wormcast(kcont,MASTER,"KMR - putkfile");
	if ((fp = fbdopen(filename,alc->dir,"w",BUFSIZ)) == NULL) {
		fileerror("Failed to open file %s",errno,filename);
		return(0);
	}
	windmill("Writing karyotype - ");
	if (alc->timing) {
		timings[local] = tick() - start_time;
		setuptim(&local, &start_time, "putkfile--grab & seqsize head");
	}
/*
 *	write header
 */
	head = (struct object *) makemain(110,NULL,NULL,alc->cell_h,NULL);
	k = seqsize(head);
	if (alc->timing) {
		timings[local] = tick() - start_time;
		setuptim(&local, &start_time, "putkfile--loop calling seqsize");
	}
/*
 * seek to end of file to speed up file write 
 */
	for (i=0; i< kcont->maxnumber ; i++ )
		if ( kcont->acl[i]&ACTIVE == ACTIVE) {
			if (objlist[i] != NULL)
				k += seqsize(objlist[i]);
		}
	if (alc->timing) {
		timings[local] = tick() - start_time;
		setuptim(&local, &start_time, "putkfile--preexpand & writehead");
	}
	if (preexpand(fp,k) == 0) {
		fileerror("Failed to write file",errno);
		stat = 0;
	} else {
/*
 * write header + segmented objects
 */
		writeobj(fp,head);
		for (i=0; i< kcont->maxnumber ; i++ ) {
			if (alc->timing) {
				timings[local] = tick() - start_time;
				setuptim(&local, &start_time, "putkfile--writeobj loop");
			}
			if (kcont->acl[i]&ACTIVE == ACTIVE) {
				if (objlist[i]==0 && alc->debug) {
					screengo(24,60);
					fprintf(stdout,"Null object number %d\n",i);
				} else {
					if (writeobj(fp,objlist[i]) == -1) {
						fileerror("Failed, can't write object",errno);
						stat = 0;
						break;
					}
					windmill(NULL);
				}
			}
		}
	}
	if (alc->timing) {
		curr_time--;
		setuptim(&local, &start_time, "putkfile--freeobj & close");
	}
	head->plist = NULL;
	freeobj(head);
	fbclose(fp);
	if (stat) 
		windmillend(NULL);
	else {	
		windmillend("");
		unlink(filename);
	}
	if (alc->timing) {
		timings[local] = tick() - start_time;
		timings[first] = tick() - first_time;
		do_timings();
	}
	return(stat);
}


/*
 *   P R E E X P A N D  --  Preexpand file for speed and check that there is
 *							actually enough space on the disc
 *
 *	returns
 *			0 if fail
 *			1 if success
 */
preexpand(f,s)
FILE *f;
{
	long	size;
	int freeblks, totblks, percent, start_time, local;
	char tmpbuf[32];
	
	if (alc->timing)
		setuptim(&local, &start_time, "preexpand");
	*tmpbuf = '/';
	_gs_devn(f->_fd, &tmpbuf[1]);
	dfree(tmpbuf, &freeblks, &totblks);
	percent = (freeblks * 100) / totblks;
	if (alc->debug)
		fprintf(debug_channel, "preexpand:dev %s free %d tot %d percent %d\n", 
			tmpbuf, freeblks, totblks, percent);
	if (percent < 10) {
		fprintf(stdout, "Warning: Disc is %d%% full \007\n", (100 - percent));
		fprintf(stdout, "Please backup disc as soon as possible.\n");
	}
#ifdef OLD
	fseek(f,(long)s,0);
	putc(0,f);
	fflush(f);
	fseek(f,(long)0,0);
	getstat(2,f->_fd,&size);
#endif
	_ss_size(f->_fd,s);
/*	fseek(f,(long)0,0); */
	size = _gs_size(f->_fd);
	if (alc->debug)
		fprintf(debug_channel,"Preexpand - want %d, got %d\n",s,size);
	if (alc->timing) {
		timings[local] = tick() - start_time;
	}
	if (s == size)
		return(1);
	else {
		errno = 248;			/* Media full */
		return(0);
	}
}

/*
 *	D I S P _ M E S S G  --  Display message telling user to press return
 *							 once case setup from getid
 */
static
disp_messg()
{
	screengo(OUTLINE - 1,0);
	printf("Check cell id. is correct, and then press <Return>\n");
}

/*
 *	G E T I D
 *
 *	where
 *		Filetype is (CELLFILE, KARYFILE, ...)
 *		mode is     (READ, WRITE, NULL)
 */
getid(filetype,mode)
int		filetype, mode;
{
	char	filename[32];
	char	dirbuf[DIRSIZ+20];
	char	workbuf[40];
	int		Excmd, Specmd, Patcmd, Sldcmd, Cellcmd, cmd, ret, patset;
	int		(*linkerbug) ();
	struct	cyto_file *cfp;

	ret = 0;
	patset = 0;
	dirbuf[0] = '\0'; workbuf[0] = '\0';
	downmenu();
	putprompt();
	newmenu("Check case id");
	if ((filetype == CELLFILE) || (filetype == KARYFILE)) {
		cfp = &(alc->nameids);
		cfp->type = filetype;
		linkerbug = setcase;
		cmmd("case ID;:k38%+s%-17s%f;t2",workbuf,cfp->case,linkerbug); Specmd = mmcmd;
		linkerbug = setslide;
		cmmd("slide ID;:k38%+s%-5s%f;t3",workbuf,cfp->slide,linkerbug); Sldcmd = mmcmd;
		cmmd("cell number;:k38<0/9999>%d",&cfp->cellno); Cellcmd = mmcmd;
		linkerbug = setname;
		cmmd("NEF coordinate;:k38%+S%-39s%f",workbuf,cfp->patnam,linkerbug); Patcmd = mmcmd;
	}
	else if ((filetype == METFILE) || (filetype == SCOREFILE)) {
		cfp = &(alc->curbay->nameids);
		linkerbug = setcase;
		cfp->type = filetype;
		cmmd("case ID;:k38%+s%-17s%f;t2",workbuf,cfp->case,linkerbug); Specmd = mmcmd;
		linkerbug = setslide;
		cmmd("slide ID;:k38%+s%-5s%f",workbuf,cfp->slide,linkerbug); Sldcmd = mmcmd;
		linkerbug = setname;
		cmmd("patient name;:k38%+S%-17s%f",workbuf,cfp->patnam,linkerbug); Patcmd = mmcmd;
			Cellcmd = -99;
	}
	else if (filetype == FLEXFILE) {
		cfp = &(alc->flex_screen->id->nameids);
		cfp->type = filetype;
		cmmd("case ID;:k38%+s%-17s%f;t2",workbuf,cfp->case,setcase); Specmd = mmcmd;
		cmmd("slide ID;:k38%+s%-5s%f;t3",workbuf,cfp->slide,setslide); Sldcmd = mmcmd;
		cmmd("flexible screen number;:k38<0/9999>%d",&cfp->cellno); Cellcmd = mmcmd;
		cmmd("patient name;:k38%+S%-39s%f",workbuf,cfp->patnam,setname); Patcmd = mmcmd;
	}
	linkerbug = show_dir;
	cmmd("show directory;:k30%-+s%f%-f",alc->dir,linkerbug,disp_messg);mouse(MIDBT);
	linkerbug = setdir;
	cmmd("set working directory;%?s%-s%f",dirbuf,alc->dir,linkerbug);
	cmmd("quit operation"); mouse(LONGLB); Excmd = mmcmd;
	while (1) {
		if ((cmd = thismu()) == Excmd)
			break;
		if (cmd == Sldcmd) {
			displin(0,Specmd);
			displin(0,Sldcmd);
			dispheader();
			if(!patset) {
				cfp->patnam[0] = '\0';
				displin(0, Patcmd);
			}	
			putprompt();
		} else if (cmd == Cellcmd) {
			displin(0,Specmd);
			displin(0,Sldcmd);
			displin(0,Cellcmd);
			dispheader();
			if(!patset) {
				cfp->patnam[0] = '\0';
				displin(0, Patcmd);
			}	
			putprompt();
		} else if (cmd == Patcmd)
			patset = 1;

		if (strcmp(cfp->case,"UNSET") && ((cmd == 0))) {
			if (makefilename(filename,cfp)) {
				switch(filetype) {
				  case CELLFILE:
					if ((mode != READ) || fileexists(alc->dir,filename)) {
						strcpy(alc->picname,filename);
						ret = 1;
					}
					break;
				  case KARYFILE:
					if ((mode != READ) || fileexists(alc->dir,filename)) {
						strcpy(alc->picname,filename);
						ret = 1;
					}
					break;
				  case METFILE:
				  case SCOREFILE:
					if ((mode != READ) || fileexists(alc->dir,filename)) {
						strcpy(alc->metfile,filename);
						ret = 1;
					}
					break;
				  case FLEXFILE:
					if ((mode != READ) || fileexists(alc->dir,filename)) {
						strcpy(alc->flex_screen->id->filename,filename);
						ret = 1;
					}
					break;
				  case GDATFILE:
				  default:
					screengo(ENDLINE,1);
					fprintf(output,"Unknown file type request\n");
					goto getid_ex;
				}
			}
			if (ret == 1) {
				if (mode == WRITE) {
					if (checkoverwrite(alc->dir,filename))
						goto getid_ex;
					else
						putprompt();
				} else
					goto getid_ex;
			} else {
				screengo(ENDLINE,2);
				fprintf(output,"Can't open file %s\n", filename);
				putprompt();
			}
		}
		ret  = 0;
	}
getid_ex:
	upmenu();
	return(ret);
}


/*
 *	C H E C K O V E R W R I T E  --
 *
 */
checkoverwrite(dir,fil)
char	*dir,*fil;
{
	int local, start_time;
	char check[32];

	if (alc->timing)
		setuptim(&local, &start_time, "checkoverwrite");
	fnamtranslate(check, fil);				/* Translate filename sep chars */
	if (alc->debug) fprintf(debug_channel,"Checkoverwrite %s %s\n",dir,check);
	if (fileexists(dir,check)) {
		if (alc->debug) fprintf(debug_channel,"Checkoverwrite exists\n");
		screengo(ENDLINE,1);
		fprintf(output,"Warning - file already exists, Overwrite ? (M) :");
		if (alc->timing)
			timings[local] = tick() - start_time;
		if (tryboth(1))
			return(1);
		else
			return(0);
	} else
		return(1);
}

/*
 *	F I L E E R R O R  --  Decode file error codes and print message
 *
 */
fileerror(str,err,p1,p2,p3)
char	*str;
int		err, p1, p2, p3;
{
	fprintf(stderr,str,p1,p2,p3);
	switch (err) {
		case 214:
			fprintf(stderr," - file permission error");
			break;
		case 216:
			fprintf(stderr," - file not found");
			break;
		case 248:
			fprintf(stderr," - disc is full");
			break;
		case 253:
			fprintf(stderr," - file is busy");
			break;
		default:
			if (err)
				fprintf(stderr," (%d)",err);
		
	}
	fprintf(stderr,"\n");
}
/*
 * D O _ T I M I N G S -- open file and write timings out
 *
 */
static
do_timings()
{
	FILE *fp, *fopen();
	char buffer[80];
	int i = 0;
		
	sprintf(buffer, "/dd/usr/%s/timings.seat%d", alc->user, alc->seat);
	if ((fp = fopen(buffer, "a")) != NULL) {
		while (strings[i] != NULL) {
			fprintf(fp, "%s  %d\n", strings[i], timings[i]);
			i++;
		}
		fprintf(fp, "\n");
		fclose(fp);
	} else
		fprintf(stderr, "Can't open timing file\n");
	curr_time = -1;
	/* reset arrays */
	for (i = 0; i < MAXTIMS; i++) {
		timings[i] = 0;
		strings[i] = NULL;
	}
}
/*
 * S E T U P T I M-- set up string and start time if timing
 *
 */	
static
setuptim(loc, time, str)
int *loc, *time;
char *str;
{
	if (curr_time < MAXTIMS - 1)
		curr_time++;
 	*loc = curr_time;
	strings[*loc] = str;
	*time = tick();
}

