/*
 *      F I L E N A M E . C  --  Routines associate with filename
 *								generation and checking
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *
 *  Date:    2nr Jun 1987
 *
 *  Modifications
 * 03-02-92 joanne	FNAMEDELIM -> _ (MRC standard).
 *			Beware if alc->fnamedelim1,2 are not default because 
 *			then all sorts of horrible things happen.
 *  21 Jan 1990		ih			Changed sprintf for scorefile in makefilestring at to agree with orig.
 *	17 Dec 1990		jimp@MRC	Added FLEXFILE type
 *		Something here I don't understand -- "makefilename()" constructs
 *		filename with FNAMEDELIM, here defined to be $, then calls
 *		"fnamtranslate()", which will only do anything in the particular
 *		circumstance that alc->fnamesep[12] == FNAMEDELIM.  Just what
 *		is going on here??
 *
 *	25 Jul 1989		dcb		Add SCOREFILE type
 *	19 May 1989		dcb		Added fnambacktranslate
 *	18 May 1989		dcb		fnamtranslate/strtrans added here
 *
 */
#include <stdio.h>
#include <direct.h>
#include <kyy.h>

/*
 *	Filename field limits (Should be somewhere else )
 */
#define CELLMAXNO	9999
#define FNAMEDELIM	'_'

extern FILE *debug_channel;

extern char * index();

/*
 *	M A K E F I L E S T R I N G  --  Make up a file name string that can
 *									be shown to user
 *
 *	Returns pointer to string
 */
char *
makefilestring(cur_case,buffer)
struct cyto_file *cur_case;
char	*buffer;
{
	switch(cur_case->type) {
		case METFILE:
			sprintf(buffer,"slide %s (metaphase list)",cur_case->slide);
			break;
		case KARYFILE:
			sprintf(buffer,"slide %s cell %d (karyotype)",cur_case->slide,cur_case->cellno);
			break;
		case CELLFILE:
			sprintf(buffer,"slide %s cell %d",cur_case->slide,cur_case->cellno);
			break;
		case SCOREFILE:
			sprintf(buffer,"slide %s (scored)",cur_case->slide,cur_case->cellno);
			break;
		case FLEXFILE:
			sprintf(buffer,"slide %s screen no. %d",cur_case->slide,cur_case->cellno);
			break;
		default:
			*buffer = '\0';
	}
	return(buffer);
}


/*
 *	I S C Y T O
 *
 *	Name format is:
 *
 *			AAAAAAAAAAAAAAA$BBBB$M		metaphase list
 *			AAAAAAAAAAAAAAA$BBBB$D		Gen Data file
 *			AAAAAAAAAAAAAAA$BBBB$9999	cell
 *			AAAAAAAAAAAAAAA$BBBB$9999K	karyotype
 *			AAAAAAAAAAAAAAA$BBBB$S		aberration scored met list
 *			AAAAAAAAAAAAAAA$BBBB$9999F	flexible screen
 *
 *		Where AAA is Patient name
 *			  BBB is Slide Id
 *			  999 is cell number
 */
iscyto(name,cfp)
char	*name;
struct cyto_file *cfp;
{
	char	work[32], *dol1, *dol2, *ptr;
	char	*patient, *slide;
	int		typ;						/* What we think */
	int		cellno;

	typ = 0;
	strntcpy(work,name,FNAMELEN);
	patient = slide = NULL;
	if (alc->debug) fprintf(debug_channel,"iscyto :%s: ",work);
	if ((dol1 = index(work,FNAMEDELIM)) == NULL) return(0);
	if ((dol2 = index(dol1+1,FNAMEDELIM)) == NULL) return(0);

	if (dol1 != work) patient = work;
	if (dol1+1 != dol2) slide = dol1+1;
	ptr = dol2+1;
	switch (*ptr) {

		case 'm':
		case 'M':
			typ = METFILE;
			break;
		case 'd':
		case 'D':
			typ = GDATFILE;
			break;
		case 's':
		case 'S':
			typ = SCOREFILE;
			break;
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			cellno = atoi(ptr);
			while (isnum(*ptr))
				ptr++;
			switch (*ptr) {
				case 'k':
				case 'K':
					typ = KARYFILE;
					break;
				case 'f':
				case 'F':
					typ = FLEXFILE;
					break;
				case '\0':
					typ = CELLFILE;
					break;
				default:
					return(0);
			}
			break;
		default:
			return(0);
	}
/*
 * Now terminate string fields and check fields against limits
 */
	*dol1 = '\0';
	*dol2 = '\0';
	if (!patient || (strlen(patient) > CASELENGTH)) return(0);
	if (slide && (strlen(slide) > SLIDELENGTH)) return(0);
				
	if (cfp != NULL) {
		strcpy (cfp->case,patient);
		if (slide)
			strcpy(cfp->slide,slide);
		else 
			cfp->slide[0] = '\0';
		switch (typ) {
			case CELLFILE:
			case KARYFILE:
			case FLEXFILE:
				cfp->cellno = cellno;
				break;
			case METFILE:
			case GDATFILE:
			case SCOREFILE:
				cfp->cellno = 0;
				break;
			default:
				return(0);
		}
		cfp->type = typ;
	}
	return(typ);
}

/*
 *	M A K E F I L E N A M E  --  Make up a cytoscan file name
 *								using data in cfp struct, and check
 *								it is syntatically OK
 *
 *	where
 *		filetype is (CELLFILE, KARYFILE etc)
 *		filename is pointer to where to put it
 */
makefilename(filename,cfp)
char	*filename;
struct	cyto_file	*cfp;
{
	char	filebuf[32];

	if ((cfp == NULL) || !strncmp(cfp->case,"UNSET",5))
		return(0);
	switch (cfp->type) {
		case METFILE:
			sprintf(filebuf,"%s%c%s%cM",cfp->case,FNAMEDELIM,cfp->slide,FNAMEDELIM);
			break;
		case GDATFILE:
			sprintf(filebuf,"%s%c%s%cD",cfp->case,FNAMEDELIM,cfp->slide,FNAMEDELIM);
			break;
		case CELLFILE:
			sprintf(filebuf,"%s%c%s%c%d",cfp->case,FNAMEDELIM,cfp->slide,FNAMEDELIM,cfp->cellno);
			break;
		case KARYFILE:
			sprintf(filebuf,"%s%c%s%c%dK",cfp->case,FNAMEDELIM,cfp->slide,FNAMEDELIM,cfp->cellno);
			break;
		case SCOREFILE:
			sprintf(filebuf,"%s%c%s%cS",cfp->case,FNAMEDELIM,cfp->slide,FNAMEDELIM);
			break;
		case FLEXFILE:
			sprintf(filebuf,"%s%c%s%c%dF",cfp->case,FNAMEDELIM,cfp->slide,FNAMEDELIM,cfp->cellno);
			break;
		default:
			return(0);
	}
	fnamtranslate(filebuf, filebuf); /* Copy and translate */
	if (alc->debug) fprintf(debug_channel,"New f/name %s\n",filebuf);
	if (iscyto(filebuf,NULL)) {
		strcpy(filename,filebuf);
		return(1);
	} else
		return(0);
}

/*
 *	F N A M T R A N S L A T E -- Translate fnamesep1 & 2 chars to _ and .
 */
fnamtranslate(to, from)
char *to, *from;
{
	strcpy(to, from); /* Copy across */
	strtrans('_', alc->fnamesep1, to);	/* Translate fnamesep1 to _ */
	strtrans('.', alc->fnamesep2, to);	/* Translate fnamesep2 to . */
}

/*
 *	F N A M B A C K T R A N S L A T E --
 *	Translate _ and . to fnamesep1 & 2 chars
 */
fnambacktranslate(to, from)
char *to, *from;
{
	strcpy(to, from); /* Copy across */
	strtrans(alc->fnamesep1, '_', to);	/* Translate _ to fnamesep1 */
	strtrans(alc->fnamesep2, '.', to);	/* Translate . to fnamesep2 */
}

/*
 *	S T R T R A N S -- Translate all occurrances of from_ch to to_ch in str
 *	up to a NULL in str
 */
strtrans(to_ch, from_ch, str)
char to_ch, from_ch, *str;
{
	while(*str) {
		if (*str == from_ch) *str = to_ch;
		str++;
	}
} /* end strtrans */

/*
 *	I S N U M  --  Return true if char is numeric
 *
 */
static
isnum(n)
char	n;
{
	return ((n>='0') && (n <= '9'));
}


