/*
 *      H E A D F I L L . C  --  Routines to fill file header records
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    9th December 1986
 *
 *  Modifications
 *
 *	21 Jan 1991		ih			Added include chromanal.h,wstruct.h to compile
 *	19 Dec 1990		JimP@MRC	Added flex_head_fill()
 *	07 Mar 1990		dcb			Use fluor flag, and camera type to set dig_type
 *								field in cellheadfill
 *	16 Aug 1989		CAS			Stuff to get fluor status from dig_type
 *	 6 Jul 1989		SEAJ		Added version 3 header to convcellhead()
 *	11 Apr 1989		dcb/SEAJ	Assign slide width to metaphase list header
 *	21 Jan 1988		SCG/CAS		Explicit clear of comments etc in cellfill
 *	12 Jan 1988		SCG			In convcellhead copy also the comments
 *	26 Aug 1987		CAS			Default dig type is DIGFIP, don't set
 *								digtype in cellheadfill (RK...)
 *	13 Aug 1987		SCG			Normal/Abnormal taken out
 *	30 Jul 1987		CAS			Correct update of header in karyheadfill
 *								It also helps to get the right number of parameters..
 *	27 Jul 1987		CAS			String inconsistancies fixed
 *	24 Jul 1987		CAS			Properly fixed makemethead
 *	23 Jul 1987		CAS/GJP		Free's in make routines needed ptr's..
 *	14 Jul 1987		CAS			Mods to operation of 'convsellhead'
 *	13 Jul 1987		SCG			New versions of metfile and cellfile
 *								header structs moded
 *	29 Jan 1987		CAS			Set/use dig_type in headers
 *	 7 Jan 1987		CAS			Init cellid in cellheadfill
 *	 5 jan 1987		CAS			Added make cell + met routines
 *	18 Dec 1986		CAS			Added metheadfill routine
 */
#include <stdio.h>
#include <allcontr.h>
#include <slides.h>
#include <kcontrol.h>
#include <fileheaders.h>
#include <chromanal.h>
#include <wstruct.h>
#include <flex.h>

extern struct allkontr *alc;
extern struct kcontrol *kcont;


/*
 *	M A K E C E L L H E A D  --  Make a cell header record
 */
makecellhead(hptr)
struct	cell_header **hptr;
{

	if(*hptr != NULL)
		Free(*hptr);
	*hptr = (struct cell_header *) makeplist(sizeof(struct cell_header));
	
}

/*
 *	M A K E M E T H E A D  --  Make a metlist header record
 *
 *	Warning - be sure we own the old one if we are going to free it
 */
struct met_header *
makemethead(hptr)
struct	met_header *hptr;
{
	if (hptr != NULL)
		Free(hptr);
	return((struct met_header *) makeplist(sizeof(struct met_header)));
}

/*
 *	C E L L H E A D F I L L  --  Fill cell header data structure
 */
cellheadfill(hptr,cfp)
struct	cell_header *hptr;
struct	cyto_file *cfp;
{
	char	buffer[50];
	int		i;
	for (i=0; i<50; i++)
		buffer[i] = ' ';
	hptr->size = sizeof(struct cell_header);
	hptr->version = HEADER_VERSION;
	hptr->file_type = CELLFILE;
	strncpy(hptr->date,alc->mydate,DATELENGTH);
	strncpy(hptr->patnam,cfp->patnam, PATLENGTH);
	strncpy(hptr->lab_name,alc->sitename,LABLENGTH);
	strncpy(hptr->opid,alc->user,OPERLENGTH);
	strncpy(hptr->patid,cfp->case,CASELENGTH);
	strncpy(hptr->slideid,cfp->slide,SLIDELENGTH);
	hptr->cellid = cfp->cellno;
	strncpy(hptr->comments[0],buffer,TEXTLENGTH);
	strncpy(hptr->comments[1],buffer,TEXTLENGTH);
	*(hptr->comments[0]) = '\0';
	*(hptr->comments[1]) = '\0';
	strncpy(hptr->krtype,buffer,TEXTLENGTH);
	strncpy(hptr->supid,buffer,OPERLENGTH);
	strncpy(hptr->supdate,buffer,DATELENGTH);
	*(hptr->krtype) = '\0';
	*(hptr->supid) = '\0';
	*(hptr->supdate) = '\0';
	if (alc->fluor) hptr->dig_type |= DIGFLUOR;	/* Add fluor. flag */
	hptr->dig_type |= ((alc->cameratype & TYPEMASK) << 4 ); /* Cam. type */
}

/*
 *	C O N V C E L L H E A D  --  Copy version 0.4.3. header and prior to
 *								 new header (post 0.4.3).
 */
struct cell_header *
convcellhead(ohptr)
struct	cell_header *ohptr;
{
	struct cell_header_1 *v1ptr;
	struct	cell_header *hptr;

	switch (ohptr->version) {
		case 1:
			v1ptr = (struct cell_header_1 *) ohptr;
			hptr = (struct cell_header *) makeplist(sizeof(struct cell_header));
			hptr->version = HEADER_VERSION;
			hptr->file_type = v1ptr->file_type;
			strncpy(hptr->date,v1ptr->date,DATELENGTH);
			hptr->patnam[0] = '\0';
			strncpy(hptr->lab_name,v1ptr->lab_name,LABLENGTH);
			strncpy(hptr->opid,v1ptr->opid,OPERLENGTH);
			strncpy(hptr->patid,v1ptr->patid,CASELENGTH);
			strncpy(hptr->slideid,v1ptr->slideid,SLIDELENGTH);
			hptr->cellid = v1ptr->cellid;
			if (v1ptr->dig_type==0)
				hptr->dig_type = DIGFIP;
			else
				hptr->dig_type = v1ptr->dig_type;
			hptr->x_coord = v1ptr->x_coord;
			hptr->y_coord = v1ptr->y_coord;
			hptr->z_coord = v1ptr->z_coord;
			hptr->x_size = v1ptr->x_size;
			hptr->y_size = v1ptr->y_size;
			
			strncpy(hptr->comments[0],v1ptr->comments[0],TEXTLENGTH);
			strncpy(hptr->comments[1],v1ptr->comments[1],TEXTLENGTH);
			strncpy(hptr->krtype,v1ptr->krtype,TEXTLENGTH);
			strncpy(hptr->supdate,v1ptr->supdate,DATELENGTH);
			
			Free(ohptr);
			return(hptr);
		case 2:
		case 3:
			return(ohptr);
	}
	
}


/*
 *	K A R Y H E A D F I L L  --  Fill karyotype header data structure
 */
karyheadfill(hptr)
struct	cell_header *hptr;
{
	hptr->size = sizeof(struct cell_header);
	hptr->version = HEADER_VERSION;
	hptr->file_type = KARYFILE;
	strncpy(hptr->date,alc->mydate,DATELENGTH);
	strncpy(hptr->lab_name,alc->sitename,LABLENGTH);
	strncpy(hptr->opid,alc->user,OPERLENGTH);
}

/*
 *	M E T H E A D F I L L  --  Fill metaphase list header data structure
 *
 *				Copy variables from structure to metaphase list header.
 *		
 */
metheadfill(met_h, bp)
struct	met_header *met_h;
struct	mult_slide *bp;
{
	
	met_h->size = sizeof(struct met_header);
	met_h->version = HEADER_VERSION;
	met_h->file_type = METFILE;
	strncpy(met_h->date,alc->mydate,DATELENGTH);
	strncpy(met_h->lab_name,alc->sitename,LABLENGTH);
	strncpy(met_h->opid,alc->user,OPERLENGTH);
	strncpy(met_h->patid,bp->nameids.case,CASELENGTH);
	strncpy(met_h->patnam, bp->nameids.patnam, PATLENGTH);
	strncpy(met_h->slideid,bp->nameids.slide,SLIDELENGTH);
	met_h->nrecs = bp->totin;
	if(bp->compos > bp->combot) 
		met_h->comments = bp->compos - bp->combot;
	else
		met_h->comments = 1;
	met_h->tissue = bp->typeids.ktistype;
	met_h->stain = bp->typeids.kfstype;
	met_h->slwidth = bp->slwidth;
									/* Box limits */
	met_h->bbot = bp->mbox.bbot;
	met_h->btop = bp->mbox.btop;
	met_h->bleft = bp->mbox.bleft;
	met_h->bright = bp->mbox.bright;
										/* Last subset */
	met_h->lastsub[0] = bp->wh.classd[0];
	met_h->lastsub[1] = bp->wh.juu[0];
	met_h->lastsub[2] = bp->wh.uubl[0];
	met_h->lastsub[3] = bp->wh.unum[0];
	met_h->lastsub[4] = bp->whb.dicc[0];
	met_h->lastsub[5] = bp->whb.fragg[0];
	met_h->lastsub[6] = bp->whb.ringg[0];
	met_h->lastsub[7] = bp->whb.inbx[0];
	
	met_h->allrecs = (short) bp->allrecs;
	met_h->listlow = (short) bp->listlow;
	met_h->listhigh = (short) bp->listhigh;
	met_h->minrank = (short) bp->minrank;
	met_h->maxrank = (short) bp->maxrank;
	met_h->sany = (short) bp->sany;
	met_h->ngrps = (short) bp->ngrps;
	met_h->byg = (short) bp->byg;
	met_h->retrain = bp->retrain;
	
}

/*
 *	C E L L H E A D E X T R A C T
 *
 */
cellheadextract(hptr)
struct	cell_header *hptr;
{
	kcont->FIP = DIGFIP;
	alc->fluor = 0;
	if (hptr->dig_type) {
		kcont->FIP = hptr->dig_type & DIGMASK;
		if (hptr->dig_type & DIGFLUOR)
			alc->fluor = 1;
	}
}

/*
 *	 F L E X _ H E A D  _ F I L L  --  Fill flexible screen header data structure
 *
 */
flex_head_fill(fs)
struct flexible_screen *fs;
{
	struct	flex_header *hptr = fs->id->header;
	struct	cyto_file *cfp = &(fs->id->nameids);
	char	buffer[50];
	int		i;
	for (i=0; i<50; i++)
		buffer[i] = ' ';
	hptr->size = sizeof(struct flex_header);
	hptr->version = HEADER_VERSION;
	hptr->file_type = FLEXFILE;
	strncpy(hptr->date,alc->mydate,DATELENGTH);
	strncpy(hptr->patnam,cfp->patnam, PATLENGTH);
	strncpy(hptr->lab_name,alc->sitename,LABLENGTH);
	strncpy(hptr->opid,alc->user,OPERLENGTH);
	strncpy(hptr->patid,cfp->case,CASELENGTH);
	strncpy(hptr->slideid,cfp->slide,SLIDELENGTH);
	hptr->cellid = cfp->cellno;
	hptr->dig_type = ((alc->cameratype & TYPEMASK) << 4 ); /* Cam. type */
	if (alc->fluor) hptr->dig_type |= DIGFLUOR;	/* Add fluor. flag */
}


