/*
 * ident.c		Jim Piper		21-8-85
 *
 * Interactively identify displayed object by pointing, and
 * return a command.  Use the objectlist, marklist, activelist
 * type of data structure used in multi-processor karyotyping.
 *
 * Modifications
 *
 *  08 Jan 1992		Joanne		Included test on identify_set in locate().
 *  29 Jul 1991		ih		Check mouse cursor pos against boundaries
 *	20 Mar 1990		dcb		Move type display up by 80 (new pos = 112)
 *	14 Feb 1989		CAS		Ditched all that I_opened_mouse junk - Unused anyway
 *  18 Jan 1989		SEAJ	Check that current object is active in objident
 *							and delete old box in markident.
 *	17 Aug 1988		CAS		Objident - dont care if current is active or not
 *	 5 Aug 1988		CAS		Removed some old mouse technology from initident
 */
#include <cmdsys.h>
#include <kyy.h>
#include <interact.h>
#include <chromanal.h>
#include <kcontrol.h>

#define LOCATEADD	8
#define ABS(x) (x>=0?x:-x)

extern struct allkontr *alc;

extern FILE *debug_channel;

struct ipoint cursor;		/* Struct for cursor arrow */

static FILE save_mouse;		/* An IOB used to save mouse file iob
								params - Restored because of the aggro 
								caused by setbuf */
char *tag[7] = {
	"Unknown",
	"CHROM",
	"COMPO",
	"OVLAP",
	"PIECE",
	"NUCLEUS",
	"NOISE"
};

static	char charbanc[40];		/* Used by dispntype as what to erase */
/*
 *	I N I T I D E N T  --  start this identify/command session
 */
initident(kcont,cursor,ifile)
struct kcontrol *kcont;
register struct ipoint *cursor;
FILE **ifile;
{
	kcont->current_obj = 0;
	cursor->type = 40;
	cursor->style = 6;
	cursor->k = kcont->f->ox;
	cursor->l = kcont->f->oy;
	charbanc[0] = '\0';
	colour(OVERLAY1);
	intens(1);
	PICFRM(cursor,kcont->f);
}

/*
 *	E N D I D E N T  --  end this identify/command session
 */
endident(objarray, kcont, cursor, ifile)
struct onject **objarray;
struct kcontrol *kcont;
struct ipoint *cursor;
FILE **ifile;
{
	intens(0);
	PICFRM(cursor,kcont->f);
	markident(0,objarray,kcont,0);
}

/*
 *	M A R K I D E N T  --  display rectangle round identified object.
 *						   display object type and count (if composite)
 *						   at bottom of screen.
 *
 *	n = objnum (1 -> kcont->maxnumber)
 */
markident(n,objarray,kcont,onoff)
register int		n,onoff;
register struct object **objarray;
register struct kcontrol *kcont;
{
	register struct object *obj;

	colour(OVERLAY2);
	intens(0);
	unbox();
	if ((n>=1) || !onoff) {
		if (!n) n = 1;
		obj = (struct object *) objarray[n-1];
		typendisp(n,obj,0);
		dispotype(obj,kcont->f,0);
		
		if (!onoff) return;
		intens(onoff); 
	   
		boxobj(obj,kcont->f,OVERLAY2);
		typendisp(n,obj,onoff);
		dispotype(obj,kcont->f,onoff);
	} else {
		if (alc->debug) {
			fprintf(debug_channel,"Markident - negative objnum\n");
			fflush(debug_channel);
		}
	}
}

/*
 *	V I S I T O B J  --  visit a specified object
 *
 *	Objnum (1 -> n)
 */
visitobj(objnum,objarray,kcont,cursor)
register int		objnum;
register struct object **objarray;
register struct kcontrol *kcont;
register struct ipoint *cursor;
{
	struct intervaldomain *idom;
	if (alc->debug) fprintf(debug_channel,"visitobj %d\n",objnum);
	if ((objnum > 0) && (objnum < kcont->maxnumber)) {
		idom = objarray[objnum-1]->idom;
		cursor->l = (idom->lastln + idom->line1)/2;
		cursor->k = (idom->lastkl - idom->kol1)/2;
/*		kcont->current_obj = objident(objarray,kcont,cursor);*/
		kcont->current_obj = objnum;
		markident(kcont->current_obj,objarray,kcont,1);
	}
}

/*
 *	T Y P E N D I S P  --  display object type and composite count 
 *							at bottom of screen
 *
 *	n = objnum (1 -> kcont->maxnumber)
 */
typendisp(n,obj,onoff)
struct chromosome *obj;
{
	register struct chromplist *plist;
	register int confmarker;
	static char *doubtful = "??";
	int dispcol;
	
	if ( (alc->ddgschannel->type == FS_VIRTUOSO) ||
		 (alc->ddgschannel->type == FS_VIRTUOSO_INVERT) )
		 dispcol = 2400;  /* direct translation gives 1800 but 2400 more central */
		 					/* WRONG.. For what its worth.. direct translation is 2224 CAS.. */
	else
		dispcol = 1200;
		
	colour (OVERLAY2);
	tsize(80,80) ;
	if (*charbanc) {
			intens(0);
			dispstring(doubtful,dispcol,112);
			dispstring(charbanc,dispcol + 200, 112);
			charbanc[0] = '\0';
	}
	if (onoff==0) return;
	plist = obj->plist;
	/* light up the doubtful sign if object type not finally determined */
	if (plist->otype > CHROMOSOME && plist->otconf < 100)
		confmarker = 1;
	else
		confmarker = 0;
	intens(confmarker);
	dispstring(doubtful,dispcol, 112);
	if (plist->otype == 2 || plist->otype == 3)
		sprintf ( charbanc, "%d: %s  %d",n,tag[plist->otype],plist->history[2]);
	else
		sprintf ( charbanc, "%d: %s",n,tag[plist->otype]);
	intens(1);
	dispstring(charbanc,dispcol+200,112);
	tsize(50,50);
}

/*
 * display object type and composite count at bottom right of screen
 *
 *	This is superceded by typendisp but is still used in some places
 */
typedisp(obj)
struct chromosome *obj;
{
	register struct chromplist *plist;
	plist = obj->plist;
	tsize(130,150);
	dispstring(tag[plist->otype],3000,112);
	if (plist->otype == 2 || plist->otype == 3)
		dispnum(plist->history[2],3800, 112);
}

/*
 *	O B J I D E N T  --  identify the next object
 */
objident(objarray,kcont,cursor)
struct object **objarray;
struct kcontrol *kcont;
struct ipoint *cursor;
{
	static int oldchr = 0;
	register int new;
	new = locate(objarray,kcont,cursor);
	if (new != kcont->current_obj) {
		if (new == 0) {
			if ((kcont->current_obj != 0) && (kcont->acl[kcont->current_obj-1] & ACTIVE))
				oldchr = kcont->current_obj;
			else {
				kcont->current_obj = 0;
				oldchr = 0;
			}
		} 
	}
	return(new ? new : oldchr);
}

/*
 *	L O C A T E 
 *
 * find if (curs->k,curs->l) uniquely identifies an object.
 * If so, return object serial number, otherwise -1.
 * Assumption: objects DO NOT OVERLAP.
 *
 * Technique: first see if point lies within right rectangle
 * surrounding any object, and slightly bigger.  Second, for each such
 * object, see which rectangle is smallest.  Third, if ambiguous,
 * see which is nearest centre.
 */
locate(objlist,kcont,curs)
register struct chromosome **objlist;
register struct kcontrol *kcont;
struct ipoint *curs;
{
	register int i,k,l,nn;
	int dk,dl,d,k1,kl,l1,ll,hit,area,minarea,mindist;
	register struct intervaldomain *iidom;
	
	minarea = 10000000;
	mindist = 1000;
	k = curs->k;
	l = curs->l;
	hit = 0;
	for (i=0; i<kcont->maxnumber; i++) {
		if (kcont->identify_set != (kcont->acl[i] & (IDENTIFY1|IDENTIFY2)))
			continue;
		if (!(kcont->acl[i] & ACTIVE)) {
			nn = objlist[i]->plist->Cotype; 
			if ( nn != NOISE && nn != NUCLEUS ) continue;
	        }
		iidom = objlist[i]->idom;
		k1 = iidom->kol1 - LOCATEADD;
		if (k >= k1) {
			kl = iidom->lastkl + LOCATEADD;
			if (k <= kl) {
				l1 = iidom->line1 - LOCATEADD;
				if (l >= l1) {
					ll = iidom->lastln + LOCATEADD;
					if (l <= ll) {
						area = (kl-k1) * (ll-l1);
						if (area <= minarea) {
							dk = 2*k-k1-kl;
							dl = 2*l-l1-ll;
							d = ABS(dk)+ABS(dl);
							if (area < minarea || d<mindist) {
								hit = i+1;
								minarea = area;
								mindist = d;
							}
						}
					}
				}
			}
		}
	}
	return(hit);
}

/*
 *	P I C F R M
 *
 */
PICFRM(){}


/*
 * additional routine to keep a mouse-driven cursor up to date.
 * If scale is negative, it represents the (-ve) reciprocal of the
 * desired scale.  In this case, work in scaled-up coordinate
 * system and keep the unused remainder of the mouse movement
 * for next time.
 */
mousecurs(curs,f,m,scale,curscolour)
register struct ipoint *curs;
register struct pframe *f;
register struct m2mouse *m;
register int scale, curscolour;
{
	register int xscale,yscale,div;
	static xrem = 0, yrem =0;
	if (scale >= 0)
		div = 1;
	else {
		div = -scale;
		scale = 1;
	}
	xscale = yscale = scale;
	if (f->ix < 0)
		xscale = -xscale;
	if (f->iy < 0)
		yscale = -yscale;
	colour(curscolour);
	intens(0);
	picframe(curs,f);
	if (div > 1) {
		xrem += m->XINC * xscale;
		curs->k += xrem/div;
		xrem %= div;
		yrem += m->YINC * yscale;
		curs->l += yrem/div;
		yrem %= div;
	}
	else {
		curs->k += m->XINC * xscale;
		curs->l += m->YINC * yscale;
	}
	intens(1);
	picframe(curs,f);
}

/*
 *	M O D C C O U N T  --  modify the count of an object, and change its
 *							type appropriately
 */
modccount(objnum,objarray,kcont,inc)
register int		objnum, inc;
register struct chromosome **objarray;
register struct	kcontrol *kcont;
{
	register struct chromplist *plist;

	plist = objarray[objnum-1]->plist;
	/*
	 * Check if attention should be turned off.
	 * Ensure that doubtful counts are correctly handled
	 * by converting to CHROMOSOME before modifying count.
	 */
	checkattention(0,objnum-1,kcont);	
	if (plist->otype > CHROMOSOME && plist->otconf < 100)
		plist->otype = CHROMOSOME;
	plist->otconf = 100;
	if (inc > 0) {
		switch(plist->otype) {
			default:
				if (plist->history[2] == 0)
					plist->history[2] = 1;
			case CHROMOSOME:
			case COMPOSITE:
			case OVERLAP:
				plist->otype = COMPOSITE;
				break;
			case NOISE:
			case NUCLEUS:
			case PIECE:
				plist->otype = CHROMOSOME;
				break;
		}
		plist->history[2]++;
		alc->numchroms++;
	} else {
		switch(plist->otype) {
			default:
			case CHROMOSOME:
				plist->otype = NOISE;
				plist->history[2] = 0;
				alc->numchroms--;
				break;
			case COMPOSITE:
			case OVERLAP:
				plist->history[2]--;
				if (plist->history[2] == 1)
					plist->otype = CHROMOSOME;
				alc->numchroms--;
				break;
			case NOISE:
				plist->otype = NUCLEUS;
				break;
			case NUCLEUS:
				plist->otype = PIECE;
				break;
			case PIECE:
				plist->otype = NOISE;
				break;
		}
	}
	intens(1);
	countdisp(alc->numchroms,1);
	typendisp(objnum,objarray[objnum-1],1);
	dispotype(objarray[objnum-1],kcont->f,1);
}




