/*
 *      S Y S T E M . C  --  Description
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    8th October 1986
 *
 *  Modifications
 *
 *	16 Aug 1989		ARL			Added extern declaration for cyto_dir()
 *	 1 Jun 1989		CAS			Def errno for v2.2
 *	19 May 1989		dcb			use fnambacktranslate in show_dir !!
 *	18 May 1989		dcb			fnamtranslate/strtrans to filename.c
 *	18 May 1989		dcb			Put translation in show_dir too.
 *	16 May 1989		dcb			Put translation in fbdopen too !!, and call as
 *								fnamtranslate. Malloc space for translated str.
 *	 8 May 1989		dcb			setcase: Add ch trans. from user specified
 *								to . and _ Also o/p what is illegal. Put
 *								strtrans function in here until it goes in gplib
 *	 5 May 1989		dcb			Rip out: tisstmu, knowntissue, knownstain,
 *								gettissue, getstain, settissue,	setstain.
 *								These function can now be found in newdentify.c
 *	16 Mar 1989		CAS/dcb		Change Malloc to Calloc in remove
 *	23 Jul 1989		CAS			Extra param to remove
 *	14 Feb 1989		CAS			Mods to stop chdir's - new fbdopen
 *	23 Jan 1989		SCG/dcb		knowntissue/knownstain return 0 if given 0 properly
 *								also increase DISPTISSUE/DISPSTAIN by 1
 *	 1 Dec 1988		CAS			woolzexitfn returned here.. + new woolz stuff
 *								incorperated back in
 *	18 Nov 1988		dcb			woolzexitfn() moved to woolz/subs/wtcheck.c
 *	09 Sep 1988		SCG			Knowntissue and knownstain return 0 if given 0
 *	17 Aug 1988		CAS			Changes to config handling
 *	27 Jun 1988		SCG			Moved gettissue, getstain, knowntissue, knownstain,
 *								settissue, setstain to here from newdentify.c
 *	14 Apr 1988		SEAJ		Changed setname to allow name to be removed
 *	 2 Feb 1988		CAS			Init workbuf in dir options
 *
 */
#include <cmdsys.h>
#include <module.h>
#include <procid.h>
#include <direct.h>
#include <kyy.h>
#include <slides.h>

#define ERRORLOG "/dd/sys/syserrors"

#define DIRSTART 5			/* Line to start directory display */

#define DIR_PERM 0x3f		/* default dir access permisions */
#define DISPTISSUES	5		/* Max number of tissue types to display */
#define DISPSTAINS	3		/* Max number of stain types to display */

extern int errno;
extern struct allkontr *alc;
extern FILE *debug_channel;

extern  int	(*quit_fn)();

extern	char	*index();
		int 	dirsort();
		int		show_dir(), show_subdir(), make_subdir(), cyto_dir();
		int		screendump();

/*
 *	T E S T C H I L D R E N --
 *
 *		Test and accept the deaths of our children
 *
 */
testchildren()
{
	struct	pdsc	process_desc, *pd;
	int	pid;

	pid = 0;
	pd = &process_desc;
	if (gprdsc(getpid(),&process_desc) != -1) {
		if (pd->_cid) {
			if (testsib(pd->_cid)) {
				pid = wait(0);
/*				fprintf(stderr,"Process %d is no more\n",pid); */
			}
		}
	}
	return(pid);
}

/*
 *	T E S T S I B  --
 *
 */
static
testsib(id)
int		id;
{
	struct	pdsc	process_desc, *pd;

	pd = &process_desc;
/*	fprintf(stderr,"Testing process %d\n",id); */
	if (gprdsc(id,pd) != -1) {
		if (pd->_pmodul == NULL)
			return(1);
		if (pd->_sid)
			return(testsib(pd->_sid));
	}
	return(0);
}

/*
 *	S E T Q U I T F N  --  Set up quit function
 *
 */
setquitfn(proc)
int (*proc)();
{
	quit_fn = proc;
}

/*
 *	W O O L Z E X I T F N  --
 *
 */
woolzexitfn(err, called_from)
int		err;
char	*called_from;
{
	char	buffer[100];
	sprintf(buffer, "Woolz error no. %d [%s] detected, called from %s\n",
		err, woolz_err_msg(err), called_from);
	fprintf(stderr,"%s",buffer);
	if (woolz_fatal(err)) {
		errorlog(buffer);
		kill(getpid(),WOOLZSIG);
	}
	return(err);
}

/*
 *	M E M E X I T F N   --  Memory alloc errors come here
 *
 */
memexitfn(err,str)
int		err;
char	*str;
{
	char	buffer[100];
	sprintf(buffer,"Memory error %d %s\n",err,str);
	errorlog(buffer);
	fprintf(stderr,"Memory corruption error\n");
	kill(getpid(),WOOLZSIG);
}

/*
 *	E R R O R L O G  --  Write messsage to error log file
 *
 */
errorlog(message)
char	*message;
{
	char	buffer[100];
	int		fd, time, date, day, tick;

	if ((fd = open(ERRORLOG,2)) == -1)
		if ((fd = create(ERRORLOG,2,0x1b)) == -1) 
			return(0);
	lseek(fd,0,2);		/* get to eof */
	_sysdate(0,&time,&date,&day,&tick);
	sprintf(buffer,"%d/%d/%d %d:%d  %s (seat %d) ",date&255,(date>>8)&255,(date>>16)&65535,(time>>16)&255,(time>>8)&255,_prgname(),alc->seat);
	writeln(fd,buffer,strlen(buffer));
	writeln(fd,message,strlen(message));
	close(fd);
}

/*
 *	R E M O V E  --
 *
 */
remove(dir,filename)
char *dir,*filename;
{
	char	*bptr;
	int		len;
	len = strlen(dir) + strlen(filename);
	bptr = (char *) Calloc(1, len+2);
	if (*dir != '\0') {
		strcpy(bptr,dir);
		strcat(bptr,"/");
	}
	strcat(bptr,filename);
	unlink(bptr);
	Free(bptr);
}

/*
 *	I C A N R E A D  --  Test if I have read permissions on file
 *
 *	returns	1 - if yes
 *			0 - if no
 */
icanread(dirname,filename)
char	*dirname, *filename;
{
	char	buffer[100];

	if (strlen(dirname) + strlen(filename) > 98) return(0);
	sprintf(buffer,"%s/%s",dirname,filename);
	if (access(buffer,0x1) ==0)
		return(1);
	else
		return(0);
}


/*
 *	C H A N G E D I R  --  Change directory - but check if there is room
 *							to store it first, then check the chd worked
 *
 */
changedir(dir)
char *dir;
{
	int		newdirlen;
	char	newname[DIRSIZ];

	if (!dir || (*dir == '\0')) return(0);
	switch (*dir) {
	case '.':
		putprompt();
		fprintf(stderr,"Illegal directory name %s\n",dir);
		return(0);
	case '/':
		if ((newdirlen = strlen(dir)) >= DIRSIZ) {
			putprompt();
			fprintf(stderr,"Directory name is too long\n");
			return(0);
		} else {
			strcpy(newname,dir);
		}
		break;
	default:
		if ((newdirlen = (strlen(alc->dir) + strlen(dir))) >= DIRSIZ) {
			putprompt();
			fprintf(stderr,"Directory name is too long\n");
			return(0);
		} else {
			sprintf(newname,"%s/%s",alc->dir,dir);
		}
		break;
	}
#ifdef DOCHDIR
	if (chdir(newname) == 0) {
		strncpy(alc->dir,newname,DIRSIZ);
	} else {
		putprompt();
		fprintf(stderr,"Can't access directory %s\n",dir);
		return(0);
	}
#else
	strncpy(alc->dir,newname,DIRSIZ);
#endif
	return(1);
}

/*
 *	F B D O P E N  --  Open big buffered file in speciried directory
 *
 */
FILE *fbdopen(name,dir,mode,bufsiz)
char	*name, *dir, *mode;
int		bufsiz;
{
	char	*buf, *nam2;
	FILE	*fp, *fbopen();

	nam2 = (char *)Malloc(strlen(name)+1);
	fnamtranslate(nam2, name); /* Copy and translate */
#ifdef DOCHDIR
	chdir(dir);
	fp = fbopen(nam2,mode,bufsiz);
#else
	buf = (char *) Malloc(strlen(dir)+strlen(nam2)+2);
	strcpy(buf,dir);
	strcat(buf,"/");
	strcat(buf,nam2);
	fp = fbopen(buf,mode,bufsiz);
	Free(buf);
#endif
	Free(nam2);
	return(fp);
}

/*
 *	S E T D I R  --  Set directory (if given) and give dir options
 *
 */
setdir(dir)
char	*dir;
{
	char *command;
	char	workbuf[100];

	if (*dir != '\0')
		if (changedir(dir)) {
			*dir = '\0';
			return;
		}
	*dir = '\0';
	workbuf[0] = '\0';
	newmenu("Directory Operations");
	cmmd("new directory; :k30 %+s %-s %f",dir,alc->dir,changedir);
 	cmmd("show directory;:k30%-+s%f",alc->dir,show_dir);
	cmmd("show sub-directories;%-+s%f",alc->dir,show_subdir);
	cmmd("make sub-directory;:k30 %s%f",workbuf,make_subdir);
	while(thismu()) {
		switch (nncmd)  {
		case 1: *dir = '\0';break;
		}
	}
}

/*
 *	S H O W _ S U B D I R 
 */
show_subdir(dirname)
char	*dirname;
{
	int		dp, size, found, kolumn;
	struct dirent *wmem, *nptr;
	char	buffer[255];

	if (strlen(dirname) >= (255-FNAMELEN-1)) {
		screengo(ENDLINE);
		fprintf(output,"Can't cope with directory name - too long\n");
		return(0);
	}
	vclear(ENDLINE);
	screengo(1,20);
	fprintf(output,"Sub directories in %s\n",dirname);
	scrollpart(5,ENDLINE);
	screengo(5,0);

	if ((dp = open(dirname,0x81)) == -1) {
		return(0);
	}
	size = _gs_size(dp);
	if ((wmem = (struct dirent *) Malloc(size)) == NULL)
		return(0);
	read(dp,wmem,size);
	close(dp);
	wmem += 2;
	qsort (wmem, size/32-2, 32, dirsort);

	found = 0;
	kolumn = 0;
	for (nptr=wmem; nptr < wmem+(size/32)-2; nptr++) {
		sprintf(buffer,"%s/%s",dirname,nptr->dir_name);	
		if (alc->debug > 3)
			fprintf(debug_channel,"Dir check %s\n",buffer);
		strntcpy(buffer,nptr->dir_name,FNAMELEN);
		if ((access(buffer,0x81)) == 0) {
			if ((kolumn == 3) && (strlen(buffer) > 19)) {
				fprintf(output,"\n");
				kolumn = 0;
			}
			scrn_column(1+20*kolumn);
			fprintf(output,"%s",buffer);
			if (strlen(buffer) > 19) kolumn++;
			if (++kolumn > 3) {
				fprintf(output,"\n");
				kolumn = 0;
			}
			found++;
		} 
	}
	if (!found)
		fprintf(output,"No sub directories found\n");
	else
		fprintf(output,"\n");
	wmem -= 2;
	Free (wmem);
    if (go_on());
    vclear(ENDLINE);
    scrollpart(OUTLINE,ENDLINE);
    setdisplay(BUST);
}

/*
 *	M A K E _ S U B D I R
 */
make_subdir(newname)
char	*newname;
{
	if (*newname == '\0')
		return(0);
	if (fileexists(newname)) {
		screengo(ENDLINE,0);
		fprintf(output,"File %s already exists\n",newname);
		return(0);
	}
	if (mknod(newname,DIR_PERM) == -1) {
		screengo(ENDLINE,0);
		fprintf(output,"Can't create directory %s error %d\n",newname,errno);
		return(0);
	}
	return(1);
}

/*
 *	S H O W _ D I R  --  Directory of given directory.  Only
 *						shows potentially interesting files.
 *
 */
show_dir(dirname)
char	*dirname;
{
	int		dp, typ, size, nfiles;
	struct cyto_file *cfiles, *cfp;
	struct dirent *wmem, *nptr;

	vclear(ENDLINE);
	screengo(1,20);
	printf("Directory of %s\n",dirname);
	scrollpart(DIRSTART,ENDLINE);
	screengo(DIRSTART,0);

	if ((dp = open(dirname,0x81)) == -1) {
		return(0);
	}
	size = _gs_size(dp);
	if ((wmem = (struct dirent *) Malloc(size)) == NULL)
		return(0);
	cfiles = (struct cyto_file *) Calloc(size/32, sizeof(struct cyto_file));
	cfp = cfiles;
	nfiles = 0;
	read(dp,wmem,size);
	close(dp);
	qsort (wmem, size/32, 32, dirsort);
	for (nptr=wmem; nptr < wmem+(size/32); nptr++) {
		fnambacktranslate(nptr->dir_name, nptr->dir_name); /* dcb */
		if (alc->debug)
			fprintf(debug_channel,"%s/%s",dirname,nptr->dir_name);
		if (typ = iscyto(nptr->dir_name,cfp)) {
			if (alc->debug) {
				fprintf(debug_channel,"\t\ttype %d pat %s sli %s cell %d\n",typ,
					cfp->case,cfp->slide,cfp->cellno);
			}
			cfp++;
			nfiles++;
		}
	}
	cyto_dir(cfiles,nfiles,0);
	Free (wmem);
	Free (cfiles);
    vclear(ENDLINE);
    scrollpart(OUTLINE,ENDLINE);
    setdisplay(BUST);
}

/*
 *	D I R S O R T 
 */
dirsort(f1,f2)
struct dirent *f1, *f2;
{
	return(strcasecmp(f1->dir_name,f2->dir_name));
}

static char legal_chars[] = "0123456789_.ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
static char legal_first[] = "0123456789_ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
static char legal_name[]  = " !\"#$%&'()*+,-./0123456789:;<=>?@_ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
/*
 *	S E T C A S E  --  Get case name and check it is OK before setting
 *						up case - If name is junk - print message
 *
 */
setcase(new,current)
char	*new,*current;
{
	char	*temp, *check;

	if (alc->debug) {
		putprompt();
		fprintf(stdout,"Current case %s, new case %s\n",current,new);
	}
	if (!*new) {
		putprompt();
		fprintf(stdout,"Case name must be specified\n");
		return(0);
	}
	if (strlen(new) > CASELENGTH) {
		putprompt();
		fprintf(stdout,"Case name too long - ignored\n");
		return(0);
	}
	check = (char *)Malloc(strlen(new)+1);
	fnamtranslate(check, new);				/* Translate filename sep chars */
	if (index(legal_first,*check) == NULL) {
		putprompt();
		fprintf(stderr,"Illegal first char (%c) in case name\n", *check);
		return(0);
	}
	temp = check + 1;
	while (*temp) {
		if (index(legal_chars,*temp) == NULL) {
			putprompt();
			fprintf(stderr,"Illegal char (%c) in case name\n", *temp);
			return(0);
		}
		temp++;
	}
	Free(check);
	strcpy(current,new);
	return(1);
}
/*
 *	S E T N A M E  --  Get patient name and check it is OK before setting
 *						up name - If name is junk - print message
 *
 */
setname(new,current)
char	*new,*current;
{
	char	*temp;

	if (alc->debug) {
		putprompt();
		fprintf(stdout,"Current name %s, new name %s\n",current,new);
	}
	if (!*new) {			/* changed to allow removal of patient name */
		*current = NULL;
		return(1);
	}
	if (strlen(new) > PATLENGTH) {
		putprompt();
		fprintf(stdout,"Patient name too long - ignored\n");
		return(0);
	}
	temp = new;
	while (*temp) {
		if (index(legal_name,*temp) == NULL) {
			putprompt();
			fprintf(stderr,"Illegal char in patient name\n");
			*current = '\0';
			return(0);
		}
		temp++;
	}
	strcpy(current,new);
	return(1);
}
/*
 *	S E T S L I D E  --  Get slide name and check it is OK before setting
 *						up case - If name is junk - print message
 *
 */
setslide(new,current)
char	*new,*current;
{
	char	*temp;

	if (alc->debug) {
		putprompt();
		fprintf(stdout,"Current slide %s, new name %s\n",current,new);
	}
	if (strlen(new) > SLIDELENGTH) {
		putprompt();
		fprintf(stdout,"Slide name too long - ignored\n");
		return(0);
	}
	temp = new;
	while (*temp) {
		if (index(legal_chars,*temp) == NULL) {
			putprompt();
			fprintf(stderr,"Illegal char in slide name\n");
			return(0);
		}
		temp++;
	}
	strcpy(current,new);
	return(1);
}


/*
 *
 *  D I S P C E L L K M
 *
 */
static char	templ[] = "Case                  Sld      Cell      ";
dispcellkm()
{
char	strg[41];
struct	cyto_file *cfp;

		cfp = &alc->nameids;
		strncpy(strg, templ, 40);
		strncpy(&strg[5], cfp->case, strlen(cfp->case));
		strncpy(&strg[26], cfp->slide, strlen(cfp->slide));
		sprintf(&strg[36], "%d", cfp->cellno);
		bigmessage(0, 0, strg);	/* Display enlarged message at the top. */
		
}
/*
 *
 *  D I S P C E L L M F
 *
 */
dispcellmf()
{
char	strg[41];
struct	cyto_file *cfp;

		if (alc->config & CONF_TV) 
			cfp = &alc->nameids;
		else
			cfp = &alc->curbay->nameids;
		strncpy(strg, templ, 40);
		strncpy(&strg[5], cfp->case, strlen(cfp->case));
		strncpy(&strg[26], cfp->slide, strlen(cfp->slide));
		sprintf(&strg[36], "%d", cfp->cellno);
		bigmessage(0, 0, strg);	/* Display enlarged message at the top. */
		
}


/*
 *	B I T N O  --  return 2 ** (n-1)
 */
bitno(n)
int		n;
{
	return(1 << (n-1));
}

