/*
 * kycount.c		17/1/84		Jim Piper (ccount.c)
 *
 * interactively count a cell file
 *
 *  Modifications
 *
 *	 9 Jan 1991		CAS			Voids..
 *	05 Mar 1990		dcb			Remove colfn, and background colour option
 *	22 Aug 1989		CAS			Remove colour display from 'background' option
 *   2 Mar 1989		SEAJ		Added erasecursor to countit.
 *	 2 Mar 1989		CAS			Remove old MIZAR/VAX defines
 *	13 jun 1988		CAS			addplist to set up plist
 *	23 May 1988		CAS			Change colour calls
 *	 2 Nov 1987		SCG/CAS		Mod to get count right on entry
 *	25 Sep 1987		CAS			Check legal obj in 'upkount'
 *	21 Sep 1987		CAS			move explode into kcont
 *	 4 Sep 1987		CAS			Reverse calls to curscol + cursframe so
 *								we don't crash if count is the first thing we do......
 **	31 Jul 1987		jimp		Don't include noise in count !!
 *	27 Mar 1987		jimp		Chromcurs() uses mouse stopping stuff.
 *	07 Jan 1987		jimp		"chromcurs" altered to use CURRENT cursor
 *								colour - must be explicitly set:
 *								Consequent calls to "curscol", "cursframe".
 *	18 Nov 1986		jimp		Call to dispprobotypes at start of countit
 *	16 Oct 1986		CAS			Call to visitobj for select
 *	13 Oct 1986		CAS			General tidyup of colours
 *	11 Oct 1986		GJP			Change to restrict some options with an
 *								alc->devel flag
 *	 9 Oct 1986		CAS			chromcurs -> ident
 *	 8 Oct 1986		CAS			Re-vamp mouse tracking
 *	 6 Oct 1986		CAS			General tidying
 *								changed ser to kcont->current_obj
 *								so ident routines would work
 *	 2 Oct 1986		CAS			kickkcont -> capture
 *	 1 Oct 1986		SCG/CAS		Removed 'show badness' from interact
 *					CAS			Change kcont to *kcont
 *								Fixed so we can put it into kmr + mf
 *	29 Sep 1986		CAS			Moved cell reading to fileio.c
 *								Found cell reading (rpic) not used
 *	17 Sep 1986		CAS			Renamed initkcont -> cleankcont
 *   8 Aug 1986		GJP			Change umouse to mukont->umouse
 *	 7 Aug 1986		GJP			Remove the cmdspace includes
 *  11 Jun 1986		GJP			Set up count varable in allcontrol after read 
 *								all objects.
 *	22 May 1986		CAS			Store colour in alc - so it is remembered 
 *								This is a very kludgy fix
 *								Removed all colour if in demo mode
 *	20 May 1986		CAS			Changed "" to <> in Mizar includes
 *	14 May 1986		Denis		bgcolour  moved to  colour.c; menu change
 *	13 May 1986		Denis		firescale
 *	 1/9/86			Denis		Use cmdsys mouse facilities
 *
 */
#include <cmdsys.h>
#include <kyy.h>
#include <interact.h>
#include <chromanal.h>
#include <kcontrol.h>

#undef P2(a,b,c)
#define P2(a,b,c)	fprintf(output,"a b %d c %d\n",b,c)

#define MAX(i,j)	(i>j? i: j)
#define MIN(i,j)	(i<j? i: j)
#define WIDTH		8

#define BOUNDARY 0
#define HULL 1
#define RECTANGLE 2
#define EXPLODE 3

/*
 *	System data structures
 */
extern struct kcontrol *kcont;

extern FILE *debug_channel;

extern int chromcurs();		/* Cursor tracking routine */
extern int changecolour();


extern char *colours[];

static struct chromosome *tobj;
extern struct ipoint cursor;
static int dbhr[3],BDcmd;

/*
 *	C H G M O D E 
 *
 */
static
chgmode() {
	int i;
	intens ( dbhr[nncmd-BDcmd] );
	if (nncmd - BDcmd == EXPLODE ) {
		for ( i=0;i<EXPLODE;++i) {
				dbhr[i]=0; 
				displin(0,BDcmd+i);
		}
	}
	switch(nncmd-BDcmd) {
		case BOUNDARY: allbound(kcont);  break;
		case HULL : allhull(kcont); break;
		case RECTANGLE : allrect(kcont); break;
		case EXPLODE : allexplode(kcont);
	}
	
	intens(1);
}

/*
 *	A L L E X P L O D E
 *
 */
static
allexplode() {
	colour(255);
	fast();
	clear();
	slow();
	colour(GREYCOLS);
	intens(1);
	alldisplay(kcont);
	colour(OVERLAY2);
	dispexpblobs(kcont);
	countdisp(alc->numchroms,1);
}

/*
 *	U P K O U N T  --  If pointing at an object - up its count
 *
 */
static
upkount(howmuch)
{
    tsleep(1);
	if (!checkobj(kcont->current_obj)) return(0);
	modccount(kcont->current_obj,kcont->ool,kcont,howmuch);
}

/*
 *	C H R O M C U R S  --  General cursor tracking routine
 *
 *	Assumes that you have a menu on screen and that line 1 contains
 *	the select chrom/object option.
 *
 *	Now called EITHER if mouse moved OR if mouse stopping has
 *	just been detected.  Only identify object in latter case.
 *
 */
chromcurs() {
	int		newobj;

	if (alc->m2s->moving) {
		if (alc->debug) fprintf(stderr, "Mouse moving\n");
		tmousecurs();
	} else {
		newobj = objident(kcont->ool,kcont,&cursor);	/* is cursor over an object ? */
		if (kcont->current_obj != newobj) {
			kcont->current_obj = newobj;
			markident(kcont->current_obj,kcont->ool,kcont,1);
			displin(0,1);
			putprompt();
		}
	}
}

#ifdef UNUSED
/*
 *	N E W C D I S P
 *
 */
static
newcdisp(kcont)
struct kcontrol *kcont;
{

	/* set picture output */
		llut();
		fast();
		clear();
		slow();
		lutfrig();
		colour(GREYCOLS);

		alldisplay(kcont);
		colour(OVERLAY2);
		dispexpblobs(kcont);
}
#endif UNUSED

/*
 *	C O U N T A L L  --
 *
 */
countall(kcont,input)
register struct kcontrol *kcont;
FILE	*input;
{
	int c, objnum=0, oldnum = -1;
	struct chromplist *plist;
	struct pframe *frm;

	frm = kcont->f;
	setmsv(0);
	newmenu("countall");
		cmd(1,"raise count"); mouse (LEFTBT);
		cmd(2,"next object"); mouse (RIGHTBT);
		cmd(3,"lower count"); mouse (MIDBT);
		cmd(4,"previous object"); mouse (LONGLB);
	for(;;) {
		fprintf(stderr,"In for loop");
		while (! kcont->acl[objnum])
			if (++objnum >= kcont->maxnumber) {
				fprintf(stderr,"Before markident delete");
				markident(kcont->current_obj,kcont->ool,kcont,0);	/* CAS TEST */
				return(0);
			}
		if ( objnum != oldnum ) {
			tobj = kcont->ool[objnum];
			plist = tobj->plist;
			kcont->current_obj = objnum;
			fprintf(stderr,"Before markident");
			markident(kcont->current_obj,kcont->ool,kcont,1);	/* CAS TEST */
			oldnum = objnum;
		}
		switch(thismu()) {
			case 0:
				curscol(OVERLAY2);
				setmsv(chromcurs);
    			return(0);
			case 1:
				modccount(kcont->current_obj,kcont->ool,kcont,1);
				continue;
			case 2:
				if (++objnum >= kcont->maxnumber)
					return(0);
				break;	
			case 3:
				modccount(kcont->current_obj,kcont->ool,kcont,-1);
				continue;
			case 4:
				if (objnum > 0)
					objnum-=1;
				break;
		}
	}
}

#ifdef UNUSED
/*
 *	C O L F N -- Here to avoid the fight to get cmdsys to pass parameters
 *				the way one would want..
 */
static
colfn()
{
	changecolour(&alc->metacolour_1,&alc->metacolour_2,0);
}
#endif

/*
 *	C O U N T I T
 *
 */
countit(gotcell)
{
	FILE *in;
	register int c;
	int		Sel_cmd;
	int	(*linkerbug)();
 	register struct chromplist *plist;

	dbhr[BOUNDARY] = 0 ;
	dbhr[HULL] = 0 ;
	dbhr[RECTANGLE] = 0 ;

	addplist(kcont);				/* set up plists if not yet done */
	dispprobotypes(kcont);	
	if (gotcell) {
		/*
		 * count chromosomes, possible composites, and display
		 */
		for (alc->numchroms=0, c=0; c<kcont->maxnumber; c++) {
 			plist = kcont->ool[c]->plist;
 			if (plist->otype == CHROMOSOME)
 				plist->history[2] = 1;
 			else if (plist->otype == COMPOSITE && plist->history[2] == 0)
 				plist->history[2] = 1;
 			else if (plist->otype == NOISE || plist->otype == NUCLEUS)
 				plist->history[2] = 0;
 			alc->numchroms += plist->history[2];
 		}			
		countdisp(alc->numchroms,1);
		initident(kcont,&cursor,&mukont->umouse);
	} else if (alc->debug)
		fprintf(stderr,"No cell in interact mu\n");
	cursframe(0);
	curscol(OVERLAY2);
	setmsv(chromcurs);
	newmenu("interact");
		cmmd ("select object;%d",&kcont->current_obj);Sel_cmd = mmcmd;
		cmmd ("add to count;%g%+i",upkount,1);mouse(LEFTBT);
		cmmd ("decrease count;%g%+i",upkount,-1);mouse(MIDBT);
		cmmd ("boundaries;%t%g",&dbhr[BOUNDARY],chgmode);
		BDcmd = mmcmd;
		cmmd ("hulls;%t%g",&dbhr[HULL],chgmode);
		cmmd ("rectangles;%t%g",&dbhr[RECTANGLE],chgmode);
		if (alc->devel) {
			cmmd ("one-by-one;%g%+i%+i",countall,kcont,input); /* should return */
			cmmd ("explode;%t%g",&kcont->explode,chgmode);
		}
#ifdef UNUSED
		cmmd ("background colour;%f",colfn);
#endif
	if (alc->debug)
		fprintf(debug_channel,"Before thismu\n");
	while (thismu())
		if (nncmd == Sel_cmd)
			visitobj(kcont->current_obj,kcont->ool,kcont,&cursor);
	if (gotcell) {
		endident(kcont->ool,kcont,&cursor,&mukont->umouse);
	}
	setmsv(0);
	countdisp(0,0);			/* Un-display count */
	erasecursor();			/* erase the cursor */
}

