/*
 *		L L U T . C  --  Lookup table stuff
 *
 *	Modifications
 *
 *	05 Mar 1990		dcb		Remove fixrgb to kyy/mf/metlist
 *	19 May 1988		CAS		General fiddling + new simplehue routine
 *	20 Jul 1987		CAS		Removed the setrgb(8,0,0,255)
 *
this is the scheme used to determine a 'contrasting colour' -
if an (r,g,b) is closest to one vertex of the cube, the most contrasty
is taken to be the opposite vertex, and the next most those opposite
any diagonally opposite one on the same face 

                                Blue

                                |
                                |
                                4-----------6
                               /|          /|
                             /  |        /  |
                            5 ----------7   |
    corner number =         |   0-------|---2---- Green
 Red + 2*Green + 4*Blue     |  /        |  /
                            | /         | /
                            |/          |/
                            1-----------3
                           /
                          /
                         Red
*/
#define S 32768

/* corner furthest away from
                       0   1   2   3   4   5   6   7  
				       -   R   G   RG  B   RB  BG  RGB     */
static int srr[] = { 255,  0,255,  0,255,  0,255,  0}; 
static int sg[] =  { 255,255,  0,  0,255,255,  0,255};/* last entry - ???*/
static int sb[] =  { 255,255,255,255,  0,  0,  0,  0};
/* corners diagonally (on same face) away from
                       0   1   2   3   4   5   6   7      */

static int tr1[] = { 255,  0,255,255,255,255,255,255}; 
static int tg1[] = {   0,255,  0,  0,  0,255,  0,  0};
static int tb1[] = { 255,  0,  0,255,  0,  0,255,  0};

static int tr2[] = {   0,  0,  0,  0,  0,  0,255,  0};
static int tg2[] = { 255,  0,  0,255,255,255,  0,  0};
static int tb2[] = { 255,255,255,255,  0,255,255,255};

/*
 *	L I N E A R H U E  --
 *
 */
linearhue(p,r,g,b,m)
short *p ; /* division points */
short *r ; /* red values */
short *g ; /* green values */
short *b ; /* blue values */
int m  ; /* no. of reserved planes */
{
	int i = 0 ; /* table load point */
	int pdiff,rdiff,gdiff,bdiff;
	int pinv;
	int rv,gv,bv;	/* scaled-up r,g,b */
	int ko;
	int power2;
	int dn;	/* reduced intensity offset whn surround is bright */

	switch( m) {
		case 0: power2 = 1; break;
		case 1: power2 = 2; break;
		default: power2 = 4; break;
	}

	while (*p != 256 ) {
		pdiff = *(p+1) - *p;
		if (!pdiff) continue;
		pinv = (S*power2)/pdiff;
		rdiff = (*(r+1) - *r ) * pinv;
		gdiff = (*(g+1) - *g ) * pinv;
		bdiff = (*(b+1) - *b ) * pinv;
		rv = S*(*r) ;
		gv = S*(*g) ;
		bv = S*(*b) ;
		while (pdiff-- > 0  && (i < 255)) {
			register rvn,gvn,bvn,ko;
			rvn = rv/S;
			gvn = gv/S;
			bvn = bv/S;
			if (rvn < 0) rvn =0; if (rvn >255) rvn=255;
			if (gvn < 0) gvn =0; if (gvn >255) gvn=255;
			if (bvn < 0) bvn =0; if (bvn >255) bvn=255;
			if (i) setrgb(i,rvn,gvn,bvn);		/* Leave background alone */
			i++;
			if (m) {
				ko=(rvn>127)+2*(gvn>127)+4*(bvn>127);
				dn = rvn+gvn+bvn > 384 ? 64 : 0;
				presetrgb(i++,srr[ko],sg[ko],sb[ko],dn);
				--pdiff;
				if (m==2) {
				    presetrgb(i++,tr1[ko],tg1[ko],tb1[ko],dn);
				    presetrgb(i++,tr2[ko],tg2[ko],tb2[ko],dn);
				    pdiff -= 2;
				}
			}
			rv += rdiff;
			gv += gdiff;
			bv += bdiff;
		}
 		++r; ++g; ++b; ++p;
	}
}

/*
 *	S I M P L E H U E  --  Dont bother with a the clever contrast stuff
 */
simplehue(p,r,g,b,m)
short *p ; /* division points */
short *r ; /* red values */
short *g ; /* green values */
short *b ; /* blue values */
int m  ; /* no. of reserved planes */
{
	int i = 0 ; /* table load point */
	int pdiff,rdiff,gdiff,bdiff;
	int pinv;
	int rv,gv,bv;	/* scaled-up r,g,b */
	int power2;

	power2 = 1 << m;
	while (*p != 256 ) {
		pdiff = *(p+1) - *p;
		if (!pdiff) continue;
		pinv = (S*power2)/pdiff;
		rdiff = (*(r+1) - *r ) * pinv;
		gdiff = (*(g+1) - *g ) * pinv;
		bdiff = (*(b+1) - *b ) * pinv;
		rv = S*(*r) ;
		gv = S*(*g) ;
		bv = S*(*b) ;
		while (pdiff > 0 && i < 255 ) {
			register rvn,gvn,bvn,ko;
			rvn = rv/S;
			gvn = gv/S;
			bvn = bv/S;
			if (rvn < 0) rvn =0; if (rvn >255) rvn=255;
			if (gvn < 0) gvn =0; if (gvn >255) gvn=255;
			if (bvn < 0) bvn =0; if (bvn >255) bvn=255;
			if (i) setrgb(i,rvn,gvn,bvn);		/* leave background alone */
			rv += rdiff;
			gv += gdiff;
			bv += bdiff;
			i += power2;
			pdiff -= power2;
		}
 		++r; ++g; ++b; ++p;
	}
}

presetrgb(i,r,g,b,down) register int r,g,b,down; {
	register two=2;
	if (down) {
		if ( (r != 0 ) + (g != 0) + ( b!=0 ) >1 ) two = 4;
		if (r) r /= two;
		if (g) g /= two;
		if (b) b /= two;
	}
	setrgb(i,r,g,b);
	
}
