/*
 *		N E W D E N T I F Y . C  --
 *
 *
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *  Based on:newdintify.c
 *           Denis Rutovitz, June/July 1985
 *           MRC CAPCU
 *           Edinburgh
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *  Modifications
 *
 *      28 Jan 1992	Joanne		data for classifier and layout are read
 *					from file rather than from data structure,
 *					so set_def_vcv not called anymore.
 *	15 Jan 1991		CAS			debug_channel used when not open
 *	02 Apr 1990		dcb			call set_def_vcv in int_cultures instead of
 *								set_def. assign alc->tissues[0], ->stains[0]
 *								in init_cultures. Delete set_def completely
 *								All this to prevent cross process memory problem
 *	20 Mar 1990		dcb			Initialise newtis, newstn in tisstmu
 *	12 Jun 1989		dcb			Don't reset doshade in tisstmu if shading not done
 *	 1 Jun 1989		CAS			Change environ handling
 *	30 May 1989		dcb			Debug in set_def_box/lin/vcv
 *	19 May 1989		dcb			Density range check altered, and sets doshade
 *								exit tisstmu by return
 *	17 May 1989		dcb			mouse buttons in moretissues/morestains
 *	15 May 1989		dcb/GJP		call initclass from init_cultures
 *	15 May 1989		dcb/SEAJ	set_def_box/lin/vcv as separate functions
 *								fix bug in findentry.
 *	12 May 1989		dcb/SEAJ	Change params to findentry (takes cyto_types) and
 *								unstatic set_def, freelin, alloclin.
 *	10 May 1989		dcb			More tissues/stains etc.
 *	 5 May 1989		dcb			New tissues/stains menu etc. Bring tisstmu
 *								here static fn's where poss. Remove showkhead
 *								and idinfo (GGG)
 *	 1 Mar 1989		CAS			Remove MIZAR/VAX stuff
 *	27 Feb 1989		SCG			rectified classifier assignments
 *	23 Jan 1989		SCG/dcb		rectified tissues/stains to include defaults
 *	16 Jan 1989		dcb			kickid initialises lastname field as well
 *	08 Sep 1988		SCG			mod in freevcv
 *	19 Aug 1988		SCG			Introduced changes for switchalbe vcv classifiers
 *	27 Jun 1988		SCG			gettissue, getstain, settissue, setstain,
 *								knowntissue, knownstain moved to system.c
 *	10 May 1988		SCG			Initialise rankcf from classifier train file
 *	23 Mar 1988		CAS			Tidy set/get tissue/stain code
 *	17 Jan 1988		CAS			Remove chdir to dd/mky..
 *	
 */
#include <cmdsys.h>
#include <chromanal.h>
#include <slides.h>
#include <kyy.h>
#include <kcontrol.h>
#include <metscan.h>

/*
 * defines
 */
#define OPERATOR_FILE "/dd/sys/operators"
#define MAXOPT 14
#define DISPTOTAL 12
#define DISPHALF 6
#define DEF_CHR_NUM 24
#define MIN_TIS_NUM 50
#define MIN_STN_NUM 6
#define CBUFLEN 256

extern FILE *debug_channel;
extern struct allkontr *alc;

static char ourop[14];

/*
 * default linear classifier
 */
static int larr[DEFLINFEATS][5] = {
	{2, 0, 0, 1, COUNTPR},
	{3, 0, 0, 1, SFPR},
	{7, 0, 0, 1, DVPR},
	{6, 0, 0, 1, SKODPR}
};

/*
 * default box classifier
 */
static int barr[DEFBOXFEATS][3] = {
	{0, XMIN, XMAX},
	{1, YMIN, YMAX},
	{2, COUNTMIN, COUNTMAX},
	{3, SFMIN, SFMAX},
	{4, CAREAMIN, CAREAMAX},
	{5, OCTAREAMIN, OCTAREAMAX},
	{6, SKODMIN, SKODMAX},
	{7, DVMIN, DVMAX},
	{8, CODMIN, CODMAX}
};	

/*
 * default vcv classifier
 */



static int featids[DEFVCVFEATS] = {
	1, 3, 9, 10, 11, 12, 13, 17, 19, 21
};


static int vcvfeatm[DEFVCVFEATS][MAXCLASS - 1] = {
    {292, 275, 177, 149, 123, 109, 66, 28, 12, 4, -1, -3, -54, -76, -78, -114, -130, -123, -194, -167, -218, -208, 58, -181},
    {201, 134, 159, -2, 6, 54, 141, 50, 133, 55, 173, 52, -89, -96, -76, 98, 52, -31, 55, 199, -122, -36, 120, -95},
    {-88, -128, -83, -55, -36, -19, -105, -21, -120, -16, -140, -9, 57, 84, 92, 55, -47, 66, 146, 8, 178, 109, -80, 104},
    {-113, 14, -80, -14, 19, 33, 31, 68, 81, -1, 48, 120, 45, -107, -102, 98, -17, 23, -85, -70, 7, -67, 55, 27},
    {103, 52, -46, -63, -96, -57, 44, -17, 101, -22, 232, 35, 0, -52, -100, -41, 133, -68, -68,  82, -107, -14, 15,-31},
    {-126, -50, -20, -1, -48, 103, -17, 49, -8, 31, 22, -22, 65, 23, -46, 35, 67, 102, -63, -76, 25, -56, -72, 82},
    {-80, 3, 141, 37, 40, 14, 69, 93, 76, 113, -53, -95, -92, -18, 8, 69, -10, 70, -96, -33, -99, -94, 83, -69},
    {91, 11, 4, -132, -149, -57, 29, -47, 64, -51, 136, -37, -31, -72, -92, 67, 48, -54, 64, 167, 16, 55, 6, -6},
    {-185, -36, -10, -2, 12, 26, 52, 76, 39, 85, -12, -4, -113, -62, -57, 107, 53, 56, 26, 38, -19, 15, 52, -27},
    {151, 2, -47, -119, -61, -59, -1, -35, -13, 8, 35, 75, -20, 23, 39, 76, -69, -5, 65, -128, 82, -45, -5, -40}
};

static int vcvfeatsd[DEFVCVFEATS][MAXCLASS - 1] = {
    {1000, 755, 941, 607, 510, 1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000},
    {295, 523, 150, 946, 1000, 176, 554, 336, 467, 296, 230, 88, 49, 46, 74, 125, 57, 121, 27, 49, 34, 26, 809, 47},
    {946, 1000, 1000, 1000, 554, 671, 442, 398, 146, 566, 167, 268, 276, 408, 263, 84, 64, 211, 169, 74, 78, 106, 700, 204},
    {263, 432, 602, 404, 259, 343, 252, 188, 117, 152, 87, 73, 55, 55, 53, 71, 44, 61, 24, 26, 21, 27, 299, 83},
    {722, 579, 310, 542, 336, 633, 373, 393, 120, 339, 119, 90, 276, 124, 152, 41, 68, 132, 97, 46, 62, 71, 464, 194},
    {270, 309, 227, 266, 120, 130, 185, 150, 125, 116, 85, 72, 58, 64, 65, 70, 27, 52, 38, 20, 24, 21, 242, 52},
    {368, 471, 128, 330, 153, 341, 182, 173, 124, 129, 61, 53, 174, 67, 89, 51, 48, 55, 116, 38, 62, 62, 308, 122},
    {351, 305, 321, 244, 196, 217, 138, 136, 111, 137, 89, 75, 132, 104, 130, 68, 68, 103, 128, 63, 86, 91, 222, 131},
    {317, 200, 138, 186, 120, 199, 149, 121, 90, 87, 90, 59, 90, 67, 70, 57, 51, 47, 68, 33, 29, 39, 215, 73},
    {356, 445, 274, 245, 186, 238, 151, 173, 126, 105, 151, 78, 68, 80, 52, 34, 71, 53, 42, 22, 17, 29, 276, 72}
};

static double logsqrtdet[MAXCLASS - 1] = {
	-4.17364e+01, -3.84634e+01, -4.09258e+01, -3.73258e+01, -3.78843e+01,
	-3.92105e+01, -3.79475e+01, -3.78642e+01, -4.02520e+01, -3.85448e+01,
	-3.93441e+01, -3.97798e+01, -3.86768e+01, -3.95346e+01, -3.94818e+01,
	-4.08774e+01, -4.05505e+01, -3.85535e+01, -4.34407e+01, -4.04939e+01,
	-4.18571e+01, -4.21680e+01, -3.79024e+01, -3.92196e+01
};

static double scalef[MAXCLASS - 1] = {
	1.80785e+06, 1.00025e+06, 1.12641e+06, 7.09317e+05, 5.24467e+05,
	8.17324e+05, 5.42356e+05, 4.73699e+05, 5.28679e+05, 4.67304e+05,
	3.60817e+05, 3.06241e+05, 3.03608e+05, 3.03883e+05, 3.10881e+05,
	2.92945e+05, 2.37440e+05, 2.32728e+05, 4.96306e+05, 1.72364e+05,
	2.34341e+05, 2.80429e+05, 7.60205e+05, 3.08781e+05
};



extern int setdir();
extern char *getenv();
extern int	setcase();		/* check entered case name (length, chars etc) */
extern int	setname();		/* check entered patient name (length, chars etc) */
extern int	setslide();		/* check entered slide name */
/*extern int	tisstmu();*/		/* Tissue/stain options menu */
/*
 * Tissue/stain manipulating functions
 */
/*extern*/ char	*gettissue();
/*extern*/ char	*getstain();
/*extern*/ int settissue();
/*extern*/ int setstain();
/*extern*/ int knowntissue();
/*extern*/ int knownstain();
int gettissueindex();
int getstainindex();

int	getopt();
extern int changedir();

static char *skipword();
extern char *Calloc(), *Malloc();
static char *Realloc();		/* Defined here so rip out later */
static char *get_next_three();

/*
 *	K I C K I D
 *
 */
kickid() 
{
	int	i;
	struct	mult_slide *bp;

	init_cultures(alc->culturesfile);
 	settissue(0, &alc->typeids);
 	setstain(0, &alc->typeids);
 	bp = alc->frsbay;
 	for(i = 0; i < alc->maxbays; i++) {
	 	settissue(0, &bp->typeids);
 		setstain(0, &bp->typeids);
		strcpy(bp->nameids.case,"UNSET");
		*bp->nameids.slide = 0;
 		bp++;
 	}
	strcpy(alc->nameids.case,"UNSET");
	if (alc->lastname) strcpy(alc->lastname->case,"UNSET"); /* May remove later */
	*alc->nameids.slide = 0;
}

/*
 *	S K I P W O R D --
 *
 */
static char *
skipword(ptr)
char *ptr;
{
	while (*ptr && (*ptr != '\t') && (*ptr != ' '))
		++ptr;
	while ((*ptr == '\t') || (*ptr == ' '))
		++ptr;
	return(ptr);
}


#ifdef GGG
/*
 *	S H O W K H E A D  --
 *
 */
static int
showkhead()
{
	int line;
	line =  17;
	
	clearlns(line,line+1);
	screengo(line,10);
	fprintf (output,"case ID: %s", alc->nameids.case);
	screengo (line,39);
	fprintf (output,"%s", alc->typeids.ktissue);
	screengo (line+1,10);
	fprintf (output,"slide ID: %s", alc->nameids.slide);
	screengo(line+1,39);
	fprintf (output,"%s", alc->typeids.kfstain);
	screengo(line+1,60);
	fprintf (output,"%s", alc->mydate);
	screengo(outline,2);
}

/*
 *	I D I N F O
 *
 */
idinfo()
{
	int i,j,nti,nst,nste, Cl_cmd, Tscmd;
	char	dirbuf[40];
	char	workbuf[41];
	int		(*linkerbug)(), (*linkerbug2)();
	struct	cyto_file *cfp;
	struct	cyto_types *ctp;
	
	dirbuf[0] = '\0'; workbuf[0] = '\0';
	cfp = &alc->nameids;
	ctp = &alc->typeids;
	newmenu("Idinfo");
	linkerbug = setcase;
	cmmd ("case ID; :k38 %+s%-17s%f;t2",workbuf,cfp->case,linkerbug);
	linkerbug = setslide;
	cmmd ("slide ID; :k38 %+s%-5s%f",workbuf,cfp->slide,linkerbug);
	cmmd ("cell number; :k38 %d",&cfp->cellno); Cl_cmd = mmcmd;
	linkerbug = setname;
	cmmd ("patient name; :k38 %+S%-17s%f",workbuf,cfp->patnam,linkerbug);
	cmmd ("tissue/stain options"); Tscmd = mmcmd;
		linkerbug = setdir;
	cmmd ("set working directory;%?s:k38%-s%f",dirbuf,alc->dir,linkerbug);
		linkerbug = showkhead; linkerbug2 = getopt;
	cmmd ("operator;:k38%-s%-f%+i%g",ourop,linkerbug,0,linkerbug2);
	
	while (thismu() ) {
	    if(nncmd == Cl_cmd) {
		        displin(0,1);
		       	displin(0,2);
				displin(0,3);
				displin(0,Cl_cmd);
				displin(0,0);			/* And do header */
		}
		if(nncmd == Tscmd) {			/* Tissue/stain options menu */
			downmenu();
			tisstmu(ctp);
			upmenu();
		}
	    putprompt();
      showkhead();
	}
	return ;
}
#endif

/*
 *	G E T O P T  --  Get operator
 *
 *	If environ set - look in environment first
 */
getopt(environ)
int	environ;
{
	FILE *opfile;
	char opnms[154],*oplist[MAXOPT],*opnm;
	int cc;
	int i,opnum = 0;

	if (opnm = getenv("USER")) {
		strncpy (ourop,opnm,13);
		ourop[13] = '\0';
		strncpy (alc->kscorer,opnm,SCORERLENGTH);
		alc->kscorer[SCORERLENGTH-1]='\0';
		strncpy(alc->user,opnm,OPERLENGTH);
		if (alc->debug)
			fprintf(debug_channel,"OPER %d %s\n",opnm,opnm);
		return;
	}
	opnm = opnms;
	oplist[0] = opnm;
	opfile = fopen(OPERATOR_FILE,"r");
	if ( !opfile ) goto endfil1;
	while(1) {
		switch ( cc = getc(opfile)) {
			default:	*opnm++ = cc; break;
			case EOF:	goto endfil;
			case '\n':  if ( oplist[opnum] < opnm ) {
							*opnm++ =0;
							oplist[++opnum] = opnm;
							break;
						}
		}
		if ( opnum > MAXOPT) {
			fprintf(stderr,"Too many op. names\n");
			return;
		}
		
	}
endfil:	fclose(opfile);
endfil1:
	if (alc->debug) fprintf(debug_channel,"getopt %10s\n", ourop);
	*ourop = 0;
	newmenu( "Name, please: ");
	for ( i = 0; i < opnum ; ++i ) cmmd(oplist[i]);
	cmmd ( "new operator;%10s",ourop);
	while (thismu() <0 );
	if (nncmd) {
		if (nncmd <= opnum) {
			strcpy (ourop,oplist[nncmd-1]);
		} else {
		fprintf(output,"File name ?");
		if (!yesno(0)) {
			strcpy (opnm,ourop);
			++ opnum;
			
			if(opnum > MAXOPT) {
		fprintf(output,"Too many operator names on file. Delete ?");
		if (yesno()) { fprintf( output, "Which number ?");
			   if (thismu()) 
				for (i=nncmd; i < opnum; ++i)
						oplist[i]=oplist[i+1];
			   }
			}
				
			opfile = fopen(OPERATOR_FILE,"w");
			if (opfile) {
				for ( i = 0; i < opnum; ++i ) 
					fprintf(opfile,"%s\n",oplist[i]);
				fprintf(output,
				   "%s added to file 'operators'\n",ourop);
				sleep(2);
				fclose(opfile);
			}
		}		
	}
}			
		strncpy (alc->kscorer,ourop,SCORERLENGTH);
		alc->kscorer[SCORERLENGTH-1]=0;
		strncpy(alc->user,ourop,OPERLENGTH);
		alc->user[OPERLENGTH-1]=0;
}	
	
static char *month[] = {"January", "February", "March", "April",
					"May", "June", "July", "August", "September",
					"October", "November", "December"};

/*
 *	D O D A T E  --  get date and put into alc
 *
 */
dodate()
{
	int		time,date,day,tick;

	_sysdate(0,&time,&date,&day,&tick);
	alc->day = date & 255;
	alc->month = (date>>8) & 255;
	alc->year = ((date>>16)&65535) - 1900;

	if ((alc->month>0 ) && (alc->month<=12))
		sprintf (alc->mydate,"%d %s %d",alc->day,month[alc->month-1],1900+alc->year);
}

/*
 * S E T _ D E F _ B O X  --
 */
int
set_def_box()
{
int i;
BOXHEAD *bhd;
	if (alc->debug)
		fprintf(debug_channel, "set_def_box:\n");
	alc->rankcf->bhd = freebox(alc->rankcf->bhd);
	bhd = allocbox(DEFBOXFEATS);
	/*
	 * Initialise box classifier
	 */
	for(i = 0; i < DEFBOXFEATS; i++)
		_strass(bhd->bfd[i], barr[i], sizeof(struct boxfdisc));
	bhd->tis = 0;
	bhd->stn = 0;
	bhd->drange = DEFDRANGE;
	bhd->nfeats = DEFBOXFEATS;
 	alc->rankcf->bhd = bhd;
}

/*
 * S E T _ D E F _ L I N  --
 */
int
set_def_lin()
{
int i;
LINHEAD *lhd;

	if (alc->debug)
		fprintf(debug_channel, "set_def_lin:\n");
	alc->rankcf->lhd = freelin(alc->rankcf->lhd);
	lhd = alloclin(DEFLINFEATS);
	/*
	 * Initialise linear classifier
	 */
	for(i = 0; i < DEFLINFEATS; i++) {
		lhd->lfd[i]->feat1 = larr[i][0];
		lhd->lfd[i]->feat2 = larr[i][1];
		lhd->lfd[i]->feat3 = larr[i][2];
		lhd->lfd[i]->aflag = larr[i][3];
		lhd->lfd[i]->const = (double) larr[i][4];
	}
	lhd->tis = 0;
	lhd->stn = 0;
	lhd->drange = DEFDRANGE;
	lhd->nfeats = DEFLINFEATS;

 	alc->rankcf->lhd = lhd;
}

/*
 * S E T _ D E F _ V C V  --
 */
int
set_def_vcv()
{
int i, j;
VCVHEAD *vhd;
VCVTYPE *vtp;

	if (alc->debug)
		fprintf(debug_channel, "set_def_vcv:\n");
	alc->rankcf->vhd = freevcv(alc->rankcf->vhd); /* This frees the whole lot */
	vhd = allocvcv();
	vtp = allocvcvtype(DEFVCVFEATS, DEF_NUM_CHR);

	/*
	 * Initialise vcv classifier
	 */
	_strass(vtp->scalef, scalef, sizeof(double) * DEF_NUM_CHR);
	_strass(vtp->logsqrtdet, logsqrtdet, sizeof(double) * DEF_NUM_CHR);
	for(i = 0; i < DEFVCVFEATS; i++) {
		vtp->vfd[i]->feat = featids[i];
		_strass(vtp->vfd[i]->featmean, vcvfeatm[i], sizeof(int) * DEF_NUM_CHR);
		_strass(vtp->vfd[i]->featsd, vcvfeatsd[i], sizeof(int) * DEF_NUM_CHR);
	}
	vtp->nclasses = DEF_NUM_CHR;
	vtp->nfeats = DEFVCVFEATS;
 	vhd->vtp[0] = vtp;
	vhd->tis = 0;
	vhd->stn = 0;
	vhd->celltype = 0;

 	alc->rankcf->vhd = vhd;
}


/*
 * A L L O C B O X ---
 *
 */
static BOXHEAD *
allocbox(nfeats)
int nfeats;
{
BOXHEAD *bhd;
struct boxfdisc *bfdd;
int i;
	
	bhd = (BOXHEAD *) Calloc(1, sizeof(BOXHEAD));
	bhd->bfd = (struct boxfdisc **) Calloc(nfeats, sizeof(struct boxfdisc *));
	bfdd = (struct boxfdisc *) Calloc(nfeats, sizeof(struct boxfdisc));
		
	bhd->nfeats = nfeats;
	for(i = 0; i < nfeats; i++)
		bhd->bfd[i] = bfdd++;

	return(bhd);
}

/*
 * A L L O C L I N ---
 *
 */
LINHEAD *
alloclin(nfeats)
int nfeats;
{
LINHEAD *lhd;
struct linfdisc *lfdd;
int i;
	
 	lhd = (LINHEAD *) Calloc(1, sizeof(LINHEAD));
	lhd->lfd = (struct linfdisc **) Calloc(nfeats, sizeof(struct linfdisc *));
	lfdd = (struct linfdisc *) Calloc(nfeats, sizeof(struct linfdisc));
	
	lhd->nfeats = nfeats;	
	for(i = 0; i < nfeats; i++)
		lhd->lfd[i] = lfdd++;

	return(lhd);
}

/*
 * A L L O C V C V T Y P E ---
 *
 */
VCVTYPE *
allocvcvtype(nfeats, nclasses)
int  nclasses, nfeats;
{
VCVTYPE *vtp;
struct vcvfdisc **vfd, *vfdd;
double *fpt;
int i, *ipt;

	if(!nfeats || !nclasses)
		return(NULL);
	vtp = (VCVTYPE *) Calloc(1, sizeof(VCVTYPE));
	fpt = (double *) Calloc(1, sizeof(double) * nclasses * 2);
	ipt = (int *) Calloc(1, sizeof(int) * nclasses * nfeats * 2);
	vfdd = (struct vcvfdisc *) Calloc(nfeats, sizeof(struct vcvfdisc));
		
	vtp->vfd = (struct vcvfdisc **) Calloc(nfeats, sizeof(struct vcvfdisc *));
	
	vtp->nfeats = nfeats;
	vtp->nclasses = nclasses;
	for(i = 0; i < nfeats; i++, ipt += 2 * nclasses) {
		vtp->vfd[i] = vfdd++;
		vtp->vfd[i]->featmean = ipt;
		vtp->vfd[i]->featsd = ipt + nclasses; 
	}
	vtp->logsqrtdet = fpt;
	vtp->scalef = fpt + nclasses;
	return(vtp);
}

/*
 * A L L O C V C V ---
 *
 */
VCVHEAD *
allocvcv()
{
VCVHEAD *vhd;
	
	return(vhd = (VCVHEAD *) Calloc(1, sizeof(VCVHEAD)));
}


/*
 * F R E E V C V ---
 *
 */
static VCVHEAD *
freevcv(vhd)
VCVHEAD *vhd;
{
	if(vhd != NULL) {
		if (vhd->vtp[0] != NULL) {
			Free(vhd->vtp[0]->logsqrtdet);
			Free(vhd->vtp[0]->vfd[0]->featmean);
			Free(vhd->vtp[0]->vfd[0]);
			Free(vhd->vtp[0]->vfd);
			Free(vhd->vtp[0]);
		}
		Free(vhd);
	}
	return(NULL);
}

/*
 * F R E E L I N ---
 *
 */
LINHEAD *
freelin(lhd)
LINHEAD *lhd;
{
	if(lhd != NULL) {
		Free(lhd->lfd[0]);
		Free(lhd->lfd);
		Free(lhd);
	}
	return(NULL);
}

/*
 * F R E E B O X ---
 *
 */
static BOXHEAD *
freebox(bhd)
BOXHEAD *bhd;
{

	if(bhd != NULL) {
		Free(bhd->bfd[0]);
		Free(bhd->bfd);
		Free(bhd);
	}
	return(NULL);
}



/*
 *	T I S S T M U  ---  Menu for tissues and stains
 */
tisstmu(ctp)
struct	cyto_types *ctp;
{
int  old_drange, Ecmd, Tcmd, Scmd, changetissue(), changestain();
char newtis[TISSUELENGTH], newstn[STAINLENGTH], currtis[TISSUELENGTH],
	currstn[STAINLENGTH];

	if(alc->num_tissues == 0 && alc->num_stains == 0) {
		fprintf(stdout, "No tissue or stain types supplied\n");
		return;
	}

	strcpy(currtis, gettissue(ctp->ktistype));	
	strcpy(currstn, getstain(ctp->kfstype));	
	old_drange = alc->curbay->drange;
	newtis[0] = '\0';
	newstn[0] = '\0';

	newmenu("Tissue/stain options");
	cmmd("tissue ;%?25+s%-s%f", newtis, currtis, changetissue); mouse(MIDBT);
		Tcmd = mmcmd;
	cmmd("stain ;%?25+s%-s%f", newstn, currstn, changestain); mouse(RIGHTBT);
		Scmd = mmcmd;
	if (alc->config & CONF_FIPAL)
		cmmd("density range (0/6);:k40<0/6>%d",&alc->curbay->drange);
	else
		cmmd("density range (0/3);:k40<0/3>%d",&alc->curbay->drange);
	cmmd("exit <Return> "); mouse(LONGLB);
		Ecmd = mmcmd;
		
	while(thismu() && (nncmd != Ecmd)) {
		if(nncmd == Tcmd)
			ctp->ktistype = gettissueindex(currtis);
		if(nncmd == Scmd)
			ctp->kfstype = getstainindex(currstn);
	}
	if (alc->curbay->drange != old_drange) {
		doshade = -1;
	}
}

/*
 * M O R E T I S S U E S --- give menu of available tissues
 *
 */
int
moretissues() 
{
int left = alc->num_tissues, i, j = 0, more, top, Mcmd, Ecmd;
char cmdstr[DISPHALF][TISSUELENGTH + 8];
	
	do {
		downmenu();
		newmenu("Known tissues");
		top = (left > DISPTOTAL) ? DISPTOTAL: left;
		more = (top < DISPTOTAL) ? 0: 1;
		for(i = 0; i < top; i++)
			if(i > DISPHALF) {
				sprintf(cmdstr[i - (DISPHALF + 1)], ":L-7k40;%s", alc->tissues[j+i].descr);
				cmmd(cmdstr[i - (DISPHALF + 1)]);
			} else
				cmmd(alc->tissues[j+i].descr);

		if(more) {
			cmmd(":L-7k40;more <cr> ... ");
			mouse(MIDBT);
			Mcmd = mmcmd;
		}
		if(top > DISPHALF)
			cmmd(":L-7k40;exit ");
		else
			cmmd("exit ");
		mouse(LONGLB);
		Ecmd = mmcmd;

		while(1) {
			nncmd = thismu();
			if(more && (nncmd == Mcmd || nncmd == 0)) {
				j += top;
				break;
			}
			if(!more && !nncmd) {
				j = 0;
				left = alc->num_tissues;
				break;
			}
			if(nncmd == Ecmd) {
				upmenu();
				return(0);
			}
			if(nncmd > 0) {
				upmenu();
				return(j + nncmd);
			}
		}
		upmenu();
		if(!more && !nncmd)
			continue;
			
		left = left - DISPTOTAL;
	} while(left > 0);
}

/*
 * M O R E S T A I N S --- give menu of available stains
 *
 */
int
morestains() 
{
int left = alc->num_stains, i, j = 0, more, top, Mcmd, Ecmd;
char cmdstr[DISPHALF][TISSUELENGTH + 8];
	
	do {
		downmenu();
		newmenu("Known stains");
		top = (left > DISPTOTAL) ? DISPTOTAL: left;
		more = (top < DISPTOTAL) ? 0: 1;
		for(i = 0; i < top; i++)
			if(i > DISPHALF) {
				sprintf(cmdstr[i - (DISPHALF + 1)], ":L-7k40;%s", alc->stains[j+i].descr);
				cmmd(cmdstr[i - (DISPHALF + 1)]);
			} else
				cmmd(alc->stains[j+i].descr);

		if(more) {
			cmmd(":L-7k40;more ... ");
			mouse(MIDBT);
			Mcmd = mmcmd;
		}
		if(top > DISPHALF)
			cmmd(":L-7k40;exit ");
		else
			cmmd("exit ");
		mouse(LONGLB);
		Ecmd = mmcmd;
		while(1) {
			nncmd = thismu();
			if(more && (nncmd == Mcmd || nncmd == 0)) {
				j += top;
				break;
			}
			if(!more && !nncmd) {
				j = 0;
				left = alc->num_stains;
				break;
			}
			if(nncmd == Ecmd) {
				upmenu();
				return(0);
			}
			if(nncmd > 0) {
				upmenu();
				return(j + nncmd);
			}
		}
		upmenu();
		if(!more && !nncmd)
			continue;
			
		left = left - DISPTOTAL;
	} while(left > 0);
}

/*
 *	K N O W N T I S S U E
 *
 */
knowntissue(tis)
int		tis;
{
	int	i;
	for (i=0; i< alc->num_tissues; i++) {
		if (alc->tissues[i].index == tis) 
			return(i);
	}
	return(-1);
}

/*
 *	K N O W N S T A I N
 *
 */
knownstain(stn)
int		stn;
{
	int	i;
	for (i=0; i< alc->num_stains; i++) {
		if (alc->stains[i].index == stn) 
			return(i);
	}
	return(-1);
}

/*
 *	G E T T I S S U E  --  Get pointer to tissue string
 *						   for given tissue type
 */
char *
gettissue(n)
int n;
{
	int		i;
	for (i=0; i< alc->num_tissues; i++) {
		if (alc->tissues[i].index == n) {
			return(alc->tissues[i].descr);
		}
	}
	return(NULL);
}
/*
 *	G E T S T A I N  --  Get pointer to stain type string
 *						 for given stain type
 */
char *
getstain(n)
int		n;
{
	int		i;
	for (i=0; i< alc->num_stains; i++) {
		if (alc->stains[i].index == n) {
			return(alc->stains[i].descr);
		}
	}
	return(NULL);
}

/*
 * U N S C R A M --- Unscamble the feature descriptor expression
 *
 */
unscram(buf, lfd)
char *buf;
struct linfdisc *lfd;
{
int featid[3], count, divisor, multipl, i, order, breakout;
char *st, numbuf[256];


	featid[0] = featid[1] = featid[2] = -1;
	lfd->aflag = -1;
	count = divisor = multipl = 0;
	if(buf != NULL) {
		i = 0;
		st = buf;
		breakout = 0;
		while(1) {
			/*
			 * no letters allowed
			 */
			if(isalpha(*st))
				return(FAIL);
			
			/*
			 * unscramble string
			 */	
			switch (*st) {
				case '/':
					if(multipl == 1)
						order = 1;
					divisor++;
					if(i > 0)
						featid[count++] = atoi(numbuf);
					else
						return(FAIL);
					i = 0;
					break;
				case '*':
					if(divisor == 1)
						order = -1;
					multipl++;
					if(i > 0)
						featid[count++] = atoi(numbuf);
					else
						return(FAIL);
					i = 0;
					break;
				case ' ':
				case '\0':
				case '\t':
				case '\n':
					if(i > 0)
						featid[count++] = atoi(numbuf);
					else
						return(FAIL);
					breakout = 1;
					break;
				default:
					numbuf[i++] = *st;
					numbuf[i] = '\0';
					break;
			}
			if(breakout)
				break;
			st++;
		}
	}
	if(count == 1) {
		lfd->aflag = 1;
	} else if(count == 2) {
		if(divisor)
			lfd->aflag = 2;
		else
			lfd->aflag = 3;
	} else if(count == 3) {
		if(divisor == 2)
			lfd->aflag = 4;
		else if(multipl == 2)
			lfd->aflag = 5;
		else if((multipl == 1) && (multipl == 1)) {
			if(order == 1) 
				lfd->aflag = 6;
			else 
				lfd->aflag = 7;
		}
	} else if(count == 0)
		return(FAIL);
		
	lfd->feat1 = featid[0];
	lfd->feat2 = featid[1];
	lfd->feat3 = featid[2];
	return(SUCCESS);
}								


/*
 * I N I T _ C U L T U R E S ---
 *
 */
init_cultures(fnam)
char *fnam;
{
FILE *fp;
char *ptr, buffer[CBUFLEN];
int tis, stn, i = 1, j = 1, set = 1;

	/*
	 * grab memory to hold tissues and stains
	 */
	alc->tissues = (struct tisdata *) Calloc(MIN_TIS_NUM, 
		sizeof(struct tisdata));
	alc->stains = (struct stndata *) Calloc(MIN_STN_NUM, 
		sizeof(struct stndata));

	/*
	 * Initialise for defaults which are always present
	 */
	alc->tissues[0].index = 0;
	strcpy(alc->tissues[0].descr, "default tissue");
	alc->stains[0].index = 0;
	strcpy(alc->stains[0].descr, "default stain");
	alc->num_tissues = 1;
	alc->num_stains = 1;

	if ((fp = fopen(fnam, "r")) == NULL) {
 		screengo(ENDLINE, 0);
 		fprintf(stderr, "Can't open cultures data file\n");
 		sleep(1);
 		alc->num_tissues = 1;
 		alc->num_stains = 1;
 		return(FAIL);
 	} else {
		while (fgets(buffer, CBUFLEN, fp)) {
			buffer[strlen(buffer) - 1] = '\0';	/* Kill <cr> */
			if (strncmp(buffer,"Tissue",6) == NULL) {
				set = 0;
				ptr = skipword(buffer);
				tis = atoi(ptr);
				alc->tissues[alc->num_tissues].index = tis;
				alc->tissues[alc->num_tissues].numchr = DEF_CHR_NUM;
				ptr = skipword(ptr);
				strncpy(alc->tissues[alc->num_tissues].descr, ptr, TISSUELENGTH);
				if(++alc->num_tissues == (MIN_TIS_NUM * i)) {
					i++;
					alc->tissues = (struct tisdata *) 
						Realloc(alc->tissues, 
						(i - 1) * MIN_TIS_NUM * sizeof(struct tisdata),
						i * MIN_TIS_NUM * sizeof(struct tisdata));
				}
			} else if (strncmp(buffer,"Stain",5) == NULL) {
				ptr = skipword(buffer);
				stn = atoi(ptr);
				alc->stains[alc->num_stains].index = stn;
				strncpy(alc->stains[alc->num_stains].descr,
							skipword(ptr), STAINLENGTH);
				if(++alc->num_stains == MIN_STN_NUM * j) {
					j++;
					alc->stains = (struct stndata *) 
						Realloc(alc->stains, 
						(j - 1) * MIN_STN_NUM * sizeof(struct stndata),
						j * MIN_STN_NUM * sizeof(struct stndata));
				}
			} else if (strncmp(buffer,"Class",5) == NULL) {
				if(!set) {
					ptr = skipword(buffer);
					alc->tissues[alc->num_tissues].numchr = atoi(ptr);
					set = 1;
				}
			}
		}
	}
	fclose(fp);
	return(SUCCESS);
}

/*
 * C H A N G E T I S S U E --- look through available tissues & string match
 *							requested tissue; if found copy to current and
 *							return 1 otherwise return 0.
 */
int
changetissue(new, current)
char	*new,*current;
{
	char	*temp;
	register int i;

	if (alc->debug) {
		putprompt();
		fprintf(debug_channel,
			"Current tissue %s, new tissue %s\n",current,new);
	}

	if(*new != '\0') {
		for(i = 0; i < alc->num_tissues; i++) {
			if(!strncmp(new, alc->tissues[i].descr, strlen(new))) {
				strcpy(current, alc->tissues[i].descr);
				return;
			}
		}
		*new = '\0';
	}
	if(i = moretissues())
		strcpy(current, alc->tissues[i - 1].descr);
}

/*
 * C H A N G E S T A I N --- look through available stains & string match
 *							requested stain; if found copy to current and
 *							return 1 otherwise return 0.
 */
int
changestain(new, current)
char	*new,*current;
{
	char	*temp;
	register int i;

	if (alc->debug) {
		putprompt();
		fprintf(debug_channel, "Current stain %s, new stain %s\n",current,new);
	}
	if(*new != '\0') {
		for(i = 0; i < alc->num_tissues; i++) {
			if(!strncmp(new, alc->stains[i].descr, strlen(new))) {
				strcpy(current, alc->stains[i].descr);
				return;
			}
		}
		*new = '\0';
	}
	if(i = morestains())
		strcpy(current, alc->stains[i - 1].descr);
}


/*
 *	S E T T I S S U E  --  Look up tissue type in tissue type table
 *							and set up alc fields
 *
 *	return  1  if matched
 *			-1 if none in table
 *			0  otherwise
 */
settissue(n, ctp)
int		n;
struct	cyto_types *ctp;
{
	int		i;
	char	*tmp;
	
	if (alc->num_tissues == 0) {					/* Check table */
		ctp->ktistype = 0;
		return(-1);
	}
	if (n == 0) {								/* If 0 give default (first) */
		ctp->ktistype = alc->tissues[0].index;
		return(1);
	}
	if (tmp = gettissue(n)) {
		ctp->ktistype = n;
		return(1);
	}
	return(0);
}

/*
 *	S E T S T A I N  --  Look up stain type in stain type table
 *							and set up alc fields
 *
 *	return  1  if matched
 *			-1 if none in table
 *			0  otherwise
 */
setstain(n, ctp)
int		n;
struct	cyto_types *ctp;
{
	int		i;
	char	*tmp;
	
	if (alc->num_stains == 0) {
		ctp->kfstype = 0;
		return(-1);
	}	
	if (!n) {								/* If 0 give default (first) */
		ctp->kfstype = alc->stains[0].index;
		return(1);
	}
	if (tmp = getstain(n)) {
		ctp->kfstype = n;
		return(1);
	}
	return(0);
}

/*
 *	G E T T I S S U E I N D E X  --  Get index given string
 *
 */
int 
gettissueindex(str)
char *str;
{
	register int  i;
	for (i=0; i< alc->num_tissues; i++) {
		if (!strcmp(str, alc->tissues[i].descr)) {
			return(alc->tissues[i].index);
		}
	}
	return(-1);
}

/*
 *	G E T S T A I N I N D E X  --  Get index given string
 *
 */
int 
getstainindex(str)
char *str;
{
	register int  i;
	for (i=0; i< alc->num_stains; i++) {
		if (!strcmp(str, alc->stains[i].descr)) {
			return(alc->stains[i].index);
		}
	}
	return(-1);
}

/*
 * F I N D E N T R Y ---
 *
 */
findentry (rcf, type, tid)
struct rankcf *rcf;
int type;
struct cyto_types *tid;
{
int i, j, k, drange, nfeats, nclasses, boxfts, linfts, nnum, ctype;
int tis, stn;
double atof();
char buffer[CBUFLEN], *ptr;
FILE *fp;
BOXHEAD *bhd;
LINHEAD *lhd;
VCVHEAD *vhd;
VCVTYPE *vtp;

	if ((fp = fopen(alc->culturesfile, "r")) == NULL) {
 		screengo(ENDLINE, 0);
 		fprintf(stderr, "Can't open cultures data file\n");
 		sleep(1);
 		return(FAIL);
 	} else {
#ifdef OSK
/*		windmill("Searching cultures file ");*/
#endif
 		switch(type) {
 		case BOX:
 			rcf->bhd = freebox(rcf->bhd);
 			break;
 		case LIN:
			rcf->lhd = freelin(rcf->lhd);
			break;
 		case VCV:
 			rcf->vhd = freevcv(rcf->vhd);
 			rcf->vhd = allocvcv();
 			break;
 		default:
 			break;
 		}
		while (fgets(buffer, CBUFLEN, fp)) {
			buffer[strlen(buffer) - 1] = '\0';	/* Kill <cr> */
			if(strncmp(buffer, "Box", 3) == NULL) {
				if(type == BOX) {
					ptr = get_next_three(buffer, &tis, &stn, &drange);
					if(tis == tid->ktistype && stn == tid->kfstype && 
							drange == alc->curbay->drange) {
						ptr = skipword(ptr);
						nfeats = atoi(ptr);
						bhd = allocbox(nfeats);
						bhd->tis = tis;
						bhd->stn = stn;
						bhd->drange = drange;
						i = 0;
						while (fgets(buffer, CBUFLEN, fp)) {
							if(*buffer != '*') {
								bhd->bfd[i]->feat = atoi(buffer);
								ptr = skipword(buffer);
								bhd->bfd[i]->lower = atoi(ptr);
								ptr = skipword(ptr);
								bhd->bfd[i]->upper = atoi(ptr);
								if(++i == nfeats) break;
							}
						}
						rcf->bhd = bhd;
						fclose(fp);
#ifdef OSK
						windmillend(", done\n");
#endif
						return(SUCCESS);
					}
				}
			} else if(strncmp(buffer, "Lin", 3) == NULL) {
				if(type == LIN) {
					ptr = get_next_three(buffer, &tis, &stn, &drange);
					if(tis == tid->ktistype && stn == tid->kfstype && 
							drange == alc->curbay->drange) {
						ptr = skipword(ptr);
						nfeats = atoi(ptr);
						lhd = alloclin(nfeats);
						lhd->tis = tis;
						lhd->stn = stn;
						lhd->drange = drange;
						i = 0;
						while (fgets(buffer, CBUFLEN, fp)) {
							if(*buffer != '*') {
								unscram(buffer,	lhd->lfd[i]);
								ptr = skipword(buffer);
								lhd->lfd[i]->const = (double) atof(ptr);
								if(++i == nfeats) break;
							}
						}
						rcf->lhd = lhd;
						fclose(fp);
#ifdef OSK
						windmillend(", done\n");
#endif
						return(SUCCESS);
					}
				}
			} else if(strncmp(buffer, "Vcv", 3) == NULL) {
				if(type == VCV) {
					ptr = get_next_three(buffer, &tis, &stn, &drange);
					ptr = skipword(ptr);
					ctype = atoi(ptr); /* This will be dummy entry */
					if(tis == tid->ktistype && stn == tid->kfstype) {
						rcf->vhd->tis = tis;
						rcf->vhd->stn = stn;
						ptr = skipword(ptr);
						nfeats = atoi(ptr);
						ptr = skipword(ptr);
						nclasses = atoi(ptr);
						vtp = allocvcvtype(nfeats, nclasses);
						nnum = nclasses / 2;
						while (fgets(buffer, CBUFLEN, fp)) {
							j = 0;
							if(*buffer != '*') {
								ptr = buffer;
								do {
									vtp->logsqrtdet[j] = atof(ptr);
									ptr = skipword(ptr);
								} while(++j != nnum);
								break;
							}
						}
						while (fgets(buffer, CBUFLEN, fp)) {
							if(*buffer != '*') {
								ptr = buffer;
								do {
									vtp->logsqrtdet[j] = atof(ptr);
									ptr = skipword(ptr);
								} while(++j != nclasses);
								break;
							}
						}
						while (fgets(buffer, CBUFLEN, fp)) {
							j = 0;
							if(*buffer != '*') {
								ptr = buffer;
								do {
									vtp->scalef[j] = atof(ptr);
									ptr = skipword(ptr);
								} while(++j != nnum);
							}
							break;
						}
						while (fgets(buffer, CBUFLEN, fp)) {
							if(*buffer != '*') {
								ptr = buffer;
								do {
									vtp->scalef[j] = atof(ptr);
									ptr = skipword(ptr);
								} while(++j != nclasses);
							}
							break;
						}
						i = 0;
						do {
							while (fgets(buffer, CBUFLEN, fp)) {
								if(*buffer != '*') {
									j = 0;
									vtp->vfd[i]->feat = atoi(buffer);
									ptr = skipword(buffer);
									do {
										vtp->vfd[i]->featmean[j] = atoi(ptr);
										ptr = skipword(ptr);
									} while(++j != nclasses);
									break;
								}
							}
							j = 0;
							while (fgets(buffer, CBUFLEN, fp)) {
								if(*buffer != '*') {
									j = 0;
									ptr = buffer;
									do {
										vtp->vfd[i]->featsd[j] = atoi(ptr);
										ptr = skipword(ptr);
									} while(++j != nclasses);
									break;
								}
							}
						} while(++i != nfeats);
						rcf->vhd->vtp[0] = vtp;
						fclose(fp);
#ifdef OSK
						windmillend(", done\n");
#endif
						return(SUCCESS);
					}
				}
			}
		}
		fclose(fp);
#ifdef OSK
		windmillend("\n");
#endif
		return(FAIL);
 	}
}
	
/*
 * G E T _ N E X T _ T H R E E ---
 *
 */
static char *
get_next_three(buf, first, secd, third)
char *buf;
int *first, *secd, *third;
{
	
	buf = skipword(buf);
	*first = atoi(buf);
	buf = skipword(buf);
	*secd = atoi(buf);
	buf = skipword(buf);
	*third = atoi(buf);
	return(buf);
}

/*
 *	Realloc - Function used here only (Thanks scg), but lets do
 *	a nicer version. Could perhaps do with a gplib Realloc, if
 *	so then can rip this out
 */
static char *
Realloc(ptr, old, new)
char *ptr;
int old, new;
{
	char *tptr;

	tptr = Calloc(1, new);
	bcopy(ptr, tptr, old > new ? old : new);
	Free(ptr);
	return(tptr);
}
