/*
 *	S Y S I N I T . C   -   Initialise system global data
 *							These are site dependant facts
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *  Date:    8th May 1986
 *
 *  Modifications
 *
 * 02-12-91 jimp	Read frame-store number as hexadecimal (since it may
 *			in fact be frame-store ADDRESS.  If > 4, do not
 *			subtract 1.
 *	2  Feb 1991		ih/gjp		Added code to set flex_screen in alc->options
 *	14 Nov 1990		ih			Added an alcalloc to calloc alc & sub-structs
 *	20 Aug 1990		ih			Added piabase
 *	29 Mar 1990		dcb			Added bgcolour, and removed kary/meta colour + nseats
 *	20 Mar 1990		dcb			seat-lut, seat-station-camera, seat-station-lut
 *								and init. cameratype to zero.
 *	02 Jan 1990		dcb			Staticise word options to prevent name clash
 *	16 Dec 1989		dcb			Read in satellite process priority
 *	12 Dec 1989		dcb/GJP			Add menu_options
 *	23 Oct 1989		dcb			Add master/slave process priority & od cachesize
 *	29 Sep 1989		dcb			Add setting for archive method and type
 *	14 Aug 1989		ARL			Added code for satellite recognition
 *	 1 Jun 1989		CAS			Change environ handling
 *	19 May 1989		dcb			matrixfile to varenhance, and initialise
 *	16 May 1989		SEAJ			Added timing initialisation.
 *	11 May 1989		SEAJ			Added culturesfile.
 *  20 Mar 1989		SEAJ		Added matrixfile.
 *	01 Feb 1989		SCG/dcb		trivial bug in allocbays
 *	01 Feb 1989		dcb			Introduce 'frontend' for front end configuration
 *	24 Jan 1989		SCG/dcb		grab space for scores structure
 *	24 Jan 1989		CAS			Add a camera 'MASK' option for cameras that need
 *								the left + top stripping
 *	23 Jan 1989		dcb			Introduce analogue front end stuff
 *	16 Jan 1989		dcb			Introduce monitor MONITOR_ALL/MONITOR_PERCASE
 *	13 Jan 1989		dcb			stage_type as sys settable parameter
 *	 6 Dec 1988		CAS			wclevel now user setable
 *	28 Nov 1988		SCG			Initialise wclevel
 *	 3 Nov 1988		SEAJ		Initialise ccoord
 *	25 Oct 1988		CAS			Extension of camera config
 *	 3 Oct 1988		CAS			New option for hardcopy type
 *	23 Sep 1988		CAS			Options to user setable
 *	14 Sep 1988		CAS			Mousescale default -> 2
 *	17 Aug 1988		CAS			New config options + s/ware options
 *	26 May 1988		CAS			Fix cfip9 check string
 *	17 May 1988		CAS			Hooks for flexible display formatting
 *	21 Mar 1988		CAS			Make line buffer bigger + add stuff for
 *								transfer facility
 *	 9 Mar 1988		CAS			Added combifip version selection
 *	29 Jan 1988		CAS			Added mouse process and slave config
 *
 */
#include <stdio.h>
#include <allcontr.h>
#include <slides.h>
#include <cfipp.h>
#include <colour.h>
#include <ddgs.h>
#include <hardcopy.h>
#include <transfer.h>
#include <kcontrol.h>


#define TOXICMU 1
#define NONTOXICMU 0
/*
 * Machine met-scan default thold values
 */
#define DEFMETTHR1 11
#define DEFMETTHR2 33

/*
 *	Known versions of combifip to use
 */
struct cfipversion {
		char	*name;
		int		version;
		char	*loadarray;
	} cfvers[] = {{"4", 4, "211C 6732 6337 211A 7FF4 FDF2 2130 EFE1"} ,
				{"5", 5, "211C 67AA 6362 211A 250C 250C 2130 6BC9"} ,
				{"7", 7, "211C 67A0 6362 211A 250C 250C 2130 6BBF"} ,
/*				{"9.pre", 7, "211C 678F 6369 211A 250C 250C 2130 6BB2"} , */
				{"9", 9, "211C 678F 6369 211A 250C 250C 2130 6BB3"} ,
				{NULL, 0, NULL} };

struct wordoption {
		char	*opt;
		int		flag;
	};

/*
 *	Configuration options - per seat
 */
static struct wordoption configs[] = {	{ "FIP",	CONF_FIP	},
								{ "TV",		CONF_TV		},
								{ "MLR",	CONF_MLR	},
								{ "KTYPE",	CONF_KTYPE	},
								{ "SAT",	CONF_SAT	},
								{ NULL,		0			} };

static struct wordoption frontendconfigs[] = {	{ "ANALOGUE",	CONF_FIPAL },
								{ NULL,		0			} };
/*
 *	Software options - system wide
 */
static struct wordoption options[] = {	{ "FUSION",	OPT_FUSION	},
								{ "FLUOR",	OPT_FLUOR	},
								{ "MANKARY",OPT_MANKAR	},
								{ "FLEXSCREEN",OPT_FLEX },
								{ NULL,		0			} };

/*
 *	Hard copy options
 */
static struct wordoption hcopyopts[] = {{ "VGR",		HC_VGR			},
								{ "DIR300",		HC_DIR300		},
								{ "DIR300_QUE",	HC_DIR300_QUE	},
								{ "DIR300_OBJ",	HC_DIR300_OBJ	},
								{ NULL,			0				} };
/*
 *	Camera Type
 */
static struct wordoption cameraopt[] = {{ "DAGE68",	DAGE68	},
								 { "PULNIX",	PULNIX	},
								 { "MASK",		MASK	},
								 { NULL,			0	} };

static struct wordoption monitoropt[] = {{ "MONITOR_PERCASE", MONITOR_PERCASE },
								  { "MONITOR_ALL", MONITOR_ALL },
								  { NULL,			0	} };

/*
 *	Archiving options
 */
static struct wordoption archopt[]	=	{{ "OD", OD_ARCH },
								 { "TAPE", TAPE_ARCH },
								 { "FLOPPY", FLOPPY_ARCH },
								 { NULL, 0 }};
/*
 *	Menu options
 */
static struct wordoption menuopt[] =	{{ "TVSTAY", TVSTAY },
								 { NULL, 0 }};

/*
 * Default values
 */
#define DEF_CHECK_LEVEL 0
#define SCANLEFT -12900
#define SCANRIGHT 12950
#define SCANTOP -18246 
#define SCANBOT 23433
#define STAGE 0		/* Default unmodified stage */
#define SEAT 1
#define NBAYS 4
#define BAYLIM 4
#define HARDCOPY HC_VGR
#define	SITENAME ""
#define FSTORE 1
#define FSTORETYPE FS_VME512
#define OURTERM "/term"
#define	MIPROCTERM "/t1"
#define MIPROCBAUD 9600
#define PIABASE 0xfe0800;
#define CFIPVERSION "7"
#define DEBUGFILE "/nil"
#define MOUSETERM "/t5"
#define MOUSEBAUD 1200
#define	PRINTERTERM "/t3"
#define DEVEL 0
#define KMRDEBUG 0
#define TIMINGDBG 0
#define MOUSESCALE 2
#define FNAMESEP1 '_'
#define FNAMESEP2 '.'
#define USERGRPNUM 10
#define ONEGROUP 0
#define MONITOR 0
#define LANGUAGE ENGLISH
#define MOVESPEED 4000
#define SCANSPEED 4000
#define ARCHIVEMETHOD "DATE" /* Default archiving method */
#define CACHESIZE 400
#define SATPRIOR 130	/* Default priority for satellite process */

#ifdef UNUSED
#define KARY_COL_1 CL_GREY
#define KARY_COL_2 0
#define META_COL_1 CL_GREY
#define META_COL_2 0
#endif

/*
 * Files
 */
#define SYS_RC "/dd/sys/kmrc"
#define USR_RC "kmrc"
#define CULTURESFILE "/dd/params/cultures"
#define VARENHANCEFILE "/dd/params/varenhance"

/*
 * General
 */
#define BUFLEN 512

/*
 *	External references
 */
int setscreenformat();		/* Set up screen ktype layout */
int	sethcpyformat();
extern int inithcpy();		/* Init hardcopy params */

/*
 * Internal references
 */
static char *getp2();

/*
 * Hold offsets temporarily until bays set up
 */
static int *xoff, *yoff, *zoff;

/*
 * Internal data
 */
static	int		seat;
static	int		system;		/* Flag used to enable/disable option
								when reading system/user 'rc' files */

/*
 * A L C A L L O C -- allocate space for alc
 */
struct allkontr *
alcalloc()
{
	struct allkontr *alc;

	alc = (struct allkontr *)Calloc(1,sizeof(struct allkontr));
	alc->cfipp=(struct cfipparams *) Calloc(1,sizeof(struct cfipparams));
	alc->hcpy=(struct hardcopy *)Calloc(1,sizeof(struct hardcopy));
	return(alc);
}
		
/*
 * S Y S I N I T  - main routine to read initialisation files
 *					and set up data structures accordingly
 *
 */
sysinit(alc)
struct allkontr *alc;
{
	char	tmpbuf[40];				/* Junk buffer */
	char	*user, *getenv();
	int		i;

	tmpbuf[0] = '/';
	_gs_devn(0,&tmpbuf[1]);			/* Get device on path 0 (stdin) */
	if (strlen(tmpbuf) < 10)
		strcpy(alc->ourterm,tmpbuf);
	else
		strcpy(alc->ourterm,OURTERM);		/* If all else fails - set default */
/*
 * set defaults
 */
	seat = 0;								/* don't know seat yet */
	alc->config = 0;
	alc->options = 0;
	alc->hcpy->type = HARDCOPY;
	strcpy(alc->sitename,SITENAME);
 	strcpy(alc->miprocterm,MIPROCTERM);
	strcpy(alc->mouseterm,MOUSETERM);
	strcpy(alc->printerterm,PRINTERTERM);
	strcpy(alc->debugfile,DEBUGFILE);
	strcpy(alc->culturesfile, CULTURESFILE);
	strcpy(alc->matrixfile, VARENHANCEFILE);
	alc->lutfile[0] = '\0';	/* No file by default */
	strcpy(alc->arch_method, ARCHIVEMETHOD);
	alc->arch_type = 0;	/* No archiving by default */
	alc->cachesize = CACHESIZE;
	alc->menu_options = 0;
	alc->miprocbaud = MIPROCBAUD;
	alc->piabase = PIABASE;	
	alc->fstore = FSTORE;
	alc->fstoretype = FSTORETYPE;
	alc->cameratype = 0;
	alc->mouseproc = -1;				/* Internal by default */
	if (alc->nslaves) {
		for (i=0; i<alc->nslaves; i++)
			alc->slaveconf[i] = SL_USE;
	} else {
		alc->slaveconf[0] = SL_USE;			/* Default 3 slaves */
		alc->slaveconf[1] = SL_USE;
		alc->slaveconf[2] = SL_USE;
	}
	alc->devel = DEVEL;
	alc->debug = KMRDEBUG;
	alc->timing = TIMINGDBG;
#ifdef UNUSED
	alc->karycolour_1 = KARY_COL_1;
	alc->karycolour_2 = KARY_COL_2;
	alc->metacolour_1 = META_COL_1;
	alc->metacolour_2 = META_COL_2;
#endif
	alc->bg_colour.red = 
	alc->bg_colour.green = 
	alc->bg_colour.blue =  0;
	alc->mousescale = MOUSESCALE;
	alc->fnamesep1 = FNAMESEP1;			/* default file name sep chars */
	alc->fnamesep2 = FNAMESEP2;
	alc->usergroupnum = USERGRPNUM;
	alc->onegroup = ONEGROUP;
	alc->mprior = MPRIOR;
	alc->sprior = SPRIOR;
	alc->satprior = SATPRIOR;
	alc->monitor = MONITOR;				/* Monitor to file status */
	alc->language = LANGUAGE;
	alc->toxicmu = NONTOXICMU;			/* Small score menu */
	alc->ccoord = OFF;					/* coordinate conversion off */
	alc->maxbays = NBAYS;
	xoff = (int *) Calloc(BAYLIM, sizeof(int));
	yoff = (int *) Calloc(BAYLIM, sizeof(int));
	zoff = (int *) Calloc(BAYLIM, sizeof(int));
	for(i = 0; i < BAYLIM; i++) {
		xoff[i] = AV_XOFFSET;
		yoff[i] = AV_YOFFSET;
		zoff[i] = AV_ZOFFSET;
	}
	alc->scantop  = SCANTOP;
	alc->scanbot  = SCANBOT;		
	alc->scanleft =	SCANLEFT;
	alc->scanright = SCANRIGHT;
	alc->stage_type = STAGE;
	alc->wclevel = DEF_CHECK_LEVEL;
	initcfipp(alc->cfipp);
	inithcpy(alc->hcpy);
/*
 * Look for system 'rc' file
 */
 	system = 1;								/* Enable system options */
 	dofile(SYS_RC,alc);
/*
 * Then user 'rc' file
 */
 	system = 0;				/* Disable system options */
	if (user = getenv("USER")) {
		sprintf(tmpbuf,"/dd/usr/%s/%s",user,USR_RC);
	 	dofile(tmpbuf,alc);
	 	if (alc->debug) printf("kmrc: %s\n",tmpbuf);
	} else
		dofile(USR_RC,alc);
/*
 * Grab space for bays and initialise offsets
 */
	allocbays(alc);
	Free(xoff);
	Free(yoff);
	Free(zoff);
	xoff = yoff = zoff = NULL;
/*
 * Any more defaults
 */
	alc->seat = (seat ? seat : SEAT);
/*
 *	Make sure slave count is OK
 */
	alc->nslaves = 0;
	for (i=0; i<MAXSLAVE; i++)
		if (alc->slaveconf[i] == SL_USE)
			alc->nslaves++;
}

/*
 *	I N I T C F I P P --
 *
 */
initcfipp(cfipp)
struct cfipparams *cfipp;
{
	cfipp->movespeed = MOVESPEED;
	cfipp->scanspeed = SCANSPEED;
	cfipp->speed = UNSET;

	cfipp->tholds.dmthresh1 = DEFMETTHR1;
	cfipp->tholds.dmthresh2 = DEFMETTHR2;
	cfipp->tholds.thresh1 = UNSET;
	cfipp->tholds.thresh2 = UNSET;

	cfipp->focus.permove = 28;
	cfipp->focus.inswathe = 300;
	cfipp->focus.between = 300;
	cfipp->focus.excursion = 1000;
	cfipp->autofocus = UNSET;
	cfipp->highres = UNSET;
	miploadarray(CFIPVERSION,cfipp);
}

/*
 *	D O F I L E  --
 *
 */
dofile(name,alc)
char	*name;
struct allkontr *alc;
{ 	
	FILE	*rc;
	char	*buffer,*value;

	buffer = (char *) Malloc(BUFLEN);
	if ((rc=fopen(name,"r")) != NULL) {
		while(fgets(buffer,BUFLEN,rc) != NULL) {
			if (*buffer != '*') {			/* Watch for comment lines */
				value = getp2(buffer,strlen(buffer),0);
				if (*buffer)
					process(buffer,value,alc);
			}
		}
		fclose(rc);
	}
	Free(buffer);
}

/*
 * Please excuse this code - this is a prototype version and could
 *	do with being improved ....
 */
static process(ptr,val,alc)
char	*ptr, *val;
struct allkontr *alc;
{
	int		i, ns;
	char	*p2, *p3, *p4;

	if (system) {
		if (!strcmp(ptr,"miprocterm")) {
			if (strlen(val) < TSIZ)
				strcpy(alc->miprocterm, val);
			else 
				message("Can't set miprocterm to %s\n",val);
/* printer term */
		} else if (!strcmp(ptr,"printerterm")) {
			if (strlen(val) < TSIZ)
				strcpy(alc->printerterm, val);
			else
				message("Can't set printerterm to %s\n",val);
/* Site name */
		} else if (!strcmp(ptr,"sitename")) {
			strncpy(alc->sitename, val, LABLENGTH);
/* miproc baud */
		} else if (!strcmp(ptr,"miprocbaud")) {
			alc->miprocbaud = atoi(val);
/* pia base */
		} else if (!strcmp(ptr,"piabase")) {
			if ((val[0]=='0')&&(val[1]=='x'))
				alc->piabase = axtoi(val+2);
			else
				alc->piabase = atoi(val);
			if (!alc->piabase) alc->piabase=PIABASE; /* goto default if trashed */
/* combifip - which version to use */
		} else if (!strcmp(ptr,"combifip")) {
			strcpy(alc->cfipp->program,val);
			miploadarray(val,alc->cfipp);
/* cfipcheck - data to check combifip loaded OK */
		} else if (!strcmp(ptr,"cfipcheck")) {
			strcpy(alc->cfipp->checkarray,val);
#ifdef UNUSED
/* nseats */
		} else if (!strcmp(ptr,"nseats")) {
			alc->nseats = atoi(val);
#endif
/* seat - term */
		} else if (!strncmp(ptr,"seat",4)) {
			if (!strcmp(ptr+5,"term")) {
				if (!strcmp(val,alc->ourterm)) {
					seat = *(ptr+4) - '0';
				}
			} else if (seat && ((*(ptr+4) - '0') == seat)) {
/* seat - config */
				if (!strcmp(ptr+5,"config")) {
					if (i = word_option(val, configs)) {
						alc->config |= i;
					} else
						message("%s is an invalid configuration parameter\n",val);
/* seat - mouse */
				} else if (!strcmp(ptr+5, "mouse")) {
					if (strlen(val) < TSIZ)
						strcpy(alc->mouseterm, val);
					else
						message("Can't set mouseterm to %s\n",val);
					if ((p2 = getp2(val,0,1)) != NULL)
						alc->mouseproc = atoi(p2);
/* seat - camera */
				} else if (!strcmp(ptr+5, "camera")) {
					if (i = word_option(val, cameraopt)) {
						alc->cameratype |= i;
					} else
						message("Invalid camera configuration %s\n", val);
/* seat - lut */
				} else if (!strcmp(ptr+5, "lut")) {
					if (strlen(val) <= 40)
						strcpy(alc->lutfile, val);
					else
						message("lut filename %s too long\n", val);
/* seat - slave */
				} else if (!strcmp(ptr+5, "slave")) {
					for (i=0; i<MAXSLAVE; i++) {
						alc->slaveconf[i] = 0;
					}
					ns = 0;
					while ((val != NULL) && (*val != '\0')) {
						i = atoi(val);
						if (i >= 0 && i < MAXSLAVE)
							alc->slaveconf[i] = SL_USE;
						if (alc->nslaves && (++ns >= alc->nslaves))
							break;
						val = getp2(val,0,1);
					}
					alc->nslaves = ns;
/* seat - fstore */
				} else if (!strcmp(ptr+5, "fstore")) {
					sscanf(val,"%x",&alc->fstore);
					if (alc->fstore > 10 || alc->fstore < 0)
						alc->fstore++; /* to circumvent the -1 in km.c */
					p2 = getp2(val,0,1);
					if (p2 != NULL) {
						if (!strcmp(p2,"VME512")) {
							alc->fstoretype = FS_VME512;
						} else if (!strcmp(p2,"VIRTUOSO_INVERT")) {
							alc->fstoretype = FS_VIRTUOSO_INVERT;
						} else if (!strcmp(p2,"VIRTUOSO")) {
							alc->fstoretype = FS_VIRTUOSO;
						} else
							message("%s is an illegal framestore type\n",p2);
					}
/* seat - station */
				} else if (!strncmp(ptr+5,"station",7)) {
					if (*(ptr+12) - '0' == alc->satno) {	/* Must be us */
/* seat - station - camera */
						if (!strcmp(ptr+13, "camera")) {
							alc->cameratype = word_option(val, cameraopt);
/* seat - station - lut */
						} else if (!strncmp(ptr+13, "lut", 3)) {
							if (((p2 = getp2(val,0,1)) != NULL) && (strlen(p2) <= 40))
								strcpy(alc->lutfile, p2);
							else
								message("lut filename %s too long\n", p2);
						}
					}
				}
			}
/* front end config */
		} else if (!strcmp(ptr,"frontend")) {
			if (i = word_option(val,frontendconfigs)) {
				alc->config |= i;
			} else
				message("%s is an invalid congfiguration parameter\n",val);
/* Hardcopy type */
		} else if (!strcmp(ptr,"hardcopy")) {
			if ((alc->hcpy->type = word_single(val,hcopyopts)) == 0)
				message("Invalid hardcopy type (%s)\n",val);
/* user - group number */
		} else if (!strcmp(ptr,"usergroupnum")) {
			alc->usergroupnum = atoi(val);
/* all users same or different group switch */
		} else if (!strcmp(ptr,"onegroup")) {
			alc->onegroup = atoi(val);
/* Master process priority */
		} else if (!strcmp(ptr,"master_priority")) {
			alc->mprior = atoi(val);
/* Slave process priority */
		} else if (!strcmp(ptr,"slave_priority")) {
			alc->sprior = atoi(val);
/* Satellite process priority */
		} else if (!strcmp(ptr,"sat_priority")) {
			alc->satprior = atoi(val);
/* file name seperation chars */
		} else if (!strcmp(ptr,"fnamesep1")) {
			alc->fnamesep1 = *val;
		} else if (!strcmp(ptr,"fnamesep2")) {
			alc->fnamesep2 = *val;
		} else if (!strcmp(ptr,"nbays")) {
			alc->maxbays = atoi(val);
/* Distances between bays. */
		} else if (!strcmp(ptr,"bay1x")) {
			xoff[0] = atoi(val);
		} else if (!strcmp(ptr,"bay1y")) {
			yoff[0] = atoi(val);
		} else if (!strcmp(ptr,"bay1z")) {
			zoff[0] = atoi(val);
		} else if (!strcmp(ptr,"bay2x")) {
			xoff[1] = atoi(val);
		} else if (!strcmp(ptr,"bay2y")) {
			yoff[1] = atoi(val);
		} else if (!strcmp(ptr,"bay2z")) {
			zoff[1] = atoi(val);
		} else if (!strcmp(ptr,"bay3x")) {
			xoff[2] = atoi(val);
		} else if (!strcmp(ptr,"bay3y")) {
			yoff[2] = atoi(val);
		} else if (!strcmp(ptr,"bay3z")) {
			zoff[2] = atoi(val);
		} else if (!strcmp(ptr,"bay4x")) {
			xoff[3] = atoi(val);
		} else if (!strcmp(ptr,"bay4y")) {
			yoff[3] = atoi(val);
		} else if (!strcmp(ptr,"bay4z")) {
			zoff[3] = atoi(val);
/* Slide - left/right limits */
		} else if (!strcmp(ptr,"scanleft")) {
			alc->scanleft = atoi(val);
		} else if (!strcmp(ptr,"scanright")) {
			alc->scanright = atoi(val);
/* Stage type */
		} else if (!strcmp(ptr, "stagetype")) {
			alc->stage_type = atoi(val);
/* Language */
		} else if (!strcmp(ptr,"language")) {
			alc->language = atoi(val);
/* OD archiving method */
		} else if (!strcmp(ptr,"arch_method")) {
			if (strlen(val) < TEXTLENGTH)
				strcpy(alc->arch_method, val);
			else
				message("Can't set OD archive method\n");
/* OD Archiving type */
		} else if (!strcmp(ptr,"archive")) {
			if (i = word_option(val,archopt)) {
				alc->arch_type |= i;
			} else
				message("%s is an invalid archiving media\n",val);
/* OD cachesize */
		} else if (!strcmp(ptr,"cachesize")) {
			alc->cachesize = atoi(val);
		}
	}
/*
 * USER SETABLE PARAMETERS 
 */
/* devel */
	if (!strcmp(ptr,"devel")) {
		alc->devel = atoi(val);
/* debug */
	} else if (!strcmp(ptr,"debug")) {
		alc->debug = atoi(val);
/* debugfile */
	} else if (!strcmp(ptr,"debugfile")) {
		if (strlen(val) < 40)
			strcpy(alc->debugfile, val);
		else
			message("Can't set debugfile to %s\n",val);
/* timing */
	} else if (!strcmp(ptr, "timing")) {
		alc->timing = atoi(val);
/* mipdebug */
	} else if (!strcmp(ptr,"mipdebug")) {
		alc->mipdebug = atoi(val);
/* monitor */
	} else if (!strcmp(ptr, "monitor")) {
		if ((alc->monitor = word_option(val, monitoropt)) == 0)
			message("Invalid monitoring level (%s)\n",val);
		alc->lastname = (struct cyto_file*)Calloc(1, sizeof(struct cyto_file));
/* Woolz object checking level */
	} else if (!strcmp(ptr, "wclevel")) {
		alc->wclevel = atoi(val);
/* Options */
	} else if (!strcmp(ptr,"options")) {
		if (i = word_option(val,options)) {
			alc->options = i;
		} else
			message("%s is an invalid option parameter\n",val);
/* Menu options */
	} else if (!strcmp(ptr,"menu_options")) {
		if (i = word_option(val,menuopt)) {
			alc->menu_options = i;
		} else
			message("%s is an invalid menu option\n",val);
/* Transfer line */
	} else if (!strcmp(ptr,"transline")) {
		if (!alc->trans) alc->trans = (struct transfer *) Calloc(1,sizeof(struct transfer));
		if (strlen(val) < TSIZ)
			strcpy(alc->trans->line, val);
		else
			message("Can't set transfer line to %s\n",val);
		if ((p2 = getp2(val,0,1)) != NULL)
			alc->trans->baud = atoi(p2);
/* Transfer start */
	} else if (!strcmp(ptr,"transstart")) {
		if (!alc->trans) alc->trans = (struct transfer *) Calloc(1,sizeof(struct transfer));
		if (*val) {
			alc->trans->startup = (char *) Malloc(strlen(val)+1);
			strcpy(alc->trans->startup,val);
		}
/* Transfer end */
	} else if (!strcmp(ptr,"transend")) {
		if (!alc->trans) alc->trans = (struct transfer *) Calloc(1,sizeof(struct transfer));
		if (*val) {
			alc->trans->closedown = (char *) Malloc(strlen(val)+1);
			strcpy(alc->trans->closedown,val);
		}
/* Screen karyotype format/layout */
	} else if (!strcmp(ptr,"screenform")) {
		setscreenformat(val);
/* Hardcopy karyotype format/layout */
	} else if (!strcmp(ptr,"hardcopyform")) {
		sethcpyformat(val);
/* Cultures file */
	} else if (!strcmp(ptr, "culturesfile")) {
		strcpy(alc->culturesfile, val);
/* Variable enhancement file */
	} else if (!strcmp(ptr, "varenhance")) {
		strcpy(alc->matrixfile, val);
/* colours */
	} else if (!strcmp(ptr,"bgcolour")) {
		alc->bg_colour.red = atoi(val);
		if ((p2 = getp2(val,0,1)) != NULL) alc->bg_colour.green = atoi(p2);
		if ((p2 = getp2(p2,0,1)) != NULL) alc->bg_colour.blue = atoi(p2);
#ifdef UNUSED
	} else if (!strcmp(ptr,"karycolour1")) {
		alc->karycolour_1 = atoi(val);
	} else if (!strcmp(ptr,"karycolour2")) {
		alc->karycolour_2 = atoi(val);
	} else if (!strcmp(ptr,"metacolour1")) {
		alc->metacolour_1 = atoi(val);
	} else if (!strcmp(ptr,"metacolour2")) {
		alc->metacolour_2 = atoi(val);
#endif
/* Mouse scale */
	} else if (!strcmp(ptr,"mousescale")) {
		alc->mousescale = atoi(val);
/* Score menu */
	} else if (!strcmp(ptr,"scoremu")) {
		if(!strcmp(val,"extended"))
			alc->toxicmu = 1;
/* Focus parameters */
	} else if (!strcmp(ptr,"focus")) {
		p2 = getp2(val,0,1); p3 = getp2(p2,0,1); p4 = getp2(p3,0,1);
		if (p4 != NULL) {
			alc->cfipp->focus.permove = atoi(val);
			alc->cfipp->focus.inswathe = atoi(p2);
			alc->cfipp->focus.between = atoi(p3);
			alc->cfipp->focus.excursion = atoi(p4);
		}
/* SCAN + MOVE Speeds */
	} else if (!strcmp(ptr,"movespeed")) {
		alc->cfipp->movespeed = atoi(val);
	} else if (!strcmp(ptr,"scanspeed")) {
		alc->cfipp->scanspeed = atoi(val);
/* default metaphase scan thresholds */		
	} else if (!strcmp(ptr,"metthresh")) {
		alc->cfipp->tholds.dmthresh1 = atoi(val);
		if ((p2 = getp2(val,0,1)) != NULL)
			alc->cfipp->tholds.dmthresh2 = atoi(p2);
	}
}

/*
 *	G E T P 2  --  Get pointer to parameter 
 *					if 'equ' false - skip down line till we find
 *					'=' on line else just skip to next word.
 *
 */
static char *
getp2(ptr,len,equ)
char	*ptr;
int		len, equ;
{
	int		inword, seeneq, instring;
	char	*retptr = NULL;
	static char	*endptr;

	if (equ) {
		inword = 1;
		seeneq = 1;
	} else {
		inword = 0;
		seeneq = 0;
		endptr = ptr + len;
	}
	instring = 0;
	while (ptr <= endptr) {
		switch (*ptr) {
		case '=':
			if (instring)
				break;
			seeneq = 1;
			inword = 0;
			*ptr = '\0';
			break;
		case ' ':
		case '\t':
			if (instring)
				break;
		case '\n':
		case '\0':
			if (inword) {
				*ptr = '\0';
				if (retptr)
					return(retptr);
				else
					instring = 0;
			}
			inword = 0;
			break;
		case '"':
			if (seeneq) {
				if (inword) {
					instring = 1;
				} else if (instring) {
					*ptr = '\0';		/* Kill trailing '"' */
					inword = 1;			/* and wait for delimiter */
					instring = 0;
				} else {
					instring = 1;
					retptr = ptr + 1;
				}
			}
			break;
		default:
			if (instring)
				break;
			if (!inword) {
				inword = 1;
				if (seeneq && !retptr) {
					retptr = ptr;
				}
			}
		}
		ptr++;
	}
	return((*retptr) ? retptr : endptr);
}

/*
 *	W O R D _ O P T I O N  --  Handle options where flags need setting dependant
 *								on one or more 'special words'
 */
word_option(str,opts)
char	*str;
struct wordoption opts[];
{
	int		ret = 0;
	struct wordoption *otab;

	while (str && *str) {
		otab = opts;
		while (otab->opt) {
			if (!strcasecmp(otab->opt,str)) {
				ret |= otab->flag;
				break;
			}
			otab++;
		}
		str = getp2(str,0,1);
	}
	return(ret);
}

/*
 *	W O R D _ S I N G L E   --  Handle options where a string need matching
 *
 */
word_single(str,opts)
char	*str;
struct wordoption opts[];
{
	struct wordoption *otab;

	while (str && *str) {
		otab = opts;
		while (otab->opt) {
			if (!strcasecmp(otab->opt,str)) {
				return(otab->flag);
			}
			otab++;
		}
	}
	return(0);
}

/*
 *	A L L O C B A Y S
 *
 *				Grabs space for "maxbays" consequtive chunks of memory
 *				for mult_slide structures and sets the pointer in alc.
 *				
 */
int 
allocbays(alc)
struct allkontr *alc;
{
struct mult_slide *bp;
int	i;

	if(alc->maxbays <= 0) {
		message("allocbays: alc->maxbays not defined\n");
		alc->frsbay = alc->curbay = NULL;
		return(-1);
	}
	if((bp = (struct mult_slide *) (Calloc(sizeof(struct mult_slide),
											 alc->maxbays))) != NULL) {
		alc->frsbay = alc->curbay = bp;
		for(i = 0; i < alc->maxbays; i++) {
			if((bp->scp = (struct scores *) (Calloc(sizeof(struct scores), 1)))
											 == NULL)
										return(-1);
			bp->xoff = xoff[i];
			bp->yoff = yoff[i];
			bp->zoff = zoff[i];
			bp++;
		}
		return(0);
	} else {
		message("allocmslides: Calloc returned NULL\n");
		alc->frsbay = alc->curbay = NULL;
		return(-1);
	}
}
/*
 *	M I P L O A D A R R A Y  --  Set up combifip load check array
 *								 given the version of the program
 */
miploadarray(name,cfipp)
char	*name;
struct cfipparams *cfipp;
{
	struct cfipversion *cv;
	cv = cfvers;
	while (cv->name) {
		if (!strcmp(cv->name,name)) {
			strcpy(cfipp->checkarray,cv->loadarray);
			strcpy(cfipp->program,cv->name);
			cfipp->pversion = cv->version;
			return(1);
		}
		cv++;
	}
	message("Failed to set combifip version %s\n",name);
	return(0);
}

#ifdef CASPLAY
addtorcfile(file,tmpfile,ptr,val)
char	*file,*ptr,*val;
{
	FILE	*rc, *new;
	char	*buffer,*value;

	buffer = (char *) Malloc(BUFLEN);
	if ((new = fopen(tmpfile,"w")) != NULL) {
		if ((rc=fopen(file,"r")) != NULL) {
			while(fgets(buffer,BUFLEN,rc) != NULL) {
				if (*buffer != '*') {			/* Watch for comment lines */
					value = getp2(buffer,strlen(buffer),0);
					if (!strcmp(ptr,buffer)) {
					}
				}
				fputs(buffer,strlen(buffer),new);
			}
			fclose(rc);
		}
		fclose(new);
		sprintf(buffer,"copy -r %s %s",tmpfile,file);
		system(buffer);
		unlink(tmpfile);
	}
	Free(buffer);
}
#endif CAS
/*
 *	M E S S A G E  ---
 *
 */
static int
message(p1,p2,p3,p4,p5,p6,p7,p8)
int		p1,p2,p3,p4,p5,p6,p7,p8;
{
	fprintf(stderr,p1,p2,p3,p4,p5,p6,p7,p8);
	tsleep(150);
}
