/*
 *      T V D I G I T I S E . C -- Digitise from TV image
 *
 *
 *  Written: G J Page
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    16th July 1986
 *
 *
 * Modifications
 *
 *  16 Apr 1991		ih			Don't turn off alloc_wait before fsconstruct
 *	16 Apr 1991		CAS			In devel mode monitor vwaitf return status..
 *   5 Apr 1991		ih			Only apply current lut to none fipdig image
 *                              This is a temporary fix until capture/tvdigitise
 *                              are de-coupled properly.
 *	28 Mar 1991		ih/cas		Fix bug in fixed integtime with virt driver
 *	15 Jan 1991		CAS/GJP		Uses debug_channel when not open.
 *  11 Jul 1990     ih			Set alc->digitised for Sc etc
 *	27 Mar 1990		dcb			Only exit threshops if accept option chosen
 *	19 Mar 1990		dcb			Switch appropriate LUT on altering fluor status
 *	16 Mar 1990		dcb			Merge with satdig.c, put satellite no as field in alc (no longer passed to read_shade)
 *	14 Mar 1990		dcb			Call lut_normalise instead of normalise
 *	12 Mar 1990		dcb			lin_colour_lut in restoreddgs, apply_lut in initialisefg, reverse sense of invert flag in upthresh/downthresh
 *	07 Mar 1990		dcb			Use current lut vals in upthresh/downthresh,
 *								cameratype_lut in initialisefg. Remove fluor stuff, and swap
 *								dig_type assign and cellheadfill call in filepic
 *	28 Feb 1990		dcb			Replace llut, nlut calls by apply_lut
 *	 4 Jan 1990		CAS			Pass null satellite data into read/write shade
 *	12 Dec 1989		dcb			Add TVSTAY to remain in tv digitise menu
 *	23 Oct 1989		dcb			Set display BUST after show histo. Flush input at start
 *	17 Aug 1989		CAS			Fluor stored in file header + alc
 *	11 Aug 1989		CAS			Integtime now controlled in driver
 *	31 Jul 1989		CAS			Automatic iris facility for fluorescence
 *	25 Jul 1989		CAS			Added call to line average routine for pulnix
 *								long integration time
 *	22 Mar 1989		dcb			Drop out after digitise (Non f/fuse) or filing
 *								f/fuse systems
 *	19 Mar 1989		dcb/GJP		set kcont->FIP to DIGTV, change mouse buttons
 *	31 Mar 1989		dcb			threshops returns threshold value, or 0 if quit
 *	14 Mar 1989		dcb			Rip out call to kickkcont in initialisewoolz
 *	30 Jan 1989		CAS			Edge + mask defines out to kyy.h
 *	27 Jan 1989		CAS			Strip of shading correction edge at same time
 *								as we do edge masking.
 *	25 Jan 1989		CAS			Set up masking to 17 + 8
 *	24 Jan 1989		CAS			Further tidy to PULNIX stuff. Integtime 0 => normal digitisation
 *								Picture edge masking
 *  24 Jan 1989		SEAJ		Added message when there is nothing to file,
 *								or call to digitise with a cell in.
 *	31 Oct 1988		CAS			Added some checking around fusion stuff
 *	27 Oct 1988		CAS			Mods to shading calls
 *	24 Oct 1988		CAS			Camera types to allcontr
 *	16 Aug 1988		CAS			Options for pulnix/peltier/fluorescence
 *	18 Aug 1988		CAS			Fluorescence as kmrc option
 *	17 Aug 1988		CAS			Fusion now a switchable option
 *	16 Aug 1988		CAS			Fluor lut stuff
 *	15 Aug 1988		CAS			Mouse button to quit from threshold
 *								Try reshade on FS_VIRTUOSO
 *	12 Aug 1988		CAS			Split fusion stuff out
 *	 5 Aug 1988		CAS			Fix to blue wash threshold
 *	 4 Aug 1988		CAS			Invert fac (in devel) for fluor
 *	29 Jul 1988		CAS			Start on field fusion
 *  18 Jul 1988		SEAJ		Made 'no' to shading correction work in reshade.
 *  18 Jul 1988		SEAJ		Changed go_on(1) to go_on().
 *	 6 Jul 1988		CAS			tidy cmmd lines
 *	13 Jun 1988		CAS			Added call to addplist before filing
 *	13 May 1988		CAS			Indicate shading on/off in digitise menu
 *	 1 Feb 1988		CAS			proceed changed to 'please focus, proceed'
 *								various message changes (cjc)
 */
#include <cmdsys.h>
#include <kyy.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <fileheaders.h>
#include <ddgs.h>
#include <sai.h>
#include <setsys.h>

#define TRUE (1)
#define FALSE (0)

static struct object *histo;
static struct object *rectobj=NULL;		/* Fstore object */
static int thresh = 20;					/* Current threshold */
static int ttog=1;						/* Autothreshold toggle */
static int shading = 0;					/* Shading correction on/off */
static int step=4;						/* Manual threshold increment */
static min_lab_siz = 2;					/* Min size object to label */
static int imageinvert = 0;				/* Inverted image in fstore (fluorescence) */
static int integtime = -1;				/* Integration time for image (automatic)*/
static int lut_modify = TRUE;			/* Modify data using gamma LUTS */

/*
 *	Fuse stuff
 */
#define FUSEMAX 3	/* Max number of fuse images */

extern struct object *fuseobj[FUSEMAX][MAXOBJS];	/* Fuse object lists */
extern struct pframe fuseframe[FUSEMAX+1];			/* Fuse frames */
extern int fusecount[FUSEMAX];						/* Number of objects in fuseobj lists */
extern int fuseframes;								/* Number of frames grabbed for fusing */

/*
 *	External things to import
 */
extern FILE *debug_channel;
extern struct kcontrol *kcont;
extern struct ipoint cursor;

/*
 *	Internal function forward declarations
 */
static int filecell();					/* Sort out filing */
static int getcell();
static int getsecond();
static int dohisto();
static shademenu();
static int switchfluor(tog);
static reshade(), dispshade();
static int pulnix(), pgrab(), oldpgrab(), itimefn();

/*
 * External routines to import
 */
extern struct object *fsframehisto();
extern struct object *fsconstruct();
extern struct object *fsthreshold();
extern struct object *fshisto();
extern struct pframe *makepframe();
extern struct object *makerect();
extern fsmemerror();
extern int woolzexitfn();

extern int fieldfuse();
extern read_shade(), write_shade();
extern apply_shade();					/* Do shading correction */
extern calc_shade();					/* Calc shading correction */
extern display_shade();					/* Display shading correction */

/*
 *	T V D I G I T I S E 
 *
 */
 tvdigitise()
 {
 	static int firstime = 1;
 	char	tmpstr[20], integstr[20];
 	int Ex_cmd, Fi_cmd;
	if (firstime) {
		imageinvert = alc->ddgschannel->invert;
		firstime = 0;
	}
	sprintf(tmpstr,"%d",integtime);
	itimefn(tmpstr,integstr,&integtime);		/* Initialise */
	alc->fluor = (imageinvert != alc->ddgschannel->invert);
	alc->digitised=1;
	kcont->FIP = DIGTV;	/* This is where the image came from */
	initialisefg();
	initialisewoolz();
	read_shade();
	fgdigitise();
	newmenu ("Digitise");
		cmmd("digitise cell ;%-+d%f",&(alc->fluor),getcell); mouse(MIDBT);
		Fi_cmd = mmcmd;
		if (alc->options & OPT_FUSION) {
			cmmd("digitise stray chromosomes ;%-+d%f",&(alc->fluor),getsecond);
				mouse(LONGMB);
			cmmd("fuse field ;%f",fieldfuse);mouse(RIGHTBT);
			cmmd("file cell ;%f",filecell);mouse(LEFTBT); Fi_cmd = mmcmd;
		}
		cmmd("show histogram ;%f",dohisto);mouse(LONGRB);
		cmmd("automatic threshold;:k40%T",&ttog);
		cmmd("shading correction;:k40%-T%-+d%f",&shading,&(alc->fluor),shademenu);
/*		cmmd("manual threshold utilities;%x%f","Manual Threshold Utilites",threshops); */
#ifdef HISTONLY
		cmmd("histogram;%g",histonly);	
#endif
		if (alc->options & OPT_FLUOR)
			cmmd("fluorescence;:k40%-T%f",&(alc->fluor),switchfluor);
		if (alc->devel) {
			cmmd("camera type;%d",&alc->cameratype);
			cmmd("gamma correct data;:k40%T", &lut_modify);
		}
		if ((alc->options & OPT_FLUOR) && (alc->cameratype & PULNIX)) {
			cmmd("exposure time;:k40%+s%-s%-+d%f",tmpstr,integstr,&integtime,itimefn);
		}
		cmmd("exit"); mouse(LONGLB); Ex_cmd = mmcmd;
	while (1) {
/*
 *	Drop through if exit selected (Always)
 *	Or if digitise (non f/fuse), or file (f/fuse) selected (Normal system)
 *	unless menu_option TVSTAY is set and not a satellite.
 *	(Satellites ignore TVSTAY)
 */
		do {
			thismu();					/* Stay in this menu */
		} while ((nncmd != Ex_cmd) && 	/* while "not exit" and */
				((nncmd != Fi_cmd) ||	/* Not filed, or TVSTAY set & not sat.*/
				((alc->menu_options == TVSTAY) && !(alc->config & CONF_SAT))));

		if (fuseframes || alc->cellin) {
			if (check("Digitised image not stored, proceed (M):")) {
				if (alc->config & CONF_SAT)
					oclear(0,0,alc->ddgschannel->lineln-1,alc->ddgschannel->lineno-1,0);
				break;
			}
		} else {
			if (alc->config & CONF_SAT)
 				oclear(0,0,alc->ddgschannel->lineln-1,alc->ddgschannel->lineno-1,0);
			break;
		}
	}
/*
 *	if theres a list of objects not yet freed then get rid of them
 */
	if (alc->cellin) {
		release(kcont->ool,&alc->lkount);
		alc->lbelld = 0;
		alc->cellin=0;
		kcont->maxnumber = 0;
		freefuse();
	}
	fgfreeze();
	clear();
	restoreddgs();
	restorewoolz();
}

/*
 *	S H A D E M E N U  --  Shading options menu
 *
 */
static shademenu(shading,fluor)
int		*shading, *fluor;
{
	int	othresh, ottog;
	fgfreeze();
	othresh = thresh; ottog = ttog;
	newmenu("Shading Correction");
	cmmd("get new shading values;%-+d%f",fluor,reshade);
	cmmd("save shading correction;%+i%f",0,write_shade);
/*	cmmd("show histogram;%f",dohisto);mouse(LONGLB); */
	cmmd("display current shading;%f",dispshade);
	cmmd("shading correction;:k30%T",shading);mouse(RIGHTBT);
	while (thismu())
		;
	ttog = ottog; thresh = othresh;
	fgdigitise();
}

/*
 *	I T I M E F N  --  Set up integration time value and indicator string
 *
 *	val		0  - Off
 *			-1 - Automatic
 *			other - normal
 */
static int itimefn(new,old,val)
char	*new, *old;
int		*val;
{
	if (*new == 'A' || *new == 'a' || *new == '-') {
		*val = -1;
		strcpy(old,"Automatic");
	} else {					/* All other responses - just get number (default 0 - live */
		*val = atoi(new);
		if (*val <= 0) {
			strcpy(old,"Live");
			*val = 0;
		} else
			sprintf(old,"%d",*val);
	}
}


/*
 *	S W I T C H F L U O R  --
 *
 */
static int switchfluor(tog)
int		*tog;
{
	imageinvert = imageinvert ? 0 : 1;
	*tog = (imageinvert != alc->ddgschannel->invert);
/*	printf("invert %d, imageinv %d\n",alc->ddgschannel->invert,imageinvert); */
	cameratype_lut();			/* Select appropriate LUT */
	apply_lut(alc->lut->lut);	/* Now apply it */
}

/*
 *	M A R K T H R E S H
 *
 */
static 
markthresh(thres,scale)
int thres,scale;
{
	lwidth(5);
	moveto(thres*scale + 512,512);
	lineby(0,256);
	lwidth(1);
}

/*
 *	S E T F R A M E 
 *
 */
setframe(f,scale)
struct pframe *f;
int scale;
{
	f->type=60;
	f->scale=scale;
	f->ix=1;
	f->iy=1;
	f->ox=f->oy= 0;
	f->dx=f->dy= 0;
}


/*
 *	U P T H R E S H  --	Raise displayed threshold
 *
 */
upthresh()
{
	register int t;
	int lutval;
	UBYTE *gl;

	ttog = 0;
	t = thresh;
	thresh+=step;
	if (thresh > 255 ) thresh = 255 ;
	gl = alc->lut->lut->green_lut;

	if (imageinvert)	{
		for ( ; t < thresh ; t++) {
			if (alc->ddgschannel->invert) {
				lutval = 255 - *(gl+t);
			} else {
				lutval = *(gl+t);
			}
			setrgb(t, lutval, lutval, lutval);
		}
	} else {
		for ( ; t < thresh ; t++)
			setrgb(t,0,128,128);
	}
}

/*
 *	D O W N T H R E S H   --	Lower displayed threshold
 *
 */
downthresh()
{
	register int t;
	int lutval;
	UBYTE *gl;

	ttog = 0;
	t = thresh;
	thresh-=step;
	if (thresh < 0 ) thresh = 0;
	gl = alc->lut->lut->green_lut;

	if (imageinvert) {
		for ( ; t >= thresh ; t--) {
			setrgb(t,0,128,128);
		}
	} else {
		for ( ; t >= thresh ; t--) {
			if (alc->ddgschannel->invert) {
				lutval = 255 - *(gl+t);
			} else {
				lutval = *(gl+t);
			}
			setrgb(t, lutval, lutval, lutval);
		}
	}
}

/*
 *	S E T T H R E S H  --  
 *
 */
setthresh()
{
	register int i;
	
	ttog = 0; 			/* ensure automatic threshold is off */
	if (alc->kcn->FIP!=DIGFIP) apply_lut(alc->lut->lut);	/* Current LUT for 
	                                                       none fip image */

	if (imageinvert) {
		for (i=255;i>=thresh;i--) 
			setrgb(i,0,128,128);
	} else {
		for (i=0;i<=thresh;i++) 
			setrgb(i,0,128,128);
	}
}

/*
 *	T H R E S H O P S  --  
 *
 */
threshops(title)
char	*title;
{
	int		oldttog, stat;
	int		Xcpt_cmd, Quit_cmd;
	
	oldttog = ttog;
	setthresh();		/* highlight current level upon entry */
	newmenu(title);
	cmmd("set threshold;:k30<1/255>%d%f",&thresh,setthresh);
	cmmd("step size;:k30<1/255>%d",&step);
	cmmd("decrease by step size;%g",downthresh);mouse(LEFTBT);
	cmmd("increase by step size;%g",upthresh);mouse(RIGHTBT);
	cmmd("accept");mouse(MIDBT);Xcpt_cmd = mmcmd;
	Quit_cmd = cmmd("quit");mouse(LONGLB);
	while (1) {
		thismu();
		if (nncmd > 1) {
			displin(0,1);
			putprompt();
		}
		if (nncmd == Quit_cmd) {
			ttog = oldttog;		/* Quit - leave manthr as it was */
			stat = 0;
			break;
		} else if (nncmd == Xcpt_cmd) {
			stat = thresh;
			break;
		}
	}
	apply_lut(alc->lut->lut);	/* Apply current LUT */
	return(stat);
}

#ifdef HISTONLY
/*
 *	H I S T O N L Y 
 *
 */
 static histonly() {
	struct pframe f;

	fgfreeze();

	printf("Scanning for histogram, please wait ..\n");
	setframe(&f,16);
	f.dx=f.dy= 512;
	if (alc->ddgschannel->type==FS_VME512) {
		histo = fshisto(rectobj,255);
	} else {
		histo = fsframehisto(alc->ddgschannel->frame,
			alc->ddgschannel->lineln,alc->ddgschannel->lineno,255);
	}
	if (alc->debug) printf ("After getting histogram\n");

	histothreshsmooth(histo);
		
	/* only automatically calculate threshold if auto thresh on */
	if (ttog)
		thresh = compthr3(histo);
	
	if (alc->ddgschannel->type==FS_VME512) {
		histodup(histo);
	}
	histonormalise(histo);
	picframe(histo,&f);
	markthresh(thresh,16);
	freeobj(histo);
	go_on();		/* wait for histo to be looked at before going live again */
	fgdigitise();
}
#endif HISTONLY

/*
 *	F I L E P I C
 *
 */
static 
filepic() 
{
	int		i;

	while (1) {
		if (!getid(CELLFILE,WRITE)) return(0);
		alc->cell_h->dig_type = DIGTV;
		cellheadfill(alc->cell_h,&(alc->nameids));
		alc->cell_h->x_coord = 0 ;
		alc->cell_h->y_coord = 0 ;
		alc->cell_h->z_coord = 0 ;
		alc->cell_h->x_size = 0 ;
		alc->cell_h->y_size = 0 ;
		windmill("Writing cell - ");
		if (alc->debug)
			fprintf(debug_channel,"Filing cell to %s\n",alc->picname);
		if (putfile(alc->picname,alc->fipdig,kcont->ool,alc->lkount))
			break;
	}
	return(1);
}

/*
 *	G E T C E L L
 *
 */
static int 
getcell(fluor)
int		*fluor;
{
	if (fuseframes) {
		if (!check("Fusion images will be lost, proceed (M):"))
			return(0);
	}
	if (alc->cellin) {
		if (!check("Digitised image not stored, proceed (M):"))
			return(0);
		release(kcont->ool,&alc->lkount);
		alc->lbelld = 0;
		alc->cellin=0;
		kcont->maxnumber = 0;
		freefuse();
	}
	
	if (!digitise(*fluor)) return(0);
	if (alc->fipdig == NULL)
		fprintf(stderr,"Digitisation error, no cell generated\n");
	else {
		/*
		 * now we can use the conventional routines.
		 */
		printf("Normalising\n");
		if (lut_modify)
			apply_lut(lut_normalise(alc->fipdig, alc->lut->lut));
		else
			normalise(alc->fipdig);
		if (alc->debug) {
			fprintf (debug_channel,"Done normalise\n");
			elapsedtime();	
		}
		printf("Extracting chromosomes\n"); 
		label(alc->fipdig,&alc->lkount,kcont->ool,MAXOBJS,3);
		alc->cellin = 1;
		alc->lbelld = 1;
		if (alc->debug)
			elapsedtime();
		freeobj(alc->fipdig);
		alc->fipdig = NULL;
	}
	if (!(alc->options & OPT_FUSION))
		filecell();
	return(1);
}

/*
 *	G E T S E C O N D  --  Grab second and subsequent fields for field fusion
 *
 */
static int getsecond(fluor)
int		*fluor;
{
	if (!alc->cellin) {
		printf("No main cell image digitised yet\n");
		return(0);
	}
	if (fuseframes >= FUSEMAX) {
		printf("Can't hold any more images to fuse\n");
		return(0);
	}
	if (!digitise(*fluor)) return(0);
	if (alc->fipdig == NULL)
		fprintf(stderr,"Digitisation error, no cell generated\n");
	else {
		/*
		 * now we can use the conventional routines.
		 */
		printf("Normalising\n");
		if (lut_modify)
			apply_lut(lut_normalise(alc->fipdig, alc->lut->lut));
		else
			normalise(alc->fipdig);
		if (alc->debug) {
			fprintf (debug_channel,"Done normalise\n");
			elapsedtime();	
		}
		printf("Extracting chromosomes\n"); 
		label(alc->fipdig,&fusecount[fuseframes],fuseobj[fuseframes],MAXOBJS,3);
		fuseframes++;
		if (alc->debug)
			elapsedtime();
		freeobj(alc->fipdig);
		alc->fipdig = NULL;
	}
	return(1);
}


/*
 *	D I G I T I S E   --  Digitise field and generate woolz object
 *							in alc->fipdig
 *
 */
static int
digitise(fluor)
int		fluor;
{
	int		stat, i, j, background;

	apply_lut(alc->lut->lut);	/* Apply current LUT */

	if (!grab(fluor,integtime,"Please focus, proceed (M):"))
		return(0);
	if (shading) {
		printf("Applying shading correction\n");
		apply_shade(alc->ddgschannel->frame,imageinvert);
		if ((alc->cameratype & MASK) && !fluor)
			mask_image(MASKTOP+EDGE,MASKBOT+EDGE,MASKLEFT+EDGE,MASKRIGHT+EDGE,imageinvert ? 255 : 0);
		else
			mask_image(EDGE,EDGE,EDGE,EDGE,imageinvert ? 255 : 0);
	} else {
		if ((alc->cameratype & MASK) && !fluor)
			mask_image(MASKTOP,MASKBOT,MASKLEFT,MASKRIGHT,imageinvert ? 255 : 0);
	}
	if (ttog) {
		printf("Computing thresholds\n");
		if (alc->debug) {
			fprintf (debug_channel,"About to threshold\n");
			fflush(debug_channel);
		}
		switch (alc->ddgschannel->type) {
			case FS_VME512:
				histo = fshisto(rectobj,255);
				break;
			case FS_VIRTUOSO:
			case FS_VIRTUOSO_INVERT:
				histo = fsframehisto(alc->ddgschannel->frame,
					alc->ddgschannel->lineln,alc->ddgschannel->lineno,255);
				break;
		}
		if (alc->debug) {
			fprintf (debug_channel,"About to compute threshold\n");
			fflush(debug_channel);
		}
		thresh = compthr3(histo);
		freeobj(histo);
	}
	downmenu();
	stat = threshops("Check Threshold");
	upmenu();
	if (!stat)
		return(0);

	switch (alc->ddgschannel->type) {
		/*
		 * special routine fsthreshold on account of top-byte FF in VME-512
		 */
		case FS_VME512:

			alc->fipdig = fsthreshold(rectobj,thresh);
			if (alc->debug) fprintf (debug_channel, "Done threshold thres:%d area:%d\n",thresh,area(alc->fipdig));
		/*
		 * make a new type-21 grey-table not in frame store,
		 * and mask out the top byte FF once and for all.
		 */
			alc->fipdig = (struct object *) fsnewgrey(alc->fipdig);
			if (alc->debug) fprintf (debug_channel,"Done newgrey\n");
			break;

		/*
		 * special routine fsthreshold on account of bytes in virtuoso
		 */
		case FS_VIRTUOSO:
		case FS_VIRTUOSO_INVERT:
			if (alc->debug) {
				fprintf (debug_channel,"About to fsconstruct with threshold of %d\n",thresh);
				fflush(debug_channel);
			}
			printf("Removing background\n");
		
		/*
		 * empirically it has been show that as well as the expected half
		 * lines in lines 0 and max, which fsconstruct itself filters out, the
		 * bottom line of those remaining has an artificial near black interval
		 * spanning half the line. this too is removed therefore. bdp 7/8/87 
		 */
		
			Alloc_exit(fsmemerror);
			if (alc->debug)
				starttimer();
			alc->fipdig = fsconstruct(alc->ddgschannel->frame,
				alc->ddgschannel->lineno-1,alc->ddgschannel->lineln,thresh,imageinvert);
			if (alc->debug) {
				fprintf (debug_channel,"Done threshold thres:%d area:%d\n",thresh,area(alc->fipdig));
				fprintf (debug_channel,"Done fsconstruct\n");
				elapsedtime();
			}
			Alloc_exit(woolzexitfn);
			break;
	}
	return(1);
}

/*
 *	R E S H A D E  --  Reset shading array
 *
 */
static reshade(fluor)
int		*fluor;
{
	unsigned char	*sh;
	int		i;
	switch(alc->ddgschannel->type) {
		case FS_VIRTUOSO_INVERT:
		case FS_VIRTUOSO:
			printf("Please move to a blank field\n");
			if (grab(*fluor,integtime,"Proceed (M): ")) {
				printf("calculating new shading values\n");
				calc_shade(alc->ddgschannel->frame,imageinvert);
			}
			shading = 1;
			break;
		default:
			printf("Sorry - reshade does not function with this hardware\n");
	}
}

/*
 *	G R A B  --  Actually do business of grabbing an image
 *					Use appropriate scheme for type of prep (fluor etc)
 *
 *	Return 1 of OK, 0 if error or negative response ro proceed prompt
 */
grab(fluor,integtime,message)
int		fluor, integtime;
char	*message;
{
	char	tmpbuf[6];
	int		pd;
	if (fluor && (alc->cameratype & PULNIX) && integtime) {
		sprintf(tmpbuf,"/v%d\n",alc->ddgschannel->devno);
		if ((pd = open(tmpbuf,0)) == -1) {
			fprintf(stderr,"Failed to access display %s\n",tmpbuf);
			if (!oldpgrab(integtime,message))
				return(0);
			else
				lineaverage();
		} else {
			if (!pgrab(pd,integtime,message)) {
				close(pd);
				return(0);
			} else
				lineaverage();
			close(pd);
		}
	} else {
		fgdigitise();
		if (!check(message)) {
			fgfreeze();
			return(0);
		}
		fgfreeze();
	}
	return(1);
}

/*
 *	P G R A B  --  Grab pulnix image
 *
 *	Integtime - if -1 automatic, else use value set
 */
static int pgrab(pd,integtime,message)
int		pd,integtime;
char	*message;
{
	int		autoiris, done, field, lastitime;

	autoiris = (integtime == -1) ? 1 : 0;
	if (integtime <= 0) integtime = 20;
	integtime = integtime * 10 / 25;		/* Get it to be fields not ticks */
	printf("%s",message);
	vitime(pd,1);
	vwaitf(pd,&lastitime);
	vitime(pd,integtime | 1);
	field = vwaitf(pd,&lastitime);
/*	if (alc->devel) printf("Itime %d\n",integtime | 1); */
	lastitime = integtime;
	if (autoiris) {
		while ((done = gone_on()) < 0) {
			integtime = calcintegtime(integtime,field);
			if (integtime > 240) integtime = 240;		/* Pulnix sensible limit */
			vitime(pd,integtime | 1);
			field = vwaitf(pd,&integtime);
			if (alc->devel > 10) {
				if (field == -1)
					printf("error, integtime %d\n",integtime);
				else
					printf("integtime %d\n",integtime);
			}
		}
	} else
		done = go_on();
	vitime(pd,0);
	return(done);
}

static int oldpgrab(integtime,message)
int		integtime;
char	*message;
{
	int		autoiris, priority, pid, done, time, field, stime, newintegtime;
	autoiris = (integtime == -1) ? 1 : 0;
	pid = getpid();
	priority = getpr(pid);
	setpr(pid,200);
	printf("%s",message);
	field = grabonefield();
	tsleep(integtime);
	while ((done = gone_on()) < 0) {
		field = grabonefield();
		if (autoiris) {
			time = tick();
			newintegtime = calcintegtime(integtime,field);
			if (newintegtime > 600) newintegtime = 600;		/* Pulnix sensible limit */
			stime = newintegtime - (tick() - time);
			if (stime > 0) {
				integtime = newintegtime;
				tsleep(stime);
			} else {
				if (alc->devel) {
					printf("Integration - calculation took too long by %d at %d\n",-stime, integtime);
					integtime = tick() - time;			/* Include message time */
				} else
					integtime = newintegtime - stime;
			}
		} else
			tsleep(integtime);
	}
	setpr(pid,priority);
	return(done);
}

/*
 *	M A S K _ I M A G E  --  Mask off bad bits round edge of image
 *
 */
mask_image(top,bot,left,right,background)
int		top,bot,left,right,background;
{
	int		i,j;
	register unsigned char *pptr;
	switch (alc->ddgschannel->type) {
		case FS_VME512:
			break;
		case FS_VIRTUOSO:
		case FS_VIRTUOSO_INVERT:
			pptr = alc->ddgschannel->frame;
			for (i=0; i<top; i++) {
				for (j=0; j<alc->ddgschannel->lineln; j++) {
					*pptr++ = background;
				}
			}
			for (i=top; i<alc->ddgschannel->lineno-bot; i++) {
				pptr = alc->ddgschannel->frame + (i*alc->ddgschannel->lineln);
				for (j=0; j<left; j++) {
					*pptr++ = background;
				}
				pptr = alc->ddgschannel->frame + ((i+1)*alc->ddgschannel->lineln);
				for (j=0; j<right; j++) {
					*--pptr = background;
				}
			}
			pptr = alc->ddgschannel->frame + (alc->ddgschannel->lineno-bot) * alc->ddgschannel->lineln;
			for (i=0; i<bot; i++) {
				for (j=0; j<alc->ddgschannel->lineln; j++) {
					*pptr++ = background;
				}
			}
			break;
	}
}

/*
 *	I N I T I A L I S E W O O L Z
 *
 */
static
initialisewoolz()
{
	extern int packcheck;
	extern SMALL highlow;
	packcheck = 0;
	highlow = 1;
	if (alc->ddgschannel->type==FS_VME512){
		if (!rectobj)
			rectobj = makerect(0,511,0,511,alc->ddgschannel->frame
					,0,NULL,rectobj);
	}
}

/*
 *	R E S T O R E W O O L Z
 *
 */
static
restorewoolz()
{
	if (alc->ddgschannel->type==FS_VME512){
		if (rectobj) {
			freeobj(rectobj);
			rectobj=NULL;
		}
	}
}

/*
 *
 *	I N I T I A L I S E F G
 *
 */
static
initialisefg()
{
	if (alc->ddgschannel->type==FS_VME512){
		if (alc->debug) fprintf(debug_channel,"VME512 framestore\n");
		fgsetup();
		setfglut(1);
		setfgsync(0);
	} else {
		if (alc->debug) fprintf(debug_channel,"Virtuoso framestore\n");
	}
	cameratype_lut();			/* Select appropriate LUT */
	apply_lut(alc->lut->lut);	/* Now apply it */

	clear();
	if (alc->ddgschannel->type==FS_VME512)
		fast();
}

/*
 *	R E S T O R E D D G S  --
 *
 */
static restoreddgs()
{
	if (alc->ddgschannel->type==FS_VME512) {
		slow();
	}
	lin_colour_lut(); /* Apply linear LUT, and set overlay colours */
}

/*
 *	D I S P S H A D E  --
 *
 */
static dispshade()
{
	if (display_shade(alc->ddgschannel->frame,imageinvert)) {
		threshops("Shading display");
	}
	shading = 1;
}

/*
 *	D O H I S T O 
 *
 */
static int 
dohisto()
{
	fgdigitise();
	disp_histo(alc->ddgschannel->frame);
	/*
	 *	Histogram clears overlay, so need to re-draw menu for satellites
	 */
	if (alc->config & CONF_SAT) setdisplay(BUST);
}
	

/*
 *	F I L E C E L L  --  File all what we got, call fieldfuse 1st if
 *							any bits still to be merged
 *
 */
static int filecell()
{
	if (alc->cellin) {
		if (fuseframes) {
			fieldfuse();
		}
		addplist(kcont);				/* Add plists etc */
		filepic();
/*
 *	Tidy up
 */
		release(kcont->ool,&alc->lkount);
		alc->lbelld = 0;
		alc->cellin = 0;
		kcont->maxnumber = 0;
		freefuse();
	} else {
		fprintf(stderr, "There is no current cell to be filed!\n");
		putprompt();
	}
}

/*
 *	H I S T O D U P
 *
 *		Input histogram
 *		Output histogram
 *		Function duplicate even histogram entries in odd bins
 *		Usage CRS VME 512 only has 7 bit look up on input this gives
 *				strange histogram unless smoothed by this technique
 *
 */
static histodup(obj)
struct object *obj;
{
	register struct histogramdomain *hdom;
	register int i, *h;
	hdom = (struct histogramdomain *) obj->idom;
	h = hdom->hv;
	for (i=0; i<hdom->npoints; i+=2) {
		h[1] = h[0] = h[0]+h[1];
		h+=2;
	}
}


