/*
 *	A L L C O N T R . H		-  General incarnation wide data structure
 *
 *
 *  Written: Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *  Date:    
 *
 *  Modifications
 *
 *	24 Jun 1991		CAS			Motorised stage option
 *  2  Feb 1991		ih/gjp		Added an OPT_FLEX def
 *	17 Dec 1990		jimp@MRC	Added flexible screen pointer "flex_screen"
 *	20 Aug 1990		ih				Added a piabase field
 *  11 Jul 1990     ih				Added a digitised flag to complement fluor
 *	05 Apr 1990		dcb				Remove nseats, karycolour etc
 *	28 Mar 1990		dcb				Add bg_colour
 *	16 Mar 1990		dcb				Add satno field
 *	07 Mar 1990		dcb				TYPEMASK to mask off cameratype
 *	30 Jan 1990		dcb				Add lutfile, lut, luts
 *	12 Dec 1989		dcb				Add satprior
 *	11 Dec 1989		GJP				Added menu_option flag
 *	23 Oct 1989		dcb				Add mprior/sprior, and od cachesize
 *  06 Oct 1989     ARL             Added satellite configuration flag
 *	04 Sep 1989		dcb				Add arch_method & arch_type
 *	23 Aug 1989		dcb				Add system path no. of miproc channel
 *									and remove demo field
 *	16 Aug 1989		CAS				Fluorescence field
 *  16 May 1989		SEAJ			Added timing option.
 *  11 May 1989		SEAJ			Added culturesfile.
 *	 9 May 1989		dcb				rankcf simple pointer
 *	 5 May 1989		dcb				struct prepdata to tisdata, and stndata
 *									and make them pointers
 *	14 Mar 1989		SEAJ			Added matrixfile for variable enhancement.
 *	24 Jan 1989		CAS				More camera flags
 *	24 Jan 1989		dcb				New config  flag CONF_FIPAL to denote
 *									presence of new analogue front end card.
 *	13 Jan 1989		dcb				stage_type & lastname in allkontr
 *	28 Nov 1988		SCG				wclevel in allcontrol structure
 *	24 Oct 1988		CAS				Camera types
 *	 3 Oct 1988		CAS				Another h/c type + camera type field
 *	22 Aug 1988		CAS				More on options
 *	17 Aug 1988		CAS				New config options + s/ware options
 *	17 May 1988		CAS				Struct ptrs for screen/hardcopy layout
 *	21 Mar 1988		CAS				Added hook for intermachine transfer
 *	29 Jan 1988		CAS				Add slave configuration + mouse processor
 */
#ifndef ALLCONTR_H
#define ALLCONTR_H

#include <preps.h>
#include <fileheaders.h>
#include <ddgs.h>
#include <lut.h>

/*
 *	Configuration type flags
 */
#define CONF_FIP	0x001	/* FIP hardware is present  */
#define CONF_TV		0x002	/* we are using TV digitisation  */
#define CONF_MLR	0x004	/* Enable Met list review */
#define CONF_KTYPE	0x008	/* Allow Karyotyping */
#define CONF_FIPAL	0x010	/* V2 analogue card present */
#define	CONF_SAT	0x020	/* Seat is satellite configured */
#define CONF_MSTAGE	0x040	/* Seat has motorised stage */

/*
 *	Options Flags
 */
#define OPT_FUSION	0x001	/* Field Fusion */
#define OPT_FLUOR	0x002	/* Fluorescence */									
#define OPT_MANKAR	0x004	/* Manual Karyotype */
#define OPT_FLEX	0x008	/* Flexible screen */

/*
 *	Camera types + flags
 */
#define DAGE68	0x0001
#define PULNIX	0x0002
#define	MASK	0x0100
#define TYPEMASK 0xf	/* Mask off camera type only */
/*
 *	Slave stuff
 */
#define MAXSLAVE 8
#define SL_USE 1			/* If set use this slave */

/*
 *	Language values
 */
#define ENGLISH	1
#define GERMAN	2

/*
 *	Hardcopy types
 */
#define HC_VGR 1
#define HC_DIR300 2
#define HC_DIR300_QUE 3
#define HC_DIR300_OBJ 4

/*
 *	CmdSys logging
 */
#define MONITOR_PERCASE 1
#define MONITOR_ALL 2

/*
 *	Archive types
 */
#define OD_ARCH		1
#define TAPE_ARCH	2
#define FLOPPY_ARCH	4

/*
 *	Menu options
 */
#define TVSTAY 1	/* Stay in tvigitise menu */

/*
 *	Misc String lengths
 */
#define TSIZ 10
#define DIRSIZ 40

struct allkontr {
	int		seat;				/* Seat number */
	int		maxbays;			/* Number of bays in stage */
	int		scantop,			/* Physical stage scan limits */
			scanbot,
			scanleft,			/* left/right limit of slide */
			scanright;
	int		stage_type;			/* Different stages, and mods. to them */
	int		config;				/* If bits set then these parts of the 
									hardware are present */
	int		menu_options;		/* Particular configurable menu options */
	int		options;			/* Enable/Disable s/ware options */
	int		language;			/* Text display language */
	int		fluor;				/* Fluorescence image ? */
	int		digitised;			/* Digitised image or filed? */
	int		kmr_pid,			/* Store process id's for signalling etc */
			mf_pid;
	int		mprior,				/* Master priority (kmr/mf) */
			sprior,				/* Slave priority */
			satprior;			/* Satellite priority */
	int		satno;				/* Satellite no */
	char	miprocterm[TSIZ],	/* Miproc terminal line */
			ourterm[TSIZ],		/* Our terminal line */
			mouseterm[TSIZ],	/* Our mouse line */
			printerterm[TSIZ],	/* Printer line */
			debugfile[40],		/* File (or terminal) to write debug data to */
			lutfile[40];		/* Look up table info */
	int		miprocbaud;			/* Miproc baud rate */
	int		miproc;				/* System path number of miproc channel */
	int		piabase;			/* Base address of pia */
	int		fstore;				/* Our frame store number */
	int		fstoretype;			/*  and its type */
	int		cameratype;			/* Type of camara we are using */
	int		nslaves;			/* Number of slaves in use */
	int		slaveconf[MAXSLAVE];/* Slave processor configuration */
	int		mouseproc;			/* Processor to run mouse process on */
	DDGS	*ddgschannel;		/* Information about current ddgs device */
	struct	lut_assoc *lut,		/* The current look_up_table */
			*luts;				/* Available set of LUTS */
	struct	transfer *trans;	/* Info about machine to transfer data to */
	int		usergroupnum,		/* Add new user option select */
			onegroup;
	int		devel,				/* replacement for demo... */
			debug,				/* Debug level (0=no debug) */
			timing,				/* Timing option (0=Off) */
			wclevel,			/* object checking in woolz */
			monitor;			/* If set write a log of whats done
									(soak test evaluation etc) */
	struct cyto_file *lastname;	/* Last case name to delete last log file */
	int		mipdebug;			/* Debug miproc comms */
	struct rgb bg_colour;		/* Background colour */
	int		mousescale;			/* Default mouse movement scale */
	char	fnamesep1,			/* file name '_' translation char */
			fnamesep2;			/* file name '.' translation char */

	char	sitename[LABLENGTH+1];	/* Name of this laboratory/site */
	int		userno;					/* User id number (10.10 etc) */
	char	user[OPERLENGTH],		/* User name */
			dir[DIRSIZ];			/* working directory */
	char	arch_method[TEXTLENGTH];/* Archiving method - general length */
	int		arch_type;				/* Archiving type(s) */
	int		cachesize;				/* OD cache size */
	struct	cell_header *cell_h;	/* File header storage */
	struct	cell_header *kary_h;
	struct	gdat_header *gdat_h;
	struct	cyto_file nameids;		/* Holds name ids */
	struct	cyto_types typeids;		/* Holds stain/tissue ids */
	struct	screenform	*scrf;		/* Screen ktype format */
	struct	hcopyform	*hcpf;		/* Hardcopy ktype format */
	char	matrixfile[80];			/* Filename for variable enhancement data matrices */
	char	culturesfile[80];		/* Filename for cultures file */
	char	kscorer[SCORERLENGTH];	/* Scorer id */
	int		toxicmu;				/* Long or short score menu */
	int		ccoord;					/* Coordinate conversion flag */

	char	metfile[28];			/* Composed working metlist filename */
	char	picname[28];			/* Composed working cell/kary filename */
	char	mydate[DATELENGTH];		/* ie 31st November 1986\0 */
	int		day,month,year;
	char	revdate[DATELENGTH];	/* review date */
	struct	mult_slide	*frsbay,	/* ptr to first bay structure */
						*curbay;	/* ptr to current bay structure */
	int		killscan;				/* interrupt handler flag for metscan */
	struct 	object 	*fipdig;		/* Pointer to digitised object */

	int 	lkount;					/* count of segmented objects - dbuf */
	int 	numchroms;				/* number of chromosomes counted*/
	int 	cellin;					/* signal: a cell has been read in and 
										not freed */
	int 	lbelld;					/* signal: the current cell has been 
										labelled */
	int 	*cellkounts;			/* array of cell counts */
	int 	*cellinfo;				/* array of other cell info */

/*
 *	Assoc structures
 */
 	struct 	kcontrol 	*kcn;		/* pointer to kcontrol structure */
	struct 	m2mouse 	*m2s;	 	/* pointer to mouse structure */
	struct	rankcf		*rankcf;	/* Pointer to ranking coefficients. */
	int		num_tissues,			/* Number of tissues in tables */
			num_stains;				/* Number of stains in tables */
	struct	tisdata *tissues;		/* Id's and strings for tissues/stains */
	struct	stndata *stains;
	struct cfipparams *cfipp;		/* Copy of current combifip params */
	struct hardcopy *hcpy;			/* Hardcopy data */
	struct flexible_screen *flex_screen;
/*	int 	*maywant[10];		   and when we do - we can put them in */

};

#endif ALLCONTR_H

