/*
 *      C F I P P . H  --  Combifip data
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *  Date:    18th August 1986
 *
 *  Modifications
 *
 *	13 Jan 1988		SCG			Added tholds structure
 *	06 Aug 1987		SCG			min/max scan speed defines
 *	28 Jul 1987		CAS			Added high res + define of UNSET
 *	23 Jul 1987		CAS			Added autofocus + position fields
 *	20 Jul 1987		CAS			Added focus + move/scan speed 
 *
 */

#define UNSET -1
#define MINSCANSPEED 100
#define MAXSCANSPEED 7353

/*
 *	Structure to contain a mirror of the state of miproc combifip parameters
 *	Plus one or two things not held by the miproc but wanted in the same context
 */
struct cfipparams {
	char	program[10];		/* Current program name extension */
	char	checkarray[40];		/* Load check array */
	int		pversion;			/* Version number */
	int		speed;				/* Current move speed */
	int		movespeed;			/* Default x-y move speed */
	int		scanspeed;			/* Default scan speed */
	struct tholds {
			int		thresh1;	/* Lower miproc thold */
			int		thresh2;	/* Upper miproc thold */
			int		dmthresh1;	/* Lower default met-scan thold */
			int		dmthresh2;	/* Upper default met-scan thold */
		} tholds; 
	struct focus {
			int		permove;	/* Steps per focus move */
			int		inswathe;	/* inswathe steps per mm */
			int		between;	/* between swathe steps per mm */
			int		excursion;	/* Total focus excursion */
		} focus;
	int		xs,ys,zs;			/* Current stage position */
	int		autofocus;			/* Is autofocus ON/OFF ? */
	int		highres;			/* High res mode */
	int		afeinterpolate;		/* Devel afe 1/2 res pixel mode */
};

