/*
 * 	S L I D E S . H --
 *
 * Modifications
 *
 *	24 Aug 1989		dcb		Plant a "newlist" field in mult_slides for
 *							recording when a new list is created/read in
 *	12 May 1989		dcb/SEAJ include LINHEAD in mult_slides also include class.h
 *	24 Jan 1989		SCG/dcb	include struct scores
 *  29 Sep 1988		SEAJ	Added in coordinate conversion defines
 *	06 May 1988		SCG		Modified cluster classifier params in the
 *							bay structure.
 *  21 Apr 1988		SEAJ	Changed thflag to be an integer not char.
 *	13 Aug 1987		SCG		Normal/Abnormal taken out
 *	06 Aug 1987		SCG		Scan speed in mult_slide structure
 *	24 Jul 1987		CAS		Put fields in box structure back the old way round
 *							so all the initialisation works...(*%$@@!!!)
 *	23 Jul 1987		CAS		Remove 'focs' from bay struct
 *							move cyto_types + prepdata to alc to improve
 *							 header dependancy problems
 *	21 Jul 1987		CAS		Remove focus params from bay structure (ferr1,2,stepm
 *	15 Jul 1987		SCG		Add focus done in bay struct (focN38/focbox)
 *	15 Jul 1987		CAS		Maxtissues to 7 max stains to 3
 *	 9 Jul 1987		SCG		Removed redundant defines and structs
 *	22 May 1987		SCG		Major mod. Introduced structs mipclust,
 *							mlclust, mult_slide, metlink, cyto_types.  
 *							Moved structs cyto_file, and box from elsewhere
 *							for four-slide stage.
 *	 5 Feb 1987		CAS		Moved slide stuff here from kyy
 *	30 Jan 1987		SCG		Defines for multiple slide scans
 *	16 Jan 1987		SCG		Max metaphase number increased to 2000.
 *	28/4/86     Denis       re-arrangement of xval,yval; intro xwide,
 *							ywide
 */
#include <preps.h>
#include <class.h>
#include <fileheaders.h>

/*
 *	Stage/slide stuff
 *
 *	X - Y position (screen coords)
 */
#define SLMYTOP 4000
#define SLMYBOT 490
#define SLMXBOT 1750
#define SLMXTOP 2950
/*
 *	Size of slide on screen (screen coords)
 */
#define SLMWIDE (SLMXTOP-SLMXBOT)	/* Width */
#define SLMHIGH (SLMYTOP-SLMYBOT)	/* */
#define FROSTH  880				/* Height of frosted/label bit */
/*
 *	Scan limits (screen coords)
 */
#define BOTLIM 1100
#define TOPLIM 3185				/* slide simulation limits */
/*
 *	slide -> box translation params
 */
#define XMID 2350
#define YMID 2245 				/* mid of tv slide equal to NE finder N38 */
#define CONV 21.4 				/* conv factor from ddgs units to microns */

#define XSCALE 		4	/* Digs units per object col drawn along x-axis. */
#define YSCALE 		3	/* Digs units per object row drawn along y-axis. */

/*
 *	data structures for Miproc met record
 */
#define COMLONG 241				/* max length of cluster record */
#define MAXREC 2000				/* max number of permitted cluster records */
#define AV_XOFFSET 45018 		/* Default offset between bays */
#define AV_YOFFSET 0	 		/* Default offset between bays */
#define AV_ZOFFSET 0	 		/* Default offset between bays */

/*
 * defines for coordinate conversion
 */
#define CONVOFF 	0	/* coordinate conversion off */
#define MICROSCOPE 	1	/* convert to microscope verniers */
#define ENGLISHNE	2	/* convert to English england finder */
#define SWEDISHNE	3	/* convert to Swedish england finder */

/*
 *
 *   M I P C L U S T   M L C L U S T   C L U S T D A T
 *
 *			mipclust - records as sent from the miproc
 *			mlclust - records as held in the Mizar
 *			clustdat - old Mizar records, kept for compatibility
 *
 */
struct mipclust {
	
/*  0 */	short a;
/*  2 */	short mxpos;		/* y pos on slide after Mz reset */
/*  4 */	short b;			/* cluster x extent after Mz reset */
/*  6 */	short mypos;		/* cluster y extent after Mz reset */
/*  8 */	short zval;
/* 10 */	short nobj;
/* 12 */	short sf;
/* 14 */	short aobj;
/* 16 */	short octa;
/* 18 */	char skt;
/* 19 */	char dv;			/* cluster density variation */
/* 20 */	char inbox;			/* copied here by Mz from Miproc qscore */
/* 21 */	char meanod;
/* 22 */	unsigned char mfscore;
/* 23 */	char qscore;
/* 24 */	short mcom;
/* 26 */	short mxwide;
/* 28 */	short mywide;
/* 30 */	char c1[2];

};

struct mlclust {
	
/*  0 */	short xval;			/* x pos on slide, after Mz reset */
/*  2 */	short yval;			/* y pos on slide after Mz reset */
/*  4 */	short xwide;		/* cluster x extent after Mz reset */
/*  6 */	short ywide;		/* cluster y extent after Mz reset */
/*  8 */	short zval;
/* 10 */	short nobj;
/* 12 */	short sf;
/* 14 */	short aobj;
/* 16 */	short octa;
/* 18 */	char skt;
/* 19 */	char dv;			/* cluster density variation */
/* 20 */	char inbox;			/* copied here by Mz from Miproc qscore */
/* 21 */	char meanod;
/* 22 */	short qrank;		/* Miproc: char mfscore/qscore  */
/* 24 */	short com;			/* pointer to comment rel addr (or zero) */
/* 26 */	char fclass;		/* see bit-codes , above [ Miproc xwide ] */
/* 27 */	char quality;		/* user set */
/* 28 */	char udic,ufrag,uring; /* [Miproc ywide ] */
/* 31 */	char chrom;
/* 32 */	short recnr;
/* 34 */	short groupno;

};

struct clustdat {
	
/*  0 */	short xval;			/* x pos on slide, after Mz reset */
/*  2 */	short yval;			/* y pos on slide after Mz reset */
/*  4 */	short xwide;		/* cluster x extent after Mz reset */
/*  6 */	short ywide;		/* cluster y extent after Mz reset */
/*  8 */	short zval;
/* 10 */	short nobj;
/* 12 */	short sf;
/* 14 */	short aobj;
/* 16 */	short octa;
/* 18 */	char skt;
/* 19 */	char dv;			/* cluster density variation */
/* 20 */	char inbox;			/* copied here by Mz from Miproc qscore */
/* 21 */	char meanod;
/* 22 */	short qrank;		/* Miproc: char mfscore/qscore  */
/* 24 */	short com;			/* pointer to comment rel addr (or zero) */
/* 26 */	char fclass;		/* see bit-codes , above [ Miproc xwide ] */
/* 27 */	char quality;		/* user set */
/* 28 */	char udic,ufrag,uring; /* [Miproc ywide ] */
/* 31 */	char chrom;

};
/*
 *  M E T L I N K
 *
 *		Link structure for metaphase list
 *
 */
struct metlink {

	struct	metlink *nrank, *prank,		/* Rank pointers. */
					*nsub, *psub,		/* Subset pointers. */
					*ngrp, *pgrp;		/* Group pointers. */
	struct	mlclust *mlrec;				/* Metaphase rec pointer. */
	
};

/*
 *  B O X
 *
 */
struct box {
	int	bbot, btop,
		bleft, bright;
	
};

/*
 * S C O R E S --- keep a running total of the fragile x scores
 */
struct scores {
	int sctot;
	int fxyes;
	int fxno;
	int fxmaybe;
};
	
/*
 *  M U L T _ S L I D E
 *
 *		This structure hold bay and slide relevant info.
 *
 */
struct mult_slide {

	int active;						/* Bay on/off for scanning. */

	struct scores *scp;				/* keeps a running total of frax scores */	
	struct cyto_file nameids;		/* Holds name id's. */
	struct cyto_types typeids;		/* Holds stain/tissue id's. */
	LINHEAD *lhd;					/* Linear classifier for this bay */	
	struct box mbox;				/* Box size for this slide. */
	int  xlow, xhigh, ylow, yhigh;	/* Scan x-y limits. */
	int  stop, sbot, sleft, sright;	/* Slide x-y limits. */
	int	 slwidth;					/* Slide width */
	int  xoff, yoff, zoff;			/* Offset to next bay datum. */
	int  xcpoff, ycpoff, zcpoff;	/* Capture offsets. */
	int  focN38, focbox;			/* Focus set at N38 + box centre */
	int  focslc;					/* Focus offset to slide centre. */
	int  focbxc;					/* Focus offset to box centre. */
	
	int  totin;						/* Number of records in list. */
	int  usbltot;					/* Number of usables in list. */
	int  revtot;					/* Number reviewed in list. */
	int  newlist;					/* Set when new list created */
	char	*compos, *comtop,		/* Comment pointers */
			*combot;
	
	struct metlink					/* METAPHASE LIST POINTERS */
			*startptr, *currptr,	/* Ptr to first and cur. chain. */
			*frankptr,				/* Ptr to first rank in list. */
			*fsubptr, *csubptr,		/* Ptr to first and cur. submemb. */
			*fgrpptr, *cgrpptr;		/* Ptr to first and cur. group memb. */
			
	struct {						/* SUBSET DISCRIMINANTS */
		char  classd[8], juu[8],
			  uubl[8], unum[8];
	} wh;
	
	struct {
		char  dicc[8], fragg[8],
			  ringg[8], inbx[8];
	} whb;
	int		allrecs, listlow,
			listhigh, minrank,
			maxrank, sany, ngrps,
			byg;
									/* SCAN PARAMETERS */		
	int	thresh1, thresh2;			/* Thresholds for scan. */
	int thflag;						/* Manual thold flag. */
	int	shadsw;						/* Shading correction. */
	int	drange;						/* Density range. */
	int	ygate;						/* Scan array setting. */
	int	retrain;					/* Miproc classifier off/on. */
	int	mcells;						/* Number of candidates. */
	int	mswathe;					/* Number of swathes. */
	int	sspeed;						/* Scan speed. */
	
									/* CLASSIFIER PARAMETERS */
	int xmin, xmax,
		countmin, countmax,			/* cluster count lims */
		careamin, careamax,			/* cluster area lims */
		octareamin, octareamax,		/* octagon area lims */
		codmin, codmax,				/* cluster od lims */
		dvmax,						/* density variation; only maximum */
		sfmax,						/* shape factor; only maximum */
		skodmax;					/* sk/od feature; only maximum */
		
	int iod, clus1, clus2, area;	/* limb clustering params; untrainable */
	
};
