/*
 * flex_curs.c		Jim Piper and Bill Hill @MRC		22-11-90
 *
 * Cursor driving and identification routines for the flexible screen.
 * Because of CMDSYS limitations I think that static-global variables
 * are unavoidable here.
 * The routines here are modelled loosely on those in kycount.c and
 * cursmove.c.
 * 
 * MODIFICATIONS
 * -------------			
 * 27-06-91 ih		Check FS->stat exists before using it
 * 30-01-91 joan	flex_curs2(), extra function added so that box 
 *			cannot fall off the screen.
 * 29-01-91 jimp	flex_curs3() things extended to include boxes.
 * 05-12-90 jimp	flex_curs3() things added.
 * 28-11-90 jimp	Simplifications and rearrangements.
 *			flex_curs2() things added.
 *
 */
#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <cmdsys.h>
#include <display.h>
#include <fileheaders.h>
#include <flex.h>


/*
 * static-global variables used by the parameterless functions
 * passed by setmsv() to CMDSYS for flexible screen identification
 */
static int flex_xtop = 6144;
static int flex_ytop = 4600;
static struct flexible_screen *FS;
static struct ipoint flex_cursor = {40,6,0,0};
static struct ivector flex_rect_diag = {30,0,0,0,0,0};
static struct pframe flex_curs_f = {60,1,0,0,0,0,1,1,NULL};
static struct m2mouse *flex_mouse;
static struct object *flex_poly;
#define flex_mousescale 32
#define flex_curscolour OVERLAY2

/*
 * Get the cursor position in clean fashion
 */
flex_curs_pos(x,y)
int *x, *y;
{
	*x = flex_cursor.k;
	*y = flex_cursor.l;
}


/*
 * Get the maximum x and y screen values
 */
flex_screen_limits()
{
/*	*flex_xtop = ;
	*flex_ytop = ;	*/
}
/*
 * This function initialises identification in a flexible screen
 * by setting up static-global variables used by the parameterless
 * functions below, and drawing the cursor in the center of the
 * screen.
 */
flex_curs1_init(fs)
struct flexible_screen *fs;
{
	FS = fs;
	if (FS->stat)
		FS->stat->current_obj = NULL;
	flex_cursor.k = flex_cursor.l = 2048;
	flex_mouse = mukont->mouss;
	intens(1);
	colour(flex_curscolour);
	picframe(&flex_cursor,&flex_curs_f);
}

/*
 * This function ends identification in a flexible screen -
 * essentially it only deletes the cursor, any displayed
 * boxes, and resets fs->stat->current_obj
 */
flex_curs1_end()
{
	intens(0);
	colour(flex_curscolour);
	if (FS->stat&&(FS->stat->current_obj != NULL)) {
		flex_draw_boxes_nd(FS->stat->current_obj);
		FS->stat->current_obj = NULL;
	}
	picframe(&flex_cursor,&flex_curs_f);
	intens(1);
}

/*
 * This function is passed as an argument to setmsv() to set up
 * mouse-driven identification in the flexible screen.  Note that there
 * is NO identification in the menu - it's hard with a recursive structure!
 */
flex_curs1()
{
	struct compounda *id_obj, *flex_objident_fs();

	if (flex_mouse->moving)
		mousecurs(&flex_cursor,&flex_curs_f,flex_mouse,flex_mousescale,flex_curscolour);
	else {
		id_obj = flex_objident_fs(FS,&flex_cursor);	/* is cursor over an object ? */
		if (FS->stat&&(FS->stat->current_obj != id_obj)) {
			colour(flex_curscolour);
			intens(0);
			flex_draw_boxes_nd(FS->stat->current_obj);
			FS->stat->current_obj = id_obj;
			intens(1);
			flex_draw_boxes_nd(id_obj);
		}
	}
}


/*
 * This function identifies a node or leaf object by comparing the
 * bounding irect with the cursor -- both are in screen coordinates
 */
struct compounda *flex_objident_fs(fs,curs)
struct flexible_screen *fs;
struct ipoint *curs;
{
	struct compounda *ca, *flex_objident_nd();
	struct compounda *hit_obj = NULL;
	int i,d,mindist = 1000000;

	for (i=0; i<fs->cao->n; i++) {
		ca = flex_objident_nd(fs->cao->o[i],curs,&d);
		if (ca != NULL) {
			if (d < mindist) {
				hit_obj = ca;
				mindist = d;
			}
		}
	}
	return(hit_obj);
}

/*
 * Check whether inside the irect for this node.
 * If so, find distance to irect centre, and return the node
 * and distance UNLESS recursive call to this function returns
 * a node higher up tree, in which case return that instead.
 */
struct compounda *flex_objident_nd(ca,curs,d)
struct compounda *ca;
struct ipoint *curs;
int *d;
{
	struct compounda *rca,*mca;
	int i,mindist,rd,kd,ld;
	struct irect *r;

	if (ca == NULL || ca->type != 81)
		return(NULL);
	if (ca->o[0] == NULL || ca->o[0]->type != 20)
		return(NULL);
	r = (struct irect *) ca->o[0]->idom;
	if (r->irl[0] > curs->l || r->irl[2] < curs->l || r->irk[0] > curs->k || r->irk[2] < curs->k)
		return(NULL);
	/*
	 * A hit on this irect -- compute max of (x,y) distance to centre
	 */
	mca = ca;
	kd = curs->k - (r->irk[0] + r->irk[2])/2;
	if (kd < 0)
		kd = -kd;
	ld = curs->l - (r->irl[0] + r->irl[2])/2;
	if (ld < 0)
		ld = -ld;
	mindist = kd;
	if (mindist < ld)	/* remember, MAX of (x,y) distance to centre */
		mindist = ld;
	/*
	 * look up the tree for a better fit
	 */
	for (i=1; i<ca->n; i++) {
		rca = flex_objident_nd(ca->o[i],curs,&rd);
		if (rca != NULL) {
			if (rd < mindist) {
				mca = rca;
				mindist = rd;
			}
		}
	}
	*d = mindist;
	return(mca);
}


/*
 * This function initialises a moving-box cursor for moving
 * the current node.  The moving box is stored by its diagonal vector.
 */
flex_curs2_init(fs)
struct flexible_screen *fs;
{
	struct compounda *ca = fs->stat->current_obj;
	struct irect *r;

	if (ca == NULL) {
		fprintf(stderr,"flex_curs2_init: nothing identified!\n");
		return(0);
	}
	r = (struct irect *) ca->o[0]->idom;
	flex_rect_diag.k2 = r->irk[2];
	flex_rect_diag.l2 = r->irl[2];
	flex_rect_diag.k1 = r->irk[0];
	flex_rect_diag.l1 = r->irl[0];
	flex_mouse = mukont->mouss;
	intens(1);
	colour(flex_curscolour);
	flex_pic_box_curs(&flex_rect_diag,&flex_curs_f);
}

/*
 * This function ends the moving-box cursor, returning final position
 */
flex_curs2_end(x,y)
int *x, *y;
{
	intens(0);
	colour(flex_curscolour);
	flex_pic_box_curs(&flex_rect_diag,&flex_curs_f);
	*x = flex_rect_diag.k2;
	*y = flex_rect_diag.l2;
}

/*
 * This function is passed as an argument to setmsv() to set up
 * mouse-driven object movement in the flexible screen, moving
 * a small box at each corner of a rectangle, stored by its
 * diagonal as a Woolz ivector structure.  When movement stops,
 * the rectangle itself is drawn.
 */
flex_curs2()
{
/*
	if (flex_mouse->moving)
*/
		flex_mousetrack2(&flex_rect_diag,&flex_curs_f,flex_mouse,flex_mousescale,flex_curscolour);
/*
	else {

	}
*/
}


/*
 * Track the mouse movement with a cursor.
 * Same logic as "mousecurs" (ident.c) but different display.
 */
flex_mousetrack2(cv,f,m,scale,curscolour)
register struct ivector *cv;
register struct pframe *f;
register struct m2mouse *m;
register int scale, curscolour;
{
	register int xscale,yscale,div;
	static xrem = 0, yrem =0;
	if (scale >= 0)
		div = 1;
	else {
		div = -scale;
		scale = 1;
	}
	xscale = yscale = scale;
	if (f->ix < 0)
		xscale = -xscale;
	if (f->iy < 0)
		yscale = -yscale;
	colour(curscolour);
	intens(0);
	flex_pic_box_curs(cv,f);
	if (div > 1) {
		xrem += m->XINC * xscale;
		cv->k1 += xrem/div;
		cv->k2 += xrem/div;
		xrem %= div;
		yrem += m->YINC * yscale;
		cv->l1 += yrem/div;
		cv->l2 += yrem/div;
		yrem %= div;
	}
	else {
		cv->k1 += m->XINC * xscale;
		cv->k2 += m->XINC * xscale;
		cv->l1 += m->YINC * yscale;
		cv->l2 += m->YINC * yscale;
	}
	flex_check_limits(cv);
	intens(1);
	flex_pic_box_curs(cv,f);
}

/*
 * make sure that moving box stays on the screen
 */
flex_check_limits(cv)
register struct ivector *cv;
{ 
	register boxwidth, boxlength;
	boxwidth = cv->k2 - cv->k1;
	boxlength = cv->l2 - cv->l1;
	if (cv->k1 < 40) {
		cv->k1 = 40;
		cv->k2 = boxwidth + 40;
	}
	if (cv->l1 < 40) {
		cv->l1 = 40;
		cv->l2 = boxlength + 40;  
	}
	if (cv->k2 > flex_xtop - 40) {
		cv->k2 = flex_xtop - 40;
		cv->k1 = flex_xtop - boxwidth - 40;
	}
	if (cv->l2 > flex_ytop - 40) {
		cv->l2 = flex_ytop - 40;
		cv->l1 = flex_ytop - boxlength - 40;
	}
}


/*
 * draw a cursor for a moving box
 */
flex_pic_box_curs(cv,f)
struct ivector *cv;
struct pframe *f;
{
	struct ivector v;
	v.type = 30;
	lwidth(2);

	v.k1 = cv->k1;
	v.l1 = cv->l1;
	v.k2 = v.k1 - 48;
	v.l2 = v.l1 - 48;
	picframe(&v,f);

	v.l1 = cv->l2;
	v.l2 = v.l1 + 48;
	picframe(&v,f);

	v.k1 = cv->k2;
	v.k2 = v.k1 + 48;
	picframe(&v,f);

	v.l1 = cv->l1;
	v.l2 = v.l1 - 48;
	picframe(&v,f);

	lwidth(1);
}


/*
 * This function initialises either a fixed-point arrow cursor (mode 1)
 * or a fixed-end box cursor (mode 2), of specified line width.
 */
flex_curs3_init(fs,x,y,mode,w)
struct flexible_screen *fs;
int x, y, mode, w;
{
	struct object *makemain();
	struct polygondomain *pdom, *makepolydmn();
	register struct ivertex *vtx;
	struct flex_poly_plist *plist;

	pdom = makepolydmn(1,NULL,0,5,1);
	pdom->nvertices = 5;
	vtx = pdom->vtx;
	flex_cursor.k = vtx[0].vtX = x + 64;
	switch (mode) {
		case 1:		/* arrow */
			vtx[0].vtY = y;
			vtx[1].vtX = x;
			vtx[1].vtY = y;
			flex_arrow_head(vtx);
			break;
		case 2:
			vtx[0].vtY = y + 64;
			vtx[1].vtX = vtx[2].vtX = x;
			vtx[2].vtY = vtx[3].vtY = y;
			flex_poly_box(vtx);
			break;
	}
	plist = (struct flex_poly_plist *) Malloc(sizeof(struct flex_poly_plist));
	plist->size = sizeof(struct flex_poly_plist);
	plist->linewidth = w;
	plist->polytype = mode;
	flex_poly = makemain(10,pdom,NULL,plist,NULL);
	fs->stat->current_obj = (struct compounda *) flex_poly;
	colour(flex_curscolour);
	intens(1);
	lwidth(w);
	picframe(flex_poly,&flex_curs_f);
}

/*
 * This function is passed as an argument to setmsv() to set up
 * mouse-driven object movement in the flexible screen, moving
 * the tail of an arrow whose point is fixed -- held as polygon.
 */
flex_curs3()
{
	flex_mousetrack3(flex_poly,&flex_curs_f,flex_mouse,flex_mousescale,flex_curscolour);
}

flex_curs3_end()
{
	if (FS->stat&&(FS->stat->current_obj != NULL)) {
		intens(0);
		colour(flex_curscolour);
		picframe(FS->stat->current_obj,&flex_curs_f);
		lwidth(1);
	}
}

/*
 * Track the mouse movement with a either fixed-point arrow cursor
 * or a fixed-end box (determined by whether 5-vertex or 2-vertex polygon).
 * Same logic as "mousecurs" (ident.c) but different display.
 */
flex_mousetrack3(p,f,m,scale,curscolour)
struct object *p;
register struct pframe *f;
register struct m2mouse *m;
int scale, curscolour;
{
	register int xscale,yscale,div;
	struct polygondomain *pdom = (struct polygondomain *)p->idom;
	register struct ivertex *vtx = pdom->vtx;
	static xrem = 0, yrem =0;
	if (scale >= 0)
		div = 1;
	else {
		div = -scale;
		scale = 1;
	}
	xscale = yscale = scale;
	if (f->ix < 0)
		xscale = -xscale;
	if (f->iy < 0)
		yscale = -yscale;
	colour(curscolour);
	intens(0);
	picframe(p,f);
	if (div > 1) {
		xrem += m->XINC * xscale;
		vtx[0].vtX += xrem/div;
		xrem %= div;
		yrem += m->YINC * yscale;
		vtx[0].vtY += yrem/div;
		yrem %= div;
	}
	else {
		vtx[0].vtX += m->XINC * xscale;
		vtx[0].vtY += m->YINC * yscale;
	}
	switch (((struct flex_poly_plist *)p->plist)->polytype) {
		case 1:
			flex_arrow_head(vtx);
			break;
		case 2:
			flex_poly_box(vtx);
			break;
	}
	intens(1);
	picframe(p,f);
}

flex_arrow_head(vtx)
register struct ivertex *vtx;
{
	int xdiff,ydiff;
	xdiff = (vtx[0].vtX - vtx[1].vtX)/3;
	ydiff = (vtx[0].vtY - vtx[1].vtY)/3;
	vtx[2].vtX = vtx[1].vtX + xdiff - ydiff;
	vtx[2].vtY = vtx[1].vtY + ydiff + xdiff;
	vtx[3].vtX = vtx[1].vtX;
	vtx[3].vtY = vtx[1].vtY;
	vtx[4].vtX = vtx[1].vtX + xdiff + ydiff;
	vtx[4].vtY = vtx[1].vtY + ydiff - xdiff;
}

flex_poly_box(vtx)
register struct ivertex *vtx;
{
	vtx[1].vtY = vtx[4].vtY = vtx[0].vtY;
	vtx[3].vtX = vtx[4].vtX = vtx[0].vtX;
}

