/*
 * flex_file.c		Jim Piper and Bill Hill @MRC		18-12-90
 *
 * MODIFICATIONS
 * -------------
 *	 9 Feb 1991	CAS		Case on menu options
 * NEED TO FINISH THE FOLLOWING CHANGE fs->stat->modified SO THAT THE
 * HACK WITH "WRITTEN"/"OLDWRITTEN" CAN BE REMOVED........
 * 30-01-91 jimp	Added flex_file_query() for checking if filed.
 * 			Added setting of fs->stat->modified after any change,
 *			re-setting after writing to disc or replacing the
 *			flexible screen structure.
 * 25-01-01 jimp	Exit from filing menu if successful read or replace.
 *			Do not query replacing an empty screen.
 *			Added filing messages.
 * 21-12-90 jimp	Added (i) proper test for non-written file before
 *			deleting, (ii) call to flex_back_links_sc() after
 *			reading from disc.
 *
 */
#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <cmdsys.h>
#include <allcontr.h>
#include <fileheaders.h>
#include <flex.h>

/*
 *	External data references
 */
extern int errno;
extern struct allkontr *alc;
extern FILE *debug_channel;

/*
 * Options to store, retrieve, replace flexible screen.
 *
 * Temporarily (??), have defined "struct flexible_screen **fs" so that we
 * can test most of the system without all the "allcontrol" rubbish.
 *
 * Checking whether the existing screen has been written occupies much
 * of the code in this menu and its support functions.
 */
flex_file_menu(fs)
struct flexible_screen **fs;
{
	struct flexible_screen *flex_make();
	int m, written=0, readin=0;
	int flex_write_ms(), flex_read_ms(), flex_replace_ms();
	downmenu();
	newmenu("Flexible Screen Filing Options");
	cmmd ("store this screen;%x%x%f",*fs,&written,flex_write_ms); mouse(LEFTBT);
	cmmd ("delete this screen, create a new one;%x%x%f",fs,&written,flex_replace_ms); mouse(MIDBT);
	cmmd ("retrieve a screen;%x%x%x%f",fs,&written,&readin,flex_read_ms); mouse(RIGHTBT);
	cmmd ("exit"); mouse(LONGLB);
	for (;;) {
		m = thismu();
		switch (m) {
			case 1:
				break;
			case 2:
				upmenu();
				return(1);
			case 3:
				if (readin) {
					upmenu();
					return(1);
				}
				break;
			case 4:
				upmenu();
				return(1);
			default:
				bleep();
				break;
		}
	}
}


static struct object head = {110,0,0,0,0};
/*
 * Check ID, file to disc.  Return success/failure in
 * "*written" since cmdsys cannot allocate a function value.
 */
int flex_write_ms(fs,written)
struct flexible_screen *fs;
int *written;
{
	FILE *fbdopen(), *fp;

	/*
	 * "getid()" assumes that alc->flex_screen == fs.
	 */
	if (alc->flex_screen != fs) {
		printf("WRITE_FLEX_MS: Program error\n");
		*written = 0;
		return(0);
	}
	if (!getid(FLEXFILE,WRITE)) {
		*written = 0;
		return(0);
	}

	if ((fp = fbdopen(fs->id->filename,alc->dir,"w",BUFSIZ)) == NULL) {
		fileerror("Failed to open file %s",errno,fs->id->filename);
		return(0);
	}
	flex_head_fill(fs);
	head.plist = (struct propertylist *)fs->id->header;
	printf("Filing ...... ");
	fflush(stdout);
	writeobj(fp,&head);
	writeobj(fp,fs->cao);
	fbclose(fp);
	printf("done\n");
	*written = 1;
	fs->stat->modified = 0;
	return(1);
}


/*
 * Read a flexible screen from disc.  First check if the old has
 * been written, and if not, whether it should be.
 *
 * Care is needed.  We need a complete flexible screen structure
 * in order to do the identification things, but than readobj()
 * will read in an entire compound object, allocating space as it
 * does so, so we are left with too much space allocated.
 * 
 */
int flex_read_ms(fs,oldwritten,readin)
struct flexible_screen **fs;
int *oldwritten, *readin;
{
	FILE *fbdopen(), *fp;
	struct object *head, *ca, *readobj();

	*readin = 0;
	if (!flex_replace_ms(fs,oldwritten))
		return(0);

	/*
	 * "getid()" is assuming that *fs == alc->flex_screen,
	 * this was done by "flex_replace_ms()" call above.
	 */
	if (!getid(FLEXFILE,READ))
		return(0);


	if ((fp = fbdopen((*fs)->id->filename,alc->dir,"r",BUFSIZ)) == NULL) {
		fileerror("Failed to open file %s",errno,(*fs)->id->filename);
		return(0);
	}
	printf("Reading ....... ");
	fflush(stdout);
	head = readobj(fp);
	if (head == NULL) {
		fbclose(fp);
		printf("Incorrect file format\n");
		return(0);
	}
	switch (head->type) {
		case 110:
			(*fs)->id->header = (struct flex_header *) head->plist;
			head->plist = NULL;
			freeobj(head);
			if ((*fs)->id->header->file_type != FLEXFILE) {
				printf("Wrong header type\n");
				fbclose(fp);
				return(0);
			}
			ca = readobj(fp);
			fbclose(fp);
			if (ca == NULL || ca->type != 81) {
				printf("Header but no body\n");
				return(0);
			}
			break;
		case 81:
			fbclose(fp);
			ca = head;
			break;
	}
	/*
	 * Have successfully read the compound object --
	 * so free out the one allocated by flex_make(),
	 * and run through the structure filling in "->assoc".
	 */
	printf("done\n");
	if (alc->debug) {
		fprintf(debug_channel,"flex_read_ms freeing *fs->cao\n");
		Show_alloc();
	}
	freeobj((*fs)->cao);
	if (alc->debug) {
		fprintf(debug_channel,"flex_read_ms making new *fs->cao\n");
		Show_alloc();
	}
	(*fs)->cao = (struct compounda *)ca;
	if (alc->debug) {
		fprintf(debug_channel,"flex_read_ms calling flex_back_links_sc\n");
		Show_alloc();
	}
	flex_back_links_sc(*fs);
	if (alc->debug) {
		fprintf(debug_channel,"flex_read_ms calling flex_draw_sc\n");
		Show_alloc();
	}
	flex_draw_sc(*fs,(*fs)->stat->boxes);
	
	*readin = 1;
	return(1);
}


/*
 * Replace the screen without writing - but at
 * least ask if it has not been written away !
 * Retain the flex_status structure in new screen,
 * using structure assignment (!)
 */
int flex_replace_ms(fs,oldwritten)
struct flexible_screen **fs;
int *oldwritten;
{
	struct flex_status stat;
	int i, empty = 1;
	struct compounda *cao = (*fs)->cao;
	
	for (i=0; i<cao->n; i++)
		if (cao->o[i] != NULL)
			empty = 0;
	if (!empty && !(*oldwritten) && !flex_replace_query())
		return(0);
	intens(0);
	flex_draw_sc(*fs,(*fs)->stat->boxes);
	intens(1);
	stat = *((*fs)->stat);
	flex_del_sc(*fs);
	alc->flex_screen = *fs = flex_make(500);
	*((*fs)->stat) = stat;
	(*fs)->stat->modified = 0;
	*oldwritten = 0;
	return(1);
}

/*
 * A screen exists and for some reason we wish to find out whether
 * to delete it or not - so ask the question.
 */
int flex_replace_query()
{
	bleep();
	screengo(ENDLINE,1);
	fprintf(output,"WARNING -- PREVIOUS FLEXIBLE SCREEN NOT FILED\n");
	return(go_on());
}


/*
 * If an unwritten screen exists -- should it be filed?
 * Depends if it has been modified since creation/last writing...
 */
int flex_file_query(fs)
struct flexible_screen *fs;
{
	int written;

	if (fs->stat->modified) {
		bleep();
			screengo(ENDLINE,1);
		fprintf(output,"Flexible screen filed ??\n");
		if (!go_on()) {
			downmenu();
			flex_write_ms(fs,&written);
			upmenu();
		}
	}
}


