/*
 * flex_menu.c		Jim Piper and Bill Hill @MRC		04-12-90
 *
 * MODIFICATIONS
 * -------------
 * 25-06-91 ih		Extra debug
 * 24-06-91 ih		Check for null flex screen in flex_menu
 * 06-02-91 jimp	Added ideogram generation menu.  Increased permitted
 *			text maximum size.  Changed the groups/all toggle.
 * 30-01-91 jimp	Added setting of fs->stat->modified after any change.
 * 29-01-91 jimp	Added permanent box-tracking, arrow/box-line width.
 * 28-01-91 jimp	Yet another bug in grouping - selection of objects.
 *			Text size changing added.
 * 27-01-91 jimp	Added rotate function, and enhancement options menu.
 * 23-01-91 jimp	Implemented IRS's wishes re grouping.
 * 19-01-91 jimp	Modified/added menus to be closer to IRS spec.
 * 18-01-91 jimp	After successful merge, clear fs->current_obj.
 * 21-12-90 jimp	Added cursframe(fs->stat->Phase) and
 *			setmsv(flex_TrackCursor) at end of format screen option.
 * 20-12-90 BH		added function flex_DrawSplitScreen() to redraw the
 *			split flexible screen at end of format screen option.
 * 18-12-90 jimp	"Boxes" in fs->stat leads to simpler code.
 *			Added flex_file_menu() called from main menu.
 * 06-12-90 jimp	HACK to re-draw an existing screen at large scale
 *			using flex_frames_big().  Better, should have
 *			a status structure in struct flexible_screen
 *			(21-12-90 we now do!).
 *
 */
#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <allcontr.h>
#include <display.h>
#include <cmdsys.h>
#include <fileheaders.h>
#include <flex.h>

extern struct allkontr *alc;
extern FILE *debug_channel;

flex_menu(fs)
struct flexible_screen *fs;
{
	int m;
	int toggle_size = 1;
	int flex_curs1(), flex_TrackCursor();

	if (!fs) {
		if (alc->debug)
			fprintf(debug_channel,"flex_menu passed NULL alc->flex_screen pointer\n");
		return(0);
	}
	if (alc->debug) {
		fprintf(debug_channel,"entering flex_menu\n");
		Show_alloc();
	}
		
	flex_frames_big(fs);
	clear();
	flex_draw_sc(fs,fs->stat->boxes);

	flex_curs1_init(fs);
	setmsv(flex_curs1);

	newmenu("Format Flexible Screen");
 	cmmd ("object grouping and movement"); mouse(MIDBT);
	cmmd ("arrows and text (labeling)"); mouse(RIGHTBT);
	cmmd ("enhancement options"); mouse(LEFTBT);
	cmmd ("ideogram generation"); mouse(LONGRB);
	cmmd ("object presentation"); mouse(LONGMB);
 	cmmd ("exit"); mouse(LONGLB);

	for (;;) {
		m = thismu();
		switch (m) {
			case 1:
				downmenu();
				flex_group_move_menu(fs);
				upmenu();
				break;
			case 2:
				downmenu();
				flex_arrow_text_menu(fs);
				upmenu();
				break;
			case 3:
				downmenu();
				flex_enhancement_menu(fs);
				upmenu();
				break;
			case 4:
				downmenu();
				flex_ideogram_menu(fs);
				upmenu();
				break;
			case 5:
				downmenu();
				flex_presentation_menu(fs);
				upmenu();
				break;
			case 6:
				flex_curs1_end();
 				flex_DrawSplitScreen();
 				cursframe(fs->stat->Phase);
				setmsv(flex_TrackCursor);
				return(0);
			default:
				bleep();
				break;
		}
	}
	if (alc->debug) {
		fprintf(debug_channel,"exiting flex_menu\n");
		Show_alloc();
	}
}


flex_group_move_menu(fs)
struct flexible_screen *fs;
{
	int tracking_boxes = 0;
	int m, newx, newy;
	int flex_curs1(), flex_curs2();

	if (alc->debug) {
		fprintf(debug_channel,"entering flex_group_move_menu\n");
		Show_alloc();
	}
	newmenu("Object Grouping and Movement");
 	cmmd ("pick up/drop;");  mouse(MIDBT);
 	cmmd ("put down in new position"); mouse(RIGHTBT);
 	cmmd ("group"); mouse(LONGMB);
 	cmmd ("delete;"); mouse(LONGRB);
 	cmmd ("exit"); mouse(LONGLB);
	for (;;) {
		m = thismu();
		switch (m) {
			case 1:
				if (fs->stat->current_obj != NULL) {
					if (tracking_boxes) {
						flex_curs2_end(&newx,&newy);
						tracking_boxes = 0;
						setmsv(flex_curs1);
					}
					else {
						flex_curs2_init(fs);
						tracking_boxes = 1;
						setmsv(flex_curs2);
					}
				}
				else
					bleep();
				break;
			case 2:
				if (tracking_boxes) {
					flex_curs2_end(&newx,&newy);
					flex_newpos_ms(fs->stat->current_obj,newx,newy,fs->stat->boxes);
					fs->stat->current_obj = NULL;
					tracking_boxes = 0;
					setmsv(flex_curs1);
					fs->stat->modified = 1;
				}
				else
					bleep();
				break;
			case 3:
				if (tracking_boxes)
					bleep();
				else {
					downmenu();
					flex_group_menu(fs);
					upmenu();
				}
				break;
			case 4:
				if (tracking_boxes || fs->stat->current_obj == NULL)
					bleep();
				else if (flex_delete_ms(fs->stat->current_obj,fs->stat->boxes)) {
					fs->stat->modified = 1;
					fs->stat->current_obj = NULL;
				}
				else
					bleep();
				break;
			case 5:
				return(0);
			default:
				bleep();
				break;
		}
	}
	if (alc->debug) {
		fprintf(debug_channel,"exiting flex_group_move_menu\n");
		Show_alloc();
	}
}


flex_group_menu(fs)
struct flexible_screen *fs;
{
	int i, m;
	struct compounda *node2, *node1 = NULL;

	if (alc->debug) {
		fprintf(debug_channel,"entering flex_group_menu\n");
		Show_alloc();
	}
	newmenu("Grouping Ojects");
 	cmmd ("select 1st object for grouping"); mouse(MIDBT);
	cmmd ("select 2nd object"); mouse(RIGHTBT);
 	cmmd ("ungroup;"); mouse(LEFTBT);
	cmmd ("toggle boxes;:k40%t",&(fs->stat->boxes)); mouse(LONGMB);
	cmmd ("exit"); mouse(LONGLB);
	for (;;) {
		m = thismu();
		switch (m) {
			case 1:
				if (node1 != NULL)
					flex_highlight_nd(node1,0);
				node1 = fs->stat->current_obj;
				if (node1 == NULL)
					bleep();
				else if (flex_leaf(node1) && flex_level(node1) != 1) {
					fprintf(stderr,"member of a group\n");
					node1 = NULL;
					putprompt();
					bleep();
				}
				else
 					flex_highlight_nd(node1,1);
 				break;
			case 2:
				node2 = fs->stat->current_obj;
				if (node1 == NULL || node2 == NULL || node2 == node1) {
					bleep();
					break;
				}
				else if (flex_leaf(node2) && flex_level(node2) != 1) {
					fprintf(stderr,"member of a group\n");
					putprompt();
					node2 = NULL;
					bleep();
					break;
				}
				flex_highlight_nd(node1,0);
				if (!flex_newgroup_ms(node1,node2,fs->stat->boxes))
					bleep();
				else
					fs->stat->modified = 1;
				node1 = node2 = fs->stat->current_obj = NULL;
				break;
			case 3:
				if (fs->stat->current_obj == NULL)
					bleep();
				else if (flex_ungroup_ms(fs->stat->current_obj,fs->stat->boxes)) {
					fs->stat->modified = 1;
					fs->stat->current_obj = NULL;
				}
				else
					bleep();
				break;
			case 4:
				colour(OVERLAY1);
				intens(fs->stat->boxes);
				for (i=0; i<fs->cao->n; i++)
					flex_draw_boxes_nd(fs->cao->o[i]);
				break;
			case 5:
				flex_highlight_nd(node1,0);
				return(0);
			default:
				bleep();
				break;
		}
	}
	if (alc->debug) {
		fprintf(debug_channel,"exiting flex_group_menu\n");
		Show_alloc();
	}
}


flex_arrow_text_menu(fs)
struct flexible_screen *fs;
{
	int m, x, y;
	int tracking_arrow = 0;
	int tracking_box = 0;
	static int textsize=6;
	static int linewidth=1;
	int flex_curs1(), flex_curs3();

	newmenu("Arrows and Text (Labeling)");
	cmmd ("line/arrow width;:k40<1/5>%d",&linewidth);
	cmmd ("set arrow head/tail"); mouse(MIDBT);
	cmmd ("set box start/finish"); mouse(RIGHTBT);
	cmmd ("text size;:k40<3/48>%d",&textsize);
	cmmd ("set text position"); mouse(LEFTBT);
 	cmmd ("delete object;"); mouse(LONGRB);
	cmmd ("exit"); mouse(LONGLB);
	for (;;) {
		m = thismu();
		switch (m) {
			case 1:
				break;
			case 2:
				if (tracking_arrow) {
					tracking_arrow = 0;
					flex_curs3_end();
					flex_arrow_box_ms(fs,fs->stat->boxes);
					fs->stat->modified = 1;
					flex_curs1_init(fs);
					setmsv(flex_curs1);
				}
				else if (tracking_box)
					bleep();
				else {
					flex_curs_pos(&x,&y);
					flex_curs1_end();
					flex_curs3_init(fs,x,y,1,linewidth);
					tracking_arrow = 1;
					setmsv(flex_curs3);
				}
				break;
			case 3:
				if (tracking_box) {
					tracking_box = 0;
					flex_curs3_end();
					fs->stat->modified = 1;
					flex_arrow_box_ms(fs,fs->stat->boxes);
					flex_curs1_init(fs);
					setmsv(flex_curs1);
				}
				else if (tracking_arrow)
					bleep();
				else {
					flex_curs_pos(&x,&y);
					flex_curs1_end();
					flex_curs3_init(fs,x,y,2,linewidth);
					tracking_box = 1;
					setmsv(flex_curs3);
				}
				break;
			case 4:
				break;
			case 5:
				if (tracking_arrow || tracking_box)
					bleep();
				else {
					flex_curs_pos(&x,&y);
					flex_get_text_ms(fs,x,y,fs->stat->boxes,textsize);
					fs->stat->modified = 1;
					putprompt();
				}
				break;
			case 6:
				if (tracking_arrow || tracking_box || fs->stat->current_obj == NULL)
					bleep();
				else if (flex_delete_ms(fs->stat->current_obj,fs->stat->boxes)) {
					fs->stat->modified = 1;
					fs->stat->current_obj = NULL;
				}
				else
					bleep();
				break;
			case 7:
				return(0);
			default:
				bleep();
				break;
		}
	}
}


flex_presentation_menu(fs)
struct flexible_screen *fs;
{
	int m, flex_dostring();
	int percent = 125;
	char string[8];

	flex_dostring(&(fs->stat->alltoggle), string);
	newmenu("Object Presentation");
	cmmd ("by groups/all;%+t:k40%-s%f", &(fs->stat->alltoggle), string, flex_dostring); mouse(MIDBT);
	cmmd ("rescale;:k40%d",&(fs->stat->scale));
	cmmd ("percentage enlargement factor;:k40<50/190>%d",&percent);
 	cmmd ("rotate;"); mouse(RIGHTBT);
	cmmd ("enlarge;"); mouse(LEFTBT);
	cmmd ("straighten"); mouse(LONGRB);
	cmmd ("recover original"); mouse(LONGMB);
	cmmd ("exit"); mouse(LONGLB);
	for (;;) {
		m = thismu();
		switch (m) {
			case 1:
				break;				
			case 2:
				flex_change_frames(fs->cao,fs->stat->dx,fs->stat->dx,fs->stat->dy,fs->stat->dy,fs->stat->scale);
				fs->stat->modified = 1;
				flex_setbox_fs(fs);
				clear();
				flex_draw_sc(fs,fs->stat->boxes);
				break;
			case 3:
				break;
			case 4:
				if (!flex_rotate_ms(fs->stat->current_obj,fs->stat->boxes))
					bleep();
				else
					fs->stat->modified = 1;
				break;
			case 5:
				if (!flex_enlarge_ms(fs,percent))
					bleep();
				else
					fs->stat->modified = 1;
				break;
			case 6:
				if (!flex_straighten_ms(fs))
					bleep();
				else
					fs->stat->modified = 1;
				break;
			case 7:
				if (!flex_recover_ms(fs))
					bleep();
				else
					fs->stat->modified = 1;
				break;
			case 8:
				return(1);
		}
	}
}
					

flex_enhancement_menu(fs)
struct flexible_screen *fs;
{
	int m, flex_dostring();
	int d1=1, d2=1;
	int varenh_menu(), adjustlut();
	char string[8];

	flex_dostring(&(fs->stat->alltoggle), string);
	newmenu("Enhancement Options");
	cmmd ("by groups/all;%+t:k40%-s%f", &(fs->stat->alltoggle), string, flex_dostring); mouse(MIDBT);
	cmmd ("normalise"); mouse(RIGHTBT);
	cmmd ("enhance"); mouse(LEFTBT);
	cmmd("variable enhancement setup;%+i%f", FLEXIBLE_SCREEN, varenh_menu); mouse(LONGRB);
	cmmd ("recover original"); mouse(LONGMB);
	cmmd("look-up-tables;%-+d%-+d%+i%f", &d1, &d2, fs->stat->Phase, adjustlut);
	cmmd ("exit"); mouse(LONGLB);
	for (;;) {
		m = thismu();
		switch (m) {
			case 1:
				break;
			case 2:
				if (!flex_normalise_ms(fs))
					bleep();
				else
					fs->stat->modified = 1;
				break;
			case 3:
				if (!flex_enhance_ms(fs))
					bleep();
				else
					fs->stat->modified = 1;
				break;
			case 4:
				fs->stat->modified = 1;
				break;
			case 5:
				if (!flex_recover_ms(fs))
					bleep();
				else
					fs->stat->modified = 1;
				break;
			case 6:
				break;
			case 7:
				return(1);
		}
	}
}


flex_ideogram_menu(fs)
struct flexible_screen *fs;
{
	int m,x,y,flex_cycle_res();
	static int class=1;
	static int resolution=300;

	newmenu("Ideogram Generation");
	cmmd ("class;:k40<1/24>%d",&class);
	cmmd ("change resolution;:k40%-d%f",&resolution,flex_cycle_res); mouse(RIGHTBT);
	cmmd ("class of selected chromosome;"); mouse(LEFTBT);
	cmmd ("place ideogram"); mouse(MIDBT);
 	cmmd ("delete object;"); mouse(LONGRB);
	cmmd ("exit"); mouse(LONGLB);
	for (;;) {
		m = thismu();
		switch (m) {
			case 1:
			case 2:
				break;
			case 3:
				class = flex_ideoclass_ms(fs,class);
				displin(0,1);
				putprompt();
				break;
			case 4:
				flex_curs_pos(&x,&y);
				flex_ideoput_ms(fs,class,resolution,x,y);
				break;
			case 5:
				if (flex_delete_ms(fs->stat->current_obj,fs->stat->boxes)) {
					fs->stat->modified = 1;
					fs->stat->current_obj = NULL;
				}
				else
					bleep();
				break;
			case 6:
				return(1);
		}
	}
}

flex_cycle_res(r)
int *r;
{
	*r += 300;
	if (*r > 900)
		*r = 300;
}

/*
 * copy whether "by groups" or "all" into given string
 *
 */
static
flex_dostring(toggle, string)
int *toggle;
char *string;
{
	if (*toggle)
		strcpy(string, "all");
	else
		strcpy(string, "groups");
}

