/*
 * flexident.c		Jim Piper and Bill Hill @ MRC HGU 	04-12-90
 *
 * This file contains functions which relate to the identification of objects
 * within quadrants of the screen.
 *
 * MODIFICATIONS
 * -------------
 * 3 Jul 1991	ih	Added check for active object in flex_MetLocate
 */

#include <stdio.h>
#include <cmdsys.h>
#include <kyy.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <display.h>
/*#include <dispform.h>*/
#include <fileheaders.h>
#include <flex.h>

#define FLEX_ABS(A)	((A)>=0?(A):-(A))
#define FLEX_CLOSE 	8 /* Location proximity control for metaphase objects */

extern int	flex_WhichQuadrant();
extern int	kylocate();

int		flex_KaryTrack(),
		flex_MetTrack(),
		flex_TrackCursor();
void		flex_InitCursor(),
		flex_TermCursor();

static void	flex_MetLocate();

extern struct kcontrol *kcont;
extern struct pframe *idf;
extern struct ipoint cursor;

static int	FlexQuadrant;

/************************************************************************
* Function:	flex_InitCursor						*
* Returns:	void							*
* Purpose:	Initialise the cursor for flexible screen so that the	*
*		cursor position is wrt the idf.				*
* Parameters:	int Phase:		segmetation (0) or karyotype (1)*
************************************************************************/
void		flex_InitCursor(Phase)
int		Phase;
{
  int		(**Track)();

  Track = alc->flex_screen->stat->TrackFuncs;
  *(Track) = (int(*)() )0;
  *(++Track) = (int(*)() )0;
  *(++Track) = (int(*)() )0;
  *(++Track) = (int(*)() )0;
  cursframe(Phase);
  setmsv(flex_TrackCursor);
}

/************************************************************************
* Function:	flex_TermCursor						*
* Returns:	void							*
* Purpose:	Terminate the cursor for flexible screen.		*
* Parameters:	-							*
************************************************************************/
void		flex_TermCursor()
{
  int		(**Track)();

  Track = alc->flex_screen->stat->TrackFuncs;
  setmsv((int(*)() )0);
  *(Track) = (int(*)() )0;
  *(++Track) = (int(*)() )0;
  *(++Track) = (int(*)() )0;
  *(++Track) = (int(*)() )0;
}

/************************************************************************
* Function:	flex_KaryTrack						*
* Returns:	int:			always returns 0		*
* Purpose:	Function used to track the cursor when in a karyotype 	*
*		quadrant of the screen. Although this function is 	*
*		passed the cursor position in ddgs coordinates, they're	*
*		not used! The coordinates used are wrt the frame idf.	*
* Parameters:	int xPos:		cursor x position in ddgs units	*
*		int yPos:		cursor y position in ddgs units	*
************************************************************************/
int		flex_KaryTrack(xPos, yPos)
int		xPos,
		yPos;
{
  register struct flex_status *Stat;
  register int	Id;

  Stat = alc->flex_screen->stat;
  kylocate(&cursor);
  if((Id = Stat->KaryId.no) >= 0)
  {
    Stat->SelectObj = *(kcont->mol + Id);
    if((kcont->ivf + Id)->iy < 0)
      Stat->SelectWayUp =  -1;
    else
      Stat->SelectWayUp =  1;
  }
  return(0);
}

/************************************************************************
* Function:	flex_MetTrack						*
* Returns:	int:			always returns 0		*
* Purpose:	Function used to track the cursor when in a metaphase	*
*		quadrant of the screen.					*
* Parameters:	int xPos:		cursor x position in ddgs units	*
*		int yPos:		cursor y position in ddgs units	*
************************************************************************/
int		flex_MetTrack(xPos, yPos)
register int	xPos,
		yPos;
{
  register struct pframe *New;

  New = alc->flex_screen->stat->Frames + FlexQuadrant;
  xPos = frinvxdig(xPos, New);
  yPos = frinvydig(yPos, New);
  flex_MetLocate(kcont->ool, kcont->maxnumber, New, xPos, yPos);
  return(0);
}

/************************************************************************
* Function:	flex_TrackCursor					*
* Returns:	int:			always returns 0		*
* Purpose:	Cursor tracking function which is passed to the mouse	*
*		handler. Type of tracking depends on whether the mouse	*
*		is moving whether an object has been picked up and the	*
*		quadrant which the mouse is in.				*
* Parameters:	-							*
************************************************************************/
int	 	flex_TrackCursor()
{
  register int	xPos,
		yPos;
  register int 	(*Func)();
  register struct flex_status *Stat;
  register struct pframe *Frame;

  Stat = alc->flex_screen->stat;
  if(Stat->ObjPickedUp)
  {
    xPos = frdigsxpos(cursor.k, idf);
    yPos = frdigsypos(cursor.l, idf);
    tmousecurs(); 			    /* HACK should be object boundary */
  }
  else
  {
    if(alc->m2s->moving)
      tmousecurs();
    else
    {
      xPos = frdigsxpos(cursor.k, idf);
      yPos = frdigsypos(cursor.l, idf);
      FlexQuadrant = flex_WhichQuadrant(xPos, yPos);
      Func = Stat->TrackFuncs[FlexQuadrant];
      if(Func)
	((int(*)() )Func)(xPos, yPos);
    }
  }
}

/************************************************************************
* Function:	flex_MetLocate						*
* Returns:	void							*
* Purpose:	Locate a metaphase object, set kcont->curent_obj and	*
*		draw a box around it. Location uses the following	*
*		algorithm: First check if point lies within a right	*
*		rectangle surrounding any object, and slightly bigger.	*
*		Second, for each such object, see which rectangle is	*
*		smallest. Third, if ambiguous, see which object is	*
*		nearest the centre. This function is based on locate().	*
* Parameters:	struct object **Objs:	list of metaphase objects	*
*		int NumOfObjs:		number of objects		*
*		struct pframe *Frame:	frame containing the objects	*
*		int xPos:		given x position wrt frame	*
*		int yPos:		given y position wrt frame	*
************************************************************************/
static void
flex_MetLocate(Objs, NumOfObjs, Frame, xPos, yPos)
struct chromosome **Objs;
int		NumOfObjs;
register struct pframe *Frame;
register int	xPos,
		yPos;
{
  register struct intervaldomain *IntDom;
  register struct chromosome **ThisObj;
  register int	Index,
		Area;
  int		Dist,
		Hit = -1,
		MinArea = 1000000,
		MinDist = 1000000;
	int nn;

	for(Index = 0, ThisObj = Objs; Index < NumOfObjs; ++Index, ++ThisObj) {
		if(*ThisObj) {
			if (!(kcont->acl[Index] & ACTIVE)) {
				nn = (*ThisObj)->plist->Cotype; 
				if ( nn != NOISE && nn != NUCLEUS ) continue;
			}
    		IntDom = (*ThisObj)->idom;
    		if(((IntDom->kol1 - FLEX_CLOSE) < xPos) &&
    		((IntDom->lastkl + FLEX_CLOSE) > xPos) &&
    		((IntDom->line1 - FLEX_CLOSE) < yPos) &&
    		((IntDom->lastln + FLEX_CLOSE) > yPos) &&
    		((Area = (IntDom->lastkl - IntDom->kol1 + (2 * FLEX_CLOSE)) * 
    		(IntDom->lastln - IntDom->line1 +
    		(2 * FLEX_CLOSE))) <= MinArea) &&
 		   	((Dist = FLEX_ABS(xPos + xPos - IntDom->kol1 - IntDom->lastkl) +
    		FLEX_ABS(yPos + yPos - IntDom->line1 - 
    		IntDom->lastln)) < MinDist)) {
    			Hit = Index;
    			MinArea = Area;
    			MinDist = Dist;
    		}
    	}
  	}
  	Index = Hit;
  	if(Index != -1) {
		unbox();
    	boxobj(*(Objs + Index), Frame, OVERLAY2); 
    	alc->flex_screen->stat->SelectObj = *(Objs + Index);
    	alc->flex_screen->stat->SelectWayUp =  1;
    }
}
