/*
 * flexsubs0.c		Jim Piper and Bill Hill @ MRC HGU	04-12-90
 *
 * This file contains functions that are either called from the main flexible
 * screen menu or else just to service these functions.
 *
 * Modifications
 *
 *	 7 Feb 1991	CAS		TABS !!!!!
 *						Circumvent compiler /= bug....
 * 06-02-91 jimp	Squeeze() has set ->assoc -- must be set to NULL.
 *					Corrected display origin for metaphase in quadrant so
 *					that displayed centrally.
 * 31-01-91 jimp	flex_DupChromo() is told whether cell is a FIP or TV
 *					digitisation and calls squeeze() in the former case.
 *					Done by setting stat->DigType in flex_Initialise.
 *					flex_PickUp() can now toggle PickedUp.  Display
 *					of PickedUp objects added.
 * 30-01-91 jimp	Added call to flex_file_query() in flex_Terminate.
 * 					Added setting of fs->stat->modified after any change.
 * 29-01-91 jimp	flex_TermKaryotype - got the karyotype cursor back.
 * 28-01-91 jimp	Given->idom->linkcount-- and Dup->idom->linkcount++
 *					in flex_DupChromo().
 * 25-01-91 jimp/joan	Insist that object is "picked-up" in flex_CopyObj.
 *					Initial screen sized for 500 objects.
 * 24-01-91 BH		Added check that cursor is in the flex quadrant to
 *					function flex_PutDown(). Added code to rescale a
 *					metaphase quadrant's frame while trying to preserve
 *					original ratio in flex_InitMetaphase().
 * 21-12-90 jimp	Stat->SelectWayUp set from plist->dispor if phase==1.
 *					Dup->assoc set to NULL (newgrey(obj) drops obj there).
 * 20-12-90 BH		Flex globals now in flex_status structure, changed
 *					parameters to funcs flex_Shift(From)|(To)Quad(X)|(Y).
 *					Added function flex_DrawSplitScreen().
 * 06-12-90 jimp	HACK to re-draw an existing screen at small scale
 *					using flex_frames_small().  Better, should have
 *					a status structure in struct flexible_screen.
 *
 */

#include <stdio.h>
#include <cmdsys.h>
#include <kyy.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <display.h>
#include <dispform.h>
#include <fileheaders.h>
#include <flex.h>
#include <ddgs.h>


extern struct intervaldomain *newidomain();
extern struct object *newgrey();
extern struct flexible_screen *flex_make();
extern struct compounda *flex_fill_next_slot(),
    			*flex_fill_spot();
extern int	flex_KaryTrack(),
		flex_MetTrack(),
		flex_ScaleFromQuad(),
		flex_ScaleToQuad(),
		flex_ShiftFromQuadX(),
		flex_ShiftFromQuadY(),
		flex_ShiftToQuadX(),
		flex_ShiftToQuadY(),
		flex_TrackCursor(),
		flex_WhichQuadrant();
extern void	flex_InitCursor(),
		flex_InitQuadrants(),
		flex_SplitScreen(),
		flex_TermCursor();

int		flex_CopyObj(),
		flex_PickUp(),
		flex_PutDown(),
		flex_Initialise();
void		flex_DrawSplitScreen(),
		flex_Terminate();

static struct chromosome *flex_DupChromo();
static void	flex_InitDisp(),
		flex_InitKaryotype(),
		flex_InitMetaphase(),
  		flex_DisplayObjs(),
		flex_TermKaryotype(),
		flex_TermMetaphase();

extern struct kcontrol *kcont;
extern struct pframe *idf;
extern struct ipoint cursor;

static struct pframe SavedFrame;

/************************************************************************
* Function:	flex_Initialise						*
* Returns:	int			0 if all ok -1 if error		*
* Purpose:	Initialise: object identification variables, save the	*
*			kcontrol pframe, cut the display into quadrants, set	*
*			cursor tracking functions, display metaphase or kary in	*
*			a quadrant and initialise the flexible screen quadrant.	*
* Parameters:	int Phase:		flag set to 0 for metaphase	*
*								and to 1 for karyotyping	*
************************************************************************/
int		flex_Initialise(Phase)
int		Phase;
{
	int		Status = 0;

	_strass(&SavedFrame, kcont->f, sizeof(struct pframe));
	if (alc->flex_screen)
		alc->flex_screen->stat->SelectObj = (struct chromosome *)0;
	else {
		if((alc->flex_screen = flex_make(500)) == (struct flexible_screen *)0)
			Status = -1;
	}
	if (Status == 0) {
		fast();
		colour(255);
		clear();
		slow();
		alc->flex_screen->stat->Phase = Phase;
		flex_InitQuadrants();
		flex_InitCursor(Phase);
		if (Phase == 0)
			flex_InitMetaphase(1);
		else
			flex_InitKaryotype(1);
		flex_DrawSplitScreen();
		alc->flex_screen->stat->DigType = DIGTV;	/* default */
		if (Phase == 0 && alc->cell_h != NULL)
			alc->flex_screen->stat->DigType = (alc->cell_h->dig_type & DIGMASK);
	}
	return(Status);
}

/************************************************************************
* Function:	flex_DrawSplitScreen					*
* Returns:	void							*
* Purpose:	Draw split flex screen with source and flex quadrants.	*
*			Note this function assumes that only quadrant 1 is used	*
*			as as source screen.					*
* Parameters:	-							*
************************************************************************/
void		flex_DrawSplitScreen()
{
	if (alc->flex_screen) {
		clear();
		if(alc->flex_screen->stat->Phase == 0)
			flex_DisplayObjs(alc->flex_screen->stat->Frames + 1, kcont->ool,
					 kcont->maxnumber);
		else
			displayall(kcont, 0); 			/* note second parameter is unused! */
		flex_SplitScreen();
		flex_frames_small(alc->flex_screen);	 
		flex_draw_sc(alc->flex_screen, alc->flex_screen->stat->boxes);
	}
}

/************************************************************************
* Function:	flex_Terminate						*
* Returns:	void							*
* Purpose:	Terminate: cursor tracking, restore display, restore 	*
*		the kcontrol pframe.					*
* Parameters:	-							*
************************************************************************/
void		flex_Terminate()
{
	alc->flex_screen->stat->ObjPickedUp = 0;
	flex_TermCursor();
	flex_file_query(alc->flex_screen);
	fast();
	clear();
	slow();
	if(alc->flex_screen->stat->Phase)
		flex_TermKaryotype(1);
	else
		flex_TermMetaphase();
	_strass(kcont->f, &SavedFrame, sizeof(struct pframe));
}

/************************************************************************
* Function:	flex_PickUp						*
* Returns:	int:			always 0			*
* Purpose:	Pick up an object, this just sets a flag since object	*
*		must already have been selected.	However, if already	*
*		picked up then put it down again.			*
* Parameters:	-							*
************************************************************************/
int		flex_PickUp()
{
	struct flex_status *Stat;

	Stat = alc->flex_screen->stat;
	if (Stat->ObjPickedUp) {
		Stat->ObjPickedUp = 0;
		lwidth(3);
		boxobj(Stat->SelectObj,Stat->Phase?kcont->ivf+Stat->KaryId.no:Stat->Frames+1,-OVERLAY2);
		intens(1);
		lwidth(1);
	}
	else if(Stat->SelectObj) {
		Stat->ObjPickedUp = 1;
		Stat->SelectWayUp = Stat->Phase==0? 1: Stat->SelectObj->plist->dispor;
		lwidth(3);
		boxobj(Stat->SelectObj,Stat->Phase?kcont->ivf+Stat->KaryId.no:Stat->Frames+1,OVERLAY2);
		lwidth(1);
	}
	else
		printf("No object selected\n");
	return(0);
}

/************************************************************************
* Function:	flex_CopyObj						*
* Returns:	int:			always 0			*
* Purpose:	Copy the object that has already been selected to the	*
*		next slot in the flexible screen. Note that the object	*
*		actually is copied in its entirety, this would not be	*
*		required if link counts were available.			*
* Parameters:	-							*
************************************************************************/
int		flex_CopyObj()
{
	struct flex_status 	*Stat;
	struct compounda	*Node;
	struct object		*Obj;

	colour(GREYCOLS);
	Stat = alc->flex_screen->stat;
	if(Stat->ObjPickedUp && Stat->SelectObj)
	{
		Obj = (struct object *)flex_DupChromo(Stat->SelectObj,Stat->DigType);
		Stat->SelectWayUp = Stat->Phase==0? 1: Stat->SelectObj->plist->dispor;
		Node = flex_fill_next_slot(alc->flex_screen, Obj, 4, Stat->SelectWayUp);
		if(Node)
			flex_draw_objs_nd(Node);
		Stat->ObjPickedUp = 0;			 /* make object no longer picked up */
		lwidth(3);
		boxobj(Stat->SelectObj,Stat->Phase?kcont->ivf+Stat->KaryId.no:Stat->Frames+1,-OVERLAY2);
		intens(1);
		lwidth(1);
		Stat->modified = 1;
	}
	else
		printf("No object selected\n");
	Stat->ObjPickedUp = 0;			 /* make object no longer picked up */
	return(0);
}

/************************************************************************
* Function:	flex_PutDown						*
* Returns:	int:			always 0			*
* Purpose:	Put the selected (and picked up) object down at the	*
*		current cursor position in the flexible screen.		*
* Parameters:	-							*
************************************************************************/
int		flex_PutDown()
{
	int		xPos,
		yPos;
	struct compounda *Node;
	struct object	*Obj;
	struct flex_status *Stat;

	colour(GREYCOLS);
	Stat = alc->flex_screen->stat;
	if(Stat->ObjPickedUp && Stat->SelectObj)
	{
		xPos = frdigsxpos(cursor.k, idf);
		yPos = frdigsypos(cursor.l, idf);
		if(flex_WhichQuadrant(xPos, yPos) == 0)
		{
			Obj = (struct object *)flex_DupChromo(Stat->SelectObj,Stat->DigType);
			Node = flex_fill_spot(alc->flex_screen, Obj, xPos, yPos, 4,
					Stat->SelectWayUp);
			if(Node)
	flex_draw_objs_nd(Node);
			Stat->ObjPickedUp = 0;			 /* make object no longer picked up */
			lwidth(3);
			boxobj(Stat->SelectObj,Stat->Phase?kcont->ivf+Stat->KaryId.no:Stat->Frames+1,-OVERLAY2);
			intens(1);
			lwidth(1);
			Stat->modified = 1;
		}
		else
			printf("Not in flexible screen\n");
	}
	else
		printf("No object selected\n");
	return(0);
}

/************************************************************************
* Function:	flex_InitKaryotype					*
* Returns:	void							*
* Purpose:	Initialise the karyotype display for given quadrant by	*
*		shifting each of the kary chromosome display frames 	*
*		into the given quadrant. Set karyotype cursor tracking	*
*		for this quadrant.					*
* Parameters:	int Quad:		given quadrant [0-3]		*
************************************************************************/
static void	flex_InitKaryotype(Quad)
int		Quad;
{
	struct flex_status *Stat;
	register struct pframe *Frame;
	register struct kdisppos *Pos;
	register int	*StackCol;
	register int	Count;

	Stat = alc->flex_screen->stat;
	Frame = Stat->Frames + Quad;
	Pos = alc->scrf->disppos; 				 		 /* positions of chomosomes */
	kcont->kdispscale = flex_ScaleToQuad(kcont->kdispscale); 	 /* new scale */
	Count = MAXCLASS;
	while(Count-- > 0)
	{
		Pos->col = flex_ShiftToQuadX(Pos->col, Frame);
		Pos->lin = flex_ShiftToQuadY(Pos->lin, Frame);
		Pos->space = flex_ScaleToQuad(Pos->space);
		Pos->height = flex_ScaleToQuad(Pos->height);
		++Pos;
	}
	alc->scrf->stacklin = flex_ShiftToQuadY(alc->scrf->stacklin, Frame);
	Count = NSTACKPOS + 1;
	StackCol = alc->scrf->stackcol;
	while(Count-- > 0)
		*StackCol++ = flex_ShiftToQuadX(*StackCol, Frame);
	alc->scrf->chromclass.x = flex_ShiftToQuadX(alc->scrf->chromclass.x, Frame);
	alc->scrf->chromclass.y = flex_ShiftToQuadY(alc->scrf->chromclass.y, Frame);
	initselect(alc->m2s, &(Stat->KaryId), &cursor, 1);
	*(Stat->TrackFuncs + Quad) = flex_KaryTrack;
}

/************************************************************************
* Function:	flex_InitMetaphase					*
* Returns:	void							*
* Purpose:	Initialise the metaphase display for given quadrant and	*
*		set metaphase cursor tracking function for this quad.	*
* Parameters:	int Quad:		quadrant [0-3] for metaphase	*
************************************************************************/
static void 	flex_InitMetaphase(Quad)
int		Quad;
{
	register int	Tmp;
	register struct pframe *Frame;
	int		xMin, yMin, xMax, yMax;
	
	Frame = alc->flex_screen->stat->Frames + Quad;
	Frame->ix = flex_ScaleToQuad(SavedFrame.ix * SavedFrame.scale);
	Frame->iy = flex_ScaleToQuad(SavedFrame.iy * SavedFrame.scale);
	Frame->scale = 1;
	Tmp = 1;	/* make scale the lowest common denominator of the two increments */
	while((Tmp <= Frame->ix) && (Tmp <= Frame->iy)) {
		if(((Frame->ix % Tmp) == 0) && ((Frame->iy % Tmp) == 0))
			Frame->scale = Tmp;
		++Tmp; 
	}
	if (Frame->scale > 1) {
		Frame->ix = Frame->ix / Frame->scale;	/* Compiler bug edition #321 don't use /= */
		Frame->iy = Frame->iy / Frame->scale;	/*  with words in structures..  CAS */
	}
	*(alc->flex_screen->stat->TrackFuncs + Quad) = flex_MetTrack;
	kcont->current_obj = 0;
}

/************************************************************************
* Function:	flex_TermKaryotype					*
* Returns:	void							*
* Purpose:	Restore the karyotype display and chromosome frames to	*
*		their initial state.					*
* Parameters:	int Quad:		quadrant [0-3] for karyotype	*
************************************************************************/
static void 	flex_TermKaryotype(Quad)
int		Quad;
{
	register struct kdisppos *Pos;
	register struct pframe *Frame;
	register int	*StackCol;
	register int	Count;
	int kycurs();

	Frame = alc->flex_screen->stat->Frames + Quad;
	Pos = alc->scrf->disppos; 							 /* positions of chomosomes */
	kcont->kdispscale = flex_ScaleFromQuad(kcont->kdispscale);
	Count = MAXCLASS;
	while(Count-- > 0)
	{
		Pos->col = flex_ShiftFromQuadX(Pos->col, Frame);
		Pos->lin = flex_ShiftFromQuadY(Pos->lin, Frame);
		Pos->space = flex_ScaleFromQuad(Pos->space);
		Pos->height = flex_ScaleFromQuad(Pos->height);
		++Pos;
	}
	alc->scrf->stacklin = flex_ShiftFromQuadY(alc->scrf->stacklin, Frame);
	Count = NSTACKPOS + 1;
	StackCol = alc->scrf->stackcol;
	while(Count-- > 0)
		*StackCol++ = flex_ShiftFromQuadX(*StackCol, Frame);
	alc->scrf->chromclass.x = flex_ShiftFromQuadX(alc->scrf->chromclass.x, Frame);
	alc->scrf->chromclass.y = flex_ShiftFromQuadY(alc->scrf->chromclass.y, Frame);
	cursframe(1);
	displayall(kcont, 0);				/* displayall() has an unused second parameter! */
/*	endselect(&cursor, 1); */
	setmsv(kycurs);
}

/************************************************************************
* Function:	flex_TermMetaphase					*
* Returns:	void							*
* Purpose:	Restore the metaphase display to its initial state.	*
* Parameters:	-							*
************************************************************************/
static void 	flex_TermMetaphase()
{
	dispall();
}

/************************************************************************
* Function:	flex_DisplayObjs					*
* Returns:	void							*
* Purpose:	Display given objects in given frame.			*
* Parameters:	struct pframe *Frame:		given frame		*
*				struct object **Obj:		objects to be displayed	*
*				int Count:			number of objects	*
************************************************************************/
static void	flex_DisplayObjs(Frame, Obj, Count)
register struct pframe 	*Frame;
register struct object	**Obj;
register int		Count;
{
	colour(GREYCOLS);
	while(Count--) {
		if(*Obj)
			picframe(*Obj, Frame);
		++Obj;
	}
}

/************************************************************************
* Function:	flex_DupChromo						*
* Returns:	struct chromosome *:	duplicated chromosome		*
* Purpose:	Given a pointer to a chromosome allocate memory & copy	*
*		the given chromosome.					*
* Parameters:	struct chromosome *Given: given chromosome to duplicate	*
*		int DigType - whether square (TV) or rectangular (FIP)	*
*		-- copying must be appropriate				*
************************************************************************/
static struct chromosome *flex_DupChromo(Given,DigType)
register struct chromosome *Given;
int DigType;
{
	register struct chromosome *Dup;

	switch (DigType) {
		case DIGTV:
			Dup = (struct chromosome *)newgrey(Given);
			Dup->assoc = NULL;
			if (Given->idom) {
				Dup->idom = newidomain(Given->idom);
				Given->idom->linkcount--;	/* pop since pushed by newgrey() */
				Dup->idom->linkcount++;	/* since set zero by newidomain() */
			}
			break;
		case DIGFIP:
		default:			/* pre - May 1990 will be DigType zero */
			Dup = (struct chromosome *) squeeze(Given,(double)1.0,(double)0.75);
			Dup->assoc = NULL;
			break;
	}
	if (Given->plist) {
		Dup->plist = (struct chromplist *)Malloc(sizeof(struct chromplist));
		if(Dup->plist)
			_strass(Dup->plist, Given->plist, sizeof(struct chromplist));
	}
	return(Dup);
}
