/*
 * flexsubs1.c		Jim Piper and Bill Hill @MRC		23-10-90
 *
 * Small utility functions for the flexible screen - NO MENUS PERMITTED HERE !!
 * Construction/destruction routines.
 * See also flex.h, flex.c, flexsubs2.c, flexsubs3.c.
 *
 * MODIFICATIONS
 * -------------
 * 30-01-91 jimp	Flex_fill_next_slot() now computes (X,Y) from the
 *			position in fs->cao that will be filled.
 * 21-12-90 jimp	Added new function flex_back_links_sc().
 *			Fixed a bug in flex_del_nd().
 * 17-12-90 jimp	Tidy up allocation of identification space.
 * 30-11-90 jimp	Root compound object indicated by cao->assoc== cao.
 *			Bug-fix in flex_extend().
 * 28-11-90 jimp	Printing on #ifdef DEBUG.
 *			Flex_fill_next_slot() now returns the leaf-node.
 * 27-11-90 jimp	fs->current_obj initialised to NULL.
 *
 */
#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <fileheaders.h>
#include <flex.h>
#include <allcontr.h>

extern FILE *debug_channel;
extern struct allkontr *alc;

/*
 * Make an empty flexible screen, with space originally for n objects.
 */
struct flexible_screen *flex_make(n)
int n;
{
	struct flexible_screen *fs;
	struct compounda *makecompounda();

	fs = (struct flexible_screen *)Calloc(1, sizeof(struct flexible_screen)
		+ sizeof(struct flex_status) + sizeof(struct flex_id)
		+ sizeof(struct flex_header) );
	fs->id = (struct flex_id *)(fs+1);
	fs->id->header = (struct flex_header *) (fs->id+1);
	fs->stat = (struct flex_status *) (fs->id->header+1);
	fs->cao = makecompounda(81,1,n,NULL,0);
	fs->cao->assoc = (struct object *) fs->cao;	/* Indicates the root */
	fs->stat->current_obj = NULL;
	fs->stat->scale = S_SMALL;
	fs->stat->dx = X_SMALL;
	fs->stat->dy = Y_SMALL;
	return(fs);
}


/*
 * free allocated space from a flexible screen
 */
flex_del_sc(fs)
struct flexible_screen *fs;
{
	flex_del_nd(fs->cao);
	Free(fs);
}


/*
 * make a "node" compound object and a single "leaf" compound object
 * contained within the "node", all to contain the input
 * object which should be of type 1 (chromosome), 10 (polygon)
 * or 70 (text).  Give it the frame f (allocated and set up outside).
 */
struct compounda *flex_makeleaf(obj,f)
struct object *obj,*f;
{
	struct compounda *lf, *nd, *makecompounda();
	struct object *makeirect();
	
	if (obj->type != 1 && obj->type != 10 && obj->type != 70) {
		fprintf(stderr,"Flex_makeleaf: illegal object type %d\n",obj->type);
		return(NULL);
	}
	lf = makecompounda(81,1,2,NULL,0);
	lf->o[0] = f;
	lf->o[1] = obj;
	nd = makecompounda(81,1,2,NULL,0);
	nd->o[0] = makeirect();
	nd->o[1] = (struct object *) lf;
	((struct compounda *) nd->o[1])->assoc = (struct object *) nd;
	flex_setbox_nd(nd);
	return(nd);
}


/*
 * Make a new compounda object with more space, copy contents
 * from input object, replace parent links, remove input shell
 */
struct compounda *flex_extend(ca,nl)
struct compounda *ca;
{
	struct compounda *nca, *oca, *makecompounda();
	int i;
	struct object *o;

	/*
	 * make new and copy objects from old to new, delete from old
	 */
	nca = makecompounda(81,1,nl,NULL,0);
	for (i=0; i<ca->n; i++) {
		nca->o[i] = o = ca->o[i];
		if (o != NULL && o->type == 81)
			((struct compounda *)o)->assoc = (struct object *) nca;
		ca->o[i] = NULL;
	}
	/*
	 * replace links to and from parent node of old
	 */
	oca = (struct compounda *) ca->assoc;
	if (oca != NULL && oca->type == 81) {
		for (i=0; i<oca->n; i++)
			if (oca->o[i] == (struct object *) ca)
				oca->o[i] = (struct object *) nca;
		if (oca != ca)
			nca->assoc = (struct object *) oca;
		else {
			fprintf(stderr,"FLEX_EXTEND: extending top-level (OUCH)");
			nca->assoc = (struct object *) nca;
		}
	}
	flex_del_nd(ca);
	return(nca);
}



/*
 * delete a flexible screen object -- recursively
 */
flex_del_nd(obj)
struct object *obj;
{
	int i;
	struct compounda *ca;
	if (obj == NULL)
		return(0);
	switch (obj->type) {
		case 1:
			if (obj->assoc != NULL) {
				obj->assoc->plist = NULL;
				freeobj(obj->assoc);
			}
			freeobj(obj);
			break;
		case 81:
			ca = (struct compounda *) obj;
			for (i=0; i<ca->n; i++)
				flex_del_nd(ca->o[i]);
			freeobj(ca);
			break;
		default:
			freeobj(obj);
			break;
	}
}


/*
 * Fill the next slot on a top-left quadrant layed out in
 * horizontal rows.....  Return NULL if no more space,
 * either in flexible screen object, or on display screen.
 * If inv < 0, will set up frame to display object inverted.
 *
 * Routine assumes (1) top left quadrant (despite the "scale" parameter)
 * (2) even spacing of objects (3) no intervening movement of objects.
 */
struct compounda *flex_fill_next_slot(fs,o,scale,inv)
struct flexible_screen *fs;
struct object *o;
int scale,inv;
{
	struct compounda *flex_fill_spot(), *ca, *cao;
	int dx, dy, ndx, i;

	/*
	 * Find first slot at top level of flexible screen hierarchy,
	 * return NULL if completely full.
	 */
	cao = fs->cao;
	for (i=0; i<cao->n; i++)
		if (cao->o[i] == NULL)
			break;
	if (i == cao->n)
		return(NULL);
	dy = 4000 - 500 * (i / 14);
	dx = 100 + 200*(i % 14);
	if (dy < 2300)
		return(NULL);
	ca = flex_fill_spot(fs,o,dx,dy,scale,inv);
	return(ca);
}

/*
 * Make a leaf in the flexible screen at specified screen location
 * I.e. dx, dy are in DIGS coordinates.
 * If inv < 0, will set up frame to display object inverted.
 * Does not yet take account of right-way up....
 */
struct compounda *flex_fill_spot(fs,o,dx,dy,scale,inv)
struct flexible_screen *fs;
struct object *o;
int dx,dy,scale,inv;
{
	struct object *flex_makeleaf();
	struct compounda *ca = fs->cao;
	struct pframe *f, *makepframe();
	int i = 0;
	if (inv >= 0)
		f = makepframe(scale,dx,dy,o->idom->kol1,o->idom->line1);
	else {
		f = makepframe(scale,dx,dy,o->idom->lastkl,o->idom->lastln);
		f->ix = f->iy = -1;
	}
	while (ca->o[i] != NULL)
		if (++i >= ca->n)
			return(NULL);
	ca->o[i] = (struct object *) flex_makeleaf(o,f);
	((struct compounda *)ca->o[i])->assoc = (struct object *)ca;
	return((struct compounda *)ca->o[i]);
}


/*
 * Make the back-links in the compound objects,
 * typically used after reading from disc.
 */
flex_back_links_sc(fs)
struct flexible_screen *fs;
{
	struct compounda *ca = fs->cao;
	int i;

	for (i=0; i<ca->n; i++)
		flex_back_links_nd(ca,ca->o[i]);
	ca->assoc = (struct object *) ca;
}

flex_back_links_nd(ca,co)
struct compounda *ca,*co;
{
	int i;

	if (co != NULL && co->type == 81) {
		co->assoc = (struct object *) ca;
		for (i=0; i<co->n; i++)
			flex_back_links_nd(co,co->o[i]);
	}
}
