/*
 * flexsubs2.c		Jim Piper and Bill Hill @MRC		23-10-90
 *
 * Small utility functions for the flexible screen - NO MENUS PERMITTED HERE !!
 * Checking, Merging, Joining nodes.
 * See also flex.h, flex.c, flexsubs1.c, flexsubs3.c
 *
 * MODIFICATIONS
 * -------------
 * 25-01-91 ih		Extra debug in flex_leaf & flex_level
 * 23-01-91 jimp	Added flex_level(); took message out of
 *			flex_ungroup_nd().
 * 17-01-91 jimp	Bug-fix in flex_ungroup_nd()
 * 04-12-90 jimp	flex_ungroup_nd() returns parent node, flex_group_nd(),
 *			flex_merge_nd(), flex_add_nd() return group node.
 * 30-11-90 jimp	Added flex_ungroup_nd().
 * 28-11-90 jimp	Printing on #ifdef DEBUG.
 *
 */
#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <fileheaders.h>
#include <allcontr.h>
#include <flex.h>

extern FILE *debug_channel;
extern struct allkontr *alc;
/*
 * set of recursive functions to check flexible 
 * screen structure conforms to definitions.
 */
static check_depth = 0;
flex_check_sc(fs)
struct flexible_screen *fs;
{
	int i;
	struct compounda *ca = fs->cao;
	struct object *o;

	f_c_pr(0,"Flexible Screen Structure:");
	check_depth = 1;
	for (i=0; i<ca->n; i++) {
		o = ca->o[i];
		if (o == NULL)
			f_c_pr(check_depth,"Nu");
		else if (o->type == 81) {
			f_c_pr(check_depth,"Co");
			flex_check_nd(o);
		}
		else
			fprintf(stderr,"Flex_check_sc: incorrect object type %d\n",o->type);
	}
}

f_c_pr(d,s)
int d;
char *s;
{
	while (d-- > 0)
		fprintf(stderr,"  ");
	fprintf(stderr,"%s\n",s);
}

flex_check_nd(co)
struct compounda *co;
{
	int i;
	struct object **o = co->o;
	
	check_depth++;
	if (*o == NULL)
		fprintf(stderr,"Flex_check_nd: NULL first object\n");
	else if ((*o)->type == 20) {
		f_c_pr(check_depth,"Re");
		for (++o,i=1; i<co->n; i++,o++)
			if (*o == NULL)
				f_c_pr(check_depth,"Nu");
			else if ((*o)->type == 81) {
				f_c_pr(check_depth,"Co");
				flex_check_nd(*o);
			}
			else fprintf(stderr,"Flex_check_nd: incorrect object type %d\n",(*o)->type);
	}
	else if ((*o)->type == 60) {
		f_c_pr(check_depth,"Fr");
		++o;
		if (*o == NULL)
			fprintf(stderr,"Flex_check_nd: NULL leaf object\n");
		else if ((*o)->type == 1)
			f_c_pr(check_depth,"Ob");
		else if ((*o)->type == 10)
			f_c_pr(check_depth,"Po");
		else if ((*o)->type == 70)
			f_c_pr(check_depth,"Tx");
		else
			fprintf(stderr,"Flex_check_nd: Illegal leaf type %d\n",(*o)->type);
	}
	check_depth--;
}


/*
 * MERGE two nodes to make a single new node at the same level --
 * Cannot be used with leaf-nodes since the merged "irect" would
 * no longer permit identification of each leaf component.
 */
struct compounda *flex_merge_nd(nd1,nd2)
struct compounda *nd1, *nd2;
{
	struct compounda *parent, *flex_extend();
	int i1,i2;
	
	if (flex_leaf(nd1) || flex_leaf(nd2)) {
		fprintf(stderr,"FLEX_MERGE_ND: not permitted to merge leaves, use GROUP\n");
		return(NULL);
	}
	if (compound_count(nd1) + compound_count(nd2) > nd1->n)
		nd1 = flex_extend(nd1, nd1->n + nd2->n);
	i2 = 1;	/* skip the "irect" */
	for (i1 = 1; i1 < nd1->n; i1++) {
		if (nd1->o[i1] == NULL) {
			for (; i2 < nd2->n; i2++) {
				if (nd2->o[i2] != NULL) {
					nd1->o[i1] = nd2->o[i2];
					if (nd1->o[i1]->type == 81)
						((struct compounda *)nd1->o[i1])->assoc = (struct object *) nd1;
					nd2->o[i2] = NULL;
					break; /* from inner for-loop */
				}
			}
		}
	}
	parent = (struct compounda *) nd2->assoc;
	if (parent == NULL)
		fprintf(stderr,"FLEX_MERGE_ND: unset parent compounda\n");
	else {
		for (i1 = 0; i1 < parent->n; i1++)
			if (parent->o[i1] == (struct object *) nd2)
				parent->o[i1] = NULL;
	}
	nd2->assoc = NULL;
	flex_del_nd(nd2);
	flex_setbox_nd(nd1);
	return(nd1);
}

/*
 * GROUP two nodes to make a single new node at the same level --
 * Attach new group where first object used to be.
 * E.g. used to group related chromosome classes, or to group
 * a chromosome with a comment.
 * The original objects thus end up one level higher up the tree.
 */
struct compounda *flex_group_nd(nd1,nd2)
struct compounda *nd1, *nd2;
{
	struct compounda *nnd, *parent, *makecompounda();
	struct object *makeirect();
	int i;

	if (alc->debug) {
		fprintf(debug_channel,"entering flex_group_nd\n");
		Show_alloc();
	}
	nnd = makecompounda(81,1,3,NULL,0);
	nnd->o[0] = makeirect();
	nnd->o[1] = (struct object *) nd1;
	nnd->o[2] = (struct object *) nd2;
	/*
	 * attach new group where first object used to be
	 */
	parent = (struct compounda *) nd1->assoc;
	for (i=0; i<parent->n; i++)
		if (parent->o[i] == (struct object *) nd1)
			parent->o[i] = (struct object *) nnd;
	nnd->assoc = (struct object *) parent;
	/*
	 * remove old reference to second object
	 */
	parent = (struct compounda *) nd2->assoc;
	for (i=0; i<parent->n; i++)
		if (parent->o[i] == (struct object *) nd2)
			parent->o[i] = NULL;
	/*
	 * plant new backward references
	 */
	nd1->assoc = (struct object *) nnd;
	nd2->assoc = (struct object *) nnd;
	flex_setbox_nd(nnd);
	if (alc->debug) {
		fprintf(debug_channel,"entering flex_group_nd\n");
		Show_alloc();
	}
	return(nnd);
}


/*
 * ADD a node to an existing group - nd is added to gp
 */
struct compounda *flex_add_nd(gp,nd)
struct compounda *nd, *gp;
{
	int i;
	struct compounda *parent, *flex_extend();
	if (flex_leaf(gp)) {
		fprintf(stderr,"FLEX_ADD_ND: not permitted to add to a leaf, use GROUP\n");
		return(NULL);
	}
	if (compound_count(gp) >= gp->n)
		gp = flex_extend(gp, gp->n * 2);
	for (i=1; i<gp->n; i++)
		if (gp->o[i] == NULL) {
			gp->o[i] = (struct object *) nd;
			break;
		}
	/*
	 * remove old reference to newly grouped object
	 */
	parent = (struct compounda *) nd->assoc;
	for (i=0; i<parent->n; i++)
		if (parent->o[i] == (struct object *) nd)
			parent->o[i] = NULL;
	nd->assoc = (struct object *) gp;
	flex_setbox_nd(gp);
	return(gp);
}


/*
 * How many elements of a compound object are non-NULL ?
 */
int compound_count(ca)
struct compounda *ca;
{
	int i,count;
	if (ca->type != 81) {
		fprintf(stderr,"COMPOUND_COUNT: not a compounda (%d)\n",ca->type);
		return(0);
	}
	for (count=0,i=0; i<ca->n; i++)
		if (ca->o[i] != NULL)
			count++;
	return(count);
}

/*
 * Return TRUE if a leaf-node, otherwise false.
 * Here defined as a two-level structure, with a pframe as the
 * first object in the lower level.
 */
int flex_leaf(nd)
struct compounda *nd;
{
	struct compounda *nnd;
	if (alc->debug) {
		fprintf(debug_channel,"entering flex_leaf\n");
		Show_alloc();
	}
	if (nd == NULL || nd->type != 81)
		return(0);
	nnd = (struct compounda *) nd->o[1];
	if (nnd == NULL || nnd->type != 81)
		return(0);
	if (nnd->o[0] == NULL || nnd->o[0]->type != 60)
		return(0);
	return(1);
}

/*
 * Return the level of the node (fs->cao is level zero,
 * next is level one, etc.).
 */
int flex_level(nd)
struct compounda *nd;
{
	int level = 0;
	if (alc->debug) {
		fprintf(debug_channel,"entering flex_level\n");
		Show_alloc();
	}
	if (nd == NULL)
		return(-1);
	while (nd != (struct compounda *) nd->assoc) {
		nd = (struct compounda *) nd->assoc;
		level++;
	}
	if (alc->debug) {
		fprintf(debug_channel,"exiting flex_level\n");
		Show_alloc();
	}
	return(level);
}


/*
 * Ungroup a node.  Attach the sub-nodes to this node's parent
 * and delete this one.  Watch out for the top-level object in
 * the flexible screen, which cannot be treated this way.
 * Nor can leaf-nodes, but nodes containing only one sub-node
 * may be ungrouped - reducing the number of levels in this
 * branch by one.
 * Return the parent node (in case flex_extend was called).
 */
struct compounda *flex_ungroup_nd(ca)
struct compounda *ca;
{
	struct compounda *nca, *pca, *flex_extend();
	int i, j;
	if (ca == NULL || ca->type != 81)
		return(NULL);
	pca = (struct compounda *) ca->assoc;
	/* top-level of hierarchy ? */
	if (pca == ca)
		return(NULL);
	/* a leaf ? */
	if (flex_leaf(ca))
		return(NULL);
	/* enough space in parent ? */
	if (compound_count(pca) + compound_count(ca) > pca->n)
		pca = flex_extend(pca, pca->n + ca->n);
	/* delete the old parent-child reference */
	for (i=0; i<pca->n; i++)
		if (pca->o[i] == (struct object *) ca)
			pca->o[i] = NULL;
	/* Do the copying */
	for (i=1, j=1; i<ca->n; i++) {
		if (ca->o[i] != NULL) {
			for (; j<pca->n; j++) {
				if (pca->o[j] == NULL) {
					pca->o[j] = ca->o[i];
					((struct compounda *)(pca->o[j]))->assoc = (struct object *) pca;
					ca->o[i] = NULL;
					break;
				}
			}
		}
	}
	freeobj(ca);
	return(pca);
}
