/*
 * flexsubs3.c		Jim Piper and Bill Hill @MRC		23-10-90
 *
 * Small utility functions for the flexible screen - NO MENUS PERMITTED HERE !!
 * Display and associated routines.
 * See also flex.h, flex.c, flexsubs1.c, flexsubs2.c.
 *
 * MODIFICATIONS
 * -------------
 * 26-06-91 ih		Extra debug in flex_highlight
 * 29-01-91 jimp	Bug fix flex_setbox_nd().  Code for arrow/line width.
 * 28-01-91 jimp	Text objects "text size" made dependent on that
 *			stored in object * pframe->scale
 * 23-01-91 jimp	altered the action of flex_highlight_nd(), to draw
 *			a wide-line box.
 * 04-12-90 jimp	added flex_highlight_nd()
 * 03-12-90 Bill Hill	removed clear() from flex_draw_sc().
 * 30-11-90 jimp	Added flex_reposition_nd()
 * 28-11-90 jimp	Simplifications and rearrangements.
 * 27-11-90 jimp	Added colour switching in display.
 *
 */
#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <display.h>
#include <fileheaders.h>
#include <flex.h>
#include <allcontr.h>

extern FILE *debug_channel;
extern struct allkontr *alc;


/*
 * Compute the DIGS coordinate bounding box of a "node"
 * compound object, and update the "irect", the first element.
 * Called recursively, so that the current objects "irect"
 * may be computed either from leaf-object bounds or lower
 * level nodes' "irects".
 */
flex_setbox_fs(fs)
struct flexible_screen *fs;
{
	int i;
	struct compounda *ca = fs->cao;
	if (ca != NULL)
		for (i=0; i<ca->n; i++)
			flex_setbox_nd(ca->o[i]);
}

flex_setbox_nd(nd)
struct compounda *nd;
{
	int i,j,swap;
	int l1,ll,k1,kl;
	int fl1,fll,fk1,fkl;
	struct compounda *no;
	struct irect *r;
	struct intervaldomain *idmn;
	struct polygondomain *pdom;
	struct textdomain *tdom;
	struct pframe *f;
	struct ivertex *vtx;

	if (nd == NULL || nd->type != 81)
		return(0);
	if (nd->o[0]->type == 60) {
		return(0);
	}
	l1 = k1 = 30000;
	ll = kl = -30000;
	for (i=1; i<nd->n; i++) {
		no = (struct compounda *) nd->o[i];
		if (no == NULL)
			continue;
		flex_setbox_nd(no);
		if (no->o[0]->type == 20) {		/* irect */
			r = (struct irect *) no->o[0]->idom;
			fl1 = r->irl[0];
			fk1 = r->irk[0];
			fll = r->irl[2];
			fkl = r->irk[2];
		}
		else if (no->o[0]->type == 60) {	/* pframe */
			f = (struct pframe *) no->o[0];
			idmn = ((struct object *) no->o[1])->idom;
			switch (no->o[1]->type) {
			case 1:
				fl1 = (idmn->line1 - f->oy) * f->iy * f->scale + f->dy;
				fll = (idmn->lastln - f->oy) * f->iy * f->scale + f->dy;
				fk1 = (idmn->kol1 - f->ox) * f->ix * f->scale + f->dx;
				fkl = (idmn->lastkl - f->ox) * f->ix * f->scale + f->dx;
				break;
			case 10:
				pdom = (struct polygondomain *) idmn;
				vtx = pdom->vtx;
				fk1 = fkl = vtx->vtX;
				fl1 = fll = vtx->vtY;
				for (j=1,vtx++; j<pdom->nvertices; j++,vtx++) {
					if (vtx->vtX < fk1)
						fk1 = vtx->vtX;
					else if (vtx->vtX > fkl)
						fkl = vtx->vtX;
					if (vtx->vtY < fl1)
						fl1 = vtx->vtY;
					else if (vtx->vtY > fll)
						fll = vtx->vtY;
				}
				fl1 = (fl1 - f->oy) * f->iy * f->scale + f->dy;
				fll = (fll - f->oy) * f->iy * f->scale + f->dy;
				fk1 = (fk1 - f->ox) * f->ix * f->scale + f->dx;
				fkl = (fkl - f->ox) * f->ix * f->scale + f->dx;
				break;
			case 70:
				tdom = (struct textdomain *) idmn;
				fl1 = (tdom->l - f->oy) * f->iy * f->scale + f->dy;
				fll = fl1 + f->scale * tdom->spacing;
				fk1 = (tdom->k - f->ox) * f->ix * f->scale + f->dx;
				fkl = fk1 + f->scale * tdom->spacing * tdom->stringlen;
				break;
			default:
				break;
			}
		}
		/*
		 * If f->ix and f->iy are negative then must
		 * swap fk1 with fkl and fl1 with fll.
		 */
		if (fl1 > fll) {
			swap = fl1; fl1 = fll; fll = swap;
		}
		if (fk1 > fkl) {
			swap = fk1; fk1 = fkl; fkl = swap;
		}
		/*
		 * Update limits
		 */
		if (fl1 < l1)
			l1 = fl1;
		if (fll > ll)
			ll = fll;
		if (fk1 < k1)
			k1 = fk1;
		if (fkl > kl)
			kl = fkl;
	}
	r = (struct irect *) nd->o[0]->idom;
	r->irl[0] = r->irl[3] = l1 - 24;
	r->irl[1] = r->irl[2] = ll + 24;
	r->irk[0] = r->irk[1] = k1 - 24;
	r->irk[2] = r->irk[3] = kl + 24;
}

/*
 * Draw the flexible screen.
 * If boxes non-zero, draw the bounding boxes.
 */
flex_draw_sc(fs,boxes)
struct flexible_screen *fs;
int boxes;
{
	struct compounda *nd = fs->cao;
	int i;
	if (alc->debug) {
		fprintf(debug_channel,"flex_draw_sc:\n");
	}
	if (nd == NULL || nd->type != 81)
		return(0);
	for (i=0; i<nd->n; i++) {
		colour(GREYCOLS);
		if (alc->debug) {
			fprintf(debug_channel,"flex_draw_objs:\n");
		}
		flex_draw_objs_nd(nd->o[i]);
		if (boxes) {
			colour(OVERLAY1);
			if (alc->debug) {
				fprintf(debug_channel,"flex_draw_boxes_nd:\n");
			}
			flex_draw_boxes_nd(nd->o[i]);
		}
	}
			
}

/*
 * Draw every object below the input node.
 */
flex_draw_objs_nd(nd)
struct compounda *nd;
{
	int i;
	struct pframe *f;
	struct flex_poly_plist *plist;

	if (nd == NULL || nd->type != 81)
		return(0);
	else if (nd->o[0]->type == 60) {
		f = (struct pframe *) nd->o[0];
		switch (nd->o[1]->type) {
			case 10:
				plist = (struct flex_poly_plist *) nd->o[1]->plist;
				if (plist != NULL)
					lwidth(plist->linewidth);
				break;
			case 70 :
				i = f->scale * ((struct text *)nd->o[1])->tdom->spacing;
				tsize(i,i);
				break;
			default:
				break;
		}
		picframe(nd->o[1],f);
		lwidth(1);
	}
	else if (nd->o[0]->type == 20)
		for (i=1; i<nd->n; i++)
			flex_draw_objs_nd(nd->o[i]);
}


/*
 * This object displays the identified node or leaf by drawing
 * the irect in the current colour and intensity.
 * Also recursively draw the rectangles surrounding every node
 * contained within the current node (if not a leaf).
 */
static struct pframe rf = {60,1,0,0,0,0,1,1,NULL};
flex_draw_boxes_nd(ca)
struct compounda *ca;
{
	int i;
	if (ca == NULL)
		return(0);
	switch (ca->type) {
		case 20:
			picframe(ca,&rf);
			break;
		case 81:
			for (i=0; i<ca->n; i++)
				flex_draw_boxes_nd(ca->o[i]);
			break;
		default:
			break;
	}
	return(0);
}


/*
 * Routines for making the screen smaller or larger.
 * This involves altering every pframe and recomputing every irect.
 */
flex_frames_small(fs)
struct flexible_screen *fs;
{
	flex_change_frames(fs->cao,X_SMALL,fs->stat->dx,Y_SMALL,fs->stat->dy,S_SMALL);
	fs->stat->dx = X_SMALL;
	fs->stat->dy = Y_SMALL;
	fs->stat->scale = S_SMALL;
	flex_setbox_fs(fs);
}

flex_frames_big(fs)
struct flexible_screen *fs;
{
	flex_change_frames(fs->cao,X_LARGE,fs->stat->dx,Y_LARGE,fs->stat->dy,S_LARGE);
	fs->stat->dx = X_LARGE;
	fs->stat->dy = Y_LARGE;
	fs->stat->scale = S_LARGE;
	flex_setbox_fs(fs);
}

flex_change_frames(o,nx,ox,ny,oy,ns)
struct object *o;
int ox,nx,oy,ny,ns;
{
	struct pframe *f;
	struct compounda *ca;
	int i;

	if (o == NULL)
		return(0);
	switch (o->type) {
		case 60:
			f = (struct pframe *) o;
			f->dx = nx + ns*(f->dx - ox) / f->scale;
			f->dy = ny + ns*(f->dy - oy) / f->scale;
			f->scale = ns;
			return(1);
		case 81:
			ca = (struct compounda *) o;
			for (i=0; i<ca->n; i++)
				flex_change_frames(ca->o[i],nx,ox,ny,oy,ns);
			return(1);
		default:
			return(0);
	}
}


/*
 * Reposition objects in a node by tweaking the leaf pframes
 */
flex_reposition_nd(ca,x,y)
struct compounda *ca;
int x, y;
{
	int i;
	struct pframe *f;
	if (ca == NULL)
		return(0);
	switch (ca->type) {
		case 60:
			f = (struct pframe *) ca;
			f->dx += x;
			f->dy += y;
			return(0);
		case 81:
			for (i=0; i<ca->n; i++)
				flex_reposition_nd(ca->o[i],x,y);
			return(0);
		default:
			return(0);
	}
}


flex_highlight_nd(ca,col)
struct compounda *ca;
int col;
{
	if (alc->debug) {
		fprintf(debug_channel,"entering flex_highlight_nd\n");
		Show_alloc();
	}
	colour(OVERLAY2);
	intens(col);
	lwidth(3);
	flex_draw_boxes_nd(ca);
	lwidth(1);
	if (alc->debug) {
		fprintf(debug_channel,"exiting flex_highlight_nd\n");
		Show_alloc();
	}
}

