/*
 * flexsubs4.c		Jim Piper and Bill Hill @MRC		30-11-90
 *
 * Menu support routines - these do some box drawing etc., and interface
 * to the actual compound object manipulation routines.
 *
 * MODIFICATIONS
 * -------------
 * 3 Feb 1992 Joanne	Added extra bit in ideogram filename to identify 
 *			current species
 * 25-06-91 ih		Extra debug in flex_newgroup
 * 06-02-91 jimp	Added ideogram service functions.
 * 31-01-91 jimp	Message "cannot delete a group" added to
 *			flex_delete_ms().
 * 29-01-91 jimp	flex_arrow_ms() renamed flex_arrow_box_ms().
 * 28-01-91 jimp	Textsize now provided from menu.
 * 23-01-91 jimp	Added flex_newgroup_ms() for IRS's style of grouping,
 *			corrected box deletion in various procedures.  Added
 *			flex_delete_query().
 * 19-01-91 jimp	Added flex_delete_ms(),
 * 18-01-91 jimp	flex_newpos_ms() - bug fixed
 * 17-01-91 jimp	flex_ungroup_ms() - another bug fixed
 * 04-12-90 jimp	flex_ungroup_ms() bug fixed
 */
#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <display.h>
#include <fileheaders.h>
#include <flex.h>
#include <cmdsys.h>
#include <allcontr.h>

extern FILE *debug_channel;
extern struct allkontr *alc;

extern char *getspec();

/*
 * Reposition a node.  Undraw the images, move and redraw.
 * If boxes, do the same with the boxes.  In this case, must
 * also recompute the boxes lower down the tree (towards the
 * root) since they will probably have changed.
 */
flex_newpos_ms(ca,x,y,boxes)
struct compounda *ca;
int x, y, boxes;
{
	struct irect *r = (struct irect *) ca->o[0]->idom;
	struct compounda *pca, *qca, *rca;

	x -= r->irk[2];
	y -= r->irl[2];

	intens(0);
	colour(GREYCOLS);
	flex_draw_objs_nd(ca);

	qca = pca = ca;
	while ((rca = (struct compounda *) pca->assoc) != pca) {
		qca = pca;
		pca = rca;
	}

	colour(OVERLAY1 | OVERLAY2);	/* get rid of the I.D. box */
	flex_draw_boxes_nd(qca);
	colour(GREYCOLS);
	flex_reposition_nd(ca,x,y);
	flex_setbox_nd(qca);
	intens(1);
	flex_draw_objs_nd(ca);
	if (boxes) {
		colour(OVERLAY1);
		flex_draw_boxes_nd(qca);
	}
}


flex_ungroup_ms(ca,boxes)
struct compounda *ca;
int boxes;
{
	struct compounda *pca, *qca, *rca, *flex_ungroup_nd();
	qca = pca = ca;
	while ((rca = (struct compounda *) pca->assoc) != pca) {
		qca = pca;
		pca = rca;
	}
	intens(0);
	colour(OVERLAY1 | OVERLAY2);	/* get rid of the I.D. box */
	flex_draw_boxes_nd(qca);
	rca = flex_ungroup_nd(ca);
	if (rca == NULL) {
		fprintf(stderr,"not a group!\n");
		putprompt();
	}
	else {
		qca = rca;
		/*
		 * don't set boxes for the top level - there ain't a box
		 */
		if (rca != pca)
			flex_setbox_nd(rca);
	}
	intens(1); /* Methinks we need this here, not just in if below - ih*/
	if (boxes) {
		colour(OVERLAY1);
		flex_draw_boxes_nd(qca);
	}
	return(rca==NULL?0:1);
}


flex_newgroup_ms(ca1,ca2,boxes)
struct compounda *ca1,*ca2;
int boxes;
{
	struct compounda *pca;
	struct compounda *flex_group_nd(), *flex_merge_nd(), *flex_add_nd();

	if (alc->debug) {
		fprintf(debug_channel,"entering flex_newgroup_ms\n");
		Show_alloc();
	}
	intens(0);
	colour(OVERLAY1|OVERLAY2);
	flex_draw_boxes_nd(ca1);
	flex_draw_boxes_nd(ca2);
	if (flex_leaf(ca1) && flex_leaf(ca2))
		pca = flex_group_nd(ca1,ca2);
	else if (!flex_leaf(ca1) && !flex_leaf(ca2))
		pca = flex_merge_nd(ca1,ca2);
	else if (flex_leaf(ca1))
		pca = flex_add_nd(ca2,ca1);
	else
		pca = flex_add_nd(ca1,ca2);
	flex_setbox_nd(pca);
	if (boxes) {
		intens(1);
		colour(OVERLAY1);
		flex_draw_boxes_nd(pca);
	}
	if (alc->debug) {
		fprintf(debug_channel,"exiting flex_newgroup_ms\n");
		Show_alloc();
	}
	return(1);
}


flex_group_ms(ca1,ca2,boxes)
struct compounda *ca1,*ca2;
int boxes;
{
	struct compounda *pca, *qca, *rca, *flex_group_nd();
	rca = pca = qca = flex_group_nd(ca1,ca2);
	while ((rca = (struct compounda *) pca->assoc) != pca) {
		qca = pca;
		pca = rca;
	}
	intens(0);
	colour(OVERLAY1 | OVERLAY2);	/* get rid of the I.D. box */
	flex_draw_boxes_nd(qca);
	flex_setbox_nd(qca);
	if (boxes) {
		intens(1);
		colour(OVERLAY1);
		flex_draw_boxes_nd(qca);
	}
	return(1);
}

flex_merge_ms(ca1,ca2,boxes)
struct compounda *ca1,*ca2;
int boxes;
{
	struct compounda *pca, *qca, *rca, *flex_merge_nd();
	intens(0);
	colour(OVERLAY1 | OVERLAY2);	/* get rid of the I.D. box */
	flex_draw_boxes_nd(ca1);
	flex_draw_boxes_nd(ca2);
	rca = pca = qca = flex_merge_nd(ca1,ca2);
	if (rca == NULL) {
		if (boxes) {
			intens(1);
			colour(OVERLAY1);
			flex_draw_boxes_nd(ca1);
			flex_draw_boxes_nd(ca2);
		}
		return(0);
	}
	while ((rca = (struct compounda *) pca->assoc) != pca) {
		qca = pca;
		pca = rca;
	}
	intens(0);
	colour(OVERLAY1 | OVERLAY2);	/* get rid of the I.D. box */
	flex_draw_boxes_nd(qca);
	flex_setbox_nd(qca);
	if (boxes) {
		intens(1);
		colour(OVERLAY1);
		flex_draw_boxes_nd(qca);
	}
	return(1);
}

flex_add_ms(ca1,ca2,boxes)
struct compounda *ca1,*ca2;
int boxes;
{
	struct compounda *pca, *qca, *rca, *flex_add_nd();
	intens(0);
	colour(OVERLAY1 | OVERLAY2);	/* get rid of the I.D. box */
	flex_draw_boxes_nd(ca1);
	rca = pca = qca = flex_add_nd(ca1,ca2);
	if (rca == NULL) {
		if (boxes) {
			intens(1);
			colour(OVERLAY1);	/* restore the I.D. box */
			flex_draw_boxes_nd(ca1);
		}
		return(0);
	}
	while ((rca = (struct compounda *) pca->assoc) != pca) {
		qca = pca;
		pca = rca;
	}
	flex_setbox_nd(qca);
	if (boxes) {
		intens(1);
		colour(OVERLAY1);
		flex_draw_boxes_nd(qca);
	}
	return(1);
}


/*
 * Delete an object, but only if it is a leaf at level 1.
 * NULL checked in menu.
 */
int flex_delete_ms(ca,boxes)
struct compounda *ca;
int boxes;
{
	struct compounda *pca;
	int i;
	if (flex_level(ca) > 1) {
		printf("Cannot delete within a group\n");
		putprompt();
		return(0);
	}
	else if (!flex_leaf(ca)) {
		printf("Cannot delete a group\n");
		putprompt();
		return(0);
	}
	else if (flex_delete_query(ca)) {
		/*
		 * undraw
		 */
		intens(0);
		colour(GREYCOLS);
		flex_draw_objs_nd(ca);
		colour(OVERLAY1 | OVERLAY2);
		flex_draw_boxes_nd(ca);
		/*
		 * delete link from parent
		 */
		pca = (struct compounda *) ca->assoc;
		for (i=0; i<pca->n; i++)
			if (pca->o[i] == (struct object *) ca)
				pca->o[i] = NULL;
		/*
		 * delete the leaf object
		 */
		flex_del_nd(ca);
		return(1);
	}
	else
		return(0);
}

flex_arrow_box_ms(fs,boxes)
struct flexible_screen *fs;
int boxes;
{
	struct compounda *ca, *flex_makeleaf();
	struct pframe *f, *makepframe();
	int i;
	f = makepframe(fs->stat->scale,0,0,0,0);
	flex_poly_rescale(fs->stat->current_obj,fs->stat->scale);
	ca = flex_makeleaf(fs->stat->current_obj,f);
	fs->stat->current_obj = NULL;
	for (i=0; i<fs->cao->n; i++) {
		if (fs->cao->o[i] == NULL) {
			fs->cao->o[i] = (struct object *) ca;
			ca->assoc = (struct object *) fs->cao;
			break;
		}
	}
	flex_setbox_nd(ca);
	colour(GREYCOLS);
	intens(1);
	flex_draw_objs_nd(ca);
	if (boxes) {
		colour(OVERLAY1);
		flex_draw_boxes_nd(ca);
	}
	return(1);
}

/*
 * rescale the polygon - divide vertex values by scale
 */
flex_poly_rescale(p,scale)
struct object *p;
int scale;
{
	int i;
	struct polygondomain *pdom = (struct polygondomain *)p->idom;
	struct ivertex *vtx = pdom->vtx;
	for (i=0; i<pdom->nvertices; i++,vtx++) {
		vtx->vtX /= scale;
		vtx->vtY /= scale;
	}
}

flex_get_text_ms(fs,x,y,boxes,textsize)
struct flexible_screen *fs;
int x,y,boxes,textsize;
{
	struct text *t, *maketext();
	struct compounda *ca, *flex_makeleaf();
	struct pframe *f, *makepframe();
	char *ss, s[100], *Malloc();
	int i, n=0;
	fprintf(stderr,"Type a line of text, press return when complete:\n");
	while ((s[n++] = getchar()) != '\n')
		;
	s[--n] = '\0';
	if (n == 0)
		return(0);
	ss = Malloc(n+1);
	strcpy(ss,s,n);
	t = maketext(ss,0,0,n);
	t->tdom->spacing = textsize;
	f = makepframe(fs->stat->scale,x,y,0,0);
	ca = flex_makeleaf(t,f);
	for (i=0; i<fs->cao->n; i++) {
		if (fs->cao->o[i] == NULL) {
			fs->cao->o[i] = (struct object *) ca;
			ca->assoc = (struct object *) fs->cao;
			break;
		}
	}
	flex_setbox_nd(ca);
	colour(GREYCOLS);
	intens(1);
	flex_draw_objs_nd(ca);
	if (boxes) {
		colour(OVERLAY1);
		flex_draw_boxes_nd(ca);
	}
	return(n);
}


bleep()
{
	fprintf(stderr,"\007");
	tsleep(25);
	fprintf(stderr,"\007");
}

/*
 * Does the user really wish to delete an object ??
 */
int flex_delete_query(ca)
struct compounda *ca;
{
	int answer;
	bleep();
	flex_highlight_nd(ca,1);
	screengo(ENDLINE,1);
	fprintf(output,"DO YOU REALLY WANT TO DELETE THIS OBJECT ??\n");
	answer = go_on();
	putprompt();
	flex_highlight_nd(ca,0);
	return(answer);
}


/*
 * Find the class (plist->pgroup) of the current
 * object if a karyotyped chromosome.  If not,
 * then return the input class.
 */
int flex_ideoclass_ms(fs,class)
struct flexible_screen *fs;
int class;
{
	struct compounda *ca;
	struct chromosome *o;
	if ((ca = fs->stat->current_obj) == NULL) {
		bleep();
		return(class);
	}
	if (!flex_leaf(ca)) {
		bleep();
		return(class);
	}
	ca = (struct compounda *) ca->o[1];
	if (ca == NULL || ca->type != 81) {
		bleep();
		return(class);
	}
	o = (struct chromosome *) ca->o[1];
	if (o == NULL || o->type != 1) {
		bleep();
		return(class);
	}
	if (o->plist->pgroup > 0 && o->plist->pgroup <= 24)
		class = o->plist->pgroup;
	else {
		bleep();
		printf("Not yet classified\n");
		putprompt();
	}
	return(class);
}

/*
 * Place the specified ideogram at the specified location
 */
int flex_ideoput_ms(fs,class,resolution,x,y)
struct flexible_screen *fs;
int class,resolution,x,y;
{
	struct object *ido, *readobj();
	struct compounda *ca, *flex_fill_spot();
	FILE *in, *fbopen();
	char s[100], *getspec();
/*
 *	sprintf(s,"/dd/params/ideograms/ideo.%d.%d", class,resolution);
 *	put extra bit in file name to identify the current species
 */
	sprintf(s,"/dd/params/ideograms/ideo.%s.%d.%d",getspec(),class,resolution);
	in = fbopen(s,"r",BUFSIZ);
	if (in != NULL) {
		ido = readobj(in);
		fbclose(in);
		if (ido != NULL) {
			ca = flex_fill_spot(fs,ido,x,y,fs->stat->scale,1);
			if (ca != NULL) {
				flex_setbox_nd(ca);
				intens(1);
				colour(GREYCOLS);
				flex_draw_objs_nd(ca);
				if (fs->stat->boxes) {
					colour(OVERLAY1);
					flex_draw_boxes_nd(ca);
				}
				return(1);
			}
		}
	}
	fprintf(stderr, "file %s for ideograms not found\n", s);
	putprompt();
	return(0);
}

