/*
 * flexsubs5.c		Jim Piper and Bill Hill @MRC		19-01-91
 *
 * Enhancement routines.
 *
 ********************************************************************
 * THERE IS SCOPE FOR A CONSIDERABLE AMOUNT OF CODE-SAVING HERE BY
 * EXTRACTING COMMON CODE AS SUBROUTINES, OR POSSIBLY BY PASSING
 * FUNCTIONS TO A MORE GENERAL PROCEDURE.
 ********************************************************************
 *
 * MODIFICATIONS
 * -------------
 * 25-06-91 ih		Added Show_alloc in flex_enlarge_ms
 * 18-06-91 ih		Check for NULL fs->stat
 * 28-01-91 jimp	Alter pframe if straightening/recovering a leaf.
 * 27-01-91 jimp	Added flex_rotate_ms(), flex_normalise_ms(),
 *			flex_enhance_ms(), flex_normalise_nd(),
 *			flex_enhance_nd().
 * 25-01-91 jimp	Added flex_straighten_nd(), finished flex_straighten_ms().
 * 19-01-91 jimp	Added flex_enlarge_ms(),
 *			flex_straighten_ms(), flex_recover_ms().
 */
#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <display.h>
#include <cmdsys.h>
#include <fileheaders.h>
#include <kcontrol.h>
#include <allcontr.h>
#include <flex.h>

extern FILE *debug_channel;
extern struct allkontr *alc;

/*
 * Enlarge by specified percentage;
 * either just the objects in the current group, or all image
 * objects, depending on the setting of fs->stat->alltoggle.
 * If by groups, then fs->stat->current_obj must be non-NULL.
 * Return 0 for failure, 1 for success.
 */
int
flex_enlarge_ms(fs,percent)
struct flexible_screen *fs;
int percent;
{
	int i;
	struct compounda *ca, *pca, *qca, *rca;
	if (alc->debug) {
		fprintf(debug_channel,"entering flex_enlarge_ms:\n");
		Show_alloc();
	}
	if (!fs->stat) return(0);
	if (!fs->stat->alltoggle) {
		ca = fs->stat->current_obj;
		if (ca == NULL)
			return(0);
		pca = ca;
		while ((rca = (struct compounda *) pca->assoc) != pca) {
			qca = pca;
			pca = rca;
		}
		intens(0);
		colour(GREYCOLS);
		flex_draw_objs_nd(ca);
		colour(OVERLAY1 | OVERLAY2);	/* get rid of the I.D. box */
		flex_draw_boxes_nd(qca);
		flex_enlarge_nd(ca,percent);
		flex_setbox_nd(qca);
		intens(1);
		colour(GREYCOLS);
		flex_draw_objs_nd(ca);
		if (fs->stat->boxes) {
			colour(OVERLAY1);
			flex_draw_boxes_nd(qca);
		}
		if (alc->debug) {
			fprintf(debug_channel,"exiting flex_enlarge_ms:\n");
			Show_alloc();
		}
		return(1);
	}
	else {
		fprintf(stderr,"Processing...");
		for (i=0; i<fs->cao->n; i++) {
			if (fs->cao->o[i] != NULL) {
				flex_enlarge_nd(fs->cao->o[i],percent);
				fprintf(stderr,"+");
				flex_setbox_nd(fs->cao->o[i]);
			}
		}
		fprintf(stderr," ...done\n");
		putprompt();
		clear();
		flex_draw_sc(fs,fs->stat->boxes);
	}
	if (alc->debug) {
		fprintf(debug_channel,"exiting flex_enlarge_ms:\n");
		Show_alloc();
	}
	return(1);
}
		

/*
 * Straighten:
 * either just the objects in the current group, or all image
 * objects, depending on the setting of fs->stat->alltoggle.
 * If by groups, then fs->stat->current_obj must be non-NULL.
 * Return 0 for failure, 1 for success.
 */
int
flex_straighten_ms(fs)
struct flexible_screen *fs;
{
	int i;
	struct compounda *ca, *pca, *qca, *rca;
	if (!fs->stat) return(0);
	if (!fs->stat->alltoggle) {
		ca = fs->stat->current_obj;
		if (ca == NULL)
			return(0);
		pca = ca;
		while ((rca = (struct compounda *) pca->assoc) != pca) {
			qca = pca;
			pca = rca;
		}
		intens(0);
		colour(GREYCOLS);
		flex_draw_objs_nd(ca);
		colour(OVERLAY1 | OVERLAY2);	/* get rid of the I.D. box */
		flex_draw_boxes_nd(qca);
		flex_straighten_nd(ca);
		flex_setbox_nd(qca);
		intens(1);
		colour(GREYCOLS);
		flex_draw_objs_nd(ca);
		if (fs->stat->boxes) {
			colour(OVERLAY1);
			flex_draw_boxes_nd(qca);
		}
		return(1);
	}
	else {
		fprintf(stderr,"Processing...");
		for (i=0; i<fs->cao->n; i++) {
			if (fs->cao->o[i] != NULL) {
				flex_straighten_nd(fs->cao->o[i]);
				fprintf(stderr,"+");
				flex_setbox_nd(fs->cao->o[i]);
			}
		}
		fprintf(stderr," ...done\n");
		putprompt();
		clear();
		flex_draw_sc(fs,fs->stat->boxes);
	}
	return(1);
}
		

/*
 * Normalise:
 * either just the objects in the current group, or all image
 * objects, depending on the setting of fs->stat->alltoggle.
 * If by groups, then fs->stat->current_obj must be non-NULL.
 * Return 0 for failure, 1 for success.
 */
int
flex_normalise_ms(fs)
struct flexible_screen *fs;
{
	int i;
	struct compounda *ca;
	intens(1);
	colour(GREYCOLS);
	if (!fs->stat) return(0);
	if (!fs->stat->alltoggle) {
		ca = fs->stat->current_obj;
		if (ca == NULL)
			return(0);
		flex_normalise_nd(ca);
		flex_draw_objs_nd(ca);
		return(1);
	}
	else {
		fprintf(stderr,"Processing...");
		for (i=0; i<fs->cao->n; i++) {
			ca = (struct compounda *) fs->cao->o[i];
			if (ca != NULL) {
				flex_normalise_nd(ca);
				flex_draw_objs_nd(ca);
				fprintf(stderr,"+");
			}
		}
		fprintf(stderr," ...done\n");
		putprompt();
	}
	return(1);
}
		

/*
 * Enhance:
 * either just the objects in the current group, or all image
 * objects, depending on the setting of fs->stat->alltoggle.
 * If by groups, then fs->stat->current_obj must be non-NULL.
 * Return 0 for failure, 1 for success.
 */
int
flex_enhance_ms(fs)
struct flexible_screen *fs;
{
	int i;
	struct compounda *ca;
	intens(1);
	colour(GREYCOLS);
	if (!fs->stat) return(0);
	if (!fs->stat->alltoggle) {
		ca = fs->stat->current_obj;
		if (ca == NULL)
			return(0);
		flex_enhance_nd(ca);
		flex_draw_objs_nd(ca);
		return(1);
	}
	else {
		fprintf(stderr,"Processing...");
		for (i=0; i<fs->cao->n; i++) {
			ca = (struct compounda *) fs->cao->o[i];
			if (ca != NULL) {
				flex_enhance_nd(ca);
				flex_draw_objs_nd(ca);
				fprintf(stderr,"+");
			}
		}
		fprintf(stderr," ...done\n");
		putprompt();
	}
	return(1);
}
		

/*
 * Recover:
 * either just the objects in the current group, or all image
 * objects, depending on the setting of fs->stat->alltoggle.
 * If by groups, then fs->stat->current_obj must be non-NULL.
 * Return 0 for failure, 1 for success.
 */
flex_recover_ms(fs)
struct flexible_screen *fs;
{
	int i;
	struct compounda *ca, *pca, *qca, *rca;
	if (!fs->stat) return(0);
	if (!fs->stat->alltoggle) {
		ca = fs->stat->current_obj;
		if (ca == NULL)
			return(0);
		pca = ca;
		while ((rca = (struct compounda *) pca->assoc) != pca) {
			qca = pca;
			pca = rca;
		}
		intens(0);
		colour(GREYCOLS);
		flex_draw_objs_nd(ca);
		colour(OVERLAY1 | OVERLAY2);	/* get rid of the I.D. box */
		flex_draw_boxes_nd(qca);
		flex_recover_nd(ca);
		flex_setbox_nd(qca);
		intens(1);
		colour(GREYCOLS);
		flex_draw_objs_nd(ca);
		if (fs->stat->boxes) {
			colour(OVERLAY1);
			flex_draw_boxes_nd(qca);
		}
		return(1);
	}
	else {
		for (i=0; i<fs->cao->n; i++) {
			flex_recover_nd(fs->cao->o[i]);
			flex_setbox_nd(fs->cao->o[i]);
		}
		clear();
		flex_draw_sc(fs,fs->stat->boxes);
	}
	return(1);
}


/*
 * Enlarge all the objects in the branch from this node
 * by the specified percentage.
 */
int
flex_enlarge_nd(ca,percent)
struct compounda *ca;
int percent;
{
	int i;
	struct object *o, *no, *squeeze();
	double f;
	if (ca == NULL || ca->type != 81)
		return(0);
	f = (double) percent / 100.0;
	for (i=0; i<ca->n; i++) {
		o = ca->o[i];
		if (o != NULL) {
			switch (o->type) {
				case 1:
					ca->o[i] = no = squeeze(o,f,f);
					no->plist = o->plist;
					if (o->assoc == NULL)
						no->assoc = o;
					else {
						no->assoc = o->assoc;
						o->plist = NULL;
						o->assoc = NULL;
						freeobj(o);
					}
					break;
				case 81:
					flex_enlarge_nd(o,percent);
					break;
				default:
					break;
			}
		}
	}
}

/*
 * Straighten all the objects in the branch from this node
 */
int flex_straighten_nd(ca)
struct compounda *ca;
{
	int i;
	struct object *o, *no, *autostraight();
	struct pframe *f;
	if (ca == NULL || ca->type != 81)
		return(0);
	for (i=0; i<ca->n; i++) {
		o = ca->o[i];
		if (o != NULL) {
			switch (o->type) {
				case 1:
					ca->o[i] = no = autostraight(o);
					/*
					 * Assume a standard leaf structure,
					 * find and modify the pframe.
					 * Check just in case.....
					 */
					f = (struct pframe *) ca->o[0];
					if (f->type == 60) {
						if (f->iy > 0) {
							f->ox = no->idom->kol1;
							f->oy = no->idom->line1;
						}
						else {
							f->ox = no->idom->lastkl;
							f->oy = no->idom->lastln;
						}
					}
					else
						printf("Flex_straight_nd: leaf structure error\n");
					no->plist = o->plist;
					if (o->assoc == NULL)
						no->assoc = o;
					else {
						no->assoc = o->assoc;
						o->plist = NULL;
						o->assoc = NULL;
						freeobj(o);
					}
					break;
				case 81:
					flex_straighten_nd(o);
					break;
				default:
					break;
			}
		}
	}
}
					
					
/*
 * Normalise all chromosome objects in this branch.
 */
int
flex_normalise_nd(ca)
struct compounda *ca;
{
	int i;
	struct object *o, *no, *newgrey();
	if (ca == NULL || ca->type != 81)
		return(0);
	for (i=0; i<ca->n; i++) {
		o = ca->o[i];
		if (o != NULL) {
			switch (o->type) {
				case 1:
					ca->o[i] = no = newgrey(o);	/* links plist */
					normalise(no);
					if (o->assoc == NULL)
						no->assoc = o;
					else {
						no->assoc = o->assoc;
						o->plist = NULL;
						o->assoc = NULL;
						freeobj(o);
					}
					break;
				case 81:
					flex_normalise_nd(o);
					break;
				default:
					break;
			}
		}
	}
}


/*
 * Enhance all chromosome objects in this branch.
 * Requires "kcontrol" structure - inserted as a global.
 */
int
flex_enhance_nd(ca)
struct compounda *ca;
{
	int i;
	extern struct kcontrol *kcont;
	struct object *o, *no, *enhance(), *varenhance();
	if (ca == NULL || ca->type != 81)
		return(0);
	for (i=0; i<ca->n; i++) {
		o = ca->o[i];
		if (o != NULL) {
			switch (o->type) {
				case 1:
					if (kcont->enh != NULL)
						no = varenhance(o, kcont->enh);
					else
						no = enhance(o);
					ca->o[i] = no;
					no->plist = o->plist;
					if (o->assoc == NULL)
						no->assoc = o;
					else {
						no->assoc = o->assoc;
						o->plist = NULL;
						o->assoc = NULL;
						freeobj(o);
					}
					break;
				case 81:
					flex_enhance_nd(o);
					break;
				default:
					break;
			}
		}
	}
}


/*
 * Recover all the objects in the branch from this node.
 */
int
flex_recover_nd(ca)
struct compounda *ca;
{
	int i;
	struct object *o, *no;
	struct pframe *f;
	if (ca == NULL || ca->type != 81)
		return(0);
	for (i=0; i<ca->n; i++) {
		o = ca->o[i];
		if (o != NULL) {
			switch (o->type) {
				case 1:
					if (o->assoc != NULL) {
						ca->o[i] = no = o->assoc;
						no->plist = o->plist;
						o->plist = NULL;
						freeobj(o);
					/*
					 * Assume a standard leaf structure,
					 * find and modify the pframe (which
					 * may have been changed by e.g.
					 * straightening).
					 * Check just in case.....
					 */
						f = (struct pframe *) ca->o[0];
						if (f->type == 60) {
							if (f->iy > 0) {
								f->ox = no->idom->kol1;
								f->oy = no->idom->line1;
							}
							else {
								f->ox = no->idom->lastkl;
								f->oy = no->idom->lastln;
							}
						}
						else
							printf("Flex_recover_nd: leaf structure error\n");
					}
					break;
				case 81:
					flex_recover_nd(o);
					break;
				default:
					break;
			}
		}
	}
}
					
					


/*
 * Rotate an object, but only if it is a leaf containing a chromosome.
 * Code largely copied from kmr/rotchrom.c.
 */
#define RADIANS	.01745
#define RAD 300		/* radius of displayed circle */
#define ROUND	80	/* from rotchrom.c (header file better !) */
#define QROUND	20
int flex_rotate_ms(ca,boxes)
struct compounda *ca;
int boxes;
{
	struct compounda *lca, *pca, *qca, *rca;
	struct object *o, *retobj, *spin();
	int ang, opt, xcr,ycr,xcn,ycn;
	int rotcurs(), flex_curs1();
	
	if (!flex_leaf(ca))
		return(0);
	lca = (struct compounda *) ca->o[1];
	o = lca->o[1];
	if (o->type != 1)
		return(0);
	
	pca = ca;
	while ((rca = (struct compounda *) pca->assoc) != pca) {
		qca = pca;
		pca = rca;
	}
	intens(0);
	colour(OVERLAY1|OVERLAY2);
	flex_draw_boxes_nd(qca);
	/*
	 * Find object center in DDGS coordinates
	 */
	xcr = (o->idom->lastkl + o->idom->kol1)/2;
	ycr = (o->idom->lastln + o->idom->line1)/2;
	xcn = frdigsxpos(xcr,lca->o[0]); /* object middle in digs coords */
	ycn = frdigsypos(ycr,lca->o[0]);

	flex_curs1_end();
	colour(OVERLAY2);
	intens(1);
	mycircle(xcn,ycn,RAD);
	setIang(0,1,xcn,ycn);
	setmsv(rotcurs);

	rotcurs();
	ang = opt = 0;
	retobj = NULL;

	downmenu();
	newmenu("Rotate Chromosome");
	cmmd("angle;%d",&ang);
	cmmd("rotate so that present cursor position is top");mouse(MIDBT);
	if ((opt = thismu()) == 2)
		ang = ((QROUND-getIang()) * 360) / ROUND;
	upmenu();

	if (opt && ang != 0) {
		intens(0);
		colour(GREYCOLS);
		flex_draw_objs_nd(ca);
		lca->o[1] = retobj = spin(o,(double)ang*RADIANS);
		retobj->plist = o->plist;
		if (o->assoc) {
			retobj->assoc = o->assoc;
			o->plist = NULL;
			freeobj(o);
		}
		else
			retobj->assoc = o;
		flex_setbox_nd(qca);
		intens(1);
		colour(GREYCOLS);
		flex_draw_objs_nd(ca);
	}
	eraserotcurs();
	intens(0);
	colour(OVERLAY2);
	mycircle(xcn,ycn,RAD);
	intens(1);
	if (boxes) {
		colour(OVERLAY1);
		flex_draw_boxes_nd(qca);
	}
	setmsv(flex_curs1);
	return(opt);
}



