/*
 * autosplit.c
 *
 * Routine to attach Cytoscan interaction to Ji Liang's
 * auto-touch-split and auto-overlap-split routines.
 *
 * Entries are
 *	autosplit(kcont,TYPE) - attempt to split current object, if apparently
 *		successful, display boundaries and return TRUE.
 *	endautosplit(kcont,TYPE) - new objects accepted - add to kcont.
 *	cleanautosplit(kcont) - new objects not accepted - clean up.
 * FOR NOW, new objects/boundaries are stored in local/static storage
 * until picked up by "endautosplit".
 *
 * MODIFICATIONS
 * -------------
 *	07 Jan 1992	Joanne	One of the objects identified after splitting.
 *	05 Mar 1990	dcb			Replace OVERLAY3 by OVERLAY2
 *	15 Feb 1989	CAS/SEAJ	Too many objects handled properly lower down.
 *							messages made consistant and more appropriate
 *	24 Nov 1988		SCG/dcb	modified calls to split_comps and split_overs to
 *							take into account the length of the remaining 
 *							object ptr array
 * jimp 09-09-87	Failure message added.
 * jimp 20-08-87	SI into kcont
 * jimp 03-08-87	Modified for seg_control.h
 * jimp 30-07-87	Modified for numbers_toggle
 * jimp	16-07-87	Modified for different call to split_comps()
 * jimp 29-07-87	Restored lost code for overlap split
 */
#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <display.h>
#include <seg_control.h>



autosplit(kcont,TYPE)
struct kcontrol *kcont;
int		TYPE;
{
	struct object *ibound();
	struct pframe *f = kcont->f;
	struct chromosome *obj;
	int i, res;
	
	kcont->SI->objnum = kcont->current_obj - 1;
	obj = kcont->ool[kcont->SI->objnum];
	if (TYPE == COMPOSITE)
		res = split_comps(obj, kcont->SI->objl, (MAXOBJS - kcont->maxnumber), &kcont->SI->splitnum,f,OVERLAY1,OVERLAY1);
	else if (TYPE == OVERLAP) {
		/*
		 * allow the overlap only if there is enough object pointer space!
		 */
		if((MAXOBJS - kcont->maxnumber) > 2) {
			res = split_overs(obj,kcont->SI->objl,kcont->SI->objl+1,f,OVERLAY1,OVERLAY1);
			kcont->SI->splitnum = 2;
		} else {
			fprintf(stderr,"Too many objects! Automatic decomposition failed.\007\n");
			return(0);
		}
	}
	if (res) {
		for (i=0; i<kcont->SI->splitnum; i++) {
			kcont->SI->boundlist[i] = ibound(kcont->SI->objl[i],1);
			colour(OVERLAY2);
			intens(1);
			picframe(kcont->SI->boundlist[i],f);
		}
		return(1);
	}
	else {
		fprintf(stderr,"Automatic decomposition failed: please split manually\007\n");
		return(0);
	}
}

/*
 * Auto-split judged satisfactory.  Clean up the display,
 * attach and fill property lists for the new objects.
 */
endautosplit(kcont,TYPE)
struct kcontrol *kcont;
{
	struct chromplist *qlist, *plist, *makechromplist();
	struct object *blobmark();
	struct chromosome *obj;
	struct pframe *f;
	int i,number;
	number = kcont->maxnumber;
	i_clean();
	w_clean();
	f = kcont->f;
	kcont->acl[kcont->SI->objnum] &= ~ACTIVE;
	obj = kcont->ool[kcont->SI->objnum];
	qlist = obj->plist;
	qlist->otype = TYPE;
	qlist->Cotype = TYPE;
	qlist->otconf = 80;
	qlist->history[1] = 8;	/* AUTOMATICALLY split */
	for (i=number; i<number+kcont->SI->splitnum; i++) {
		kcont->oth[i].bound = kcont->SI->boundlist[i-number];
		kcont->ool[i] = kcont->SI->objl[i-number];
		kcont->bol[i] = blobmark(kcont->ool[i]);
		colour(OVERLAY1);
		picframe(kcont->bol[i],f);
		if (kcont->SI->numbers_toggle)
			frameobjnum(i+1,kcont->ool[i],f);
		kcont->ool[i]->plist = makechromplist();
		plist = kcont->ool[i]->plist;
		plist->number = i+1;
		plist->pnumber = kcont->SI->objnum+1;
		plist->otype = UNKNOWN; 	/* not necessarily chromosome */
		plist->Cotype = UNKNOWN;
		probotype(kcont->ool[i]);
		kcont->acl[i] = ACTIVE|INTERACTING|IDENTIFY1;
/*		checkattention(1,i,kcont);*/
		plist->history[0] = 8;	/* result of AUTOMATIC split */
	}
	kcont->maxnumber += kcont->SI->splitnum;
	return(kcont->SI->splitnum);	/* compatible with "gotpnow" */
}

/*
 * Clean up the display if the auto-split is judged to be
 * unsatisfactory, and delete the unsatisfactory objects.
 */
cleanautosplit(kcont)
struct kcontrol *kcont;
{
	int i;
	colour(OVERLAY2);
	intens(0);
	for (i=0; i<kcont->SI->splitnum; i++) {
		picframe(kcont->SI->boundlist[i],kcont->f);
		freeobj(kcont->SI->objl[i]);
		freeobj(kcont->SI->boundlist[i]);
	}
}
