/*
 *      C H R O M L Y S E . C  --  Do chromosome analysis
 *
 *
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *
 *	Based on chromlyse.c
 *           Jim Piper, Denis Rutovitz
 *           MRC CAPCU Edinburgh.  Version of 14 Dec. 85
 *
 *
 *  Modifications
 *
 * 06 Feb 1992	jimp	i<=MAXSLAVE -> i<MAXSLAVE in chromlyseinit
 *     20 Jan 1992	Joanne		initclass() instead of doclassifier()
 *	7 Jan 1992	Joanne		Go via anamenu() to segmenu() so that 
 *					the count option is included.
 *					Added freeing of kcont->oth[].bound
 *	10 Jun 1991		ih			Null mol->plist after freeing ool->plist
 *  10 Jun 1991		ih/cas		Check that kol!=ool before freeing kol and NULL ool after this check
 *   5 Jun 1991		ih			Extra debug in cleanup+return(0) in kdetails
 *   9 Apr 1991		ih			Check alc->debug before using debug_channel
 *                              in kdetails!
 *  11 Jul 1990		ih			Clear digitised flag after dispgo for filed images
 *	19 Mar 1990		dcb			Default chrom length = LONG (long)
 *	12 Mar 1990		dcb			call lin_colour_lut after cell read in chromlyse
 *	01 Mar 1990		dcb			remove call to setcolour
 *	23 Oct 1989		dcb			Pass initslave the slave priority
 *	22 Sep 1989		dcb			chromlyse: main loop reads while(!finished)
 *	21 Aug 1989		CAS			setcolour for fluor
 *	12 Jul 1989		SEAJ		Removed fixclasses, called initclass instead.
 *	 1 Jun 1989		CAS			Def errno for v2.2
 *	12 May 1989		dcb/SEAJ	Change call to findentry
 *	10 May 1989		dcb			Use findentry to get VCV classifier.
 *								Drop through to segment
 *	 4 May 1989		dcb			In anamenu - fix paretheses in vhd ptr check,
 *								and	remove index from vhd
 *	12 Apr 1989		SEAJ		Added gotcell to call to kinteract.
 *	27 Feb 1989		SCG			bug in classifier assignment
 *  17 Feb 1989		SEAJ		Copied chromlyse to newchromlyse and rewrote.
 *	14 Feb 1989		CAS			file opening changed, messages changed etc
 *	01 Feb 1989		dcb			Comment text in monitor_on
 *	16 Jan 1989		dcb			static monitor_on to cope with logging on a
 *								per case basis, if alc->monitor == 1
 *	02 Dec 1988		dcb/GJP		chromlyseinit turns on debugging in slaves
 *								(if indicated)
 *	19 Aug 1988		SCG			Modifications for switchable chromosome classifiers
 * 	 8 Jul 1988		CAS/SEAJ	Check that mol plist is removed when there is
 *								an ool list != mol list
 *	 7 Jul 1988		CAS/SEAJ	Check that plist is removed in cleanup
 *	 6 Jul 1988		CAS			Tidy cmmd's
 *	23 May 1988		CAS			Change params to colour calls
 *	 2 Feb 1988		CAS			Pass phase to kinteract
 *	29 Jan 1988		CAS			Added configurable slave startup
 */
#include <signal.h>
#include <cmdsys.h>
#include <wstruct.h>
#include <multip.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <class.h>
#include <allcontr.h>
#include <colour.h>

extern int closecfile();
extern int countit();
extern int segmenu();
extern int SI_attach();

extern int errno;
extern struct allkontr *alc;	/* 'allcontrol' structure */
extern struct kcontrol *kcont;	/* karyotyping control structure */
extern FILE *debug_channel;

static maxsla;
static gotcell[3]; /* these should be in alc or kont ??? */

/* 
 *	C H R O M L Y S E  --  Main chromosome analysis procedure 
 *
 *	Phase indicates whether called as analyse or review.
 *		analyse - phase = 0
 *		review  - phase = 1
 *
 */
chromlyse(phase)
int		phase;
{
	int stat, finished = 0, met_displayed = 0, ktype = 0;
	int i;
	static int type=0;
	struct chromplist *plist, *makechromplist();

	if (alc->debug)
		fprintf (debug_channel,"In chromlyse:\n");
	chromlyseinit();
	SI_attach(kcont);
	if (initgotcell(phase) == ERROR)
		return(ERROR);
		
	initclass(kcont);		/* in case of resegment from karyotype
					 * review, the classifier needs to be 
					 * initialised
					 */
	if (phase == 0) {
		lin_colour_lut();	/* Linear LUT + colours */
		cameratype_lut();	/* Set lut type to be correct for this cell */
		dispall();
		met_displayed = 1;
		stat = anamenu();
	} else
		stat = KARYOTYPE;
	finished = 0;
	while (!finished) {
		switch(stat) {
			case SEGMENT :
				if (alc->debug)
					fprintf(debug_channel, "CHROMLYSE - SEGMENT\n");
				if (initgotcell(phase) == ERROR)
					return(ERROR);
				if (gotcell[1] > 0) {
					if (alc->debug)
						fprintf(debug_channel, "CHROMLYSE - call to initclass\n");
				}
				if (!met_displayed) {
					showcell();
					met_displayed = 1;
				}
				if (alc->debug)
					fprintf(debug_channel, "CHROMLYSE - call segmenu(gotcell)\n");
				stat = segmenu(gotcell);
				break;
			case KARYOTYPE :
				if (alc->debug)
					fprintf(debug_channel, "CHROMLYSE - KARYOTYPE\n");
				if (!ktype) {
					met_displayed = 0;
					if (phase == 0) {
						if (!segexitcheck())
							return(ERROR);
						kcont->mstatus |= SEGEND;
						/* wait for feature measurement to finish */
						if (alc->debug)
							fprintf(debug_channel, "CHROMLYSE - waiting for feature measurement - waitfeats(kcont)\n");
						printf( "WAITING FOR FEATURE MEASUREMENT TO FINISH\n");
						waitfeats(kcont);
						if (alc->debug) {
								fprintf(debug_channel,"CHROMLYSE - Segmentation finished\n");
							kdetails(kcont);
						}
						/* copy back objects processed on slave processors */
						if (gotcell[1] && gotcell[2]) {	/* been in karyotype before */
							for (i = 0; i < kcont->maxnumber; i++ ) {
								if (kcont->acl[i] & PROCESSED) {
									if (alc->debug)
										fprintf(debug_channel, "CHROMLYSE - kcopyback object %d\n", i);
									kcopyback(kcont, i, i+1);
								}
							}
						} else {
							if (alc->debug)
								fprintf(debug_channel, "CHROMLYSE - kcopyback ALL objects\n");
							kcopyback(kcont,0, kcont->maxnumber);
						}
						/* classify in this process */
						if (alc->debug) {
							fprintf(debug_channel,"CHROMLYSE - Starting Classification - class(kcont)\n");
							kdetails(kcont);
						}
						class(kcont);
						if (alc->debug)
							fprintf(debug_channel, "CHROMLYSE - returned from class\n");
						gotcell[2] = gotcell[1] = 1; /* we have got the processed and 
														the rotated objects */
					}
					if (initgotcell(phase) == ERROR) {
						return(ERROR);
					}
					upkary(phase);
				}
				if (alc->debug) {
					fprintf(debug_channel, "CHROMLYSE - call to kinteract(phase, gotcell)\n");
					fflush(debug_channel);
				}
				stat = kinteract(phase, gotcell);
				ktype = 0;
				if (alc->debug)
					fprintf(debug_channel, "CHROMLYSE - mark all objects PROCESSED\n");		
				for (i = 0; i < kcont->maxnumber; i++) 
					kcont->acl[i] &= ~PROCESSED;
				break;
			case MANKARY :
				if (alc->debug)
					fprintf(debug_channel, "\nCHROMLYSE - MANKARY\n");
				stat = mankary();
				break;
			case EXIT :
				if (alc->debug)
					fprintf(debug_channel, "\nCHROMLYSE - EXIT\n");
				stat = ktypeopts(phase);
				ktype = 1;
				break;
			case QUIT :
				if (alc->debug)
					fprintf(debug_channel, "\nCHROMLYSE - QUIT\n");
				finished = 1;
		}
	}	

	if (alc->debug) {
		fprintf(debug_channel,"Before kdetails\n");
		fflush(debug_channel);
		kdetails(kcont);
		fprintf(debug_channel,"Before cleanup\n");
		fflush(debug_channel);
	}
	cleanup(kcont);	
	if (alc->debug)
		kdetails(kcont);
}
/*
 * I N I T G O T C E L L -- Read file if necessary and initialise gotcell
 *
 */
initgotcell(phase)
int phase;
{
	if(alc->debug) {
		kdetails(kcont);
		fflush(debug_channel);
	}
	if (!gotcell[phase]) {
		if(alc->debug) {
			fprintf(debug_channel,"Not gotcell phase %d\n",phase);
			fflush(debug_channel);
		}
		if (!chekcell(phase)) {			/* get cell, slide, id. etc. */
			cleanup(kcont);
			return(ERROR);
		}
		if (!readcfile(phase)) {		/* read the file */
			closecfile();
			cleanup(kcont);
			return(ERROR);
		}
		closecfile();
		if (phase == 0)
			monitor_on("| Analyse\n");	/* CSYS logging on if specified */
		else
			monitor_on("| Karyotype\n");/* CSYS logging on if specified */
		if (alc->debug)
			kdetails(kcont);
		gotcell[phase] = 1;
		setgotcell(gotcell);		/* YUK - export gotcell data */
		if (alc->debug) {
			fprintf(debug_channel,"Alloc_check, after read file\n");
			Alloc_check();
		}
	}
	if(alc->debug) {
		fprintf(debug_channel,"Initgotcell - returning 1\n");
		fflush(debug_channel);
	}
	return(1);
}

/*
 * A N A M E N U -- Analyse option menu.
 *
 */
anamenu()
{
	int tis, stn;
	int	Quit_cmd,Seg_cmd, Tissue_cmd;

	findentry(alc->rankcf, VCV, &alc->typeids);
	if (alc->debug) {
		fprintf(debug_channel,"Alloc_check, after classifier init\n");
		Alloc_check();
	}
	downmenu();
	newmenu("Analyse options.");
	cmmd("count;%g%+i", countit, &gotcell[0]);mouse(LEFTBT);
	cmmd("segmentation");mouse(MIDBT); Seg_cmd = mmcmd;
	if (alc->devel) {
		cmmd("tissues/stain options"); Tissue_cmd = mmcmd;
	}
	cmmd("quit");mouse(RIGHTBT); Quit_cmd = mmcmd;
	while (1) {
		thismu();
		if(nncmd == Seg_cmd) {
			upmenu();
			return(SEGMENT);
			break;
		} else if (nncmd == Quit_cmd) {
			return(QUIT);
			break;
		} else if (nncmd == Tissue_cmd) {
			downmenu();
			tisstmu(&alc->typeids);
			upmenu();
			/*
			 *	initialise chromosome classifier
			 */
			fprintf(output, "Redefining classifier, tissue type %d, stain %d\n",
				alc->typeids.ktistype, alc->typeids.kfstype);
			findentry(alc->rankcf, VCV, &alc->typeids);
		}
	}
	upmenu();
}


/*
 *	C H R O M L Y S E I N I T  --  Initialise slaves, gotcell values and
 *								   graphics. Turn on debug if indicated
 *
 */
chromlyseinit()
{
	int		i, slave;

	if (alc->debug) {
		fprintf (debug_channel,"In chromlyseinit\n");
		fflush(debug_channel);
	}
/*
 *	Gotcell needs revamping.  Its used in here and kinteract to indicate
 *	which forms are in memory (raw cell, karyotype etc).  It should not
 *	be cleared here - but since I don't know where to move it to I have 
 *  added a cleanup to ensure that its ok to forget about the cells.
 *										CAS  30 Sep 1986
 */
	cleanup(kcont);

/*
 * Initialise gotcell to zero. 
 */
	for (i=0;i<sizeof gotcell;i++) {
		gotcell[i]=0;
	}
			
	for (slave=0, i=0; i<MAXSLAVE ; i++) {
		if (alc->slaveconf[i] & SL_USE) {
			if (alc->debug)
				printf("Chromlyse - initslave kcont 0x%x slave %d proc %d prior %d\n",
						kcont, slave, i, alc->sprior);
			initslave(kcont,slave,i, alc->sprior);
			if (alc->debug >= 3)
				commdslave(kcont,slave,SLAVE_DEBUG);
			slave++;
		}
	}
/* 
 *	Initialise display. 
 */
	dispgo();
	/* tell hardcopy routines etc what kind of framestore image we have */
	alc->digitised=0;
}

/*
 *	C L E A N U P  --  clean the given kcontrol structure of all hanging
 *						woolz objects by freeing memory and clearing
 *						out the pointers
 *
 */
cleanup(kcont)
struct kcontrol *kcont;
{
	int		i, slave, object;

	for (slave=0; slave<alc->nslaves; slave++)
		killslave(kcont,slave,SIGKILL);
	if (alc->debug) {
		fprintf(debug_channel,"Cleanup - done slaves\n");
		fflush(debug_channel);
	}
 	for (object=0; object<kcont->maxnumber; object++) {
		if (alc->debug) {
			fprintf(debug_channel,"Cleanup - object %d: ",object);
 			space_details(kcont->ool[object]);
 		}
 /*
  *	Free ool object if different from mol object.  ool == mol in ktype rev
  *	after resegmentation....
  */
 		if (kcont->ool && kcont->ool[object]) {
 			if (!kcont->mol || (kcont->mol[object] != kcont->ool[object])) {
 				if (alc->debug)
 					fprintf(debug_channel,"Freeing ool object\n");
	 			freeobj(kcont->ool[object]);
	 			if (kcont->mol && kcont->mol[object] && (kcont->mol[object]->plist != kcont->ool[object]->plist)) {
	 				if (alc->debug)
 						fprintf(debug_channel,"Freeing mol plist\n");
	 				Free(kcont->mol[object]->plist);
	 				kcont->mol[object]->plist = NULL;
	 			}
	 		} else {
 				if (alc->debug)
 					fprintf(debug_channel,"Freeing ool plist\n");
	 			Free(kcont->ool[object]->plist);
	 			kcont->mol[object]->plist=NULL;
	 		}
 		} else {
 			if (kcont->mol && kcont->mol[object]) {
 				if (alc->debug)
 					fprintf(debug_channel,"No ool - freeing mol plist\n");
 				Free(kcont->mol[object]->plist);
 				kcont->mol[object]->plist=NULL;
 			}
 		}
		/*N.B. bols are not objects but ipoints (4 shorts) --ih*/
 		space_details(kcont->bol[object]);
 		if (kcont->bol && kcont->bol[object]) {
			freeobj(kcont->bol[object]);
			kcont->bol[object] = NULL;
		}
		if (kcont->kol && kcont->kol[object]) {
			if((kcont->kol[object]!=kcont->mol[object])&&(kcont->kol[object]!=kcont->ool[object])) {
 				kcont->kol[object]->plist = NULL;
 				freeobj(kcont->kol[object]);
 			}
 			kcont->kol[object] = NULL;
 		}
 		if (kcont->oth && kcont->oth[object].bound) {
 			freeobj(kcont->oth[object].bound);
			kcont->ool[object] = NULL;
		}
	}
	if (alc->debug) {
		fprintf(debug_channel,"Cleanup - done ool's and bol's\n");
		fflush(debug_channel);
	}
 	kdeletemol(kcont);
 	cleankcont(kcont);		/* clean out pointers  */
	subinitkc(kcont);		/* and comand registers  */
}

/*
 *	S U B I N I T K C  --  Reset slave control part of kcont structure
 *
 */
subinitkc(kc)
struct kcontrol *kc ;	/* karyotyping control structure */
{
	int slave;
	lock7(&(kc->lock));
	kc->maxnumber = 0;
	kc->mstatus = 0;
	for (slave=0; slave<=NSLAVES; slave++) {
		kc->command[slave] = 0;
		kc->status[slave] = 0;
		kc->processor[slave] = 0;
	}
	unlock7(&(kc->lock));
}

/*
 *	C H E K C E L L
 *
 */
chekcell(phase)
int		phase;
{
	int		stat;

	while(1) {
		switch (phase) {
			case 0:
				stat = getid(CELLFILE,READ);
				break;
			case 1:
				stat = getid(KARYFILE,READ);
				break;
			default:
				return(0);
		}
		if (stat == 0)
			return(0);
		else {
			if (alc->debug)
				fprintf(debug_channel,"Chekcell, picname %s/%s\n",alc->dir,alc->picname);
			if (opencfile() == -1) {
				fprintf(stderr, "Please try again\n");
				sleep(2);
			} else 
				return(1);
		}
	}
}


/*
 *	N E W C E L L  --
 *
 */
newcell() {
		SMALL *activelist;
		int i;
		
		activelist = kcont->acl;
		for (i=0; i<MAXOBJS; i++) activelist[i] = 0;
}

/*
 *	S P A C E _ D E T A I L S
 *
 */
space_details(obj)
struct object *obj;
{
	if (alc->debug >1) {
		if (obj) {
			switch (obj->type) {
			case 1:
				fprintf(debug_channel, "OBJ=%x I=%x IFS=%x V=%x VFS=%x P=%x\n",
					obj,obj->idom,(obj->idom)?obj->idom->freeptr:0,
					obj->vdom,(obj->vdom)?obj->vdom->freeptr:0,obj->plist);
				break;
			case 40:
				fprintf(debug_channel, "POINT=%x\n", obj);
				break;
			default:
				fprintf(debug_channel,"Invalid object \n");
			}
		} else {
			fprintf(debug_channel,"NULL object\n");
		}
	}
}

/*
 *		K D E T A I L S
 *
 */
kdetails(kcont)
struct kcontrol *kcont;
{			
	int object;
	if (alc->debug) {
		fprintf(debug_channel,"K C O N T at %x",kcont);
		fprintf(debug_channel," seat %d term %s mouse %s\n",
				kcont->seat,kcont->terminal,kcont->mouse);
	}
	if(!kcont->maxnumber) {
		if (alc->debug)	fprintf(debug_channel,"No objects\n");
		return(0);
	}
 	for (object=0; object<kcont->maxnumber; object++) {
		if (alc->debug) fprintf(debug_channel,"Object %d: ",object);
 		if (kcont->ool[object])
 			space_details(kcont->ool[object]);
 		else if (alc->debug)
			fprintf(debug_channel,"\n");
 	}
}

/*
 *	M O N I T O R _ O N
 *	Turn on cmdsys logging to file name "caseid.log", and delete log file for
 *	previous case. Files are created in the users home directory. If the file
 *	exists, logging is turned off.
 *	Logging is only enabled on a per case basis if alc->monitor == 1
 *	This function is included here as a static because at the moment it is
 *	only called from within this module.
 *	comment_text is written to the file
 */
monitor_on(comment_text)
char *comment_text;
{
	char buf[60];
	if (alc->monitor == 1) {
		if ((alc->lastname->case) &&
			(strcmp(alc->lastname->case, "UNSET"))) { /* Last name not UNSET */
			CSYS_log(NULL);							  /* Turn off logging */
			sprintf(buf, "/dd/usr/%s/%s$%s$%dL",
				alc->user, alc->lastname->case, alc->lastname->slide,
				alc->lastname->cellno);
			if (unlink(buf) != 0) {					  /* Delete last log file */
				fprintf(stderr, "Cannot delete log file %s. (Error %d)\n",
					buf, errno);
			}
			if (alc->debug)
				fprintf(debug_channel, "CSYS logging off, %s deleted\n", buf);
		} /* end if UNSET */
		sprintf(buf, "/dd/usr/%s/%s$%s$%dL", alc->user, alc->nameids.case,
			alc->nameids.slide,	alc->nameids.cellno);
		CSYS_log(buf);
		CSYS_text(comment_text);
		_strass(alc->lastname, &(alc->nameids), sizeof(struct cyto_file));
		if (alc->debug)
			fprintf(debug_channel, "CSYS logging on to %s\n", buf);
	}
} /* end monitor_on */
