/*
 *      D I S P F O R M . C  --  Display format/layout stuff
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *  Date:    17th May 1988
 *
 *  Modifications
 *
 *   6 Feb 1992		Joanne		Display NEF coordinates (patnam) in idbox
 *   3 Feb 1992		Joanne		Changed position of idbox and classbox,
 *					removed setscreenscale (no scaling !!)
 *  20 Jan 1992		Joanne		Read data for karyotype layout from 
 *					file rather than from data structure
 *   8 May 1989		SEAJ		Added call to convscrformat to setscreenformat.
 *  24 Apr 1989		SEAJ		Added sethcpyformat to read hardcopy structures.
 *  11 Apr 1989		SEAJ		Added height field to structure defines.
 *	19 Sep 1988		CAS			Date field big enough, init class bopx
 *	23 Aug 1988		CAS			Mor flex display stuff
 *	19 Aug 1988		CAS			Get display position stuff off disk
 */
#include <stdio.h>
#include <allcontr.h>
#include <chromanal.h>
#include <dispform.h>
#include <ddgs.h>

extern struct screenform *convscrformat();
extern struct allkontr *alc;


/*
 * The bottom row is a 24-position stack, filling from the left.
 * The first 12 locations are reasonably well spaced, but then things
 * get a bit tighter.
 * We need a stack occupancy and a column-origin table.
 */
int defstackcol[NSTACKPOS+2] = {-1,
	 250, 650, 1050,1450,1850,2250,
	2650,3050,3450,3850	
};  /* rest filled in later */
/*
 *	Default ID Box text
 */
struct idtext defidtext_english[] = {
		{ 1, 16,   98, 510, 40, 'C','a','s','e',' ' },
		{ 1,  4, 1018, 510, 40, 'S','l','i','d','e',' ' },
		{ 1,  0, 1458, 510, 40, 'C','e','l','l',' ' },
		{ 1, 17,   98,  70, 40, 'O','p',' ','i','d',' ' },
		{ 1, 17, 1138,  70, 40, 'D','a','t','e',' ' },
		{ 0 },	/* Supervisor acceptance */
		{ 0 },	/* Supervisor date */
		{ 1, 10,   98, 210, 40 },	/* NEF coordinates */
		{ 1, 39,   98, 310, 40, 'C','o','m','m','e','n','t','s',' ' },
		{ 1, 39,   98, 210, 40, ' ',' ',' ',' ',' ',' ',' ',' ',' ' },
		{ 1, 36,   98, 410, 40, 'K','a','r','y','o','t','y','p','e',' ' },
		{ 1, 33,   98, 610, 40, 'L','a','b','o','r','a','t','o','r','y',' ','n','a','m','e',' ' }
	};
struct idtext defidtext_german[] = {
		{ 1, 16,   98, 510, 40, 'F','a','l','l',' ' },
		{ 1,  4, 1018, 510, 40, 'S','l','i','d','e',' ' },
		{ 1,  0, 1458, 510, 40, 'C','e','l','l',' ' },
		{ 1, 17,   98,  70, 40, 'O','p',' ','i','d',' ' },
		{ 1, 17, 1138,  70, 40, 'D','a','t','u','m',' ' },
		{ 0 },	/* Supervisor acceptance */
		{ 0 },	/* Supervisor date */
		{ 1, 10,   98, 210, 40 },	/* NEF coordinates */
		{ 1, 40,  534, 310, 40, '\0' },		/* Comments 1 */
		{ 1, 40,  534, 210, 40, '\0' },		/* Comments 2 */
		{ 1, 37,   98, 410, 40, 'K','a','r','y','o','t','y','p',' ' },
		{ 1, 30,   98, 610, 40, 'G','e','n','e','t','i','s','c','h','e','s',' ','L','a','b','o','r','.',' ' }
	};

/*
 *	S E T S C R E E N F O R M A T  --  Setup screen format structure
 *										from filename suplied
 */
setscreenformat()
{
	int		fd;
	
	alc->scrf = (struct screenform *) Calloc(1,sizeof(struct screenform));
	alc->scrf->stacklin = 200;			/* default */
	alc->scrf->idbox.position.x = 3800;
	alc->scrf->idbox.position.y = 16;
	alc->scrf->idbox.width = 2068;		/* Id box params */
	alc->scrf->idbox.height = 860;
/*
	strcpy(alc->scrf->idbox.banner.text,"W.R.G.L.");
	strcpy(alc->scrf->idbox.banner.text,"MRC  HGU");
*/
	strcpy(alc->scrf->idbox.banner.text,"N.R.P.B.");
	alc->scrf->idbox.banner.x = 710;
	alc->scrf->idbox.banner.y = 710;
	alc->scrf->idbox.banner.size = 80;
	alc->scrf->chromclass.x = 5420;
	alc->scrf->chromclass.y = 120;
	switch (alc->language) {
		case ENGLISH:
			_strass(&alc->scrf->idbox.case,defidtext_english,12 * sizeof(struct idtext));
			break;
		case GERMAN:
			_strass(&alc->scrf->idbox.case,defidtext_german,12 * sizeof(struct idtext));
			break;
	}

	karyinitpos(); 
	_strass(alc->scrf->stackcol,defstackcol, (NSTACKPOS + 2) * sizeof(int));
}



/*
 *	S E T H C P Y F O R M A T  --  Setup hardcopy display format
 *
 */
sethcpyformat(file)
char	*file;
{
	int fd;
	
	alc->hcpf = NULL;
	if ((fd = open(file, 1)) == -1) {
		fprintf(stderr,"Can't open hardcopy format file %s\n", file);
		return(0);
	}
	alc->hcpf = (struct hcopyform *) Calloc(sizeof(struct hcopyform), 1);
	alc->hcpf->size = sizeof(struct hcopyform);
	alc->hcpf->version = VERSION;
	alc->hcpf->hco = NULL;
	alc->hcpf->chromclasses = 0;
	alc->hcpf->viewed_stack = 0;
	
	readbinfile(fd);
	close(fd);
	return(1);
}
/*
 * R E A D B I N F I L E
 *
 */
readbinfile(fd)
int fd;
{
	read(fd, alc->hcpf, sizeof(struct hcopyform));
	if (alc->hcpf->hco != NULL)
		readall(fd, &(alc->hcpf->hco));
}
/*
 * R E A D A L L
 *
 */
readall(fd, ptr)
int fd;
struct hcobject **ptr;
{
	while (*ptr != NULL) {
		*ptr = (struct hcobject *) Calloc (sizeof(struct hcobject), 1);
		read(fd, *ptr, sizeof(struct hcobject));
		if ((*ptr)->ass != NULL)
			readall(fd, &((*ptr)->ass));
		ptr = &((*ptr)->next);
	}
}

