/*
 *      H A R D C O P Y . C  --  Hardcopy utilities
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    11th Sept 1987
 *
 *  Modifications
 *
 *  11 Jul 1990     ih removed shit from text colour logic-just rely on alc->fluor and new flag alc->digitised to decide if background dark or light
 *  10 Jul 1990     ih/dcb change text colour to overlay0 | fc for fluor==ddgs->inv
 *                  and ~fc for flour & !ddgs->inv (digitised fluor image)
 *	29 Nov 1989		dcb/GJP	do_mets: Fix aspect ratio adjustment calculation
 *	14 Sep 1989		dcb		dispname: remove oldmode
 *	22 Aug 1989		CAS		Mod to dispname colour for fluor
 *	14 Jul 1989		SEAJ	Sorted out aspect ratio in do_mets().
 *	 4 Jul 1989		SEAJ	Check Cpgroup in do_kars()  and setupframe().
 *  14 Jun 1989		SEAJ	Made object count be set to 0 each time spoolobjs 
 *							are grabbed.
 *	 8 Jun 1989		dcb		Handle quit case from laser3, make lkopts static
 *  30 May 1989		CAS/SEAJ	Fixed interpolate.
 *  25 May 1989		SEAJ	Fixed timing bug in wspoolobj and added checks for 
 *							numbers of spooled objects.
 *  22 May 1989		SEAJ	Reset timing arrays in do_timing.
 *  17 May 1989		SEAJ  	Added in timings.
 *  24 Apr 1989		SEAJ	Added laser 3 hard copy stuff.
 *	22 Feb 1989		CAS		Use status from mputc in spool
 *	17 Jan 1989		CAS		Added basic text gen, for testing
 *  11 Jan 1989		SEAJ	% added to disc full message.
 *	 5 Jan 1989		CAS		Minor change to position of met in laser ktype
 *	 7 Dec 1988		CAS		Make display name work with 8 bit data
 *  29 Nov 1988		SEAJ	Added disc allocation test to spoolscreen.
 *	19 Oct 1988		CAS		Further refinement of obj spooling
 *	 3 Oct 1988		CAS		Support object based spooling
 *	19 Sep 1988		CAS		Quit used to not undisp casename
 *	11 Aug 1988		CAS		Check for spool failures (ie disc full etc)
 *	12 May 1988		CAS		the 's' was lost in copies option
 *	27 Apr 1988		CAS		Get Case name pos calc right
 *  22 Apr 1988		SEAJ	Added in spaces to hc menu opts 1 & 2 to stop line deletion.
 *	18 Apr 1988		CAS		Move Case label to top right
 *	24 Mar 1988		CAS		Allow removal of case id from screen, add mouse options
 *	 9 Mar 1988		CAS		Option to write case id on screen.
 *	 2 Feb 1988		CAS		Down/up menu needed
 *	 1 Feb 1988		CAS		Press return to print message
 */
#include <cmdsys.h>
#include <allcontr.h>
#include <spool.h>
#include <dir300.h>
#include <hardcopy.h>
#include <display.h>
#include <wstruct.h>
#include <chromanal.h>
#include <dispform.h>
#include <kcontrol.h>
#include <fbio.h>

#define MAXSPOOLOBJS 300
#define TXTSIZE 10
#define MAXTIMS 300
#define TXTCOLOUR 0xFC	/* colour of text to contrast with brightfield backgnd */
#define BRIGHT_TEXT_X 64 /*text box sizes for bright field images */
#define BRIGHT_TEXT_Y 64
#define FLUOR_TEXT_X 90  /* and fluorescent ones */
#define FLUOR_TEXT_Y 90

extern struct allkontr *alc;
extern struct kcontrol *kcont;
extern DDGS *ddgsopen();
extern FILE *debug_channel;

static int timings[MAXTIMS], curr_time = -1;
static char* strings[MAXTIMS];

struct spoolobj {
			struct object *object;
			struct pframe *frame;
		};
/*
 *	Local (static) functions
 */
static int sizefunc();
static int repeatfunc();
static int dispname();
/*
 *	Local storage
 */
char cellid[5];					/*  space for character cellno */
static int onoff = 0, objcount;
static struct spoolobj *sptr;
struct  hcobject *classptr[MAXCLASS];
struct 	hcobject *stackptr[NSTACKPOS];

/*
 *	D I S P _ M E S S G  --  Display message telling user to press return
 *							 once case setup from getid
 */
static disp_messg()
{
	screengo(OUTLINE - 1,0);
	printf("Press <Return> to start print\n");
}

/*
 *	H C O P T S  --	Hard copy options
 *
 */
static int hcopts()
{
	int		repeat,size,ret = 1;
	int		opt, Quit_cmd;

	newmenu("Hard copy options");
	cmmd("number of copies;:k30%+d%-d%f",&repeat,&alc->hcpy->repeat,repeatfunc);
	cmmd("picture size;:k30%+d%-d%f%-f",&size,&alc->hcpy->picsize,sizefunc,disp_messg);
	cmmd("display case name;:k30%t%f",&onoff,dispname); mouse(MIDBT);
	if (alc->devel)
		cmmd("interpolate;:k30<1/3>%d",&alc->hcpy->interpolate);
	Quit_cmd = cmmd("quit (don't print)"); mouse(LEFTBT);
	while(opt = thismu()) {
		if (opt == Quit_cmd) {
			ret = 0;
			break;
		}
	}
	return(ret);
}

/*
 *	R E P E A T F U N C  --  Function to validate and set repeat count
 *
 */
static repeatfunc(newcount,count)
int		*newcount, *count;
{
	if (*newcount > 0) {
		*count = *newcount;
	} else
		printf("repeat count must be greater than zero\n");
}

/*
 *	S I Z E F U N C  -- Function to validate and set new image size
 *
 */
static sizefunc(newsize,picsize)
int		*newsize, *picsize;
{
	if ((*newsize > 0) && (*newsize < 4)) {
		*picsize = *newsize;
	} else
		printf("picture size should be between 1 and 3\n");
}

/*
 *	D I S P N A M E  --
 *
 */
dispname(onoff)
int		*onoff;
{
	char	buffer[100], *wsave();
	static char *savebuf = NULL;
	int		xp,yp;
	DDGS	*dgp;
	int textx,texty;	/* for holding text box dimensions */

	if (*onoff) {
		if (savebuf)				/* Left over buffer ? */
			Free(savebuf);
		dgp = alc->ddgschannel;
		yp = (dgp->maxy<<dgp->yshift) - 200;
		xp = (dgp->maxx<<dgp->xshift) - 2150;
		sprintf(buffer,"Case: %s %s %d",alc->nameids.case,alc->nameids.slide,alc->nameids.cellno);
		moveto(xp,yp);
		if (alc->fluor) {
			textx=FLUOR_TEXT_X;
			texty=FLUOR_TEXT_Y;
		}
		else {
			textx=BRIGHT_TEXT_X;
			texty=BRIGHT_TEXT_Y;
		}
		tsize(textx,texty);
		savebuf = wsave(textx*strlen(buffer),texty);
		moveto(xp,yp);
		intens(1);
		if (alc->fluor && alc->digitised) {
			if (alc->debug) {
				fprintf(debug_channel,"hardcopy:fluorescent digitised image--black background\n");
			}
			/*digitised fluorescence image */
			colour(OVERLAY0 | ((~TXTCOLOUR)&0xFC));
		} else {
		 	/*all other cases*/
		 	if (alc->debug) 
		 		fprintf(debug_channel,"hardcopy:bright background\n");
			colour(OVERLAY0 | TXTCOLOUR);
		}
		if (alc->fluor) {
			colourmask(0);		/* what the hell does this do -- ih*/
		}
		text(buffer);
		if (alc->fluor)
			intens(1);			/* and this ????? */
	} else {
		wrestore(savebuf,0);
		savebuf = NULL;
	}
}

/*
 *	S C R D U M P  --
 *
 */
screendump()
{
	int		stat, local, start_time, first_time, first;

	if (alc->timing) {
		setuptim(&first, &first_time, "scrdump--total time");
		setuptim(&local, &start_time, "scrdump--hcopts");
	}
	downmenu();
	if (onoff)
		dispname(&onoff);
	if (stat = hcopts()) {
		if (alc->timing) {
			timings[local] = tick() - start_time;
			setuptim(&local, &start_time, "scrdump--spoolscreen");
		}
		switch (alc->hcpy->type) {
			case HC_VGR:
			case HC_DIR300:
				break;
			case HC_DIR300_QUE:
			case HC_DIR300_OBJ:
				windmill("Copying Screen - Please wait.... ");
				if (spoolscreen(alc->ddgschannel,alc->hcpy->picsize,alc->hcpy->picsize))
					windmillend("done\n");
				else
					windmillend("Failed to write spool file\n");
				break;
		}
		if (alc->timing) {
			timings[local] = tick() - start_time;
			setuptim(&local, &start_time, "scrdump--end");
		}
	}
	upmenu();
	if (onoff) {
		onoff = 0;
		dispname(&onoff);
		onoff = 1;
	}
	if (alc->timing) {
		timings[local] = tick() - start_time;
		timings[first] = tick() - first_time;
		do_timings();
	}
	return(stat);
}

/*
 *	L A S E R K T Y P E  -- Print karyotype on laser printer.
 *
 */
laserktype()
{
	struct spoolobj *sol;
	int		stat, i, local, start_time, first, first_time;

	if (alc->timing) {
		setuptim(&first, &first_time,  "laserktype--total time");
		setuptim(&local, &start_time, "laserktype--setup spool objs");
	}
	switch (alc->hcpy->type) {
		case HC_DIR300_OBJ:
			if (stat = lkopts()) {
				objcount = 0;
				sol = (struct spoolobj *) Calloc(sizeof(struct spoolobj),MAXSPOOLOBJS);
				sptr = sol;
				if (alc->hcpf == NULL)
					use_scrf_hcpy();
				else
					setuphcpy();
				if (alc->timing) {
					timings[local] = tick() - start_time;
					setuptim(&local, &start_time, "laserktype--spoolobj");
				}
				windmill("Spooling - Please wait.... ");
				if (spoolobject(alc->ddgschannel,sol,1,1))
					windmillend("done\n");
				else
					windmillend("Failed to write spool file\n");
			}
			break;
		default:
			if (alc->timing) {
				timings[local] = tick() - start_time;
				setuptim(&local, &start_time, "laserktype--screendump");
			}
			screendump();
			if (alc->timing) {
				timings[local] = tick() - start_time;
				timings[first] = tick() - first_time;
				do_timings();
			}
			return(0);
			break;
	}
	if (alc->timing) {
		timings[local] = tick() - start_time;
		setuptim(&local, &start_time, "laserktype--free objects");
	}
	if (stat) {	/* dcb */
		sptr = sol;
		for (i=0; i<MAXSPOOLOBJS; i++) {
			if (sptr->object && (sptr->object->type == 70)) {
				sptr->object->vdom = NULL;			/* Text not malloc'd */
				freeobj(sptr->object);
			}
			if (sptr->frame)
				freeobj(sptr->frame);
 				sptr++;
		}
		Free(sol);
	}
	if (alc->timing) {
		timings[local] = tick() - start_time;
		timings[first] = tick() - first_time;
		do_timings();
	}
	return(stat);
}

/*
 *	L K O P T S  --	Laser karyotype print options
 *
 */
static
lkopts()
{
	int		repeat,size,ret = 1;
	int		opt, Quit_cmd;
	newmenu("Karyotype print options");
	cmmd("number of copies;:k30%+d%-d%f",&repeat,&alc->hcpy->repeat,repeatfunc);
	Quit_cmd = cmmd("quit (don't print)"); mouse(LEFTBT);
	while(opt = thismu()) {
		if (opt == Quit_cmd) {
			ret = 0;
			break;
		}
	}
	return(ret);
}
/*
 * U S E _ S C R F _ H C P Y
 *
 */
use_scrf_hcpy()
{
	struct kdisppos *dp;
	int i, xmin, xmax, ymin, ymax,x, y, kminx, kminy;
	int local, start_time, first, first_time;
	struct pframe *fr, *makepframe();
/*
 *	Put karyotype chroms into list
 */
 	if (alc->timing) {
 		setuptim(&first, &first_time,  "use_scrf_hcpy--total time");
 		setuptim(&local, &start_time, "use_scrf_hcpy--karyotype");
 	}
	kminx = 9999999; kminy = 9999999;
	for (i=0; i<kcont->maxnumber; i++) {
		if (objcount >= MAXSPOOLOBJS)
			break;
		if (kcont->acl[i] & ACTIVE) {
			fr = kcont->ivf+i;
			sptr->frame = makepframe(fr->scale*2,fr->dx*2,fr->dy*2,fr->ox,fr->oy);
			sptr->frame->ix = fr->ix;
			sptr->frame->iy = fr->iy;
			sptr->object = (struct object *) kcont->mol[i];
/*
 *	Calculate bottom left corner position
 */
			y = frdigsypos(kcont->mol[i]->idom->line1,fr);
			x = frdigsxpos(kcont->mol[i]->idom->kol1,fr);
			if (x < kminx)
				kminx = x;
			if (y < kminy)
				kminy = y;
			objcount++;
			sptr++;
		}
	}
	if (alc->timing) {
		timings[local] = tick() - start_time;
		setuptim(&local, &start_time, "use_scrf_hcpy--class numbers");
	}
/*
 * Put class numbers into list
 */
	fr = makepframe(8,0,0,0,0);
	fr->ix = 1;
	fr->iy = 2;
	sptr->frame = fr;
	dp = alc->scrf->disppos;
	for (i=0; i<alc->scrf->undefclass; i++) {
		if (objcount >= MAXSPOOLOBJS)
			break;
		if (*dp->tag) {
			x = (dp->col-(strlen(dp->tag) * 40 / 2))>>alc->ddgschannel->xshift;
			y = (dp->lin-96) >> alc->ddgschannel->yshift;
			sptr->object = (struct object *) maketext(dp->tag,y,x*2,strlen(dp->tag)+1);
			objcount++;
			sptr++;
		}
		dp++;
	}
	if (alc->timing) {
		timings[local] = tick() - start_time;
		setuptim(&local, &start_time, "use_scrf_hcpy--metaphase");
	}
/*
 *	Stick metaphase in bottom left corner
 * set frame such that top left point at kminx/ddgs, (kminy/ddgs - gap) and scale 2
 */
	metsize(&xmin,&xmax,&ymin,&ymax);
	fr = makepframe(2,0,0,xmin-(kminx>>alc->ddgschannel->xshift),ymax-(kminy>>alc->ddgschannel->yshift) + 50);
	fr->ix = 4;
	if (kcont->FIP == DIGFIP)
		fr->iy = 3;
	else
		fr->iy = 4;
	sptr->frame = fr;
	for (i=0; i<kcont->maxnumber; i++) {
		if (objcount >= MAXSPOOLOBJS)
			break;
		if (kcont->ool[i] != NULL && !kcont->ool[i]->plist->history[1]) {
			sptr->object = (struct object *) kcont->ool[i];
			objcount++;
			sptr++;
		}
	}
	if (alc->timing) {
		timings[local] = tick() - start_time;
		timings[first] = tick() - first_time;
	}
}
/*
 *	S P O O L S C R E E N  --
 *
 */
spoolscreen(dg,pixwid,linrep)
DDGS	*dg;
int		pixwid,linrep;
{
	SFILE	*sfp, *lspoolopen();
  	int		margin,rotate = 0, freeblks, totblks, percent;
  	int 	local, start_time, first, first_time;
  	char 	tmpbuf[34];

	if (alc->timing) {
		setuptim(&first, &first_time,  "spoolscreen--total time");
		setuptim(&local, &start_time, "spoolscreen--open & setup");
	}
	switch (alc->ddgschannel->type) {
		case FS_VME512:
			sfp = lspoolopen(dg->lineno,dg->lineln);
			break;
		case FS_VIRTUOSO:
		case FS_VIRTUOSO_INVERT:
			if (pixwid > 2) {							/* Print rotated */
				sfp = lspoolopen(dg->lineln,dg->lineno);
				rotate = 1;
			} else
				sfp = lspoolopen(dg->lineno,dg->lineln);
			break;
	}
	if (sfp == NULL) {
		printf("Can't open screenprint file - screenprint abandoned\n");
		if (alc->timing) {
			timings[local] = tick() - start_time;
			timings[first] = tick() - first_time;
		}
		return;
	}
	sfp->repcount = (alc->hcpy->repeat > 0) ? alc->hcpy->repeat : 1;
	sfp->ps.laser.enhance = alc->hcpy->interpolate;
	if (alc->hcpy->interpolate <= 1) {
		sfp->ps.laser.pixwidth = pixwid;
		sfp->ps.laser.linerep = linrep;
	} else {
		sfp->ps.laser.pixwidth = 1;
		sfp->ps.laser.linerep = 1;
	}
	margin = (IWIDTH - (sfp->ps.laser.ncols) * sfp->ps.laser.pixwidth) / (2 * sfp->ps.laser.pixwidth);
	if (margin <0 ) margin = 0;
	sfp->ps.laser.margin = margin;
	sfp->type = SQULUT;
	if (alc->timing) {
		timings[local] = tick() - start_time;
		setuptim(&local, &start_time, "spoolscreen--disc check");
	}
	*tmpbuf = '/';
	_gs_devn(sfp->fp->_fd, &tmpbuf[1]);
	dfree(tmpbuf, &freeblks, &totblks);
	percent = (freeblks * 100) / totblks;
	if (alc->debug)
		fprintf(debug_channel, "spoolscreen:dev %s free %d tot %d percent %d%%\n", 
			tmpbuf, freeblks, totblks, percent);
	if (percent < 10) {
		fprintf(stdout, "Warning: Disc is %d%% full \007\n", (100 - percent));
		fprintf(stdout, "Please backup disc as soon as possible.\n");
	}
	if (alc->timing) {
		timings[local] = tick() - start_time;
		setuptim(&local, &start_time, "spoolscreen--dump & close");
	}
	if (ddgslutdump(dg,sfp->fp->_fd)) {
		if (ddgssdump(dg,sfp->fp->_fd,rotate)) {
			spoolclose(sfp);
			if (alc->timing) {
				timings[local] = tick() - start_time;
				timings[first] = tick() - first_time;
			}
			return(1);
		}
	}
	spoolquit(sfp);
	if (alc->timing) {
		timings[local] = tick() - start_time;
		timings[first] = tick() - first_time;
	}
	return(0);
}

/*
 *	S P O O L O B J E C T  --
 *
 */
spoolobject(dg,sol,pixwid,linrep)
DDGS	*dg;
struct spoolobj *sol;
int		pixwid,linrep;
{
	SFILE	*sfp, *lspoolopen();
	int		i, local, start_time;

	if (alc->timing)
		setuptim(&local, &start_time, "spoolobject");
	sfp = lspoolopen(dg->lineno,dg->lineln);
	if (sfp == NULL) {
		return(0);
	}
	sfp->repcount = (alc->hcpy->repeat > 0) ? alc->hcpy->repeat : 1;
	sfp->ps.laser.enhance = alc->hcpy->interpolate;
	if (alc->hcpy->interpolate <= 1) {
		sfp->ps.laser.pixwidth = pixwid;
		sfp->ps.laser.linerep = linrep;
	} else {
		sfp->ps.laser.pixwidth = 1;
		sfp->ps.laser.linerep = 1;
	}
	sfp->ps.laser.margin = 0;
	sfp->ps.laser.textcolour = 255;
	sfp->type = WOOLZ;
	for (i=0; i<256; i++) {		/* Set up LUT */
		if (mputc(i,sfp->fp) == -1) {
			spoolquit(sfp);
			return(0);
		}
	}
	if (wspoolobj(sol,sfp->fp)) {
		spoolclose(sfp);
		if (alc->timing)
			timings[local] = tick() - start_time;
		return(1);
	} else {
		spoolquit(sfp);
		if (alc->timing)
			timings[local] = tick() - start_time;
		return(0);
	}
}

/*
 *	W S P O O L O B J  --  Write spool objects to file
 *
 */
wspoolobj(sol,fp)
struct spoolobj *sol;
FILE	*fp;
{
	int		ret, count = 0, local, start_time;
	
	if (alc->timing)
		setuptim(&local, &start_time, "wspoolobj");
	ret = 1;
	if (objcount >= MAXSPOOLOBJS)
		fprintf(stderr, "Too many objects to spool\n");
	while (sol->object) {
		if (sol->frame)
			if (writeobj(fp,sol->frame) != 0) {
				ret = 0;
				break;
			}
		if (writeobj(fp,sol->object) != 0) {
			ret = 0;
			break;
		}
		windmill(NULL);
		count++;
		if (count < MAXSPOOLOBJS)
			sol++;
		else
			break;
	}
	if (alc->timing)
		timings[local] = tick() - start_time;
	return(ret);
}

metsize(xmin,xmax,ymin,ymax)
int		*xmin,*xmax,*ymin,*ymax;
{
 	SMALL aclsav[MAXOBJS];
 	int i;

	if (!chekread(0)) return;
	for (i=0; i<kcont->maxnumber; i++) {
		aclsav[i] = kcont->acl[i];
		if (kcont->ool[i] != NULL && !kcont->ool[i]->plist->history[1])
			kcont->acl[i] |= ACTIVE;
	}
	cellsize(kcont->ool,kcont->maxnumber,xmin,xmax,ymin,ymax);
	for (i=0; i<kcont->maxnumber; i++)
		kcont->acl[i] = aclsav[i];
}

/*
 *	I N I T H C P Y  --  Init hardcopy params
 *
 */
inithcpy(hcpy)
struct hardcopy *hcpy;
{
	hcpy->repeat = 1;
	hcpy->picsize = 3;
	hcpy->interpolate = 1;
}
/*
 * S E T U P H C P Y
 *
 */
setuphcpy()
{
	int i, local, start_time;
	
	if (alc->timing)
		setuptim(&local, &start_time, "setuphcpy");
	for (i = 0; i < MAXCLASS; i++)
		classptr[i] = NULL;
	for (i = 0; i < NSTACKPOS; i++)
		stackptr[i] = NULL;
	do_mets(alc->hcpf->hco);
	do_text(alc->hcpf->hco);
	setupclasses(alc->hcpf->hco);
	do_kars();
	if (alc->timing)
		timings[local] = tick() - start_time;
}
/*
 * D O _ T E X T
 *
 */
do_text(ptr)
struct hcobject *ptr;
{
	struct pframe *f;
	int x, y, local, start_time;
	
	if (alc->timing)
		setuptim(&local, &start_time, "do_text");
	while (ptr != NULL) {
		if (objcount >= MAXSPOOLOBJS)
			break;
		if (ptr->ass != NULL)
			do_text(ptr->ass);
		if (ptr->type == TXT) {
			f = makepframe(8, 0, 0, 0, 0);
			f->ix = ptr->b.width/20;
			f->iy = ptr->b.height/20;
			if (alc->debug)
				fprintf(debug_channel, "DO_TEXT :ix %d iy %d x %d y %d width %d height %d text !%s!\n", f->ix, 
					f->iy, ptr->b.posn.x, ptr->b.posn.y, ptr->b.width, ptr->b.height,
					ptr->txt);
			sptr->frame = f;
			x = (ptr->b.posn.x/f->ix) >> alc->ddgschannel->xshift;
			y = (ptr->b.posn.y/f->iy) >> alc->ddgschannel->yshift;
			sptr->object = (struct object *) maketext(ptr->txt, y, x, 
														strlen(ptr->txt) + 1);
			objcount++;
			sptr++;
			if (objcount >= MAXSPOOLOBJS)
				break;
			x = strlen(ptr->txt) * TXTSIZE * f->ix * f->scale;
			x = ((x + ptr->b.posn.x)/f->ix) >> alc->ddgschannel->xshift;
			switch(ptr->asstxt) {
				case -1 :
					sptr--;
					objcount--;
					break;
				case CASE :
					sptr->object = (struct object *) maketext(alc->cell_h->patid,
										y, x, strlen(alc->cell_h->patid) + 1);
					break;
				case SLIDE :
					sptr->object = (struct object *) maketext(alc->cell_h->slideid,
										y, x, strlen(alc->cell_h->slideid) + 1);
					break;
				case CELL :
					sprintf(cellid, "%d", alc->cell_h->cellid);
					sptr->object = (struct object *) maketext(cellid,
										y, x, strlen(cellid) + 1);
					break;
				case OPERATOR :
					sptr->object = (struct object *) maketext(alc->cell_h->opid,
										y, x, strlen(alc->cell_h->opid) + 1);
					break;
				case DATE :
					sptr->object = (struct object *) maketext(alc->cell_h->date,
										y, x, strlen(alc->cell_h->date) + 1);
					break;
				case SUPERVISOR :
					sptr->object = (struct object *) maketext(alc->cell_h->supid,
										y, x, strlen(alc->cell_h->supid) + 1);
					break;
				case SUPERDATE :
					sptr->object = (struct object *) maketext(alc->cell_h->supdate,
										y, x, strlen(alc->cell_h->supdate) + 1);
					break;
				case PATNAME :
					sptr->object = (struct object *) maketext(alc->cell_h->patnam,
										y, x, strlen(alc->cell_h->patnam) + 1);
					break;
				case COMMENT1 :
					sptr->object = (struct object *) maketext(alc->cell_h->comments[0],
										y, x, strlen(alc->cell_h->comments[0]) + 1);
					break;
				case COMMENT2 :
					sptr->object = (struct object *) maketext(alc->cell_h->comments[1],
										y, x, strlen(alc->cell_h->comments[1]) + 1);
					break;
				case KARYOTYP :
					sptr->object = (struct object *) maketext(alc->cell_h->krtype,
										y, x, strlen(alc->cell_h->krtype) + 1);
					break;
				case LABNAME :
					sptr->object = (struct object *) maketext(alc->cell_h->lab_name,
										y, x, strlen(alc->cell_h->lab_name) + 1);
					break;
			}
			objcount++;
			sptr++;
		}
		ptr = ptr->next;
	}
	if (alc->debug)
		fprintf(debug_channel, "End of Do_text\n");
	if (alc->timing)
		timings[local] = tick() - start_time;
}

/*
 *  D O _ M E T S
 *
 */
do_mets(ptr)
struct hcobject *ptr;
{
	struct pframe *f;
	int xmin, ymin, xmax, ymax, i, scale, width, height, local, start_time;
	
	if (alc->timing)
		setuptim(&local, &start_time, "do_mets");
	while (ptr != NULL) {
		if (objcount >= MAXSPOOLOBJS)
			break;
		if (ptr->ass != NULL)
			do_mets(ptr->ass);
		if (ptr->type == METAPHASE) {
			metsize(&xmin, &xmax, &ymin, &ymax);
			width = xmax - xmin;
			height = ymax - ymin;
			if (kcont->FIP == DIGFIP)
				height = (height * 3)/4;
			if (ptr->b.width < ptr->b.height)
				scale = ptr->b.width / width;
			else
				scale = ptr->b.height / height;
			f = makepframe(1, ptr->b.posn.x, ptr->b.posn.y, 0, 0);
			if (alc->debug)
				fprintf(debug_channel, "DO_METS :x %d y %d\n", ptr->b.posn.x, ptr->b.posn.y);
			if (scale <= 1) {
				f->ix = 4;
				if (kcont->FIP == DIGFIP)
					f->iy = 3;
				else
					f->iy = 4;
			} else {
				f->ix = scale;
				if (kcont->FIP == DIGFIP)
					f->iy = (scale * 3)/4;
				else
					f->iy = scale;
			}
			if (alc->debug)
				fprintf(debug_channel, "DO_METS :width %d height %d bwidth %d bheight %d scale %d ix %d iy %d\n", width, height, ptr->b.width, 
					ptr->b.height, scale, f->ix, f->iy);
			sptr->frame = f;
			for (i = 0; i < kcont->maxnumber; i++) {
				if (objcount >= MAXSPOOLOBJS)
					break;
				if (kcont->ool[i] != NULL && !kcont->ool[i]->plist->history[1]) {
					sptr->object = (struct object *) kcont->ool[i];
					objcount++;
					sptr++;
				}
			}
		}
		ptr = ptr->next;
	}
	if (alc->timing)
		timings[local] = tick() - start_time;
}
/*
 * S E T U P C L A S S E S -- put class and stack ptrs into array in order.
 *
 */
setupclasses(ptr)
struct hcobject *ptr;
{
	while (ptr != NULL) {
		if (ptr->ass != NULL)
			setupclasses(ptr->ass);
		if (ptr->type == CLASS)
			classptr[ptr->class] = ptr;
		else if (ptr->type == STK)
			stackptr[ptr->class] = ptr;
		ptr = ptr->next;
	}
}
/*
 * D O _ K A R S
 *
 */
do_kars()
{
	int i, class, num[MAXCLASS], local, start_time;
	struct pframe *f;

	if (alc->timing)
		setuptim(&local, &start_time, "do_kars");
	for (i = 0; i < MAXCLASS; i++)
		num[i] = 0;
	/* calculate the number of chromosomes in each class */
	for (i = 0; i < kcont->maxnumber; i++) {
		if (kcont->acl[i] & ACTIVE) {
			if (kcont->mol[i]->plist->Cpgroup)
				class = kcont->mol[i]->plist->Cpgroup;
			else
				class = kcont->mol[i]->plist->pgroup;
			if (kcont->mol[i]->plist->disppos > num[class])
				num[class] = kcont->mol[i]->plist->disppos;
		}
	}
	for (i = 0; i < kcont->maxnumber; i++) {
		if (objcount >= MAXSPOOLOBJS)
			break;
		if (kcont->acl[i] & ACTIVE) {
			f = makepframe(1, 0, 0, 0, 0);
			setupframe(f, kcont->mol[i], num);
			sptr->frame = f;
			sptr->object = (struct object *) kcont->mol[i];
			objcount++;
			sptr++;
		}
	}
	if (alc->timing)
		timings[local] = tick() - start_time;
}

/*
 * S E T U P F R A M E 
 *
 */
setupframe(f, obj, num)
struct pframe *f;
struct chromosome *obj;
int num[];
{
	struct intervaldomain *idom;
	struct hcobject *ptr;
	int indwidth, width, disppos, class;
	
	f->type = 60;
	f->scale = alc->hcpf->karscale;
	idom = obj->idom;
	width = idom->lastkl - idom->kol1;
	width *= f->scale;
	if (obj->plist->Cpgroup)
		class = obj->plist->Cpgroup;
	else
		class = obj->plist->pgroup;
	if (class > alc->scrf->undefclass) {
		if (obj->plist->disppos > alc->hcpf->viewed_stack)
			ptr = stackptr[alc->hcpf->viewed_stack];
		else
			ptr = stackptr[obj->plist->disppos];
		if (ptr == NULL)
			return(0);
		indwidth = ptr->b.width;
		disppos = 0;
	} else {
		ptr = classptr[class];
		if (ptr == NULL)
			return(0);
		indwidth = ptr->b.width / num[class];
		disppos = obj->plist->disppos - 1;
	}
	if (alc->debug)
		fprintf(debug_channel, "width %d width * scale %d box width %d box (%d, %d) indwidth %d\n", 
			width, width*f->scale, ptr->b.width, ptr->b.posn.x, ptr->b.posn.y, indwidth);
	if (width >= indwidth)
		f->dx = ptr->b.posn.x + (disppos * indwidth);
	else {
		width = indwidth - width;
		width /= 2;
		f->dx = ptr->b.posn.x + (disppos * indwidth) + width;
	}
	f->dy = ptr->b.posn.y;
	if (obj->plist->dispor == -1) {
		f->ix = -1;
		f->iy = -1;
		f->oy = idom->lastln;
		f->ox = idom->lastkl;
	} else {
		f->ix = 1;
		f->iy = 1;
		f->ox = idom->kol1;
		f->oy = idom->line1;
	}
	f->oy += obj->plist->dispopts;
	if (alc->debug)
		fprintf(debug_channel, "SETUPFRAME :class %d indwidth %d ix %d iy %d ox %d oy %d dx %d dy %d\n\n",
		class, indwidth, f->ix, f->iy, f->ox, f->oy, f->dx, f->dy);
	return(1);
}
/*
 * D O _ T I M I N G S -- open file and write timings out
 *
 */
static
do_timings()
{
	FILE *fp, *fopen();
	char buffer[80];
	int i = 0;
		
	sprintf(buffer, "/dd/usr/%s/timings.seat%d", alc->user, alc->seat);
	if ((fp = fopen(buffer, "a")) != NULL) {
		while (strings[i] != NULL) {
			fprintf(fp, "%s  %d\n", strings[i], timings[i]);
			i++;
		}
		fprintf(fp, "\n");
		fclose(fp);
	} else
		fprintf(stderr, "Can't open timing file\n");
	curr_time = -1;
	/* reset arrays */
	for (i = 0; i < MAXTIMS; i++) {
		timings[i] = 0;
		strings[i] = NULL;
	}
}
/*
 * S E T U P T I M-- set up string and start time if timing
 *
 */	
static
setuptim(loc, time, str)
int *loc, *time;
char *str;
{
	if (curr_time < MAXTIMS - 1)
		curr_time++;
	*loc = curr_time;
	strings[*loc] = str;
	*time = tick();
}

