/*
 *		K A R Y S U B S 1 . C  --  Karyotype interaction
 *
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *	Based on: karysubs1.c
 *			  Jim Piper, MRC CAPCU
 *			  23 Jan 1984
 *
 *  Modifications
 *
 *   4 Feb 1992		Joanne  added "mode" and STACKHEIGHT for hanging chromosomes from top 
 *   3 Feb 1992		Joanne  moved karyotype layout to the right
 *  21 Jan 1992		Joanne	Added "karyinitpos" to read karyogram positions
 *				from file rather than from data structure
 *	 4 Jul 1989		SEAJ		Check Cpgroup in stacktidy().
 *	12 Jun 1989		dcb			Put null at end of string in boxtext.
 *	 9 Mar 1989		dcb			Draw thicker id box lines to prevent flicker
 *	 6 Mar 1989		Jimp		re_use_if_pos -> Get correct class size
 *	 2 Dec 1988		CAS			Move reposition to karylib
 *								, kdisplay, unkdisplay, hiddenstackcount
 *								remove_class to karylib to reduce duplication.
 *								VIEWED_STACK to dispform.h
 *	 2 Dec 1988		CAS/SEAJ	Keep track of kccp stack pos list in reorderchr
 *								and moved the call to it from karyogram () to kary()
 *								Moved newclass to karylib/karysubs2
 *	 3 Oct 1988		CAS/GJP		Set up frame type
 *	 5 Sep 1988		CAS			Fiddling with display
 *	23 Aug 1988		CAS			More flex display stuff
 *	19 Aug 1988		CAS			Get display stuff off disk
 *	05 Aug 1988		SCG			Mods for pick'an'place, rearrange most likely
 *								chroms out of the stack
 *	17 May 1988		CAS			Start of flexible ktypebox stuff - currently
 *								only the box banner.
 */

#include <stdio.h>
#include <allcontr.h>
#include <wstruct.h>
#include <interact.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <display.h>
#include <dispform.h>

#define TWOWIDE	24
#define BIGNUM 100000
#define MIN(a,b) (((a) > (b)) ? (b) : (a))
#define STACKHEIGHT  600   /* to "hang" chromosomes you need the height of the frame */

/*
 *	Local storage
 */
struct clconf {
	int 	objnum;
	int 	conf;
};
static struct clconf clcr[MAXCHILD];

/*
 * externals
 */
extern struct allkontr *alc;
extern struct kcontrol *kcont;
extern int DOWN;		/* defined in karyfix.c */
extern int kdispscale;		/* defined in karyfix.c */
extern char *getspec();		/* defined in utilities.c */

/*
 * globals
 */
struct text *textobj;
struct kclasscont kcc[MAXCLASS];
extern struct allkontr *alc;

/*
 *	K A R Y S C A L E  --
 *
 */
karyscale(s)  {
	kcont->kdispscale = s;
	showkary();
}

static char filename[100];
karyinitpos()
{
	register int i;
	register struct kdisppos *d = alc->scrf->disppos;
	FILE *f, *fbopen();
	int p1,p2,p3,p4,p5;
	char *getspec();
	
	/*
	 *   get filename  by combining known string (dir+name)
	 *   and current species as suffix
	 */
	sprintf(filename, "/dd/params/ky_layout.%s",getspec());

	f = fbopen(filename,"r",BUFSIZ);
	if (f == NULL) {
		fprintf(stderr,"Cannot open %s for karyotype layout\n",filename);
		kary_exit(2);
	}
	fscanf(f,"%d %d",&alc->scrf->undefclass, &alc->scrf->mode);
	for (i=0; i<=alc->scrf->undefclass; i++) {
		/*
		 * avoid problems with word-length by always reading
		 * integers (kdisppos members are mostly SMALLs).
		 */
		fscanf(f,"%d %d %d %d %d %s",&p1,&p2,&p3,&p4,&p5,d->tag);
		d->col = p1 + 300;
		d->lin = p2;
		d->space = p3;
		d->height = p4;
		d->npos = d->defaultpos = p5;
		d++;
	}
	d--;
	alc->scrf->viewed_stack = d->npos;	/* "abnormals" class size */
	d->npos = d->defaultpos = 0;	/* necessary to correctly stack abnormals */
	fbclose(f);
}



/*
 *	K A R Y O G R A M  -- construct initial karyogram from classified objects
 *
 */
karyogram(ikcont)
struct kcontrol *ikcont;
{
	int i, j, stack;
 	struct chromplist *plist;
	struct kclasscont *kccp;
	/*
	 * clear various things
	 */
	kcont = ikcont;	
	kccp = kcc;
	for (i=0; i<MAXCLASS; i++) {
		alc->scrf->disppos[i].npos = alc->scrf->disppos[i].defaultpos;
		kccp->n = kccp->maxn = 0;
		for (j=0; j<MAXCHILD; j++)
			kccp->kno[j] = -1;
		kccp++;
	}
	for (i=0; i<=NSTACKPOS; i++)
		unstack(i);
	for (i=VIEWED_STACK+1; i<=NSTACKPOS; i++)
		alc->scrf->stackcol[i] = 30000;
	/*
	 * Modify disppos->npos values dependent on number of active objects.
	 */
	modnpos(kcont);

	/*
	 * classified objects placed in notional positions
	 * i.e. not actually displayed yet (see "displayall")
	 */
	for (i=0; i<kcont->maxnumber; i++) {
		if ((ACTIVE & kcont->acl[i]) != 0) {
			firstclass(i,kcont,alc->scrf->disppos);
		}
	}
}
/*
 *  M O D N P O S --
 *
 *  Modify the disppos[i].npos value depending on the number of active
 *  chromosomes in the cell - aim to caterfor triploid, tetraploid, and
 *  prevent stack overflow.
 */
modnpos(kcont)
struct kcontrol *kcont;
{
int i, actin;

	actin = 0;
	for(i = 0; i < kcont->maxnumber; i++) 
		if((ACTIVE & kcont->acl[i]) != 0)
			actin++;
	actin -= 35;		/* POSSIBLY NOT SUITABLE FOR NON-HUMAN CHROMS. */
	if(actin < 0)
		actin = 0;
	actin /= 23;		/* POSSIBLY NOT SUITABLE FOR NON HUMAN CHROMS> */
	for (i = 0; i < alc->scrf->undefclass; i++)
		alc->scrf->disppos[i].npos += actin;
		
}


/*
 *	T I D Y S C R E E N  --
 *
 * tidy up screen.  In particular, move remaining abnormals to left
 * of screen and put trisomies in correct group positions.
 */
tidyscreen(kcont)
struct kcontrol *kcont;
{
	register int i,j,l;
	struct kident k;
	register struct kclasscont *kc = kcc;
	struct kdisppos *dp = alc->scrf->disppos;
	for (i=0; i<alc->scrf->undefclass; i++) {
		if (kc->n > dp->npos || 
			(kc->n < dp->npos && dp->npos > dp->defaultpos)) {
			k.kclass = i;
			/*
			 * delete class contents from present positions
			 */
			for (j=0; j<kc->maxn; j++) {
				k.pos = j+1;
				k.no = kc->kno[j];
				if (k.no >= 0) {
					k.stackpos = kc->sp[j];
					if (k.stackpos >= 0)
						unstack(k.stackpos);
					unkdisplay(&k,kcont);
					kc->sp[j] = -1;
				}
			}
			/*
			 * pack down class contents structure
			 * at end, kc->n and kc->maxn should be same.
			 */
			for (j=0; j<kc->maxn; j++) {
				if (kc->kno[j] < 0) {
					kc->maxn--;
					for (l=j; l<kc->maxn; l++) {
						kc->kno[l] = kc->kno[l+1];
					}
					j--;
				}
			}
			/*
			 * increment the number permitted in class display
			 */
			dp->npos = kc->n;
			if(dp->npos < dp->defaultpos)
				dp->npos = dp->defaultpos;
			/*
			 * and display the class contents
			 */
			for (j=0; j<kc->n; j++) {
				k.pos = j+1;
				k.no = kc->kno[j];
				kc->sp[j] = k.stackpos = -1;
				/*  then must update disppos. */
				kcont->mol[k.no]->plist->disppos = k.pos;
				kdisplay(&k,kcont);
			}
		}
		kc++;
		dp++;
	}
	stacktidy(kcont);
	tsize(50, 50);
	disptags();
	hiddenstackcount();
}



/*
 *	D I S P L A Y A L L  --  display every chromosome in its current position
 *
 */
displayall(kcont,thereiscellinfo)
struct kcontrol *kcont;
int thereiscellinfo;
{
	struct kident kid;
	register int i, j;
	register struct kclasscont *kccp;
	colour(255);
	fast();
	mpclear(kcont);
	slow();
	colour(GREYCOLS);
	tsize(50,50);
	kccp = kcc;
	intens(1);
	for (i=0; i<alc->scrf->undefclass; i++) {
		for (j=0; j<kccp->maxn; j++) {
			kid.no = kccp->kno[j];
			if (kid.no >= 0) {
				kid.pos = kcont->mol[kid.no]->plist->disppos;
				kid.stackpos = kccp->sp[j];
				kid.kclass = i;
				kdsetup(&kid,kcont);
			}
		}
		kccp++;
	}
	disptags();
	hiddenstackcount();
	/*
	 * multi-slave display
	 */
	mpipic(kcont,GREYCOLS,1);
}


kdsetup(k,kcont)
struct kident *k;
struct kcontrol *kcont;
{
	struct chromosome *obj, *newgrey();
	int normit,i,l;
	char tag[10];
	struct pframe *f = kcont->ivf+k->no;

	/*
	 * set the picture frame
	 */
	setframe(f,k,kcont);

	/*
	 * construct and display the object's tag,
	 * but only if in the stack area.
	 */
	if (k->pos > alc->scrf->disppos[k->kclass].npos) {
		i = 0;
		if (alc->scrf->disppos[k->kclass].tag[0]) {
			strcpy(tag,alc->scrf->disppos[k->kclass].tag);
			i = strlen(tag);
			tag[i++] = 'a' + k->pos - 1;
		}
		tag[i] = '\0';
		colour(OVERLAY1);
		tsize(50,50);
		if (alc->scrf->mode == TOP) 
			dispstring(tag,f->dx,f->dy-STACKHEIGHT);
		else
			dispstring(tag,f->dx,f->dy-96);
	}
	/*
	 * mark the object if so specified
	 */
	if (k->no >= 0) {
		obj = kcont->mol[k->no];
		if (obj->plist->dispmark & PERM_MARK)
			dispmark(f);
	}
}

/*
 * set the display frame "f" to suitable for display of chromosome
 * described in identification structure "k".  The following are
 * relevant:
 *	- Chromosome class
 *	- ordinal within the class, and whether the "disppos" structure
 *	  currently expects this many chromosomes.  If not, then a
 *	  "stack" location is provided.
 *	- whether chromosome should be inverted
 *	- modifications for aspect ratio and video recording
 */
setframe(f,k,kcont)
register struct pframe *f;
register struct kident *k;
struct kcontrol *kcont;
{
	struct chromosome *obj = kcont->mol[k->no];
	struct object *objp = (struct object *) obj; /* to avoid compile time
												    error reports !! */
	register struct intervaldomain *idom;
	register struct kdisppos *dispp;
	int width;

	f->type = 60;
	idom = objp->idom;
	width = idom->lastkl - idom->kol1;
	dispp = alc->scrf->disppos + k->kclass;
	f->scale = kcont->kdispscale;
	f->ox = (idom->kol1 + idom->lastkl)/2;
	f->dy = dispp->lin - DOWN;
	if (f->dy < 100)
		f->dy = 100;
	if (alc->scrf->mode && TOP)
		f->dy += dispp->height;

	/*
 	 * 3 cases to be considered:
	 *	1)	k->pos > dispp->npos
	 *	2)	dispp->npos == 2    &&    k->pos <= 2
	 *	3)	k->pos <= dispp->npos > 2
	 */
	if (k->pos > dispp->npos) {
		f->dx = alc->scrf->stackcol[k->stackpos];
		f->dy = alc->scrf->stacklin;
		if (alc->scrf->mode && TOP)
			f->dy += STACKHEIGHT;
	} else if (dispp->npos == 2) {
		switch (k->pos) {
		case 1:
			f->dx = dispp->col - TWOWIDE - kcont->kdispscale*width/2;
			break;
		case 2:
		default:
			f->dx = dispp->col + TWOWIDE + kcont->kdispscale*width/2;
			break;
		}
	} else {
		/* this gives the left hand edge of box - wrong */
		f->dx = dispp->col - dispp->space/2 + 
				(2*k->pos-1)*dispp->space/(2*dispp->npos);
	}

	/*
	 * to be inverted ?
	 */
	if (obj->plist->dispor == -1) {
		f->ix = -1;
		f->iy = -1;
		if (alc->scrf->mode && TOP)
			f->oy = idom->line1;
		else
			f->oy = idom->lastln;
	} else {
		f->ix = 1;
		f->iy = 1;
		if (alc->scrf->mode && TOP)
			f->oy = idom->lastln;
		else
			f->oy = idom->line1;
	}
	f->oy += obj->plist->dispopts;
}


/*
 * re-use a position just emptied to get an object from
 * bottom line into correct position where appropriate.
 * The position and class are specified in identifier
 * structure "k".
 */
re_use_if_poss (k,kcont)
register struct kident *k;
struct kcontrol *kcont;
{
	struct kident kn;
	register int i;
	register struct kclasscont *kc = kcc + k->kclass;
	/*
	 * do nothing in various circumstances:
	 *	- spare position non-empty (an error ?)
	 *	- spare position is a stack position - then do a stack tidy
	 */
	if (kc->n == kc->maxn)
		return;
	if (k->pos > alc->scrf->disppos[k->kclass].npos) {
		stacktidy(kcont);
		return;
	}
	if (kc->kno[k->pos-1] >= 0)
		return;
	/*
	 * position seems (i) not stack, (ii) empty.
	 * see if there is anything to fill it
	 */
	for (i=kc->maxn-1; i>=alc->scrf->disppos[k->kclass].npos; i--) {
		if (kc->kno[i] >= 0) {
			/* delete from first position */
			kn.no = kc->kno[i];
			kn.pos = i+1;
			kn.stackpos = kc->sp[i];
			kn.kclass = k->kclass;
			unkdisplay(&kn,kcont);
			remove_class(&kn);
			/* add to better position */
			newclass(&kn,k->kclass,kcont,alc->scrf->disppos);
			kdisplay(&kn,kcont);
			stacktidy(kcont);	/* in case object removed from stack */
			return;
		}
	}
}

/*
 * display the standard class tags on screen
 */
disptags()
{
	register int i;
	register struct kdisppos *dp;
	int		xsize, ysize;

	findtsize(&xsize,&ysize);
	dp = alc->scrf->disppos;
	colour(OVERLAY1);
	for (i=0; i<alc->scrf->undefclass; i++) {
		if (*dp->tag) 
			if (alc->scrf->mode == TOP) 
				dispstring(dp->tag,dp->col-(strlen(dp->tag) * xsize / 2),dp->lin+48);
			else
				dispstring(dp->tag,dp->col-(strlen(dp->tag) * xsize / 2),dp->lin-96);
		dp++;
	}
	update();
}

/*
 *	D I S P I D B O X  --  Display id box for karyotype (patient data etc)
 *							Unpaint old box, and if onoff is 1 evaluate
 *							and paint new one.
 *
 */
dispidbox(x,y,onoff)
int		x,y,onoff;
{
	static struct cell_header oldheader;

	intens(0);
	disp_kr_header(x,y,&oldheader);
	colour(OVERLAY2);
	intens(1);
	if (onoff) {
		disp_kr_header(x,y,alc->cell_h);
		_strass(&oldheader,alc->cell_h,sizeof(struct cell_header));
	}
}

/*
 * 	D I S P _ K R _ H E A D E R   Displays a caption on the karyogram.
 *
 */
static
disp_kr_header(xpos, ypos, headerptr)
int	xpos, ypos;
struct	cell_header *headerptr;
{
	int	oldxtsize, oldytsize;
	struct databox *id;

	id = &alc->scrf->idbox;
	findtsize(&oldxtsize, &oldytsize);	/* Get previous text size. */
	xpos = id->position.x + id->width/2;
	ypos = id->position.y + id->height/2;			/* Set current box centre */
	moveto(xpos,ypos);	/* id->position.x + id->width/2,id->position.y + id->height/2);/* Set current box centre */
	lwidth(2);										/* Draw thicker box lines */
	box(id->width, id->height);						/* Draw box. */
	box(id->width - 16, id->height - 32);			/* Draw inside border. */
	lwidth(1);
	tsize(id->banner.size, id->banner.size);									/* Change text size. */
	disptext(id->banner.text,id->position.x + id->banner.x, id->position.y + id->banner.y);

	boxtext(&id->case,headerptr->patid,CASELENGTH,id->position.x,id->position.y);
	boxtext(&id->slide,headerptr->slideid,SLIDELENGTH,id->position.x,id->position.y);
	boxnum(&id->cell,headerptr->cellid,id->position.x,id->position.y);
	boxtext(&id->operator,headerptr->opid,20,id->position.x,id->position.y);
	boxtext(&id->date,headerptr->date,DATELENGTH,id->position.x,id->position.y);
	boxtext(&id->supervisor,headerptr->supid,OPERLENGTH,id->position.x,id->position.y);
	boxtext(&id->superdate,headerptr->supdate,DATELENGTH,id->position.x,id->position.y);
	boxtext(&id->patname,headerptr->patnam,PATLENGTH,id->position.x,id->position.y);
	boxtext(&id->comment1,headerptr->comments[0],TEXTLENGTH,id->position.x,id->position.y);
	boxtext(&id->comment2,headerptr->comments[1],TEXTLENGTH,id->position.x,id->position.y);
	boxtext(&id->karyotype,headerptr->krtype,TEXTLENGTH,id->position.x,id->position.y);
	boxtext(&id->labname,headerptr->lab_name,LABLENGTH,id->position.x,id->position.y);

	tsize(oldxtsize, oldytsize);		/* Recover previous text size. */
}

/*
 *	D I S P T E X T __  
 *
 */
static
disptext(str,x,y)
char	*str;
int		x,y;
{
	moveto(x,y);
	text(str);
}

/*
 *	B O X T E X T  --  Display text in ID box
 *
 */
static
boxtext(tp,str,len,x,y)
struct	idtext *tp;
char	*str;
int		len, x, y;
{
	char	buffer[120];

	if (tp->display) {
		len = MIN(len,tp->len);
		strcpy(buffer,tp->text);
		strncpy(&buffer[strlen(buffer)],str,len);
		buffer[strlen(tp->text) + len] = '\0';	/* Null at end of string */
		moveto(tp->x + x,tp->y + y);
		tsize(tp->size,tp->size);
		text(buffer);
	}
}

/*
 *	B O X N U M  --  Display a number in idbox
 *
 */
boxnum(tp,num,x,y)
struct	idtext *tp;
int		num, x, y;
{
	char	buffer[120];

	if (tp->display) {
		strcpy(buffer,tp->text);
		sprintf(&buffer[strlen(buffer)],"%d",num);
		moveto(tp->x + x,tp->y + y);
		tsize(tp->size,tp->size);
		text(buffer);
	}
}

	
/*
 *	S T A C K T I D Y  --	Keep the abnormals/overflow stack pushed to left
 *							hand corner of the screen.
 */
stacktidy(kcont)
register struct kcontrol *kcont;
{
	register struct kclasscont *kccp;
	register int i,j,k,objnum;
	int class;
	struct kident kid;
	
	for (i=j=1; j<=NSTACKPOS; j++) {
		/*
		 * stack location in use ?
		 */
		if ((objnum = tellstack(j)) > 0) {
			/*
			 * any gaps in stack ? if no, continue
			 */
			if (i == j) {
				i++;
				continue;
			}
			/*
			 * otherwise identify this object, undisplay it.
			 */
			objnum--;
			if (kcont->mol[objnum]->plist->Cpgroup)
				class = kcont->mol[objnum]->plist->Cpgroup - 1;
			else
				class = kcont->mol[objnum]->plist->pgroup - 1;
			kccp = kcc + class;
			kid.no = objnum;
			kid.pos = kcont->mol[objnum]->plist->disppos;
			kid.stackpos = j;
			kid.kclass = class;
			if (j <= VIEWED_STACK)
				unkdisplay(&kid,kcont);
			/*
			 * now insert object in first hole in stack and display it
			 */
			unstack(j);
			for (k=0; k<kccp->maxn; k++)
				if (kccp->kno[k] == objnum) {
					kid.stackpos = kccp->sp[k] = stack(objnum);
					if (kccp->sp[k] != i)
						fprintf(stderr,"Stacktidy - ouch\n");
					break;
				}
			if (i <= VIEWED_STACK)
				kdisplay(&kid,kcont);
			i++;
		}
	}
	hiddenstackcount();
}

/*
 * C O M P F 1 ---
 *
 */
compf1(pt1, pt2)
struct clconf *pt1, *pt2;
{
	if (pt1->conf > pt2->conf)
		return(-1);
	else if (pt1->conf < pt2->conf)
		return(1);
	else
		return(0);
}

/*
 * R E O R D E R C H R --- reorder chroms according to confidence level
 *
 */
reorderchr(kcont)
struct kcontrol *kcont;
{
	register int i, j, k, n;

	/*
	 * rearrange each class
	 */
	for (i = 0; i < MAXCLASS; i++) {
		for (j = 0, n = 0; j < kcc[i].maxn; j++) {
			if (kcc[i].kno[j] != -1) {
				clcr[n].objnum = kcc[i].kno[j];
				clcr[n].conf = kcont->mol[clcr[n].objnum]->plist->pgconf;
				n++;
			}
		}
		
		if (n > 1)
			qsort(clcr, n, sizeof(struct clconf), compf1);
			
		for (j = 0, n = 0; j < kcc[i].maxn; j++) {
			if (kcc[i].kno[j] != -1) {
				kcc[i].kno[j] = clcr[n++].objnum;
				kcont->mol[kcc[i].kno[j]]->plist->disppos = j + 1;
				if (kcc[i].sp[j] >= 0) {
					unstack(kcc[i].sp[j]);
					kcc[i].sp[j] = stack(kcc[i].kno[j]);
				}
			}
		}
	}
}				


