/*
 * kenlarge.c 		31-03-87		Jim Piper
 *
 * Multi-slave driver for the enlarge-class karyotype display function.
 *
 * Modifications
 * -------------
 *
 * 27 Jan 1989	SEAJ	Make function return the maximum length of class.
 * 15-07-87		jimp	Get the horizontal spacing right !!
 */

#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <multip.h>
#include <allcontr.h>
#include <dispform.h>

extern struct allkontr *alc;

#define ENLARGE_FACTOR	150		/* Percentage enlargement by interpolation */
#define ENLARGE_SCALE	16		/* Enlarged object display scale */


/*
 * E N L A R G E
 *
 * Interpolated display in larger scale frame of all chromosome
 * in a particular group.
 */
int enlarge(kcont,class)
struct kcontrol *kcont;
int class;
{
	register struct chromosome *nobj;
	register int ac,i,objnum,num;
	register struct pframe *f;
	struct kident kid1;
	int width,fx,classmem[MAXCHILD], length, max_length=0;
	register SMALL *statpt;
	
	if (class >= 0 && class <= MAXCLASS) {
		fast();
		mpclear(kcont);
		slow();
		num = 0;
		for (i=0; i<MAXCHILD; i++)
			if (kdentify(&kid1,class,i)) {
				nobj = kcont->mol[kid1.no];
				classmem[num] = kid1.no;
				f = kcont->ivf + kid1.no;
				f->scale = ENLARGE_SCALE;
				width = f->scale * ENLARGE_FACTOR *
					(nobj->idom->lastkl - nobj->idom->kol1)/100;
				length = f->scale * ENLARGE_FACTOR *
					(nobj->idom->lastln - nobj->idom->line1)/100;
				if (max_length < length)
					max_length = length;
				f->dy = 512;
				if (alc->scrf->mode == TOP)
					f->dy += length;
				if (num == 0)
					fx = 1024;
				f->dx = fx + width/2;
/*				fx += 128 + width;*/
				fx += 2*(128 + width);				
				num++;
			}

		/* set function bits in activity list */
		for (i=0; i<kcont->maxnumber; i++) 
			kcont->acl[i] |= (FUNCBEGIN|FUNCEND|MASTCOPY|MPROC|SLFREE);

		/* clear function bits in activity word of specified object */
		for (i=0; i<num; i++)
			kcont->acl[classmem[i]] &= ~(FUNCBEGIN|FUNCEND|MASTCOPY|MPROC|SLFREE);
	
		/* clear function bits in slave status words, and start slaves */
		kcont->auxstat = ENLARGE_FACTOR;		
		func_start_slaves(kcont,K_ENLARGE);

		/* wait for completion */
		for (i=0; i<num; i++) {
			objnum = classmem[i];
			statpt = kcont->acl + objnum;
			while (!(*statpt & FUNCEND))
				tsleep(2);
			/* permit slaves to free memory */
			*statpt |= MASTCOPY;
		}

		/* wait for slaves to finish */
		func_finish_slaves(kcont);
	}
	return(max_length);
}
