/*
 *			K M . C  --  Karyotyping and Metaphase finding 
 *
 *
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *	Based on km.c (1985)
 *			 MRC Clinical + Population Cytogenetis Unit
 *			 Edinburgh
 *
 *
 *  Modifications
 *
 *  25 Mar 1992	jimp	Forks "kmf" not "mf"
 *  07 Jan 1992		Joanne		Allocated space for kcont->oth and Setbuf(stderr, NULL).
 *  19 Apr 1991		ih			Return after certain (harmless) exceptions
 *	15 Jan 1991		CAs			Init debug_channel to stderr if debug off in case
 *								some nurd uses it anyway
 *	 9 Jan 1991		CAS			Voids..
 *	12 Mar 1990		dcb			Init with lin_colour_lut, instead of setcolour
 *	01 Mar 1990		dcb			pre-declare readlutfile
 *	28 Feb 1990		dcb			Read LUT file after calling sysinit(). I will
 *								restructure this to have "read kmrc", & "init"
 *								as separate modules. (someday!)
 *	12 Dec 1989		dcb			Replace HOME_MENU_NAME with version & write
 *								version to syserrors on exception (version.h)
 *	27 Oct 1989		dcb			Version 9.0BETA
 *	23 Oct 1989		dcb			set master priority after sysinit
 *	14 Sep 1989		dcb			Version 8.7
 *	18 Aug 1989		CAS			Force flush in suicide
 *  14 Jun 1989		SEAJ		Version 8.3 (laser3 bug)
 *	13 jun 1989		dcb			Version 8.2
 *	 1 Jun 1989		CAS			Def errno for v2.2
 *								Change environ handling
 *	26 May 1989		dcb			Version 8.0 (2nd Jun V8.1.1)
 *	19 May 1989		dcb/GJP		Initialise kcont-FIP to -1
 *	10 May 1989		dcb			include class.h here (should this be in kyy.h)
 *	 9 May 1989		dcb			Allocate rankcf here as well (cf. alc->hcpy)
 *	 1 Mar 1989		dcb			Version 0.7.12
 *	 1 Mar 1989		CAS			Remove MIZAR/VAX stuff
 *	27 Feb 1989		dcb			Version 0.7.11
 *  22 Feb 1989		GJP			Version to 7.10 - 4
 *	01 Feb 1989		dcb			Comment into monitor file
 *	01 Feb 1989		SCG/dcb		clear after dinit
 *	16 Jan 1988		dcb			All logging only on for monitor == MONITOR_ALL
 *	 8 Dec 1988		CAS			Version 0.7.9
 *	 4 Dec 1988		CAS			Monitor logfile in home dir
 *	02 Dec 1988		dcb			Turn on MP_debug if debug level > 3
 *	 1 Dec 1988		CAS			Removed wtcheck.h include - not used
 *	28 Nov 1988		SCG			call woolz_check_level with alc->wclevel
 *	18 Nov 1988		SCG/dcb		Open debug channel in woolz_check_debug()
 *								also set level with woolz_check_level()
 *								include wtcheck.h as additional header
 *	17 Nov 1988		CAS			Enable cmdsys log if monitor set
 *	23 Sep 1988		CAS			Version 0.7.6
 *	19 Aug 1988		CAS			No longer need karyinitpos
 *	18 Aug 1988		CAS			Changes to configuration options
 *	25 Jul 1988		CAS			Version 0.7.5
 *	15 Jun 1988		CAS			Version 0.7.3
 *	23 May 1988		CAS			Change llut to setcolour
 *	17 May 1988		CAs			Change where command line is checked
 *	22 Apr 1988		SEAJ		change 'exit' to 'logout' in home menu
 *	 5 Apr 1988		CAs			Vers 0.7.1  release
 *	 1 Mar 1988		CAS			Vers 0.7.0 for nofold 7100's
 *	11 Feb 1988		CAS			Vers 0.6.6
 *	 8 Feb 1988		CAS			Vers 0.6.5
 *	 2 Feb 1988		CAS			Exception handling
 *	 1 Feb 1988		CAS			digitise -> image capture (cjc)
 *	27 Jan 1988		CAS			Reorder startup sequences, add fstore
 *								initialisation check
 *	25 Jan 1988		CAS			Vers 0.6.4
 */
#define INIFILE
#include <cmdsys.h>
#include <chromanal.h>
#include <cfipp.h>
#include <kyy.h>
#include <slides.h>
#include <kcontrol.h>
#include <signal.h>
#include <hardcopy.h>
#include <module.h>
#include <procid.h>
#include <version.h>

#define LOGFILE		"kmr.log"	/* Kmr cmdsys command log */

/*
 *	MF Home menu options
 */
#define MF_Fipops 1
#define MF_Find 2
#define MF_ListRev 3
#define MF_Capture 4
#define MF_Digitise 5

/* Conversion from km cmd numbers to Mf ones */
static	int		MF_cmds[9] = {0,0,0,0,0,0,0,0,0};	

extern int errno;

/*
 *		Global system data structure pointers
 *
struct allkontr *alc;			defined in kyy.h */
struct kcontrol *kcont;

FILE	*debug_channel = NULL;			/* System debug file */


static struct pframe mainframe = { 60, 8, 0, 0, 0, 0, 1, 1, 0 };

/*
 *	External function references
 */
extern int chromlyse();
extern int utilities();
extern int woolzexitfn();
extern int memexitfn();
extern int dispcellkm();
extern int tvdigitise();
extern int idinfo();
extern DDGS *ddgsopen();
extern int screendump();	/* Dump screen to hardcopy device */
extern	int	(*setexception())();
extern struct lut_assoc *readlutfile();

/*
 *	Local function references
 */
static upcontrols();
static	fstoreerror();

int m2child = 0;

/*
 *	Interrupt handling storage
 */
int	(*quit_fn)() = NULL;		/* Function to call on quit signal */

gethelp() {} /* dummy */

/*
 *	I believe this routine can be called from CMDSYS
 */
static
myjobs(nj) {
/*	switch (nj) {
		case -1 : getid();
	}
 */
	if (alc->debug)
		fprintf(stderr,"Warning -- myjobs called in km.c - job %d\n",nj);
}


/*
 * J N T E R R U P T  --  Interrupt handler
 *
 */
jnterrupt(sig)
int sig;
{
	static int busy = 0, inquit = 0;
	if (busy) return(0);
	busy = 1;
	if (alc->debug) {
		printf("KMR - seen sig %d\n",sig);
	}
	switch (sig) {
		case SIGWAKE:
			break;
		case SIGINT:
		case SIGQUIT:
			if (inquit) return(0);
			inquit = 1;
			if (quit_fn) {
				quit_fn();
			} else {
				if (alc->mf_pid) {
					if (kill(alc->mf_pid,KMR_INT) == -1) {
						if (alc->debug)
							fprintf(stderr,"KMR - mf sig failed, error %d\n",errno);
						suicide();
					} else {
						busy = 0; tsleep(0); busy = 1;
					}
				} else
					suicide();
			}
			inquit = 0;
			break;
		case MF_INT:
			if (quit_fn) {
				quit_fn();
			} else {
				suicide();
			}
			kill(alc->mf_pid,SIGWAKE);
			break;
		case MF_DIES:
			shutdown();
		case WOOLZSIG:
			fprintf(stderr,"Fatal error\007\007\n");
			shutdown();
		default:
			printf( "kmr: Signal %d received \n",sig);
	}
	busy = 0;
}

/*
 *	S U I C I D E  --
 */
static
suicide()
{
	char	c, c2;
	screengo(ENDLINE,0);
	while (_gs_rdy(fileno(stdin)) > 0)
		read(fileno(stdin),&c,1);
	printf("\nQuit seen - Do you wish to stop CYTOSCAN ? ");
	fflush(output);
	read(fileno(stdin),&c,1);
	if (c != '\n')
		do {
			read(fileno(stdin),&c2,1);
		} while (c2 != '\n');
	if (alc->debug)
		printf("stop %d\n",c);
	if ((c == 'y') || (c == 'Y')) {
		shutdown();
	}
	while (c != '\n') {
		if (ferror(stdin))
			clearerr(stdin);
		if (feof(stdin))
			cleareof(stdin);
		c = getc(stdin);
	}
}

/*
 *	E X C E P T I O N  --  Exception traps come here
 *
 */
exception(exc,addr)
int		exc,addr;
{
	procid	pd;
	int		fd, time, date, day, tick;
	char	buffer[120];

	gprdsc(getpid(),&pd);
	sprintf(buffer,"Exception %d, addr 0x%x, base 0x%x (offset 0x%x)\n",
		exc, addr, pd._pmodul, addr - (int)pd._pmodul);
	errorlog(buffer);
	/* don't shut down for zero div,chk & trapv exceptions */
	switch (exc) {
		case 20:
		case 24:
		case 28:
			setexception(exception);
			return(0);
		default:
			break;
	}
	shutdown();
}

/*
 *	S H U T D O W N  --  Shut all down
 *
 */
static 
shutdown()
{
	tidyworld();
	fprintf(stderr,"\nCYTOSCAN: shutdown\n");
	alldone(1);
}

/*
 *	T I D Y W O R L D  --
 *
 */
static
tidyworld()
{
	int slave;

	system("tmode echo eof=4");
	if (alc->kcn) {
		for (slave=0; slave<=alc->nslaves ; slave++)
			killslave(alc->kcn,slave,SIGKILL);
	}
}


/*
 *	M A I N
 *
 */
main(argc,argv,argp)
int  argc;
char **argv,**argp;
{
	int seatnumber = 1;
	char buffer[120];
	int		(*linkerbug)(), (*tmpexcept)();

	setbuf(stderr,NULL);
	setmemalloc(48*1024);
	alc = (struct allkontr *) Calloc(1,sizeof(struct allkontr));
	intercept(jnterrupt);
	setexception(exception);
	set_woolz_exit(woolzexitfn);
	set_kary_exit(woolzexitfn);
	set_chrom_exit(woolzexitfn);
	Alloc_exit(memexitfn);
	alc->cfipp = (struct cfipparams *) Calloc(1, sizeof(struct cfipparams));
	alc->hcpy  = (struct hardcopy *) Calloc(1, sizeof(struct hardcopy));
	alc->rankcf = (struct rankcf *) Calloc(1, sizeof(struct rankcf));

	alc->kmr_pid = getpid();
	if (argc > 1) {
		alc->nslaves = atoi(argv[1]);
		if (alc->nslaves < 0  || alc->nslaves > MAXSLAVE) {
			fprintf(stderr,"bad slave spec %d\n",alc->nslaves);
			alc->nslaves = 0;
		}
#ifdef DEBUG
		if (alc->debug)
			printf("no of slaves %d\n",alc->nslaves);
#endif
	}
	if ((argc > 2) && (argv[2][0] == 'd')) alc->devel = 1;
	sysinit(alc);
	setpr(alc->kmr_pid, alc->mprior);
	alc->lut = alc->luts = readlutfile();
/*
 *	Start up menu handler
 */
	mouseport(alc->mouseterm);	/* Where we want the mouse to be */
	muini(stdout);				/* Init cmdsys + mouse */
	alc->m2s = mukont->mouss;	/* Get mouse struct from cmdsys */
	set_header_fn(dispcellkm);	/* Do this display routine instead of the 
							  * up menu line
							  */

	pastmu = 0;				/* do recall up-menus on command string */
	uplow = 0;				/* do not identify upper-lower case */
	dojob = myjobs;			/* may be called from 'lead' at request of slave */
	linkerbug = screendump;
	cmd (GLOBAL,"Screenprint;%g",linkerbug); 
	screengo(ENDLINE,0);	/* for benefit of time message */
/*
 *	debug_channel should only be opened if debug is on
 */
	if (alc->debug) {
	 	debug_channel = fopen(alc->debugfile,"w");
		Alloc_debug(debug_channel);
		freeobj_debug(debug_channel);
		Makestruct_debug(debug_channel);
		woolz_check_debug(debug_channel);
		Classifier_debug(debug_channel);
		/*
		 * If debug level 3 or more, turn on multiproccessing debug.
		 * slave debug turned on when we call chromlyseinit
		 */
		if (alc->debug >= 3)
			MP_debug(debug_channel); /* Multip debugging */
	} else
		debug_channel = stderr;
	woolz_check_level(alc->wclevel);

#ifdef DEBUG
	if (alc->debug)
		printf("Config %d seat %d, term %s, mouse %s\n miproc %s baud %d, debugfile  %s\n",
		alc->config,alc->seat,alc->ourterm,
			alc->mouseterm, alc->miprocterm, alc->miprocbaud, alc->debugfile);
#endif
/*
 *	Open the frame store on the correct device
 */
	tmpexcept = setexception(fstoreerror);
	if (alc->debug) {
		fprintf(stderr,"Fstore tempexcept 0x%x, exception 0x%x\n",tmpexcept,exception);
		fprintf(stderr,"Fstore open address %x\n",alc->fstore-1);
	}
	alc->ddgschannel = ddgsopen(alc->fstoretype,alc->fstore - 1);
	dinit();				/* Init framestore */
	if (alc->debug)
		fprintf(stderr,"Fstore dinit\n");
	clear();
	if (alc->debug)
		fprintf(stderr,"Fstore cleared\n");
	lin_colour_lut();		/* Apply linear LUT + colour overlays */

	setexception(tmpexcept);
	if (!alc->scrf) setscreenformat();    
/*	screenformscale();		/* Scale format stuff for framestore type */
/*	karyinitpos();			/* copy desired kary layout coordinates across */
	upcontrols();
	if (alc->debug) {
		Show_alloc();
		Show_free();
	}
	kickid();				/* initialise id stuff */
	dodate();					/* Get system date */
	getopt(1);				/* Get user name */
	sprintf(buffer,"/dd/usr/%s",alc->user);
	if (!changedir(buffer)) {
		getwd(alc->dir,40);
	}
	if (alc->monitor == MONITOR_ALL) {
		sprintf(buffer,"/dd/usr/%s/%s%d",alc->user,LOGFILE, alc->seat);
		CSYS_log(buffer);
		CSYS_text("| Monitoring all actions\n");
	}
	setjmp(homeplace);
#ifdef DEBUG
	if (alc->debug)
		printf("seat %d, term %s, mouse %s, miproc %s, debugfile %s\n",alc->seat,alc->ourterm,
			alc->mouseterm, alc->miprocterm, alc->debugfile);
#endif
	makecellhead(&alc->cell_h);
	home();					/* home is where the action is */
	
	if ( alc->kcn) {
		if (kcont->processor[0]) killslave(alc->kcn, 0, 9);
	}
	alldone(0);
}

/*
 *	M E T F I N D  --  
 *
 */
metfind() 
{
	char	buffer[30];
	sprintf(buffer,"%d",alc);
	if(alc->frsbay == NULL) {
		fprintf(debug_channel, "Bay structures not set up.\n");
		return(0);
	}
	nncmd = MF_cmds[nncmd];
	nowfork("kmf",buffer);
}

/*
 * HOME MENU
 *
 */
home ()
{
	int	Exit_cmd;
	char home_menu_name[80];

	sprintf(home_menu_name, "    Cytoscan    Version %s  ", VERSION);
	newmenu ( home_menu_name );
	if (alc->config & CONF_FIP) {
		if (alc->seat == 1) {
		 	cmmd ("microscope control;%g",metfind);  mouse(LEFTBT);
	  		MF_cmds[mmcmd]=MF_Fipops;
	  		cmmd ("find metaphases;%g", metfind);mouse(MIDBT);
	  		MF_cmds[mmcmd]=MF_Find;
	  	} else {
		 	cmmd ("microscope control        (Not available on this seat)");
	  		cmmd ("find metaphases           (Not available on this seat)");
	  	}
	}
  	if (alc->config & CONF_MLR) {
  		cmmd ("metaphase list review;%g",metfind); mouse(LONGMB);
  		MF_cmds[mmcmd]=MF_ListRev;
  	}
	if (alc->config & CONF_FIP) {
		if (alc->seat == 1) {
			if (alc->config & CONF_TV)
				cmmd ("array capture;%g",metfind);
			else
				cmmd ("capture;%g",metfind);
			mouse(RIGHTBT);
	  		MF_cmds[mmcmd]=MF_Capture;
	  	} else {
			cmmd ("capture                   (Not available on this seat)");
	  	}
	}
	if (alc->config & CONF_TV) {
		if (alc->config & CONF_FIP) {
			cmmd ("TV image capture;%g",metfind);
		} else {
			cmmd ("image capture;%g",metfind); mouse(RIGHTBT);
		}
		MF_cmds[mmcmd]=MF_Digitise;
	}	
	if (alc->config & CONF_KTYPE) {
		cmmd ("analyse;%g%+i",chromlyse,0); mouse(LONGLB);
		cmmd ("karyotype (review);%g%+i",chromlyse,1);
	}
	cmmd ("house keeping;%g",utilities);
	cmmd ("logout"); Exit_cmd = mmcmd;
	if (alc->debug) {
		Show_alloc();
		Show_free();
	}
	while (thismu() != Exit_cmd) {
		while (testchildren())		 /* Any dead offspring ?? */
			;
		if (alc->debug) {
			Show_alloc();
			Show_free();
			fflush(debug_channel);
		}
	}
}


/*
 *	U P L I S T
 *
 */
static struct chromosome ** 
uplist() {
	return ( (struct chromosome **) 
				Calloc ( MAXOBJS,sizeof(struct chromosome *)));
}

/*
 *	U P C O N T R O L S  --  Generate and initialise kcontrol structure
 *
 */
static
upcontrols() 
{
	kcont = (struct kcontrol *) Calloc ( 1, sizeof( struct kcontrol ) );
	subinitkc(kcont);			/* Initialise slaves etc */	
	initkcontrol(kcont);
	alc->kcn = kcont;			/* make kcontrol structure available through
										   'allcontrol' system */
/*
 *	Now overide defaults with sensible values
 */
	kcont->ool = uplist();
	kcont->rol = uplist();
	kcont->bol = (struct object **) uplist();
	kcont->mol = uplist();
	kcont->kol = uplist();
/*	kcont->grol1 = NULL;		/ * Becomes keeplist in karyotyping. */
	kcont->f = &mainframe;
	kcont->ivf = (struct pframe *) Calloc ( MAXOBJS,sizeof(struct pframe));
	kcont->acl = (SMALL *)Calloc (MAXOBJS,sizeof(SMALL));
	kcont->oth = (struct othings *) Calloc (MAXOBJS, sizeof(struct othings));
	kcont->seat = alc->seat - 1;
	kcont->terminal = alc->ourterm;
	kcont->mouse = alc->mouseterm;
	kcont->ddgschannel = alc->ddgschannel;
	kcont->FIP = -1;	/* Neither TV nor FIP */
#ifdef DEBUG
	if (alc->debug) {
		printf("kcont - 0x%x seat %d\n",kcont, kcont->seat);
		printf("kcont - mouse %s terminal %s\n",kcont->mouse, kcont->terminal);
	}
#endif
}

/*
 *	F S T O R E E R R O R  --   farme store acces error handler
 *
 */
static fstoreerror()
{
	fprintf(stderr,"Framestore - incorrect address or missing\n");
	shutdown();
}
