/*
 *	K R E S E G . C  --  Re-segment from karyotype, possibly in window display.
 *
 *
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *	Based on:	reseg.c, by Denis Rutovitz (1986)
 *				Medical Research Council
 *				Edinburgh
 *
 *	Date:		6th February 1987
 *
 *
 *  Modifications
 *
 *	 4 Jul 1989		SEAJ		Added call to newpgroup.
 *	10 May 1989		dcb			resegment: Give up if classifier not initialised
 *  28 Feb 1989		SEAJ		Added extra parameter to kcopyback.
 *  21 Feb 1989		SEAJ		Added extra paramter ignore to call to dispbig.
 *	 4 Dec 1988		CAS			Change params to newclass/swapclass
 *	19 Aug 1988		SCG			Modifications for switchable chromosome classifiers
 *	13 may 1988		CAS			don't box reseg results
 *	 2 Feb 1988		CAS			remove unnecessary kcont init
 *
 */
#include <stdio.h>
#include <wstruct.h>
#include <interact.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <multip.h>
#include <display.h>
#include <class.h>
#include <allcontr.h>
#include <dispform.h>


extern struct allkontr *alc;
extern struct kcontrol *kcont;
	
static struct classifier *CLFR;
static struct kident *kid1, *kid2;
static struct pframe *framewas;

static *gotcell;   /* tells how we have approached keryfix - 
                      i.e., which cell lists are available to us .
                      Source of info is chromlyse.c, which
                      passes it here by call to setgotcell    
  
  gotcell[]    1		if available, 0 if not
  ---------
	[0]: original segmented cell
	[1]: got the processed and rotated objects
	[2]: got the karyotyped objects
                   */

static int oldmaxnumber;   /* preserves original maxn during reseg */
static donecopy;           /* tells whether 'mol' copied to 'ool' because
                              original objects not available */
static wind_style = 0;
static struct chromosome *keepobj;

kickkry1(k1,k2)
struct kident *k1,*k2;
{
	kid1 = k1;
	kid2 = k2;
}
                      
setgotcell(gc) int *gc; { 
	gotcell = gc; 
	donecopy = 0;  /* in case second cell ! */
}
	

static interactstyle = 0; /* to switch from window display as default; */

/*
 * Called from karyfix.c.
 * Parameter w_scale:  If zero:
 * If original objects in kcont->ool, then do segmentation in metaphase
 * cell display, otherwise segment the kcont->mol object in a default
 * sized window.
 * If w_scale non-zero, the segment ->ool object (if available)
 * or ->mol object (otherwise) in window, scale w_scale.
 */
resegment(w_style,w_scale)
{
	struct chromosome *nobj;
	struct intervaldomain *idom;
	int  kmmm, i;
	int temp;
	struct pframe *f,*dispbig();
	track1();

	tsize(50,50);	/* Ensure correct text size */
	if (w_style == 0 && !gotcell[0])
		w_style++;		/* force window if no original objects */	
	wind_style = w_style; /* if zero, then not wind_style */	

/*
 * note: donecopy 0 unless !gotcell[0] and we've been here before
 */
	if (! donecopy && ! gotcell[0] ) {
		for ( i = 0; i < kcont->maxnumber ; ++i) {
		    kcont->ool[i]=kcont->mol[i];
			kcont->acl[i] = ACTIVE | MASTCOPY | MSTART | MEND;
		}
		if(alc->rankcf->vhd == NULL) {
			printf("Classifier not initialised\n");
			return(-1);
		}
		
		CLFR = (struct classifier *) kcont->C;
	/*
	 * set up lists of chromosomes and objects
	 */
		clobjsetup(kcont,0,kcont->maxnumber);	
		normvec(CLFR->otype1_3_list,CLFR->ntype1_3,CLFR->vn);
		donecopy = 1;
	} else {
		CLFR = (struct classifier *) kcont->C;
		if (kcont->ool[kid1->no] == NULL)
			kcont->ool[kid1->no] = kcont->mol[kid1->no];
	}
	if ( ! wind_style ) {
		nobj = kcont->ool[kid1->no];
		/*
		 * clear screen, display original objects in original frame
		 */
		showcell();
		f = kcont->f;
    }
    else {
    	erasecursor();
    	intens(1);
		nobj = kcont->mol[kid1->no];
		keepobj = kcont->ool[kid1->no];
		kcont->ool[kid1->no] = nobj;
    	/*
    	 * Pick up frame so that window can be placed in appropriate
    	 * position.  Display in window.  Set the "phase" parameter
    	 * to "dispbig()" to zero.
    	 */
		f = kcont->ivf+kid1->no;
		f = dispbig(nobj,f,0,w_scale, 0);
		/*
		 * If using karyotyped object and plist->dispor == -1,
		 * may need to negate the MOUSESCALE.  Probably also wish
		 * to reduce MOUSESCALE to make easier drawing.......
		 * ---------------TO BE DONE------------------------
		 */
	}

	/* 
	 * plant new frame in kcont for mouse,
	 * save old frame, initialise mouse tracking,
	 * show the cursor at object centre.
	 */
	framewas = kcont->f;
	kcont->f = f;
	setmsv(track1);
	cursframe(0); /* use phase 0 cursor */
	idom = nobj->idom;
	showcursor((idom->line1+idom->lastln)/2,(idom->kol1+idom->lastkl)/2);
	oldmaxnumber = kcont->maxnumber; 
}

/* called from karyfix.c */
endreseg()
{ 
	int newobj,class,FIPsave,MAXsave;
	struct chromplist *plist;
	/*
	 * kick feature measurement in slaves
	 *
	 * If segmenting a rotated object, must (i) recover
	 * old ool[] object, (ii) set kcont->FIP flag for
	 * suitable aspect ratio in slaves.
	 */
	kcont->mstatus |= SEGBEGIN ;
	FIPsave = kcont->FIP;
	if (wind_style) {
		kcont->ool[kid1->no] = keepobj;
		kcont->FIP = DIGTV;
	}
	fmeasstart(kcont);
	kcont->mstatus |= SEGEND;	/* fmeasstart clears SEGEND !!!! */
	waitfmeas(kcont);
	kcont->FIP = FIPsave;
	kcont->mstatus = 0;

	/*
	 * if fail to segment then do nothing,
	 * otherwise copy back measured objects to master,
	 * classify with the likelihood classifier,
	 * display in appropriate locations with temporary highlight.
	 */
	if (kcont->maxnumber > oldmaxnumber) {
		kcopyback(kcont,oldmaxnumber, kcont->maxnumber);
	    clobjsetup(kcont,oldmaxnumber,kcont->maxnumber);
	    for (newobj=0; newobj<CLFR->numchr; newobj++) 
			kcont->gacl1[newobj] = ACTIVE;
	    mplikl(kcont);
		kcont->f = framewas;
		/*
		 * get the karyotype display back with all the old objects
		 * but without the new, since they have not yet gone
		 * through the "newclass"/"kdisplay" mill.
		 */
	    if ( wind_style )
	    	putbackwindow();
	    else {
	    	MAXsave = kcont->maxnumber;
	    	kcont->maxnumber = oldmaxnumber;
	    	displayall(kcont);
	    	kcont->maxnumber = MAXsave;
	    }
		unkdisplay(kid1,kcont);
        remove_class(kid1);
        re_use_if_poss(kid1,kcont);
	    
        for (newobj=oldmaxnumber; newobj<kcont->maxnumber; newobj++) {
			/* insert class assignments in property lists */
        	class = CLFR->chrocl[newobj-oldmaxnumber].newclass;
			/*
			 * manual segmentation no longer forces resultant objects
			 * to be of otype CHROMOSOME, so when karyotyping we have
			 * to ensure that this does in fact happen.
			 */
			plist = kcont->mol[newobj]->plist;
/*			plist->dispmark = TEMP_MARK_REQ; */
			plist->otype = plist->Cotype = CHROMOSOME;
	        initialorientation(newobj,kcont);
			kid1->no = newobj;
			newpgroup(kid1, class, kcont);
			newclass(kid1,class,kcont,alc->scrf->disppos);
	        ((struct chromosome **)kcont->kol)[newobj] = kcont->mol[newobj];
	        kdisplay(kid1,kcont);
	    }
	}
	else if (wind_style) {
		curscol(OVERLAY2);
		putbackwindow();
		kcont->f = framewas;
	}
	else
		displayall(kcont); 
	cursframe(1); /* use phase 1 cursor */
	showcursor(500,500);
}
