/*
 * manualstraight.c		Jim Piper		09-01-87
 *
 * C o p y r i g h t  (C)  Medical Research Council 1987
 *
 *	Intellectual Property Rights  Image Recognition Systems
 *
 * Display chromosome, draw axis with mouse, straighten, restore
 * original display.
 *
 * Modifications
 *
 *	22 Aug 1989		CAS			Remove unused colour.h include
 *	21 Feb 1989		SEAJ		added parameter ignore to call to dispbig.
 *   7 Feb 1989		SEAJ		modifications  for initialisation of nvertices=0
 *  16 Dec 1988		SEAJ		Check object returned from getstraight for NULL.
 *	14 Dec 1988		SEAJ		Added extra debugging.
 *   5 Dec 1988		SEAJ		Set kcont->SI->state to CUT | STRT when rubber-banding
 *								is switched off.
 *   6 May 1988		SEAJ		Added in invert_poly to be used when the
 *								chromosome will end up upside down.
 *  26 Apr 1988		SEAJ		Initialise arc from cursor position in straightbegin.
 *	16 Oct 1987		CAS			putbackbox -> putbackwindow (for consistancy)
 *	30 Sep 1987		CAS			Debugging
 *					CAS/SCG		don't call getstraight if no vertices
 *	24-09-87		jimp@IRS	Rubber-band stuff brought in line with
 *								segfixbis.c.  CUT, ENTRY, STRT to seg_control.h
 *	20-08-87		jimp@IRS	SI into kcont
 *	18-08-87		jimp		Rubber-banding option, manstrbegin and manstrend
 *								amalgamated since no further use.
 *	04-08-87		jimp		manstrbegin, manstrend moved from segfixbis.
 *	31-03-87		jimp		Perform the actual straightening in a slave.
 *	18-03-87		jimp		Use new track3() (segfixbis.c) for lower gearing.
 *	04-02-87		jimp		Now uses facilities from zoombox.c
 */

#include <cmdsys.h>
#include <wstruct.h>
#include <chromanal.h>
#include <interact.h>
#include <kcontrol.h>
#include <display.h>
#include <seg_control.h>
#include <allcontr.h>

extern struct allkontr *alc;
extern FILE *debug_channel;
extern struct ipoint cursor;

/*
 * function declarations
 */
extern track1(),track3();		/* Cursor tracking functions */
extern newpseg(), ck_bkpseg();
static int invert_flag;

struct chromosome *manualstraight(kcont,objnum)
struct kcontrol *kcont;
{
	struct chromosome *nobj, *getstraight();
	static struct pframe *strframe;
	struct pframe *ofr = kcont->ivf + objnum;
	struct chromosome *obj = kcont->mol[objnum];
	int mencom;
	extern rubber_track2();
	register struct polygondomain *pdom;

	if (alc->debug)
		fprintf(debug_channel, "manualstraight: before straightbegin\n");
	straightbegin(obj,ofr,&strframe,kcont);
	pdom = (struct polygondomain *)kcont->SI->polyobj->idom;
	if (kcont->SI->rubber_straight) {
		pdom->nvertices = 0;
		setmsv(rubber_track2);
	}
	else
		setmsv(track1);
	newmenu("Manually straighten chromosome - draw axis\n");
	if (kcont->SI->rubber_straight) {
		cmmd("start/bend");mouse(LEFTBT);
	}
	else {
		cmmd("start;%g%",newpseg);mouse(LEFTBT);
	}
	cmmd("back up;%g",ck_bkpseg);mouse(RIGHTBT);
	cmmd("o.k. - straighten");mouse(MIDBT);
	cmmd("forget it");mouse(LONGLB);
	do {
		mencom = thismu();
		switch (mencom) {
			case 0:
			case 4:		/* forget it and return same */
				nobj = obj;
				break;
			case 1:
				if (kcont->SI->rubber_straight) {
					pdom->nvertices++;
					pdom->vtx[pdom->nvertices].vtX = kcont->SI->arc.k1 = kcont->SI->arc.k2;
					pdom->vtx[pdom->nvertices].vtY = kcont->SI->arc.l1 = kcont->SI->arc.l2;
					kcont->SI->state = CUT | STRT;
				} else {
					kcont->SI->state = CUT | STRT;
					setmsv(track3); 
				}
				break;
			case 3:
				if (kcont->SI->rubber_straight) {
					if (kcont->SI->arc.k2 != kcont->SI->arc.k1 || kcont->SI->arc.l2 != kcont->SI->arc.l1)
						pdom->nvertices++;
				}
				if (pdom->nvertices > 1) {
					nobj = getstraight(kcont,objnum,kcont->SI->polyobj,strframe);
					if (nobj == NULL)
						nobj = obj;
				} else
					nobj = obj;
				break;
	    }
	} while (mencom == 1 || mencom == 2);
	straightend(kcont);
	return(nobj);
}


/*
 *	S T R A I G H T B E G I N  --  
 *
 */
static
straightbegin(obj,ofr,f,kcont)
struct chromosome *obj;
struct pframe *ofr, **f;
struct kcontrol *kcont;
{
	struct pframe *dispbig();
	register struct intervaldomain *idom = obj->idom;
	struct object *makemain();
	struct polygondomain *pdom, *makepolydmn();
	
	invert_flag = 0;
	/* display in a window */
	*f = dispbig(obj,ofr,0,16, 0);
	/* plant new frame in kcont for mouse, save old frame */
	kcont->SI->framewas = kcont->f;
	kcont->f = *f;
	cursframe(0);
 	if ( (*f)->ix > 0 ) {
		showcursor(idom->lastln,(idom->kol1+idom->lastkl)/2);
		invert_flag = 1;
	 } else
		showcursor(idom->line1,(idom->kol1+idom->lastkl)/2);
	pdom = makepolydmn(1,NULL,0,IPOLSIZE,1);
	kcont->SI->polyobj = makemain(10,pdom,NULL,NULL,NULL);
	kcont->SI->arc.k2 = pdom->vtx[0].vtX = cursor.k;
	kcont->SI->arc.l2 = pdom->vtx[0].vtY = cursor.l;
}

/*
 *	S T R A I G H T E N D  --  
 *
 */
static
straightend(kcont)
struct kcontrol *kcont;
{
	kcont->f = kcont->SI->framewas;
	cursframe(1);
	if (kcont->SI->polyobj) {
		freeobj(kcont->SI->polyobj);
		kcont->SI->polyobj = NULL;
	}
	putbackwindow();
	quitmenu();
}


/*
 *  G E T S T R A I G H T
 *
 */
static
struct chromosome *getstraight(kcont,objnum,poly,f)
struct kcontrol *kcont;
struct object *poly;
struct pframe *f;
{
	struct chromosome *strobj, *sing_mp_k_serv();
	struct chromosome *turnchrom();
	struct object *mpol, *makemain();
	struct polygondomain *pdom, *npdom, *mpdom, *uspaxis8();
	/*
	 * delete polygon display and cursor
	 */
	intens(0);
	colour(OVERLAY1);	/* from track3 specification */
	picframe(poly,f);
	setmsv(0);
	erasecursor();
	/*
	 * This polygon will be a bit rough and will have arcs of widely
	 * differing lengths.  So the treatment is
	 *	(1) Multiply by 8 (rather than use floating point)
	 *	(2) Unit space
	 *	(3) Smooth.  Need to use multiple calls to "polysmooth(pdom,1)"
	 *		since e.g. "polysmooth(pdom,3)" would overflow short integers.
	 *	(4) Unit space again
	 */
	pdom = (struct polygondomain *) poly->idom;
	/* IF (the chromosome was inverted AND the polygon was drawn top to bottom)
	 * OR (if the chromosome was not inverted and the polygon was drawn bottom
	 * to top) THEN invert the polygon to avoid the chromosome being produced 
	 * upsidedown
	 */
	if (invert_flag) {
		if (pdom->vtx[0].vtY > pdom->vtx[pdom->nvertices-1].vtY)
			invert_poly(pdom);
	} else {
		if (pdom->vtx[0].vtY > pdom->vtx[pdom->nvertices-1].vtY)
			invert_poly(pdom);
	}
	if (alc->debug > 2) {
		fprintf(debug_channel, "manualstraight: poly pdom:\n");
		showpoly(pdom);
	}
	polymult(pdom,8);
	if (alc->debug > 2) {
		fprintf(debug_channel, "manualstraight: after polymult poly pdom:\n");
		showpoly(pdom);
	}
	npdom = uspaxis8(pdom);
	if (alc->debug > 2) {
		fprintf(debug_channel, "manualstraight:after uspaxis8 poly npdom:\n");
		showpoly(npdom);
	}
	polysmooth(npdom,1);
	polysmooth(npdom,1);
	polysmooth(npdom,1);
	if (alc->debug > 2) {
		fprintf(debug_channel, "manualstraight: after 3 * polysmooth poly npdom:\n");
		showpoly(npdom);
	}
	mpdom = uspaxis8(npdom);
	if (alc->debug > 2) {
		fprintf(debug_channel, "manualstraight: after uspaxis8 poly mpdom:\n");
		showpoly(mpdom);
	}
	mpol = makemain(10,mpdom,NULL,NULL,NULL);
	f->scale /= 8;
	f->ox *= 8;
	f->oy *= 8;
	intens(1);
	picframe(mpol,f);
	kcont->gool1 = (char *) mpol;
	strobj = sing_mp_k_serv(kcont,K_STR_POL,objnum);
	intens(0);
	picframe(mpol,f);
	freeobj(mpol);
	Free(npdom);
	return(strobj);
}

/*
 * I N V E R T _ P O L Y -- Turn polygon the other way up
 *
 */
 
invert_poly (pdom)
struct polygondomain *pdom;
{
	int temp, i, n;
	
	n = pdom->nvertices;
	for (i = 0; i < n/2; i++) {
		temp = pdom->vtx[i].vtX;
		pdom->vtx[i].vtX = pdom->vtx[n-i-1].vtX;
		pdom->vtx[n-i-1].vtX = temp;

		temp = pdom->vtx[i].vtY;
		pdom->vtx[i].vtY = pdom->vtx[n-i-1].vtY;
		pdom->vtx[n-i-1].vtY = temp;
	}
}

/*
 * S H O W P O L Y
 *
 */
showpoly(poly)
struct polygondomain *poly;
{
	int i;
	
	fprintf(debug_channel, "nverts = %d\n", poly->nvertices);
	for (i = 0; i < poly->nvertices; i++)
		fprintf(debug_channel, "vertex %d: X %d Y %d\n", i, poly->vtx[i].vtX,
											poly->vtx[i].vtY);
}
