/*
 *	O D _ A R C H I V E --	User interface to Optical Disk archiving from
 *							application
 *
 * 	Written: David C Burgess
 *			Image Recognition Systems
 *			720 Birchwood Boulevard
 *			Birchwood Science Park
 *			Warrington
 *			Cheshire
 *			WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 * 	Date:	26th September 1989
 *
 * 	Modifications
 *  7     Jun 1990 ih   Added check for process exit status to allow abort.
 *  5     Jun 1990 ih   Generalised getfilelist to take a pointer to a decision
 *                      procedure and a parameter so that cyto file lists can be
 *                      generated by e.g. wildcard name matches as well as date
 *                      Also fixed arch_dir to return 000000 when dt->tm_year<0
 *  16    May 1990 ih   Fixed prompt bugs in retrieve by date (bad use of strncmp)
 *  5     Apr 1990 ih/dcb Added a check on retrieve case.
 *  19-22 Feb 1990 ih   Added code to archive/retrieve and show by case.
 *                      Added new structure, dirlist and function getdirlist
 *                      to produce a list of archive directories and argv
 *                      lists and simplified getfilelist. (i.e. generalised)
 *	15 Dec 1989	dcb		Add delete function to single case archive
 *	14 Dec 1989	dcb		Reverse archive date logic to archive files earlier
 *						than date supplied, change fn. later to earlier
 */
#include <fileheaders.h>
#include <cmdsys.h>
#include <kyy.h>
#include <modes.h>
#include <errno.h>
#include <direct.h>
#include <dir.h>
#include <time.h>
#include <signal.h>

#ifdef TEST
#define ODMOD		"w0"						/* OD module name */
#define ODDEV		"/w0"						/* OD device */
#define ARCHIVE		"/w0/archive"				/* Archive directory */
#define ARCH_METHOD	"/w0/archive/arch_method"	/* Archive method file */
#define ARCH_NULL	"/w0/archive/000000"		/* Archive file for no date */
#else
#define ODMOD		"wr0"						/* OD module name */
#define ODDEV		"/wr0"						/* OD device */
#define ARCHIVE		"/wr0/archive"				/* Archive directory */
#define ARCH_METHOD	"/wr0/archive/arch_method"	/* Archive method file */
#define ARCH_NULL	"/wr0/archive/000000"		/* Archive file for no date */
#endif
#define COPYSW1		"-b=100"					/* Copy switch 1 */
#define COPYSW2		"-rw="						/* Copy switch 2 */
#define TRUE	(1)
#define FALSE	(0)
#define ABS(x) (((x)>0) ? (x) : -(x))
#define AM_DATE	1
#define	AM_HASH	2
#define	AM_CASE	3
#define ESC			'\033'
#define STARTLINE	5
#define SEPCHAR		":"
#define WR_FIRST	1
#define MAX_DIR	16/* max characters in dir name*/
#define MAX_DIR_LEN MAX_DIR+1					/* max characters in dir string*/

/*
 *	Externals
 */
extern int errno;
extern FILE *debug_channel;
extern char	workingfile[]; /* Used to display file being processed */
extern int iscyto();
extern int setcase();
extern int show_dir();
extern int os9forkc();
extern char **environ;

/*
 *	Forward declarations
 */
int od_arch(), od_retrieve(), od_show(); od_init();
int convert_date();
int earlier(),match();
char *strsave(), *strnsave(), *Calloc(), *pr_date(), *arch_dir();
struct filelist *Freefilelist();
struct dirlist *getdirlist();
static int quitfn();
void parse_case();
void Freedirlist();
char **getargvlist();

struct filelist {
	char	*name;			/* Complete file name */
	char	*casename;		/* Just the case name */
	char	*date;			/* Last modify date */
	struct filelist	*next;	/* Next file in list */
};

struct dirlist {
	int nargs;				/* Number of files in this archive */
	struct filelist *files; /* List of files */
	char dir[MAX_DIR_LEN];		/* Name of archive */
	struct dirlist *next;   /* Next archive in list */
};

static struct {
	int pos[MAX_DIR];
	int tot;
} case_characters;

/*
 *	O D _ A R C H I V E _ M E N U -- Main od archive menu
 *
 */
od_archive_menu()
{
	char	workbuf[40], csn[CASELENGTH+1];
	static char datestr[80]; /* Better not overrun ! */
	static char disp_date[15];	/* Display date string */
	int	(*linkerbug)();
	int Quit_cmd;
	struct tm dt;

	workbuf[0] = '\0';
	strcpy(disp_date, "UNSET");
	dt.tm_year = -1; dt.tm_mon = 1; dt.tm_mday = 1;
	dt.tm_hour = 0; dt.tm_min = 0;

	strcpy(csn,alc->nameids.case);
	newmenu ("OD archive operations");
	cmmd ("set archive/retrieve date;:k40%-+d%?+S%-s%f",
		&dt, datestr, disp_date, convert_date);
	linkerbug = setcase;
	cmmd ("case ID;:k40%+s%-17s%f", csn, alc->nameids.case, linkerbug);
	linkerbug = show_dir;
	cmmd ("show working directory;:k40%-+s%f",alc->dir,linkerbug); mouse(LONGMB);
	cmmd ("show archive directory;%-+d%f", &dt, od_show); mouse(LONGRB);
	cmmd ("archive case(s);%-+d%+i%f", &dt, 0, od_arch); mouse(MIDBT);
	cmmd ("archive and delete case(s);%-+d%+i%f", &dt, 1, od_arch); mouse(RIGHTBT);
	cmmd ("retrieve case;%-+d%-+d%f", &dt, csn, od_retrieve); mouse(LEFTBT);
	if ((getuid() & 0xFFFF0000) == 0) {		/* Only allow if super user */
		cmmd ("initialise blank disk;%f", od_init);
	}
	Quit_cmd = cmmd ("exit"); mouse(LONGLB);

	while (thismu() && (nncmd != Quit_cmd)) ;
#ifndef TEST
			wrcache(ODDEV, 0);	/* Disable cacheing */
#endif
}

/*
 *	O D _ A R C H -- Archive cases to OD
 */
od_arch(dt, delflag)
struct tm *dt;
int delflag;
{
	int		pid, stat, nargs, arch_method;
	char	cur_dir[100], ad[100], swch[100], buffer[100], dir[MAX_DIR_LEN];
	char 	**argv;
	struct filelist *ll;
	struct dirlist *dirlist, *dl;
	int		ef;
	FILE	*fopen();
	int (*func)();
	void *parm;
	char csn[MAX_DIR_LEN];

	if (!odinstalled()) return(0);
/*
 *	Get current dir so we can return at end - We need to work from
 *	the directory with the files in so they are saved pathless
 */
	if (getwd(cur_dir,100) == 0) cur_dir[0] = '\0';
	else if (alc->debug)
		fprintf(stderr,"Directory was %s\n",cur_dir);
	if (alc->debug) fprintf(debug_channel, "Changing to directory %s\n", alc->dir);
	chdir(alc->dir);

	checkdir(ARCHIVE);
	if ((arch_method = get_arch_method()) < 0) return(0);
/*
 *	This section deals with archiving when no date is given
 */
	if (dt->tm_year < 0) { /* Date UNSET */
		if (strncmp(alc->nameids.case,"UNSET",5) == 0) {
			fprintf(output, "Must supply case name or archive date\n");
			return(0);
		}
		switch (arch_method) {
 			case AM_DATE:
 				sprintf(csn,"%s",alc->nameids.case);
 				parm=csn;
 				break;
 			case AM_CASE:
 				sprintf(csn,"%s*",alc->nameids.case);
 				parm=csn;
 				break;
 		}
		func=match;
	}
	else {
		parm=dt;
		func=earlier;
 	}
/*
 *	This part archives to each directory
 */
	if ((nargs = getfilelist(parm, func, &ll)) == 0) {
		fprintf(output, "Can't create file list\n");
		ll = Freefilelist(ll);	/* Free save linked list */
		if (cur_dir[0])	chdir(cur_dir);
		return(0);
	}
	printfile(ll);	/* Pretty print of files to archive */
	if (!check("OK to archive these files ? (M): ")) {
		ll = Freefilelist(ll);	/* Free save linked list */
		if (cur_dir[0])	chdir(cur_dir); /* Change back to old dir */
		return(0);
	}
	switch (arch_method) {
		case AM_DATE:
			dirlist = getdirlist(ll,NULL);
			strcpy(dirlist->dir,arch_dir(dt)); /* Archive directory */
			dirlist->nargs=nargs;
			break;
		case AM_CASE:
			dirlist = getdirlist(ll,parse_case);
			break;
		default:
			return(0); /*should never get here*/
	}
	stat=0;
	for (dl=dirlist;dl&&((stat&0xffff)==0);dl=dl->next) {
		if (*dl->dir == '\0') {
			fprintf(output,"Can't construct archive for casename %s\n",dl->files->casename);
			get_archive(dl->dir,dl->files->casename);
			setdisplay(BUST);
			clearlns(STARTLINE, ENDLINE);
			screengo(STARTLINE, 1);
			fflush(stdout);
		}
		sprintf(ad, "%s/%s", ARCHIVE, dl->dir);
		argv = getargvlist(dl,3);
		argv[0] = "copy";
		argv[1] = COPYSW1;						/* Fixed switches */
		sprintf(swch, "%s%s", COPYSW2, ad);			/* Variable switches */
		argv[2] = swch;
		if (alc->debug)
			fprintf(debug_channel,"Archive directory = %s, switches = %s %s\n",
					ad, COPYSW1, swch);
		printf("\n");
		checkdir(ad);
/*
 *	Fork copy
 */
		if ((pid = os9exec(os9forkc, argv[0], argv, environ, 0, 0, 3)) < 0) {
			fprintf(stderr, "Can't fork %s\n", argv[0]);
		} else {
			while (wait(&stat) != pid) ;
		}
		Free(argv);
	}
/*
 *	Now test for, and optionally delete
 */
 	if (delflag&&((stat&0xffff)==0)) {
		if (check("OK to delete these files ? (M): ")) {
			for (dl=dirlist;dl&&((stat&0xffff)==0);dl=dl->next) {
				argv = getargvlist(dl,1);
				argv[0] = "del";	/* Insert command in free slot */
				if ((pid = os9exec(os9forkc, argv[0], argv, environ, 0, 0, 3)) < 0) {
					fprintf(stderr, "Can't fork %s\n", argv[0]);
				} 
				else {
					printf("Deleting files ... please wait"); fflush(stdout);
					while (wait(&stat) != pid) ;
					printf("\ndone\n");
				}
			}
		}
		Free(argv);
 	}
	Freedirlist(dirlist);			/* Free save linked list */
	if (cur_dir[0])	chdir(cur_dir); /* Change back to old dir */
	testchildren();					/* Check for dead offspring.. */
} /* end od_arch */

/*
 *	O D _ R E T R I E V E -- Retrieve case
 *
 */
od_retrieve(dt, csn)
struct tm *dt; /* retrieve date */
char *csn;
{
	int method;

	if (!odinstalled()) return(0);

	if (alc->debug) fprintf(debug_channel, "Case name: %s\n", csn);
	if (alc->debug) fprintf(debug_channel, "Case name after UNSET check: %s\n", csn);

	if ((method = get_arch_method()) < 0) return(0);
	switch (method) {
		case AM_DATE:
			retrieve_by_date(dt, csn);
			break;
		case AM_CASE:
			retrieve_by_case(csn);
			break;
		case AM_HASH:
			printf("This archive method is not yet supported\n");
			break;
		default:
			printf("Unknown archive method\n");
			break;
	}
}

/*
 *	 R E T R I E V E _ B Y _ D A T E -- Retrieve cases using the archive date
 */
retrieve_by_date(dt, csn)
struct tm *dt;
char *csn;
{
	char ad[100];
	char *arch;
	int cancel=FALSE;

	if (dt->tm_year < 0) {
		if (!check("Warning: All OD archives will be searched, continue ? (M): "))
			cancel = TRUE;
		else if (strncmp(csn,"UNSET",5)==0) {
			if (!check("Warning: This will retrieve entire optical disk!! continue ? (M): "))
				cancel = TRUE;
			else if (!check("Are you really sure ? (M): "))
				cancel = TRUE;
		}
	} else if (strncmp(csn,"UNSET",5)==0) {
		if (!check("Warning: This will retrieve all cases archived on this date, continue ? (M): "))
			cancel = TRUE;
	}
	if (cancel) {
		printf("\nRetrieve cancelled\n");
		return(0);
	}
	if (dt->tm_year < 0) { 
		arch = "*";
	}
	else {
		arch = arch_dir(dt);
	}
	csn = ((strncmp(csn,"UNSET",5) == 0) ? "*" : csn);
	setdisplay(BUST);
	clearlns(STARTLINE, ENDLINE);
	screengo(STARTLINE, 1);
	fflush(stdout);                       
	sprintf(ad, "dir %s/%s/%s\\$*\\$*\n",ARCHIVE, arch, csn);
	sysexec("chs", ad);
	if (check("OK to retrieve these files ? (M): ")) {
		sprintf(ad, "copy \\-r %s/%s/%s\\$*\\$* \\-w=%s\n",
			ARCHIVE, arch, csn, alc->dir);

		if (alc->debug) fprintf(debug_channel, "Copy cmd: %s\n", ad);
		sysexec("chs", ad);
	}
} /* end retrieve_by_date */

/*
 * R E T R I E V E _ B Y _ C A S E -- retrieve by case name
 */
retrieve_by_case(csn)
char *csn;
{
	char ad[100],dircsn[MAX_DIR_LEN];
	char *arch;
	char cancel = FALSE;
	
	if (strncmp(csn,"UNSET",5) == 0) {
		if (!check("Warning: This will retrieve entire optical disk!! continue ? (M): "))
			cancel = TRUE;
		else if (!check("Are you really sure ? (M): "))
			cancel = TRUE;
	}
	else {
		parse_case(dircsn,csn);
		if (*dircsn == '\0') {
			get_archive(dircsn,csn);
		}
	}
	if (cancel) {
		printf("\nRetrieve cancelled\n");
		return(0);
	}
	arch = ((strncmp(csn,"UNSET",5) == 0) ? "*" : dircsn);
	csn = ((strncmp(csn,"UNSET",5) == 0) ? "*" : csn); /* exchange * .. */
	setdisplay(BUST);
	clearlns(STARTLINE, ENDLINE);
	screengo(STARTLINE, 1);
	fflush(stdout);                       
	sprintf(ad, "dir %s/%s/%s\\$*\\$*\n",ARCHIVE, arch, csn); /* ..with here ?*/
	sysexec("chs", ad);
	if (check("OK to retrieve these files ? (M): ")) {
		sprintf(ad, "copy \\-r %s/%s/%s\\$*\\$* \\-w=%s\n",
			ARCHIVE, arch, csn, alc->dir);

		if (alc->debug) fprintf(debug_channel, "Copy cmd: %s\n", ad);
		sysexec("chs", ad);
	}
}
	

/*
 *	O D _ S H O W - Show archive sub-directories
 */
od_show(dt)
struct tm *dt;
{
	int method;
	char *csn;

	csn = alc->nameids.case;
	if (!odinstalled()) return(0);
	clearlns(STARTLINE, ENDLINE);
	screengo(STARTLINE, 1);
	fflush(stdout);
	if ((method = get_arch_method()) < 0) return(0);
	downmenu();
	switch (method) {
		case AM_DATE:
			show_by_date(dt);
			break;
		case AM_CASE:
			show_by_case(csn);
			break;
		case AM_HASH:
			printf("This archive method is not yet supported\n");
			break;
		default:
			printf("Unknown archive method\n");
			break;
	}
	upmenu();
}

/*
 *  S H O W _ B Y _ D A T E
 */
show_by_date(dt,csn)
struct tm *dt;
{
	char buf[50];

	if (dt->tm_year < 0) {					/* Date UNSET */
		sprintf(buf, "dir -d %s", ARCHIVE);	/* Show archive directories */
		system(buf);
		check("Type <Return> or use (M) to continue ");
	} else {								/* Date set */
		sprintf(buf, "%s/%s", ARCHIVE, arch_dir(dt)); /* Show contents */
		show_dir(buf);
	}
	printf("\n");
	setdisplay(BUST);
}

/*
 *  S H O W _ B Y _ C A S E
 */
show_by_case(csn)
char *csn;
{
	char buf[50];
	char dir[MAX_DIR_LEN];
	
	if (strncmp(csn,"UNSET",5) == 0) {					/* Case UNSET */
		sprintf(buf, "dir -d %s", ARCHIVE);	/* Show archive directories */
		system(buf);
		check("Type <Return> or use (M) to continue ");
	} else {							/* Case set */
		parse_case(dir,csn);
		if (*dir == '\0') {
			get_archive(dir,csn);
		}
		sprintf(buf, "%s/%s", ARCHIVE, dir); /* Show contents */
		show_dir(buf);
	}
	printf("\n");
	setdisplay(BUST);
}	 

/*
 *  G E T _ A R C H I V E -- Get an archive directory name manually
 */
get_archive(dir,casen)
char *dir,*casen;
{
	int Quit_cmd;
	
	downmenu();
	newmenu ("OD Archive Directory Name");
	cmmd("archive directory for case ;:k40%16S", dir);
	Quit_cmd = cmmd ("exit"); mouse(LONGLB);
	fprintf(output,"Can't construct directory for case %s\n",casen);
	while (thismu() && (nncmd != Quit_cmd)) ;
	if (*dir == '\0') {
		strcpy(dir,"000000");
		fprintf(output,"No directory specified, set to default: %s\n",dir);
	}
	upmenu();
	setdisplay(BUST);
}

/*
 *	O D _ I N I T -- Initialise blank optical disk
 */
od_init()
{
	char buf[35+LABLENGTH];
	if (!odinstalled()) return(0);

	fprintf(output, "Initialising blank disk, please wait\n");
	sprintf(buf, "wrinit501 %s@", ODDEV);
	if (alc->debug) 
		fprintf(debug_channel,"od_init -- about to execute %s\n",buf);
#ifdef TEST
	if (ssystem(buf) != 0) {
#else
	if (system(buf) != 0) {
#endif
		fprintf(stderr, "Error (%d) initialising optical disk\n", errno);
		return(-1);
	}
	sprintf(buf, "wrformat %s -q -g=16 -d=\"%s Archive\"",
		ODDEV, alc->sitename);
	if (alc->debug) 
		fprintf(debug_channel,"od_init -- about to execute %s\n",buf);
#ifdef TEST
	if (ssystem(buf) != 0) {
#else
	if (system(buf) != 0) {
#endif
		fprintf(stderr, "Error (%d) formatting optical disk\n", errno);
		return(-1);
	}
	fprintf(output, "Optical disk is now ready for use\n");
} /* end od_init */

#ifdef TEST
ssystem(str)
{
	printf("I'd give %s to system\n", str);
	return(0);
}
#endif

/*
 *	O D I N S T A L L E D
 */
static int
odinstalled()
{
	static int checked=FALSE;
	char	*ptr;

	if (!checked) {
		if ((ptr = (char *) modlink(ODMOD, 0)) == (char *) -1) {
			screengo(ENDLINE,2);
			fprintf(output,"OD software not installed\n");
			return(0);
		}
		munlink(ptr);
		checked = TRUE;
#ifndef TEST
		wrmode(ODDEV, SEPCHAR, WR_FIRST);	/* Set correct mode */
#endif
	}
	wrcache(ODDEV, 0);					/* disable cacheing to clear cache */
	wrcache(ODDEV, alc->cachesize);		/* and re-enable */
	return(1);
}

/*
 *	C O N V E R T _ D A T E -- Convert string passed to date structure values
 *	also places a standard format string in disp_str
 */
convert_date(dt, str, disp_str)
struct tm *dt;
char *str, *disp_str;
{
	int wn = FALSE;
/*
 *	Set defaults
 */
 	dt->tm_year = 0;
	dt->tm_mon  = 1;
	dt->tm_mday = 1;
	dt->tm_min  = 0;
	dt->tm_sec  = 0;
/*
 *	Assign from string
 */
	if (alc->debug) fprintf(debug_channel, "convert date %s\n", str);
	if (*str == '\0') {
		if (alc->debug) printf("Unset date\n");
		dt->tm_year = -1;
		strcpy(disp_str, "UNSET");
		return(0);
	}
	dt->tm_year = atoi(str);
	*str++ = '\0';	/* Used */
	if (skip_to_nxt_num(&str)) dt->tm_mon  = atoi(str);
	if (skip_to_nxt_num(&str)) dt->tm_mday = atoi(str);
	if (skip_to_nxt_num(&str)) dt->tm_hour = atoi(str);
	if (skip_to_nxt_num(&str)) dt->tm_min  = atoi(str);
/*
 *	Check ranges
 */
	dt->tm_year = ABS(dt->tm_year);
	while (dt->tm_year > 99999) dt->tm_year -= 100000;
	while (dt->tm_year > 99) dt->tm_year -= 100;
	if ((dt->tm_mon > 12) || (dt->tm_mon < 0)) { dt->tm_mon = 1; wn = TRUE; }
	if ((dt->tm_mday > 31) || (dt->tm_mday < 0)) { dt->tm_mday = 1; wn = TRUE; }
	if ((dt->tm_hour > 23) || (dt->tm_hour < 0)) { dt->tm_hour = 0; wn = TRUE; }
	if ((dt->tm_min > 59) || (dt->tm_min < 0)) { dt->tm_min = 0; wn = TRUE; }
	if (wn) fprintf(output, "Warning: Invalid date format\n");
/*
 *	Create display string
 */
 	sprintf(disp_str, "%02d/%02d/%02d %02d:%02d",
		dt->tm_year, dt->tm_mon, dt->tm_mday, dt->tm_hour, dt->tm_min);
	return(0);

} /* end convert_date */

/*
 *	S K I P _ T O _ N X T _ N U M -- Skip to next numeric value in string
 *	and adjust ptr position accordingly
 *	Returns TRUE if there is still some string left, or
 *	FALSE if the end of string (NULL) is detected
 */
skip_to_nxt_num(str)
char **str;
{
	while ((**str >= '0') && (**str <= '9')) {
		if (**str == '\0') return(FALSE);
		(*str)++;
	}
	while ((**str < '0') || (**str > '9')) {
		if (**str == '\0') return(FALSE);
		(*str)++;
	}
	return(TRUE);
} /* end skip_to_nxt_num */

/*
 *	S T R S A V E --	Copy string to a safe place
 *						Return pointer to it if successful, else NULL
 */
char *
strsave(str)
char *str;
{
	char *p, *Malloc();

	if (( p= Malloc(strlen(str) + 1)) != NULL) /* get space */
		strcpy(p, str);	/* copy it there */
	return(p);
}

/*
 *	S T R N S A V E --	Copy n characters somewhere safe, and place NULL
 *						at the end of the string
 */
char *
strnsave(str, n)
char *str;
int n;
{
	char *p, *Malloc();

	if (( p = Malloc(n + 1)) != NULL) {/* get space */
		strncpy(p, str, n);	/* copy it there */
		*(p+n) = '\0';
	}
	return(p);
}

/*
 *	A R C H _ D I R -- Return the archive directory name, given the archive date
 */
char *
arch_dir(dt)
struct tm *dt;
{
	static datestr[15];
	if (dt->tm_year<0) {
		sprintf(datestr,"%s","000000");
	}
	else {
		sprintf(datestr, "%02d%02d%02d", dt->tm_year, dt->tm_mon, dt->tm_mday);
	}
	return(datestr);
}

/*
 * P A R S E _ C A S E -- Parse the case name according to 'case_characters'
 */
void
parse_case(dirname,casename)
char *casename, *dirname;
{
	int i,j,k;
	char casen[MAX_DIR_LEN];
	
	if (alc->debug) {
		fprintf(debug_channel,"parse_case - casename: %s\n",casename);
	}
	for (i=0,j=0;i<case_characters.tot;i++) {
		if ((k = case_characters.pos[i]) < strlen(casename)) {
			dirname[j] = casename[k];
			j++;
		}
		else {
			if (alc->debug) {
				fprintf(debug_channel,"Character position %d rejected from casename %s\n",k,casename);
			}
		}
	}
	dirname[j] = '\0';
	if (alc->debug) {
		fprintf(debug_channel,"parse_case - dirname: %s\n",dirname);
	}
}

/*
 *	E A R L I E R -- Check two dates, formats are different
 *	Returns:	TRUE if date supplied as 1st param (struct tm) is earlier
 *				(or equal) to that in second param (ll->date)
 *				FALSE if if 1st param is later
 *	Call is similar to:
 *		if (earlier(ll, dt))
 *	NOTE:	Does not cope with wrap round at end of century.
 */
earlier(ll, date2)
struct filelist *ll;
struct tm *date2;
{
	char *date1;	/*5 byte array */
	
	date1=ll->date;
	if ((int)(*date1) > date2->tm_year) return(FALSE);
	if ((int)(*date1++) < date2->tm_year) return(TRUE);
	if ((int)(*date1) > date2->tm_mon) return(FALSE);
	if ((int)(*date1++) < date2->tm_mon) return(TRUE);
	if ((int)(*date1) > date2->tm_mday) return(FALSE);
	if ((int)(*date1++) < date2->tm_mday) return(TRUE);
	if ((int)(*date1) > date2->tm_hour) return(FALSE);
	if ((int)(*date1++) < date2->tm_hour) return(TRUE);
	if ((int)(*date1) > date2->tm_min) return(FALSE);
	if ((int)(*date1++) < date2->tm_min) return(TRUE);
	return(TRUE);	/* Exactly equal to */
} /* end earlier */

/*
 * M A T C H -- Decide if file casename field in ll->casename matches
 *              the string csn, using wildcarding. At present only a postfix
 *              '*' is checked for in matchstr
 */
match(ll,matchstr)
struct filelist *ll;
char *matchstr;
{
	char *ptr = ll->casename;

	while ((*ptr != 0)&&(*matchstr != 0)) {
		if (*ptr++ != *matchstr) {
			if (*matchstr != '*')
				return 0;
			else
				return 1;
		}
		matchstr++;
	}
	if ((*matchstr==0)&&(*ptr==0))
		return 1;
	else if (*matchstr=='*')
		return 1;
	else
		return 0;
}
/*
 *	F R E E F I L E L I S T -- Free elements of filelist linked list
 */
struct filelist *
Freefilelist(list)
struct filelist *list;
{
	struct filelist *linkptr, *free_entry;

	linkptr = list;
	while (linkptr) {
		if (alc->debug) {
			fprintf(debug_channel,"about to free entry at 0x%x\n",linkptr);
			fprintf(debug_channel,"contains %s\n",linkptr->name);
		}
		free_entry = linkptr;
		linkptr = linkptr->next;
		if (alc->debug) fprintf(debug_channel,"freeing name entry at 0x%x\n",free_entry->name);
		Free(free_entry->name);
		if (alc->debug) fprintf(debug_channel,"freeing case name entry at 0x%x\n",free_entry->casename);
		Free(free_entry->casename);
		if (alc->debug) fprintf(debug_channel,"freeing date entry at 0x%x\n",free_entry->date);
		Free(free_entry->date);
		if (alc->debug) fprintf(debug_channel,"freeing list entry at 0x%x\n",free_entry);		
		Free(free_entry);
	}
	return(NULL);
} /* end Freefilelist */

/*
 *	G E T F I L E L I S T -- Construct a link list of files to archive
 *
 *	Entry:	dt, Archive before date, the list of files returned, will have at
 *			least one member of the case with a date earlier than this.
 *			link_list, Parameter for returning linked list of files
 *	Exit:	A pointer to a linked list of filelist structures is returned
 *			as "link_list". The argv list points at strings within link_list
 *			The caller must ensure memory is freed correctly.
 *			Freefilelist(link_list); will free the filelist,
 *			Free(argv) then frees the array.
 */
int
getfilelist(parm, dec_proc, link_list)
void *parm;
int (*dec_proc)();
struct filelist **link_list;
{
	DIR		*dirp;
	struct	direct *entry;
	struct	fildes desc;
	struct	filelist *ll, *rl, **fl, **sl, **rm, *nextll, *nextrl, *lastrl;
	struct	filelist *list=NULL, *savelist=NULL, *remainder=NULL;
	int		typ, nargs, ef;
	struct cyto_file cur_file;
/*	static char arch_method[] = "DATE";*/

	dirp = opendir(".");
	readdir(dirp);						/* Skip '.' and '..' */
	readdir(dirp);

	fl = &list;
	nargs = 0;
	while ((entry = readdir(dirp)) != NULL) {
		if (typ = iscyto(entry->d_name,&cur_file)) {
			if ((ef = open(entry->d_name, S_IREAD)) < 0) { /* Can't read */
				if ((ef = open(entry->d_name, S_IFDIR)) < 0) { /* Directory ? */
					screengo(ENDLINE,0);
					fprintf(output,"File %s %s is read protected\007\n",
							cur_file.case,makefilestring(&cur_file,workingfile));
					workingfile[0] = '\0';
					if (alc->debug)
						fprintf(stderr, "Warning: Can't read %s (error %d)\n",
							entry->d_name, errno);
						fprintf(debug_channel, "Can't read %s (error %d)\n",
							entry->d_name, errno);
				} else {
					close(ef);			/* Close, now finished with */
				}
			} else {
/* 				printf("%s\n", entry->d_name);*/
				_gs_gfd(ef, &desc, sizeof(struct fildes));	/* Get the file info */
				*fl = (struct filelist *)Calloc(1, sizeof(struct filelist));
				(*fl)->name = strsave(entry->d_name);
				(*fl)->casename = strsave(cur_file.case);
				(*fl)->date = strnsave(desc.fd_date, 5);
				fl = &((*fl)->next);
				close(ef);			/* Close, now finished with */
			}
		} /* end if iscyto... */
	} /* end while */
/*
 *	Print list to see if correct
 */
	if (alc->debug) pr_list("Raw file list\n", list);
/*
 *	Now split into two lists, one consisting of files which satisfy the decision
 *	procedure (the savelist), and the other consisting of files which don't
 *	(the remainder).
 *	NOTE: This code destroys the original linking
 */
	sl = &savelist;
	rm = &remainder;
	for (ll = list; ll; ll = ll->next) {
		if ((*dec_proc)(ll, parm)) {	/* Satisfies decision proc, add to save list */
			*sl = ll;
			sl = &(ll->next);
			nargs++;				/* One more to archive */
		} else {					/* Later, add to remainder list */
			*rm = ll;
			rm = &(ll->next);
		}
	}
	*rm = NULL;
	*sl = NULL;	/* Terminate lists properly */
	if (alc->debug) {
		pr_list("Save list\n", savelist);
		pr_list("Remaindered list\n", remainder);
	}
/*
 *	Scan remainder list for cases mentioned in savelist
 */
	for (ll = savelist; ll; ll = ll->next) {
		lastrl = NULL;	/* To indicate at start of list */
		for (rl = remainder; rl; rl = rl->next) {
			if (strcmp(ll->casename, rl->casename) == 0) { /* Link to savelist */
				nargs++;			/* One more to archive */
				nextrl = rl->next;	/* Keep next in remainder list */
				nextll = ll->next;	/* Keep hold of next in chain */
				ll->next = rl;		/* Add new file to save list */
				ll = ll->next;		/* Advance ll to point at it */
				ll->next = nextll;	/* Link back next in save list chain */
				if (lastrl == NULL)	{			/* Still at start of list */
					rl = remainder = nextrl;	/* Re-init start of list */
				} else {
					lastrl->next = nextrl;		/* Join remainder back up */
					rl = lastrl;				/* To give a current rl ptr */
				}
			} else {				/* Don't link file in to savelist */
				lastrl = rl;		/* Update last value of rl */
			}
			if (rl == NULL) break;
		} /* end for rl ... */
	} /* end for ll ... */
	if (alc->debug) pr_list("Final savelist\n", savelist);
	closedir(dirp);
/*
 *	Remove remaindered list
 */
 	if (alc->debug) pr_list("Final remainder list\n",remainder);
 	remainder = Freefilelist(remainder);
 	if (alc->debug) fprintf(debug_channel,"Remainder removed\n");

	if (nargs == 0) {
		fprintf(output, "No files to archive\n");
	}
	*link_list = savelist;
	return(nargs);
} /* end getfilelist */

/*
 *  G E T D I R L I S T -- Get a list of archive directories from a filelist
 *
 *  Entry:  
 *		This function accepts a pointer to a link list of file structures
 *		in list and a pointer to a function in getdir. It iteratively scans
 *		list producing a set of seperate lists, each one having a common
 *		archive directory. This archive directory is decided by the function
 *		*getdir, which is passed as the second parameter. This function will 
 *		typically take a filename string and use it to create a directory name.
 *			
 *	Exit:	
 *		The function returns a link list of 'dirlist' structures, one for each
 *		filelist.
 *		A dirlist consists of an archive directory string, a pointer to a
 *		filelist and a pointer to an array of pointers to the files 
 *		(an argv list).
 *		The first 3 elements of this list are free for use as a command
 *		name, and command switches.
 */
struct dirlist *
getdirlist(list, getdir)
struct filelist *list;
(*getdir)();
{
	
	struct filelist *ll, *head, *headr;
	struct filelist **sl, **rm;
	struct dirlist *dl,*dirlist;
	char **avp;
	char dir[MAX_DIR_LEN],checkdir[MAX_DIR_LEN];
	int nargs;
	
	dirlist = NULL;
	/*
	 * First deal with the special case of a null getdir function -- just
	 * build a single dirlist entry
	 */
	if (getdir == NULL) {
		if (alc->debug) {
			fprintf(debug_channel,"getdirlist:\nnull getdir specified, building single dirlist\n");
		}
		dirlist = (struct dirlist *)Calloc(1,sizeof(struct dirlist));
		dirlist->files=list;
		dirlist->next=NULL;
		return(dirlist);
	}
	/*
	 * This loop succesively splits the given file list into two groups
	 * the first consisting of files to archive to the same directory as
	 * the head member, (head), the second consisting of any remainders, (headr)
	 * The procedure is then applied iteratively to the remainder until no
	 * remainder is left. At the end of each pass a dirlist structure is created
	 * for the list at 'head'
	 */
		
	for (head=list;head != NULL;head = headr) {
		nargs=0;
		/* get the directory for the head of the list */
		(*getdir)(dir,head->casename);
		if (alc->debug) {
			fprintf(debug_channel,"creating list for %s\n",dir);
		}
		/*
		 * This loop splits the list at 'head' into two depending on whether the
		 * archive directory of each member file is the same as the head member
		 * or not. This code is adapted from the corresponding loop in getfilelist
		 */
		if (alc->debug) {
			fprintf(debug_channel,"get_dir_list:splitting list at 0x%x\n",head);
		}
		sl = &(head->next); /* the first entry matches by definition */
		rm = &headr;
		nargs = 1;
		for (ll = head->next; ll; ll = ll->next) {
			(*getdir)(checkdir,ll->casename);
			if (strcmp(dir,checkdir) == 0) {
				/* the next file matches the first so add to sl */
				if (alc->debug) {
					fprintf(debug_channel,"adding %s to directory %s\n",ll->name,dir);
				}
				*sl = ll;
				sl = &(ll->next);
				nargs++;				/* One more in this archive */
			} 
			else {					/* Else add to remainder list */
				if (alc->debug) {
					fprintf(debug_channel,"%s excluded, directory is %s\n",ll->name,dir);
				}
				*rm = ll;
				rm = &(ll->next);
			}
		}
		*rm = NULL;
		if (alc->debug) fprintf(debug_channel,"remainder ends at 0x%x\n",rm);
		*sl = NULL;	/* Terminate lists properly */
		if (alc->debug) fprintf(debug_channel,"save ends at 0x%x\n",sl);
		if (dirlist==NULL) {
			dirlist=(struct dirlist *)Calloc(1,sizeof(struct dirlist));
			dl=dirlist;
			if (alc->debug) {
				fprintf(debug_channel,"Malloced space for first dirlist entry at 0x%x",dl);
			}
		}
		else {
			if (alc->debug) {
				fprintf(debug_channel,"cur dirlist at 0x%x, contents are:\n",dl);
				fprintf(debug_channel,"name %s, nargs %d\n",dl->dir,dl->nargs);
				fprintf(debug_channel,"about to malloc new entry\n");
			}
			dl->next=(struct dirlist *)Calloc(1,sizeof(struct dirlist));
			if (alc->debug) fprintf(debug_channel,"malloced space at 0x%x\n",dl->next);
			dl=dl->next;
			if (alc->debug) {
				fprintf(debug_channel,"Malloced space for dirlist entry at 0x%x",dl);
			}
		}
		strcpy(dl->dir,dir);
		dl->nargs=nargs;
		dl->files=head;
	}
	return (dirlist);
}

/*
 * G E T A R G V L I S T -- get an argv file list from a dirlist entry
 */
char **
getargvlist(dl,freeslots)
struct dirlist *dl;
int freeslots;
{
	char **argv, **avp;
	int nargs;
	struct filelist *ll;
	
	if (alc->debug) {
		fprintf(debug_channel,"\nbuilding argv list\n");
	}
	nargs = dl->nargs+freeslots+1;  /* include one for null at end*/
	if (alc->debug) {
		fprintf(debug_channel,"%d arguments, %d freeslots\n",dl->nargs,freeslots);
	}
	argv = avp = (char **)Calloc(nargs, (sizeof(char *)));	/* Pointer list */
	if (alc->debug) {
		fprintf(debug_channel,"Malloced 0x%x bytes for argv list\n",nargs);
	}
	avp += freeslots; /* Skip freeslots, these are filled in later by caller */
	for (ll = dl->files; ll; ll = ll->next) {
		if (alc->debug) {
			fprintf(debug_channel,"adding %s to argv list\n",ll->name);
		}
		*avp++ = ll->name;
	}
	*avp = NULL;
	return(argv);
}
	
void
Freedirlist(dl)
struct dirlist *dl;
{
	struct dirlist *hl;

	hl=dl;
	while (hl) {
		if (alc->debug) fprintf(debug_channel,"freeing file list at 0x%x\n",hl->files);
		if (alc->debug) fprintf(debug_channel,"file list begins with %s\n",hl->files->name);
		Freefilelist(hl->files);
		hl=hl->next;
		if (alc->debug) fprintf(debug_channel,"freeing dirlist entry at 0x%x\n",dl);
		Free(dl);
		dl=hl;
	}
}
/*
 *	C H E C K D I R -- Check if supplied directory exists. If not, create it.
 */
checkdir(dirname)
char *dirname;
{
	int ef;
	if (alc->debug)
		fprintf(debug_channel,"checking for directory %s\n",dirname);
	if ((ef = open(dirname, S_IFDIR)) < 0) {		/* Directory not there */
		if (alc->debug)
			fprintf(debug_channel,"making directory %s\n",dirname);
		makdir(dirname, "rw", 0x3f, 1024);			/* No, so make it ewrewr */
		if ((ef = open(dirname, S_IFDIR)) < 0) {	/* OK to use ? */
			if (alc->debug) {
				fprintf(debug_channel, "Can't open archive directory %s (%d)\n",
					                   dirname, errno);
			}
			fprintf(output, "Can't open archive directory %s (%d)\n",
				dirname, errno);
			return(FALSE);
		}
	}
	close(ef);
	return(TRUE);
}

/*
 *	S Y S E X E C --	Execute command given as first param
 *						With subsequent params. as params to the command
 *						params is a space separated list of parameters
 *						(Is that clear ?). Uses os9forkc to fork command
 */
sysexec(cmd, params)
char *cmd, *params;
{
	int stat, pid;

	if (alc->debug)
		fprintf(debug_channel, "sysexec: command %s, with %s\n", cmd, params);

	if ((pid = os9forkc(cmd, strlen(params), params, 0, 0, 0, 0, 3)) < 0) {
		fprintf(stderr, "Can't fork %s\n", cmd);
	} else {
		while (wait(&stat) != pid) ;
	}
	return(stat);
}

#ifdef WE_WOULD_USE_THIS_METHOD_IF_SHELL_DID_NOT_STICK_ITS_OAR_IN
/*
 *	S Y S E X E C --	Execute command given as first param
 *						With subsequent params. as params to the command
 *						params is a space separated list of parameters
 *						(Is that clear ?)
 */
sysexec(cmd, params)
char *cmd, *params;
{
	char *p, **argv, **avp;
	int argc, inspace, stat, pid;

	fprintf(debug_channel, "sysexec: command %s, with %s\n", cmd, params);
	p = params;
	inspace = TRUE;		/* Hypothetical space before start of command */
	argc = 2;			/* The command itself, and null termination */
	while (*p != '\0') {
		if (*p++ == ' ') inspace = TRUE; /* We are in whitespace */
		else if (inspace) {
			argc++;
			inspace = FALSE;	/* Not anymore, until next space */
		}
	}
	printf("No. args = %d\n", argc);
	argv = avp = (char **)Calloc(argc, sizeof(char *));
	*avp++ = cmd;
	p = params;
	inspace = TRUE;		/* Hypothetical space before start of command */
	while (*p != '\0') {
		if (*p == ' ') inspace = TRUE; /* We are in whitespace */
		else if (inspace) {
			*avp++ = p;
/*			printf("Arg: %s\n", p);*/
			inspace = FALSE;	/* Not anymore, until next space */
		}
		p++;
	}
	*avp = NULL;	/* Terminate arg list */
	if ((pid = os9exec(os9forkc, argv[0], argv, environ, 0, 0, 3)) < 0) {
		fprintf(stderr, "Can't fork %s\n", argv[0]);
	} else {
		while (wait(&stat) != pid) ;
	}
}
#endif

/*
 *	G E T _ A R C H _ M E T H O D --	Return the archive method as an
 *										integer value
 *	This function will need to also return method parameters for the case method
 *	NOTE: We don't want to overwrite the method specified in alc->arch_method,
 *	since this means the archive method for a new OD will depend on whether
 *	another disk has been used first.
 */

get_arch_method()
{
	int am_fp;
	char am_str[80];
	int am_int;

	if ((am_fp = open(ARCH_METHOD, S_IREAD)) < 0) {
		if (errno != E_PNNF) {	/* It's not there so need to create it */
			fprintf(output, "Error in reading archive method (%d)\n", errno);
			return(-1);
		}
		strcpy(am_str, alc->arch_method); /* So get default */
		if (alc->debug) {
			fprintf(debug_channel,"get_arch_method :\nam_str: %s, alc->arch_method %s\n",am_str,alc->arch_method);
		}
		/*
		 *	Must be legal method before we try to write the file
		 */
		if ((am_int = convert_arch_method(am_str)) < 0) return(-1);
		if ((am_fp = create(ARCH_METHOD, S_IWRITE,
				S_IREAD | S_IWRITE | S_IOREAD)) < 0) { /* will need pub read */
			fprintf(output, "Error creating archive method file (%d)\n", errno);
			return(-1);
		}
		strcat(am_str, "\n");
		write(am_fp, am_str, strlen(am_str));
	} else {
		am_int = readln(am_fp, am_str, 80);
		am_str[am_int] = '\0'; /* Null end of string */
		if ((am_int = convert_arch_method(am_str)) < 0) return(-1);
	}
	if (alc->debug)
		fprintf(debug_channel, "Archive method = %s\n", am_str);
	close(am_fp);
	return(am_int);
} /* end get_arch_method */

/*
 *	C O N V E R T _ A R C H _ M E T H O D --	Convert archive method string to
 *												integer format.
 */
convert_arch_method(str)
char *str;
{
	if (strncmp("DATE", str, 4) == 0)
		return(AM_DATE);
	else if (strncmp("CASE", str, 4) == 0) {
		int i=0; /* counter for position list */
		char *pstr=(str+4);		/* point to blank before char position list */
		
		if (alc->debug) {
			fprintf(debug_channel, "converting arch method for : ");
			fprintf(debug_channel, "%s\n",str);
		}		
		while ((*pstr) && (i<MAX_DIR)) {
			if (alc->debug) {
				fprintf(debug_channel,"case characters left: %s\n",pstr);
			}
			if (sscanf(pstr,"%d",&case_characters.pos[i])) {
				if (alc->debug) fprintf(debug_channel,"convert_arch_method:\n");
				case_characters.pos[i]--;	/* adjust for count from zero */
				if (alc->debug) fprintf(debug_channel,"position %d: character pos %d\n",i,case_characters.pos[i]);
				if (case_characters.pos[i] > -1) /* gotcha to trap negatives */
					i++;
				else if (alc->debug)
					fprintf(debug_channel,"Case character position %d at directory character %d",case_characters.pos[i],i);
			}
			while ((*pstr) && (*pstr == ' ')) pstr++; /* gobble blanks */
			while ((*pstr) && (*pstr != ' ')) pstr++; /* gobble previous number */
		}
		if ((case_characters.tot = i) == 0) {
			fprintf(output,"Couldn't create directory for %s\n",str);
		}
		if (alc->debug) {
			fprintf(debug_channel,"got case arch method, %d character positions\n",i);
		}
		if (i==0) {
			return(-1);
		}
		else {
			return(AM_CASE);
		}
	}
	else if (strncmp("HASH", str, 4) == 0) {
		fprintf(output, "HASH");
		/*return(AM_HASH);*/
	}
	else {
		fprintf(output, "Unknown archive method\n");
		return(-1);
	}
	fprintf(output, " archive method is not yet supported\n");
	return(-1);
}

/*
 *	P R I N T F I L E --	print list of files given
 */
printfile(alist)
struct filelist *alist;
{
	int line, col, filesperline=1;

	clearlns(STARTLINE, ENDLINE);
	line = STARTLINE;
	col = 1;
	while (alist != NULL) {
		screengo(line, col);
		printf("%s", alist->name);
		if ((filesperline++ % 2) == 0) { line++; col = 1; }
		else col += 30;
		alist = alist->next;
	}
	fflush(stdout);
	setdisplay(BUST);
}

/*
 *	The following are debug functions
 */
/*
 *	P R _ L I S T -- Print linked list of filelist structures
 */
pr_list(str, list)
char *str;
struct filelist *list;
{
	struct filelist *ll;

	if (!alc->debug) return(0);
	fprintf(debug_channel, "%s", str);
 	for (ll = list; ll; ll = ll->next) {
 		fprintf(debug_channel, "File: %s, Date: %s\n",
 			ll->name, pr_date(ll->date));
 	}
}

/*
 *	P R _ D A T E -- Convert date to yy/mm/dd/hh:mm format, given date in
 *	5 byte format.
 */
char *
pr_date(date)
char *date;
{
	static datestr[15];

	sprintf(datestr, "%d/%d/%d %d:%d",
		date[0], date[1], date[2], date[3], date[4]);
	return(datestr);
}


