/*
 *      R E P O R T . C  --  Write case summary report
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    23 March 1988
 *
 *  Modifications
 *
 *	17 Aug 1988		CAS		Case report - get ktype stuff to work
 *	31 Mar 1988		CAS		Check string lengths in copy's
 *
 */
#include <stdio.h>
#include <kyy.h>
#include <fileheaders.h>
#include <modes.h>
#include <direct.h>
#include <dir.h>
#include <spool.h>

extern FILE *debug_channel;

/*
 *	Stuff we user from elsewhere
 */
struct propertylist *convcellhead(), *convmethead();

/*
 *	Local forward defs 
 */
struct case_report *reportcase();

struct case_report *repcase();
struct slide_report *repslide(), *replist();
struct cell_report *repcell();

struct case_report {
		char	case[PATLENGTH+1];
		char	patnam[PATLENGTH+1];
		char	lab_name[LABLENGTH+1];
		struct slide_report *slide;
	};
struct slide_report {
		struct	slide_report *next;
		char	slideid[SLIDELENGTH+1];
		int		tissue;
		int		stain;
		struct	list_report *mlist;
		struct	cell_report *cell;
	};
struct list_report {
	};
struct cell_report {
		struct	cell_report *next;
		int		cellno;
		int		xcoord, ycoord;
		char	karyotype[TEXTLENGTH+1];
		char	comments[2][TEXTLENGTH+1];
	};

/*
 *	R E P O R T  --  Generate report and write to file
 *
 */
report(casename,dirname,opfile,level)
char	*casename, *dirname;
FILE	*opfile;
int		level;
{
	struct case_report *rep;
	rep = reportcase(casename,dirname,level);
	reportwrite(rep,opfile,level);
	reportfree(rep);
}

/*
 *	R E P O R T C A S E  --  Make report structure for supplied case
 *
 */
struct case_report *reportcase(casename,dirname,level)
char	*casename, *dirname;
int		level;
{
	DIR		*dp;
	int		typ;
	char	buffer[80];
	struct	direct *dirent;
	struct cyto_file cur_file;
	struct case_report *rep = NULL;

	rep = (struct case_report *) Calloc(1,sizeof(struct case_report));
	if (dp = opendir(dirname)) {
		readdir(dp);						/* Skip '.' and '..' */
		readdir(dp);
		while(dirent = readdir(dp)) {
			if (typ = iscyto(dirent->d_name,&cur_file)) {
				if (strncasecmp(cur_file.case,casename,CASELENGTH) == NULL) {
					sprintf(buffer,"%s/%s\n",dirname,dirent->d_name);
					if ((fp = fopen(buffer,"r")) != NULL) {
						reportfile(rep,fp);
						fclose(fp);
					} else {
						fprintf(stderr,"File %s %s is read protected\007\n",
								cur_file.case,makefilestring(&cur_file,buffer));
					}
				}
			}
		}
		closedir(dp);
	} else
		fprintf(stderr,"Failed to access directory %s\n",dirname);
	return(rep);
}

/*
 *	R E P O R T F I L E  --  Add stuff from this file to report structure
 *
 */
reportfile(rep,fp)
struct	case_report *rep;
FILE	*fp;
{
	struct cell_header *cell;
	struct met_header *met;
	struct object *obj, *readobj();
	struct	case_report *crep;
	struct	slide_report *srep;
	obj = readobj(fp);
	if (obj && (obj->type == 110) && obj->plist) {
		switch (((struct cell_header *)obj->plist)->file_type) {
			case CELLFILE:
			case KARYFILE:
				obj->plist = convcellhead(obj->plist);
				cell = (struct cell_header *) obj->plist;
				printf("Report %s %s %d\n",cell->patid,cell->slideid,cell->cellid);
				crep = repcase(rep,cell->patid,cell->patnam,cell->lab_name);
				srep = repslide(crep,cell->slideid);
				if (cell->file_type == CELLFILE)
					repcell(srep,cell->cellid,cell->x_coord,cell->y_coord);
				else
					repktype(srep,cell->cellid,cell->x_coord,cell->y_coord,cell->krtype,cell->comments[0],cell->comments[1]);
				break;
			case METFILE:
/*				obj->plist = convmethead(obj->plist); */
				met  = (struct met_header *) obj->plist;
				if (met->version == HEADER_VERSION) {
					printf("Report %s %s MET\n",met->patid,met->slideid);
					crep = repcase(rep,met->patid,met->patnam,met->lab_name);
					srep = replist(crep,met->slideid,met->tissue,met->stain);
				}
				break;
			default:
				break;
		}
	}
	freeobj(obj);
}

/*
 *	R E P O R T W R I T E  --  Write actual report to o/p file
 *
 *	Level is used to set verbosity.  Level 1 is for intermachine transfer
 *									 Level 2 is for human readable report
 */
reportwrite(rep,fp,level)
struct	case_report *rep;
FILE	*fp;
int		level;
{
	struct slide_report *srep;
	struct cell_report *crep;
	switch (level) {
		case 1:
			fprintf(fp,"*Case 3\n%s\n",rep->case);
			fprintf(fp,"%s\n",rep->patnam);
			fprintf(fp,"%s\n",rep->lab_name);
			break;
		case 2:
			fprintf(fp,"        CYTOSCAN  Case Summary \n\n");
			fprintf(fp,"Case:    %s\n",rep->case);
			fprintf(fp,"Patient: %s\n",rep->patnam);
			fprintf(fp,"Lab:     %s\n",rep->lab_name);
			break;
	}
	srep = rep->slide;
/*
 *	Process each slide in turn
 */
	while (srep) {
		switch (level) {
			case 1:
				fprintf(fp,"*Slide 3\n");
				fprintf(fp,"%s\n",srep->slideid);
				fprintf(fp,"%d %s\n",srep->tissue,gettissue(srep->tissue));
				fprintf(fp,"%d %s\n",srep->stain,getstain(srep->stain));
				break;
			case 2:
				fprintf(fp,"\nSlide %s\n",srep->slideid);
				if (srep->tissue) {
					fprintf(fp,"Tissue :%s",gettissue(srep->tissue));
					fprintf(fp,", Stain %s\n",getstain(srep->stain));
				}
				break;
		}
/*
 *	For each slide -> do each cell
 */
		crep = srep->cell;
		while (crep) {
			switch (level) {
				case 1:
					fprintf(fp,"*Cell 5\n");
					fprintf(fp,"%d\n",crep->cellno);
					fprintf(fp,"%d %d\n",crep->xcoord,crep->ycoord);
					fprintf(fp,"%s\n",crep->karyotype);
					fprintf(fp,"%s\n",crep->comments[0]);
					fprintf(fp,"%s\n",crep->comments[1]);
					break;
				case 2:
					fprintf(fp,"\n  Cell %d\n",crep->cellno);
					fprintf(fp,"  Location %d %d\n",crep->xcoord,crep->ycoord);
					fprintf(fp,"  Karyotype: %s\n",crep->karyotype);
					fprintf(fp,"  Comments:  %s\n",crep->comments[0]);
					fprintf(fp,"             %s\n",crep->comments[1]);
					break;
			}
			crep = crep->next;
		}
		srep = srep->next;
	}
}

/*
 *	R E P O R T F R E E  --  Free off all space grabbed for report
 *
 */
reportfree(rep)
struct	case_report *rep;
{
	struct slide_report *srep, *stmp;
	struct cell_report *crep, *ctmp;

	srep = rep->slide;
	while (srep) {
		crep = srep->cell;
		while (crep) {
			ctmp = crep->next;
			Free(crep);
			crep = ctmp;
		}
		stmp = srep->next;
		Free(srep);
		srep = stmp;
	}
	Free(rep);
}

/*
 *	R E P C A S E  --  Put case info into report structure
 *
 */
struct case_report *repcase(rep,patid,patnam,labnam)
struct	case_report *rep;
char	*patid, *patnam, *labnam;
{
	if (rep->case[0] == '\0') {
		strncpy(rep->case,patid,CASELENGTH);
		strncpy(rep->patnam,patnam,PATLENGTH);
		strncpy(rep->lab_name,labnam,LABLENGTH);
	}
	return(rep);
}

/*
 *	R E P S L I D E  --  Put slide info into report structure
 *
 */
struct slide_report *repslide(rep,slideid)
struct case_report *rep;
char	*slideid;
{
	struct slide_report *srep;
	srep = rep->slide;
	if (srep == NULL) {
		srep = (struct slide_report *) Calloc(1,sizeof(struct slide_report));
		rep->slide = srep;
	} else {
		while (1) {
			if (!(strncasecmp(srep->slideid,slideid,SLIDELENGTH)))
				return(srep);
			if (!srep->next) {
				srep->next = (struct slide_report *) Calloc(1,sizeof(struct slide_report));
				srep = srep->next;
				break;
			} else
				srep = srep->next;
		}		
	}
	strncpy(srep->slideid,slideid,SLIDELENGTH);
	return(srep);
}

/*
 *	R E P L I S T  --  Put basic list info in report structure
 *
 */
struct slide_report *replist(rep,slideid,tissue,stain)
struct case_report *rep;
char	*slideid;
int		tissue, stain;
{
	struct slide_report *srep;
	srep = repslide(rep,slideid);
	srep->tissue = tissue;
	srep->stain = stain;
	return(srep);
}

/*
 *	R E P C E L L  --  Put cell info into report structure
 *
 */
struct cell_report *repcell(srep,cellno,xc,yc)
struct	slide_report *srep;
int		cellno, xc, yc;
{
	struct cell_report *crep;
	crep = srep->cell;
	if (crep == NULL) {
		crep = (struct cell_report *) Calloc(1,sizeof(struct cell_report));
		srep->cell = crep;
	} else {
		while (1) {
			if (crep->cellno == cellno)
				return(crep);
			if (!crep->next) {
				crep->next = (struct cell_report *) Calloc(1,sizeof(struct cell_report));
				crep = crep->next;
				break;
			} else
				crep = crep->next;
		}		
	}
	crep->cellno = cellno;
	crep->xcoord = xc;
	crep->ycoord = yc;
	return(crep);
}

/*
 *	R E P K T Y P E  --  Put karyotype info into report structure
 *
 */
repktype(srep,cellno,xc,yc,kart,com1,com2)
struct	slide_report *srep;
int		cellno, xc, yc;
char	*kart,*com1,*com2;
{
	struct cell_report *crep;
	crep = repcell(srep,cellno,xc,yc);
	strncpy(crep->karyotype,kart,TEXTLENGTH);
	strncpy(crep->comments[0],com1,TEXTLENGTH);
	strncpy(crep->comments[1],com2,TEXTLENGTH);
}

