/*
 *      R O T C H R O M . C  --  Chromosome rotation
 *
 *
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *  Written: Denis Rutovitz  (Nov 1986)
 *			 Medical Research Council
 *			 Clinical and Population Cytogenetics Unit
 *			 Edinburgh
 * 
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *  Modifications
 *
 *	27 Jan 1991	jimp@MRC	getIang(), setIang() added for flexible screen.
 *	12 may 1988		CAS			down/up menu calls
 *	15 Apr 1988		CAS			Silly bug from below not erasing arrow properly
 *	28 Mar 1988		CAS			ensure Iang remains in sensible limits
 *								and tidy up code... + get rotate sense right
 *	25 Mar 1988		CAS			Initialise Iang (static)
 *	25 Jan 1988		CAS			removed all p4/2N crap
 *	18 Nov 1987		CAS			Menu options to lower case
 *	02 Sep 1987		jimp@IRS	Display cursor at user's choice of angle.
 *								Only X movement significant.
 *	20 Aug 1987		jimp@IRS	mymouse and mouss ==> alc->m2s
 *	31 Mar 1987		jimp		Send rotate off to a slave service function.
 *								New function parameters.
 *	11 Mar 1987		jimp		Start fixing into IRS system....
 */
#include <cmdsys.h>
#include <wstruct.h>
#include <allcontr.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <interact.h>
#include <display.h>

/*
 * external variables used
 */
extern struct allkontr *alc;

/*
 * external function reference
 */

#define c30 87		/* used in construction of arrow head only */
#define s30 50
#define RADIANS	.01745
#define RAD 300		/* radius of displayed circle */

/*
 * local variables
 */
#define ROUND	80
#define HROUND	40
#define QROUND	20
#define MFACTOR	10000
/*
 *	Sin and Cos tables
 */
static int csr[41],snr[41];
static int cs [ ] = { 
	10000,9969,9876,9723,9510,9238,8910,8526,8090,7604,7071,
	6494,5877,5224,4539,3826,3090,2334,1564,784,0,-784,
	-1564,-2334,-3090,-3826,-4539,-5224,-5877,-6494,-7071,-7604,-8090,
	-8526,-8910,-9238,-9510,-9723,-9876,-9969,-10000
	} ;
static int sn [ ] = { 
	0,784,1564,
	2334,3090,3826,4539,5224,5877,6494,7071,7604,8090,8526,
	8910,9238,9510,9723,9876,9969,10000,9969,9876,9723,9510,
	9238,8910,8526,8090,7604,7071,6494,5877,5224,4539,3826,
	3090,2334,1564,784,0
	} ;
static struct pframe *myframe;
static int xcn,ycn; /* centre of rotation (digs coordinates)*/
static int xcd,ycd; /* digs coords of cursor */
static int Iang = 0;

/*
 *	M Y C I R C L E  --  
 *
 * this isnt very good with very differnet radii, but its OK in the
 * range of 300 or so. Should adjust number of segments to size
 */
mycircle(xcentre,ycentre,rad)
register xcentre,ycentre,rad ;
{
	register int i;
	register int *csrp, *snrp;
	static int lastrad=0;
	if (rad < 1) return;
	
	if (lastrad != rad) { /* if this radius differs from last one,
	                           make new tables csr,snr */
		int *csp,*snp;
		csp = cs;  csrp = csr;
		snp = sn;  snrp = snr;
		while ( csp < cs + HROUND) {
			*csrp++ = (*csp++ * rad) / MFACTOR;
			*snrp++ = (*snp++ * rad) / MFACTOR;
		}
		lastrad = rad;
	}
	moveto(xcentre,ycentre);
	moveby(rad,0);
	
	csrp = csr; snrp = snr;
	while (csrp < csr + HROUND) lineto(xcentre + *csrp++, ycentre + *snrp++);
	csrp = csr; snrp = snr;
	while (csrp < csr + HROUND) lineto(xcentre - *csrp++, ycentre - *snrp++);
}

eraserotcurs()
{
	intens(0);
	digscurs();
	intens(1);	
}

static first = 1; /* tells whether initial or subseuent activation of rotcurs */

/*
 * Functions added to permit this module to be used
 * by the flexible screen things.
 */
setIang(a,f,x,y)
int a,f,x,y;
{
	Iang = a;
	first = f;
	xcn = x;
	ycn = y;
}
int getIang()
{
	return(Iang);
}

/*
 *	R O T C U R S  --
 *
 */
rotcurs()
{
	register int scalprod,xdel;
	int xd,yd,dqw,dq;
	int i;
	
	if (!first) {
		eraserotcurs();
	}
	first = 0;
	xdel = myframe->ix > 0 ? alc->m2s->XINC : - alc->m2s->XINC;
	if ((xdel >= ROUND) || (xdel <= -ROUND)) xdel = 0;
	Iang += xdel;
	if (Iang >= ROUND)
		Iang -= ROUND;
	else if (Iang < 0)
		Iang += ROUND;
	if (Iang >= HROUND) {
		xcd = xcn -csr[Iang-HROUND];
		ycd = ycn -snr[Iang-HROUND];
	} else {
		xcd = xcn +csr[Iang];
		ycd = ycn +snr[Iang];
	}
	intens(1);
	digscurs();
}

/*
 *  T U R N C H R O M
 *
 */
struct chromosome *
turnchrom(ang,kcont,objnum)
struct kcontrol *kcont;
{
	struct chromosome *retobj;
	register struct chromosome *nobj = kcont->mol[objnum]; 
	register struct pframe *f = kcont->ivf + objnum;
	register int xcr,ycr;	/* current cursor position */
	int		opt;
	
	myframe = f;
	xcr = (nobj->idom->lastkl + nobj->idom->kol1)/2;
	ycr = (nobj->idom->lastln + nobj->idom->line1)/2;
	xcn = frdigsxpos(xcr,myframe); /* object middle in digs coords */
	ycn = frdigsypos(ycr,myframe);

	erasecursor();
	unbox();
	colour(OVERLAY2);
	intens(1);
	mycircle(xcn,ycn,RAD);
	setmsv(rotcurs);
	Iang = QROUND - ang*ROUND/360;
	first = 1;
	rotcurs();
	retobj=0;
	if (ang) {
		ang = -ang;		/* Get sense right */
		opt = 1;
	} else {
		downmenu();
		newmenu("Rotate Chromosome");
		cmmd("angle;%d",&ang);
		cmmd("rotate so that present cursor position is top");mouse(MIDBT);
		if ((opt = thismu()) == 2)
			ang = ((QROUND-Iang) * 360) / ROUND;
		upmenu();
	}
/*
 *	If we really want to do rotate then
 *	use a slave service function to do the rotation.
 */
	if (opt) {
		kcont->auxstat = ang;
		retobj = (struct chromosome *) sing_mp_k_serv(kcont,K_ROTATE,objnum);
	}
	eraserotcurs();
	intens(0);
	colour(OVERLAY2);
	mycircle(xcn,ycn,RAD);
	intens(1);
	return(retobj);
}

/*
 *	D I G S C U R S  --
 *
 */
digscurs()
{
	colour(OVERLAY1);
	moveto(xcn,ycn);
	lineto(xcd,ycd);
	arrowhd(xcd-xcn,ycd-ycn,100);
}

/*
 *	A R R O W H D  --
 *
 */
arrowhd(x,y,size)
register int x,y,size;
{
	int x1,x2,y1,y2;
	register int dd = 100*RAD;
	x1 = ((-x * c30 - y *s30 )*size)/dd;
	y1 = ((-y * c30 + x *s30 )*size)/dd;
	x2 = ((-x * c30 + y *s30 )*size)/dd;
	y2 = ((-y * c30 - x *s30 )*size)/dd;
	moveby(x1,y1);
	lineby(-x1,-y1);
	moveby(x2,y2);
	lineby(-x2,-y2);
}
