 /*
 *      S E G F I X . C  --  Metaphase Segmentation
 *
 *
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *  Taken from: seginteract and segfix 11/85, 1/86
 *				Jim Piper et al.
 *				Medical Research Council
 *				Clinical and Population Cytogenetics Unit
 *				Edinburgh
 *
 *  Date:    10th January 1988
 *
 *  Modifications
 *
 *  13 Feb 1992		Joanne		All the splitting now in splitup.c
 *  10 Feb 1992		Joanne		COMPOSITES left over after segmentation,
 *					are changed into CHROMOSOMES
 *  28 Jan 1992		Joanne		change-species option added to 
 *					segmentation menu 
 *  7 Jan 1992		Joanne		ck_newpseg() and ck_bpdseg() made extern
 *					New module splitup() made which contains
 *					doautosplit() and splitup().
 *					oversplit() changed.
 *  2  Feb 1991 	ih/gjp		Check OPT_FLEX before allowing flex screen
 *  20 Nov 1990		Bill Hill @ MRC HGU	put in flexible screen stuff
 *  19 Jul 1990		dcb			seg_display_opts: pass phase to enhanceopt
 *	20 Mar 1990		dcb/GJP		Add confirm prompt on quit segmentation. Add
 *								multi-classifier selection
 *	05 Mar 1990		dcb			Rmove colfn, & background colour option
 *	01 Mar 1990		dcb			Remove setcolour from segmenu
 *	19 Sep 1989		dcb			Remove colours[] declr, and double include of
 *								colours.h
 *	22 Aug 1989		CAS			Remove colour display from 'background' option
 *	21 Aug 1989		CAS			setup colours automatically for fluor
 *	12 Jul 1989		SEAJ		Removed call to fixclasses.
 *	 4 Jul 1989		SEAJ		pgroup nolonger set in ischromosome().
 *	31 May 1989		dcb			Only call set_dev_vcv if things change
 *  25 May 1989		SEAJ		Make prob noise really noise in scrap_prob_noise.
 *	19 May 1989		dcb/SEAJ	segmenu call trackcursor for nncmd > 1 !!
 *  18 May 1989		SEAJ		Fixed line missing in splitup
 *	15 May 1989		dcb			Increase functionality of doclassifier, change
 *								class to chromlength and access from chromlyse
 *  11 May 1989		SEAJ		Changed segmentation menu ... reinit classifier.
 *  19 Apr 1989		SEAJ		Bug with manual karyotype {}.
 *  11 Apr 1989		SEAJ		Swapped white & black for htnormalise.
 *  17 Mar 1989		SEAJ		Added variable enhancement to metaphase.
 *  22 Feb 1989		dcb/SEAJ	Overlap to use rubber_overlap flag.
 *	17 Feb 1989		SEAJ		Modified segmenu and mankary for new chromlyse.c.
 *	15 Feb 1989		CAS			Mods to oversplit/splitup to tidy menu code
 *	 7 Feb 1989		SEAJ		Initialise nvertices to 0 in splitup and oversplit
 *								and make sure that it is then used correctly.
 *  18 Jan 1989		SEAJ		Changed order of commands for nncmd>2 in segmenu.
 *  11 Jan 1989		SEAJ		Added check for maximum number of objects into 
 *								joinup.
 *   7 Dec 1988		CAS			Allow inactive objects in ischromosome so that
 *								we can reactivate things.
 *	18 Nov 1988		CAS			Removed double obj check in splitup
 *	27 Sep 1988		CAS			Bad {} in segfix, around mankary
 *								Also need to initialise man_cmd in case its not setup
 *	19 Sep 1988		CAS			colfn - use correct set when called from ktype
 *	13 Aug 1988		CAS			linkerbug on undo in segmenu
 *	22 Aug 1988		CAS			Man kary as an option
 *	17 Aug 1988		CAS			Bug in ischrom - lists accessed before checkobj..
 *	12 Aug 1988		SEAJ		Initialise string in mankary
 *	11 Aug 1988		SEAJ		Fix in nxtnum
 *   8 Aug 1988		SEAJ		Removed ischromosome facility in devel, it is
 *								no longer needed.
 *   4 Aug 1988		SEAJ		Added manual karyotype option.
 *	15 Jul 1988		SEAJ		Added clear background to Segment and display 
 *								options
 *	 6 Jul 1988		CAS			Facility in ischromosome to force group
 *								only used in devel for testing
 *	 1 Jun 1988		CAS			Fix incomming count to cope with NUC's
 *	23 May 1988		CAS			Changes to colour calls
 *	12 May 1988		CAS			Option to restore (un-norm, un-enhance)
 *	24 Mar 1988		CAS			Ensure intens(1) in metenh + metnorm
 *	11 Mar 1988		CAS			methtnormalise
 *	10 Mar 1988		CAS			metenhance - releas + check for NULL ptrs
 *	 2 Feb 1988		CAS			remove unnecessary kcont init
 */
#include <cmdsys.h>
#include <wstruct.h>
#include <allcontr.h>
#include <chromanal.h>
#include <interact.h>
#include <kcontrol.h>
#include <colour.h>
#include <class.h>
#include <display.h>
#include <seg_control.h>

#define OVER_MAXLEFT	200	/* Max amount allowd to be left in overlap
				split before we assume an object has been
				missed */
#define MAXOPTS 3			/* Maximum number of display options in mankary */

/*
 * EXTERNALS
 */
extern struct allkontr *alc;
extern struct kcontrol *kcont;
extern FILE *debug_channel;
extern struct ipoint cursor;
/*extern int chromlength; */

/*
 * GLOBALS USED EXTERNALLY ELSEWHERE
 */
struct seg_interact SI = {0};
/*
 * GLOBALS ONLY USED IN HERE
 */
int chromnum, subnum, ninclass;
char *opt[] = {
	"",
	"Boundaries",
	"Hulls",
	"Rectangles"
};

static struct ct {
	int	type;
	char *str;
} ctypes[] = {{SHORT, "short"}, {LONG, "long"}, {RBAND, "R-band"}};
#define NCTYPES = sizeof(ctypes)/sizeof(struct ct)

/*
 * FUNCTION DECLARATIONS
 */
extern int track1(),track2();		/* Cursor tracking functions */
extern int rubber_track2();
/*extern int chromcurs();*/
/*extern int setcolour(), changecolour();*/
extern dispblobs(), clearplane();
extern bkpseg();
extern i_clean();
extern int enhanceopt();
extern int flex_Main();
extern splitup(), doautosplit();
extern ck_newpseg(), ck_bkpseg();
extern getspecies();
clean();

/*
 *	Forward declarations
 */
int seg_display_opts();
static bdselect();
/*static oversplit();*/
disp_prob_noise();
int axmsg(), selchromnum(), nxtnum(), backup();
int dispchromnum(), strint(), optmenu(), strmatch(), mankary(), doclassifier();
static int options();
static char* chrom_typestr();
static int typestr_chrom();
static select_classifier(), change_classifier();

#define  UNCERTAINMAX 20	/* This is only nec since we have a fixed 
							size stack on the bottom line.  This
							stack is NSTACKPOS elements long.
							Here we play the probabilities game and
							assume no more than 14 more generated
							during karytype phase......
							A NASTY KLUDGE... */


/*
 *	J O I N U P 
 *
 */
static joinup() {
	struct chromosome **objlist;
	int dunjoin = 0;
	if (!checkobj(kcont->current_obj)) {
		return;
	}
	if ( ! kcont->SI->jselect )
		kcont->SI->jselect = kcont->current_obj;
	if ( ! kcont->SI->jobject || kcont->SI->jobject == kcont->SI->jselect) {
		fprintf(stderr,"Please select other object to be joined\n");
		kcont->SI->jobject = kcont->SI->jselect;
		inset(kcont,kcont->SI->jobject-1);
		backlight(kcont->ool[kcont->SI->jobject-1],kcont->f,1); /* 3*/
	} else {
		intens(1);
		objlist = kcont->ool;
		inset(kcont,kcont->SI->jselect-1);
		backlight(objlist[kcont->SI->jselect-1],kcont->f,1); /* 3*/
		printf ("About to join objects %d and %d\n",kcont->SI->jobject,kcont->SI->jselect);
		dunjoin = 1;
		if ( go_on() ) {
			if (alc->debug) {
				fprintf(debug_channel,"jobj plist 0x%x\n",kcont->ool[kcont->SI->jobject]->plist);
				fprintf(debug_channel,"jsel plist 0x%x\n",kcont->ool[kcont->SI->jselect]->plist);
				fflush(debug_channel);
			}
			if (kcont->maxnumber >= MAXOBJS)
				fprintf(stderr, "There are too many objects for join to be performed.\007\n");
			else {
				if (kcont->ool[kcont->SI->jobject-1]->plist->otype != NOISE)
					kcont->SI->numchroms--;
				if (kcont->ool[kcont->SI->jselect-1]->plist->otype != NOISE)
					kcont->SI->numchroms--;
				join(kcont->SI->jobject,kcont->SI->jselect);
				kcont->SI->numchroms++;
				inclear(kcont,kcont->SI->jobject-1);
				inclear(kcont,kcont->SI->jselect-1);
			}
		}
	}
	if (dunjoin) {
		unbacklight();
		backlight(objlist[kcont->SI->jobject-1],kcont->f,0);
		kcont->SI->jobject = 0;
	}
	putprompt();
	kcont->SI->jselect = 0;
}

/*
 *	I S N O I S E 
 *
 */
static
isnoise(i)
{
	if (!checkobj(kcont->current_obj)) {
		return;
	}
	if (noise(kcont->current_obj,i)) --kcont->SI->numchroms;
}

/*
 *	I S C H R O M O S O M E
 *
 */
static
ischromosome(class)
int		*class;
{
	int onum;
	struct chromplist *plist;
	SMALL *ac;

	onum = kcont->current_obj-1;
	if ((onum < 0) || (onum >= kcont->maxnumber)) {
		checkobj(kcont->current_obj);				/* Unify error handling */
		return;
	}
	plist = kcont->ool[onum]->plist;
	ac = &kcont->acl[onum];
	if (!(*ac & ACTIVE) && !((plist->otype == NOISE) || (plist->otype == NUCLEUS))) {
		return;
	}
	/*
	 * wait if features being measured, set INTERACTING bit
	 */
	inset(kcont,onum);	
	checkattention(0,onum,kcont);
	if (plist->otype == NOISE || plist->otype == NUCLEUS) {
		if ((*ac & ACTIVE) == 0) {
			*ac |= ACTIVE;
			kcont->SI->numchroms++;
		}
	}
	if (*ac & ACTIVE) {
		if (*class) {
			plist->Cpgroup = *class;
			plist->pgconf = 100;
		} else {
			*ac &= ~(MSTART|MEND|MPROC);	/* force feature (re-)measurement
											and clear processor field */
		}
		plist->otype = plist->Cotype = CHROMOSOME;
		plist->otconf = 100;
	}
	*class = 0;
	/*
	 * clear INTERACTING bit
	 */
	inclear(kcont,onum);
}


undo();
static chromcurs();
changedisplay();

#ifdef UNUSED
/*
 *	C O L F N
 *
 */
colfn(phase)
int		phase;
{
	if (phase) 
		changecolour(&alc->karycolour_1,&alc->karycolour_2,phase);
	else
		changecolour(&alc->metacolour_1,&alc->metacolour_2,phase);
}
#endif

/*
 *	R E N O R M D I S P 
 *
 */
renormdisp()
{
	int i;
	renormalisedisplay(kcont);
	/* do a quick fix on probable object types before displaying */
	for (i=0; i<kcont->maxnumber; i++)
		probotype(kcont->ool[i]);
	colour(OVERLAY1);
	tsize(50,50);
	dispblobs(kcont);
	dispserials(kcont);
}

/*
 *	W R I T E M E T  --  Devel write back modified metaphase
 *
 */
writemet(fname)
char	*fname;
{
	putfile(fname,NULL,kcont->ool,kcont->maxnumber);
}

/*
 *	M E T E N H A N C E  --  Enhance the metaphase (SLOWLY..)
 *
 */
metenhance()
{
	struct object **objlist,*enhobj,*enhance(), *varenhance();
	SMALL *acl;
	int ac,objnum;
	
	colour(GREYCOLS);
	intens(1);
	objlist = (struct object **) kcont->ool;
	acl = kcont->acl;
	for (objnum = 0; objnum < kcont->maxnumber; objnum++) {
		if (*objlist) {
			ac = *acl++;
			ac &= ACTIVE;
			if (ac == ACTIVE) {
				if (kcont->enh == NULL)
					enhobj = enhance(*objlist);
				else
					enhobj = varenhance(*objlist, kcont->enh);
				picframe(enhobj,kcont->f);
				if (enhobj != *objlist) {
					enhobj->plist = NULL;
					freeobj(enhobj);
				}
			}
		}
		objlist++;
	}
	colour(OVERLAY1);
	tsize(50,50);
	if (kcont->SI->blobs_toggle) dispblobs(kcont);
}

/*
 *	M E T H T N O R M A L I S E  --  Dark spol removal of metaphase (SLOWLY..)
 *
 */
methtnormalise()
{
	struct object **objlist, *nobj, *newgrey();
	SMALL *acl;
	int ac,objnum;
	
	colour(GREYCOLS);
	intens(1);
	objlist = (struct object **) kcont->ool;
	acl = kcont->acl;
	for (objnum = 0; objnum < kcont->maxnumber; objnum++) {
		if (*objlist) {
			ac = *acl++;
			ac &= ACTIVE;
			if (ac == ACTIVE) {
				nobj = newgrey(*objlist);
				if (kcont->enh == NULL)
					htnormalise(nobj,(float) 1.0, (float) 1.0);
				else
					htnormalise(nobj, (float)(kcont->enh->black)/10, 
												(float)(kcont->enh->white)/10);
				picframe(nobj,kcont->f);
				if (nobj != *objlist) {
					nobj->plist = NULL;
					freeobj(nobj);
				}
			}
		}
		objlist++;
	}
	colour(OVERLAY1);
	tsize(50,50);
	if (kcont->SI->blobs_toggle) dispblobs(kcont);
}

/*
 *	M E T R E S T O R E  --  Re-display (the slow way)
 *
 */
metrestore()
{
	struct object **objlist, *nobj, *newgrey();
	SMALL *acl;
	int ac,objnum;
	
	colour(GREYCOLS);
	intens(1);
	objlist = (struct object **) kcont->ool;
	acl = kcont->acl;
	for (objnum = 0; objnum < kcont->maxnumber; objnum++) {
		if (*objlist) {
			ac = *acl++;
			ac &= ACTIVE;
			if (ac == ACTIVE) {
				picframe(*objlist,kcont->f);
			}
		}
		objlist++;
	}
	colour(OVERLAY1);
	tsize(50,50);
	if (kcont->SI->blobs_toggle) dispblobs(kcont);
}


/*
 *	S I _ A T T A C H  --	SI needs allocating in km.c with everything else..]
 *							so how this little nasty happened - I don't know
 *
 */
SI_attach(kcont)
struct kcontrol *kcont;
{
	kcont->SI = &SI;
}

/*
 *	S E G M E N U 
 *
 */
segmenu(gotcell)
int	gotcell[];
{
	int Exit_cmd, Quit_cmd, Man_cmd, Flex_cmd = -2, i, j, holdnum = 0;
	int	(*linkerbug)(), currentspecies();
	static int zero = 0;
	char	tempfil[40], clstr[10], clinput[10];
	
	clinput[0] = '\0';
	if (alc->debug)
		fprintf(debug_channel,"In seg menu\n");
	SI_attach(kcont);
	if (!kcont->SI->initialised) {
		seg_control_init(kcont->SI);
		dispblobs(kcont);
	}
	/*
	 *	Draw operators attention to dodgy objects
	 */
	dispprobotypes(kcont);

	/*
	 *	Start feature measurement put back into chromlyse ih/cas
	 */
	startfmeas();
	/*
	 * set alc->m2s as structure, iselect for locate output, 0 as phase
	 * indicator (i.e., segmentation) 
	 */
	if (gotcell[1] == 1) {
		if ((kcont->current_obj > 0) && (kcont->current_obj < kcont->maxnumber) &&
					(kcont->acl[kcont->current_obj-1]))
	 		holdnum = kcont->current_obj;
	}
	initselect( alc->m2s, &kcont->current_obj, &cursor, 0);
	if (holdnum)
		kcont->current_obj = holdnum;
	else 
		for (kcont->current_obj = 1; 
		! kcont->acl[kcont->current_obj-1] && kcont->current_obj < kcont->maxnumber;
		++kcont->current_obj );
	visitobj(kcont->current_obj,kcont->ool,kcont,&cursor);
#ifdef UNUSED
	if (alc->fluor)
		setcolour(CL_REVERSE,CL_BLACK,0);
	else
		setcolour(alc->metacolour_1,alc->metacolour_2,0);
#endif
	trackcursor();
	/*
	 * count chromosomes, possible composites, and display
	 */
	for (kcont->SI->numchroms=0, i=0; i<kcont->maxnumber; i++)
		if ( (kcont->ool[i]->plist->otype != NOISE) && 
		   (kcont->ool[i]->plist->otype != NUCLEUS) && 
		   (kcont->acl[i] & ACTIVE) )    /* to be removed?? */
			kcont->SI->numchroms++;
	countdisp(kcont->SI->numchroms,1);

	if (alc->debug)
		fprintf(debug_channel, "SEGMENU - downmenu()\n");
	downmenu();
	newmenu ( "Segmentation");
	cmmd ("current object;%d",&kcont->current_obj);
	cmmd ("split composite;%g",splitup);mouse(LEFTBT);
	cmmd ("join;%?d%g",&kcont->SI->jselect,joinup);mouse(LONGRB);
	cmmd ("eliminate object;%g%+i",isnoise,0);mouse(LONGLB);
	cmmd ("restore chromosome;%-+d%f",&zero, ischromosome); mouse(MIDBT);
	cmmd ("species (human/mouse/chinham);%f", getspecies);
	cmmd ("display options;%+i%f",0,seg_display_opts);
	if (alc->options & OPT_MANKAR) {
		 cmmd ("manual karyotype");Man_cmd = mmcmd;
	}
	if (alc->options & OPT_FLEX) {
		cmmd ("flexible screen;%+i%f", 0, flex_Main);  /* segment: phase is 0 */
	}
	cmmd ("exit and karyotype"); Exit_cmd = mmcmd;mouse(LONGMB);
	if (alc->devel > 1)
		cmmd ("rewrite spread;file %s%f",tempfil,writemet);
	cmmd ("quit segment");Quit_cmd = mmcmd;

	while (1) {
		setextradisp(currentspecies);
		thismu();
		if (nncmd == Exit_cmd) {
			setextradisp(NULL);
			for (i=0; i<kcont->maxnumber; i++) {
				if ( (kcont->ool[i]->plist->otype == COMPOSITE) && 
				   (kcont->acl[i] & ACTIVE) )   {
					kcont->current_obj = i+1;
					ischromosome(&zero);
				 }
			} 
			endselect(&cursor,0);
			erasecursor();
			setmsv(0);
			if (alc->debug)
				fprintf(debug_channel, "SEGMENU - upmenu() - return(KARYOTYPE)\n");
			upmenu();
			return(KARYOTYPE);
		}
		if (nncmd == Man_cmd) {
			setextradisp(NULL);
			upmenu();
			return(MANKARY);
		}
		if ((nncmd == Quit_cmd) &&
			 check("Quit, are you sure (y/n) ? ")) {
			setextradisp(NULL);
			endselect(&cursor,0);
			erasecursor();
			setmsv(0);
			kcont->mstatus |= ABORT;
			waitfeats(kcont);
			upmenu();
			return(QUIT);
		} else if (nncmd > 1) {
			countdisp(kcont->SI->numchroms,1);
			trackcursor();
			doselect(kcont->ool,kcont,&cursor);
			displin(0,1);
			putprompt();
		} else if (nncmd == 1) {
			if ( kcont->current_obj < 0 || kcont->current_obj > kcont->maxnumber) {
				fprintf(output, "Out of range !\n");
				putprompt();				
			} else {
				if (!(kcont->acl[kcont->current_obj-1] & ACTIVE)) {
					fprintf(output,"Inactive !\n");
					putprompt();
				} else {
					erasecursor();
					visitobj(kcont->current_obj,kcont->ool,kcont,&cursor);
				}
			}
		}
	}
}


currentspecies()
{
	char *getspec(), *getcurclass();

	screengo(6,45);
	printf("CURRENT SPECIES   : %s   ",getspec());
	screengo(7,45);
	printf("CURRENT CLASS.TYPE: %s   ", getcurclass());
}


/*
 *	C H R O M _ T Y P E S T R -- Return chromosome classifier type string
 *	dependent on the integer value passed.
 *	SHORT = short, LONG = long, RBAND = R-band
 */
static char *
chrom_typestr(classifier_type)
int classifier_type;
{
	int i;

	for (i=0; i<NCTYPES; i++)
		if (ctypes[i].type == classifier_type) return(ctypes[i].str);
	return("unknown");
}

/*
 *	T Y P E S T R _ C H R O M -- Get chromosome number
 *	Given the chrom type, return the integer number
 */
static int
typestr_chrom(str)
char *str;
{
	int i, len, rval = -1;
	
	if (*str != '\0') { /* Search for type */
		len = strlen(str);
		for (i=0; i<NCTYPES; i++)
			if (!strncmp(str, ctypes[i].str, len)) {
				rval = ctypes[i].type;
				break;
			}
		*str = '\0';
	}
	return(rval);
}

/*
 *	S E G _ D I S P L A Y _ O P T S
 *
 *
 */
seg_display_opts(phase)
int phase;
{
	int Clback, cmd;
	int	(*linkerbug)();

	setextradisp(NULL);
	if (phase)
		newmenu("Re-segmentation and display options");
	else
		newmenu("Segmentation and display options");
	cmmd("use windows;:k50%t",&kcont->SI->windowstyle);
	cmmd("window scale;:k50<4-24>%d",&kcont->SI->wind_scale);
	cmmd("use auto-split facilities;:k50%t",&kcont->SI->autosplit_toggle);
	cmmd("use rubber-banding for splits;:k50%t",&kcont->SI->rubber_segment);
	cmmd("use rubber-banding for overlaps;:k50%t",&kcont->SI->rubber_overlap);
	cmmd("use rubber-banding for straightening;:k50%t",&kcont->SI->rubber_straight);
#ifdef UNUSED
	cmmd("background colour;%-+d%f",phase,colfn);
#endif
	linkerbug = clearplane;
	cmmd ("clear background;%-+d%+i%f",alc->ddgschannel, OVERLAY1 | OVERLAY2, linkerbug); Clback = mmcmd;
	if (phase == 0) {
		linkerbug = dispblobs;
		cmmd("display object serial numbers;:k50%+i%t%f",kcont,&kcont->SI->numbers_toggle,linkerbug);
		cmmd("display spots on objects;:k50%+i%t%f",kcont,&kcont->SI->blobs_toggle,linkerbug);
		linkerbug = disp_prob_noise;
		cmmd("highlight unresolved noise spots;:k50%t%+i%f",&kcont->SI->noise_toggle,kcont,linkerbug);
		cmmd("enhancement options;%+i%+i%f",SEGMENT, phase, enhanceopt);
		if (alc->devel){
			cmmd("renormalise display;%g",renormdisp);
		}
	}
	while (cmd = thismu()) {
		if (cmd == Clback) {
			if (phase == 0) {
				kcont->SI->numbers_toggle = 0;
				kcont->SI->blobs_toggle = 0;
				kcont->SI->noise_toggle = 0;
				displin(0, 9);
				displin(0, 10);
				displin(0, 11);
				putprompt();
			}
		}
	}
	setextradisp(currentspecies);

}

/*
 *	D I S P _ P R O B _ N O I S E  --
 *
 */
disp_prob_noise(toggle,kcont)
int *toggle;
struct kcontrol *kcont;
{
	register int objnum;
	for (objnum=0; objnum<kcont->maxnumber; objnum++) {
		if ((kcont->acl[objnum]&ACTIVE) && kcont->ool[objnum]->plist->otype == NOISE)
			checkattention(*toggle,objnum,kcont);
	}
}

/*
 *	S C R A P _ P R O B _ N O I S E  ---
 *
 */
scrap_prob_noise(kcont)
struct kcontrol *kcont;
{
	register int objnum;
	for (objnum=0; objnum<kcont->maxnumber; objnum++) {
		if ((kcont->acl[objnum]&ACTIVE) && kcont->ool[objnum]->plist->otype == NOISE) {
			checkattention(0,objnum,kcont);
			/* make it really noise */
			kcont->ool[objnum]->plist->Cotype = NOISE;
			kcont->ool[objnum]->plist->otconf = 100;
			kcont->acl[objnum] &= ~ACTIVE;
		}
	}
}

/*
 *	S E G E X I T C H E C K
 *
 * Assumption : if ACTIVE and type neither UNKNOWN nor CHROMOSOME
 * then uncertain object type, which will go into stack as abnormal
 * chromosome in karyogram.  But unresolved NOISE will be lost first.
 */
segexitcheck()
{
	register int i;
	register struct chromplist *plist;
	int uncertaincount = 0;
	scrap_prob_noise(kcont);
	for (i=0; i<kcont->maxnumber; i++) {
		if (kcont->acl[i] & ACTIVE) {
			plist = kcont->ool[i]->plist;
			if (plist->otype != UNKNOWN && plist->otype != CHROMOSOME)
				uncertaincount++;
		}
	}
	if (uncertaincount <= UNCERTAINMAX)
		return(1);
	else {
		fprintf(stderr,"TOO MANY DOUBTFUL TYPE OBJECTS UNRESOLVED\n");
		return(0);
	}
}




/*
 *	B D S E L E C T  --
 *
 */
static
bdselect()
{
	if (!checkobj(kcont->current_obj)) {
		return;
	}
	showbound(kcont->current_obj,1);
}

/*
 *	C L E A N
 *
 */
clean ( )
{
	i_clean();
	kcont->acl[kcont->current_obj-1] |= ACTIVE;
	if (alc->debug)
		fprintf(output,"Just reactivated object %d\n",kcont->current_obj);
}



/*
 *	T R A C K C U R S O R  --
 *
 */
static
trackcursor() {
	curscol(OVERLAY2);
	setmsv(chromcurs);
}

/*
 * M A N K A R Y -- Routine to do manual pick and place
 *
 */
mankary() {
	int Exit_cmd, Quit_cmd, displ;
	char str[25], tmpstr[25], optstr[25];

	displ = 1;
	*tmpstr = '\0';
	*str = '\0';
	strcpy(optstr, opt[displ]);
	ninclass = 2;
	chromnum = 1;
	subnum = 1;	
	countdisp(0, 0);
	dispchromnum(1);
	downmenu();
	newmenu("Manual Karyotype");
	cmmd("current object;%d", &kcont->current_obj);
	cmmd("chromosome number selection;:k40%25s%f", str, strint);
	cmmd("number in class;:k40%d", &ninclass);
	cmmd("display options;:k40%-+d%-s%?+s%f", &displ, optstr, tmpstr, options);
	cmmd("go on;%-+i%f",1, nxtnum);mouse(RIGHTBT);
	cmmd("select"); mouse(MIDBT);
	cmmd("backup;%f", backup); mouse(LEFTBT);
	cmmd("quit"); Quit_cmd = mmcmd;
	cmmd("exit"); mouse(LONGMB); Exit_cmd = mmcmd;
	
	while(1) {
		thismu();
		if (nncmd == Exit_cmd || nncmd == Quit_cmd) {
			dispchromnum(0);
			erasecursor();
			upmenu();
			if (nncmd == Exit_cmd)
					return(KARYOTYPE);
			else
				return(SEGMENT);
		}
		if (nncmd > 4) {
			displin(0,1);
			putprompt();
			trackcursor();
			doselect(kcont->ool,kcont,&cursor);
		} else if (nncmd == 1) {
			if ( kcont->current_obj < 0 || kcont->current_obj > kcont->maxnumber) {
				fprintf(output, "Out of range !\n");
				putprompt();				
			} else {
				if (!(kcont->acl[kcont->current_obj-1] & ACTIVE)) {
					fprintf(output,"Inactive !\n");
					putprompt();
				} else {
					erasecursor();
					visitobj(kcont->current_obj,kcont->ool,kcont,&cursor);
				}
			}
		}
		if (nncmd == 4) {
			displin(0,4);
			putprompt();
		}
		if (nncmd == 6) {
			selchromnum(displ);
		}
	}
}

/*
 * N X T N U M --
 *
 */
nxtnum(onoff)
int onoff;
{
	if (subnum >= ninclass || onoff) {
		switch (chromnum) {
			case 24 :
				chromnum++;
				subnum = 1;
				break;
			case 25 :
				if (onoff) {
					chromnum = 1;
					subnum = 1;
				} else 
					subnum++;
				if (subnum == 26)
					subnum = 1;
				break;
			default:
				chromnum++;
				subnum = 1;
  				break;
		}
	} else {
		subnum++;
		if (subnum > 26)
			subnum = 1;
	}
	dispchromnum(1);
}

/*
 * S E L C H R O M N U M
 *
 */
selchromnum(option)
int option;
{ 
int num;
char string[3];

	colour(OVERLAY1);
	switch(option) {
		case 1 :
			dispbound(kcont->ool[kcont->current_obj-1], kcont->f);
			break;
		case 2 :
			disphull(kcont->ool[kcont->current_obj-1], kcont->f);
			break;
		case 3 :
			disprect(kcont->ool[kcont->current_obj-1], kcont->f);
			break;
	}
	intens(0);
	makestring(string, kcont->ool[kcont->current_obj-1]->plist->Cpgroup, 0);
	frameobjstr(string, kcont->ool[kcont->current_obj-1], kcont->f);
	intens(1);
	makestring(string, chromnum, 0);
	frameobjstr(string, kcont->ool[kcont->current_obj-1], kcont->f);
	num = chromnum;
	ischromosome(&num);
	nxtnum(0);
}

/*
 * B A C K U P --
 *
 */
backup() { 
	if (chromnum != 1) {
		chromnum--;
		subnum = ninclass;
		dispchromnum(1);
	} else {
		if (subnum != 1)
			subnum--;
	}
}
	
/*
 * D I S P C H R O M N U M --
 *
 */
dispchromnum(onoff)
int onoff;
{
	static char string[4];

	tsize(130, 150);
	colour(OVERLAY2);
	intens(0);
	dispstring(string, 100, 100);
	if (onoff) {
		intens(1);
		makestring(string, chromnum, subnum);
		dispstring(string, 100, 100);
	}
}

/*
 * 	M A K E S T R I N G --
 *
 */
makestring(str, num1, num2)
char *str;
int num1, num2;
{
	char str2;
	
	if (num2 != 0)
		str2 = 'a' + num2 - 1;
	else 
		str2 = '\0';
	switch(num1) {
		case 23 :
			sprintf(str, "X%c", str2);
			break;
		case 24 :
			sprintf(str, "Y%c", str2);
			break;
		case 25 :
			sprintf(str, "A%c", str2);
			break;
		default :
			sprintf(str, "%d%c", num1, str2);
			break;
	}
}

/*
 * O P T I O N S --
 *
 */
static int options(number, thisopt, tmpstr)
int *number;
char *thisopt, *tmpstr;
{
	int i, j, ok;
	int opt_is[MAXOPTS];

	for (i=0; i < MAXOPTS; i++)
		opt_is[i] = 0;
	if (*number != 0)
		opt_is[*number-1] = 1;		
	if (*tmpstr != '\0') {
		for (j = 1; j <= MAXOPTS; j++) {
			if ((ok = strmatch(tmpstr, opt[j])) != 0)
				break;
		}
		if (ok > 0) {
			for (i=0; i < MAXOPTS; i++)
				opt_is[i] = 0;
			opt_is[j] = 1;
			*number = j;
			strcpy(thisopt, opt[j]);
		} else
			optmenu(number, thisopt);
	} else
		optmenu(number, thisopt);
	*tmpstr = '\0';
}

/*
 * O P T M E N U --
 *
 */
optmenu(number, thisopt)
int *number;
char *thisopt;
{
	int i, cmd;
	int opt_is[MAXOPTS];
	
	for (i=0; i < MAXOPTS; i++)
		opt_is[i] = 0;
	if (*number != 0)
		opt_is[*number-1] = 1;
	newmenu("Display Options");
	cmmd("boundaries;:k40%-t", &opt_is[mmcmd]);
	cmmd("hulls;:k40%-t", &opt_is[mmcmd]);
	cmmd("rectangles;:k40%-t", &opt_is[mmcmd]);
	while (cmd = thismu()) {
		if (cmd > 0) {
			*number = cmd;
			for (i=0; i < MAXOPTS; i++)
				opt_is[i] = 0;
			opt_is[cmd-1] = 1;
			mukont->mubust = 1;
		}
	}
	strcpy(thisopt, opt[*number]);
}
/*
 * S T R I N T --
 *
 */
strint(str)
char *str;
{
	int number;
	char character, *strbeg;
	
	strbeg = str;
	character = *str;
	if (*str == 'X' || *str == 'x') {
		subnum = 1;
		chromnum = 23;
		dispchromnum(1);
	} else if (*str == 'Y' || *str == 'y') {
		subnum = 1;
		chromnum = 24;
		dispchromnum(1);
	} else if (*str == 'A' || *str == 'a') {
		subnum = 1;
		chromnum = 25;
		dispchromnum(1);
	} else if (isdigit(character) != 0 || character == '-' || character == '+') {
		number = atoi(str);
		if (number > 0 && number < 26) {
			subnum = 1;
			chromnum = number;
			dispchromnum(1);
			while(isdigit(*str) || *str == '-' || *str == '+')
				str++;
		} else {
			printf("%d is not a valid number!\n",number);
			return(0);
		}
	}
	str = strbeg;
	switch(chromnum) {
		case 23 :
			sprintf(str, "Xa");
			break;
		case 24 :
			sprintf(str, "Ya");
			break;
		case 25 :
			sprintf(str, "Aa");
			break;
		default :
			sprintf(str, "%da", chromnum);
			break;
	}
}
/*
 *  S T R M A T C H -- String comparison taking no notice of case.
 *
 */
strmatch(s1,s2)
char *s1,*s2;
{
	int match;

	match = 1;
	while (*s1 && *s2) {
		if (toupper(*s1 & 0x7f) != toupper(*s2 & 0x7f)){
			match = 0;
			break;
		};
		s1++; s2++;
	}
	return(match);
}

