/*
 * segfixbis.c Jim Piper/Denis Rutovitz 29/12/85
 * adapted from:  Jim Piper's segfuncs2.c, seginteract.c and interact.c
 *
 * Modifications
 *
 *     May 1992		Joanne	make an extra object of the bit(s) leftover
 *				after manual splitting of an overlap
 *   1 May 1992		Joanne	free objects and boundaries if there are too 
 *				many objects in unscramble()
 *  29 Apr 1992		Joanne  display boundary after extraction of axes
 *   7 Jan 1992		Joanne	updated activelist[] and moved ck_newpseg() and 
 *				ck_bkpseg(),as in version 9.5,from segfix.c to this module
 *  23 Feb 1989		SEAJ		Modified bkpseg for nvertices correction.
 *  21 Feb 1989		SEAJ		Added parameter ignore to call to dispbig.
 *	15 Feb 1989		CAS/SEAJ	Cope with poly_split return status if too many
 *								objects.
 *								Removed deactivation of orig object from splitbegin
 *   7 Feb 1989		SEAJ		modified use of nvertices and initialised vtx[0]
 *								in splitbegin.
 *	02 Feb 1989		SCG/SEAJ	ensure manual overlap checks for object high
 *								water mark (MAXOBJS)
 *	02 Dec 1988		SCG/dcb		fix logic for previous mod
 *	24 Nov 1988		SCG/dcb		give available objptr array length to poly_split
 *	23 Nov 1988		SCG/dcb		prevent j_clean from undrawing empty pdom
 *	18 Apr 1988		SEAJ		Changed gotpnow so that noise objects are 
 *								no longer counted.
 */
#include <stdio.h>
#include <kyy.h>		/* contains wstruct.h, allcontr.h */
#include <chromanal.h>
#include <interact.h>
#include <kcontrol.h>
#include <colour.h>
#include <display.h>
#include <seg_control.h>
#include <allcontr.h>

#define OVER_MAXLEFT 200

/*
 * externals defined in other modules
 */
extern struct allkontr *alc;
extern struct kcontrol *kcont;
extern FILE *debug_channel;		/* Debug channel file pointer */
extern struct ipoint cursor;
extern int track2();


/*
 *	S P L I T B E G I N  --  begin to split a composite
 *
 */
splitbegin(obbjnum)
int		obbjnum;
{
	struct chromosome *obj,**objlist;
	struct intervaldomain *idom;
	struct object **marklist, *makemain();
	struct polygondomain *pdom, *makepolydmn();
	SMALL *activelist;
	struct pframe *f, *dispbig();
	
	objlist = kcont->ool;
	marklist = (struct object **) kcont->bol;
	f = kcont->f;
	activelist = kcont->acl;

	kcont->SI->objnum = obbjnum-1;
	kcont->SI->eggno = 0;			/* in case we're dealing with an overlap */
	
	obj = objlist[kcont->SI->objnum];
	idom = obj->idom;
	intens(0);
	colour(OVERLAY1);
	picframe(marklist[kcont->SI->objnum],f);
	frameobjnum(kcont->SI->objnum+1,obj,f);
	erasecursor();
	intens(1);
	/*
	 * karyfix.c plants zero in kcont->SI->windowstyle if doing windowed
	 * resegmentation, to prevent double windowing - ouch !
	 */
	if (kcont->SI->windowstyle) {
		f = dispbig(objlist[kcont->SI->objnum],f,0,kcont->SI->wind_scale, 1);
		kcont->SI->framewas = kcont->f;
		kcont->f = f;
		cursframe(0);
	}
	curscol(OVERLAY1);
	if (farcursor(obj->idom) ) 
		showcursor((idom->line1+idom->lastln)/2,(idom->kol1+idom->lastkl)/2);
	else  showcursor( cursor.l, cursor.k);
	pdom = makepolydmn(1,NULL,0,IPOLSIZE,1);
	kcont->SI->polyobj = makemain(10,pdom,NULL,NULL,NULL);
	activelist[kcont->SI->objnum] &= ~ACTIVE;
	kcont->SI->arc.k2 = pdom->vtx[0].vtX = cursor.k;
	kcont->SI->arc.l2 = pdom->vtx[0].vtY = cursor.l;
}

/*
 *	U N S C R A M 1  --  segment an overlap: - this begins in same way
 *						 as split ...
 */
#define WIDTH 8
unscram1() {
	register struct chromosome **objlist;
	struct chromosome *extractov();
	register struct chromplist *plist;
	struct chromplist *makechromplist();
	struct object *ibound(),*makemain();
	struct polygondomain *pdom;
	register int i;
	
	if (alc->debug) {
		fprintf(debug_channel,"*** unscram1()\n");
		fflush(debug_channel);
		Extractov_debug(debug_channel);
	}
	objlist = kcont->ool;
	i = kcont->maxnumber + kcont->SI->eggno;
/* extract object and picture results */
	objlist[i] = extractov(objlist[kcont->SI->objnum],kcont->SI->polyobj,WIDTH);
	if (alc->debug) {
		fprintf(debug_channel,"Done extractov\n");
		fflush(debug_channel);
		Extractov_debug(NULL);
	}
	i_clean();
	if (objlist[i] == NULL) {
		printf("Not part of this object\007\n");
		if (alc->debug)
			fprintf(debug_channel,"NULL object from extractov\n");
	} else if (wzemptyidom(objlist[i]->idom)) {
		freeobj(objlist[i]);
		printf("Not part of this object\007\n");
		if (alc->debug)
			fprintf(debug_channel,"Empty object from extractov\n");
	} else {
		objlist[i]->plist = makechromplist();
		plist = objlist[i]->plist;
		plist->number = i+1;
		plist->pnumber = kcont->SI->objnum+1;
		plist->Cotype = CHROMOSOME;
		plist->otype = CHROMOSOME;
		plist->history[0] = 2;	/* result of manual extraction from overlap */
		colour(OVERLAY2);
		kcont->SI->boundlist[kcont->SI->eggno] = ibound(objlist[i],1);
		picframe(kcont->SI->boundlist[kcont->SI->eggno],kcont->f);
		kcont->SI->eggno++;
	}
	pdom = makepolydmn(1,NULL,0,IPOLSIZE,1);
	kcont->SI->polyobj = makemain(10,pdom,NULL,NULL,NULL);
 }

/*
 *	D O N T U N S C R A M B L E
 *
 * If interaction killed, need to tidy up
 *
 */
dontunscramble()
{
	register int i;
	register struct chromosome **objlist;
	
	objlist = kcont->ool;
	intens(0);
	colour(OVERLAY2);
	for (i=0; i<kcont->SI->eggno; i++) {
		picframe(kcont->SI->boundlist[i],kcont->f);
		freeobj(kcont->SI->boundlist[i]);
/*		objlist[kcont->maxnumber+i]->vdom = NULL;*/
		freeobj(objlist[kcont->maxnumber+i]);
	}
}
 

/*
 *	U N S C R A M B L E  --  Unscramble overlap
 *
 * 15/5/86 make changes if kcont->SI->eggno is zero , ie nothing has been done
 */
unscramble()
{
 	register int i;
	register struct chromosome **objlist, *unionn(), *diff, *d1;
	register struct object **marklist;
	struct object *blobmark(), *ibound(), *diffdom();
	register struct chromplist *qlist, *makechromplist();

	if (alc->debug)
		fprintf(debug_channel,"*** unscramble()\n");	
	objlist = kcont->ool;
	marklist = (struct object **) kcont->bol;

	i_clean(); 	
	w_clean();
	if ((kcont->SI->eggno) && 
			(kcont->SI->eggno + kcont->maxnumber <= MAXOBJS)) { 
		kcont->acl[kcont->SI->objnum] &= ~ACTIVE;
		qlist = objlist[kcont->SI->objnum]->plist;
		qlist->Cotype = OVERLAP;
		qlist->otype = OVERLAP;
		qlist->otconf = 100;
		qlist->history[1] = 2;	/* manually segmented */
		qlist->history[2] = kcont->SI->eggno;	/* count */
		for (i=kcont->maxnumber; i<kcont->maxnumber+kcont->SI->eggno; i++) {
			kcont->oth[i].bound = ibound(objlist[i],1);
			colour(OVERLAY2);
			picframe(kcont->oth[i].bound, kcont->f);
			
			marklist[i] = blobmark(objlist[i]);
			colour(OVERLAY1);
			picframe(marklist[i],kcont->f);
			if (kcont->SI->numbers_toggle)
				frameobjnum(i+1,objlist[i],kcont->f);
			kcont->acl[i] |= ACTIVE|INTERACTING|IDENTIFY1;
		}
		for (i=0; i<kcont->SI->eggno; i++) {
			freeobj(kcont->SI->boundlist[i]);
		}
		
		d1 = objlist[kcont->SI->objnum];
		for (i=0; i<kcont->SI->eggno; i++) {
			diff = (struct chromosome *) diffdom (d1,objlist[kcont->maxnumber + i]);
			if (i > 0)
				freeobj(d1);
			d1 = diff;
		}
		diff->vdom = objlist[kcont->SI->objnum]->vdom;
		objlist[kcont->SI->objnum]->vdom->linkcount++;
		kcont->maxnumber += kcont->SI->eggno;
		objlist[kcont->maxnumber] = diff;		
		
		if ((kcont->SI->eggno > 0) && 
		    (area(objlist[kcont->maxnumber]) > OVER_MAXLEFT)) {
			objlist[kcont->SI->objnum]->plist->history[2]++;
			i = kcont->maxnumber;
			objlist[i]->plist = makechromplist();
			qlist = objlist[i]->plist;
			qlist->number = i+1;
			qlist->pnumber = kcont->SI->objnum+1;
			qlist->Cotype = UNKNOWN;
			qlist->otype = UNKNOWN;
			probotype(objlist[i]);
			qlist->history[0] = 2;

			intens(1);
			marklist[i] = blobmark(objlist[i]);			
			colour(OVERLAY1);
			picframe(marklist[i],kcont->f);
			if (kcont->SI->numbers_toggle)
				frameobjnum(i+1,objlist[i],kcont->f);
			kcont->acl[i] = ACTIVE|INTERACTING|IDENTIFY1;

			colour(OVERLAY2);
			kcont->oth[i].bound = ibound(objlist[i],1);
			picframe(kcont->oth[i].bound,kcont->f);
			kcont->SI->eggno++;
			kcont->maxnumber++;
		}
		else 
			freeobj(diff);
		return(kcont->SI->eggno);
	} else {
		if(kcont->SI->eggno + kcont->maxnumber > MAXOBJS)
			fprintf(stderr, "Too many objects! Overlap failed.\007\n");
		for (i=0; i<kcont->SI->eggno; i++) {
			freeobj(kcont->SI->boundlist[i]);
			freeobj(kcont->ool[kcont->maxnumber +i]);
		}
		intens(1);
		curscol(OVERLAY2);
		return(1);						/* we are leaving an object on 
											the screen */
	};
}


/*
 *	G O T P N O W  --  we have got the split polygon. Go ahead and do it
 *
 */
gotpnow()
{
	register int i,kc,lc;
	int k, num;
	register struct chromosome **objlist;
	struct chromosome *obj;
	struct object **marklist, *blobmark();
	register SMALL *activelist;
	struct pframe *f;
	register struct chromplist *qlist;
	struct chromplist *makechromplist();

	num = 0;
	if (alc->debug)
		fprintf(debug_channel,"In gotpnow\n");
	j_clean();
	w_clean();
	objlist = kcont->ool;
	obj = objlist[kcont->SI->objnum];
	marklist = (struct object **) kcont->bol;
	f = kcont->f;
	activelist = kcont->acl;

	kc = cursor.k;
	lc = cursor.l;
	if (kcont->SI->polyobj == NULL) {
		picframe(marklist[kcont->SI->objnum],f);
		colour(OVERLAY1);
		if (kcont->SI->numbers_toggle)
			frameobjnum(kcont->SI->objnum+1,obj,f);
	} else {			 /* split original and picture results */
		if (alc->debug) {
			fprintf(debug_channel,"Before poly_split\n");
			Polysplit_debug(debug_channel);
		}
		if (poly_split(obj, kcont->SI->polyobj, &objlist[kcont->maxnumber], 
				(MAXOBJS - kcont->maxnumber), &k,1) == 0) {
			fprintf(stderr,"Too many objects! Split failed.\007\n");
		} else {
			activelist[kcont->SI->objnum] &= ~ACTIVE;
			if (alc->debug) {
				Polysplit_debug(NULL);
				fprintf(debug_channel,"Poly - deactivate %d\n",kcont->SI->objnum+1); /* kcont->SI->objnum 0->n-1 */
			}
			qlist = (struct chromplist *)obj->plist;
			qlist->otype = COMPOSITE;
			qlist->Cotype = COMPOSITE;
			qlist->otconf = 100;
			qlist->history[1] = 1;	/* manually split */
			qlist->history[2] = k;
			for (i=kcont->maxnumber; i<kcont->maxnumber+k; i++) {
				kcont->oth[i].bound = ibound(objlist[i],1);
				colour(OVERLAY2);
				picframe(kcont->oth[i].bound,f);
				marklist[i] = blobmark(objlist[i]);
				colour(OVERLAY1);
				picframe(marklist[i],f);
				if (kcont->SI->numbers_toggle)
					frameobjnum(i+1,objlist[i],f);
				qlist = makechromplist();
				objlist[i]->plist = qlist;
				qlist->number = i+1;
				qlist->pnumber = kcont->SI->objnum+1;
				qlist->otype = UNKNOWN; 	/* not necessarily chromosome */
				qlist->Cotype = UNKNOWN;
				probotype(objlist[i]);
				if ( qlist->otype != NOISE)
					num = num + 1;
				activelist[i] = ACTIVE | INTERACTING | IDENTIFY1;				
/*				checkattention(1,i,kcont);*/
				colour(OVERLAY1);
				qlist->history[0] = 1;	/* result of manual split */
			}
			kcont->maxnumber += k;
		}
		freeobj(kcont->SI->polyobj);
		kcont->SI->polyobj = NULL;
	}
	erasecursor();
	return(num);
}





/*
 *	C K _ B K P S E G
 *
 */
ck_bkpseg()
{
	if (kcont->SI->state & CUT ) {
		if ((kcont->SI->state&TOUCH) && kcont->SI->rubber_segment)
			bkpseg(1);
		else if ((kcont->SI->state&OVLP) && kcont->SI->rubber_overlap)
			bkpseg(1);
		else if ((kcont->SI->state&STRT) && kcont->SI->rubber_straight)
			bkpseg(1);
		else
			bkpseg(5);
	}
}

/*
 *	C K _ N E W P S E G  --  Start or Start / bend
 *
 */
ck_newpseg()
{
	if (kcont->SI->state & ENTRY ) {
		newpseg();
		kcont->SI->state |= CUT ;
		kcont->SI->state &= ~ENTRY;
	}
}




/*
 *	N E W P S E G  --  start new polygon segment
 *
 */
newpseg() {
	register int kol,lin;
	register struct polygondomain *pdom = (struct polygondomain *)kcont->SI->polyobj->idom;
	kol = cursor.k;
	lin = cursor.l;
	pdom->vtx[pdom->nvertices].vtX = kol;
	pdom->vtx[pdom->nvertices].vtY = lin;
	pdom->nvertices++;
	kcont->SI->arc.k1 = kcont->SI->arc.k2 = kol;
	kcont->SI->arc.l1 = kcont->SI->arc.l2 = lin;
}
	
/*
 *	B K P S E G  --  back up a bit
 *
 */
bkpseg(nseg)
register int nseg;
{
	register struct polygondomain *pdom = (struct polygondomain *)kcont->SI->polyobj->idom;
	register struct ivertex *vtx = pdom->vtx + pdom->nvertices - 1;
	colour (OVERLAY1);
	intens(0);
	for ( ; pdom->nvertices>0 && nseg>0; nseg--) {
		picframe(&kcont->SI->arc,kcont->f);
		kcont->SI->arc.k2 = kcont->SI->arc.k1;
		kcont->SI->arc.l2 = kcont->SI->arc.l1;
		if (--pdom->nvertices > 0)
			vtx--;
		kcont->SI->arc.k1 = vtx->vtX;
		kcont->SI->arc.l1 = vtx->vtY;
	}
	intens(1);
	showcursor(kcont->SI->arc.l2,kcont->SI->arc.k2,OVERLAY2);
}

track2()
{
	trackn(1);
}

track3()
{
	trackn(-2);
}

trackn(scale)
{
	register struct polygondomain *pdom = (struct polygondomain *)kcont->SI->polyobj->idom;
	if (alc->m2s->moving) trackx(scale);			/* in cursmove.c */
	pdom->vtx[pdom->nvertices].vtX = kcont->SI->arc.k2 = cursor.k;
	pdom->vtx[pdom->nvertices].vtY = kcont->SI->arc.l2 = cursor.l;
	if ((++pdom->nvertices + 1) > IPOLSIZE) {
		fprintf(stderr,"Polygon too big\n");
		--pdom->nvertices;
		return;
	}
	colour (OVERLAY1);
	picframe(&kcont->SI->arc,kcont->f);
	kcont->SI->arc.k1 = kcont->SI->arc.k2;
	kcont->SI->arc.l1 = kcont->SI->arc.l2;
}

/*
 *	R U B B E R _ T R A C K 2  --  Rubber-band mouse tracking functions
 *
 */
rubber_track2()
{
	rubber_trackn(1);
}

/*
 *	R U B B E R _ T R A C K N  --
 *
 */
rubber_trackn(scale)
{
	register struct polygondomain *pdom = (struct polygondomain *)kcont->SI->polyobj->idom;
	register struct ivertex *vtx = pdom->vtx + pdom->nvertices;
	if (alc->m2s->moving) trackx(scale);			/* in cursmove.c */
	if (pdom->nvertices >= 1) {
		colour (OVERLAY1);
		intens(0);
		picframe(&kcont->SI->arc,kcont->f);
	}
	vtx->vtX = kcont->SI->arc.k2 = cursor.k;
	vtx->vtY = kcont->SI->arc.l2 = cursor.l;
	if (pdom->nvertices >= 1) {
		intens(1);
		picframe(&kcont->SI->arc,kcont->f);
	}
}

/*
 *	S I Z E _ L E F T  --  Calculate size of object left after averlap split
 *
 */
size_left()
{
	struct	chromosome **objlist, *merged, *unionn();
	struct object *diff, *diffdom();
	int		left;

	return (0);
	objlist = kcont->ool;
	merged = unionn(objlist[kcont->maxnumber],kcont->SI->eggno,NULL);
	diff = diffdom(merged,objlist[kcont->SI->objnum]);
	left = area(diff);
	freeobj(diff);
	freeobj(merged);
	return(left);
}

/*
 *	A C T I V A T I O N 
 *
 */
activation(objnum,activity) {
	if (activity==ACTIVE)
		kcont->acl[objnum-1] |= ACTIVE;
	else
		kcont->acl[objnum-1] &= ~ACTIVE;
}



i_clean()
{
	j_clean();
	if (kcont->SI->polyobj) {
		freeobj(kcont->SI->polyobj);
		kcont->SI->polyobj = NULL;
	}
}

static int
j_clean()
{
	register struct polygondomain *pdom = (struct polygondomain *)kcont->SI->polyobj->idom;
	int i;
	/*
	 * undraw only if non-null pointer and non-empty pdom
	 */
	if (wzemptypdom(kcont->SI->polyobj) == 0) {
		intens(0);
		colour (OVERLAY1);
		picframe(kcont->SI->polyobj,kcont->f);
		fflush(stderr);
	}
	intens(1);
}

w_clean()
{
    if ( kcont->SI->windowstyle ) {
    	erasecursor();
    	putbackwindow();
		kcont->f = kcont->SI->framewas;
		cursframe(0);
		curscol(OVERLAY2);		/* shows the cursor in restored frame */
    }
}



/*
 *	U N D O  --  Undo an interaction
 *
 */
undo(objnum)
{
	if (!checkobj(objnum)) return;
	return(segundo(kcont,objnum-1));
}


/*
 *	J O I N
 *
 */
join(j0,j1)
register int j0,j1;
{
	register struct chromosome *obj,**objlist;
	struct object **marklist;
	struct pframe *f;
	
	if (!checkobj(j0)) return;
	if (!checkobj(j1)) return;
	objlist = kcont->ool;
	marklist = (struct object **) kcont->bol;
	f = kcont->f;
	obj=objlist[j0];
	intens(0);
	colour(OVERLAY1);
	picframe(marklist[j0-1],f);
	frameobjnum(j0,obj,f);
 	obj=objlist[j1];
	picframe(marklist[j1-1],f);
	frameobjnum(j1,obj,f);
	segjoin(kcont,j0,j1);
}

/*
 *
 * noise or nucleus ... t = 0, noise, 1, nucleus
 *
 */
noise(objnum,t) {
	register int prevotype;
	register struct chromosome **objlist;
	register SMALL *activelist;
	register struct chromplist *qlist;
	
	/* delete attention marker if present (beware effect on colour) */
	checkattention(0,objnum-1,kcont);
	objlist = kcont->ool;
	activelist = kcont->acl;
	if (!checkobj(objnum)) return(0);
	--objnum;
	activelist[objnum] &= ~ACTIVE;
	qlist = objlist[objnum]->plist;
	prevotype = qlist->otype;
	qlist->Cotype = (t ? NUCLEUS: NOISE);
	qlist->otype = qlist->Cotype;
	qlist->otconf = 100;
	qlist->history[2] = 0;
	/* return amount by which count to be reduced */
	if (prevotype <= OVERLAP)
		return(1);
	else
		return(0);
}

/*
 * join two objects
 */
segjoin(kcont,j0,j1)
struct kcontrol *kcont;
{
	struct chromosome **objlist, *uobjl[2], *unionn();
	struct object **marklist;
	struct chromplist *plist, *qlist, *makechromplist();
	SMALL *activelist;
	struct pframe *f;
	int i,j,k;
	
	objlist = kcont->ool;
	marklist = (struct object **) kcont->bol;
	f = kcont->f;
	activelist = kcont->acl;
	
	intens(0);
	for (j=0; j<=1; j++) {
		i = j==0? j0: j1; 
		i--;
		picframe(marklist[i],f);
		frameobjnum(i+1,objlist[i],f);
		activelist[i] &= ~ACTIVE;
		qlist = objlist[i]->plist;
		qlist->otype = 4;	/* piece */
		qlist->Cotype = 4;	/* piece */
		qlist->otconf = 100;
		qlist->history[1] = 4;	/* manually joined */
		qlist->history[2] = 0;
		uobjl[j] = objlist[i];
	}
	intens(1);
	objlist[kcont->maxnumber] = unionn(uobjl, 2, 1);
	marklist[kcont->maxnumber] = blobmark(objlist[kcont->maxnumber]);
	picframe(marklist[kcont->maxnumber],f);
	if (kcont->SI->numbers_toggle)
		frameobjnum(kcont->maxnumber+1,objlist[kcont->maxnumber],f);
	objlist[kcont->maxnumber]->plist = makechromplist();
	plist = (struct chromplist *)objlist[kcont->maxnumber]->plist;
	plist->number = kcont->maxnumber+1;
	plist->pnumber = j0;		/* 1st parent */
	plist->history[2] = j1;	/* 2nd parent */
	plist->otype = 1; 	/* chromosome */
	plist->otconf = 100;
	plist->history[0] = 4;	/* result of manual join */
	activelist[kcont->maxnumber] = ACTIVE;
	kcont->maxnumber++;
}

/*
 * undo an interaction
 */
segundo(kcont,objnum)
struct kcontrol *kcont;
{
	int i,k,parent,numchange;
	struct chromosome **objlist;
	struct object **marklist;
	struct chromplist *plist,*qlist;
	SMALL *activelist;
	struct pframe *f;
	
	objlist = kcont->ool;
	marklist = (struct object **) kcont->bol;
	f = kcont->f;
	activelist = kcont->acl;
	
	plist = objlist[objnum]->plist;
	numchange = 0;
	switch (plist->otype) {
	case 1:
		/*
	 	* find if derived object, find parent
	 	*/
		parent = plist->pnumber;
		if (parent <= 0) {
			fprintf(stderr,"NOT a valid derived object\n");
			return(0);
		}
		/*
	 	 * find other derived objects and permanently de-activate
	 	 */
		intens(0);
		for (i=0; i<kcont->maxnumber; i++) {
			if (objlist[i]->plist->pnumber == parent) {
				picframe(marklist[i],f);
				frameobjnum(i+1,objlist[i],f);
				activelist[i] &= ~ACTIVE;
				if (objlist[i]->plist->otype <= 1)
					numchange--;
			}
		}
		/*
	 	 * re-activate and re-display parent
	 	 */
		intens(1);
		parent--;
		qlist = objlist[parent]->plist;
		qlist->history[1] = 0;
		qlist->otype = qlist->Cotype = 1;
		picframe(marklist[parent],f);
		if (kcont->SI->numbers_toggle)
			frameobjnum(parent+1,objlist[parent],f);
		activelist[parent] |= ACTIVE;
		numchange++;
		/*
		 * if a join, get other parent
		 */
		if (plist->history[0] == 4) {
			parent = plist->history[2] - 1;
			qlist = objlist[parent]->plist;
			qlist->history[1] = 0;
			qlist->otype = qlist->Cotype = 1;
			picframe(marklist[parent],f);
			if (kcont->SI->numbers_toggle)
				frameobjnum(parent+1,objlist[parent],f);
			activelist[parent] |= ACTIVE;
			numchange++;
		}
		return(numchange);

	case 0:
	case 2:
	case 3:
	case 4:
	case 5:
	default:
		fprintf(stderr,"Object %d now assumed to be a chromosome\n",objnum+1);
		plist->otype = 1;
		activelist[objnum] = 1;
		return(1);
	}
}

