/*
 * splitup.c
 *
 * Extracted from original segfix.c 12 Jan 1988
 *
 * Note the split between "splitup" and "subsplit", and in
 * particular the kcont->SI->windowstylesave handling (see
 * karyfix.c and "oversplit" in segfix.c).
 *
 * MODIFICATIONS
 * -------------
 * Joanne 29-04-92   	Made sure that boundary is removed to be redrawn after splitting
 * Joanne 13-02-92	All the splitting, "touch" and "overlap", can be done
 *			while staying inside the window
 * Joanne 09-01-92	unbox() after window is deleted
 * Joanne 07-01-92      doautosplit() as in  new version 9.7, but splitup() as
 *			in old version
 * jimp 30-5-91		call chromcurs() after autosplit success exit
 * joanne 23-10-91	extra parameter for dispbig() added; 
 * jimp 10-02-88	Unbox before split, not before window.
 * jimp 05-02-88	Unbox before window.
 * jimp 22-01-88	More tidying.
 * jimp 21-01-88	Menu rearrangement.  Pay no attention to
 *			auto-split toggle, since fully controlled here.
 * jimp 13-01-88	Remain in window after auto-split.
 *
 */

#include <cmdsys.h>
#include <wstruct.h>
#include <allcontr.h>
#include <chromanal.h>
#include <interact.h>
#include <kcontrol.h>
#include <colour.h>
#include <display.h>
#include <seg_control.h>

#define OVER_MAXLEFT	200	/* Max amount allowd to be left in overlap
				split before we assume an object has been
				missed */

/*
 * EXTERNALS
 */
extern struct allkontr *alc;
extern struct kcontrol *kcont;


/*
 * EXTERNAL functions defined for menus etc.
 */
extern track1(),track2(),rubber_track2(),chromcurs();
extern ck_newpseg(),ck_bkpseg();
extern dontunscramble();

/*
 * FORWARD declarations
 */
int axmsg();


/*
 *	S P L I T U P  --  Split two touching chromosomes
 *
 */
splitup() 
{
#define CO_cmd 1
#define MS_cmd 2
#define AS_cmd 3
#define MO_cmd 4
#define AO_cmd 5
#define EX_cmd 6
	register int i,objnum;		/* should be parameter */
	int stat,which,prevmaxobjs,windowstylesave;
	struct pframe *f, *dispbig();

	setmsv(0);
	erasecursor();
	objnum = kcont->current_obj - 1;
	if (!checkobj(objnum+1)) {
		quitmenu();
		return;
	}
	prevmaxobjs = kcont->maxnumber;
	if(kcont->acl[objnum]&ACTIVE!=ACTIVE){
		fprintf(output,"Inactive !\n");
		quitmenu();
		return;
	}
	windowstylesave = kcont->SI->windowstyle;
	/* delete attention marker if present (beware effect on colour) */
	checkattention(0,objnum,kcont);
	intens(1);	/* may have been set 0 by checkattention */
	unbox();  	/* remove identifying box before displaying window */
	if (windowstylesave) {
		f = dispbig(kcont->ool[objnum],kcont->f,0,kcont->SI->wind_scale, 1);
		kcont->SI->framewas = kcont->f;
		kcont->SI->objnum = objnum;
		kcont->f = f;
		cursframe(0);
		kcont->SI->windowstyle = 0;
	}
	inset(kcont,objnum);
	kcont->identify_set = IDENTIFY1;
		
	newmenu("split touching objects");
	cmd (CO_cmd,"current object;%d",&kcont->current_obj);
	cmd (MS_cmd,"manual split touch"); mouse(LEFTBT);
	cmd (AS_cmd,"autosplit touch"); mouse(RIGHTBT);
	cmd (MO_cmd,"manual split overlap"); mouse(LONGMB);
	cmd (AO_cmd,"autosplit overlap"); mouse(LONGRB);
	cmd (EX_cmd,"exit from split subsystem"); mouse(MIDBT);
	while (1) {
		thismu();
		switch(nncmd) {
			case AS_cmd:
				kcont->SI->state = TOUCH | ENTRY ;	
				downmenu();
				stat = doautosplit(COMPOSITE);
				quitmenu();
				upmenu();
				if (stat) {
					chromcurs();
					setmsv(chromcurs);
					break;
				}
				/* else fall into manual split touch */
			case MS_cmd:
				objnum = kcont->current_obj - 1;
				downmenu();
				subsplittouch(objnum);
				quitmenu();
				upmenu();
				setmsv(chromcurs);
				break;
			case AO_cmd:
/*				i_clean();	
				dontunscramble();  */
				kcont->SI->state = OVLP | ENTRY;
				downmenu();
				stat = doautosplit(OVERLAP);
				upmenu();
				if (stat) {
					chromcurs();
					setmsv(chromcurs);
					break;
				}
				/* else fall into manual split overlap */
			case MO_cmd:
				objnum = kcont->current_obj - 1;
				downmenu();
				subsplitoverlap(objnum);
				quitmenu();
				upmenu();
				setmsv(chromcurs);
				break;
			case EX_cmd:
				kcont->SI->windowstyle = windowstylesave;
				w_clean();
				erasecursor();
				kcont->identify_set = 0;
			 	setmsv(0);
				inclear(kcont,objnum);
				colour(OVERLAY1);
				for (i=prevmaxobjs; i<kcont->maxnumber; i++) {
					kcont->acl[i] &= ~(INTERACTING|IDENTIFY1);
					if (kcont->acl[i] & ACTIVE) {
						if (kcont->SI->blobs_toggle)
							picframe(kcont->bol[i],kcont->f);
						if (kcont->SI->numbers_toggle)
							frameobjnum(i+1,kcont->ool[i],kcont->f);
						checkattention(1,i,kcont);
						colour(OVERLAY1);
					}
				}
				quitmenu();
				return;
			default:
				break;
		}
	}
}

subsplittouch(objnum)
register int objnum;
{
	int Start_cmd, Back_cmd, Extr_cmd, Quit_cmd;
	register struct polygondomain *pdom;
	struct object *boundary;
	char title[60];

	/* delete attention marker if present (beware effect on colour) */
	checkattention(0,objnum,kcont);
	/*
	 * Erase object boundary in OVERLAY2 plane
	 */
	boundary = kcont->oth[objnum].bound;
	if (boundary != NULL) {
		colour(OVERLAY2);	/* was OVERLAY3 */
		intens(0);
		picframe(boundary,kcont->f);
	}
	splitbegin(objnum+1);
	kcont->SI->state = TOUCH | ENTRY ;
	pdom  = (struct polygondomain *)kcont->SI->polyobj->idom;
	curscol(OVERLAY1);
	if (kcont->SI->rubber_segment) {
		pdom->nvertices = 0;
		setmsv(rubber_track2);
	} else
		setmsv(track1);
	if (alc->debug) {
		printf("Manual split begin on %d\n",kcont->current_obj);
	}
	sprintf ( title, "Split Composite Object %d\n", kcont->current_obj);
	newmenu( title );
	if (kcont->SI->rubber_segment) {
		cmmd("start/bend"); Start_cmd = mmcmd; mouse(LEFTBT);
	} else {
		cmmd("start;%g%",ck_newpseg); Start_cmd = mmcmd; mouse(LEFTBT);
	} 
	cmmd("back up;%g",ck_bkpseg); Back_cmd = mmcmd; mouse(LONGRB);
	cmmd("o.k. - extract"); Extr_cmd = mmcmd; mouse(MIDBT);
	cmmd("forget it");Quit_cmd = mmcmd; mouse(LONGLB);
	while (1) {
		thismu();
		if (nncmd == 0 || nncmd == Quit_cmd) {		/* forget it and return */
			clean();
			return;
		}
		if (nncmd == Start_cmd) {
			curscol(OVERLAY2);
			if (kcont->SI->rubber_segment) {
				pdom->vtx[pdom->nvertices].vtX = kcont->SI->arc.k1 = kcont->SI->arc.k2;
				pdom->vtx[pdom->nvertices].vtY = kcont->SI->arc.l1 = kcont->SI->arc.l2;
				pdom->nvertices++;
				kcont->SI->state = CUT | TOUCH;
			}
			else {
				if (alc->debug)
					printf("Cursor now track2 0x%x\n",track2);
				setmsv(track2);
			}
		}
		if (nncmd == Extr_cmd) {
			if (kcont->SI->rubber_segment) {
				if (kcont->SI->arc.k2 != kcont->SI->arc.k1 || kcont->SI->arc.l2 != kcont->SI->arc.l1)
					pdom->nvertices++;
			}
			if(kcont->SI->state & CUT) {
				kcont->SI->numchroms += gotpnow() - 1;
				return;
			} else {		
				clean();
				return;
			}
		}
	} /* end while() */
}


subsplitoverlap(objnum)
register int objnum;
{
	int Start_cmd, Back_cmd, Acpt_cmd, Extr_cmd, Quit_cmd;
	register struct polygondomain *pdom;
	struct object *boundary;
	char title[60];

	/* delete attention marker if present (beware effect on colour) */
	checkattention(0,objnum,kcont);
	/*
	 * Erase object boundary in OVERLAY2 plane
	 */
	boundary = kcont->oth[objnum].bound;
	if (boundary != NULL) {
		colour(OVERLAY2);	/* was OVERLAY3 */
		intens(0);
		picframe(boundary,kcont->f);
	}
	splitbegin(objnum+1);
	kcont->SI->state = OVLP | ENTRY ;
	pdom  = (struct polygondomain *)kcont->SI->polyobj->idom;
	curscol(OVERLAY1);
	if (kcont->SI->rubber_overlap) {
		pdom->nvertices = 0;
		setmsv(rubber_track2);
	} else
		setmsv(track1);
	kcont->SI->axchrom = 1;
	if (alc->debug) {
		printf("Manual split overlap begin on %d\n",kcont->current_obj);
	}
	sprintf ( title, "Split Composite Object %d\n", kcont->current_obj);
	newmenu( title );
	if (kcont->SI->rubber_overlap) {
		cmmd("start/bend"); Start_cmd = mmcmd; mouse(LEFTBT);
	} else {
		cmmd("start;%g%",ck_newpseg); Start_cmd = mmcmd; mouse(LEFTBT);
	} 
	cmmd("back up;%g",ck_bkpseg); Back_cmd = mmcmd; mouse(LONGRB);
	cmmd("accept axis"); Acpt_cmd = mmcmd; mouse(MIDBT);
	cmmd("o.k. - extract"); Extr_cmd = mmcmd; mouse(LONGMB);
	cmmd("forget it");Quit_cmd = mmcmd; mouse(LONGLB);
	while (1) {
		thismu();
		if (nncmd ==0 || nncmd == Quit_cmd) {			/* forget it and return */
			dontunscramble();
			clean();
			return;
		}
		if (nncmd == Start_cmd) {
			curscol(OVERLAY2);
			if (kcont->SI->rubber_overlap) {
				pdom->vtx[pdom->nvertices].vtX = kcont->SI->arc.k1 = kcont->SI->arc.k2;
				pdom->vtx[pdom->nvertices].vtY = kcont->SI->arc.l1 = kcont->SI->arc.l2;
				pdom->nvertices++;
				kcont->SI->state = CUT | OVLP;
			}
			else {
				if (alc->debug)
					printf("Cursor now track2 0x%x\n",track2);
				setmsv(track2);
			}
		}
		if (nncmd == Acpt_cmd) {
			if (kcont->SI->state & CUT) {
				if (kcont->SI->rubber_overlap) {
					if (kcont->SI->arc.k2 != kcont->SI->arc.k1 || kcont->SI->arc.l2 != kcont->SI->arc.l1)
						pdom->nvertices++;
				}
				unscram1();
				/*
				 * Note that unscram1() makes new
				 * polygon (yuckkkk).
				 */
				pdom = (struct polygondomain *)kcont->SI->polyobj->idom;
				curscol(OVERLAY1);
				if (kcont->SI->rubber_overlap) {
					pdom->nvertices = 0;
					setmsv(rubber_track2);
				} else
					setmsv(track1);
				++kcont->SI->axchrom;
				axmsg();
				kcont->SI->state=OVLP|ENTRY;
			}
		}
		if ( nncmd == Extr_cmd) {
	 		if (kcont->SI->state & CUT) 
	 			fprintf(stderr,"accept axis (M) ??\007\n");
	 		else if (size_left() > OVER_MAXLEFT) 
 				fprintf(stderr,"Please select all objects\007\n");
	 		else {
				kcont->SI->numchroms += unscramble()-1;
				return;
			}
	 	}
	} /* end while() */
}


/*
 *	A X M S G  --  Ask user to draw next chrom axis
 *
 */
static
axmsg() {
	screengo ( 16,0);
	printf( "Draw axis of %d%s chromosome",kcont->SI->axchrom,
		kcont->SI->axchrom==1?"st": (kcont->SI->axchrom==2?"nd": (kcont->SI->axchrom==3?"rd":"th")));
	putprompt();
}


/*
 *	D O A U T O S P L I T  --
 *
 */
doautosplit(type)
int		type;
{
#define Acpt_cmd 1
#define Rejt_cmd 2
	char title[60];
	struct object *boundary;
	
	boundary = kcont->oth[kcont->current_obj - 1].bound;
	if (boundary != NULL) {
		colour(OVERLAY2);	
		intens(0);
		picframe(boundary,kcont->f);
	}
	/* delete attention marker if present (beware effect on colour) */
	checkattention(0,kcont->current_obj-1,kcont);
	setmsv(NULL);
	erasecursor();
	printf("Autosplitting, please wait...");fflush(stdout);
	if (kcont->SI->autosplit_toggle && autosplit(kcont,type)) {
		printf("\n");
		sprintf(title,"Results of auto-split of object %d\n",kcont->current_obj);
		newmenu(title);
		cmd(Acpt_cmd,"accept auto-split result");mouse(MIDBT);
		cmd(Rejt_cmd,"reject auto-split result");mouse(LONGLB);
		while (1) {
			switch(thismu()) {
				case Acpt_cmd:
					kcont->SI->numchroms += endautosplit(kcont,type) - 1;
					return(1);
				case Rejt_cmd:
					cleanautosplit(kcont);
					return(0);
			}
		}
	} else
		printf("\n");
	return(0);		/* failed */
}

