/*
 *      T A P E U T I L . C  --  User frendly interface to tape handling
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *	Extracted from Utilities.c by CAS
 *
 *  Date:    9th March 1987
 *
 *  Modifications
 *
 *  27 Jun 1991		ih			Make pointer indexing secure in nextfile
 *	16 Dec 1989		dcb			Tape extract changes to alc->dir before extract
 *	30 Oct 1989	CAS/dcb			pipestdin/pipestdout, open pipe for read/write
 *	 1 Jun 1989		CAS			Def errno for v2.2
 *	21 Jun 1989		CAS			Force eot before wfm + call directly
 *								rather than via system call to mt. This should
 *								reduce problems with fm's at start of tapes
 *	23 Jul 1989		CAS			Extra param to remove
 *	 6 Jul 1988		CAS			Tidy cmmd lines
 *	 1 Jun 1988		CAS			'C' to 'c' in menu (confirm each case)
 *	21 Apr 1988		CAs			Slight fix to access tests in extract..
 *	30 Mar 1988		CAS			Check prompt and allow extract overwrite
 *	11 Feb 1988		CAS			Changed path pipeing to restore std fd's quickly
 *	25 Jan 1988		CAS			Mods for nrw cmdsys
 */
#include <cmdsys.h>
#include <kyy.h>
#include <modes.h>
#include <direct.h>
#include <dir.h>

#define TAPE_PREFIX		"filemark_1.0"
#define MAGTAPE	"mt0"
#define MAGTAPEDEV	"/mt0"

extern int errno;
extern FILE *debug_channel;

static	int		Tape_cmd = 0;			/* Menu command lines */

extern	char	workingfile[];			/* Used in menues to display
											file being processed */

extern int iscyto();
extern int setcase();
extern int show_dir();
static int tape_case(), tape_rew(), tape_dir(), tape_extract();
static int quitfn();

/*
 *	T A P E _ M E N U  --  Main tape menu
 *
 */
tape_menu()
{
	static int	confirm = 1;
	char	workbuf[40], casename[CASELENGTH+1];
	int	(*linkerbug)();

	workbuf[0] = '\0';
	strcpy(casename,alc->nameids.case);
	newmenu ("tape options");
	linkerbug = setcase;
	cmmd ("case ID;:k30%+s%-17s%f",workbuf,alc->nameids.case,linkerbug);
	cmmd ("confirm each file;:k30%T",&confirm);
	linkerbug = show_dir;
	cmmd ("show working directory;:k30%-+s%f",alc->dir,linkerbug);
	cmmd ("show tape directory;%f",tape_dir);
	cmmd("copy case to tape;:k30%?+s%x%-s%f",casename,alc->dir,workingfile,
												tape_case);Tape_cmd = mmcmd;
	cmmd ("extract case from tape;%?+s%-+d%f",casename,&confirm,tape_extract);
	cmmd ("rewind tape;%f",tape_rew);

	while (thismu())
		strcpy(casename,alc->nameids.case);
	Tape_cmd = 0;
}


/*
 *	T A P E _ C A S E  --  Copy all files relating to given case to tape
 *
 *	Currently no test if tar fails etc..
 */
tape_case(casename,dirname)
char	*casename, *dirname;
{
	DIR		*dp;
	struct	direct *dirent;
	int		tarstdin;
	int		typ, nfiles, pid, status, prefix, mt;
	char	buffer[40];
	char	cur_dir[100], pref_file[100];
	struct cyto_file cur_file;
	static	char tar[]    = "tar",
				 tarcmd[] = "-eb=150";

	if (!tapeinstalled()) return(0);
	fprintf(stderr,"Tape %s %s\n",casename, dirname);
	if (Tape_cmd) displin(0,Tape_cmd);
	screengo(ENDLINE,0);
	nfiles = 0;
	if (dp = opendir(dirname)) {
 		if ((strlen(dirname)+strlen(TAPE_PREFIX)+2) < 100) {
 			sprintf(pref_file,"%s/%s\n",dirname,TAPE_PREFIX);
			prefix = touch(pref_file);
			if (alc->debug)
				printf("Prefix file %s\n",pref_file);
		} else {
			if (alc->debug)
				printf("No prefix file\n");
			prefix = 0;
		}
/*
 *	Get current dir so we can return at end - Only tar needs to be
 *	in directory with the files in so that files are saved pathless
 */
		if (getwd(cur_dir,100) == 0) cur_dir[0] = '\0';
		else if (alc->debug)
			fprintf(stderr,"Dir was %s\n",cur_dir);
		chdir(dirname);
		pipestdin();
		pid = os9fork(tar,strlen(tarcmd)+1,tarcmd,0,0,0,128);
		tarstdin = unpipestdin();
/*
 *	Change back to old dir - dont need it anymore
 */
 		if (cur_dir[0])
			chdir(cur_dir);
		if (pid > 0) {
			if (alc->debug)
				fprintf(stderr,"Tar pid %d\n",pid);
			if (prefix) {
				write(tarstdin,pref_file,strlen(pref_file));
			}
			readdir(dp);						/* Skip '.' and '..' */
			readdir(dp);
			while(dirent = readdir(dp)) {
				if (typ = iscyto(dirent->d_name,&cur_file)) {
					if (strcasecmp(cur_file.case,casename) == NULL) {
						sprintf(buffer,"%s\n",dirent->d_name);
						if (icanread(dirname,dirent->d_name)) {
							write(tarstdin,buffer,strlen(buffer));
							nfiles++;
							showfile(&cur_file,Tape_cmd);
						} else {
							screengo(ENDLINE,0);
							fprintf(output,"File %s %s is read protected\007\n",
									cur_file.case,makefilestring(&cur_file,workingfile));
							workingfile[0] = '\0';
						}
					}
				}
			}
		}
		close(tarstdin);
		closedir(dp);
		while (wait(&status) != pid)
			;
		if (prefix) {
			remove("",pref_file);
		}
		if (status != 0)
			fprintf(stderr,"Backup failed exit status %d\n", status);
		else
			fprintf(stderr,"Number of files backed up %d\n",nfiles);
		if ((mt = mtopen(MAGTAPEDEV)) != -1) {
			if (mtskip(mt,3,1) != 0)
				mtwritefm(mt,1);
			mtclose(mt);
		}
	} else
		fprintf(stderr,"Failed to open %s\n",dirname);
	testchildren();		/* Check for dead offspring.. */
}

/*
 *	T A P E _ E X T R A C T  --  Extract case from tape
 *
 */
tape_extract(casename,confirm)
char	*casename;
int		*confirm;
{
	int		pid, tarstdin, tarstdout, stat;
	struct	cyto_file cur_file;
	static	char	tar[] = "tar",
					tarcmd[] = "-xqheob=150",
					cr = '\015', yes = 'y';
	char	cur_dir[100], filename[30], filestring[40];
	if (!tapeinstalled()) return(0);
	vclear(ENDLINE);
	screengo(1,20);
	printf("Extract Case from Tape");
	scrollpart(5,ENDLINE);
	screengo(5,0);
/*
 *	Find current dir & return afterwards
 */
	if (getwd(cur_dir,100) == 0) cur_dir[0] = '\0';
	else if (alc->debug)
		fprintf(stderr,"Dir was %s\n",cur_dir);
	chdir(alc->dir);

	pipestdin();
	pipestdout();
	pid = os9fork(tar,strlen(tarcmd)+1,tarcmd,0,0,0,128);
	tarstdin = unpipestdin();
	tarstdout = unpipestdout();
	while (stat = nextfile(tarstdout,&cur_file)) {
		if (stat == 1) {
			if (!strcasecmp(casename,cur_file.case)) {
				makefilename(filename,&cur_file);
				makefilestring(&cur_file,filestring);
				if (!access(filename,0x0)) {			/* File exists */
					if (!access(filename,0x2)) {		/* And I have permission to overwrite */
						printf ("File %s %s exists - overwrite ?",cur_file.case,filestring);
						if (tryboth(1))
							write (tarstdin,&yes,1);
					} else {
						printf ("File %s %s exists and is protected  - Can't overwrite\n",cur_file.case,filestring);
					}
				} else if (*confirm) {
					printf ("Extract %s %s ?",cur_file.case,filestring);
					if (tryboth(1))
						write (tarstdin,&yes,1);
				} else {
					printf ("Extracting %s %s\n",cur_file.case,filestring);
					write (tarstdin,&yes,1);
				}
			} else {
				if (alc->devel || alc->debug) {
					printf("Skipping file %s %s\n",cur_file.case,filestring);
				}
			}
		} else if (stat== -1) {
			if (alc->debug)
				printf("Non cyto file\n");
		}
		write (tarstdin,&cr, 1);
	}
	close(tarstdin);
	close(tarstdout);
/*
 *	Change back to old dir
 */
	if (cur_dir[0])	chdir(cur_dir);
	go_on();
	vclear(ENDLINE);
	scrollpart(OUTLINE,ENDLINE);
	setdisplay(BUST);
}

/*
 *	N E X T F I L E  --
 */
nextfile(inpath,cur_file)
int	inpath;
struct	cyto_file *cur_file;
{
	char	buffer[200], *ptr, *nend, *index();
	char *lim=buffer+199;

	ptr = buffer;
	read(inpath,ptr,1);ptr++; /* set up buffer */
	while ((ptr<lim)&&read(inpath,ptr,1)) {
		if ((*ptr == ' ') && (*(ptr-1) == '?')) {
			*(ptr+1) = '\0';
			nend = index(buffer,':');
			if (nend) *nend = '\0';
			ptr = buffer;
			while ((ptr<lim)&&(*ptr!='\015'))
				ptr++;
			while ((ptr<lim)&&(*ptr<'A'))
				ptr++;
			if (alc->debug)
				printf("Buffer %s\n",buffer);
			if (iscyto(buffer,cur_file)) {
				return(1);
			} else {
				return(-1);
			}
		} else
			ptr++;
	}
	return(0);
}

/*
 *	T A P E _ R E W  --  Rewind tape
 *
 */
tape_rew()
{
	int		mt, ret;

	ret = 0;
	if (!tapeinstalled()) 
		return(0);
	if ((mt = mtopen(MAGTAPEDEV)) != -1) {
		ret = 1;
		if (mtrewind(mt) == 0) {
			screengo(ENDLINE,2);
			fprintf(stderr,"Tape rewind failed - error %d\n",errno);
			ret = 0;
		}
		mtclose(mt);
	} else {
		screengo(ENDLINE,2);
		fprintf(stderr,"Can't access mag tape\n");
	}
	return(ret);
}

/*
 *	T A P E _ D I R  --  Show tape directory
 *
 */
tape_dir()
{
	struct cyto_file cur_file;
	char	buffer[100], *last, *rindex();
	int		tarstdout, pid, status, nc;
	static	char tar[]    = "tar",
				 tarcmd[] = "-dheb=150";

	if (!tapeinstalled()) return(0);
	vclear(ENDLINE);
	screengo (1,20);
	printf("Tape Directory");
	scrollpart(5,ENDLINE);
	screengo(5,0);

	tape_rew();
	pipestdout();
	pid = os9fork(tar,strlen(tarcmd)+1,tarcmd,0,0,0,128);
	tarstdout = unpipestdout();
/*
 * Loose first 2 lines of header from tar 
 */
	readln(tarstdout,buffer,100);
	readln(tarstdout,buffer,100);
/*
 *	read file names
 */
	while ((nc = readln(tarstdout,buffer,100)) > 0) {
		buffer[nc-1] = '\0';				/* Stomp on '\n' */
		if (iscyto(buffer,&cur_file)) {
			switch(cur_file.type) {
				case METFILE:
					printf("Case %s slide %s (metaphase list)\n",
						cur_file.case,cur_file.slide);
					break;
				case KARYFILE:
					printf("Case %s slide %s cell %d (karyotype)\n",
						cur_file.case,cur_file.slide,cur_file.cellno);
					break;
				case CELLFILE:
					printf("Case %s slide %s cell %d\n",
						cur_file.case,cur_file.slide,cur_file.cellno);
					break;
			}
		} else {
			if ((last = rindex(buffer,'/')) != NULL) {
				if (strncmp(last+1,TAPE_PREFIX,8) == NULL) {
					printf("\n");
				}
			}
		}
	}
	close(tarstdout);
	while (wait(&status) != pid)
		;
	if (status != 0)
		fprintf(stderr,"Tape read failed - error %d\n",status);
	tape_rew();

	go_on();
	vclear(ENDLINE);
	scrollpart(OUTLINE,ENDLINE);
	setdisplay(BUST);
}


/*
 * T A P E I N S T A L L E D
 */
static
tapeinstalled()
{
	char	*ptr;

	if ((ptr = (char *) modlink(MAGTAPE,0)) == (char *) -1) {
		screengo(ENDLINE,2);
		fprintf(stderr,"Tape software not installed\n");
		return(0);
	}
	munlink(ptr);
	return(1);
}


/*
 *	P I P E S T D I N  --  Set path 0 to be a pipe
 */
static
pipestdin()
{
	int		stdinpath;

	setquitfn(quitfn);			/* NEED TO DISABLE ^C HERE */
	if ((stdin->_fd != 0) && (alc->debug || alc->devel))
		fprintf(stderr,"Warning - paths confused\n");
	stdinpath = dup(0);
	close(0);
	open("/pipe", S_IREAD | S_IWRITE);
	stdin->_fd=stdinpath;
	return(0);
}

/*
 *	U N P I P E S T D I N  --  Restore stdin path to 0 and return dup of
 *								current path 0
 */
static
unpipestdin()
{
	int	stdinpath, retpath;

	retpath = dup(0);
	close(0);
	stdinpath = dup(stdin->_fd);
	close(stdin->_fd);
	stdin->_fd = stdinpath;
	setquitfn(NULL);		/* NEED TO RE_ENABLE ^C HERE */
	return(retpath);
}

/*
 *	P I P E S T D O U T  --  Make path 1 a pipe
 *
 */
static
pipestdout()
{
	int		stdoutpath;

	if ((stdout->_fd != 1) && (alc->debug || alc->devel))
		fprintf(stderr,"Warning - paths confused\n");
	stdoutpath = dup(1);
	close(1);
	open("/pipe", S_IREAD | S_IWRITE);
	stdout->_fd=stdoutpath;
	return(1);
}

/*
 *	U N P I P E S T D O U T  --  Restore stdout to path 1 and return
 *								 dup of current path 1
 */
static
unpipestdout()
{
	int	stdoutpath, retpath;

	retpath = dup(1);
	close(1);
	stdoutpath = dup(stdout->_fd);
	close(stdout->_fd);
	stdout->_fd = stdoutpath;
	return(retpath);
}


/*
 *	Q U I T F N  --  Local - null quit function to disable interrupt
 */
static
quitfn()
{
}

