/*
 *      T R A N S F E R . C  --  Inter machine data transfer
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *  Date:    21st March 1988
 *
 *  Modifications
 *
 *	 1 Jun 1989		CAS			Def errno for v2.2
 */
#include <stdio.h>
#include <allcontr.h>
#include <transfer.h>
#include <sgstat.h>
#include <ctype.h>
#include <modes.h>

#define SYSDATA		"/dd/params/kmrdat"
#define TRANSFILE	"transfer"

struct sysdata {
		int		trans;		/* Next number to use for file transfer */
	};

extern int errno;
extern struct allkontr *alc;
extern FILE *debug_channel;

/*
 *	Local forward declarations
 */
char	*expandseg();
char	*astring();
#define TIMEOUT		250		/* 5 seconds */

/*
 *	T R A N S F E R  --
 *
 */
transfer(casename,dirname)
char	*casename,*dirname;
{
	int		cp, stat;
	struct	sgbuf sgsave;
	char	tmpbuf[128];
	struct	transfer *trans;

	if ((trans = alc->trans) == NULL) {
		fprintf(stderr,"No Remote Transfer Data\n");
		return(0);
	}
	sprintf(tmpbuf,"xmode %s baud=%d",trans->line, trans->baud);
	system(tmpbuf);
	if ((cp = open(trans->line,S_IREAD|S_IWRITE|S_ISHARE)) < 0) {
		if (alc->debug)
			fprintf(debug_channel,"Failed to open transfer line, error %d\n",errno);
		fprintf(stderr,"Failed to open remote transfer line\n");
		return(0);
	}
	raw(cp,&sgsave);
	if (makereport(casename,dirname,tmpbuf)) {
		if (trans->startup) stat = mumble(cp,trans->startup,alc->debug);
		if (stat && senddata(tmpbuf) && trans->closedown)
			mumble(cp,trans->closedown,alc->debug);
	} else {
		fprintf(stderr,"Failed to generate case summary file\n");
	}
	cook(cp,&sgsave);
	close(cp);
}

/*
 *	M A K E R E P O R T  --  generate report file to send
 *
 */
makereport(casename,dirname,reportfile)
char	*casename,*dirname,*reportfile;
{
	int		fd;
	FILE	*fp;
	struct sysdata sysdat;

	if ((fd = open(SYSDATA,3)) == -1) {
		if ((fd = create(SYSDATA,2,0x1b)) == -1)
			return(0);
		else {
			sysdat.trans = 1;
		}
	} else {
		read(fd,&sysdat,sizeof(struct sysdata));
		if (++sysdat.trans > 999) sysdat.trans = 1;
	}
	lseek(fd,0L,0);
	write(fd,&sysdat,sizeof(struct sysdata));
	close(fd);
	sprintf(reportfile,"%s.%03d",TRANSFILE,sysdat.trans);
	if ((fp = fopen(reportfile,"w")) == NULL)
		return(0);
	report(casename,dirname,fp,1);
	fclose(fp);
	return(1);
}

/*
 *	S E N D D A T A  --  Send case summary report file using kermit
 *
 */
senddata(reportfile)
char	*reportfile;
{
	char	buffer[80];
	sprintf(buffer,"kermit -sl %s %s",alc->trans->line,reportfile);
	fprintf(stderr,"Transfering case summary file\n");
	system(buffer);
	sprintf(buffer,"kermit -ql %s",alc->trans->line);
	system(buffer);
	unlink(reportfile);
	return(1);
}

/*
 *	M U M B L E  --
 *
 */
mumble(cp,command,dlevel)
int		cp, dlevel;
char	*command;
{
	char	buffer[128], *cptr;
	int		ssize;
	while (command = expandseg(command,buffer,&ssize)) {
		if (ssize == 0) continue;
		write(cp,buffer,ssize);
		if (dlevel) fprintf(debug_channel,"Sent: %s\n",astring(buffer,ssize));
		if ((command = expandseg(command,buffer,&ssize)) != NULL) {
			if (ssize == 0) continue;
			if (dlevel) fprintf(debug_channel,"Want: %s\n",astring(buffer,ssize));
			if (!matchseg(buffer,ssize,cp,dlevel)) {
				fprintf(stderr,"Remote line handshake failed\n");
				return(0);
			}
		}
	}
	return(1);
}

/*
 *	M A T C H S E G  --  Ultra simplistic char matcher
 *
 */
matchseg(mptr,len,cp,dlevel)
char	*mptr;
int		len,cp,dlevel;
{
	char	buffer[100], *bptr, *end;
	int		time, cnt, i;
	end = mptr+len;
	time = 0;
	if (dlevel) fprintf(debug_channel,"Got: ");
	while (mptr < end) {
		while ((cnt = _gs_rdy(cp)) <= 0) {
			tsleep(2);
			if (++time > TIMEOUT) {
				if (dlevel) fprintf(debug_channel," - Failed\n");
				return(0);
			}
		}
		time = 0;
		if (cnt >= 100) cnt = 99;
		read(cp,buffer,cnt);
		for (i=0; i<cnt; i++)
			buffer[i] &= 0x7f;
		if (dlevel) fprintf(debug_channel,"%s",astring(buffer,cnt));
		bptr = buffer;
		while (cnt--) {
			if (*bptr == *mptr)
				mptr++;
			bptr++;
		}
	}
	if (dlevel) fprintf(debug_channel,"\n");
	return(1);
}

/*
 *	E X P A N D S E G  --
 *
 */
char	*expandseg(cptr,bptr,ssize)
char	*cptr, *bptr;
int		*ssize;
{
	char	*bstart;
	bstart = bptr;	
	if ((*cptr == '\0') || (*cptr == '-')) {
		return(NULL);
	}
	while (*cptr && (*cptr != '-')) {
		if (*cptr == '\\') {
			switch(*++cptr) {
				case 'n':
					*bptr++ = '\n';
					cptr++;
					break;
				default:
					if (isxdigit(*cptr)) {
						*bptr++ = atox(cptr);
						cptr++;
					} else
						*bptr++ = *cptr++;
			}
		} else
			*bptr++ = *cptr++;
	}
	*ssize = bptr-bstart;
	if (*cptr == '-') cptr++;
	return(cptr);
}

static atox(ptr)
char	*ptr;
{
	int		val;
	val = (*ptr > 9) ? *ptr - '7' : *ptr - '0';
	ptr++;
	val = val * 256 + (*ptr > 9) ? *ptr - '7' : *ptr - '0';
	return(val);
}

/*
 *	A S T R I N G  --  Generate a readable ascii string from one with
 *						control chars in it.
 *	NB - Has a static expansion buffer..
 */
char	*astring(ptr,len)
char	*ptr;
int		len;
{
	static	char	buffer[128];
	char	*bptr;
	bptr = buffer;
	while ((bptr < &buffer[127]) && len--) {
		*ptr &= 0x7f;
		if (*ptr < ' ') {
			*bptr++ = '^';
			*bptr++ = *ptr+'@';
		} else if (*ptr == 0x7f) {
			*bptr++ = '^';
			*bptr++ = '?';
		} else
			*bptr++ = *ptr;
		ptr++;
	}
	*bptr = '\0';
	return(buffer);
}
