/*
 *	A F E _ C T R L . H -- Struct defns & defines for new analogue front end
 *
 *
 *  Written: David Burgess
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1989)
 *
 *  Date:    23rd January 1989
 *
 *  Modifications
 *	10 Feb 1989	dcb		Remove Variable max_range, and introduce fixed MAX_RANGE
 *
 */

/*
 * Hardware bit positions
 */
#define EXIST_GAIN	3	/* Bits 0,1 Existing gain setting */
#define ADD_GAIN	4	/* Bit 2 Additional gain setting */
#define INT_DEN		4	/* Bit 2 Intensity/Density */
#define INTERP		8	/* Bit 3 Interpolate on/off */
#define TEST_SIG	3	/* Bits 0,1 Test signal source */
/*
 *	Boards
 */
#define VIDEO		1	/* Referencing video card */
#define AUTO_FOCUS	2	/* Referencing auto focus card */
/*
 * Mode definitions, these represent bit positions within
 * a register.
 */
#define	RAMP_IP		0	/* Test ramp input */
#define	VIDEO_IP	3	/* Video input (normal) with const. level test sig */
#define	LEVEL_IP	2	/* Constant level test input */
#define INTERP_OFF	8	/* Turn off interpolating h/ware */
#define INTERP_ON	0	/* Turn on interpolating h/ware */
#define DENSITY		0	/* Select density mode */
#define INTENSITY	4	/* Select intensity mode */
/*
 *	Miscellaneous
 */
#define DEF_GAIN	4.6	/* Default gain setting */
#define AFE_PE		1	/* Function parameter error */
#define AFE_ME		2	/* Miproc error */
#define AFE_CI		3	/* Can't init afe struct */
#define MAX_RANGE	230	/* Scale up to this range (safe) */
/*
 *	Gain, available dynamic range, and control bit structure
 */
struct gain_range {
	int gain;			/* Real gain * 10 */
	int	control_bits;	/* Bit pattern to give this gain */
};

/*
 *	Analogue front end 'ghost' register contents
 */
struct analogue_front_end {
	int	su_cm_0_3,	/* Sub_unit 0, command 3	*/
		su_cm_2_2,	/* Sub_unit 2, command 2	*/
		su_cm_2_3,	/* Sub_unit 2, command 3	*/
		su_cm_2_6,	/* Sub_unit 2, command 6	*/
		su_cm_2_7;	/* Sub_unit 2, command 7	*/
	float gain;		/* Actual gain value 		*/
};


