/*
 *      C A P T U R E . C  --  Linear array image capture
 *
 *
 *  Written: G J Page
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *  Date:    19th June 1986
 *
 *	This version taken from the file digitise.c and modified to give
 *	simpler menu text.  Calling sequence is the same as digitise.
 *
 *	Modifications
 *
 *	24 Jun 1991		CAs		Parameterise setoffsets
 *	21 Feb 1991		CAS		Remove redundant + erroneous alloc_digbuf call
 *	15 Jan 1991		CAS/GJP		debug_channel used when not open
 *  11 Jul 1990		ih		Set alc->digitised in arraydigitise for Sc
 *	27 Mar 1990		dcb		Don't alter density setting in stray. Print message
 *							if move to next met without filing. Init. step size
 *							in arraydigitise. Don't flag image in if
 *							digitisation failed. Set linear lut before display
 *							for thresholding. Display message in stray to
 *							differentiate from main field
 *	23 Mar 1990		dcb		test_gen under #ifdef TEST, clear display in dobox
 *							if no captured image
 *  15 Mar 1990		ih		Removed filed=TRUE from freecell and use alc->cellin
 *                          for cell loaded to fix memory free bug after filing cell
 *	12 Mar 1990		dcb		lin_colour_lut instead of fixrgb.
 *	07 Mar 1990		dcb		Swap dig_type assign, and cellheadfill call
 *	15 Dec 1989		dcb		Calc. frame scale for main fuse field too.
 *							Removed	digspeed from devel. Free fusion images
 *	14 Dec 1989		dcb		flush after "Starting digitisation scan ... "
 *	23 Oct 1989		dcb		Put checkoverwrite back in, display message	during
 *							scan, reset boxbgn before cell display. Don't free
 *							cell after filing so it can be used for adjusting
 *							capture area, just flag the fact it's filed 
 *	10 Oct 1989		dcb			Pass disp_dig a box structure pointer
 *								stray doesn't set frame ox, oy
 *	 4 Sep 1989		dcb			setaframe: ox/oy set to 0
 *	31 May 1989		dcb			Yet more debug
 *	25 May 1989		dcb			Make next_met work, and do on <Return> as well
 *								filepic does all filing work. Keep record of
 *								old case, and reset cells filed if it changes
 *								copy alc->nameids to bay nameids. Removed
 *								checkoverwrite as this is done later
 *	23 May 1989		dcb			More debug out
 *	22 May 1989		dcb			Put in next_met function. Chg cell focus msg
 *	19 May 1989		GJP			Take out next_met line
 *	18 May 1989		dcb			Extensive re-write of most bits to include blue
 *								wash, and field fusion on array capture. Top
 *								level menu is now arraydigitise
 *	14 Mar 1989		dcb			Rip out kickkcont, digitise and count (Bounded
 *								by ifdef GGG's). Make all functions except
 *								setoffsets, init_offset, and capture non-static
 *	20 Feb 1989		dcb			erase correct histogram
 *  10 Feb 1989		dcb			For new afe card, 1st histo. done with sh corr.
 *								off, and always at calib gain. Shading corr.
 *								is ALWAYS re-done.
 *	09 Feb 1989		dcb			init dhrange to DEFDHRANGE. #define DEFDHRANGE 6
 *								for new afe card (old cards unaffected by this)
 *	08 Feb 1989		dcb			Trivial, add LL mouse for exit in Image capture
 *	24 Jan 1989		dcb			Add new code for new analogue front end
 *   2 Nov 1988		SEAJ		Added extra test to setbox() to catch unset 
 *								xwide & ywide.
 *  30 Sep 1988		SEAJ		Changed call to mic_conv
 *	17 Aug 1988		CAS			New config option switching
 *	 2 Aug 1988		CAS			Devel option to disable normalisation
 *  18 Jul 1988		SEAJ		Changed go_on(1) to go_on()
 *	20 Jun 1988		SCG			set_scan_timeout before dig scan
 *	13 Jun 1988		CAS			Fix wait for digitise time
 *	10 Jun 1988		CAS			Shuffle some devel stuff
 *	23 May 1988		CAS			Change colour calls
 *	10 Mar 1988		CAS			This file header
 *	22 Jan 1988		CAS			New cmdsys
 *	13 Jan 1988		SCG			use setthresh to set thresholds
 *	12 Jan 1988		SCG			in filepic if in single mode set all cell
 *								dimensions to zero and set the right ids in
 *								the header
 */
#include <cmdsys.h>
#include <slides.h>
#include <kyy.h>
#include <cfipp.h>
#include <fipcomm.h>
#include <chromanal.h>
#include <kcontrol.h>

#define TRUE	(1)
#define FALSE	(0)
#define MAIN_FIELD (1)
#define STRAY_FIELD (0)

extern struct kcontrol *kcont;
extern char	*fiperr[];
extern	FILE *debug_channel;
extern char *conversionstr[];
char *Calloc();

/*
 *	Fuse stuff
 */
#define FUSEMAX 3	/* Max number of fuse images - should be common */

extern struct object *fuseobj[FUSEMAX][MAXOBJS];	/* Fuse object lists */
extern struct pframe fuseframe[FUSEMAX+1];			/* Fuse frames */
extern int fusecount[FUSEMAX];						/* Number of objects in fuseobj lists */
extern int fuseframes;								/* Number of frames grabbed for fusing */


static int manthflag = 0;	/* Set when a manual thresh is set */
static int normalflag = 1;	/* Usually want to normalise */
static int htnormflag = 0;	/* Histo normalise to remove dark spots when on. */
static int castelflag = 0;	/* S/ware repair castelation effect of NAFF h/ware */
static int digspeed = 70;	/* Default digitisation speed */
static int min_lab_siz = 2;	/* min size object in label */
static int autofoc=0;			/* let m/c find focus */
static int filelabel = 1;	/* Default display Cells filed */
static int filed = TRUE;	/* Cell has been filed */
static int dump=0, invdump = 0;
static int stepreset, stoptime;

/* Single is nearly static - only other place used is in digsubs
and there the routine is currently not used.. */
static int single;		/* signal: ad-hoc object - no list */

#define MFXFACT		1.0	/* Converts mln->xwide to microns.  */
#define MFYFACT		0.65 /* Coverts mln->ywide to microns. */
#define CAPXFACT	0.1	/* Converts microns to capture scan steps. */
#define CAPYFACT	0.13 /* Converts microns to capture scan array elements.*/
#define SAFACTOR 	1.3	/* Metaphase window blow-up factor for safety. */
#define DXB		2046	/* Default box xwidth in digs. */
#define DYB		2046	/* Default box ywidth in digs. */
#ifdef GGG
#define DXB1	1025    /* 2048 - DXB/2 */
#define DYB1	1025	/* 2048 - DXB/2 */
#define DXB2	3071	/* 2048 + DXB/2 */
#define DYB2	3071	/* 2048 + DXB/2 */
#endif
#define OFFPOS	  18    /* Line at which shonum is drawn */
#define THRESH_ADD 6    /* Threshold offset */
#define CALIB_GAIN 4.6	/* Gain to use for initial histo scan */
#define DEFDHRANGE	2	/* Index to control bits for default density range */
#define FORW	1		/* Nextrec stuff */
#define BACKW	-1		/* More nextrec stuff */

/* Wots this */
#ifdef GGG
static char *nontoxhead[] = {
	"Nmb X-cord Y-cord              Comments\n--- ------ ------              --------\n",
	"Nmb  MicX   MicY               Comments\n---  ----   ----               --------\n",
	"Nmb  English NE                Comments\n---  ----------                --------\n",
	"Nmb  Swedish NE                Comments\n---  ----------                --------\n"
};
#endif

static xmid, ymid;			/* Object and box centre rel to screen centre. */
#ifdef GGG
static struct box	dbox = { DYB1,DYB2,DXB1,DXB2 }, 
					oldbox = { DYB1, DYB2, DXB1, DXB2 };
#endif
static struct box *dbox=NULL, *oldbox=NULL;

#ifdef GGG
static countobs;			/* signal: object counting only */
#endif
static int thresh=2, dhrange=DEFDHRANGE;
static autofile = 0;
static int boxbgn=0;
#ifdef GGG
static char	*digsfil;		/* Pointer to array keeps score of filed cells*/
#endif
static int countfil;		/* Number of unique cells digitised */
static int box_driven = 0;
static int fuse_init = 0;	/* Initialisation of fuseobjs/counts */

/*
 *	Internal function references
 */
static centrit(), setcprcd();
static int getcell(), dobox(), dgtfacilities();
static int filepic();
static int capturecell();
static int shonum();
static int stray();
static int initbox();
static int next_met();
static int setslstr();
#ifdef TEST
static int test_gen();
#endif

/*
 *	External function references
 */
extern	int bay_move(), baynn(), changeobj(), mic_conv(), fipops();
extern int threshops();
extern int fieldfuse();
extern int getlist();

/*
 *		S E T O F F S E T S
 *	Defined here, used globally
 *
 */
int
setoffsets(mpt,xs,ys,zs)
struct	mlclust *mpt;
{
struct	mult_slide *bp;
	bp = alc->curbay;
	bp->xcpoff = xs - mpt->xval;
	bp->ycpoff = ys - mpt->yval;
	bp->zcpoff = zs - (mpt->zval+bp->focslc);
	
}

/*
 *  I N I T _ O F F S E T.
 *
 *	This routine initialises the static offsets to zero.
 *	It is called from getlist (metlist.c) and getmph (findmet.c).
 * 
 */
init_offset()
{
	alc->curbay->xcpoff = alc->curbay->ycpoff = alc->curbay->zcpoff = 0;
}

/*
 *	A R R A Y D I G I T I S E -- Top level digitise menu
 *
 */
arraydigitise( ) 
{
int shadsww, temp, Quitcmd, Prevcmd, Nextcmd, Boxcmd, ii, ir, Rankcmd,
		Stepcmd, bayno, Mvcmd, Rdcmd, i;
struct	mult_slide *bp;
char convstr[20];
int	(*linkerbug) ();

	alc->digitised=1;						/* tell Sc etc about image type */
	bp = alc->curbay;
	bayno = baynn(bp);						/* Set bay number */
	single=0;								/* initialise to assume from list */
	countfil = 0;
#ifdef GGG
	if(digsfil)	Free(digsfil);				/* Free previous lot */
	if(bp->startptr) {						/* Grab memory for count list. */
		fprintf(stderr, "Grabbing memory for digi-list\n");
		digsfil = (char *) Calloc(bp->totin + 1, 1); 
	}
	else
		digsfil = NULL;
#endif

	fuseframes = 0;
	if (!fuse_init) {
		for (i=0; i<FUSEMAX; i++) {
			fusecount[i] = 0;
			fuseobj[i][0] = NULL;
			setaframe(&fuseframe[i], 4, 3);	 /* Set defaults for all frames */
		}
		setaframe(&fuseframe[FUSEMAX], 4, 3); /* Main fr. with 4/3 asp. ratio */
		fuse_init = 1;
	}
	initbox(&dbox);
	initbox(&oldbox);
	if(!boxbgn) dobox(0);
	shadsww = bp->shadsw;
	bp->shadsw = 800;				/* shading range */
	setcprcd(&ii, &ir);
	if (tomip0("xs 2") == FS_FAIL) 	/* Reset step size */
		return(0);
	stepreset = 1;

	llut();						/* Correct LUT for array */
	newmenu ("Array capture");
	cmmd ("capture metaphase;%g",capturecell); mouse(MIDBT);
	cmmd ("read metaphase list;%g",getlist); Rdcmd = mmcmd;
	cmmd ("record No.;:k40%d", &ii); Stepcmd = mmcmd;
	cmmd ("rank;:k40%d", &ir); Rankcmd = mmcmd;
 	cmmd ("previous metaphase"); mouse(LEFTBT); Prevcmd = mmcmd;
	cmmd ("next metaphase"); mouse(RIGHTBT); Nextcmd = mmcmd;
	cmmd ("microscope control;%g", fipops); mouse(LONGMB);
		linkerbug = mic_conv;
	cmmd("coordinate conversion;:k40 %-s%f", convstr, linkerbug); mouse(LONGRB);
	cmmd ("exit;%-f", shonum); mouse(LONGLB); Quitcmd = mmcmd;

	while (1) {
		thismu();
		if(nncmd <= 0) {
			putprompt();
			continue;
		}
		if (nncmd == Quitcmd) {
			if (filed || check("Digitised image not stored, proceed (M): ")) {
				filed = TRUE; /* as if */
				printf("\n");
				break;	/* Exit this level */
			}
			else {
				printf("\n");
				putprompt();
				setdisplay(BUST);
			}
		}
		if(nncmd == Mvcmd) {				/* Move bay */
			if(bay_move(bayno)) {
				countfil = 0;
#ifdef GGG
				if(digsfil)
					Free(digsfil);			/* Free previous lot */
				if(alc->curbay->startptr)	/* Grab memory for count list. */
					digsfil = (char *) Calloc(alc->curbay->totin + 1, 1); 
				else
					digsfil = NULL;
#endif
				setcprcd(&ii, &ir);			/* Get record and display */
				displin(0, Stepcmd);
				displin(0, Rankcmd);
				displin(0, 0);
				shonum();
				setbox();
			}
			bayno = baynn(alc->curbay);
			displin(0, Mvcmd);				/* Update bay line */
			putprompt();
		}
		if (nncmd == Rdcmd) {				/* read new list and update */
			setcprcd(&ii, &ir);
			displin(0, Stepcmd);
			displin(0, Rankcmd);
			displin(0, 0);
			shonum();
			setbox();
		}
		if (nncmd == Stepcmd || nncmd == Nextcmd || !nncmd ||
							    nncmd == Rankcmd || nncmd == Prevcmd) { 
			if(nncmd == Stepcmd) {
				if(!findrec(ii))
					fprintf(output,"Request outside list bounds\007\n");
			} else if(nncmd == Prevcmd) {
				if(!nextrec(BACKW))
					fprintf(output,"Beginning of subset list\007\n"); 
			} else if(nncmd == Nextcmd) {
				if(!nextrec(FORW))
					fprintf(output,"End of subset list\007\n"); 
			} else if(nncmd == Rankcmd) {
				if(!findrank(ir))
					fprintf(output,"Request outside list bounds\007\n");
			}
			setcprcd(&ii, &ir);
			displin(0, Stepcmd);
			displin(0, Rankcmd);
			displin(0, 0);
			shonum();
			setbox();
			putprompt();
		}
		if(!nncmd)				/* Dont redisplay whole screen. */
			setdisplay(NOTBUST);
		
		if(alc->debug > 1) {
			fprintf(debug_channel, "Capture main menu:");
			Show_alloc();
		}
	}

	boxbgn = 0;
	bp->shadsw = shadsww;
	single = 0;				/* assume default is from list */

	if (alc->cellin) {
		freecell();			/* Free off any cells done */
	}
	if(alc->debug > 1) {
		fprintf(debug_channel, "Capture main menu: EXIT");
		Show_alloc();
	}
#ifdef GGG
	if (digsfil) {
		Free(digsfil);		/* Free memory associated with count list */
		digsfil = NULL;
	}
#endif
} /* end of arraydigitise */

/*
 *	C A P T U R E C E L L -- Display and action capture menu
 */
static 
capturecell()
{
int	Quitcmd, x, y;
char convstr[20], slstr[7];
int		(*linkerbug)();

	strcpy(convstr, conversionstr[alc->ccoord]);
/*	single = 1;*/			/* show that we are in single cell mode. */
	setslstr(&single, slstr);
	newmenu("Capture metaphase");
	cmmd("capture metaphase;%g", getcell); mouse(MIDBT);
	if (alc->options & OPT_FUSION) {
		cmmd("capture stray chromosomes;%g", stray); mouse(LONGMB);
		cmmd("fuse field;%g", fieldfuse); mouse(RIGHTBT);
	}
	cmmd("file metaphase;%g",filepic); mouse(LEFTBT);
	linkerbug = dobox;
	cmmd("adjust capture area;:k40%g %+i", linkerbug, 1) ;mouse(LONGRB);
	cmmd("facilities;%g",dgtfacilities);
	cmmd("next metaphase <Return>;%g", next_met);
	cmmd("single/list;:k40%-+t%-s%f", &single, slstr, setslstr);
	if (alc->devel) {
		cmmd ("restore from file;:k40%T", &invdump);
		cmmd ("dump to file;:k40%T", &dump);
#ifdef TEST
    	cmmd ("generate test wedge;%g", test_gen);
#endif
	}
	cmmd ("exit;"); mouse(LONGLB); Quitcmd = mmcmd;

	do {
		if (!thismu()) next_met();
		putprompt();
	} while (nncmd != Quitcmd);
	/*single = 0;*/			/* may wish to go back to list */
} /* end of capturecell */

/*
 *	S E T S L S T R -- Set string passed as either single, or list
 *	according to val passed. Needed as function to call on cmdsys line
 */
static int
setslstr(val, str)
int *val;
char *str;
{
	strcpy(str, *val ? "single" : "list");
}

/*
 *		G E T C E L L -- Digitises cell, normalises, etc, and labels
 *
 */
static
getcell()
{
	int i, temp1, temp2, castval, thr, size;
	int sx, sy;
	int xdoff, ydoff, stat;
	struct object *t0, *t1, *dbufconstruct();
	unsigned char *dbuf, *array_thr();	/* (thresholded) digitisation buffer */

	if (fuseframes) {
		if (!check("Fusion images will be lost, proceed (M):"))
			return(0);
	}
	freefuse(); /* Free them then ... */

	if (!filed) {
		if (!check("Digitised image not stored, proceed (M):"))
			return(0);
		printf("\n");
		filed = TRUE;
	}
	if (alc->cellin) {
		freecell();				/* Release the old one */
	}
	/*
	 * Window centre coords wrt screen centre (2048, 2048) in digs
	 */
	xmid = (dbox->bright + dbox->bleft)/2  - alc->ddgschannel->xcenter;
	ymid = (dbox->btop + dbox->bbot)/2  - alc->ddgschannel->ycenter;
	/*
	 * xdoff, ydoff in microns. Rotated object thus ymid to xdoff and vice-versa
	 */
	xdoff = (int) (((float) ymid / (float) YSCALE) * CAPXFACT);
	ydoff = (int) (((float) xmid / (float) XSCALE) * CAPYFACT);

	llut();
	if (invdump) {
		if ((stat = restore("rawimage")) < 0) return(1);
		stopset(0);		/* Set zero stoptime */
		stepreset = 1;	/* Assume no fip operations so nothing to reset */
	}
	else if ((stat = array_read(xdoff, ydoff, MAIN_FIELD)) < 0) {
		switch (stat) {
		case -1: return(1);		/* Error return from this function */
		case -2: return(gc_mip_tidy(xdoff, ydoff));	/* Recover position */
		}
	}
	/*
	 *	Now display for BLU wash
	 */
	boxbgn = FALSE;	/* Re-draw box stuff */
	llut();
	disp_dig(dbox);			/* This knows where the buffer is */
	if (dump) {
		filedump("rawimage");
		return(gc_mip_tidy(xdoff, ydoff));	/* quit */
	}

	do {
		downmenu();
		thr = threshops("Array capture threshold select");
		upmenu();
		if (!thr)
			return(gc_mip_tidy(xdoff, ydoff));	/* quit */
		printf("Removing background\n");
		dbuf = array_thr(thr, &size);	/* Now threshold to new buffer */
	} while (dbuf == NULL);
	alc->cellin = 1;
	kcont->FIP = DIGFIP;
	filed = FALSE;
	/*
	 *	Make a Woolz object out of it
	 */
	if (alc->fipdig) {
		freeobj(alc->fipdig);
		alc->fipdig = NULL;
	}
	if ((alc->fipdig = dbufconstruct(dbuf, size)) == NULL) {
		alc->cellin = FALSE;
		filed = TRUE;
		return(gc_mip_tidy(xdoff, ydoff));	/* No object */
	}
	if (alc->debug) fprintf(debug_channel, "After dbufconstruct\n");

	/*
 	 *	Debug/devel mode numerical castellation check
 	 */
	if (alc->devel || alc->debug) {
		castval = mcastel(alc->fipdig);
		if (castval != 0)
			fprintf(stderr,"\007Castellation value %d\n",castval);
	}
	if (stoptest() && !stepreset) {
		if (tomip0("xs 2") == FS_FAIL) 		
			return(gc_mip_tidy(xdoff, ydoff));
		stepreset = 1;
	}
/*
 *	Four possible scenarios here: 
 *	either 1: check + fix castelation  (with or without smart histo)
 *		   3: htnormalise does a normalise ignoring the top and bottom n% of
 *			  pixell values.  This in effect ignores 'dark spots'.
 *		   4: straight normalise
 */
	printf("Normalising\n");
	if (castelflag) {
		if (htnormflag)
			compute_decastel_tables(alc->fipdig,&t0,&t1,8,255,(float) 1.0);
		else
			compute_decastel_tables(alc->fipdig,&t0,&t1,8,255,(float) 0.0);
		apply_decastel_table(alc->fipdig,t0,t1);
	} else if (normalflag) {
		if (htnormflag)
			htnormalise(alc->fipdig,(float) 1.0, (float) 1.0);
		else
			normalise(alc->fipdig);
	}

	if (alc->debug)
		fprintf(debug_channel, "getcell: After normalise\n");
	if (!stepreset && stoptest()) {
		if (tomip0("xs 2") == FS_FAIL) 		
			return(gc_mip_tidy(xdoff, ydoff));
		stepreset = 1;
	}
/*
 *	position object in centre of screen area, and display,
 *	only if fusion option selected
 */
	if (!(alc->options & OPT_FUSION)) blow_up();
	if (!stepreset) {
		stopwait();
		if (tomip0("xs 2") == FS_FAIL) 		
			return(gc_mip_tidy(xdoff, ydoff));
	}

#ifdef GGG
	if (countobs) {
		imgdisp();
		if (! single) movrecord(); 
	}
#endif
/*
 *	Label the digitised object
 */
	printf("Extracting chromosomes\n"); 
	if (alc->debug) fprintf(debug_channel, "getcell: Before label\n");
	label(alc->fipdig, &alc->lkount, kcont->ool, MAXOBJS, min_lab_siz);
	if (alc->debug)
		fprintf(debug_channel,"getcell: label %d objs\n", alc->lkount);
	findscale(kcont->ool, alc->ddgschannel->xcenter,
		alc->ddgschannel->ycenter, &sx, &sy);
	if (alc->debug)
		fprintf(debug_channel, "getcell: Returned ix, iy = %d, %d\n", sx, sy);
	fuseframe[FUSEMAX].type = 60;
	fuseframe[FUSEMAX].ix = sx;
	fuseframe[FUSEMAX].iy = sy;
	fuseframe[FUSEMAX].scale = 1;
	fuseframe[FUSEMAX].dx = 0;
	fuseframe[FUSEMAX].dy = 0;
	kcont->maxnumber = alc->lkount;
	alc->lbelld = 1;
	freeobj(alc->fipdig);
	alc->fipdig = NULL;
	if (!(alc->options & OPT_FUSION)) filecell();
	return(gc_mip_tidy(xdoff, ydoff));
}

/*
 *	S T R A Y -- Grab second and subsequent fields for field fusion
 *
 */
static int
stray()
{
	int temp1, temp2, castval, thr, size;
	int xdoff, ydoff, stat;
	int sx, sy;
	struct object *t0, *t1, *dbufconstruct();
	unsigned char *dbuf, *array_thr();	/* (thresholded) digitisation buffer */

	fprintf(output, "Capture stray chromosomes\n");

	if (!alc->cellin) {
		printf("No main cell image digitised yet\n");
		return(0);
	}
	if (fuseframes >= FUSEMAX) {
		printf("Can't hold any more images to fuse\n");
		return(0);
	}
	if (alc->debug) {
		fprintf(debug_channel, "Stray started:\n");
		print_kcont_all_ol();
	}
	/*
	 * Window centre coords wrt screen centre (2048, 2048) in digs
	 */
	xmid = (dbox->bright + dbox->bleft)/2  - alc->ddgschannel->xcenter;
	ymid = (dbox->btop + dbox->bbot)/2  - alc->ddgschannel->ycenter;
	/*
	 * xdoff, ydoff in microns. Rotated object thus ymid to xdoff and vice-versa
	 */
	xdoff = (int) (((float) ymid / (float) YSCALE) * CAPXFACT);
	ydoff = (int) (((float) xmid / (float) XSCALE) * CAPYFACT);

 	llut();
	if (invdump) {
		if ((stat = restore("rawimage")) < 0) return(1);
		stopset(0);		/* Set zero stoptime */
		stepreset = 1;	/* Assume no fip operations so nothing to reset */
	}
	else if ((stat = array_read(xdoff, ydoff, STRAY_FIELD)) < 0) {
		switch (stat) {
		case -1: return(1);		/* Error return from this function */
		case -2: return(gc_mip_tidy(xdoff, ydoff));	/* Recover position */
		}
	}
	/*
	 *	Now display for BLUUU wash
	 */
	boxbgn = FALSE;	/* Re-draw box stuff */
	llut();
	disp_dig(dbox);			/* This knows where the buffer is */
	if (dump) {
		filedump("rawimage");
		return(gc_mip_tidy(xdoff, ydoff));	/* quit */
	}

	do {
		downmenu();
		thr = threshops("Array capture threshold select");
		upmenu();
		if (!thr)
			return(gc_mip_tidy(xdoff, ydoff));	/* quit */
		printf("Removing background\n");
		dbuf = array_thr(thr, &size);	/* Now threshold to new buffer */
	} while (dbuf == NULL);

	/*
	 *	Make a Woolz object out of it
	 */
	if (alc->fipdig) {
		freeobj(alc->fipdig);
		alc->fipdig = NULL;
	}
	if ((alc->fipdig = dbufconstruct(dbuf, size)) == NULL) {
		alc->cellin = FALSE;
		filed = TRUE;
		return(gc_mip_tidy(xdoff, ydoff));	/* No object */
	}

	if (alc->debug) fprintf(debug_channel, "After dbufconstruct\n");

	/*
 	 *	Debug/devel mode numerical castellation check
 	 */
	if (alc->devel || alc->debug) {
		castval = mcastel(alc->fipdig);
		if (castval != 0)
			fprintf(stderr,"\007Castellation value %d\n",castval);
	}
	if (stoptest() && !stepreset) {
		if (tomip0("xs 2") == FS_FAIL) 		
			return(gc_mip_tidy(xdoff, ydoff));
		stepreset = 1;
	}
/*
 *	Four possible scenarios here: 
 *	either 1: check + fix castelation  (with or without smart histo)
 *		   3: htnormalise does a normalise ignoring the top and bottom n% of
 *			  pixell values.  This in effect ignores 'dark spots'.
 *		   4: straight normalise
 */
	printf("Normalising\n");
	if (castelflag) {
		if (htnormflag)
			compute_decastel_tables(alc->fipdig,&t0,&t1,8,255,(float) 1.0);
		else
			compute_decastel_tables(alc->fipdig,&t0,&t1,8,255,(float) 0.0);
		apply_decastel_table(alc->fipdig,t0,t1);
	} else if (normalflag) {
		if (htnormflag)
			htnormalise(alc->fipdig,(float) 1.0, (float) 1.0);
		else
			normalise(alc->fipdig);
	}

	if (alc->debug)
		fprintf(debug_channel, "stray: After normalise\n");
	if (!stepreset && stoptest()) {
		if (tomip0("xs 2") == FS_FAIL) 		
			return(gc_mip_tidy(xdoff, ydoff));
		stepreset = 1;
	}
/*
 *	position object in centre of screen area.
 */
	/*blow_up();*/
	if (!stepreset) {
		stopwait();
		if (tomip0("xs 2") == FS_FAIL) 		
			return(gc_mip_tidy(xdoff, ydoff));
	}

#ifdef GGG
	if (countobs) {
		imgdisp();
		if (! single) movrecord(); 
	}
#endif
/*
 *	Label the digitised object
 */
	printf("Extracting chromosomes\n"); 
	if (alc->debug) fprintf(debug_channel, "stray: Before label\n");
	label(alc->fipdig, &fusecount[fuseframes], fuseobj[fuseframes],
												MAXOBJS, min_lab_siz);
	if (alc->debug)
		fprintf(debug_channel,"Stray label %d objs\n", fusecount[fuseframes]);
	findscale(fuseobj[fuseframes], alc->ddgschannel->xcenter,
		alc->ddgschannel->ycenter, &sx, &sy);
	if (alc->debug)
		fprintf(debug_channel, "Returned ix, iy = %d, %d\n", sx, sy);
	fuseframe[fuseframes].type = 60;
	fuseframe[fuseframes].ix = sx;
	fuseframe[fuseframes].iy = sy;
	fuseframe[fuseframes].scale = 1;
/*
	fuseframe[fuseframes].ox = 20;
	fuseframe[fuseframes].oy = -20;
*/
	fuseframe[fuseframes].dx = 0;
	fuseframe[fuseframes].dy = 0;
	fuseframes++;
	freeobj(alc->fipdig);
	alc->fipdig = NULL;
	return(gc_mip_tidy(xdoff, ydoff));
} /* end stray */

/*
 *		C A L C S C A L E
 *
 */
static
calcscale(tt1, tt2) 
int	*tt1, *tt2;
{
float temp = 0.0;
int t = 1, boxx, boxy, diff, length;

	boxx = abs(dbox->bleft - dbox->bright);
	boxy = abs(dbox->btop - dbox->bbot);
	
	length = boxx - boxy;				/* length is largest between x and y */
	diff = length > 0 ? XSCALE : YSCALE;/* diff used to calculate showdc pars */
	length = boxx > boxy ? boxx : boxy; /* reassign length to largest */
	temp = ((float)(alc->ddgschannel->ycenter*2) / (float)length) * (float) diff;
	while( temp > (t * diff) ) ++t;
	if( t > 1 ) --t;
	*tt1 = 3*t;
	*tt2 = 4*t;
	
	if (alc->debug) {					/* Useful debug info. */
		fprintf(debug_channel,"calcscale: boxx %d boxy %d\n", boxx, boxy);
		fprintf(debug_channel,"calcscale: length %d diff %d\n", 
				length, diff);
		fprintf(debug_channel,"calcscale: temp is %.2f t is %d\n", temp, t);
		fprintf(debug_channel,"calcscale: tt1 is %d tt2 is %d\n", *tt1, *tt2);
	}
}

/*
 *		S E T B O X
 *
 */
static
setbox() 
{
int nn,dxb,dyb;
struct	mlclust *mpt;

	if (!alc->curbay->startptr)
		return(0);
    mpt = alc->curbay->currptr->mlrec;
    if ((mpt->xwide <= 0) || (mpt->ywide <= 0))
    	return(0);
    dxb = (int) ((float) mpt->ywide * MFYFACT * SAFACTOR *
    			(float) XSCALE / CAPYFACT);
    dyb = (int) ((float) mpt->xwide * MFXFACT * SAFACTOR *
    			(float) YSCALE / CAPXFACT);
	if (alc->debug)
		fprintf(debug_channel,"in setbox dxb is %d dyb is %d\n", dxb, dyb);
    boxsiz(dxb,dyb);
    boxbgn=0;
    dobox(0);
}

/*
 *		B O X S I Z
 *
 */
static
boxsiz(xx,yy)
 int xx, yy;
 {
	DDGS *fs;

	fs = alc->ddgschannel;
    dbox->bleft = fs->xcenter - (xx/2);
    dbox->bbot = fs->ycenter - (yy/2);
    dbox->bright = fs->xcenter + (xx/2);
    dbox->btop = fs->ycenter + (yy/2);
 }

#ifdef GGG
/*
 *		C H E C K _ D I G _ T H O L D S
 *
 *		Checks that thresholds are sensible.  Overrides with manually
 *		selected ones if flag set.
 *
 */
static
int check_dig_tholds(lth1, lth2)
int  lth1, lth2;
{

	screengo(ENDLINE, 0);
	fprintf(output, "Evaluated threshold %d\n", lth1);
	if (!manthflag) {
		if (lth1 < 0 || lth1 > 255-THRESH_ADD ) {
			fprintf(output,"Unsatisfactory automatic threshold\n");
			downmenu();
			dgtfacilities();
			upmenu();
		} else {
			thresh = lth1;
			fprintf(output,"Use automatic threshold (%d)?\n",
				lth1 + THRESH_ADD);
			if (!yesno(1)) /* yesno - default yes - return true if no !!!! */
				thresh = lth1 + THRESH_ADD;
			else {
				downmenu();
				dgtfacilities();
				upmenu();
			}
		}
	} else
		fprintf(output, "Manual threshold %d overrides calculated one.\n", 
						 thresh);
		
}
#endif

#ifdef GGG
/*
 *		M O V R E C O R D
 *
 */
static
movrecord() 
{
struct	mlclust *mpt;
struct	mult_slide *bp;

	if (!alc->curbay->startptr) {
		screengo(ENDLINE,0);
		fprintf(output,"No list on this bay\n");
		return(0);
	}
#ifdef GGG
	if (single) return(0);
#endif
	movespeed();
	box_driven = 0;				/* use default instead of chosen box. */
	bp = alc->curbay;
 	mpt = bp->currptr->mlrec;
	stagepos(mpt->xval + bp->xcpoff, mpt->yval + bp->ycpoff,
		   mpt->zval + bp->focslc + bp->zcpoff);
	return(1);
}
#endif

/*
 *		C E N T R I T
 *
 *	Returns:
 *		0 - sucess
 *		-1 - fail
 */
static
centrit()
{
struct	mlclust *mpt;
struct	mult_slide *bp;

	bp = alc->curbay;
	
	screengo(ENDLINE,1);
	tomip0("xs 2");
	fprintf(output,"Please check cell in focus\n"); 
	if (! go_on()) { 
		setdisplay(BUST);
		tomip0("xs 0");
		return(-1);
	}
	if (!single && bp->startptr != NULL) {
/*	if (bp->startptr != NULL) {*/
		if (getpos(2)) return(-1);
		setoffsets(bp->currptr->mlrec,xs,ys,zs);
		screengo(ENDLINE,0);
	}
	tomip0("xs 0");
	return(0) ;
}

/*
 *		D O B O X
 *
 */
static 
dobox(i) 
{
int	mrow, mcol;
	if (alc->debug) {
		fprintf(debug_channel,"dbox: bleft=%d, bright=%d, btop=%d, bbot=%d\n",
			dbox->bleft, dbox->bright, dbox->btop, dbox->bbot);
		fprintf(debug_channel,"oldbox: bleft=%d, bright=%d, btop=%d, bbot=%d\n",
			oldbox->bleft, oldbox->bright, oldbox->btop, oldbox->bbot);
	}
	if (i || !boxbgn) {
		if(!i) qclear();
		if(i){
			if(alc->cellin) {		/* Show only when object retained. */
				mrow = abs(oldbox->bleft - oldbox->bright) / (2 * XSCALE);
				mcol = abs(oldbox->btop - oldbox->bbot) / (2 * YSCALE);
				showdc(YSCALE, XSCALE, xmid, ymid, mrow, mcol);
			} else {
			    qclear();
			}
 	   		box_driven = 1;
		} 
		lin_colour_lut();
#ifdef GGG
		gratic(XSCALE * 200);
#endif
		boxbgn = 1;	
		if (!i) dwb(2, dbox, 1, 1);
	}
	if (i) newbox( "Digitisation Frame", dbox ,1,1);
}

/*
 *		M A N T H R
 *
 */
static
manthr(t)
int *t ; 
{
	/* since manual threshold set, disable auto thresh */
	manthflag = 1;
}

#ifdef GGG
/*
 *	O N E M O R E
 *
 */
static
onemore() 
{	
		screengo(ENDLINE,0);
		fprintf(output," Single cell [not from cell-list] ?\n");
		single = ! yesno(0);
}
#endif

/*
 *	A F _ M I P _ S E T U P.
 *
 *				This routine sets the miproc to handle digitisation when
 *				autofocus is switched on.  Future development.
 *
 */
 static
 af_mip_setup()
 {
 
	if (tomip0("it 480") != FS_OK)	/* Set integration time. */
		return(FS_FAIL);
	if (speed(2000) != FS_OK)		/* Set speed. */
		return(FS_FAIL);
	if (tomip0("fv 1") != FS_OK)	/* Cancel default speed. */
		return(FS_FAIL);
	if (tomip0("sz -100") != FS_OK)	/* Focus search. */
		return(FS_FAIL);
	if (! waitword ("ok")) {
		fprintf(output,"focus not found\n");
		centrit();
	} else
		fprintf(output,"focus found\n");
		
	if (tomip0("it 2000") != FS_OK)	/* Set integration time. */
		return(FS_FAIL);
	if (speed(digspeed) != FS_OK)	/* Set speed. */
		return(FS_FAIL);
	return(FS_OK);
}
/*
 *
 *  G C _ M I P _ T I D Y.
 *
 *			Leaves the miproc in a sensible state when leaving getcell.
 *
 */
static
gc_mip_tidy(xdoff, ydoff)
int	xdoff, ydoff;
{
	int	stat = 0;

	if (invdump) return(0);
	if (tomip1("x", xs-xdoff) != FS_OK)  return(FS_FAIL); /* Return x, y posn */
	if (tomip1("y", ys-ydoff) != FS_OK)  return(FS_FAIL);
	stopwait();
	if (tomip0("xs 2") != FS_OK)  return(FS_FAIL);	/* Set step size back. */
	if (highres(OFF) != FS_OK)  return(FS_FAIL);
	getpos(2);
	return(movespeed());											
}

/*
 *	F I L E C E L L
 */
static 
filecell()
{
	downmenu();
	filepic();
	upmenu();
} /* end filecell */
 
/*
 *	F I L E P I C
 *
 */
static 
filepic() 
{
static char oldcase[PATLENGTH+1] = " ", oldslide[SLIDELENGTH+1] = " ";
static int oldcell = -1;

int		notfiled, i;
struct	mlclust *mpt;
struct	cyto_file *cfp;

	if ( !alc->cellin ) {
		fprintf(output,"no cell to file\n");
		return(0);
	}
	lin_colour_lut();
	blow_up();
	if (filelabel) filcount(countfil);		
	/*
	 * in single mode use the karyotype id for the cell and set all
	 * the cell dimensions to zero since they are not known
	 */
/*	if (alc->curbay->startptr != NULL) {*/
	if(single) {
		cfp = &alc->nameids;
		alc->cell_h->x_coord = 0; 
		alc->cell_h->y_coord = 0;
		alc->cell_h->z_coord = 0;
		alc->cell_h->x_size = 0;
		alc->cell_h->y_size = 0;
	} else {
		cfp = &alc->curbay->nameids;
		mpt = alc->curbay->currptr->mlrec;
		alc->cell_h->x_coord = mpt->xval;
		alc->cell_h->y_coord = mpt->yval;
		alc->cell_h->z_coord = mpt->zval;
		alc->cell_h->x_size = mpt->xwide;
		alc->cell_h->y_size = mpt->ywide;
 	}
	cfp->type = CELLFILE;				/* Type of file to be made */
	if ( ! alc->lbelld ) {
		fprintf(output, "Labelling image\n");
		alc->lkount = labeldc(kcont,min_lab_siz); 
		if (alc->debug)
			fprintf(debug_channel,"labelled, %d objects\n",alc->lkount);
	}
	notfiled = 0;
	while (1) {
		if (single) {
/*		if (alc->curbay->startptr) {*/
			if (!getid(CELLFILE,WRITE)) return(0);
		} else {
			if (notfiled || !makefilename(alc->picname,cfp))
				if (getid(CELLFILE,WRITE) == 0)
					return(0);
		}
		if (single)
			_strass(&(alc->curbay->nameids), &(alc->nameids),
				sizeof(struct cyto_file));
		else
			_strass(&(alc->nameids), &(alc->curbay->nameids),
				sizeof(struct cyto_file));
		if (checkoverwrite(alc->dir,alc->picname) == 0) {
			notfiled = 1;
			continue;
		}
		if (alc->debug) {
			fprintf(debug_channel,"Filing cell to %s\n",alc->picname);
			print_cyto(cfp);
		}
		alc->cell_h->dig_type = DIGFIP;
		cellheadfill(alc->cell_h, cfp);
		if (alc->debug) print_cell_h(alc->cell_h);
		if (putfile(alc->picname,alc->fipdig,kcont->ool,alc->lkount))
			break;
		notfiled = 1;
	}
/*	if(!single){				*/		/* Count the cells filed. */
#ifdef GGG
	if (!(alc->curbay->startptr)) {
		if (cfp->cellno && (cfp->cellno < alc->curbay->totin))
			digsfil[cfp->cellno - 1] = '1';
		for(i = 0, countfil = 0; i < alc->curbay->totin; ++i) {
			if(digsfil[i] == '1') { 
				++countfil;  
			}
		}
	}
	return(1);
#endif
	if (!strcmp(oldcase, alc->nameids.case)) {
		countfil = 1;						/* Reset cells filed */
		strcpy(oldcase, alc->nameids.case);	/* Keep a record */
		strcpy(oldslide, alc->nameids.slide);
		oldcell = alc->nameids.cellno;
	}
	else if (strcmp(oldslide, alc->nameids.slide) ||	/* Slideid diferent */
			 (oldcell != alc->nameids.cellno)) {		/* cellno different */
		countfil++;										/* One more filed */
		strcpy(oldslide, alc->nameids.slide);
		oldcell = alc->nameids.cellno;
	}
	if (filelabel) filcount(countfil);
	filed = TRUE;	/* Flag filed, but retain the cell - no freecell(); */
}

/*
 *	D G T F A C I L I T I E S
 *
 *	Put up the facilities available in digitisation
 */
static
dgtfacilities()
{
	int Quitcmd, Thr_cmd,Offcmd,shval,Shcmd;
	
	if(doshade > -1)
		shval = 1;
	else
		shval = 0;
		
	newmenu ( "Image capture facilities" );
	cmmd("threshold;:k40 <1/255>%d%f",&thresh,manthr); Thr_cmd = mmcmd;
/*	cmmd("manual threshold;:k40%T",	&manthflag);*/
	cmmd("zero offsets");Offcmd=mmcmd; mouse(LEFTBT);
	cmmd("shading correction;:k40%T",&shval);Shcmd =mmcmd; mouse(MIDBT);
	cmmd("dark spot removal;:k40%T",&htnormflag); mouse(LONGMB);
	cmmd("de-castelate;:k40%T",&castelflag); mouse(LONGRB);
	cmmd("display number of cells filed ;:k40%T", &filelabel); mouse(RIGHTBT);
	cmmd("digitisation speed;:k40%d",&digspeed);
	if (alc->devel) {
		cmmd("normalise;:k40%T",&normalflag);
		cmmd("density range;%d",&dhrange);
		cmmd("minimum obj size;:k40%d",&min_lab_siz);
		if (alc->devel > 10) {
			cmmd("auto-file;%t",&autofile);
			cmmd("auto-focus;%t",&autofoc);
		}
	}
	cmmd("exit "); Quitcmd = mmcmd; mouse(LONGLB);
	while ( 1 ) {
		thismu();
		if (nncmd == Quitcmd) return(0);
		if (nncmd == Shcmd) {
			if (shval)
				doshade = 0;	/* Do it */
			else
				doshade = -1;	/* Don't do it */
		}
		if (nncmd == Thr_cmd) {
			displin(0,Thr_cmd);
			displin(0,Thr_cmd+1);
			putprompt();
		} else if ( nncmd == Offcmd ) {
				init_offset();
				continue;
		}
	}
}

/* 
 *	S E T C P R C D ---  Set record and rank
 */
static 
setcprcd(ii, ir)
int	*ii, *ir;
{
struct	mult_slide *bp;
struct	mlclust *mpt;

	bp = alc->curbay;
	if(bp->startptr == NULL)
		*ii = *ir = 0;
	else {
		mpt = bp->currptr->mlrec;
		*ii = bp->nameids.cellno = mpt->recnr;
		*ir = mpt->qrank;
		recall(mpt->xval+bp->xcpoff, mpt->yval+bp->ycpoff,
			   mpt->zval+bp->focslc+bp->zcpoff);
	}
}
/*
 *	F R E E O B J L I S T  --  Free and object list
 *
 *		Experimental routine
 *
 */
static
freeobjlist(objlist,count)
struct object **objlist;
int		count;
{
	int		i;

	for (i=0; i<count; i++) {
		if (objlist[i]) {
			freeobj(objlist[i]);
			objlist[i] = NULL;
		}
	}
}

/*
 *
 *	B O X C O P Y
 *
 *	Copy new box into old box.  Old box used for display purposes.
 *
 */
static
boxcopy()
{
	oldbox->bleft = dbox->bleft;
	oldbox->bright = dbox->bright;
	oldbox->btop = dbox->btop;
	oldbox->bbot = dbox->bbot;
}

/*
 *	A R R A Y _ R E A D -- Read cell from array
 *	Produce appropriate prompts etc
 *	Returns:	 0	-	OK
 *				-1	-	Error return, no stage move required
 *				-2	-	Error return, stage move required
 */
array_read(xdoff, ydoff, field)
int xdoff, ydoff, field;
{
	int errnum, stat;
	int		dix, diy;			/* Digitise command params */

	if (!(alc->config)&CONF_FIP){
		fprintf(output,"Unable to digitise, no FIP in configuration\n");
		return(-1);
	}

	if (!single && !alc->curbay->startptr) {
		fprintf(output,"Please specify list or select single\n");
		return(-1);
	}

	if (!single && alc->curbay->startptr && !box_driven)
/*	if (alc->curbay->startptr && !box_driven)*/
		setbox();
	/*
	 * Always calculate scan length and y gate from box.
	 */
	diy = ((dbox->bright - dbox->bleft)) / XSCALE;
	dix = ((dbox->btop - dbox->bbot)) / YSCALE;
	
	if (alc->debug) {
		fprintf(debug_channel,"array_read: diy is %d dix is %d\n", diy, dix);
		fprintf(debug_channel,"array_read: xmid is %d ymid is %d\n", 
				xmid, ymid);
	}
	if (getpos(-1) == -1) return(-1);
	/*
	 *	The speed must be set to 2000 before a small move is attempted.
	 *	At v 4000 small moves are not done correctly
	 */
	if ((highres(ON) != FS_OK) ||
		(autofocus(OFF) != FS_OK) ||
		(speed(2000) != FS_OK) ||
		(tomip0("fv 1") != FS_OK)) return(-2); /* Error - needs recovery */
	
	screengo(ENDLINE,0);
	/*
	 * correct in x, y but not in z and update
	 */
	tomip0("xs 2");					/* Step size 1 um */
	tomip1("x", xs+xdoff);
	tomip1("y", ys+ydoff);
	getpos(2);
	/*
	 *	last check microscope and miproc OK
	 */
	if (!checkmcmp(2))			
		return(-2);
/*   	erasehisto();*/
	/*
	 * If new afe present, AND digitising main field.
	 * Do histogram, and set gain accordingly.
	 * Re-do shading if range is altered
	 */
	if ((field == MAIN_FIELD) && alc->config & CONF_FIPAL) { /* New card present */
		if ((stat = set_afe_gain()) < 0) return(stat);
    } /* end if config & CONF_FIPAL */

	if (doshade > -1) {				/* Check if done before	*/
		if (hrshade())
			return(-2);
		else
			doshade = -1;			/* Do it only once		*/
	}

	tomip0("xs 0");
	/*
	 * Do histogram (or final histogram)
	 */
#ifdef GGG
	if (hrhisto(&temp1,&temp2, 500)) {
	   fiptim("Histogram reset.");
	   return(gc_mip_tidy(xdoff, ydoff)); -2?
    }
	/*
	 * Check thresholds
	 */
    check_dig_tholds(temp1, temp2);
    showhisto(thresh, 255);
#endif
	if (centrit() == -1) {		/* Centre cell and continue */
		putprompt();
		return(-2);
	}

	if(threshset(thresh, 255) != FS_OK)	return(-2);

	speed(digspeed);
		
	if (autofoc){
		if (af_mip_setup())  return(-2);
	}
	
	boxcopy();					/* Old box no longer needed. */
	screengo(ENDLINE,0);
	fprintf(output,"Starting digitisation scan .... " );
	fflush(output);

#ifdef GGG
	if (alloc_digbuf(NULL) == NULL)		/* Allocate dig buffer first */
		return(-2);
#endif

	tomip2("di",diy,dix);		/* Tell miproc how many lines and cols. */
	set_scan_timeout();	

	if (errnum = waitfip(FOR_DSIG)) {
		stopset(dix/2 + 48);
		fprintf(output,"Digitisation Fail - error %s\n",fiperr[errnum]);
		display();
		shonum();
		stopwait();
		erasehisto();
		return(-2);
	}
/*
 *	Must have been OK
 *	Restore step size as soon as we can.  We need to calculate when this will 
 *	be possible so we use the box size + the stage speed to calculate
 *	when it will have stopped.
 *	Note stoptime = current_tick + (box_size/2 + overrun)/speed*ticks_per_sec
 *	old calc. was "stoptime = "tick() + ((dix/2 + 48) * digspeed / 100);"
 *	Note stopset automatically adds tick()
 */
	stopset((dix/2 + 48) / (digspeed * 100));
	stepreset = 0;
	fprintf(output,"scan completed\n" );
	return(0);	/* OK */
} /* array_read */

#ifdef TEST
/*
 *	T E S T _ G E N -- Generate test pattern
 */
static
test_gen()
{
	int xdoff, ydoff;
	int		dix, diy;			/* Digitise command params */
								/* offsets calculated as newmidpoint minus 
								   oldmidpoint */
	/*
	 * Window centre coords wrt screen centre
	 * (2048, 2048) in digs
	 */
	xmid = (dbox->bright + dbox->bleft)/2  - alc->ddgschannel->xcenter;
	ymid = (dbox->btop + dbox->bbot)/2  - alc->ddgschannel->ycenter;
	/*
	 * xdoff, ydoff in microns.  Rotated object 
	 * hence ymid to xdoff and vice-versa
	 */
	xdoff = (int) (((float) ymid / (float) YSCALE) * CAPXFACT);
	ydoff = (int) (((float) xmid / (float) XSCALE) * CAPYFACT);
	/*
	 * Always calculate scan length and y gate from box.
	 */
	diy = ((dbox->bright - dbox->bleft)) / XSCALE;
	dix = ((dbox->btop - dbox->bbot)) / YSCALE;
	
	if (alc->debug) {
		fprintf(debug_channel,"test_gen: diy is %d dix is %d\n", diy, dix);
		fprintf(debug_channel,"test_gen: xmid is %d ymid is %d\n", 
				xmid, ymid);
	}

	screengo(ENDLINE,0);
	test_wedge();
	boxbgn = FALSE;	/* Re-draw box stuff */
	disp_dig(1024, 1024);			/* This knows where the buffer is */
	threshops("Test wedge threshold select");
	return(0);
} /* end test_gen */
#endif

/*
 *	S E T _ A F E _ G A I N -- Set the gain of the analogue front end
 *	takes histogram and adjusts gain accordingly to give maximum use of
 *	available dynamic range
 */
set_afe_gain()
{
	int temp1, temp2;
	float act_gain;

	/*
	 * If new afe present, do histogram, and set
	 * gain accordingly. Re-do shading if range is altered
	 */
	if (alc->debug)
		fprintf(debug_channel, "New afe present, called set_afe_gain\n");
	tomip0("xs 0");				/* Step size 0.1 um 	*/
	tomip0("sh -1");			/* Shading corr. off 	*/
	density_range(DEFDHRANGE);	/* Set default density range */
	/*
	 *	Now obtain histogram
	 */
	if (hrhisto(&temp1,&temp2, 500)) {	/* Re - histogram */
   		fiptim("Histogram reset 1.");
		tomip0("sh 1");				/* Shading corr. on		*/
		return(-1);					/* Error - higher level will recover */
	}
	/*
	 * Get range number for gain, return actual gain setting
	 * in act_gain. (Prob. unused at present)
	 */
	dhrange = get_gain(gettop(), CALIB_GAIN, &act_gain);
	if (alc->debug)
		fprintf(debug_channel, "Gain ctrl bits = %d, actual gain = %f\n",
			dhrange, act_gain);
	/*
	 * New gain set will be set by hrshade
	 */
	doshade = 0;				/* Re-do shading ALWAYS	*/
	tomip0("sh 1");				/* Shading corr. on		*/
	tomip0("xs 2");				/* Shading done at 1 um	*/
	return(0);					/* OK return */
} /* end set_afe_gain */

/*
 *	P R I N T _ C Y T O -- Debug, print contents of cyto structure
 */
print_cyto(cyto)
struct cyto_file *cyto;
{
	if (cyto == NULL) {
		fprintf(debug_channel, "print_cyto: Passed NULL pointer\n");
		return(-1);
	}
	fprintf(debug_channel, "type=%d\ncase=%s\npatient=%s\nslide=%s\ncellno=%d\n",
		cyto->type, cyto->case, cyto->patnam, cyto->slide, cyto->cellno);
	return(0);
} /* end print_cyto */

/*
 *	P R I N T _ C E L L _ H -- Print the cell_h structure contents
 */
print_cell_h(ch)
struct cell_header *ch;
{
	if (ch == NULL) {
		fprintf(debug_channel, "print_cell_h: Passed NULL pointer\n");
		return(-1);
	}
	fprintf(debug_channel, "size=%d, version=%d, file_type=%d\ndate=%s, lab_name=%s\n",
		(int)ch->size, ch->version, ch->file_type, ch->date, ch->lab_name);
	fprintf(debug_channel, "opid=%s, patid=%s, patnam=%s\n",
		ch->opid, ch->patid, ch->patnam);
	fprintf(debug_channel, "slideid=%s, cellid=%d, dig_type=%d\n",
		ch->slideid, ch->cellid, ch->dig_type);
	fprintf(debug_channel, "x_coord=%d, y_coord=%d, z_coord=%d\n",
		ch->x_coord, ch->y_coord, ch->z_coord);
	fprintf(debug_channel, "x_size=%d, y_size=%d\n",
		ch->x_size, ch->y_size);
	fprintf(debug_channel, "comments[1]=%s, comments[2]=%s, krtype=%s\n",
		ch->comments[1], ch->comments[2], ch->krtype);
	fprintf(debug_channel, "supid=%s, supdate=%s\n",
		ch->supid, ch->supdate);
	return(0);

} /* end print_cell_h */

#ifdef GGG

/*
 *		I M G D I S P
 *
 *	This is called from the segmentation stage
 *
 */
static 
imgdisp() 
{
	setcolour(alc->metacolour_1,alc->metacolour_2,0);
	if (!kcont->ool) {
		if (alc->debug)
			fprintf(debug_channel,"imgdisp: no kcont->ool\n");
		fprintf(output, "\007No kcont->ool\n"); /* ho hum */
		return(-1);
	}
	if (!alc->cellin)
		fprintf(output,"\007Warning: No cell in!\n");
	else {
		if (!alc->lbelld) {
			fprintf(stderr,"Segmenting image\n");
			alc->lkount = labeldc(kcont,min_lab_siz);
			if (alc->debug)
				fprintf(debug_channel,"imgdisp: %d objects segmented\n",
						alc->lkount);
		}
	}
	qclear();
	colour(GREYCOLS);
	mastdisplay(kcont);
	setcolour(alc->metacolour_1,alc->metacolour_2,0);
	downmenu();
	countit(alc->cellin);
	upmenu();
}
#endif

/*
 *		H R S H A D E
 *
 * do the shading stuff
 *
 */
static 
hrshade() 
{ 
		tomip0("xs 2");
		tomip0("it 2000");
		speed(500);					/* new addition for better signal and
								 	   faster shading */
		if (runshade(dhrange,alc->curbay->shadsw) ) 
				return(-1);
		return(0);
} /* end hrshade */

/*
 *	S H O N U M  --  Displays record num, xyz position
 *							and xyz offset
 *
 */
static 
shonum() 
{
char string[60];
struct	mlclust	*mpt;
struct	mult_slide *bp;

	bp = alc->curbay;
	clearlns(OFFPOS, OFFPOS+1);
	sprintf(string,"Position:     ");
	if (alc->curbay->startptr != NULL) {
		mpt = bp->currptr->mlrec;
		sprintf(string+13,"x: %d      ", mpt->xval);
		sprintf(string+23,"y: %d      ", mpt->yval);
		sprintf(string+33,"z: %d\0", mpt->zval);
	}
	screengo (OFFPOS, 10);
	fprintf (output, "%s", string);
	screengo (OFFPOS + 1, 10);
	sprintf(string,"Offsets:      ");
	sprintf(string+13,"x: %d      ", bp->xcpoff);
	sprintf(string+23,"y: %d      ", bp->ycpoff);
	sprintf(string+33,"z: %d\0", bp->zcpoff);
	fprintf (output, "%s", string);
	putprompt();
} /* end shonum */

/*
 *	F R E E C E L L -- Free off cell object lists etc.
 */
static
freecell()
{
	if (alc->debug)
		fprintf(debug_channel, "freecell: Started\n");
	if (kcont->maxnumber == 0) {
		kcont->maxnumber = alc->lkount;
	}
#ifdef GGG
	freeobjlist(kcont->bol,kcont->maxnumber);
#endif
	release(kcont->ool,&alc->lkount);
	kcont->maxnumber = 0;
	if (alc->fipdig) {
		freeobj(alc->fipdig);
		alc->fipdig = NULL;
	}
	alc->cellin = 0;
	alc->lbelld = 0;
	kcont->FIP = 0;
	freefuse();
} /* end freecell */

print_kcont_all_ol()
{
	print_kcont_cl("ool", kcont->ool);
	print_kcont_cl("rol", kcont->rol);
	print_kcont_cl("mol", kcont->mol);
	print_kcont_cl("kol", kcont->kol);
	print_kcont_ol("bol", kcont->bol);
}

print_kcont_cl(name, list)
char *name;
struct chromosome **list;
{
	fprintf(debug_channel, "%s list\n", name);
	while (*list) fprintf(debug_channel, "0x%x ", *list++);
	fprintf(debug_channel, "\n");
}

print_kcont_ol(name, list)
char *name;
struct object **list;
{
	fprintf(debug_channel, "%s list\n", name);
	while (*list) fprintf(debug_channel, "0x%x ", *list++);
	fprintf(debug_channel, "\n");
}

/*
 *	S T O P S E T	--	
 */
stopset(time)
int time;
{
	stoptime = tick() + time;
	return(0);
} /* end stopset */

/*
 *	S T O P T E S T	--	
 */
stoptest()
{
	return(tick() >= stoptime);
} /* end stoptest */

/*
 *	S T O P W A I T	--	
 */
stopwait()
{
	int timeleft;

	timeleft = stoptime - tick();
	if (timeleft > 0)
		tsleep(timeleft);
} /* end stopwait */

/*
 *	F I N D S C A L E -- Calc ix, iy values for scale 2
 *	in order that objects in oblist will fit in area dx by dy
 */
findscale(oblist, dx, dy, sx, sy)
struct object **oblist;
int dx, dy, *sx, *sy;
{
	int maxx=0, minx=4096, maxy=0, miny=4096, ax, ay;
	float s;
	struct object *obj;
	if (alc->debug)
		fprintf(debug_channel, "findscale: Scaling to fix %d x %d\n", dx, dy);
/*
 *	Step thru objs to find x/y limits == cellsize()
 */
	while ((obj = *oblist) != NULL) {
 		if (obj->idom->lastkl > maxx) maxx = obj->idom->lastkl;
 		if (obj->idom->kol1 < minx) minx = obj->idom->kol1;
 		if (obj->idom->lastln > maxy) maxy = obj->idom->lastln;
 		if (obj->idom->line1 < miny) miny = obj->idom->line1;
		oblist++;
 	}
	if (alc->debug)
		fprintf(debug_channel, "Obj. list bound coords\nmaxx:%d, minx:%d\nmaxy:%d, miny:%d\n",
	 		maxx, minx, maxy, miny);
	ax = maxx - minx;
	ay = maxy - miny;
	if (ay > ax) { 				/* ((4*ax)/3)) {*/
		if (alc->debug) fprintf(debug_channel, "Using y rule\n");
		s = (float)dy/(float)ay;
		*sx = (int)((s*(float)4)/(float)3);
		*sy = (int)s;
	} else {
		if (alc->debug) fprintf(debug_channel, "Using x rule\n");
		s = (float)dx/(float)ax;
		*sx = (int)s;
		*sy = (int)((s*(float)3)/(float)4);
	}
	if (alc->debug)
		fprintf(debug_channel, "s = %f, sx, sy values are %d, %d\n",
			s, *sx, *sy);
} /* end findscale */

/*
 *	I N I T B O X -- Initialise box to be in centre of screen
 *	Do nothing if box already initialised.
 */
initbox(boxp)
struct box **boxp;
{
	DDGS *fs;

	fs = alc->ddgschannel;

	if (!*boxp) {
		if ((*boxp = (struct box *)Calloc(1, sizeof(struct box))) == NULL) {
			fprintf(stderr, "Can't allocate box space\n");
			return(-1);
		}
		(*boxp)->bleft = (int)fs->xcenter - (DXB/2);
		(*boxp)->bright = (int)fs->xcenter + (DXB/2);
		(*boxp)->btop = (int)fs->ycenter + (DYB/2);
		(*boxp)->bbot = (int)fs->ycenter - (DYB/2);
	}
	if (alc->debug)
		fprintf(debug_channel, "bleft=%d, bright=%d, btop=%d, bbot=%d\n",
		(*boxp)->bleft, (*boxp)->bright, (*boxp)->btop, (*boxp)->bbot);
} /* end initbox */

/*
 *	B L O W _ U P -- Expand cell to fit available screen size
 */
blow_up()
{
	int mrow, mcol, tt1, tt2, xmin, xmax, ymin, ymax;

	if (alc->fipdig) { /* From digitised object */
		mcol = (alc->fipdig->idom->line1 + alc->fipdig->idom->lastln) / 2;
		mrow = (alc->fipdig->idom->kol1 + alc->fipdig->idom->lastkl) / 2;
		calcscale(&tt1,&tt2); 		/* Calc. scales for showdc */
	}
 	else {	/* From labelled (possibly fused) object */
		findscale(kcont->ool, alc->ddgschannel->xcenter*2,
			alc->ddgschannel->ycenter*2, &tt2, &tt1);
		cellsize(kcont->ool, alc->lkount, &xmin, &xmax, &ymin, &ymax);
		mcol = (ymin + ymax)/2;
		mrow = (xmin + xmax)/2;
 	}
	showdc(tt1, tt2, xmid, ymid, mrow, mcol);
}

/*
 *	S H O W D C 
 *	Display either digitised object or fused object list.
 *	Moved here from dbuf.c
 */
showdc(p, q, xmid, ymid, midrow, midcol)
int p, q, xmid, ymid, midrow, midcol;
{ 
	struct pframe f;
	struct object **oblist;
	int c;

	/* initialise display */
	qclear();

	/*
	 *	set display frame and display normalised object
	 */
	f.scale = 1;
	f.iy = p;
	f.ix = q;
	f.dx = alc->ddgschannel->xcenter + xmid;
	f.dy = alc->ddgschannel->ycenter + ymid;
	f.ox = midrow;
	f.oy = midcol;
	colour(GREYCOLS & 0xf8);
	intens(1);
	if (alc->fipdig) {
		picframe(alc->fipdig,&f);
	} else {
		oblist = (struct object **)kcont->ool;
		for(c = alc->lkount; c>0; c--) {
			if (*oblist != NULL) {
				picframe(*oblist, &f);
				oblist++;
			}
		}
	}
} /* end showdc */

/*
 *	S E T A F R A M E - Set frame with aspect ratio
 *	i.e. different ix/iy scales
 */
static int
setaframe(frame, ix, iy)
struct pframe *frame;
SMALL ix, iy;
{
	frame->type = 60;
	frame->ix = ix;
	frame->iy = iy;
	frame->scale = 1;
	frame->ox = -20;
	frame->oy = -20;
	frame->dx = 0;
	frame->dy = 0;
}

/*
 *	N E X T _ M E T
 */
static int
next_met()
{
	int ii, ir;

	if (filed || check("Digitised image not stored, proceed (M): ")) {
		filed = TRUE; /* pretend */
		if(!nextrec(FORW))
			fprintf(output,"End of subset list\007\n"); 
		setcprcd(&ii, &ir);			/* Get record and display */
		setbox();
		return(0);
	}
}
