/*
 *      C C O N V . C  --  Coordinate conversion low-level stuff
 *
 *	THIS MODULE IS CLEAN - Keep it that way.
 *
 *	Extracted from mic_conv.c which is a tangled mess.....
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1991)
 *
 *  Date:    18th June 1991
 *
 *  Modifications
 *
 *	31 Jul 1991		CAS		Further tidy -> new cconv to here, menus -> cconv_menu.c
 */
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <cconv.h>

/*
 *	External functions needed
 */
extern	char	*skipwhite(), *skipword();


/********************************************************************
 *
 *		COORDINATE CONVERSION STUFF
 *
 ********************************************************************/
#include <ctype.h>
#include <math.h>
/*#include <cconv.h> */

static struct microscope Cytoscan = {
					0,						/* Control */
					0,						/* Id number */
					{ -11648, -11674 },		/* Z50 */
					{  11649,  27149 },		/* A10 */
					1, 1,					/* left handed, inverted */
					"Cytoscan"				/* Name */
			};

/*
 *	C O O R D C O N V E R T  --
 */
coordconvert(m_from, m_to, p_from, p_to)
struct microscope *m_from, *m_to;
struct coord *p_from, *p_to;
{
	cc_init(m_from);
	cc_init(m_to);
	cc_correct(m_from,p_from,p_to);		/* correct sense of everything */
	cc_translate(m_from,m_to,p_to,p_to);
	cc_scale(m_from,m_to,p_to,p_to);
	cc_rotate(m_from,m_to,p_to,p_to);
	cc_uncorrect(m_to,p_to,p_to);		/* Mangle sense for target mic */
}

/*
 *	-- export pointer to cytoscan mic coord conv structure
 */
getcytoscan_mic()
{
	return(&Cytoscan);
}

/*
 *	C C _ I N I T  --  coordinate conversion initialise Microscope struct
 */
cc_init(mic)
struct microscope *mic;
{
	int		deltax, deltay, tmp;
	if ((mic == NULL) || (mic->control & CC_INIT))
		return(0);
	deltax = mic->Z50.x - mic->A10.x;
	deltay = mic->Z50.y - mic->A10.y;
/*	printf("mic %s:",mic->id); */
	if (abs(deltax) > abs(deltay)) {
/*		printf(" x-y swap"); */
		mic->control |= CC_XYSWAP;
		tmp = deltax; deltax = deltay; deltay = tmp;
	}
	if (deltax < 0) {
		mic->control |= CC_NEGX;
/*		printf(" neg_x"); */
	}
	if (deltay < 0) {
		mic->control |= CC_NEGY;
/*		printf(" neg_y"); */
	}
	mic->scale = (int) sqrt((double)deltax * (double)deltax 
							+ (double)deltay * (double)deltay);

	cc_correct(mic,&(mic->Z50),&(mic->cor_Z50));
	cc_correct(mic,&(mic->A10),&(mic->cor_A10));
/*
 *	Compute sin + cos of angle of the z50/a10 line to use as a
 *	rotation datum
 */
	mic->sinth = (double) abs(deltax) / (double) mic->scale;
	mic->costh = (double) abs(deltay) / (double) mic->scale;

/*	printf(" cor z50 %d %d, a10 %d %d, scale %d\n",mic->cor_Z50.x, mic->cor_Z50.y,
							mic->cor_A10.x, mic->cor_A10.y, mic->scale); */
	mic->control |= CC_INIT;
}

/*
 *	C C _ C O R R E C T  --  Correct mic coords to internal format
 *								Basically fixes for gross orientation problems
 *								where axes are different
 */
cc_correct(mic,old,new)
register struct microscope *mic;
register struct coord *old, *new;
{
	register int	tmp;

	if (mic->control & CC_XYSWAP) {
		tmp = old->x;
		new->x = old->y;
		new->y = tmp;
	} else if (new != old) {
		new->x = old->x;
		new->y = old->y;
	}
	if (mic->control & CC_NEGX) {
		new->x = -new->x;
	}
	if (mic->control & CC_NEGY) {
		new->y = -new->y;
	}
}

/*
 *	C C _ U N C O R R E C T  --  change internal coords to external format
 *								Basically fixes for gross orientation problems
 *								where axes are different
 */
cc_uncorrect(mic,old,new)
register struct microscope *mic;
register struct coord *old, *new;
{
	register int	tmp;

	if (new != old) {
		new->x = old->x;
		new->y = old->y;
	}
	if (mic->control & CC_NEGY) {
		new->y = -new->y;
	}
	if (mic->control & CC_NEGX) {
		new->x = -new->x;
	}
	if (mic->control & CC_XYSWAP) {
		tmp = new->x;
		new->x = new->y;
		new->y = tmp;
	}
}

cc_translate(m_from, m_to, p_from, p_to)
struct microscope *m_from, *m_to;
struct coord *p_from, *p_to;
{
	
}

cc_scale(m_from, m_to, p_from, p_to)
struct microscope *m_from, *m_to;
struct coord *p_from, *p_to;
{
	p_to->x = (p_from->x * m_to->scale) / m_from->scale;
	p_to->y = (p_from->y * m_to->scale) / m_from->scale;
}

/*
 *	C C _ R O T A T E  --  Rotate a point about its origin from one mic frame
 *							to the other. This does 2 rotates since to save
 *							re-evaluating the sin/cos terms each time. Ie rotate
 *							clockwise by A and counterclockwise by B
 *
 *	Uses standard 2x2 rotation matrices:
 *	Clockwise:
 *		[x]  =  [cos A   -sin A] [x]
 *		[y]     [sin A    cos A] [y]
 *	Counter-clockwise
 *		[x]  =  [ cos A   sin A] [x]
 *		[y]     [-sin A   cos A] [y]
 */
cc_rotate(m_from, m_to, p_from, p_to)
struct microscope *m_from, *m_to;
struct coord *p_from, *p_to;
{
	double		tmpx, tmpy;
/* Clockwise */
	tmpx = (double) p_from->x * m_from->costh - (double) p_from->y * m_from->sinth;
	tmpy = (double) p_from->x * m_from->sinth + (double) p_from->y * m_from->costh;
/* Counter clockwise */
	p_to->x = (int) (tmpx * m_to->costh + tmpy * m_to->sinth);
	p_to->y = (int) (tmpy * m_to->costh - tmpx * m_to->sinth);
}

/*
 *	M I C R E A D  -- Find microscope data in microscope data base file
 *						Read into struct `mic'
 *
 */
int	micread(name,mic)
char *name;
struct	microscope *mic;
{
	FILE	*fp;
	struct	microscope tmp;

	if ((fp = fopen(MSCOPEFILE, "r")) == (FILE *) -1 ) {
		printf("Unable to read microscope file.\007\n");
		return(0);
	}

	while(read_mic_line(fp,&tmp) != -1) {
		if (strcasecmp(tmp.id, name) == 0) {
			_strass(mic,&tmp,sizeof(struct microscope));
			mic->control = 0;
			fclose(fp);
			return(1);
		}
	}
	fclose(fp);
	return(0);
}

/*
 *	R E A D _ M I C _ L I N E  --  Read 1 line from MIC file
 */
read_mic_line(fp,mic)
FILE		*fp;
struct microscope *mic;
{
	char	buffer[100], *bptr;
	int		i;

	while (1) {
		if (fgets(buffer,100,fp) == NULL)
			return(-1);
		bptr = buffer;
		bptr = skipwhite(bptr);
		if (isdigit(*bptr)) {
			mic->number = atoi(bptr);
			bptr = skipwhite(skipword(bptr));		/* Point to mic name */
			i = 0;
			while (*bptr && (*bptr != ' ') && (*bptr != '\t')) {
				if (i < (MAXNAME-1))
					mic->id[i++] = *bptr++;
				else
					bptr++;
			}
			mic->id[i] = '\0';
			bptr = skipwhite(bptr);
			mic->Z50.x = (int) (atof(bptr) * 1000 );
			bptr = skipwhite(skipword(bptr));
			mic->Z50.y = (int) (atof(bptr) * 1000 );
			bptr = skipwhite(skipword(bptr));
			mic->A10.x = (int) (atof(bptr) * 1000 );
			bptr = skipwhite(skipword(bptr));
			mic->A10.y = (int) (atof(bptr) * 1000 );
			bptr = skipwhite(skipword(bptr));
			if (strncmp(bptr, "LH", 2) == 0)
				mic->left_handed = 1;
			else
				mic->left_handed = 0;
			bptr = skipwhite(skipword(bptr));
			if (strncmp(bptr, "INV", 3) == 0)
				mic->inverted = 1;
			else
				mic->inverted = 0;
			return(1);
		}
	}
}

/*
 * F L O A T _ S T R -- convert fixed point number to string
 *						Number is *1000
 *						Result is format [-]XX.Y[5]
 *
 */
float_str(buffer, number)
char	*buffer;
int		number;
{
	int		whole, fract, dig1, negative = 0, five = 0;

	if (number < 0) {
		negative = 1;
		number = -number;
	}
	number += 25;		/* Round up (0-24 -> 0, 25-74 -> '5', 75-99 -> '10') */
	whole = number / 1000;				/* Integer part of number */
	fract = number - (whole * 1000);	/* decimal part of number */

	dig1 = fract / 100;
	if ((fract - (100 * dig1)) >= 50)
		five = 1;
	sprintf(buffer,"%s%d.%d%s",negative ? "-" : "", whole, dig1, five ? "5" : "");
	return(buffer);
}


/*
 *	R O T A T E -- Rotate a point through angle theta.
 *
 */	
rotate(newx, newy, oldx, oldy, sinth, costh)
float	*newx, *newy, oldx, oldy, sinth, costh; 
{
	*newx = oldx * costh - oldy * sinth;
	*newy = oldx * sinth + oldy * costh;
}

/*
 *	N E X T L I N E  --
 *
 */
static int nextbufline(fd,buffer,len)
int		fd, len;
char	*buffer;
{
	int		cnt;

	while ((cnt = readln(fd,buffer,len)) == 1)
		;
	if (cnt <= 0)
		return(0);
	else {
		buffer[cnt-1] = '\0';
		return(cnt);
	}
}

/*
 * S Q U A R E -- square two integer and return the sum
 *
 */
int square(x, y)
register int x, y;
{
	return((x * x) + (y * y));
}

/*
 *	A T A N 2 -- Proper arctan evaluation through 360 degrees/2*pi radians
 *
 */
float	atan2(Dy, Dx)
float	Dy, Dx;
{
	float	ttht, Pi;
	
	Pi = 4 * atan(1.);
	
	if (!Dx) {
		return ((Dy < 0) ? -Pi/2 : Pi/2);
	}
	ttht = atan(fabs(Dy/Dx));
	if (Dy/Dx <= 0) {
		return ((Dx < 0) ? (Pi - ttht) : -ttht);
	} else {
		return ((Dy < 0) ? (Pi + ttht) : ttht);
	}
}

/*
 *	C H E K S T R -- Check that string is sensibly numeric.
 *						Resets to "0.000" if not
 */
int chekstr(mystr)
char	*mystr;
{
	char	*cptr;

	cptr = mystr;
	if ((*cptr) == '+') {			/* Remove a leading + if it's there. */
		do {
			*cptr = *(cptr+1);
		} while((*cptr++) != '\0');
		cptr = mystr;
	} else if ((*cptr) == '-') {	/* Skip over a leading minus. */
		cptr = mystr+1;
	}

	while ((*cptr) != '\0') {		/* Check the rest for sensible values. */
		if(!isdigit(*cptr) && (*cptr) != '.') {
			sprintf(mystr, "0.000");
			return(0);
		}
		cptr++;
	}
	return(1);
}

/*
 *	S K I P W O R D  --
 *
 */
static char *skipword(ptr)
char	*ptr;
{
	while (*ptr && (*ptr != ' ') && (*ptr != '\t'))
		ptr++;
	return(ptr);
}

/*
 *	S K I P W H I T E  --
 *
 */
static char *skipwhite(ptr)
char	*ptr;
{
	while ((*ptr == ' ') || (*ptr == '\t'))
		ptr++;
	return(ptr);
}

