/*
 *      D R I V B O X . C  --  Drive a box around with a mouse
 *
 *
 *  Taken from drivbox.c  Nov 85 : Denis Rutovitz
 * 
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:	29 jan 1987
 *
 *	invoked by call to newbox (caption,&box), 
 *	uses woolz cursor objects [ styles 4 and 6 as now ]
 *	connected to mouse reader in thismu by call to setmsv -
 *
 *	user picks up edge of box by moving cursor to it style changes 
 *	when acquired moves edge along with cursor until either loses due
 *	to too rapid movement or middle mouse button
 *
 *	test prgm:  trybox.c 
 *
 *	Getting timing acceptable rather tricky - hence
 *		a) partial box erase (as requd)
 *		b) min movement before redisplay
 *
 *	All subroutines local except     newbox , dwb
 *
 * Modifications
 *
 *	17 May 1989		dcb			Pick hard limits from ddgschannel
 *	14 Sep 1988		CAS			Mousescale from alc (scaled by 8)
 *	29 Jul 1987		CAS			Pick up mouss from alc->m2s
 *	 9 Mar 1987		CAS			Added better slide limit checking
 *	 9 Feb 1987		CAS			Scan limits now in alc
 *	 5 Feb 1987		CAS			Add box edge checking based on SLIDE scan
 *								limits
 *	29 Jan 1987		CAS			Problems with box edges going out of desirable
 *								range, negative areas etc
 *								Good general hack
 *	15 Sep 1986		CAS			Added a lot of undef's - bad move - there
 *								is obviously a problem with include files
 *								here somewhere..
 *   8 Aug 1986		GJP			Change umouse to mukont->umouse
 *	 9/4/86   denis/mark       boxdrive limited to slide in metaphase finding
 */


#include <cmdsys.h>
#include <wstruct.h>
#include <allcontr.h>
#include <slides.h>

#define MIN(a,b) (((a)>(b)) ? (b) : (a))
#define MAX(a,b) (((a)>(b)) ? (a) : (b))


/*
 *	Set up defined constants
 */
#define MINBOX	100	/* Smallest box size we allow */
#define MINMOVE 32	/* if less than MINMOVE points box to cursor,
													 leave box alone */
#define MAXLATCH 64	/* if futher than MAXLATCH from edge, unlatch */
#define SCALE 475
#define BORDER 100	/* Can't go within this dist. of hard limits */
extern struct allkontr	*alc;

/*
 *	Current box absolute limits
 */
static	int		left_lim    = BORDER,
				right_lim	= BORDER,
				top_lim		= BORDER,
				bottom_lim	= BORDER;

/*
 *	arrays to determine when to latch on to an edge (and unlatch)
 */
static lray0[] = { -4096, 500,600,3400,3500, 4096};
static kray0[] = { -4096, 500,600,3400,3500, 4096};
/*
 *	Box stretch modes - actual values determined by newbox menu
 */
#define SEPARATELY 0			/* sides move separately */
#define TOGETHER  -1			/* sides move together */
#define OPPOSITE   1			/* sides move oppositely */
static int mode = 2; 

/*
 *	Box sides
 */
#define LEFT	1
#define	RIGHT	2
#define	BOTTOM	3
#define TOP		4

/*
 *	Cursor styles - should be in woolz header but temp here for now
 */
#define CURS_BOX	4
#define CURS_ARROW	6

/*
 *	Local static data
 */
static int lastlatch = LEFT;	/* Last thing we were latched to */
static *lrayw, *krayw;
static struct ipoint cursor;
static int oleft,oright,otop,obot;
static struct box *boxp;
static latchon;
static struct pframe fr;
static curscolr, boxcolr;

/*
 *	C U R S T Y L E  --  Change cursor style + redisplay
 */
static
curstyle(i)
{
		colour(curscolr);
		intens(0); /* erase old cursor prelimanry to changing 
						cursor style */
		picframe(&cursor,&fr);
		cursor.style = i;
		intens(1);
		picframe(&cursor,&fr);
}

/*
 *	B O X W I N D O W  --  Set outside limits for metaphase finding
 *							box.  These should be set from some global
 *							data somewhere - but you know how it is..
 */
boxwindow()
{
	struct	box lbox;
	slide_to_box(&lbox,alc->scanleft,alc->scanright,alc->scantop,alc->scanbot);
/*
 *	left_lim = 1800;
 *	right_lim = 2900;
 *	bottom_lim = 1150;
 *	top_lim = 3135;
 */
	left_lim = lbox.bleft;
	right_lim = lbox.bright;
	bottom_lim = lbox.bbot;
	top_lim = lbox.btop;
}

/*
 *	Check met_find box limits and reset if nec - Comments on boxwindow
 *	also apply here
 */
slidelimits(xlow,xhigh,ylow,yhigh)
int	*xlow,*xhigh,*ylow,*yhigh;
{
	int		temp;

	if (*xlow > *xhigh) {		/* If limits reversed - swap them back */
		temp = *xlow;
		*xlow = *xhigh;
		*xhigh = temp;
	}
	if (*ylow > *yhigh) {
		temp = *ylow;
		*ylow = *yhigh;
		*yhigh = temp;
	}
	if (*xlow < alc->scanleft ) *xlow = alc->scanleft;
	if (*xhigh > alc->scanright) *xhigh = alc->scanright;
	if (*ylow < alc->scantop) *ylow = alc->scantop;
	if (*yhigh > alc->scanbot) *yhigh = alc->scanbot;
}

/*
 *	S L I D E _ T O _ B O X  --  Convert stage/slide coords to box (screen)
 *									co-ords
 */
slide_to_box(mbox,xlow,xhigh,ylow,yhigh)
struct box *mbox;
int	xlow,xhigh,ylow,yhigh;
{
	mbox->bbot =  YMID - ( (yhigh+800)  / CONV );
	mbox->btop =  YMID - ( (ylow-800)  / CONV );
	mbox->bleft =  XMID + ( xlow  / CONV );
	mbox->bright =  XMID + ( xhigh  / CONV );
}

/*
 *	B O X _ T O _ S L I D E  --  Convert box (screen) co-ord to stage/slide
 *								coords
 */
box_to_slide(mbox,xlow,xhigh,ylow,yhigh)
struct box *mbox;
int	*xlow,*xhigh,*ylow,*yhigh;
{
	*yhigh = (( YMID - mbox->bbot ) * CONV)-800 ;
	*ylow = 800-(( mbox->btop - YMID) * CONV) ; 
	*xlow = ( mbox->bleft - XMID ) * CONV ;
	*xhigh = ( mbox->bright - XMID ) * CONV ;
}

/*
 *	D I G W I N D O W  --  Sets outside box limits for digitisation box
 */
digwindow()
{
	left_lim = bottom_lim = BORDER;
	right_lim = (alc->ddgschannel->xcenter * 2) - BORDER;
	top_lim = (alc->ddgschannel->ycenter * 2) - BORDER;
}

/*
 *	L C U R L I M S  --  Set up vertical range that cursor will latch
 *							to top/bottom of box
 */
static
lcurlims()
{
	lray0[1] = boxp->bbot - MAXLATCH;
	lray0[2] = boxp->bbot + MAXLATCH;
	lray0[3] = boxp->btop - MAXLATCH;
	lray0[4] = boxp->btop + MAXLATCH;
	lray0[5] = alc->ddgschannel->ycenter * 2;
}

/*
 *	K C U R L I M S  --  Set up horizontal range that cursor will latch
 *							to left/right of box
 */
static 
kcurlims() 
{
	kray0[1] = boxp->bleft - MAXLATCH;
	kray0[2] = boxp->bleft + MAXLATCH;
	kray0[3] = boxp->bright - MAXLATCH;
	kray0[4] = boxp->bright + MAXLATCH;
	kray0[5] = alc->ddgschannel->xcenter * 2;
}

/*
 *	B T E D G E  --  Redisplay bottom/top edge of box (Minimum line drawing)
 *
 *	(if diff>0 box gets smaller)
 */
static btedge ( oedge,edge,diff ) 
{
	colour (boxcolr);
	intens (0);
	if ( diff >  0 ) {
		moveto (boxp->bleft,oedge); lineto (boxp->bleft,edge);
		moveto (boxp->bright,oedge); lineto (boxp->bright,edge); 
	} 
	moveto (boxp->bleft,oedge); lineto(boxp->bright,oedge);
	intens(1);
	if ( diff <  0 ) {
		moveto (boxp->bleft,oedge); lineto (boxp->bleft,edge);
		moveto (boxp->bright,oedge); lineto (boxp->bright,edge);
	} 
	moveto (boxp->bleft,edge); lineto(boxp->bright,edge);
	lcurlims();
}

/*
 *	L R E D G E  --  Redisplay left/right edge of box
 *
 * (if diff>0 box gets bigger)
 */
static lredge ( oedge,edge,diff ) 
{
		colour (boxcolr);
		intens (0);
		if ( diff < 0 ) {
			moveto (oedge,boxp->bbot); lineto (edge,boxp->bbot);
			moveto (oedge,boxp->btop); lineto (edge,boxp->btop); 
		} 
		moveto (oedge,boxp->bbot); lineto(oedge,boxp->btop);
		intens(1);
		if ( diff > 0 ) {
			moveto (oedge,boxp->bbot); lineto (edge,boxp->bbot);
			moveto (oedge,boxp->btop); lineto (edge,boxp->btop); 
		} 
		moveto (edge,boxp->bbot); lineto(edge,boxp->btop);
		kcurlims();
}


/*
 *	U N D R I V E  --  if latched - unlatch
 *						else latch to next edge round box from previous
 *						latched one
 */
undrive() 
{
	if (latchon > 0) {
		lastlatch = latchon;
		latchon = -20 ; 
		curstyle(CURS_ARROW);
		dwb(1);	/* redisplay box in case showing cursor damage */
	} else {
		intens(0);
		colour(curscolr);
		picframe(&cursor,&fr);
		switch ( lastlatch ) {
			case TOP:
					cursor.k = boxp->bleft;
					latchon = LEFT;
					cursor.l = (boxp->bbot+boxp->btop)/2;
					break;
			case BOTTOM:
					cursor.k = boxp->bright;
					latchon = RIGHT;
					cursor.l = (boxp->bbot+boxp->btop)/2;
					break;
			case LEFT:
					cursor.l = boxp->bbot;
					latchon = BOTTOM;				
					cursor.k = (boxp->bleft+boxp->bright)/2;
					break;
			case RIGHT:
					cursor.l = boxp->btop;
					latchon = TOP;
					cursor.k = (boxp->bleft+boxp->bright)/2;
					break;
		}
		curstyle(CURS_BOX);
	}
}

/*
 *	B O X C U R S  --  Called from CMDSYS when the mouse moves
 */
static boxcurs()
{
	register int m1, *lray,*kray, kswitch, lswitch, mydiff,newlatch,temp;
	int oldlatch,limit;

	oldlatch = latchon;
	colour (curscolr);
	intens(0);		/* delete old cursor */
	picframe(&cursor,&fr);
	intens(1);
	cursorpos(&cursor,boxp,latchon,mode,alc->m2s->XINC*8*alc->mousescale,alc->m2s->YINC*8*alc->mousescale);

/*
 *	Now we have updated cursor, if latchon is negative don't look to
 *	see if were near enough to latch something.  this is to give time to
 *	move the cursor away after we unlatch else we would re-latch before
 *	we had time to move
 */
	if (latchon<0 ) {
		picframe(&cursor,&fr);
		++latchon;
		return;
	} 
/*
 *	Check whether we are in range of any of the box sides
 *
 *	Check vertically first
 */
 	newlatch = 0;
	m1 = cursor.l;
	kray = krayw;
	lray = lrayw;
	while ( m1 > *lray ) ++ lray;
	while ( m1 < *lray ) --lray;
	switch ( lswitch = lray - lray0 ) {
		case 0:									/* Below the box */
			if (latchon > 0) newlatch = BOTTOM;
			break;
		case 4: 								/* Above the box */
			if (latchon > 0) newlatch = TOP;
			break;
		case 1:									/* Here we are in the box */
		case 2:									/* vertically - now check */
		case 3:									/* horizontally */
			m1 = cursor.k;
			while ( m1 > *kray ) ++kray;
			while ( m1 < *kray ) --kray;
			switch ( kswitch = kray - kray0 ) {
/*
 * if latched on and too left or too right, force latch to left or right 
 */
				case 0:
					if (latchon > 0) newlatch = LEFT;	/* Well left of box */
					break;
				case 4:
					if (latchon > 0) newlatch = RIGHT;	/* Well right of box */
					break;
/*
 * if inside box, and latched, stay with it ; if not, check new latch 
 */
				case 1:
					newlatch = LEFT;
					break;
				case 2:
					switch (lswitch) {
						case 1:
							newlatch = BOTTOM;
							break;
						case 2:
							if (latchon > 0)
								newlatch = latchon;
							break;
						case 3:
							newlatch = TOP;
							break;
					}
					break;
				case 3:
					newlatch = RIGHT;
					break;
		}
		break;
	}	
	
	switch (newlatch) {
		case LEFT:
			if (latchon != LEFT) {
				cursor.k = boxp->bleft;
				latchon = LEFT;
			}
			if ( iabs ( cursor.k - oleft ) < MINMOVE ) break;
			boxp->bleft = cursor.k; 
Ltl1:		lredge ( oleft, boxp->bleft, (mydiff = oleft - boxp->bleft ));
			oleft = boxp->bleft;
			if ((mode != SEPARATELY) && latchon == LEFT ) {
				boxp->bright += ((mode == OPPOSITE) ? mydiff : -mydiff);
				goto Rtl1;
			}
			break;
		case RIGHT:
			if (latchon != RIGHT) {
				cursor.k = boxp->bright;
				latchon = RIGHT;
			}
			if (iabs(cursor.k - oright) < MINMOVE) break;
			boxp->bright = cursor.k; 
Rtl1:		lredge (oright, boxp->bright, (mydiff = boxp->bright - oright));
			oright = boxp->bright;
			if ((mode != SEPARATELY) && latchon == RIGHT) {
				boxp->bleft -= ((mode == OPPOSITE) ? mydiff : -mydiff);
				goto Ltl1;
			}
			break;
		case TOP:
			if (latchon != TOP) {
				cursor.l = boxp->btop;
				latchon = TOP;
			}
			if ( iabs ( cursor.l - otop  ) < MINMOVE ) break;
			boxp->btop = cursor.l; 
Ttl1:		btedge ( otop, boxp->btop, mydiff = otop - boxp->btop );
			otop = boxp->btop;
			if ((mode != SEPARATELY) && latchon == TOP ) {
				boxp->bbot -= ((mode == TOGETHER) ? mydiff : -mydiff);
				goto Btl1;
			}
			break;
		case BOTTOM:
			if (latchon != BOTTOM) {
				cursor.l = boxp->bbot;
				latchon = BOTTOM;
			}
			if ( iabs ( cursor.l - obot  ) < MINMOVE ) break;
			boxp->bbot = cursor.l; 
Btl1:		btedge ( obot, boxp->bbot, mydiff = boxp->bbot - obot );
			obot = boxp->bbot;
			if ((mode != SEPARATELY) && latchon == BOTTOM ) {
				boxp->btop += ((mode == TOGETHER) ? mydiff : -mydiff);
				goto Ttl1;
			}
			break;
	}
	if ( !oldlatch && (latchon > 0)) 
		cursor.style = CURS_BOX;
	picframe(&cursor,&fr);
	krayw = kray;
	lrayw = lray;
}

/*
 *	C U R S O R P O S  --  Calculate new valid cursor position
 */
static
cursorpos(cursor,boxp,latchon,mode,xinc,yinc)
struct ipoint *cursor;
struct box *boxp;
int		latchon, mode, xinc, yinc;
{
	int	highlim, lowlim, boxmid;

/*
 *	First calculate limits horizontally
 */
	if (latchon <= 0) {
		highlim = (alc->ddgschannel->xcenter * 2) - BORDER;
		lowlim = BORDER;
	} else {
		highlim = right_lim;
		lowlim = left_lim;
		switch (mode) {
			case OPPOSITE:
				boxmid = (boxp->bright - boxp->bleft - MINBOX)/2;
				if (boxmid < 0) boxmid = 0;
		  		if (latchon == LEFT) {
		  			lowlim =  MAX(left_lim,boxp->bleft - (right_lim - boxp->bright));
		  			highlim = boxp->bleft +  boxmid;
		  		} else if (latchon == RIGHT) {
		  			lowlim = boxp->bright - boxmid;
		  			highlim = MIN(right_lim,boxp->bright + (boxp->bleft - left_lim));
		  		}
		  		break;
		  	case TOGETHER:
		  		if (latchon == LEFT) {
		  			lowlim = left_lim;
		  			highlim = boxp->bleft + (right_lim - boxp->bright);
		  		} else if (latchon == RIGHT) {
		  			lowlim = boxp->bright - (boxp->bleft - left_lim);
		  			highlim = right_lim;
		  		}
		  		break;
		  	case SEPARATELY:
		  		if (latchon == LEFT) {
		  			lowlim = left_lim;
		  			highlim = boxp->bright - MINBOX; 
		  		} else if (latchon == RIGHT) {
		  			lowlim = boxp->bleft + MINBOX;
		  			highlim = right_lim;
		  		}
		  		break;
		}
	}
/*
 *	Having got the limits - update the 'x' cursor position
 */
	cursor->k += xinc;
	if (highlim < lowlim) {
		if (latchon == LEFT)
			highlim = lowlim;
		else if (latchon == RIGHT)
			lowlim = highlim;
	}
	if (cursor->k > highlim) cursor->k = highlim;
	else if (cursor->k < lowlim) cursor->k = lowlim;
/*
 *	Now do vertical check
 */
	if (latchon <= 0) {
		highlim = (alc->ddgschannel->ycenter * 2) - BORDER;
		lowlim = BORDER;
	} else {
		highlim = top_lim;
		lowlim = bottom_lim;
		switch (mode) {
			case OPPOSITE:
				boxmid = (boxp->btop - boxp->bbot - MINBOX)/2;
				if (boxmid < 0) boxmid = 0;
		  		if (latchon == BOTTOM) {
			  		lowlim = MAX(bottom_lim,boxp->bbot - (top_lim - boxp->btop));
		  			highlim = boxp->bbot +  boxmid;
		  		} else if (latchon == TOP) {
		  			lowlim = boxp->btop - boxmid;
		  			highlim = MIN(top_lim,boxp->btop + (boxp->bbot - bottom_lim));
		  		}
		  		break;
		  	case TOGETHER:
		  		if (latchon == BOTTOM) {
		  			lowlim = bottom_lim;
		  			highlim = boxp->bbot + (top_lim - boxp->btop);
		  		} else if (latchon == TOP) {
		  			lowlim = boxp->btop - (boxp->bbot - bottom_lim);
		  			highlim = top_lim;
		  		}
		  		break;
		  	case SEPARATELY:
		  		if (latchon == BOTTOM) {
		  			lowlim = bottom_lim;
		  			highlim = boxp->btop - MINBOX; 
		  		} else if (latchon == TOP) {
		  			lowlim = boxp->bbot + MINBOX;
		  			highlim = top_lim;
		  		}
		  		break;
		}
	}
	cursor->l += yinc;
	if (highlim < lowlim) {
		if (latchon == BOTTOM)
			highlim = lowlim;
		else if (latchon == TOP)
			lowlim = highlim;
	}
	if (cursor->l > highlim) cursor->l = highlim;
	else if (cursor->l < lowlim) cursor->l = lowlim;

}

/*
 *	N E W B O X  --
 */
newbox(caption, a , curcolr, bxclr )
char *caption;
struct box *a;
int		curcolr, bxclr;
{
	curscolr = curcolr;
	boxcolr  = bxclr;
	setmsv (boxcurs);
	if(!(strncmp(caption,"Search Limits"))) boxwindow();
	else digwindow();
	boxp = a;
	fr.ox = fr.dx = alc->ddgschannel->xcenter;
	fr.oy = fr.dy = alc->ddgschannel->ycenter;
	fr.func = 0;
	fr.ix =  fr.iy = fr.scale = 1;
	cursor.type = 40;
	cursor.style = CURS_ARROW;
	cursor.k = (boxp->bleft+boxp->bright)/2;
	cursor.l = (boxp->bbot+boxp->btop)/2;
	colour(curscolr);
	intens(1);
	picframe(&cursor,&fr);
	dwb(1);
	latchon = 0;
	mode = 0;
	
	newmenu ( caption );
		cmmd ( "drag box " );mouse(LEFTBT);
		cmmd ( "sides separately "); mouse(LONGRB);
		cmmd ( "bigger/smaller ");mouse(RIGHTBT);
		cmmd ( "let go/latch on ;%g",undrive);mouse(MIDBT);
		cmmd ( "exit ");mouse(LONGLB);
	while (thismu())  {
		if (nncmd==mmcmd) break;
		if (nncmd != 4) mode = nncmd-2;
	}
	latchon = 0;
/*	boxp = 0;	*/
	colour(curscolr);
	intens (0);
	picframe ( &cursor,&fr);
	intens(1);
	setmsv (NULL);
	return;

}

/*
 *	D W B  --  Draw box
 *
 *	Mode 	0 - erase previous box
 *			1 - redraw box
 *			2 - draw new box
 *	
 */
dwb(mode,bp,ccl,bcl)
int		mode, ccl, bcl;
struct box *bp;
{
		if ( mode == 2) { boxp = bp; curscolr = ccl; boxcolr = bcl; }
		colour (boxcolr);
		intens ( mode ) ;
		moveto ( boxp->bleft,boxp->bbot);
		lineto ( boxp->bleft,boxp->btop);
		lineto ( boxp->bright,boxp->btop);
		lineto ( boxp->bright,boxp->bbot);
		lineto ( boxp->bleft,boxp->bbot);
		lcurlims();
		kcurlims();
		lrayw = lray0 + 2;
		krayw = kray0 + 2;
		oleft = boxp->bleft;
		oright = boxp->bright;
		otop = boxp->btop;
		obot = boxp->bbot;
}

/*
 *	I A B S  --  Make number absolute
 */
static iabs ( k ) register k; { return ( k > 0 ? k : -k ) ; }

	
