/*
 *			F I N D M E T . C  --
 *
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *	Based on:	findmet
 *				June 85 : Mark McKie and Denis Rutovitz
 *				MRC CAPCU
 *				which was loosely based on cytel (Margaret Stark , June 85)
 *				and revel versions (Geoff Shippey, 84)
 *
 *
 *  Modifications
 *
 *	18 Jul 1991		CAS			Split menus out to mf_menu.c
 *	15 Jul 1991		CAS			Remove mipdebug/debugfp refs
 *	 4 Apr 1991		CAS			Devel option to allow afe interpolate mode
 *	15 Jan 1991		CAS/GJP		debug_channel used when not open
 *
 * 
 */
#include <time.h>
#include <cmdsys.h>
#include <kyy.h>
#include <slides.h>
#include <cfipp.h>
#include <fipcomm.h>
#include <metscan.h>
#include <afe_ctrl.h>

extern	char	*fiperr[];
extern	FILE	*debug_channel;
static  char	*stringin[30];

int	shading(), showprm(), others(), metlist(), 
	clusdis(), getbxfoc(), scanspeed(), killscan();
static mp_mip_setup(), check_met_tholds();
extern int tomip0();				/* Miproc routines */
extern int admip1();
extern int admip2();
init_offset();

/*
 *	I N I T B A Y S 
 *
 */
initbays()
{
static int firstime = 1;
int i;
struct mult_slide  *bp;

	if ((bp = alc->frsbay) == NULL) {
		if (alc->debug)
			fprintf(debug_channel, "initbays: mult_slide struct not set up\n");
		return(0);
	}
	for(i = 0; i < alc->maxbays; i++) {
		if (firstime) {
			setbaycl(bp, &bp->typeids);
			setdefaultbox(bp);
			slide_to_box(&bp->mbox,bp->xlow, 
					bp->xhigh, bp->ylow, bp->yhigh);
			bp->slwidth = alc->scanright - alc->scanleft;
			bp->active = ON;
			bp->retrain = OFF;
			bp->mcells = MCELLS;
			bp->area = AREA;
			bp->iod = IOD;
			bp->clus1 = CLUS1;
			bp->clus2 = CLUS2;
		}
		bp->thflag = OFF;
		bp->sspeed = defaultscanspeed();
		defaultmettholds(&bp->thresh1, &bp->thresh2);
		bp->mswathe = MSWATHE;
		bp->ygate = YGATE;
		bp->shadsw = SHADSW;
		bp->focN38 = 0;
		bp->focbox = 0;
		bp++;	
	}
	if (firstime) firstime = 0;
} /* end initbays */

/*
 *	S E T D E F A U L T B O X
 *
 */
setdefaultbox(bp)
struct	mult_slide *bp;
{
	bp->ylow = BYLLIM;		
	bp->yhigh = BYHLIM;		
	bp->xhigh = BXHLIM;		
	bp->xlow = BXLLIM;		
}
/*
 *	H I S T T A B  ---
 *
 *			Write histogram values and tholds.  Time
 *			of writing is also given.
 */
histtab()
{
struct sgtbuf timebuf;
 
	getime(&timebuf);
	fprintf(debug_channel, "\n\nRAW HISTOGRAM VALUES.\n");
	fprintf(debug_channel, "Time  Day: %d  Hour: %d  Min: %d  Sec: %d\n\n",
				(int) (timebuf.t_day), (int) (timebuf.t_hour),
				(int) (timebuf.t_minute), (int) (timebuf.t_second));
}

/*
 *
 *  D E F T H O L D S
 *
 *			Shading reset and histogram scan to define tholds
 *			on a particular bay.  Returns 1 if either tholds defined or
 *			manual thold flag ON.
 */
deftholds()
{
int	lth1, lth2;
struct	mult_slide *bp;

	bp = alc->curbay;
	autofocus(ON);
	if(scanspeed(bp->sspeed) != FS_OK)
		return(0);
	if (doshade < 1) {
		if (shading()) {
			return(0);
		} else
			doshade = 1;						/* Do it only once */
	}
	if(!bp->thflag) {							/* Dont do if thflag on */
		erasehisto();
		newspecimen();							/* Ensure clustering OFF ! (YUK) */
		if(scanspeed(bp->sspeed) != FS_OK)
			return(0);
		autofocus(ON);
		if (hrhisto(&lth1, &lth2, bp->shadsw)) {
			fiptim("Histogram reset.");
			return(0);
		}
		if (check_met_tholds(lth1, lth2)) {
			fprintf(output, "Unsatisfactory thresholds found.\n");
			showhisto(lth1, lth2);
			return(0);
		}
		showhisto(bp->thresh1, bp->thresh2);
	} else
		fprintf(output,"Manual thresholds set.  Histogram not done.\n");
		
												/* Confusion between max 
												   clust and max OD. */
	bp->clus2 = 2 * bp->thresh2;				/* maxOD = 2 * upper thold */
	return(1);
}

/*
 *	G E T M P H
 *
 */
getmph() 
{
int newx, newy, newslx, newsly, errnum;
struct	mult_slide *bp;

	if (killscan())
		return(0);		/* In case set whilst in bay_move */
	
	bp = alc->curbay;
	simslide();						/* Erase old crosses */
	dwb(2, &bp->mbox, 1, 1);		/* Draw box in new bay */
	newx = ((bp->mbox.bright - bp->mbox.bleft) / 2 ) + bp->mbox.bleft;
	newy = ((bp->mbox.btop - bp->mbox.bbot) / 2 ) + bp->mbox.bbot;
									/* new midpoint of box on screen */
	newslx = (newx - XMID) * CONV;
	newsly = (YMID - newy) * CONV;	/* equivalent new centre on alc->slide */
	if (stagepos(newslx, newsly, bp->focslc+bp->focbxc) == -1) 
		return(0);
	tomip1("dz",bp->focbxc);		/* set focus rel to slide surface */
	if(mp_mip_setup()) {			/* Set miproc params for a new scan */
									/* Recover proper focus */
		tomip1("dz",bp->focslc+bp->focbxc);
		return(0);
	}
	if(set_mip_class(alc->curbay, alc->cfipp->pversion)) {	/* set miproc classifier */
		tomip1("dz",bp->focslc+bp->focbxc);
		return(0);
	}
	if(doshade < 1) {
		if (shading()) {			/* Obtain shading correction vector */
			tomip1("dz",bp->focslc+bp->focbxc);
			return(0);	
		} else
			doshade = 1;			/* Do shade only once */
	}
	if(killscan()) {				/* In case set during this routine */
		tomip1("dz",bp->focslc+bp->focbxc);
		return(0);		
	}
									/* Set scan dependent miproc parameters 
									   and start scan. */
	if (gmph_mip_setup()) {
		tomip1("dz",bp->focslc+bp->focbxc);
		return(0);
	}
	screengo(ENDLINE,0);
	fprintf(output,"metaphase search ... ");
	fflush(output);
	/* If error while scan tidy up list and go on. */
	set_scan_timeout();
	if((errnum = waitfip(FOR_WSIG))) { 
		if(errnum != -1) {
			fprintf(output,"Error detected from combifip %s\n", 
					fiperr[errnum]); 
		} else {
			fprintf(output,"General miproc error. Please try again.\n");
		}
		mlistfinish();			/* Tidy up list, not done in fipops */
		fprintf(output," - %d clusters found\n", bp->totin);
	} else 
		fprintf(output,"Scan completed - %d clusters found\n", bp->totin);
	
	if (waitword("POSITION"))		/* Waits for position after scan */
		fprintf(output,"Not read POSITION\n");
	if (waitfip(FOR_QUIET))			/* Waits for miproc to be quiet */
		fprintf(output,"Miproc not quiet\n");
		
	miptimeout(0);
	tomip1("dz",bp->focslc+bp->focbxc);
	allcells(0);					/* Allcells to set inrank on */ 
	linksub();						/* Link list by subset */
	init_offset();					/* set list offsets to zero */
	init_scores(alc->curbay, 0);	/* initialise frax counts */
	init_counts(alc->curbay, 0);	/* initialise usbl+rev counts */
	return(1);
}
/*
 *	R U N S H A D E
 *
 */
runshade(dr, shad) 
int dr, shad; 
{
	fprintf (output,
	 "Recalculate shading corrections (swathe length %d density %d)\n",
		shad,dr);
	set_scan_timeout();
	if (density_range(dr)) goto Notshad;
	if (tomip1("us",shad)) goto Notshad;
	if (waitword("POSITION") ) goto Notshad; 
	if (waitfip(FOR_QUIET)) goto Notshad; 
	return(0);  
Notshad: fiptim( "shading reset");
	return(-1);
}

/*
 *	H R H I S T O
 *
 */
hrhisto(it1, it2, shd) 
int *it1, *it2, shd; 
{
char cc = '0';
int nread = 0;
int temp = 0;
	char	errorbuf[200], *errptr;

	errptr = errorbuf;
	sprintf(errptr,"hrhisto: "); errptr += 9;
	if(alc->debug == 3)					/* Print histo values for debug */
		histtab();
	screengo(ENDLINE,0);
	fprintf(output,"Do histogram scan and calculate thresholds\n");
	tsleep(2);
	set_scan_timeout();
	if (waitfip(FOR_QUIET))	return(-1); 
	if (tomip1("hi",shd)) return(-1); 
	if (waitfip(FOR_CTRLD)) {
		if(alc->debug)
			fprintf(debug_channel, "hrhisto:  Waifip not got ^D\n");
		errorlog("hrhisto - NO ^D\n");
		return(-1); 
	}
	tsleep(50);
	do {
		cc = newchar(); 
		*errptr++ = cc;
	} while ( !isdigit(cc));
	do {
		temp = 10*temp + cc - '0';
		cc = newchar();
		*errptr++ = cc;
	} while(isdigit(cc));
	*it1 = temp;
	do {
		cc = newchar(); 
		*errptr++ = cc;
	} while (!isdigit(cc));
	temp = 0;
	do {
		temp = 10*temp + cc - '0';
		cc = newchar();
		*errptr++ = cc;
	} while(isdigit(cc));
    *it2 = temp;	
    if(alc->debug == 3) {
    	fprintf(debug_channel, "Miproc tholds lower: %d  upper: %d\n\n",
    				*it1, *it2);
    }
	if (waitfip(FOR_QUIET)) {
		*errptr++ = '\n';
		*errptr = '\0';
		errorlog(errorbuf);
		return(-1); 
	}
	return(0);
}
/*
 *
 *  C H E C K _ M E T _ T H O L D S
 *
 *			Checks that sensible tholds are used.  Overrides with user
 *			selected ones if tgflag set.
 */
static
int check_met_tholds(lth1, lth2)
int  lth1, lth2;
{
struct	mult_slide *bp;
	
	bp = alc->curbay;
	screengo(ENDLINE,0);
	fprintf (output,
		 "Histogram scan completed - thresholds %d,%d\n", lth1, lth2);
	
	if(lth1 < MINTHR || lth2 > MAXTHR || lth1 > lth2) {
			return(-1);
	}
	bp->thresh1 = lth1;
	bp->thresh2 = lth2;
	return(0);
}	
/*
 *	S H A D I N G
 *
 */
shading() 
{

	if(scanspeed(alc->curbay->sspeed) != FS_OK)
		return(0);
  	runshade(alc->curbay->drange, alc->curbay->shadsw); 
}

/* pos. of default scan box (bot,top,left,right)*/


/*
 *
 * M P _ M I P _ S E T U P ---
 *
 *				
 *				This routine is called from findmet to set up all the
 *				miproc parameters necessary for metaphase find.
 *
 */
static
mp_mip_setup()
{
	int	stat = 0;
	char	temp;
	struct  mult_slide *bp;

	bp = alc->curbay;

	if ((stat = newspecimen()) != FS_OK)  		/* Set new specimen. */
		return(stat);
	if ((stat = autofocus(ON)) != FS_OK) return(stat);
	if ((stat = tomip1("sm", alc->cfipp->focus.permove)) != FS_OK)			
						return(stat);			/* Set focus slew rate. */
	if ((stat = tomip2("fe", alc->cfipp->focus.inswathe,alc->cfipp->focus.between)) != FS_OK)		
						return(stat);			/* Set focus limits. */
	if ((stat = tomip1("fe", alc->cfipp->focus.excursion)) != FS_OK) return(stat);
	if ((stat = tomip0("xs 2")) != FS_OK)					
						return(stat); 			/* Set stepsiz = 1 micron. */
	if ((stat = scanspeed(bp->sspeed)) != FS_OK)		
						return(stat); 			/* Set scan speed. */
	if ((stat = tomip1("sw", bp->ygate)) != FS_OK) 
						return(stat);			/* Set y gate. */
						
	return(FS_OK);
	
}
/*
 *
 * G M P H _ M I P _ S E T U P ---
 *
 *				
 *				This routine is called from getmph to set up all the
 *				miproc parameters for a particular metaphase find.
 *
 */
static 
gmph_mip_setup()
{
int	stat = 0;
struct  mult_slide *bp;

	bp = alc->curbay;
	if(alc->debug)
		fprintf(debug_channel,"in gmph_mip_setup\n");

	if ((stat = tomip2("xl", bp->xlow, bp->xhigh)) != FS_OK)
								return(stat);		/* Set x limits. */
	if ((stat = tomip2("yl", bp->ylow, bp->yhigh)) != FS_OK) 
								return(stat);		/* Set y limits. */
	if ((stat = density_range(bp->drange)) != FS_OK)
								return(stat);		/* Set density range. */
	if (alc->devel && (alc->config & CONF_FIPAL) && alc->cfipp->afeinterpolate)
		afe_interp(INTERP_ON, VIDEO);
	if ((stat = threshset(bp->thresh1, bp->thresh2)) != FS_OK) 
								return(stat);		/* Set tholds. */

	/*
	 * These are untrainable parameters affecting clustering
	 */
	if ((stat = tomip2("cl", bp->area, bp->iod)) != FS_OK)		
						return(stat);				/* Limb area and IOD. */
	if ((stat = tomip2("cd", bp->clus1, bp->clus2)) != FS_OK) 
								return(stat);		/* Cluster distance. */

	if (!bp->retrain) {
		if ((stat = tomip1("om", bp->mcells)) != FS_OK)  return(stat);	
	} else {
		if ((stat = tomip2("om", bp->mcells, bp->retrain)) != FS_OK) 
								return(stat);	
	}
	if ((stat = tomip1("ss", bp->mswathe)) != FS_OK) 
								return(stat);		/* Start scan. */
	miptimeout(abs(bp->xlow - bp->xhigh) * 16 /		/* Stretch timeout */
									alc->cfipp->speed);	
				
	if(alc->debug)
		fprintf(debug_channel,"out of gmph_mip_setup\n");
	return(FS_OK);
} 

/*
 * S E T B A Y C L ---
 *
 */
setbaycl(bp, ctp)
struct mult_slide *bp;
struct cyto_types *ctp;
{
register int nfeats, i, j;
int old_drange;
BOXHEAD *bhd;
LINHEAD *lhd, *alclhd;
struct linfdisc **lin;

	if(alc->debug)
		fprintf(debug_channel, "setbaycl: in\n");
		
	if((ctp == NULL) || (bp == NULL)) {
		if(alc->debug)
			fprintf(debug_channel, "setbaycl: Passed null pointer(s)\n");
		return(0);
	}

	if ((bp->typeids.ktistype != alc->rankcf->bhd->tis) ||
		(bp->typeids.kfstype != alc->rankcf->bhd->stn) ||
		(bp->drange != alc->rankcf->bhd->drange)) {
		if ((bp->typeids.ktistype == 0) && (bp->typeids.kfstype == 0)) {
			old_drange = bp->drange;
			set_def_box();
			set_def_lin();
			bp->drange = old_drange; /* Don't alter for default classifier */
		}
		else if ((findentry(alc->rankcf, BOX, &bp->typeids) == FAIL) ||
			(findentry(alc->rankcf, LIN, &bp->typeids) == FAIL)) {
			printf("Classifier not found, using default\n");
			set_def_box();
			set_def_lin();
			bp->typeids.ktistype = 0; /* Reset defaults here as well */
			bp->typeids.kfstype = 0;
		}
	}

	bhd = alc->rankcf->bhd;
	bp->drange = bhd->drange; /* dcb */

	for(i = 0; i < bhd->nfeats; i++) {
		switch(bhd->bfd[i]->feat) {
			case 0:
				bp->xmin = bhd->bfd[i]->lower;
				bp->xmax = bhd->bfd[i]->upper;
				break;
			case 2:
				bp->countmin = bhd->bfd[i]->lower;
				bp->countmax = bhd->bfd[i]->upper;
				break;
			case 3:
				bp->sfmax = bhd->bfd[i]->upper;
				break;
			case 4:
				bp->careamin = bhd->bfd[i]->lower;
				bp->careamax = bhd->bfd[i]->upper;
				break;
			case 5:
				bp->octareamin = bhd->bfd[i]->lower;
				bp->octareamax = bhd->bfd[i]->upper;
				break;
			case 6:
				bp->skodmax = bhd->bfd[i]->upper;
				break;
			case 7:
				bp->dvmax = bhd->bfd[i]->upper;
				break;
			case 8:
				bp->codmin = bhd->bfd[i]->lower;
				bp->codmax = bhd->bfd[i]->upper;
				break;
			default:
				break;
		}				
	}
	/*
	 *	Set up the linear classifier also
	 */
	alclhd = alc->rankcf->lhd;
	lhd = alloclin(alclhd->nfeats);

	lhd->tis = alclhd->tis;
	lhd->stn = alclhd->stn;
	lhd->drange = alclhd->drange;
	lhd->nfeats = alclhd->nfeats;
	lin = alclhd->lfd;
	for(i = 0; i < alclhd->nfeats; i++) {
		_strass(lhd->lfd[i], lin[i], sizeof(struct linfdisc));
	}
	bp->lhd = freelin(bp->lhd);
	bp->lhd = lhd;

	if(alc->debug)
		fprintf(debug_channel, "setbaycl: out\n");
}

/*
 * S E T _ M I P _ C L A S S
 *
 */
set_mip_class(bp, mipvers)
struct mult_slide *bp;
int mipvers;
{
int stat;

	/*
	 * These are the same in versions 7 and 9
	 */
	if ((stat = tomip2("cc", bp->countmin, bp->countmax)) != FS_OK)	
						return(stat);			/* Min max cluster counts. */
	if ((stat = tomip2("cw", bp->xmin, bp->xmax)) != FS_OK)		
						return(stat);			/* Octagon limits. */
	if ((stat = tomip2("ca", bp->careamin, bp->careamax)) != FS_OK)		
						return(stat);			/* Octagon area. */
	if ((stat = tomip2("ct", bp->codmin, bp->codmax)) != FS_OK)		
						return(stat);			/* Set mean OD. */
	if ((stat = tomip2("cb", bp->skodmax, 100)) != FS_OK)		
						return(stat);			/* Set the skirt/IOD ratio. */
						
	/* 
	 * Version variable commands; these version numbers should be hashdefined
	 */
	if(mipvers == 7) {

		if ((stat = tomip2("cs", bp->sfmax, 0)) != FS_OK)			
							return(stat);			/* Set shape factor. */
		if ((stat = tomip1("cv", bp->dvmax)) != FS_OK)				
							return(stat);			/* Set density variation. */

	} else if(mipvers == 9) {

		if ((stat = tomip2("cs", bp->sfmax, bp->dvmax)) != FS_OK)			
							return(stat);			/* Set shape factor. */
		if ((stat = tomip2("cv", bp->octareamin, bp->octareamax)) != FS_OK)				
							return(stat);			/* Set density variation. */
	}				
	return(FS_OK);
}
