/*
 *	F I P C O M M . C   --  Mizar/Miproc communications routines
 *
 *
 *	Written: Clive A Stubbings
 *			 Image Recognition Systems
 *			 720 Birchwood Boulevard
 *			 Birchwood Science Park
 *			 Warrington
 *			 Cheshire
 *
 *	Date:	 5th June 1986
 *
 *	Modifications
 *
 *	18 Jul 1991		CAS		Rename move_s to stagepos + do minor tidy up
 *	16 Jul 1991		CAS		move putfipprompt here - and don't wait for echo
 *	12 Jul 1991		CAS		error source ident in miprderr - make mipsuck ignore
 *							errors a couple of times before it gives up
 *	11 Jul 1991		CAS		Add STATIC/CSTATIC to help code maintenance
 *							Shuffle <CR>'s in tomip/tomip0
 *  21 May 1991		ih		Add a check for xon/xoff in writemip since this is turned off in the driver methinks
 *  13 Mar 1991		ih		Increase TIMEOUT to 40 from 20
 *	21 Feb 1991		CAS/IH	Extend readmip timeout + make tick independant
 *
 * The following fields in alc are required to be initialised
 *
 *		debug		- normal debug
 *		mipdebug	- miproc comms debug
 *		miprocterm	- miproc terminal line
 *		miprocbaud	- miproc baud rate
 *
 * Also the following external must exist
 *		debug_channel - filepointer for system debug (used if alc->debug)
 *		debugfp		- file pointer to comms debug channel
 *					  only used if mipdebug != 0
 */

#include <cmdsys.h>
#include <modes.h>
#include <sgstat.h>
#include <signal.h>
#include <module.h>
#include <procid.h>
#include <allcontr.h>
#include <cfipp.h>
#include <fipcomm.h>
#include <setsys.h>

#define	STATIC	static				/* Static because it is only used here */
#define	CSTATIC	static				/* Static because it causes name clashes.. */

#define ASKLEN	200
#define TIMEOUT		40				/* Miproc timeout count (default) */
#define LOAD_TMO	100				/* Timeout while waiting for load to complete */
#define MAX_MIP_PUSH_BACK 20		/* Max no of push back chars for readmip*/
#define XON 0x11
#define XOFF 0x13
static	int		miproc = 0;			/* Miproc file channel */
		int		miproc_stat;		/* Current miproc state */
static	int		tickpersec = 0;		/* System ticks per second (for timeout) */
static	int		timeout = TIMEOUT;	/* Set in 1/10ths of a second - NOTE: if system
										is idle tsleep(2) waits 1 tick, on busy
										systems it is load dependent so timeouts
										will get significantly longer (more than double) */
static  int		ahead_pointer = 0;		/* pointer to last push back character*/
static	char	aheadch[MAX_MIP_PUSH_BACK]; /* Used to store read_ahead chars*/
static	struct	sgbuf mipsave;		/* Old miprocterm line mode */
static	char	askbuffer[ASKLEN];
static	int		askbuflen;
		int		nodatumset = 1;		/* Datum not yet set ..*/
extern int errno;
extern FILE *debugfp;
extern FILE *debug_channel;
extern struct allkontr *alc;		/* The megga data structure */
extern int	xs,ys,zs;				/* Global position */

/*
 * HERE ARE THE MIPROC PROGRAM FILES ......
 */
#define MIPPRGM "/dd/miprogs/g1prog"
#define MIPDATA "/dd/miprogs/g1data"
#define MIPFLUSH "/nil"
#define PUTPGL	15
#define PUTPG	"ip $4000 $7fff\015"
#define PUTDTL	14
#define PUTDT	"id $a00 $1fff\015"
#define PUTFL	13
#define PUTF	"id $a00 $a10\015"

#define FPROMPTL	4
#define FPROMPT		"\015\012.\005"					/* Miproc prompt */
#define MPROMPT0L	9
#define MPROMPT0	"command?:"						/* Mipex prompt */
#define MPROMPTL	14
#define MPROMPT		"\012\000\000\000\000command?:"	/* Mipex prompt */
#define CPROMPT0L	6
#define CPROMPT0	"\012\000\000\000\000:"			/* Combifip prompt */
#define CPROMPT1L	24
#define CPROMPT1	"\012\000\000\000\000syntax error\015\012\000\000\000\000:"	/* Combifip prompt */

/*
 * Command sequences
 */
#define STARTMIPEX	"SP 101\015"		/* Start mipex from mipmon */
#define STARTMIPEXL	7
#define STARTCFIP	"ru\015"			/* Run combifip */
#define STARTCFIPL	3
#define CFIPINIT	"&e"				/* Init combifip */

/*
 *	Error messages to o/p
 */
char *fiperr[] = {
	"",
	"command syntax",
	"stage at end stop",
	"focus not found",
	"excessive focus change",
	"digitisation overflow",
	"unknown error" 
};

/*
 *	External functions
 */
extern char	*index();


/*
 * M I P T I M E O U T  ---  Set miproc timeout
 *
 *	If param is 0 - restore default
 */
miptimeout(count)
int		count;
{
	int		old;

	old = timeout - TIMEOUT;
	if (old < 0) old = 0;
	timeout = abs(count) + TIMEOUT;
	if (alc->mipdebug > 1) {
		fprintf (debugfp,"Timeout %d\n",timeout);
		fflush(debugfp);
	}
	return(old);
}

/*
 *  S T R I N D E X  --  Find position of one string in another
 *
 * returns:
 *		Pointer to s2 in s1 or NULL if not found.
 */
char *
strindex(s1,l1,s2,l2)
char	*s1, *s2;
int		l1, l2;
{
	char	*end1;

	end1 = s1 + l1 -l2;
	while (s1 <= end1) {
		while (*s1 != *s2)
			s1++;
		if (s1 > end1) return(NULL);
		if (strmatch(s1,s2,l2) == FS_OK) return (s1);
		s1++;
	}
	return(NULL);
}

/*
 *  T O M I P  --  Send string to miproc and wait for reply then
 *						the combifip prompt
 *						Put the reply in the askbuffer
 *
 * Returns FS_OK/FS_FAIL
 */
STATIC tomip(str)
char	*str;
{
	int		stat;
	char	*new;
	if (kickmip() != FS_OK) return(FS_FAIL);
	if (alc->mipdebug) {
		fprintf(debugfp,"To miproc tmo(%d): ",timeout);
		printstring(debugfp,str,strlen(str));
		fprintf(debugfp,"\n");
	}
	new = str;
	while (*new) {
		if (*new == '\015') {
			if ((stat = writemip(str,new-str+1)) != FS_OK) {
				miproc_stat = stat;
				return(FS_FAIL);
			}
			stat = frommip();
			str = ++new;
		} else
			new++;
	}
	if (new != str) {
		if (((stat = writemip(str,new-str)) != FS_OK) ||
			((stat = writemip("\015",1)) != FS_OK)) {
			miproc_stat = stat;
			return(FS_FAIL);
		}
		stat = frommip();
	}
	if (stat != FS_OK)
		stat = recover();
	return(stat);
}


/*
 *  T O M I P 0  --  Send string to miproc and wait for reply then
 *						the combifip prompt
 *						Put the reply in the askbuffer
 *
 * Returns FS_OK/FS_FAIL
 */
tomip0(str)
char	*str;
{
	return(tomip(str));
}

/*
 *  T O M I P 1  --  Send string to miproc and wait for reply then
 *						the combifip prompt
 *						Put the reply in the askbuffer
 *
 * Returns FS_OK/FS_FAIL
 */
tomip1(str,p1)
char	*str;
int		p1;
{
	char	buffer[128];
	
	sprintf(buffer,"%s %d\015",str,p1);
	return(tomip(buffer));
}

/*
 *  T O M I P 2  --  Send string to miproc and wait for reply then
 *						the combifip prompt
 *						Put the reply in the askbuffer
 *
 * Returns FS_OK/FS_FAIL
 */
tomip2(str,p1,p2)
char	*str;
int		p1,p2;
{
	char	buffer[128];
	
	sprintf(buffer,"%s %d %d\015",str,p1,p2);
	return(tomip(buffer));
}

/*
 *  A D M I P 1  --  Send string to miproc and wait for reply then
 *						the combifip prompt
 *						Put the reply in the askbuffer
 *
 * Returns FS_OK/FS_FAIL
 */
admip1(str,p1)
char	*str;
int		*p1;
{
	char	buffer[128];
	
	sprintf(buffer,"%s %d\015",str,*p1);
	return(tomip(buffer));
}


/*
 *  A D M I P 2  --  Send string to miproc and wait for reply then
 *						the combifip prompt
 *						Put the reply in the askbuffer
 *
 * Returns FS_OK/FS_FAIL
 */
admip2(str,p1,p2)
char	*str;
int		*p1,*p2;
{
	char	buffer[128];
	
	sprintf(buffer,"%s %d %d\015",str,*p1,*p2);
	return(tomip(buffer));
}

/*
 *	S T A G E P O S  --  Move stage (x, y + z)
 *
 * Returns FS_OK/FS_FAIL
 */
stagepos(x,y,z)
int		x,y,z;
{
	char	command[15], reply[30];
	int		oldtimeout, stat;

	stat = FS_OK;
	if (alc->mipdebug) fprintf(debugfp,"MOVE_S %d %d %d\n",x,y,z);
	if (alc->cfipp->autofocus != OFF) {
		if (autofocus(OFF) != FS_OK) return(FS_FAIL);
	} else
		if (kickmip() != FS_OK) return(FS_FAIL);
/* Send 'x' command + check echo immediately */
	sprintf(command,"x %d\015",x);
	writemip(command,strlen(command));
/* Send 'y' command - but it will not be echoed till later */
	sprintf(command,"y %d\015",y);
	write(miproc,command,strlen(command));
/* get prompt back from x command */
	readmip(reply,5,0);					/* read LF NULL NULL NULL NULL */
/* set 'x' move timeout here */
	oldtimeout = miptimeout(abs(x-xs)*10/alc->cfipp->speed);
	readmip(reply,15,0);				/* CR LF 4 (NULL) 'POSITION:' */
	readmip(reply,27,0);				/* read x, y, z */
	readmip(reply,6,0);					/* CR LF 4 (NULL) */
/* Get echo'd 'y' command mixed with ':' prompt */
	readmip(reply,strlen(command) + 1);
	str2cmp(reply,command,":");
/* Send out 'z' command */
	sprintf(command,"z %d\015",z);
	write(miproc,command,strlen(command));
/* get prompt back from 'y' command */
	readmip(reply,5,0);					/* read LF NULL NULL NULL NULL */
	miptimeout(abs(y-ys)*10/alc->cfipp->speed);	/* set 'y' move timeout */
	readmip(reply,15,0);				/* read CR LF 4 (NULL) 'POSITION:' */
	readmip(reply,27,0);				/* read x, y, z */
	readmip(reply,6,0);					/* CR LF 4 (NULL) */
/* Get echo'd 'z' command mixed with ':' prompt */
	readmip(reply,strlen(command) + 1);
	str2cmp(reply,command,":");
/* get prompt back from 'z' command */
	readmip(reply,5,0);					/* read LF NULL NULL NULL NULL */
/* set 'z' move timeout here */
	if (readmip(reply,15,0) != 15)			/* read CR LF 4 (NULL) 'POSITION:' */
		stat = FS_FAIL;
	else if (readmip(reply,27,0) != 27)		/* read x, y, z */
		stat = FS_FAIL;
	else {
		sscanf(reply,"%d %d %d",&xs,&ys,&zs);
		if (alc->devel)
			printf("New position %d %d %d\n",xs,ys,zs);
		if (readmip(reply,6,0) != 6)			/* CR LF 4 (NULL) */
			stat = FS_FAIL;
		else if (readmip(reply,1,0) != 1)		/* ':' */
			stat = FS_FAIL;
	}
	miptimeout(oldtimeout);					/* Restore old timeout */
	return(stat);
}


/*
 *  F R O M M I P  --  Wait for reply then
 *						the combifip prompt
 *						Put the reply in the askbuffer
 *
 * Returns FS_OK/FS_FAIL
 */
STATIC frommip()
{
	char	*bptr, *strend, *j;
	char	*errptr;
	int		nchars, time;
	int		errtype;

	if (tickpersec == 0) {
		setuptimeout();
	}
	bptr = askbuffer;
	askbuflen = 0;
	if (alc->mipdebug > 1) fprintf(debugfp,"Frommip: ");
	if (readmip(bptr,1,0) == 0) return(FS_FAIL);
	bptr++;
	askbuflen = 1;
	time = timeout * tickpersec;
	while (time-- > 0) {
		while ((nchars = readmip(bptr,(&askbuffer[ASKLEN]-bptr),1)) != 0) {
			bptr += nchars;
			askbuflen += nchars;
			if (askbuflen >= CPROMPT0L) {
				if ((strend = strindex(askbuffer,bptr-askbuffer,CPROMPT0,CPROMPT0L)) != NULL) {
					askbuflen = strend-askbuffer;
					for (j = (bptr-1); j >= (strend+CPROMPT0L); j--) {
						unreadmip(*j);
					}
					if (alc->mipdebug > 1) {
						fprintf(debugfp,"Found prompt - buffer :");
						printstring(debugfp,askbuffer,askbuflen);
						fprintf(debugfp,":\n");
					}
					if ((errptr = strindex(askbuffer,askbuflen,"error",5)) != NULL) {
						if ((errtype = cfiperr(askbuffer,askbuffer+askbuflen)) != CF_SYNT) {
							putprompt();
							fprintf(stderr,"Error detected from combifip %s\n",fiperr[errtype]);
							return(FS_FAIL);
						}
					}
					return(FS_OK);
				}
			}
		}
		tsleep(2);
	}
	if (alc->mipdebug > 1) {
		fprintf(debugfp,"No prompt in ask :");
		printstring(debugfp,askbuffer,askbuflen);
		fprintf(debugfp,":\n");
	} 
	return(FS_FAIL);
}

/*
 *	N E W S P E C I M E N  --  Send new specimen to cfip
 *
 *	Returns: FS_OK/FS_FAIL
 */
newspecimen()
{
	char	tbuf[2];
	int		stat;
	if (tomip0("ns") != FS_OK)
		return(FS_FAIL);
	/*
	 * Ensure stable state
	 * 1. Remove extra characters up to ':'
	 */
	do {
		if ((stat = readmip(tbuf,1,0)) < 0) {	/* Read until timeout */
			fprintf(stderr,"newspecimen: readmip error (%d)\n", stat);
			return(FS_FAIL);
		}
	} while ((stat == 1) && (*tbuf&127) != ':');

	/*
	 * Now check the miproc state to make sure we're OK
	 */
	if ((stat = fipstat()) != FS_CFIP) {
		fprintf(stderr, "newspecimen: miproc in incorrect state (%d)\n", stat);
		return(FS_FAIL);
	}
	alc->cfipp->speed = UNSET;
	alc->cfipp->tholds.thresh1 = 10;
	alc->cfipp->tholds.thresh2 = 60;
	if (tomip0("fv 1") != FS_OK)  
		return(FS_FAIL);
	return(FS_OK);
}

/*
 *	A U T O F O C U S  --	Change autofocus state (ON/OFF)
 *
 *	Returns: FS_OK/FS_FAIL
 */
autofocus(state)
int		state;
{
	if (state != alc->cfipp->autofocus) {
		if (tomip1("af",(state == 1 ? 1 : -1)) == FS_OK) {
			alc->cfipp->autofocus = state;
		} else {
			return(FS_FAIL);
			alc->cfipp->autofocus = UNSET;
		}
	}
	return(FS_OK);
}

/*
 *	H I G H R E S  --  Change miproc between high/low res modes
 *
 *	Returns:	FS_OK/FS_FAIL
 */
highres(state)
int		state;
{
	register int	highres;
	static	 int	savespeed = 0;

	highres = alc->cfipp->highres;
	if (highres != state) {
		if (tomip1("hr",state ? 1 : -1) == FS_OK) {
			alc->cfipp->highres = state;
			if (state == ON) {
				if ((highres == UNSET) || (highres == OFF)) {
					savespeed = alc->cfipp->speed;
					alc->cfipp->speed = 100;
				}
				alc->cfipp->autofocus = OFF;
			} else {
				if ((highres == ON) && (savespeed)) {
					alc->cfipp->speed = savespeed;
					savespeed = 0;
				}
				alc->cfipp->autofocus = ON;
			}
		} else {
			alc->cfipp->highres = UNSET;
			alc->cfipp->autofocus = UNSET;
			savespeed = 0;
			/* What has happened to the speed - ????? */
			return(FS_FAIL);
		}
	}
	return(FS_OK);
	
}

/*
 *	M O V E S P E E D  --
 *
 *	Returns:	FS_OK/FS_FAIL
 */
movespeed()
{
	return(speed(alc->cfipp->movespeed));
}

/*
 *	S C A N S P E E D  --
 *
 *	Returns:	FS_OK/FS_FAIL
 */
scanspeed(sm)
{
	if (sm < MINSCANSPEED || sm > MAXSCANSPEED)
		return(FS_FAIL);
	else 
		return(speed(sm));
}

/*
 *	D E F A U L T S C A N S P E E D  --
 *
 *	Returns:	Default scan speed
 */
defaultscanspeed()
{
	return(alc->cfipp->scanspeed);
}

/*
 *	S P E E D  --
 *
 *	Returns:	FS_OK/FS_FAIL
 */
speed(sp)
int		sp;
{
	if (alc->cfipp->speed != sp) {
		if (tomip1("v",sp) == FS_OK)
			alc->cfipp->speed = sp;
		else
			return(FS_FAIL);
	}
	return(FS_OK);
}
/*
 *	T H R E S H S E T --
 *
 *	Returns:	FS_OK/FS_FAIL
 */
threshset(th1, th2)
int		th1, th2;
{
	if ((alc->cfipp->tholds.thresh1 != th1) || (alc->cfipp->tholds.thresh2 != th2)) {
		if (tomip2("th", th1, th2) == FS_OK) {
			alc->cfipp->tholds.thresh1 = th1;
			alc->cfipp->tholds.thresh2 = th2;
		} else
			return(FS_FAIL);
	}
	return(FS_OK);
}
/*
 *	D E F A U L T M E T T H O L D S  --
 *
 *	Returns:	FS_OK/FS_FAIL
 */
defaultmettholds(th1, th2)
int	*th1, *th2;
{
	*th1 = alc->cfipp->tholds.dmthresh1;
	*th2 = alc->cfipp->tholds.dmthresh2;
	
}

/*
 *  M I P W O R D  --  search for a word in the frommip reply buffer
 *						Return NULL or a pointer to it
 *
 */
char *
mipword(word)
char	*word;
{
	return(strindex(askbuffer,askbuflen,word,strlen(word)));
}

/*
 * C F I P E R R   --   Look for error type word from combifip
 *
 */
cfiperr(start,end)
char 	*start, *end;
{
	if (strindex(start,end-start,"syntax",6)) return(CF_SYNT);
	if (strindex(start,end-start,"endstop",7)) return(CF_ENDST);
	if (strindex(start,end-start,"focus not",9)) return(CF_FNFND);
	if (strindex(start,end-start,"focus exc",9)) return(CF_FOCEX);
	if (strindex(start,end-start,"dig o/f",7)) return(CF_DIGOF);
	if (alc->mipdebug) {
		fprintf(debugfp,"Combifip error:");
		printstring(debugfp,start,end-start);
		fprintf(debugfp,"\n");
	} else if (alc->debug) {
		fprintf(debug_channel,"Combifip error:");
		printstring(debug_channel,start,end-start);
		fprintf(debug_channel,"\n");
	}
	if (alc->devel) {
		putprompt();
		fprintf(stderr,"Combifip error :\007");
		printstring(stderr,start,end-start);
		fprintf(stderr,"\n");
	}
	return(CF_UNKN);
}

/*
 *  R E C O V E R  --  Test fip stat and try to recover
 *
 */
STATIC recover()
{
	int		stat;
	
	if (alc->mipdebug) fprintf(debugfp,"Recover:\n");
	stat = fipstat();
	switch (stat) {
		case FS_MIPMON:
			printf("Combifip crashed with :");
			printstring(stdout,askbuffer,askbuflen);
			printf(": - trying to re-boot\n");
			if (bootmip() == FS_CFIP)
				return(FS_OK);
			else
				return (FS_FAIL);
		case FS_CFIP:
			return(FS_OK);
		default:
			return(FS_FAIL);
	}
}

/*
 *  K I C K M I P  --  Test miproc state and boot if neccessary
 *
 *	Returns FS_OK/FS_FAIL
 */
STATIC kickmip()
{
	int		tmotemp, stat;
	
	stat = FS_OK;
	if (nodatumset) {
		screengo(ENDLINE,1);
		printf("No datum set - are you sure ? ");
		if (!tryboth(1)) 
			return(FS_FAIL);
		nodatumset = 0;
	}
	tmotemp = timeout;			/* Save current timeout */
	timeout = TIMEOUT;			/* Make it short */
	miproc_stat = fipstat();
	if (miproc_stat != FS_CFIP) {
		stat = bootmip();
		if (stat == FS_OK)
			cfipinit();
	}
	timeout = tmotemp;			/* Restore at end */
	return(stat);
}

/*
 *  C F I P I N I T  --  Send initialising params to cfip
 * This would be better in fipops but since it needs to be called from
 * kickmip it will have to be here.
 * Send:  init string, ns, fv 1, v 4000.
 */
STATIC cfipinit()
{
	char	buf[4];
	int		stat;
	
	if (tomip0(CFIPINIT) != FS_OK) return(miproc_stat);
	if (newspecimen() != FS_OK) return(miproc_stat);
	if (speed(alc->cfipp->movespeed) == FS_FAIL) return(miproc_stat);
	return(FS_OK);
}

/*
 *	R E B O T M I P --  Re boot miproc
 *
 */
rebootmip()
{
	writemip("mj\015",3);
	return(bootmip());
}

/*
 *  B O O T M I P  --   Get miproc into a 'running combifip' state
 *
 * Return  FS_OK/FS_FAIL
 */
STATIC bootmip()
{
	int		stat;
	int		progress,			/* How far we have got with the boot */
			needload,			/* Need to load combifip */
			loadedonce;			/* Loaded cfip once already */
	char	namebuf[100];

	progress = 0;
	needload = 0;
	loadedonce = 0;
	miptimeout(0);
/*
 *	Indicate we no longer know miproc state
 */
	alc->cfipp->speed = UNSET;
	alc->cfipp->autofocus = UNSET;
	alc->cfipp->highres = UNSET;

	if (alc->mipdebug) fprintf(debugfp,"Bootmip\n");
	stat = fipstat();				/* Get current miproc state */
/*
 * While status is not running combifip and the error is not fatal
 * keep trying to improve the situation
 */
	while (stat != FS_CFIP) {
		switch (stat) {
		  case FS_MIPMON:
			progress = FS_MIPMON;
			printf("Starting Mipex\n");
			stat = mipexstart();
			break;
		  case FS_MIPEX:
			progress = FS_MIPEX;
			if (needload) {
				if (loadedonce) goto wayout;
				loadedonce = 1;
				/*temp code to make sure miproc input flushed ih*/
				sprintf(namebuf,"%s",MIPFLUSH);
				stat = pload(namebuf,PUTF,PUTFL);
				/* end temp*/
				sprintf(namebuf,"%s%s",MIPPRGM,alc->cfipp->program);
				if ((stat = pload(namebuf,PUTPG,PUTPGL)) != FS_OK)
					goto wayout;
				sprintf(namebuf,"%s%s",MIPDATA,alc->cfipp->program);
				if ((stat = pload(namebuf,PUTDT,PUTDTL)) != FS_OK)
					goto wayout;
				needload = 0;
			} else {
				printf("Starting Combifip\n");
				stat = cfipstart();
			}
			break;
		  case FS_UNEXC:
		  case FS_FAIL:
			if (progress == FS_MIPEX) {
				stat = fipstat();
				needload = 1;
				break;
			} else
				goto wayout;
		  case FS_NECHO:
		  case FS_NSMPX:
		  case FS_NSCFP:
		  case FS_QUIET:
		  	goto wayout;
		  default:
			stat = fipstat();
		}
	}
wayout:
	if (stat == FS_CFIP) {
		printf("Miproc booted, running combifip\n");
	} else {
		printf("Miproc Comms Failure - Error ");
		mipmessage(stat);
		printf("\n");
	}
	return ((stat == FS_CFIP) ? FS_OK : FS_FAIL);
}



/*
 *  T E S T L O A D E D  --  Test if combifip is loaded
 *
 */
STATIC cfiploaded()
{
	if (miproc_stat != FS_MIPEX)
		return(FS_FAIL);
	mipsuck();
	if (writemip("dp $4000 $4007\015",15) != FS_OK)
		return(FS_FAIL);
	if (getmip(miproc,"\012\000\000\000\000 ",6) != FS_OK)
		return(FS_FAIL);
	if (getmip(miproc,alc->cfipp->checkarray,strlen(alc->cfipp->checkarray)) != FS_OK)
		return(FS_FAIL);
	if (getmip(miproc,"\015\012\000\000\000\000\004",7) != FS_OK)
		return(FS_FAIL);
	if (getmip(miproc,MPROMPT0,MPROMPT0L) != FS_OK)
		return(FS_FAIL);
	return(FS_OK);
}

/*
 * P L O A D  --  Parallel load a file
 *
 */
STATIC pload(file,ldcmd,ldcmdlen)
char	*file, *ldcmd;
int		ldcmdlen;
{
	int		pid, stat, i, sin, nchar;
	char	buffer[60];
	mipsuck();
	sin = dup(0);
	close (0);
	if (open (file,S_IREAD) != -1) {
		sprintf(buffer,"%d",alc->piabase);
		pid = os9fork("piaw",strlen(buffer)+1,buffer,0,0,0,128);
		close (0);
		dup(sin);
		close(sin);
	} else {
		dup(sin);
		close(sin);
		return(FS_NFILE);
	}
	if (alc->mipdebug) {
		fprintf(debugfp,"forked piaw pid %d\n",pid);
		if (alc->mipdebug>1) {
			printf("forked piaw\n");
		}
	}
	sleep(3);
	if (writemip(ldcmd,ldcmdlen) == FS_OK) {
		if (alc->mipdebug>1) printf("sent load command to miproc\n");
		if (getmip(miproc,"\012\000\000\000\000",5) == FS_OK) {
			for (i=0; i<LOAD_TMO; i++) {
				if (!procactive(pid)) {
					if (alc->mipdebug > 1) fprintf(debugfp,"Process piaw ended\n");
					if (pid == wait(&stat)) {
						stat &= 65535;
						if (alc->mipdebug) fprintf(debugfp,"Exit status %d\n",stat);
						if (stat) return(FS_FAIL);
						return(getmip(miproc,MPROMPT0,MPROMPT0L));
					}
				}
				tsleep(20);
			}
			if (alc->mipdebug) fprintf(debugfp,"piaw timed out\n");
		}
	}
	kill(pid,SIGKILL);
	return(FS_FAIL);
}

/*
 * P R O C A C T I V E  ---  Look to see if a process is active..
 *							 Should be in a library
 *
 */
STATIC procactive(pid)
int		pid;
{
	struct	pdsc	pdesc;
	gprdsc(pid,&pdesc);
	if (pdesc._pmodul) return(1);
	return(0);
}


/*
 * M I P E X S T A R T  --  Start Mipex in Miproc
 *
 */
STATIC mipexstart()
{
	int		len, stat;
	char	buf[20];
	if (!miproc)
		miproc_stat = fipstat();
	if (miproc_stat != FS_MIPMON) return(miproc_stat);
	if (alc->mipdebug) fprintf(debugfp,"Mipexstart - Ready\n");
	mipsuck();
	if (writemip(STARTMIPEX,STARTMIPEXL) == FS_OK) {
		if ((stat = getmip(miproc,MPROMPT0,MPROMPT0L)) != FS_OK) {
			len = readmip(buf,20,0);
			if (alc->mipdebug) {
				fprintf(debugfp,"mipexstart failed: ");
				printstring(debugfp,buf,len);
				fprintf(debugfp,"\n");
			}
			miproc_stat = FS_NSMPX;
			return(FS_NSMPX);
		} else {
			miproc_stat = FS_MIPEX;
			return(FS_MIPEX);
		}
	} else
		return(FS_NSMPX);
}

/*
 * C F I P S T A R T  --  Start combifip code
 *
 * Check its loaded
 * Try to start it
 */
STATIC cfipstart()
{
	int		len, stat;
	char	buf[20];
	if (!miproc)
		miproc_stat = fipstat();
	if (miproc_stat != FS_MIPEX) return(miproc_stat);
	if (alc->mipdebug) fprintf(debugfp,"Combifipstart - Ready\n");
	if (cfiploaded() != FS_OK) {
		mipsuck();
		return(FS_FAIL);
	}
	mipsuck();
	if (alc->mipdebug > 1) fprintf(debugfp,"going to write\n");
	if (writemip(STARTCFIP,STARTCFIPL) == FS_OK) {
		if ((stat = getmip(miproc,CPROMPT0,CPROMPT0L)) == FS_OK) {
			miproc_stat = FS_CFIP;
			return(miproc_stat);
		} else {
			len = readmip(buf,20,0);
			if (alc->mipdebug) {
				fprintf(debugfp,"combifip start failed: ");
				printstring(debugfp,buf,len);
				fprintf(debugfp,"\n");
			}
			miproc_stat = stat;
			return(stat);
 		}
	} else
		return(FS_NSCFP);
}

 

/*
 * F I P S T A T  --  Find out what state the Miproc is in
 *
 */
fipstat()
{
	int		len;
	char	tmpbuf[256];
	char	pbuf[40];
	char	*ptr;

	miproc_stat = FS_FAIL;
	if (!alc->miproc) { /* Line not yet open by anyone */
		sprintf(tmpbuf,"xmode %s baud=%d",alc->miprocterm, alc->miprocbaud);
		if (alc->mipdebug) fprintf(debugfp,"fipstat: xmode is :%s:\n",tmpbuf);
		system(tmpbuf);
		if ((miproc = open(alc->miprocterm,S_IREAD|S_IWRITE|S_ISHARE)) < 0) {
			if (alc->mipdebug)
				fprintf(debugfp,"Miproc open failed..  %d\n",errno);
			return(FS_NOPEN);
		}
		raw (miproc,&mipsave);
		alc->miproc = syspath(getpid(), miproc);	/* Save miproc system path no */
	} else {
		if (!miproc) {	/* This process hasn't opened miproc channel */
			miproc = sysdup(alc->miproc);		/* Duplicate from system path */
			if (alc->mipdebug)
				fprintf(debugfp,"fipstat: sysdup %d to local path %d\n",
					alc->miproc, miproc);
		}
	}
/*
 * Write a <CR> to miproc and see what comes back
 */
	mipsuck();
	if (writemip("\015",1) != FS_OK)
		miproc_stat = FS_NECHO;
	else {
		ptr = tmpbuf;
		while ((len = readmip(ptr,tmpbuf+sizeof(tmpbuf)-ptr,2)) > 0) {
			ptr += len;
			if (alc->mipdebug > 1) {
				fprintf(debugfp,"Fipstat - read %d :",len);
				printstring(debugfp,tmpbuf,ptr-tmpbuf);
				fprintf(debugfp,":\n");
			}
			if (ptr >= tmpbuf+sizeof(tmpbuf) - 1) {
				if (alc->mipdebug) fprintf(debugfp, "Fipstat - buffer overflow\n");
				break;
			}
			if ((ptr-tmpbuf >= CPROMPT0L) && 
					(strmatch(CPROMPT0,tmpbuf,ptr-tmpbuf) == FS_OK)) {
				miproc_stat = FS_CFIP;
				break;
			} else if ((ptr-tmpbuf >= CPROMPT1L) && 
					(strmatch(CPROMPT1,tmpbuf,ptr-tmpbuf)) == FS_OK) {
				miproc_stat = FS_CFIP;
				break;
			} else if ((ptr-tmpbuf >= MPROMPTL) && 
					(strmatch(MPROMPT,tmpbuf,ptr-tmpbuf) == FS_OK)) {
				miproc_stat = FS_MIPEX;
				break;
			} else if ((ptr-tmpbuf >= FPROMPTL) &&
					(strmatch(FPROMPT,tmpbuf,ptr-tmpbuf) == FS_OK)) {
				miproc_stat = FS_MIPMON;
				break;
			}
		}
		if (ptr == tmpbuf)
			miproc_stat = FS_QUIET;	
	}
	return(miproc_stat);
}


/*
 * G E T M I P   --   Get a string from the miproc and match with one
 *						given
 *
 * Returns:
 *	FS_FAIL/FS_OK
 */
STATIC getmip(chan,string,len)
int		chan;
char	*string;
int		len;
{
	char	ch;
	if (alc->mipdebug > 1) fprintf(debugfp,"getmip: ");
	while (readmip(&ch,1,0) > 0) {
		if (alc->mipdebug  > 1) printstring(debugfp,&ch,1);
		if (ch != *string) {
			unreadmip(ch);
			if (alc->mipdebug) {
				fprintf(debugfp,"[FAIL]\ngetmip: Unexpected char from miproc :");
				printstring(debugfp,&ch,1);
				fprintf(debugfp,":\n");
			}
			return(FS_UNEXC);
		}
		string++;
		if (!--len) {
			if (alc->mipdebug > 1) fprintf(debugfp,"[OK]\n");
			return(FS_OK);
		}
	}
	if (alc->mipdebug > 1) fprintf(debugfp,"[FAIL]\n");
	return(FS_FAIL);
}

/*
 *  M I P C H A R S  --  Seehow many chars are ready on miproc line
 *						This is a temporary kludge till I fix it properly
 *						Blame it on life...
 *
 */
mipchars()
{
	return(ahead_pointer + getstat(1,miproc));
}


/*
 * W R I T E M I P  --   Write string to miproc and read and verify echo
 *
 * Returns FS_FAIL/FS_OK
 */
writemip(string,len)
char	*string;
int		len;
{
	char	ch;

	if (alc->mipdebug > 2) {
		fprintf(debugfp,"Writemip: ");
		fflush(debugfp);
	}
	while (len--) {
		write(miproc,string,1);
		if (alc->mipdebug > 2) {
			printstring(debugfp,string,1);
			fflush(debugfp);
		}
		if (readmip(&ch,1,0) == 0) {
			if (alc->mipdebug) fprintf(debugfp,": read timed out\n");
			return(FS_FAIL);
		}
		/*if (ch != *string) {
			if (ch==XOFF) {
				while(readmip(&ch,1,0)==0);
				if (readmip(&ch,1,0) == 0) {
					if (alc->mipdebug) fprintf(debugfp,": read timed out\n");
					return(FS_FAIL);
				}
			}
			else if (ch==XON) {
				if (readmip(&ch,1,0) == 0) {
					if (alc->mipdebug) fprintf(debugfp,": read timed out\n");
					return(FS_FAIL);
				}
			}
		}*/
		if (ch != *string++) {
			if (alc->mipdebug) {
				fprintf(debugfp,": Unexpected char \"");
				printstring(debugfp,&ch,1);
				fprintf(debugfp,"\"\n");
			}
			if (alc->debug) printf("Unexpected char from miproc\n");
			return(FS_FAIL);
		}
	}
	if (alc->mipdebug > 2) fprintf(debugfp,"\n");
	return(FS_OK);
}

/*
 *	P U T F I P P R O M P T  --
 *
 */
putfipprompt() 
{ 
	if (alc->mipdebug) {
		fprintf(debugfp, "putfipprompt()\n");
		fflush(debugfp);
	}
	write(miproc,"\015",1);
/*	writemip("\015",1);  */
}

/*
 * R E A D M I P  --  Read (or timeout reading) a string from miproc
 *
 *	Control value - once
 *
 *	0  -  Read until timeout
 *	1  -  Read once - if no chars - return immediately
 *	2  -  Read until 1 or more chars then return - timeout if none
 */
readmip(string,len,once)
char	*string;
int		len, once;
{
	int		time, nchar, tchar;
	tchar = 0;
	
	if (tickpersec == 0) {
		setuptimeout();
	}
	if (alc->mipdebug > 2)
		fprintf(debugfp,"readmip - want %d chars\n",len);
	while (ahead_pointer != 0) {
		*string++ = aheadch [--ahead_pointer];
		tchar++;
		if (!--len)
			return(tchar);
	}
	time = timeout * tickpersec;
	while (time-- > 0) {
		if ((nchar = getstat(1, miproc)) > 0) {
			if (alc->mipdebug > 2)
				fprintf(debugfp,"read - %d chars avail\n",nchar);
			if (nchar>len) nchar = len;
			if ((nchar = read(miproc,string,nchar)) < 0) {
				if (alc->mipdebug > 2)
					fprintf(debugfp,"readmip: read error %d\n", errno);
				miprderr("readmip");
				return(0);	/* No chars., is this the correct value to rtn ? */
			}
			len -= nchar;
			tchar += nchar;
			while (nchar-- > 0) {
				*string &= 127;
				if (alc->mipdebug > 2) {
					fprintf(debugfp,"(");
					printstring(debugfp,string,1);
					fprintf(debugfp,")");
					fflush(debugfp);
				}
				string++;
			}
			if (len <= 0)
				return(tchar);
		} else if (once==1) 
			return(tchar);
		else if ((once == 2) && tchar)
			return(tchar);
		else
			tsleep(2);
	}
	return(tchar);
}

/*
 * U N R E A D M I P  --  Take back a char
 *
 */
STATIC unreadmip(ch)
char	ch;
{
	int i;
	
	if (ahead_pointer >= MAX_MIP_PUSH_BACK) {
		fprintf(stderr,"Miproc error, data overrun\n");
		if (alc->mipdebug) {
			fprintf(debugfp,"error:-miproc comms push back stack full\n");
			fprintf(debugfp,"stack holds: ");
			printstring(debugfp,aheadch,ahead_pointer);
			fprintf(debugfp,"\n");
			fflush(debugfp);
		}
	} else {
		aheadch [ahead_pointer++] = ch;
		if (alc->mipdebug > 2) {
			fprintf(debugfp,"unread: ");
			fprintf(debugfp,"(");
			printstring(debugfp,&ch,1);
			fprintf(debugfp,")");
			fflush(debugfp);
		}
	}
}


/*
 * M I P S U C K   --   Read and dump any pending input
 *
 */
mipsuck()
{
	int		nchars, errcnt;
	char	buf[120];

	ahead_pointer = errcnt = 0;
	if (alc->mipdebug > 1) fprintf(debugfp,"Suck o/p:");
	while ((nchars = _gs_rdy(miproc)) > 0) {
		if ((nchars = read(miproc, buf, ((nchars<120) ? nchars : 119))) < 0) {
			miprderr("mipsuck");
			if (++errcnt >= 3)
				return (-1);
		}
		if (alc->mipdebug > 1 && (nchars>0)) {
			fprintf(debugfp,"[%d]",nchars);
			printstring(debugfp,buf,nchars);
		}
		tsleep(2);
	}
	if (alc->mipdebug > 1) fprintf(debugfp,"\n");
}

/*
 * P R I N T S T R I N G  --  Print a string with control chars in a readable
 *								form
 */
STATIC printstring(fchan,buf,len)
FILE	*fchan;
char	*buf;
int		len;
{
	int		i;
	char	ch;
	for (i=0; i<len; i++) {
		ch = *buf++;
		ch &= 127;
		if (ch < 32) {
			putc('^',fchan);
			putc(ch+'@',fchan);
		} else if (ch == 127) {
			putc('^',fchan);
			putc('?',fchan);
		} else
			putc(ch,fchan);
	}
}

/*
 * S T R M A T C H  --  Match two strings - char for char (including nulls)
 *
 */
CSTATIC strmatch(str1,str2,len)
char	*str1, *str2;
int		len;
{
	if (!len)
		return(FS_OK);
	while (len--) {
		if (*str1++ != *str2++)
			return(FS_FAIL);
	}
	return(FS_OK);
}

/*
 * M I P M E S S A G E  --   Print error message
 *
 */
STATIC mipmessage(code)
int		code;
{
	switch (code) {
	case FS_OK:
		printf("OK");
		break;
	case FS_CFIP:
		printf("Running Combifip");
		break;
	case FS_MIPEX:
		printf("Running Mipex");
		break;
	case FS_MIPMON:
		printf("Running Mipmon");
		break;
	case FS_FAIL:
		printf("fail");
		break;
	case FS_NOPEN:
		printf("can't open line %s",alc->miprocterm);
		break;
	case FS_NECHO:
		printf("Command not echoed");
		break;
	case FS_UNKN:
		printf("Unknown prompt");
		break;
	case FS_QUIET:
		printf("No response");
		break;
	case FS_UNEXC:
		printf("Unexpected character");
		break;
	case FS_NLOAD:
		printf("Load failed");
		break;
	case FS_NSMPX:
		printf("Can't start mipex");
		break;
	case FS_NSCFP:
		printf("Can't start Combifip");
		break;
	case FS_NFILE:
		printf("Can't open file");
		break;
	default:
		printf("Unspecified code (%d)",code);
	}
}

/*
 * M I P R D E R R -- Print an error message to system error file
 */
STATIC miprderr(str)
char	*str;
{
	char buf[80];

	sprintf(buf, "Can't read miproc (%s), error %d\n",str,errno);
	fprintf(stderr, "%s", buf);
	errorlog(buf);
	return (-1);
} /* end miprderr */

/*
 *	S E T U P T I M E O U T  --  Setup timout correction factor to provide
 *								 a fairly predictable timeout over different
 *								 systems with different clock rates
 */
STATIC setuptimeout()
{
	if ((tickpersec = _getsys(D_TckSec,2)) < 0)
		tickpersec = 100;
	if ((tickpersec /= 10) < 1)
		tickpersec = 1;
}
