/*
 *	F I P O P S . C  --  Kyy fip operations
 *
 *	Provides Control of Miproc, and Preprocessor operations via Miproc
 *
 *	adapted from interrupt routine,
 *	authors: Margaret Stark/Jim Piper, May-June 1985
 *
 * Modifications
 *
 *	18 Jul 1991		CAS			Ditch old stagepos - use new move_s (fipcomm)
 *								Don't die if mlistfill openpia fails
 *								Low level bay stuff to here
 *	17 Jul 1991		CAS			Shuffling in waitfip to tighten putfipprompt/openpia
 *								and move putfipprompt to fipcomm (to be lower level)
 *  13 Mar 1991		ih			Make scan timeout value tick proportional
  */

#include <cmdsys.h>
#include <kyy.h>
#include <slides.h>
#include <cfipp.h>
#include <fipcomm.h>
#include <setsys.h>
#include <sgstat.h>

#define CTRLQ 17
#define Z_SPEED		1400			/* Z motor speed */
/* 
 * set s.t. wtim = 512 when scanspeed = 4000; as it was
 */
/*#define TDIS4000 0x7fffffff*//*for debugging to effectively remove fip timeouts*/
#define TDIS4000 2048000

static hardboot,noerrs,hbn;
static int wtim;
static char word_received[128], *wp;
static char bfdebug[81],*bfd;
static char word_wanted[20];
static int	Xcmd, Ycmd, Zcmd;

static errmsgs();

/* Forward declarations */
char newchar();

/* Imported stuff */
extern int	miproc_stat;			/* Current miproc state */
extern int	nodatumset;				/* No datum yet set for stage */
extern FILE *debug_channel, *debugfp;

extern int	rebootmip(), admip1();
extern char *mipword();

#ifdef GGG
/*
 *	S T G M O V  --  Move stage (x,y or z)
 *
 */
stgmov(m)
int		m;
{
	if (alc->debug) printf("stagemove \n");
	switch(m) {
		case 1:  if (tomip1("x",xs) == FS_FAIL) return(FS_FAIL);
				 break;
		case 2:  if (tomip1("y",ys) == FS_FAIL) return(FS_FAIL);
				 break;
		case 3:  if (tomip1("z",zs) == FS_FAIL) return(FS_FAIL);
	}
	getpos(2);
	return(FS_OK);
}
#endif GGG

#ifdef DEFUNCT_CAS
/*
 * S T A G E P O S  --  Set stage poisition
 *
 * Returns:
 *		FS_OK/FS_FAIL
 */
stagepos(x,y,z)
int	x,y,z;
{
	int		stat;
	if ((stat=getpos(2)) != FS_OK)
		return(stat);
	if (alc->mipdebug) fprintf(debugfp,"Stagepos x %d y %d z %d\n",x,y,z);
	if (autofocus(OFF) == FS_FAIL) stat = FS_FAIL;
	if (x != xs) {
		miptimeout(abs(x-xs)*10/alc->cfipp->speed);
		if (tomip1("x",x) == FS_FAIL) stat = FS_FAIL;
	}
	if ((stat != FS_FAIL) && (y != ys)) {
		miptimeout(abs(y-ys)*10/alc->cfipp->speed);
		if (tomip1("y",y) == FS_FAIL) stat = FS_FAIL;
	}
	if ((stat != FS_FAIL) && (z != zs)) {
		miptimeout(abs(z-zs)*10/Z_SPEED);
		if (tomip1("z",z) == FS_FAIL) stat = FS_FAIL;
	}
	/* Restore default timeout. */
	miptimeout(0);	
	if (stat != FS_OK) 
		printf("Stage move failed\n");
	else
		stat = getpos(2);
	return(stat);
}
#endif

/*
 *	R E C A L L  --  synonym for stagepos
 */
recall(x,y,z)      /* issues recall command to fip */
{
	stagepos(x,y,z);
}


/*
 *	C H E C K M C M P  --  Check lens and slide state before scanning
 *							or digitisation.
 *
 *	Returns 1 if OK - 0 otherwise
 */
checkmcmp(whatnow)
{
	int	stat;
	if ( lastact != whatnow ) {
		vclear(ENDLINE);
		bigmessage( 6,1,"Check:");
		switch (whatnow) {
			case 2 :	bigmessage( 8,10,"X 100");
						bigmessage( 10,10,"Slide oiled");
						break;
			case 1 :	bigmessage( 8,10,"X 20");
						bigmessage( 10,10,"No oil !");
		}
		bigmessage( 12,10,"Condensor set");
		stat = go_on();
		clearbig(6);
		clearbig(8);
		clearbig(10);
		clearbig(12);
		screengo(ENDLINE,0);	
	}
	setdisplay(BUST);
	if (stat)
		lastact = whatnow;
	return(stat);
}

/*
 *  G E T P O S  --  get current stage position
 *
 * mode - 0 don't prompt
 *		  1 prompt with ?\015 and redisplay
 *		  2 prompt with ?\015 but Don't display
 *
 * There is a suspicion of something more here re digitise etc
 * Returns:
 *		FS_OK/FS_FAIL
 */
getpos(mode)
int		mode;
{
char	*bptr;

	if (!mode || (tomip0("?") == FS_OK)) {
		if ((bptr=mipword("POSITION:")) != NULL) {
			bptr += 9;
			sscanf(bptr,"%d %d %d",&xs,&ys,&zs);
			if (alc->mipdebug) 
				fprintf(debugfp,"Getpos x %d y %d z %d\n",xs,ys,zs);
			
			/* rewrite positions on screen unless a remote call 
			( mode is -1 or 2 when getpos called from boxcorns.c and 
			capture.c  ie mouse movement ) */
			if ( mode != 2 && mode != -1 ) { 
				if (Xcmd) {
					displin(0,Xcmd);displin(0,Ycmd);displin(0,Zcmd);
					putprompt();
				}
			}
			return(FS_OK);
		}
	}
	return(FS_FAIL);
}



/*
 * S E T P O S  --  Set origin
 *
 */
static
setpos(option)
int	*option;
{
int	olx, oly, olz;
struct	mult_slide *bp;

										/* Return on stupid values */
	if(*option < 0 || *option > alc->maxbays)
		return(FS_FAIL);
	bp = alc->curbay;
	nodatumset = 0;
	if(getpos(2) == FS_FAIL)				/* Get pos but dont update */
		return(FS_FAIL);
	olx = xs;
	oly = ys;
	olz = zs;
	if (tomip0("d") != FS_OK) return(FS_FAIL);
	if(getpos(1) == FS_FAIL)				/* Get pos and update */
		return(FS_FAIL);
	else if(alc->curbay > alc->frsbay && *option == 0)
		updatoffs(bp, olz - zs, olx - xs, oly - ys);
	return(FS_OK);
}

/*
 *
 *	U P D A T Z O F F
 *
 *				Update focus offsets.
 *
 */
static updatoffs(bp, fdiff, xdiff, ydiff)
struct	mult_slide *bp;
int	fdiff, xdiff, ydiff;
{
	if (bp > alc->frsbay) {				/* Update previous bay offsets */
		bp->zoff -= fdiff;				/* Update this bay offsets */
		bp->xoff -= xdiff;					
		bp->yoff -= ydiff;					
		bp--;
		bp->zoff += fdiff;				/* And previous bay offsets */
		bp->xoff += xdiff;					
		bp->yoff += ydiff;					
	}
}

/*
 *	C H A N G E O B J  --  Move to a position from which the objective
 *							can be changed....
 *	alc->stage_type = 0 - Standard unmodified stage
 *	alc->stage_type = 1 - Modified (cut-away) stage
 *	alc->stage_type = 2, 3 etc. - Other stage types as yet undefined.
 */
changeobj()
{
	int	ret_x, ret_y, ret_z;
	if (getpos(1) == FS_FAIL) return(FS_FAIL);
	ret_x = xs;
	ret_y = ys;
	ret_z = zs;
	if (autofocus(OFF) != FS_OK) return(FS_FAIL);
	switch (alc->stage_type) {
		default:
			fprintf(stderr,
				"Stage type is set to illegal value. Using default\n");
			alc->stage_type = 0; /* Assuming 0 is the default of course !! */
		case 0:	/* Unmodified stage */
			if (stagepos(0,0,zs) != FS_OK) return(FS_FAIL);
			if (tomip0("dz 20000") != FS_OK) return(FS_FAIL);
			if (stagepos(0,0,0) != FS_OK) return(FS_FAIL);	/* Max mip z move = 32k */
			if (stagepos(0,0,-20000) != FS_OK) return(FS_FAIL);
			ch_obj_prompt();
			if (stagepos(0,0,0) != FS_OK) return(FS_FAIL);	/* Max mip z move = 32k */
			if (stagepos(0,0,20000) != FS_OK) return(FS_FAIL);
			if (tomip1("dz",ret_z) != FS_OK) return(FS_FAIL);
			break;
		case 1:	/* Modified (cut-away) stage */
			if (stagepos(0,0,0) != FS_OK) return(FS_FAIL);
			ch_obj_prompt();
			break;
		} /* end switch */
	/*
	 * Now return stage to original position
	 */
	if (stagepos(ret_x,ret_y,ret_z) != FS_OK) return(FS_FAIL);
	getpos(1);
	return(FS_OK);
}

/*
 * 	C H _ O B J _ P R O M P T
 *	"Change objective (M)" prompt
 *	Correct responses are: Mouse mid, return, y, or Y
 */
static
ch_obj_prompt()
{
	screengo(ENDLINE,1);

	fprintf(stdout,"Change objective (M) ");
	fflush(stdout);
	while(gone_on() <= 0); /* gone_on returns -1 if no input */
}

/*
 * F I P O P S  --  Fip options menu
 *
 */
fipops()
{
	int x, y, z, option=0, Dtcmd;
	static int running = 0;
	int	Gcmd, Cbcmd, Qcmd;
	int	bayno;

	x = xs; y = ys; z = zs;
	kbegin = 35;

	if(running)
		getpos(2);
		
	if ((bayno = baynn(alc->curbay)) == -1) {
		fprintf(output,"fipops: bay structures not set up!\n");
		return(0);
	}
	newmenu("Microscope Control");
		cmmd ( "x- stage move;%-d%+d",&xs,&x); Xcmd = mmcmd;
		cmmd ( "y- stage move;%-d%+d",&ys,&y); Ycmd = mmcmd;
		cmmd ( "z- stage move;%-d%+d",&zs,&z); Zcmd = mmcmd;
		cmmd ( "position ;%g%+i",getpos,2); Gcmd = mmcmd; mouse(RIGHTBT);
		cmmd ( "set origin (N38) ;%?+d%f", &option, setpos);
			mouse(MIDBT);
			Dtcmd = mmcmd;
		if (alc->devel || alc->debug || alc->mipdebug)
			cmmd ("speed;%x<100/16000>%d steps/second%f","v",&alc->cfipp->speed,admip1);
		cmmd ( "change objective ;%g",changeobj); mouse(LONGRB);
		cmmd ( "bay no; %d", &bayno);Cbcmd = mmcmd;
		cmmd ( "exit ");Qcmd = mmcmd; mouse(LONGLB);
		if (alc->devel) {
			cmmd ( "reboot Miproc;%g",rebootmip);
		}
	while (thismu()) {
		if (nncmd == Xcmd) {
			if(abs(x) > 32000)
				fprintf(output,"Requested move outside bounds of bay.\n");
			else stagepos(x,ys,zs);
			x = xs;
			displin(0,Xcmd);
			putprompt();
		}
		if (nncmd == Ycmd) {
			if(abs(y) > 32000)
				fprintf(output,"Requested move outside bounds of bay.\n");
			else stagepos(xs,y,zs);
			y = ys;
			displin(0,Ycmd);
			putprompt();
		}
		if (nncmd == Zcmd) {
			if(abs(z) > 32000)
				fprintf(output,"Requested move outside bounds of bay.\n");
			else stagepos(xs,ys,z);
			z = zs;
			displin(0,Zcmd);
			putprompt();
		}
		if (nncmd == Gcmd) {
			x = xs; y = ys; z = zs;
			displin(0,Xcmd);
			displin(0,Ycmd);
			displin(0,Zcmd);
			putprompt();
		}
		if (nncmd == Cbcmd) {
			bay_move(bayno);
			bayno = baynn(alc->curbay);
			displin(0,Cbcmd);
			putprompt();
		}
		if (nncmd == Dtcmd) {
			if(option > 0 && option <= alc->maxbays) {
				alc->curbay = alc->frsbay+option-1;
				bayno = baynn(alc->curbay);
				displin(0, Cbcmd);
				putprompt();
			}
		}
		if (nncmd == Qcmd) {
			break;
		}
		running = 1;
		option=0;
	}
	Xcmd = 0; Ycmd = 0; Zcmd = 0; Gcmd = 0;
}
/*
 *  W A I T W O R D
 *
 */
waitword(w) 
char *w; 
{
	strcpy(word_wanted,w);
	waitfip(FOR_WORD);
}

/*
 *  W A I T F I P  --  Wait for some state from the fip
 *
 * 'prompt' interpretation:
 * -1	FOR_CTRLD  detect CTRL+D after CTRL+Q and return 0
 * -2	FOR_WSIG   expect CTRL+Q BW : then change to FOR_CTRLD
 * -5	FOR_WORD   look for 'word_wanted'
 * -6	FOR_QUIET  wait for Miproc to go quiet
 * -7	FOR_DSIG   look for digitise signal (BD after CTRL+Q)
 *
 */
waitfip(prompt)
{
int nchars, wtime, errtype; 
register char c; 
int ret_stat, wasC=0, opstat;

	ret_stat = FS_OK;				/* Assume all is OK */
	errtype = 0;
	if (alc->mipdebug) {
		fprintf(debugfp,"Entering waitfip prompt %d (%c)\n",prompt,
				prompt > 0 ?  prompt : '-' );	
		if (prompt == FOR_WORD) 
			fprintf(debugfp,"[ waiting for %s ]\n", word_wanted);
	}
	bfd = bfdebug;
	wp = word_received; 
	if (prompt == FOR_QUIET) {
		if (fipstat() == FS_CFIP)
			return(ret_stat);
		else
			return(FS_FAIL);
	}
	if (prompt != FOR_WORD ) *word_wanted = 0;

	/* outer loop reads successive characters from Miproc */
	while(1) {
		wtime = 0;
				
#ifdef NOFIP
		nchars = -1;
		if (prompt== FOR_QUIET) {
			if (alc->mipdebug) fprintf(debugfp,"return FOR_QUIET\n");
			return(ret_stat);
		}
		while ( nchars < 1) { 
		nchars = 1;
#endif
#ifndef NOFIP
/* inner loop counts up wtime each time nothing is in buffer */
		if (prompt== FOR_QUIET) {
			if (alc->mipdebug) fprintf(debugfp,"return FOR_QUIET\n");
			mipsuck();
			return(ret_stat);
		}
		while((nchars = mipchars()) <= 0) {
#endif
			if (++wtime > wtim) { /* time expired */
				return (FS_FAIL);
			}
			tsleep(4);
			if (alc->mipdebug) {
				if ((wtime&15) == 15) 
					fprintf(debugfp,"Counting down Miproc response ... %d \n",
						(wtim - wtime) / 64);
			} 
		}
		if (alc->mipdebug)
			fprintf(debugfp,"Waitfip nchars %d\n",nchars);
		while (nchars-- > 0) {
			if((c = newchar()) != CTRLQ)
				wasC = 0;
			switch (c) {
				case CTRLQ:	/* ctrl-Q */
					c = newchar();
					--nchars;
					if (c == '4') {
						if (prompt == FOR_CTRLD)
							return(ret_stat);
						else if(wasC) {
							wasC = 0;
							if (killscan()) {
								writemip("K",1);
								putfipprompt();
								mlistfinish();
								return(ret_stat);
							}
						}
					} else if (errmsgs(prompt, c, &ret_stat)) {
						return(ret_stat);
					}
					break;
				case 'B':
					c = newchar(); 
					--nchars;
					if (alc->mipdebug) fprintf(debugfp,"Waitfip - B%c\n",c);
					switch (c) {
					case 'I': 
							putfipprompt();
							mlistinit(); 
							break;
					case 'C': 
							setpr(alc->mf_pid,getpr(alc->mf_pid)+256);
							tsleep(2);	/* Delay here causes no problems and
											ensures a `clean slice' for putfipprompt()
											and openpia() which must not be split */
							putfipprompt();
							if ((opstat= openpia(0,8,alc->piabase)) < 0) {
								char errstr[80];
								sprintf(errstr, "openpia fails %d\n", opstat);
								if (alc->devel || alc->debug || alc->mipdebug)
									fprintf(stderr, "%s", errstr);
								errorlog(errstr);
/*
 *								kill(alc->kmr_pid,MF_DIES);
 *								exit(1);
 */
							} else {
								setpr(alc->mf_pid,getpr(alc->mf_pid)-256);
								mlistfill(); 	
							}
							closepia(alc->piabase);		/* release miproc */
							break;
					case 'H': 
							putfipprompt(); 
							denshisto(); 
							break;
					case 'W': 
							if (prompt!= FOR_WSIG) break;
							putfipprompt();
							mlistfinish();
							if (alc->mipdebug) 
								fprintf(debugfp,"Waitmip want ctrl-d\n");
							prompt= FOR_CTRLD;
							break;
					case 'D': 
							if (prompt!= FOR_DSIG) break;
							putfipprompt();
							if (!dbuffill())
								ret_stat = FS_FAIL;
							if (alc->mipdebug)
								fprintf(debugfp,"done dbuffill\n");
							prompt = FOR_QUIET;
							break;
					default:
							if (errmsgs(prompt, c, &ret_stat)) {
								return(ret_stat);
							}
					}
					break;
				case 'D':
					if (alc->mipdebug) 
						fprintf(debugfp,"Waitfip - %c\n",c);
					putfipprompt(); 
					break;
				case 'U':
					if (alc->mipdebug) 
						fprintf(debugfp,"Waitfip - %c\n",c);
					putfipprompt(); 
					break;
				case 'C': 
					wasC = 1;
					if (alc->mipdebug) 
						fprintf(debugfp,"Waitfip - %c\n",c);
					putfipprompt(); 
					break;
				default:
					if (errmsgs(prompt, c, &ret_stat)) {
						return(ret_stat);
					}
			}
		}
	}
	if (alc->mipdebug) {
		fprintf(debugfp,"End waitfip stat %d\n",ret_stat);
		fflush(debugfp);
	}
	return(ret_stat);
}

/*
 *  E R R M S G S --
 * 
 *				Accumulates characters sent from miproc and checks for
 *				an error message.
 */
static
errmsgs(prompt, c, ret_stat)
register char c; 
int *ret_stat;
{
	
	/* Accumulate 'word_received' to compare with 'word_wanted' */
	if (( isalpha(c) || c == ' ' || c == '/' )&& wp<word_received + 127) {
		*wp++ = c;	
		return(0);
	} else if (wp != word_received) {
		if (alc->mipdebug > 3)
			fprintf(debugfp,"Waitfip - terminated by %d\n",c);
		*wp = 0;
		if (prompt == FOR_WORD) {
			if (alc->mipdebug > 2) {
				fprintf(debugfp,"Waitfip cmp %s with %s\n",word_received,word_wanted);
				fflush(debugfp);
			}
			if (strindex(word_received, wp - word_received, word_wanted,
   				 		 strlen(word_wanted))) {
				if (alc->mipdebug > 2) {
					fprintf(debugfp,"Waitfip - Got word\n");
					fflush(debugfp);
				}
				return(1);
			}
		} else {
			if (alc->mipdebug > 2) {
				fprintf(debugfp,"Waitfip Word recd %s\n",word_received);
				fflush(debugfp);
			}
			if ((wp-word_received) && (strindex(word_received,wp-word_received,
												"error",5)) != NULL) {
				*ret_stat = cfiperr(word_received,wp);
			}
		}
		wp = word_received;
	}
	if (*ret_stat != 0) 
		return(1);
	return(0);
}



/*
 *	N E W C H A R  --
 *
 */
char
newchar() 
{
	char	c, cc, fred[4], *buf;

	if (readmip(&cc,1,0) != 1) {
		if (alc->mipdebug)
			fprintf(debugfp,"newchar() error\n");
		return(0);
	}
	if (alc->mipdebug) {
		c = cc & 127;
		buf = fred;
		if (c < 32) {
			*buf++ = '^';
			*buf++ = c+'@';
		} else if (c == 127) {
			*buf++ = '^';
			*buf++ = '?';
		} else
			*buf++ = c;
		*buf = '\0';
		fprintf(debugfp,"newchar \"%s\"\n", fred);
	}
	return (cc);
}

/*
 * S E T _ S C A N _ T I M E O U T ---
 *
 */
set_scan_timeout()
{
	int tickpersec;
	
	wtim = TDIS4000 / alc->cfipp->speed;
	if ((tickpersec = _getsys(D_TckSec,2)) < 0)
		tickpersec = 100;
	wtim *= tickpersec;
	if ((wtim /= 10) < 1)
		wtim = 1;
	if (alc->devel)
		wtim *= 3; /* For Mark J */
	if(alc->debug)
		fprintf(debugfp, "swathe time: %d\n", wtim);
}

/*
 *
 *	B A Y _ M O V E  ---
 *
 *			Moves from current bay to datum of bay <newbay>.
 *
 */
bay_move(newbay)
int	newbay;
{
	if(newbay == baynn(alc->curbay))  
		return(1);
	else if(newbay <= alc->maxbays && newbay > 0) {
		if(bay_jump(newbay)) {
			fprintf(output, "Cannot complete move\n");
			return(0);
		}
		return(1);
	} else {
		fprintf(output, "Bad bay number requested (%d).  Bays 1 to %d available\n",
				newbay, alc->maxbays);
		return(0);
	}
}

/*
 *
 *	B A Y _ J U M P  ---
 *
 *			Moves from current bay to datum of bay <newbay>.
 *			Return	FS_OK if it succeeds else returns FS_FAIL.
 *
 */
static
int	bay_jump(newbay)
int	newbay;
{
int	diff, idiff, xdiff, ydiff, zdiff, lastxbit, xoffset = 32000;
struct	mult_slide *bp;

	movespeed();
	diff = newbay - baynn(alc->curbay);
	stagepos(0, 0, 0);						/* Move to datum */
	if(diff > 0)
		idiff = 1;
	else 
		idiff = -1;

	if(tomip1("xs", 2))  return(FS_FAIL);
		
	while(diff) {
		getpos(2);							/* Get current position */
		bp = alc->curbay;
		if(idiff < 0)
			bp--;
		tomip2("d", -idiff * xoffset, ys);
		tomip1("dz", zs);
		xdiff = idiff * (bp->xoff - xoffset);
		ydiff = idiff * bp->yoff;
		zdiff = idiff * bp->zoff;
		do {
			if (stagepos(xdiff, ydiff, zdiff) != FS_OK)
							return(FS_FAIL);
		} while (xs < xdiff - 2 || xs > xdiff + 2 || ys < ydiff - 2 || 
				 ys > ydiff + 2 ||  zs < zdiff - 2 || zs > zdiff + 2); 
				 
		if(tomip0("d"))
			return(FS_FAIL);
		
		diff -= idiff;
		alc->curbay += idiff;
	}
	getpos(2);								/* Update current position */
	return(FS_OK);
}
/*
 *
 *	B A Y N N --
 *
 *				Calculate from ptr integer no of bay.
 *
 */
int
baynn(bpt1)
struct	mult_slide *bpt1;
{
int	nn = 0;

	if(!bpt1 || !alc->frsbay || bpt1 < alc->frsbay) {
		fprintf(output,"Baynn: Illegal bay ptrs.\n");
		return(0);
	} else {
/*
 *		Compiler bug -> can't work out how to move the struct size 
 *		into d1 so we will have to do it explicitly ....  CAS 
 */
		nn = ((char *) bpt1 - (char *) alc->frsbay) / 
										sizeof(struct mult_slide);
		if(nn >= alc->maxbays) {
			fprintf(output,"Baynn: Bay no outside possible range!\n");
			return(0);
		}
		return(nn+1);
	}
}

/*
 *	K I L L S C A N
 */
killscan()
{
	int		num;
	char	sbuf[64];
	struct	sgbuf oldmode;

	if (alc->killscan)
		return(1);
	if ((num = _gs_rdy(fileno(stdin))) > 0) {
		raw(fileno(stdin), &oldmode);
		do {
			if (!read(fileno(stdin), sbuf, 1)) {
				cook(fileno(stdin), &oldmode);
				return(0);
			} else if (*sbuf == '\033') {
				cook(fileno(stdin), &oldmode);
				alc->killscan = 1;
				return(1);
			} else if (*sbuf == '\021') {
				if(alc->debug)
					fprintf(debug_channel, "Received XON\n");
				_ss_rel(fileno(stdout));
			} else {
				cook(fileno(stdin), &oldmode);
				return(0);
			}
		} while ((num = _gs_rdy(fileno(stdin))) > 0);
		cook(fileno(stdin), &oldmode);
	}
	return(0);
}

