/*
 *	D B U F . C -- Array digitise functions
 *	Digitising data via the PIA; filing; labelling; display
 *
 *	NOTE
 *	ONLY 5 BITS USED FOR GREY DOISPLAY SINCE 3 USED FOR OTHER THINGS
 *	(SEE "simslide.c")
 *	Modification History
 *  06 Oct 1989     arl         moved release() to oddsods.c
 *	25 May 1989		dcb			255 runs done properly
 *	23 May 1989		dcb			Check *(out-3) in merging to detect EOL + single
 *								data item case. This may be the 'slipped line
 *								problem. Also finish removing debug
 *	19 May 1989		dcb			Debug to alc->debug
 *	17 May 1989		dcb			Cope with merging > 255 zeros in run.
 *								Use bcopy instead of quickcpy
 *	xx May 1989		dcb			Major rewrite, Remove 'out' param to dbuffill,
 *								split dfuffill to just fill	add dbuffconstruct
 *								for construction of woolz object prob.
 *								digbuf a static global, objdig - ptr to raw
 *								digitised object, PREBYTES, bytes after digbuf,
 *								at which objdig starts currently 4 lines * 1024
 *								bytes = 4096 introduce dbuflen to hold no. of
 *								bytes in objdig	move showdc to digitise.c etc
 *								digbuf is 0x300000 (bytes)
 *	31 Jul 1987 CAS/GJP 		Return digbuf if no objects seen
 *	20 Jul 1987		CAS			Extra 'bit' of colour in 'showdc'
 *	15 Jul 1987		CAS			Dig buffer now up to 400000......
 *	02 Apr 1987		SCG			Modify showdc to display with offset.
 *	11 Mar 1987		CAS			Catch nfipobj fail
 *	05 Mar 1987		GJP			Extra diags in dbuff
 *	20 jan 1987		CAS			Moved mcastel + normalise calls to capture
 *	15 Jan 1987		CAS			make thnormalise + normalise mut exclusive
 *	14 Jan 1987		CAS			added htnormalise in devel mode
 *	13 Jan 1987		CAS			removed interrupt(2) after piaopen fail.
 *	12 Jan 1987		CAS			Bump priority up arround digitise
 *	10 Dec 1986		CAS/SCG		Minor change to readpia after digfail
 *	10 Oct 1986		CAS/GJP		Mod to releas - params and use freeobj
 *	 9 Oct 1986		CAS			MRC Mods - 25/6 + 17/6 (Jimp) - Add freeplist
 *								to release and add call to mcastel after nfipobj
 *	30 Sep 1986		CAS			Convert malloc to Malloc
 *								and free to Free
 *	29 Sep 1986		CAS			Moved 'putfile' to fileio.c
 *	27 Aug 1986		CAS			Explicitly zero'd alc->fipdig in dbuffill
 *	26 Aug 1986		CAS			Moved closepia in dbuffill
 *								Enlarged dig buffer to 225000
 *	22 Aug 1986		CAs			If get buffer overflow - read rest of data
 *								from miproc before returning
 *	18 Aug 1986		CAS			Retrurn status from dbuffill
 *	25 Jul 1986		CAS			Added timeout to openpia call
 *	24 Jul 1986		CAS			More diagnostics
 *	 8 Jul 1986		GJP			More diagnostics
 *	12 Jun 1986		GJP			Add extra diagnostics to track down filling bugs
 *	 							and attempt to cope with them
 *	20 May 1986		CAS			Made maxdig 150000 (was 100000)
 *	   Aug 1985		Denis		Split into separate pia, label, display, file subrs.
 *	   Jun 1985		Marge/Jim	Cytel version
 */
#include <cmdsys.h>
#include <kyy.h>

#define MAXDIG  300000
#define MINDIG  10000
#define PREBYTES 4096	/* 4 lines of 1024 bytes */
#define BLOCKSIZE 256

char *Malloc();
static maxdig = MAXDIG;
extern FILE	*debug_channel;

/*
 *	Ptr to digitisation buf, & raw digitised object
 *	Note these buffers are contiguous, and that objdig = digbuf+PREBYTES
 *	dbuflen is the actual number of bytes in the buffer
 */
static int dbuflen;
static unsigned char *digbuf, *objdig;

static
fixmaxdig(k) int *k; {
	if (*k > MAXDIG) *k = MAXDIG;
	if (*k < 100 ) *k = 100;
	maxdig = *k;
}

/*
 *	D B U F F I L L  --  Read digitisation from miproc
 *
 *	Returns -  1   OK
 *			   0   Fail
 */
dbuffill()
{
	int i,k;
	struct object *nfipobj();
    
	if (alc->fipdig) {
		freeobj(alc->fipdig);
		if (alc->debug)	fprintf(output,"alc->fipdig after freeobj 0x%x - set to NULL\n",alc->fipdig);
		alc->fipdig = NULL;
	}

	if (digbuf)	free_digbuf();
	alloc_digbuf(NULL);

	if ((i = openpia(0,8)) < 0) {		/* Open with timeout */
		fprintf(output,"openpia fails %d\n",i);
		if (alc->debug)
			fprintf(debug_channel,"OPENPIA failed error %d\n",i);
		free_digbuf();
		return(0);
	}
/*
 *	Read digitisation from pia -
 *	Done with an elevated priority to try to avoid miproc timeouts
 *	Upped priority elevation, the rest of the system will grind to a halt
 */
	setpr(alc->mf_pid,getpr(alc->mf_pid)+256);
	dbuflen = readpia8(objdig,maxdig-PREBYTES);	/* dtb */
	setpr(alc->mf_pid,getpr(alc->mf_pid)-256);
	if (alc->debug)
		fprintf(debug_channel,"readpia8 read 0x%X. (maxdig=0x%X)\n",
			dbuflen, maxdig);
	if (dbuflen >= (maxdig-PREBYTES)) {
		fprintf(output,"buffer overflow - digitisation incomplete\n");
		while (readpia(objdig, maxdig) != 0); /* Remove extra info */
	}
	if (alc->debug)	fprintf(debug_channel,"Done read - now to close\n");
	closepia(0);
	if (alc->debug) {
		fprintf(debug_channel,"DBUF : bytes copied = %d\n",dbuflen);
	}
	if (alc->debug)
		fprintf(debug_channel, "dbuffill: Digitisation buffer filled\n");
	return(1); /* OK */
} /* end dbuffill */

#ifdef GGG
/*
 *	D B U F C O N S T R U C T -- Construct woolz object from buffer
 *	construct Woolz standard object (type 21 grey-table)
 *	Make dig data into a real woolz type object.  Uses the orig digitisation
 *	as the grey table for the object
 */
dbufconstruct(blength)
int blength;
{
	if (alc->debug)
		fprintf(output,"Before nfipobj\n");
	alc->fipdig = nfipobj(digbuf, blength);
	if (alc->debug)
		fprintf(output,"After nfipobj\n");
	if (alc->fipdig == 0) {
		fprintf(output,"No objects seen - digitisation failed\n");
		free_digbuf();
		return(-1);	/* Error return */
	}
	return(0); /* OK return */
}
#endif

/*
 *	F R E E _ D I G B U F --  Free the digitisation buffer
 *	and NULL its pointer
 */
free_digbuf()
{
	if (alc->debug)
		fprintf(debug_channel, "Freeing digitisation buffer at 0x%X\n", digbuf);
	if (digbuf != NULL) { /* Protect and survive */
		Free(digbuf);
		digbuf = NULL;
		objdig = NULL;
		dbuflen = 0;
	}
}

/*
 *	A L L O C _ D I G B U F -- Allocate space for digitisation buffer
 */
alloc_digbuf(newptr)
unsigned char * newptr;
{
	if (digbuf) {
		fprintf(debug_channel, "alloc_digbuf: Buffer already allocated\n");
		return; /* Already allocated */
	}
	Alloc_wait(0);	/* Don't wait */
	if (alc->debug)
		fprintf(debug_channel,"Allocate digitisation buffer\n");
	if (newptr) {
		digbuf = newptr;
		if (alc->debug)
			fprintf(debug_channel, "alloc_digbuf: Using newptr\n");
	}
	else {
		maxdig = MAXDIG;
		while ((digbuf = (unsigned char *)Malloc(maxdig)) == NULL) { 
			if (alc->debug)
				fprintf(debug_channel, "\007Allocation (0x%X) failed\n",
						maxdig);
			maxdig = (int) (maxdig * 0.9);	/* 90 % */
			if (maxdig < MINDIG) return(NULL); /* Must be > MINDIG */
		} 
	}
	if (alc->debug)
		fprintf(debug_channel, "Digbuf allocated at 0x%X, size \n",
			digbuf, maxdig);
	objdig = digbuf+PREBYTES;
	dbuflen = 0;
	Alloc_wait(1);		/* Return to out of memory waiting ... */
	return(digbuf);
}

/*
 *	D I S P _ D I G -- Display digitised image for BLU wash
 *	This function hits the framestore direct, mainly to bump up speed
 *
 */
disp_dig(dix, diy)
int dix, diy;	/* Rows & cols to display (digs) */
{
	
	register unsigned char *fr;
	unsigned char *frame;	/* The framestore direct hit address(es) */
	register unsigned char *twoline;	/* Pointer to current bit in two line buffer */
	register unsigned char *dig;		/* Pointer to bit in dig. buffer */
	unsigned char *digend;		/* End of digitised data */
	register int i;
	int lines, ll, current_x_pos, bytes_per_line;
	int ypos;
	int incr = 1;				/* Column to column increment (default) */
	register unsigned char *line1, *line2; /* Ptrs to 2 line buffer */
	DDGS	*fs;	/* The f/store structure */

	if (alc->debug)
		fprintf(debug_channel, "disp_dig: Started\n");
 	fs = alc->ddgschannel;
 	dig = objdig;				/* Initialise to start */
	digend = objdig+dbuflen;	/* Initialise to end */
	bytes_per_line = fs->lineln; /* Length of a line (pixels) */
	if (fs->type == FS_VME512) {
		if (alc->debug)
			fprintf(debug_channel, "VME 512 present\n");
		bytes_per_line *= 2;
		incr = 2; /* Col to col incr is by word */
	}
/*
 *	Set display position to be middle of screen
 *	Origin in this case is top LH corner of screen
 */
	current_x_pos = (fs->lineln - dix/2)/2;					/* Centre x */
	current_x_pos *= incr;									/* in bytes */
	ypos = (fs->lineno - diy/2)/2;
	frame = fs->frame + (ypos*bytes_per_line);	/* Centre y */
	if (alc->debug > 1) {
		fprintf(debug_channel, "current_x_pos=%d, ypos=%d\n",
			current_x_pos, ypos);
		fprintf(debug_channel, "f/s=0x%X, bytes/line=%d\n", frame, bytes_per_line);
		fprintf(debug_channel, "dig = 0x%X, digend = 0x%X\n", dig, digend);
		fprintf(debug_channel, "digbuf=0x%X\n", digbuf);
	}
/*
 *	Now start the display
 */
	llut();
	fast();
	clear();
	for (twoline = digbuf; twoline <dig; twoline++) *twoline = 0;

	while (dig < digend) {			/* Still some to display */
		lines = 2;					/* Uncompact to two line buffer */
		twoline = line1 = digbuf;	/* Init. two line buffer ptrs */
		while (lines) {				/* Still lines to do */
 			while(*dig)
 				*twoline++ = *dig++; /* Transfer normal data */
			if (!*dig) {			/* Special code */
				dig++;
				if (*dig) {			/* Blank pixel count */
					for (i= *dig++; i>0; i--) *twoline++ = 0; /* Insert them */
				} else {	/* End of line */
					*twoline++ = 0; /* Bung in a zero for good measure */
					dig++; /* Advance over 2nd zero */
					lines--;	/* end of line, so done another one */
					if (lines == 1) line2 = twoline; /* Start of 2nd line */
				}
			} /* end of special codes */
		} /* end of while lines ... */
		/*
		 *	Now display to f/store
		 *	F/store pixel = average of 2x2 block from 2 line buffer
		 */
		fr = frame + current_x_pos;
		ll = (line2-line1) >> 1;

		for (i=0; i < ll; i++) {
				*fr = (*line1++ + *line1++ + *line2++ + *line2++) >> 2;
			fr += bytes_per_line;	/* Move down a line */
		}

		current_x_pos += incr;		/* Move to next column (The byte address) */
	} /* end while dig < digend ... */ 

	slow();
	return(0);
} /* end disp_dig */

/*
 *	A R R A Y _ T H R -- Threshold array digitised data
 *	Uses objdig, the raw object data.
 *	Places data in new smaller buffer, frees old larger one
 *	Returns pointer to this buffer, and actual size of this buffer
 */
unsigned char *
array_thr(thr, size)
int thr;	/* Threshold to use */
int *size;	/* Size of new buffer */
{
	register unsigned char *digend;	/* End of digitised data */
	register unsigned char *dig;	/* Pointer to bit in dig. buffer */
	register unsigned char *out;	/* Thresholded output */
	unsigned char *newbuf;			/* Temp ptr to new smaller buffer */
	register int i, bp, t;			/* Loop ct, blank pix. run ct, & thr */
	int merges=0;

	t = thr;
	if (alc->debug)
		fprintf(debug_channel, "array_thr: Original buffer size = %d\n", dbuflen);
	dig = objdig;				/* Initialise to start */
	digend = objdig+dbuflen;	/* Initialise to end */
	out = digbuf;				/* Thresholded output */

	if (alc->debug)
		fprintf(debug_channel,"array_thr: dig=0x%X, dbuflen=%d (0x%X), thr=%d, out=0x%X\n",
			dig, dbuflen, dbuflen, t, out);

	while (dig < digend) {
		if (*dig >= t) *out++ = *dig++;		/* Over thr. data */
		else if (*dig)	{					/* Under thr data */
			bp=0;
			while ((dig < digend) && (*dig) && (*dig < t)) {
				bp++;			/* Under thr data, acc run */
				dig++;			/* To look at next */
			}
/*
 *	Check to see if run could be merged with a previous run
 *	Last but 2 is zero (Introduces a zero run)
 *	Last but one is data (i.e. Not end of line marker)
 *	Last but 3 is non-zero (This detects end of line marker followed by a
 *	single data item)
 */
			if (!*(out-2) && *(out-1) && *(out-3)) {
				merges++;
				bp += *(out-1);				/* Add to prev. count */
				out -= 2;
			}
			if ((bp>255) && (alc->debug)) {
				fprintf(stderr, "\007Zero run > 255\n");
			}
			while (bp > 255) {
				*out++ = 0;			/* Introduce run of zeros */
				*out++ = 255;		/* Store full length run */
				bp -= 255;			/* Done up to 1st 255 zeros */
			}
			if (bp)	{				/* Remainder to do */
				*out++ = 0;
				*out++ = bp;
			}
		}
		else {
			*out++ = *dig++;	/* Copy zero special */
			*out++ = *dig++;	/* + other code */
		}
		if (out > dig) {
			fprintf(stderr,"\007Thresholded object occupies too much space!\n");
			fprintf(stderr,"Please re-try threshold\n");
			if (alc->debug)
				fprintf(debug_channel, "Merges = %d\n", merges);
			*size = 0;
			return(NULL);
			break;
		}
	} /* end while */

	*size = out - digbuf + 1;
	if (alc->debug)
		fprintf(debug_channel, "New buffer size = %d\nMerges=%d",
			*size, merges);

	if ((newbuf = (unsigned char*)Malloc(*size)) == NULL) {
		fprintf(stderr, "Failed to allocate copy buffer\n");
		return(NULL);
	}
	if (alc->debug)
		fprintf(debug_channel, "Allocate copy buffer at 0x%X, size=%d (0x%X)\n",
			newbuf, *size, *size);
/*	quickcpy(newbuf, digbuf, *size); use this when it works !!		*/
/*	NOTE: bcopy parameters are the wrong way round */
	bcopy(digbuf, newbuf, *size);			/* Straight copy of data */
	free_digbuf();							/* Free old big buffer */
	alloc_digbuf(newbuf);					/* And assign new smaller one */
	dbuflen = *size;
	digend = digbuf + *size;
	return(digbuf);							/* Return pointer to buffer */
} /* end array_thr */

/*
 *	Q U I C K C P Y -- Copy buffer to buffer
 *
 */
static
quickcpy(to, from, size)
unsigned char *to, *from;
int size;
{
	register unsigned *out, *in;
	register unsigned sz;
	register int i;

	out = (unsigned *)to;
	in = (unsigned *)from;
	sz = size/4;	/* Size given in bytes, copy in in longs */

	for (i=0; i<sz; i++)
		*out++ = *in++;	/* Straight copy of data */
} /* end quickcpy */

/*
 *	T E S T _ W E D G E -- Write a test wedge to the digitisation buffer
 */
test_wedge()
{
	register int i, j;
	register unsigned char *buf;
	int nlines;

	if (digbuf) free_digbuf();
	alloc_digbuf(NULL);

	buf = digbuf+PREBYTES;
	dbuflen = maxdig-PREBYTES-100;
	nlines = dbuflen/1026;

	for (i=0; i<nlines; i++) {
		for (j=0; j<1024; j++) *buf++ = j;
		*buf++ = 0;
		*buf++ = 0; /* end of line = 0, 0 */
	}
	return(1);

} /* end of test_wedge */
 

filedump(filename)
char *filename;
{
	unsigned char *dig, *digend;
	FILE *fp, *fopen();
	int blocks, i;

	fprintf(stderr, "Dumping image to %s\n", filename);
	if ((fp = fopen(filename, "w")) == NULL) {
		fprintf(stderr, "Can't open %s for write\n", filename);
		return(-1);
	}
	dig = objdig;
	digend = objdig+dbuflen;
	if (alc->debug) {
		fprintf(output, "filedump: dig = 0x%X, digend = 0x%X\n", dig, digend);
	}
	if ((dbuflen) > maxdig) {
		fprintf(output, "Digitisation buffer pointer error, can't dump\n");
		return(-1);
	}
	blocks = (dbuflen/BLOCKSIZE) - 1;
	if (blocks <= 0) {
		fprintf(output, "No image to dump\n");
		return(-1);
	}
	windmill("Dumping ");
	for (i=0; i<blocks; i++) {
		if (!(i%50)) windmill(NULL);
		write(fileno(fp), dig, BLOCKSIZE);
		dig += BLOCKSIZE;
	}
	windmillend("complete\n");
	fclose(fp);
}

/*
 *	R E S T O R E -- Restore dumped image from file
 */
restore(filename)
char *filename;
{
	unsigned char *dig, *digend;
	FILE *fp, *fopen();
	int i=0;

	fprintf(stderr, "Restoring image from %s\n", filename);
	if (!digbuf) {
		alloc_digbuf(NULL);
	}
	if ((fp = fopen(filename, "r")) == NULL) {
		fprintf(stderr, "Can't open %s for read\n", filename);
		return(-1);
	}
	dig = objdig;
	digend = objdig + maxdig - PREBYTES - BLOCKSIZE;

	if (alc->debug)
		fprintf(output, "dig = 0x%X, digend = 0x%X\n", dig, digend);
	windmill("Loading ");

	while (read(fileno(fp), dig, BLOCKSIZE)) {
		dig += BLOCKSIZE;
		if (dig > digend) break;
		if (!(i++ % 50)) windmill(NULL);
	}
	windmillend("complete\n");
	dbuflen = dig - objdig;
	if (alc->debug)
		fprintf(debug_channel, "\nRead %d (0x%X) bytes\n", dbuflen, dbuflen);
	*dig++ = 0; *dig++ = 0;
}
