/*
 *	File:	listfuncs.c
 *		
 *		Metaphase list manipulation functions.
 *
 *
 *	Written by:	S. C. Georgiou
 *				720 Birchwood Boulevard,
 *				Birchwood Science Park,
 *				Warrington,
 *				Cheshire.
 *
 *	Date: 28 May 1987.
 *
 *
 *	Modifications:
 *
 *	15 Jan 1991		CAS/GJP		debug_channel used when not open
 *	31 Oct 1989		dcb				methtostruct: Return error if unknown hdr
 *									readnewlist: Use returned val from methtostruct
 *	30 Oct 1989		dcb				Allow reading of V 3 header as V 2
 *	 5 May 1989		dcb				methtostruct,Rip out copy to ktissue kfstain
 *	08 Feb 1989		dcb				Trivial added ? after abandon list msg.
 *	29 Nov 1988		SEAJ			Added checks for disc allocation.
 *	24 Jun 1988		SEAJ			Compilation bug fixes for below
 *  23 Jun 1988		SCG/SEAJ		Bug fixed in methtostruct
 *	16 Jun 1988		SCG				Fill the tissue and stain strings when
 *									reading a list
 *	 6 Jun 1988		SCG/SEAJ		Minor mod in linksub (by groups).
 *   2 Jun 1988		SCG/SEAJ		Changed loop counter in linksub from ngrps
 *									to count.
 *  12 May 1988		SEAJ			Added in windmills into readnewlist.
 *  11 May 1988		SCG/SEAJ		Re-modified linksub to use qsort on the rank
 *									ordered linked list.
 *   9 May 1988		SCG/SEAJ		changed calls to chekattr
 *	09 May 1988		SCG/SEAJ		writelist does not switch usables to include
 *   6 May 1988		SCG & SEAJ		Modified linksub to make it faster.
 *	13 Aug 1987		SCG				Normals/abnormals taken out
 *	07 Aug 1987		SCG				Change calls to chekattr
 *	05 Aug 1987		SCG				Put some vars in registers for speed
 *	24 Jul 1987		CAS				Fixed junk code around makemethead..
 *	22 Jul 1987		GJP				Change debugging level in linklist
 *	22 Jul 1987		CAS				Fix strcpy bug in methtostruct
 *									linksub - if fsubptr = NULL set currptr
 *									to startptr
 *	17 Jul 1987		SCG				linksub reset currptr to fsubptr
 *	17 Jul 1987		SCG				If when writelist called inrank is
 *										on only switch also usables
 */
												/* Includes */
#include	<cmdsys.h>
#include	<kyy.h>
#include	<slides.h>
												/* Defines */
extern	FILE *debug_channel;
struct	object *readobj(), *makemain();
struct	mipclust *grabmbuf();
struct	clustdat *grabmolbuf();
struct	mlclust *grabmlbuf();
struct	metlink *grabnlnk();
extern	int allcells(), setrtb(), mksubtxt(), knowntissue(), knownstain();
extern	struct met_header *makemethead();
static	int readcomms(), readnewlist(), readoldlist(), methtostruct();

#define	FORW 1
#define BACKW -1
#define BLSIZE 256
/*
 *	W R I T E L I S T
 *
 *				Take a file pointer and write list.
 */
writelist(fq)
FILE	*fq;
{
struct	met_header *met_h;
struct	mult_slide	*bp;
struct	object *head;
int	totout, k, stat = 0;

	bp = alc->curbay;
	if (bp->startptr == NULL) {						/* No list in */
		fprintf(output, "No list in - can't file\n");
		return;
	}
	if(fq == NULL) {								/* Bad file pointer */
		if(alc->debug)
			fprintf(debug_channel,"writelist:  null file pointer\n");
		return(0);
	}
	windmill("Writing metaphase list - ");
	met_h = makemethead(NULL);							/* Make file header */
	metheadfill(met_h, bp);
	head = makemain(110,NULL,NULL,met_h,NULL);

	k = seqsize(head);
	if (met_h->comments)
		k += (bp->compos - bp->combot);
	k += bp->totin * sizeof(struct mlclust);
	if (stat = preexpand(fq, k)) {
		fseek (fq, (long) seqsize(head), 0);			/* position at end of 
													   header */
 		if (alc->debug) {
	 		fprintf(debug_channel,"Filing comments\n");
 			fflush(debug_channel);
 		}
		if (met_h->comments) {							/* if comments file'em */
			met_h->comments =  bp->compos - bp->combot;
			fbwrite(bp->combot, met_h->comments, 1, fq);
			if (alc->debug)
				fprintf(stderr,"Write comments 0x%x len %d\n",
						bp->combot, met_h->comments);
		}
		windmill(NULL);
 		if (alc->debug) {
	 		fprintf(debug_channel,"Filing data\n");
 			fflush(debug_channel);
 		}		
 													/* file list */
	 	fbwrite(bp->startptr->mlrec, bp->totin * sizeof(struct mlclust), 1, fq);
		totout = alc->curbay->totin;
		if (alc->debug) {
			fprintf(stderr,"List filed (%d records)\n",totout);
			fprintf (debug_channel, "%d records filed",totout);
			fflush(debug_channel);
		}
		met_h->nrecs = totout;
		fseek (fq,0,0);
		writeobj(fq,head);								/* Now write header */
	}
	freeobj(head);
	windmillend(NULL);
}
/*
 *	R E A D L I S T
 *
 *				Take a file pointer and read list.
 */
readlist(fq)
FILE	*fq;
{
int		nn, requd, dispgone=0, notfound = 0, size, comments, nrecs;
struct	met_header *met_h;
struct	object *head;
struct	mult_slide *bp;

	bp = alc->curbay;
	if(bp->startptr != NULL) {			/* If a list exists remove it. */
		fprintf(output, "Read in new list. Abandon previous one ? \n");
		if(yesno(1))  					/* Confirm the removal of old list. */
			return(0);
		freelist();						/* Free old list */
	}
	head = readobj(fq);
	if (!head || (head->type != 110)) {
		fprintf(stderr,"metaphase list file has illegal format\n");
		freeobj(head);
		return(0);
	}
	met_h = (struct met_header *) head->plist;
	head->plist = NULL;
	freeobj(head);
	

	switch (met_h->version) {
		case 1:								/* Old type header */
			if(!readoldlist(fq, (struct met_header_1 *) met_h, bp)) {
				fprintf(output, "Unable to read old list.\n");
				Free(met_h);				/* Free header */
				return(0);
			}
			break;
		case 2:								/* New type header */
		case 3:
			if(!readnewlist(fq, met_h, bp)) {
				fprintf(output, "Unable to read list.\n");
				Free(met_h);				/* Free header */
				return(0);
			}
			break;
		default:
			fprintf(output,"Header version unknown.\n");
			Free(met_h);					/* Free header */
			return(0);
	}
 	if (alc->debug) {
 		fprintf(debug_channel,"metlist - nrecs %d\n",bp->totin);
 		fflush(debug_channel);
 	}
	Free(met_h);						/* Free header */
	return(1);
}
/* 
 *	R E A D O L D L I S T --- Reads a metlist of version 0.4.3 and prior
 */
static
int readoldlist(fq, omet_h, bp)
FILE	*fq;
struct	met_header_1 *omet_h;
struct	mult_slide *bp;
{
struct	clustdat *mcbuf;
struct	mlclust *mptbuf;
struct	metlink *linkptr;
int	size,nn;
 
	fprintf(output, "Old format list read.  Converted to new format.\n");
	if(omet_h->nrecs < 1) {
		fprintf(output,"Corrupted file!\n");
		return(0);
	}
	readcomms(fq, omet_h->comments, bp);		/* Read comments first */
	mcbuf = grabmolbuf(omet_h->nrecs);			/* Grab record buffer */
	size = omet_h->nrecs * sizeof(struct clustdat);
												/* Read and check that block
												   size is correct */
	if ((nn = fbread(mcbuf, 1, size, fq)) != size) {
		if ((nn % sizeof(struct clustdat)) != 0) {
			fprintf(output, "Corrupted file!\n");
			bp->startptr = NULL;
			Free(mcbuf);						/* Free the buffer */
			return(0);
		}
		if (!nn) {
			fprintf(output, "Empty file.  No records!\n");
			bp->startptr = NULL;
			Free(mcbuf);						/* Free the buffer */
			return(0);
		}
	}
	mptbuf = grabmlbuf(omet_h->nrecs);			/* Grab mlclust block */
	oldtonew(mptbuf, mcbuf, omet_h->nrecs);		/* Convert clustdat to 
												   mlclust */
	Free(mcbuf);								/* Free record buffer */
	linkptr = grabnlnk(omet_h->nrecs);
	linkblk(mptbuf, linkptr, omet_h->nrecs);
 	methtostruct(omet_h, bp);					/* Header to struct */
	frank();									/* Re-rank old list */
 	allcells(0);								/* Define all as subset */
 	if (alc->debug) {
 		fprintf(debug_channel,"metlist - Read data %d\n",
 									nn/sizeof(struct clustdat));
 		fflush(debug_channel);
 	}
 	return(1);
 }
/* 
 *	R E A D N E W L I S T --- Reads a metlist created after version 0.4.3 
 */
static
int readnewlist(fq, met_h, bp)
FILE	*fq;
struct	met_header *met_h;
struct	mult_slide *bp;
{
struct	mlclust *mptbuf;
struct	metlink *linkptr;
int	size,nn, number, remd, i;
char *lmpt;

	if(met_h->nrecs < 1) {
		fprintf(output,"Corrupted file!\n");
		return(0);
	}
	readcomms(fq, met_h->comments, bp);			/* Read comments first */
	windmill("Reading list - ");
	mptbuf = grabmlbuf(met_h->nrecs);			/* grab records buffer */
	lmpt = (char *) mptbuf;	
	
	number = met_h->nrecs/BLSIZE;
	remd = met_h->nrecs - (BLSIZE * number);
	size = BLSIZE * sizeof(struct mlclust);
	for (i=0; i<number; i++) {
		if ((nn = fbread(lmpt, 1, size, fq)) != size) {
			if ((nn % sizeof(struct mlclust)) != 0) {
				fprintf(output, "Corrupted file!\n");
				bp->startptr = NULL;				/* No list in */
				Free(mptbuf);						/* Free the buffer */
				return(0);
			}
			if (!nn) {
				fprintf(output, "Empty file.  No records!\n");
				bp->startptr = NULL;
				Free(mptbuf);						/* Free the buffer */
				return(0);
			}
		}
		lmpt += nn;
		windmill(NULL);
	}
	size = remd * sizeof(struct mlclust);
	if ((nn = fbread(lmpt, 1, size, fq)) != size) {
		if ((nn % sizeof(struct mlclust)) != 0) {
			fprintf(output, "Corrupted file!\n");
			bp->startptr = NULL;				/* No list in */
			Free(mptbuf);						/* Free the buffer */
			return(0);
		}
		if (!nn) {
			fprintf(output, "Empty file.  No records!\n");
			bp->startptr = NULL;
			Free(mptbuf);						/* Free the buffer */
			return(0);
		}
	}
	lmpt += nn;
	windmill(NULL);
	
	linkptr = grabnlnk(met_h->nrecs);
	linkblk(mptbuf, linkptr, met_h->nrecs);
 	if (methtostruct(met_h, bp) < 0) {
 		Free(mptbuf);
 		Free(linkptr);
 		return(0);
 	}
 	if (alc->debug) {
 		fprintf(debug_channel,"metlist - Read data %d\n",
 											nn/sizeof(struct mlclust));
 		fflush(debug_channel);
	}
 	windmill(NULL);
	setrtb();									/* Obtain rtb and link
													   by ranks */
	windmillend(NULL);
	return(1);
}
/*
 *	R E A D C O M M S --- Read comments
 */
 static
 int	readcomms(fq, comments, bp)
 FILE	*fq;
 int	comments;
 struct	mult_slide *bp;
 {
 
	if (comments) {		 		/* Read if they exist */
	 	if (alc->debug) {
 			fprintf(debug_channel,"metlist - comments %d\n",comments);
 			fflush(debug_channel);
 		}
		if (bp->combot) {
			if (bp->comtop - bp->combot <  comments) {
				Free(bp->combot);
				bp->combot=0;
			}
		}
		if (!bp->combot) {
			bp->combot = (char *) Malloc(comments + 100);
			bp->comtop = bp->combot + comments + 100;
		}
		bp->compos = bp->combot + comments;
		fread(bp->combot,comments,1,fq);
	}
	
}
/*
 *	O L D T O N E W --- Convert a block of clustdat recs to mlclust recs
 */
static
oldtonew(mptbuf, mcbuf, tot)
struct	mlclust *mptbuf;
struct	clustdat *mcbuf;
int	tot;
{
int	i;

	for(i = 0; i < tot; i++) {
								/* clustdat is a subset of mlclust, so use
								   wholesale copy */
		_strass(mptbuf, mcbuf, sizeof(struct clustdat));
		mptbuf->recnr = i+1;
		mptbuf++;
		mcbuf++;
	}
}
/*
 *	F R E E L I S T
 *
 *			Free list at the current bay.
 */
freelist()
{

	if(alc->curbay->startptr == NULL) {
		alc->curbay->nameids.cellno = 0;
		alc->curbay->totin = 0;
		return;
	}
	Free(alc->curbay->startptr->mlrec);
	Free(alc->curbay->startptr);
	alc->curbay->startptr = NULL;
	alc->curbay->nameids.cellno = 0;
	alc->curbay->totin = 0;
	
}
/*
 *	G R A B N L N K -- Allocate n structs of type metlink.
 */
struct metlink *grabnlnk(n)
int	n;
{

	return((struct metlink *) (Calloc(n, sizeof(struct metlink))));
	
}
/*
 *	G R A B M B U F -- Allocate n structs of type mipclust.
 */
struct mipclust *grabmbuf(n)
int	n;
{

	return((struct mipclust *) (Calloc(n, sizeof(struct mipclust))));
	
}
/*
 *	G R A B M L B U F -- Allocate n structs of type mlclust.
 */
struct mlclust *grabmlbuf(n)
int	n;
{

	return((struct mlclust *) (Calloc(n, sizeof(struct mlclust))));
	
}
/*
 *	G R A B M O L B U F -- Allocate n structs of type clustdat.
 */
struct clustdat *grabmolbuf(n)
int	n;
{

	return((struct clustdat *) (Calloc(n, sizeof(struct clustdat))));
	
}
/*
 *	N E X T R E C
 *
 *		Gets next record in current subset if step +ve or previous if
 *		step -ve.  Returns 1 if successful, 0 if no list in or
 *		at beginning or end of list.
 */
nextrec(step)
int	step;
{
struct	mult_slide	*bp;

	bp = alc->curbay;
	if(bp->startptr == NULL) {				/* No list in this bay */
		fprintf(output, "No list in.\n");
		return(0);
	}
	if(bp->fsubptr == NULL) {				/* No records in subset */
		fprintf(output, "Empty subset.\n");
		return(0);
	}
			
	if(step > 0) {							/* Take +ve step if poss */
		if(bp->csubptr->nsub == NULL) {		/* Cant move forward */
			if(bp->currptr != bp->csubptr)	/* Reset currptr */
				bp->currptr = bp->csubptr;
			return(0);
		} else								/* Move forward from current sub
											   position; reset currptr */
			bp->currptr = bp->csubptr = bp->csubptr->nsub;
			
	} else if(step < 0) {					/* Take -ve step if poss */
		if(bp->csubptr->psub == NULL) {		/* Cant move backward */
			if(bp->currptr != bp->csubptr)	/* Reset currptr */
				bp->currptr = bp->csubptr;
			return(0);
		} else								/* Move backward from current sub
											   position; reset currptr */
			bp->currptr = bp->csubptr = bp->csubptr->psub;
	}
											/* Put no in ids struct */
	bp->nameids.cellno = bp->currptr->mlrec->recnr;
	return(1);
}
/*
 *	F I N D R E C
 *
 *		Find record defined by arg nn starting from beginning of list.
 *		Returns 1 if successful, 0 if no list in or request outside 
 * 		bounds of list.
 *		
 */
findrec(nn)
int	nn;
{
struct	mult_slide	*bp;

	if(alc->debug)
		fprintf(debug_channel,"findrec: in\n");
	bp = alc->curbay;
	if(bp->startptr == NULL) {
		fprintf(output, "No list in.\n");
		return(0);
	}
	if(nn > bp->totin || nn < 1)		/* Check request */
		return(0);
	else 								/* -1 because list starts at 0 */
		bp->currptr = bp->startptr + nn - 1;
		
	bp->nameids.cellno = bp->currptr->mlrec->recnr;
	if(alc->debug)
		fprintf(debug_channel,"findrec: out\n");
	return(1);
	
}			
/*
 *	F I N D R A N K
 *
 *		Find rank defined by arg nn.  Returns 1 if successful, 0 
 *		if no list in or request outside bounds of list.
 *		
 */
findrank(nn)
int	nn;
{
int	i, rleft;
struct	mult_slide	*bp;

	bp = alc->curbay;
	if(bp->startptr == NULL) {
		fprintf(output, "No list in.\n");
		return(0);
	}
	if(nn > bp->totin || nn < 1)
		return(0);
	else {
		rleft = nn - bp->currptr->mlrec->qrank;
		if(rleft > 0) {
			for(i = 0; i < rleft; i++)
				bp->currptr = bp->currptr->nrank;
		} else {
			rleft = -rleft;
			for(i = 0; i < rleft; i++)
				bp->currptr = bp->currptr->prank;
		}
	}
	bp->nameids.cellno = bp->currptr->mlrec->recnr;
	return(1);
}			
/*
 *	L I N K B L K
 *
 *				Link metlink structs to mlclust structs to make list.
 *				Assign first metlink ptr to alc->curbay->startptr.
 */
linkblk(mptbuf, linkptr, nn)
struct mlclust *mptbuf;
struct metlink *linkptr;
int	nn;
{
int	i;
register struct metlink *lptr;
register struct mlclust *mpt, *optr;

	if(alc->debug)
		fprintf(debug_channel,"linkblk: in\n");

	lptr = linkptr;
	mpt = mptbuf;
	for(i = 0; i < nn; i++) {
		mpt->recnr = i+1;					/* Set records */
		lptr->mlrec = mpt;					/* Link records */
		lptr++;								/* Next link */
		mpt++;								/* Next record */
	}
											/* Set first and current pointers
											   to beginning of list */
	alc->curbay->startptr = alc->curbay->currptr = linkptr;
	
	if(alc->debug == 3) {
		optr = alc->curbay->startptr->mlrec;
		fprintf(debug_channel,"linkblk linked following records\n");
		for(i = 0; i < nn; i++) {
			fprintf(debug_channel,"Rec %d xpos %d ypos %d rank %d\n", 
					optr->recnr, optr->xval, optr->yval, optr->qrank);
			optr++;
		}
	}
	if(alc->debug)
		fprintf(debug_channel,"linkblk: out\n");
}
/*
 *	L I N K R A N K S
 *
 *				Link list in rank order according to the order given
 *				by array pointed by rtb.
 */
linkranks(rtb)
int	*rtb;
{
int	i;
register struct metlink *lptr, *pptr, *oldptr;

	if(alc->debug)
		fprintf(debug_channel,"linkrank: in\n");

	lptr = alc->curbay->startptr;				/* Get first link */
												/* Assign frankptr
												   to first rank */
	alc->curbay->frankptr = lptr + *rtb;
	pptr = alc->curbay->frankptr;				/* Keep this pointer */
	pptr->prank = NULL;							/* NULL prank since
												   beginning of list */
	for(i = 1; i <= alc->curbay->totin; i++) {	/* Link the rest */
		(lptr + *rtb)->prank = pptr;
		pptr->nrank = (lptr + *rtb);
		pptr = lptr + *(rtb++);
	}
	pptr->nrank = NULL;							/* NULL nrank since
												   end of list */
	if(alc->debug == 3) {
		oldptr = alc->curbay->frankptr;
		fprintf(debug_channel,"linkrank linked following records\n");
		do {
			fprintf(debug_channel,"Rec %d rank %d\n", oldptr->mlrec->recnr,
									                  oldptr->mlrec->qrank);
		} while((oldptr = oldptr->nrank) != NULL);
	}
	if(alc->debug)
		fprintf(debug_channel,"linkrank: out\n");
}

/*
 *	L I N K S U B
 *
 *		Link all records in list belonging to the current subset. 
 *		Return 1 if successful otherwise return 0.
 *		
 */
linksub()
{
register struct	mult_slide *bp;
register int ranklow, rankhigh, count, i;
int nsects, listlow, listhigh, val, empty;
int comparison();
struct	metlink *oldptr, *hold_ptr, **array;

	if(alc->debug)
		fprintf(debug_channel,"linksub: in\n");
	bp = alc->curbay;
	if(bp->startptr == NULL) {
		fprintf(output, "No list in.\n");
		return(0);
	}
	bp->fsubptr = NULL;							/* For link to check first 
												   time */
	if(bp->byg) {								/* By groups (inrank on) */
		array = (struct metlink **) Calloc(bp->ngrps, sizeof(struct metlink *));
		bp->currptr = bp->frankptr;
		count = 0;
		empty = 1;
		ranklow = 1;
		rankhigh = bp->maxrank;
		while (bp->currptr != NULL) {
			while (count < bp->ngrps) {
				if(chekattr(bp, bp->currptr->mlrec, ranklow, rankhigh)) {
					array[count] = bp->currptr;
					count++;
					empty = 0;
				}
				if ((bp->currptr = bp->currptr->nrank) == NULL)
					break;		/* end of list */
			}
			if (!empty) {
				hold_ptr = bp->currptr;
				if (bp->ngrps > 1)
					qsort(array, count, sizeof(struct metlink *), comparison);
				for (i=0; i < count; i++) {
					bp->currptr = array[i];
					link(1);
				}
				bp->currptr = hold_ptr;
				count = 0;
			}
			empty = 1;
		}
		Free(array);
	} else {									/* Not by groups subset */
		bp->currptr = bp->startptr + bp->listlow - 1;
		for(i = bp->listlow - 1; i < bp->listhigh; i++) {
			if(chekattr(bp, bp->currptr->mlrec, bp->minrank, bp->maxrank))  
				link(1);
			bp->currptr++;
		}
	}
	bp->currptr = bp->csubptr = bp->fsubptr;	/* Point to first sub-rec */
	if (bp->currptr == NULL)
		bp->currptr = bp->startptr;
	if (alc->debug == 3) {
		fprintf(debug_channel,"linksub linked following records\n");
		if((oldptr = bp->fsubptr) != NULL) {
			do {
				fprintf(debug_channel,"%d\n", oldptr->mlrec->recnr);
			} while((oldptr = oldptr->nsub) != NULL);
		}
	}
	if(alc->debug)
		fprintf(debug_channel,"linksub: out\n");
	return(1);
}
/*
 * C O M P A R I S O N -- Compare record numbers
 *
 */
int comparison(rec1, rec2)
struct metlink **rec1, **rec2;
{
	if ((*rec1)->mlrec->recnr < (*rec2)->mlrec->recnr)
		return(-1);
	else if ((*rec1)->mlrec->recnr == (*rec2)->mlrec->recnr)
		return(0);
	else
		return(1);
}
/*
 *	L I N K
 *
 *		Link a record in list belonging to the current subset. 
 *		Return 1 if successful otherwise return 0.
 *		
 */
link(mode)
int	mode;
{
struct	mult_slide *bp;

	bp = alc->curbay;
	if(mode == 1) {						/* Link by subset */
	
		if(bp->fsubptr == NULL) {		/* First record */
			bp->fsubptr = bp->csubptr = bp->currptr;
										/* Previous ptr of first rec NULL */
			bp->currptr->psub = bp->currptr->nsub = NULL;	
		} else {						/* Subsequent records */
			bp->currptr->psub = bp->csubptr;
			bp->currptr->nsub = NULL;
			bp->csubptr->nsub = bp->currptr;
			bp->csubptr = bp->currptr;
		}
	} else if(mode == 2) {				/* Link by group */
		if(bp->fgrpptr == NULL) {
			bp->fgrpptr = bp->cgrpptr = bp->currptr;
			bp->currptr->pgrp = NULL;
		} else {
			bp->currptr->pgrp = bp->cgrpptr;
			bp->cgrpptr->ngrp = bp->currptr;
			bp->cgrpptr = bp->currptr;
		}
	}
	return(1);
}

/*
 *	R E L I N K
 *
 *		Relink a record in list according to the following rules: 
 *			a.  If type == INCLUDED and record does not belong to
 *			    current subset unlink it from current subset.
 *			b.  If type == EXCLUDED and record belongs to current
 *			    subset link it in the current subset BEHIND the
 *				record pointed to by csubptr.
 *		
 */
relink(type, where)
int	type, where;
{
struct	mult_slide *bp;

	bp = alc->curbay;
	if(alc->debug)
		fprintf(debug_channel,"relink: in\n");
		
	if(bp->startptr == NULL)  return(0);
	if(type == 1) {								/* Was included in subset */
												/* Excluded from subset */
		if(!chekattr(bp, bp->currptr->mlrec, bp->minrank, bp->maxrank)) {
		
			if(bp->currptr->psub != NULL)		/* Not beginning of list */
				bp->currptr->psub->nsub = bp->currptr->nsub;
			else								/* Beginning of list */
				bp->fsubptr = bp->currptr->nsub;
				
			if(bp->currptr->nsub != NULL)		/* Check not end of list */
				bp->currptr->nsub->psub = bp->currptr->psub;
			
			if(bp->currptr->nsub != NULL) {		/* Reset csubptr to next
												   subset record if there */
				if(bp->csubptr == bp->currptr)	/* Reset only when csubptr
												   was pointing there */
					bp->csubptr = bp->currptr->nsub;
				
			} else if(bp->currptr->psub != NULL) {/* else reset to previous */
			
				if(bp->csubptr == bp->currptr)	/* Reset only when csubptr
												   was pointing there */
					bp->csubptr = bp->currptr->psub;
				
			} else								/* No recs in subset */
				bp->fsubptr = bp->csubptr = NULL;
				
												/* Set ptrs of unlinked rec
												   to NULL */
			bp->currptr->nsub = bp->currptr->psub = NULL;
		}
	} else if(type == 0) {						/* Was excluded from subset
												   to be included in subset */
		if(chekattr(bp, bp->currptr->mlrec, bp->minrank, bp->maxrank)) {
												/* Set ptrs of new record */
			if(bp->csubptr != NULL) {
				if(where == BACKW) {			/* link behind currptr */
					bp->currptr->nsub = bp->csubptr;
					bp->currptr->psub = bp->csubptr->psub;
												
												/* Not beginning of list */
					if(bp->csubptr->psub != NULL)		
						bp->csubptr->psub->nsub = bp->currptr;
					else						/* Beginning of list */
						bp->fsubptr = bp->currptr;
					bp->csubptr->psub = bp->currptr;
					
				} else {						/* Link infront of currptr */
					bp->currptr->psub = bp->csubptr;
					bp->currptr->nsub = bp->csubptr->nsub;
												
												/* Not end of list */
					if(bp->csubptr->nsub != NULL)		
						bp->csubptr->nsub->psub = bp->currptr;
					bp->csubptr->nsub = bp->currptr;
				}
			} else {							/* First rec in subset */
				bp->csubptr = bp->fsubptr = bp->currptr;
				bp->csubptr->nsub = bp->csubptr->psub = NULL;
			}
		}
	}
	if(alc->debug)
		fprintf(debug_channel,"relink: out\n");
	return(1);
}		
/*
 *	M E T H T O S T R U C T 
 *
 *				Copy variables from metaphase list header to structure.
 *		
 */
methtostruct(met_h, bp)
struct	met_header *met_h;
struct	mult_slide *bp;
{
	struct	met_header_1 *omet_h;
	int i, j, rval=0;
	
	switch (met_h->version) {
		case 1:
			omet_h = (struct met_header_1 *) met_h;
			bp->totin = omet_h->nrecs;
			bp->compos = bp->combot + omet_h->comments;
			bp->typeids.ktistype = omet_h->tissue;
			bp->typeids.kfstype = omet_h->stain;
			strncpy(bp->nameids.case,omet_h->patid,CASELENGTH);
			strcpy(bp->nameids.patnam,'\0');
			strncpy(bp->nameids.slide,omet_h->slideid,SLIDELENGTH);
			break;
		case 2:
		case 3:
			bp->totin = met_h->nrecs;
			bp->slwidth = met_h->slwidth;
			bp->compos = bp->combot + met_h->comments;
			bp->typeids.ktistype = met_h->tissue;
			bp->typeids.kfstype = met_h->stain;
			strncpy(bp->nameids.case,met_h->patid,CASELENGTH);
			strncpy(bp->nameids.patnam,met_h->patnam,PATLENGTH);
			strncpy(bp->nameids.slide,met_h->slideid,SLIDELENGTH);
										/* Box limits */
			bp->mbox.bbot = met_h->bbot;
			bp->mbox.btop = met_h->btop;
			bp->mbox.bleft = met_h->bleft;
			bp->mbox.bright = met_h->bright;
										/* Last subset */
			if(met_h->lastsub[0] == 'I')  strcpy(bp->wh.classd,"Include");
			else if(met_h->lastsub[0] == 'E')  strcpy(bp->wh.classd,"Exclude");
			else *(bp->wh.classd) = '\0';
		
			if(met_h->lastsub[1] == 'I')  strcpy(bp->wh.juu,"Include");
			else if(met_h->lastsub[1] == 'E')  strcpy(bp->wh.juu,"Exclude");
			else *(bp->wh.juu) = '\0';
			
			if(met_h->lastsub[2] == 'I')  strcpy(bp->wh.uubl,"Include");
			else if(met_h->lastsub[2] == 'E')  strcpy(bp->wh.uubl,"Exclude");
			else *(bp->wh.uubl) = '\0';
			
			if(met_h->lastsub[3] == 'I')  strcpy(bp->wh.unum,"Include");
			else if(met_h->lastsub[3] == 'E')  strcpy(bp->wh.unum,"Exclude");
			else  *(bp->wh.unum) = '\0';
		
			if(met_h->lastsub[4] == 'I')  strcpy(bp->whb.dicc,"Include");
			else if(met_h->lastsub[4] == 'E')  strcpy(bp->whb.dicc,"Exclude");
			else  *(bp->whb.dicc) = '\0';
		
			if(met_h->lastsub[5] == 'I')  strcpy(bp->whb.fragg,"Include");
			else if(met_h->lastsub[5] == 'E')  strcpy(bp->whb.fragg,"Exclude");
			else  *(bp->whb.fragg) = '\0';
		
			if(met_h->lastsub[6] == 'I')  strcpy(bp->whb.ringg,"Include");
			else if(met_h->lastsub[6] == 'E')  strcpy(bp->whb.ringg,"Exclude");
			else  *(bp->whb.ringg) = '\0';
		
			if(met_h->lastsub[7] == 'I')  strcpy(bp->whb.inbx,"Include");
			else if(met_h->lastsub[7] == 'E')  strcpy(bp->whb.inbx,"Exclude");
			else  *(bp->whb.inbx) = '\0';
	
			bp->allrecs = (int) met_h->allrecs;
			bp->listlow = (int) met_h->listlow;
			bp->listhigh = (int) met_h->listhigh;
			bp->minrank = (int) met_h->minrank;
			bp->maxrank = (int) met_h->maxrank;
			bp->sany = (int) met_h->sany;
			bp->ngrps = (int) met_h->ngrps;
			bp->byg = (int) met_h->byg;
			bp->retrain = met_h->retrain;
			break;
		default:
			fprintf(output,"Header version unknown.\n");
			rval = -1;
			break;
	}
	return(rval);
}


