/*
 *		M E T L I S T . C  -- list filing and printing operations, and 
 *								control menu for branching to "listops" 
 *
 *   author: Denis R.		Mon Jun 24 08:37:48 GMT 1985
 *
 *	Modifications
 *
 *	16 Jul 1991		CAS			Moved menus to mf_menu.c
 *	24 Jun 1991		CAS			Prior stage handling
 *	22 Feb 1991		CAS			Chuck a formfeed out at the end
 *	
 */

#define PRPRIOR 127				/* Priority for 'lwcat' */
#define FORMFEED '\014'			/* ASCII for form feed */
#define PAGELENGTH 66			/* Lines / printer page */


#include <cmdsys.h>
#include <fbio.h>
#include <kyy.h>
#include <slides.h>
#include <spool.h>
#include <metscan.h>

extern int errno;
extern FILE *debug_channel;

static int	totout,			/* Count of records written to file */
			linesout,		/* Line count for list pagination */
			dblsp = 1;		/* Double line spacing on output, default ON */

			
static char	classnam[10];
static char *heading[] = {
	"Nmb CytoNmb    X      Y\n------------------------- \n",
	"Nmb CytoNmb   MicX   MicY\n-------------------------\n",
	"Nmb CytoNmb   E.F.(Graticules Ltd.)\n----------------------------------- \n",
	"Nmb CytoNmb   E.F.(Albert Sass)\n------------------------------- \n"
};
static char *longhead[] = {
	"Nmb X-cord Y-cord Xwd Ywd   Z  No Sf Area  Ar8  Sk Vr Rank Dns Class\n\
--- ------ ------ --- ---   -  -- -- ----  ---  -- -- ---- --- -----\n",
	"Nmb  MicX    MicY   Xwd Ywd  Z   No Sf Area Ar8  Sk Vr Rank Dns Class\n\
---  ----    ----   --- ---  -   -- -- ---- ---  -- -- ---- --- -----\n", 
	"Nmb  E.F.(Grat.Ltd)  Xwd Ywd   Z  No  Sf Area Ar8 Sk Vr Rank Dns Class\n\
---  --------------  --- ---   -  --  -- ---- --- -- -- ---- --- -----\n",
	"Nmb  E.F.(A.Sass)    Xwd Ywd   Z  No  Sf Area Ar8 Sk Vr Rank Dns Class\n\
---  ------------    --- ---   -  --  -- ---- --- -- -- ---- --- -----\n"
};
static char *toxhead[] = {
	"Nmb X-cord Y-cord Dic   Acen   Comments\n--- ------ ------ ----- ----   --------\n",
	"Nmb  MicX   MicY  Dic   Acen   Comments\n---  ----   ----  ---   ----   --------\n",
	"Nmb  English NE   Dic   Acen   Comments\n---  ----------   ---   ----   --------\n",
	"Nmb  Swedish NE   Dic   Acen   Comments\n---  ----------   ---   ----   --------\n"
};
static char *nontoxhead[] = {
	"Nmb X-cord Y-cord              Comments\n--- ------ ------              --------\n",
	"Nmb  MicX   MicY               Comments\n---  ----   ----               --------\n",
	"Nmb  E.F.(Graticules Ltd.)     Comments\n---  --------------------      --------\n",
	"Nmb  E.F.(Albert Sass)         Comments\n---  -----------------         --------\n"
};

static char Errstr[] = "Error reading ";
extern char *Malloc(), *gettissue(), *getstain();
extern struct object *makemain(), *readobj();
extern struct met_header *metheadfill();
int		getlist(), printone(), askfile();
static	printentry(), classcode(), scoreentry();
int	frank(), subset(), init_offset(), rank_lims_set(), 
		displist(), setfx();

/*
 * C A L L _ S U B S E T --
 *
 */
call_subset()
{
	subset();
	displist();
}

/*
 * M O V E _ T O _ B A Y --
 *
 */
move_to_bay(bayno)
int *bayno;
{
	if (bay_move(*bayno)) {
		mksubtxt();
		clearlns(6, 8);				/* Clear subset window */
		displin(0, 1);				/* Update subset window */
		displin(0, 0);				/* Update header */
		showbays();					/* Highlight new bay */
		displist();					/* Show new list subset crosses */
	}
	*bayno = baynn(alc->curbay);
}

/*
 * S E T _ C U R B A Y -- set curbay for list filing, reading, ...
 *
 */
set_curbay(num)
int *num;
{
	if ((*num < 0) || (*num > alc->maxbays)) {
		fprintf(stderr, "%d is not a valid bay number.\n", *num);
		*num = baynn(alc->curbay);
	} else {
		alc->curbay = alc->frsbay + (*num - 1);
	}
}
/*
 * R E A D _ N E W L I S T --
 *
 */
read_newlist()
{
	getlist();						/* Call to getlist */
	mksubtxt();						/* Make new sub text */
	alc->curbay->active = ON;
	showbays();						/* Redraw new bay */
	displist();						/* Show new list subset crosses */
}

/*
 *	F I L E L I S T  --
 *
 *	Make name and file whole list.
 */
filelist()
{
int mm, nc, iw, zero = 0;
FILE	*fq, *fbdopen();

	if (! alc->curbay->startptr) {
		fprintf(output, "No list in!\007\n");
		return;
	}
	alc->curbay->nameids.type = METFILE;
	makefilename(alc->metfile, &(alc->curbay->nameids));

/*
 *	Check file id
 */
	while(1) {
		if (!getid(METFILE,WRITE))
			return(0);
		if (alc->debug)
			fprintf(debug_channel,"Write metaphase file %s\n",alc->metfile);
		if ((fq = fbdopen(alc->metfile,alc->dir,"w",BUFSIZ)) != NULL)
			break;
		if (alc->debug)
			fprintf(output,"\ncan't open %s error %d\007\n",alc->metfile,errno);
		else
			fileerror("Failed to open metaphase list file",errno);
	}
	writelist(fq);
	fbclose(fq);
}

/*
 *	G E T L I S T  --  Read in a metaphase list
 *
 */
getlist()
{
int		nn, requd, dispgone=0;
FILE	*fq, *fbdopen();

	if (alc->curbay->startptr) {		/* If a list exists remove it. */
		fprintf(output, "Read in new list. Abandon previous one ? \n");
		if(yesno(1)) 				/* Confirm the removal of old list. */
			return(0);
	}
	alc->curbay->nameids.type = METFILE;
	makefilename(alc->metfile, &(alc->curbay->nameids));
	while(1) {
		if (!getid(METFILE,READ))
			return(0);
		if (alc->debug)
			fprintf(debug_channel,"Read metaphase file %s\n",alc->metfile);
		if ((fq = fbdopen(alc->metfile,alc->dir,"r",BUFSIZ)) != NULL)
			break;
		fileerror("Failed to open %s\n",errno,alc->metfile);
	}
	freelist();
	if(!readlist(fq)) {
		fbclose(fq);
		return(0);
	} else {
		fprintf(output,"Reordering list\n");
		linksub();		/* Link as dictated by subset */
		init_offset();	/* Set capture offsets to zero */
		init_scores(alc->curbay, 1);
		init_counts(alc->curbay, 1);
		alc->curbay->newlist = 1;	/* This is a new list */
		fbclose(fq);
		return(1);
	}
}

/*
 *	S H O W L I S T
 *
 */
showlist()
{
	int j,k, count;
	struct	metlink *oldcpt, *oldspt;
									/* If no list or no subset return */
	if (! alc->curbay->startptr) {
		fprintf(output, "No list in!\007\n");
		return(0);
	}
	if (!alc->curbay->fsubptr) {
		fprintf(output,"Empty subset!\007\n");
		return(0);
	}
	k=20;
	count = 0;
	ourstyle = outstyle ? outstyle : 2;
	linesout=2;
	scrollpart(0,ENDLINE);
	vclear(ENDLINE);
	screengo(0,0);
	fprintf(output, "Case  %s   Scorer %s\n\n",
					alc->curbay->nameids.case, alc->kscorer);
	fprintf ( output, ourstyle==1 ? longhead[alc->ccoord]: heading[alc->ccoord]);
	oldspt = alc->curbay->csubptr;
	oldcpt = alc->curbay->currptr;
	alc->curbay->csubptr = alc->curbay->fsubptr;
	do {
		count++;
		printentry(alc->curbay->csubptr->mlrec, output, count);
		if (++linesout >=k) {
			if (! go_on()) goto Fixscroll ;
			scrollpart(0,ENDLINE);
			vclear(ENDLINE);
			screengo(0,0);
			linesout=0;
		}
	} while(nextrec(1));
	go_on();

Fixscroll:
	alc->curbay->csubptr = oldspt;
	alc->curbay->currptr = oldcpt;
	alc->curbay->nameids.cellno = alc->curbay->currptr->mlrec->recnr;
	scrollpart(OUTLINE,ENDLINE);
	vclear(ENDLINE);
	display();
	return(0);
}

/*
 *	P R I N T L I S T
 *
 */
printlist(dblsp)
int		*dblsp;
{
	int j,lastline,page, count; 
	FILE *outfile; SFILE *sfp;
	struct	metlink *oldcpt, *oldspt;
	struct	mult_slide *bp;
	char	tmpbuf[40];
	
	bp = alc->curbay;
									/* If no list or no subset return */
	if (!bp->startptr) {
		fprintf(output, "No list in!\007\n");
		return(0);
	}
	if (!bp->fsubptr) {
		fprintf(output,"Empty subset!\007\n");
		return(0);
	}
	ourstyle = outstyle ? outstyle : 2;
	if ((sfp = spoolopen()) == NULL) {
		fprintf(stderr,"Unable to open print file\n");
		return(1);
	}
	outfile = sfp->fp;
	lastline = PAGELENGTH - 8; /* give 50 lines at single space ih*/
	linesout = 0;
	page = 0;
	oldspt = bp->csubptr;
	oldcpt = bp->currptr;
	bp->csubptr = bp->fsubptr;
	count = 0; /* Init here, so Nmb is continuously incrementing */
	do {
		if (linesout == 0) {
			fprintf(outfile, "%-40s%36s %3d\n",alc->mydate,"Page",++page);
			fprintf(outfile, "\016         Cytoscan Metaphase List\024\n\n");
			sprintf(tmpbuf,"Case: %s  %s",bp->nameids.case,bp->nameids.slide);
			if (alc->ccoord)
				fprintf(outfile, "%-40s%25s %s\n",tmpbuf,"Scope:      ",scopename());
			else
				fprintf(outfile, "%-40s\n",tmpbuf);
				fprintf(outfile, "Name: %-40s%19s %s\n",bp->nameids.patnam,"Operator:   ",
																alc->kscorer);
			sprintf(tmpbuf, "Prep: %s : %s",
				gettissue(bp->typeids.ktistype), getstain(bp->typeids.kfstype));
			if (alc->ccoord)
				fprintf(outfile, "%-40s%25s %d.%d\n\n",tmpbuf,"Slide width:",
							alc->curbay->slwidth/1000, alc->curbay->slwidth%1000);
			else
				fprintf(outfile, "%-40s\n\n", tmpbuf);
			fprintf(outfile, ourstyle==1 ? longhead[alc->ccoord]: heading[alc->ccoord]);
			linesout = 9;
		}
		count++;
		printentry(bp->csubptr->mlrec, outfile, count);
		if (dblsp) {
			fprintf(outfile, "\n");
			linesout++;
		}
		if (++linesout >= lastline) {
			putc(FORMFEED,outfile);
			linesout=0;
		}
	} while(nextrec(1));
	bp->csubptr = oldspt;
	bp->currptr = oldcpt;
	bp->nameids.cellno = bp->currptr->mlrec->recnr;
	if (linesout) putc(FORMFEED,outfile);
	spoolclose(sfp);
	return(0);
}

/*
 *	D I S P L I S T
 *
 */
displist()
{
	register int ix, iy, tot, conversion;
	int xk1, yk1; 
	char str[5];
	struct	metlink *oldcpt, *oldspt;
   	register struct	mlclust *mpt;

	simslide();								/* Erase old crosses */
	dwb(2, &alc->curbay->mbox, 1, 1);		/* Draw box */
	tot = 0;
									/* If no list or no subset return */
	if (alc->curbay->startptr && alc->curbay->fsubptr) {
		oldspt = alc->curbay->csubptr;
		oldcpt = alc->curbay->currptr;
		alc->curbay->csubptr = alc->curbay->fsubptr;
		colour(1);
		conversion = CONV * 256;
		do {
			mpt = alc->curbay->csubptr->mlrec;
			ix = ((mpt->xval<<8) / conversion) + XMID;
			iy = YMID - ((mpt->yval<<8) / conversion);
			moveto(ix, iy);
			cross(8);
			tot++;
		} while(nextrec(1));
		alc->curbay->csubptr = oldspt;
		alc->curbay->currptr = oldcpt;
	}
	xk1 = SLMXTOP;					/* Display subset total box */
	yk1 = SLMYBOT;
	moveto(2000,yk1-340);
	colour (2);
	tsize (64,64);
	text( "Subset total ... ");
	tsize(64,64); 
	moveto (xk1+200,yk1-340);
	colour(255);
	pixbegin(8,1,1,640,200);	/*clear the old value (blue wash)*/
	pixel(4); 
	pixend();
	sprintf (str, "%4d", tot);
	moveto (xk1+300,yk1-315);
	colour(1);
	tsize(128,128);
	text (str);
	tsize(64,64); 
 	return(1);
}

/*
 *	P R I N T O N E  --
 *
 */
printone(mpt,l) 
struct mlclust	*mpt;
int	l;
{
	screengo(l,0);
	fprintf ( output, ourstyle==1 ? longhead[alc->ccoord]: heading[alc->ccoord]);
	printentry (mpt, output, 0);
}

/*
 *	P R I N T E N T R Y --
 *
 */
static
printentry(mpt,outfile, count) 
struct mlclust *mpt;
FILE *outfile;
int count;
{
	int		micxc, micyc;
	char string[8], fxval[8], fyval[8];


	if (alc->ccoord)
		convert(mpt->xval, mpt->yval, fxval, fyval, string);
	
	if (ourstyle == 2) { /* short listing */
		switch (alc->ccoord) {
			case CONVOFF:
			    fprintf( outfile,"%3d  %3d    %6d %6d\n", count, mpt->recnr, mpt->xval, mpt->yval);
				break;
			case MICROSCOPE :
			    fprintf( outfile,"%3d  %3d  %8s %8s\n", count, mpt->recnr, fxval, fyval);
				break;
			case ENGLISHNE :
			case SWEDISHNE :
			    fprintf( outfile,"%3d  %3d          %s\n", count, mpt->recnr, string);
				break;
		} /* end of switch */
	} else {  /* long listing */
		classcode(mpt->fclass);			/* Call classcode to replace code with class string. */
		switch (alc->ccoord) {
			case CONVOFF :
			    fprintf( outfile,"%3d %6d %6d %3d %3d %4d %2d %3d %3d %4d",
				    	mpt->recnr,mpt->xval, mpt->yval, mpt->xwide, mpt->ywide,
		    			mpt->zval, mpt->nobj, mpt->sf, mpt->aobj, mpt->octa);
		    	break;
		    case MICROSCOPE :
			    fprintf(outfile,"%3d%8s%8s %3d %3d %4d %2d %3d %3d %4d",
				    	mpt->recnr, fxval, fyval, mpt->xwide, mpt->ywide,mpt->zval,
		    			mpt->nobj, mpt->sf,mpt->aobj,mpt->octa);
		    	break;
		    case ENGLISHNE :
		    case SWEDISHNE :
			    fprintf(outfile,"%3d      %5s       %3d %3d %4d %2d %3d %3d %4d",
				    	mpt->recnr, string, mpt->xwide, mpt->ywide,mpt->zval,
		    			mpt->nobj, mpt->sf,mpt->aobj,mpt->octa);
		    	break;
		} /* end of switch */
		fprintf(outfile,"  %2d %2d %4d %2d %s\n",mpt->skt,mpt->dv,
			mpt->qrank,mpt->meanod, classnam);
	}
}

/*
 *	S C O R E S H E E T
 *
 */
scoresheet()
{
	int i,j,lastline,page, sctot=0, frtot=0, prsc, prfr; 
	FILE *outfile; SFILE *sfp;
	struct	mlclust *fspt;
	struct	mult_slide *bp;
	
	bp = alc->curbay;
									/* If no list or no subset return */
	if (!bp->startptr) {
		fprintf(output, "No list in!\007\n");
		return(0);
	}
	if ((sfp = spoolopen()) == NULL) {
		fprintf(stderr,"Unable to open print file\n");
		return(1);
	}
	outfile = sfp->fp;
	lastline = PAGELENGTH - 8;	/* 25 entries per page on double space ih */
	linesout = 0;
	page = 0;
	fspt = bp->startptr->mlrec;
	for (i = 0; i < bp->totin; i++) {
		if (linesout == 0) {
			fprintf(outfile, "\nCase:  %-16s Slide:  %-4s Scorer: %s  %s Page %d\n",
				bp->nameids.case, bp->nameids.slide, alc->kscorer, 
				alc->mydate,++page);
			fprintf(outfile, "Patient:  %-39s\n", bp->nameids.patnam);
			fprintf(outfile, "Tissue:  %-15s  Stain:  %-15s\n\n", 
				gettissue(bp->typeids.ktistype), getstain(bp->typeids.kfstype));
			fprintf(outfile, alc->toxicmu ? toxhead[alc->ccoord] : nontoxhead[alc->ccoord]);
			linesout = 7;
		}
		if ((fspt->com != NULL) || (fspt->udic != -1) || (fspt->uring != -1) ||
				(fspt->ufrag != '\0'))
			scoreentry(fspt, outfile);

		fspt++;
		if (linesout >= lastline) {
			putc(FORMFEED,outfile);
			linesout=0;
		}
	} 
	sctot = bp->scp->fxyes + bp->scp->fxno;
	if (!sctot)
		prfr = 0;
	else
		prfr = (100 *  bp->scp->fxyes) / sctot;
		
											/* Statistics summary */
	if (linesout) putc(FORMFEED,outfile);
	fprintf(outfile, "\nCase:  %-16s Slide:  %-4s Scorer: %s  %s Page %d\n",
		bp->nameids.case, bp->nameids.slide,alc->kscorer,alc->mydate,++page);
	fprintf(outfile, "Patient:  %-39s\n", bp->nameids.patnam);
	fprintf(outfile, "Tissue:  %-15s  Stain:  %-15s\n\n", 
		gettissue(bp->typeids.ktistype), getstain(bp->typeids.kfstype));
	fprintf(outfile, "\n\nSTATISTICS SUMMARY\n\n");
	fprintf(outfile, "Total scored   : %4d\n", sctot);
	fprintf(outfile, "Total fragile x: %4d    Percentage over total scored :%d\n",
					bp->scp->fxyes, prfr);
	putc(FORMFEED,outfile);
	spoolclose(sfp);
	return(0);
}

/*
 *	S C O R E E N T R Y --
 *
 */
static
scoreentry(mpt,outfile) 
struct mlclust *mpt; 
FILE *outfile;
{
	int 	i, nc1, nc2, nc3, micxc, micyc;
	char	comstring[2][40], scecom[40], chromcom[40], 
			fstr[9], dnum[6], anum[6], string[8], fxval[8], fyval[8];
	struct	mult_slide *bp;

	bp = alc->curbay;

	if (alc->ccoord > 0)
		convert(mpt->xval, mpt->yval, fxval, fyval, string);

	if(mpt->uring != -1)					/* dicentrics string */
		sprintf(dnum,"%3d", (int)mpt->uring);
	else
		*dnum = '\0';
		
	if(mpt->udic != -1)						/* acentrics string */
		sprintf(anum,"%3d", (int)mpt->udic);
	else
		*dnum = '\0';
		
	if (alc->toxicmu) {		/* toxheader */
		switch (alc->ccoord) {
			case CONVOFF :	
			    fprintf( outfile,"%3d %6d %6d %3s %3s\n", mpt->recnr, mpt->xval,
												    	mpt->yval, dnum, anum);
			case MICROSCOPE :
			    fprintf( outfile,"%3d %8s %8s %3s %3s\n ", mpt->recnr, fxval,
													    	fyval, dnum, anum);
				break;
			case ENGLISHNE :
			case SWEDISHNE :
			    fprintf( outfile,"%3d   %s  %3s %3s\n", mpt->recnr, string, dnum, anum);
			    break;
		} /* end if switch */
	} else {
		switch (alc->ccoord) {
			case CONVOFF :
			    fprintf( outfile,"%3d %6d %6d\n ", mpt->recnr, mpt->xval, mpt->yval);
				break;
			case MICROSCOPE :
			    fprintf( outfile,"%3d %8s %8s\n ", mpt->recnr, fxval, fyval);
			    break;
			case ENGLISHNE :
			case SWEDISHNE :
			    fprintf( outfile,"%3d   %s\n ", mpt->recnr, string);
			    break;
		} /* end of switch */
	} /* end of if */
	linesout++;
	if (mpt->com)  {
		strcpy(comstring[0], bp->combot+mpt->com); 
		if(*comstring[0]) {
			fprintf(outfile,"                             Comment  : %s\n", 
															comstring[0]);
			++linesout;
		}
		nc1 = strlen(comstring[0]);
		strcpy(comstring[1], bp->combot+mpt->com + nc1 + 1); 
		if (*comstring[1]) {
			fprintf(outfile,"                             Comment  : %s\n", 
															comstring[1]);
			++linesout;
		}
		nc2 = strlen(comstring[1]);
		strcpy(scecom, bp->combot+mpt->com + nc1 + nc2 + 2); 
		if (*scecom) {
			fprintf(outfile,"                             SCE      : %s\n", 
															scecom);
			++linesout;
		}
		nc3 = strlen(scecom);
		strcpy(chromcom, bp->combot+mpt->com + nc1 + nc2 + nc3 + 3); 
		if (*chromcom) {
			fprintf(outfile,"                             Karyotype: %s\n", 
															chromcom);
			++linesout;
		}
	}
	if(mpt->ufrag) {
		setfx(mpt, fstr);
		fprintf(outfile,"                             Fragile x: %s\n", 
														fstr);
		++linesout;
	}
}

/*
 *	F I X F O R M  -- Set forstr
 *
 */
fixform(forstr)
char	*forstr;
{
	if (outstyle == 1) outstyle = 2;	/* Just switch between short/long */
	else outstyle = 1;
	switch (outstyle) {
		case 0 : strcpy(forstr,"default"); break;
		case 1 : strcpy(forstr,"long"); break;
		case 2 : strcpy(forstr,"short");
	}
}

/*
 *	S E T S P A C I N G  --  Set print line spacing
 */
setspacing(space)
int		*space;
{
	printf("spacing %d\n",*space);
	dblsp = *space;
}

/*
 *  A S K F I L E  --  See if we want to file list and if we do - file it
 *
 */
askfile(wherefrom)
int wherefrom;
{
	struct mult_slide *bp;
	
	bp = alc->curbay;
		
	screengo(ENDLINE,0);
	fprintf (output, "File list ? \n"); if ( yesno(1)) return;

	/*
	 * modify the subset if called from select and subset was ALL
	 */
	if(wherefrom == SELECT) {
		if(bp->startptr != NULL) {
			if(bp->byg == 1) {
				strcpy(bp->wh.uubl, "Include");
				mksubtxt();
				linksub();
				displist();
			}
		}
	}
	filelist();
}
/*
 *
 *	C L A S S C O D E ---
 *
 *				This routine interprets a bit code for a string whose pointer
 *				is a static.
 */
static
classcode(cde)
char	cde;
{

	if(cde & USABLE)
		sprintf(classnam, "Usable");
	else if(cde & JUNK)
		sprintf(classnam, "Junk");
	else if(cde & UNUMET)
		sprintf(classnam, "Unumet");
	else
		sprintf(classnam, "Unclass");
	return;
	
}

/*
 *
 *  I N I T _ C O U N T S
 *
 *				Counts the number of usables in a metaphase list and
 *				puts up the initial display.
 */
int
init_counts(bp, how)
struct	mult_slide *bp;
int how;
{
register int i;
register struct mlclust *mpt;

	bp->usbltot = bp->revtot = 0;					/* Reset to zero */			
	
	if(how && bp->startptr != NULL && bp->totin > 0) {
		for(i = 0, mpt = bp->startptr->mlrec; i < bp->totin; i++, mpt++) {
			if(mpt->fclass & USABLE)					/* Count usables */
					bp->usbltot++;
			if(mpt->fclass & (USABLE | JUNK | UNUMET))	/* Count reviewed */
					bp->revtot++;
		}
	}
}
/*
 * I N I T _ S C O R E S --- initialise scores when a new list appears
 *
 */
init_scores(bp, how)
struct mult_slide *bp;
int how;
{
register struct mlclust *mpt;
register struct scores *scp;
register int i;

	scp = bp->scp;
	scp->sctot = scp->fxyes = scp->fxno = scp->fxmaybe = 0;

	/*
	 * if how is set its a previously scored list so look it up
	 */	
	if(how && bp->startptr != NULL && bp->totin > 0)
		for(i = 0, mpt = bp->startptr->mlrec; i < bp->totin;  i++, mpt++)
			modscores('\0', mpt->ufrag, scp);
			
}

/*
 *	F I X R G B -- Set up some pretty colours
 */
fixrgb() {
			setrgb(0,255,255,255);
			setrgb(6,120,120,200);
			setrgb(4,190,190,255);
			setrgb(5,200,0,0); /* this is a darkened red - when written to
								  a "4" background as "1" gives 5 */
			setrgb(1,200,0,0);
			setrgb(2,0,128,0);
/*			setrgb(8,0,0,255);	*/
			setrgb(255,0,0,0);
}
