/*
 *		M F . C   --   
 *
 *  Written:
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 * Modifications
 *
 *  19 Apr 1991		ih			Return after certain (harmless) exceptions
 *	15 Jan 1991		CAs			Init debug_channel to stderr if debug off in case
 *								some nurd uses it anyway
 *	12 Dec 1989		dcb			Put vsn no. in exception proc. from version.h
 *	 2 Jun 1989		CAS			Define errno for v2.2
 *	19 May 1989		dcb			capture to arraydigitise
 *	28 Nov 1988		SCG			call woolz_check_level with alc->wclevel
 *	18 Nov 1988		SCG/dcb		Call woolz_check_debug and woolz_check_level
 *								added additional header wtcheck.h
 *	18 Aug 1988		CAS			Mods to config + Call CS capture directly
 *	19 Apr 1988		SCG/SEAJ	Call initbays in main
 *	28 Jan 1988		CAS			Extension of errorlog facility
 *	22 Jan 1988		CAS			New cmdsys
 */

#define INIFILE 1		/* Cmdsys - root section - define storage */

#include <cmdsys.h>
#include <kyy.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <signal.h>
#include <module.h>
#include <procid.h>
#include <wtcheck.h>
#include <version.h>

extern int errno;

/*
 *	System data structures
 */
struct kcontrol *kcont;

/*
 *	Files
 */
FILE *debug_channel = NULL;				/* system debug file */
FILE *debugfp = NULL;					/* Miproc comms Debug filepointer */

int		(*quit_fn)() = NULL;		/* Call this when we get ^c */
extern int woolzexitfn();			/* call this when woolz fatal error */
extern int memexitfn();

digitise();
identify();
interrupt();
findmet();
fipops();
metlist();
slavemenu();
kickslave();
FILE	*smartopen();

extern int dispcellmf();


/*
 *	I N T E R R U P T  --  MF interrupt catcher
 *
 */
interrupt(sig)
int sig;
{
	static int busy = 0;

	if (busy) return;
	busy = 1;
	if (alc->debug) {
		printf("MF - seen sig %d\n",sig);
	}
	switch (sig) {
		case SIGWAKE:
			break;
		case SIGINT:
		case SIGQUIT:
		case KMR_INT:
			if (quit_fn) {
				quit_fn();
			} else {
				if (alc->kmr_pid) {
					kill(alc->kmr_pid,MF_INT);
/*
 *	This sleep will be broken by the next signal we get - Note that signal
 *	will cause us to have 'nested interrupts'
 */
 					busy = 0;
 					tsleep(0);
 					busy = 1;
				}
			}
			break;
		case 9:				/* Kill from alldone in cmdsys */
			if (alc->debug || alc->mipdebug) {
				fclose(debug_channel);
			}
			exit(0);
			break;
		case WOOLZSIG:
		case 102:
		case 101:
			finish(); /*tidy graphics */
			kill(alc->kmr_pid,MF_DIES);		/* Tell kmr... */
			if (alc->debug) {
					fprintf(debug_channel,"MF terminated by signal %d\n",sig);
				fflush(debug_channel);
			}
			exit(1);
			break;
		default:
			printf( "mf: Signal %d received \n",sig);
	}
	busy = 0;
}

/*
 *	E X C E P T I O N  --
 *
 */
exception(exc,addr)
int		exc,addr;
{
	procid	pd;
	int		fd, time, date, day, tick;
	char	buffer[100];

	gprdsc(getpid(),&pd);
	sprintf(buffer,"Exception %d, addr 0x%x, base 0x%x (offset 0x%x)\n",
		exc, addr, pd._pmodul, addr - (int)pd._pmodul);
	errorlog(buffer);
	/* don't shut down kmr for zero div,chk & trapv exceptions */
	switch (exc) {
		case 20:
		case 24:
		case 28:
			setexception(exception);
			return;
		default:
			break;
	}
	interrupt(101);
}

/*
 *	M A I N  --  Mf main loop
 */
main(argc,argv)
int argc; char **argv;
{
	int cc;

	sleep(1) ; /* allow caller to settle down */
	intercept(interrupt);
	setexception(exception);
	if ( argc < 3 ) {
		fprintf(stderr, "Incorrect invocation of mf - no parameter\n");
		exit(1);
	}

/*	mukont = ( struct mukontr *) atoi(argv[1]); */
	alc = (struct allkontr *) atoi(argv[2]);
	kcont = (struct kcontrol *) alc->kcn;
	setmemalloc(32*1024);
	alc->mf_pid = getpid();
	set_woolz_exit(woolzexitfn);
	set_kary_exit(woolzexitfn);
	set_chrom_exit(woolzexitfn);
	Alloc_exit(memexitfn);

/*
 *	Start up debuging
 *
 *	2 different channels - debugfp for miproc
 *						 - debug_channel for system
 *
 */
	if (alc->mipdebug) {
		fprintf(stderr,"About to open %s\n",alc->debugfile);
		if (debugfp = smartopen(alc->debugfile,"w") ) {
			fprintf(debugfp,"M I P  D E B U G   I N F O R M A T I O N\n");
		} else {
			fprintf (stderr,"Failed to open debug file error number %d\n"
				,errno);
		}
		fflush(debugfp);
	}
 	if (alc->debug) {
 		if (debugfp)
 			debug_channel = debugfp;
 		else {
			fprintf(stderr,"About to open %s\n",alc->debugfile);
			if ((debug_channel = smartopen(alc->debugfile,"w")) ){
				fprintf(debug_channel,"G E N E R A L  D E B U G   I N F O R M A T I O N\n");
 			} else {
				fprintf (stderr,"Failed to open debug file error number %d\n"
					,errno);
			}
		}
		fflush(debug_channel);
		Alloc_debug(debug_channel);
		Makestruct_debug(debug_channel);
		freeobj_debug(debug_channel);
		woolz_check_debug(debug_channel);
/*		if (alc->debug > 3) {
			Label_debug(debug_channel);
		} */
	} else
		debug_channel = stderr;
/*
 *	Set up initial combifip parameters
 */
	if (alc->debug)
		Show_alloc(); 	
	woolz_check_level(alc->wclevel);
	/*
	 * initialise bays
	 */
	initbays();
	set_header_fn(dispcellmf);
	if (alc->debug)
		fprintf(debug_channel,"MF: before slavemain argv[1] %s\n",argv[1]);
	slavemain(argv[1],slavemenu,kickslave);
	
}

/*
 *	S M A R T O P E N
 *
 */
static FILE *
smartopen(name,attr)
char	*name,*attr;
{
	char	buffer[60];
	char	*passed_name=name;

	if ((*name++ == '/') && (*name++ =='t') && (*name >='0') &&
			(*name++ <='9') && (*name == '\0')) {
			return(fopen(passed_name,attr));
	} else {
		sprintf(buffer,"%s.mf",passed_name);
		fprintf(stderr,"MF open %s\n",buffer);
		return(fopen(buffer,attr));
	}
}


/*
 *	K I C K S L A V E
 *
 */
kickslave()
{
	static int first_time = 1;
	if (alc->debug)
		fprintf(debug_channel,"MF: kickslave\n");
	if (first_time) {
		allcells(1);
	}
	xs = ys = zs = 0;
	if (alc->debug)
		fprintf(debug_channel,"MF: ddgs devno %x\n",alc->ddgschannel->devno);
	ddgsdup(alc->ddgschannel);
	first_time = 0;
}


/*
 *	S L A V E M E N U  --  Slave side of km / mf menu
 *
 */
slavemenu(nn) 
{
	testchildren();			/* Look for any dead offspring */
		switch (nn) {
			case 1: fipops();
			case 2: scanmu(); break;
			case 3: metlist(); break;
			case 4: arraydigitise(); break;
			case 5: tvdigitise(); break;
		}
		if (alc->debug) {
			Show_alloc();
			fflush(debug_channel);
		}
}

/*
 *  G E T H E L P  --  Null routine for cmdsys
 *
 */
gethelp(){}
