/*
 *      M F _ M E N U . C  --  Metaphase find menu stuff
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1991)
 *
 *  Date:    18th July 1991
 *
 *  Modifications
 *
 */
#include <cmdsys.h>
#include <kyy.h>
#include <slides.h
#include <cfipp.h>
#include <fipcomm.h>

extern FILE *debug_channel;
extern int showmhead(), setcase(), setslide(), setname(), killbay();
extern int newbox(), cornerset(), shading(), tomip0(), admip1();
static options(), setprm(), clust();

/*
 *
 *	S C A N M U  ---
 *
 *			Menu to set up bay options and initiate scan for multiple slides.
 *
 */
scanmu()
{
int bayno, By_cmd, Au_cmd, Ss_cmd, Ex_cmd, stat;

	if (!initscan())								/* Define params on each bay */
		return(0);
	bayno = baynn(alc->curbay);
	
	newmenu("Metaphase Find");
	cmmd("single scan;%?+d%-f", &bayno, showmhead);	mouse(LEFTBT); Ss_cmd = mmcmd;
	cmmd("bay set up;%?+d", &bayno); mouse(MIDBT); By_cmd = mmcmd;
	cmmd("autoscan;%-f", showmhead); mouse(RIGHTBT); Au_cmd = mmcmd;
	cmmd("exit <return> "); mouse(LONGLB); Ex_cmd = mmcmd;
	while(thismu() && (nncmd != Ex_cmd)) {
		if (nncmd == By_cmd) {				/* Bay set up menu */
			downmenu();
			selbay(bayno);
			upmenu();
		} else if(nncmd == Au_cmd) {		/* Scan all active bays */
			if (doscan(NULL) == FS_OK) {
				downmenu();
				metlist();
				upmenu();
			} else
				putprompt();
			unsetflag();
		} else if (nncmd == Ss_cmd) {		/* Single scan */
			downmenu();
			stat = singlebay(bayno);
			upmenu();
			if (stat && doscan(&bayno) == FS_OK)  {
				downmenu();
				metlist();
				upmenu();
			}
			unsetflag();
		}
		showmhead();
		bayno = baynn(alc->curbay);			/* Default to current bay */
	}
}

/*
 *
 *  S E L B A Y ---
 *
 *			Set up parameters for scanning on chosen bays.
 */
static
selbay(bayno) 
int	bayno;
{
int i, j, nti, nst, nste, Bd_cmd, Bc_cmd, By_cmd, Excmd, nn, Spcmd,
	Ptcmd, Slcmd, Tscmd, Thcmd, firstime = 1, Fccmd, patset, ok, deadbay;
struct	mult_slide *bp;
char	dirbuf[40], wbuf[41], casebuf[CASELENGTH+1], nambuf[PATLENGTH+1], 
		slbuf[SLIDELENGTH+1];
int		(*lbug1)(), (*lbug2)();
	
	dirbuf[0] = '\0'; wbuf[0] = '\0';
	qclear();
	fixrgb();
												/* check mic and miproc 
											       initialised */
	if (!checkmcmp(1))
		return(0);
	if(!movefoc(bayno))
		return(0);
	bp = alc->curbay;
	simslide();
	dwb(2, &bp->mbox, 1, 1);					/* show scanned area */
	patset = 0;								/* patnam not set by user */
	strcpy(casebuf, bp->nameids.case);
	strcpy(nambuf, bp->nameids.patnam);
	strcpy(slbuf, bp->nameids.slide);
	deadbay = bayno;
	
	newmenu ("Bay set-up");
		lbug1 = setcase;
	cmmd ("case ID;:k38%+s%-17s%f;t2",wbuf,casebuf,lbug1);
		Spcmd = mmcmd;
		lbug1 = setslide;
	cmmd ("slide ID;:k38%+s%-5s%f",wbuf,slbuf,lbug1);
		Slcmd = mmcmd;
	lbug1 = killbay;
	cmmd("deactivate bay number;:k40%d%f", &deadbay, lbug1);
		lbug1 = setname;
	cmmd ("patient name;:k38%+S%-39s%f",wbuf,nambuf,lbug1);
		Ptcmd = mmcmd;
	cmmd ("tissue/stain options"); mouse(LONGRB);
		Tscmd = mmcmd;
    cmmd ("scan options;%g", options); mouse(MIDBT);
	cmmd ("define focus");mouse(RIGHTBT);
		Fccmd = mmcmd;
	cmmd ("define thresholds");mouse(LEFTBT);
		Thcmd = mmcmd;
	cmmd ("parameters;%g", setprm);
   	cmmd ("bay [RETURN next bay];:k40%d", &bayno);mouse(LONGMB);
	   	By_cmd = mmcmd;
    cmmd ("exit;%-f", showmhead);mouse(LONGLB);
    	Excmd = mmcmd;

	while (1) {
		thismu();
		if(nncmd == Slcmd) {
			strcpy(bp->nameids.case, casebuf);
			strcpy(bp->nameids.slide, slbuf);
			/*
			 * Reset patient name
			 */
			if(!patset) { 
				*nambuf = '\0';
				*(bp->nameids.patnam) = '\0';
				displin(0, Ptcmd);
			}	
			displin(0, Spcmd);
			displin(0, Slcmd);
			displin(0, 0);
			putprompt();
		}
		if(nncmd == Ptcmd) {
			strcpy(bp->nameids.patnam, nambuf);
			displin(0, Ptcmd);
			patset = 1;
			putprompt();
		}
		if(nncmd == Slcmd) {				/* If id change ask for focus
											   only once */
			bp->focN38 = 0;
			bp->focbox = 0;
			putprompt();
		}
		if(nncmd == Fccmd) {				/* Explicit set focus request */
			bp->focN38 = 0;
			bp->focbox = 0;
			deffocs();
			putprompt();
		}

		if(nncmd == Excmd) {  				/* EXIT MENU */
			if (bp->active)	{				/* if bay is active */
				ok = deffocs();				/* define focus */
				if(ok == 1)					/* state 0 = don't want to do it */
					deftholds();			/* state 2 = already done */
			}
			break;
		}
		if(nncmd == Thcmd) {				/* Define thresholds */
			deftholds();
			putprompt();
		}
		if(nncmd == By_cmd || !nncmd) {
			if (bp->active) {
				ok = deffocs();
				if ( ok == 1 )				/* state 0 = don't want to do it */
					deftholds();			/* state 2 = already done */
			}
			if(!nncmd) {
				setdisplay(NOTBUST);					/* Dont redraw whole screen */
				if((bayno = baynn(alc->curbay) + 1) > alc->maxbays)
					bayno = 1;
			}
											/* Drive to next bay if poss */
			if(movefoc(bayno)) {
				erasehisto();				/* Erase previous bay histo */
				bp = alc->curbay;
				strcpy(casebuf, bp->nameids.case);
				strcpy(nambuf, bp->nameids.patnam);
				strcpy(slbuf, bp->nameids.slide);
				displin(0, 0);
				displin(0, Spcmd);
				displin(0, Ptcmd);
				displin(0, Slcmd);
				displin(0, By_cmd);
				simslide();
											/* show scanned area */
				dwb(2, &alc->curbay->mbox, 1, 1);			
			} 
			bayno = baynn(alc->curbay);
			displin(0, By_cmd);
			putprompt();
		} 
		if(nncmd == Tscmd) {				/* Tissues/stains menu */
			downmenu();
			tisstmu(&bp->typeids);
			upmenu();
			setbaycl(bp, &bp->typeids);
		}
	showmhead();
    }		
	erasehisto();							/* Erase bay histo */
	return(1);
}

/*
 *
 *  S I N G L E B A Y ---
 *
 *			Set up parameters for scanning on a bays.
 */
static
singlebay(bayno) 
int	bayno;
{
int i, j, Bd_cmd, Bc_cmd, Excmd, nn, Spcmd, Ptcmd, Slcmd, Tscmd, Fmcmd, patset;
struct	mult_slide *bp;
char	dirbuf[40], wbuf[41], casebuf[CASELENGTH+1], nambuf[PATLENGTH+1], 
		slbuf[SLIDELENGTH+1];
int		(*lbug1)(), (*lbug2)();
	
	dirbuf[0] = '\0'; wbuf[0] = '\0';
	qclear();
	fixrgb();
												/* check mic and miproc 
											       initialised */
	if (!checkmcmp(1))
		return(0);
	if(!movefoc(bayno))
		return(0);
	bp = alc->curbay;
												/* force redef. of focs */
	bp->focN38 = 0;
	bp->focbox = 0;
	
	simslide();
	dwb(2, &bp->mbox, 1, 1);					/* show scanned area */
	patset = 0;								/* patnam not set by user */
	strcpy(casebuf, bp->nameids.case);
	strcpy(nambuf, bp->nameids.patnam);
	strcpy(slbuf, bp->nameids.slide);
	
	newmenu ("Metaphase Scan");
		lbug1 = setcase;
	cmmd ("case ID;:k38%+s%-17s%f;t2",wbuf,casebuf,lbug1); Spcmd = mmcmd;
		lbug1 = setslide;
	cmmd ("slide ID;:k38%+s%-5s%f",wbuf,slbuf,lbug1); Slcmd = mmcmd;
		lbug1 = setname;
	cmmd ("patient name;:k38%+S%-38s%f",wbuf,nambuf,lbug1); Ptcmd = mmcmd;
	cmmd ("tissue/stain options"); mouse(LONGRB); Tscmd = mmcmd;
    cmmd ("scan options;%g", options); mouse(MIDBT);
	cmmd ("parameters;%g", setprm);
    cmmd ("metaphase scan;%-f", showmhead);mouse(LEFTBT); Fmcmd = mmcmd;
    cmmd ("exit"); mouse(LONGLB); Excmd = mmcmd;

	while (1) {
		thismu();
		if(nncmd == Slcmd) {
			strcpy(bp->nameids.case, casebuf);
			strcpy(bp->nameids.slide, slbuf);
			/*
			 * Reset patient name
			 */
			if(!patset) { 
				*nambuf = '\0';
				*(bp->nameids.patnam) = '\0';
				displin(0, Ptcmd);
			}	
			displin(0, Spcmd);
			displin(0, Slcmd);
			displin(0, 0);
			putprompt();
		}
		if(nncmd == Ptcmd) {
			displin(0, Ptcmd);
			strcpy(bp->nameids.patnam, nambuf);
			patset = 1;
			putprompt();
		}
		if(nncmd == Fmcmd) {		
			if(deffocs())
				if(deftholds()) 
					return(1);
			putprompt();
		}
		if (nncmd == Excmd)
			return(0);
			
		if (nncmd == Tscmd) {				/* Tissues/stains menu */
			downmenu();
			tisstmu(&bp->typeids);
			upmenu();
			/*
			 *	Set new box classifier
			 */
			setbaycl(bp, &bp->typeids);
			putprompt();
		}
		showmhead();
    }
}

/*
 *	O P T I O N S
 *
 */
static options()
{
int	 lth1, lth2, Bd_cmd, Bc_cmd, Hs_cmd, shval, Sh_cmd, kbwas, Ex_cmd;
struct  mult_slide *bp;
int		(*linkerbug)();

	kbwas = kbegin;								/* Remember old column */
												/* kbegin is in cmdsys.h */
	kbegin = 40;								/* Set column to 40 */

	bp = alc->curbay;
	shval = (doshade < 1)  ? 1 : 0;

	newmenu("Options");
	cmmd ("bay scan;%T", &bp->active);
	cmmd ("boxdrive ;%g%x%+i%+i%+i",newbox,"Search Limits",&bp->mbox,1,1);
		mouse(MIDBT);Bd_cmd = mmcmd;
	cmmd ("boxcorners ;%g%+i%+i%+i%+i%-+d", cornerset, &bp->xlow,
    					&bp->xhigh, &bp->ylow, &bp->yhigh, &bp->mbox);
    	mouse(LONGRB); Bc_cmd = mmcmd;
	cmmd ("histogram;"); Hs_cmd = mmcmd;
	cmmd ("shading correction;%T", &shval); Sh_cmd = mmcmd;
	linkerbug = shading;
	cmmd ("shading reset;swathe length\\:<100/10000>%?d%g", &bp->shadsw, linkerbug);
	if (alc->devel) {
		cmmd ("focus slew;%d",&(alc->cfipp->focus.permove));
		cmmd ("focus excursion;%d %d",&(alc->cfipp->focus.inswathe),&(alc->cfipp->focus.between));
		cmmd ("focus max excursion;%d",&(alc->cfipp->focus.excursion));	
		cmmd ("afe interpolate;%T",&(alc->cfipp->afeinterpolate));
	}
	if (alc->debug || alc->devel) {
		linkerbug = tomip0;
		cmmd("set z-datum;%x%f","dz 0", linkerbug);
	}
	cmmd("exit "); mouse(LONGLB); Ex_cmd = mmcmd;
	while(thismu() && (nncmd != Ex_cmd)) {
    	if (nncmd == Bd_cmd)
    		box_to_slide(&bp->mbox,&bp->xlow,&bp->xhigh,&bp->ylow,&bp->yhigh);
    	if (nncmd == Bc_cmd || nncmd == Bd_cmd)
    		bp->focbox = 0;
    	if (nncmd == Sh_cmd) {
    		doshade = shval ? 0 : 1;
    	}
		if(nncmd == Hs_cmd) {
			erasehisto();
			newspecimen();							/* Ensure clustering OFF ! (YUK) */
			if(scanspeed(bp->sspeed) != FS_OK)
				return(0);
			autofocus(ON);
			if (doshade < 1) {
				if (shading()) return(0);
				else {
					shval = 0;
					doshade = 1;			/* Do it only once */
					displin(0, Sh_cmd);
				}
			}
			if (hrhisto(&lth1, &lth2, bp->shadsw)) {
				fiptim("Histogram reset.");
			} else {
				showhisto(lth1, lth2);
				fprintf(output, "Thresholds found:  Lower %d  Upper %d\n", 
					lth1, lth2);
			}
		}
		putprompt();
	}
	kbegin = kbwas;
	putprompt();
} 
/*
 *	S E T P R M
 *
 */
static setprm()
{
int		Thr_cmd, Mth_cmd, Xlim_cmd, Ylim_cmd, Def_cmd, kbwas, Sscmd;
struct  mult_slide *bp;
struct  cyto_types *ctp;
int		(*linkerbug)();

	bp = alc->curbay;
	ctp = &bp->typeids;
	kbwas = kbegin;								/* Remember old column */
												/* kbegin is in cmdsys.h */
	kbegin = 40;								/* Set column to 40 */

	newmenu("Metaphase Finding Parameters");
	cmmd("candidates;<1/2000>%d",&bp->mcells);
	Thr_cmd = cmmd("thresholds;Lower <1/255>%d Upper <1/255>%?d ",
			&bp->thresh1,&bp->thresh2);
	Mth_cmd = cmmd("manual threshold;%T", &bp->thflag);
	cmmd("swathes;<1/500>%d",&bp->mswathe);
	Sscmd = cmmd("scan speed;%d", &bp->sspeed);
	Xlim_cmd = cmmd("x Limits;Min %d Max %d",&bp->xlow,&bp->xhigh);
	Ylim_cmd = cmmd("y Limits;Min %d Max %d",&bp->ylow,&bp->yhigh);
	Def_cmd = cmmd("restore default box");  
	linkerbug = admip1;
	cmmd("y-gate;options(0,1,2)- <0/2>%x%d%f","sw",&bp->ygate, linkerbug);
	cmmd("cluster classifier;%g",clust);
	while (thismu()) {
		if (alc->debug) fprintf(debug_channel,"Man thresh toggle %d\n",bp->thflag);
		if(nncmd == Thr_cmd) {
			if (bp->thresh1 > bp->thresh2) {
				bp->thresh1 = bp->thresh2 - 1;
				fprintf(output,"Lower threshold reset to %d\n", bp->thresh1);
			}
			bp->thflag = 1;				/* Force thresh overide */
			displin(0,Mth_cmd);
		} else if ((nncmd == Xlim_cmd) || (nncmd == Ylim_cmd) || 
												(nncmd == Def_cmd)) {
			dwb(0);			/* Erase old box */
			if(nncmd == Def_cmd)
				setdefaultbox(bp);
			slidelimits(&bp->xlow,&bp->xhigh,&bp->ylow,&bp->yhigh);
			slide_to_box(&bp->mbox,bp->xlow,bp->xhigh,bp->ylow,bp->yhigh);
			dwb(2,&bp->mbox,1,1);
			displin(0, Xlim_cmd);
			displin(0, Ylim_cmd);
		}
		if(nncmd == Sscmd) {
			if((bp->sspeed > MAXSCANSPEED) || (bp->sspeed < MINSCANSPEED)) {
				fprintf(output, "Scan speed set outside allowed range! Reset to default.\n");
				bp->sspeed = defaultscanspeed();
				displin(0, Sscmd);
			}
		}
		if(nncmd == Def_cmd || nncmd == Xlim_cmd || nncmd == Ylim_cmd)
			bp->focbox = 0;
		putprompt();
	}
	kbegin = kbwas;									/* Reset column */
}

/*
 *	C L U S T
 *
 */
static 
clust() 
{
struct  mult_slide *bp;
	int old_drange;	/* Kludge */

	bp = alc->curbay;
	old_drange = bp->drange;

	newmenu("cluster classifier");
	cmmd("object count;min %d max %d", &bp->countmin, &bp->countmax);
	cmmd("width (microns);min %d max %d", &bp->xmin, &bp->xmax);
	cmmd("area (pixels);min %d max %d", &bp->careamin, &bp->careamax);
	cmmd("octagon area (pixels);min %d max %d", &bp->octareamin, &bp->octareamax);
	cmmd("mean cluster OD;min %d max %d", &bp->codmin, &bp->codmax);
	if (alc->config & CONF_FIPAL)
		cmmd("density range (0/6);:k40<0/6>%d",&bp->drange);
	else
		cmmd("density range (0/3);:k40<0/3>%d",&bp->drange);
	cmmd("density variation;max %d", &bp->dvmax);
	cmmd("shape factor;max %d", &bp->sfmax);
	cmmd("skirt OD;max %d", &bp->skodmax);
	if(alc->devel) {
		cmmd("limb area;max %d", &bp->area);
		cmmd("limb distance;max %d", &bp->clus1);
		cmmd("limb IOD;max %d", &bp->iod);
		cmmd("limb mean OD;max %d", &bp->clus2);
	}
	cmmd("retrain;%T", &bp->retrain);
	while (thismu());
	if (bp->drange != old_drange) {
		doshade = -1;
	}
}

