/*
 *	File:	M I C _ C O N V . C
 *		
 *		Microscope coordinate conversion.
 *
 *
 *	Written by:	S. C. Georgiou
 *				720 Birchwood Boulevard,
 *				Birchwood Science Park,
 *				Warrington,
 *				Cheshire.
 *
 *	Date: 4 Dec 1986.
 *
 *
 *	Modifications:
 *
 *	18 Jun 1991		CAS			Start a major cleanup
 *	13 Mar 1989		SEAJ		Changed name of nextline to nextbufline.
 *	28 Feb 1989		SCG/dcb		Use initialised slide width as first measure
 *  14 Feb 1989 	SEAJ		Added extra parameter to add_to_list for backward 
 *								conversion.
 *	23 Jan 1989		SEAJ		Added extra prompts.
 *  16 Jan 1989		SEAJ		Allow for different cases in micread.
 *  13 Dec 1988		SEAJ		Changed conversionstr to lower case.
 *   7 Dec 1988		SEAJ		Modified messages printed for backward conversion.
 *   1 Nov 1988		SEAJ		Added in local list and metlist stuff.
 *	14 Sep 1987		CAS			Disable slide measure on seat 2
 *	29 Jul 1987		CAS			'scopename' fn to get scope name outside
 *	16 Jul 1987		SCG			Lower case menu options
 *	10 Jul 1987		SCG			Backward conversion and slwidth in mult_slides
 *	16 Apr 1987		CAS			Change coord file name + can't open message
 *	15 Apr 1987		SCG			A10 ref. point instead of A5.
 *	14 Apr 1987		SCG			Read mic coords from file. mm instead of mu.
 *	17 Mar 1987		SCG			Tidied up the code. Introduced LH, RH  
 *								stage1 philosophy.  Made code more
 *								efficient.
 *	05 Mar 1987		SCG			Further mods to cope with 180 rotation
 *								of slide w.r.t. NE finder on target mic.
 *	03 Mar 1987		SCG			Put options for different types of
 *								mics and the ability to measure slide width.
 *	19 Feb 1987		SCG			Cludge option under devel mode.
 *	12 Feb 1987		SCG			Modified functionality of this to agree with
 *								mic_conv.c in /dd/usr/sotiris/src/trans_coord
 *	16 Dec 1986		SCG			Modification to prevent hanging when both
 *								sets of coordinates are zero.
 *
 *
 */

#include	<cmdsys.h>
#include	<kyy.h>
#include	<slides.h>
#include	<math.h>
#include	<cconv.h>

/* external definitions */
extern	FILE	*debugfp;			/* Miproc comms debug file. */
extern	FILE	*debug_channel;		/* Systems debug file. */

/*	Local Defines */
#define DIRSTART 5
#define SCREENFUL 15
#define MAXOPTS 4		/* maximum number of options for conversion */
#define RADIUS 250		/* radius of circle on NE finder */
#define RADIUS2 62500	/* radius squared */
#define DIST 338		/* distance to center of quadrants */

char 	*conversionstr[] = {
				"conversion off",
				"microscope verniers",
				"england finder (Graticules Ltd.)",
				"england finder (Albert Sass)"
			};						/* type of conversion string */

/* structure definitions */
static struct coordlist {
	struct coord pt;
	char str[20];
	struct coordlist *next;
};
static struct microscope mic = {0, 0, {0, 0},{0, 0}, 1, 1, 'U', 'N', 'S', 'E', 'T'};

static char *stage[] = {
				"Right-handed",
				"Left-handed"
			};
static char *optics[] = {
				"Non-inverted",
				"Inverted"
			};
/* global defines */
static struct coord cytoZ50 = { -11648, -11674 },	/* Cytoscan Z50 coordinates */
					cytoA10 = { 11649, 27149},		/* Cytoscan A10 coordinates*/
					cytoM30 = { 902, 7727 },		/* Cytoscan M30 coordinates for check */
					middle;							/* coordinates of midpoint of slide
																on target mic */
																
static	float	sin1, cos1, 		/* First rotation angle sin & cos. */
				sin2, cos2; 		/* Second rotation angle sin & cos. */
static	int		xoff, yoff;			/* Translation offsets. */
                                       
static	int 	iflag;				/* Set when coords reflected. */
static  int     copied = 0;			/* Set when backward conversion list has been made met list */
static struct coordlist *start = NULL;
/* Forward declarations. */
static	int mic_conv(), select_mic(), 
			slmeasure(), setiflag(), eval_ang(), eval_mids(), backconv(),
			convertback(), conv_coords(), cyto_ne(), ne_cyto(), cyto_sne(),
			sne_cyto(), checkstr(), chekstr(),
			checkref(), conv_str(), isnumber(), square(),
			add_to_list(), make_met(), print_list(), free_list();
/*
 *	External stuff
 */
extern	int micread(), shownames(), float_str();
extern	float atan2();

/*
 * routines used else where
 */
int convert(), scopename();

/*
 * M I C _ C O N V	 -- select microscope and type of conversion
 *
 */
mic_conv(toggle)
char *toggle;
{
	int i, cmd, Quitcmd;
	int option_is[MAXOPTS];
	char micname[MAXNAME];

	copied = 0;
	start = NULL;
	strcpy(micname, "UNSET");
	for (i = 0; i < MAXOPTS; i++)
		option_is[i] = 0;
	option_is[alc->ccoord] = 1;
	
	newmenu("Coordinate Conversion");
	mmcmd = 0;
	cmmd("coordinate conversion off;:k40%-t", &option_is[mmcmd]);
	cmmd("microscope verniers;:k40%-t", &option_is[mmcmd]);
	cmmd("england finder (Graticules Ltd.);:k40%-t", &option_is[mmcmd]);
	cmmd("england finder (Albert Sass);:k40%-t", &option_is[mmcmd]);
	cmmd("select microscope;%?+s%f", micname, select_mic);
	cmmd("backward conversion;%f", backconv);
	cmmd("quit"); Quitcmd = mmcmd;
	
	while (cmd = thismu()) {
		if (cmd == Quitcmd) {
			alc->ccoord = 0;
			break;
		} else if (cmd > 0 && cmd <= 4) {
			alc->ccoord = cmd - 1;
			for (i=0; i<MAXOPTS; i++)
				option_is[i] = 0;
			option_is[alc->ccoord] = 1;
			for (i = 1; i <= 4; i++)
				displin(0, i);
			putprompt();
		}
		if ((alc->ccoord == MICROSCOPE) && (!strcmp(mic.id, "UNSET"))) {
			screengo(ENDLINE, 0);
			fprintf(output, "Please select microscope id.\n");
			putprompt();
		}
	}
	if (alc->ccoord == MICROSCOPE) {
		if(mic.Z50.x == mic.A10.x && mic.Z50.y == mic.A10.y) {	/* Bad coords given.*/
			fprintf(output, "Coords define same point.  No conversion.\007\n");
			alc->ccoord = 0;
		}
	} else if (alc->ccoord > MICROSCOPE) {
		if (!(strcmp(mic.id, "UNSET")) || (mic.Z50.x == 0 && mic.Z50.y == 0 && 
										mic.A10.x == 0 && mic.A10.y == 0)) {
			mic.Z50.x = cytoZ50.x;
			mic.Z50.y = cytoZ50.y;
			mic.A10.x = cytoA10.x;
			mic.A10.y = cytoA10.y;
			if (!mic.inverted) {			/* non inverted mic */
				mic.inverted = 1;			/* make it inverted */
				if (mic.left_handed)		/* left handed mic */
					mic.left_handed = 0;	/* make it right handed */
				else						/* right handed mic */
					mic.left_handed = 1;	/* make it left handed */
			}
		}
		if (alc->debug)
			fprintf(debug_channel, "A10 = (%d, %d) Z50 = (%d, %d)\n", mic.A10.x, 
												mic.A10.y, mic.Z50.x, mic.Z50.y);
	}
	setiflag();
	eval_ang();
	eval_mids();
	strcpy(toggle, conversionstr[alc->ccoord]);
	if (!copied && start != NULL) {
		fprintf(output, "Backward conversion list not filed. Do you wish to file it?\n");
		if (!yesno(1))
			make_met();
	}
	putprompt();
	free_list();
}

/*
 *  S E L E C T _ M I C -- Obtain microscope coordinates of the datum point or 
 *						   optionally of two points to use in the conversion 
 *						   from local to microscope coordinates. 
 */
select_mic(string)
char *string;
{
int  Scmd, Icmd, Wcmd, i;
char Z50x[9], Z50y[9], A10x[9], A10y[9], lenstr[9], tmpstr[9], langstr[10];
char stagestr[13], opticsstr[13], slidestr[13];

	/* Convert slide width to string. */
	float_str(lenstr, alc->curbay->slwidth);
	/* Convert coords to strings. */
	float_str(Z50x, mic.Z50.x);
	float_str(Z50y, mic.Z50.y);
	float_str(A10x, mic.A10.x);
	float_str(A10y, mic.A10.y);
	strcpy(stagestr, stage[mic.left_handed]);
	strcpy(opticsstr, optics[mic.inverted]);
	*tmpstr = '\0';
	screengo(ENDLINE, 0);
	if (!strcmp(string, "UNSET") || !micread(string,&mic)) {
		fprintf(output, "Please select microscope id. and measure slide width.\n");
		newmenu("Select Microscope");
		cmmd("microscope id;:k40 %+s%-s", string, mic.id);  Icmd = mmcmd;
		cmmd("show microscope directory;%f", shownames); mouse(MIDBT);
		cmmd("microscope coords for Z50;:k40 x %8s y %8s %-+d %-+d %f", Z50x, 
										Z50y, &mic.Z50.x, &mic.Z50.y, checkstr);
		cmmd("microscope coords for A10;:k40 x %8s y %8s %-+d %-+d %f", A10x, 
										A10y, &mic.A10.x, &mic.A10.y, checkstr);
		cmmd("stage;:k40%+T%-s%-+d%f", &mic.left_handed, stagestr, stage, conv_str);
		cmmd("optics;:k40%+T%-s%-+d%f", &mic.inverted, opticsstr, optics, conv_str);
		if (alc->config & CONF_FIP) {
			cmmd("slide measure;%g", slmeasure);  Scmd = mmcmd;
		} else {
			cmmd("slide measure             (Not available on this seat)");	Scmd = 0;
		}
		cmmd("slide width;:k40%+s%-s", tmpstr, lenstr); Wcmd = mmcmd;
	
		while(thismu()) {
			switch(nncmd) {
				case 1:
					if(micread(string,&mic)) {
						float_str(Z50x, mic.Z50.x);
						float_str(Z50y, mic.Z50.y);
						float_str(A10x, mic.A10.x);
						float_str(A10y, mic.A10.y);
						strcpy(stagestr, stage[mic.left_handed]);
						strcpy(opticsstr, optics[mic.inverted]);
						for (i = 0; i <= 6; i++)
							displin(0, i);
					} else {
						strcpy(string, "UNSET");
						displin(0, Icmd);
						screengo(ENDLINE, 0);
						fprintf(output, "Microscope id match not found.\007\n");
					}
					putprompt();
					break;
				case 8:
					if(!chekstr(tmpstr) || (*tmpstr == '-') || (*tmpstr == '\0')) {/* Check for stupid strings. */
						screengo(ENDLINE, 0);
						fprintf(output, "Illegal value.\007\n");
					} else {
						alc->curbay->slwidth = (int) (1000.0 * atof(tmpstr));
						strcpy(lenstr, tmpstr);
						screengo(ENDLINE, 0);
						displin(0, Wcmd);
						putprompt();
					}
					break;
				case 7 :
					float_str(lenstr, alc->curbay->slwidth);
					displin(0, Wcmd);
					screengo(ENDLINE, 0);
					putprompt();
					break;
			}
		}
	}
	if (alc->ccoord == MICROSCOPE) {
		if(mic.Z50.x == mic.A10.x && mic.Z50.y == mic.A10.y) {	/* Bad coords given.*/
			fprintf(output, "Coords define same point.  No conversion.\007\n");
			alc->ccoord = 0;
			return;
		}
	} else if (alc->ccoord > MICROSCOPE) {
		if (mic.Z50.x == 0) {
			mic.Z50.x = cytoZ50.x;
			mic.Z50.y = cytoZ50.y;
			mic.A10.x = cytoA10.x;
			mic.A10.y = cytoA10.y;
			if (!mic.inverted) {			/* non inverted mic */
				mic.inverted = 1;			/* make it inverted */
				if (mic.left_handed)		/* left handed mic */
					mic.left_handed = 0;	/* make it right handed */
				else						/* right handed mic */
					mic.left_handed = 1;	/* make it left handed */
			}
		}
	}
}

/*
 * C O N V E R T -- Routine for external use of conversions
 *
 */
convert(cytox, cytoy, strx, stry, string)
short cytox, cytoy;
char *strx, *stry, *string;
{
	int microx, microy;
	
	*string = '\0';
	switch(alc->ccoord) {
		case MICROSCOPE :
			conv_coords(cytox, cytoy, &microx, &microy);
			float_str(strx, microx);
			float_str(stry, microy);
			break;
		case ENGLISHNE :
			cyto_ne(cytox, cytoy, string);
			break;
		case SWEDISHNE :
			cyto_sne(cytox, cytoy, string);
			break;
	}
}

/*
 *	S C O P E N A M E  --  External access to microscope name
 *
 */
scopename()
{
	return(mic.id);
}

/*
 *  S L M E A S U R E -- Helps the user to find the width of the slide.
 *
 */
static
slmeasure()
{
int	idiff;

	idiff = alc->scanright - alc->curbay->slwidth;
	stagepos(idiff, 0, 0);
	fprintf(output, "Position at left edge of slide and confirm.\n");
	if(!yesno(1)) {				/* Get position of edge. */
		getpos(2);
		if(xs > -11000 || xs < -15000) {
			fprintf(output, "Bad width value.\007\n");
		} else
			alc->curbay->slwidth = alc->scanright - xs;
	}

}
/*
 *  S E T I F L A G  -- Sets iflag.  Determines whether microscope reflecting 
 *						or not. 
 */
static
setiflag()
{
int	d1, d2;

	d1 = mic.Z50.y - mic.A10.y;
	d2 = mic.Z50.x - mic.A10.x;
	if(mic.inverted) {			/* Inverted microscope. */
	
		if((d1 < 0 && d2 > 0) || (d1 > 0 && d2 < 0)) {	/* Reflecting. */
			iflag = 1;
		} else 											/* Non reflecting. */
			iflag = 0;
			
	} else {				/* Non inverted microscope. */

		if((d1 < 0 && d2 < 0) || (d1 > 0 && d2 > 0)) {	/* Reflecting. */
			iflag = 1;
		} else 											/* Non reflecting. */
			iflag = 0;
			
	}
}
/*
 *  E V A L _ A N G  -- Calculates angles and offsets required for transformation. 
 *						Actual rotation angle theta2 - theta1.  Used two angles
 *						to make sure that sign is right.
 */
static
eval_ang()
{
float	gly, glx, lcx, lcy, theta, theta1, theta2, Pi;
int		xpos, ypos;

	if(iflag) {							/* Consider reflection. */
		mic.Z50.x = -mic.Z50.x;
		mic.A10.x = -mic.A10.x;
	}
	
	gly = (float) (mic.Z50.y - mic.A10.y);	/* Evaluate first angle. */
	glx = (float) (mic.Z50.x - mic.A10.x);
	theta1 = atan2(gly, glx);
	sin1 = (float) sin(theta1);
	cos1 = (float) cos(theta1);
	
	lcy = (float) (cytoZ50.y - cytoA10.y);	/* Evaluate second angle. */
	lcx = (float) (cytoZ50.x - cytoA10.x);
	theta2 = atan2(lcy, lcx);
	sin2 = (float) sin(theta2);
	cos2 = (float) cos(theta2);

	/* Evaluate offsets. */
	offset((float) mic.Z50.x,(float) mic.Z50.y,(float) cytoZ50.x,(float) cytoZ50.y);
		   
	if(iflag) {							/* Correct for reflection. */
		mic.Z50.x = -mic.Z50.x;
		mic.A10.x = -mic.A10.x;
	}
	
	if(alc->debug) {
		Pi = 4 * atan(1.);
		theta = (theta2 - theta1) * 180 / Pi;
		conv_coords(cytoM30.x, cytoM30.y, &xpos, &ypos);	
		fprintf(debug_channel, "Rotation angle %f\n", theta);
		fprintf(debug_channel, "Translation in x %d and y %d\n", xoff, yoff);
		fprintf(debug_channel, "Transformed point at M30 %d %d\n", xpos, ypos);
	}
		   
}
/*
 *  E V A L _ M I D S  -- Calculates coords of a point of slide on target mic 
 *						  about which the slide is rotated.  When the slide is
 *						  horiz. this point's x coord is half way along the 
 *						  length of the NE finder and its y coord half way 
 *						  along the width of the slide. If slide vertical 
 *						  coords are other way round.  The slide is rotated 
 *						  when transforming to a LH stage (inverted mic) or a 
 *						  RH stage (non-inverted mic).
 */
static
eval_mids()
{
float	n38x, n38y, d1, d2;

	d1 = (float) (mic.Z50.x - mic.A10.x);
	d2 = (float) (mic.Z50.y - mic.A10.y);

	if(fabs(d1) > fabs(d2)) {						/* Slide horizontal. */
											/* 28 - Diff between A10 N38 */
											/* 40 - Diff between A10 Z50 */
											/* 12 - Diff between A10 N38 */
											/* 24 - Diff between A10 Z50 */
		n38x = d1 * 28 / 40 + (float) mic.A10.x;
		n38y = d2 * 12 / 24 + (float) mic.A10.y;
		middle.x = (int) n38x;
		if(d2 > 0) 
			middle.y = (int) n38y - (alc->scanright - alc->curbay->slwidth / 2);
		else
			middle.y = (int) n38y + (alc->scanright - alc->curbay->slwidth / 2);
			
	} else {							/* Slide vertical. */
	
		n38x = d1 * 12 / 24 + (float) mic.A10.x;
		n38y = d2 * 28 / 40 + (float) mic.A10.y;
		if (alc->debug)
			fprintf(debug_channel, "alc->scanright = %d slwidth = %d\n", alc->scanright, alc->curbay->slwidth);
		if(d1 > 0)
			middle.x = (int) n38x - (alc->scanright - alc->curbay->slwidth / 2);
		else
			middle.x = (int) n38x + (alc->scanright - alc->curbay->slwidth / 2);
		middle.y = (int) n38y;
	}
	if(alc->debug) {
		fprintf(debug_channel, "middle.x %d middle.y %d\n", middle.x, middle.y);
	}
}
/*
 *  B A C K C O N V  --  Microscope to Cytoscan conversion.
 *
 */
static
backconv()
{
static struct coord mscope, cytoscan;
char	strx[8], stry[8], NEstr[8], SNEstr[8], tmpstr[20];
int ok, Cytocmd, Backcmd, Movecmd;

	cytoscan.x = 0;
	cytoscan.y = 0;
	strx[0] = '\0';
	stry[0] = '\0';
	NEstr[0] = '\0';
	SNEstr[0] = '\0';
	tmpstr[0] = '\0';
	
	screengo(ENDLINE, 0);
	switch (alc->ccoord) {
		case CONVOFF :
			fprintf(output, "Coordinate conversion is switched off.\007\n");
			fprintf(output, "Please return to previous menu and select type of conversion required.\n");
			break;
		case MICROSCOPE :
			fprintf(output, "Please enter Microscope coordinates.\n");
			break;
		case ENGLISHNE :
		case SWEDISHNE :
			fprintf(output, "Please enter England finder reference.\n");
			break;
	}
	newmenu("Backward Conversion");
	switch (alc->ccoord) {
		case MICROSCOPE :
		 	cmmd("microscope coords;:k40 x %8s :k51 y %8s%-+d%-+d%f", strx, stry, 
								&mscope.x, &mscope.y, checkstr); Backcmd = mmcmd;
			break;
		case ENGLISHNE :
			cmmd("england finder (Graticules Ltd.);:k40 %s", NEstr); Backcmd = mmcmd;
			break;
		case SWEDISHNE :
			cmmd("england finder (Albert Sass);:k40 %s", SNEstr); Backcmd = mmcmd;
			break;
	} 	/* end of switch */
	cmmd("cytoscan coords;:k40 x %d :k51 y %d", &cytoscan.x, &cytoscan.y); Cytocmd = mmcmd;
	if (alc->config & CONF_FIP)
		cmmd("move to new cytoscan position"); mouse(MIDBT); Movecmd = mmcmd;
	cmmd("add coordinates to local list;%-+d%-+d%-+s%f", &cytoscan.x, &cytoscan.y, 
				tmpstr, add_to_list);
	cmmd("show local list; %f", print_list);
	cmmd("file list;%f", make_met);
	while(thismu()) {
		if (alc->ccoord == MICROSCOPE) {
			if(mic.Z50.x == mic.A10.x && mic.Z50.y == mic.A10.y) {
				fprintf(output, "Z50 A10 coords define same point.  No conversion.\007\n");
				putprompt();
				ok = 0;
			} else if (!strcmp(mic.id, "UNSET")) {
				fprintf(output, "Microscope is UNSET. No conversion.\007\n");
				ok = 0;
			} else
				ok = 1;
		} else if ((alc->ccoord > MICROSCOPE) && (!strcmp(mic.id, "UNSET"))) {
			mic.Z50.x = cytoZ50.x;
			mic.Z50.y = cytoZ50.y;
			mic.A10.x = cytoA10.x;
			mic.A10.y = cytoA10.y;
			/* make microscope one of the two defaults */
			if (!mic.inverted) {			/* non inverted mic */
				mic.inverted = 1;			/* make it inverted */
				if (mic.left_handed)		/* left handed mic */
					mic.left_handed = 0;	/* make it right handed */
				else						/* right handed mic */
					mic.left_handed = 1;	/* make it left handed */
			}
			ok = 1;
		}
		if (ok) {
			setiflag();
			eval_ang();
			eval_mids();
			if (nncmd == Cytocmd) {  /* typed in cytoscan coords */
				if ((abs(cytoscan.x) > 32000) || (abs(cytoscan.y) > 32000)) {
					fprintf(output, "Illegal cytoscan coords. No conversion.\n");
					cytoscan.x = 0;
					cytoscan.y = 0;
					displin(0, Cytocmd);
				} else {
					switch (alc->ccoord) {
						case MICROSCOPE :
							conv_coords((short) cytoscan.x, (short) cytoscan.y, &mscope.x, &mscope.y);
							float_str(strx, mscope.x);
							float_str(stry, mscope.y);
							sprintf(tmpstr, "%s8%12s", strx, stry);
							break;
						case ENGLISHNE :
							cyto_ne((short) cytoscan.x, (short) cytoscan.y, NEstr);
							strcpy(tmpstr, NEstr);
							break;
						case SWEDISHNE :
							cyto_sne((short) cytoscan.x, (short) cytoscan.y, SNEstr);
							strcpy(tmpstr, SNEstr);
							break;
						default :
							ok = 0;
							break;
					} 	/* end of switch */
					if (ok)
						displin(0,Backcmd);
				}
			} else if (nncmd == Backcmd) {
				switch (alc->ccoord) {
					case MICROSCOPE :
						convertback(mscope.x, mscope.y, &cytoscan.x, &cytoscan.y);
						sprintf(tmpstr, "%s\t%s", strx, stry);
						break;
					case ENGLISHNE :
						if (checkref(NEstr))
							strcpy(tmpstr, NEstr);
							ne_cyto(NEstr, &cytoscan.x, &cytoscan.y);
						break;
					case SWEDISHNE :
						if (checkref(SNEstr))
							strcpy(tmpstr, SNEstr);
							sne_cyto(SNEstr, &cytoscan.x, &cytoscan.y);
						break;
					default :
						ok = 0;
						break;
				} 	/* end of switch */
				if ((abs(cytoscan.x) > 32000) || (abs(cytoscan.y) > 32000)) {
					fprintf(output, "Illegal cytoscan coords. No conversion.\n");
					cytoscan.x = 0;
					cytoscan.y = 0;
					tmpstr[0] = '\0';
					ok = 0;
					displin(0, Cytocmd);
				}
				if (ok)
					displin(0,Cytocmd);
			} else if (nncmd == Movecmd) {
				if ((abs(cytoscan.x) > 32000) || (abs(cytoscan.y) > 32000)) {
					fprintf(output, "Requested move is outside bounds of bay.\007\n");
					break;
				} else {
					stagepos(cytoscan.x, cytoscan.y, zs);
					xs = cytoscan.x;
					ys = cytoscan.y;
					fprintf(output, "Is metaphase centered?\n");
					if (!yesno(1)) {
						getpos(2);
						cytoscan.x = xs;
						cytoscan.y = ys;
						displin(0, Cytocmd);
						putprompt();
						fprintf(output, "Add metaphase to local list?\n");
						if (!yesno(1))
							add_to_list(&cytoscan.x, &cytoscan.y, tmpstr);
						putprompt();
					}
				}
			}
			putprompt();
		} /* end of if ok */
	} /* end of while */
}
/*
 *  C O N V E R T 	B A C K  -- Microscope to Cytoscan conversion.
 *
 */ 
static 
convertback(oldx, oldy, newx, newy)
int	oldx, oldy; 
int	*newx, *newy;
{
float  nx, ny, rot1x, rot1y, rot2x, rot2y, bix, biy;

	if(alc->debug) {
		fprintf(debug_channel, "In converback.\n");
	}
	if((mic.inverted && mic.left_handed) || (!mic.inverted && !mic.left_handed)) {	/* Correct for 180 deg 
													relative rotation. */
		nx = (float) (oldx - middle.x);				/* Subtract offsets. */
		ny = (float) (oldy - middle.y);
													/* Rotate by Pi rads. */
		rotate(&rot1x, &rot1y, nx, ny, 0.0, -1.0);	
		rot1x = rot1x + (float) middle.x;			/* Add offsets. */
		rot1y = rot1y + (float) middle.y;
		
	} else {										/* No relative rotation. */
		rot1x = (float) oldx;
		rot1y = (float) oldy;
	}
	if(iflag)
		rot1x = -rot1x;
	rot1x -= (float) xoff;
	rot1y -= (float) yoff;
	rotate(&rot2x, &rot2y, rot1x, rot1y, sin2, cos2);
	rotate(&bix, &biy, rot2x, rot2y, -sin1, cos1);
	*newx = (int) bix;
	*newy = (int) biy;
	if(alc->debug) {
		fprintf(debug_channel, "Out of conv_coords.\n");
	}
}
/*
 *  C O N V _ C O O R D S  -- Converts from cytoscan coordinates in microns to 
 *							  microscope coordinates in microns.
 */ 
static 
conv_coords(oldx, oldy, newx, newy)
short	oldx, oldy; 
int	*newx, *newy;
{
float  nx, ny, rot1x, rot1y, rot2x, rot2y;

	if(alc->debug) {
		fprintf(debug_channel, "In conv_coords.\n");
	}

	rotate(&nx, &ny, (float) oldx, (float) oldy, sin1, cos1);
	
	if(alc->debug) 
		fprintf(debug_channel,"First trans: %f %f\n", nx, ny);
		
	rotate(&rot1x, &rot1y, nx, ny, -sin2, cos2);
	
	if(alc->debug) 
		fprintf(debug_channel,"Second trans: %f %f\n", rot1x, rot1y);
		
	rot1x += (float) xoff;
	if(iflag) {
		rot1x = -rot1x;
	}
	rot1y += (float) yoff;
	
	if(alc->debug) 
		fprintf(debug_channel,"Third trans: %f %f\n", rot1x, rot1y);
		
	if((mic.inverted && mic.left_handed) || (!mic.inverted && !mic.left_handed)) {	/* Correct for 180 deg 
													relative rotation. */
		rot1x = rot1x - (float) middle.x;				/* Subtract offsets. */
		rot1y = rot1y - (float) middle.y;
		rotate(&rot2x, &rot2y, rot1x, rot1y, 0.0, -1.0);	/* Rotate by Pi rads. */
		rot2x = rot2x + (float) middle.x;			/* Add offsets. */
		rot2y = rot2y + (float) middle.y;
		*newx = (int) rot2x;
		*newy = (int) rot2y;
		
	} else {										/* No relative rotation. */
	
		*newx = (int) rot1x;
		*newy = (int) rot1y;
	}
	
	if(alc->debug) 
		fprintf(debug_channel,"Fourth trans: %d %d\n", *newx, *newy);
	
	if(alc->debug)
		fprintf(debug_channel, "Out of conv_coords.\n");
}
/*
 * C Y T O _ N E -- Cytoscan coordinates to New England
 *
 */
cyto_ne(x, y, string)
short x, y;				/* cytoscan coords */
char *string;
{
	static struct coord scope, new, n, sub;
	int d1, d2, xd, yd, quad, tmp;
	char ch;
	
	/* convert cytoscan coords to microscope coords */
	conv_coords(x, y, &scope.x, &scope.y);
	if (alc->debug)
		fprintf(debug_channel, "x = %d y = %d scope = (%d, %d)\n", x, y, scope.x, scope.y);
	d1 = mic.Z50.x - mic.A10.x;
	d2 = mic.Z50.y - mic.A10.y;
	
	xd = scope.x - mic.A10.x;
	yd = scope.y - mic.A10.y;
	if (alc->debug)
		fprintf(debug_channel, "d1 = %d d2 = %d xd = %d yd = %d\n", d1, d2, xd, yd);
	
	if (abs(d1) > abs(d2)) {						/* slide horizontal */
		new.x = ((xd * 40000) / d1) + 10000;
		new.y = ((yd * 24000) / d2);
	} else {										/* slide vertical */
		new.x = ((xd * 24000) / d1);
		new.y = ((yd * 40000) / d2) + 10000;
	}		
	
	n.x = new.x / 1000;	/* letter */
	n.y = new.y / 1000;	/* number */
	
	/* calculate fractional remainder */
	sub.x = new.x - (n.x * 1000);
	sub.y = new.y - (n.y * 1000);
	if (abs(d1) > abs(d2)) {					/* slide horizontal */
		tmp = sub.x;
		sub.x = sub.y;
		sub.y = tmp;
		tmp = n.x;								/* swap n(x, y) so that number */
		n.x = n.y;								/* and letter are correct */
		n.y = tmp;
	}
	if (square(sub.x, sub.y) <= RADIUS2)
		quad = 0;
	else {
		if (sub.x <= 500) {
			if (sub.y <= 500)
				quad = 4;
			else {
				n.y++;				/*square above */
				sub.y = 1000 - sub.y;
				if (square(sub.x, sub.y) <= RADIUS2) quad = 0;
				else
					quad = 3;
			}
		} else {					/* x > 500 */
			if (sub.y < 500) {
				n.x++;				/* square above */
				sub.x = 1000 - sub.x;
				if (square(sub.x, sub.y) < RADIUS2) quad = 0;
				else
					quad = 2;
			} else {				/* square diagonally above */
				n.x++;
				n.y++;
				sub.x = 1000 - sub.x;
				sub.y = 1000 - sub.y;
				if (square(sub.x, sub.y) <= RADIUS2) quad = 0;
				else
					quad = 1;
			}
		}
	}
	if (alc->debug)
	 	fprintf(debug_channel, "new  = (%d, %d) n = (%d, %d) sub = (%d, %d) ", 
					new.x, new.y, n.x, n.y, sub.x, sub.y);
	if (n.x < 8)
		ch = n.x + 'A';
	else
		ch = n.x + 1 + 'A';
	sprintf(string, "%c%d/%d", ch, n.y, quad);
}
/*
 *  N E _ C Y T O -- New England references to Cytoscan coords
 *
 */
ne_cyto(string, cytox, cytoy)
char *string;
int *cytox, *cytoy;
{
	static struct coord new;
	int x, y, quad, d1, d2, xd, yd;
	char *str;

	if (alc->debug)
		fprintf(debug_channel, "string = %s cytox = %d cytoy = %d\n", string, *cytox, *cytoy);
	/* convert string into x, y, quad */
	while(*string == ' ')
		string++;			/* delete white space */
	if (*string <= 'I')
		x = *string - 'A';
	else
		x = *string - 'A' - 1;
	string++;
	str = string;
	while (*string != '/' && *string != '\0')
		string++;
	*string = '\0';
	y = atoi(str);
	string++;
	quad = atoi(string);
	if (alc->debug)
		fprintf(debug_channel, "x = %d y = %d quad = %d\n", x, y, quad);
	
	d1 = mic.Z50.x - mic.A10.x;
	d2 = mic.Z50.y - mic.A10.y;
	
	if (abs(d1) > abs(d2)) {				/* slide horizontal */
		xd = y - 10;
		yd = x;
		new.x = (xd * d1) / 40 + mic.A10.x;
		new.y = (yd * d2) / 24 + mic.A10.y;
	} else {								/* slide vertical */
		xd = x;
		yd = y - 10;
		new.x = (xd * d1) / 24 + mic.A10.x;
		new.y = (yd * d2) / 40 + mic.A10.y;
	}
	if (alc->debug)
	 	fprintf(debug_channel, "d1 = %d d2 = %d xd = %d yd = %d new = (%d, %d)\n", d1, d2, xd, yd, new.x, new.y);
	switch (quad) {
		case 0 :
			break;
		case 1 :
			new.y += DIST;
			if (abs(d1) > abs(d2))
				new.x -= DIST;
			else				
				new.x += DIST;
			break;
		case 2 :
			if (abs(d1) > abs(d2))
				new.y += DIST;
			else
				new.y -= DIST;
			new.x += DIST;
			break;
		case 3 :
			if (abs(d1) > abs(d2))
				new.y -= DIST;
			else				
				new.y += DIST;
			new.x -= DIST;
			break;
		case 4 :
			new.y -= DIST;
			if (abs(d1) > abs(d2))
				new.x += DIST;
			else
				new.x -= DIST;
			break;
	}
	convertback(new.x, new.y, cytox, cytoy);
	if (alc->debug)
	 	fprintf(debug_channel, "new = (%d, %d) cyto = (%d, %d)\n", new.x, new.y, *cytox, *cytoy);
}
/*
 * C Y T O _ S N E -- Cytoscan coordinates to Swedish New England
 *					  NB  The whole of this routine is calculated in
 *						  10's of microns instead of microns to avoid
 *						  overflow problems.
 */
cyto_sne(x, y, string)
short x, y;
char *string;
{
	static struct coord scope, new, n;
	int d1, d2, xd, yd, tmp;
	char ch;
	
	/* convert cytoscan coords to microscope coords */
	conv_coords(x, y, &scope.x, &scope.y);
	if (alc->debug)
	 	fprintf(debug_channel, "x=%d y=%d scope=(%d, %d)\n", x, y, scope.x, scope.y);
	
	d1 = (mic.Z50.x - mic.A10.x) / 10;
	d2 = (mic.Z50.y - mic.A10.y) / 10;
	
	xd = (scope.x - mic.A10.x) / 10;
	yd = (scope.y - mic.A10.y) / 10;
	
	if (abs(d1) > abs(d2)) {				/* slide horizontal */
		new.x = ((xd * 8090) / d1) + 2160;
		new.y = ((yd * 4850) / d2) + 150;
	} else {								/* slide vertical */
		new.x = ((xd * 4850) / d1) + 150;
		new.y = ((yd * 8090) / d2) + 2160;
	}	
	n.x = new.x / 100;	/* letter */
	n.y = new.y / 100;	/* number */
	if (abs(d1) > abs(d2)) {		/* swap number & letter if horizontal */
		tmp = n.x;
		n.x = n.y;
		n.y = tmp;
	}
	if (alc->debug)
		fprintf(debug_channel, "d1=%d d2=%d xd=%d yd=%d new=(%d, %d) n=(%d, %d)\n", d1, d2, xd, yd, new.x, new.y, n.x, n.y);
	
	if (n.x <= 25)
		ch = n.x + 'A';
	else
		ch = n.x - 26 + 'a';
	sprintf(string, "%c%d", ch, n.y);
}
/*
 * S N E _ C Y T O -- Swedish New England references to Cytoscan coords
 *					  NB  All calculations are in 10's microns instead
 *						  of microns to stop overflow problems.
 */
sne_cyto(string, cytox, cytoy)
char *string;
int *cytox, *cytoy;
{
	static struct coord new;
	int x, y, d1, d2, xd, yd;
	
	while (*string == ' ')
		string++;
	if (*string <= 'Z')
		x = *string - 'A';
	else
		x = *string - 'a' + 26;
	string++;
	y = atoi(string);
	
	d1 = (mic.Z50.x - mic.A10.x) / 10;
	d2 = (mic.Z50.y - mic.A10.y) / 10;
	
	if (abs(d1) > abs(d2)) {				/* slide horizontal */
		yd = (x - 1) * 100;					/* x + 0.5 - 1.5 */
		xd = (y * 100) - 2110; 				/* y + 0.5 - 21.6 */
		new.x = (((xd * d1) / 8090) * 10) + mic.A10.x;
		new.y = (((yd * d2) / 4850) * 10) + mic.A10.y;
	} else {								/* slide vertical */
		xd = (x - 1) * 100;					/* x + 0.5 - 1.5 */
		yd = (y * 100) - 2110; 				/* y + 0.5 - 21.6 */
		new.x = (((xd * d1) / 4850) * 10) + mic.A10.x;
		new.y = (((yd * d2) / 8090) * 10) + mic.A10.y;
	}
	if (alc->debug)
		fprintf(debug_channel, "x=%d y=%d d1=%d d2=%d xd=%d yd=%d new=(%d, %d)\n",
					x, y, d1, d2, xd, yd, new.x, new.y);
	convertback(new.x, new.y, cytox, cytoy);
}
/*
 * A D D _ T O _ L I S T --
 *
 */
add_to_list(x, y, str)
int *x, *y;
char *str;
{
	static struct coordlist *ptr;

	if (start == NULL) {
		start = (struct coordlist *) Calloc (1, sizeof(struct coordlist));
		ptr = start;
	} else {
		ptr->next = (struct coordlist *) Calloc (1, sizeof(struct coordlist));
		ptr = ptr->next;
	}
	ptr->next = NULL;
	ptr->pt.x = *x;
	ptr->pt.y = *y;
	strcpy(ptr->str, str);
}
/*
 * F R E E _ L I S T --
 *
 */
free_list()
{
	struct coordlist *ptr, *bptr;
	
	ptr = start;
	bptr = ptr;
	while(ptr != NULL) {
		ptr = ptr->next;
		Free(bptr);
		bptr = ptr;
	}
	start = NULL;
}
/*
 * P R I N T _ L I S T --
 *
 */
print_list()
{
	struct coordlist *ptr;
	int count, flag = 0;
	
	ptr = start;
	vclear(ENDLINE);
	screengo(1, 20);
	printf("Local Metaphase List\n\n");
	switch (alc->ccoord) {
		case MICROSCOPE :
			printf("     MicX        MicY         X           Y\n\n");
			break;
		case ENGLISHNE :
		case SWEDISHNE :
			printf("               NE Ref.        X            Y\n\n");
			break;
	};
	count = 0;
	while (ptr != NULL ) {
		count++;
		printf(" %20s%12d%12d\n", ptr->str, ptr->pt.x, ptr->pt.y);
		if (count > SCREENFUL) {
			count = 0;
			if (!go_on()) {
				flag = 1;
				break;
			}
			clearlns(DIRSTART, ENDLINE);
			screengo(DIRSTART, 0);
		}
		ptr = ptr->next;
	}
	if (!flag)
		if (go_on());
	vclear(OUTLINE, ENDLINE);
	setdisplay(BUST);
}
/*
 * M A K E _ M E T --
 *
 */
make_met()
{
	struct mult_slide *bp;
	
	bp = alc->curbay;
	if (bp->startptr != NULL) {
		fprintf(output, "Write new list over current one?\n");
		if (yesno(1))
			return(0);
		freelist();
	}
	bp->nameids.type = METFILE;
	makefilename(alc->metfile, (&bp->nameids));
	makenewlist();
	filelist();
	free_list();
	copied = 1;
	init_counts(bp, 1);
	showbays();
	displist();
}
/*
 * M A K E N E W L I S T --
 *
 */
int makenewlist()
{
	struct mult_slide *bp;
	struct coordlist *ptr;
	struct metlink *mlp;
	struct mlclust *clp;
	int count = 0;
	
	ptr = start;
	if (ptr == NULL) {
		fprintf(output, "The current local list is empty.\007\n");
		return(0);
	}
	while (ptr != NULL) {
		count++;
		ptr = ptr->next;
	}
	bp = alc->curbay;
	bp->totin = count;
	bp->usbltot = count;
	
	mlp = (struct metlink *) Calloc(count, sizeof(struct metlink));
	clp = (struct mlclust *) Calloc(count, sizeof(struct mlclust));
	bp->startptr = mlp;
	ptr = start;
	count = 1;
	while (ptr != NULL) {
		mlp->mlrec = clp;
		clp->recnr = count;
		clp->qrank = count;
		clp->fclass |= USABLE;			/* mark all entries usable */
		clp->xval = ptr->pt.x;
		clp->yval = ptr->pt.y;
		if (count == 1) {				/* start of list */
			mlp->prank = NULL;
			mlp->psub = NULL;
			mlp->pgrp = NULL;
		} else {
			mlp->prank = mlp - 1;
			mlp->psub = mlp - 1;
			mlp->pgrp = mlp - 1;
		}
		if (count == bp->totin) {		/* end of list */
			mlp->nrank = NULL;
			mlp->nsub = NULL;
			mlp->ngrp = NULL;
		} else {
			mlp->nrank = mlp + 1;
			mlp->nsub = mlp + 1;
			mlp->ngrp = mlp + 1;
		}
		if (ptr->next != NULL) {
			clp++;
			mlp++;
			count++;
		}
		ptr = ptr->next;
	}
	allcells(1);				/* make sure all necessary variables are set up */
	linksub();					/* relink llist according to allcells */
	return(1);
}


/*
 * C H E C K S T R -- checks strings entered for faults 
 *
 */
checkstr(str1, str2, x, y)
char *str1, *str2;
int *x, *y;
{
	if(!chekstr(str1)) {		/* Check for stupid strings. */
		screengo(ENDLINE, 0);
		fprintf(output, "Illegal value.  Reset to 0.\007\n");
	}
	if(!chekstr(str2)) {		/* Check for stupid strings. */
		screengo(ENDLINE, 0);
		fprintf(output, "Illegal value.  Reset to 0.\007\n");
	}
	*x = (int) (1000.0 * atof(str1));
	*y = (int) (1000.0 * atof(str2));
}
/*
 * C H E C K R E F -- check input NE reference 
 *
 */
checkref(string)
char *string;
{
	char str[20], *str1, *str2;
	int num;
	
	strcpy(str, string);
	str1 = str;
	while (*str1 == ' ')
		str1++;
	if (alc->ccoord == 2) {
		if (*str1 < 'A' || *str1 > 'Z' || *str1 == 'I') {
			fprintf(output, "Illegal first character.\007\n");
			return(0);
		} else {
			str1++;
			str2 = str1;
			while(isdigit(*str1) && *str1 != '\0')
				str1++;
			if (*str1 == '\0') {
				if (isnumber(str2))
					num = atoi(str2);
				else
					num = -1;
			} else {
				*str1 = '\0';
				if (isnumber(str2))
					num = atoi(str2);
				else
					num = -1;
				str1++;
			}
			if (num < 0 || num > 75) {
				fprintf(output, "Illegal number.\007\n");
				return(0);
			} else {
				if (*str1 == '\0')
					return(1);
				if (isnumber(str1))
					num = atoi(str1);
				else
					num = -1;
				if (num < 0 || num > 4) {
					fprintf(output, "Illegal quadrant number.\007\n");
					return(0);
				}
			}
		}
		return(1);
	} else {
		if ((*str1 >= 'a' && *str1 <= 'z') || (*str1 >='A' && *str1 <= 'Z')) {
			str1++;
			if (isnumber(str1))
				num = atoi(str1);
			else
				num = -1;
			if (num < 20 || num > 120) {
				fprintf(output, "Illegal number.\007\n");
				return(0);
			} else
				return(1);
		} else {
			fprintf(output, "Illegal character.\007\n");
			return(0);
		}
	}
}
/*
 * C O N V _ S T R
 *
 */
conv_str(num, str, array)
int *num;
char *str, *array[];
{
	strcpy(str, array[*num]);
}		

/*
 * I S N U M B E R --
 *
 */
 static int isnumber(string)
 char *string;
 {
 	char *str;
 
	str = string;	
 	while(*str != '\0') {
 		if (!isdigit(*str))
 			return(0);
 		else
 			return(1);
 	}
 }

/*
 * O F F S E T -- Rotate local to global and evaluate offsets.
 *
 */
offset(gx1, gy1, lx1, ly1)
float  gx1, gy1, lx1, ly1;
{
	float	nx, ny, rot1x, rot1y;

	rotate(&nx, &ny, lx1, ly1, sin1, cos1);
	rotate(&rot1x, &rot1y, nx, ny, -sin2, cos2);
	xoff = (int) (gx1 - rot1x);
	yoff = (int) (gy1 - rot1y);
}

