/*
 * mlist.c
 * read mataphase list via PIA
 * authors: Margaret Stark and Jim Piper, May 85
 *
 * Modifications
 *
 *	15 Jan 1991		CAS/GJP		debug_channel used when not open
 *	09 Apr 1990		dcb			Move open/close pia up a level (mlistfill)
 *	24 Aug 1989		dcb			Set newlist in mlistinit
 *	 3 Jun 1988		SCG			Remove bad records
 *	10 Aug 1987		SCG			Removed redrawing of bay in mlistfinish
 *	06 Aug 1987		SCG			moded showbays call
 *	13 Jul 1987		SCG			Major mods for 4-slide stage.  
 *								Met recs now held on a linked list.
 *	 5 Feb 1987		CAS			Gen tidy
 *	16 Jan 1987		SCG			Enlarged box for alc->totin in graphics.
 *	18 dec 1986		CAs			Changed hhh in metlistfinish
 *	 3 Dec 1986		CAS			Print status if openpia fails
 *	 2 Dec 1986		CAS			Fix around mlistfill - apparenlty lost in
 *								confusion with 'kermit'
 *	28 Nov 1986		SCG			Modified mlistfill to call jigfirst only
 *								when a non zero number of records have
 *								been read in.
 *	17 Nov 1986     SCG			Modified mlistinit, mlistfill, crossmets and
 *								jigrecord.
 *								Now totw = 0 on first entry unless mlistinit
 *								sets it to alc->totin.  See notes for reason
 *								behind this code modification.  The objective
 *								is to allow the merging of lists.
 *	 6 Oct 1986		CAS			Changed what code did if mlist buffer
 *								reached limit.  Also signal kmr before death
 *	25 Jul 1986		CAS			Added timeout to openpia call
 *	22 May 1986		CAS			Changed "" to <> for include
 *	28 Apr 1986    Denis    	'jigrecord' function to implement new
 *								record structure 'mipclust' structure in
 *								jigrecord to facilitate re-jig.
 *	10/4/86						in crossmets() for loop now begins at totw
 *								not tow+1
 *	15/11/85					coors. of crosses now linked to actual
 *								alc->slide position.
 *	3/11/85						alc->slide simulation display transferred 
 *								to file simslide.c
 *	22/10/85		Denis		lut setup into cubroutine for digitise
 *	16/8/85						alc->totin intiatilised -1 ( for header) 
 *								reduced -1 at end (???)
 *	10 Oct 1985		Mark M	New rank inserted
 *	8/10/85						removed call to crossmets() after 32 seen
 *								in list
 *	June 85			Denis R		progress display
 */

#include <cmdsys.h>
#include <kyy.h>
#include <slides.h>

extern FILE *debug_channel, *debugfp;

struct	metlink *grabnlnk();
struct  mlclust *grabmlbuf();
struct  mipclust *grabmbuf();
extern	baynn();
int	drawcross();

static int startflag;
static int totw, 							/* total recs before last read */
		   total, 							/* total including last read */
		   oldtot,							/* in append mode number of recs
		   									   in old list */
		   totbad;
		   
static int yk1,xk1;
static struct mipclust *mclp, 				/* variable buffer pointer */
					   *mlist;				/* first rec pointer */

static first = 2;

/*
 *
 *	M L I S T I N I T.  Initialise the list before data read.
 *
 */
 
mlistinit() 
{

	mclp = mlist;			/* Leave two records at front of list. */
	if (showmets) crossmets(0); 
	total = 0;
	freelist();				/* Free old list */
	oldtot = alc->curbay->totin = alc->curbay->usbltot = 
			 alc->curbay->revtot = totbad = 0;
	startflag = 1;
	alc->curbay->newlist = 1;	/* Start of something new */
}

static cheksafe() 
{
}
/*
 *	J I G R E C O R D  --  Copy from mipclust buffer to mlclust buffer.
 *
 */
static jigrecord(bufptr, mpt, nrecs) 
struct	mipclust *bufptr;
struct	mlclust  *mpt;
int	nrecs;
{
int	i;

	for (i = 0; i < nrecs; i++) {
		_strass(mpt, bufptr, sizeof(struct mipclust));
		mpt->xval = bufptr->mxpos;	
		mpt->yval = bufptr->mypos;	
		mpt->xwide = bufptr->mxwide;
		mpt->ywide = bufptr->mywide;
		mpt->fclass = mpt->quality = mpt->ufrag = 0;
		mpt->udic = mpt->chrom = mpt->uring = -1;	/* Do not default to
													   zero */
		mpt->inbox = notsecondary ? bufptr->mfscore :
				bufptr->qscore;						/* qscore or mfscore */
		bufptr++;									/* increment pointers */
		mpt++;
	}
	if(alc->debug == 3) {					/* Dump list data in high debug
											   mode */
		mpt -= nrecs;
		fprintf(debug_channel,"Linked list block\n");
		fprintf(debug_channel,"-----------------\n");
		for (i = 0; i < nrecs; i++) {
			fprintf(debug_channel,"%3d: %6d %6d %6d %6d %4d %4d %4d\n", i, 
					mpt[i].xval, mpt[i].yval, mpt[i].xwide, mpt[i].ywide,
					mpt[i].zval, mpt[i].nobj, mpt[i].sf); 
		}
	}
}

/*
 *	M L I S T F I L L  --  Read from buffer into list at each swathe.
 *	Now assumes PIA is opened before this function is called
 *	and closed after it returns
 */
mlistfill() 
{
int i, j, k, jnow, size, totmax, nread, bn, badrecs;

	size = 1024;					/* Size of chunk to get from miproc */
	nread = 0;						/* amount read this call to mlistfill */
	/*
	 *	Read PIA until no more data or no more space.
	 *	If no more space - dump rest of data
	 */
	while (1) {
		totmax = (MAXREC - (size/sizeof(struct mipclust) + 1));
		if (total >= totmax) { 
			size = (MAXREC - total) * sizeof(struct mipclust);
			if (size <= 0) {
				fprintf(stderr, "metaphase list overflow\n");
				flushpia();
				break;
			}
		}
		mclp = mlist + total; 
		if(startflag) {						/* Read the first record and dump */
			i = readpia((char *) mclp, sizeof(struct mipclust),alc->piabase);
			i = 0;
			startflag = 0;
		} else								/* Read a chunk */
			i = readpia((char *) mclp, size,alc->piabase);
		
		nread += i;
		if (i == 0) break;
		if ((i % sizeof(struct mipclust)) != 0) {	
											/* Something's wrong */
			fprintf(stderr,"mlistfill(): loss of %d-byte synch %d\n",
				sizeof(struct mipclust),i);
			kill(alc->kmr_pid,MF_DIES);
			exit(1);
		}
		checkswathe(&i, &badrecs);
		totbad += badrecs;
		if (alc->debug)
			fprintf(debug_channel, "MLF - size %d, totin %d tot_max %d\n",
					size, alc->curbay->totin,totmax);
		total += i / sizeof(struct mipclust);
		alc->curbay->totin += i / sizeof(struct mipclust);
				
	}
	if(alc->debug == 3) {					/* Dump miproc data in high debug
											   mode */
		fprintf(debug_channel,"Data from Miproc\n");
		fprintf(debug_channel,"----------------\n");
		for (j = totw; j < total; ++j)
			fprintf(debug_channel,"%3d: %6d %6d %6d %6d %4d %4d %4d\n", j, 
					mlist[j].mxpos, mlist[j].mypos, mlist[j].mxwide, 
					mlist[i].zval, mlist[i].nobj, mlist[i].sf); 
	}
	if (showmets) crossmets(1);   
}

/*
 *	F L U S H P I A  --  Read pia till no more data comes.
 *
 */
 
static
flushpia()
{
	char	*buffer;
	
	if (buffer = (char *) Malloc(1024)) {
		while (readpia(buffer,1024,alc->piabase))
			;
		Free(buffer);
	}
}

/*
 *  M L I S T F I N I S H.  Tidy up the list after data read.
 *
 */
 
mlistfinish(out)
FILE *out;
{
	int		bn;

	if (italk)
		fprintf(output,"%d records received",alc->curbay->totin);
	if(alc->devel)
		fprintf(output, "Total bad records found: %d\n", totbad);
	if(alc->debug)
		fprintf(debug_channel, "Total bad records found: %d\n", totbad);
	if(alc->curbay->totin) {	/* Set flag to indicate we have a list and rank. */
		copytolist(mlist, alc->curbay->totin);
		frank();
	}
	if (showmets) crossmets(2);
}

/*
 *
 * C R O S S M E T S.   Set display.
 *
 *			set look-up tables for alc->slide-simulation display, crosses, 
 *			etc.  Mapping : -7500 - 7500 to 1000 - 4000
 *					        -10000 - 20000 to same scale, centred on 2000
 */
crossmets(ii) 
{
int jnow,ix,iy,bn;
char str[8];

	switch (ii) {
		case 0: 
			totw = 0; 
			xk1 = SLMXTOP;
			yk1 = SLMYBOT;
			moveto(2000,yk1-340);
			colour (2);
			tsize (64,64);
			text( "Clusters found ... ");
			tsize(64,64); 
			return;
		case 1: 
			for (jnow = totw; jnow < total; ++jnow ) {
				ix = ((mlist[jnow].mxpos) / CONV ) + XMID ; 
				iy = YMID - ((mlist[jnow].mypos) / CONV ) ; 
				drawcross(ix, iy, 1);
			}
			moveto (xk1+200,yk1-340);
			colour(255);
			pixbegin(8,1,1,640,200);	/*clear the old value (blue wash)*/
			pixel(4); 
			pixend();
			sprintf ( str, "%4d", alc->curbay->totin);
			moveto (xk1+300,yk1-315);
			colour(1);
			tsize(128,128);
			text (str);
			tsize(64,64); 
			break;
		case 2:
			colour(255);
			update();
	}
	totw = total;
}
/*
 *
 * M B U F A L L O C.   Grabs mipclust buffer for temp. holding list.
 *
 */
mbufalloc(nrecs)
int	nrecs;
{

	if((mlist = grabmbuf(nrecs)) == NULL) {
		if(alc->debug)
			fprintf(debug_channel, "Grabmbuf: Unable to grab mlist buffer\n");
		return(0);
	}
	return(1);
}
/*
 *
 * F R E E M B U F.   Frees mipclust buffer for temp. holding list.
 *
 */
mbuffree()
{

	Free(mlist);
	mlist = NULL;
	
}
/*
 *
 *  C O P Y T O L I S T
 *
 *			Grab space for list, copy buffer to it, link and attach to
 *			mult_slides struct.
 */
copytolist(bufptr, nrecs)
struct mipclust *bufptr;
int	nrecs;
{
struct	metlink *linkptr;
struct  mlclust *mptbuf;

	if (alc->debug)
		fprintf(debug_channel,"copytolist: in\n");
	linkptr = grabnlnk(nrecs);
	mptbuf = grabmlbuf(nrecs);
	jigrecord(bufptr, mptbuf, total);
	linkblk(mptbuf, linkptr, alc->curbay->totin);
	if (alc->debug)
		fprintf(debug_channel,"copytolist: out\n");
	
}
/*
 *	D R A W C R O S S
 */
drawcross(ix, iy, col)
int	ix, iy, col;
{

	colour(col);
	moveto(ix,iy);
	moveby (-10,-10);
	lineby (20,20);
	moveby (-20,0);
	lineby (20,-20);

}	

/*
 * C H E C K S W A T H E --- remove bad records from a subset of the list
 *
 */
checkswathe(stot, badrecs)
int *stot, *badrecs;
{
int tot, gotit;
register int i, j;
register struct mipclust *lmcp;

	tot = (*stot) / sizeof(struct mipclust);
	for(i = 0, j = 0, lmcp = mclp, *badrecs = 0; i < tot; i++) {
		if(!checkrec(lmcp + i))
			(*badrecs)++;
		else {
			if((*badrecs))
				_strass(lmcp + j, lmcp + i, sizeof(struct mipclust));
			j++;
		}
	}
	*stot = (tot - *badrecs) * sizeof(struct mipclust);
	if(alc->devel)
		fprintf(output, "bad records found: %d\n", *badrecs);
	if(alc->debug)
		fprintf(debug_channel, "bad records found: %d\n", *badrecs);
}		

/*
 * C H E C K R E C --- check the nine features in the miproc cluster
 *		record for stupid values; return 0 if so 1 otherwise; if the miproc
 *		worked properly this would not be necessary.
 */
checkrec(rptr)
register struct mipclust *rptr;
{
	if(rptr == NULL) 
		return(0);
	else {
		if(rptr->aobj <= 0) {
			if(alc->debug == 3) 
				fprintf(debug_channel, "aobj fail, aobj: %d\n", rptr->aobj);
			return(0);
		}
		if(rptr->nobj <= 0) {
			if(alc->debug == 3) 
				fprintf(debug_channel, "nobj fail, nobj: %d\n", rptr->nobj);
			return(0);
		}
		if(rptr->octa <= 0) {
			if(alc->debug == 3) 
				fprintf(debug_channel, "octa fail, octa: %d\n", rptr->octa);
			return(0);
		}
		if(((int) rptr->skt) <= 0) {
			if(alc->debug == 3) 
				fprintf(debug_channel, "skt fail, skt: %d\n", (int) rptr->skt);
			return(0);
		}
		if(rptr->mxwide <= 0) {
			if(alc->debug == 3) 
				fprintf(debug_channel, "xwide fail, xwide: %d\n", rptr->mxwide);
			return(0);
		}
		if(rptr->mywide <= 0) {
			if(alc->debug == 3) 
				fprintf(debug_channel, "ywide fail, ywide: %d\n", rptr->mywide);
			return(0);
		}
		if(rptr->meanod <= 0) {
			if(alc->debug == 3) 
				fprintf(debug_channel, "meanod fail, meanod: %d\n", rptr->meanod);
			return(0);
		}
		if(rptr->sf < 100) {
			if(alc->debug == 3) 
				fprintf(debug_channel, "sf fail, sf: %d\n", rptr->sf);
			return(0);
		}
		if(((int) rptr->dv) <= 0) {
			if(alc->debug == 3) 
				fprintf(debug_channel, "dv fail, dv: %d\n", (int) rptr->dv);
			return(0);
		}
		return(1);
	}
}			

