/*
 *      M L R _ M E N U . C  --  Met list review menu stuff
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1991)
 *
 *  Date:    18th July 1991
 *
 *  Modifications
 *
 *	 1 Aug 1991		CAS		conversionstr duplicate def in mic_conv - hook into that one
 *
 */
#include <cmdsys.h>
#include <kyy.h>
#include <slides.h>
#include <errno.h>
#include <metscan.h>
#include <disp_defs.h>

/*
 *	External functions used
 */
extern int nowfork();
extern int call_subset(), select(), score(), rscore(), mic_conv();
extern int printlist(), scoresheet(), move_to_bay(), set_curbay();
extern int filelist(), showlist(), read_newlist();
extern int fixform(), setspacing();
extern int chekudic(), chekuring(), chromtog(), fxtog();
extern int draw_scores_box();
/*
 *	Internal forward declarations
 */
int		inout();

/*
 *	External variables used
 */
extern int outstyle;

extern char comstring[2][40], scecom[40], chromcom[40];		/* Menu stuff - imported */
extern char *listclasstags[];

/*
 *	Internal static constants/variables
 */
#ifdef CAS_DEFUNCT		/* Duplicate in mic_conv */
static char *conversionstr[] = {		/* type of conversion string */
	"conversion off",
	"microscope verniers",
	"england finder (Graticules Ltd.)",
	"england finder (Albert Sass)"
};
#endif
extern char *conversionstr[];
static char forstr[10] = "short";

/*
 *  M E T L I S T  --
 *
 */
metlist()
{
	int  (*linkerbug)(), bayno, Ex_cmd, mstage;
	char convstr[40], buf[15];

	strcpy(convstr, conversionstr[alc->ccoord]);
	qclear();
	fixrgb();
	bayno = baynn(alc->curbay);
	outstyle = 1;	/* Set long initially */
	fixform(forstr);		/* now toggle it to short */
	mksubtxt();		/* Make new sub text */
	showbays();
	displist();
	if ((alc->config & (CONF_MSTAGE|CONF_FIP)) == CONF_MSTAGE) {
		if ((mstage = stageopen("/t6")) < 0) {
			alc->config &= ~CONF_MSTAGE;
			printf("Failed to access motorised stage (%d)\n",errno);
		}
	}
	newmenu ("Metaphase List Review"); 
	cmmd ("subset selection;:l6 :k10 %-s:l7 :k10 %-s:l8 :k10 %-s %g",
			isub,esub,rsub,call_subset); mouse(LEFTBT);
	if (alc->config & CONF_FIP) {
		cmmd (":L3;select cells;%g", select); mouse(MIDBT);
		if (alc->toxicmu) {		/* Call up aberrations package */
			sprintf(buf, "-s %d", alc);	/* Pass abe the alc ptr as param to -s */
			linkerbug = nowfork;
			cmmd(":L3;aberration scoring;%x%x%f", "abe", buf, linkerbug);
		} else {
			cmmd (":L3;score cells;%g", score); mouse(RIGHTBT);
		}
		cmmd (":L3;retrain score;%g", rscore);
	} else {
		if (alc->config & CONF_MSTAGE) {
			cmmd (":L3;select cells;%g%+i", select, mstage); mouse(MIDBT);
			cmmd (":L3;score cells;%g%+i", score, mstage); mouse(RIGHTBT);
		} else {
			cmmd (":L3;select cells              (Not available on this seat)");
			cmmd (":L3;score cells               (Not available on this seat)");
		}
		cmmd (":L3;retrain score             (Not available on this seat)");
	}
	cmmd (":L3;print score sheet;%f", scoresheet);
	cmmd (":L3;print list;%f",printlist);
	cmmd (":L3;list input/output;%f", inout); mouse(LONGMB);
	if (alc->seat == 1) {
		cmmd (":L3;bay;:k40%d%f", &bayno, move_to_bay);
	} else
		cmmd (":L3;bay                       (Not available on this seat)");
	linkerbug = mic_conv;
	cmmd (":L3;coordinate conversion;:k40%-s%f", convstr, linkerbug);
	cmmd(":L3;exit <return>"); mouse(LONGLB); Ex_cmd = mmcmd;
	while (thismu() && (nncmd != Ex_cmd));
	if ((alc->config & CONF_MSTAGE|CONF_FIP) == CONF_MSTAGE)
		stageclose(mstage);
	return(0); 
}


/*
 * I N O U T --
 *
 */
inout()
{
	struct mult_slide *hold;
	int bayno, Ex_cmd;
	static int dblsp = 1;		/* Double line spacing on output, default ON */
	
	setspacing(&dblsp);
	hold = alc->curbay;
	bayno = baynn(alc->curbay);
	newmenu("List Input/Output");
	if (alc->config & CONF_FIP)
		cmmd("current bay number (no move);:k40%d%f", &bayno, set_curbay);
	else
		cmmd("current bay number (Not available on this seat)");
	cmmd("show list;%f",showlist);mouse(RIGHTBT);
	cmmd("file list;%f",filelist); mouse(LEFTBT);
	cmmd("read list;%g",read_newlist); mouse(MIDBT);
	cmmd("format spec.;:k40%-s%f",forstr,fixform); mouse(LONGMB);
	cmmd("double line spaced output.;:k40%T%f",&dblsp,setspacing); mouse(LONGRB);
	cmmd("exit <return>"); Ex_cmd = mmcmd; mouse(LONGLB);
	while (thismu() && (nncmd != Ex_cmd));
	
	alc->curbay = hold;
	mksubtxt();						/* Make new sub text */
	showbays();						/* Redraw new bay */
	displist();						/* Show new list subset crosses */
}

/* listops.c */

#define FORW 1
#define BACKW -1

#define XSTART 128
#define YSTART 128


/*
 *
 *  S E L E C T  --
 *
 */
select(mstage) 
int		mstage;
{ 
	int 	ii, ir, Jcmd, Ucmd, Ocmd, Scmd, Rcmd, Qtcmd, Com1_cmd, Com2_cmd;
	int		otype, Pcmd, kbwas, pclass, nclass, Nrcmd, Dtcmd;
	struct	mult_slide *bp;
	char	buf[20];

	bp = alc->curbay;
	kbwas = kbegin;							/* Reset display to col 40 */
	kbegin = 40;
	
	ourstyle = outstyle ? outstyle : 1;
	if(bp->startptr == NULL) {		/* Check for list presence */
		fprintf(output, "No list in!\n");
		return(0);
	}
	if (alc->config & CONF_FIP)
		movespeed();
	else if (alc->config & CONF_MSTAGE)
		mstagedatum(mstage);
	if (alc->config & CONF_FIP|CONF_MSTAGE)
		setslrcd(&ii, &ir, &otype, mstage);				/* Get attr of current rec */

	newmenu("Select Cells");
	cmmd("junk;%-s",listclasstags[Junk]); mouse(RIGHTBT); Jcmd = mmcmd;
	cmmd("usable;%-s",listclasstags[Usable]); mouse(MIDBT); Ucmd = mmcmd;
	cmmd("other metaphase;%-s",listclasstags[Othermet]); mouse(LEFTBT); Ocmd = mmcmd;
	cmmd("record No.;%d", &ii); Scmd = mmcmd;
	cmmd("rank;%d", &ir); Rcmd = mmcmd;
	cmmd("previous record");mouse(LONGMB); Pcmd = mmcmd;
	cmmd("next record [Return]"); Nrcmd = mmcmd;
	cmmd("determine offsets"); Dtcmd = mmcmd;
	if (alc->config & CONF_MSTAGE) {
		sprintf(buf, "-s %d ",alc->ddgschannel);
		cmmd("measure/count;%x%x%f", "measure", buf, nowfork);
		mouse(LONGRB);
	}
    cmmd("exit"); Qtcmd = mmcmd;

	do {
		thismu();
		pclass = classnum(bp->currptr->mlrec);
		if(nncmd == Ucmd || nncmd == Ocmd || nncmd == Jcmd) {
			if(nncmd == Ucmd)
				classline(Usable);
			else if(nncmd == Jcmd)
				classline(Junk);
			else if(nncmd == Ocmd)
				classline(Othermet);
			updatebay(bp);
			relink(otype, BACKW);
			if(pclass != nncmd) {
				displin(0, nncmd);
				if(pclass)
					displin(0, pclass);
				pclass = nncmd;
			}	
			putprompt();
			if(!nextrec(FORW)) 
				fprintf(output,"End of subset list\007\n");
		} else if(nncmd == Pcmd) {
			if(!nextrec(BACKW))
				fprintf(output,"Beginning of subset list\007\n");
		} else if(nncmd == Scmd) {
			if(!findrec(ii))
				fprintf(output,"Request outside list bounds\007\n");
		} else if(nncmd == Rcmd) {
			if(!findrank(ir))
				fprintf(output,"Request outside list bounds\007\n");
		} else if(nncmd == Nrcmd || !nncmd)
			if(!nextrec(FORW)) 
				fprintf(output,"End of subset list\007\n");
				
		setslrcd(&ii, &ir, &otype, mstage);	/* Get attr of current rec */
		if(pclass)  displin(0, pclass);
		if(nclass = classnum(bp->currptr->mlrec))  displin(0, nclass);
		displin(0,Scmd);			/* Redisplay current record. */
		displin(0,Rcmd);			/* Redisplay current rank. */
		displin(0, 0);				/* Redisplay menu header. */
		putprompt();
		if(nncmd == Dtcmd) {
			centrit(mstage);
			putprompt();
		}
		if(!nncmd)						/* No need to redraw whole screen. */
			setdisplay(NOTBUST);
	} while (nncmd != Qtcmd);
	kbegin = kbwas;

	askfile(SELECT); 
}

/*
 *
 *  R S C O R E --- Insert retrain score to all members of a list
 *
 */
rscore() 
{ 
int ii, ir, Sccmd, Scmd, Rcmd, otype, Pcmd, kbwas, Nrcmd,
	metscore, oldscore, i, Qtcmd, scarr[MAXSCORE + 1], Dtcmd;
struct	mult_slide *bp;
struct mlclust *sptr;
char	sbuf[256];

	bp = alc->curbay;
	kbwas = kbegin;							/* Reset display to col 40 */
	kbegin = 40;
	
	ourstyle = outstyle ? outstyle : 1;
	if(bp->startptr == NULL) {		/* Check for list presence */
		fprintf(output, "No list in!\n");
		return(0);
	}
	/*
	 * fill the scarr array; first initialise to zero
	 */
	for(i = 0; i <= MAXSCORE; i++)
		scarr[i] = 0;
		
	for(i = 0, sptr = bp->startptr->mlrec; i < bp->totin; i++, sptr++) { 
		/*
		 * if by some miracle quality was set outside bounds reset to zero
		 */
		if(((metscore = (int) sptr->quality) > MAXSCORE) || (metscore < 0))
			sptr->quality = 0;
		else
			scarr[metscore]++;
	}
	
	/*
	 * this is graphics score display; but I'm not sure where to put it
	 */
/*	 
	disptabl(XSTART, YSTART, 1);
	dispscore(XSTART, YSTART, scarr, 1);
*/
	/*
	 * Move to the first record in the list.  In this menu the default
	 * move is to the next record number not the next subset record
	 */
	ii = 1; 
	findrec(ii);
	 	
	movespeed();
	setslrcd(&ii, &ir, &otype,-1);				/* Get attr of current rec */
											/* Get score of first rec */
	oldscore = metscore = bp->currptr->mlrec->quality;	

	newmenu("Retrain Score");
	cmmd("score;%-d", &metscore); mouse(RIGHTBT); Sccmd = mmcmd;
	cmmd("record No.;%d", &ii); Scmd = mmcmd;
	cmmd("rank;%d", &ir); Rcmd = mmcmd;
	cmmd("previous record");mouse(LONGMB); Pcmd = mmcmd;
	cmmd("next record [Return]"); Nrcmd = mmcmd;
	cmmd("determine offsets"); Dtcmd = mmcmd;
	cmmd("exit"); Qtcmd = mmcmd;

	do {
		thismu();
		dispscr(scarr);
		putprompt();
		if(nncmd == Sccmd) {
			fprintf(output, "Entered scoring loop.\n");
			screengo(17, 10);
			fprintf(output, "Enter code (0 - %d) or x to exit\n", MAXSCORE);
			putprompt();
			while(1) {
				while(1) {
					gets(sbuf);
					metscore = atoi(sbuf);
					if((sbuf[0] == 'x') || (metscore >= 0 && metscore <= MAXSCORE))
						break;
					else {
						fprintf(output," \007Illegal score\n");
						putprompt();
					}
				}
				if(sbuf[0] == 'x') {		/* Reset metscore and exit loop */
					metscore = bp->currptr->mlrec->quality;
					break;
				}
				bp->currptr->mlrec->quality = metscore;
				class_score(metscore);		/* Associate class with score */
				dispscore(XSTART, YSTART, scarr, 0);
				scarr[oldscore]--;
				scarr[metscore]++;
		
/*				dispscore(XSTART, YSTART, scarr, 1); */
				dispscr(scarr);
				
				displin(0, Sccmd);			/* Display this score */
				
				ii++;						/* To request next record */
				if(!findrec(ii)) {
					screengo(ENDLINE, 0);
					fprintf(output,"End of list\007\n");
				}
				setslrcd(&ii, &ir, &otype, -1);	/* Get attr of current rec */
				oldscore = metscore = bp->currptr->mlrec->quality;
				displin(0, Sccmd);
				displin(0, Scmd);			/* Redisplay current record. */
				displin(0, Rcmd);			/* Redisplay current rank. */
				displin(0, 0);				/* Redisplay menu header. */
				putprompt();
			}
			clearlns(16,16);
			putprompt();
			fprintf(output, "Exited scoring loop.\n");
			putprompt();
		} else if(nncmd == Pcmd) {
			ii--;
			if(!findrec(ii))
				fprintf(output,"Request outside list bounds\007\n");
		} else if(nncmd == Scmd) {
			if(!findrec(ii))
				fprintf(output,"Request outside list bounds\007\n");
		} else if(nncmd == Rcmd) {
			if(!findrank(ir))
				fprintf(output,"Request outside list bounds\007\n");
		} else if(nncmd == Nrcmd || !nncmd) {
			ii++;
			if(!findrec(ii))
				fprintf(output,"Request outside list bounds\007\n");
		}
		setslrcd(&ii, &ir, &otype, -1);	/* Get attr of current rec */
		oldscore = metscore = bp->currptr->mlrec->quality;
		displin(0, Sccmd);
		displin(0, Scmd);			/* Redisplay current record. */
		displin(0, Rcmd);			/* Redisplay current rank. */
		displin(0, 0);				/* Redisplay menu header. */
		putprompt();
		if(nncmd == Dtcmd) {
			centrit();
			putprompt();
		}
		if(!nncmd)						/* No need to redraw whole screen. */
			setdisplay(NOTBUST);
	} while (nncmd != Qtcmd);
	kbegin = kbwas;
	
/*
	disptabl(XSTART, YSTART, 0);
	disptabl(XSTART, YSTART, scarr, 0);
*/	
	askfile(RSCORE); 
}

/*
 *
 *  S C O R E  --
 *
 */
score(mstage) 
int		mstage;
{ 
int ii, ir, Jucmd, Otcmd, Scmd, Pcmd, Rcmd, Qtcmd, otype, Dtcmd,
	Com1_cmd, Com2_cmd, Sce_cmd=0, Ucmd, Jcmd, kbwas, Chr_cmd, Dicmd=0,
	Ascmd=0, Frcmd=0, Nrcmd;
	char	fxstrg[9],dnum[4],anum[4];
	char	chstr[40];

	kbwas = kbegin;							/* Reset display to col 40 */
	kbegin = 40;

	ourstyle = outstyle ? outstyle : 1;
	
	if(alc->curbay->startptr == NULL) {		/* Check for list presence */
		fprintf(output, "No list in!\n");
		return(0);
	}
	strcpy(chstr, chromcom);
	intens(1);
	draw_scores_box();
	
	if (alc->config & CONF_FIP)
		movespeed();
	else if (alc->config & CONF_MSTAGE)
		mstagedatum(mstage);
	if (alc->config & CONF_FIP|CONF_MSTAGE)
		setscrcd(&ii, &ir, &otype, dnum, anum, fxstrg, mstage);
	
	newmenu("Score Cells");
	cmmd("junk;%-s",listclasstags[Junk]); mouse(RIGHTBT);
			Jucmd = mmcmd;
	cmmd("other metaphase;%-s",listclasstags[Othermet]); mouse(LONGRB);
			Otcmd = mmcmd;
	cmmd("fragile x;%-s%f", fxstrg, fxtog); mouse(LEFTBT);
			Frcmd = mmcmd;
	cmmd("comment 1:;:k20?-?%39S;t5",comstring[0]);
			Com1_cmd = mmcmd;
	cmmd("        2:;:k20?-?%39S",comstring[1]);
			Com2_cmd = mmcmd;
	if (alc->toxicmu) {							/* Extended menu only for
												   toxicology work */
		cmmd("dicentrics;%s%f", dnum, chekudic);
				Dicmd = mmcmd;
		cmmd("acentrics;%s%f", anum, chekuring);
				Ascmd = mmcmd;
		cmmd("SCE      :;:k20?-?%39S",scecom);
				Sce_cmd = mmcmd;
	} 
	cmmd("karyotype:;:k20?-?%-S%?39S%f", chstr, chromtog);
			Chr_cmd = mmcmd;
	cmmd("record No.;%d", &ii);
			Scmd = mmcmd;
	cmmd("rank;%d", &ir);
			Rcmd = mmcmd;
	if (alc->config & CONF_FIP) {
		cmmd("determine offsets");
		Dtcmd = mmcmd;
	}
	cmmd("previous record");mouse(LONGMB);
			Pcmd = mmcmd;
	cmmd("next record [Return]");mouse(MIDBT);
			Nrcmd = mmcmd;
    cmmd("exit"); mouse(LONGLB);
			Qtcmd = mmcmd;

	do {
		thismu();
		if(nncmd == Com2_cmd) { 
			displin(0, Com1_cmd);
			displin(0, Com2_cmd);
			putprompt();
		}		
		if(alc->toxicmu && nncmd == Sce_cmd) { 
			displin(0, Sce_cmd);
			putprompt();
		}		
		if(nncmd == Chr_cmd) { 
			displin(0, Chr_cmd);
			putprompt();
		}		
										/* Stepping thru the met list. */
		if (!nncmd || nncmd == Scmd || nncmd == Pcmd || nncmd == Jucmd || 
						nncmd == Rcmd || nncmd == Otcmd || nncmd == Nrcmd) {
						
			putcom();					/* Putcom before move because possibly
										   more than one comment */
			if(nncmd == Jucmd || nncmd == Otcmd){
				if(nncmd == Jucmd)
					classline(Junk);
				if(nncmd == Otcmd)
					classline(Othermet);
				relink(otype,BACKW);
				displin(0, Jucmd);
				displin(0, Otcmd);
				putprompt();
				if(!nextrec(FORW)) 
					fprintf(output,"End of subset list\007\n");
			} else if(nncmd == Pcmd) {
				relink(otype,BACKW);
				if(!nextrec(BACKW))
					fprintf(output,"Beginning of subset list\007\n");
			} else if(nncmd == Scmd) {
				relink(otype,BACKW);
				if(!findrec(ii))
					fprintf(output,"Request outside list bounds\007\n");
			} else if(nncmd == Rcmd) {
				relink(otype,BACKW);
				if(!findrank(ir))
					fprintf(output,"Request outside list bounds\007\n");
			} else if(!nncmd || nncmd == Nrcmd) {
				relink(otype,BACKW);
				if(!nextrec(FORW)) 
					fprintf(output,"End of subset list\007\n");
			}
										/* Set display bits and move */
			setscrcd(&ii, &ir, &otype, dnum, anum, fxstrg, mstage);
			displin(0, 0);				/* Displins faster than display */
			displin(0, Jucmd);
			displin(0, Otcmd);
			displin(0, Frcmd);
			if(alc->toxicmu) {
				displin(0, Dicmd);
				displin(0, Ascmd);
				displin(0, Sce_cmd);
			}
			displin(0, Chr_cmd);
			displin(0, Com1_cmd);
			displin(0, Com2_cmd);
			displin(0, Scmd);
			displin(0, Rcmd);
			putprompt();
		}
		if(alc->toxicmu && nncmd == Dicmd) {
			chekudic(dnum);
			displin(0, Dicmd);
			putprompt();
		}
		if(alc->toxicmu && nncmd == Ascmd) {
			chekuring(anum);
			displin(0, Ascmd);
			putprompt();
		}
		if(nncmd == Dtcmd) {
			centrit(-1);
			putprompt();
		}
		if(!nncmd)						/* Dont redraw whole screen */
			setdisplay(NOTBUST);
	} while (nncmd != Qtcmd);
	kbegin = kbwas;

	intens(0);
	draw_scores_box();
	intens(1);

	askfile(SCORE); 
}

