/*
 *      M S T A G E . C  --  Prior stage handling stuff
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1991)
 *
 *  Date:    24th June 1991
 *
 *  Modifications
 *
 *	31 Jul 1991		CAS		Further tidy -> remove cconv stuff to cconv
 *
 */
#include <stdio.h>
#include <sgstat.h>
#include <cconv.h>

static struct microscope Prior;
struct sgbuf saved_mode;

/*
 *	External stuff
 */
extern int errno;

/*
 *	S T A G E O P E N  --
 *
 */
stageopen(line)
char	*line;
{
	int		fd;
	if ((fd = open(line,3)) >= 0) {
		raw(fd,&saved_mode);
		baud(fd,9600);
		if (micread("PRIOR",&Prior) == 0) {
			stageclose(fd);
			fd = -1;
			errno = 0;
		}
	}
	return(fd);
}

/*
 *	C L O S E S T A G E  --
 *
 */
stageclose(fd)
int		fd;
{
	cook(fd,&saved_mode);
	close(fd);
}

mstagedatum(fd)
int		fd;
{
	static int datumed = 0;
	if (fd >= 0) {
		if (!datumed) {
			mstagepos(fd,0,0);
			printf("Move to N38 datum position\n"); 
			go_on();
			priorcommand(fd,"Z\015",NULL);
		}
		datumed = 1;
	}
}

/*
 *	M S T A G E P O S  --
 *
 */
mstagepos(fd,x,y)
int		fd, x, y;
{
	struct	coord	from, to;
	from.x = x;
	from.y = y;
	coordconvert(getcytoscan_mic(),&Prior,&from,&to);
	priormove(fd,to.x,to.y);
}

mstagegetpos(fd,x,y)
int		fd, *x, *y;
{
	char	buffer[100], *bptr;
	struct microscope *cyto;
	struct	coord	from, to;

	priorflush(fd);
	priorcommand(fd,"P\015",buffer);
	from.x = atoi(buffer);
	bptr = buffer;
	while (*bptr && ((*bptr & 0x7f) != ','))
		bptr++;
	if (*bptr) bptr++;
	from.y = atoi(bptr);
/*	printf("PRIOR pos: \"%s\" - %d %d",buffer,from.x,from.y); */

	coordconvert(&Prior,getcytoscan_mic(),&from,&to);
/*	printf("  --> %d %d\n",to.x,to.y); */
	*x = to.x;
	*y = to.y;
}


/*
 *	P R I O R M O V E  --  
 *
 */
priormove(fd,x,y)
int		fd, x, y;
{
	char	buf[80];
	sprintf(buf,"G,%d,%d\015",x,y);
	priorcommand(fd,buf,NULL);
}

/*
 *	P R I O R C O M M A N D  --  Send command to stage - very cautiously
 *
 */
priorcommand(fd,com,reply)
int		fd;
char	*com, *reply;
{
	int		len, timeout;

/*	printf("PRIOR (%d): %s",fd,com); */
	len = strlen(com);
	while (len) {
		if (write(fd,com++,1) != 1) {
			printf("mstage - write error %d\n",errno);
			break;
		}
		len--;
		tsleep(2);
	}
	if (reply) {
		timeout = 500;
		do {
			if ((len = _gs_rdy(fd)) <= 0) {
				tsleep(2);
			} else {
				read(fd,reply,1);
				if (*reply++ == '\015') {
					break;
				}
			}
		} while (timeout-- > 0);
		*reply = '\0';
	} else
		priorflush(fd);
}

priorflush(fd)
int		fd;
{
	int		len;
	char	buf[10];
	while ((len = _gs_rdy(fd)) > 0) {
		if (len > 10) len = 10;
		read(fd,buf,len);
	}
}
