/*
 *	File:	mult_slides.c
 *
 *		Routines to handle multiple slide scanning.
 *
 *	Written by:
 *
 *				S C Georgiou
 *				720 Birchwood Boulevard
 *				Birchwood Science Park
 *				Warrington
 *				Cheshire
 *
 *	Date: 30.01.1987
 *
 *
 *	Modifications:
 *
 *	18 Jul 1991		CAS			Split out menu stuff -> mf_menu.c
 *								Low level bay move stuff to fipops
 *											
 */
#include <cmdsys.h>
#include <kyy.h>
#include <slides.h
#include <fipcomm.h>
#include <disp_defs.h>

/* 
 *  Defines 
 */
#define HALFXVIEW 500
#define HALFYVIEW 500

								
/*
 * Used here; defined elsewhere
 */
extern	FILE *debug_channel;
extern	int	tisstmu(), newbox(), cornerset(), deftholds(), metlist(),
			mbuffree(), set_quit_fn();
extern char *gettissue(), *getstain();

/*
 * Used here exclusively; defined here
 */
static	int	interrscan(), scaninfo(), selbay(), singlebay(), getbxfoc();

/*
 * Used here and elsewhere; defined here
 */
int		bay_move(), baynn(), bayshow(), showmhead(), doscan(),
		killscan(), unsetflag(), killbay(), deffocs(), initscan(), movefoc();

/*
 *	I N I T S C A N --  Initialise scan params and bays
 *
 */
int initscan()
{
	showmets = 1;					/* Show crosses on slide sim */
	talk = 0;
	italk = 0;
	if (movespeed() == FS_OK)		/* in case come straight to do scan */
		return(1);
	else
		return(0);
}

/*
 *	D O S C A N  ---
 *
 *			Scan either all active bays (doscan(NULL)) or any chosen
 *			bay whether active or not.
 *
 */
int doscan(bayno)
int	*bayno;
{
	int	oneonly = 0, i, j, firsttime = 1, firstbay, anymore;
	struct	mult_slide *bp;

	if (!mbufalloc(MAXREC)) 
		return(FS_FAIL);
	qclear();
	fixrgb();
	firstbay = baynn(alc->curbay);				/* in case ctrl-c before 
												   firstbay defined */
												/* If no bays active no need
												   to move anywhere */
	if (!bayno) {							
		for (j = 0, bp = alc->frsbay, anymore = 0; j < alc->maxbays; j++) {
			if (bp->active == 1) {
				anymore = 1;
				if (!bp->focN38 || !bp->focbox) {
					fprintf(output, "Focus not defined on bay %d\n", baynn(bp));
					if (!go_on())
						return(FS_FAIL);
				}
			}
			bp++;
		}
		if(!anymore) {
			fprintf(output,"No active bays. Please set parameters before scan.\n");
			return(FS_FAIL);
		}
	}
	scaninfo();									/* Show big message etc */	
	if (!bayno) {								/* Scan all active bays */
		for(i = 1; i <= alc->maxbays; i++) {
			if(killscan())						/* Check status flag */
				break;
			if (!movefoc(i)) {
				mbuffree();
				return(FS_FAIL);					/* Move */
			}
			showbays();							/* Show bay contents */
			if(alc->curbay->active) {			/* Check */
				if(firsttime) {					/* Keep first scanned bay */
					firstbay = i;
					firsttime = 0;
				}
				showmhead();
				if(killscan())					/* Check status flag */
					break;
				getmph();						/* Scan */
				updatebay(alc->curbay);
			}
												/* If no more bays on, break */
			for(j=i, anymore=0; j < alc->maxbays; j++) {
				bp = alc->frsbay+j;				/* From next onwards */
				if(bp->active == 1)
					anymore = 1;
			}
			if(!anymore)
				break;
			if(killscan())					/* If last scan killed do
												   not do any more */
				break;
		}
	} else {
		if(!killscan()) {					/* Check status flag */
			showmhead();
			getmph();							/* Scan */
			firstbay = *bayno;
		}
		erasehisto();							/* Erase histo left by 
												   deftholds */
	}
	mbuffree();
	movefoc(firstbay);
	return(FS_OK);
}

/*
 *	D E F F O C S
 *
 *			Define focus at N38 and box centre
 */
deffocs()
{
	int ok;

	if (!alc->curbay->focN38) {
		stagepos(0, 0, alc->curbay->focslc);	/* Move closer in focus */
		ok = focusit("at datum position");  	/* Update focus */
		if ( ok ) {
			getpos(2);
			alc->curbay->focslc = zs;
			alc->curbay->focN38 = 1;
		}
		if ( ok )
			ok = getbxfoc();
		if ( ok )
			return(1);
		else
			return(0);
	} else if (alc->curbay->focbox)
			return(2);
	getbxfoc();							/* Update focus at box centre */
	
}
/*
 *	G E T B X F O C
 *
 *			Prompt user to indicate correct focus at box centre
 */
static
getbxfoc()
{
int	newx, newy, newslx, newsly, ok;
struct	mult_slide *bp;

	bp = alc->curbay;
	if (!bp->focbox) {
		newx = (( bp->mbox.bright - bp->mbox.bleft ) / 2 ) + bp->mbox.bleft;
		newy = (( bp->mbox.btop - bp->mbox.bbot) / 2 ) + bp->mbox.bbot;
										/* new midpoint of box on screen */
		newslx = ( newx - XMID ) * CONV;
		newsly = ( YMID - newy ) * CONV; 
										/* Focus close to N38 is known anyway */
		if(abs(newslx) < HALFXVIEW && abs(newsly) < HALFYVIEW) {
			bp->focbxc = 0;
			bp->focbox = 1;
			return(1);
		}
		if (stagepos(newslx,newsly,bp->focslc+bp->focbxc) == -1) return(0);
		ok = focusit("at box center");	/* To find focus at box center */
		if( ok ) {
			getpos(2);
			bp->focbxc = zs - 			/* Get focus distance rel to slide top */
						bp->focslc;
			bp->focbox = 1;
			return(1);
		} else
			return(0);
	}
}

/*
 *	S H O W M H E A D  --
 *
 */
showmhead()
{
int line, bayno;
struct	mult_slide *bp;

	line =  16;
	bp = alc->curbay;
	bayno = baynn(bp);
	
	clearlns(line,line+2);
	screengo(line,10);
	fprintf (output,"Case: %s", bp->nameids.case);
	screengo (line,39);
	fprintf (output,"%s", gettissue(bp->typeids.ktistype));
	screengo (line+1,10);
	fprintf (output,"Slide ID: %s", bp->nameids.slide);
	screengo(line+1,39);
	fprintf (output,"%s", getstain(bp->typeids.kfstype));
	screengo(line+1,60);
	fprintf (output,"%s", alc->mydate);
	screengo(line, 60);
	fprintf(output, "density    : %d", bp->drange);
	screengo(line+2,10);
	fprintf (output,"Current bay: %d", bayno);
	screengo(line+2,39);
	fprintf (output,"Lower thold: %d", bp->thresh1);
	screengo(line+2,60);
	fprintf (output,"Upper thold: %d", bp->thresh2);
	screengo(outline,2);
}

/*
 *
 *	M O V E F O C  ---
 *
 *			Move to newbay and correct focus.
 *
 */
movefoc(newbay)
int	newbay;
{
	if (bay_move(newbay)) {
		stagepos(xs, ys, zs+alc->curbay->focslc);
		return(1);
	} else
		return(0);
}

/*
 *
 *   F O C U S I T
 *
 *			Indicate and set focus datum.
 */
static 
focusit(str)
char	*str;
{
	screengo(ENDLINE,1);
	fprintf(output,"Please indicate correct focus %s\n", str); 
	if (! go_on())
			return(0); 
	return(1);
}

/*
 *	I N T E R R S C A N
 */
static
interrscan()
{

	alc->killscan = 1;
	screengo(ENDLINE, 0);
	fflush(stdin);
	printf("Scan will be terminated soon.  Please wait.\n");
	
}

/*
 *	S C A N I N F O --  Clear screen and inform usr that scan in progress
 */
static
scaninfo()
{

	vclear(ENDLINE);
	setdisplay(BUST);
	bigmessage(8, 10, "SCAN IN PROGRESS");
	screengo(10, 23);
	fprintf(output, "To kill scan press ESC once");
	fflush(output);
	
}


/*
 *	U N S E T F L A G ---
 */
unsetflag()
{

	alc->killscan = 0;
	
}	

/*
 * K I L L B A Y -- remote deactivation of bay
 *
 */
killbay(num)
int *num;
{
	struct mult_slide *bp;
	
	if ((*num < 0) || (*num > alc->maxbays)) {
		fprintf(stderr, "%d is outside possible bay range.\n", *num);
		*num = baynn(alc->curbay);
	} else {
		bp = alc->frsbay + (*num - 1);
		bp->active = 0;
	}
}

