/*
 *	N F I P O B J 2 -- Make and fill type 21 object from raw image data
 *
 *	Written: D C Burgess
 *	4th April 1989
 *	 4 Sep 1989		dcb		Fix bug in make21 causing crash
 *	19 May 1989		dcb		debug under alc->debug
 */
#include <stdio.h>
#include <allcontr.h>
#include <wstruct.h>
#include <ddgs.h>


struct object *make21(), *fill21(), *makemain();
struct intervaldomain *makedomain();
extern struct allkontr *alc;
extern FILE *debug_channel;

/*
 *	P R I N T _ O B J _ D A T A -- Print all info on type 21 object
 */
print_obj_data(obj)
struct object *obj;
{
	int i, j, k, npix, nlines;
	GREY *pixptr;
	struct intervaldomain *idom;
	struct intervalline *ivl;
	struct interval *iv;
	struct ivtable *ivt;
	struct valintline *vil;
	struct valueline *val;

	printf("Object data\n-----------\n");
	printf("Object = 0X%x, type=%d\n", obj, obj->type);
	idom = obj->idom;
	printf("Interval domain (Type=%d, linkcount=%d)\n",
		idom->type, idom->linkcount);
	printf("Line1=%d, lastln=%d, kol1=%d, lastkl=%d\n",
		idom->line1, idom->lastln, idom->kol1, idom->lastkl);
	nlines = idom->lastln - idom->line1 -1;
	ivl = idom->intvlines;
	for (i=idom->line1; i<=idom->lastln; i++, ivl++) {
		printf("Line %d: nintvs=%d: ", i, ivl->nintvs); fflush(stdout);
		iv = ivl->intvs;
		for (j=0; j<ivl->nintvs; j++, iv++) {
			printf("(%d,%d), ", iv->ileft, iv->iright);
			fflush(stdout);
		}
		printf("\n");
	}

	ivt = (struct ivtable *)obj->vdom;
	printf("\nValue data\n----------\n");
	printf("Type=%d, linkcount=%d, line1=%d, lastln=%d, bckgrnd=%d\n",
		ivt->type, ivt->linkcount, ivt->line1, ivt->lastln, ivt->bckgrnd);
	vil = ivt->vil;
	printf("Freeptr = 0x%x, vil = 0x%x\n", ivt->freeptr, vil);
	for (i=idom->line1; i<=idom->lastln; i++, vil++) {
		printf("Line %d: nintvs=%d: ", i, vil->nintvs); fflush(stdout);
		val = vil->vtbint;
		for (j=0; j<vil->nintvs; j++, val++) {
			printf("[(%d,%d) - ", val->vkol1, val->vlastkl);
			npix = val->vlastkl-val->vkol1+1;
			pixptr = val->values;
			for (k=0; k<npix; k++) printf("%2X ", (int)*pixptr++);
			printf("] "); fflush(stdout);
		}
		printf("\n");
	}
} /* end print_obj_data */

/*
 *	D B U F C O N S T R U C T -- Construct woolz type 21 interval
 *	structured object from the raw image data
 */
struct object *
dbufconstruct(buf, buflen)
unsigned char *buf;
int buflen;
{
	struct object *obj;
	if ((obj = make21(buf, buflen)) == NULL) {
		fprintf(stderr, "No objects seen - digitisation failure\n");
		free_digbuf();
		return(NULL);			/* Error return */
	}
	obj = fill21(obj, buf, buflen);
	return(obj);
}

/*
 *	M A K E 2 1 -- Make space for the type21 interval structured value
 *	table, also the interval domain. Init most fields
 *	Returns a pointer to the object thus constructed
 *	Also allocates space for a separate grey table
 */
struct object *
make21(digbuf, dbuflen)
unsigned char *digbuf;
int dbuflen;
{
	int nints, prev_nints, nlines, lineno, line1, lastln, lastkl, kol, kol1;
	int specials, grey_size;
	unsigned char *dig, *digend;

	struct object *obj;
	struct intervaldomain *idom;
	struct ivtable *v;
	struct valintline *vil;
	struct valueline *val;

	if (alc->debug)
		fprintf(debug_channel, "make21: Started\n");

	specials = nints = prev_nints = nlines = lineno = line1 = lastln = lastkl = kol = 0;
	kol1 = 1024;

	dig = digbuf;
	digend = digbuf + dbuflen - 2;
	*(digend) = 0;		/* NOT +1, +2 */
	*(digend+1) = 0;	/* Ensure a proper termination for sticky cases */

	if (!dig) {
		fprintf(stderr, "make21: Not passed valid digitisation pointer\n");
		return(NULL);
	}
	if (alc->debug)
		fprintf(debug_channel, "make21: dig=0x%x, digend=0x%x, dbuflen=%d\n",
			dig, digend, dbuflen);

	while (dig < digend) {
		if (*dig) {						/* Normal pixel data */
			if (kol < kol1) kol1 = kol;	/* Update for lowest 1st column posn. */
			nints++;					/* An interval */
			while (*dig) {
				dig++;					/* Skip its elements */
				kol++;					/* Count the kols */
			}
			if (kol-1 > lastkl)
				lastkl = kol-1;			/* Update lastkl for highest kol pos */
		}
		else {							/* Special code */
			specials++;					/* One more to take account of */
			dig++;						/* What is it ? */
			if (*dig) {					/* A run of zeros */
				kol += *dig;			/* Add it to position */
			}
			else {						/* end of line */
				if (prev_nints != nints) { /* Data found on line */
					nlines++;			/* Count 1 more line */
					lastln = lineno;	/* Update last ln where there's data */
				}
				lineno++;				/* Move to next line */
				if (!nints) line1++;	/* Start here, no intervals found yet */
				kol = 0;				/* Reset to 0 */
				prev_nints = nints;		/* Set previous nints count */
			}
			dig++;
		}
	} /* end while dig < digend ... */
	if (alc->debug) {
		fprintf(debug_channel, "line1=%d, lastln=%d\nkol1=%d, lastkl=%d\n",
			line1, lastln, kol1, lastkl);
		fprintf(debug_channel, "no of data lines = %d, nints = %d\n",
			nlines, nints);
	}
	/*
	 *	For space considerations, nlines should be the total
	 *	lines between first and last lines INCLUDING blank lines
	 *	THUS ....
	 */
	nlines = lastln-line1 + 1;
	grey_size = (dbuflen - 2*specials)*2;
	if (alc->debug) {
		fprintf(debug_channel, "Total nlines = %d, nints = %d\n",
			nlines, nints);
		fprintf(debug_channel, "Size of orig. dig. buf. (bytes) = %d (0x%x)\n",
			dbuflen, dbuflen);
		fprintf(debug_channel, "Size of reqd. grey table (bytes) = %d (0x%x)\n",
			grey_size, grey_size);
		fprintf(debug_channel, "Saves %dK\n", dbuflen - grey_size);
	}
	if (nints == 0) return(NULL);		/* No objects seen */
	/*
	 *	Now construct object structures, and assign space
	 */
	idom = makedomain(1, line1, lastln, kol1, lastkl); /* The interval domain */
	/*
	 * Space for the intervals (freeptr is pointer for freeing this chunk)
	 */
	idom->freeptr = (char *)Malloc(nints * sizeof(struct interval));
	obj = makemain(1, idom, NULL, NULL, NULL);			/* The main object */
	/*
	 *	Space for interval structured grey table as one contiguous block
	 */
	v = (struct ivtable *)Calloc(1, sizeof(struct ivtable) +
	 					 			(nlines * sizeof(struct valintline)) +
	 					 			(nints * sizeof(struct valueline)) );
	vil = (struct valintline *)(v + 1);	/* The valintline bit */
	/*
	 * NOTE: - val is an object of struct valintline, thus the
	 * following addition of nlines works correctly & skips the
	 * valintline bit of the contiguous memory area
	 */
	val	= (struct valueline *)(vil + nlines);	/* The valueline bit */
	vil->vtbint = val;							/* Init the 1st ptr to it */
	/*
	 *	Init elements of 1st intervalline structure
	 */
	idom->intvlines->nintvs = 0;
	idom->intvlines->intvs = (struct interval *)idom->freeptr;
	/*
	 *	Assign elements of ivtable
	 */
	v->type = 21;							/* Interval structured value tbl */
	v->linkcount = 1;						/* In use */
	/*
	 *	Space for grey table
	 *	Grey tbl IS NOT digitise buf (Not now anyway)
	 */
	v->freeptr = (char *)Malloc(grey_size);
	v->line1 = line1;
	v->lastln = lastln;
	v->bckgrnd = 0;
	v->vil = vil;
	v->original = obj;						/* Pointer to orig. object */
	obj->vdom = (struct valuetable *)v;		/* Assign value domain to object */
	return(obj);							/* Return for further processing */
} /* end make21 */

/*
 *	F I L L 2 1 -- Fill the interval structures
 *	by analysing the raw image data
 */
struct object *
fill21(obj, digbuf, dbuflen)
struct object *obj;
unsigned char *digbuf;
int dbuflen;
{
	int kol, dk1, line, line1, lastln;
	register unsigned char *dig, *digend;
	struct interval *ivt;
	struct intervalline *intvlines;
	struct valintline *vil;
	struct valueline *val;
	struct ivtable *ivtab;
	GREY *values;

	if (alc->debug)
		fprintf(debug_channel, "fill21: Started, obj = 0x%x\n", obj);

	ivt = (struct interval *)obj->idom->freeptr;	/* Interval structures */
	intvlines = obj->idom->intvlines;				/* Intervalline structures*/
	ivtab = (struct ivtable *)obj->vdom;			/* Get an ivtab struct */
	vil = ivtab->vil;								/* Valintline structures */
	val = vil->vtbint;								/* Valueline structures */

	dig = digbuf;
	digend = digbuf + dbuflen;
	line = kol = 0;
	dk1 = obj->idom->kol1;				/* Interval domain kol1 */
	line1 = obj->idom->line1;
	lastln = obj->idom->lastln;
	values = (GREY *)ivtab->freeptr;	/* The grey table of values */

	while (dig < digend) {				/* Stuff in the buff */
		if (*dig) {						/* 1st pixel in interval */
			vil->nintvs++;				/* Another interval */
			val->vkol1 = kol;			/* 1st kol of interval */
			ivt->ileft = kol-dk1;		/* 1st kol relative to idom->kol1 */
			val->values = values;		/* The grey values */
			while (*dig) {				/* While in interval */
				kol++;
				*values++ = (GREY)*dig++;	/* Transfer interval data across */
			}
			val->vlastkl = kol-1;		/* End of interval */
			ivt->iright = kol-1-dk1;	/* End relative to idom->kol1 */
			val++; ivt++;				/* Next interval structures */
		}
		else {							/* Special codes */
			dig++;						/* Point at it */
			if (*dig) {					/* Run of zeros */
				kol += *dig;			/* Adjust kol */
			}
			else {						/* End of line */
				if (line < line1) { kol=0; line++; dig++; continue; }
				if (vil->nintvs) {		/* Intervals have been assigned */
					intvlines->nintvs = vil->nintvs; /* Same no. of intervals */
					if (line == lastln) break; /* Reached the end */
				}
				intvlines++;			/* Next line of intervals */
				vil++;					/* Next line of intervals */
				vil->nintvs = 0;		/* Use as flag for iv. on line detect */
				vil->vtbint = val;		/* Start of next interval */
				intvlines->intvs = ivt;	/* Start of next interval */
				intvlines->nintvs = 0;	/* No intervals yet */
				line++;
				kol = 0;				/* Back to kol 0 */
			}
			dig++;						/* Advance over special code */
		} /* end of special codes */
	} /* end of while dig < digend ... */
	free_digbuf();						/* Free original buffer */
	return(obj);
} /* end fill21 */
