/* 
 *	Title: rank.c 
 *
 *	Copyright (c) and Intellectual Property Rights Image Recognition Systems
 *
 *	Written by: S C Georgiou
 *				Image Recognition Systems
 *				720 Birchwood Boulevard
 *				Cheshire
 *
 * Mods:
 *
 *  11 May 1989		SEAJ		Added alc->culturesfile.
 *	10 May 1989		dcb			frank: initialise linear classifier
 *	27 Feb 1989		SCG			bug in classifier assignment
 *	12 Sep 1988		SCG			lhd->nfeats not nfeats
 *	19 Aug 1988		SCG			Include class.h
 *	16 Jun 1988		SCG			cope when tis = 0 stn = 0
 *	 8 Jun 1988		SCG			put things in registers to make faster
 *	23 May 1988		SCG			makefeat function defined and used
 *	10 May 1988		SCG			trainable classification
 *					SCG			4-slide stage mods
 *	 2 feb 1987		CAS			replace rcf with alc->rankcf
 *	 9 Jan 1987		CAS			rcf structure access - decrement tiss + st
 *								since now indexed from 1 not 0
 *  17 Dec 1986		SCG/CAS		Fixed the ranking bug.  rcf structure
 *								was not initialised properly.
 *	30 Sep 1986		CAS			Convert malloc to Malloc
 *								and free to Free
 *	20 Apr 1986		Denis		rank compression to 1 byte removed
 *
 */
#include <stdio.h>
#include <slides.h>
#include <kyy.h>
#include <metscan.h>


extern struct allkontr *alc;
extern FILE *debug_channel;

static	float *qrb;


/* 
 * R C P --- rank variable comparison function 
 *			returns 1 if first operand worse, -1 if better, 0 if equal
 *			to second 
 */    
rcp(ip,jp) 
register int *ip, *jp; 
{
	register float temp;
	if((temp = *(qrb + *jp) - *(qrb + *ip)) > 0)
		return(1);
	else
		return(-1);
} 

/* 
 * R C P 2 --- rank variable comparison function 
 *			returns -1 if first operand worse, 1 if better, 0 if equal
 *			to second 
 */    
rcp2(ip,jp) 
register int *ip, *jp; 
{
	register float temp;
	if((temp = *(qrb + *jp) - *(qrb + *ip)) > 0)
		return(-1);
	else
		return(1);
} 

/*
 * F R A N K --- rank a metaphase list
 *
 */
frank() 
{
	register struct mlclust *k;
	struct mlclust *mpt;
	register float *qrp, *pars, feat;
	register int i;
	int  n, t, j, *rtb, *rtp, feat1, feat2, feat3, tis, stn;
	struct mult_slide *bp;
	struct cyto_types *ctp;
	LINHEAD *lhd;
	float fvec[MAXBOXFEATS], makefeat();

	if(alc->debug)
		fprintf(debug_channel,"Rank: in\n");
		
	bp = alc->curbay;

	/* 
	 * provide rank value table in consecutive memory locations for greater
	 * efficiency in subsequent sorting 
	 */
	mpt = bp->startptr->mlrec;
	qrp = qrb = (float *) Malloc(bp->totin * sizeof (float));
	ctp = &bp->typeids;
	lhd = bp->lhd;
	pars = (float *) Calloc(lhd->nfeats, sizeof(float));

	/*
	 * construct the parameters vector
	 */
	for(i = 0; i < lhd->nfeats; i++) {
		if(alc->debug)
			fprintf(debug_channel, "feats %d %d %d   flag %d  const %lf\n",
					lhd->lfd[i]->feat1, lhd->lfd[i]->feat2, lhd->lfd[i]->feat3, lhd->lfd[i]->aflag, lhd->lfd[i]->const);
		*(pars + i) = (float) lhd->lfd[i]->const;
	}
	/*
	 * go through all the list, construct the features and then the sum
	 */
	for(k = mpt; k < mpt + bp->totin; ++k) {
		defvec(fvec, k);
		*qrp = 0.0;
		for(i = 0; i < lhd->nfeats; i++) {
			feat = makefeat(fvec, lhd->lfd[i]);
			*qrp += feat * (*(pars + i));
			if(alc->debug)
				fprintf(debug_channel, "feat: %f  val: %f\n", feat, *qrp);
		}
		qrp++;
	}
	/* 
	 * provide table of ordinal numbers of cells for qsort to sort 
	 */
	rtp = rtb = (int *) Malloc ( bp->totin * sizeof (int) );
	for ( j = 0 ; j < bp->totin; ++j ) *rtp++ = j;
	
	/* 
	 * now sort, using  rcp (rank comparison function) to determine relative
	 * order 
	 */
	qsort(rtb, bp->totin, sizeof(int), rcp);
	
	/* 
	 * now copy ordinal table into qrank in mmlist - make lowest rank
	 * (best) 1 
	 */
	rtp = rtb;
	j = 1;
	n=0;
	for ( k = mpt; k < mpt + bp->totin; ++k ) 
			(mpt + *rtp++ ) ->qrank = j++ ;
	linkranks(rtb);							/* Link in rank order */
	Free (rtb);
	Free (qrb);
	Free (pars);
	if(alc->debug)
		fprintf(debug_channel,"Rank: out\n");
}

/*
 *	S E T R T B ---  Sets the rank order table for previously ranked lists
 */
setrtb()
{
int n,j,i,*rtb,*rtp;
float *qrp;
struct	mult_slide *bp;
struct	mlclust *mpt;

	if(alc->debug)
		fprintf(debug_channel,"setrtb: out\n");
		
	bp = alc->curbay;
	if(bp->startptr == NULL)
		return;
	mpt = bp->startptr->mlrec;
	
	qrp = qrb = (float *) Malloc(bp->totin * sizeof (float));
	for(i = 0; i < bp->totin; i++)			/* Obtain ranks on list */
		*qrp++ = (float) (mpt++)->qrank;
											/* provide table of ordinal 
											   numbers of cells for qsort 
											   to sort */
	rtp = rtb = (int *) Malloc(bp->totin * sizeof (int));
	for (j = 0 ; j < bp->totin; ++j) *rtp++ = j;
											/* sort using rcp to determine 
											   relative order */
	qsort(rtb, bp->totin, sizeof(int), rcp2);
	
	linkranks(rtb);							/* Link in rank order */
	Free (rtb);								/* Free space */
	Free (qrb);
	
	if(alc->debug)
		fprintf(debug_channel,"setrtb: out\n");
}

/*
 * D E F V E C --- extract vector from cluster record
 *
 */
defvec(fvec, mpt)
register float *fvec;
register struct mlclust *mpt;
{


	if(alc->debug)
		fprintf(debug_channel, "defvec: in\n");	

	fvec[0] = (float) mpt->xwide;
	fvec[1] = (float) mpt->ywide;
	fvec[2] = (float) mpt->nobj;
	fvec[3] = (float) mpt->sf;
	fvec[4] = (float) mpt->aobj;
	fvec[5] = (float) mpt->octa;
	fvec[6] = (float) mpt->skt;
	fvec[7] = (float) mpt->dv;
	fvec[8] = (float) mpt->meanod;
	
	if(alc->debug)
		fprintf(debug_channel, "defvec: out\n");
}

/*
 * M A K E F E A T ---
 *	The scaling factors are: for each division multiply by 100
 *				 for each multiplication divide by 100
 *
 */
float
makefeat(fvec, lfd)
struct linfdisc *lfd;
float fvec[];
{
register int feat1, feat2, feat3;
register float feat;

	if(alc->debug)
		fprintf(debug_channel, "makefeat: in\n");	

	feat1 = lfd->feat1;
	feat2 = lfd->feat2;
	feat3 = lfd->feat3;
	switch(lfd->aflag) {
		case 1:
			feat = fvec[feat1];
			break;
		case 2:
			feat = 100. * fvec[feat1] / fvec[feat2];
			break;
		case 3:
			feat = 0.01 * fvec[feat1] * fvec[feat2];
			break;
		case 4:
			feat = 10000. * fvec[feat1] / fvec[feat2] / fvec[feat3];
			break;
		case 5:
			feat = 0.0001 * fvec[feat1] * fvec[feat2] * fvec[feat3];
			break;
		case 6:
			feat = fvec[feat1] * fvec[feat2] / fvec[feat3];
			break;
		case 7:
			feat = fvec[feat1] * fvec[feat3] / fvec[feat2];
			break;
		default:
			break;
	}
	if(alc->debug)
		fprintf(debug_channel, "makefeat: out\n");
	return(feat);
}

