/*
 *		S H A D E . C  --  Software shading correction for the RK
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    September 1987
 *
 *	Note: These routines are all designed explicitly for use with a virtuoso
 *		  framestore.  This module has implicit knowledge of the framestore.
 *
 *	This is one of those strange bits of software that has evolved during
 *	experimentation. It is still in prototype form and I still don't really
 *	understand why it works as well as it seems to.
 *
 *  Modifications
 *
 *	31 Jul 1989		CAS			Moved line average code to virtuoso module
 *	25 Jul 1989		CAS			Added code to do a 2 line average across a virtuoso
 *	 2 Jun 1989		CAS			Extra comma in ovline call picked up by 2.2 compiler
 *	 7 Mar 1989		CAS			Added extra param to oclear call
 *	30 Jan 1989		CAS			Edge define out to kyy.h cos we need it elsewhere
 *	 8 Dec 1988		CAS/dcb		Fix highest to check 255 not 0
 *					CAS			Increment missing in shade_smooth, also
 *								edge hardcoded as 2
 *	27 Oct 1988		CAS			Support inverted images
 *	15 Aug 1988		CAS			Make writeshading work properly
 *	 1 Feb 1988		CAS			Added message confirmin shade write
 *	27 Jan 1988		CAS			debug out messages
 *	18 Nov 1987		CAS			Major tidy up, pulled some stuff from tvdig
 *	9-13 Nov 1987	CAS			New scale_shade fn
 *
 */
#include <cmdsys.h>
#include <kyy.h>

#define LINELN	768		/* These are implicit in some calculations - especially */
#define LINENO	576		/* some of the assembler routines - so don't get clever
							and expect changing them here to work... */

#define SMOOTHSIZE	1	/* Number of surrounding pixels to smooth over (1 => 3x3, 2=> 5x5) */
#define SHADEFILE "/dd/params/rkshade"

static unsigned char *shadevector = NULL;	/* Shading correction */
static unsigned char *shadevector2 = NULL;	/* Shading correction */

extern struct allkontr *alc;

/*
 *	C A L C S H A D E  --  Calculate a shading correction table
 *
 */
calc_shade(frame,invert)
register unsigned char	*frame;
int		invert;
{
	register int	sum,nhoriz, nvert;
	int		size,low,high,tick();
	unsigned char	*ctb, *worktab, *Malloc(), *temp, *oframe;

	oframe = frame;
/*	printf("start %d, frame 0x%x, shadevector 0x%x\n",tick(),frame,shadevector); */
	if (!shadevector)
		shadevector = Malloc((alc->ddgschannel->lineln/4 +1) * (alc->ddgschannel->lineno/4 +1));
	nvert = LINENO/4;
	size = nvert * (LINELN/4);
	worktab = Malloc(size);
	ctb = worktab;
/*	printf("calc %d\n",tick()); */
	while (nvert--)  {
		nhoriz = LINELN/4;
		while (nhoriz--) {
			sum = *(frame + LINELN);
			sum += *(frame + LINELN*2);
			sum += *(frame + LINELN*3);
			sum += *frame++;
			sum += *(frame + LINELN);
			sum += *(frame + LINELN*2);
			sum += *(frame + LINELN*3);
			sum += *frame++;
			sum += *(frame + LINELN);
			sum += *(frame + LINELN*2);
			sum += *(frame + LINELN*3);
			sum += *frame++;
			sum += *(frame + LINELN);
			sum += *(frame + LINELN*2);
			sum += *(frame + LINELN*3);
			sum += *frame++;
			sum >>= 4;
			if (invert)
				*ctb++ = (255-sum);
			else
				*ctb++ = sum;
		}
		frame += (3*LINELN);
	}
	if (alc->debug) {
		printf("limits %d\n",tick());
		low = lowest(worktab,size);
		high = highest(worktab,size);
		printf("Shading lowest %d, highest %d\n",low,high);
		temp = shadevector;
		shadevector = worktab;
		display_shade(oframe,invert);
		shadevector = temp;
		printf("smooth %d\n",tick());
	}
 	smooth_shade(shadevector,worktab,LINELN/4,LINENO/4,SMOOTHSIZE,2);
	if (alc->debug) {
	 	printf("bottom cut %d\n",tick());
		display_shade(oframe,invert);
	}
	low = lowest(shadevector,size);
	if (low) {
		subtract(shadevector,size,low);
		low = 0;
	}
	if (alc->debug) {
		high = highest(shadevector,size);
		printf("Shading correction range (after smooth) %d to %d\n",low,high);
	}
	Free(worktab);
/*	printf("done %d\n",tick()); */
}

/*
 *	A P P L Y _ S H A D E  --
 *
 */
apply_shade(frame,invert)
int		frame;
int		invert;
{
	int		size;
	if (shadevector) {
		if (!shadevector2) {
			size = ((alc->ddgschannel->lineln/4 +1) * (alc->ddgschannel->lineno/4 +1));
			shadevector2 = Malloc(size);
		}
		scale_shade(frame,shadevector2,shadevector,invert);
		if (invert)
			do_invshade(alc->ddgschannel->frame,shadevector2);
		else
			do_shade(alc->ddgschannel->frame,shadevector2);
		Free(shadevector2);
		shadevector2 = NULL;
	}
}


/*
 *	R E A D _ S H A D E  --  Read default shading correction vector from file
 *
 */
read_shade()
{
	int		sfile,size;
	char	namebuf[80];
	if (shadevector != NULL)
		return;
	sprintf(namebuf,"%s%d",SHADEFILE,alc->seat);
	if ((sfile = open(namebuf,1)) != -1) {
		size = ((alc->ddgschannel->lineln/4 +1) * (alc->ddgschannel->lineno/4 +1));
		if (shadevector == NULL)
			shadevector = Malloc(size);
		if (read(sfile,shadevector,size) != size) {
			fprintf(stderr,"Shading correction file is corrupt\n");
			Free(shadevector);
			shadevector = NULL;
		}
		close(sfile);
	}
}

/*
 *	W R I T E _ S H A D E  --  Rewrite default shading correction
 *
 */
write_shade()
{
	int		sfile,size;
	char	namebuf[80];
	if (shadevector == NULL)
		return;
	sprintf(namebuf,"%s%d",SHADEFILE,alc->seat);
	if ((sfile = open(namebuf,2)) == -1) {
		if ((sfile = create(namebuf,2,0x1b)) == -1) {
			fprintf(stderr,"Unable to write shading correction to disc\n");
			return;
		}
	}
	size = ((alc->ddgschannel->lineln/4 +1) * (alc->ddgschannel->lineno/4 +1));
	write(sfile,shadevector,size);
	close(sfile);
	printf("Shading correction saved\n");
}

/*
 *	D I S P L A Y  _ S H A D E  --	Display shading correction on fstore
 *
 */
display_shade(frame,invert)
register unsigned char	*frame;
int		invert;
{
	register int	nhoriz, nvert;
	unsigned char	*ctb;

	if (alc->debug)
		printf("display shade, frame 0x%x, shadevector 0x%x\n",
			frame, shadevector);

	if (shadevector == NULL)
		return(0);
	ctb = shadevector;
	nvert = LINENO/4;
	while (nvert--)  {
		nhoriz = LINELN/4;
		while (nhoriz--) {
			*(frame + LINELN) = *ctb;
			*(frame + LINELN*2) = *ctb;
			*(frame + LINELN*3) = *ctb;
			*frame++ = *ctb;
			*(frame + LINELN) = *ctb;
			*(frame + LINELN*2) = *ctb;
			*(frame + LINELN*3) = *ctb;
			*frame++ = *ctb;
			*(frame + LINELN) = *ctb;
			*(frame + LINELN*2) = *ctb;
			*(frame + LINELN*3) = *ctb;
			*frame++ = *ctb;
			*(frame + LINELN) = *ctb;
			*(frame + LINELN*2) = *ctb;
			*(frame + LINELN*3) = *ctb;
			*frame++ = *ctb++;
		}
		frame += (3*LINELN);
	}
	return(1);
}

/*
 *	S M O O T H _ S H A D E  --
 *
 */
#ifdef SLOWCODE
smooth_shade(new,old,lineln,lineno,smoothsize,edge)
unsigned char *new, *old;
int		lineln, lineno,smoothsize,edge;
{
	register int i, j, row, col, offset,increment;
	register short	tot, cnt;
	register unsigned char *start, *oldp;
	oldp = old;
	offset = smoothsize + smoothsize * lineln;		/* top corner of smooth box */
	increment = lineln - (1 + 2 * smoothsize);		/* how to get from last point
														in one row to 1st in next */
	for (row=0; row < edge; row++) {
		for (col=0; col < lineln; col++)
			*new++ = *oldp++;
	}
	for (row=edge; row < lineno-edge; row++) {
		for (col=0; col < edge; col++)
			*new++ = *oldp++;
		for (col=edge; col < lineln-edge; col++) {
			cnt = 0;
			tot = 0;
			start = oldp++ - offset;
			for (i=row-smoothsize; i<=row+smoothsize; i++) {
				if ((i < 0) || (i >= lineno)) {
					start += lineln;
				} else {
					for (j=col-smoothsize; j<=col+smoothsize; j++) {
						if ((j < 0) || (j >= lineln)) {
							start++;
						} else {
							tot += *start++;
							cnt++;
						}
					}
					start += increment;
				}
			}
			*new++ = tot/cnt;
		}
		for (col=lineln-edge; col < lineln; col++)
			*new++ = *oldp++;
	}
	for (row=lineno-edge; row < lineno; row++) {
		for (col=0; col < lineln; col++)
			*new++ = *oldp++;
	}
}
#else SLOWCODE
#asm
 ttl smooth_s
* 
* smooth_shade(new,old,lineln,lineno,smoothsize,edge)
* unsigned char *new, *old;
* int		lineln, lineno,smoothsize,edge;
*
smooth_shade:  movem.l #ss_83!3,-(sp)
		move.l #ss_85,d0 :6
		bsr _stkcheck

* 	register int i, j, row, col, offset,increment;
* 	register short	tot, cnt;
* 	register unsigned char *start, *oldp;
* 	oldp = old;
		lea		-12(sp),sp
		movea.l	16(sp),a3
* 	offset = smoothsize + smoothsize * lineln;		/* top corner of smooth box */
		move.l	20+ss_84(sp),d0
		move.l	12+ss_84(sp),d1
		bsr		_T$UMul
		add.l	20+ss_84(sp),d0
		move.l	d0,8(sp)
* 	increment = lineln - (1 + 2 * smoothsize);		/* how to get from last point
		move.l	20+ss_84(sp),d0
		lsl.l	#1,d0 :2
		addq.l	#1,d0 :2
		move.l	12+ss_84(sp),d1
		sub.l	d0,d1 :2
		move.l	d1,4(sp)
* 	for (row=0; row < edge; row++) {
		moveq.l	#0,d6 :2
		bra		ss_89
ss_87
* 		for (col=0; col < lineln; col++)
		moveq.l	#0,d7 :2
		bra		ss_93
* 			*new++ = *oldp++;
ss_91
		movea.l	12(sp),a0
		addq.l	#1,12(sp)
		move.b	(a3)+,(a0) :2
* 	}
ss_94	addq.l	#1,d7 :2
ss_93	cmp.l	12+ss_84(sp),d7
		blt		ss_91
ss_92
* 	for (row=edge; row < lineno-edge; row++) {
ss_90	addq.l #1,d6 :2
ss_89	cmp.l 24+ss_84(sp),d6
		blt ss_87
ss_88	move.l 24+ss_84(sp),d6
		bra ss_97
ss_95	moveq.l #0,d7 :2			* 		for (col=0; col < edge; col++)
		bra ss_101
ss_99	movea.l 12(sp),a0			* 			*new++ = *oldp++;
		addq.l #1,12(sp)
		move.b (a3)+,(a0) :2
ss_102	addq.l #1,d7 :2				*
ss_101	cmp.l 24+ss_84(sp),d7 :6
		blt ss_99
ss_100	move.l 24+ss_84(sp),d7		* 		for (col=edge; col < lineln-edge; col++) {
		bra ss_105
ss_103	clr.w (sp) :2				* 			cnt = 0;
		clr.w 2(sp)					* 			tot = 0;
		move.l	a3,d0 :2			* 			start = oldp++ - offset;
		addq.l	#1,a3
		sub.l	8(sp),d0
		movea.l	d0,a2 :2
		move.l d6,d0 :2				*			for (i=row-smoothsize; i<=row+smoothsize; i++) {
		sub.l 20+ss_84(sp),d0
		move.l d0,d4 :2
		bra ss_109
ss_107	tst.l d4 :2					* 				if ((i < 0) || (i >= lineno)) {
		blt ss_111
		cmp.l 16+ss_84(sp),d4
		blt ss_112
ss_111	move.l 12+ss_84(sp),d0		* 					start += lineln;
		adda.l d0,a2 :2
* 				} else {
		bra ss_114
ss_112
* 					for (j=col-smoothsize; j<=col+smoothsize; j++) {
		move.l d7,d0 :2
		sub.l 20+ss_84(sp),d0
		move.l d0,d5 :2
		bra ss_117
ss_115
* 						if ((j < 0) || (j >= lineln)) {
		tst.l d5 :2
		blt ss_119
ss_121
		cmp.l 12+ss_84(sp),d5
		blt ss_120
ss_119
* 							start++;
		addq.l #1,a2 :2
* 						} else {
		bra ss_122
ss_120
* 							tot += *start++;
		moveq.l #0,d0 :2
		move.b (a2)+,d0 :2
		add.w d0,2(sp)
* 							cnt++;
		addq.w #1,(sp) :2
* 						}
* 					}
ss_122
* 					start += increment;
ss_118	addq.l #1,d5 :2
ss_117	move.l d7,d0 :2
		add.l 20+ss_84(sp),d0
		cmp.l d0,d5 :2
		ble ss_115
ss_116	move.l 4(sp),d0
		adda.l d0,a2 :2
* 				}
* 			}
ss_114
ss_110	addq.l #1,d4 :2
ss_109	move.l d6,d0 :2
		add.l 20+ss_84(sp),d0
		cmp.l d0,d4 :2
		ble ss_107
* 			*new++ = tot/cnt;
ss_108	move.w	2(sp),d0			* tot
		ext.l	d0
		move.w	(sp),d1				* cnt
									*	ext.l d1 :2
		divu	d1,d0				*	bsr _T$LDiv (d0 = d0/d1)
		movea.l	12(sp),a0
		addq.l	#1,12(sp)			* new++
		move.b	d0,(a0)				* *new =
* 		}
* 		for (col=lineln-edge; col < lineln; col++)
ss_106	addq.l #1,d7 :2
ss_105	move.l 12+ss_84(sp),d0
		sub.l 24+ss_84(sp),d0
		cmp.l d0,d7 :2
		blt ss_103
ss_104	move.l 12+ss_84(sp),d0
		sub.l 24+ss_84(sp),d0
		move.l d0,d7 :2
		bra ss_125
* 			*new++ = *oldp++;
ss_123	movea.l 12(sp),a0
		addq.l #1,12(sp)
		move.b (a3)+,(a0) :2
* 	}
ss_126	addq.l #1,d7 :2
ss_125	cmp.l 12+ss_84(sp),d7
		blt ss_123
ss_124
* 	for (row=lineno-edge; row < lineno; row++) {
ss_98	addq.l #1,d6 :2
ss_97	move.l 16+ss_84(sp),d0
		sub.l 24+ss_84(sp),d0
		cmp.l d0,d6 :2
		blt ss_95
ss_96	move.l 16+ss_84(sp),d0
		sub.l 24+ss_84(sp),d0
		move.l d0,d6 :2
		bra ss_129
ss_127
*		for (col=0; col < lineln; col++)
		moveq.l #0,d7 :2
		bra ss_133
* 			*new++ = *oldp++;
ss_131	movea.l 12(sp),a0
		addq.l #1,12(sp)
		move.b (a3)+,(a0) :2
* 	}
ss_134	addq.l #1,d7 :2
ss_133	cmp.l 12+ss_84(sp),d7
		blt ss_131
ss_132
* }
ss_130	addq.l #1,d6 :2
ss_129	cmp.l 16+ss_84(sp),d6
		blt ss_127
ss_128	lea 12(sp),sp
ss_86	addq.l #8,sp :2
		movem.l (sp)+,#ss_83
		rts :2
ss_85 equ 0xffffffb4 :0
ss_83 equ 0x00000df0 :0
ss_84 equ 0x00000028 :0
#endasm
#endif SLOWCODE

/*
 *	D I S P _ H I S T O  --  Calculate and display histogram on screen
 *							 in virtuoso overlay plane
 *
 */
disp_histo(frame)
unsigned char *frame;
{
	unsigned char *fptr;
	unsigned short	hist[256];
	int		i, j, size;

/*	overlay = frame + 0x70181 - 0xc08;
	olut = frame + 0x300 - 0xc08;
	creg = frame + 0x70060 - 0xc08;
	optr = overlay;
*/
	size = 96*576;
	oclear(0,0,LINELN-1,LINENO-1,0);
	ddgsoverlay(ON);
	osetrgb(1,255,0,0);
/*
 *	Calc histo
 */
 	screengo(ENDLINE,1);
	printf("Proceed (M) ?");
	fflush(stdout);
	do {
		for (i=0; i<128; i++)
			hist[i] = 0;
		fptr = frame;
		for (i=0; i<LINENO/4; i++) {
			for (j=0; j<LINELN/4; j++ ) {
				hist[*fptr>>1]++;
				fptr += 4;
			}
			fptr += (3 * LINELN);
		}
		plothisto(hist,600,340);
	} while(gone_on() <= 0);
	ddgsoverlay(OFF);
}

/*
 *	P L O T H I S T O  --  Plot the histogram
 *
 */
static plothisto(hist,x,y)
unsigned short hist[];
int		x, y;
{
	register int	i, max, divisor, height;
	unsigned char *optr;
	if (y > (LINENO - 128)) y = LINENO-128;
	oclear(x,y,x+128,y+128,0);
/*
 *	Draw axes
 */
	ovline(x,y,y+128);
	ohline(x,y,x+128);
/*
 *	Scale histogram and then display
 */
	max = 0;
	for (i=0; i<128; i++)
		if (hist[i] > max)
			max = hist[i];
	max = max >> 7;
	divisor = 0;
	while ((max = max / 2) > 1)
		divisor++;
	for (i=1; i<128; i++) {
		height = hist[i]>>divisor;
		if (height > 128) height = 128;
		ovline(x+i,y,y + height);
	}
}

/*
 *	S C A L E _ S H A D E   --  Using the current framestore data and the
 *								supplied shading martix, calc a new scaled
 *								matrix suited to the current level of illumination
 */
scale_shade(frame,newtab,oldtab,invert)
register unsigned char	*frame;
unsigned char *newtab, *oldtab;
int		invert;
{
	register int	sum,nhoriz, nvert;
	int		factor, minframe = 999999;
	int		ctbmax = 0;
	int		size,low,high,i;
	unsigned char	*ctb, *framestart, localfr;

	ctb = oldtab;
	framestart = frame;
	if (invert)
		minframe = highframe4(LINELN/4,LINENO/4,frame);
	else
		minframe = lowframe4(LINELN/4,LINENO/4,frame);
	if (alc->debug)
		printf("Scale: screen min %d\n",minframe);
	nvert = LINENO/4;
	size = nvert * (LINELN/4);
	nvert -= 2 * EDGE;			/* Skip 'EDGE' lines top and bottom */
	frame += EDGE * LINELN*4;
	ctb += EDGE * LINELN/4;
	factor = scalefactor(nvert,nhoriz,ctb,frame,invert,minframe,LINELN,EDGE);
/*	if (alc->debug)
		printf("Min factor %d, ave factor %d, scale factor %d\n",minfactor, totfactor/nterms,factor);
 */
	for (i=0; i< size; i++) {
		*newtab = (*oldtab * factor) >> 8;
		newtab++; oldtab++;
	}
/*
 *	if (alc->debug) {
		low = lowest(newtab,size);
		high = highest(newtab,size);
		printf("Shading correction range %d to %d\n",low,high);
 *	}
 */
}


/*
 *	S C A L E F A C T O R  --
 *
 *	Calculate the ratio of pixel to shading value. The final factor used is
 *	the average of (the minimun factor and the average factor). Pixels are only
 *	considered if they are more than 10 grey levels away from the minimum
 *	grey val found in the sampled image. This is to stop light points screwing
 *	things up.
 */
#ifdef SLOWCODE
scalefactor(nvert,nhoriz,ctb,frame,invert,minframe,lineln,edge)
int		nvert,invert,lineln,edge;
register int nhoriz, minframe;
register unsigned char *ctb, *frame;
{
	register int	localfr, factor, minfactor, totfactor, nterms;
	totfactor = 0;
	nterms = 0;
	minfactor = 999999;
	while (nvert--)  {
		nhoriz = (lineln >> 2) - edge - edge;
		ctb += edge;				/* skip first 'EDGE' cols */
		frame += edge << 2;			/* Adjust frame pointer accordingly */
		while (nhoriz--) {
/*
 *	Ignore all zero shading values
 *	The '10' in the factor calculation is to stop points with small grey
 *	values (white background) screwing up the minimum scale factor.
 */
			if (*ctb) {
				if (invert) 
					localfr = minframe - *frame;
				else
					localfr = *frame - minframe;
				if (localfr > 10) {
					factor = (localfr << 8) / *ctb;
					if (factor < minfactor) {
						minfactor = factor;
					}
					totfactor += factor;
					nterms++;
				}
			}
			ctb++;
			frame += 4;
		}
		ctb += edge;				/* skip last 'EDGE' cols */
		frame += edge << 2;
		frame += lineln + lineln + lineln;		/* Next line */
		printf("next line - frame 0x%x\n",frame);
	}
	printf("Done l ines\n");
	if (minfactor == 999999)
		minfactor = 0;
	factor = (totfactor/nterms + minfactor)/2;
	return(factor);
}
#else SLOWCODE
#asm
 ttl scalefac
* 
* scalefactor(nvert,nhoriz,ctb,frame,invert,minframe,lineln,edge)
* int		nvert,invert,lineln,edge;
* register int nhoriz, minframe;
* register unsigned char *ctb, *frame;
*
scalefactor:  movem.l #sf_182!3,-(sp)
		move.l d1,d4
		movea.l 0+sf_183(sp),a2
		movea.l 4+sf_183(sp),a3
		move.l 12+sf_183(sp),d5
		move.l #sf_184,d0 :6
		bsr _stkcheck

* 	register int	localfr, factor, minfactor, totfactor, nterms;
		lea		-12(sp),sp
		clr.l	(sp)			*	nterms = 0;
		clr.l	4(sp)			*	totfactor = 0;
		move.l #999999,8(sp)	*	minfactor = 999999;
* 	while (nvert--)  {
		bra sf_187
sf_188
* 		nhoriz = (lineln >> 2) - edge - edge;
		move.l 28+sf_183(sp),d0
		asr.l #2,d0 :2
		sub.l 32+sf_183(sp),d0
		sub.l 32+sf_183(sp),d0
		move.l d0,d4 :2
* 		ctb += edge;				/* skip first 'EDGE' cols */
		move.l 32+sf_183(sp),d0
		adda.l d0,a2 :2
* 		frame += edge << 2;			/* Adjust frame pointer accordingly */
		move.l 32+sf_183(sp),d0
		lsl.l #2,d0 :2
		adda.l d0,a3 :2
* 		while (nhoriz--) {
		bra sf_190
sf_191
* /*
* 			if (*ctb) {
		tst.b (a2) :2
		beq sf_193
sf_192
* 				if (invert) 
* 					localfr = minframe - *frame;
		tst.l 20+sf_183(sp)
		beq sf_195
sf_194
		moveq.l #0,d0 :2
		move.b (a3),d0 :2
		move.l d5,d1 :2
		sub.l d0,d1 :2
		move.l d1,d6 :2
* 				else
* 					localfr = *frame - minframe;
		bra sf_196
sf_195
		moveq.l #0,d0 :2
		move.b (a3),d0 :2
		sub.l d5,d0 :2
		move.l d0,d6 :2
* 				if (localfr > 10) {
sf_196
		cmpi.l #10,d6 :6
		ble sf_198
sf_197
* 					factor = (localfr << 8) / *ctb;
		move.l d6,d0 :2
		lsl.l #8,d0 :2
		moveq.l #0,d1 :2
		move.b (a2),d1 :2
		divu	d1,d0			* bsr _T$UDiv
		moveq.l	#0,d7
		move.w	d0,d7
* 					if (factor < minfactor) {
		cmp.l 8(sp),d7
		bge sf_200
sf_199
* 						minfactor = factor;
		move.l d7,8(sp)
* 					}
* 					totfactor += factor;
sf_200
		move.l d7,d0 :2
		add.l d0,4(sp)
* 					nterms++;
		addq.l #1,(sp) :2
* 				}
* 			}
sf_198
* 			ctb++;
sf_193
		addq.l #1,a2 :2
* 			frame += 4;
		addq.l #4,a3 :2
* 		}
* 		ctb += edge;				/* skip last 'EDGE' cols */
sf_190
		move.l d4,d0 :2
		subq.l #1,d4 :2
		tst.l d0 :2
		bne sf_191
sf_189
		move.l 32+sf_183(sp),d0
		adda.l d0,a2 :2
* 		frame += edge << 2;
		move.l 32+sf_183(sp),d0
		lsl.l #2,d0 :2
		adda.l d0,a3 :2
* 		frame += lineln + lineln + lineln;		/* Next line */
		move.l 28+sf_183(sp),d0
		add.l 28+sf_183(sp),d0
		add.l 28+sf_183(sp),d0
		adda.l d0,a3 :2
* 	}
* 	if (minfactor == 999999)
sf_187
		move.l 12(sp),d0
		subq.l #1,12(sp)
		tst.l d0 :2
		bne sf_188
sf_186
* 		minfactor = 0;
		cmpi.l #999999,8(sp) :8
		bne sf_202
sf_201
		clr.l 8(sp)
* 	factor = (totfactor/nterms + minfactor)/2;
sf_202
		move.l 4(sp),d0
		move.l (sp),d1 :2
		bsr _T$LDiv
		add.l 8(sp),d0
		moveq.l #2,d1 :2
		bsr _T$LDiv
		move.l d0,d7 :2
* 	return(factor);
		move.l d7,d0 :2
		lea 12(sp),sp
		bra sf_185
* }
		nop :2
sf_185
		addq.l #8,sp :2
		movem.l (sp)+,#sf_182
		rts :2
sf_184 equ 0xffffffb4 :0
sf_182 equ 0x00000cf0 :0
sf_183 equ 0x00000024 :0
#endasm
#endif SLOWCODE

/*
 *	D O S H A D E  --  Apply shading correction to current framestore
 *						image
 *
 */
#ifdef SLOWCODE
/*
 *	Warning - there is a compiler bug in here and hence the slow version
 *	won't work.
 *			if ((*a++ -= b) <0)  -> double increments 'a'
 */
do_shade(frame,shadevector)
register unsigned char	*frame;
{
	register unsigned char *convtab, conval, *wkptr;
	register int	nhoriz, nvert;
	convtab = shadevector;
	nvert = LINENO/4;
	while (nvert--)  {
		nhoriz = LINELN/4;
		while (nhoriz--) {
			if (alc->debug)
				printf("fram 0x%x\n",frame);
			if ((conval = *convtab++) == 0) {
				frame += 4;
				if (alc->debug)
					printf("conv\n");
				continue;
			}
/* 1st line */
			if ((*frame++ -= conval) < 0) {*(--frame) = '\0'; frame++; }
			if ((*frame++ -= conval) < 0) {*(--frame) = '\0'; frame++; }
			if ((*frame++ -= conval) < 0) {*(--frame) = '\0'; frame++; }
			if ((*frame++ -= conval) < 0) {*(--frame) = '\0'; frame++; }
			wkptr = frame + (LINELN-4);
/* 2nd line */
			if ((*wkptr++ -= conval) < 0) {*--wkptr = '\0'; wkptr++; }
			if ((*wkptr++ -= conval) < 0) {*--wkptr = '\0'; wkptr++; }
			if ((*wkptr++ -= conval) < 0) {*--wkptr = '\0'; wkptr++; }
			if ((*wkptr -= conval) < 0) {*wkptr = '\0';}
			wkptr += (LINELN-3);
/* 3rd line */
			if ((*wkptr++ -= conval) < 0) {*--wkptr = '\0'; wkptr++; }
			if ((*wkptr++ -= conval) < 0) {*--wkptr = '\0'; wkptr++; }
			if ((*wkptr++ -= conval) < 0) {*--wkptr = '\0'; wkptr++; }
			if ((*wkptr -= conval) < 0) {*wkptr = '\0';}
			wkptr += (LINELN-3);
/* 4th line */
			if ((*wkptr++ -= conval) < 0) {*--wkptr = '\0'; wkptr++; }
			if ((*wkptr++ -= conval) < 0) {*--wkptr = '\0'; wkptr++; }
			if ((*wkptr++ -= conval) < 0) {*--wkptr = '\0'; wkptr++; }
			if ((*wkptr -= conval) < 0) {*wkptr = '\0';}
		}
		frame += (3*LINELN);
		if (alc->debug)
			printf("**fram 0x%x\n",frame);
	}
}
#else SLOWCODE
#asm
* 
* do_shade(frame,shadevector)
*
*	Register usage
*
*	d4  -  correction value for this square
*	d5  -  horizontal loop counter
*	d6  -  vertical loop counter
*	a4  -  Temporary frame pointer
*	a3  -  correction table
*	a2  -  Frame pointer
*
 ttl do_shade
do_shade:  movem.l d4-d6/a2-a4,-(sp)
		movea.l	d0,a2					* frame pointer
*		movea.l shadevector(a6),a3		* shade vector table
		movea.l	d1,a3					* shade vector table
		move.l	#144,d6					* nvert = LINENO/4;
		bra		ds60					* while (nvert--)  {
ds7		move.l	#192,d5					* nhoriz = LINELN/4;
		bra		ds46					* while (nhoriz--) {
ds10	move.b	(a3)+,d4				* Get shading correction and incr ptr
		bne		ds13					* Only do if non zero.
		addq.l	#4,a2					* Else skip this set and go for next square
		bra		ds46
*
* 1st line of 4 pixels
*
ds13	sub.b	d4,(a2)+				* Apply correction
		bcc.s	ds16					* check for underflow
		clr.b	-1(a2)					* If it it went under zap it to zero
ds16	sub.b	d4,(a2)+
		bcc.s	ds18
		clr.b	-1(a2)
ds18	sub.b	d4,(a2)+
		bcc.s	ds20
		clr.b	-1(a2)
ds20	sub.b	d4,(a2)+
		bcc.s	ds22
		clr.b	-1(a2)
*
* 2nd line of 4 pixels
*
ds22	move.l	a2,a4					* Get copy of frame ptr
		adda.l	#764,a4					* Go down 1 line + back 4 pixels (LINELN-4)
		sub.b	d4,(a4)+				* Apply correction
		bcc.s	ds24					* check for underflow
		clr.b	-1(a4)					* If it it went under zap it to zero
ds24	sub.b	d4,(a4)+
		bcc.s	ds26
		clr.b	-1(a4)
ds26	sub.b	d4,(a4)+
		bcc.s	ds28
		clr.b	-1(a4)
ds28	sub.b	d4,(a4)
		bcc.s	ds30
		clr.b	(a4)
*
* 3rd line of pixels
*
ds30	adda.l	#765,a4					* Go down 1 line and back 3 pixels
		sub.b	d4,(a4)+				* Apply correction
		bcc.s	ds32					* check for underflow
		clr.b	-1(a4)					* If it it went under zap it to zero
ds32	sub.b	d4,(a4)+
		bcc.s	ds34
		clr.b	-1(a4)
ds34	sub.b	d4,(a4)+
		bcc.s	ds36
		clr.b	-1(a4)
ds36	sub.b	d4,(a4)
		bcc.s	ds38
		clr.b	(a4)
*
*	4th line of pixels
*
ds38	adda.l	#765,a4					* Go down 1 line and back 3 pixels
		sub.b	d4,(a4)+				* Apply correction
		bcc.s	ds40					* check for underflow
		clr.b	-1(a4)					* If it it went under zap it to zero
ds40	sub.b	d4,(a4)+
		bcc.s	ds42
		clr.b	-1(a4)
ds42	sub.b	d4,(a4)+
		bcc.s	ds44
		clr.b	-1(a4)
ds44	sub.b	d4,(a4)
		bcc.s	ds46
		clr.b	(a4)
ds46	dbra	d5,ds10					* Loop for each block on line
		adda.l	#2304,a2				* Skip down past 3 lines already done
ds60	dbra	d6,ds7					* Loop for each set of 4 lines
		movem.l	(sp)+,d4-d6/a2-a4
		rts
#endasm
#endif SLOWCODE
/*
 *	D O _ I N V S H A D E  --  Apply shading correction to current framestore
 *								image
 *
 */
#ifdef SLOWCODE
/*
 *	Warning - there is a compiler bug in here and hence the slow version
 *	won't work.
 *			if ((*a++ += b) > 255)  -> double increments 'a'
 */
do_ivnshade(frame,shadevector)
register unsigned char	*frame;
{
	register unsigned char *convtab, conval, *wkptr;
	register int	nhoriz, nvert;
	convtab = shadevector;
	nvert = LINENO/4;
	while (nvert--)  {
		nhoriz = LINELN/4;
		while (nhoriz--) {
			if (alc->debug)
				printf("fram 0x%x\n",frame);
			if ((conval = *convtab++) == 0) {
				frame += 4;
				if (alc->debug)
					printf("conv\n");
				continue;
			}
/* 1st line */
			if ((*frame++ += conval) > 255) {*(--frame) = 255; frame++; }
			if ((*frame++ += conval) > 255) {*(--frame) = 255; frame++; }
			if ((*frame++ += conval) > 255) {*(--frame) = 255; frame++; }
			if ((*frame++ += conval) > 255) {*(--frame) = 255; frame++; }
			wkptr = frame + (LINELN-4);
/* 2nd line */
			if ((*wkptr++ += conval) > 255) {*--wkptr = 255; wkptr++; }
			if ((*wkptr++ += conval) > 255) {*--wkptr = 255; wkptr++; }
			if ((*wkptr++ += conval) > 255) {*--wkptr = 255; wkptr++; }
			if ((*wkptr += conval) > 255) {*wkptr = 255;}
			wkptr += (LINELN-3);
/* 3rd line */
			if ((*wkptr++ += conval) > 255) {*--wkptr = 255; wkptr++; }
			if ((*wkptr++ += conval) > 255) {*--wkptr = 255; wkptr++; }
			if ((*wkptr++ += conval) > 255) {*--wkptr = 255; wkptr++; }
			if ((*wkptr += conval) > 255) {*wkptr = 255;}
			wkptr += (LINELN-3);
/* 4th line */
			if ((*wkptr++ += conval) > 255) {*--wkptr = 255; wkptr++; }
			if ((*wkptr++ += conval) > 255) {*--wkptr = 255; wkptr++; }
			if ((*wkptr++ += conval) > 255) {*--wkptr = 255; wkptr++; }
			if ((*wkptr += conval) > 255) {*wkptr = 255;}
		}
		frame += (3*LINELN);
		if (alc->debug)
			printf("**fram 0x%x\n",frame);
	}
}
#else SLOWCODE
#asm
* 
* do_invshade(frame,shadevector)
*
*	Register usage
*
*	d4  -  correction value for this square
*	d5  -  horizontal loop counter
*	d6  -  vertical loop counter
*	a4  -  Temporary frame pointer
*	a3  -  correction table
*	a2  -  Frame pointer
*
 ttl do_invshade
do_invshade:  movem.l d4-d6/a2-a4,-(sp)
		movea.l	d0,a2					* frame pointer
*		movea.l shadevector(a6),a3		* shade vector table
		movea.l	d1,a3					* shade vector table
		move.l	#144,d6					* nvert = LINENO/4;
		bra		di60					* while (nvert--)  {
di7		move.l	#192,d5					* nhoriz = LINELN/4;
		bra		di46					* while (nhoriz--) {
di10	move.b	(a3)+,d4				* Get shading correction and incr ptr
		bne		di13					* Only do if non zero.
		addq.l	#4,a2					* Else skip this set and go for next square
		bra		di46
*
* 1st line of 4 pixels
*
di13	add.b	d4,(a2)+				* Apply correction
		bcc.s	di16					* check for underflow
		move.b	#255,-1(a2)				* If it it went under zap it to zero
di16	add.b	d4,(a2)+
		bcc.s	di18
		move.b	#255,-1(a2)
di18	add.b	d4,(a2)+
		bcc.s	di20
		move.b	#255,-1(a2)
di20	add.b	d4,(a2)+
		bcc.s	di22
		move.b	#255,-1(a2)
*
* 2nd line of 4 pixels
*
di22	move.l	a2,a4					* Get copy of frame ptr
		adda.l	#764,a4					* Go down 1 line + back 4 pixels (LINELN-4)
		add.b	d4,(a4)+				* Apply correction
		bcc.s	di24					* check for underflow
		move.b	#255,-1(a4)				* If it it went under zap it to zero
di24	add.b	d4,(a4)+
		bcc.s	di26
		move.b	#255,-1(a4)
di26	add.b	d4,(a4)+
		bcc.s	di28
		move.b	#255,-1(a4)
di28	add.b	d4,(a4)
		bcc.s	di30
		move.b	#255,(a4)
*
* 3rd line of pixels
*
di30	adda.l	#765,a4					* Go down 1 line and back 3 pixels
		add.b	d4,(a4)+				* Apply correction
		bcc.s	di32					* check for underflow
		move.b	#255,-1(a4)				* If it it went under zap it to zero
di32	add.b	d4,(a4)+
		bcc.s	di34
		move.b	#255,-1(a4)
di34	add.b	d4,(a4)+
		bcc.s	di36
		move.b	#255,-1(a4)
di36	add.b	d4,(a4)
		bcc.s	di38
		move.b	#255,(a4)
*
*	4th line of pixels
*
di38	adda.l	#765,a4					* Go down 1 line and back 3 pixels
		add.b	d4,(a4)+				* Apply correction
		bcc.s	di40					* check for underflow
		move.b	#255,-1(a4)				* If it it went under zap it to zero
di40	add.b	d4,(a4)+
		bcc.s	di42
		move.b	#255,-1(a4)
di42	add.b	d4,(a4)+
		bcc.s	di44
		move.b	#255,-1(a4)
di44	add.b	d4,(a4)
		bcc.s	di46
		move.b	#255,(a4)
di46	dbra	d5,di10					* Loop for each block on line
		adda.l	#2304,a2				* Skip down past 3 lines already done
di60	dbra	d6,di7					* Loop for each set of 4 lines
		movem.l	(sp)+,d4-d6/a2-a4
		rts
#endasm
#endif SLOWCODE

/*
 *	L O W E S T  --
 *
 */
static int lowest(ctb,size)
unsigned char *ctb;
int		size;
{
	register int lowest;
	lowest = 256;
	while (size--) {
		if (*ctb < lowest) {
			if (*ctb == 0)
				return(0);
			lowest = *ctb;
		}
		ctb++;
	}
	return(lowest);
}
/*
 *	H I G H E S T  --
 *
 */
static int highest(ctb,size)
unsigned char *ctb;
int		size;
{
	register int highest;
	highest = 0;
	while (size--) {
		if (*ctb > highest) {
			if (*ctb == 255)
				return(255);
			highest = *ctb;
		}
		ctb++;
	}
	return(highest);
}

/*
 *	S T R I P T O P
 *
 */
static int striptop(ctb,size,val)
unsigned char *ctb;
int		size,val;
{
	while(size--) {
		if (*ctb >= val) {
			*ctb = 0;
		}
		ctb++;
	}
}

/*
 *	S U B T R A C T  --  
 *
 */
static subtract(ctb,size,val)
register unsigned char *ctb;
register int	size,val;
{
	while (size--) {
		*ctb++ -= val;
	}
}

/*
 *	L O W F R A M E 4  --  Get estimate of minimum screen value (background) 
 *							looking at one pixel in each 4x4 block
 */
lowframe4(xcount,ycount,frame)
int		xcount, ycount;
register unsigned char	*frame;
{
	register unsigned char minframe;
	register int	ncols, lineincrement;

	minframe = 255;
	lineincrement = 3 * 4 * xcount;			/* Work one line, skip three */
	while (ycount--)  {
		ncols = xcount;
		while (ncols--) {
			if (*frame < minframe)
				minframe = *frame;
			frame += 4;
		}
		frame += lineincrement;
	}
	return(minframe);
}
/*
 *	H I G H F R A M E 4  --  Get estimate of maximum screen value
 *							looking at one pixel in each 4x4 block
 */
highframe4(xcount,ycount,frame)
int		xcount, ycount;
register unsigned char	*frame;
{
	register unsigned char maxframe;
	register int	ncols, lineincrement;

	maxframe = 0;
	lineincrement = 3 * 4 * xcount;			/* Work one line, skip three */
	while (ycount--)  {
		ncols = xcount;
		while (ncols--) {
			if (*frame > maxframe)
				maxframe = *frame;
			frame += 4;
		}
		frame += lineincrement;
	}
	return(maxframe);
}

