/*
 *		S I M S L I D E  --  Slide simulation drawing for met finding
 *
 *  Written:
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Orig version:	Nov 85 : Denis Rutovitz/Mark Mckie
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    8th October 1986
 *
 *  Modifications
 *
 *	19 Jul 1991		CAS			Move mult_slide bay box display here
 *	16 Jul 1991		CAS			Consistancy mods round drawbox
 */
#include <stdio.h>
#include <kyy.h>
#include <slides.h>
#include <disp_defs.h>

#define BOXX 3450		/* Where the offsets box is */
#define BOXY 3825
#define ORIG 147
#define TOP 4036
#define LENGTH 3888		/* length of gratic lines to draw on screen */

static int numbx ,numby;

/*
 * initilisation of bay display structures
 */
static struct item item1[] = {{"Total", 0, -1}, {"Reviewed", 0, -1}, 
								{"Usable", 0, -1}};
static struct item item2[] = {{"Total", 0, -1}, {"Reviewed", 0, -1}, 
								{"Usable", 0, -1}};
static struct item item3[] = {{"Total", 0, -1}, {"Reviewed", 0, -1}, 
								{"Usable", 0, -1}};
static struct item item4[] = {{"Total", 0, -1}, {"Reviewed", 0, -1}, 
								{"Usable", 0, -1}};
static struct boxdisplay msld[] = {
	{"Bay 1", 3, 0, 0, 0, 0, 1, item1}, {"Bay 2", 3, 0, 0, 0, 0, 1, item2},
	{"Bay 3", 3, 0, 0, 0, 0, 1, item3}, {"Bay 4", 3, 0, 0, 0, 0, 1, item4}
};


/*
 *	S I M S L I D E --
 *
 * assumes     fixrgb();   already called 
 */
simslide()
{
/* display frosted glass bit */
	colour(6);
	moveto(SLMXBOT,SLMYTOP-FROSTH);
	pixbegin(8,1,1,SLMWIDE,FROSTH); 
	pixel(2);
	pixend(); 
/* Draw rest of slide */
	moveto(SLMXBOT,SLMYBOT);
	pixbegin(8,1,1,SLMWIDE,SLMHIGH-FROSTH);
	pixel(4);
	pixend(); 
/* green cross at slide centre */
	moveto(XMID,YMID);
	moveby (-10,-10);
	lineby (20,20);
	moveby (-20,0);
	lineby (20,-20);
	colour(1);
/* draw box round slide area */
	moveto ( SLMXBOT,SLMYBOT );
	lineto ( SLMXBOT,SLMYTOP);
	lineto ( SLMXTOP,SLMYTOP);
	lineto ( SLMXTOP,SLMYBOT);
	lineto ( SLMXBOT,SLMYBOT);
/* Put in top/bottom limit markers */
	moveto(SLMXBOT,BOTLIM);
	lineby(40,0);
	moveto(SLMXTOP,BOTLIM);
	lineby(-40,0);
	moveto((SLMXTOP+30),BOTLIM);
	tsize (64,64);
	text("lower limit");
	moveto((SLMXTOP+30),TOPLIM);
	text("upper limit");
	moveto(SLMXTOP,TOPLIM);
	lineby(-40,0);
	moveto(SLMXBOT,TOPLIM);
	lineby(40,0);
}
static drawscale(x,y,dir) int x,y; char dir; {
	char nline[10];
	moveto(x,y);
	switch(dir){
	case 'x':lineby(0,50);
			 numbx += 20;
			 sprintf(nline,"%2d",numbx);
			 moveto((x-32),10);
			 text(nline);		
			 break;
	case 'y':lineby(50,0);
			 numby += 20;
			 sprintf(nline,"%2d",numby);
			 moveto(10,(y-32));
			 text(nline);		
			 break;
	}
}

/*
 * U P D A T E B A Y  --- update and refresh bay box
 *
 */
int
updatebay(bp)
struct mult_slide *bp;
{
	int bayno;
	
	bayno = baynn(bp);
	updatebox(&msld[bayno - 1], bp->totin, bp->revtot, bp->usbltot);
	refreshbox(&msld[bayno - 1]);
}

/*
 *	S H O W B A Y S --- Graphics to show progress of scan
 */
showbays()
{
struct	mult_slide *bp;
int	i, xpos = 700, ypos = 4000, bayno;

	for (i = 0, bp = alc->frsbay; i < alc->maxbays; i++) {
		updatebox(&msld[i], bp->totin, bp->revtot, bp->usbltot);
		msld[i].bust = 1;
		drawbox(xpos, ypos, &msld[i]);
		intens(0);							/* Erase old highlight box */
		drawborder(&msld[i]);
		intens(1);
		if(bp == alc->curbay)
			drawborder(&msld[i]);
		if(!bp->active)
			drawboxcross(&msld[i], 1);
		bp++;
		ypos -= (msld[i].bh + ITEM_HEIGHT);
	}
}

/*
 *	S T A G E  --  Display stage offset in box at top right of screen
 *					Currently disabled
 */
stage(xstage,ystage)
int xstage,ystage;
{
	char xorig[10],yorig[10];
	sprintf(xorig,"%3d",xstage);
	sprintf(yorig,"%3d",ystage);
	moveto(3100,TOP);
	tsize(54,54);
	colour (6);
	text("Stage offset ");
	moveto(3300,(TOP-150));
	pixbegin(8,1,1,200,100);	/*clear the old value (blue wash)*/
	pixel(4); 
	pixend();
	moveto(3300,(TOP-300));
	pixbegin(8,1,1,200,100);	/*clear the old value (blue wash)*/
	pixel(4); 
	pixend();
	colour(2);
	moveto(3100,(TOP-150));
	text("x = ");
	moveto(3300,(TOP-150));
	text(xorig);
	moveto(3100,(TOP-300));
	text("y = ");
	moveto(3300,(TOP-300));
	text(yorig);
	moveto(BOXX,BOXY);
	box(900,450);
}

/*
 *	G R A T I C  --  display of graticule on screen
 *
 */
gratic(scale) 
{
	int xcors,ycors,i;
	char xdir='x';char ydir='y';
	numbx = -60;
	numby = -60;
	colour(2);
	tsize(48,52);
	moveto (ORIG -100, ORIG -100);
	text("microns");
	tsize(36,32);
	moveto(ORIG,TOP);
	lineby(0,-LENGTH);
	lineby(LENGTH,0);
	for(i=1;i<=7;++i){
		ycors = i*scale + ORIG;
		xcors = ORIG;
		drawscale(xcors,ycors,ydir);
		xcors=ycors;
		ycors=ORIG;
		drawscale(xcors,ycors,xdir);
	}
}

/*
 *	D I G C O U N T  --   displays count of digitised cells 
 *
 */
filcount(n)
int n;
{
	char str[8];
	int cent;

	cent = alc->ddgschannel->ycenter;
	moveto(cent, 150);
	colour(2);
	tsize(64,64);
	text("Cells filed ");

	moveto (cent + 800, 150-25);
	colour(255);
	pixbegin(8,1,1,640,200);	/*clear the old value (blue wash)*/
	pixel(4); 
	pixend();

	sprintf(str,"%3d",n);
	moveto(cent + 900, 150);
	colour(1);
	tsize(128,128);
	text(str);
	tsize(64,64);
}

/*
 *	D R A W B O X -- Draw box for scores, and associate x,y pos. with disp
 */
int
drawbox(x, y, disp)
int x, y;
struct boxdisplay *disp;
{
	register int i, xpos, ypos;

	disp->bh = (disp->nitems * ITEM_HEIGHT) + BOTTOM_INDENT + TOP_INDENT;
	disp->bw = BOX_WIDTH;
	disp->tx = x;
	disp->ty = y;

	/*
	 * Draw box background
	 */
	moveto(disp->tx, disp->ty-disp->bh);
	colour(BACKCOLOUR);
	pixbegin(8, 1, 1, disp->bw, disp->bh);
	pixel(4);
	pixend();
	/*
	 * Draw strings
	 */
	colour(FORECOLOUR);
	tsize(64, 64);
	for (i=0, xpos=x+LEFT_INDENT, ypos=y-ITEM_HEIGHT-TOP_INDENT; i<disp->nitems;
					i++, ypos -= ITEM_HEIGHT) {
		moveto(xpos, ypos);
		text((disp->il + i)->name);
	}
	/*
	 * Draw title
	 */
	moveto(x + TL_XOFF, y + TL_YOFF);
	tsize(104, 104);
	text(disp->title);

	disp->bust = 1;
	refreshbox(disp);
} /* end drawbox */

/*
 *	R E F R E S H B O X -- Redraw values that have changed
 */
int
refreshbox(disp)
struct boxdisplay *disp;
{
	register int i, xpos, ypos;
	char str1[10];

	tsize(64, 64);
	colour(FORECOLOUR);
	for (i=0, xpos=disp->tx+VALUE_INDENT, ypos=disp->ty-ITEM_HEIGHT-TOP_INDENT;
			i<disp->nitems;	i++, ypos -= ITEM_HEIGHT) {
		/* 
		 * display and update 
		 */
		if((disp->bust == 1) || 
				((disp->il + i)->val != (disp->il + i)->oldval)) {	
			moveto(xpos, ypos);
			intens(0);
			sprintf(str1, "%d", (disp->il + i)->oldval);
			text(str1);
			intens(1);
			sprintf(str1, "%d", (disp->il + i)->val);
			text(str1);
			(disp->il + i)->oldval = (disp->il + i)->val;
		}
	}
	disp->bust = 0;
} /* end refreshbox */

/*
 *	D R A W B O R D E R -- Put a border round the box
 *	x, y positions are taken from disp. structure
 */
int
drawborder(disp)
struct boxdisplay *disp;
{

	lwidth(2);
	colour(FORECOLOUR);
	moveto(disp->tx+(disp->bw/2), disp->ty-(disp->bh/2));
	box(disp->bw, disp->bh);
	lwidth(1);
} /* end drawborder */

/*
 * U P D A T E B O X -- Associate parameters with box structure
 *
 */
int
updatebox(box, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10)
struct boxdisplay *box;
int	p1, p2, p3, p4, p5, p6, p7, p8, p9, p10;
{
	switch (box->nitems) {
		case 10: updateitem((box->il + 9), p10);
		case  9: updateitem((box->il + 8), p9);
		case  8: updateitem((box->il + 7), p8);
		case  7: updateitem((box->il + 6), p7);
		case  6: updateitem((box->il + 5), p6);
		case  5: updateitem((box->il + 4), p5);
		case  4: updateitem((box->il + 3), p4);
		case  3: updateitem((box->il + 2), p3);
		case  2: updateitem((box->il + 1), p2);
		case  1: updateitem(box->il, p1);
		default: break;
	}
} /* end updatebox */

/*
 * U P D A T E I T E M -- Update a specific data item in disp structure
 */
static int
updateitem(it, value)
struct item *it;
int value;
{
	if (it->val != value) {			/* Must have changed */
		it->oldval = it->val;		/* Keep the old value */
		it->val = value;
	}
} /* end updateitem */

/* 
 *  D R A W C R O S S
 */
int
drawboxcross(box, col)
struct boxdisplay *box;
int col;
{
	
	lwidth(3);
	colour(col);
	moveto(box->tx, box->ty);
	lineto(box->tx+box->bw, box->ty-box->bh);
	moveto(box->tx, box->ty-box->bh);
	lineto(box->tx+box->bw, box->ty);
	lwidth(1);
}

