/*
 *		S U B S E T . C  --  cell list subset selection for ky system
 *   	authors:  Mark Mckie, Denis Rutovitz     May/June 85
 *
 *
 *
 *
 * Modifications:
 *
 *	 6 Jul 1988		CAS			Tidy cmmd lines
 *	13 Jun 1988		SCG			Possible cmdsys bug investigation
 *	 8 Jun 1988		SCG			Argument in chekattr put in registers for speed
 *   9 May 1988		SCG/SEAJ	Changed number of arguments in chekattr
 *	21 Aug 1987		SCG			small correction in mksubtxt, reordering list message
 *	13 Aug 1987		SCG			Normal/Abnormal removed; simpler subset mu
 *	07 Aug 1987		SCG			Change args of chekattr
 *	13 Jul 1987		SCG			Major mods for 4-slide stage. All subset
 *								relevant statics now removed to mult_slide
 *								struct
 *	13 Jan 1987		SCG			Modification to allcells.
 *	13 Jan 1987		SCG			Modifications to subset menu.
 *	12 Jan 1987		SCG			Findnext is modified to shown and print
 *								combinations of classifieds etc.
 *	12 Jan 1987		SCG			Allcells called from getlist and getmph
 *								with argument 2 to set inrank on.
 *	 9 Jan 1987		SCG			Extensive changes in the whole of this 
 *								module.  These changes are primarily
 *								to do with the old inrank philosophy
 *								and they have cured some lock-ups in the
 *								"for" loop in findnext.
 *	26 Nov 1986		SCG			Tidied up indentation and removed DR's
 *								diagnostics.  Routine allcells has been
 *								modified such that it responds in the
 *								following way:
 *								allcells(-1) - File usables.
 *								allcells(1) - Reset rank and record limits to
 *								default values and subset selection to all.
 *								Getmph and getlist call with allcells(1) now.
 *	 2 Oct 1986		CAS			Ensure debugging switches off
 *	16 Sep 1986		CAS			Debuging - and cosmetic
 *	15 Aug 1986		CAS			Additional debug (allcells)
 *								and Cosmetic
 *	 7 Aug 1986		GJP			Remove the cmdspace includes
 *  20 May 1986		CAS			Changed "" to <> in Mizar includes
 *  29 Apr 1986		DR			'mfscore' changed to 'inbox' in clustdat
 *  							ref. 'inbox' in whb chg to inbx (compiler!)
 *  24 Apr 1986		DR			mouse selection and auto-return if inrank
 *                              selected
 *  14 Feb 1986		mark		attempt to make rank grouping start from
 *                              ranklow setting (f mark shows changes)
 *                              Store original ranklow value in olrank.
 *  22 Dec 1985		DR			allcells(2) to reset list traverse limits
 *						    	when cell is new - called from getlist in
 *						    	metlist.c
 *  14 Oct 1985		DR			Arrangements for 'best 10'/'next best 10'
 *								etc. in rank selection : thus nrank
 *
 */
#include <cmdsys.h>
#include <kyy.h>
#include <slides.h>

#define All 1

static char *whatb[] = {
	"dicentrics", "fragile x", "acentrics", "inner box"
};
	
static char *what[] = { 
	"Classified", "Junk", "Usable", "Other Metph"
};

char *strcpy();
static any_includes();
static class_comb();

extern	FILE *debugfp;			/* Miproc comms Debugfile */
extern	FILE *debug_channel;	/* System debug file */


/*
 *		A L L C E L L S  --  
 *
 *	called with:
 *				mm = -1 from metlist, to set for usables 
 *				mm = 1 from within subset to reset to all (inrank off).
 *				mm = 0 from getlist (metlist.c) and getmph (findmet.c)
 *					   to reset subset values to default (inrank on).
 *
 */
allcells(mm)
{
struct	mult_slide *bp;

	bp = alc->curbay;
	if (alc->debug) printf("Allcells \n");
	
	/* If list in set everything to default, otherwise set to zero. */
	if(bp->startptr != NULL) {
		bp->minrank = 1; 
		bp->maxrank = bp->totin; 
		bp->listlow = 1;
		bp->listhigh = bp->totin;
	} else {
		bp->minrank = 0; 
		bp->maxrank = 0; 
		bp->listlow = 0;
		bp->listhigh = 0;
	}
	*isub = 0;
	*esub = 0;
	*rsub = 0;
	if(mm == 1) {
		bp->byg = 0;
		bp->allrecs = 1;
	} else if(mm == 0) {
		bp->byg = 1;
		bp->allrecs = 0;
	}
	bp->ngrps = 10;
	bp->sany = 0;
	
	/* Set all the subset selection pointers to zero. */
	*bp->wh.uubl = 0;
	*bp->wh.unum = 0;
	*bp->wh.juu = 0;
	*bp->wh.classd = 0;
	*bp->whb.dicc = 0;
	*bp->whb.fragg = 0;
	*bp->whb.ringg = 0;
	*bp->whb.inbx = 0;
	
	mksubtxt();
}

/*
 *	X I S W O P  --
 *
 */
xiswop(t) 
char *t; 
{
	switch (*t) {
		case 'E' : 
			*t = 0; 
			break;
		case  0  : 
			strcpy ( t,"Include" ); 
			break;
		case 'I' : 
			strcpy (t, "Exclude" );
	}
}

/*
 *	C H E K A T T R  --
 *
 *				Checks whether the current record belongs in the current
 *				subset.  Returns TRUE if so, FALSE otherwise.
 *
 */
chekattr(bp, mpt, rl, rh) 
register struct mult_slide *bp;
register struct	mlclust *mpt;
int	rl, rh;
{
char	klass, done, scordun;


	if (rl > mpt->qrank || rh < mpt->qrank) 	/* Outside rank range */
		return(0);
		
	if (bp->allrecs) 							/* Always return 1 */
		return(1);
												
	if (bp->listlow > mpt->recnr || 			/* Outside number range */
						bp->listhigh < mpt->recnr) 
		return(0);
		
	klass = mpt->fclass;
	done = (USABLE | JUNK | UNUMET) & klass;	/* To find classifieds */
		
	switch(*bp->whb.inbx) {						/* Miproc box classifier */
		case  0 : break;						/* Neither included or
												   excluded */
												/* Included */
		case 'I': if (!mpt->inbox) return(0);	/* Not this record */
				  break;						/* This record included */
				  								/* Excluded */
		case 'E': if (mpt->inbox) return(0);	/* This record excluded */
	}
	switch (*bp->wh.classd ) {
		case 0   : break;			
		case 'I' : if (!done) return(0); 
				   break;
		case 'E' : if (done) return(0);
	}
	if(! class_comb(klass, mpt, bp)) return(0);
	switch (*bp->whb.dicc ) {
		case 0   : break;			
		case 'I' : if (mpt->udic <= 0) return(0); 
				   break;
		case 'E' : if (mpt->udic > 0) return(0);
	}
	switch (*bp->whb.fragg ) {
		case 0   : break;			
		case 'I' : if (mpt->ufrag <= 0) return(0); 
				   break;
		case 'E' : if (mpt->ufrag > 0) return(0);
	}
	switch (*bp->whb.ringg ) {
		case 0   : break;			
		case 'I' : if (mpt->uring <= 0) return(0); 
				   break;
		case 'E' : if (mpt->uring > 0) return(0);
	}
	return(1);
}				
/*
 *
 *  C L A S S _ C O M B
 *
 *		Includes (or excludes) combinations of classifieds.
 *
 */
static
class_comb(klass, mpt, bp)
char	klass;
struct	mlclust *mpt;
struct	mult_slide *bp;
{

	if(klass&JUNK) {
			switch (*bp->wh.juu) {
				case 0   : return(any_includes(mpt, bp));			
				case 'I' : return(1);
				case 'E' : return(0);
			}
	} else if(klass&USABLE) {
			switch (*bp->wh.uubl ) { 
				case 0   : return(any_includes(mpt, bp));			
				case 'I' : return(1);
				case 'E' : return(0);
			}
	} else if(klass&UNUMET) {
			switch (*bp->wh.unum ) { 
				case 0   : return(any_includes(mpt, bp));			
				case 'I' : return(1);
				case 'E' : return(0);
			}
	} else
		return(any_includes(mpt, bp));
		
}		

/*
 *
 *	A N Y _ I N C L U D E S
 *
 *		Returns 0 if no includes in subset else 1.
 */
static
any_includes(mpt, bp)
struct	mlclust *mpt;
struct	mult_slide *bp;
{

	if (*bp->wh.uubl == 'I' || *bp->wh.unum == 'I' || 
										*bp->wh.juu == 'I')
		return(0);
	else 
		return(1);
	
}

/*
 *	M K S U B T X T
 * 
 *			Constructs the text drawn in subset window.  This text shows
 *			what the current subset is composed of,
 *
 */
mksubtxt() 
{ 
char *i, *ji, *je, k;
struct	mult_slide *bp;

	bp = alc->curbay;
	if(bp->startptr == NULL) {			/* NULL the strings if no list */
		*isub = 0;
		*esub = 0;
		*rsub = 0;
		return;
	}
 	strcpy( isub, "Include: "); 
 	ji = isub + 9;
 	strcpy( esub, "Exclude: ");  
 	je = esub + 9;
	if (bp->byg) {
		strcpy(rsub, "Inrank on.");
	} else {
		*rsub = 0;
	}
	i = bp->wh.classd;
	
	for (k = 0; k <= 3; ++k) {			/* Normal subsets */
		switch (*i) {
			case 0:	
				break;
			case 'I':
				strcpy (ji, what[k]);
				while (*ji++); 
				*(ji - 1) = ',';
				break;
			case 'E':
				strcpy (je, what[k]);
				while (*je++); 
				*(je - 1) = ',';
		}
		i += 8;
	}
	i = bp->whb.dicc;
	for (k = 0; k <= 3; ++k) {			/* Abnormals subset */
		switch (*i) {
			case 0: 
				break;
			case 'I': 
				strcpy (ji, whatb[k]);
				while (*ji++); 
				*(ji - 1) = ',';
				break;
			case 'E': 
				strcpy (je, whatb[k]);
				while (*je++); 	
				*(je - 1) = ',';
		}
		i += 8;
	}
	if (bp->listlow > 1) {
		sprintf (je, "numbers<%d,", bp->listlow);
		while (*je++); 
		--je;
	}
	if (bp->listhigh < bp->totin) {
		sprintf (je , "numbers>%d,", bp->listhigh);
		while (*je++); 
		--je;
	}
	if (bp->minrank > 1) {
		sprintf ( je , "ranks<%d,", bp->minrank);
		while (*je++); 
		--je;
	}
	if (bp->maxrank < bp->totin) {
		sprintf (je , "ranks>%d,", bp->maxrank);
		while (*je++); 
		--je;
	}
	if (ji > isub +9) {
		*(--ji) = 0; 
	} else {
		*isub = 0;
	}
	if (je > esub + 9) {
		*(--je) = 0; 
	} else {
		*esub =0;
	}
}

/*
 *
 *   S U B S E T --
 *
 *				Menu for subset selection.  Called from metlist (metlist.c).
 *
 */
subset() 
{
struct	mult_slide *bp;
int	Acmd, Rcmd, Ccmd, Incmd, ngrps;

	bp = alc->curbay;
	ngrps = bp->ngrps;
	
    newmenu( "Subset Selection");
	kbegin = 40;
	cmmd ("all;%t:l17:k10%-s:l18:k10%-s:l19:k10 %-s",&bp->allrecs,isub,esub,rsub);
	  	Acmd = mmcmd;
	cmmd ("classified;%-s%f",bp->wh.classd,xiswop);
	cmmd ("junk;%-s%f",bp->wh.juu,xiswop); 
	cmmd ("usable;%-s%f",bp->wh.uubl,xiswop);
	cmmd ("other metaphase;%-s%f",bp->wh.unum,xiswop);
	cmmd ("dicentrics;%-s%f",bp->whb.dicc,xiswop);
	cmmd ("fragile x;%-s%f",bp->whb.fragg,xiswop);
	cmmd ("acentrics;%-s%f",bp->whb.ringg,xiswop);
	cmmd ("record numbers;from <1/2000>%d to <1/2000>%?d", 
					&bp->listlow, &bp->listhigh);
		Ccmd = mmcmd;
	cmmd ("rank;from <1/2000>%d to <1/2000>%?d", &bp->minrank, 
					&bp->maxrank);
		Rcmd = mmcmd;
	cmmd ("inrank;%T by groups of <1/2000>%?d",&bp->byg, &ngrps); 
		Incmd = mmcmd;
		mouse(MIDBT);
	cmmd ("box;%-s%f",bp->whb.inbx,xiswop); 
	
	while(thismu()) {
		if(nncmd == Ccmd || nncmd == Rcmd) {
			checklims();
			if(nncmd == Ccmd)
				displin(0, Ccmd);
			else
				displin(0, Rcmd);
			putprompt();
		}
		if (nncmd > Acmd) {
			if(nncmd == Ccmd && (bp->listlow > 1 || bp->listhigh < 
														bp->totin)) {
				bp->byg = 0;
				*rsub = 0;
				displin(0, Incmd);
			}
			if(nncmd == Incmd) {
				if(ngrps != bp->ngrps) {
					bp->byg = 1;
					bp->ngrps = ngrps;
					displin(0, Incmd);
				}
				if(bp->byg) {
					bp->listlow = 1;
					bp->listhigh = bp->totin;
				}
				displin(0, 9);
			}
			bp->allrecs = 0;
			mksubtxt();
			clearlns(17, 19);				/* clear for subset selection */
			displin(0, All);
			putprompt();
		} else {							/* Include all */
			allcells(bp->allrecs);
			display();
		}
		if (mukont->wasmouse)  break;
	}
	fprintf(output,"Reordering list - please wait\n");
	linksub();								/* Link list with new subset */
}

/*
 *
 * C H E C K L I M S ---
 *
 *				Ensure that sensible values for listlow listhigh
 *				minrank and maxrank are always entered in the menu.
 *
 */
static
checklims()
{
struct	mult_slide *bp;

	bp = alc->curbay;
		
	/* Don't allow record values outside valid limits. */
	if(bp->listlow < 1) bp->listlow = 1;
	if(bp->listlow > bp->totin) bp->listlow = bp->totin;
	if(bp->listhigh < 1) bp->listhigh = 1;
	if(bp->listhigh > bp->totin) bp->listhigh = bp->totin;
	
	/* Dont allow rank values outside valid limits. */
	if(bp->minrank < 1) bp->minrank = 1;
	if(bp->minrank > bp->totin) bp->minrank = bp->totin;
	if(bp->maxrank < 1) bp->maxrank = 1;
	if(bp->maxrank > bp->totin) bp->maxrank = bp->totin;
	
	if(bp->maxrank < bp->minrank) {
		bp->maxrank = bp->minrank;
	}
	if(bp->listhigh < bp->listlow) {
		bp->listhigh = bp->listlow;
	}
	
	/* If no metaphase list in set everything to zero. */
	if(bp->startptr == NULL) {
		bp->minrank = 0;
		bp->maxrank = 0;
		bp->listlow = 0;
		bp->listhigh = 0;
	}
}	
