/*
 *      V I R T U O S O . C  --  Optimised / special code for virtuoso framestore
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1989)
 *
 *  Date:    31st July 1989
 *
 *  Modifications
 *
 */
#include <allcontr.h>

extern struct allkontr *alc;

#define XOFFSET	48		/* Integ time - edge of screen mask */
#define	YOFFSET	24		/* Integ time - top/bottom of screen mask */
#define COLINC	8		/* Integ time - sample evey COLINC cols */
#define	LINEINC	8		/*            - and LINEINC lines */

/*
 *	C A L C I N T E G T I M E  --  Calculate integration time for fluorescence
 *									automatic iris
 *
 *	Scan framestore looking at 1 in 16 pixels in specified (odd/even) field
 *	and determine whether to increase or decrease current integration time.
 *	Estimate new value + return it.
 *	Field is the last field grabbed, and hence the one to measure from
 *	Field = 0 - Line 0, 2... (offset 0, 600, c00...)
 *			1 - Line 1, 3... (offset 300, 900...)
 */
calcintegtime(current,field)
int		current, field;
{
	DDGS	*dg;
	register unsigned char *fptr;
	unsigned char	min;
	int		nmin, newtime;

	dg = alc->ddgschannel;
	fptr = dg->frame + dg->lineln * (YOFFSET + field) + XOFFSET;
	calcintloop(fptr,dg->lineln - 2*XOFFSET, dg->lineno - 2*YOFFSET, dg->lineln, &min, &nmin, LINEINC, COLINC);
	if (current <= 0) current = 1;
	if (min == 0) {					/* Saturation - lower the time */
		if (nmin <= 2)				/* Leave alone if only just saturating <= 0.2% */
			newtime = current;
/*
 *	Reduce integration time by a factor of twice the percentage of saturating
 *	points to give very fast pull down response. It seems to be better to
 *	overcook the pull down and recover than go the other way (Its quicker +
 *	psychologically better since image gets brighter rather than seen to
 *	apparently fade slowly)
 */
 		else if (nmin < 13) {						/* 0.3% to 1.2% */
 			newtime = current * 95 / 100;
			if (newtime == current)
				newtime--;
		} else if (nmin < 50) {						/* 1.3% to 4.9% */
			newtime = current * (1000 - 8 * nmin) / 1000;
			if (newtime == current)
				newtime--;
		} else {									/* 5.0% on */
			newtime = current >> 3;
		}
	} else if (min > 3) {				/* Too dark - lengthen the time */
		newtime = current * 255 / (256 - min);
		if (newtime == current)
			newtime++;
	} else
		newtime = current;
	if (alc->devel > 10 && (newtime != current))
		printf("Integration time (current %d, field %d) min %d, nmin %d, newtime %d\n",current,field,min,nmin,newtime);
	return(newtime);
}

/*
 *	C A L C I N T L O O P  --
 *
 *	nmin is returned as a percentage (*10) of the tested points that
 *	are at the minimum value
 */
#ifdef SLOWCODE
calcintloop(fptr,cols,lines,lineln,min,nmin,lineinc,colinc)
register unsigned char *fptr;
int	cols, lineln, lineinc, colinc;
register unsigned char *min;
register int lines, *nmin;
{
	register int	col;
	int		eoline, npoints;
	*min = 255;
	*nmin = 0;
	cols = cols/colinc + 1;
	lines = lines/lineinc + 1;
	npoints = (cols-1) * (lines-1);
	eoline = lineln - (cols-1) * colinc + (lineln * (lineinc - 1));
	while (--lines) {
		col = cols;
		while (--col) {
			if (*fptr < *min) {
				*min = *fptr;
				*nmin = 0;
			} else if (*fptr == *min)
				(*nmin)++;
			fptr += colinc;
		}
		fptr += eoline;
	}
	*nmin = *nmin * 1000 / npoints;
}
#else SLOWCODE
#asm
 ttl calcintl
*
* calcintloop(fptr,cols,lines,lineln,min,nmin,lineinc,colinc)
* int	cols, lineln, lineinc, colinc;
*
calcintloop:	link a5,#0
		movem.l	d0-d2/d4-d5/a2-a4,-(sp)
cl_params equ	0x28
		movea.l	d0,a2					; fptr
		move.l	0+cl_params(sp),d4		; lines
		movea.l	8+cl_params(sp),a3		; *min
		movea.l	12+cl_params(sp),a4		; *nmin
		move.l	#0xffffffb8,d0
		bsr		_stkcheck

* 	register int	col;			; d5

		subq.l	#8,sp				* 	int		eoline, npoints;
		move.b	#255,(a3)			* 	*min = 255;

		clr.l	(a4)				* 	*nmin = 0;
*										* 	cols = cols/colinc + 1;
		move.l	12(sp),d0				; cols
		move.l	28+cl_params(sp),d1		; colinc
		bsr		_T$LDiv					; divide
		addq.l	#1,d0					; add 1
		move.l	d0,12(sp)				; resave
*										* 	lines = lines/lineinc + 1;
		move.l	d4,d0					; lines
		move.l	24+cl_params(sp),d1		; lineinc
		bsr		_T$LDiv					; divide
		addq.l	#1,d0					; add 1
		move.l	d0,d4 :2				; resave
*										* 	npoints = (cols-1) * (lines-1);
		move.l 12(sp),d0
		subq.l #1,d0 :2
		move.l d4,d1 :2
		subq.l #1,d1 :2
		bsr _T$UMul
		move.l d0,(sp) :2
*			* 	eoline = lineln - (cols-1) * colinc + (lineln * (lineinc - 1));
		move.l 12(sp),d0				; cols => d0
		subq.l #1,d0 :2					; cols - 1 => d0
		move.l 28+cl_params(sp),d1		; colinc => d1
		bsr _T$UMul						; (cols-1) * colinc => d0
		move.l 12+cl_params(sp),d1		; lineln => d1
		sub.l d0,d1 :2					; lineln - (cols-1)*colinc => d1
		move.l d1,d2 :2					; save copy in d2 for later
		move.l 24+cl_params(sp),d0		; lineinc
		subq.l #1,d0 :2					; lineinc - 1
		move.l 12+cl_params(sp),d1		; lineln
		bsr _T$UMul						; (lineinc-1)*lineinc
		add.l d0,d2 :2					; add it
		move.l d2,4(sp)					; save to eoline

		move.l	28+cl_params(sp),d2	; Get colinc for speed
		bra		cl_29				* 	while (--lines) {
cl_30	move.l	12(sp),d5			* 		col = cols;
		bra		cl_32				* 		while (--col) {
cl_33	move.b	(a2),d0				; Get *fptr
		cmp.b	(a3),d0				; compare against *min (*fptr - *min)
		bhi		cl_38				; Ignore if (*fptr > *min)
		beq		cl_35				; If equal - need to just bump count
		move.b	d0,(a3)				; Otherwise set up new *min
		clr.l	(a4)				; Clear out min count - fall through to set count to 1
cl_35	addq.l	#1,(a4)				* 				(*nmin)++;
cl_38	adda.l	d2,a2				* 		fptr += colinc;
cl_32	subq.l	#1,d5				* 		}
		bne		cl_33
		move.l	4(sp),d0			* 		fptr += eoline;
		adda.l	d0,a2
cl_29	subq.l	#1,d4				* 	}
		bne		cl_30
		move.l	(a4),d0				* 	*nmin = *nmin * 1000 / npoints;
		move.l	#1000,d1 :6
		bsr		_T$UMul
		move.l	(sp),d1 :2
		bsr		_T$LDiv
		move.l	d0,(a4) :2

		addq.l	#8,sp			* }
		movem.l -24(a5),d2/d4-d5/a2-a4
		unlk	a5
		rts

#endasm
#endif SLOWCODE
/*
 *	L I N E A V E R A G E  --  Filter to remove inter-field problems by
 *								averaging adjacent lines
 *
 *	Average pixels from current line + next line and put in current line for
 *	lineno - 1 lines, last line set same as last-1
 */
lineaverage()
{
	DDGS	*dg;
	register unsigned char *line1, *line2;
	register int	npix, time;

	if (alc->devel > 10) {
		printf("Line average image ");
		time = tick();
	}
	dg = alc->ddgschannel;
	line1 = dg->frame;
	line2 = line1+dg->lineln;
	npix = (dg->lineno - 1) * dg->lineln;
	laloop(line1,line2,npix,dg->lineln);
	if (alc->devel > 10) {
		time = tick() - time;
		printf(" %d.%02ds\n",time/100,time%100);
	}
}

/*
 *	L A L O O P  --  Inter line average (loop for optimising)
 *
 *	This loop will average line N+1 and N for (npix contig pixels)
 *	then duplicate the 2nd last line into the last line.
 *	line1 and line2 initially point to the start of the first two
 *	lines of the image. npix is number of pixels in image minus one line.
 */
#ifdef SLOWCODE
static laloop(line1,line2,npix,lineln)
register	unsigned char *line1, *line2;
register	int	npix, lineln;
{
	npix++;
	lineln++;
	while (--npix) {
		*line1 = (*line1 + *line2++) >> 1;
		line1++;
	}
	while (--lineln)
		*(--line2) = *(--line1);
}
#else SLOWCODE
#asm
 ttl laloop
laloop	link	a5,#0
		movem.l	d0-d1/d4-d5/a2-a3,-(sp)
la_221 equ 0x00000020				; stack dist to params
		movea.l	d0,a2				; Get line 1 ptr
		movea.l	d1,a3				; Get line 2 ptr
		move.l	0+la_221(sp),d4		; Number of pixels to do
		move.l	4+la_221(sp),d5		; line length

		moveq.l	#0,d1				; Ensure d1 clear (used to store byte val)
*
*	Assume pixel count is divisible by 4 - this give about
*	a 25% speed improvement by inlining 4 pixels + then doing test
*
la_226	moveq.l	#0,d0				;  8
		move.b	(a2),d0				; 12 ; get this pix
		move.b	(a3)+,d1			; 12 ; get pix on next line
		add.w	d1,d0				;  8 ; Add them
		asr.w	#1,d0				; 12 ; divide 2
		move.b	d0,(a2)+			; 12 ; stuff back
* Second pixel
		moveq.l	#0,d0
		move.b	(a2),d0				; get this pix
		move.b	(a3)+,d1			; get pix on next line
		add.w	d1,d0				; Add them
		asr.w	#1,d0				; divide 2
		move.b	d0,(a2)+			; stuff back
* Third pixel
		moveq.l	#0,d0
		move.b	(a2),d0				; get this pix
		move.b	(a3)+,d1			; get pix on next line
		add.w	d1,d0				; Add them
		asr.w	#1,d0				; divide 2
		move.b	d0,(a2)+			; stuff back
* Fourth pixel
		moveq.l	#0,d0
		move.b	(a2),d0				; get this pix
		move.b	(a3)+,d1			; get pix on next line
		add.w	d1,d0				; Add them
		asr.w	#1,d0				; divide 2
		move.b	d0,(a2)+			; stuff back
la_225	subq.l	#4,d4				; 12 ; Decrement counter
		bhi		la_226				; 18 ; *C.*Z
		bra		la_228
la_229	move.b	-(a2),-(a3)			; Copy last line from last minus 1
la_228	subq.l	#1,d5				; d5 contains the number of pixels per line
		bne		la_229
		movem.l	-16(a5),d4-d5/a2-a3
		unlk	a5
		rts
#endasm
#endif SLOWCODE

