 /*
 *			S A T .  C -- Satellite Capture application
 *
 *
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *	Based on km.c (1985)
 *
 *
 *  Modifications
 *
 *	15 Jan 1991		CAs			Init debug_channel to stderr if debug off in case
 *								some nurd uses it anyway
 *	19 Mar 1990	dcb		Merge with cas version, rip out MF menu nos. Get satno
 *						Read LUT file after calling sysinit().
 *	12 Mar 1990	dcb		Use lin_colour_lut instead of setcolour
 *	08 Mar 1990	dcb		Use tvdigitise instead of satdigitise
 *	 4 Jan 1990	CAS		Mods for new sat libs etc
 *	15 Dec 1989	dcb		Set priority from alc->satprior
 *
 */
#define INIFILE
#include <cmdsys.h>
#include <chromanal.h>
#include <cfipp.h>
#include <kyy.h>
#include <slides.h>
#include <kcontrol.h>
#include <signal.h>
#include <hardcopy.h>
#include <module.h>
#include <procid.h>
#include <sai.h>

#define LOGFILE		"sat.log"	/* Kmr cmdsys command log */
#define SAT_HOME_MENU "   Cytoscan Satellite Capture  Version 1.1"

extern int errno;

/*
 *		Global system data structure pointers
 *
struct allkontr *alc;			defined in kyy.h */
struct kcontrol *kcont;

FILE	*debug_channel = NULL;			/* System debug file */


static struct pframe mainframe = { 60, 8, 0, 0, 0, 0, 1, 1, 0 };

/*
 *	External function references
 */
extern int woolzexitfn();
extern int memexitfn();
extern int dispcellkm();
extern int tvdigitise();
extern int idinfo();
extern DDGS *ddgsopen();
extern int back_toggle();
extern	int	(*setexception())();
extern struct lut_assoc *readlutfile();

/*
 *	Dummy function calls
 */

setscreenformat(){}
sethcpyformat(){}
inithcpy(){}
gethelp() {}

/*
 *	Local function references
 */
static	upcontrols();
static	fstoreerror();
int		chi_sleep();
int		chi_wake();

int m2child = 0;

/*
 *	Interrupt handling storage
 */
int	(*quit_fn)() = NULL;		/* Function to call on quit signal */

/*
 * J N T E R R U P T  --  Interrupt handler
 *
 */
jnterrupt(sig)
int sig;
{
	static int busy = 0, inquit = 0;
	if (busy) return;
	busy = 1;
	if (alc->debug) {
		printf("SATCAP - seen sig %d\n",sig);
	}
	switch (sig) {
		case SIGWAKE:
			break;
		case SIGINT:
		case SIGQUIT:
			if (inquit) return;
			inquit = 1;
			if (quit_fn) {
				quit_fn();
			} else
				suicide();
			inquit = 0;
			break;
		case WOOLZSIG:
			fprintf(stderr,"Fatal error\007\007\n");
			shutdown();
		default:
			printf( "Satcap: Signal %d received \n",sig);
	}
	busy = 0;
}

/*
 *	S U I C I D E  --
 */
static
suicide()
{
	char	c, c2;
	screengo(ENDLINE,0);
	while (_gs_rdy(fileno(stdin)) > 0)
		read(fileno(stdin),&c,1);
	printf("Quit seen - Do you wish to stop CYTOSCAN ? ");
	read(fileno(stdin),&c,1);
	if (c != '\n')
		do {
			read(fileno(stdin),&c2,1);
		} while (c2 != '\n');
	if (alc->debug)
		printf("stop %d\n",c);
	if ((c == 'y') || (c == 'Y')) {
		shutdown();
	}
	while (c != '\n') {
		if (ferror(stdin))
			clearerr(stdin);
		if (feof(stdin))
			cleareof(stdin);
		c = getc(stdin);
	}
}

/*
 *	E X C E P T I O N  --  Exception traps come here
 *
 */
exception(exc,addr)
int		exc,addr;
{
	procid	pd;
	int		fd, time, date, day, tick;
	char	buffer[100];

	gprdsc(getpid(),&pd);
	sprintf(buffer,"Exception %d, addr 0x%x, base 0x%x (offset 0x%x)\n",exc,addr,pd._pmodul,addr - (int)pd._pmodul);
	errorlog(buffer);
	shutdown();
}

/*
 *	S H U T D O W N  --  Shut all down
 *
 */
static shutdown()
{
	tidyworld();
	fprintf(stderr,"CYTOSCAN: shutdown\n");
	if (!sids.only_child)
		noncritical();					/* Check not still in critical region */
	exit(1);							/* alldone(1); */
}

/*
 *	T I D Y W O R L D  --
 *
 */
static
tidyworld()
{
	int slave;

	system("tmode echo eof=4");
	if (alc->kcn) {
		for (slave=0; slave<=alc->nslaves ; slave++)
			killslave(alc->kcn,slave,SIGKILL);
	}
}


/*
 *	M A I N
 *
 */
main(argc,argv,envp)
int  argc;
char **argv,**envp;
{
	int seatnumber = 1;
	char buffer[120];
	int		(*linkerbug)(), (*tmpexcept)();
	char	devbuf[40];

	setmemalloc(48*1024);
	argc = child_init(argc,argv);	/* Must come before interrupt alteration */
	if (!sids.only_child) {
		intercept(sai_handler);
		set_handler(jnterrupt);
		set_sleep(chi_sleep,chi_wake);
	} else {
		intercept(jnterrupt);
	}

	alc = (struct allkontr *) Calloc(1,sizeof(struct allkontr));

#ifdef DEBUG
/*	_gs_devn(1,devbuf);
	fprintf(stderr,"ONLY_CHILD = %d, stdout dev = %s, env[3] %s\n",sids.only_child,devbuf,getenv("USER"));
*/
#endif

	setexception(exception);
	set_woolz_exit(woolzexitfn);
	set_kary_exit(woolzexitfn);
	set_chrom_exit(woolzexitfn);
	Alloc_exit(memexitfn);
	alc->cfipp = (struct cfipparams *)Calloc(1, sizeof(struct cfipparams));
	alc->hcpy  = (struct hardcopy *)Calloc(1, sizeof(struct hardcopy));
	alc->rankcf = (struct rankcf *)Calloc(1, sizeof(struct rankcf));

	alc->kmr_pid = getpid();
	if ((argc > 1) && (argv[2][0] == 'd')) alc->devel = 1;
	alc->satno = get_satno();	/* Get satellite no. - used by sysinit */
	sysinit(alc);
	setpr(alc->kmr_pid, alc->satprior);
	alc->lut = alc->luts = readlutfile();
/*
 *	Start up menu handler
 */
	if (!sids.only_child)
		critical();
	mouseport("/nil");	/* Where we want the mouse to be */
	muini(stdout);				/* Init cmdsys + mouse */
	alc->m2s = mukont->mouss;	/* Get mouse struct from cmdsys */
	set_header_fn(dispcellkm);	/* Do this display routine instead of the 
							  * up menu line
							  */

	pastmu = 0;				/* do recall up-menus on command string */
	uplow = 0;				/* do not identify upper-lower case */
	screengo(ENDLINE,0);	/* for benefit of time message */

	linkerbug = back_toggle;
	cmd(GLOBAL, "Background;%g", linkerbug);

/*
 *	debug_channel should only be opened if debug is on
 */
	if (alc->debug) {
	 	debug_channel = fopen(alc->debugfile,"w");
		Alloc_debug(debug_channel);
		freeobj_debug(debug_channel);
		Makestruct_debug(debug_channel);
		woolz_check_debug(debug_channel);
		/*
		 * If debug level 3 or more, turn on multiproccessing debug.
		 * slave debug turned on when we call chromlyseinit
		 */
		if (alc->debug >= 3)
			MP_debug(debug_channel); /* Multip debugging */
/*		cmdsys_debug(debug_channel,0xffff); */
	} else 
		debug_channel = stderr;
	woolz_check_level(alc->wclevel);

#ifdef DEBUG
	if (alc->debug)
		printf("Config %d seat %d, term %s, mouse %s\n miproc %s baud %d, debugfile  %s\n",
		alc->config,alc->seat,alc->ourterm,
			alc->mouseterm, alc->miprocterm, alc->miprocbaud, alc->debugfile);
#endif
/*
 *	Open the frame store on the correct device
 */
	tmpexcept = setexception(fstoreerror);
	if (alc->debug)
		fprintf(stderr,"Fstore tempexcept 0x%x, exception 0x%x\n",tmpexcept,exception);
	alc->ddgschannel = ddgsopen(alc->fstoretype,alc->fstore - 1);
	dinit();				/* Init framestore */
	clear();
	lin_colour_lut();		/* and ensure sensible lut */
	setexception(tmpexcept);
/*	karyinitpos();			/* copy desired kary layout coordinates across */
	upcontrols();
	if (alc->debug) {
		Show_alloc();
		Show_free();
	}
	kickid();				/* initialise id stuff */
	
	dodate();					/* Get system date */

	getopt(1);				/* Get user name */
	sprintf(buffer,"/dd/usr/%s",alc->user);
	if (!changedir(buffer)) {
		getwd(alc->dir,40);
	}
	if (alc->monitor == MONITOR_ALL) {
		sprintf(buffer,"/dd/usr/%s/%s%d",alc->user,LOGFILE, alc->seat);
		CSYS_log(buffer);
		CSYS_text("| Monitoring all actions\n");
	}
	setjmp(homeplace);
#ifdef DEBUG
	if (alc->debug)
		printf("seat %d, term %s, mouse %s, miproc %s, debugfile %s\n",alc->seat,alc->ourterm,
			alc->mouseterm, alc->miprocterm, alc->debugfile);
#endif
	makecellhead(&alc->cell_h);
	sathome();					/* home is where the heart lies */

	exit(0);						/* alldone(0); */
}

/*
 *	S A T H O M E
 *
 */
sathome()
{
	int Ex_cmd, firsttime = 1;

	newmenu (SAT_HOME_MENU);
		cmmd("capture ;%f",tvdigitise); mouse(MIDBT);
		cmmd ("logout"); mouse(LONGLB); Ex_cmd = mmcmd;

	do {
		if (firsttime) {
			firsttime = 0;
		} else if (!sids.only_child) {
			fflush(output);
			criticalw();
		}
		thismu();
		if (alc->debug) {
			Show_alloc();
			Show_free();
			fflush(debug_channel);
		}
	} while (nncmd != Ex_cmd);
}

/*
 *	U P L I S T
 *
 */
static struct chromosome ** 
uplist()
{
	return ( (struct chromosome **) 
				Calloc ( MAXOBJS,sizeof(struct chromosome *)));
}

/*
 *	U P C O N T R O L S  --  Generate and initialise kcontrol structure
 *
 */
static
upcontrols() 
{
	kcont = (struct kcontrol *) Calloc ( 1, sizeof( struct kcontrol ) );
	initkcontrol(kcont);
	alc->kcn = kcont;			/* make kcontrol structure available through
										   'allcontrol' system */
/*
 *	Now overide defaults with sensible values
 */
	kcont->ool = uplist();
	kcont->rol = uplist();
	kcont->bol = (struct object **)uplist();
	kcont->mol = uplist();
	kcont->kol = uplist();
/*	kcont->grol1 = NULL;		/ * Becomes keeplist in karyotyping. */
	kcont->f = &mainframe;
	kcont->ivf = (struct pframe *) Calloc ( MAXOBJS,sizeof(struct pframe));
	kcont->acl = (SMALL *)Calloc (MAXOBJS,sizeof(SMALL));
	kcont->seat = alc->seat - 1;
	kcont->terminal = alc->ourterm;
	kcont->mouse = alc->mouseterm;
	kcont->ddgschannel = alc->ddgschannel;
	kcont->FIP = -1;	/* Neither TV nor FIP */
#ifdef DEBUG
	if (alc->debug) {
		printf("kcont - 0x%x seat %d\n",kcont, kcont->seat);
		printf("kcont - mouse %s terminal %s\n",kcont->mouse, kcont->terminal);
	}
#endif
}

/*
 *	F S T O R E E R R O R  --   farme store acces error handler
 *
 */
static fstoreerror()
{
	fprintf(stderr,"Framestore - incorrect address or missing\n");
	shutdown();
}

/*
 *	C H I _ S L E E P  --  Cleanup world before giving up sat video bus
 */
int
chi_sleep()
{
	fgfreeze();
	scrn_clr();
	clear();
	ddgsoverlay(ON);
	setdisplay(BUST);

	return(0);
}

/*
 *	C H I _ W A K E  --  Repaint world to resume process
 *
 */
int
chi_wake()
{
	display();
	back_restore();
	return(0);
}

