/*
 *      S A T D I G . C  -- RK Satellite System digitise from TV Image
 *
 *
 *  Written:
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    11th August 1989
 *
 *
 * Modifications
 *
 *	14 Mar 1991 ih		Add a check of alc->debug before print in initfg
 *	23 Oct 1989	dcb		Set display BUST after show histo. Flush input at start
 *
 */
#include <cmdsys.h>
#include <kyy.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <fileheaders.h>
#include <ddgs.h>
#include <sai_ext.h>

static struct object *histo;
static struct object *rectobj=NULL;		/* Fstore object */
static int thresh = 20;					/* Current threshold */
static int ttog=0;						/* Autothreshold toggle */
static int shading = 0;					/* Shading correction on/off */
static int step=4;						/* Manual threshold increment */
static min_lab_siz = 2;					/* Min size object to label */
static int imageinvert = 0;				/* Inverted image in fstore (fluorescence) */
static int integtime = 20;				/* Integration time for image */
/*
 *	Fuse stuff
 */
#define FUSEMAX 3	/* Max number of fuse images */

extern struct object *fuseobj[FUSEMAX][MAXOBJS];	/* Fuse object lists */
extern struct pframe fuseframe[FUSEMAX+1];			/* Fuse frames */
extern int fusecount[FUSEMAX];						/* Number of objects in fuseobj lists */
extern int fuseframes;								/* Number of frames grabbed for fusing */

/*
 *	External things to import
 */
extern FILE *debug_channel;
extern struct kcontrol *kcont;
extern struct ipoint cursor;

/*
 *	Internal function forward declarations
 */
static int filecell();					/* Sort out filing */
static int getcell();
static int getsecond();
static int dohisto();
static shademenu();
static int switchfluor(tog);
static int pulnix(), pgrab();
static int satdigitise();

/*
 * External routines to import
 */
extern struct object *fsframehisto();
extern struct object *fsconstruct();
extern struct object *fsthreshold();
extern struct object *fshisto();
extern struct pframe *makepframe();
extern struct object *makerect();
extern fsmemerror();
extern int woolzexitfn();
extern int utilities();

extern int fieldfuse();
extern read_shade(), write_shade();
extern apply_shade();					/* Do shading correction */
extern calc_shade();					/* Calc shading correction */
extern display_shade();					/* Display shading correction */

/*
 *	S A T D I G I T I S E 
 *
 */
 satdigitise()
 {
 	static int firstime = 1, fluor;
 	int Ex_cmd, Fi_cmd;

	if (firstime) {
		imageinvert = alc->ddgschannel->invert;
		firstime = 0;
	}
	fluor = (imageinvert != alc->ddgschannel->invert);
	kcont->FIP = DIGTV;	/* This is where the image came from */
	initialisefg();
	initialisewoolz();
	read_shade();
	fgdigitise();

	newmenu ("Image Capture");
		cmmd("capture ;%-+d%f",&fluor,getcell); mouse(MIDBT);	/* 1 */
		Fi_cmd = mmcmd;
		if (alc->options & OPT_FUSION) {
			cmmd("capture stray chromosomes ;%-+d%f",&fluor,getsecond); /* 2 */
				mouse(LONGMB);
			cmmd("fuse field ;%f",fieldfuse);mouse(RIGHTBT);	/* 3 */
			cmmd("file cell ;%f",filecell);mouse(LEFTBT); Fi_cmd = mmcmd; /* 4 */
		}
		cmmd("show histogram ;%f",dohisto);mouse(LONGRB);	/* 5 */
		cmmd("automatic threshold;:k40%T",&ttog);		/* 6 */
		cmmd("shading correction;:k40%-T%f",&shading,shademenu);	/* 7 */
/*		cmmd("manual threshold utilities;%x%f","Manual Threshold Utilites",threshops); */
		if (alc->options & OPT_FLUOR)
			cmmd("fluorescence;:k40%-T%f",&fluor,switchfluor);	/* 8 */
		if (alc->devel) {
			cmmd("camera type;%d",&alc->cameratype);	/* 9 */
		}
		if ((alc->options & OPT_FLUOR) && (alc->cameratype & PULNIX)) {
			cmmd("exposure time;:k40%d",&integtime);	/* 10 */
		}
/*		if (alc->devel)
			cmmd ("house keeping;%g",utilities); */
		cmmd ("exit"); mouse(LONGLB); Ex_cmd = mmcmd;

	while(1) {
/*
 *	Drop through if exit selected.
 */
		do {
			thismu();			/* Each function will enter shared resource */

			if (alc->debug) {
				Show_alloc();
				Show_free();
				fflush(debug_channel);
			}

		} while (nncmd != Ex_cmd && nncmd != Fi_cmd);

		if (fuseframes || alc->cellin) {
			if (check("Digitised image not stored, proceed (M):"))
			{
				oclear(0,0,alc->ddgschannel->lineln-1,alc->ddgschannel->lineno-1,0);
				break;
			}
		}
		else
		{
			oclear(0,0,alc->ddgschannel->lineln-1,alc->ddgschannel->lineno-1,0);
			break;
		}
	}
/*
 *	if theres a list of objects not yet freed then get rid of them
 */
	if (alc->cellin) {
		release(kcont->ool,&alc->lkount);
		alc->lbelld = 0;
		alc->cellin=0;
		kcont->maxnumber = 0;
		freefuse();
	}

	restorewoolz();

}

/*
 *	P G R A B  --  Grab pulnix image
 *
 */
static int pgrab(integtime)
int		integtime;
{
	int		priority,done;
	priority = getpr(getpid());
	setpr(getpid(),200);
	printf("Please focus, proceed (M):");
	while ((done = gone_on()) < 0) {
		grabonefield();
		tsleep(integtime);
	}
	setpr(getpid(),priority);
	return(done);
}

/*
 *	S W I T C H F L U O R  --
 *
 */
static int switchfluor(tog)
int		*tog;
{
	imageinvert = imageinvert ? 0 : 1;
	*tog = (imageinvert != alc->ddgschannel->invert);
/*	printf("invert %d, imageinv %d\n",alc->ddgschannel->invert,imageinvert); */
}

/*
 *	M A R K T H R E S H
 *
 */
static 
markthresh(thres,scale)
int thres,scale;
{
	lwidth(5);
	moveto(thres*scale + 512,512);
	lineby(0,256);
	lwidth(1);
}

/*
 *	S E T F R A M E 
 *
 */
setframe(f,scale)
struct pframe *f;
int scale;
{
	f->type=60;
	f->scale=scale;
	f->ix=1;
	f->iy=1;
	f->ox=f->oy= 0;
	f->dx=f->dy= 0;
}


/*
 *	U P T H R E S H  --	Raise displayed threshold
 *
 */
upthresh()
{
	register int t;
	int negt;
	
	ttog = 0;
	t = thresh;
	thresh+=step;
	if (thresh > 255 ) thresh = 255 ;
	if (imageinvert)	{
		for ( ; t < thresh ; t++) {
			if (alc->ddgschannel->invert) {
				setrgb(t,t,t,t);
			} else {
				negt = 255 - t;
				setrgb(t,negt,negt,negt);
			}
		}
	} else {
		for ( ; t < thresh ; t++)
			setrgb(t,0,128,128);
	}
}

/*
 *	D O W N T H R E S H   --	Lower displayed threshold
 *
 */
downthresh()
{
	register int t;
	int negt;

	ttog = 0;
	t = thresh;
	thresh-=step;
	if (thresh < 0 ) thresh = 0;
	if (imageinvert) {
		for ( ; t >= thresh ; t--) {
			setrgb(t,0,128,128);
		}
	} else {
		for ( ; t >= thresh ; t--) {
			if (alc->ddgschannel->invert) {
				setrgb(t,t,t,t);
			} else {
				negt = 255 - t;
				setrgb(t,negt,negt,negt);
			}
		}
	}
}

/*
 *	S E T T H R E S H  --  
 *
 */
setthresh()
{
	register int i;
	
	ttog = 0; 			/* ensure automatic threshold is off */
	if (alc->ddgschannel->invert)
		nlut();
	else
		llut();
	if (imageinvert) {
		for (i=255;i>=thresh;i--) 
			setrgb(i,0,128,128);
	} else {
		for (i=0;i<=thresh;i++) 
			setrgb(i,0,128,128);
	}
}

/*
 *	T H R E S H O P S  --  
 *
 */
threshops(title)
char	*title;
{
	int		oldttog, stat;
	int		Xcpt_cmd, Quit_cmd;
	
	oldttog = ttog;
	setthresh();		/* highlight current level upon entry */
	newmenu(title);
	cmmd("set threshold;:k30<1/255>%d%f",&thresh,setthresh);
	cmmd("step size;:k30<1/255>%d",&step);
	cmmd("decrease by step size;%g",downthresh);mouse(LEFTBT);
	cmmd("increase by step size;%g",upthresh);mouse(RIGHTBT);
	cmmd("accept");mouse(MIDBT);Xcpt_cmd = mmcmd;
	Quit_cmd = cmmd("quit");mouse(LONGLB);
	while (thismu()) {
		if (nncmd > 1) {
			displin(0,1);
			putprompt();
		}
		if (nncmd == Quit_cmd) {
			ttog = oldttog;		/* Quit - leave manthr as it was */
			stat = 0;
			break;
		} else if (nncmd == Xcpt_cmd) {
			stat = thresh;
			break;
		}
	}
	if (alc->ddgschannel->invert)
		nlut();
	else
		llut();
	return(stat);
}

#ifdef HISTONLY
/*
 *	H I S T O N L Y 
 *
 */
 static histonly() {
	struct pframe f;

	fgfreeze();

	printf("Scanning for histogram, please wait ..\n");
	setframe(&f,16);
	f.dx=f.dy= 512;
	if (alc->ddgschannel->type==FS_VME512) {
		histo = fshisto(rectobj,255);
	} else {
		histo = fsframehisto(alc->ddgschannel->frame,
			alc->ddgschannel->lineln,alc->ddgschannel->lineno,255);
	}
	if (alc->debug) printf ("After getting histogram\n");

	histothreshsmooth(histo);
		
	/* only automatically calculate threshold if auto thresh on */
	if (ttog)
		thresh = compthr3(histo);
	
	if (alc->ddgschannel->type==FS_VME512) {
		histodup(histo);
	}
	histonormalise(histo);
	picframe(histo,&f);
	markthresh(thresh,16);
	freeobj(histo);
	go_on();		/* wait for histo to be looked at before going live again */
	fgdigitise();
}
#endif HISTONLY

/*
 *	F I L E P I C
 *
 */
static 
filepic() 
{
	int		i;

	while (1) {
		if (!getid(CELLFILE,WRITE)) return(0);
		cellheadfill(alc->cell_h,&(alc->nameids));
		alc->cell_h->dig_type = DIGTV ;
		alc->cell_h->x_coord = 0 ;
		alc->cell_h->y_coord = 0 ;
		alc->cell_h->z_coord = 0 ;
		alc->cell_h->x_size = 0 ;
		alc->cell_h->y_size = 0 ;
		windmill("Writing cell - ");
		if (alc->debug)
			fprintf(debug_channel,"Filing cell to %s\n",alc->picname);
		if (putfile(alc->picname,alc->fipdig,kcont->ool,alc->lkount))
			break;
	}
	return(1);
}

/*
 *	G E T C E L L
 *
 */
static int getcell(fluor)
int		*fluor;
{
	if (fuseframes) {
		if (!check("Fusion images will be lost, proceed (M):"))
			return(0);
	}
	if (alc->cellin) {
		if (!check("Digitised image not stored, proceed (M):"))
			return(0);
		release(kcont->ool,&alc->lkount);
		alc->lbelld = 0;
		alc->cellin=0;
		kcont->maxnumber = 0;
		freefuse();
	}
	
	if (!digitise(*fluor)) return(0);
	if (alc->fipdig == NULL)
		fprintf(stderr,"Digitisation error, no cell generated\n");
	else {
		/*
		 * now we can use the conventional routines.
		 */
		printf("Normalising\n");
		normalise(alc->fipdig);
		if (alc->debug) {
			fprintf (debug_channel,"Done normalise\n");
			elapsedtime();	
		}
		printf("Extracting chromosomes\n"); 
		label(alc->fipdig,&alc->lkount,kcont->ool,MAXOBJS,3);
		alc->cellin = 1;
		alc->lbelld = 1;
		if (alc->debug)
			elapsedtime();
		freeobj(alc->fipdig);
		alc->fipdig = NULL;
	}
	if (!(alc->options & OPT_FUSION))
		filecell();
	return(1);
}

/*
 *	G E T S E C O N D  --  Grab second and subsequent fields for field fusion
 *
 */
static int getsecond(fluor)
int		*fluor;
{
	if (!alc->cellin) {
		printf("No main cell image digitised yet\n");
		return(0);
	}
	if (fuseframes >= FUSEMAX) {
		printf("Can't hold any more images to fuse\n");
		return(0);
	}
	if (!digitise(*fluor)) return(0);
	if (alc->fipdig == NULL)
		fprintf(stderr,"Digitisation error, no cell generated\n");
	else {
		/*
		 * now we can use the conventional routines.
		 */
		printf("Normalising\n");
		normalise(alc->fipdig);
		if (alc->debug) {
			fprintf (debug_channel,"Done normalise\n");
			elapsedtime();	
		}
		printf("Extracting chromosomes\n"); 
		label(alc->fipdig,&fusecount[fuseframes],fuseobj[fuseframes],MAXOBJS,3);
		fuseframes++;
		if (alc->debug)
			elapsedtime();
		freeobj(alc->fipdig);
		alc->fipdig = NULL;
	}
	return(1);
}


/*
 *	D I G I T I S E   --  Digitise field and generate woolz object
 *							in alc->fipdig
 *
 */
static int digitise(fluor)
int		fluor;
{
	int		stat, i, j, background;

	if (alc->ddgschannel->invert == 1)
		nlut();
	else
		llut();
	if (fluor && (alc->cameratype & PULNIX) && integtime) {
		if (!pgrab(integtime))
			return(0);
		else
			lineaverage();
	} else {
		fgdigitise();
		if (!check("Please focus, proceed (M):"))
			return(0);
		fgfreeze();
	}
	if (shading) {
		printf("Applying shading correction\n");
		apply_shade(alc->ddgschannel->frame,imageinvert);
		if ((alc->cameratype & MASK) && !fluor)
			mask_image(MASKTOP+EDGE,MASKBOT+EDGE,MASKLEFT+EDGE,MASKRIGHT+EDGE,imageinvert ? 255 : 0);
		else
			mask_image(EDGE,EDGE,EDGE,EDGE,imageinvert ? 255 : 0);
	} else {
		if ((alc->cameratype & MASK) && !fluor)
			mask_image(MASKTOP,MASKBOT,MASKLEFT,MASKRIGHT,imageinvert ? 255 : 0);
	}
	if (ttog) {
		printf("Computing thresholds\n");
		if (alc->debug) {
			fprintf (debug_channel,"About to threshold\n");
			fflush(debug_channel);
		}
		switch (alc->ddgschannel->type) {
			case FS_VME512:
				histo = fshisto(rectobj,255);
				break;
			case FS_VIRTUOSO:
			case FS_VIRTUOSO_INVERT:
				histo = fsframehisto(alc->ddgschannel->frame,
					alc->ddgschannel->lineln,alc->ddgschannel->lineno,255);
				break;
		}
		if (alc->debug) {
			fprintf (debug_channel,"About to compute threshold\n");
			fflush(debug_channel);
		}
		thresh = compthr3(histo);
		freeobj(histo);
	}
	downmenu();
	stat = threshops("Check Threshold");
	upmenu();
	if (!stat)
		return(0);

	switch (alc->ddgschannel->type) {
		/*
		 * special routine fsthreshold on account of top-byte FF in VME-512
		 */
		case FS_VME512:

			alc->fipdig = fsthreshold(rectobj,thresh);
			if (alc->debug) fprintf (debug_channel, "Done threshold thres:%d area:%d\n",thresh,area(alc->fipdig));
		/*
		 * make a new type-21 grey-table not in frame store,
		 * and mask out the top byte FF once and for all.
		 */
			alc->fipdig = (struct object *) fsnewgrey(alc->fipdig);
			if (alc->debug) fprintf (debug_channel,"Done newgrey\n");
			break;

		/*
		 * special routine fsthreshold on account of bytes in virtuoso
		 */
		case FS_VIRTUOSO:
		case FS_VIRTUOSO_INVERT:
			if (alc->debug) {
				fprintf (debug_channel,"About to fsconstruct with threshold of %d\n",thresh);
				fflush(debug_channel);
			}
			printf("Removing background\n");
		
		/*
		 * empirically it has been show that as well as the expected half
		 * lines in lines 0 and max, which fsconstruct itself filters out, the
		 * bottom line of those remaining has an artificial near black interval
		 * spanning half the line. this too is removed therefore. bdp 7/8/87 
		 */
		
			Alloc_exit(fsmemerror);
			Alloc_wait(0);		/* dont wait if out of memory */
			if (alc->debug)
				starttimer();
			alc->fipdig = fsconstruct(alc->ddgschannel->frame,
				alc->ddgschannel->lineno-1,alc->ddgschannel->lineln,thresh,imageinvert);
			if (alc->debug) {
				fprintf (debug_channel,"Done threshold thres:%d area:%d\n",thresh,area(alc->fipdig));
				fprintf (debug_channel,"Done fsconstruct\n");
				elapsedtime();
			}
			Alloc_exit(woolzexitfn);
			Alloc_wait(1);
			break;
	}
	return(1);
}

/*
 *	M A S K _ I M A G E  --  Mask off bad bits round edge of image
 *
 */
mask_image(top,bot,left,right,background)
int		top,bot,left,right,background;
{
	int		i,j;
	register unsigned char *pptr;
	switch (alc->ddgschannel->type) {
		case FS_VME512:
			break;
		case FS_VIRTUOSO:
		case FS_VIRTUOSO_INVERT:
			pptr = alc->ddgschannel->frame;
			for (i=0; i<top; i++) {
				for (j=0; j<alc->ddgschannel->lineln; j++) {
					*pptr++ = background;
				}
			}
			for (i=top; i<alc->ddgschannel->lineno-bot; i++) {
				pptr = alc->ddgschannel->frame + (i*alc->ddgschannel->lineln);
				for (j=0; j<left; j++) {
					*pptr++ = background;
				}
				pptr = alc->ddgschannel->frame + ((i+1)*alc->ddgschannel->lineln);
				for (j=0; j<right; j++) {
					*--pptr = background;
				}
			}
			pptr = alc->ddgschannel->frame + (alc->ddgschannel->lineno-bot) * alc->ddgschannel->lineln;
			for (i=0; i<bot; i++) {
				for (j=0; j<alc->ddgschannel->lineln; j++) {
					*pptr++ = background;
				}
			}
			break;
	}
}

/*
 *	I N I T I A L I S E W O O L Z
 *
 */
static
initialisewoolz()
{
	extern int packcheck;
	extern SMALL highlow;
	packcheck = 0;
	highlow = 1;
	if (alc->ddgschannel->type==FS_VME512){
		if (!rectobj)
			rectobj = makerect(0,511,0,511,alc->ddgschannel->frame
					,0,NULL,rectobj);
	}
}

/*
 *	R E S T O R E W O O L Z
 *
 */
static
restorewoolz()
{
	if (alc->ddgschannel->type==FS_VME512){
		if (rectobj) {
			freeobj(rectobj);
			rectobj=NULL;
		}
	}
}

/*
 *
 *	I N I T I A L I S E F G
 *
 */
static initialisefg()
{
	if (alc->ddgschannel->type==FS_VME512){
		if (alc->debug) fprintf(debug_channel,"VME512 framestore\n");
		fgsetup();
		setfglut(1);
		setfgsync(0);
	} else {
		if (alc->debug) fprintf(debug_channel,"Virtuoso framestore\n");
	}
	if (alc->ddgschannel->invert == 1)
		nlut();
	else
		llut();
		
	clear();
	if (alc->ddgschannel->type==FS_VME512)
		fast();
}

/*
 *	R E S T O R E D D G S  --
 *
 */
static restoreddgs()
{
	if (alc->ddgschannel->type==FS_VME512){
		slow();
	} else {
		if (alc->ddgschannel->invert == 1)
			nlut();
		else
			llut();
	}
}

/*
 *	R E S H A D E  --  Reset shading array
 *
 */
static reshade()
{
	unsigned char	*sh;
	int		i;
	switch(alc->ddgschannel->type) {
		case FS_VIRTUOSO_INVERT:
		case FS_VIRTUOSO:
			printf("Please move to a blank field\n");
			fgdigitise();
			if (!yesno(1)) {
				fgfreeze();
				printf("calculating new shading values\n");
				calc_shade(alc->ddgschannel->frame,imageinvert);
			} else {
				fgfreeze();
			}
			shading = 1;
			break;
		default:
			printf("Sorry - reshade does not function with this hardware\n");
	}
}

/*
 *	D I S P S H A D E  --
 *
 */
static dispshade()
{
	if (display_shade(alc->ddgschannel->frame,imageinvert)) {
		threshops("Shading display");
	}
	shading = 1;
}

/*
 *	D O H I S T O 
 *
 */
static int dohisto()
{
	fgdigitise();
	disp_histo(alc->ddgschannel->frame);
	setdisplay(BUST);
}
	
/*
 *	S H A D E M E N U  --  Shading options menu
 *
 */
static shademenu()
{
	int	othresh, ottog;
	fgfreeze();
	othresh = thresh; ottog = ttog;
	newmenu("Shading Correction");
	cmmd("get new shading values;%f",reshade);
	cmmd("save shading correction;%f",write_shade);
/*	cmmd("show histogram;%f",dohisto);mouse(LONGLB); */
	cmmd("display current shading;%f",dispshade);
	cmmd("shading correction;:k30%T",&shading);mouse(RIGHTBT);
	while (thismu())
		;
	ttog = ottog; thresh = othresh;
	fgdigitise();
}


/*
 *	F I L E C E L L  --  File all what we got, call fieldfuse 1st if
 *							any bits still to be merged
 *
 */
static int filecell()
{
	if (alc->cellin) {
		if (fuseframes) {
			fieldfuse();
		}
		addplist(kcont);				/* Add plists etc */
		filepic();
/*
 *	Tidy up
 */
		release(kcont->ool,&alc->lkount);
		alc->lbelld = 0;
		alc->cellin = 0;
		kcont->maxnumber = 0;
		freefuse();
	} else {
		fprintf(stderr, "There is no current cell to be filed!\n");
		putprompt();
	}
}

/*
 *	H I S T O D U P
 *
 *		Input histogram
 *		Output histogram
 *		Function duplicate even histogram entries in odd bins
 *		Usage CRS VME 512 only has 7 bit look up on input this gives
 *				strange histogram unless smoothed by this technique
 *
 */
static histodup(obj)
struct object *obj;
{
	register struct histogramdomain *hdom;
	register int i, *h;
	hdom = (struct histogramdomain *) obj->idom;
	h = hdom->hv;
	for (i=0; i<hdom->npoints; i+=2) {
		h[1] = h[0] = h[0]+h[1];
		h+=2;
	}
}
