/*
 *			S A T F O R K . C --	Continuously fork a direct satellite
 * 									capture application
 *
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *  Modifications
 *
 */
# include <stdio.h>
# include <modes.h>
# include <procid.h>

# define TRUE (1)
# define FALSE (0)
# define MAXAPPS (2)
# define FAIL (-2)

extern int errno;
extern int os9forkc();

char satforkid[] = "satfork V0.1";
int dbug=0;
int quit = FALSE;	/* User controlled quit */
int interrupt();

struct app_prog {
	char prog[40];
	char arg[20];
	char _stdin[10];
	char _stdout[10];
	char _stderr[10];
	int pid;
}

/*
 * Main code starts here
 */
main(argc, argv, argp)
int argc;
char **argv;
char **argp;
{
	int opt, stat, c;
	char *s, *buf;
	int num, bp;
	int i, j, id, satnum=-1;
	struct app_prog ap[MAXAPPS];
/*
 *	First set defaults
 */
 	for (i=0; i < MAXAPPS; i++) {
		ap[i].pid = 0;
		ap[i].prog[0] = '\0';
		ap[i].arg[0] = '\0';
		strcpy(ap[i]._stdin, "/nil");
		strcpy(ap[i]._stdout, "/nil");
		strcpy(ap[i]._stderr, "/nil");
	}
/*
 * Now process for options
 */
	argv++;	/* skip program name */
	argc--; /* One less */

	while (argc>0 && (*argv)[0] == '-') {
		opt = 1;	/* where to find an option */
		for (s=argv[0]+1; *s!='\0'; s++)
			switch (*s) {
			case '?': options();
			case 'v':
				printf("%s\n", satforkid);
			  	exit(0);
			case 'g': dbug++;
					break;
			case 't': /* Get number */
					stat = sscanf(*(argv+opt), "%d", &num);
					if (stat != 1)
						exit(_errmsg(1, "Invalid format for number"));
					opt++;	/* Next option is one further on */
					break;
			case 'p': /* Get slave program name */
					if (satnum++ >= MAXAPPS) {
						exit(_errmsg(1, "Too many applications specified"));
					}
					sscanf(*(argv+opt++), "%s", ap[satnum].prog);
					if (dbug) {
						printf("Prog: \"%s\"\n", ap[satnum].prog);
					}
					break;
			case 'a': /* Slave program argument */
					sscanf(*(argv+opt++), "%s", ap[satnum].arg);
					break;
			case 'i': /* Get stdin path */
					sscanf(*(argv+opt++), "%s", ap[satnum]._stdin);
					break;
			case 'o': /* Get stdout path */
					sscanf(*(argv+opt++), "%s", ap[satnum]._stdout);
					break;
			case 'e': /* Get stderr path */
					sscanf(*(argv+opt++), "%s", ap[satnum]._stderr);
					break;
			default: fprintf(stderr, "Unknown switch '%c'\n", *s);
				exit(-1);
			}
		argc -= opt;  argv += opt; opt = 1;
	}

	if (argc == 0) ; /*No further params */
	intercept(interrupt); /* Set up signal catcher */

	if (dbug) printf("Starting\n");

	id = getpid();
	if (dbug) printf("This is process %d\n", id);
/*
 *	Check we have a program specified
 */
 	if (satnum == -1) {
		exit(_errmsg(1, "No slave program specified"));
 	}

	while (!quit) {
		for (i=0; i <= satnum; i++) {
			if (ap[i].pid == 0) {
				if (dbug) printf("Forking %s\n", ap[i].prog);
				if ((ap[i].pid = satfork(&ap[i], argp)) == FAIL)
					exit(1); /* Or perhaps we could be more sophisticated !! */
			}
		}
/*
 *	Now wait until one dies
 */
		if (wait(0) < 0)
			exit(_errmsg(1, "No slave program to wait for"));
/*
 *	Which one died ?
 */
		for (j=0; j <= satnum; j++)
			ap[j].pid = checkalive(ap[j].pid, id);

	} /* and round again */
} /* end main */

/*
 *	S A T F O R K -- Fork the satellite capture process
 */
int
satfork(module, argp)
struct app_prog *module;
char **argp;
{
	int _in, _out, _err, pid, reass_err;
	char *arg[3];

	if (dbug) {
		printf("Satfork: Forking %s\n", module->prog);
		printf("Arg: %s\nStdin: %s\nStdout: %s\nStderr: %s\n",
			module->arg, module->_stdin, module->_stdout, module->_stderr);
	}
	if ((_in = dup(0)) < 0 ||
		(_out = dup(1)) < 0 ||
		(_err = dup(2)) < 0) {
			fprintf(stderr, "Satfork: Dup failed (%d)\n", errno);
			return(FAIL);
	}
	stderr->_fd = _err;
/*
 *	Close down i/o paths
 */
	if ((close(0) < 0) || (close(1) < 0) || (close(2) < 0)) {
		fprintf(stderr, "Satfork: i/o close failed (%d)\n", errno);
		return(FAIL);
	}
/*
 *	Now open child paths
 */
	if ((open(module->_stdin, S_IREAD | S_IWRITE) != 0) ||
		(open(module->_stdout, S_IREAD | S_IWRITE) != 1) ||
		(open(module->_stderr, S_IREAD | S_IWRITE) != 2)) {
			fprintf(stderr, "Satfork: i/o open failed (%d)\n", errno);
			return(FAIL);
	}

	arg[0] = module->prog;
	if (*(module->arg) == '\0')
		arg[1] = NULL;
	else
		arg[1] = module->arg;
	arg[2] = NULL;
	pid = os9exec(os9forkc, module->prog, arg, argp,0,0,3);

	reass_err = dup(_err);
	if ((close(2) < 0) || (dup(_err) != 2) || (close(_err) < 0)) {
		fprintf(stderr, "Satfork: Stderr reassignment failed"); /* ho hum */
		return(FAIL);
	}
	if ((close(0) < 0) || (dup(_in) != 0) || (close(_in) < 0)) {
		fprintf(stderr, "Satfork: Stdin reassignment failed");
		return(FAIL);
	}
	if ((close(1) < 0) || (dup(_out) != 1) || (close(_out) < 0)) {
		fprintf(stderr, "Satfork: Stdout reassignment failed");
		return(FAIL);
	}
	stderr->_fd = reass_err;

	if (pid < 0) {
		fprintf(stderr, "Satfork: Fork failed");
		return(FAIL);
	}

	tsleep(1);
	if (dbug) printf("Satfork: Child forked %d\n",pid);
	return(pid);
}	

/*
 *	C H E C K A L I V E -- Check the process with supplied id is still alive
 *							and kicking. If so return its pid else return 0
 */
checkalive(id, pid)
int id, pid;
{
	procid pbuf;

	if (id<0) return(id);
	if (dbug > 1) printf("Checking if process (id %d) is still alive\n", id);

	if (_get_process_desc(id, sizeof(procid), &pbuf) < 0) {
		if (errno == 224) return(0); /* Non existant */
		else {
			fprintf(stderr,"Checkalive: Can't obtain process descriptor (%d)\n",
				errno);
			return(-1);
		}
	}
	if (dbug > 1) {
		printf("Parent id is %d\n", pbuf._pid);
		printf("Status is %d\n", pbuf._state);
	}
	if (pid != pbuf._pid) {
		fprintf(stderr, "Process %d is not a child of satfork\n", id);
		return(0);
	}
	return(id);
}
/*
 * options() - Print options and exit
 */
options()
{
	printf("%s%s%s%s%s%s%s%s%s",
		"Syntax: satfork [<opts>] [<optinfo>] <other params>\n",
		"Function: Continuously fork satellite capture application\n",
		"Options:\n\t-?\t\tHelp info.\n",
		"\t-p\t\tSlave program name\n",
		"\t-a\t\tSlave program argument (only one allowed)\n",
		"\t-i\t\tSpecify stdin path\n",
		"\t-o\t\tSpecify stdout path\n",
		"\t-e\t\tSpecify stderr path\n",
		"\t-v\t\tVersion number\n\n");
	exit(0);
} /* end options() */

/*
 *	I N T E R R U P T --  Signal catcher
 *
 */
interrupt(sig)
int		sig;
{
	quit=TRUE;
}

