/*
 *			S A T I N P U T . C --	Test satellite input functions
 *
 *	Written: D. C. Burgess
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1989)
 *
 *  Modifications
 *
 */
# include <stdio.h>
# include <modes.h>
# include <procid.h>

# define TRUE (1)
# define FALSE (0)
# define INPUTSIZE (80)	/* Size of input buffer */
extern int errno;

char satinputid[] = "satinput V0.1";
int dbug=0;
int quit = FALSE;	/* User controlled quit */
int interrupt();

/*
 * Main code starts here
 */
main(argc, argv, argp)
int argc;
char **argv;
char **argp;
{
	int opt, stat, c;
	char *s, *buf;
	int num=1, bp;
	char inputline[INPUTSIZE];
/*
 * Process for options
 */
	argv++;	/* skip program name */
	argc--; /* One less */

	while (argc>0 && (*argv)[0] == '-') {
		opt = 1;	/* where to find an option */
		for (s=argv[0]+1; *s!='\0'; s++)
			switch (*s) {
			case '?': options();
			case 'v':
				printf("%s\n", satinputid);
			  	exit(0);
			case 'g': dbug++;
					break;
			case 't': /* Get time delay */
					stat = sscanf(*(argv+opt), "%d", &num);
					if (stat != 1)
						exit(_errmsg(1, "Invalid format for number"));
					opt++;	/* Next option is one further on */
					break;
			default: fprintf(stderr, "Unknown switch '%c'\n", *s);
				exit(-1);
			}
		argc -= opt;  argv += opt; opt = 1;
	}

	if (argc == 0) ; /*No further params */
	intercept(interrupt); /* Set up signal catcher */

	if (dbug) printf("Starting\n");
	while ((stat = readln(fileno(stdin), inputline, INPUTSIZE)) >= 0) {
		if (dbug) printf("Input (%d chars) was: %s\n", stat, inputline);
		if (strcmp(inputline, "quit") == 0) break;
		if(quit) break;
		tsleep(num);
	}
} /* end main */


/*
 * options() - Print options and exit
 */
options()
{
	printf("%s%s%s%s",
		"Syntax: satinput [<opts>] [<optinfo>] <other params>\n",
		"Function: Test satellite input\n",
		"Options:\n\t-?\t\tHelp info.\n",
		"\t-v\t\tVersion number\n\n");
	exit(0);
} /* end options() */

/*
 *	I N T E R R U P T --  Signal catcher
 *
 */
interrupt(sig)
int		sig;
{
	quit=TRUE;
}

