/*
 * controlsubs.c
 *
 * Functions to provide simple control of master-slave system.
 *
 * Modifications
 *
 *  28 Feb 1989		SEAJ		Made kcopyback take an extra parameter last, 
 *								loop now goes from first to last.
 *	 2 Mar 1988		CAS			Removed allcontr refs so this can
 *								be moved back to multip (where it came from)
 *	16 feb 1987		CAS			Changed mind - removed mastderived
 *	12 Feb 1987		CAS			kdeletemol - need to know when object is
 *								master derived (we own all) so added MASTDERIVED
 *	12 Jan 1987		CAS			More info from kcopyback if mpcopyobj fails
 *								and de-activate object
 *	16 Oct 1986		CAS			Make kcopyback - copy even if in same 
 *								physical processor
 *					CAS/GJP		Experiment with slave allocation
 *	 7 Oct 1986		CAS			Add quitslave
 *	 5 Oct 1986		CAS			Added a rather necessary call to freeobj
 *								in kdeletemol
 *  22 Sep 1986 	GJP			Remove some extranious calls to free
 *	19 Sep 1986		CAS/GJP		Added kdeletemol
 *	15 Sep 1986		CAS			Cast
 */
#include <stdio.h>
#include <multip.h>
#include <wstruct.h>
#include <chromanal.h>
#include <kcontrol.h>


/*
 *	F M E A S S T A R T
 *
 */
fmeasstart(kcont)
struct kcontrol *kcont;
{
	int slave;
	kcont->mstatus &= ~SEGEND;
	for (slave=0; slave<=NSLAVES; slave++) {
		if (kcont->processor[slave] != 0) {
			kcont->status[slave] &= ~(MEASBEGIN|MEASEND);
			commdslave(kcont,slave,CHROM_FEATURE_MEASURE);
		}
	}
}

/*
 *	W A I T F M E A S
 *
 */
waitfmeas(kcont)
struct kcontrol *kcont;
{
	int slave;
	for (slave=0; slave<=NSLAVES; slave++)
		if (kcont->processor[slave] != 0)
			waitslave(kcont,slave,MEASEND);
}

/*
 *	Q U I T S L A V E  --
 *
 */
quitslave(kcont,what)
struct kcontrol *kcont;
int		what;
{
	int slave;
	kcont->mstatus |= ABORT;
	for (slave=0; slave<=NSLAVES; slave++)
		if (kcont->processor[slave] != 0)
			waitslave(kcont,slave,what);
	kcont->mstatus &= ~ABORT;
	
}

/*
 *	K C O P Y B A C K
 *
 */
kcopyback(kc, first, last)
register struct kcontrol *kc;
int first, last;
{
	register SMALL *statpt;
	register int i, a, slavep;
	struct object *rol;
	
	statpt = kc->acl + first;
	for (i = first; i < last; i++) {
		a = *statpt;
		a &= ACTIVE;
		if (a == ACTIVE) {
			/* slave processor number */
			slavep = (*statpt & MPROC) >> 6;
			if ((rol = (struct object *) convaddr(kc->rol[i],slavep,MASTER)) != NULL) {
				kc->mol[i] = (struct chromosome *) mpcopyobj(rol,slavep,MASTER,0);
				*statpt |= MASTCOPY;
			} else  {
				kc->mol[i] = kc->ool[i];
				fprintf(stderr,"Warning - object number %d not measured\n",i);
			}
		}
		statpt++;
	}
}

/*
 *	K D E L E T E M O L  --  Delete the master copl object as and
 *							where required - but don't delete anything
 *							that has already been killed ....
 *							(Well ... thats the theory.....)
 *
 */
kdeletemol(kc)
register struct kcontrol *kc;
{
	register SMALL *statpt;
	register int i, slavep;

	statpt = kc->acl;
	for (i=0; i<kc->maxnumber; i++) {
		slavep = (*statpt & MPROC) >> 6;
		if (*statpt & MASTCOPY) {
			kc->mol[i] = (struct chromosome *) mpdeleteobj(kc->mol[i],0);
			if (kc->mol[i])
				freeobj(kc->mol[i]);
		}
		kc->mol[i] = NULL;
		statpt++;
	}
}

