/*
 *  S L _ F M E A S . C -- slave service function to compute features of 
 *						   chromosome objects (was kmeas.c)
 *
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *  Taken from: Jim Piper et al.
 *				Medical Research Council
 *				Clinical and Population Cytogenetics Unit
 *				Edinburgh
 *
 *  Date:    17th April 1985
 *
 *  Modifications
 *
 *   1 Mar 1989		SEAJ	Added activity bit PROCESSED.
 *	 6 Dec 1988		CAS		Remove messages into DEBUG defines
 *	 2 Nov 1987		SCG/CAS	Fixed stupid bug in wormcast mod... PRATT
 *	22 Oct 1987		CAS		Only wormcast in slave 1.. - Not ideal but to in
 *							practice will be OK
 *							Changed SEGEND processing to ensure no stragler
 *							objects left unmeasured
 * 24-09-87			jp/bdp	Get the numbers at bottom right for ALL displays.
 * 22-09-87			jimp	Modification to 18-09-87 stuff: don't free after
 *							MASTCOPY bit set, since MPROC no longer guaranteed
 *							(there could otherwise be problems in karyotype
 *							re-segmentation).  MASTCOPY is permanently set
 *							during karyotyping except when karyotype server
 *							functions are happening.
 * 18-09-87	jimp		Detect previously measured objects owned by this
 *			processor (NB should be OWNED BY THIS PROCESS, but that
 *			information not available in acl[] - does not matter
 *			so long as only one slave per processor).  Free the
 *			unwanted object and fix up acl[] appropriately.
 *			To make this more secure, always copy the input object,
 *			even if running in the master processor.
 * jimp	05-08-87	Leave the master to do the attention-drawing.
 *	29 Jan 1987		CAS			Cockup with kcont->FIP
 * JP	15-12-86	Wormcast tag changed to reflect module name change.
 * JP	14-11-86	Even if an operator disregards the "probotype" typing
 *			and calls a large object a chromosome (possibly owing
 *			to a finger fault), if it is too small or too big we
 *			only do the "basic" feature measurements.
 * JP	13-11-86	Reconstructed from divergent MRC and IRS versions.
 * JP	06-11-86	Name changed.
 * JP	31-10-86	If object type believed not CHROMOSOME, draw operator's
 *			further attention to this object (and don't measure).
 *			This replaces the previous test based on "maxdiam()".
 * JP	17-10-86	First guess at object type
 * JP	17-10-86	INTERACTING flag checked before running measurement
 *	 7 Oct 1986		CAS			Add ABORT option
 *  Get the size of remaining memory and shut up shop 
 * if insufficient left.    19/5/86 GJP
 */

#include <stdio.h>
#include <multip.h>
#include <wstruct.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <display.h>
#include <fileheaders.h>

#define BASIC 1
#define ENOUGH_MEM 50000
#define MINDIAM 10
#define MINDENSITY 10
#define MAXAREA 5000

char astring[80];

#ifdef DEBUG
char dstring[50];
#endif DEBUG

extern double xscale,yscale;

kmeas (slave, kcont)
struct kcontrol *kcont;
{
	struct object *obj, *robj;
	struct object *chromfeatures(), *mpcopyobj();
	struct object **ool, **rol;
	struct chromplist *plist;
	struct ipoint blob, **bol, *mblob;
	struct pframe *f;
	int i, ac, slavep, SEGBEG, slavey, ddgstype, num_xpos, segend = 0;
	SMALL *acl, *statpt;
#ifdef DEBUG
	int j;
#endif DEBUG

	/* slave process number must be converted to true processor number */
	slavep = kcont->processor[slave] >> P_OR;
	tsize(50,50);
	slavey = 32 + 80 * slave;
/*	slavey = 3800 + 80 * slave; GJP 13/5/86  */
#ifdef DEBUG
	sprintf(dstring,"slave %d pr_or %d pr_ss %d object",slave,slavep,
		kcont->processor[slave] & P_SS);
	dispstring(dstring,2400,slavey);
#endif DEBUG
	kcont->status[slave] |= MEASBEGIN;
	ool = (struct object **)convaddr(kcont->ool,MASTER,slavep);
	acl = (SMALL *)convaddr(kcont->acl,MASTER,slavep);
	rol = (struct object **)convaddr(kcont->rol,MASTER,slavep);
	bol = (struct ipoint **)convaddr(kcont->bol,MASTER,slavep);
	f = (struct pframe *)convaddr(kcont->f,MASTER,slavep);
	ddgstype = ((DDGS *)convaddr(kcont->ddgschannel,MASTER,slavep))->type;
	if (ddgstype == FS_VIRTUOSO || ddgstype == FS_VIRTUOSO_INVERT)
		num_xpos = 5940;
	else
		num_xpos = 3960;
	switch (kcont->FIP) {
		default:
		case DIGTV:
			xscale = yscale = 1.0;
			break;
		case DIGFIP:
			xscale = 1.0;
			yscale = 0.75;
			break;
		case DIGFIPEXP:
			xscale = 1.33;
			yscale = 1.0;
			break;
	}


	/* set up blob display */
	blob.type = 40;
	blob.style = 4;	/* fixed size hollow square */

	/*
	 * wait until at least one object present
	 */
	while (kcont->maxnumber <= 0)
		tsleep(5);

	/*
	 * Measure features and display in bottom corner of screen.
	 * Loop in range 1 - maxnumber until all active objects processed
	 * and segmentation marked as finished in status word, so as to catch
	 * any composites etc. subsequently re-designated as chromosomes.
	 *
	 * Don't measure features on tiny objects.  These will be noise,
	 * pieces or fragments and will either be junked, joined or
	 * treated separately in some fashion.  If left at end of segmentation
	 * then at least they have a property list (even if empty)
	 *
	 * Check that we have more than enough free memory to do the job
	 */
	i = 0;
/*   
 * print out the memory at the moment
 */
#ifdef DEBUG
 fprintf(stderr,"Memory at start %d\n",free_left());
#endif DEBUG
	while (i < kcont->maxnumber && free_left()>ENOUGH_MEM) {
#ifdef DEBUG
		intens(0);
		dispstring(dstring,600,slavey);
		sprintf(dstring,"free %d mstatus %04o maxn %d status %04o",free_left(),
			kcont->mstatus,kcont->maxnumber,kcont->status[slave]);
		intens(1);
		dispstring(dstring,600,slavey);
		if(slavep==MASTER){
			intens(0);
			dispstring(astring,600,3700);
			for (j=0;j<kcont->maxnumber;j++){
				if(acl[j]&ACTIVE==ACTIVE)
					astring[j]='*';
				else
					astring[j]=' ';
			}
			intens(1);
			dispstring(astring,600,3700);
		};
#endif DEBUG
/**********************************/
		lock7(&(kcont->lock));
		statpt = acl + i;
		if ((*statpt & (ACTIVE|MSTART|INTERACTING)) != ACTIVE) {
			unlock7(&(kcont->lock));
		} else {
			*statpt |= MSTART;
			unlock7(&(kcont->lock));
/**********************************/
			if (slavep == 1) wormcast(kcont,slavep,"sl_fmeas");
			colour(OVERLAY1);
			intens(1);
			dispnum(i+1,num_xpos,slavey);
			obj = mpcopyobj(convaddr(ool[i],MASTER,slavep),
					MASTER,slavep,0);
			/*
			 * don't display if segmentation not yet started,
			 * e.g. while objects still being read from disc
			 */
			SEGBEG = kcont->mstatus&SEGBEGIN;
			if (SEGBEG != 0) {
				mblob = (struct ipoint *)convaddr(bol[i],MASTER,slavep);
				blob.k = mblob->k;
				blob.l = mblob->l;
				picframe(&blob,f);
			}
			/* type already known not chrom ?? */
			if (((struct chromosome *)obj)->plist->otype > 1)	
				robj = obj;
			else if (probotype(obj) > 1) {	/* believed not chromosome */
				robj = obj;
			}
			else {							/* believed to be chromosome */
				/*
				 * small object test - since operator may have forced
				 * chromosome classification on us.
				 */
				plist = (struct chromplist *) obj->plist;
				if (plist->area == 0)
					plist->area = area(obj);
				if (plist->mass == 0)
					plist->mass = mass(obj);
				if (maxdiam(obj) < MINDIAM ||
					plist->mass/plist->area < MINDENSITY ||
					plist->area > MAXAREA) {
						robj = obj;
						basicfeatures(robj);
				} else
					robj = chromfeatures(obj,BASIC);
			}
			colour(OVERLAY1);
			intens(0);
			dispnum(i+1,num_xpos,slavey);
			if (SEGBEG != 0) {
				picframe(&blob,f);
				intens(1);
			}
			/*
			 * Can delete "obj" now.
			 * First set obj->plist to NULL since "obj" and "robj"
			 * share the property list.  Also test that it really is a
			 * new object (on occasion we may not have rotated original)
			 */
			if (obj != robj) {
				obj->plist = NULL;
				freeobj(obj);
			}
			/*
			 * First check to see if any previously measured
			 * objects can be abandoned.
			 */
			check_free(kcont,slavep);
/**********************************/
			lock7(&(kcont->lock));
			rol[i] = robj;
			*statpt |= (MEND | (slavep << 6) | PROCESSED);
			unlock7(&(kcont->lock));
/**********************************/
		}
		if (kcont->mstatus & ABORT) {
			break;
		}
		ac = kcont->mstatus & SEGEND;
		i++;
		if (i >= kcont->maxnumber && ac == 0) {
			/*
			 * Check to see if any previously measured
			 * objects can be abandoned.
			 */
			check_free(kcont,slavep);
			tsleep(2);
			i = 0;
		} else if (ac && !segend) {
			segend = 1;
			i = 0;					/* Force one last pass through all objects */
		}
	}
	kcont->status[slave] |= MEASEND;
#ifdef DEBUG
	if (free_left()<ENOUGH_MEM)
		fprintf(stderr,"Kmeas-insufficient memory\n");
#endif DEBUG
}

static int free_left()
{
	return(getsys(0x72,4));
}

check_free(kcont,slavep)
register struct kcontrol *kcont;
{
	register SMALL *acl, *statpt;
	register struct object **rol;
	register int i;

	acl = (SMALL *)convaddr(kcont->acl,MASTER,slavep);
	rol = (struct object **)convaddr(kcont->rol,MASTER,slavep);
	for (i=0; i<kcont->maxnumber; i++) {
		statpt = acl + i;
		lock7(&(kcont->lock));
		if ((*statpt & (ACTIVE|MSTART|MEND|INTERACTING|MASTCOPY)) ==
		    (MSTART|MEND) && ((*statpt&MPROC) >> 6) == slavep) {
			unlock7(&(kcont->lock));
			rol[i]->plist = NULL;
			freeobj(rol[i]);
			lock7(&(kcont->lock));
			rol[i] = NULL;
			*statpt &= ~(MSTART|MEND|MPROC);
#ifdef DEBUG
			fprintf(stderr,"Slave freeing rotated object number %d (mem %d)\n",i+1,free_left());
#endif DEBUG
		}
		unlock7(&(kcont->lock));
	}
}
