/*
 * 		S L _ K _ S E R V . C -- slave service functions for karyotyping
 *
 *
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *  Written By :Jimm Piper et al.
 *				Medical Research Council
 *				Clinical and Population Cytogenetics Unit
 *				Edinburgh
 *
 *  Date:    17th March 1987
 *
 *  Modifications
 *
 *   1 Mar 1989		SEAJ	Added extra activity bit PROCESSED.
 *	 6 Dec 1988		CAS		Remove messages into DEBUG defines
 *	24-09-87		jp/bdp	Get the numbers at bottom right for ALL displays.
 *	02-09-87		jimp	Space problem from normalise/enhance caused
 *							by using the master's object in MASTER slave,
 *							then doing a newgrey() in slave, then freeing
 *							intermediate enhanced object in master, leaving
 *							an orphaned domain structure.  Get round this
 *							by ALWAYS copying input object.
 *	01-09-87		jimp	Debugging space problems...  
 *	31-03-87		jimp	Mods to 30-03-87 stuff...
 *	30-03-87		jimp	Rotation added.  Uses kcont->auxstat for angle.
 *
 */


#include <stdio.h>
#include <multip.h>
#include <wstruct.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <display.h>

#define RADIANS 	.01745
#define ENOUGH_MEM 50000

#ifdef DEBUG
char dstring[50];
#endif DEBUG

struct class_order {
	unsigned char class;
	unsigned char serial;
};

sl_k_serv(slave, kcont, com)
register struct kcontrol *kcont;
{
	struct object *obj, *nobj, *poly, *spin(), *squeeze(), *straighten();
	struct object *mpcopyobj(), *enhance(), *autostraight(), *newgrey();
	struct object **mol, **rol;
	struct chromplist *plist;
	struct pframe *ivf, *f;
	register int i, notfree, slavep, slavey;
	register SMALL *acl, *statpt;
	struct class_order clo[MAXOBJS];
	int nactive,j,clorder(), ddgstype, num_xpos;
	double escale;

	/* slave process number must be converted to true processor number */
	slavep = kcont->processor[slave] >> P_OR;
	tsize(50,50);
	slavey = 32 + 80 * slave;
#ifdef DEBUG
	sprintf(dstring,"slave %d pr_or %d pr_ss %d object",slave,slavep,
		kcont->processor[slave] & P_SS);
	dispstring(dstring,2400,slavey);
#endif DEBUG
	kcont->status[slave] |= MEASBEGIN;
	mol = (struct object **)convaddr(kcont->mol,MASTER,slavep);
	acl = (SMALL *)convaddr(kcont->acl,MASTER,slavep);
	rol = (struct object **)convaddr(kcont->rol,MASTER,slavep);
	ivf = (struct pframe *)convaddr(kcont->ivf,MASTER,slavep);
	ddgstype = ((DDGS *)convaddr(kcont->ddgschannel,MASTER,slavep))->type;
	if (ddgstype == FS_VIRTUOSO || ddgstype == FS_VIRTUOSO_INVERT)
		num_xpos = 5940;
	else
		num_xpos = 3960;
	
	/*
	 * Fill in the class_order array with serial number and
	 * Paris class of the active, un-processed objects, sort it into
	 * class order, use to index the processing of the objects
	 */
	nactive = 0;
	for (i=0; i<kcont->maxnumber; i++) {
		statpt = acl + i;
		if ((*statpt & ACTIVE) && !(*statpt & FUNCBEGIN)) {
			clo[nactive].serial = i;
			obj = (struct object *)convaddr(mol[i],MASTER,slavep);
			plist = (struct chromplist *)convaddr(obj->plist,MASTER,slavep);
			clo[nactive++].class = plist->pgroup;
		}
	}
	qsort(clo,nactive,sizeof(struct class_order),clorder);

	/*
	 * Loop in range 1 - nactive until all active objects processed.
	 *	(1) Compute required transformed object, place in rol[]
	 *	(2) Wait until object status word indicates that master
	 *		copy has been made.  Meanwhile, get on processing next.
	 *	(3) Free object.
	 *
	 * Check that we have more than enough free memory to do the job
	 */
	notfree = j = 0;
	while (j < nactive && free_left()>ENOUGH_MEM) {
		i = clo[j].serial;
/**********************************/
		lock7(&(kcont->lock));
		statpt = acl + i;
		if (!(*statpt & ACTIVE) || (*statpt & SLFREE))
			unlock7(&(kcont->lock));
		/*
		 * Need to take great care freeing things, since some service
		 * functions do not duplicate domain if slave is running in master.
		 * Also, must only free those objects created in this slave.
		 */
		else if ((*statpt & MASTCOPY) && ((*statpt&MPROC)>>6) == slavep) {
			*statpt |= SLFREE;
			unlock7(&(kcont->lock));
			rol[i]->plist = NULL;
#ifdef DEBUG
			fprintf(stderr,"K_meas: freeing rol[%d] (linkcounts %d %d) (total free %d)\n",i+1,rol[i]->idom->linkcount,rol[i]->vdom->linkcount,free_left());
#endif DEBUG
			freeobj(rol[i]);
		}
		else if (*statpt & FUNCBEGIN) {
			unlock7(&(kcont->lock));
			notfree = 1;
		}
		else {
			*statpt |= FUNCBEGIN;
			unlock7(&(kcont->lock));
/**********************************/
			notfree = 1;
			colour(OVERLAY1);
			intens(1);
			dispnum(i+1,num_xpos,slavey);
			obj = mpcopyobj(convaddr(mol[i],MASTER,slavep),MASTER,slavep,0);
			colour(GREYCOLS);
			intens(0);
			f = ivf + i;
			/*
			 * compute transformation
			 */
			switch (com) {
				case K_STRAIGHTEN:
					nobj = autostraight(obj);
					/* undisplay old object */
					picframe(obj,f);
					/*
					 * massage the frame before redisplay
					 */
					f->ox = (nobj->idom->kol1 + nobj->idom->lastkl)/2;
					if (f->ix > 0)
						f->oy = nobj->idom->line1;
					else
						f->oy = nobj->idom->lastln;
					break;
				case K_STR_POL:
					if (slavep == MASTER)
						poly = (struct object *) kcont->gool1;
					else
						poly = mpcopyobj(kcont->gool1,MASTER,slavep,0);
					nobj = straighten(obj,poly,0,0);
					if (slavep != MASTER)
						freeobj(poly);
					break;
				case K_ENHANCE:
					nobj = enhance(obj);
					break;
				case K_NORMALISE:
					/*
					 * since we did not copy the grey values
					 * in mpcopyobj, must do it here since
					 * normalise OVERWRITES original grey-table
					 */
					nobj = newgrey(obj);
					normalise(nobj);
					break;
				case K_ROTATE:
					nobj = spin(obj,RADIANS*kcont->auxstat);
					/* undisplay old object */
					picframe(obj,f);
					break;
				case K_ENLARGE:
					escale = (double)kcont->auxstat / 100;
					nobj = squeeze(obj,escale,escale);
					break;
			}
			/*
			 * display new object
			 */
			intens(1);
			switch (com) {
				case K_STRAIGHTEN:
					/* only if straightening all objects */
					if (nactive > 1)
						picframe(nobj,f);
					break;
				case K_ROTATE:
				case K_STR_POL:
					break;
				default:
					picframe(nobj,f);
					break;
			}
			colour(OVERLAY1);
			intens(0);
			dispnum(i+1,num_xpos,slavey);
			/*
			 * free obj immediately
			 */
			obj->plist = NULL;
			freeobj(obj);
/**********************************/
			lock7(&(kcont->lock));
			rol[i] = nobj;
			*statpt |= (FUNCEND | (slavep << 6) | PROCESSED);
			unlock7(&(kcont->lock));
/**********************************/
			/*
			 * set j back to zero so that space clear
			 * loop is executed as often as possible
			 * (N.B. j++ later in loop)
			 * - DON'T DO WHAT JIMP DID and place this
			 * statement before "rol[i] = nobj;" !!!
			 */
			j = -1;
		}
		if (kcont->mstatus & ABORT) {
			break;
		}
		j++;
		if (j >= nactive && notfree) {
			tsleep(2);
			j = 0;
			notfree = 0;
		}
	}
	kcont->status[slave] |= FUNCEND;
#ifdef DEBUG
	if (free_left()<ENOUGH_MEM)
		fprintf(stderr,"K_serv - insufficient memory\n");
#endif DEBUG
}

static int free_left()
{
	return(getsys(0x72,4));
}

clorder(c1,c2)
register struct class_order *c1, *c2;
{
	return(c1->class - c2->class);
}
