/*
 * mpdeleteobj.c
 *
 *	Delete an object structure that has been copied by mpcopyobj
 *	from one processor to here.
 * If flag "all" is set to zero, only those parts of
 * hierarchy which contain pointer variables need deleting.  If "all" is 
 *	true, also delete tables of grey-values, interval end points, polygon
 * vertices, etc..
 *
 * NOTE: These routiunes don't delete anything - they just make the
 *		structure cosher to hit with the woolz object deleter
 *
 * Modifications
 *
 *  16 may 1991		ih			return NULL for NULL obj in mpdvdom
 *	18 feb 1987		CAS			Extend facilities in delete vdom
 *
 */

#include <stdio.h>
#include <wstruct.h>


struct object *mpdeleteobj(obj,all)
struct object *obj;
int	all;
{
	struct object *makemain(), *nobj;
	struct intervaldomain *mpdidom();
	struct polygondomain *mpdpolydom();
	struct valuetable *mpdvdom();
	struct propertylist *mpdplist();
	
	if (obj) {
		switch (obj->type) {
			case 1:
				obj->idom = mpdidom(obj->idom,all);
				obj->vdom = mpdvdom(obj->vdom,all);
				obj->plist = mpdplist(obj->plist,all);
				break;
			case 10:
				obj->idom = (struct intervaldomain *) mpdpolydom(obj->idom,all);
				obj->plist = mpdplist(obj->plist,all);
				break;
			default:
				fprintf(stderr,"MPDELETEOBJ: cannot delete type %d\n",obj->type);				
				return(NULL);
		}
	}
	return(obj);
}


struct intervaldomain *mpdidom(idom,all)
struct intervaldomain *idom;
{
	if (idom) {
		if (idom->type == 2 && all == 0)
			return(NULL);
		if (idom->type == 2)
			return(idom);
		if (!all)
			idom->intvlines = NULL;
	}
	return(idom);
}


struct polygondomain *mpdpolydom(pdom)
struct polygondomain *pdom;
{
	return(NULL);
}


struct valuetable *mpdvdom(vdom,all)
struct valuetable *vdom;
{
	if ( vdom ) {
		switch(vdom->type) {
		case 1:
/*
 *	This should take care of itself
 *			if ( !all) {
 *				vdom->freeptr = NULL ;
 *			}
 */
			return ( vdom ) ;
		case 11:
			return(vdom);
		default:
			fprintf(stderr,"MPDVDOM: cannot delete type %d space lost\n",vdom->type);
			return (NULL);
		}
	}
	return (NULL);
}


struct propertylist *mpdplist(pl,all)
register char *pl;
{
	if (all == 0)
		return(NULL);
	else
		return(pl);
}
