/*
 *		M P F U N C S . C --
 *
 * Master process functions for simplified control of
 * multi-tasking and multi-processing for MRC M68000 - OS9
 * chromosome analysis system.
 *
 * These apply the same function to ALL active slaves (which are
 * detected by having kcont->processor[slave] non-zero).
 *
 * They all wait for completion before returning to the caller,
 * so if you want something to happen asynchronously you will have
 * to do it differently.
 *
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *  Taken from: Jim Piper et al.
 *				Medical Research Council
 *				Clinical and Population Cytogenetics Unit
 *				Edinburgh
 *
 *  Date:    1st November 1985.
 *
 *  Modifications
 *
 *	 6 Jun 1991 ih			Tidy up death check in sing_mp_kserv
 *  31 May 1991 ih			Fixed silly bug in sing_mp_kserv getting sid use- kcont->processor[MASTER] not kcont->acl (dick head!!!)
 *                          Also tsleep(2) - 10 Extra work in wait slave loops
 *  30 May 1991 ih			Debug prints when slave found dead
 *   2 May 1991 ih			Check for null object in mp_k_serv
 *  17 Apr 1991 ih			Check if master slave alive in func_finish_slaves
 *	15 Apr 1991 ih			Check if master slave alive after mpcopyobj
 *	10 Apr 1991 ih			Check if commdslave was succesful in mpclear
 *	11 Jan 1991	CAS			Additional debug for slave lockup in finish_slave
 *	 1 Mar 1989	SEAJ		Added PROCESSED flag
 *  16 Dec 1988	SEAJ		Make sure sing_mp_k_serv copes with NULL objects returned
 *							from mpcopyobj.
 *	30 Jun 1988	SCG/SEAJ	mod in func_clear_bits; clear bits of ACTIVE
 *							objects only.
 *  22 Sep 1987	JIMP		Mods so that MASTCOPY not destroyed by multi-processor
 *							display routines.
 *  31 Mar 1987 JP			Changes to 30-03-87 stuff, plus some tidying.
 *  30 Mar 1987 JP			New single-object function for karyotype services.
 *  13 Mar 1987 JP			New function for karyotype services.
 */

#include <stdio.h>
#include <multip.h>
#include <wstruct.h>
#include <chromanal.h>
#include <kcontrol.h>

#define TIMEOUT 7		/* Slave finish timesout + prints status */
#define SLAVE_DEATH "my poor slave hath died on me\n"
/*
 *	M P C L E A R  --  multi-slave screen clear
 *
 */
mpclear(kcont)
register struct kcontrol *kcont;
{
	register int slave;
	register int sid=kcont->processor[0]&P_SS;
	char going;
	int wait;
	
	kcont->auxstat = 0;
	going=0;
	for (slave=0; slave<=NSLAVES; slave++) {
		if (kcont->processor[slave] != 0) {
		    if (commdslave(kcont,slave,SCREEN_CLEAR)==-1) {
#ifdef DEBUG
				char buffer[80];
				sprintf(buffer,"mpclear %s",SLAVE_DEATH);
				fprintf(stderr,"%s",buffer);
#endif DEBUG
		    	kcont->processor[slave]=0;
		    }
		    else going=1;
		}
	}
	if (!going) return(0);
	while (kcont->auxstat != ALL_CLEAR) {
		wait=Alloc_wait(1);
		while (waitchild());
		if (testproc(sid)==-1) {
			kcont->processor[0]=0;
			Alloc_wait(wait);
			return(0);
		}
		Alloc_wait(wait);
		tsleep(10);
	}
}


/*
 *	M P C P I C  ---
 *
 * multi-slave display of active objects in kcont->ool,
 * using frame kcont->f.
 */
mpcpic(kcont,colour,intens)
register struct kcontrol *kcont;
{
	/* set up colour and intensity */
	kcont->colour = colour;
	kcont->intens = intens;
	
	mpstandard(kcont,COM_DISPLAY_LIST);
}


/*
 *	M P I P I C  --
 *
 * multi-slave display of active objects in kcont->rol,
 * using appropriate frames from kcont->ivf.  Each slave
 * will only display an object marked as belonging to its
 * processor in the "MPROC" bits of the activity list.
 */
mpipic(kcont,colour,intens)
register struct kcontrol *kcont;
{
	/* set up colour and intensity */
	kcont->colour = colour;
	kcont->intens = intens;
	
	mpstandard(kcont,INDV_DISPLAY_LIST);
}

/*
 *	M P S T A N D A R D  ---  
 *
 *	standard multi-processor multi-object function driver
 */
mpstandard(kcont,funccode)
register struct kcontrol *kcont;
{
	register int ac,i,slave;

	/* clear function bits in activity list */
	func_clear_bits(kcont,(FUNCBEGIN|FUNCEND));

	/* clear function bits in slave status words, and start slaves */
	func_start_slaves(kcont,funccode);

	/* wait for slaves to finish */
	func_finish_slaves(kcont);
}

/*
 *	F U N C _ S T A R T _ S L A V E  --
 *
 * clear function bits in slave status words, and start slaves 
 */
func_start_slaves(kcont,funccode)
register struct kcontrol *kcont;
{
	register int slave,ac;
	for (slave=0; slave<=NSLAVES; slave++) {
		if (kcont->processor[slave] != 0) {
			kcont->status[slave] &= ~(FUNCBEGIN|FUNCEND|MAXNCHANGE);
			commdslave(kcont,slave,funccode);
		}
	}
}

/*
 *	F U N C _ F I N I S H _ S L A V E S  -- wait for slaves to finish 
 *
 */
func_finish_slaves(kcont)
register struct kcontrol *kcont;
{
	register int slave,ac,sid;
	int wait;
#ifdef TIMEOUT
	int		i, starttime;

	starttime = tick();
#endif
	sid=kcont->processor[MASTER]&P_SS;
	for (slave=0; slave<=NSLAVES; slave++) {
		if (kcont->processor[slave] != 0) {
			do {
				if (slave==MASTER) {
					wait=Alloc_wait(1);
					while (waitchild());
					if (testproc(sid)==-1) { /*pick up any dead offspring*/

#ifdef DEBUG
						char buffer[80];
						sprintf(buffer,"func_finish_slaves %s",SLAVE_DEATH);
						fprintf(stderr,"%s",buffer);
#endif DEBUG

						Alloc_wait(wait);
						break;
					}
					Alloc_wait(wait);
				}
				ac = kcont->status[slave];
				ac &= FUNCEND;
				if (ac == 0) {
#ifdef TIMEOUT
					if ((tick() - starttime) > 100*60*TIMEOUT) {
						starttime = tick();
						printf("Slave %d minute itch kc 0x%x\nprocs:",TIMEOUT,kcont);
						for (i=0;i<=NSLAVES;i++)
							printf(" %x",kcont->processor[i]);
						printf(" stats:");
						for (i=0;i<=NSLAVES;i++)
							printf(" %x",kcont->status[i]);
						printf("\n");
					}
#endif
					tsleep(10);
				}
            } while (ac == 0);
		}
	}
}

/*
 *	F U N C _ C L E A R _ B I T S  --
 *
 *	clear function bits in all activity words 
 */
func_clear_bits(kcont,bits)
register struct kcontrol *kcont;
unsigned short bits;
{
	register int i;
	for (i=0; i<kcont->maxnumber; i++) 
		if(kcont->acl[i] & ACTIVE)  kcont->acl[i] &= ~bits;
}

/*
 *	M P L I K L  --  multi-processor likelihood computation
 */
mplikl(kcont)
register struct kcontrol *kcont;
{
	register int ac,i,slave;

	/* clear function bits in slave status words, and start slaves */
	func_start_slaves(kcont,CHROM_LIKL_MEASURE);

	/* wait for slaves to finish */
	func_finish_slaves(kcont);
}

/*
 *	M P _ K _ S E R V  --
 *
 *	standard multi-processor multi-object karyotype service driver
 */
mp_k_serv(kcont,funccode)
register struct kcontrol *kcont;
{
	register int ac,i,slave,slavep,alldone;
	register SMALL *statpt;
	int sid,wait;
	struct chromosome *mobj, *nobj, *mpcopyobj();
	int deadslave;

	/* clear function bits in activity list */
	func_clear_bits(kcont,(FUNCBEGIN|FUNCEND|MASTCOPY|MPROC|SLFREE));
	
	/* clear function bits in slave status words, and start slaves */
	func_start_slaves(kcont,funccode);

	/* loop copying back processed but uncopied objects */
	i = 0;
	alldone = 1;
	sid=kcont->processor[MASTER]&P_SS;
	deadslave=0;
	while (i<kcont->maxnumber) {
		statpt = kcont->acl + i;
		ac = *statpt;
		/* slave processor number */
		slavep = (ac & MPROC) >> 6;
		if ((ac&ACTIVE) && !(ac&FUNCEND)) {
			if (slavep==MASTER) {
				if (!deadslave)
					alldone=0;
			}
			else
				alldone=0;
		}
		if ((ac&ACTIVE) && (ac&FUNCEND) && !(ac&MASTCOPY)) {
			nobj = kcont->mol[i];
			kcont->mol[i]=mobj=mpcopyobj(convaddr(kcont->rol[i],slavep,MASTER),slavep,MASTER,1);
			if ((slavep==MASTER)&&!deadslave) {
				wait=Alloc_wait(1);
				while (waitchild());
				if (testproc(sid)==-1) {
#ifdef DEBUG
					char buffer[80];
					sprintf(buffer,"mp_kserv %s",SLAVE_DEATH);
					fprintf(stderr,"%s",buffer);
#endif DEBUG
					deadslave=1;
				}
				Alloc_wait(wait);
			}
			if (deadslave) {
				if (kcont->mol&&kcont->mol[i]) {
					if (kcont->mol[i]!=kcont->ool[i]) {
						kcont->mol[i]->plist=NULL;
						freeobj(kcont->mol[i]);
					}
				}
				kcont->mol[i]=kcont->ool[i];
				if (nobj) {
					nobj->plist=NULL;
					freeobj(nobj);
				}
				mobj=NULL;
				nobj=NULL;
			}
			/*
		 	 * mpcopyobj(,,,1) may make a NEW propertylist.
			 * Attach the original.  Free old mol[] object
			 * if it different to the kol[] (keeplist) version.
			 */
			if (mobj) {
				if (mobj->plist != NULL)
					Free(mobj->plist);
				if (nobj&&nobj->plist)
					mobj->plist = nobj->plist;
				else
					mobj->plist=NULL;
			}
			if (nobj) {
				if (nobj != kcont->kol[i]) {
					nobj->plist = NULL;
					freeobj(nobj);
				}
			}
			*statpt |= MASTCOPY;
		}
		i++;
		if (i == kcont->maxnumber && !alldone) {
			i = 0;
			alldone = 1;
			tsleep(10);
		}
	}
	/* wait for slaves to finish */
	func_finish_slaves(kcont);
}

/*
 *	S I N G _ M P _ K _ S E R V  --
 *
 * Standard multi-processor single-object karyotype service driver.
 * All relevant slaves are started, first available does the service.
 */
struct chromosome *
sing_mp_k_serv(kcont,funccode,objnum)
register struct kcontrol *kcont;
{
	register int ac,i,slave,slavep;
	register SMALL *statpt;
	int sid;
	char ok=1;
	struct chromosome *mobj, *nobj, *mpcopyobj();

	/* set function bits in activity list */
	for (i=0; i<kcont->maxnumber; i++) 
		kcont->acl[i] |= (FUNCBEGIN|FUNCEND|MASTCOPY|MPROC|SLFREE|PROCESSED);

	/* clear function bits in activity word of specified object */
	kcont->acl[objnum] &= ~(FUNCBEGIN|FUNCEND|MASTCOPY|MPROC|SLFREE|PROCESSED);
	
	/* clear function bits in slave status words, and start slaves */
	func_start_slaves(kcont,funccode);

	statpt = kcont->acl + objnum;
	/* slave processor number */
	slavep = (*statpt & MPROC) >> 6;
	sid=kcont->processor[MASTER]&P_SS;
	/* wait for completion */
	while (!(*statpt & FUNCEND)) {
		if (slavep==MASTER) {
			int wait;
			wait=Alloc_wait(1);
			while (waitchild());
			if (testproc(sid)==-1) {
#ifdef DEBUG
				char buffer[80];
				sprintf(buffer,"sing_mp_kserv %s",SLAVE_DEATH);
				fprintf(stderr,"%s",buffer);
#endif DEBUG
				ok=0;
				if (kcont->mol&&kcont->mol[objnum]) {
					if (kcont->mol[objnum]!=kcont->ool[objnum]) {
						kcont->mol[objnum]->plist=NULL;
						freeobj(kcont->mol[objnum]);
					}
				}
				kcont->mol[objnum]=kcont->ool[objnum];
				Alloc_wait(wait);
				break;
			}
			Alloc_wait(wait);
		}
		tsleep(10);
	}
	nobj=kcont->mol[objnum];
	if (ok)
		mobj=mpcopyobj(convaddr(kcont->rol[objnum], slavep,MASTER),slavep,MASTER,1);
	else
		mobj=NULL;
	/*
	 * mpcopyobj(,,,1) may make a NEW propertylist.
	 * Attach the original.  Free old mol[] object
	 * if it different to the kol[] (keeplist) version.
	 */
	if (mobj != NULL) {
		if (mobj->plist != NULL)
			Free(mobj->plist);
		if (nobj)
			mobj->plist = nobj->plist;
		else
			mobj->plist=NULL;
	}
	*statpt |= MASTCOPY;

	/* wait for slaves to finish */
	func_finish_slaves(kcont);
	return(mobj);
}

