/*
 * sl_cpic.c		8/11/85		Jim Piper
 *
 ************************************************
 * slave service function to					*
 * display a list of objects (master addresses) *
 * in a common picture frame.
 ************************************************
 ***********************************************************************
 * MODIFICATION HISTORY
 * --------------------
 *	Who		When		What
 *	---		----		----
 *  ih  39-4-91		check for null objs in sl_cpic before calling picframe or freeobj
 *	JP	17-10-86	display.h used
 ***********************************************************************
 */


#include <stdio.h>
#include <multip.h>
#include <wstruct.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <display.h>

sl_cpic (slave, kcont)
struct kcontrol *kcont;
{
	struct object *obj;
	struct object *mpcopyobj();
	struct object **ool;
	struct pframe *f;
	int i, ac, slavep, slavey;
	SMALL *acl, *statpt;
#ifdef DEBUG
	char dstring[40];
#endif DEBUG

	/* slave process number must be converted to true processor number */
	slavep = kcont->processor[slave] >> P_OR;
#ifdef DEBUG
	slavey = 32 + 80 * slave;
	tsize(50,50);
	sprintf(dstring,"slave %d pr_or %d pr_ss %d object",slave,slavep,
		kcont->processor[slave] & P_SS);
	dispstring(dstring,2400,slavey);
#endif DEBUG
	kcont->status[slave] |= FUNCBEGIN;
	ool = (struct object **)convaddr(kcont->ool,MASTER,slavep);
	acl = (SMALL *)convaddr(kcont->acl,MASTER,slavep);
	f = (struct pframe *)convaddr(kcont->f,MASTER,slavep);

	/*
	 * wait until at least one object present
	 */
	ac = kcont->mstatus & MAXNCHANGE;
	while (ac != 0 && kcont->maxnumber <= 0)
		tsleep(5);

	/*
	 * Loop in range 1 - maxnumber until all active objects displayed
	 */
	i = 0;
	while (i < kcont->maxnumber) {
/**********************************/
		lock7(&(kcont->lock));
		statpt = acl + i;
		if ((*statpt & (ACTIVE|FUNCBEGIN)) != ACTIVE) {
			unlock7(&(kcont->lock));
		} else {
			*statpt |= FUNCBEGIN;
			unlock7(&(kcont->lock));
/**********************************/
			if (slavep == MASTER)
				obj = ool[i];
			else
				obj = mpcopyobj(convaddr(ool[i],MASTER,slavep),
						MASTER,slavep,0);
#ifdef DEBUG
			intens(1);
			colour(OVERLAY1);
			dispnum(i+1,3960,slavey);
#endif DEBUG
			intens(kcont->intens);
			colour(kcont->colour);
			if ((obj)&&(f))
				picframe(obj,f);
#ifdef DEBUG
			intens(0);
			colour(OVERLAY1);
			dispnum(i+1,3960,slavey);
#endif DEBUG
			/*
			 * If not running in master processor, can delete "obj" now.
			 * First set obj->plist to NULL since "obj" and "robj"
			 * share the property list.
			 */
			if (slavep != MASTER) {
				if (obj) {
					obj->plist = NULL;
					freeobj(obj);
				}
			}
/**********************************/
			lock7(&(kcont->lock));
			*statpt |= FUNCEND;
			unlock7(&(kcont->lock));
/**********************************/
		}
		ac = kcont->mstatus & MAXNCHANGE;
		i++;
		if (i >= kcont->maxnumber && ac != 0) {
			tsleep(2);
			i = 0;
		}
	}
	kcont->status[slave] |= FUNCEND;
}
