/*
 * sl_cpic.c		15/11/85		Jim Piper
 *
 ************************************************
 * slave service function to					*
 * display a list of objects (master addresses) *
 * in individual picture frames.                *
 *                                              *
 * This routine will only handle FIXED          *
 * kc->maxnumber                                *
 ************************************************
 ***********************************************************************
 * MODIFICATION HISTORY
 * --------------------
 *	Who		When		What
 *	---		----		----
 *  ih   2-04-91	protect call to picframe from null obj and ivf
 *	JP	17-10-86	display.h used
 ***********************************************************************
 */


#include <stdio.h>
#include <multip.h>
#include <wstruct.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <display.h>

sl_ipic (slave, kcont)
struct kcontrol *kcont;
{
	struct object *obj;
	struct object **mol;
	struct pframe *ivf;
	int i, slavep, slavey;
	SMALL *acl, *statpt;
#ifdef DEBUG
	char dstring[40];
#endif DEBUG

	/* slave process number must be converted to true processor number */
	slavep = kcont->processor[slave] >> P_OR;
#ifdef DEBUG
	slavey = 32 + 80 * slave;
	tsize(50,50);
	sprintf(dstring,"slave %d pr_or %d pr_ss %d object",slave,slavep,
		kcont->processor[slave] & P_SS);
	dispstring(dstring,2400,slavey);
#endif DEBUG
	kcont->status[slave] |= FUNCBEGIN;
	mol = (struct object **)convaddr(kcont->mol,MASTER,slavep);
	acl = (SMALL *)convaddr(kcont->acl,MASTER,slavep);
	ivf = (struct pframe *)convaddr(kcont->ivf,MASTER,slavep);

	/*
	 * Loop in range 1 - maxnumber until all active objects displayed
	 */
	i = 0;
	while (i < kcont->maxnumber) {
/**********************************/
		statpt = acl + i;
		lock7(&(kcont->lock));
		if ((*statpt & (ACTIVE|FUNCBEGIN)) != ACTIVE) {
			unlock7(&(kcont->lock));
		} else {
			*statpt |= FUNCBEGIN;
			unlock7(&(kcont->lock));
/**********************************/
			if (slavep == MASTER)
				obj = mol[i];
			else
				obj = (struct object *)
						mpcopyobj(convaddr(mol[i],MASTER,slavep),MASTER,slavep,0);
#ifdef DEBUG
			intens(1);
			colour(OVERLAY1);
			dispnum(i+1,3960,slavey);
#endif DEBUG
			intens(kcont->intens);
			colour(kcont->colour);
			if (obj&&ivf)
				picframe(obj,ivf+i);
#ifdef DEBUG
			intens(0);
			colour(OVERLAY1);
			dispnum(i+1,3960,slavey);
#endif DEBUG
			/*
			 * If not running in master processor, can delete "obj" now.
			 * First set obj->plist to NULL since "obj" and "robj"
			 * share the property list.
			 */
			if (slavep != MASTER) {
				if (obj) {
					obj->plist = NULL;
					freeobj(obj);
				}
			}
/**********************************/
			lock7(&(kcont->lock));
			*statpt |= FUNCEND;
			unlock7(&(kcont->lock));
/**********************************/
		}
		i++;
	}
	kcont->status[slave] |= FUNCEND;
}
