/*
 * sl_likl.c		Jim Piper	29/11/85
 *
 * Slave service function to compute class likelihoods.
 *
 * Modifications
 *
 ***********************************************************************
 * MODIFICATION HISTORY
 * --------------------
 *	Who		When		What
 *	---		----		----
 *	 7 Jan 1991		CAS		Lock classcont struct when possible multi-slave access
 *  12 Jul 1989		SEAJ	Temporary check on classcont->n to stop overruns 
 *	01 Aug 1988		SCG		Override returned class from idist by wanted class
 *	17 Mar 1987		jimp	Must set intens(1) outside #ifdef DEBUG !!
 *	11 Nov 1986		jimp	->gool1 replaced by ->C
 *	JP	17-10-86	display.h used
 *	14 Sep 1986		CAS		Includes and much casting
 ***********************************************************************
 */


#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <kcontrol.h>
#include <multip.h>
#include <display.h>


sl_likl (slave, kcont)
struct kcontrol *kcont;
{
	struct classifier *C;
	int i, ac, slavep, slavey;
	SMALL *acl, *statpt;
	struct ipoint blob, **bol, *mblob;
	struct pframe *f;
	struct chromosome *obj;
	struct chromplist *plist;
#ifdef DEBUG
	char dstring[40];
#endif DEBUG

	/* slave process number must be converted to true processor number */
	slavep = kcont->processor[slave] >> P_OR;
#ifdef DEBUG
	slavey = 32 + 80 * slave;
	tsize(50,50);
	sprintf(dstring,"slave %d pr_or %d pr_ss %d object",slave,slavep,
		kcont->processor[slave] & P_SS);
	dispstring(dstring,2400,slavey);
#endif DEBUG
	kcont->status[slave] |= FUNCBEGIN;
	C = (struct classifier *) convaddr(kcont->C,MASTER,slavep);
	acl = (SMALL *) convaddr(kcont->gacl1,MASTER,slavep);
	bol = (struct ipoint **) convaddr(kcont->bol,MASTER,slavep);
	f = (struct pframe *) convaddr(kcont->f,MASTER,slavep);

	/* set up blob display */
	blob.type = 40;
	blob.style = 4;	/* fixed size hollow square */
	/*
	 * Loop in range minnumber - maxnumber
	 * until all active objects classified
	 */
	i = 0;
	while (i < C->numchr) {
/**********************************/
		lock7(&(kcont->lock));
		statpt = acl + i;
		if ((*statpt & (ACTIVE|FUNCBEGIN)) == ACTIVE) {
			*statpt |= FUNCBEGIN;
			unlock7(&(kcont->lock));
/**********************************/
			intens(1);
			colour(OVERLAY1);
#ifdef DEBUG
			dispnum(i+1,3960,slavey);
#endif DEBUG
			likelihood(kcont,i,slavep);
			/*
			 * keep the operator happy by displaying
			 * a blob at the chromosome being classified.
			 */
			obj = (struct chromosome *) convaddr(C->chrlist[i],MASTER,slavep);
			plist = (struct chromplist *) convaddr(obj->plist,MASTER,slavep);
			mblob = (struct ipoint *) convaddr(bol[plist->number-1],MASTER,slavep);
			blob.k = mblob->k;
			blob.l = mblob->l;
			picframe(&blob,f);
#ifdef DEBUG
			intens(0);
			colour(OVERLAY1);
			dispnum(i+1,3960,slavey);
#endif DEBUG
/**********************************/
			lock7(&(kcont->lock));
			*statpt |= FUNCEND;
/**********************************/
		}
		unlock7(&(kcont->lock));
		i++;
	}
	kcont->status[slave] |= FUNCEND;
}



/*
 * compute likelihoods for each class, for one chromosome.
 */
likelihood(kc,n,slavep)
struct kcontrol *kc;
{
	register struct classcont *classcontp;
	register struct chromcl *chp;
	register struct classifier *C;
	struct chromosome *obj;
	struct chromplist *plist;
	int ivec[MAXDIM], fvec[MAXDIM];
	register int i,class;
	
	C = (struct classifier *) convaddr(kc->C,MASTER,slavep);
	obj = (struct chromosome *) convaddr(C->chrlist[n],MASTER,slavep);
	plist = (struct chromplist *) convaddr(obj->plist,MASTER,slavep);
	chp = &C->chrocl[n];
	/* feature vector */
	getvec(plist,fvec);
	/* normalise and convert to un-scaled integer */
	normivec(fvec,ivec,C->sfv,C->vn,C->ndim,C->nsfv);
	/* likelihood classifier */
	class = iclassdist(ivec, C, &(chp->cllika[0]), slavep) - 1;
	if((chp->trueclass = plist->Cpgroup) > 0)
		class = chp->trueclass - 1;
	chp->rawclass = class;
	chp->newclass = class;
	classcontp = &C->clcontents[class];
	lock7(&classcontp->lock);
	classcontp->chcl[classcontp->n] = 
			(struct chromcl *) convaddr(chp,slavep,MASTER);
	if (classcontp->n < MAXCHILD - 1)	/* Temporary check to stop overruns */
		classcontp->n++;
	unlock7(&classcontp->lock);
}

/*
 * classify by distance (variance-corrected) IN INTEGER ARITHMETIC,
 * computing log-likelihoods.
 *
 * "C" has correct address in processor "slavep", but beware the
 * pointers within it.  Note that the arrays within it do NOT
 * require further address conversion.
 */
#define LARGENEG -999999999999.0
iclassdist (fvec,C,liklv,slavep)
int *fvec;
struct classifier *C;
struct cllik *liklv;
{
	register int d,i,maxc;
	double maxl, lik;
	struct cllik liks[MAXCLASS];
	register struct cllik *liklvp;

	liklvp = liks;
	for (i=0; i<C->nclass; i++) {
		d = idistvar(fvec,&C->iclassmeans[i][0],&C->iclassinvvars[i][0],C->ndim);
		d = (-d/2);
		lik = (double)d;
		lik *= C->conversionf[i];
		/*
		 * the following term also includes prior probability
		 * (see "readclassparams()").
		 */
		liklvp->lval = lik + C->clinvvarsqrtdet[i];
		liklvp->class = i;
		liklvp++;
	}
	/*
	 * sort likelihoods into order for rearrangement classifier.
	 * copy first CUTOFF values into class likelihood structure
	 */
	for (i=0; i<CUTOFF; i++) {
		liklvp = liks;
		maxl = LARGENEG;
		for (d=0; d<C->nclass; d++) {
			if (liklvp->lval > maxl) {
				maxl = liklvp->lval;
				maxc = d;
			}
			liklvp++;
		}
		liks[maxc].lval = LARGENEG;
		liklv[i].lval = maxl;
		liklv[i].class = maxc;
	}
	return (liklv->class+1);
}
