/*
 * wormcast.c		Jim Piper		19/11/85
 *
 * This routine looks for worms in a "kcontrol" structure, using known
 * data consistencies to detect overwriting of data.  It can be called
 * from any processor, and should be run frequently if problems are
 * suspected.  The "comment" string permits some information about
 * calling point to be included in the error message, which is displayed
 * by DDGS on the graphics screen.
 *
 * Modifications
 *
 *	12 Aug 1988		CAS		Use BDP's limits (from woolz checker)
 *	14 Oct 1987		CAS		Don't bother noting NULL objects - Now quite OK
 *							in ktype review
 *	17 Dec 1986		jimp	Column bound checks added.
 *	15 Dec 1986		jimp	Final fprintf to stderr, not stdout.
 *							Moved from KYY to MULTIP.
 *							Less restrictive line, column bounds for objects.
 *							In particular, line and column numbers of rotated
 *							objects can be negative.
 *	15 Sep 1986		CAs		Fixed spelling of intervaldomain - 
 *							and converted to vax names, also did some casts
 *	27 May 1986   			changer diagnistics
 */
#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <multip.h>
#include <kcontrol.h>
#include <wtcklims.h>

static int pslave;
static char *comment;
static char string[200];

wormcast(kc,slavep,c)
struct kcontrol *kc;
char *c;
{
	struct chromosome *obj, **ool, **mol;
	struct intervaldomain *idom;
	struct chromplist *plist;
	SMALL *acl;
	int ac,i,j,k;

	pslave = slavep;
	comment = c;
		
	ool = (struct chromosome **) convaddr(kc->ool,MASTER,slavep);
	mol = (struct chromosome **) convaddr(kc->mol,MASTER,slavep);
	acl = (SMALL *) convaddr(kc->acl,MASTER,slavep);
	
	for (i=0; i<kc->maxnumber; i++) {
		ac = acl[i];
/*
		if ((ac & ACTIVE) != 0) {
*/

			/*
			 * object main list tests
			 */
			if (ool[i] == NULL) {
#ifdef NULLOBJTEST
				worm("null object",i);
				return(1);
#else NULLOBJTEST
				return(0);
#endif NULLOBJTEST
			}
			obj = (struct chromosome *) convaddr(ool[i],MASTER,slavep);
			if (obj->type != 1) {
				worm("object type not 1",i);
				return(1);
			}
			
			/*
			 * domain tests
			 */
			if (obj->idom == NULL) {
				worm("null domain",i);
				return(1);
			}
			idom = (struct intervaldomain *)  convaddr(obj->idom,MASTER,slavep);
			if (idom->type != 1 && idom->type != 2) {
				worm("bad domain type",i);
				return(1);
			}
			if ((idom->lastln - idom->line1) < -1 || idom->line1 < CKLINEMIN ||
				idom->lastln > CKLINEMAX) {
				worm("bad domain line bounds",i);
				return(1);
			}
			if ((idom->lastkl - idom->kol1) < -1 || idom->kol1 < CKKOLMIN ||
				idom->lastkl > CKKOLMAX) {
				worm("bad domain column bounds",i);
				return(1);
			}
			
			/*
			 * property list tests
			 */
			if (obj->plist == NULL) {
				worm("null plist",i);
				return(1);
			}
			plist = (struct chromplist *) convaddr(obj->plist,MASTER,slavep);
			if (plist->size != sizeof(struct chromplist)) {
				worm("plist wrong size",i);
				return(1);
			}
			if (plist->otype < 0 || plist->otype > 7) {
				worm("plist wrong otype",i);
				return(1);
			}
			
			/*
			 * master-copy of rotated object test
			 */
			if (mol[i] != NULL) {
				obj = (struct chromosome *) convaddr(mol[i],MASTER,slavep);
					if (obj->type != 1) {
					worm("r-object type not 1",i);
					return(1);
				}
			
				/*
				 * domain tests
				 */
				if (obj->idom == NULL) {
					worm("null r-domain",i);
					return(1);
				}
				idom = (struct intervaldomain *) convaddr(obj->idom,MASTER,slavep);
				if (idom->type != 1 && idom->type != 2) {
					worm("bad r-domain type",i);
					return(1);
				}
				if (idom->lastln - idom->line1 < -1 || idom->line1 < -200 ||
					idom->lastln > 2048) {
					worm("bad r-domain line bounds",i);
					return(1);
				}
				if (idom->lastkl - idom->kol1 < -1 || idom->kol1 < -200 ||
					idom->lastkl > 1224) {
					worm("bad r-domain column bounds",i);
					return(1);
				}

				/*
				 * property list tests
				 */
				if (obj->plist == NULL) {
					worm("null r-plist",i);
					return(1);
				}
				plist = (struct chromplist *) convaddr(obj->plist,MASTER,slavep);
				if (plist->size != sizeof(struct chromplist)) {
					worm("r-plist wrong size",i);
					return(1);
				}
				if (plist->otype < 0 || plist->otype > 7) {
					worm("r-plist wrong otype",i);
					return(1);
				}
			}
/*
		}
*/
	}		
	return(0);
}


/*
 * print an error message to screen
 */
worm(m1,i)
char *m1;
{
	/* erase old string */
	intens(0);
	dispstring(string,100,4000-100*pslave);
	/* fill string */
	sprintf(string,"%s: object %d: from %s",m1,i,comment);
	/* draw string */
	intens(1);
	dispstring(string,100,4000-100*pslave);
	sleep(10);
	fprintf(stderr,"%s",string);
}
