/*
 *      C H N A M E . C   --  Chenge case name of a cyto file
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *  Date:    2nd Jun 1987
 *
 *  Modifications
 *	 2 May 1989	dcb		Use convcellhead to cope with new + old headers
 *						include a dummy kcont defn.
 *
 */

#include <stdio.h>
#include <fbio.h>
#include <allcontr.h>
#include <kcontrol.h>
#include <wstruct.h>


struct propertylist *makeheader();
struct allkontr alcont, *alc;
struct kcontrol *kcont;

int		eflag = 0;		/* Actually do change if set */
int		debug = 0;		/* Debug if true */
int		sflag = 0;		/* Silent mode flag */
int		zflag = 0;		/* Get filenames from stdin */

FILE *debug_channel = stderr;

extern char *rindex();


main(argc,argv)
int		argc;
char	**argv;
{
	char	*swptr;
	FILE	*infile;
	char	buffer[128],newcase[40];

	alc = &alcont;
	alc->debug = 0;
	argc--; argv++;
	while ((argc > 0) && (*argv[0] == '-')) {
		swptr = *argv++; argc--;
		while (*(++swptr)) {
			switch (*swptr) {
			  case '?':
			  	printf("chname  - change case name of cytoscan file\n");
			  	printf("Syntax  - chname <options> <newcasename> [file name list]\n");
			  	printf("Options -  e    Execute output\n");
			  	printf("           d    Give debug info\n");
			  	printf("           s    Silent mode\n");
			  	printf("           z    get filenames from stdin\n");
			  	printf("           v    Print version\n");
			  	exit(0);
			  case 'd':
			  	alc->debug = 1;
			  	break;
			  case 'v':
			  	printf("Chname Version 0.2\n");
			  	exit(0);
			  case 'e':
			  	eflag = 1;
			  	break;
			  case 's':
			  	sflag = 1;
			  	break;
			  case 'z':
			  	zflag = 1;
			  	break;
			  default:
			  	;
			}
		}
	}
	if (argc) {
		strcpy(newcase,*argv++);
		argc--;
	} else {
		fprintf(stderr,"Insufficient arguements\n");
		exit(1);
	}
	if (zflag) {
		while (fgets(buffer,127,stdin)) {
			*(buffer+strlen(buffer)-1) = '\0';
			doname(buffer,newcase);
		}
	} else if (argc) {
		while (argc) {
			doname(*argv,newcase);
			argc--; argv++;
		}
	}
}

/*
 *	D O N A M E 
 */
doname(name,newcase)
char *name, *newcase;
{
	char	*pre,*npt,oldfil[100],newfil[100];
	struct cyto_file cfp;
	FILE *fq,*fo;
	char	*buffer[BUFSIZ];
	struct	object *head;
	struct	cell_header *hptr;
	struct propertylist *convcellhead();
	int		nbytes;

	strcpy(oldfil,name);
	npt = rindex(name,'/');
	if (npt == NULL) {
		npt = name;
		pre = NULL;
	} else {
		*npt++ = '\0';
		pre = name;
	}
	if (!iscyto(npt,&cfp)) {
		fprintf(stderr,"%s is not a Cytoscan file\n",npt);
		return(0);
	}
	if (alc->debug)
		printf ("Change %s to case %s in dir %s\n",npt,newcase,
				pre ? pre : ".");
	if (strcasecmp(newcase,cfp.case) == NULL) {
		fprintf(stderr,"New name is the same as old name - file ignored\n");
		return(0);
	}
	switch (cfp.type) {
		case CELLFILE:
			sprintf(newfil,"%s%s%s$%s$%d",pre,pre ? "/" : "", newcase,cfp.slide, cfp.cellno);
			break;
		case KARYFILE:
			sprintf(newfil,"%s%s%s$%s$%dK",pre,pre ? "/" : "", newcase,cfp.slide, cfp.cellno);
			break;
		case METFILE:
			sprintf(newfil,"%s%s%s$%s$M",pre,pre ? "/" : "", newcase,cfp.slide);
			break;
	}
	if (!sflag)
		printf("Change %s to %s\n",oldfil,newfil);

	if ((fq = fbopen(oldfil,"r",BUFSIZ)) == NULL) {
		fprintf(stderr,"Chname - can't read %s\n",oldfil);
		return(0);
	}
	if ((fo = fbopen(newfil,"w",BUFSIZ)) == NULL) {
		fprintf(stderr,"Chname - can't create %s\n",newfil);
		fbclose(fq);
		return(0);
	}
	head = (struct object *) readobj(fq);
	if (!head || (head->type != 110)) {
		fprintf(stderr,"Chname - file has illegal format\n");
		fbclose(fq);
		fbclose(fo);
		unlink(newfil);
		return(0);
	}
	head->plist = convcellhead(head->plist);
	hptr = (struct cell_header *) head->plist;
	strncpy(hptr->patid,newcase, CASELENGTH);
	writeobj(fo,head);
	freeobj(head);
	while ((nbytes=fbread(buffer,1,BUFSIZ,fq)) > 0) {
		fbwrite(buffer,nbytes,1,fo);
	}
	fbclose(fq);
	fbclose(fo);
}
