/*
 *      C U R S P R O C S . C -- general cursor and mouse routines.
 *
 *
 *  Rewritten by Sarah E. A. Jones.
 *				
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    21st Mar 1989.
 *
 *  Modifications
 *
 *  29 July 1991	ih	Check mouse cursor pos against boundaries
 *	19 July	1991	ih	Added code to deal with straight lines
 *
 */
#include <cmdsys.h>
#include <wstruct.h>
#include <chromanal.h>
#include <dispform.h>
#include <ddgs.h>
#include <scrformat.h>
#include <display.h>
 
static struct pframe cursframe, *cf;
extern DDGS *dg;

struct scrform *scrf; 
struct hcopyform *hcpy;

struct ipoint cursor;
struct lineint *curr_line;
struct box *curr_box;
int curr_chrom;

struct hcobject *hclocate();
/*
 * I N I T _ C U R S --
 *
 */
init_curs()
{
	cursframe.scale = 8;
	cursframe.dx = dg->xcenter;
	cursframe.dy = dg->ycenter;
	cursframe.ox = dg->lineln/2;
	cursframe.oy = dg->lineno/2;
	cursframe.ix = 1;	
	cursframe.iy = 1;
	cf = &cursframe;
	cursor.type = 40;
	cursor.style = 6;
}
/*
 * E R A S E C U R S O R --
 *
 */
erasecursor()
{
	colour(RED);
	intens(0);
	picframe(&cursor, cf);
	intens(1);
}
/*
 * S H O W C U R S O R --
 *
 */
showcursor()
{
	colour(RED);
	intens(1);
	picframe(&cursor, cf);
}
/*
 * C U R S T R A C K --
 *
 */
curstrack()
{
	
	register int xscale,yscale;
	
	xscale = yscale = 1;
	if (cursframe.ix < 0)
		xscale = -xscale;
	if (cursframe.iy < 0)
		yscale = -yscale;
	erasecursor();
	cursor.k += mukont->mouss->XINC * xscale;
	cursor.l += mukont->mouss->YINC * yscale;
	if(cursor.k<dg->minx) cursor.k=dg->minx;
	if(cursor.k>dg->maxx) cursor.k=dg->maxx;
	if(cursor.l<dg->miny) cursor.l=dg->miny;
	if(cursor.l>dg->maxy) cursor.l=dg->maxy;
	showcursor();
}
/*
 * I N I T _ D I S P --
 *
 */
init_display()
{
	clearplane(dg, 1);
	setrgb(WHITE, 255, 255, 255);
	setrgb(BLACK, 0, 0, 0);
	setrgb(RED, 255, 0, 0);
	setrgb(BLUE, 0, 0, 255);
	setrgb(GREEN, 0, 255, 0);
	setrgb(YELLOW, 255, 255, 0);
	setrgb(VIOLET, 255, 0, 255);
	setrgb(AZURE, 0, 255, 255);
}
/*
 *  C U R S
 *
 */
curs()
{
	if (mukont->mouss->moving)
		curstrack();
	else
		if (kylocate()) ;
}

/*
 *  N E W L I N E -- start a a new stretch line
 */
static int xfreeze,yfreeze;
newline()
{
	xfreeze=cursor.k<<dg->xshift;
	yfreeze=cursor.l<<dg->yshift;
}

/*
 *  S T R E T C H L I N E
 *
 */
stretchline()
{
	if (mukont->mouss->moving) {
		moveto(xfreeze,yfreeze);
		intens(0);
		lineto(cursor.k<<dg->xshift,cursor.l<<dg->yshift);
		curstrack();
		moveto(xfreeze,yfreeze);
		intens(1);
		colour(OVERLAY1);
		lineto(cursor.k<<dg->xshift,cursor.l<<dg->yshift);
	}
}

/*
 * H C C U R S
 *
 */
hccurs()
{
	if (mukont->mouss->moving)
		curstrack();
	else
		hclocate(0);
}
/*
 * C A L C _ D I S T
 *
 */
calc_dist(b, dist, cx, cy, xoff, yoff)
struct box *b;
int *dist, cx, cy, xoff, yoff;
{
	register int xdist, ydist;

/*	fprintf(stderr, "calc_dist cx %d cy %d x %d y %d\n", cx, cy, b->posn.x + b->width/2,
		 b->posn.y + b->height/2); */
	xdist = cx - (b->posn.x + xoff + b->width/2);
	ydist = cy - (b->posn.y + yoff + b->height/2);
	*dist = xdist * xdist + ydist * ydist;
}
/*
 * C A L C _ T E X T
 *
 */
calc_text(p, dist, cx, cy, xoff, yoff)
struct hcobject *p;
int *dist, cx, cy, xoff, yoff;
{
	register int xdist, ydist;
	int len;
		
	len = strlen(p->txt);
	len *= p->b.width;
	len /= 2;
	xdist = p->b.posn.x + xoff + len;
	ydist = p->b.posn.y + yoff + (p->b.height);
	*dist = ((cx - xdist) * (cx - xdist)) + ((cy - ydist) * (cy - ydist));
}
			
/*
 *	K Y L O C A T E  --  identify box in karyotype from cursor position.
 *
 */
kylocate()
{
	register int xscale, yscale;
	static int lastnearest = -1, lastinstack = -1, lastidbox = 0, lastchromclass = 0;
	static int type, lastintext = -1;
	static struct lineint *lastinline=NULL;
	int idbox = 0, chromclass = 0, settype = 0;
	int	i,j,cx,cy,bx,by,tx,ty,dx,dy,len;
	int	dist,mindist,nearest,instack,intext;
	struct box *b;
	struct datab *id;
	struct lineint *inline,*l;
	
	/*
	 * calculate DDGS-coord cursor pos,
	 * restrict to DDGS frame
	 */
	xscale = yscale = 1;
	if (cursframe.ix < 0)
		xscale = -xscale;
	if (cursframe.iy < 0)
		yscale = -yscale;
	cursor.k += mukont->mouss->XINC * xscale;
	cursor.l += mukont->mouss->YINC * yscale;
	cx = cursor.k << dg->xshift;
	cy = cursor.l << dg->yshift;
	if (cx < 0) cx =0;

	nearest = -1;
	mindist = 99999999;
	instack = -1;
	intext = -1;
	inline=NULL;
	for (i=0; i<MAXCLASS-1; i++) {
		b = &scrf->classes[i].b;
		calc_dist(b, &dist, cx, cy, 0, 0);
		if (dist < mindist) {
			mindist = dist;
			nearest = i;
		}
	}
	b = &scrf->idbox.b;
	calc_dist(b, &dist, cx, cy, 0, 0);
	if (dist < mindist) {
		mindist = dist;
		idbox = 1;
	}
	b = &scrf->chromclass;
	calc_dist(b, &dist, cx, cy, 0, 0);
	if (dist < mindist) {
		mindist = dist;
		chromclass = 1;
		idbox = 0;
	}
	for (i = 0; i < scrf->viewed_stack; i++) {
		b = &scrf->stk[i];
		calc_dist(b, &dist, cx, cy, 0, 0);
		if (dist < mindist) {
			mindist = dist;
			instack = i;
			chromclass = 0;
			idbox = 0;
		}
	}
	bx = scrf->idbox.b.posn.x;
	by = scrf->idbox.b.posn.y;
	id = &scrf->idbox;
	for (i = 0; i < MAXTEXT; i++) {
		len = strlen(id->cont[i].text);
		len *= id->cont[i].size;
		len /= 2;
		tx = bx + id->cont[i].x + len;
		ty = by + id->cont[i].y + (id->cont[i].size/2);
		dist = ((cx - tx) * (cx - tx)) + ((cy - ty) * (cy - ty));
		if (dist < mindist) {
			mindist = dist;
			intext = i;
			instack = -1;
			chromclass = 0;
			idbox = 0;
		}
	}
	for(l=scrf->linebuf.lineint;l;l=l->next) {
		dx=l->x1-cx;
		dy=l->y1-cy;
		if((dist=(dx*dx+dy*dy))<mindist) {
			mindist=dist;
			inline=l;
			instack = -1;
			chromclass = 0;
			idbox = 0;
			intext=-1;
		}
		dx=l->x2-cx;
		dy=l->y2-cy;
		if((dist=(dx*dx+dy*dy))<mindist) {
			mindist=dist;
			inline=l;
			instack = -1;
			chromclass = 0;
			idbox = 0;
			intext=-1;
		}
	}
	if ((nearest!=lastnearest)||(idbox!=lastidbox)||(chromclass!=lastchromclass)
		||(instack!=lastinstack)||(intext!=lastintext)||(inline!=lastinline)) {
		lastnearest = nearest;
		lastidbox = idbox;
		lastchromclass = chromclass;
		lastinstack = instack;
		lastintext = intext;
		lastinline=inline;
		display_box(curr_box, 0, RED, type);
		display_box(curr_box, 1, BLUE, type);
		if (chromclass) {
			type = CHROMCLASS;
			curr_box = &scrf->chromclass;
		} else if (idbox) {
			type = IDBOX;
			curr_box = &scrf->idbox.b;
		} else if (instack != -1) {
			type = STACK;
			curr_box = &scrf->stk[instack]; 
			curr_chrom = instack;
		} else if (intext != -1) {
			type = IDTEXT;
			curr_box = NULL;
			curr_chrom = intext;
		} else if (inline) {
			type = IDLINE;
			curr_box = NULL;
			curr_line = inline;
		} else {
			type = CHROM;
			curr_box = &scrf->classes[nearest].b;
			curr_chrom = nearest;
		}
		display_box(curr_box, 0, BLUE, type);
		display_box(curr_box, 1, RED, type);
		settype = 1;
	}
	if (!settype) {
		if (chromclass)
			type = CHROMCLASS;
		else if (idbox)
			type = IDBOX;
		else if (instack != -1) {
			type = STACK;
			curr_chrom = instack;
		} else if (intext != -1) {
			type = IDTEXT;
			curr_chrom = intext;
		} else if (inline) {
			type = IDLINE;
			curr_line=inline;
		} else {
			type = CHROM;
			curr_chrom = nearest;
		}
	}
	return(type);
}
/*
 * G E T N E A R E S T
 *
 */
struct hcobject *
getnearest(p, curx, cury, mindist)
struct hcobject *p;
int curx, cury, *mindist;
{
	static struct hcobject *nearest;
	int dist;
	
	while (p != NULL) {
		if (p->type == TXT)
			calc_text(p, &dist, curx, cury, XOFFSET, YOFFSET);
		else
			calc_dist(&p->b, &dist, curx, cury, XOFFSET, YOFFSET);
		if (dist < *mindist) {
			nearest = p;
			*mindist = dist;
		}
		p = p->next;
	}
	return(nearest);
}
/*
 * H C L O C A T E
 *
 */
struct hcobject *
hclocate(removed)
int removed;
{
	static struct hcobject *lastptr = NULL;
	struct hcobject *ptr = NULL;
	register int xscale, yscale;
	int cx, cy, mindist;
	
	xscale = yscale = 1;
	if (cursframe.ix < 0)
		xscale = -xscale;
	if (cursframe.iy < 0)
		yscale = -yscale;
	cursor.k += mukont->mouss->XINC * xscale;
	cursor.l += mukont->mouss->YINC * yscale;
	cx = cursor.k << dg->xshift;
	cy = cursor.l << dg->yshift;
	if (cx < 0) cx =0;

	mindist = 99999999;
	ptr = getnearest(hcpy->hco, cx, cy, &mindist);
	if (ptr != lastptr) {
		if (!removed) {
			display_group(lastptr, 0, RED);
			if (lastptr->type == GROUP)
				display_all(lastptr->ass, 1, BLUE);
			else
				display_group(lastptr, 1, BLUE);
		}
		display_group(ptr, 0, BLUE);
		display_group(ptr, 1, RED);
		lastptr = ptr;
	}
	return(ptr);
}
/*
 * B O X C U R S
 *
 */
boxcurs(ptr)
struct hcobject *ptr;
{
	if (mukont->mouss->moving)
		curstrack();
	else
		makebox(ptr);
}
/*
 * M A K E B O X
 *
 */
makebox(current)
struct hcobject *current;
{
	struct hcobject *ptr;
	register int xscale, yscale;
	int cx, cy, hold, ingroupx = 0, ingroupy = 0;
		
	xscale = yscale = 1;
	if (cursframe.ix < 0)
		xscale = -xscale;
	if (cursframe.iy < 0)
		yscale = -yscale;
	cursor.k += mukont->mouss->XINC * xscale;
	cursor.l += mukont->mouss->YINC * yscale;
	cx = (cursor.k << dg->xshift) - XOFFSET;
	cy = (cursor.l << dg->yshift) - YOFFSET;
	if (cx < 0) cx =0;
	display_box(current, 0, VIOLET);
	current->b.width = cx - current->b.posn.x;
	current->b.height = cy - current->b.posn.y;
	display_box(current, 1, VIOLET);
	ptr = hcpy->hco;
	while (ptr != NULL) {
		if (inside(current, ptr)) {
			if (ptr->b.col != VIOLET) {
				display_box(ptr, 0, BLUE);
				display_box(ptr, 1, VIOLET);
			}
		} else if (ptr->b.col != BLUE) {
			display_box(ptr, 0, VIOLET);
			display_box(ptr, 1, BLUE);
		}
		ingroupx = ingroupy = 0;
		ptr = ptr->next;
	}
}
/*
 * I N S I D E
 *
 */
int
inside(current, ptr)
struct hcobject *current, *ptr;
{
	int ingroupx = 0, ingroupy = 0;
	
	if ((current->b.width >= 0) && (ptr->b.posn.x > current->b.posn.x) &&
				(ptr->b.posn.x < current->b.posn.x + current->b.width))
		ingroupx = 1;
	else if ((ptr->b.posn.x < current->b.posn.x) &&
				(ptr->b.posn.x > current->b.posn.x + current->b.width))
		ingroupx = 1;
	if ((current->b.height >= 0) && (ptr->b.posn.y > current->b.posn.y) &&
				(ptr->b.posn.y < current->b.posn.y + current->b.height))
		ingroupy = 1;
	else  if ((ptr->b.posn.y < current->b.posn.y) &&
				(ptr->b.posn.y > current->b.posn.y + current->b.height))
		ingroupy = 1;
	if (ingroupx && ingroupy)
		return(1);
	else
		return(0);
}
