/*
 *    F I L E P R O C S . C -- Contains the file routines for kmrcmenu.c
 *
 *
 *  Written: Sarah E. A. Burton.
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    8th January 1988.
 *
 *  Modifications
 *
 *  24 Apr 1990     IH		Changed OD_arch_method to arch_method
 *   9 Apr 1990     IH      Removed all references to old colourdef struct
 *   9 Apr 1990     IH      Added archive media type check!! (should have been here already)
 *   3 Apr 1990     IH      Added include for new file (allposs.h)
 *	28 Mar 1990     IH      Added lutfile option for ordinary seats
 *	12 Dec 1989		GJP		Added menu_option
 *	26 Oct 1989		IH		Added archive, satellite and process priority opts
 *	19 Jul 1989		CAS		Errno def for v2.2
 *  22 May 1989		SEAJ	Added timing and made matrixfile varenhance.
 *  15 May 1989		SEAJ	Added cultures file.
 *  10 May 1989		SEAJ	Added fnamesep1 and 2 for filename separators, and
 *							DIR300_OBJ for laser3.
 *	20 Mar 1989		SEAJ	Added matrixfile for variable enhancement.
 *	31 Jan 1989		SEAJ	Made monitor be a string, added frontend, stagetype, 
 *							and seat<n>camera.
 *   7 Dec 1988		SEAJ	Added woolz checking level.
 *  17 Oct 1988		SEAJ	Correct bug in seat config. 
 *  23 Sep 1988		SEAJ	Added screen/hardcopy filename stuff.
 *  19 Aug 1988		SEAJ	Changes seat configuration structure and added
 *							for and back ground colours
 *  17 Aug 1988		SEAJ	If seat1config = FIP switch is_CS on.
 *  15 May 1988		SEAJ	Added in 'non-removeable' comments stuff.
 *  13 May 1988		SEAJ	Tidied up buildfile & final file layout.
 *   8 Apr 1988		SEAJ	Added in all transfer summary stuff.
 *   8 Apr 1988		SEAJ	Added in combifip version and check.
 *
 */
#include <stdio.h>
#include <ctype.h>
#include <kmrcmenu.h>
#include <allposs.h>

extern char *hardcopytype[];
extern char *languagetype[];
extern char *monitortype[];
extern char *termtype[];
extern char *fstoretype[];
extern char *configtype[];
extern char *cameratype[];
extern char *archmediatype[];
extern char *fronttype[];
extern char *seattype[];
extern char *stropt[];
extern char *menuopt[];
extern char *forcols[];
extern char *backcols[];
extern char *weekdays[];
extern char *months[];
extern char *allposs[];

extern struct allcontrol;
extern struct sysdefinition;
extern struct seatdefinition;
extern struct date;
extern int errno;

static char *getp2();
static int where = 1;
int getdate(), writecomms();

/*
 *  F I L E R E A D -- Opens file, processes data using process, closes file.
 *
 */
fileread(name, whole)
char *name;
struct allcontrol *whole;
{
	FILE *rc;
	char buffer[BUFLEN], *value; 
	struct comment *ptr;
	
	if ((rc = fopen(name, "r")) != NULL){
		while (fgets(buffer, BUFLEN, rc) != NULL){
			if (*buffer != '*'){
				value = getp2(buffer, strlen(buffer), 0);
				if (*buffer)
					process(buffer, value, whole);
			} else {
				if (buffer[1] == '*') {
					if (whole->comptr == NULL) {
						whole->comptr = (struct comment *) Calloc(1, sizeof(struct comment));
						whole->comptr->current = whole->comptr;
					} else {
						whole->comptr->current->nextcom = (struct comment *) Calloc(1, sizeof(struct comment));
						whole->comptr->current = whole->comptr->current->nextcom;
					}
					whole->comptr->current->nextcom = NULL;
					whole->comptr->current->area = where;
					strcpy( whole->comptr->current->comstr, buffer);
				}
			}
		}
		fclose(rc);
	}
	else
		printf("\nError on opening file\n");
}

/*
 *	G E T P 2  --  Get pointer to parameter 
 *					if 'equ' false - skip down line till we find
 *					'=' on line else just skip to next word.
 *
 */
static char *
getp2(ptr,len,equ)
char	*ptr;
int		len, equ;
{
	int		inword, seeneq, instring;
	char	*retptr = NULL;
	static char	*endptr;

	if (equ) {
		inword = 1;
		seeneq = 1;
	} else {
		inword = 0;
		seeneq = 0;
		endptr = ptr + len;
	}
	instring = 0;
	while (ptr <= endptr) {
		switch (*ptr) {
		case '=':
			if (instring)
				break;
			seeneq = 1;
			inword = 0;
 			*ptr = '\0';
			break;
		case ' ':
		case '\t':
			if (instring)
				break;
		case '\n':
		case '\0':
			if (inword) {
				*ptr = '\0';
				if (retptr)
					return(retptr);
				else
					instring = 0;
			}
			inword = 0;
			break;
		case '"':
			if (seeneq) {
				if (inword) {
					instring = 1;
				} else if (instring) {
					*ptr = '\0';		/* Kill trailing '"' */
					inword = 1;			/* and wait for delimiter */
					instring = 0;
				} else {
					instring = 1;
					retptr = ptr + 1;
				}
			}
			break;
		default:
			if (instring)
				break;
			if (!inword) {
				inword = 1;
				if (seeneq && !retptr) {
					retptr = ptr;
				}
			}
		}
		ptr++;
	}
	return((*retptr) ? retptr : endptr);
}

/*
 * P R O C E S S -- Takes in the buffer, processes it and stores the relevant 
 *					information.
 */
process(ptr, val, whole)
char	*ptr, *val;
struct allcontrol *whole;
{
#define DEBUG 0 		/* QD debug */

	int i, j, count;
	char *p2, *p3, *p4, *ps;
	char character;
	int number, seatno, typeseat, n[MAXPROCS];
	
	if (DEBUG) 
		fprintf(stderr,"processing variable %s with value %s\n",ptr,val);
	i = 0;
	while (i<MAXPOSS) {
		if (strcmp(ptr, allposs[i]) == 0)
			break;
		else
			i++;
	}
	if (DEBUG) 
		fprintf(stderr,"index is %d\n",i);
	switch(i) {
		case 1 :			/* sitename */
			strcpy(whole->sysdef.name, val);
			where = 1;
			break;
		case 2 : 			/* language */
			whole->sysdef.lang = atoi(val);
			where = 1;
			break;
		case 3 :			/* hardcopy */
			j = 0;
			while (strcmp(val, hardcopytype[j]) != 0 && j <= MAXCOPY)
				j++;
			if (j <= MAXCOPY)
				whole->sysdef.copy = j;
			where = 1;
			break;
		case 4 :			/* scoremu */
			if (strcmp(val, "extended") == 0)
				whole->sysdef.score = 1;
				where = 1;
			break;
		case 5 : 				/* needs pulling out */
			break;
		case 6 :				/* needs pulling out */
			break;
		case 7 : 			/* miprocterm */
			i = 0;
			while (strcmp(val, termtype[i]) != 0 && i <= MAXTYPE)
				i++;
			whole->termdef.miprocterm = i;
			whole->is_CS = 1;
			where = 1;
			break;
		case 8 : 			/* miprocbaud */
			whole->termdef.miprocbaud = atoi(val);
			whole->is_CS = 1;
			where = 1;
			break;
		case 9 : 			/* printerterm */
			i = 0;
			while (strcmp(val, termtype[i]) != 0 && i <= MAXTYPE)
				i++;
			whole->termdef.printer = i;
			where = 1;
			break;
		case 10 : 			/* karycolour1 */
			break;
		case 11 :			/* karycolour2 */
			break;
		case 12 :			/* metacolour1 */
			break;
		case 13 :			/* metacolour2 */
			break;		
		case 14 :			/* mousescale */
			whole->sysdef.mousescale = atoi(val);
			where = 3;
			break;
		case 15 :			/* devel */
			whole->debugdef.devel = atoi(val);
			where = 1;
			break;
		case 16 :			/* debug */
			whole->debugdef.debug = atoi(val);
			where = 1;
			break;
		case 17 :			/* debugfile */
			strcpy(whole->debugdef.debugfile, val);
			where = 1;
			break;
		case 18 :			/* mipdebug */
			whole->debugdef.mipdebug = atoi(val);
			where = 1;
			break;
		case 19 :			/* monitor */
			j = 0;
			while(strcmp(val, monitortype[j]) != 0 && j < MAXMON)
				j++;
			if (j < MAXMON)
				whole->debugdef.monitor = j;
			where = 1;
			break;
		case 20 :			/* usergroupnum */
			whole->userdef.usergroupnum = atoi(val);
			where = 3;
			break;
		case 21 :			/* onegroup */
			whole->userdef.onegroup = atoi(val);
			where = 3;
			break;
		case 22 :			/* focus */
			p2 = getp2(val, 0, 1);
			p3 = getp2(p2, 0, 1);
			p4 = getp2(p3, 0, 1);
			whole->cs.speed.focus.permove = atoi(val);
			whole->cs.speed.focus.inswathe = atoi(p2);
			whole->cs.speed.focus.between = atoi(p3);
			whole->cs.speed.focus.excursion = atoi(p4);
			whole->cs.speed.focen = ENABLE;
			whole->is_CS = 1;
			where = 4;
			break;
		case 23 :			/* movespeed */
			whole->cs.speed.movespeed = atoi(val);
			whole->cs.speed.moven = ENABLE;
			whole->is_CS = 1;
			where = 4;
			break;
		case 24 :			/* scanspeed */
			whole->cs.speed.scanspeed = atoi(val);
			whole->cs.speed.scanen = ENABLE;
			whole->is_CS = 1;
			where = 4;
			break;
		case 25 :			/* metthresh */
			p2 = getp2(val, 0,1);
			whole->cs.speed.metthresh.low = atoi(val);
			whole->cs.speed.metthresh.high = atoi(p2);
			whole->cs.speed.thresen = ENABLE;
			whole->is_CS = 1;
			where = 4;
			break;
		case 26 :			/* nbays */
			whole->cs.nbays = atoi(val);
			if ( whole->cs.nbays > 1)
				whole->is_CS = 1;
			where = 5;
			break;
		case 27 :			/* scanleft */
			whole->cs.sleft = atoi(val);
			where = 5;
			break;
		case 28 :			/* scanright */
			whole->cs.sright = atoi(val);
			where = 5;
			break;
		case 29 :			/* transline */
			i = 0;
			while (strcmp(val, termtype[i]) != 0 && i <= MAXTYPE)
				i++;
			whole->trans.line = i;
			whole->trans.enable = 1;
			p2 = getp2(val, 0, 1);
			whole->trans.baud = atoi(p2);
			where = 4;
			break;
		case 30 :			/* transstart */
			strcpy(whole->trans.startup, val);
			whole->trans.enable = 1;
			where = 4;
			break;
		case 31 :			/* transend */
			strcpy(whole->trans.closedown, val);
			whole->trans.enable = 1;
			where = 4;
			break;
		case 32 :			/* combifip */
			strcpy(whole->cs.program, val);
			where = 4;
			break;
		case 33 :			/* cfipcheck */
			strcpy(whole->cs.check, val);
			where = 4;
			break;
		case 34 :			/* options */
			ps = val;
			count = 0;
			while (*ps != '\0' && count < MAXOPTS) {
				j = 0;
				while(strcmp(ps, stropt[j]) != 0 && j <= MAXOPTS)
					j++;
				if (j > 0 && j <= MAXOPTS)
				 	whole->featdef.options[j-1] = 1;
				ps = getp2(ps, 0, 1);
				count++;
			}
			where = 3;
			break;
		case 35 :			/* screenform */
			strcpy(whole->featdef.screenform, val);
			where = 3;
			break;
		case 36 :
			strcpy(whole->featdef.hardcopyform, val);
			where = 3;
			break;
		case 37 :			/* woolz checking level */
			whole->debugdef.wclevel = atoi(val);
			where = 1;
			break;
		case 38 :			/* front end options */
			ps = val;
			count = 0;
			while (*ps != '\0' && count < MAXFRONT) {
				j = 0;
				while(strcmp(ps, fronttype[j]) != 0 && j <= MAXFRONT)
					j++;
				if (j < MAXFRONT)
				 	whole->cs.frontend[j] = 1;
				ps = getp2(ps, 0, 1);
				count++;
			}
			where = 4;
			break;
		case 39 :			/* stage type */
			whole->cs.stagetype = atoi(val);
			where = 4;
			break;
		case 40 :			/* matrix file for variable enhancement */
			strcpy(whole->featdef.varenhance, val);
			where = 3;
			break;
		case 41 :			/* filename separator 1 */
			strcpy(whole->featdef.fnamesep1, val);
			where = 3;
			break;
		case 42 :			/* filename separator 2 */
			strcpy(whole->featdef.fnamesep2, val);
			where = 3;
			break;
		case 43 :			/* classifiers cultures file */
			strcpy(whole->featdef.culturesfile, val);
			where = 3;
			break;
		case 44 :			/* timing debug information */
			whole->debugdef.timing = atoi(val);
			where = 1;
			break;
		case 45 :			/* master priority */
			whole->sysdef.master_priority = atoi(val);
			where = 1;
			break;
		case 46 :			/* slave priority */
			whole->sysdef.slave_priority = atoi(val);
			where = 1;
			break;
		case 47 :			/* sat priority */
			whole->sysdef.sat_priority = atoi(val);
			where = 1;
			break;
		case 48 :
			whole->sysdef.odcache = atoi(val);
			where = 1;
			break;
		case 49:            /* OD archive method */
			strcpy(whole->sysdef.od_arch_method,val);
			break;
		case 50:			/* archive media */
			ps = val;
			count = 0;
			while (*ps != '\0' && count < MAXARCH) {
				j = 0;
				while(strcmp(ps, archmediatype[j]) != 0 && j < MAXARCH)
					j++;
				if (j < MAXARCH)
				 	whole->sysdef.archmedia[j] = 1;
				ps = getp2(ps, 0, 1);
				count++;
			}
			break;
		case 51 :			/* options */
			ps = val;
			count = 0;
			while (*ps != '\0' && count < MAXMENUOPT) {
				j = 0;
				while(strcmp(ps, menuopt[j]) != 0 && j <= MAXMENUOPT)
					j++;
				if (j > 0 && j <= MAXMENUOPT)
				 	whole->featdef.menu_options[j-1] = 1;
				ps = getp2(ps, 0, 1);
				count++;
			}
			where = 3;
			break;
		case 52 :			/* needs pulling out */
			break;
		case 53 :			/* background colour */
			j = 0;
			ps = val;
			while ((*ps != '\0') && (j<3)) {
				whole->sysdef.bgcolour[j] = atoi(ps);
				ps = getp2(ps,0,1);
				j++;
			}
			break;
		case 54:
			ps=val;
			if ((*ps=='0')&&(*(ps+1)=='x'))
				whole->cs.piabase=axtoi(ps+2);
			else
				whole->cs.piabase=atoi(ps);
			break;
		
	}
	/* Seat section */
	if (strncmp(ptr, "seat", 4) == 0) {
		where = 2;
 		i = 0;
		if(strncmp(ptr+4,"Sstation",8) == 0) {
			typeseat = NET;
			if (DEBUG) printf("processing station with arguments %s\n",ptr);
			character = *(ptr+12);
			seatno = character - '0';
			if (seatno<0) seatno = 0;
			if (seatno>MAXNETWORK-1) seatno=MAXNETWORK-1;
			ptr += 13;
		}
		else {
			character = *(ptr+4);
			ptr += 5;
			if (character == 'S') {
				if (DEBUG) printf("processing satellite with arguments %s\n",ptr);
				typeseat = SAT;
			}
			else {
				typeseat = SEAT;
				if (DEBUG) printf("processing seat with arguments %s\n",ptr);
				seatno = character - '0';
				if (seatno<1) seatno=1;
				if (seatno>MAXSEATS) seatno=MAXSEATS;
			}
		}
		while(strcmp(ptr, seattype[i]) != 0 && i<=MAXSEATTYPE)
			i++;
		switch(i) {
			case 1 : 			/* terminal type */
				j = 0;
				while(strcmp(val, termtype[j]) != 0 && j<=MAXTYPE)
					j++;
				if (j <= MAXTYPE) {
					switch (typeseat) {
						case SEAT:
							whole->seatdef[seatno-1].valid = 1;
							whole->seatdef[seatno-1].term = j;
							break;
						case SAT:
							whole->satseatdef.valid = 1;
							whole->satseatdef.term = j;
							break;
						default:
							break;
					}
				}
				break;
			case 2 : 			/* configuration type */
				ps = val;
				count = 0;
				while (*ps != '\0' && count<MAXCONF) {
					j = 0;
					while(strcmp(ps, configtype[j]) != 0 && j<MAXCONF)
						j++;
					if (j<MAXCONF) {
						switch (typeseat) {
							case SEAT:
						 		whole->seatdef[seatno-1].valid = 1;
						 		whole->seatdef[seatno-1].config[j] = 1;
								break;
							default:
								break;
						}
					}
					ps = getp2(ps, 0, 1);
					count++;
				}
				if ((seatno == 1) && (j == 1) && (typeseat == SEAT)) 
					whole->is_CS = 1;
				break;
			case 3 : 			/* mouse terminal */
				j = 0;
				while ((strcmp(val,termtype[j]) != 0) && (j<=MAXTYPE))
					j++;
				p2 = getp2(val, 0, 1);
				if (isdigit(*p2))
					number = atoi(p2);
				else
					number = -1;
				if ((j <= MAXTYPE) && (typeseat == SEAT)) {
					whole->seatdef[seatno-1].valid = 1;
					whole->seatdef[seatno-1].mouse = j;
					whole->seatdef[seatno-1].mproc = number;
				};
				break;
			case 4 : 			/* fstore */
				j = 0;
				p2 = getp2(val, 0, 1);
				while (strcmp(p2, fstoretype[j]) != 0 && j<=MAXFST)
					j++;
				if (j <= MAXFST){
					switch (typeseat) {
						case SEAT:
							whole->seatdef[seatno-1].valid = 1;
							whole->seatdef[seatno-1].fstore.number = atoi(val);
							whole->seatdef[seatno-1].fstore.string = j;
							break;
						case SAT:
							whole->satseatdef.valid = 1;
							whole->satseatdef.fstore.number = atoi(val);
							whole->satseatdef.fstore.string = j;
							break;
						default:
							break;
					}
				}
				break;
			case 5 :			/* slave */
				ps = val;
				for (j=0; j<MAXPROCS; j++)
					n[j] = -1;
				j = 0;
				character = *ps;
				while (isdigit(character) && j<MAXPROCS) {
					n[j] = atoi(ps);
					j++;
					ps = getp2(ps, 0, 1);
					character = *ps;
				};
				if (typeseat == SEAT) {
					whole->seatdef[seatno-1].valid = 1;
					for (j=0; j<MAXPROCS; j++)
						whole->seatdef[seatno-1].slave[j] = n[j];
				}
				break;
			case 6 : 			/* camera type */
				ps = val;
				count = 0;
				while (*ps != '\0' && count < MAXCAM) {
					j = 0;
					while(strcmp(ps, cameratype[j]) != 0 && j < MAXCAM)
						j++;
					if (j < MAXCAM) {
						switch (typeseat) {
							case SEAT:
					 			whole->seatdef[seatno-1].valid = 1;
					 			whole->seatdef[seatno-1].camera[j] = 1;
					 			break;
							case SAT:
					 			whole->satseatdef.valid = 1;
					 			whole->satseatdef.camera[j] = 1;
					 			break;
							case NET:
					 			whole->netseatdef[seatno].valid = 1;
					 			whole->satseatdef.net = 1;
					 			whole->netseatdef[seatno].camera[j] = 1;
					 			break;
					 	}
					}
					ps = getp2(ps, 0, 1);
					count++;
				}
				if ((seatno == 1) && (j == 1) && (typeseat == SEAT)) 
					whole->is_CS = 1;
				break;
			case 7 :			/* lut file */
				switch (typeseat) {
					case SEAT:
						whole->seatdef[seatno-1].valid = 1;
						strcpy(whole->seatdef[seatno-1].lutfile,val);
						break;
					case SAT:
						whole->satseatdef.valid = 1;
						strcpy(whole->satseatdef.lutfile,val);
						break;
					case NET:
						whole->netseatdef[seatno-1].valid = 1;
						whole->satseatdef.net = 1;						
						strcpy(whole->netseatdef[seatno].lutfile,val);
						break;
				}
				break;
		}		/* switch end */
	}
	/* bay section */
	if (strncmp(ptr, "bay", 3) == 0) {
		where = 5;
		character = *(ptr+3);
		number = character - '0';
		character = *(ptr+4);
		if (character == 'x')
			whole->cs.bay[number-1].x = atoi(val);
		if (character == 'y')
			whole->cs.bay[number-1].y = atoi(val);
		if (character == 'z')
			whole->cs.bay[number-1].z = atoi(val);
	}
	/* archive section */
	if (strncmp(ptr, "archive", 7) == 0) {
		ps = val;
		count = 0;
		while (*ps != '\0' && count < MAXARCH) {
			j = 0;
			while(strcmp(ps, archmediatype[j]) != 0 && j < MAXARCH)
				j++;
			if (j < MAXARCH)
	 			whole->sysdef.archmedia[j] = 1;
			ps = getp2(ps, 0, 1);
			count++;
		}
	}
	if (strncmp(ptr, "cachesize", 9) == 0)
		whole->sysdef.odcache = atoi(val);
	if (strncmp(ptr,"arch_method",14)==0) {
		strcpy(whole->sysdef.od_arch_method,val);
	}
}

/*
 * F I L E W R I T E -- Opens the file, constructs the file using buildfile
 *						then closes the file.
 */
filewrite(name, whole)
char *name;
struct allcontrol *whole;
{
	FILE *rc;
	int user, fd, error;
	
	error = 0;
	user = 0;
	if (strcmp(name, SYS_RC) != 0)
		user = 1;
	if ((fd = open(name, 2)) == -1) {
		if (user != 1)
			fd = create(name, 2, 0x0b);
		else
			fd = create(name, 2, 0x1b);
		if (fd == -1) {
			printf("\nERROR on trying to create file.\n");
			error = 1;
		} else 
			close(fd);
	} else
		close (fd);
	if (error != 1) {
		printf("The file is now being written.\n");
		if ((rc = fopen(name, "w")) != NULL)
			buildfile(rc, whole, user);
		else{
			printf("ERROR opening file.\n");
			printf("The error is %d\n",errno);
		}
		fclose(rc);
	}
}

/*
 * B U I L D F I L E -- Constructs the file.
 * 
 */
buildfile(rc, whole, user)
FILE *rc;
struct allcontrol *whole;
int user;
{
 	struct date today;
 	int thisuser, thisgroup, i, j;
 	char str[MAXSTRING], *ptr;
	char flag = 0;
 	 	
 	thisuser = getuid();
 	thisgroup = thisuser >> 16;
 	thisuser = thisuser & 0xFFFF;
	getdate(&today);
	if ( user == 0)
		fprintf(rc, "*\n*  CYTOSCAN SYSTEM CONFIGURATION FILE \n");
	else
		fprintf(rc, "*\n* PERSONNAL SYSTEM CONFIGURATION FILE \n");
	fprintf(rc, "*  written on %s %d %s %d\n",today.day,today.daynum,today.month,today.year );	
	fprintf(rc, "*  at %d : %d by group %d, user %d\n*\n",today.hour,today.minute,
														 thisgroup, thisuser);
	if (whole->seatdef[SEAT1].valid && whole->seatdef[SEAT2].valid)
		if (whole->seatdef[SEAT1].config[0] == 1)  			/* FIP on */
			if (whole->seatdef[SEAT2].config[1] == 1)		/* TV on seat 2 */
				fprintf(rc, "*    Configuration CS2 - ");
			else
				fprintf(rc, "*    Configuration CS - ");
		else
			fprintf(rc, "*    Configuration RK2 - ");
	else if (whole->seatdef[SEAT1].config[0] == 1)			/* FIP on */
		fprintf(rc, "*    Configuration DF - ");
	else
		fprintf(rc, "*    Configuration RK - ");
	if (whole->seatdef[SEAT1].fstore.string > 1)
		fprintf(rc, "Virtuoso, ");
	else
		fprintf(rc, "CRS, ");
	if (whole->sysdef.copy > 1)
		fprintf(rc, "laserprint, ");
	else
		fprintf(rc, "honeywell, ");
	i=0;
	for (j=0;j<MAXSEATS;j++)
		if (whole->seatdef[j].valid)
			i++;
	fprintf(rc, "\n* %d Seat System",i);
	i=0;
	for (j=0;j<MAXSEATS;j++)
		if (whole->netseatdef[j].valid)
			i++;
	if (i)
		fprintf(rc, "* %d Seat Satellite Network System \n",i);
	else {
		if (whole->satseatdef.valid)
			fprintf(rc, "\n* Satellite System \n");
	}
	fprintf(rc, "*\n");
	
	if ( user == 0){
		fprintf(rc, "sitename = \"%s\"\n",whole->sysdef.name);
	 	fprintf(rc, "language = %d\t* %s\n",whole->sysdef.lang, languagetype[whole->sysdef.lang]);
		fprintf(rc, "hardcopy = %s\n",hardcopytype[whole->sysdef.copy]);
		if (whole->sysdef.score == 1)
			fprintf(rc, "scoremu = extended\n");
		fprintf(rc,"\n");
	}

	fprintf(rc, "* Debug Options \n*\n");
	if (whole->debugdef.devel != 0)
		fprintf(rc, "devel = %d\n",whole->debugdef.devel);
	if (whole->debugdef.debug != 0)
		fprintf(rc, "debug = %d\n",whole->debugdef.debug);
	fprintf(rc, "debugfile = %s\n",whole->debugdef.debugfile);
	if (whole->debugdef.wclevel != 0)
		fprintf(rc, "wclevel = %d\n", whole->debugdef.wclevel);
	if (whole->debugdef.timing != 0)
		fprintf(rc, "timing = %d\n", whole->debugdef.timing);
	if (whole->debugdef.mipdebug != 0)
		fprintf(rc, "mipdebug = %d\n",whole->debugdef.mipdebug);
	if (whole->debugdef.monitor != 0)
		fprintf(rc, "monitor = %s\n",monitortype[whole->debugdef.monitor]);
	
	if (user == 0 && whole->is_CS == 1) {
		fprintf(rc, "*\n* Miproc and Printer Definitions \n*\n");
		fprintf(rc, "miprocterm = %s      * Miproc line\n",termtype[whole->termdef.miprocterm]);
		fprintf(rc, "miprocbaud = %d\n",whole->termdef.miprocbaud);
		fprintf(rc, "printerterm = %s\n*\n",termtype[whole->termdef.printer]);
	};
	writecomms(whole, rc, 1);
	if (user == 0) {
		for (j = 0;j<MAXSEATS;j++) {
			if (whole->seatdef[j].valid) {
				fprintf(rc, "*\n* Seat %d \n*\n", j+1);
				fprintf(rc, "seat%dterm = %s\n", j+1,
				             termtype[whole->seatdef[j].term]);
				if ( whole->seatdef[j].mproc != -1)
					fprintf(rc, "seat%dmouse = %s %d\n", j+1,
					termtype[whole->seatdef[j].mouse],whole->seatdef[j].mproc);
				else
					fprintf(rc, "seat%dmouse = %s\n", j+1, termtype[whole->seatdef[j].mouse]);
				fprintf(rc, "seat%dfstore = %d %s\n", j+1, whole->seatdef[j].fstore.number,
							fstoretype[whole->seatdef[j].fstore.string]);
				*str = '\0';
				ptr = str;
				for (i=0; i < MAXCONF; i++) {
					if (whole->seatdef[j].config[i] == 1) {
						sprintf(ptr, " %s", configtype[i]);
						ptr += strlen(ptr);
					}
				}
				*ptr = '\0';
				if (*str != '\0')
					fprintf(rc, "seat%dconfig =%s\n", j+1, str);
				*str = '\0';
				ptr = str;
				for (i=0; i < MAXCAM; i++) {
					if (whole->seatdef[j].camera[i] == 1) {
						sprintf(ptr, " %s", cameratype[i]);
						ptr += strlen(ptr);
					}
				}
				*ptr = '\0';
				if (*str != '\0')
					fprintf(rc, "seat%dcamera =%s\n", j+1, str);
				if ( whole->seatdef[j].slave[0] != -1) {
					fprintf(rc, "seat%dslave = ", j+1);
					for (i=0; i<MAXPROCS; i++) {
						if ( whole->seatdef[j].slave[i] != -1)
							fprintf(rc, "%d ",whole->seatdef[j].slave[i]);
					}
					fprintf(rc, "\n");
				}
				if (whole->seatdef[j].lutfile[0] != '\0')
					fprintf(rc, "seat%dlut =%s\n",j+1, whole->seatdef[j].lutfile);
			}
		}
		if (whole->satseatdef.valid) {
			if (whole->satseatdef.net)
				fprintf(rc, "*\n*Satellite Network Section\n*\n");
			else
				fprintf(rc, "*\n* Satellite Seat Section\n*\n");
			if (whole->satseatdef.net)
				fprintf(rc, "seatSterm = %s\n","/pipe");
			else
				fprintf(rc, "seatSterm = %s\n",
			                termtype[whole->satseatdef.term]);
			fprintf(rc, "seatSfstore = %d %s\n", whole->satseatdef.fstore.number,
						fstoretype[whole->satseatdef.fstore.string]);
			writecomms(whole, rc, 2);
			*str = '\0';
			ptr = str;
			for (i=0; i < MAXCAM; i++) {
				if (whole->satseatdef.camera[i] == 1) {
					sprintf(ptr, " %s", cameratype[i]);
					ptr += strlen(ptr);
				}
			}
			*ptr = '\0';
			if (*str != '\0')
				fprintf(rc, "seatScamera =%s\n", str);
			fprintf(rc,"seatSconfig = TV SAT\n");
			if (whole->satseatdef.lutfile[0] != '\0')
				fprintf(rc, "seatSlut = %s\n",whole->satseatdef.lutfile);
			for (i=0;i<MAXNETWORK;i++) {
				if (whole->netseatdef[i].valid) {
					ptr = str;
					*ptr = '\0';
					for (j=0; j < MAXCAM; j++) {
						if (whole->netseatdef[i].camera[j] == 1) {
							sprintf(ptr, " %s", cameratype[j]);
							ptr += strlen(ptr);
						}
					}
					if (*str != '\0')
						fprintf(rc, "seatSstation%dcamera =%s\n", i, str);
					if (whole->netseatdef[i].lutfile[0] != '\0') {
						fprintf(rc, "seatSstation%dlut = %s\n",
			            		    i, whole->netseatdef[i].lutfile);
					}
				}
			}
		}
		fprintf(rc, "*\n* Login/user generation \n*\n");
		fprintf(rc, "usergroupnum = %d\n",whole->userdef.usergroupnum);
		fprintf(rc, "onegroup = %d\n*\n",whole->userdef.onegroup);
		fprintf(rc, "*\n* Process Priorities \n*\n");
		fprintf(rc, "master_priority = %d\n", whole->sysdef.master_priority);
		fprintf(rc, "slave_priority = %d\n", whole->sysdef.slave_priority);
		fprintf(rc, "sat_priority = %d\n", whole->sysdef.sat_priority);
		*str = '\0';
		ptr = str;
		for (i=0; i < MAXARCH; i++) {
			if (whole->sysdef.archmedia[i] == 1) {
				sprintf(ptr, " %s", archmediatype[i]);
				ptr += strlen(ptr);
			}
		}
		*ptr = '\0';
		if (*str != '\0') {
			fprintf(rc, "*\n* Archive Stuff \n*\n");
			fprintf(rc, "archive =%s\n", str);
		}
		if (whole->sysdef.archmedia[1]) {
			fprintf(rc, "cachesize = %d\n", whole->sysdef.odcache);
			if (strncmp(whole->sysdef.od_arch_method,"CASE",4) == 0)
				fprintf(rc, "arch_method = \"%s\"\n",whole->sysdef.od_arch_method);
			else if (strlen(whole->sysdef.od_arch_method) != 0)
				fprintf(rc, "arch_method = %s\n",whole->sysdef.od_arch_method);
			else
				fprintf(rc, "\n");
		}

	};
	fprintf(rc, "*\n* Background Colour\n*\nbgcolour = ");
	for (i=0;i<3;i++) {
		fprintf(rc,"%d ",whole->sysdef.bgcolour[i]);
	}
	fprintf(rc,"\n");
	fprintf(rc,"*\n* Mouse Scale\n*\n");
	fprintf(rc, "mousescale = %d\n*\n",whole->sysdef.mousescale);
	writecomms(whole, rc, 3);
	
	*str = '\0';
	ptr = str;
	for (i=0; i < MAXOPTS; i++) {
		if (whole->featdef.options[i] == 1) {
			sprintf(ptr, " %s", stropt[i+1]);
			ptr += strlen(ptr);
		}
	}
	*ptr = '\0';
	if (*str != '\0')
		fprintf(rc, "options =%s\n", str);
	
	*str = '\0';
	ptr = str;
	for (i=0; i < MAXMENUOPT; i++) {
		if (whole->featdef.menu_options[i] == 1) {
			sprintf(ptr, " %s", menuopt[i+1]);
			ptr += strlen(ptr);
		}
	}
	*ptr = '\0';
	if (*str != '\0')
		fprintf(rc, "menu_options =%s\n", str);
	if (whole->featdef.screenform[0] != '\0')
		fprintf(rc, "screenform = %s\n", whole->featdef.screenform);
	if (whole->featdef.hardcopyform[0] != '\0')
		fprintf(rc, "hardcopyform = %s\n", whole->featdef.hardcopyform);
	if (whole->featdef.varenhance[0] != '\0')
		fprintf(rc, "varenhance = %s\n", whole->featdef.varenhance);
	if (whole->featdef.culturesfile[0] != '\0')
		fprintf(rc, "culturesfile = %s\n", whole->featdef.culturesfile);
	if (whole->featdef.fnamesep1[0] != '\0')
		fprintf(rc, "fnamesep1 = %s\n", whole->featdef.fnamesep1);
	if (whole->featdef.fnamesep2[0] != '\0')
		fprintf(rc, "fnamesep2 = %s\n", whole->featdef.fnamesep2);
	fprintf(rc, "*\n");
	if (whole->trans.enable != 0) {
		fprintf(rc, "* transfer summary \n");
		if (whole->trans.line != 0) {
			fprintf(rc, "transline = %s %d\n", termtype[whole->trans.line], 
													whole->trans.baud);
		};
		if (strcmp(whole->trans.startup, "") != 0)
			fprintf(rc, "transstart = \"%s\"\n", whole->trans.startup);
		if (strcmp(whole->trans.closedown, "") != 0)
			fprintf(rc, "transend = \"%s\"\n", whole->trans.closedown);
	};
	if (whole->is_CS == 1){
		if (whole->cs.speed.focen != 0) {
			fprintf(rc, "*       focus parameters\n");
			fprintf(rc, "*    permove inswathe between excursion\n");
			fprintf(rc, "focus = %d %d %d %d\n",whole->cs.speed.focus.permove,
				 whole->cs.speed.focus.inswathe, whole->cs.speed.focus.between,
				 whole->cs.speed.focus.excursion);
		};
		if (whole->cs.speed.moven != 0)
			fprintf(rc, "movespeed = %d\n", whole->cs.speed.movespeed);
		if (whole->cs.speed.scanen != 0)
			fprintf(rc, "scanspeed = %d\n", whole->cs.speed.scanspeed);
		if (whole->cs.speed.thresen != 0)
			fprintf(rc, "metthresh = %d %d\n", whole->cs.speed.metthresh.low,
										 whole->cs.speed.metthresh.high);
		if (whole->cs.piabase!=0)
			fprintf(rc,"piabase = 0x%x\n",whole->cs.piabase);
		else
			fprintf(rc,"piabase = 0xfe0800\n");
		if (user == 0) {
			fprintf(rc, "*\n");
			if (strcmp(whole->cs.program, "") != 0)
				fprintf(rc, "combifip = %s\n", whole->cs.program);
			if (strcmp(whole->cs.check, "") != 0)
				fprintf(rc, "cfipcheck = \"%s\"\n", whole->cs.check);
			fprintf(rc, "stagetype = %d\n", whole->cs.stagetype);
			*str = '\0';
			ptr = str;
			for (i=0; i < MAXFRONT; i++) {
				if (whole->cs.frontend[i] == 1) {
					sprintf(ptr, " %s", fronttype[i]);
					ptr += strlen(ptr);
				}
			}
			*ptr = '\0';
			if (*str != '\0')
				fprintf(rc, "frontend =%s\n", str);
			
			writecomms(whole, rc, 4);
			fprintf(rc, "*\n*  Bay information\n*\n");
			fprintf(rc, "nbays = %d\n", whole->cs.nbays);
			fprintf(rc, "scanleft = %d\n", whole->cs.sleft);
			fprintf(rc, "scanright = %d\n", whole->cs.sright);
			if (whole->cs.nbays >= 2){
				for (j = 1; j < whole->cs.nbays; j++) {
					fprintf(rc, "*\nbay%dx = %d\n", j, whole->cs.bay[j-1].x);
					fprintf(rc, "bay%dy = %d\n", j, whole->cs.bay[j-1].y);
					fprintf(rc, "bay%dz = %d\n", j, whole->cs.bay[j-1].z);
				}
			}
			writecomms(whole, rc, 5);
			
		}
	}
}
/*
 * W R I T E C O M M S -- Routine to write out the comments in their area.
 *
 */
writecomms(whole, rc, number)
struct allcontrol *whole;
FILE *rc;
int number;
{
	struct comment *ptr;
	
	ptr = whole->comptr;
	while( ptr != NULL) {
		if (ptr->area == number)
			fprintf(rc, "%s", ptr->comstr);
		ptr = ptr->nextcom;
	}
}
				
/*
 * G E T D A T E -- Gets the date and puts it in the structure date
 * 
 */
getdate(today)
struct date *today;
{
 	int thisdate, time, tick, thismonth; 
 	short day;
 	
 	_sysdate(0, &time, &thisdate, &day, &tick);
 	today->day = weekdays[day];
 	today->daynum = thisdate & 0xFF;
 	thismonth = (thisdate >> 8) & 0xFF;
 	today->month = months[thismonth]; 
 	today->year = thisdate >> 16;
 	today->hour = time >> 16;
 	today->minute = (time >> 8) & 0xFF;
}

