/*
 *			G A M M A . C -- Create gamma curves
 *
 *
 *  Written  David Burgess
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    15th January 1990
 *
 *  Modifications
 *	14 Nov 1990	ih	Use alcalloc to set up space for alc struct
 *	10 Sep 1990 ih		Initialise cmdsys variable output explicitly in slave mode
 *	5  Sep 1990 ih		Added a Calloc(struct hardcopy) for alc->hcpy to point at !!
 *	28 Feb 1990	dcb		Moved some LUT defines to kyy/defs/lut.h
 *						Moved readlutfile(), default_lut(), Freeluts() to
 *						kyy/common/lut_correct.c
 */
#define INIFILE        /* define variables in this link */
#include <cmdsys.h>
#include <procid.h>
#include <kyy.h>
#include <cfipp.h>
#include <ddgs.h>
#include <sgstat.h>
#include <math.h>
#include <qmath.h>
#include <hardcopy.h>

#define TRUE (1)
#define FALSE (0)
#define WHITE 0
#define BLACK 255
#define RED 1
#define BLUE 2
#define GREEN 3
#define MAXGAMMA 15.6
#define MINGAMMA 0

/*
 *	System data structures etc
 */
struct kcontrol *kcont = NULL;

/*
 *	Externals
 */
extern DDGS *ddgsopen();
extern int (*setexception())();
extern int memexitfn();
extern int errno;
extern int _memmins;
extern FILE *outpout;

/*
 *	Files
 */
FILE *debug_channel = NULL;				/* system debug file */
FILE *debugfp;

/*
 *	Globals
 */
int	(*quit_fn)() = NULL;		/* Call this when we get ^c */
int quit=0;
int slave_mode;					/* Forked from another process */
char *gammaid = "0.6";			/* The version number */
struct sgbuf savestdin;
int	store = FS_VIRTUOSO_INVERT, devno = 0;
float step;
char mouseterm[5] = "/t5";

/*
 *	Function forward declarations
 */
int gamma_set(), gamma_init(), dispbanner();
int upgamma(), downgamma(), file_lut();
int change_invert(), change_lut_type(), lut_type_menu(), setgamma(), setfloat();
int greytestbar();
struct lut_assoc *readlutfile(), *default_lut(), *Freeluts();
static int disp_messg();
char *lut_typestr();
static int xo=0, yo=0;	/* Graph x, and y origin */

/*
 *	Static variables
 */
static DDGS *dg;
static int filewritten = 0;
static struct pframe cursframe, *cf;
static struct ipoint cursor;
static int clearflag = FALSE;	/* Clear display in stand-alone mode */

/*
 *	I N T E R R U P T  -- interrupt catcher
 */
interrupt(sig)
int sig;
{
	char c;

	if (quit) return;
	quit = 1;
	if (alc->debug)
		printf("GAMMA - seen sig %d\n",sig);
	if (quit_fn) quit_fn();
	else def_quit_fn();
	quit = 0;
	return;
}

/*
 *	D E F _ Q U I T _ F N -- Default quit function, call this when
 *	we get ctrl-C if no other quit function set up
 */
def_quit_fn()
{
	char c, c1;

	while (_gs_rdy(fileno(stdin)) > 0)
		read(fileno(stdin), &c, 1);	/* Flush input */
	screengo(ENDLINE,0);
	printf("\nQuit seen - Do you wish to stop GAMMA ? "); fflush(stdout);
	while (_gs_rdy(fileno(stdin)) <= 0); /* Wait for char input */
	read(fileno(stdin), &c, 1);
	write(fileno(stdout), &c, 1);
	do {
		read(fileno(stdin), &c1, 1);
		write(fileno(stdout), &c1, 1);
	} while (c1 != '\n');
	if ((c == 'y') || (c == 'Y')) {
		shutdown();
	} else {
		printf("GAMMA not stopped\n");
		putprompt();
	}
} /* end def_quit_fn */

/*
 *	S H U T D O W N  --  Shut all down
 */
static
shutdown()
{
	tidyworld();
	fprintf(stderr, "\nGAMMA: shutdown\n");
	alldone(1);
}

/*
 *	T I D Y W O R L D  --
 */
static
tidyworld()
{
	cook(fileno(stdin), &savestdin);
	system("tmode echo eof=4");
}

/*
 *	E X C E P T I O N  --  Exception traps come here
 *
 */
exception(exc,addr)
int		exc, addr;
{
	procid	pd;
	int		fd, time, date, day, tick;
	char	buffer[100];

	gprdsc(getpid(),&pd);
	sprintf(buffer,"(V%s) Exception %d, addr 0x%x, base 0x%x (offset 0x%x)\n",
		gammaid, exc, addr, pd._pmodul, addr - (int)pd._pmodul);
	errorlog(buffer);
	shutdown();
}

/*
 *	M A I N  --  Gamma main
 */
main(argc,argv, argp)
int argc; char **argv, **argp;
{
	DDGS	*ddgsopen(), *ddgsdup();
	char	*swptr, **targv, **sargv, buf[80];
	int		i, targc;

	_gs_opt(fileno(stdin), &savestdin);	/* Keep a copy for cleanup later */
	intercept(interrupt);
	setexception(exception);
	Alloc_exit(memexitfn);
	targc = argc; targv = argv;

	slave_mode = FALSE;
	argc--; argv++;			/* dump process name */
	while (argc > 0) {
		if (*argv[0] == '-') {
			swptr = *argv;
			*argv = NULL;
			while (*(++swptr)) {
				switch (*swptr) {
				case '?':	print_options();
							exit(0);
				case 'v':	printf("gamma: version %s\n", gammaid);
							exit(0);
				case 's':	slave_mode = TRUE;
							alc = (struct allkontr *)atoi(*++argv);
							if (alc->debug) printf("gamma: alc = 0x%x\n");
							*argv = NULL; argc--;
							break;
				case '1':
				case '2':
				case '3':
				case '4':	devno = *swptr - '1';
							break;
				case 'I':	store = FS_VIRTUOSO;
							break;
				case 'V':	store = FS_VIRTUOSO_INVERT;
							break;
				case 'C':	store = FS_VME512;
							break;
				case 'c':	clearflag = TRUE;
							break;
				case 'm':	argc--;
							argv++;
							strcpy(mouseterm, *argv);
							*argv = NULL;
							break;
				default:	printf("gamma: - unknown option\n");
				}
			}
		}
		argv++; argc--;
	}

/*
 *	Init global structures
 */
 	_memmins = 16384; /* 16K */
	if (!slave_mode) {
		alc=alcalloc();
		sysinit(alc);
		getwd(alc->dir, 40);	/* Set current dir (sysinit ?) */
/*
 *	Start up menu handler
 */
		mouseport(alc->mouseterm);	/* Where we want the mouse to be */
		muini(stdout);				/* Init cmdsys + mouse */
		alc->m2s = mukont->mouss;	/* Get mouse struct from cmdsys */
	}
	output=stdout;					/* Not inized in slave mode ??*/

	set_header_fn(dispbanner);		/* Top line display function */
	pastmu = 0;						/* do recall up-menus on command string */
	uplow = 0;						/* do not identify upper-lower case */

/*
 *	Start up debugging (debugfile.gamma) to debug_channel
 */
	screengo(ENDLINE, 0);
	sprintf(buf, "%s.gamma", alc->debugfile);
 	if (alc->debug) {
 		if (debugfp)
 			debug_channel = debugfp;
 		else {
			fprintf(stderr, "gamma: normal debug on. Opening %s\n",buf);
			if ((debug_channel = fopen(buf, "w")) ){
				fprintf(debug_channel,"gamma: General debug information\n");
 			} else {
				fprintf (stderr,"Failed to open debug file error %d\n" ,errno);
			}
		}
		fflush(debug_channel);
		Alloc_debug(debug_channel);
	}

	if (slave_mode) {
		slavemain(targv[1], gamma_set, gamma_init); /* Process name already dumped */
	} else {
		gamma_init();
		alc->lut = alc->luts = readlutfile();
		gamma_set();
		alldone(0);
	}
}

/*
 *	G A M M A _ I N I T --	Initialise for gamma setup
 */
gamma_init()
{
	if (slave_mode) {
		dg = ddgsdup(alc->ddgschannel);
	} else {
		alc->ddgschannel = dg = ddgsopen(store, devno);
		dinit();
		if (clearflag) clear();
		init_curs();
	}
}

/*
 *	G A M M A _ S E T -- Set gamma factor, and display
 */
gamma_set()
{
	int chg_cmd, set_cmd, dec_cmd, inc_cmd, acc_cmd, inv_cmd, exit_cmd;
	int inv, gtf;
	char gv[30], sv[30], ltype[16], home_menu_name[40];

	if (slave_mode) printf("Gamma setup\n");

/*	setmsv(curs);	*/

	init_display();
	plotaxes();
	sprintf(gv, "%5.2f", (double)(alc->lut->gamma));
	strcpy(sv, "0.1");
	setfloat(sv, &step);
	plotgammacurve(1);
	strcpy(ltype, lut_typestr(alc->lut->type));
	inv = ((alc->lut->status & INVERT) == INVERT);
	gtf = FALSE;

	sprintf(home_menu_name, "Gamma Correction Version %s", gammaid);
	newmenu(home_menu_name);
	chg_cmd = cmmd("correction for ..... ;:k40%-s%f", ltype, lut_type_menu);
	mouse(LONGRB);
	set_cmd = cmmd("set gamma value;:k40%30s%f", gv, setgamma);
	cmmd("step size;:k40%30s%-+d%f", sv, &step, setfloat);
	dec_cmd = cmmd("decrease by step size;%g", downgamma); mouse(LEFTBT);
	inc_cmd = cmmd("increase by step size;%g", upgamma); mouse(RIGHTBT);
	acc_cmd = cmmd("accept and file;%g", file_lut); mouse(MIDBT);
	if (alc->devel) {
		inv_cmd = cmmd("invert values;:k40%T%f", &inv, change_invert);
		mouse(LONGMB);
		cmmd("greyscale test bar;:k40%T%f", &gtf, greytestbar);
	}
	exit_cmd = cmmd("exit"); mouse(LONGLB);
	while (1) {
		if (!thismu() || (nncmd == exit_cmd)) {
			if (!filewritten) {
				if (check("Save correction file (M) ? ")) {
					downmenu();
					file_lut();
					upmenu();
				}
			}
			recalc_luts();
			if (gtf) {
				gtf = FALSE;		/* Flag bar removed */
				greytestbar(&gtf);	/* and remove it */
			}
			break;
		} else if ((nncmd == inc_cmd) || (nncmd == dec_cmd)) {
			sprintf(gv, "%5.2f", (double)(alc->lut->gamma));
			displin(0, set_cmd);
			putprompt();
		} else if (nncmd == chg_cmd) {	/* Change to new LUT */
			sprintf(gv, "%5.2f", (double)(alc->lut->gamma));
			displin(0, set_cmd);
			inv = ((alc->lut->status & INVERT) == INVERT);
			if (alc->devel) displin(0, inv_cmd); /* Option only under devel */
			putprompt();
		}
	}
	alldone(0);
}

/*
 *	S E T G A M M A -- Take gamma value as string, assign to LUT assoc
 *	structure, and use this modified value to alter the gamma curve
 */
setgamma(val)
char *val;
{
	setfloat(val, &(alc->lut->gamma));
	modifygammacurve();
}

/*
 *	U P G A M M A -- Increase current LUT gamma value
 */
upgamma()
{
	alc->lut->gamma += step;
	if (alc->lut->gamma > MAXGAMMA) alc->lut->gamma = MAXGAMMA;
	modifygammacurve();
}

/*
 *	D O W N G A M M A -- Decrease current LUT gamma value
 */
downgamma()
{
	alc->lut->gamma -= step;
	if (alc->lut->gamma < MINGAMMA) alc->lut->gamma = MINGAMMA;
	modifygammacurve();
}

/*
 *	C H A N G E _ I N V E R T -- Invert LUT sense
 */
change_invert(flag)
int *flag;
{
	plotgammacurve(0);					/* Erase old curve */
	if (*flag)
		alc->lut->status |= INVERT;		/* Set it */
	else
		alc->lut->status &= ~INVERT;	/* Clear it */
	invert_lut();						/* Invert LUT values */
	plotgammacurve(1);					/* Plot new */
	apply_mono_lut();					/* & apply it */
}

/*
 *	G R E Y T E S T B A R -- Draw grey scale test wedge at bottom of screen
 */
greytestbar(mode)
int *mode;
{
	char *wsave();
	static char *win=NULL;
	register int xp, i;

	xp = alc->ddgschannel->xcenter/2;
	moveto(xp, 300);

	if (*mode) {
		win = wsave(alc->ddgschannel->xcenter + 20, 320);
		colour(GREYCOLS);
		moveto(xp, 300);
		pixbegin(8, 256, 1, 12, 300);
		for (i=0; i<256; i++) pixel(i);
		pixend();
		colour(BLUE);
		lwidth(2);
		moveto(2*xp, 450);
		box(2*xp - 2, 290);
	} else {
		moveto(xp, 300);
		wrestore(win, 0);
	}
}

/*
 *	M O D I F Y G A M M A C U R V E --	Erase old gamma curve,
 *										and calculate & draw new one
 */
modifygammacurve()
{
	plotgammacurve(0);	/* Erase the old one */
	calcgammacurve(4);	/* Roughly calculate the new one */
	plotgammacurve(1);	/* Plot the new curve */
	apply_mono_lut();	/* Apply the green lut */
}

/*
 *	F I L E _ L U T -- File generated LUT
 */
file_lut()
{
	int quit_cmd, file_cmd, cmd, i;

	newmenu("Create gamma correction file");
	cmmd("output file name;:k40%40s%-f", alc->lutfile, disp_messg);
	file_cmd = cmmd("file lut"); mouse(MIDBT);
	quit_cmd = cmmd("quit"); mouse(LONGLB);

/*
 *	Calculate all LUTS with greater accuracy
 */
	recalc_luts();
 	while((cmd = thismu()) != quit_cmd) {
		if ((cmd == 0) || (nncmd == file_cmd)) {
			if (check_lut_type(UNDEF_LUT)) {
				fprintf(stderr, "Please set correction type first\n");
				break;
			}
			writelutfile(alc->lutfile);
			filewritten = 1;
			break;
		}
	}
} /* end file_lut */

/*
 *	D I S P _ M E S S G  --  Display message telling user to press return
 *			
 */
static int
disp_messg()
{
	screengo(OUTLINE - 1,0);
	printf("Press <Return> to write file\n");
}

/*
 *	L U T _ T Y P E _ M E N U -- Put up menu of correction types & select one
 */
lut_type_menu(typestr)
char *typestr;
{
	newmenu("Select correction type");
	cmmd("brightfield;%+i%-+s%f", BRIGHT_LUT, typestr, change_lut_type); mouse(MIDBT);
	cmmd("fluorescence;%+i%-+s%f", FLUOR_LUT, typestr, change_lut_type); mouse(LEFTBT);
	cmmd("special;%+i%-+s%f", SPEC_LUT, typestr, change_lut_type); mouse(RIGHTBT);
	cmmd("unchanged;:k30%-s", typestr); mouse(LONGLB);
	thismu();
}

/*
 *	C H A N G E _ L U T _ T Y P E -- Change the LUT & plot new etc
 */
change_lut_type(type, typestr)
int type;
char *typestr;
{
	struct lut_assoc *lut;

	plotgammacurve(0);						/* Erase old curve */
	strcpy(typestr, lut_typestr(type));	/* Asign new string */
/*
 *	Now scan through and see if one already exists, if not
 *	add a new one with default values.
 */
	alc->lut = NULL; /* Use as a found flag */

	for	(lut = alc->luts; lut; lut = lut->next)
		if (lut->type == type) alc->lut = lut;
	if (alc->lut == NULL)						/* Can't find requested type */
		for	(lut = alc->luts; lut; lut = lut->next)		/* Look for undefined */
			if (lut->type == UNDEF_LUT) alc->lut = lut;
	if (alc->lut == NULL) {						/* No luck, so add new space */
		lut = alc->luts;						/* Keep the rest */
		alc->luts = alc->lut = default_lut();	/* Assign new & make current */
		alc->lut->next = lut;					/* Link the rest back in */
	}
	alc->lut->type = type;
	alc->lut->status &= ~INVERT;				/* Positive LUT for all */
	plotgammacurve(1);							/* Plot new curve */
	apply_mono_lut();							/* and apply it */
	if (type == FLUOR_LUT) alc->fluor = TRUE; 	/* Reflect state in fluor flag*/
	else alc->fluor = FALSE;
} /* end change_lut_type */

/*
 *	L U T _ T Y P E S T R -- Return a type string, given integer type
 */
char *
lut_typestr(type)
int type;
{
	static char typestr[16];

	switch (type) {
	case UNDEF_LUT:		strcpy(typestr, "undefined");
						break;
	case BRIGHT_LUT:	strcpy(typestr, "brightfield");
						break;
	case FLUOR_LUT:		strcpy(typestr, "fluorescence");
						break;
	case SPEC_LUT:		strcpy(typestr, "special");
						break;
	}
	return(typestr);
} /* end lut_typestr */

/*
 *	C H E C K _ L U T _ T Y P E --	Check for presence of LUT type
 *									Return true if type found
 */
check_lut_type(type)
int type;
{
	struct lut_assoc *lut;
	int present=FALSE;

	for	(lut = alc->luts; lut; lut = lut->next)
		if (lut->type == type) present = TRUE;
	return(present);
}

/*
 * I N I T _ C U R S --
 */
init_curs()
{
	cursframe.scale = 8;
	cursframe.dx = dg->xcenter;
	cursframe.dy = dg->ycenter;
	cursframe.ox = dg->lineln/2;
	cursframe.oy = dg->lineno/2;
	cursframe.ix = 1;	
	cursframe.iy = 1;
	cf = &cursframe;
	cursor.type = 40;
	cursor.style = 6;
}

/*
 * I N I T _ D I S P --
 */
init_display()
{
	register int i;

	clearplane(dg, RED|BLUE);
	for (i=0; i<256; i++) {
		if ((i & (RED|BLUE)) == (RED|BLUE))
			setrgb(i, 0, 255, 0);		/* Green */
		else if (i & RED)
			setrgb(i, 255, 0, 0);		/* Red */
		else if (i & BLUE)
			setrgb(i, 0, 0, 255);		/* Blue */
	}
	apply_mono_lut();
}

/*
 *	P L O T G A M M A C U R V E --	Plot the current gree_lut on screen.
 *
 *	Input:	draw = 1, draw curve;
 *			draw = 0, erase curve
 */
plotgammacurve(draw)
int draw;
{
	register int in, x, y, rstep;
	register UBYTE *gl;

	colour(RED);
	lwidth(1);
	intens(draw);
/*
 *	Draw, or erase lut curve
 */
	rstep = (alc->lut->status & CSTEPMASK) >> 4;
	if (!rstep) rstep = 1;
	if (alc->debug) printf("plotgammacurve: rstep = %d\n", rstep);
	gl = alc->lut->lut->green_lut;
	moveto(xo , (int)(*gl << 3) + yo);
	for (in = 0; in <= 255; in++) {
		if (((in % rstep) == 0) || (in == 255)) {
			x = (in << 3) + xo; y = (int)(*gl << 3) + yo;
			lineto(x, y);
		}
		gl++;
	}
} /* end plotgammacurve */

/*
 *	P L O T A X E S -- Put up some axes.
 */
plotaxes()
{
	int xc, yc;

	colour(BLUE);
	lwidth(1);
	intens(1);

	xc = alc->ddgschannel->xcenter;
	yc = alc->ddgschannel->ycenter;
	xo = xc/3;
	yo = yc/2;

	moveto(xo-40, yo);
	lineto(xc+xo, yo);
	moveto(xo, yo-40);
	lineto(xo, yc+yo);
	moveto((xo+xc)/2, yo-100);
	text("Intensity");
	moveto(xo-450, yo + yc/2);
	text("Output");
}

/*
 *	C A L C G A M M A C U R V E --	Calculate a new gamma curve
 *	Input:
 *	step, determines the frequency of the calculation. The higher the step
 *	value, the less calculation is involved, and the faster	it should run.
 *	Uncalculated values are undefined
 */
calcgammacurve(step)
int step;
{
	register int in, rstep;
	register UBYTE *rl, *gl, *bl;
	float f, gamma;

	rstep = step;
	alc->lut->status &= ~CSTEPMASK;		/* Clear previous bits */
	alc->lut->status |= (rstep << 4);	/* Store used step value */
	if (rstep != 1)
		alc->lut->status &= ~EXACT;	/* Current LUT will need recalculating */
	else
		alc->lut->status |= EXACT;	/* Else it will be exact */
	gamma = alc->lut->gamma;		/* Local var. to use in calculation */
	f = 255.0 / pow((double)255.0, (double)gamma);
	rl = alc->lut->lut->red_lut;
	gl = alc->lut->lut->green_lut;
	bl = alc->lut->lut->blue_lut;

	if ((alc->lut->status & INVERT) == INVERT) {
		for (in = 0; in <= 255; in++) {
			if (((in % rstep) == 0) || (in == 255)) {
				*rl = *gl = *bl = f * qexp((float)(gamma * qln((float)in)));
			}
			rl++; gl++; bl++;
		}
	} else {
		for (in = 0; in <= 255; in++) {
			if (((in % rstep) == 0) || (in == 255)) {
				*rl = *gl = *bl = 255 - (f * qexp((float)(gamma * qln((float)in))));
			}
			rl++; gl++; bl++;
		}
	}
}

/*
 *	A P P L Y _ M O N O _ L U T -- Version of apply_lut
 *	For display purposes, this function just takes the current green_lut LUT.
 *	It does not set value one, as this is the graph colour.
 *	It also takes account of the invertedness of the framestore.
 */
apply_mono_lut()
{
	register int i, v, rstep;
	register UBYTE *gl;
	int invert_flag;
/*
 *	Modify screen LUT according to fluor flag, since we use invert values off
 *	for both brightfield, and fluorescence, made possible due to the fact that
 *	fsconstruct provides an inversion if the image is fluorescent.
 *	An alternative solution would be to modify fsconstruct to not invert the 
 *	data then it could all be done by the gamma LUT, this may be preferable.
 */
 	invert_flag = (alc->ddgschannel->invert != alc->fluor);

	rstep = (alc->lut->status & CSTEPMASK) >> 4;
	if (!rstep) rstep = 1;
	if (alc->debug) printf("apply_mono_lut: rstep = %d\n", rstep);
	gl = alc->lut->lut->green_lut;

	if (invert_flag) { /* Inverted framestore */
		for (i=0; i<256; i++, gl++) {
			if (i==1) continue;
			if (((i % rstep) == 0) || (i == 255)) v = 255 - (int)*gl;
			if ((i % 4) == 0) setrgb(i, v, v, v); /* Don't set lower 2 planes */
		}
	} else {
		for (i=0; i<256; i++, gl++) {
			if (i==1) continue;
			if (((i % rstep) == 0) || (i == 255)) v = (int)*gl;
			if ((i % 4) == 0) setrgb(i, v, v, v); /* Don't set lower 2 planes */
		}
	}
} /* end apply_mono_lut */

/*
 *	I N V E R T _ L U T -- Invert state of every 'bin'
 */
invert_lut()
{
	register UBYTE *rl, *gl, *bl;
	register int i, v;

	rl = alc->lut->lut->red_lut;
	gl = alc->lut->lut->green_lut;
	bl = alc->lut->lut->blue_lut;
	for (i=0; i<256; i++) {
		v = 255 - *gl;	/* Mono copy, use green LUT */
		*rl++ = *gl++ = *bl++ = v;
	}
} /* end invert_lut */

/*
 *	W R I T E L U T F I L E -- Write the LUTS to file
 */
writelutfile(ofile)
char *ofile;
{
	FILE *lf;
	int nluts, i, write_err=0;
	struct lut_assoc *lut;
/*
 *	Scan through LUT list to see what's there
 */
 	for (nluts=0, lut = alc->luts; lut; lut = lut->next) {
 		nluts++;
 	}
	printf("Writing %d correction tables to file %s\n", nluts, ofile);
 		
	if ((lf = fopen(alc->lutfile, "w")) == NULL) {
		fprintf(stderr, "Can't write to look-up table file %s\n", alc->lutfile);
		return(-1);
	}

	if ((putw((int)LUT_MAGIC, lf) < 0) ||
		(putw((int)LUT_VSN, lf) < 0) ||
		(putw(nluts, lf) < 0)) {
		fprintf(stderr, "Error writing to look-up table file %s\n", alc->lutfile);
		fclose(lf);
		return(-1);
	}

 	for (nluts=0, lut = alc->luts; (lut && (write_err == 0)); lut = lut->next) {
		if ((fwrite(lut, sizeof(struct lut_assoc), 1, lf) == 0) ||
			(fwrite(lut->lut, sizeof(struct look_up_table), 1, lf) == 0))
				write_err = -1;
	} /* end for lut */
	if (write_err < 0) { 
		fprintf(stderr, "Error writing to LUT file %s\n", alc->lutfile);
		fclose(lf);
		return(-1);
	}
	fclose(lf);
	return(0);
} /* end writelutfile */

/*
 *	R E C A L C _ L U T S -- Re-calculate those LUTS that need it
 */
recalc_luts()
{
	struct lut_assoc *lut;

	for (lut = alc->luts; lut; lut = lut->next) {
		if ((lut->status & EXACT) != EXACT) {
			alc->lut = lut;
			printf("Recalculating %s correction\n",lut_typestr(alc->lut->type));
			calcgammacurve(1); /* Exactly calculate new one */
		}
	}
}

/*
 *	S E T F L O A T --	Set up floating point value, given input string
 *						Applies MIN/MAXGAMMA values (These are sensible to
 *						apply to 'step' as well.
 */
setfloat(val, floatval)
char *val;
float *floatval;
{
	sscanf(val, "%f" , floatval);
	if (*floatval > MAXGAMMA) *floatval = MAXGAMMA;
	if (*floatval < MINGAMMA) *floatval = MINGAMMA;
	sprintf(val, "%5.2f", (double)(*floatval));
}

/*
 *	D I S P B A N N E R -- Banner display
 */
dispbanner()
{
	bigmessage(0, 0, "     Define Gamma Correction Tables");
	fflush(stdout);
}

/*
 *	P R I N T _ O P T I O N S -- Print option list & exit
 */
print_options()
{
	printf("%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
	"Syntax:\tgamma <-v?1234IVCm> <option info> (stand-alone mode)\n",
	"\t\t<cmdsys ptr> -s <alc ptr> (slave mode)\n",
	"Function:\tProduce gamma correction look-up tables\n",
	"Options:\n",
	"\t-v - Show version number\n",
	"\t-s - Start in slave mode\n",
	"\t-? - Print this help list\n",
	"\t-1 - Use seat 1 framestore (default)\n",
	"\t-2 - Use seat 2 framestore (3, 4 similarly)\n",
	"\t-I - Non-inverted virtuoso\n",
	"\t-V - Use virtuoso framestore (Default)\n",
	"\t-c - Clear display on startup\n",
	"\t-C - Use CRS512 framestore\n",
	"\t-m - Specify mouse terminal line (Default /t5)\n");
}
