/*
 *			G A M M A . C -- Create gamma curves
 *
 *
 *  Written  David Burgess
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    15th January 1990
 *
 *  Modifications
 *
 */
#define INIFILE        /* define variables in this link */
#include <cmdsys.h>
#include <procid.h>
#include <kyy.h>
#include <cfipp.h>
#include <ddgs.h>
#include <sgstat.h>
#include <math.h>
#include <lut.h>

#define TRUE (1)
#define FALSE (0)
#define WHITE 0
#define BLACK 255
#define RED 1
#define BLUE 2
#define GREEN 3
#define LUT_MAGIC 5002580

/*
 *	System data structures etc
 */
struct kcontrol *kcont = NULL;

/*
 *	Externals
 */
extern DDGS *ddgsopen();
extern int (*setexception())();
extern int memexitfn();
extern int errno;
extern int _memmins;

/*
 *	Files
 */
FILE *debug_channel = NULL;				/* system debug file */
FILE *debugfp;

/*
 *	Globals
 */
int	(*quit_fn)() = NULL;		/* Call this when we get ^c */
int quit=0;
int slave_mode;					/* Forked from another process */
int first=TRUE;					/* Controls certain first time initialisation */
char *gammaid = "0.1";			/* The version number */
struct sgbuf savestdin;
int	store = FS_VIRTUOSO_INVERT, devno = 3;
float gamma, step;
char mouseterm[5] = "/t5";

/*
 *	Function forward declarations
 */
int gamma_set(), gamma_init(), dispbanner();
int upgamma(), downgamma(), setfloat();
struct look_up_table *readlutfile(), *default_lut(), *Freeluts();
static int disp_messg();

/*
 *	Statics
 */
static DDGS *dg;
static char infile[80], outfile[80];
static int curr_chrom, txt = 0, filewritten = 0;
static struct pframe cursframe, *cf;
static struct ipoint cursor;
static struct look_up_table *luts;

/*
 *	I N T E R R U P T  -- interrupt catcher
 */
interrupt(sig)
int sig;
{
	char c;

	if (quit) return;
	quit = 1;
	if (alc->debug)
		printf("GAMMA - seen sig %d\n",sig);
	if (quit_fn) quit_fn();
	else def_quit_fn();
	quit = 0;
	return;
}

/*
 *	D E F _ Q U I T _ F N -- Default quit function, call this when
 *	we get ctrl-C if no other quit function set up
 */
def_quit_fn()
{
	char c;

	while (_gs_rdy(fileno(stdin)) > 0)
		read(fileno(stdin),&c,1);	/* Flush input */
	screengo(ENDLINE,0);
	printf("Quit seen - Do you wish to stop GAMMA ? "); fflush(stdout);
	while (_gs_rdy(fileno(stdin)) <= 0); /* Wait for input */
	read(fileno(stdin),&c,1);
	if ((c == 'y') || (c == 'Y')) {
		shutdown();
	}
} /* end def_quit_fn */

/*
 *	S H U T D O W N  --  Shut all down
 */
static
shutdown()
{
	tidyworld();
	fprintf(stderr, "\nGAMMA: shutdown\n");
	alldone(1);
}

/*
 *	T I D Y W O R L D  --
 */
static
tidyworld()
{
	cook(fileno(stdin), &savestdin);
	system("tmode echo eof=4");
}

/*
 *	E X C E P T I O N  --  Exception traps come here
 *
 */
exception(exc,addr)
int		exc, addr;
{
	procid	pd;
	int		fd, time, date, day, tick;
	char	buffer[100];

	gprdsc(getpid(),&pd);
	sprintf(buffer,"(V%s) Exception %d, addr 0x%x, base 0x%x (offset 0x%x)\n",
		gammaid, exc, addr, pd._pmodul, addr - (int)pd._pmodul);
	errorlog(buffer);
	shutdown();
}

/*
 *	M A I N  --  Gamma main loop
 */
main(argc,argv, argp)
int argc; char **argv, **argp;
{
	DDGS	*ddgsopen(), *ddgsdup();
	char	*swptr, **targv, **sargv, buf[80];
	int		i, targc;

	_gs_opt(fileno(stdin), &savestdin);	/* Keep a copy for cleanup later */
	intercept(interrupt);
	setexception(exception);
	Alloc_exit(memexitfn);
	targc = argc; targv = argv;

	slave_mode = FALSE;
	argc--; argv++;			/* dump process name */
	while (argc > 0) {
		if (*argv[0] == '-') {
			swptr = *argv;
			*argv = NULL;
			while (*(++swptr)) {
				switch (*swptr) {
				case '?':	print_options();
							exit(0);
				case 'v':	printf("gamma: version %s\n", gammaid);
							exit(0);
				case 's':	slave_mode = TRUE;
							alc = (struct allkontr *)atoi(*++argv);
							if (alc->debug) printf("gamma: alc = 0x%x\n");
							*argv = NULL; argc--;
							break;
				case '1':
				case '2':
				case '3':
				case '4':	devno = *swptr - '1';
							break;
				case 'I':	store = FS_VIRTUOSO;
							break;
				case 'V':	store = FS_VIRTUOSO_INVERT;
							break;
				case 'C':	store = FS_VME512;
							break;
				case 'm':	argc--;
							argv++;
							strcpy(mouseterm, *argv);
							*argv = NULL;
							break;
				default:	printf("gamma: - unknown option\n");
				}
			}
		}
		argv++; argc--;
	}

	if (alc->debug) {
		argc = targc; argv = targv;
		while(--argc) printf("%s\n", *argv++);
	}
	argc = targc; argv = targv;
	argc--; argv++;			/* dump process name */
	
/*
 * Compact params to fill holes left by switches
 */
	argc = --targc; argv = ++targv;				/* dump process name */
	sargv = targv;
	while (targc-- > 0) {						/* Squeez + fill the holes */
		if (*targv)
			*sargv++ = *targv;
		else
			argc--;
		targv++;
	}
/*
 *	Init global structures
 */
 	_memmins = 32768; /* 32K */
	if (!slave_mode) {
		alc = (struct allkontr *)Calloc(1, sizeof(struct allkontr));
		alc->cfipp = (struct cfipparams *) Calloc(1, sizeof(struct cfipparams));
		
		sysinit(alc);
		getwd(alc->dir, 40);	/* Set current dir (sysinit ?) */
/*
 *	Start up menu handler
 */
		mouseport(alc->mouseterm);	/* Where we want the mouse to be */
		muini(stdout);				/* Init cmdsys + mouse */
		alc->m2s = mukont->mouss;	/* Get mouse struct from cmdsys */
	}

	set_header_fn(dispbanner);	/* Display routine instead of up menu line */
	pastmu = 0;				/* do recall up-menus on command string */
	uplow = 0;				/* do not identify upper-lower case */

/*
 *	Start up debugging (debugfile.gamma)
 *						 - debug_channel for system
 */
	screengo(ENDLINE, 0);
	sprintf(buf, "%s.gamma", alc->debugfile);
 	if (alc->debug) {
 		if (debugfp)
 			debug_channel = debugfp;
 		else {
			fprintf(stderr, "gamma: normal debug on. Opening %s\n",buf);
			if ((debug_channel = fopen(buf, "w")) ){
				fprintf(debug_channel,"gamma: General debug information\n");
 			} else {
				fprintf (stderr,"Failed to open debug file error %d\n" ,errno);
			}
		}
		fflush(debug_channel);
		Alloc_debug(debug_channel);
	}

	if (slave_mode) {
		slavemain(targv[1], gamma_set, gamma_init);
	} else {
		gamma_init();
		alc->lut = alc->luts = readlutfile();
		gamma_set();
		alldone(0);
	}
}

/*
 *	G A M M A _ I N I T --	Initialise for gamma setup
 */
gamma_init()
{
	if (slave_mode) {
		dg = alc->ddgschannel;
	} else {
		dg = ddgsopen(store, devno);
	}

	dinit();
	clear();
	init_display();
	init_curs();
}

/*
 *	G A M M A _ S E T -- Set gamma factor, and display
 */
gamma_set()
{
	if (slave_mode) printf("Gamma setup\n");
	while (topmenu());
}

/*
 * T O P M E N U 
 *
 */
topmenu()
{
	int set_cmd, dec_cmd, inc_cmd, acc_cmd, quit_cmd;
	char gv[30], sv[30];

/*	setmsv(curs);	*/

	strcpy(gv, "1.0");
	setfloat(gv, &gamma);
	strcpy(sv, "0.1");
	setfloat(sv, &step);
	plotgammacurve(alc->lut);

	newmenu("Gamma Correction");
	set_cmd = cmmd("set gamma value;:k30%s%-+d%f", gv, &gamma, setfloat);
	cmmd("step size;:k30%s%-+d%f", sv, &step, setfloat);
	dec_cmd = cmmd("decrease by step size;%g", downgamma); mouse(LEFTBT);
	inc_cmd = cmmd("increase by step size;%g",upgamma); mouse(RIGHTBT);
	acc_cmd = cmmd("accept and file"); mouse(MIDBT);
	quit_cmd = cmmd("quit"); mouse(LONGLB);
	while (thismu()) {
		if (nncmd == acc_cmd) file_lut();
		else if (nncmd == quit_cmd) break;
		else if ((nncmd == inc_cmd) || (nncmd == dec_cmd)) {
			setfloat(gv, &gamma);
			displin(0, set_cmd);
			putprompt();
			plotgammacurve(alc->lut);
		}
		if (nncmd == set_cmd) plotgammacurve(alc->lut);
	}
}

/*
 *	S E T F L O A T -- Set up floating point value, given input string
 */
setfloat(val, floatval)
char *val;
float *floatval;
{

	printf("Val. to convert = %s\n", val);

	sscanf(val, "%f" , floatval);
	sprintf(val, "%5.2f", (double)(*floatval));
}

upgamma()
{
	gamma += step;
}

downgamma()
{
	gamma -= step;
}

/*
 *	F I L E _ L U T -- File generated LUT
 */
file_lut()
{
	char txtstr[8];
	int Quit_cmd, cmd;
	
	dotxtstr(&txt, txtstr);
	newmenu("Create LUT file");
	cmmd("output file name;:k40%s%-f", outfile, disp_messg);
	cmmd("type of output file;:k40%-s%f", txtstr, dotxtstr);
	cmmd("quit"); Quit_cmd = mmcmd;

	while((cmd = thismu()) != Quit_cmd) {
		if (cmd == 0) {
			writefile(outfile, txt);
			filewritten = 1;
			break;
		}
	}
}

/*
 *	W R I T E F I L E -- Write the LUTS to file
 */
writefile()
{}

/*
 *	D I S P _ M E S S G  --  Display message telling user to press return
 *			
 */
static int
disp_messg()
{
	screengo(OUTLINE - 1,0);
	printf("Press <Return> to write file\n");
}

/*
 *	D O T X T S T R -- Process type of file
 */
dotxtstr(typestr)
char *typestr;
{
	static int type = 0;

	switch(type) {
		case 0:	strcpy(typestr, "brightfield");
				break;
		case 1:	strcpy(typestr, "fluorescence");
				break;
		case 2:	strcpy(typestr, "special");
				break;
	}
	type = (type++) % 2;
}

/*
 *	D I S P B A N N E R -- Banner display
 */
dispbanner()
{
	bigmessage(0, 0, "     Produce Gamma Correction Tables");
	fflush(stdout);
}

/*
 *	P R I N T _ O P T I O N S -- Print option list & exit
 */
print_options()
{
	printf("%s%s%s%s%s%s%s%s%s%s%s%s%s",
	"Syntax:\tgamma <-v?1234IVCm> <option info> (stand-alone mode)\n",
	"\t\t<cmdsys ptr> -s <alc ptr> (slave mode)\n",
	"Function:\tProduce gamma correction look-up tables\n",
	"Options:\n",
	"\t-v - Show version number\n",
	"\t-s - Start in slave mode\n",
	"\t-? - Print this help list\n",
	"\t-1 - Use seat 1 framestore (default)\n",
	"\t-2 - Use seat 2 framestore (3, 4 similarly)\n",
	"\t-I - Non-inverted virtuoso\n",
	"\t-V - Use virtuoso framestore (Default)\n",
	"\t-C - Use CRS512 framestore\n",
	"\t-m - Specify mouse terminal line (Default /t5)\n");
}

/*
 * I N I T _ C U R S --
 */
init_curs()
{
	cursframe.scale = 8;
	cursframe.dx = dg->xcenter;
	cursframe.dy = dg->ycenter;
	cursframe.ox = dg->lineln/2;
	cursframe.oy = dg->lineno/2;
	cursframe.ix = 1;	
	cursframe.iy = 1;
	cf = &cursframe;
	cursor.type = 40;
	cursor.style = 6;
}

/*
 * I N I T _ D I S P --
 */
init_display()
{
	clearplane(dg, 1);
	setrgb(WHITE, 255, 255, 255);
	setrgb(BLACK, 0, 0, 0);
	setrgb(RED, 255, 0, 0);
	setrgb(BLUE, 0, 0, 255);
	setrgb(GREEN, 0, 255, 0);
}

/*
 *	P L O T G A M M A C U R V E --	Plot gamma gurve on screen from
 *									current global gamma value
 */
plotgammacurve(lut)
struct look_up_table *lut;
{
	register int e, x, y, *ilut;
	double scale = 4096.0, dgamma;
	float inter, f;

	ilut = (int *)Calloc(256,sizeof(int)); /* Working grey scale LUT copy */
	dgamma = (double)gamma;
	colour(1);
	intens(1);
	moveto(0, 0);

	f = 255.0 / pow(255.0, dgamma);
	for (e=0; e<255; e++) {
/*		*ilut = 255 * (float)pow((double)(e/f255), dgamma); */
		*ilut = f * pow((double)e, dgamma);
		x = e << 4; y = *ilut << 4;
		lineto(x, y);
		moveto(x, y);
		ilut++;
	}
}

/*
 *	R E A D L U T F I L E -- Read the look up tables from file
 */
struct look_up_table *
readlutfile()
{
	FILE *lf;
	int magic, vsn, nluts, i, read_err=0;
	struct look_up_table *luts, *lut;

	if ((lf = fopen(alc->lutfile, "r")) == NULL) {
		fprintf(stderr, "Can't open look-up table file %s\n", alc->lutfile);
		lut = default_lut();
	} else {
		if ((getw(magic) < 0) || (magic != LUT_MAGIC) ||
			(getw(vsn) < 0) || (getw(nluts) < 0)) {
				fprintf(stderr, "Invalid look-up table file %s\n", alc->lutfile);
				luts = default_lut();
		} else {
			for (i=0; ((i < nluts) || (read_err == 0)); i++) {
				if (i == 0)
					lut = luts = (struct look_up_table *)Calloc(1, sizeof(struct look_up_table));
				else {
					lut->next = (struct look_up_table *)Calloc(1, sizeof(struct look_up_table));
					lut = lut->next;
				}
				switch (vsn) {
 				case 1: fread(lut, sizeof(struct look_up_table), 1, lf);
						break;
				default: read_err = -1;
						break;
				}
			} /* end for i ... nluts */
			if (read_err < 0) { 
				fprintf(stderr, "Illegal LUT file %s\n", alc->lutfile);
				luts = Freeluts(luts);
				luts = default_lut();
			}
		}
	}
	return(luts);
}

/*
 *	D E F A U L T _ L U T -- Assign a default linear LUT
 */
struct look_up_table *
default_lut()
{
	int i;
	struct look_up_table *lut;

	lut = (struct look_up_table *)Calloc(1, sizeof(struct look_up_table));
	lut->type = BRIGHT_LUT;
	for (i=0; i<256; i++)
		lut->red_lut[i] = lut->green_lut[i] = lut->blue_lut[i] = i;
	lut->next = NULL;
	return(lut);
}

/*
 *	F R E E L U T S -- Free of linked list of LUT structures
 */
struct look_up_table *
Freeluts(luts)
struct look_up_table *luts;
{
	struct look_up_table *flut;

	while (luts) {
		flut = luts;
		luts = flut->next;
		Free(flut);
	}
	return(NULL);
}
