/*
 *      H C O P Y F M T . C  --  hardcopy formatter
 *
 *
 *  Written by Sarah E. A. Jones.
 *				
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    13th Apr 1989.
 *
 *  Modifications
 *
 */

#define INIFILE        /* define variables in this link */
#include <cmdsys.h>
#include <wstruct.h>
#include <chromanal.h>
#include <dispform.h>
#include <ddgs.h>
#include <scrformat.h>

#define MAXPAGEX 2728
#define MAXPAGEY 3864
#define WIDTH 1
#define HEIGHT 2

/*
 * external routines
 */
extern int init_curs(), erasecursor(), showcursor(), curs(), init_display(), curstrack();
extern int hccurs(), boxcurs(), inside(), display_group();
extern struct hcobject *hclocate();
extern struct hcopyform *readfile();

/*
 * external variables
 */
extern struct scrform *scrf; 
extern struct hcopyform *hcpy;
extern struct ipoint cursor;
extern int curr_chrom;
extern struct box *curr_box;

/*
 * local variables 
 */
DDGS *dg;
struct hcobject *current;
char infile[80], outfile[80];
int txt = 0, filewritten = 0;
char *asstxt_strings[] = {
	"none",
	"case",
	"slide",
	"cell",
	"operator",
	"date",
	"supervisor checked",
	"supervisor date",
	"patient name",
	"comment 1",
	"comment 2",
	"karyotype",
	"laboratory name"
};
/*
 * local routines.
 */
int kickslave(), startup();
int topmenu(), group(), getgroup(), ungroup();
int change_box(), placebox(), resize(), change_text(), sizebox(), txtsize();
int writemenu(), newbox(), rembox(), gettype();
/*
 *	I N T E R R U P T --  Signal catcher
 *
 */
interrupt(sig)
int		sig;
{
}
/*
 * M A I N -- 
 *
 */
main(argc,argv)
int		argc;
char	**argv;
{
	char	*swptr, **targv, **sargv, mouseterm[7];
	int		targc;
	int		store = FS_VME512, devno = 0;
	DDGS	*ddgsptr, *ddgsopen(), *ddgsdup();

#ifdef INTERRUPT
	intercept(interrupt);
#endif INTERRUPT
	targc = argc; targv = argv;

	strcpy(mouseterm, "/t5");
	argc--; argv++;			/* dump process name */
	while (argc > 0) {
		if (*argv[0] == '-') {
			swptr = *argv;
			*argv = NULL;
			while (*(++swptr)) {
				switch (*swptr) {
				  case '?':
				  	printf("Syntax:   hcopyfmt <options> \n");
				  	printf("Function: \n");
				  	printf("Options:  v - Show version number\n");
					printf("          1 - Use seat 1 framestore\n");
					printf("          2 - Use seat 2 framestore\n");
					printf("                  also 3 and 4\n");
					printf("          I - Virtuoso framestore (Default CRS 512)\n");
					printf("		  V - Virtuoso invert framestore(Default CRS 512)\n");
					printf("		  m - Mouse term (default /t5)\n");
				  	printf("          ? - Help\n");
				  	exit(0);
				  case 'v':
						printf("hcopyfmt - Version 0.1\n");
						exit(0);
				   case '1':
						devno = 0;
						break;
					case '2':
						devno = 1;
						break;
					case '3':
						devno = 2;
						break;
					case '4':
						devno = 3;
						break;
					case 'I':
						store = FS_VIRTUOSO;
						break;
					case 'V':
						store = FS_VIRTUOSO_INVERT;
						break;
					case 'm':
						argc--;
						argv++;
						strcpy(mouseterm, *argv);
						*argv = NULL;
						break;
					case 't':
						txt = 1;
						break;
					case 'b':
						txt = 0;	/* default */
						break;
				  default:
				 	printf("hcopyfmt - unknown option\n");
				}
			}
		}
		argv++; argc--;
	}

/*
 * Compact params to fill holes left by switches
 */
	argc = --targc; argv = ++targv;				/* dump process name */
	sargv = targv;
	while (targc-- > 0) {						/* Squeez + fill the holes */
		if (*targv)
			*sargv++ = *targv;
		else
			argc--;
		targv++;
	}
/*
 * initialise cmdsys before anything else
 */
 	mouseport(mouseterm);
	muini(stdout);
/*
 *	Use params 
 */
 	*outfile = '\0';
 	*infile = '\0';
	if (argc > 1)
		strcpy(outfile, argv[1]);
	if (argc) {
		strcpy(infile, *argv);
		if (*outfile == '\0')
			strcpy(outfile, infile);
	} else
		*infile = '\0';
/*
 * This is the start of the M A I N program
 *
 */
	dg = ddgsopen(store, devno);
	startup(infile, outfile);
}
/*
 *  K I C K S L A V E -- dummy routine
 *
 */
kickslave() {}
/*
 * I N I T I A L I S E --
 *
 */
initialise()
{
	dinit();
	clear();
	init_display();
	init_curs();
	showcursor();
}
/*
 *  S T A R T U P -- initial setup routines
 *
 */
startup(infile, outfile)
char *infile, *outfile;
{
	initialise();
	draw_paper();
	if (*infile != '\0') {
		hcpy = readfile(infile);
		display_all(hcpy->hco, 1, BLUE);
	} else
		inithcpy();
	topmenu();
	alldone(0);
}
/*
 * T O P M E N U 
 *
 */
topmenu()
{
	int Quit_cmd;
	
	erasecursor();
	setmsv(hccurs);
	newmenu("Hardcopy Formatter");
	cmmd("select box/text to be altered;%f", change_box); mouse(MIDBT);
	cmmd("new object;%f", gettype);mouse(LONGRB);
	cmmd("group;%f", group);mouse(LONGMB);
	cmmd("karyotype scale;:k40<1/50>%d", &hcpy->karscale);
	cmmd("update file;%f", writemenu);
	cmmd("quit"); Quit_cmd = mmcmd;
	while (1) {
		while (thismu() != Quit_cmd) ;
		if (!filewritten) {
			printf("Output file has not been written. Writefile?\n");
			if (!yesno()) {
				downmenu();
				writemenu();
				upmenu();
			} else
				break;
		} else
			break;
	}
}
/*
 *  D R A W _ P A P E R
 *
 */
draw_paper()
{
	colour(GREEN);
	lwidth(2);
	moveto(XOFFSET, YOFFSET);
	lineto(XOFFSET + MAXPAGEX, YOFFSET);
	lineto(XOFFSET + MAXPAGEX, YOFFSET + MAXPAGEY);
	lineto(XOFFSET, YOFFSET + MAXPAGEY);
	lineto(XOFFSET, YOFFSET);
}
/*
 * D I S P L A Y _ A L L
 *
 */
display_all(ptr, on, col)
struct hcobject *ptr;
int col, on;
{
	while (ptr!= NULL) {
		if (ptr->ass != NULL)
			display_all(ptr->ass, on, col);
		display_box(ptr, on, col);
		ptr = ptr->next;
	}
}
/*
 * I N I T _ B O X --
 *
 */
init_box(ptr, x, y, width, height)
struct hcobject *ptr;
int x, y, width, height;
{
	if (x >= 0 && y >= 0) {
		ptr->b.posn.x = x;
		ptr->b.posn.y = y;
	}
	ptr->b.width = width;
	ptr->b.height = height;
}
/*
 * D I S P L A Y _ B O X
 *
 */
display_box(ptr, on, col)
struct hcobject *ptr;
int col, on;
{
	register int x, y;

	if (ptr !=  NULL) {
		colour(col);
		lwidth(2);
		intens(on);
		x = ptr->b.posn.x + XOFFSET;	/* bottom left hand corner */
		y = ptr->b.posn.y + YOFFSET;
		ptr->b.col = col;
		if (ptr->type == TXT) {
			tsize(ptr->b.width, ptr->b.height);
			dispstring(ptr->txt, x, y);
			if (ptr->asstxt != -1) {
				colour(GREEN);
				dispstring(asstxt_strings[ptr->asstxt + 1], 
							x+(strlen(ptr->txt)*ptr->b.width), y);
				colour(col);
			}
		} else {
			moveto(x, y);
			lineto(x, y + ptr->b.height);
			lineto(x + ptr->b.width, y + ptr->b.height);
			lineto(x + ptr->b.width, y);
			lineto(x, y);
		}
	}
}
/*
 * D I S P L A Y _ G R O U P
 *
 */
display_group(ptr, on, col)
struct hcobject *ptr;
int col, on;
{
	if (ptr != NULL) {
		display_box(ptr, on, col);
		if (ptr->ass != NULL)
			display_all(ptr->ass, on, col);
	}
}
/*
 * C H A N G E _ B O X --
 *
 */
change_box()
{
	current = hclocate(0);
	erasecursor();
	setmsv(curstrack);
	boxmenu();
	erasecursor();
	setmsv(hccurs);
}
/*
 * G E T T Y P E
 *
 */
gettype()
{
	int Ex_cmd;
	
	newmenu("Select type of new object");
	cmmd("chromosome class;:k40%-+d%f", CLASS, newbox); mouse(RIGHTBT);
	cmmd("stack;:k40%-+d%f", STK, newbox);mouse(LEFTBT);
	cmmd("metaphase;:k40%-+d%f", METAPHASE, newbox);mouse(MIDBT);
	cmmd("text;:k40%-+d%f", TXT, newbox);mouse(LONGMB);
	cmmd("exit"); Ex_cmd = mmcmd; mouse(LONGLB);
	
	while (thismu() != Ex_cmd) ;
}
/*
 * N E W B O X
 *
 */
newbox(type)
int type;
{
	struct hcobject *ptr;
	int x, y;
	
	erasecursor();
	setmsv(curstrack);
	display_box(current, 0, RED);
	display_box(current, 1, BLUE);
	current = (struct hcobject *) Calloc(sizeof(struct hcobject), 1);
	ptr = hcpy->hco;
	while (ptr->next != NULL)
		ptr = ptr->next;
	ptr->next = current;
	current->next = NULL;
	current->type = type;
	x = cursor.k<<dg->xshift;
	y = cursor.l<<dg->yshift;
	if (type == TXT)
		init_box(current, x - XOFFSET, y - YOFFSET, 40, 40);
	else
		init_box(current, x - XOFFSET, y - YOFFSET, 200, 300);
	if (type == STK) {
		hcpy->viewed_stack++;
		current->class = hcpy->viewed_stack;
	} else if (type == CLASS) {
		hcpy->chromclasses++;
		current->class = hcpy->chromclasses;
	} else
		current->class = -1;
	*current->txt = '\0';
	current->ass = NULL;
	current->asstxt = -1;
	display_box(current, 1, RED);
	boxmenu();
	erasecursor();
	setmsv(hccurs);
}
/*
 * F R E E _ L I S T
 *
 */
free_list(fptr)
struct hcobject *fptr;
{
	struct hcobject *next;
	
	while (fptr != NULL) {
		if (fptr->ass != NULL)
			free_list(fptr->ass);
		display_box(fptr, 0, RED);
		next = fptr->next;
		Free(fptr);
		fptr = next;
	}
}
/*
 * R E M B O X
 *
 */
rembox()
{
	struct hcobject *ptr;
	
	ptr = hcpy->hco;
	if (current->type == STK)
		hcpy->viewed_stack--;
	if (current->type == CLASS)
		hcpy->chromclasses--;
	while(ptr->next != current && ptr != NULL)
		ptr = ptr->next;
	display_box(current, 0, RED);
	ptr->next = current->next;
	current->next = NULL;
	free_list(current);
	current = hclocate(1);
}
/*
 * C L A S S S T R
 *
 */
classstr(str)
char *str;
{
	int num;
	
	if (*str == 'X' || *str == 'x')
		current->class = 23;
	else if (*str == 'Y' || *str == 'y')
		current->class = 24;
	else if (*str == 'A' || *str == 'a')
		fprintf(stderr, "This is a class option, please select stack option.\n");
	else {
		num = atoi(str);
		if (num > 0 && num <= 22)
			current->class = num;
		else {
			fprintf(stderr, "This is not a valid number.");
			if (current->class > 0 && current->class < 23)
				sprintf(str, "%d", current->class);
			else if (current->class == 23)
				sprintf(str, "X");
			else
				sprintf(str, "Y");
		}
	}
}
/*
 * C H A S S T X T
 *
 */
chasstxt(str)
char *str;
{
	int toggle[MAXTEXT], i, cmd;
	
	for (i = 0; i < MAXTEXT; i++)
		toggle[i] = 0;
	toggle[current->asstxt + 1] = 1;
	newmenu("choose associated variable text");
	mmcmd = 0;
	cmmd("none;:k40%-t", &toggle[mmcmd]);
	cmmd("case;:k40%-t", &toggle[mmcmd]);
	cmmd("slide;:k40%-t", &toggle[mmcmd]);
	cmmd("cell;:k40%-t", &toggle[mmcmd]);
	cmmd("operator;:k40%-t", &toggle[mmcmd]);
	cmmd("date;:k40%-t", &toggle[mmcmd]);
	cmmd("supervisor acceptance;:k40%-t", &toggle[mmcmd]);
	cmmd("supervisor date;:k40%-t", &toggle[mmcmd]);
	cmmd("patient name;:k40%-t", &toggle[mmcmd]);
	cmmd("comment 1;:k40%-t", &toggle[mmcmd]);
	cmmd("comment 2;:k40%-t", &toggle[mmcmd]);
	cmmd("karyotype;:k40%-t", &toggle[mmcmd]);
	cmmd("laboratory name;:k40%-t", &toggle[mmcmd]);
	
	while (cmd = thismu()) {
		if (cmd > 0) {
			display_group(current, 0, BLUE);
			current->asstxt = cmd - 2;
			for (i = 0; i < MAXTEXT; i++)
				toggle[i] = 0;
			toggle[cmd - 1] = 1;
			mukont->mubust = 1;
			strcpy(str, asstxt_strings[current->asstxt + 1]);
			display_group(current, 1, RED);
		}
	}
}
/*
 * S T K S T R
 *
 */
stkstr(string)
char *string;
{
	int num;
	
	num = atoi(string);
	if (num < 1 || num > hcpy->viewed_stack) {
		fprintf(stderr, "%s is not a valid position, please enter a number between 1 and %d\n",
			string, hcpy->viewed_stack);
		sprintf(string, "%d", current->class);
	} else
		current->class = num;
}
/*
 *  B O X M E N U -- menu for sizing a box!
 *
 */
boxmenu()
{	
	int Ex_cmd, width, height;
	char tmpstr[80], class[4], assoc[25];

	strcpy(assoc, asstxt_strings[current->asstxt + 1]);
	if (current->class > 0 && current->class < 23)
		sprintf(class, "%d", current->class);
	else if (current->class == 23)
		sprintf(class, "X");
	else
		sprintf(class, "Y");
	strcpy(tmpstr, current->txt);
	width = current->b.width;
	height = current->b.height;
	newmenu("Box/Text Manipulation.");
	cmmd("place bottom left hand corner of box;:k40%+i%+i%f", -1, -1, placebox);
																	mouse(MIDBT);
	cmmd("resize object;%f", sizebox); mouse(LONGMB);
	if (current->type == TXT) {
		cmmd("change text;:k40%S%f", tmpstr, change_text);
		cmmd("associated variable text;:k40%-s%f", assoc, chasstxt);
		cmmd("text size;:k40width <1/320>%d:k50height <1/320>%d%f", 
										&width, &height, txtsize);
	}
	if (current->type == CLASS)
		cmmd("class number;:k40%s%f", class, classstr);
	if (current->type == STK)
		cmmd("stack position;:k40%s%f", class, stkstr);
	if (current->type == GROUP) {
		cmmd("ungroup;:k40%f", ungroup);mouse(LONGRB);
	}
	cmmd("remove object;:k40%f", rembox);
	cmmd("exit");Ex_cmd = mmcmd; mouse(LONGLB);
	
	while (thismu() != Ex_cmd);
}
/*
 * M O V E B O X E S
 *
 */
moveboxes(ptr, xdiff, ydiff)
struct hcobject *ptr;
int xdiff, ydiff;
{
	while(ptr != NULL) {
		if (ptr->ass != NULL)
			moveboxes(ptr->ass, xdiff, ydiff);
		init_box(ptr, ptr->b.posn.x+xdiff-XOFFSET, ptr->b.posn.y+ydiff-YOFFSET,
												ptr->b.width, ptr->b.height);
		ptr = ptr->next;
	}
}
/*
 * P L A C E B O X --
 *
 */
placebox(xpos, ypos)
int xpos, ypos;
{
	int x, y, xdiff, ydiff, nofit = 0;
	
	display_group(current, 0, RED);
	if (xpos < 0 && ypos < 0) {
		x = cursor.k<<dg->xshift;
		y = cursor.l<<dg->yshift;
	} else {
		x = xpos;
		y = ypos;
	}
	xdiff = x - current->b.posn.x;
	ydiff = y - current->b.posn.y;
	if ((x > XOFFSET) && (x < MAXPAGEX + XOFFSET) && (y > YOFFSET) && 
													(y < MAXPAGEY + YOFFSET)) {
		if (x + current->b.width > MAXPAGEX + XOFFSET) {
			if (current->type == GROUP)
				nofit = 1;
			else
				current->b.width = MAXPAGEX + XOFFSET - x;
		}
		if (y + current->b.height > MAXPAGEY + YOFFSET) {
			if (current->type == GROUP)
				nofit = 1;
			else
				current->b.height = MAXPAGEY + YOFFSET - y;
		}
		if (nofit)
			 fprintf(stderr, "This group will not fit on page.\n");
		else {
			if (current->ass != NULL)
				moveboxes(current->ass, xdiff, ydiff);
			init_box(current, x - XOFFSET, y - YOFFSET, current->b.width, current->b.height);
		}
	}
	display_group(current, 1, RED);
}
/*
 * S I Z E B O X
 *
 */
sizebox()
{
	int amount, Ex_cmd;
	
	if (current->type == TXT)
		amount = 10;
	else
		amount = 50;
	newmenu("Resize object");
	cmmd("increase width;:k40%-+d%-+d%+i%-+d%f", &(current->b.width), &amount, 1, 
											WIDTH, resize);	mouse(RIGHTBT);
	cmmd("decrease width;:k40%-+d%-+d%+i%-+d%f", &(current->b.width), &amount, 0, 
											WIDTH, resize); mouse(LEFTBT);
	cmmd("increase height;:k40%-+d%-+d%+i%-+d%f", &(current->b.height), &amount, 1, 
											HEIGHT, resize); mouse(LONGRB);
	cmmd("decrease height;:k40%-+d%-+d%+i%-+d%f", &(current->b.height), &amount, 0, 
											HEIGHT, resize); mouse(LONGLB);
	cmmd("amount to enlarge/shrink box;:k40%d", &amount);
	cmmd("exit"); Ex_cmd = mmcmd; mouse(LONGMB);
	while(thismu() != Ex_cmd) ;
}
/*
 * S I Z E G R O U P
 *
 */
sizegroup(ptr, pc, type)
struct hcobject *ptr;
int pc, type;
{
	while (ptr != NULL) {
		if (ptr->ass != NULL)
			sizegroup(ptr->ass, pc, type);
		if (type == WIDTH) {
			ptr->b.width = ptr->b.width * pc / 100;
			ptr->b.posn.x = ptr->b.posn.x * pc / 100;
		} else {
			ptr->b.height = ptr->b.height * pc / 100;
			ptr->b.posn.y = ptr->b.posn.y * pc / 100;
		}
		ptr = ptr->next;
	}
}
/*
 * R E S I Z E --
 *
 */
resize(number, size, inc, type)
int *number, *size, inc, type;
{
	int length, percent, oldnumber;
	
	display_group(current, 0, RED);
	oldnumber = *number;
	if (inc)
		*number += *size;
	else
		*number -= *size;
	if (type == WIDTH) {
		if (current->type == TXT) {
			length = strlen(current->txt);
			length *= *number;
			if (current->b.posn.x + length > MAXPAGEX)
				*number = (MAXPAGEX - current->b.posn.x) / strlen(current->txt);
		} else {
			if (current->b.posn.x + *number > MAXPAGEX)
				*number = MAXPAGEX - current->b.posn.x;
		}
	} else {
		if (current->b.posn.y + *number > MAXPAGEY)
			*number = MAXPAGEY - current->b.posn.y;
	}
	if (*number < 0)
		*number = 0;
	if (current->ass != NULL) {
		if (oldnumber > 0) {
			percent = *number * 100 / oldnumber;
			sizegroup(current->ass, percent, type);
		}
	}
	display_group(current, 1, RED);
}
/*
 * C H A N G E _ T E X T
 *
 */
change_text(tmpstr)
char *tmpstr;
{
	char *str;
	
	display_group(current, 0, RED);
	if (strlen(tmpstr) > DISPTEXT) {
		fprintf(stderr, "New string is too long.\n");
		strcpy(tmpstr, current->txt);
	} else {
		str = tmpstr;
		while (*str != '\0') {
			*str &= 0x7f;
			str++;
		}
		strcpy(current->txt, tmpstr);
	}
	display_group(current, 1, RED);
}
/*
 * D O T X T S T R
 *
 */
dotxtstr(txt, str)
int *txt;
char *str;
{
	if (*txt)
		strcpy(str, "text");
	else
		strcpy(str, "binary");
}
/*
 * T X T S I Z E
 *
 */
txtsize(width, height)
int *width, *height;
{
	int length;
	
	length = strlen(current->txt);
	length *= *width;
	if (current->b.posn.x + length > MAXPAGEX)
		*width = (MAXPAGEX - current->b.posn.x) / strlen(current->txt);
	if (current->b.posn.y + *height > MAXPAGEY)
		*height = MAXPAGEY - current->b.posn.y;
	display_group(current, 0, RED);
	current->b.width = *width;
	current->b.height = *height;
	display_group(current, 1, RED);
}
/*
 *	D I S P _ M E S S G  --  Display message telling user to press return
 *			
 */
static disp_messg()
{
	screengo(OUTLINE - 1,0);
	printf("Press <Return> to write file\n");
}
/*
 * W R I T E M E N U
 *
 */
writemenu()
{
	char txtstr[8];
	int Quit_cmd, cmd;
	
	dotxtstr(&txt, txtstr);
	newmenu("write output file");
	cmmd("output file name;:k40%s%-f", outfile, disp_messg);
	cmmd("type of output file;:k40%+t%-s%f", &txt, txtstr, dotxtstr);
	cmmd("quit"); Quit_cmd = mmcmd;

	while((cmd = thismu()) != Quit_cmd) {
		if (cmd == 0) {
			writefile(outfile, txt);
			filewritten = 1;
			break;
		}
	}
}
/*
 * B O X C U R S 2
 *
 */
boxcurs2()
{
	boxcurs(current);
}
/*
 * S E T T R A C K
 *
 */
settrack(inbox)
int *inbox;
{
	if (*inbox) {
		erasecursor();
		setmsv(curstrack);
		*inbox = 0;
	} else {
		display_box(current, 0, VIOLET);
		*inbox = 1;
		current->b.posn.x = (cursor.k << dg->xshift) - XOFFSET;
		current->b.posn.y = (cursor.l << dg->yshift) - YOFFSET;
		erasecursor();
		setmsv(boxcurs2);
	}
}
/*
 * G E T G R O U P
 *
 */
getgroup()
{
	struct hcobject *ptr, *prev, *p;
	
	ptr = hcpy->hco;
	prev = ptr;
	if (current->b.width < 0) {
		current->b.posn.x = current->b.posn.x + current->b.width;
		current->b.width *= -1;
	}
	if (current->b.height < 0) {
		current->b.posn.y = current->b.posn.y + current->b.height;
		current->b.height *= -1;
	}
	while (ptr != NULL) {
		if (inside(current, ptr)) { /* remove from list */
			if (ptr == prev) {
				prev = ptr->next;
				if (ptr == hcpy->hco)
					hcpy->hco = prev;
			}
			else if (ptr->next == NULL)
				prev->next = NULL;
			else
				prev->next = ptr->next;
			if (current->ass == NULL)
				current->ass = ptr;
			else {
				p = current->ass;	/* add to associated list */
				while (p->next != NULL) 
					p = p->next;
				p->next = ptr;
			}
			ptr->next = NULL;
			ptr = prev;
		} else {
			prev = ptr;
			ptr = ptr->next;
		}
	}
	ptr = hcpy->hco;
	if (ptr == NULL) {
		ptr = current;
		hcpy->hco = current;
	} else {
		while (ptr->next != NULL) /* add current to list */
			ptr = ptr->next;
		ptr->next = current;
	}
	current->next = NULL;
	display_group(current, 0, VIOLET);
	display_group(current, 1, BLUE);
	display_box(current, 0, BLUE);
	erasecursor();
	setmsv(curstrack);
}
/*
 * Q U I T _ G R O U P -- undisplay boxes in violet and free space
 *
 */
quit_group()
{
	struct hcobject *ptr;

	display_box(current, 0, VIOLET);
	ptr = hcpy->hco;
	while (ptr != NULL) {
		if (inside(current, ptr)) {
			if (ptr->b.col != BLUE) {
				display_group(ptr, 0, VIOLET);
				display_group(ptr, 1, BLUE);
			}
		}
		ptr = ptr->next;
	}
	Free(current);
}
/*
 * G R O U P
 *
 */
group()
{
	int inbox = 0, cmd, Qu_cmd, Ex_cmd;
	
	current = hclocate(0);
	display_group(current, 0, RED);
	display_group(current, 1, BLUE);
	erasecursor();
	setmsv(curstrack);
	current = (struct hcobject *) Calloc(sizeof(struct hcobject), 1);
	current->type = GROUP;
	current->b.width = 100;
	current->b.height = 100;
	current->next = NULL;
	current->ass = NULL;
	current->asstxt = -1;
	while (1) {
		downmenu();
		newmenu("Grouping");
		if (inbox) {
			cmmd("place top right hand corner of group;:k40%-+d%f", &inbox, settrack);
			mouse(MIDBT);
		} else {
			cmmd("place bottom left hand corner of group;:k40%-+d%f", &inbox, settrack);
			mouse(MIDBT);
			cmmd("accept grouping and exit;%f", getgroup);mouse(LONGMB);Ex_cmd = mmcmd;
		}
		cmmd("quit;%f", quit_group); Qu_cmd = mmcmd;
		while (cmd = thismu()) {
			if ((cmd != Ex_cmd) && (cmd != Qu_cmd))
				upmenu();
			break;
		}
		if (cmd == 0 || cmd == Qu_cmd || cmd == Ex_cmd)
			break;
	}
	erasecursor();
	setmsv(hccurs);
	upmenu();
}
/*
 * U N G R O U P
 *
 */
ungroup()
{
	struct hcobject *ptr, *p;

	display_group(current, 0, RED);	
	ptr = hcpy->hco;
	if (ptr == current) {
		hcpy->hco = current->ass;
		ptr = hcpy->hco;
	} else {
		while (ptr->next != current)
			ptr = ptr->next;
		ptr->next = current->ass;
	}
	display_all(current->ass, 1, BLUE);
	p = current->ass;
	while (p->next != NULL)
		p = p->next;
	p->next = current->next;
	current->next = NULL;
	current->ass = NULL;
	Free(current);
	current = hclocate(0);
}
