/*
 *      K M R C M E N U . C -- Program to change the KMRC user and system 
 *                             files by  a menu driven interface.
 *
 *
 *  Written: Sarah E. A. Burton.
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    8th January 1988.
 *
 *  Modifications
 *
 * 2   Feb 1991		IH		Added flexible screen to sys options menu
 * 24  Oct 1990		IH		Put processor type toggle option rather than setting pia base directly
 *  4  Oct 1990		IH		Added a piabase parm to cs struct in allcontrol
 * 10  Apr 1990     IH      Extended calcseat to take a range of seatno values
 *  9  Apr 1990     IH      Removed old colourdef references
 *  6  Apr 1990     IH      Changed seat menus and added network seats
 *  28 Mar 1990     IH      Added lut file option to ordinary seat config
 *	12 Dec 1989		GJP		Added menu options
 *	20 Oct 1989		IH		Added satellite archiving and process priority options
 *  22 May 1989		SEAJ	Added timing and made matrixfile varenhance.
 *  15 May 1989		SEAJ	Added cultures file.
 *  10 May 1989		SEAJ	Added fnamesep1 and 2 for filename separators, and
 *							DIR300_OBJ for laser3.
 *	20 Mar 1989		SEAJ	Added matrixfile for variable enhancement.
 *	31 Jan 1989		SEAJ	Made monitor be a string, added frontend, stagetype, 
 *							and seat<n>camera.
 *   7 Dec 1988		SEAJ	Added woolz level checking.
 *  17 Oct 1988		SEAJ	Corrected seat config bug
 *	26 Sep 1988		SEAJ	Modified personmenu so that it exits correctly.
 *  23 Sep 1988		SEAJ	Added screen/hardcopy filename stuff.
 *	22 Aug 1988		SEAJ	Added options into system stuff.
 *	19 Aug 1988		SEAJ	Changed seat configuration structure
 *	17 Aug 1988		SEAJ	Modified so that person can change speed & focus definitions
 *	27 Jul 1988		SEAJ	Added debug option flag.
 *  26 Jul 1988		SEAJ	More alldone stuff.
 *   1 Jul 1988		SEAJ	Moved alldone() calls for slave stuff.
 *  28 Apr 1988		SEAJ	Changed slave processors so that they always appear.
 *							Version 1.0
 *	11 Apr 1988		SEAJ	Added combicheck to check validity of check string.
 *   8 Apr 1988		SEAJ	Added in all transfer summary stuff.
 *   8 Apr 1988		SEAJ	Added in combifip version and check string.
 *
 */
#define INIFILE        /* define variables in this link */
#include <cmdsys.h>
#undef debug
#include <kmrcmenu.h>
#include <kmrctype.h>

extern fileread(), filewrite();

int topmenu(), personmenu();
int		system(), debug(), terminal(),seat(), satseat(), user(), aesthetics(), personal();
int     csconfig(), initialise(), initperson(), features();

int language(), copy(), strint(), intstr(), transfer(), optmenu();
int term(), smouse(), fstore(), config(), monitmenu(), frontmenu(), cameramenu();
int dispcol(), getdfile(), bayproc(), speedfoc(), calcbay(), calcseat(), 
	calcmov(), checkstrlen();
int kickslave();
void priority(), archive();
void seatconfig(), satseatconfig(), netseatconfig();

int develop, is_pfile, enable_debug;
char friend[30];
struct allcontrol thiscontrol, *controlptr;
struct allcontrol thisperson, *personptr;

/*
 *	I N T E R R U P T --  Signal catcher
 *
 */
interrupt(sig)
int		sig;
{
}
/*
 * M A I N
 *
 */
main(argc,argv)
int		argc;
char	**argv;
{
	char	*swptr, **targv, *param, who[20];
	int		targc, is_slave, csd_debug;
	FILE	*infile;

	is_slave = 0;
	csd_debug = 0;
	enable_debug = 0;
#ifdef INTERRUPT
	intercept(interrupt);
#endif INTERRUPT
	targc = argc; targv = argv;
	argc--; argv++;			/* dump process name */
	while (argc > 0) {
		if (*argv[0] == '-') {
			swptr = *argv;
			*argv = NULL;
			while (*(++swptr)) {
				switch (*swptr) {
				  case '?':
				  	printf("Syntax:   kmrcmenu <options> \n");
				  	printf("Function: Allows changes to be made to the KMRC files, by means of menus\n");
				  	printf("Options:  v - Show version number\n");
				  	printf("          u <username> - User File changes only\n");
				  	printf("          s - Slave program\n");
				  	printf("          d - Enable debug options\n");
				  	printf("          D - Enable cmdsys debug\n");
				  	printf("          ? - Help\n");
				  	exit(0);
				  case 'v':
					printf("kmrcmenu - Version %s\n",VERNO);
					exit(0);
				  case 'u':
				  	is_pfile = 1;
				  	if (argc <= 1){
						printf("Whose file do you wish to change?--- ");
						scanf("%s", friend);
					} else {
						param = *++argv;
						strcpy(friend, param);
						*argv = NULL; argc--;
					}
					break;
				  case 's':
				  	is_slave = 1;
				  	break;
				  case 'D':
				  	csd_debug = 1;
				  	develop = 1;
				  	break;
				  case 'd':
				  	enable_debug = 1;
				  	develop = 1;
				  	break;
				  default:
				 	printf("kmrcmenu - unknown option\n");
				}
			}
		}
		argv++; argc--;
	}
	/*
	 * This is the start of the M A I N program
	 *
	 */
	if (is_slave == 1) {
		if (is_pfile == 1)
			slavemain(targv[1], personmenu, kickslave);
		else
			slavemain(targv[1], topmenu, kickslave);
 	} else {
		muini(stdout);
		if (csd_debug)
			cmdsys_debug(CSD_AUTO, CSD_ALL);
		if (is_pfile == 1)
	 		personal(friend);
		else
	 		topmenu();
	}
}
/*
 * K I C K S L A V E -- dummy routine
 *
 */
kickslave(){}

/*
 * P E R S O N M E N U -- dummy to call personal
 *
 */
personmenu()
{
	controlptr = NULL;
	personal(friend);
}

/*
 * T O P M E N U -- The main menu for system changes
 *
 */
topmenu()
{
	int Quitcmd;
	char who[20];

	pastmu = 0;	
	who[0] = '\0';	
	initialise();
	fileread(SYS_RC, controlptr);
	newmenu("\033#6Configuration Editor");
	cmmd(":k20;system;%g",system);
	cmmd(":k20;debug options;%-+d%f",controlptr, debug);
	cmmd(":k20;terminal setup;%g",terminal);
	cmmd(":k20;seat control;%g",seatconfig);
	cmmd(":k20;single satellite control;%g",satseatconfig);
	cmmd(":k20;satellite network control;%g",netseatconfig);
	cmmd(":k20;user group information;%g",user);
	cmmd(":k20;aesthetic values;%-+d%f",controlptr,aesthetics);
	cmmd(":k20;optional features;%-+d%f",controlptr,features);
	cmmd(":k20;personal configurations...USER ;:k56%20s%f",who,personal);
	cmmd(":k20;cs system setup;%g",csconfig);
	cmmd(":k20;exit and update file;%x%-+d%f;t13",SYS_RC,controlptr,filewrite);
	cmmd(":k20;quit"); Quitcmd = mmcmd;
	while(thismu() != Quitcmd);
	alldone(0);
}	

/*
 * I N I T I A L I S E -- Initialises all variables to default values.
 *
 */
initialise()
{
	int i, j;

	controlptr = &thiscontrol;
	controlptr->is_CS = 0;
		
	strcpy(controlptr->sysdef.name, "");
	controlptr->sysdef.lang = ENGLISH;
	controlptr->sysdef.copy = VGR;
	controlptr->sysdef.score = 0;	/* not extended */
	controlptr->sysdef.master_priority = 128; /*initialise priorities*/
	controlptr->sysdef.slave_priority = 110;
	controlptr->sysdef.sat_priority = 128;
	controlptr->sysdef.archmedia[0] = 0; /*all archiving flags off by default*/
	controlptr->sysdef.archmedia[1] = 0; 
	controlptr->sysdef.archmedia[2] = 0;
	controlptr->sysdef.odcache = 400;
	for (j=0;j<3;j++)
		controlptr->sysdef.bgcolour[j] = 0;
	strcpy(controlptr->sysdef.od_arch_method,"");
	
	controlptr->trans.enable = 0;			/* disabled */
	controlptr->trans.line = 0;		/* not set */
	controlptr->trans.baud = 9600;
	strcpy(controlptr->trans.startup, "");
	strcpy(controlptr->trans.closedown, "");

/*
 * Set up default seat values
 */
	controlptr->seatdef[SEAT1].valid = 1;
	controlptr->seatdef[SEAT1].term = TERM;
	controlptr->seatdef[SEAT2].term = T2;
	controlptr->satseatdef.term = T3;	
	controlptr->seatdef[SEAT1].mouse = T5;
	controlptr->seatdef[SEAT2].mouse = T4;
	controlptr->seatdef[SEAT1].lutfile[0] = '\0';
	controlptr->seatdef[SEAT2].lutfile[0] = '\0';
	controlptr->satseatdef.lutfile[0] = '\0';
	for (i=1;i<MAXSEATS;i++) {
		controlptr->seatdef[i].valid = 0;
	}
	controlptr->satseatdef.valid = 0;
	controlptr->satseatdef.net = 0;
	for (i=0;i<MAXNETWORK;i++) {
		controlptr->netseatdef[i].valid = 0;
	}
	for (i=0;i<MAXSEATS;i++) {
		controlptr->seatdef[i].fstore.number = i+1;
		controlptr->seatdef[i].fstore.string = VME512;
		controlptr->seatdef[i].mproc = -1;
		for (j=0; j<MAXPROCS; j++) 
			controlptr->seatdef[i].slave[j] = -1;
		for (j=0; j <= MAXCONF; j++)
			controlptr->seatdef[i].config[j] = 0;
		for (j=0; j < MAXCAM; j++)
			controlptr->seatdef[i].camera[j] = 0;
	};
	controlptr->satseatdef.fstore.number = i+1;
	controlptr->satseatdef.fstore.string = VME512;	
	for (j=0;j < MAXCAM;j++)
		controlptr->satseatdef.camera[j] = 0;
	for (i=0;i<MAXNETWORK;i++) {
		for (j=0; j < MAXCAM; j++) {
			controlptr->netseatdef[i].camera[j] = 0;
		}
	}
	
	controlptr->termdef.printer = T3;
	controlptr->termdef.miprocterm = T1;
	controlptr->termdef.miprocbaud = 9600;
	
 	controlptr->sysdef.mousescale = 1;
	
	for (i=0; i < MAXOPTS; i++)
		controlptr->featdef.options[i] = 0;
	for (i=0; i < MAXMENUOPT; i++)
		controlptr->featdef.menu_options[i] = 0;
	controlptr->featdef.screenform[0] = '\0';
	controlptr->featdef.hardcopyform[0] = '\0';
	controlptr->featdef.varenhance[0] = '\0';
	controlptr->featdef.culturesfile[0] = '\0';
	 	
 	controlptr->debugdef.debug = DISABLE;
 	controlptr->debugdef.wclevel = DISABLE;
 	controlptr->debugdef.mipdebug = DISABLE;
 	controlptr->debugdef.devel = DISABLE;
 	controlptr->debugdef.monitor = DISABLE;
 	controlptr->debugdef.timing = DISABLE;
 	strcpy(controlptr->debugdef.debugfile, "/dd/tmp/kmr.bug");
 	
 	controlptr->userdef.usergroupnum = 1;
 	controlptr->userdef.onegroup = OFF;

	for(i = 0; i < MAXFRONT; i++)
		controlptr->cs.frontend[i] = 0;
	controlptr->cs.stagetype = 0;
	controlptr->cs.nbays = 4;
	controlptr->cs.sleft = -12900;
	controlptr->cs.sright = 12950; 	
 	controlptr->cs.speed.focus.permove = 56;
	controlptr->cs.speed.focus.inswathe = 500;
	controlptr->cs.speed.focus.between = 500;
	controlptr->cs.speed.focus.excursion = 2000;
	controlptr->cs.speed.movespeed = 4000;
	controlptr->cs.speed.scanspeed = 4000;
	controlptr->cs.speed.metthresh.low = 11;
	controlptr->cs.speed.metthresh.high = 33;
	controlptr->cs.speed.focen = DISABLE;
	controlptr->cs.speed.moven = DISABLE;
	controlptr->cs.speed.scanen = DISABLE;
	controlptr->cs.speed.thresen = DISABLE;
	controlptr->cs.bay[0].x = 45000;
	controlptr->cs.bay[0].y = 0;
	controlptr->cs.bay[0].z = 0;
	controlptr->cs.bay[1].x = 45000;
	controlptr->cs.bay[1].y = 0;
	controlptr->cs.bay[1].z = 0;
	controlptr->cs.bay[2].x = 45000;
	controlptr->cs.bay[2].y = 0;
	controlptr->cs.bay[2].z = 0;
	controlptr->cs.piabase=0xfe0800;
	strcpy(controlptr->cs.program, "");
	strcpy(controlptr->cs.check, "");
	controlptr->comptr = NULL;
}
/*
 * I N I T P E R S O N
 *
 */
initperson()
{
	int i;
	
	personptr = &thisperson;
	personptr->is_CS = 0;
 	
 	personptr->debugdef.debug = DISABLE;
 	personptr->debugdef.wclevel = DISABLE;
 	personptr->debugdef.mipdebug = DISABLE;
 	personptr->debugdef.devel = DISABLE;
 	personptr->debugdef.monitor = DISABLE;
 	personptr->debugdef.timing = DISABLE;
 	strcpy(personptr->debugdef.debugfile, "/dd/tmp/kmr.bug");
 	
 	personptr->cs.speed.focus.permove = 56;
	personptr->cs.speed.focus.inswathe = 500;
	personptr->cs.speed.focus.between = 500;
	personptr->cs.speed.focus.excursion = 2000;
	personptr->cs.speed.movespeed = 4000;
	personptr->cs.speed.scanspeed = 4000;
	personptr->cs.speed.metthresh.low = 11;
	personptr->cs.speed.metthresh.high = 33;
	personptr->cs.speed.focen = DISABLE;
	personptr->cs.speed.moven = DISABLE;
	personptr->cs.speed.scanen = DISABLE;
	personptr->cs.speed.thresen = DISABLE;
	
 	personptr->sysdef.mousescale = 1;
 	
	for (i=0; i < MAXOPTS; i++)
		personptr->featdef.options[i] = 0;
	for (i=0; i < MAXMENUOPT; i++)
		personptr->featdef.menu_options[i] = 0;
	personptr->featdef.screenform[0] = '\0';
	personptr->featdef.hardcopyform[0] = '\0';
	personptr->featdef.varenhance[0] = '\0';
	personptr->featdef.culturesfile[0] = '\0';

 	personptr->trans.enable = 0;	/* disabled */
	personptr->trans.line = 0;		/* not set */
	personptr->trans.baud = 9600;
	strcpy(personptr->trans.startup, "");
	strcpy(personptr->trans.closedown, "");
	personptr->comptr = NULL;
}
/************************************************
 *    Section for changing system configuration
 ************************************************/

/*
 * S Y S T E M -- main system menu for changes.
 *
 */
system()
{
	char name[30], score[10], lang[15], tmpstr[10], hcopy[15], archstr[MAXSTR], *ptr;
	int Exitcmd, i;
	
	strcpy(lang,languagetype[controlptr->sysdef.lang]);
	strcpy(hcopy,hardcopytype[controlptr->sysdef.copy]);
	ptr = archstr;
	for (i=0; i < MAXARCH; i++) {
		if (controlptr->sysdef.archmedia[i] == 1) {
			sprintf(ptr, "%s ", archmediatype[i]);
			ptr += strlen(ptr);
		}
	}
	newmenu("\033#6System Changes");
	cmmd(":k20;sitename;:k50 %50S",controlptr->sysdef.name);
	cmmd(":k20;language;:k50%-s%?+s%f",lang,tmpstr,language);
	cmmd(":k20;hardcopy;:k50%-s%f",hcopy,copy);
	cmmd(":k20;extended score menu;:k50%T",&controlptr->sysdef.score);
	cmmd(":k20;transfer summary;%-+d%f", controlptr, transfer);
	cmmd(":k20;process priorities;%-+d%f", controlptr, priority);
	cmmd(":k20;archive options;%-+d%-+d%f", controlptr, archstr, archive);
	while (thismu())
		;
}

/*
 *	L A N G U A G E  -- Routine which investigates the data entered, sets up
 *						the language if it has been entered, otherwise it
 *						calls the language menu.
 *
 *
 */
language(lang,tmpstr)
char	*lang,*tmpstr;
{
	int		i, j, ok;
	int		lang_is[MAXLANG];
	
	for (i=0; i<MAXLANG; i++)
		lang_is[i] = 0;
	if (controlptr->sysdef.lang != 0)
		lang_is[controlptr->sysdef.lang-1] = 1;
	if (*tmpstr != '\0'){
		ok = 0;
		j = 0;
		while (ok == 0 && j < MAXLANG){
			j++;
			ok = strcasecmp(tmpstr, languagetype[j]);
		};
		if (ok > 0){
			controlptr->sysdef.lang = j;
			for (i=0; i<MAXLANG; i++)
				lang_is[i] = 0;
			lang_is[j] = 1;
			strcpy(lang, languagetype[j]);
		} else
			langmenu(lang);
	}
	else
		langmenu(lang);
	strcpy(tmpstr, '\0');
}
	
/*
 * L A N G M E N U -- Language menu.
 *
 */
 
langmenu(lang)
char *lang;
{
	int Exitcmd, i, cmd;
	int lang_is[MAXLANG];
	
	for (i=0; i<MAXLANG; i++)
		lang_is[i] = 0;
	if (controlptr->sysdef.lang != 0)
		lang_is[controlptr->sysdef.lang-1] = 1;
	newmenu("\033#6Language Change");
	mmcmd = 0;
	cmmd(":k20;english;:k40%-t",&lang_is[mmcmd]);
	cmmd(":k20;german;:k40%-t",&lang_is[mmcmd]);
	while (cmd = thismu()) {
		if (cmd > 0){
			controlptr->sysdef.lang = cmd;
			for (i=0; i<MAXLANG; i++)
				lang_is[i] = 0;
			lang_is[cmd-1] = 1;
			mukont->mubust = 1;
		}
	}
	strcpy(lang,languagetype[controlptr->sysdef.lang]);
}

/*
 *	C O P Y  -- Main menu for changing the hard copy machine.
 *
 */
copy(hcopy)
char *hcopy;
{
	int Exitcmd, i, cmd;
	int copy_is[MAXCOPY];
	char copyname[10];

	for (i = 0; i<MAXCOPY; i++)
		copy_is[i] = 0;
	if (controlptr->sysdef.copy != 0)
		copy_is[controlptr->sysdef.copy-1] = 1;
	newmenu("\033#6Change Hard Copy Machine");
	mmcmd = 0;
	cmmd(":k20;honeywell VGR;:k60%-t",&copy_is[mmcmd]);
	cmmd(":k20;laserprinter;:k60%-t",&copy_is[mmcmd]);
	cmmd(":k20;spooled laser printer;:k60%-t",&copy_is[mmcmd]);
	cmmd(":k20;spooled laser printer with laser3;:k60%-t", &copy_is[mmcmd]);
	
	while (cmd = thismu()) {
		if (cmd > 0){
			controlptr->sysdef.copy = cmd;
			for (i=0; i<MAXCOPY; i++)
				copy_is[i] = 0;
			copy_is[cmd-1] = 1;
			mukont->mubust = 1;
		}
	}
	strcpy(hcopy, hardcopytype[controlptr->sysdef.copy]);
}

/*
 *  T R A N S F E R --  Sets up all transfer information.
 *
 */
transfer(controlptr)
struct allcontrol *controlptr;
{
	char transline[6], tmpstr[10];
	int cmd, baud, i, temp;
	
	tmpstr[0] = '\0';
	strcpy( transline, termtype[controlptr->trans.line]);
	baud = controlptr->trans.baud;
	while (1) {
		downmenu();
		newmenu("\033#6Transfer Summary");
		if (controlptr->trans.enable == 0)
			cmmd(":k20;enable transfer summary;%+t", &controlptr->trans.enable);
		else {
			cmmd(":k20;disable transfer summary;%+t", &controlptr->trans.enable);
			cmmd(":k20;line;:k50%-+d%-s%?+s%f", &controlptr->trans.line, transline,
														tmpstr, term);
			cmmd(":k20;baud;:k50<50/38400>%d", &baud);
			cmmd(":k20;startup sequence;:k50 %S", controlptr->trans.startup);
			cmmd(":k20;closedown sequence;:k50 %S", controlptr->trans.closedown);
		};
		while ( cmd = thismu()) {
			if (cmd == 3) {
				i = 0;
				while (baud >= validbaud[i])
					i++;
				temp = validbaud[i-1] + ((validbaud[i-1] - validbaud[i])/2);
				if (baud < temp) {
					controlptr->trans.baud = validbaud[i-1];
					baud = validbaud[i-1];
				} 
				else {
					controlptr->trans.baud = validbaud[i];
					baud = validbaud[i];
				}
			}
			upmenu();
			break;
		}
		if (cmd == 0)
			break;
	}
	upmenu();
}
/*
 * P R I O R I T Y -- process priority menu
 */
void
priority(controlptr)
struct allcontrol *controlptr;
{
	
	newmenu("\033#6Process Priorities");
	cmmd(":k20;master priority;:k50<0/65535>%d",&controlptr->sysdef.master_priority);
	cmmd(":k20;slave  priority;:k50<0/65535>%d",&controlptr->sysdef.slave_priority);
	cmmd(":k20;satellite  priority;:k50<0/65535>%d",&controlptr->sysdef.sat_priority);
	while (thismu());
}

/*
 * A R C H I V E -- archive options menu
 */
void
archive(controlptr,thisarch)
char *thisarch;
struct allcontrol *controlptr;
{
	char *strbeg;
	int i;

	newmenu("\033#6Archive Options");
 	cmmd(":k20;tape;:k55%T",&controlptr->sysdef.archmedia[0]);
 	cmmd(":k20;od;:k55%T",&controlptr->sysdef.archmedia[1]);
 	cmmd(":k20;floppy;:k55%T",&controlptr->sysdef.archmedia[2]);
	cmmd(":k20;OD cachesize;:k50%d",&controlptr->sysdef.odcache);
	cmmd(":k20;OD archive method;:k50%S",controlptr->sysdef.od_arch_method);
 	while (thismu()) ;
 	strbeg = thisarch;
	for (i=0; i < MAXARCH; i++) {
		if (controlptr->sysdef.archmedia[i] == 1) {
			sprintf(thisarch, "%s ", archmediatype[i]);
			thisarch += strlen(thisarch);
		}
	}
	*thisarch = '\0';
	thisarch = strbeg;
}

/*
 * O P T M E N U -- options menu
 *
 */
optmenu(number, thisopt)
int number[];
char *thisopt;
{
	int i;
	char *strbeg;
	 	
 	newmenu("\033#6System Options");
 	cmmd(":k20;field fusion;:k55%T",&number[0]);
 	cmmd(":k20;manual karyotype;:k55%T",&number[1]);
 	cmmd(":k20;fluorescence;:k55%T",&number[2]);
 	cmmd(":k20;flexible screen;:k55%T",&number[3]);
 	while (thismu()) ;
 	strbeg = thisopt;
	for (i = 0; i < MAXOPTS; i++) {
		if (number[i] == 1) {
			sprintf(thisopt, "%s ", stropt[i+1]);
			thisopt += strlen(thisopt);
		}
	}
	*thisopt = '\0';
	thisopt = strbeg;
}


/*
 * M E N U O P T M E N U -- options menu
 *
 */
menuoptmenu(number, thisopt)
int number[];
char *thisopt;
{
	int i;
	char *strbeg;
	 	
 	newmenu("\033#6Menu Options");
 	cmmd(":k20;stay in TV digitise;:k55%T",&number[0]);
 	while (thismu()) ;
 	strbeg = thisopt;
	for (i = 0; i < MAXMENUOPT; i++) {
		if (number[i] == 1) {
			sprintf(thisopt, "%s ", menuopt[i+1]);
			thisopt += strlen(thisopt);
		}
	}
	*thisopt = '\0';
	thisopt = strbeg;
}

/*
 *	D E B U G  --
 *
 */
debug(controlptr)
struct allcontrol *controlptr;
{
	char dbgstr[12], wcstr[12], monstr[18];
	int cmd, Dbg_cmd;
	
	convert(&controlptr->debugdef.debug, dbgstr);
	convert(&controlptr->debugdef.wclevel, wcstr);
	strcpy(monstr, monitortype[controlptr->debugdef.monitor]);
	while (1){
		downmenu();
		newmenu("\033#6Debug Options");
		cmmd(":k20;system monitor;:k55%-+d%-s%f",&controlptr->debugdef.monitor, 
										monstr, monitmenu);
		if (enable_debug) {
			cmmd(":k20;miproc debug options;:k55%T",&controlptr->debugdef.mipdebug);
			cmmd(":k20;debug timing information;:k55%T",&controlptr->debugdef.timing);
			cmmd(":k20;debug options;:k55<0/3>%+d%-s%f",&controlptr->debugdef.debug,
										dbgstr, convert);Dbg_cmd = mmcmd;
			cmmd(":k20;woolz checking;:k55<0/3>%+d%-s%f",&controlptr->debugdef.wclevel,
										wcstr, convert);
			if (develop == 1)
				cmmd(":k20;development options;:k55%T",&controlptr->debugdef.devel);
			if (controlptr->debugdef.debug > 0)
				cmmd(":k20;debug file;:k55 %50s",controlptr->debugdef.debugfile);
		}
		while (cmd = thismu()){
			if (cmd == Dbg_cmd) {
				upmenu();
				break;
			}
		}
		if ( cmd == 0 )
			break;				/* normal exit */
	}
	upmenu();
}

/*
 * M O N I T M E N U -- menu for selecting type of monitoring
 *
 */
monitmenu(number, string)
int *number;
char *string;
{
	int i, cmd;
	int monit_is[MAXTYPE];
	
	for (i=0; i<MAXTYPE; i++)
		monit_is[i] = 0;
	monit_is[*number] = 1;
	newmenu("\033#6Types of System Monitoring");
	mmcmd = 0;
	cmmd(":k20;monitoring off;:k55%-t",&monit_is[mmcmd]);
	cmmd(":k20;continuous system monitoring;:k55%-t",&monit_is[mmcmd]);
	cmmd(":k20;per case monitoring;:k55%-t",&monit_is[mmcmd]);
	while (cmd = thismu()) {
		if (cmd > 0){
			*number = cmd-1;
			for (i=0; i<MAXMON; i++)
				monit_is[i] = 0;
			monit_is[cmd-1] = 1;
			mukont->mubust = 1;
		}
	}
	strcpy(string, monitortype[*number]);
}
/*
 * T E R M I N A L -- main menu for changing the printer and microproc 
 *												information.
 */
terminal()
{
	char termprint[6], miproct[6], tmpstr[10];
	int baud, i, temp, cmd;
	
	tmpstr[0] = '\0';
	strcpy(termprint, termtype[controlptr->termdef.printer]);
	strcpy(miproct, termtype[controlptr->termdef.miprocterm]);
	baud = controlptr->termdef.miprocbaud;
	
	newmenu("\033#6Terminal Changes");
	cmmd(":k20;printer line;:k50%-+d%-s%?+s%f",&controlptr->termdef.printer,
									termprint,tmpstr,term);
	cmmd(":k20;miproc line;:k50%-+d%-s%?+s%f",&controlptr->termdef.miprocterm,
									miproct,tmpstr,term);
	cmmd(":k20;miproc baud rate;:k50<50/38400>%d",&baud);
	
	while (cmd = thismu()) {
		if (cmd == 3) {
			i = 0;
			while(i < 12)
				i++;
			i = 0;
			while (baud >= validbaud[i]){
				i++;
			};
			temp = validbaud[i-1] + ((validbaud[i] - validbaud[i-1])/2);
			if (baud < temp){
				controlptr->termdef.miprocbaud = validbaud[i-1];
				baud = validbaud[i-1];
			}
			else{
				controlptr->termdef.miprocbaud = validbaud[i];
				baud = validbaud[i];
			}
			mukont->mubust = 1;
		}
	}
}
/********************************************
 * This is the section for changing the 
 * seat configuration.
 ********************************************/
/*
 * S E A T -- Main menu for seat control
 */

/*
 * S E A T C O N F I G -- Sub menu for entering normal seat configuration.
 *
 */
void
seatconfig()
{
	char thisterm[6], thismouse[6], thisconfig[MAXSTRING], thisfstore[16];
	char tmpstr[MAXSTRING], procs[MAXSTRING], thiscamera[MAXSTRING], *ptr;
	int seatno;
	int cmd, Exitcmd, i;
	int firstseat = 1;
	int lastseat = MAXSEATS;
	
	seatno = 1;
	tmpstr[0] = '\0';
	procs[0] = '\0';
	intstr(procs, &seatno);
	strcpy(thisterm, termtype[controlptr->seatdef[SEAT1].term]);
	strcpy(thismouse, termtype[controlptr->seatdef[SEAT1].mouse]);
	strcpy(thisfstore, fstoretype[controlptr->seatdef[SEAT1].fstore.string]);
	ptr = thisconfig;
	for (i=0;i<MAXCONF;i++) {
		if (controlptr->seatdef[SEAT1].config[i] == 1) {
			sprintf(ptr, "%s ", configtype[i]);
			ptr += strlen(ptr);
		}
	}
	*ptr = '\0';
	ptr = thiscamera;
	for (i=0;i<MAXCAM;i++) {
		if (controlptr->seatdef[SEAT1].camera[i] == 1) {
			sprintf(ptr, "%s ", cameratype[i]);
			ptr += strlen(ptr);
		}
	}
	*ptr = '\0';
	do {
		downmenu();
		newmenu("\033#6Seat Configuration");
		cmmd(":k20;seat number ;:k55%d%-+d%-+d%f",
				&seatno, &firstseat, &lastseat, calcseat);
		cmmd(":k20;terminal;:k55%-+d%-s%?+s%f",
		        &controlptr->seatdef[seatno-1].term,thisterm,tmpstr,term);
		cmmd(":k20;frame store;%-+d:k55%-s:k72%-d%f",
		        &controlptr->seatdef[seatno-1], thisfstore,
		        &controlptr->seatdef[seatno-1].fstore.number,fstore);
		cmmd(":k20;configuration;:k55%-+d%-s%f",
		        controlptr->seatdef[seatno-1].config,thisconfig,config);
		cmmd(":k20;camera;:k55%-+d%-s%f",
		        controlptr->seatdef[seatno-1].camera,thiscamera, cameramenu);
		cmmd(":k20;mouse;:k55%-+d%-s%?+s%f",
		        &controlptr->seatdef[seatno-1].mouse,thismouse,tmpstr,term);
		cmmd(":k20;slave processors;:k55%30S%-+d%f",procs,&seatno,strint);
		cmmd(":k20;gamma correction file;:k55%s",
				controlptr->seatdef[seatno-1].lutfile);
		if (develop == 1) 
			cmmd(":k20:l20;mouse processor number;:k55<-1/7>%d",&controlptr->seatdef[seatno-1].mproc);
		cmd = thismu();
		if (cmd == 1) {
			intstr(procs, &seatno);
			strcpy(thisterm,
			       termtype[controlptr->seatdef[seatno-1].term]);
			strcpy(thismouse,
			       termtype[controlptr->seatdef[seatno-1].mouse]);
			strcpy(thisfstore,
			       fstoretype[controlptr->seatdef[seatno-1].fstore.string]);
			ptr = thisconfig;
			for (i=0;i<=MAXCONF;i++) {
				if (controlptr->seatdef[seatno-1].config[i] == 1) {
					sprintf(ptr, "%s ", configtype[i]);
					ptr += strlen(ptr);
				}
			}
			*ptr = '\0';
			ptr = thiscamera;
			for (i=0;i<MAXCAM;i++) {
				if (controlptr->seatdef[seatno-1].camera[i] == 1) {
					sprintf(ptr, "%s ", cameratype[i]);
					ptr += strlen(ptr);
				}
			}
			*ptr = '\0';
		}
		controlptr->seatdef[seatno-1].valid = 1;
		upmenu();
	} while (cmd);
}

/*
 * S A T S E A T C O N F I G -- Sub menu for entering satellite seat configuration.
 *
 */
void
killnet();
void
satseatconfig()
{
	char thisterm[6], thisconfig[MAXSTRING], thiscamera[MAXSTRING];
	char thisfstore[16];
	char tmpstr[MAXSTRING], procs[MAXSTRING], *ptr;
	int cmd, Exitcmd, i, flag;
	
	*tmpstr ='\0';
	strcpy(thisterm, termtype[controlptr->satseatdef.term]);
	strcpy(thisfstore,
	       fstoretype[controlptr->satseatdef.fstore.string]);
	ptr = thiscamera;
	*ptr = '\0';
	for (i=0;i<MAXCAM;i++) {
		if (controlptr->satseatdef.camera[i] == 1) {
			sprintf(ptr, "%s ", cameratype[i]);
			ptr += strlen(ptr);
		}
	}
	*ptr = '\0';
	do {
		downmenu();
		newmenu("\033#6Satellite Seat Configuration");
		cmmd(":k20;terminal;:k55%-+d%-s%?+s%f",
		    &controlptr->satseatdef.term,thisterm,tmpstr,term);
		cmmd(":k20;frame store;%-+d:k55%-s:k72%-d%f",
		    &controlptr->satseatdef,thisfstore,
		    &controlptr->satseatdef.fstore.number,fstore);
		cmmd(":k20;camera;:k55%-+d%-s%f",
	        controlptr->satseatdef.camera,thiscamera, cameramenu);
		cmmd(":k20;gamma correction file;:k55%s",
			controlptr->satseatdef.lutfile);
		cmmd(":k20;Disable Network;%-+d%f",
			  controlptr,killnet);
		cmd = thismu();
		upmenu();
		controlptr->satseatdef.valid = 1;
	} while (cmd);
}
/*
 * K I L L N E T -- temporary measure for killing network off
 */
void
killnet()
{
	int i;
	for (i=0;i<MAXNETWORK;i++) {
		controlptr->netseatdef[i].valid = 0;
	}
	controlptr->satseatdef.net = 0;
}
/*
 * N E T S E A T C O N F I G -- Sub menu for entering satellite network configuration.
 *
 */
void
netseatconfig()
{
	char thiscamera[MAXSTRING];
	char tmpstr[MAXSTRING], *ptr;
	char thislutfile[MAXSTRING];
	int netseatno, cmd, Exitcmd, i, flag;
	int lastseat = MAXNETWORK - 1;
	int firstseat = 0;
	
	netseatno = 0;
	do {
		tmpstr[0] = '\0';
		ptr = thiscamera;
		*ptr = '\0';
		for (i=0;i<MAXCAM;i++) {
			if (controlptr->netseatdef[netseatno].camera[i] == 1) {
				sprintf(ptr, "%s ", cameratype[i]);
				ptr += strlen(ptr);
			}
		}
		*ptr = '\0';
        
		downmenu();
		newmenu("\033#6Network Configuration");
		cmmd(":k20;network seat number ;:k55%d%-+d%-+d%f",
				&netseatno, &firstseat, &lastseat, calcseat);
		cmmd(":k20;camera;:k55%-+d%-s%f",
		   	    controlptr->netseatdef[netseatno].camera,thiscamera, cameramenu);
		cmmd(":k20;gamma correction file;:k55%s",
			controlptr->netseatdef[netseatno].lutfile);
		if ((cmd = thismu()) == 2) {
			ptr = thiscamera;
			*ptr = '\0';
			for (i=0;i<MAXCAM;i++) {
				if (controlptr->netseatdef[netseatno].camera[i] == 1) {
					sprintf(ptr, "%s ", cameratype[i]);
					ptr += strlen(ptr);
				}
			}
			*ptr = '\0';
		}
		upmenu();
		controlptr->netseatdef[netseatno].valid = 1;
		controlptr->satseatdef.net = 1;
	} while (cmd);
}

/*
 * T E R M --
 *
 */
term(number, thisterm, tmpstr)
int *number;
char *thisterm, *tmpstr;
{
	int i, j, ok;
	int term_is[MAXTYPE];
	
	for (i=0; i<MAXTYPE; i++)
		term_is[i] = 0;
	if (*number != 0)
		term_is[*number-1] = 1;
	if (*tmpstr != '\0'){
		ok = 0;
		j = 0;
		while (ok == 0 && j < MAXTYPE){
			j++;
			ok = strcasecmp(tmpstr, termtype[j]);
		};
		if (ok > 0){
			for (i=0; i<MAXTYPE; i++)
				term_is[i] = 0;
			term_is[j] = 1;
			*number = j;
			strcpy(thisterm, termtype[j]);
		}else
			termmenu(number, thisterm);
	} else
		termmenu(number, thisterm);
	strcpy(tmpstr, '\0');
}

/*
 * T E R M M E N U -- Menu which is used if optional string is not present
 *
 */
termmenu(number, thisterm)
int *number;
char *thisterm;
{
	int i, cmd;
	int term_is[MAXTYPE];
	
	for (i=0; i<MAXTYPE; i++)
		term_is[i] = 0;
	if (*number != 0)
		term_is[*number-1] = 1;
	newmenu("\033#6Terminal Types");
	mmcmd = 0;
	cmmd(":k20;/term;:k40%-t",&term_is[mmcmd]);
	cmmd(":k20;/t1;:k40%-t",&term_is[mmcmd]);
	cmmd(":k20;/t2;:k40%-t",&term_is[mmcmd]);
	cmmd(":k20;/t3;:k40%-t",&term_is[mmcmd]);
	cmmd(":k20;/t4;:k40%-t",&term_is[mmcmd]);
	cmmd(":k20;/t5;:k40%-t",&term_is[mmcmd]);
	while (cmd = thismu()) {
		if (cmd > 0){
			*number = cmd;
			for (i=0; i<MAXTYPE; i++)
				term_is[i] = 0;
			term_is[cmd-1] = 1;
			mukont->mubust = 1;
		}
	}
	strcpy(thisterm, termtype[*number]);
}
/*
 * F S T O R E --
 *
 */
 fstore(seat, thisstring)
 struct seatdefinition *seat;
 char *thisstring;
 {
 	int i, cmd;
 	int fstore_is[MAXFST];
 	
 	for (i=0; i<MAXFST; i++)
 		fstore_is[i] = 0;
  	if (seat->fstore.string != 0)
 		fstore_is[seat->fstore.string-1] = 1;
 	newmenu("\033#6Frame Store Changes");
 	mmcmd = 0;
 	cmmd(":k20;framestore location;:k60<1/4>%d",&seat->fstore.number);
 	cmmd(":k20;crs vme512;:k60%-t",&fstore_is[mmcmd-1]);
 	cmmd(":k20;inverted virtuoso;:k60%-t",&fstore_is[mmcmd-1]);
 	cmmd(":k20;virtuoso;:k60%-t",&fstore_is[mmcmd-1]);
 	while (cmd = thismu()) {
 		if (cmd > 1) {
 			seat->fstore.string = cmd-1;
 			for (i=0; i<MAXFST; i++)
 				fstore_is[i] = 0;
 			fstore_is[cmd-2] = 1;
 			mukont->mubust = 1;
 		}
 	}
 	strcpy(thisstring, fstoretype[seat->fstore.string]);
 };
 
/*
 * C O N F I G --
 *
 */
config(number, thisconfig)
int number[];
char *thisconfig;
{
 	int i;
	char *strbeg;
	 	
 	newmenu("\033#6Configuration Types");
 	cmmd(":k20;scanning stage;:k55%T",&number[0]);
 	cmmd(":k20;tv;:k55%T",&number[1]);
 	cmmd(":k20;metaphase list review;:k55%T",&number[2]);
 	cmmd(":k20;karyotype;:k55%T",&number[3]);
 	while (thismu()) ;
 	strbeg = thisconfig;
	for (i=0; i < MAXCONF; i++) {
		if (number[i] == 1) {
			sprintf(thisconfig, "%s ", configtype[i]);
			thisconfig += strlen(thisconfig);
		}
	}
	*thisconfig = '\0';
	thisconfig = strbeg;
 }
 
/*
 * C A M E R A M E N U -- menu for choosing seat camera options
 *
 */
cameramenu(number, string)
int number[];
char *string;
{
	int i, cmd;
	char *str;
	 	
 	newmenu("\033#6Camera Types");
 	cmmd(":k20;dage68;:k55%T",&number[0]);
 	cmmd(":k20;pulnix;:k55%T",&number[1]);
 	cmmd(":k20;mask;:k55%T",&number[2]);
 	while (cmd = thismu()) {
 		if ((number[0] == 1) && (number[1] == 1)) {
 			fprintf(stderr, "Dage68 and pulnix is an illegal combination!!\n");
 			number[cmd - 1] = 0;
 			displin(0, cmd);
 			putprompt();
 		}
 	}
 	str = string;
	for (i=0; i < MAXCAM; i++) {
		if (number[i] == 1) {
			sprintf(str, "%s ", cameratype[i]);
			str += strlen(str);
		}
	}
	*str = '\0';
 }
/*
 * S T R I N T -- converts slave string into integer array
 *
 */
strint(str, n)
char *str;
int *n;
{
	int j, number;
	char character, *tmp;
	
	tmp = str;
	j = 0;
	while (*str != '\0' && j < MAXPROCS) {
		character = *str;
		if (isdigit(character) != 0 || character == '-' || character == '+') {
			if (j < MAXPROCS) {
				number = atoi(str);
				if (number < MAXPROCS && number > -2)
					controlptr->seatdef[*n-1].slave[j] = number;
				else {
					printf("%d is not a valid slave number!\n",number);
					break;
				};
				if (character == '-' || character == '+')
					str++;
				str++;
				j++;
			}
		}
		else	
			str++;
	};
    while (j < MAXPROCS) {
    	controlptr->seatdef[*n-1].slave[j] = -1;
    	j++;
    };
    if (j = MAXPROCS)
    	*str = '\0';
    str = tmp;
    intstr(str, n);
}

/*
 * I N T S T R -- Converts the slave process numbers into a string
 *
 */
intstr(str, n)
char *str;
int *n;
{
	int j;
	
	j = 0;
	while (controlptr->seatdef[*n-1].slave[j] != -1 && j < MAXPROCS) {
		*str = controlptr->seatdef[*n-1].slave[j] + '0';
		j++;
		str++;
		*str = ' ';
		str++;
	};
	*str = '\0';
}

/******************************************************
 *   This is the section for changing user information
 ******************************************************/
 
/*
 * U S E R -- main menu for user information
 *
 */
user()
{
	newmenu("\033#6User Login Setup");
	cmmd(":k20;user, or group number;:k55<1/255>%d",&controlptr->userdef.usergroupnum);
	cmmd(":k20;onegroup;:k55%T",&controlptr->userdef.onegroup);
	while (thismu())
		mukont->mubust = 1;
}

/*
 * A E S T H E T I C S -- Main menu to change the background colour and mouse scale
 *
 */
aesthetics(controlptr)
struct allcontrol *controlptr;
{

	newmenu("\033#6Background Colour & Mouse Scale");
	cmmd(":k20;RED;:k50%d",&controlptr->sysdef.bgcolour[0]);
	cmmd(":k20;GREEN;:k50%d",&controlptr->sysdef.bgcolour[1]);
	cmmd(":k20;BLUE;:k50%d",&controlptr->sysdef.bgcolour[2]);
	cmmd(":k20;mouse movement scale;:k50<1/4>%d",
	      &controlptr->sysdef.mousescale);
	while (thismu());
}
/*
 * D I S P C O L -- Display the colour options
 *
 */
dispcol()
{
	screengo(6, 10);
	printf("Foreground colours :             Background colours : \n");
	printf("                     1 = GREY                         0 = NORMAL\n");
	printf("                     2 = FIRE                         1 = CLEAR\n");
	printf("                     3 = GRASS                        2 = SHADED\n");
	printf("					 4 = REVERSE					  3 = BLACK\n");
}
/*
 * F E A T U R E S -- Define options and screen/hardcopy forms
 *
 */
features(controlptr)
struct allcontrol *controlptr;
{
	char tmpstr[10], thisopt[MAXSTRING], thismenuopt[MAXSTRING], 
			fname1[MAXSTRING], fname2[MAXSTRING];
	char *ptr;
	int i;
	
	strcpy(fname1, controlptr->featdef.fnamesep1);
	strcpy(fname2, controlptr->featdef.fnamesep2);
	tmpstr[0] = '\0';
	ptr = thisopt;
	for (i=0; i < MAXOPTS; i++) {
		if (controlptr->featdef.options[i] == 1) {
			sprintf(ptr, "%s ", stropt[i+1]);
			ptr += strlen(ptr);
		}
	}
	*ptr = '\0';
	ptr = thismenuopt;
	for (i=0; i < MAXMENUOPT; i++) {
		if (controlptr->featdef.menu_options[i] == 1) {
			sprintf(ptr, "%s ", menuopt[i+1]);
			ptr += strlen(ptr);
		}
	}
	*ptr = '\0';

	newmenu("\033#6Optional Features");
	cmmd(":k10;system options;:k50%-+d%-s%f", controlptr->featdef.options, thisopt, optmenu);
	cmmd(":k10;menu options;:k50%-+d%-s%f", controlptr->featdef.menu_options, thismenuopt, menuoptmenu);
	cmmd(":k10;screen format filename;:k50%s", controlptr->featdef.screenform);
	cmmd(":k10;hardcopy format filename;:k50%s", controlptr->featdef.hardcopyform);
	cmmd(":k10;variable enhancement matrix file;:k50%s", controlptr->featdef.varenhance);
	cmmd(":k10;classifier cultures file;:k50%s", controlptr->featdef.culturesfile);
	cmmd(":k10;case name character translation for '_';:k60%s%-+s%f", 
							fname1, controlptr->featdef.fnamesep1, checkstrlen);
	cmmd(":k10;case name character translation for '.';:k60%s%-+s%f", 
							fname2, controlptr->featdef.fnamesep2, checkstrlen);
	
	while(thismu())
		;
}
/*
 * P E R S O N A L -- Change personal details
 *
 */
personal(who)
char *who;
{
	char buffer[60];
	int cmd, Quitcmd;
	struct allcontrol temp, *tmp;

	initperson();
	sprintf(buffer, USR_RC, who);
	fileread(buffer, personptr);
	if (personptr->is_CS != 1) {
		if (controlptr == NULL) {
			tmp = &temp;
			fileread(SYS_RC, tmp);
			if (tmp->is_CS == 1)
				personptr->is_CS = 1;
		} else if (controlptr->is_CS == 1)
				personptr->is_CS = 1;
	}
	
	newmenu("\033#6Personal Details");
	cmmd(":k20;debug options;%-+d%f", personptr, debug);
	if (personptr->is_CS == 1)
		cmmd(":k20;speed and focus options;%-+d%f", &personptr->cs.speed, speedfoc);
	cmmd(":k20;aesthetics;%-+d%f", personptr, aesthetics);
	cmmd(":k20;optional features;%-+d%f", personptr, features);
	cmmd(":k20;transfer summary;%-+d%f", personptr, transfer);
	if (personptr->is_CS == 1)
		cmmd(":k20;exit and update file;%x%-+d%f;t7",buffer, personptr, filewrite);
	else
		cmmd(":k20;exit and update file;%x%-+d%f;t6",buffer, personptr, filewrite);
	cmmd(":k20;quit");
	Quitcmd = mmcmd;
	while((cmd = thismu()) != Quitcmd) {
		if (cmd == 2)
			personptr->is_CS = 1;
	}
	if (is_pfile == 1)
		alldone(0);
}

/*
 *  S T A G E T O G -- togglethe stage string
 *
 */
stagetog(number, string)
int *number;
char *string;
{
	strcpy(string, stgtype[*number]);
}

/*
 *  P R O C T O G -- togglethe proc string
 *
 */
proctog(number, string)
int *number;
char *string;
{
	strcpy(string,proctype[*number]);
}

/*
 * C S C O N F I G -- main menu for changing the CS system
 *
 */
csconfig()
{
	int cmd, i, Combi_cmd, proc;
	char tmpstr[MAXCHECK],frontstr[MAXSTRING],stagestr[MAXSTRING],procstr[MAXSTRING];
	char piastr[10];
	char *ptr;
	
	if (controlptr->is_CS == 0)
		controlptr->is_CS = 1;

	ptr = frontstr;
	for (i=0; i<MAXFRONT; i++) {
		if (controlptr->cs.frontend[i] == 1) {
			sprintf(ptr, "%s ", fronttype[i]);
			ptr += strlen(ptr);
		}
	}
	*ptr = '\0';
	strcpy(stagestr, stgtype[controlptr->cs.stagetype]);
	strcpy(tmpstr, controlptr->cs.check);
	if (controlptr->cs.piabase==0xfe0800) {
		proc=0;
	}
	else {
		proc=1;
	}
	strcpy(procstr,proctype[proc]);
	newmenu("\033#6CS System Changes");
	cmmd(":k10;front end configuration;:k50%-+d%-s%f", controlptr->cs.frontend, 
														frontstr, frontmenu);
	cmmd(":k10;stage type;:k50%+t%-s%f", &controlptr->cs.stagetype, stagestr, 
														stagetog);
	cmmd(":k10;number of bays;:k50<1/4>%d",&controlptr->cs.nbays);
	cmmd(":k10;scan left limit;:k50<1/31000>%d",&controlptr->cs.sleft);
	cmmd(":k10;scan right limit;:k50<1/31000>%d",&controlptr->cs.sright);
	cmmd(":k10;speed/focus;%-+d%f",&controlptr->cs.speed, speedfoc);
	cmmd(":k10;bay changes;%f",bayproc);
	cmmd(":k10;combifip version;:k50%10s", controlptr->cs.program);
	cmmd(":k10;combifip check string;:k50%40S", tmpstr); Combi_cmd = mmcmd;
	cmmd(":k10;processor type;:k50%+t%-s%f",&proc,procstr,proctog);
	while(cmd = thismu()) {
		if (cmd == Combi_cmd) {
			if (combicheck(tmpstr) >= 0 ) {
				strcpy(controlptr->cs.check, tmpstr);
			} else {
				printf("This is not a valid check string!\n");
				strcpy( tmpstr, controlptr->cs.check);
				mukont->mubust = 1;
			}
		}
	}
	controlptr->cs.piabase=(proc?0xff008300:0xfe0800);
}
/*
 *  F R O N T M E N U -- sets up front end string
 *
 */
frontmenu(number, string)
int number[];
char *string;
{
	int i;
	char *str;
	 	
 	newmenu("\033#6Front End Configuration");
 	cmmd(":k20;analogue;:k55%t",&number[0]);
 	while (thismu()) ;
 	str = string;
	for (i = 0; i < MAXFRONT; i++) {
		if (number[i] == 1) {
			sprintf(string, "%s ", fronttype[i]);
			string += strlen(string);
		}
	}
	*string = '\0';
	string = str;
}

/*
 * S P E E D F O C -- Sets up the speeds and focus values.
 *
 */
speedfoc(speed)
struct speeddefinition *speed;
{
	int cmd;
	
	while(1) {
		downmenu();
		newmenu("\033#6Speed/Focus Changes");
		if (speed->focen != 0)
			cmmd(":k20;disable focus definitions;%+t",&speed->focen);
		else
			cmmd(":k20;enable focus definitions;%+t",&speed->focen);
		if (speed->moven != 0)
			cmmd(":k20;disable move speed;:k50%+t",&speed->moven);
		else
			cmmd(":k20;enable move speed;%+t",&speed->moven);
		if (speed->scanen != 0)
			cmmd(":k20;disable scan speed;%+t",&speed->scanen);
		else
			cmmd(":k20;enable scan speed;%+t",&speed->scanen);
		if (speed->thresen != 0)
			cmmd(":k20;disable thresholds;%+t",&speed->thresen);
		else
			cmmd(":k20;enable threshold definitions;%+t",&speed->thresen);
		if (speed->focen != 0) {
			cmmd(":k20;focus steps per move;:k60<1/16000>%d%f",&speed->focus.permove, calcmov);
			cmmd(":k20;focus steps in swathe;:k60<1/7400>%d",&speed->focus.inswathe);
			cmmd(":k20;focus steps between swathes;:k60<1/20000>%d",&speed->focus.between);
			cmmd(":k20;focus total excursion;:k60<1/20000>%d",&speed->focus.excursion);
		};
		if (speed->moven != 0)
			cmmd(":k20;move speed;:k60<1/16000>%d",&speed->movespeed);
		if (speed->scanen != 0)
			cmmd(":k20;scan speed;:k60<1/7400>%d",&speed->scanspeed);
		if (speed->thresen != 0) {
			cmmd(":k20;metaphase low threshold;:k60<8/60>%d",&speed->metthresh.low);
			cmmd(":k20;metaphase high threshold;:k60<8/60>%d",&speed->metthresh.high);
		};
		while(cmd = thismu()) {
			upmenu();
			break;
		}
		if (cmd == 0)
			break;
	}

}
/*
 * C A L C M O V -- Calculates nearest multiple of 128
 *
 */
calcmov(number)
int *number;
{
	int mod;
	
	mod = *number % 28;
	if (mod <= 14)
		*number = *number - mod;
	else
		*number = *number + (28 - mod);
}
 
/*
 * B A Y P R O C
 *
 */
bayproc()
{
	int cmd, from, to;
	
	from = 1;
	to = 2;
	while (1) {
		downmenu();
		newmenu("\033#6Bay Distance Changes");
		cmmd(":k20;distance between bays;:k47%d:k50and:k54%-d%-+d%f",&from,
												&to,&controlptr->cs.nbays,calcbay);
		cmmd(":k20;bay x value;:k40%d",&controlptr->cs.bay[from-1].x);
		cmmd(":k20;bay y value;:k40%d",&controlptr->cs.bay[from-1].y);
		cmmd(":k20;bay z value;:k40%d",&controlptr->cs.bay[from-1].z);
		while (cmd = thismu()) {
			if (cmd == 1){
				upmenu();
				break;
			}
		}
		if (cmd == 0)
			break;
	}
	upmenu();				/* normal exit */
}
/* 
 * C A L C B A Y
 *
 */
calcbay(from, to, nbays)
int *nbays, *to, *from;
{
	if (*from >= *nbays)
		*from = *nbays - 1;
	if (*from < 1)
		*from = 1;
	*to = *from + 1;
}
/*
 * C A L C S E A T -- Check for valid entry of seat number default to nearest
 *
 */
calcseat(seatno, firstseat, lastseat)
int *seatno,*firstseat,*lastseat;
{
	if (*seatno > *lastseat)
		*seatno = *lastseat;
	if (*seatno < *firstseat)
		*seatno = *firstseat;
}
/*
 * C H E C K S T R L E N -- check string length for fnamesep(s)
 *
 */
checkstrlen(str1, str2)
char *str1, *str2;
{
	if (strlen(str1) > MAXSEP) {
		fprintf(stderr, "This string is too long, only one character is allowed\n");
		*str2 = *str1;
		*(str2+1) = '\0';
		strcpy(str1, str2);
	} else
		strcpy(str2, str1);
}
/*
 *  S T R C A S E C M P -- String comparison taking no notice of case.
 *
 */
/*strcasecmp(s1,s2)
char *s1,*s2;
{
	int match;

	match = 1;
	while (*s1 && *s2) {
		if (toupper(*s1 & 0x7f) != toupper(*s2 & 0x7f)){
			match = 0;
			break;
		};
		s1++; s2++;
	}
	return(match);
}
*/
/*
 *  C O M B I C H E C K -- checks if combifip check value is valid.
 *
 */
int combicheck(str)
char *str;
{
	int i, j, ok;
	char ch;
	
	i = 0;
	ok = 0;
	if (strlen(str) > 39)
		return(-1);

	if (strlen(str) == 0)
		return(ok);	
		
	while ( i <= 37) {
		for (j = 1; j <= 4; j++) {
			ch = str[i];
			if ((ch >= '0' && ch <= '9') || (ch >= 'A' && ch <= 'F'))
				ok = 1;
			else
				return(-1);
			i++;
		}
		if (i < 38) {
			ch = str[i];
			if ( ch != ' ')
				return(-1);
			else
				i++;
		}
	}
	return(ok);
}
			
/*
 * C O N V E R T -- convert debug numbers into strings
 *
 */
convert(number, string)
int *number;
char *string;
{
	strcpy(string, debug_strings[*number]);
}
