/*
 *      K M R C M E N U . H -- Header file for : kmrcmenu.c,
 *												 fileprocs.c.
 *
 *
 *  Written: Sarah E. A. Burton.
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    8th January 1988.
 *
 *  Modifications
 *	 2 Feb 1991		IH		Increase MAXOPTS to 4
 *	 4 Oct 1990		IH		Added piabase parm to cs struct
 *   9 Apr 1990     IH      Removed colourdef structure
 *   3 Apr 1990     IH      Moved MAXPOSS def to new file allposs.h
 *  30 Mar 1990     IH		Added satellite and network seat definitions
 *  28 Mar 1990     IH      Added lutfile to seatdefinition structure
 *	12 Dec 1989		GJP		Added menu_option stuff and sat priority
 *	27 Oct 1989		IH		Added version number and satellite seat defs
 *  22 May 1989		SEAJ	Added timing and made matrixfile varenhance.
 *  15 May 1989		SEAJ	Added cultures file.
 *  10 May 1989		SEAJ	Added fnamesep1 and 2 for filename separators, and
 *							DIR300_OBJ for laser3.
 *	20 Mar 1989		SEAJ	Added matrixfile for variable enhancement.
 * 	31 Jan 1989		SEAJ	Made monitor be a string, added frontend, stagetype, 
 *							and seat<n>camera.
 * 	 7 Dec 1988		SEAJ	Added woolz checking level. 
 * 	23 Sep 1988		SEAJ	Added screen/hardcopy filename stuff.
 * 	22 Aug 1988		SEAJ	Added options into system definition.
 * 	19 Aug 1988		SEAJ	Changed seat configuration structure.
 * 	16 May 1988		SEAJ	Added in comment structure for non-removeable comments.
 * 	 8 Apr 1988		SEAJ	Added in combifip version and check.
 * 	 7 Apr 1988		SEAJ	Added in transfer summary definitions.
 *
 */
#define SYS_RC "/dd/sys/kmrc"
#define USR_RC "/dd/usr/%s/kmrc"

/* Current version number */
#define	VERNO	"1.5"

#define BUFLEN 60		/* Max buffer length */
#define MAXSTRING 30	/* Max string length */
#define MAXSTR 450   	/* Max string length for setup and closedown */
#define MAXFILNAM 80	/* Max filename length */
#define MAX_NAME_LEN 50 /* Max number of chars in any name */
#define MAXLANG 2		/* Max number of languages avail */
#define MAXCOPY 4       /* Max number of copy machines avail */
#define MAXTERM 2		/* Max number of poss terminals */
#define MAXSEATTYPE 6	/* Max number of poss seat options */
#define MAXTYPE 6		/* Max number of terminal types */
#define MAXCONF 4		/* Max number of configuration types */
#define MAXARCH 3		/* Max number of archive methods */
#define MAXFST 3		/* Max number of fstore types */
#define MAXSEATS 2		/* Max number of seats possible */
/* Max number of satellite network seats possible */
#define MAXNETWORK 8
#define MAXBAYS 3		/* Number of bays - 1, for bay<n>x,y,z */
#define MAXPROCS 8      /* Max number of mouse processes */
#define MAXPROGLEN 10  	/* Max program name length for combifip */
#define MAXCHECK 40		/* Max string length for combifip check */
#define MAXFRONT 1		/* Max number of frontend options */
#define MAXOPTS 4		/* Max number of options available */
#define MAXMENUOPT 1	/* Max number of menu selection options */
#define MAXSEP 2		/* Max number of characters for filename separators */
#define MAXMON 3		/* Max number of monitoring options allowed */
#define MAXCAM 3		/* Max number of camera options */
#define SEAT1 0			/* Defines array indices for seat array */
#define SEAT2 1
#define SEAT 0			/* Non satellite system type */
#define SAT 1			/* Single satellite tag value */
#define NET 2			/* Satellite network tag value */

#define ENGLISH 1 		/* Language definitions to make code readable */
#define GERMAN  2
#define VGR 1			/* Hard copy types to make code readable */
#define DIR300 2

#define DIR300_QUE 3	/* Terminal types to make code readable */ 
#define TERM 1
#define T1 2
#define T2 3
#define T3 4
#define T4 5
#define T5 6

#define VME512 1		/* Frame store types to make code readable */
#define VIRTUOSO_INVERT 2
#define VIRTUOSO 3

#define FIP 1			/* Configuration types to make code readable */
#define TV 2
#define NOFIP 3
#define NONE 4

#define GREY 1			/* Define foreground and background colours */
#define FIRE 2
#define GRASS 3
#define CLEAR 4
#define SHADE 5
#define REVERSE 6

#define DISABLE 0		/* Define enable and disable */
#define ENABLE 1

struct sysdefinition {
	char name[MAX_NAME_LEN];
	int lang;
	int copy;
	int score;
	int master_priority;
	int slave_priority;
	int sat_priority;
	int archmedia[MAXARCH];		/*archiving option flags*/
	int odcache;		/* od cache size*/
	char od_arch_method[MAX_NAME_LEN];
	int bgcolour[3];
	int mousescale;
};
struct fstoredefinition {
	int number;
	int string;
};
/* control structure for an ordinary seat */
struct seatdefinition {
	int term;
	int mouse;
	int mproc;
	struct fstoredefinition fstore;
	int config[MAXCONF];
	int slave[MAXPROCS];
	int camera[MAXCAM];
	char lutfile[MAXSTRING];
	char valid;
	int dummy;
};
/* control structure for a single satellite seat */
struct satseatdefinition {
	int term;
	int dummy1;
	int dummy2;
	struct fstoredefinition fstore;
	int dummy3[MAXCONF];
	int dummy4[MAXPROCS];
	int camera[MAXCAM];
	char lutfile[MAXSTRING];
	char valid;
	int net;		/* temp flagg for network */
};
/* control structure for a satellite network seat */
struct netseatdefinition {
	int dummy0;
	int dummy1;
	int dummy2;
	struct fstoredefinition dummy3;
	int dummy4[MAXCONF];
	int dummy5[MAXPROCS];
	int camera[MAXCAM];
	char lutfile[MAXSTRING];
	char valid;
	int dummy6;
};
struct termdefinition {
	int printer;
	int miprocterm;
	int miprocbaud;
};
struct featdefinition {
	int options[MAXOPTS];
	int menu_options[MAXMENUOPT];
	char screenform[MAXFILNAM];
	char hardcopyform[MAXFILNAM];
	char varenhance[MAXFILNAM];
	char culturesfile[MAXFILNAM];
	char fnamesep1[MAXSEP];
	char fnamesep2[MAXSEP];
};
struct debugdefinition {
	char debugfile[MAX_NAME_LEN];
	int debug;
	int wclevel;
	int mipdebug;
	int devel;
	int monitor;
	int timing;
};
struct userdefinition {
	int usergroupnum;
	int onegroup;
};
struct focusdefinition {
	int permove;
	int inswathe;
	int between;
	int excursion;
};
struct thrhdefinition {
	int low;
	int high;
};
struct speeddefinition {
	struct focusdefinition focus;
	int movespeed;
	int scanspeed;
	struct thrhdefinition metthresh;
	int focen;
	int moven;
	int scanen;
	int thresen;
};
struct baydefinition {
	int x;
	int y;
	int z;
};
struct csdefinition {
	int nbays;
	int sleft;
	int sright;
	struct speeddefinition speed;
	struct baydefinition bay[MAXBAYS];
	char program[MAXPROGLEN];
	char check[MAXCHECK];
	int frontend[MAXFRONT];
	int stagetype;
	int piabase;
};
struct transfer {
	int enable;
	int baud;
	int line;
	char startup[MAXSTR];
	char closedown[MAXSTR];
};

struct comment {
	int area;
	struct comment *nextcom, *current;
	char comstr[BUFLEN];
};
struct allcontrol {
	int is_CS;
	struct sysdefinition sysdef;
	struct seatdefinition seatdef[MAXSEATS];
	struct satseatdefinition satseatdef;
	struct netseatdefinition netseatdef[MAXNETWORK];
	struct termdefinition termdef;
	struct featdefinition featdef;
	struct debugdefinition debugdef;
	struct userdefinition userdef;
	struct csdefinition cs;
	struct transfer trans;
	struct comment *comptr;
};
struct date {
		int daynum;
		char *day;
		char *month;
		int year;
		int hour;
		int minute;
};


