/*
 *      L A S E R W R I T E . C  --  Take framestore image + produce postscript
 *									  image file
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    21st August 1988
 *
 *  Modifications
 *
 */
#include <stdio.h>
#include <ddgs.h>

/*
 *	Local general storage
 */
DDGS	*dg;				/* DDGS device structure */

/*
 *	External routines
 */
DDGS	*ddgsopen();

/*
 *	I N T E R R U P T --  Signal catcher
 *
 */
interrupt(sig)
int		sig;
{
}

main(argc,argv)
int		argc;
char	**argv;
{
	char	*swptr, **targv, **sargv;
	int		targc;
	int		store = FS_VME512, devno = 0;
	int		cmdsys_slave = 0;

#ifdef INTERRUPT
	intercept(interrupt);
#endif INTERRUPT
	targc = argc; targv = argv;
	argc--; argv++;			/* dump process name */
	while (argc > 0) {
		if (*argv[0] == '-') {
			swptr = *argv;
			*argv = NULL;
			while (*(++swptr)) {
				switch (*swptr) {
					case '?':
						printf("Syntax:   laserwrite <options>\n");
						printf("Function: \n");
						printf("Options:  v - Show version number\n");
						printf("          V - Virtuoso framestore (Default CRS 512)\n");
						printf("          I - Unmodified Virtuoso framestore\n");
						printf("          1 - Use seat 1 framestore\n");
						printf("          2 - Use seat 2 framestore\n");
						printf("                  also 3 and 4\n");
						printf("          ? - Help\n");
						exit(0);
					case 'v':
						printf("laserwrite - Version 0.1\n");
						exit(0);
					case '1':
					case '2':
					case '3':
					case '4':
						devno = *swptr - '1';
						break;
					case 'V':
						store = FS_VIRTUOSO_INVERT;
						break;
					case 'I':
						store = FS_VIRTUOSO;
						break;
/*					case 'x':
						param = *++argv;
						*argv = NULL; argc--;
						break;
 */
					default:
						printf("Piced - unknown option\n");
				}
			}
		}
		argv++; argc--;
	}
#ifdef PARAMS
/*
 * Compact params to fill holes left by switches
 */
	argc = --targc; argv = ++targv;				/* dump process name */
	sargv = targv;
	while (targc-- > 0) {						/* Squeez + fill the holes */
		if (*targv)
			*sargv++ = *targv;
		else
			argc--;
		targv++;
	}
/*
 *	Use params 
 */
	if (argc) {
		while (argc--) {
			if ((infile = fopen(*argv,"r")) == NULL) {
				fprintf(stderr,"Can't open file %s\n",*argv);
			}
			dofile(infile);
			fclose(infile);
			argv++;
		}
	} else {
		dofile(stdin);
	}
#endif PARAMS
	dg = ddgsopen(store,devno);
	dinit();
	dumpps(stdout);
}


dumpps(fp)
FILE *fp;
{
	int		i,j,k,byt;
	register unsigned char	*scr1;
	register unsigned short	*scr2;
	fprintf(fp,"%%!\n");
	fprintf(fp,"%% Screen image generated by laserwrite\n");
	fprintf(fp,"%%\n");
	fprintf(fp,"/concatprocs\n  { /proc2 exch cvlit def\n");
	fprintf(fp,"    /proc1 exch cvlit def\n");
	fprintf(fp,"    /newproc proc1 length proc2 length add\n");
	fprintf(fp,"      array def\n");
	fprintf(fp,"    newproc 0 proc1 putinterval\n");
	fprintf(fp,"    newproc proc1 length proc2 putinterval\n");
	fprintf(fp,"    newproc cvx\n");
	fprintf(fp,"  } def\n");
	fprintf(fp,"/inch { 72 mul } def\n");
	fprintf(fp,"/picstr %d string def\n",dg->lineln/8);
	fprintf(fp,"/imagelogo\n  { %d %d 1 [%d 0 0 %d 0 %d]\n",dg->lineln,dg->lineno,
						dg->lineln,-dg->lineno,dg->lineno);
	fprintf(fp,"     { currentfile picstr\n");
	fprintf(fp,"    readhexstring pop } image\n");
	fprintf(fp,"  } def\n\n");
	fprintf(fp,"  0.5 inch 3 inch translate\n");
	fprintf(fp,"  7 inch 5.25 inch scale\n");
	fprintf(fp,"gsave\n");
	fprintf(fp,"  {1 exch sub} currenttransfer concatprocs settransfer\n");
	fprintf(fp,"\n\n");
	fprintf(fp,"imagelogo\n");
	switch (dg->framepacking) {
		case BYTEPACKED:
			scr1 = dg->frame;
			for (i=0; i<dg->lineno; i++) {
				for (j=0; j<dg->lineln; j+=8) {
					byt = 0;
					for (k=0;k<8;k++) {
						byt = byt << 1;
						byt = byt | (*scr1++ ? 1 : 0);
					}
					fprintf(fp,"%02X",byt);
				}
				fprintf(fp,"\n");
			}
			break;
		case SHORTPACKED:
			scr2 = (unsigned short *) dg->frame;
			for (i=0; i<dg->lineno; i++) {
				for (j=0; j<dg->lineln; j+=8) {
					byt = 0;
					for (k=0;k<8;k++) {
						byt = byt << 1;
						byt = byt | (*scr2++ ? 1 : 0);
					}
					fprintf(fp,"%2X",byt);
				}
				fprintf(fp,"\n");
			}
			break;
	}
	fprintf(fp,"\ngrestore\nshowpage\n");
}
