/*
 *      N E W N A M E . C  --  Change cytoscan cell name to new format
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *  Date:    4th December 1986
 *
 *	This no longer compiles in the new system.  It is left here in case
 *	there is ever a requirement to do this again.  I doubt it will ever be
 *	required
 *						CAS 10th February 1988
 *
 *  Modifications
 *
 */

#include <stdio.h>
#include <fileheaders.h>
#include <slides.h>

/*
 *	data structure for kmr version 0.1 met header
 */
#define IDLONG 9	/* space for specimen id in header record */
#define SLONG 6 	/* space for slide id in header record */
#define SCLONG 4	/* space for scorer's name in header record */
#define TSLONG 12	/* space for tissue-comment in header record */
#define STLONG 10	/* space for fstain-comment in header record */
#define AILONG 10	/* space for aim-comment in header record */

struct hstruct { 
/*  0 */ short nrecs;	/* no. of records, excluding header */
/*  2 */ short comments; /* amount of storage requd for comments
			    (on files, appended) */
/*  4 */ short scored;
/*  6 */ char abnormals;
/*  7 */ char dics;
/*  8 */ char frags;
/*  9 */ char rings;
/* 10 */ char patient[IDLONG];
/* 19 */ char slide[SLONG];
/* 23 */ char scorer[SCLONG];
/* 29 */ char date[3];
/* 32 */ char tissue[TSLONG];
/* 44 */ char fstain[STLONG];
/* 54 */ char aim[AILONG];
};

/*
 *	Old filename format lengths
 */
#define FNAMECASE	15
#define CELLLENGTH	4

struct propertylist *makeheader();

int		eflag = 0;		/* Actually do change if set */
int		hflag = 0;		/* Insert headers */
int		debug = 0;		/* Debug if true */
int		sflag = 0;		/* Silent mode flag */
int		zflag = 0;		/* Get filenames from stdin */


main(argc,argv)
int		argc;
char	**argv;
{
	char	*swptr;
	FILE	*infile;
	char	buffer[128];

	argc--; argv++;
	while ((argc > 0) && (*argv[0] == '-')) {
		swptr = *argv++; argc--;
		while (*(++swptr)) {
			switch (*swptr) {
			  case '?':
			  	printf("newname - change name of old format cytoscan file\n");
			  	printf("Syntax  - newname <options> [file name list]\n");
			  	printf("Options -  e    Execute output\n");
			  	printf("           h    Insert file headers (implies -e)\n");
			  	printf("           d    Give debug info\n");
			  	printf("           s    Silent mode\n");
			  	printf("           z    get filenames from stdin\n");
			  	printf("           v    Print version\n");
			  	exit(0);
			  case 'd':
			  	debug = 1;
			  	break;
			  case 'v':
			  	printf("Newname Version 1.2  (c) Image Recognition Systems\n");
			  	exit(0);
			  case 'e':
			  	eflag = 1;
			  	break;
			  case 'h':
			  	hflag = 1;
			  	break;
			  case 's':
			  	sflag = 1;
			  	break;
			  case 'z':
			  	zflag = 1;
			  	break;
			  default:
			  	;
			}
		}
	}
	if (zflag) {
		while (fgets(buffer,127,stdin)) {
			*(buffer+strlen(buffer)-1) = '\0';
			if (doname(buffer) == 0)
				fprintf(stderr,"File name %s is not Cytoscan format\n",buffer);
		}
	} else if (argc) {
		while (argc) {
			if (doname(*argv) == 0)
				fprintf(stderr,"File name %s is not Cytoscan format\n",*argv);
			argc--; argv++;
		}
	}
}

/*
 *	D O N A M E 
 */
doname(name)
char *name;
{
	char *slide, *cellno, *ext, *temp, *temp2;
	int		kar, met, dat, cell, typ;
	char	work[30], new[30], command[100];

	kar = met = dat = cell = 0;
	slide = cellno = ext = NULL;
	extractname(work,name);
/*
 *	Do bit at beginning
 */
 	if ((slide = (char *) rindex(work,'_')) == NULL) {
		if (index(work,'.') == NULL) return(0);
		slide = NULL;
	} else {
		if ((temp = (char *) index(slide,'.')) == NULL) return(0);
		if ((temp2 = (char *) bsearch(slide,work,'_')) != NULL) {
			if (((temp-temp2) <= 4) && (temp2 != work))
				slide = temp2;
		} else if (temp == slide+1)
			slide = NULL;
	}
	if (slide) slide++;
/*
 * Do bit at end
 */
 	temp = (char *) rindex(work,'.');
	if (temp == NULL)
		return(0);
	switch (*(temp+1)) {
	case 'k':
	case 'K':
		if (*(temp+2) != '\0')
			return(0);
		ext = temp+1;
		*temp = '\0';
		kar = 1;
		if ((temp = (char *) bsearch(temp,work,'.')) == NULL) return(0);
		if (slide && (temp < slide)) return(0);
		if ((*(temp+1) < '0') || (*(temp+1) > '9')) return (0);
		cellno = temp+1;
		*temp = '\0';
		break;
	case 'm':
	case 'M':
		if (*(temp+2) != '\0')
			return(0);
		ext = temp+1;
		*temp = '\0';
		met = 1;
		break;
	case 'd':
	case 'D':
		if (*(temp+2) != '\0')
			return(0);
		ext = temp+1;
		*temp = '\0';
		dat = 1;
		break;
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
		cellno = temp+1;
		*temp = '\0';
		cell = 1;
		break;
	default:
		return(0);
	}
/*
 * Terminate case name
 */
	if (slide)
		*(slide-1) = '\0';
	else if (cellno)
		*(cellno-1) = '\0';
/*
 * Check field lengths
 */
	if (strlen(work) > FNAMECASE) return(0);
	else (remdollars(work,'_'));
	if (slide && (strlen(slide) > SLIDELENGTH)) return(0);
	else (remdollars(slide,'.'));
	if (cellno && (strlen(cellno) > CELLLENGTH)) return(0);
/*
 *	Now do fix
 */
	if (debug)
		printf("kar %d met %d cell %d dat %d case %s slide %s cell %s ext %s\n",
				kar,met,cell,dat,work,slide,cellno,ext);
	if (kar) {
		typ = KARYFILE;
		sprintf(new,"%s$%s$%sK",work,slide,cellno);
	} else if (cell) {
		typ = CELLFILE;
		sprintf(new,"%s$%s$%s",work,slide,cellno);
	} else if (met) {
		typ = METFILE;
		sprintf(new,"%s$%s$M",work,slide);
	} else if (dat) {
		typ = GDATFILE;
		sprintf(new,"%s$%s$D",work,slide);
	}
	if (!hflag) {
		sprintf(command,"rename %s %s",name,new);
		if (!sflag)
			printf("%s\n",command);
		if (eflag)
			system(command);
	} else {
		makenewfile(typ,name,new,work,slide,atoi(cellno));
	}
	return(1);
}

/*
 *	M A K E N E W F I L E
 *
 */
makenewfile(typ,old,new,pat,slide,cellno)
int		typ, cellno;
char	*pat, *slide;
char	*old, *new;
{
	struct object *obj;
	struct	propertylist *header;
	char	filename[512], *ptr;
	FILE	*fp;
	int		fold, count, i;
	char	*filebuf;

	if ((filebuf = (char *) malloc(8192)) == NULL) {
		fprintf(stderr,"Can't get memory buffer\n");
		return(0);
	}
	if ((fold = open(old,1)) == -1) {
		fprintf(stderr,"Can't read file %s, error %d\n",old,errno);
		free (filebuf);
		return(0);
	}
	if ((ptr = (char *) rindex(old,'/')) != NULL) {
		strncpy(filename,old,(ptr-old+1));
		strcpy(filename+(ptr-old+1),new);
	} else
		strcpy(filename,new);
	if ((fp = fopen(filename,"w")) == NULL) {
		fprintf(stderr,"Can't create new file %s, error %d\n",filename,errno);
		free(filebuf);
		close(fold);
		return(0);
	}
	if (typ == METFILE) {
		header = (struct propertylist *) convmetheader(fold,pat,slide);
	} else {
		header = (struct propertylist *) makeheader(typ,pat,slide,cellno);
	}
	if (header == NULL) {
		free(filebuf);
		close(fold);
		fclose(fp);
		return(0);
	}
	obj = (struct object *)  makemain(110,NULL,NULL,header,NULL);

	if (!sflag)
		printf("Converting %s to %s\n",old,new);
	writeobj(fp,obj);
	fflush(fp);
	freeobj(obj);
	while (count = read(fold,filebuf,8192)) {
		write(fp->_fd,filebuf,count);
	}
	free(filebuf);
	fclose(fp);
	close(fold);
}

/*
 *	C O N V M E T H E A D E R
 */
convmetheader(fd,pat,slide)
int		fd;
char	*pat, *slide;
{
	struct	met_header *met_h;
	struct	hstruct		old_met;

	if (read(fd,&old_met,sizeof(struct hstruct)) != sizeof(struct hstruct))
		return(NULL);
	met_h = (struct met_header *) makeheader(METFILE,pat,slide,0);
	met_h->nrecs = old_met.nrecs;
	met_h->comments = old_met.comments;
	strncpy(met_h->opid,old_met.scorer,SCLONG);
	met_h->opid[SCLONG] = '\0';
	sprintf(met_h->date,"%d/%d/%d",old_met.date[0],old_met.date[1],
				old_met.date[2]);
/*
 *	Translate text string tissue/stain to numeric code using
 *	first letter  of each type
 */
	switch (*old_met.tissue) {
		case 'p':					/* periph. blood */
			met_h->tissue = 1;
			break;
		case 'a':					/* amniotic fluid */
			met_h->tissue = 2;
			break;
		case 'c':					/* chorionic villus */
			met_h->tissue = 3;
			break;
		case 'b':					/* bone marrow */
			met_h->tissue = 4;
			break;
		default:
			met_h->tissue = 1;
	}
	switch (*old_met.fstain) {
		case 'g':					/* giemsa banded */
			met_h->stain = 1;
			break;
		case 'o':					/* orcein homogenous */
			met_h->stain = 2;
			break;
		default:
			met_h->stain = 1;
	}
	return(met_h);
}

/*
 *	M A K E H E A D E R
 *
 */
struct propertylist *
makeheader(typ,pat,slide,cellno)
int		typ, cellno;
char	*pat, *slide;
{
	struct cell_header *cell_h;
	struct met_header *met_h;
	struct gdat_header *gdat_h;
	switch (typ) {
		case CELLFILE:
			cell_h = (struct cell_header *) makeplist(sizeof(struct cell_header));
			cell_h->version = HEADER_VERSION;
			cell_h->file_type = CELLFILE;
			strcpy(cell_h->patid,pat);
			strcpy(cell_h->slideid,slide);
			cell_h->cellid = cellno;
			cell_h->dig_type = DIGFIP;
			return(cell_h);
		case KARYFILE:
			cell_h = (struct cell_header *) makeplist(sizeof(struct cell_header));
			cell_h->version = HEADER_VERSION;
			cell_h->file_type = KARYFILE;
			strcpy(cell_h->patid,pat);
			strcpy(cell_h->slideid,slide);
			cell_h->cellid = cellno;
			cell_h->dig_type = DIGFIP;
			return(cell_h);
		case METFILE:
			met_h = (struct met_header *) makeplist(sizeof(struct met_header));
			met_h->version = HEADER_VERSION;
			met_h->file_type = METFILE;
			strcpy(met_h->patid,pat);
			strcpy(met_h->slideid,slide);
			return(met_h);
		case GDATFILE:
			gdat_h = (struct gdat_header *) makeplist(sizeof(struct gdat_header));
			gdat_h->version = HEADER_VERSION;
			gdat_h->file_type = GDATFILE;
			strcpy(gdat_h->patid,pat);
			strcpy(gdat_h->slideid,slide);
			return(gdat_h);
		
	}
}

/*
 *	B S E A R C H  --  Search back down a string for given char
 */
bsearch(end,start,ch)
char	*end, *start, ch;
{
	char	*ptr;
	for (ptr = end-1; ptr >= start; ptr--) {
		if (*ptr == ch)
			return(ptr);
	}
	return(NULL);
}

/*
 *	E X T R A C T N A M E
 */
extractname(work,name)
{
	char	*temp;

	if ((temp = (char *) rindex(name,'/')) == NULL)
		strcpy(work,name);
	else 
		strcpy(work,temp+1);
}

/*
 *	R E M D O L L A R S
 */
remdollars(str,ch)
char	*str,ch;
{
	while (*str) {
		if (*str == '$')
			*str = ch;
		str++;
	}
}


