/*
 *      P I C E D . C  --  Picture editor
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    21st August 1988
 *
 *  Modifications
 *
 */
#define INIFILE		/* CMDSYS Startup */
#include <cmdsys.h>
#include <ddgs.h>
#include <wstruct.h>
#define FRAMEFILE	"framemaker"
/*
 *	Local definitions
 */
#define OVERLAY1 1

/*
 *	Local general storage
 */
DDGS	*dg;				/* DDGS device structure */
unsigned char	*frame;		/* Image to be worked on */
static struct ipoint cursor;
static cursorcolour = OVERLAY1;
static struct pframe cframe;


/*
 *	Forward declarations
 */
int	mainmenu();
int	kickslave();
static int zoom(), restore();
static int threshold(), binarise(), tidyup(), edit(), framefile();
static int edittrack(), eddisp(), edsetpix(), edclrpix();

/*
 *	External routines
 */
DDGS	*ddgsopen();

/*
 *	I N T E R R U P T --  Signal catcher
 *
 */
interrupt(sig)
int		sig;
{
}

main(argc,argv)
int		argc;
char	**argv;
{
	char	*swptr, **targv, **sargv;
	int		targc;
	int		store = FS_VME512, devno = 0;
	int		cmdsys_slave = 0;

#ifdef INTERRUPT
	intercept(interrupt);
#endif INTERRUPT
	targc = argc; targv = argv;
	argc--; argv++;			/* dump process name */
	while (argc > 0) {
		if (*argv[0] == '-') {
			swptr = *argv;
			*argv = NULL;
			while (*(++swptr)) {
				switch (*swptr) {
					case '?':
						printf("Syntax:   piced <options>\n");
						printf("Function: \n");
						printf("Options:  v - Show version number\n");
						printf("          V - Virtuoso framestore (Default CRS 512)\n");
						printf("          I - Unmodified Virtuoso framestore\n");
						printf("          1 - Use seat 1 framestore\n");
						printf("          2 - Use seat 2 framestore\n");
						printf("                  also 3 and 4\n");
						printf("          s - cmdsys slave process\n");
						printf("          ? - Help\n");
						exit(0);
					case 'v':
						printf("Piced - Version 0.1\n");
						exit(0);
					case '1':
					case '2':
					case '3':
					case '4':
						devno = *swptr - '1';
						break;
					case 'V':
						store = FS_VIRTUOSO_INVERT;
						break;
					case 'I':
						store = FS_VIRTUOSO;
						break;
					case 's':
						cmdsys_slave = 1;
						break;
/*					case 'x':
						param = *++argv;
						*argv = NULL; argc--;
						break;
 */
					default:
						printf("Piced - unknown option\n");
				}
			}
		}
		argv++; argc--;
	}
#ifdef PARAMS
/*
 * Compact params to fill holes left by switches
 */
	argc = --targc; argv = ++targv;				/* dump process name */
	sargv = targv;
	while (targc-- > 0) {						/* Squeez + fill the holes */
		if (*targv)
			*sargv++ = *targv;
		else
			argc--;
		targv++;
	}
/*
 *	Use params 
 */
	if (argc) {
		while (argc--) {
			if ((infile = fopen(*argv,"r")) == NULL) {
				fprintf(stderr,"Can't open file %s\n",*argv);
			}
			dofile(infile);
			fclose(infile);
			argv++;
		}
	} else {
		dofile(stdin);
	}
#endif PARAMS
	printf("Piced - initialising\n");
	initialise(store,devno);
	printf("copy\n");
	copyscreentobuffer(frame);
	if (cmdsys_slave == 1)
		slavemain(targv[1], mainmenu, kickslave);
	else {
		mouseport("/t3");
		muini(stdout);
		mainmenu();
	}
}

mainmenu()
{
	int		x = 0, y= 0, scale = 1;
	int		thr = 0, Ex_cmd = 0;;
	newmenu("Picture editor");
	cmmd("restore;%f",restore);
	cmmd("zoom;<0/999>%d<0/999>%d<1/8>%d%f",&x,&y,&scale,zoom);
	cmmd("threshold;<0/255>%d%f",&thr,threshold);
	cmmd("binarise;%f",binarise);
	cmmd("tidy up;%f",tidyup);
	cmmd("edit;<0/999>%d<0/999>%d%f",&x,&y,edit);
	cmmd("Write framemaker file;%f",framefile);
	Ex_cmd = cmmd("exit");
	while(thismu() != Ex_cmd)
		;
	alldone(0);
}

kickslave()
{
}

/*
 *	E D I T   --   Edit image on screen
 *
 */
static int edit(x,y)
int		*x,*y;
{
	static int scale = 4;
	int		hhit = 1, vhit = 1;
	copyscreentobuffer(frame);
	eddisp(x,y,&scale);
	setupcursor();
	setmsv(edittrack);
	newmenu("edit");
	cmmd("scale;%-+d%-+d<1/20>%d%f",x,y,&scale,eddisp);
	cmmd("position;<0/999>%d<0/999>%d%-+d%f",x,y,&scale,eddisp);
	cmmd("add pixel;%-+d%-+d%-+d%-+d%-+d%f",x,y,&scale,&hhit,&vhit,edsetpix);mouse(LEFTBT);
	cmmd("remove pixel;%-+d%-+d%-+d%-+d%-+d%f",x,y,&scale,&hhit,&vhit,edclrpix);mouse(RIGHTBT);
	cmmd("hitcount;horizontal %d vertical %d\n",&hhit,&vhit);
	while (thismu());
	setmsv(NULL);
	erasecursor();
	llut();
	copybuffertoscreen(frame);
}
static int edsetpix(x,y,scale,hhit,vhit)
int		*x,*y,*scale,*hhit,*vhit;
{
	int		cx, cy, i ,j, hor, vert,pntx,pnty,realx,realy;
	unsigned char	*pnt;
	cx = cursor.k - (cursor.k % *scale);
	cy = cursor.l - (cursor.l % *scale);
	for (vert=0;vert < *vhit; vert++) {
		pnty = cy + vert * *scale;
		if (pnty >= dg->lineno) break;
		realy = (pnty / *scale) + *y;
		for (hor=0; hor < *hhit; hor++) {
			pntx = cx + hor * *scale;
			if (pntx >= dg->lineln) break;
			realx = (pntx / *scale) + *x;
			for (i=pnty; i<pnty+ *scale;i++) {
				pnt = (unsigned char *) dg->frame+((dg->lineno - i - 1) * dg->lineln) + pntx;
				for (j=0;j< *scale;j++)
					*pnt++ = 255-OVERLAY1;
			}
			*(frame + ((dg->lineno - realy) * dg->lineln) + realx) = 255;
		}
	}
}
static int edclrpix(x,y,scale,hhit,vhit)
int		*x,*y,*scale,*hhit,*vhit;
{
	int		cx, cy, i ,j, hor, vert,pntx,pnty,realx,realy;
	unsigned char	*pnt;
	cx = cursor.k - (cursor.k % *scale);
	cy = cursor.l - (cursor.l % *scale);
	for (vert=0;vert < *vhit; vert++) {
		pnty = cy + vert * *scale;
		if ((pnty >= dg->lineno) || (pnty < 0)) continue;
		realy = (pnty / *scale) + *y;
		for (hor=0; hor < *hhit; hor++) {
			pntx = cx + hor * *scale;
			if (pntx >= dg->lineln || (pntx < 0)) continue;
			realx = (pntx / *scale) + *x;
			for (i=pnty; i<pnty+ *scale;i++) {
				pnt = (unsigned char *) dg->frame+((dg->lineno - i - 1) * dg->lineln) + pntx;
				for (j=0;j< *scale;j++)
					*pnt++ = 0;
			}
/*			fprintf(stderr,"plot %d %d, real %d %d\n",pntx,pnty,realx,realy); */
			*(frame + ((dg->lineno - realy) * dg->lineln) + realx) = 0;
		}
	}
}

static int eddisp(x,y,scale)
int		*x, *y, *scale;
{
	int		fixedy;
	fixedy = dg->lineno - *y;
	zoombuffer(frame,x,&fixedy,*scale);
	*y = dg->lineno - fixedy;
	clearplane(dg,1);
	lutplane(OVERLAY1,255,0,0);
}
	
/*
 *	Z O O M  --  Zoom data and display on framestore
 *
 */
static int zoom(xstart,ystart,scale)
int		*xstart,*ystart,*scale;
{
	int		x,y, xsize, ysize, fixedy;
	x = *xstart << dg->xshift;
	y = *ystart << dg->yshift;
	xsize = (dg->lineln / *scale) << dg->xshift;
	ysize = (dg->lineno / *scale) << dg->yshift;
	clearplane(dg,1);
	lutplane(OVERLAY1,255,0,0);
	colour(OVERLAY1);
	lwidth(1);
	moveto(x,y);
	lineby(xsize,0);
	lineby(0,ysize);
	lineby(-xsize,0);
	lineby(0,-ysize);
	sleep(4);
	llut();
	fixedy = dg->lineno - *ystart;
	zoombuffer(frame,xstart,&fixedy,*scale);
	*ystart = dg->lineno - fixedy;
}

/*
 *	Write out data in a ascii hex file for frame maker
 */
framefile()
{
	register unsigned char byt;
	register int	i, lines, cols;
	register unsigned char	*scr1;
	register unsigned short	*scr2;
	int		x, y, xsize, ysize;
	FILE	*out;
	char	buffer[40];

	x = 100; y = 100;
	xsize = 200; ysize = 200;
	putbox(x<<dg->xshift,y<<dg->yshift,xsize<<dg->xshift,ysize<<dg->yshift);
	newmenu("Write framemaker file");
	cmmd("Box bottom/left;%d%d",&x,&y);
	cmmd("Box size;%d%d",&xsize,&ysize);
	while (thismu()) {
		xsize = (xsize+7) & 0xffff8;
		xsize = (xsize+7) & 0xffff8;
		unputbox();
		putbox(x<<dg->xshift,y<<dg->yshift,xsize<<dg->xshift,ysize<<dg->yshift);
	}
	unputbox(x,y,xsize,ysize);
	llut();
	printf("Writing file %s\n",FRAMEFILE);
	sprintf(buffer,"%s.pic",FRAMEFILE);
	out = fopen(buffer,"w");
	fprintf(out,"frameimage\n");
	lines = ysize;

	switch (dg->framepacking) {
		case BYTEPACKED:
			while (lines--) {
				scr1 = dg->frame + (dg->lineln * (dg->lineno - y - lines + 1)) + x;
				cols = xsize;
				while (cols > 0) {
					cols -= 8;
					for (i=0; i<8; i++) {
						byt<<=1;
						byt += (*scr1++) ? 1:0;
					}
					fprintf(out,"%02X",byt);
				}
				fprintf(out,"\n");
			}
			break;
		case SHORTPACKED:
			while (lines--) {
				scr2 = ((unsigned short *)dg->frame) + (dg->lineln * (dg->lineno - y - lines + 1)) + x;
				cols = xsize;
				while (cols > 0) {
					cols -= 8;
					for (i=0; i<8; i++) {
						byt<<=1;
						byt += (*scr1++)&0xff ? 1:0;
					}
					fprintf(out,"%02X",byt);
				}
				fprintf(out,"\n");
			}
			break;
	}
	fclose(out);
	sprintf(buffer,"%s.ps",FRAMEFILE);
	out = fopen(buffer,"w");
	fprintf(out,"%s%s%s%s%s",
			"%Graphic Inset\n%by C A Stubbings\n%Oct 1990\n",
			"/height exch def\n/width exch def\n/concatprocs\n  { /proc2 exch cvlit def\n",
			"    /proc1 exch cvlit def\n    /newproc proc1 length proc2 length add",
			" array def\n    newproc 0 proc1 putinterval\n    newproc proc1 length proc2 putinterval\n",
			"    newproc cvx\n  } def\n"
		);
	fprintf(out,"/picstr %d string def\n/frameimage\n",xsize/8);
	fprintf(out,"{ %d %d 1 [%d 0 0 %d 0 %d]\n",xsize,ysize,xsize,-ysize,ysize);
	fprintf(out,"%s%s",
			"     { currentfile picstr\n   readhexstring pop } image\n  } def\n  width height scale\n",
			"gsave\n  {1 exch sub} currenttransfer concatprocs settransfer\n"
		);
	fprintf(out,"#include \"%s.pic\"\ngrestore\n",FRAMEFILE);
	fclose(out);
}

/*
 *	T H R E S H O L D  --
 *
 */
threshold(thr)
register int	*thr;
{
	register int	size;
	register unsigned char	*scr1;
	register unsigned short	*scr2;
	size = dg->lineno * dg->lineln;

	switch (dg->framepacking) {
		case BYTEPACKED:
			scr1 = dg->frame;
			while (size--) {
				if (*scr1 < *thr)
					*scr1 = 0;
				scr1++;
			}
			break;
		case SHORTPACKED:
			scr2 = (unsigned short *) dg->frame;
			while (size--) {
				if (*scr2 < *thr)
					*scr2 = 0;
				scr2++;
			}
			break;
	}
}

/*
 *	B I N A R I S E  --  Make all non zero pixels 255
 *
 */
static int binarise()
{
	register int	size;
	register unsigned char	*scr1;
	register unsigned short	*scr2;

	size = dg->lineno * dg->lineln;
	switch (dg->framepacking) {
		case BYTEPACKED:
			scr1 = dg->frame;
			while (size--) {
				if (*scr1)
					*scr1 = 255;
				scr1++;
			}
			break;
		case SHORTPACKED:
			scr2 = (unsigned short *) dg->frame;
			while (size--) {
				if (*scr2)
					*scr2 = 255;
				scr2++;
			}
			break;
	}
}
/*
 *	T I D Y U P  --  Tidy up image - fill holes remove noise
 *
 */
static int tidyup()
{
	register int	i, j, neigb;
	register unsigned char	*line, *prev, *next;

	prev = dg->frame + 1;
	line = prev + dg->lineln;
	next = line + dg->lineln;
	for (i=1; i<dg->lineno-1; i++) {
		for (j=1; j<dg->lineln-1;j++) {
			neigb = (*prev != 0) + (*(prev-1) != 0) + (*(prev+1) !=0) + (*next != 0) +
					(*(next-1) != 0) + (*(next+1) != 0) + (*(line-1) != 0) + (*(line+1) != 0);
			if (neigb < 3)
				*line = 0;
			else if (neigb >= 5)
				*line = 255;
			line++; prev++; next++;
		}
		line+=2; prev+=2; next+=2;
	}
}


setupcursor()
{
	cframe.scale = 8;
	cframe.dx = dg->xcenter;
	cframe.dy = dg->ycenter;
	cframe.ix = 1;
	cframe.iy = 1;
	cframe.ox = dg->lineln/2;
	cframe.oy = dg->lineno/2;
	cursor.type = 40;
	cursor.style = 6;
}
/*
 *	E R A S E C U R S O R
 *
 */
erasecursor()
{
	colour(cursorcolour);
	intens(0);
	picframe(&cursor,&cframe);
	intens(1);
}
showcursor()
{
	colour(cursorcolour);
	intens(1);
	picframe(&cursor,&cframe);
}

static int edittrack()
{
	register int xscale,yscale;
	xscale = yscale = 1;
	if (cframe.ix < 0)
		xscale = -xscale;
	if (cframe.iy < 0)
		yscale = -yscale;
	erasecursor();
	cursor.k += mukont->mouss->XINC * xscale;
	cursor.l += mukont->mouss->YINC * yscale;
/*	printf("cursor %d %d\n",cursor.l, cursor.k); */
	showcursor();
}





lutplane(plane,r,g,b)
int		plane,r,g,b;
{
	int		i;
	for (i=0; i<256; i++) {
		if (i & plane) {
			setrgb(i,r,g,b);
		}
	}
}

/*
 *	Z O O M B U F F E R  --
 *
 */
zoombuffer(buffer,xstart,ystart,scale)
unsigned char *buffer;
int		*xstart, *ystart,scale;
{
	register int	i,j,xcount,ycount;
	register unsigned char *in_line_pos, *out_line_pos;
	unsigned char *in_line_start, *out_line_start;

	if (*xstart > dg->lineln - (dg->lineln/scale))
		*xstart = dg->lineln - (dg->lineln/scale);
	if (*ystart < dg->lineno/scale)
		*ystart = dg->lineno/scale;
/*	printf("buffer 0x%x, xstart %d, ystart %d\n",buffer,*xstart,*ystart); */
	in_line_start = buffer + (dg->lineln * *ystart) + *xstart;
	out_line_start = dg->frame + (dg->lineno - 1) * dg->lineln;
	ycount = dg->lineno;
	while (ycount > 0) {
		for (i=0;i<scale;i++) {
			in_line_pos = in_line_start;
			out_line_pos = out_line_start;
			xcount = dg->lineln;
/*			printf("in 0x%x, out 0x%x\n",in_line_pos,out_line_pos); */
			while (xcount > 0) {
				for (j=0; j<scale; j++) {
					*out_line_pos++ = *in_line_pos;
					xcount--;
				}
				in_line_pos++;
			}
			out_line_start -= dg->lineln;
			ycount--;
		}
		in_line_start -= dg->lineln;
	}
}

/*
 *	R E S T O R E   --
 *
 */
static int restore()
{
	copybuffertoscreen(frame);
}

/*
 *	C O P Y S C R E E N T O B U F F E R
 *
 */
copyscreentobuffer(buffer)
register unsigned char	*buffer;
{
	register int	size;
	register unsigned char	*scr1;
	register unsigned short	*scr2;

	size = dg->lineln * dg->lineno;
	switch (dg->framepacking) {
		case BYTEPACKED:
			scr1 = dg->frame;
			while (size--)
				*buffer++ = *scr1++;
			break;
		case SHORTPACKED:
			scr2 = (unsigned short *) dg->frame;
			while (size--)
				*buffer++ = *scr2++;
			break;
	}
}

/*
 *	C O P Y B U F F E R T O S C R E E N  --
 *
 */
copybuffertoscreen(buffer)
register unsigned char	*buffer;
{
	register int	size;
	register unsigned char	*scr1;
	register unsigned short	*scr2;

	size = dg->lineln * dg->lineno;
	switch (dg->framepacking) {
		case BYTEPACKED:
			scr1 = dg->frame;
			while (size--)
				*scr1++ = *buffer++;
			break;
		case SHORTPACKED:
			scr2 = (unsigned short *) dg->frame;
			while (size--)
				*scr2++ = *buffer++;
			break;
	}
}

/*
 *	I N I T I A L I S E  --
 *
 */
initialise(devtype,devno)
int		devtype,devno;
{
	int		size;

	dg = ddgsopen(devtype,devno);
	dinit();
	size = dg->lineln * dg->lineno;
	Alloc_wait(0);
	if ((frame = (unsigned char *) Calloc(size,1)) == NULL) {
		fprintf(stderr,"Can't get enough memory\n");
		exit(99);
	}
	Alloc_wait(1);
}

putbox(x,y,xsize,ysize)
int		x, y, xsize, ysize;
{
	clearplane(dg,1);
	lutplane(OVERLAY1,255,0,0);
	colour(OVERLAY1);
	lwidth(1);
	moveto(x,y);
	lineby(xsize,0);
	lineby(0,ysize);
	lineby(-xsize,0);
	lineby(0,-ysize);
}

unputbox()
{
	clearplane(dg,1);
}
