/*
 *	Q M A T H - Quick(ish) maths functions
 *
 *
 *  Written  David Burgess
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    31st January 1990
 *
 *  Modifications
 *
 */
#include <stdio.h>
#include <math.h>
#include <time.h>

#define LNSQRT2 0.3465735902799726547086160
#define HALFSQRT2	0.7071067811865475244
#define SQRT2		1.4142135623730950488
#define A0			-0.5527074855
#define B0			-6.632718214

#define LN2			0.693147180559945309
#define MAXARG		88.0
#define P0			0.24999999
#define P1			0.00416028863
#define Q0			0.5
#define Q1			0.049987178

union intfloat {
	int hi;
	float re;
};

float qln(), qexp();

main()
{
	float num, mylog;
	double os9log;

	printf("Number :"); fflush(stdout);
	scanf("%f", &num);
	if (num < 0) timeit(-num);
	else {
		mylog = qln(num);
		os9log = log(num);
		printf("My log is %f\n", mylog);
		printf("OS9 log is %f\n", os9log);
		printf("My exp is %f\n", qexp(mylog));
		printf("OS9 exp is %f\n", exp(os9log));
	}
}

timeit(limit)
float limit;
{
	char ans[20];
	struct sgtbuf timebuf;
	float num;

	/*
	 *	My log
	 */
	getime(&timebuf);
	printf("\007Start mylog %d:%d:%d\n", timebuf.t_hour, timebuf.t_minute, timebuf.t_second);
	for (num = 0.1; num < limit; num += 0.1) qln(num);
	getime(&timebuf);
	printf("\007Done mylog %d:%d:%d\n", timebuf.t_hour, timebuf.t_minute, timebuf.t_second);

	/*
	 *	OS9 log
	 */
	getime(&timebuf);
	printf("\007Start os9log %d:%d:%d\n", timebuf.t_hour, timebuf.t_minute, timebuf.t_second);
	for (num = 0.1; num < limit; num += 0.1) log(num);
	getime(&timebuf);
	printf("\007Done os9log %d:%d:%d\n", timebuf.t_hour, timebuf.t_minute, timebuf.t_second);

	limit /= 100.0;
	/*
	 * My exp
	 */
	getime(&timebuf);
	printf("\007Start myexp %d:%d:%d\n", timebuf.t_hour, timebuf.t_minute, timebuf.t_second);
	for (num = 0.1; num < limit; num += 0.001) qexp(num);
	getime(&timebuf);
	printf("\007Done myexp %d:%d:%d\n", timebuf.t_hour, timebuf.t_minute, timebuf.t_second);

	/*
	 *	OS9 exp
	 */
	getime(&timebuf);
	printf("\007Start os9exp %d:%d:%d\n", timebuf.t_hour, timebuf.t_minute, timebuf.t_second);
	for (num = 0.1; num < limit; num += 0.001) exp(num);
	getime(&timebuf);
	printf("\007Done os9exp %d:%d:%d\n", timebuf.t_hour, timebuf.t_minute, timebuf.t_second);
}

float
qln(x)
float x;
{
	union intfloat f;
	float z, w;
	int i, n;

	f.re = x;
/*	printf("float %5.2f, int = 0x%x\n", f.re, f.hi);*/
	n = 2 * ((f.hi >> 23) - 0x80);
	f.hi = (f.hi & 0x807fffff) | 0x40000000; /* Make exponent 0 */
/*	printf("float %5.2f, exponent = %d\n", f.re, n);*/

	if (f.re < HALFSQRT2) {
		f.re *= SQRT2;
		n -= 1;
	}
	z = 2.0 * (f.re - 1.0)/(f.re + 1.0);
	w = z * z;
	return(n * LNSQRT2 + z + z * (w*A0/(w+B0)));
}


float
qexp(x)
float x;
{
	union intfloat h;
	float g, pz, qz, y;
	int w;

	if (x > MAXARG) fprintf(stderr, "qexp: overflow\n");
	else if (x < -MAXARG) return(0.0);
	else {
		y = x/LN2;
		w = y + 0.5;
		y = (y - w) * LN2;
		g = y * y;

		pz = (P1 * g + P0) * y;
		qz = Q1 * g + Q0;
		h.re = pz/(qz-pz) + 0.5;
		h.hi = h.hi + ((w+1) << 23);
		return(h.re);
	}
}

