/*
 *      S C R F O R M A T . C -- Flexible screen/hardcopy formatter
 *
 *
 *  Written by Sarah E. A. Jones.
 *				
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    21st Mar 1989.
 *
 *  Modifications
 *  18 Jul 1991 ih		Put in raw process mode (c.f. setscreen)
 *	05 Apr 1990	dcb		Put checks on input string lengths, and vsn no. in hdr
 *
 */

#define INIFILE        /* define variables in this link */
#include <cmdsys.h>
#include <wstruct.h>
#include <chromanal.h>
#include <dispform.h>
#include <display.h>
#include <ddgs.h>
#include <scrformat.h>

/*
 * external routines
 */
extern int init_curs(), erasecursor(), showcursor(), curs(), init_display(), curstrack();
extern int newline(), stretchline();
extern struct scrform *readfile();
extern int writefile(), init_box();

extern struct scrform *scrf;
extern struct ipoint cursor;
extern struct box *curr_box;
extern struct lineint *curr_line;
extern int curr_chrom;
/*
 * local variables 
 */
DDGS *dg;
char infile[80], outfile[80];
int txt = 0, raw=0, filewritten = 0;
char *scrformatid = "0.3";			/* The version number */

/*
 * local routines.
 */
int kickslave(), startup();
int boxmenu(), placebox(), resize(), placestack(), viewed_stack();
int changetag(), change_object(), placetext(), chstr(), chsize(), disptext();
int label(), writemenu();
int placeline(),delline(),create_object(),create_box(),create_line();
/*
 *	I N T E R R U P T --  Signal catcher
 *
 */
interrupt(sig)
int		sig;
{
}

/*
 * M A I N -- 
 *
 */
main(argc,argv)
int		argc;
char	**argv;
{
	char	*swptr, **targv, **sargv, mouseterm[7];
	int		targc;
	int		store = FS_VME512, devno = 0;
	DDGS	*ddgsptr, *ddgsopen(), *ddgsdup();

#ifdef INTERRUPT
	intercept(interrupt);
#endif INTERRUPT
	targc = argc; targv = argv;

	strcpy(mouseterm, "/t5");
	argc--; argv++;			/* dump process name */
	while (argc > 0) {
		if (*argv[0] == '-') {
			swptr = *argv;
			*argv = NULL;
			while (*(++swptr)) {
				switch (*swptr) {
					case '?':
						printf("Syntax:   scrformat <options> <input file> <output file> \n");
						printf("Function: Edit and create karyogram templates\n");
						printf("Options:  v - Show version number\n");
						printf("          1 - Use seat 1 framestore\n");
						printf("          2 - Use seat 2 framestore\n");
						printf("                  also 3 and 4\n");
						printf("          I - Virtuoso framestore (Default CRS 512)\n");
						printf("		  V - Virtuoso invert framestore(Default CRS 512)\n");
						printf("		  m - Mouse term (default /t5)\n");
						printf("		  r - raw mode (convert text file to binary without user interaction)\n");
				  		printf("          ? - Help\n");
				  		exit(0);
			  		case 'v':
						printf("Scrformat: Version %s\n", scrformatid);
						exit(0);
				   	case '1':
						devno = 0;
						break;
					case '2':
						devno = 1;
						break;
					case '3':
						devno = 2;
						break;
					case '4':
						devno = 3;
						break;
					case 'I':
						store = FS_VIRTUOSO;
						break;
					case 'V':
						store = FS_VIRTUOSO_INVERT;
						break;
					case 'm':
						argc--;
						argv++;
						strcpy(mouseterm, *argv);
						*argv = NULL;
						break;
					case 't':
						txt = 1;
						break;
					case 'b':
						txt = 0;	/* default */
						break;
					case 'r':
						raw=1;	/* rawcopy (no screen edit) */
						break;
					default:
						printf("scrformat - unknown option\n");
				}
			}
		}
		argv++; argc--;
	}

/*
 * Compact params to fill holes left by switches
 */
	argc = --targc; argv = ++targv;				/* dump process name */
	sargv = targv;
	while (targc-- > 0) {						/* Squeez + fill the holes */
		if (*targv)
			*sargv++ = *targv;
		else
			argc--;
		targv++;
	}
/*
 * initialise cmdsys before anything else
 */
 	if (!raw) {
	 	mouseport(mouseterm);
		muini(stdout);
	}
/*
 *	Use params 
 */
 	*outfile = '\0';
 	*infile = '\0';
	if (argc > 1)
		strcpy(outfile, argv[1]);
	if (argc) {
		strcpy(infile, *argv);
		if (*outfile == '\0') {
			if (raw) {
				printf("\n\nWhat is the output file ? --- ");
				scanf("%s", outfile);
			}
			else
				strcpy(outfile, infile);
		}
	}
	else {
		printf("\n\nWhat is the input file ? --- ");
		scanf("%s", infile);
		if (raw) {
			printf("\n\nWhat is the output file ? --- ");
			scanf("%s", outfile);
		}
		else
			strcpy(outfile, infile);
	}
/*
 * This is the start of the M A I N program
 *
 */
 	if(raw) {
 		scrf=readfile(infile);
 		writefile(outfile,txt);
 	}
 	else {
		dg = ddgsopen(store, devno);
		startup(infile, outfile);
	}
}
/*
 *  K I C K S L A V E -- dummy routine
 *
 */
kickslave() {}

/*
 * I N I T I A L I S E --
 *
 */
initialise()
{
	dinit();
	clear();
	init_display();
	init_curs();
	showcursor();
}

/*
 *  S T A R T U P -- initial setup routines
 *
 */
startup(infile, outfile)
char *infile, *outfile;
{
	initialise();
	scrf = readfile(infile);
	display_screen();
	topmenu();
	alldone(0);
}

/*
 * D I S P L A Y _ S C R E E N --
 *
 */
display_screen()
{
	int i;
	struct lineint *l;
	
	disp_box(&scrf->idbox.b, 1, BLUE);
	dispboxtext(&scrf->idbox.b, 1, BLUE);
	for (i = 0; i < MAXCLASS; i++) {
		if (scrf->classes[i].b.posn.x == -1)
			break;
		disp_class(&scrf->classes[i], 1, BLUE);
		}
	display_box(&scrf->chromclass, 1, BLUE, CHROMCLASS);
	for (i = 0; i < scrf->viewed_stack; i++)
		display_box(&scrf->stk[i], 1, BLUE, STACK);
	for(l=scrf->linebuf.lineint;l;l=l->next)
		display_line(l,1,BLUE);
}

/*
 * D I S P B O X T E X T 
 *
 */
dispboxtext(b, on, col)
struct box *b;
int on, col;
{
	int x, y, i;
	struct datab *id;
	
	x = b->posn.x;
	y = b->posn.y;
	id = &scrf->idbox;
	for (i = 0; i <= MAXTEXT; i++)
		dispidtext(&id->cont[i], x, y, on, col);
}

/*
 * D I S P I D T E X T
 *
 */
dispidtext(line, x, y, on, col)
struct idtext *line;
int x, y, col, on;
{
	colour(col);
	intens(on);
	if (line->display) {
		tsize(line->size, line->size);
		dispstring(line->text, x + line->x, y + line->y);
	}
}	

/*
 * D I S P _ C L A S S
 *
 */
disp_class(cl, on, colr)
struct classpos *cl;
int on, colr;
{
	int x, length;
	
	disp_box(&cl->b, on, colr);
	intens(on);
	colour(colr);
	tsize(40, 40);
	length = strlen(cl->tag);
	length *= 40;
	length = cl->b.width - length;
	length /= 2;
	dispstring(cl->tag, (cl->b.posn.x + length), cl->b.posn.y - 80);
}

/*
 * D I S P L A Y _ L I N E
 */
display_line(lineint,on,col)
int on,col;
struct lineint *lineint;
{
	intens(on);
	colour(col);
	moveto(lineint->x1,lineint->y1);
	lineto(lineint->x2,lineint->y2);
}

/*
 * T O P M E N U 
 *
 */
topmenu()
{
	int Quit_cmd;
	char home_menu_name[40];
	
	setmsv(curs);	
	sprintf(home_menu_name, "Screen Formatter Version %s", scrformatid);
	newmenu(home_menu_name);
	cmmd("create box/line/text;%f",create_object);mouse(LEFTBT);
	cmmd("select box/line/text to be altered;%f",change_object);mouse(MIDBT);
	cmmd("choose label text to be displayed;%f",label);mouse(RIGHTBT);
	cmmd("update file;%f",writemenu);
	cmmd("quit");Quit_cmd=mmcmd;
	while (1) {
		while (thismu() != Quit_cmd);
		if (!filewritten) {
			printf("Output file has not been written. Write file?\n");
			if (!yesno()) {
				downmenu();
				writemenu();
				upmenu();
			} else
				break;
		} else 
			break;
	}
}

/*
 * C R E A T E _ O B J E C T
 */
create_object()
{
	int Ex_cmd;

	setmsv(curstrack);
	newmenu("Create objects.");
	cmmd("create line;:k40%f",create_line);mouse(LEFTBT);
	cmmd("create id box - not yet implemented;:k40%f",create_box);mouse(MIDBT);
	cmmd("exit");Ex_cmd=mmcmd;
	while(thismu()!=Ex_cmd);
	setmsv(curs);
}

/*
 * C H A N G E _ O B J E C T
 *
 */
change_object()
{
	int objtype;
	
	objtype = kylocate();
	setmsv(curstrack);	
	switch(objtype) {
		case CHROM :
			chrom_box();
			break;
		case STACK :
			stack_line();
			break;
		case IDBOX :
			boxmenu(&scrf->idbox.b, IDBOX);
			break;
		case CHROMCLASS :
			boxmenu(&scrf->chromclass, CHROMCLASS);
			break;
		case IDTEXT :
			change_text();
			break;
		case IDLINE :
			change_line();
			break;
	}
	setmsv(curs);
}

/*
 * L A B E L 
 *
 */
label()
{
	char tmpstr[MAXTEXT][12];
	int i, type, Ex_cmd;
	
	type = kylocate();
	display_box(curr_box, 0, RED, type);
	display_box(curr_box, 1, BLUE, type);
	erasecursor();
	setmsv(NULL);
	dispboxtext(&scrf->idbox.b, 0, BLUE);
	dispboxtext(&scrf->idbox.b, 1, RED);
	for (i = 0; i < MAXTEXT - 1; i++) {
		if (scrf->idbox.cont[i].display)
			strcpy(tmpstr[i], "displayed");
		else
			strcpy(tmpstr[i], "undisplayed");
	}
	newmenu("Choose label text");
	cmmd("case;:k40%-s%-+d%-+d%f", tmpstr[CASE], CASE, RED, disptext);
	cmmd("slide;:k40%-s%-+d%-+d%f", tmpstr[SLIDE], SLIDE, RED, disptext);
	cmmd("cell;:k40%-s%-+d%-+d%f", tmpstr[CELL], CELL, RED, disptext);
	cmmd("operator;:k40%-s%-+d%-+d%f", tmpstr[OPERATOR], OPERATOR, RED, disptext);
	cmmd("date;:k40%-s%-+d%-+d%f", tmpstr[DATE], DATE, RED, disptext);
	cmmd("supervisor;:k40%-s%-+d%-+d%f", tmpstr[SUPERVISOR], SUPERVISOR, RED, disptext);
	cmmd("superdate;:k40%-s%-+d%-+d%f", tmpstr[SUPERDATE], SUPERDATE, RED, disptext);
	cmmd("patname;:k40%-s%-+d%-+d%f", tmpstr[PATNAME], PATNAME, RED, disptext);
	cmmd("comment1;:k40%-s%-+d%-+d%f", tmpstr[COMMENT1], COMMENT1, RED, disptext);
	cmmd("comment2;:k40%-s%-+d%-+d%f", tmpstr[COMMENT2], COMMENT2, RED, disptext);
	cmmd("karyotype;:k40%-s%-+d%-+d%f", tmpstr[KARYOTYP], KARYOTYP, RED, disptext);
	cmmd("lab name;:k40%-s%-+d%-+d%f", tmpstr[LABNAME], LABNAME, RED, disptext);
	cmmd("exit"); Ex_cmd = mmcmd; mouse(LONGLB);
	while (thismu() != Ex_cmd) ;
	
	dispboxtext(&scrf->idbox.b, 0, RED);
	dispboxtext(&scrf->idbox.b, 1, BLUE);
	display_box(curr_box, 0, BLUE, type);
	display_box(curr_box, 1, RED, type);
	showcursor();
	setmsv(curs);
}
/*
 * C H R O M _ B O X 
 *
 */
chrom_box()
{
	char tmpstr[20];
	int Ex_cmd;
	
	strcpy(tmpstr, scrf->classes[curr_chrom].tag);
	newmenu("change chromosome box");
	cmmd("move/resize;%-+d%-+d%f", &scrf->classes[curr_chrom].b, CHROM, boxmenu);
																mouse(MIDBT);
	cmmd("change tag;:k40%6+s%-s%f", tmpstr, scrf->classes[curr_chrom].tag, changetag);
	cmmd("default number in class;:k40%d", &scrf->classes[curr_chrom].defaultpos);
	cmmd("number in class;:k40%d", &scrf->classes[curr_chrom].npos);
	cmmd("exit"); Ex_cmd = mmcmd;mouse(LONGLB);
	
	while (thismu() != Ex_cmd);
	disp_class(&scrf->classes[curr_chrom], 0, RED);
	disp_class(&scrf->classes[curr_chrom], 1, BLUE);
}
/*
 * C H A N G E  _ T E X T 
 *
 */
change_text()
{
	struct idtext *id;
	char tmpstr[DISPTEXT], display[DISPTEXT];
	int size, Ex_cmd;
	
	id = &scrf->idbox.cont[curr_chrom];
	if (id->display)
		strcpy(display, "displayed");
	else
		strcpy(display, "undisplayed");
	strcpy(tmpstr, id->text);
	size = id->size;
	newmenu("Change text.");
	if (curr_chrom != BANNER) {
		cmmd("text displayed/not displayed;:k40%-s%-+d%-+d%f", display, curr_chrom, RED, disptext);
																mouse(LONGMB);
	}
	cmmd("place text;:k40%f", placetext); mouse(MIDBT);
	cmmd("label text;:k40%40S%-+S%f", tmpstr, display, chstr);
	cmmd("length of input text;:k40%d", &id->len);
	cmmd("text size;:k40%d%f", &size, chsize);
	cmmd("exit");Ex_cmd = mmcmd; mouse(LONGLB);
	
	while(thismu() != Ex_cmd);
	dispidtext(id, scrf->idbox.b.posn.x, scrf->idbox.b.posn.y, 0, RED);
	dispidtext(id, scrf->idbox.b.posn.x, scrf->idbox.b.posn.y, 1, BLUE);
}

/*
 * C H A N G E  _ L I N E
 *
 */
change_line()
{
	int Ex_cmd;
	int zap=0;

	newmenu("Change line.");
	cmmd("place left point;:k40%+i%+i%-+d%f",curr_line,LEFT,&zap,placeline);mouse(LEFTBT);
	cmmd("place right point;:k40%+i%+i%-+d%f",curr_line,RIGHT,&zap,placeline);mouse(RIGHTBT);
	cmmd("delete line;:k40%+i%-+d%f",curr_line,&zap,delline);
	cmmd("exit");Ex_cmd=mmcmd;mouse(MIDBT);
	
	while(thismu()!=Ex_cmd);
	if (zap) {
		curr_line->prev->next=curr_line->next;
		curr_line->next->prev=curr_line->prev;
		scrf->linebuf.count--;
		curr_line=NULL;
	}
}

/*
 * C R E A T E _ B O X
 */
create_box()
{
}

/*
 * C R E A T E _ L I N E
 */
create_line()
{
	struct lineint *lineint,*p,*n;
	int cmd, Draw_cmd, Forg_cmd, Ex_cmd;
	int x1,x2,y1,y2;
	
	newmenu("Create line.");
	cmmd("start/finish");Draw_cmd=mmcmd;mouse(LEFTBT);
	cmmd("forget it");Forg_cmd=mmcmd;mouse(MIDBT);
	cmmd("exit");Ex_cmd=mmcmd;mouse(RIGHTBT);
	
	while((cmd=thismu())!=Ex_cmd) {
		if(cmd==Draw_cmd) {
			lineint=(struct lineint *)Calloc(1,sizeof(struct lineint));
			lineint->x1=cursor.k<<dg->xshift;
			lineint->y1=cursor.l<<dg->yshift;
			newline();
			setmsv(stretchline);
			while(((cmd=thismu())!=Draw_cmd)&&(cmd!=Forg_cmd));
			lineint->x2=cursor.k<<dg->xshift;
			lineint->y2=cursor.l<<dg->yshift;
			setmsv(curstrack);
			if(cmd==Draw_cmd) {
				for(n=p=scrf->linebuf.lineint;n;p=n,n=n->next);
				p->next=lineint;
				scrf->linebuf.count++;
				intens(0);
				moveto(lineint->x1,lineint->y1);
				lineto(lineint->x2,lineint->y2);
				intens(1);
				colour(OVERLAY2);
				moveto(lineint->x1,lineint->y1);
				lineto(lineint->x2,lineint->y2);
			}
			else {
				moveto(lineint->x1,lineint->y1);
				intens(0);
				lineto(lineint->x2,lineint->y2);
				intens(1);
			}
		}
	}
}

/*
 * D I S P T E X T
 *
 */
disptext(on, curr, col)
char *on;
int curr, col;
{
	if (scrf->idbox.cont[curr].display) {
		strcpy(on, "undisplayed");
		dispidtext(&scrf->idbox.cont[curr], scrf->idbox.b.posn.x, 
											scrf->idbox.b.posn.y, 0, col);
		scrf->idbox.cont[curr].display = 0;
	} else {
		strcpy(on, "displayed");
		scrf->idbox.cont[curr].display = 1;
		dispidtext(&scrf->idbox.cont[curr], scrf->idbox.b.posn.x, 
											scrf->idbox.b.posn.y, 1, col);
	}
}
/*
 * P L A C E T E X T
 *
 */
placetext()
{
	int x, y;
	
	dispidtext(&scrf->idbox.cont[curr_chrom], scrf->idbox.b.posn.x, 
											scrf->idbox.b.posn.y, 0, RED);
	x = cursor.k << dg->xshift;
	y = cursor.l << dg->yshift;	
	scrf->idbox.cont[curr_chrom].x = x - scrf->idbox.b.posn.x;
	scrf->idbox.cont[curr_chrom].y = y - scrf->idbox.b.posn.y;
	dispidtext(&scrf->idbox.cont[curr_chrom], scrf->idbox.b.posn.x, 
											scrf->idbox.b.posn.y, 1, RED);
}

/*
 * P L A C E L I N E
 */
placeline(lineint,pos,zap)
int pos,*zap;
struct lineint *lineint;
{
	int x1,x2,y1,y2;
	
	/* clear old line */
	moveto(lineint->x1,lineint->y1);
	intens(0);
	colour(OVERLAY1);
	lineto(lineint->x2,lineint->y2);
	/* draw new one */
	if(pos==LEFT) {
		x1=cursor.k<<dg->xshift;
		y1=cursor.l<<dg->yshift;	
		x2=x1+lineint->x2-lineint->x1;
		y2=y1+lineint->y2-lineint->y1;
	}
	else {
		x2=cursor.k<<dg->xshift;
		y2=cursor.l<<dg->yshift;	
		x1=x2+lineint->x1-lineint->x2;
		y1=y2+lineint->y1-lineint->y2;
	}
	moveto(x1,y1);
	intens(1);
	lineto(x2,y2);
	lineint->x1=x1;
	lineint->y1=y1;
	lineint->x2=x2;
	lineint->y2=y2;
	*zap=0;
}

/*
 * D E L L I N E
 */
delline(lineint,zap)
int *zap;
struct lineint *lineint;
{
	intens(0);
	colour(OVERLAY1);
	moveto(lineint->x1,lineint->y1);
	lineto(lineint->x2,lineint->y2);
	intens(1);
	*zap=1;
}

/*
 * C H S T R
 *
 */
chstr(str, display)
char *str, *display;
{
	if (*str == '\0') {
		scrf->idbox.cont[curr_chrom].display = 1;
		disptext(display, curr_chrom, RED);
		displin(0, 1);
		putprompt();
	} else {
		dispidtext(&scrf->idbox.cont[curr_chrom], scrf->idbox.b.posn.x, 
												scrf->idbox.b.posn.y, 0, RED);
		if (strlen(str) <= DISPTEXT)
			strcpy(scrf->idbox.cont[curr_chrom].text, str);
		else {
			fprintf(stderr, "Input string is too long\n");
			strncpy(scrf->idbox.cont[curr_chrom].text, str, DISPTEXT-1);
		}
		if (scrf->idbox.cont[curr_chrom].display) {
			dispidtext(&scrf->idbox.cont[curr_chrom], scrf->idbox.b.posn.x, 
												scrf->idbox.b.posn.y, 1, RED);
		} else {
			disptext(display, curr_chrom, RED);
			displin(0, 1);
			putprompt();
		}
	}
}	
/*
 * C H S I Z E
 *
 */
chsize(size)
int *size;
{
	dispidtext(&scrf->idbox.cont[curr_chrom], scrf->idbox.b.posn.x, 
											scrf->idbox.b.posn.y, 0, RED);
	scrf->idbox.cont[curr_chrom].size = *size;
	dispidtext(&scrf->idbox.cont[curr_chrom], scrf->idbox.b.posn.x, 
											scrf->idbox.b.posn.y, 1, RED);
}
/*
 * I N C S T A C K 
 *
 */
incstack(num, inc)
int num, inc;
{
	struct box *b;
	int i;
	
	if (curr_chrom == -1) {
		fprintf(stderr, "There is no current box.\n\007");
		return(0);
	}
	for (i = 0; i < scrf->viewed_stack; i++) {
		if (i == curr_chrom)
			display_box(&scrf->stk[i], 0, RED, STACK);
		else
			display_box(&scrf->stk[i], 0, BLUE, STACK);
	}
	for (i = 0; i < scrf->viewed_stack; i++) {
		b = &scrf->stk[i];
		if (inc) {
			b->width += num;
			b->posn.x += (i * num);
		} else {
			b->width -= num;
			b->posn.x -= (i * num);
			if (b->width < 0)
				b->width = 0;
			if (b->posn.x < 0)
				b->posn.x = 0;
		}
	}
	for (i = 0; i < scrf->viewed_stack; i++) {
		if (i == curr_chrom)
			display_box(&scrf->stk[i], 1, RED, STACK);
		else
			display_box(&scrf->stk[i], 1, BLUE, STACK);
	}
	for (i = scrf->viewed_stack; i < NSTACKPOS + 2; i++) {
		if (inc) {
			scrf->stk[i].width += num;
			scrf->stk[i].posn.x += (i * num);
		} else {
			scrf->stk[i].width -= num;
			scrf->stk[i].posn.x -= (i * num);
			if (scrf->stk[i].width < 0)
				scrf->stk[i].width = 0;
			if (scrf->stk[i].posn.x < 0)
				scrf->stk[i].posn.x = 0;
		}
	}
}
/*
 * S T A C K _ L I N E
 *
 */
stack_line()
{
	int Ex_cmd, number, increase = 50;
	
	number = scrf->viewed_stack;
	newmenu("Change stack line");
	cmmd("place box;%f", placestack);mouse(MIDBT);
	cmmd("increase box width;%-+d%-+i%f", increase, 1, incstack);mouse(RIGHTBT);
	cmmd("decrease box width;%-+d%-+i%f", increase, 0, incstack);mouse(LEFTBT);
	cmmd("number of chromosomes in stack line;:k50%d%f", &number, viewed_stack);
	cmmd("abnormal stack label;:k50%6s", scrf->classes[ABNORMAL].tag);
	cmmd("amount to increase/decrease by;:k50%d", &increase);
	cmmd("exit"); Ex_cmd = mmcmd;mouse(LONGLB);
	
	while(thismu() != Ex_cmd) ;
}
/*
 * V I E W E D _ S T A C K
 *
 */
viewed_stack(number)
int *number;
{
	int i;
	
	if (*number >= NSTACKPOS + 2) {
		fprintf(stderr, "This number of positions is not available.\n");
		*number = scrf->viewed_stack;
	} else {
		for (i = 0; i < scrf->viewed_stack; i++) {
			if (i == curr_chrom)
				display_box(&scrf->stk[i], 0, RED, STACK);
			else
				display_box(&scrf->stk[i], 0, BLUE, STACK);
		}
		if (*number <= 0) {
			curr_chrom = -1;
			curr_box = NULL;
		}
		if ((curr_chrom >= *number) || (scrf->viewed_stack == 0 && *number > 0)) {
			curr_chrom = *number - 1;
			curr_box = &scrf->stk[curr_chrom];
		}
		scrf->viewed_stack = *number;
		for (i = 0; i < scrf->viewed_stack; i++) {
			if (i == curr_chrom)
				display_box(&scrf->stk[i], 1, RED, STACK);
			else
				display_box(&scrf->stk[i], 1, BLUE, STACK);
		}
	}
}
/*
 *  B O X M E N U -- menu for sizing a box!
 *
 */
boxmenu(b, type)
struct box *b;
int type;
{
	int amount = 50, Ex_cmd;
	
	newmenu("Box Manipulation.");
	if (type == STACK) {
		cmmd("place bottom left hand corner of box;:k40%-+d%-+d%f", b, type, 
												placestack);mouse(MIDBT);
	} else {
		cmmd("place bottom left hand corner of box;:k40%-+d%-+d%f", b, type, 
												placebox);mouse(MIDBT);
	}
	if (type != CHROMCLASS) {
		cmmd("enlarge box;:k40%-+d%-+d%+i%-+d%f", b, &amount, 1, type, resize);
																mouse(RIGHTBT);
		cmmd("shrink box;:k40%-+d%-+d%+i%-+d%f", b, &amount, 0, type, resize);
																mouse(LEFTBT);
		cmmd("amount to enlarge/shrink box;:k40%d", &amount);
	}
	cmmd("exit");Ex_cmd = mmcmd; mouse(LONGLB);
	
	while (thismu() != Ex_cmd);
}
/*
 * D I S P _ B O X --
 *
 */
disp_box(b, on, col)
struct box *b;
int on, col;
{
	register int x, y;
	
	colour(col);
	lwidth(2);
	intens(on);
	x = b->posn.x;	/* bottom left hand corner */
	y = b->posn.y;
	moveto(x, y);
	lineto(x, y + b->height);
	lineto(x + b->width, y + b->height);
	lineto(x + b->width, y);
	lineto(x, y);
}

/*
 * D I S P L A Y _ B O X --
 *
 */
display_box(b, on, col, type)
struct box *b;
int on, col;
{
	switch(type) {
		case IDBOX :
		case CHROMCLASS:
		case STACK :
			if (b == NULL)
				return(0);
			disp_box(b, on, col);
			break;
		case CHROM :
			disp_class(&scrf->classes[curr_chrom], on, col);
			break;
		case IDTEXT :
			dispidtext(&scrf->idbox.cont[curr_chrom], scrf->idbox.b.posn.x, 
												scrf->idbox.b.posn.y, on, col);
			break;
		case IDLINE :
			display_line(curr_line,on,col);
			break;
	}
}
/*
 * R E C A L C T X T P O S
 *
 */
recalctxtpos(newx, newy)
int newx, newy;
{
	int i, tx, ty;
	struct box *b;
	
	b = &scrf->idbox.b;	
	for (i = 0; i < MAXTEXT; i++) {
		tx = b->posn.x + scrf->idbox.cont[i].x;
		ty = b->posn.y + scrf->idbox.cont[i].y;
		if ((tx < b->posn.x) || (tx > b->posn.x + b->width) || (ty < b->posn.y)
						|| (ty > b->posn.y + b->height)) {
			/* outside of text box */
			scrf->idbox.cont[i].x = tx - newx;
			scrf->idbox.cont[i].y = ty - newy;
		}
	}
}			
/*
 * P L A C E B O X --
 *
 */
placebox(b, type)
struct box *b;
int type;
{
	int x, y;
	
	display_box(b, 0, RED, type);
	if (type == IDBOX)
		dispboxtext(&scrf->idbox.b, 0, BLUE);
	x = cursor.k<<dg->xshift;
	y = cursor.l<<dg->yshift;
	if (type == IDBOX)
		recalctxtpos(x, y);
	init_box(b, x, y, b->width, b->height);
	if (type == IDBOX)
		dispboxtext(&scrf->idbox.b, 1, BLUE);
	display_box(b, 1, RED, type);
}
/*
 * P L A C E S T A C K
 *
 */
placestack()
{
	struct box *b;
	int i, x, y;
	
	if (curr_chrom == -1) {
		fprintf(stderr, "There is no current box.\n\007");
		return(0);
	}
	x = cursor.k<<dg->xshift;
	y = cursor.l<<dg->yshift;
	for (i = 0; i < scrf->viewed_stack; i++) {
		b = &scrf->stk[i];
		if (i == curr_chrom) {
			display_box(b, 0, RED, STACK);
			init_box(b, x, y, b->width, b->height);
			display_box(b, 1, RED, STACK);
		} else {
			display_box(b, 0, BLUE, STACK);
			init_box(b, b->posn.x, y, b->width, b->height);
			display_box(b, 1, BLUE, STACK);
		}
	}
	for (i = scrf->viewed_stack; i < NSTACKPOS + 2; i++)
		init_box(&scrf->stk[i], scrf->stk[i].posn.x, y, scrf->stk[i].width, 
														scrf->stk[i].height);
}
/*
 * R E S I Z E --
 *
 */
resize(b, size, inc, type)
struct box *b;
int *size, inc, type;
{
	int xper, yper, oldwidth, oldheight, tx, ty, i;
	
	display_box(b, 0, RED, type);
	if (type == IDBOX)
		dispboxtext(b, 0, BLUE);
	oldwidth = b->width;
	oldheight = b->height;
	if (inc) {
		b->height += *size;
		b->width += *size;
	} else {
		b->height -= *size;
		if (b->height < 0)
			b->height = 0;
		b->width -= *size;
		if (b->width < 0)
			b->width = 0;
	}
	if (type == IDBOX) {
		for (i = 0; i < MAXTEXT; i++) {
			tx = b->posn.x + scrf->idbox.cont[i].x;
			ty = b->posn.y + scrf->idbox.cont[i].y;
			if ((tx > b->posn.x) && (tx < b->posn.x + b->width) && (ty > b->posn.y)
							&& (ty < b->posn.y + b->height)) {
				/* inside of text box */
				xper = scrf->idbox.cont[i].x * 1000 / oldwidth;
				yper = scrf->idbox.cont[i].y * 1000 / oldheight;
				scrf->idbox.cont[i].x = (xper * b->width) / 1000;
				scrf->idbox.cont[i].y = (yper * b->height) / 1000;
				if (scrf->idbox.cont[i].x <= 0)
					scrf->idbox.cont[i].x = 24;
				if (scrf->idbox.cont[i].y <= 0)
					scrf->idbox.cont[i].y = 24;
			}
		}
		dispboxtext(b, 1, BLUE);
	}
	display_box(b, 1, RED, type);
}
/*
 * C H A N G E T A G
 *
 */
changetag(tmpstr, string)
char *tmpstr, *string;
{
	disp_class(&scrf->classes[curr_chrom], 0, RED, 0);
	if (strlen(tmpstr) > 6) {
		fprintf(stderr, "New tag is too long.\n");
		strcpy(tmpstr, string);
	} else {
		strcpy(string, tmpstr);
	}
	disp_class(&scrf->classes[curr_chrom], 1, RED, 0);
}
/*
 * D O T X T S T R
 *
 */
dotxtstr(txt, str)
int *txt;
char *str;
{
	if (*txt)
		strcpy(str, "text");
	else
		strcpy(str, "binary");
}
/*
 *	D I S P _ M E S S G  --  Display message telling user to press return
 *			
 */
static disp_messg()
{
	screengo(OUTLINE - 1,0);
	printf("Press <Return> to write file\n");
}
/*
 * W R I T E M E N U
 *
 */
writemenu()
{
	char txtstr[8];
	int Quit_cmd, cmd;
	
	dotxtstr(&txt, txtstr);
	newmenu("write output file");
	cmmd("output file name;:k40%80s%-f", outfile, disp_messg);
	cmmd("type of output file;:k40%+t%-s%f", &txt, txtstr, dotxtstr);
	cmmd("quit"); Quit_cmd = mmcmd;

	while((cmd = thismu()) != Quit_cmd) {
		if (cmd == 0) {
			writefile(outfile, txt);
			filewritten = 1;
			break;
		}
	}
}

display_group()
{
}
display_all()
{
}
