/*
 *      S E T H C O P Y .  C -- Setup screen layout file
 *
 *
 *  Written: Sarah E. A. Jones
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    10th July 1987
 *
 *  Modifications
 *	14 Sep 1989		dcb		Errno for V2
 *
 */
#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <dispform.h>
#include <ddgs.h>
#include <scrformat.h>

#define SCALE 6
#define MAXOPTS 7

/*
 * Local structures
 */
static struct sfhead {
	short size;
	short version;
};
struct hcobject *ptr;
char *options[] = {
	"class",
	"stack",
	"metaphase",
	"text",
	"chromclass",
	"viewed",
	"karyotype scale"
};
extern struct hcopyform *hcpy;
extern DDGS *dg;
extern int errno;

static char *skipword();
static char	*skipwhite();
static int setclass(), settext(), setmet();
/*
 *	N E X T L I N E  --
 *
 */
static int nextline(fd,buffer,len)
int		fd, len;
char	*buffer;
{
	int		cnt;

	while ((cnt = readln(fd,buffer,len)) == 1)		/* loose blank lines */
		;
	if (cnt <= 0)
		return(0);
	else {
		buffer[cnt-1] = '\0';
		return(cnt);
	}
}

/*
 *	S K I P W O R D  --
 *
 */
static char *skipword(str)
char	*str;
{
	while (*str && (*str != ' ') && (*str != '\t'))
		str++;
	str = skipwhite(str);
	return(str);
}	

/*
 *	S K I P W H I T E  --
 *
 */
static char *skipwhite(str)
char	*str;
{
	while (*str && ((*str == ' ') || (*str == '\t')))
		str++;
	return(str);
}
/*
 * M A T C H W O R D
 *
 */
int
matchword(str1, str2)
char *str1, *str2;
{
	while((*str1 != '\0') && (*str2 != '\0')) {
		if (toupper(*str1 & 0x7f) != toupper(*str2 & 0x7f))
			return(1);
		str1++;
		str2++;
	}
	return(0);
}
/*
 * G E T N E X T
 *
 */
getnext()
{
	if (hcpy->hco == NULL) {
		hcpy->hco = (struct hcobject *) Calloc(sizeof(struct hcobject), 1);
		ptr = hcpy->hco;
	} else {
		ptr->next = (struct hcobject *) Calloc(sizeof(struct hcobject), 1);
		ptr = ptr->next;
	}
	ptr->next = NULL;
	ptr->ass = NULL;
}
/*
 * R E A D F I L E
 *
 */
struct hcopyform *
readfile(infile)
char *infile;
{
	int fd, txt;
	struct sfhead tmp, *sf;
	
	sf = &tmp;
	if ((fd = open(infile, 1)) == -1) {
		fprintf(stderr,"Can't open file %s\n", infile);
		exit(errno);
	} else {
		read(fd, sf, sizeof(struct sfhead));
		if (sf->version == VERSION)
			txt = 0;
		else
			txt = 1;
		lseek(fd, 0, 0);
	}
	inithcpy();
	if (txt)
		readtxtfile(fd);
	else
		readbinfile(fd);
	convert(hcpy->hco, 1);
	close(fd);
	return(hcpy);
}
/*
 * I N I T H C O P Y
 *
 */
inithcpy()
{
	hcpy = (struct hcopyform *) Calloc(sizeof(struct hcopyform), 1);
	hcpy->size = sizeof(struct hcopyform);
	hcpy->version = VERSION;
	hcpy->hco = NULL;
	hcpy->chromclasses = 0;
	hcpy->viewed_stack = 0;
	hcpy->karscale = 32;	
}
/*
 *	R E A D T X T F I L E --
 *
 */
static
readtxtfile(in)
int in;
{
	int		i;
	char	buffer[200], *buf;

	while (nextline(in, buffer, sizeof(buffer))) {
		buf = buffer;
		i = 0;
		while ((i < MAXOPTS) && (matchword(buf, options[i])))
			i++;
		if (i < MAXOPTS) {
			buf = skipword(buf);
			switch(i) {
				case VIEWED :
					hcpy->viewed_stack = atoi(buf);
					break;
				case CHCLASS :
					hcpy->chromclasses = atoi(buf);
					break;
				case CLASS :
					setclass(CLASS, buf);
					break;
				case STK :
					setclass(STK, buf);
					break;
				case METAPHASE :
					setmet(buf);
					break;
				case TXT :
					settext(buf);
					break;
				case KARSCALE :
					buf = skipword(buf);
					hcpy->karscale = atoi(buf);
					break;
			}
		}
	}
}
/*
 * S E T B O X
 *
 */
static
char *
setbox(str)
char *str;
{
	getnext();
	str = skipwhite(str);
	ptr->b.posn.x = atoi(str);
	str = skipword(str);
	ptr->b.posn.y = atoi(str);
	str = skipword(str);
	ptr->b.width = atoi(str);
	str = skipword(str);
	ptr->b.height = atoi(str);
	return(str);
}
/*
 * S E T C L A S S
 *
 */
static
setclass(type, str)
int type;
char *str;
{
	int length;

	str = setbox(str);	
	ptr->type = type;
	str = skipword(str);
	ptr->class = atoi(str);
}

/*
 * S E T M E T
 *
 */
static
setmet(str)
char *str;
{
	str = setbox(str);
	ptr->type = METAPHASE;
}
/*
 * S E T T E X T
 *
 */
static
settext(str)
char *str;
{
	str = setbox(str);
	ptr->type = TXT;
	str = skipword(str);
	ptr->asstxt = atoi(str);
	str = skipword(str);
	strcpy(ptr->txt, str);
}
/*
 * C O N V E R T
 *
 */
convert(p, small)
struct hcobject *p;
int small;
{
	while (p != NULL) {
		if (p->ass != NULL)
			convert(p->ass, small);
		if (small)
			makesmall(p);
		else
			makebig(p);
		p = p->next;
	}
}
/*
 * M A K E S M A L L 
 *
 */
makesmall(p)
struct hcobject *p;
{
	if (p->type != TXT) {
		p->b.width /= SCALE;
		p->b.height /= SCALE;
	}
	p->b.posn.x /= SCALE;
	p->b.posn.y /= SCALE;
}
/*
 * M A K E B I G
 *
 */
makebig(p)
struct hcobject *p;
{
	if (p->type != TXT) {
		p->b.width *= SCALE;
		p->b.height *= SCALE;
	}
	p->b.posn.x *= SCALE;
	p->b.posn.y *= SCALE;
}
/*
 * R E A D B I N F I L E
 *
 */
readbinfile(fd)
int fd;
{
	read(fd, hcpy, sizeof(struct hcopyform));
	if (hcpy->hco != NULL)
		readall(fd, &(hcpy->hco));
}
/*
 * R E A D A L L
 *
 */
readall(fd, ptr)
int fd;
struct hcobject **ptr;
{
	while (*ptr != NULL) {
		*ptr = (struct hcobject *) Calloc (sizeof(struct hcobject), 1);
		read(fd, *ptr, sizeof(struct hcobject));
		if ((*ptr)->ass != NULL)
			readall(fd, &((*ptr)->ass));
		ptr = &((*ptr)->next);
	}
}
/*
 * W R I T E F I L E
 *
 */
writefile(outfile, txt)
char *outfile;
int txt;
{
	FILE *rc;
	int fd;
		
	if ((fd = open(outfile, 2)) == -1) {
		fd = creat(outfile, 2);
		if (fd == -1) {
			printf("ERROR on trying to create output file\n");
			return(0);
		}
	}
	convert(hcpy->hco, 0);
	if (txt) {
		close(fd);
		printf("The text file is now being written.\n");
		if ((rc = fopen(outfile, "w")) != NULL) {
			writetxt(rc);
			fclose(rc);
		} else {
			printf("ERROR on opening text file.\n");
			printf("The error is %d\n", errno);
		}
	} else {
		printf("The binary file is now being written\n");
		writebinfile(fd);
		close(fd);
	}
	convert(hcpy->hco, 1);
}
/*
 * W R I T E T X T 
 *
 */
static
writetxt(fp)
FILE *fp;
{
	fprintf(fp, "chromclass  %d\n", hcpy->chromclasses);
	fprintf(fp, "viewed   %d\n", hcpy->viewed_stack);
	fprintf(fp, "karyotype scale  %d\n", hcpy->karscale);
	writedata(fp, hcpy->hco);
}
/*
 * W R I T E D A T A
 *
 */
writedata(fp, ptr)
FILE *fp;
struct hcobject *ptr;
{
	while (ptr != NULL) {
		writebox(fp, ptr);
		if (ptr->ass != NULL)
			writedata(fp, ptr->ass);
		ptr = ptr->next;
	}
}
/*
 * W R I T E B O X
 *
 */
writebox(fp, ptr)
FILE *fp;
struct hcobject *ptr;
{
	switch(ptr->type) {
		case TXT :
			fprintf(fp, "text ");
			break;
		case CLASS :
			fprintf(fp, "class ");
			break;
		case STK :
			fprintf(fp, "stack ");
			break;
		case METAPHASE :
			fprintf(fp, "metaphase ");
			break;
	}
	fprintf(fp, "%4d %4d %4d %4d", ptr->b.posn.x, ptr->b.posn.y, ptr->b.width,
													ptr->b.height);
	if (ptr->type == TXT) {
		fprintf(fp, " %4d", ptr->asstxt);
		fprintf(fp, " %s\n", ptr->txt);
	} else if (ptr->type == CLASS || ptr->type == STK)
		fprintf(fp, " %d\n", ptr->class);
	else
		fprintf(fp, "\n");
}
/*
 * W R I T E B I N F I L E
 *
 */
writebinfile(fd)
int fd;
{
	write(fd, hcpy, sizeof(struct hcopyform));
	if (hcpy->hco != NULL)
		writeall(fd, hcpy->hco);
}
/*
 * W R I T E A L L
 *
 */
writeall(fd, ptr)
int fd;
struct hcobject *ptr;
{
	while (ptr != NULL) {
		write(fd, ptr, sizeof(struct hcobject));
		if (ptr->ass != NULL)
			writeall(fd, ptr->ass);
		ptr = ptr->next;
	}
}
