/*
 *      S E T S C R E E N . C  --  Setup screen layout file
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    10th July 1987
 *
 *  Modifications
 *
 *	19 Jul 1989		CAS			Define errno for v2.2
 *   5 Apr 1989		SEAJ		Added height to disppos and viewed_stack.
 *	19 Sep 1988		CAS/SEAJ	Change blankline test in nextline
 */
#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <dispform.h>

extern int errno;
/*
 *	Local global storage
 */
int		verbose = 0;

/*
 *	Local forward declarations
 */
static char *skipword();
static char	*skipwhite();

/*
 *	I N T E R R U P T --  Signal catcher
 *
 */
interrupt(sig)
int		sig;
{
}

main(argc,argv)
int		argc;
char	**argv;
{
	char	*swptr, **targv, **sargv;
	int		targc;
	int		infile,outfile;

#ifdef INTERRUPT
	intercept(interrupt);
#endif INTERRUPT
	targc = argc; targv = argv;
	argc--; argv++;			/* dump process name */
	while (argc > 0) {
		if (*argv[0] == '-') {
			swptr = *argv;
			*argv = NULL;
			while (*(++swptr)) {
				switch (*swptr) {
					case '?':
						printf("Syntax:   setscreen <options> <infile> <outfile> <options>\n");
						printf("Function: \n");
						printf("Options:  v - Show version number\n");
						printf("          d - Display settings\n");
						printf("          ? - Help\n");
						exit(0);
					case 'v':
						printf("Setscreen - Version 0.1\n");
						exit(0);
					case 'd':
						verbose = 1;
						break;
#ifdef PARAMS
					case 'x':
						param = *++argv;
						*argv = NULL; argc--;
						break;
#endif PARAMS
					default:
						printf("setscreen - unknown option\n");
				}
			}
		}
		argv++; argc--;
	}
/*
 * Compact params to fill holes left by switches
 */
	argc = --targc; argv = ++targv;				/* dump process name */
	sargv = targv;
	while (targc-- > 0) {						/* Squeez + fill the holes */
		if (*targv)
			*sargv++ = *targv;
		else
			argc--;
		targv++;
	}
/*
 *	Use params 
 */
	if (argc > 1) {
		if ((outfile = creat(argv[1],2)) == -1) {
			fprintf(stderr,"Can't open file %s\n",argv[1]);
			exit(errno);
		}
	} else
		outfile = fileno(stdout);
	if (argc) {
		if ((infile = open(*argv,1)) == -1) {
			fprintf(stderr,"Can't open file %s\n",*argv);
			exit(errno);
		}
	} else
		infile = fileno(stdin);
	dofile(infile,outfile);
	if (infile != fileno(stdin))
		close(infile);
	if (outfile != fileno(stdout))
		close(outfile);
}


/*
 *	D O F I L E --
 *
 */
dofile(in,out)
int		in,out;
{
	int		i, cnt;
	struct screenform *scrf;
	char	buffer[200];
	scrf = (struct screenform *) Calloc(sizeof(struct screenform),1);
	scrf->size = sizeof(struct screenform);
	scrf->version = VERSION;
	getidbox(in,&scrf->idbox);
	for (i=0; i<MAXCLASS; i++) {
		if (!nextline(in,buffer,sizeof(buffer)))
			break;
		getdisppos(buffer,&scrf->disppos[i]);
		if (verbose) fprintf(stderr,"Class %s, at %d %d, Def %d, actual %d space %d height %d\n",scrf->disppos[i].tag,
							scrf->disppos[i].col,scrf->disppos[i].lin,
							scrf->disppos[i].defaultpos,scrf->disppos[i].npos,
							scrf->disppos[i].space, scrf->disppos[i].height);
		if (scrf->disppos[i].col == -1)
			break;
	}
	if (nextline(in,buffer,sizeof(buffer)))
		getstacklin(buffer,&scrf->stacklin);
	if (verbose) fprintf(stderr,"Stackline %d\n",scrf->stacklin);
	if (nextline(in,buffer,sizeof(buffer)))
		getstacklin(buffer,&scrf->viewed_stack);
	if (verbose) fprintf(stderr,"Viewed_stack %d\n",scrf->viewed_stack);
	if (nextline(in,buffer,sizeof(buffer)))
		getstackcol(buffer,scrf->stackcol,NSTACKPOS+2);
	if (verbose) {
		fprintf(stderr,"Stack columns:");
		for (i=0; i<NSTACKPOS+2; i++) {
			if (scrf->stackcol[i] == 0)
				break;
			fprintf(stderr," %d",scrf->stackcol[i]);
		}
		fprintf(stderr,"\n");
	}
	if (nextline(in,buffer,sizeof(buffer)))
		getcclasspos(buffer,&scrf->chromclass.x,&scrf->chromclass.y);
	if (nextline(in,buffer,sizeof(buffer)))
		if(!strncasecmp(buffer,"lines",5))
			getlines(in,atoi(buffer+6),&scrf->linebuf);
	fprintf(stderr,"Class box %d %d\n",scrf->chromclass.x,scrf->chromclass.y);
	
	write(out,scrf,sizeof(struct screenform));
	write(out,scrf->linebuf.lineint,scrf->linebuf.count*sizeof(struct lineint));
}
getidbox(in,idbox)
int		in;
struct	databox *idbox;
{
	char	buffer[200];

	if (nextline(in,buffer,sizeof(buffer))) {
		getboxpos(buffer,&idbox->position,&idbox->width,&idbox->height);
	}
	if (verbose) fprintf(stderr,"IDBox at %d %d, width %d, height %d\n",idbox->position.x,
				idbox->position.y,idbox->width,idbox->height);
	if (nextline(in,buffer,sizeof(buffer))) {
		getbanner(buffer,&idbox->banner);
	}
	if (verbose) fprintf(stderr,"Box Banner :%s: at %d %d size %d\n",idbox->banner.text,idbox->banner.x,idbox->banner.y,idbox->banner.size);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->case);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->slide);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->cell);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->operator);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->date);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->supervisor);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->superdate);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->patname);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->comment1);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->comment2);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->karyotype);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->labname);
}

/*
 *	G E T B O X T E X T  -- 
 *
 */
getboxtext(str,tp)
char	*str;
struct	idtext *tp;
{
	tp->display = atoi(str);
	if (tp->display) {
		str = skipword(str);
		tp->len = atoi(str);
		str = skipword(str);
		tp->x = atoi(str);
		str = skipword(str);
		tp->y = atoi(str);
		str = skipword(str);
		tp->size = atoi(str);
		str = skipword(str);
		strncpy(tp->text,str,DISPTEXT -1);
		if (verbose)
			fprintf(stderr,"IDBOX text :%s: at %d %d, size %d, len %d\n",tp->text,tp->x,tp->y,tp->size,tp->len);
	}
}

/*
 *	G E T B O X P O S  --
 *
 */
getboxpos(str,position,width,height)
char	*str;
struct	pos	*position;
int		*width, *height;
{
	position->x = atoi(str);
	str = skipword(str);
	position->y = atoi(str);
	str = skipword(str);
	*width = atoi(str);
	str = skipword(str);
	*height = atoi(str);
}

/*
 *	G E T B A N N E R  --  
 *
 */
getbanner(str,ban)
char	*str;
struct	txt *ban;
{
	ban->x = atoi(str);
	str = skipword(str);
	ban->y = atoi(str);
	str = skipword(str);
	ban->size = atoi(str);
	str = skipword(str);
	strncpy(ban->text,str,40);
}

/*
 *	G E T D I S P P O S  --
 *
 */
getdisppos(str,kdp)
char	*str;
struct kdisppos *kdp;
{
	str = skipwhite(str);
	kdp->col = atoi(str);
	str = skipword(str);
	kdp->lin = atoi(str);
	str = skipword(str);
	kdp->defaultpos = atoi(str);
	str = skipword(str);
	kdp->npos = atoi(str);
	str = skipword(str);
	kdp->space = atoi(str);
	str = skipword(str);
	kdp->height = atoi(str);
	str = skipword(str);
	strncpy(kdp->tag,str,5);
}

/*
 *	G E T S T A C K L I N  --
 *
 */
getstacklin(str,line)
char	*str;
int		*line;
{
	str = skipwhite(str);
	*line = atoi(str);
}

/*
 *	G E T S T A C K C O L  --
 *
 */
getstackcol(str,stack,max)
char	*str;
int		*stack, max;
{
	str = skipwhite(str);
	while (max-- && str && *str) {
		*stack++ = atoi(str);
		str = skipword(str);
	}
}

/*
 *	G E T C C L A S S P O S  --  Get position of chro class box
 *
 */
getcclasspos(str,x,y)
char	*str;
int		*x,*y;
{
	str = skipwhite(str);
	*x = atoi(str);
	str = skipword(str);
	*y = atoi(str);
}
/*
 * G E T L I N E S --
 */
getlines(in,count,linebuf)
int in,count;
struct linebuf *linebuf;
{
	int i;
	char buffer[200],*str;

	linebuf->lineint=Calloc(count,sizeof(struct lineint));
	for(i=0;(i<count)&&nextline(in,buffer,sizeof(buffer));i++) {
		str=buffer;
		linebuf->lineint[i].x1=atoi(str);
		str=skipword(str);
		linebuf->lineint[i].y1=atoi(str);
		str=skipword(str);
		linebuf->lineint[i].x2=atoi(str);
		str=skipword(str);
		linebuf->lineint[i].y2=atoi(str);
	}
	linebuf->count=i;
}

/*
 *	N E X T L I N E  --
 *
 */
static int nextline(fd,buffer,len)
int		fd, len;
char	*buffer;
{
	int		cnt;

	while ((cnt = readln(fd,buffer,len)) == 1)		/* loose blank lines */
		;
	if (cnt <= 0)
		return(0);
	else {
		buffer[cnt-1] = '\0';
		return(cnt);
	}
}

/*
 *	S K I P W O R D  --
 *
 */
static char *skipword(str)
char	*str;
{
	while (*str && (*str != ' ') && (*str != '\t'))
		str++;
	str = skipwhite(str);
	return(str);
}	

/*
 *	S K I P W H I T E  --
 *
 */
static char *skipwhite(str)
char	*str;
{
	while (*str && ((*str == ' ') || (*str == '\t')))
		str++;
	return(str);
}
