/*
 *      S E T S C R F O R M . C  --  Setup screen layout file
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    10th July 1987
 *
 *  Modifications
 *
 *  20 Jul 1991		ih			Add line interval handling, also some tidying up of error checking
 *	14 Sep 1989		dcb			Errno for V2.2
 *  22 Mar 1989		SEAJ		Added height into disppos.
 *  22 Mar 1989		SEAJ		Modified to be called from somewhere else.
 *	19 Sep 1988		CAS/SEAJ	Change blankline test in nextline
 */
#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <dispform.h>
#include <scrformat.h>


/*
 * Local structures
 */
static struct sfhead {
	short size;
	short version;
};

/*
 *	Local global storage
 */
static struct screenform *scrformat;

/*
 * external globals
 */
extern struct scrform *scrf;
extern int errno;

/*
 * external functions
 */
extern struct screenform *convscrformat();
extern void screxit();

/*
 *	Local forward declarations
 */
static char *skipword();
static char	*skipwhite();
static int writetxt(), convert();

/*
 * R E A D F I L E
 *
 */
struct scrform *
readfile(infile)
char *infile;
{
	int fd,txt,size;
	struct sfhead tmp, *sf;

	sf = &tmp;
	if ((fd = open(infile, 1)) == -1) {
		fprintf(stderr,"Can't open file %s\n", infile);
		alldone(errno);
	} else {
		read(fd, sf, sizeof(struct sfhead));
		lseek(fd, 0, 0);
		if ((sf->size==(size=_gs_size(fd)))&&(sf->version<=VERSION))
			txt = 0;
		else
			txt = 1;
	}
	scrformat = (struct screenform *) Calloc(sizeof(struct screenform), 1);
	scrf = (struct scrform *) Calloc(sizeof(struct scrform), 1);
	scrformat->size=size;
	scrformat->version = VERSION;
	if (txt)
		readtxtfile(fd);
	else
		readbinfile(fd);
	close(fd);
	init_struct(scrf, scrformat);
	return(scrf);
}
/*  N.B. the file read routines use a struct screenform scrformat.
 *  Since it is used only for storing file images the linked
 *  line interval lists do not need to be physically
 *  linked until they are copied to the shared scrf structure. --ih.
 *
 *
 *	R E A D T X T F I L E --
 *
 */
static
readtxtfile(in)
int in;
{
	int		i, cnt;
	char	buffer[200];

	fprintf(stderr, "This is a text file\n");
	getidbox(in,&scrformat->idbox);
	for (i=0; i<MAXCLASS; i++) {
		if (!nextline(in,buffer,sizeof(buffer)))
			break;
		getdisppos(buffer,&scrformat->disppos[i]);
		if (scrformat->disppos[i].col == -1)
			break;
	}
	if (nextline(in,buffer,sizeof(buffer)))
		getstackint(buffer,&scrformat->stacklin);
	if (nextline(in,buffer,sizeof(buffer)))
		getstackint(buffer,&scrformat->viewed_stack);
	if (nextline(in,buffer,sizeof(buffer)))
		getstackcol(buffer,scrformat->stackcol,NSTACKPOS+2);
	if (nextline(in,buffer,sizeof(buffer)))
		getcclasspos(buffer,&scrformat->chromclass.x,&scrformat->chromclass.y);
	if (nextline(in,buffer,sizeof(buffer)))
		if(!strncasecmp(buffer,"lines",5))
			getlines(in,atoi(buffer+6),&scrformat->linebuf);
}

/*
 * R E A D B I N F I L E --
 *
 */
static
readbinfile(in)
int in;
{
	struct lineint *p,*l;
	int i;
	
	read(in,scrformat,sizeof(struct screenform));
	if((scrformat->version>=3)&&(scrformat->linebuf.count)) {
		scrformat->linebuf.lineint=(struct lineint *)Calloc(scrformat->linebuf.count,sizeof(struct lineint));
		read(in,scrformat->linebuf.lineint,scrformat->linebuf.count*sizeof(struct lineint));
	}
	scrformat = convscrformat(scrformat);
}

/*
 * The following routines are for parsing the text file input
 *
 *
 * G E T I D B O X
 *
 */
getidbox(in,idbox)
int		in;
struct	databox *idbox;
{
	char	buffer[200];

	if (nextline(in,buffer,sizeof(buffer))) {
		getboxpos(buffer,&idbox->position,&idbox->width,&idbox->height);
	}
	if (nextline(in,buffer,sizeof(buffer))) {
		getbanner(buffer,&idbox->banner);
	}
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->case);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->slide);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->cell);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->operator);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->date);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->supervisor);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->superdate);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->patname);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->comment1);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->comment2);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->karyotype);
	if (nextline(in,buffer,sizeof(buffer)))
		getboxtext(buffer,&idbox->labname);
}

/*
 *	G E T B O X T E X T  -- 
 *
 */
getboxtext(str,tp)
char	*str;
struct	idtext *tp;
{
	str = skipwhite(str);
	tp->display = atoi(str);
	str = skipword(str);
	tp->len = atoi(str);
	str = skipword(str);
	tp->x = atoi(str);
	str = skipword(str);
	tp->y = atoi(str);
	str = skipword(str);
	tp->size = atoi(str);
	str = skipword(str);
	strncpy(tp->text,str,DISPTEXT -1);
}

/*
 *	G E T B O X P O S  --
 *
 */
getboxpos(str,position,width,height)
char	*str;
struct	pos	*position;
int		*width, *height;
{
	str = skipwhite(str);
	position->x = atoi(str);
	str = skipword(str);
	position->y = atoi(str);
	str = skipword(str);
	*width = atoi(str);
	str = skipword(str);
	*height = atoi(str);
}

/*
 *	G E T B A N N E R  --  
 *
 */
getbanner(str,ban)
char	*str;
struct	txt *ban;
{
	str = skipwhite(str);
	ban->x = atoi(str);
	str = skipword(str);
	ban->y = atoi(str);
	str = skipword(str);
	ban->size = atoi(str);
	str = skipword(str);
	strncpy(ban->text,str,40);
}

/*
 *	G E T D I S P P O S  --
 *
 */
getdisppos(str,kdp)
char	*str;
struct kdisppos *kdp;
{
	str = skipwhite(str);
	kdp->col = atoi(str);
	str = skipword(str);
	kdp->lin = atoi(str);
	str = skipword(str);
	kdp->defaultpos = atoi(str);
	str = skipword(str);
	kdp->npos = atoi(str);
	str = skipword(str);
	kdp->space = atoi(str);
	str = skipword(str);
	kdp->height = atoi(str);
	str = skipword(str);
	strncpy(kdp->tag,str,5);
}

/*
 *	G E T S T A C K I N T --
 *
 */
getstackint(str,line)
char	*str;
int		*line;
{
	str = skipwhite(str);
	*line = atoi(str);
}

/*
 *	G E T S T A C K C O L  --
 *
 */
getstackcol(str,stack,max)
char	*str;
int		*stack, max;
{
	str = skipwhite(str);
	while (max-- && str && *str) {
		*stack++ = atoi(str);
		str = skipword(str);
	}
}

/*
 *	G E T C C L A S S P O S  --  Get position of chro class box
 *
 */
getcclasspos(str,x,y)
char	*str;
int		*x,*y;
{
	str = skipwhite(str);
	*x = atoi(str);
	str = skipword(str);
	*y = atoi(str);
}

/*
 * G E T L I N E S --
 */
getlines(in,count,linebuf)
int in,count;
struct linebuf *linebuf;
{
	int i;
	char buffer[200],*str;
	struct lineint *l;

	if(!count) {
		linebuf->count=0;
		return;
	}
	l=linebuf->lineint=(struct lineint *)Calloc(count,sizeof(struct lineint));
	for(i=0;(i<count)&&nextline(in,buffer,sizeof(buffer));i++) {
		str=buffer;
		l->x1=atoi(str);
		str=skipword(str);
		l->y1=atoi(str);
		str=skipword(str);
		l->x2=atoi(str);
		str=skipword(str);
		l->y2=atoi(str);
		l++;
	}
	linebuf->count=i;
}

/*
 *	N E X T L I N E  --
 *
 */
static int nextline(fd,buffer,len)
int		fd, len;
char	*buffer;
{
	int		cnt;

	while ((cnt = readln(fd,buffer,len)) == 1)		/* loose blank lines */
		;
	if (cnt <= 0)
		return(0);
	else {
		buffer[cnt-1] = '\0';
		return(cnt);
	}
}

/*
 *	S K I P W O R D  --
 *
 */
static char *skipword(str)
char	*str;
{
	while (*str && (*str != ' ') && (*str != '\t'))
		str++;
	str = skipwhite(str);
	return(str);
}	

/*
 *	S K I P W H I T E  --
 *
 */
static char *skipwhite(str)
char	*str;
{
	while (*str && ((*str == ' ') || (*str == '\t')))
		str++;
	return(str);
}

/*
 * I N I T _ S T R U C T initialise the shared scrf structure
 *
 */
init_struct(new, old)
struct scrform *new;
struct screenform *old;
{
	int i;
	
	init_box(&new->idbox.b, old->idbox.position.x, old->idbox.position.y, old->idbox.width,
			old->idbox.height);
	init_banner(&new->idbox.cont[BANNER], &old->idbox.banner);
	init_id(&new->idbox.cont[CASE], &old->idbox.case);
	init_id(&new->idbox.cont[SLIDE], &old->idbox.slide);
	init_id(&new->idbox.cont[CELL], &old->idbox.cell);
	init_id(&new->idbox.cont[OPERATOR], &old->idbox.operator);
	init_id(&new->idbox.cont[DATE], &old->idbox.date);
	init_id(&new->idbox.cont[SUPERVISOR], &old->idbox.supervisor);
	init_id(&new->idbox.cont[SUPERDATE], &old->idbox.superdate);
	init_id(&new->idbox.cont[PATNAME], &old->idbox.patname);
	init_id(&new->idbox.cont[COMMENT1], &old->idbox.comment1);
	init_id(&new->idbox.cont[COMMENT2], &old->idbox.comment2);
	init_id(&new->idbox.cont[KARYOTYP], &old->idbox.karyotype);
	init_id(&new->idbox.cont[LABNAME], &old->idbox.labname);
	for (i = 0; i < MAXCLASS; i++) {
		scrf->classes[i].b.posn.x = -1;
		scrf->classes[i].b.posn.y = -1;
	}
	for (i = 0; i < MAXCLASS; i++) {
		init_class(&old->disppos[i], &new->classes[i]);
		if (old->disppos[i].col == -1)
			break;
	}
	init_box(&new->chromclass, old->chromclass.x, old->chromclass.y, 200, 200);
	for (i = 1; i < NSTACKPOS+2; i++) {
		init_box(&new->stk[i-1], old->stackcol[i], old->stacklin, 
							old->disppos[ABNORMAL].space, old->disppos[ABNORMAL].height);
	}
	new->viewed_stack = old->viewed_stack;
	init_linebuf(&new->linebuf,&old->linebuf);
}

/*
 * I N I T _ B A N N E R
 *
 */
init_banner(new, old)
struct txt *old;
struct idtext *new;
{
	new->x = old->x;
	new->y = old->y;
	new->size = old->size;
	strcpy(new->text, old->text);
	new->display = 1;
	new->len = 0;
}

/*
 * I N I T _ I D
 *
 */
init_id(new, old)
struct idtext *new, *old;
{
	new->display = old->display;
	new->len = old->len;
	new->x = old->x;
	new->y = old->y;
	new->size = old->size;
	strcpy(new->text, old->text);
}

/*
 * I N I T _ C L A S S 
 *
 */
init_class(kd, cl)
struct kdisppos *kd;
struct classpos *cl;
{
	init_box(&cl->b, (kd->col - kd->space / 2), kd->lin, kd->space, kd->height);
	cl->defaultpos = kd->defaultpos;
	cl->npos = kd->npos;
	strcpy(cl->tag, kd->tag);
}

/*
 * I N I T _ B O X --
 *
 */
init_box(b, x, y, width, height)
struct box *b;
int x, y, width, height;
{
	if (x >= 0 && y >= 0) {
		b->posn.x = x;
		b->posn.y = y;
	}
	b->width = width;
	b->height = height;
}

/*
 * I N I T L I N E B U F
 */
init_linebuf(new,old)
struct linebuf *new,*old;
{
	int i;
	struct lineint *l,*p;

	new->count=old->count;
	if(new->count) {
		new->lineint=(struct lineint *)Calloc(new->count,sizeof(struct lineint));
		_strass(new->lineint,old->lineint,new->count*sizeof(struct lineint));
	}
	/* link together */
	p=NULL;
	l=new->lineint;
	for(i=0;i<new->count;i++) {
		l->prev=p;
		l->next=l+1;
		p=l;
		l=l->next;
	}
	p->next=NULL;
}

/*
 * W R I T E F I L E
 *
 */
writefile(outfile, txt)
char *outfile;
int txt;
{
	FILE *rc;
	int fd;
		
	if ((fd = open(outfile, 2)) == -1) {
		fd = creat(outfile, 2);
		if (fd == -1) {
			printf("ERROR on trying to create output file\n");
			return(0);
		}
	}
	if (txt) {
		close(fd);
		printf("The text file is now being written.\n");
		if ((rc = fopen(outfile, "w")) != NULL) {
			writetxt(rc);
			fclose(rc);
		} else {
			printf("ERROR on opening text file.\n");
			printf("The error is %d\n", errno);
		}
	} else {
		printf("The binary file is now being written\n");
		convert(scrf, scrformat);
		write(fd, scrformat, sizeof(struct screenform));
		write(fd, scrformat->linebuf.lineint, scrformat->linebuf.count*sizeof(struct lineint));
		close(fd);
	}
}

/*
 * W R I T E T X T 
 *
 */
static
writetxt(fp)
FILE *fp;
{
	struct datab *id;
	struct classpos *cl;
	int i;
	struct lineint *l;
	
	id = &scrf->idbox;
	fprintf(fp, "%5d %5d %5d %5d\n", id->b.posn.x, id->b.posn.y, id->b.width, 
																id->b.height);
	fprintf(fp, "%4d %4d %3d %s\n", id->cont[BANNER].x, id->cont[BANNER].y,
								id->cont[BANNER].size, id->cont[BANNER].text);
	for (i = 0; i <= LABNAME; i++) {
		fprintf(fp, "%2d %3d %5d %4d %3d %s\n", id->cont[i].display, id->cont[i].len,
			id->cont[i].x, id->cont[i].y, id->cont[i].size, id->cont[i].text);
	}
	for (i = 0; i < MAXCLASS; i++) {
		cl = &scrf->classes[i];
		if (i == ABNORMAL)
			fprintf(fp, "-1 -1 0 0 %4d %4d %s\n", cl->b.width, cl->b.height, 
					cl->tag);
		else 
			fprintf(fp, "%5d %5d %2d %2d %4d %4d %s\n", (cl->b.posn.x + cl->b.width/2), 
			cl->b.posn.y, cl->defaultpos, cl->npos, cl->b.width, cl->b.height, 
			cl->tag);
	}
	fprintf(fp, "%4d\n", scrf->stk[0].posn.y);
	fprintf(fp, "%2d\n", scrf->viewed_stack);
	fprintf(fp, "-1 ");
	for (i = 0; i < scrf->viewed_stack; i++)
		fprintf(fp, "%5d ", scrf->stk[i].posn.x);
	fprintf(fp, "\n");
	fprintf(fp, "%5d %5d\n", scrf->chromclass.posn.x, scrf->chromclass.posn.y);
	if(scrf->linebuf.count) {
		fprintf(fp,"lines %d\n",scrf->linebuf.count);
		for(l=scrf->linebuf.lineint;l;l=l->next)
			fprintf(fp,"%d %d %d %d\n",l->x1,l->y1,l->x2,l->y2);
    }
}
/*
 * C O N V E R T convert a scrf struct back to a file image
 *
 */
static
convert(scrf, screenfmt)
struct scrform *scrf;
struct screenform *screenfmt;
{
	int i;
	struct lineint *d,*s;
	
	copy_box(&scrf->idbox.b, &screenfmt->idbox.position.x, &screenfmt->idbox.position.y, 
		&screenfmt->idbox.width, &screenfmt->idbox.height);
	copy_banner(&screenfmt->idbox.banner, &scrf->idbox.cont[BANNER]);
	init_id(&screenfmt->idbox.case, &scrf->idbox.cont[CASE]);
	init_id(&screenfmt->idbox.slide, &scrf->idbox.cont[SLIDE]);
	init_id(&screenfmt->idbox.cell, &scrf->idbox.cont[CELL]);
	init_id(&screenfmt->idbox.operator, &scrf->idbox.cont[OPERATOR]);
	init_id(&screenfmt->idbox.date, &scrf->idbox.cont[DATE]);
	init_id(&screenfmt->idbox.supervisor, &scrf->idbox.cont[SUPERVISOR]);
	init_id(&screenfmt->idbox.superdate, &scrf->idbox.cont[SUPERDATE]);
	init_id(&screenfmt->idbox.patname, &scrf->idbox.cont[PATNAME]);
	init_id(&screenfmt->idbox.comment1, &scrf->idbox.cont[COMMENT1]);
	init_id(&screenfmt->idbox.comment2, &scrf->idbox.cont[COMMENT2]);
	init_id(&screenfmt->idbox.karyotype, &scrf->idbox.cont[KARYOTYP]);
	init_id(&screenfmt->idbox.labname, &scrf->idbox.cont[LABNAME]);
	for (i = 0; i < MAXCLASS; i++) {
		copy_class(&screenfmt->disppos[i], &scrf->classes[i]);
		if (i == ABNORMAL) {
			screenfmt->disppos[i].col = -1;
			screenfmt->disppos[i].lin = -1;
			screenfmt->disppos[i].npos = screenfmt->disppos[i].defaultpos = 0;
		}
	}
	screenfmt->chromclass.x = scrf->chromclass.posn.x;
	screenfmt->chromclass.y = scrf->chromclass.posn.y;
	screenfmt->stacklin = scrf->stk[0].posn.y;
	screenfmt->stackcol[0] = -1;
	for (i = 0; i < NSTACKPOS + 1; i++)
		screenfmt->stackcol[i+1] = scrf->stk[i].posn.x;
	screenfmt->viewed_stack = scrf->viewed_stack;
	_strass(&screenfmt->linebuf,&scrf->linebuf,sizeof(struct linebuf));
	if(scrf->linebuf.count) {
		d=screenfmt->linebuf.lineint=(struct lineint *)Calloc(scrf->linebuf.count,sizeof(struct lineint));
		for(i=0,s=scrf->linebuf.lineint;i<scrf->linebuf.count;i++) {
			_strass(d,s,sizeof(struct lineint));
			d++;
			s=s->next;
		}
	}
	else {
		screenfmt->linebuf.count=0;
		screenfmt->linebuf.lineint=NULL;
	}
	screenfmt->version=VERSION;
	screenfmt->size=sizeof(struct screenform)+screenfmt->linebuf.count*sizeof(struct lineint);
}
/*
 * C O P Y _ B O X
 *
 */
copy_box(b, x, y, width, height)
struct box *b;
int *x, *y, *width, *height;
{
	*x = b->posn.x;
	*y = b->posn.y;
	*width = b->width;
	*height = b->height;
}
/*
 * C O P Y _ B A N N E R
 *
 */
copy_banner(new, old)
struct txt *new;
struct idtext *old;
{
	new->x = old->x;
	new->y = old->y;
	new->size = old->size;
	strcpy(new->text, old->text);
}
/*
 * C O P Y _ C L A S S
 *
 */
copy_class(kd, cl)
struct kdisppos *kd;
struct classpos *cl;
{
	kd->col = cl->b.posn.x + cl->b.width / 2;
	kd->lin = cl->b.posn.y;
	kd->space = cl->b.width;
	kd->height = cl->b.height;
	kd->defaultpos = cl->defaultpos;
	kd->npos = cl->npos;
	strcpy(kd->tag, cl->tag);
}
