/*
 *	T E L L H E A D E R . C
 *
 *
 *	Written: Graham John Page
 *			 Image Recognition Systems
 *			 720 Birchwood Boulevard
 *			 Birchwood Science Park
 *			 Warrington
 *			 Cheshire
 *
 *	Date:	 2 Feb 1987
 *
 * print file header contents.
 *
 * Modifications
 *
 *
 */
#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <fileheaders.h>

#define TRUE 0
#define FALSE -1


main(argc,argv)
char **argv;
int argc;
{
	char	*swptr;
	FILE	*fptr;

	argc--; argv++;
	while ((argc > 0) && (*argv[0] == '-')) {
		swptr = *argv++; argc--;
		while (*(++swptr)) {
			switch (*swptr) {
			  case '?':
			  	printf("Syntax:  tellheader [<options>] [<file list>]\n");
				printf("Function : Print out file header details for");
				printf(" Cytoscan format files\n");
				printf("Options :\n");
				printf("\t-v\tShow version number\n");
			  	exit(0);
			  case 'v':
			  	printf("Tellheader Version 0.2 (c) Image Recognition Systems\n");
			  	exit(0);
			  default:
			 	printf("Tellheader - unknown option\n");
			}
		}
	}
	if (argc <1) {
		tellheader(stdin);
		while(!feof(stdin)){
			getc(stdin);
		}
	} else {
		while (argc > 0) {
			printf("File : %s\n",*argv);
			if ((fptr = fopen(*argv++,"r")) != NULL) {
				 tellheader(fptr);
				 fclose(fptr);
			} else {
				printf("Open error\n");
			}
			printf("\n\n");
			argc--;
		}
	}
}

tellheader(fptr)
FILE *fptr;
{
	struct object *obj, *readobj();

	if ((obj = readobj(fptr)) != NULL) {
		if (obj->type == 110) {
			if (obj->plist != NULL) {
				displayheader(obj->plist);
			} else {
				printf("\nNO PROPERTY LIST\n\n");
			}
		} else {
			printf("First Object in file is not a valid file header\n");
			freeobj(obj);
		}
	} else {
		printf("NULL Object\n");
	}
}

/*
 *	D I S P L A Y H E A D E R  --  Work out what type of header and call 
 *									relevant routine
 *
 */
displayheader(h)
struct	gdat_header *h;		/* Any header type will do here */
{
	switch (h->file_type) {
		case CELLFILE:
			cell_props(h);
			break;
		case KARYFILE:
			kary_props(h);
			break;
		case METFILE:
			met_props(h);
			break;
		case GDATFILE:
			gdat_props(h);
			break;
		case FLEXFILE:
			flex_props(h);
			break;
		default:
			printf("Invalid file header type\n");
			
	}
}

char *
dig_string(index)
int index;
{
	static char type[50];

	switch (index&DIGMASK) {
		case DIGFIP    : sprintf (type,"DIGFIP");
				 break;
		case DIGTV     : sprintf (type,"DIGTV");
				 break;
		case DIGFIPEXP : sprintf (type,"DIGFIPEXP");
				 break;
		default:
			sprintf(type,"Invalid digitisation type %d",index);
			break;
	}
	if (index&DIGFLUOR) strcat(type," DIGFLUOR");
	return(type);
}

/*
 *	F L E X _ P R O P S 
 *
 */
flex_props(flex)
struct flex_header *flex;
{
	printf("\tFlexible screen header\n");
	switch (flex->version) {
		case 1:
			printf("Header version 1, size %d\n",flex->size);
			printf("Date              : %s\n",flex->date);
			printf("Lab name          : %s\n",flex->lab_name);
			printf("Opid              : %s\n",flex->opid);
			printf("Patient id        : %s\n",flex->patid);
			printf("Slide id          : %s\n",flex->slideid);
			break;
		default:
			printf("Header version %d, size %d -- UNDEFINED\n",flex->version, flex->size);
	}
}

/*
 *	G D A T _ P R O P S 
 *
 */
gdat_props(gdat)
struct gdat_header *gdat;
{
	printf("\tGeneral data header\n");
	switch (gdat->version) {
		case 1:
			printf("Header version 1, size %d\n",gdat->size);
			printf("Date              : %s\n",gdat->date);
			printf("Lab name          : %s\n",gdat->lab_name);
			printf("Opid              : %s\n",gdat->opid);
			printf("Patient id        : %s\n",gdat->patid);
			printf("Slide id          : %s\n",gdat->slideid);
			break;
		case 2:
			printf("Header version 2, size %d\n",gdat->size);
			printf("Date              : %s\n",gdat->date);
			printf("Lab name          : %s\n",gdat->lab_name);
			printf("Opid              : %s\n",gdat->opid);
			printf("Patient id        : %s\n",gdat->patid);
			printf("Slide id          : %s\n",gdat->slideid);
			break;
		default:
			printf("Header version %d, size %d -- UNDEFINED\n",gdat->version, gdat->size);
	}
}

/*
 *	M E T _  P R O P S  --  Met list header
 *
 */
met_props(met)
struct met_header *met;
{
	struct met_header_1 *metv1;
	struct met_header_2 *metv2;

	printf("Metaphase list header\n");
	switch (met->version) {
		case 1:
			metv1 = (struct met_header_1 *) met;
			printf("Header version 1, size %d\n",metv1->size);
			printf("Date              : %s\n",metv1->date);
			printf("Lab name          : %s\n",metv1->lab_name);
			printf("Opid              : %s\n",metv1->opid);
			printf("Patient id        : %s\n",metv1->patid);
			printf("Slide id          : %s\n",metv1->slideid);
			printf("\tMetaphase data\n");
			printf("Number of records : %d\n",metv1->nrecs);
			printf("Comment storage   : %d\n",metv1->comments);
			printf("Tissue type       : %d\n",metv1->tissue);
			printf("Stain             : %d\n",metv1->stain);
			break;
		case 2:
			metv2 = (struct met_header_2 *) met;
			printf("Header version 2, size %d\n",metv2->size);
			printf("Date              : %s\n",metv2->date);
			printf("Lab name          : %s\n",metv2->lab_name);
			printf("Opid              : %s\n",metv2->opid);
			printf("Patient id        : %s\n",metv2->patid);
			printf("Slide id          : %s\n",metv2->slideid);
			printf("\tMetaphase data\n");
			printf("Number of records : %d\n",metv2->nrecs);
			printf("Comment storage   : %d\n",metv2->comments);
			printf("Tissue type       : %d\n",metv2->tissue);
			printf("Stain             : %d\n",metv2->stain);
			break;
		case 3:
			printf("Header version 2, size %d\n",met->size);
			printf("Date              : %s\n",met->date);
			printf("Lab name          : %s\n",met->lab_name);
			printf("Opid              : %s\n",met->opid);
			printf("Patient id        : %s\n",met->patid);
			printf("Patient name      : %s\n",met->patnam);
			printf("Slide id          : %s\n",met->slideid);
			printf("\tMetaphase data\n");
			printf("Number of records : %d\n",met->nrecs);
			printf("Comment storage   : %d\n",met->comments);
			printf("Tissue type       : %d\n",met->tissue);
			printf("Stain             : %d\n",met->stain);
			printf("Slide width       : %d\n",met->slwidth);
			break;
		default:
			printf("Header version %d, size %d -- UNDEFINED\n",met->version, met->size);
	}
}

/*
 *	C E L L _ P R O P S  --
 *
 */
cell_props(cell)
struct cell_header *cell;
{
	printf("Cell file header\n");
	cellcommon(cell);
}

/*
 *	C E L L C O M M O N
 *
 */
cellcommon(cell)
struct cell_header *cell;
{
	struct cell_header_1 *cellv1;
	struct cell_header_2 *cellv2;

	switch (cell->version) {
		case 1:
			cellv1 = (struct cell_header_1 *) cell;
			printf("Header version 1, size %d\n",cellv1->size);
			printf("Date              : %s\n",cellv1->date);
			printf("Lab name          : %s\n",cellv1->lab_name);
			printf("Opid              : %s\n",cellv1->opid);
			printf("Patient id        : %s\n",cellv1->patid);
			printf("Slide id          : %s\n",cellv1->slideid);
			printf("Cell Number       : %d\n",cellv1->cellid);
			printf("\tCell data\n");
			printf("Digitisation type : %s\n",dig_string(cellv1->dig_type));
			printf("          X coord : %d\n",cellv1->x_coord);
			printf("          Y coord : %d\n",cellv1->y_coord);
			printf("          Z coord : %d\n",cellv1->z_coord);
			printf("          X size  : %d\n",cellv1->x_size);
			printf("          Y size  : %d\n",cellv1->y_size);
			break;
		case 2:
			cellv2 = (struct cell_header_2 *) cell;
			printf("Header version 2, size %d\n",cellv2->size);
			printf("Date              : %s\n",cellv2->date);
			printf("Lab name          : %s\n",cellv2->lab_name);
			printf("Opid              : %s\n",cellv2->opid);
			printf("Patient id        : %s\n",cellv2->patid);
			printf("Slide id          : %s\n",cellv2->slideid);
			printf("Cell Number       : %d\n",cellv2->cellid);
			printf("\tCell data\n");
			printf("Digitisation type : %s\n",dig_string(cellv2->dig_type));
			printf("          X coord : %d\n",cellv2->x_coord);
			printf("          Y coord : %d\n",cellv2->y_coord);
			printf("          Z coord : %d\n",cellv2->z_coord);
			printf("          X size  : %d\n",cellv2->x_size);
			printf("          Y size  : %d\n",cellv2->y_size);
			break;
		case 3:
			printf("Header version 2, size %d\n",cell->size);
			printf("Date              : %s\n",cell->date);
			printf("Lab name          : %s\n",cell->lab_name);
			printf("Opid              : %s\n",cell->opid);
			printf("Patient id        : %s\n",cell->patid);
			printf("Patient name      : %s\n",cell->patnam);
			printf("Slide id          : %s\n",cell->slideid);
			printf("Cell Number       : %d\n",cell->cellid);
			printf("\tCell data\n");
			printf("Digitisation type : %s\n",dig_string(cell->dig_type));
			printf("          X coord : %d\n",cell->x_coord);
			printf("          Y coord : %d\n",cell->y_coord);
			printf("          Z coord : %d\n",cell->z_coord);
			printf("          X size  : %d\n",cell->x_size);
			printf("          Y size  : %d\n",cell->y_size);
			break;
		default:
			printf("Header version %d, size %d -- UNDEFINED\n",cell->version, cell->size);
	}
}

kary_props(kary)
struct cell_header *kary;
{
	struct cell_header_1 *karyv1;
	struct cell_header_2 *karyv2;

	printf("Karyotype file header\n");
	cellcommon(kary);
	printf("\tKaryotype data\n");
	switch(kary->version) {
		case 1:
			karyv1 = (struct cell_header_1 *) kary;
			printf("Comments 1        : %s\n",karyv1->comments[1]);
			printf("Comments 2        : %s\n",karyv1->comments[2]);
			printf("karyotype         : %s\n",karyv1->krtype);
			printf("Supervisior       : %s\n",karyv1->supid);
			printf("Checked date      : %s\n",karyv1->supdate);
			break;
		case 2:
			karyv2 = (struct cell_header_2 *) kary;
			printf("Comments 1        : %s\n",karyv2->comments[1]);
			printf("Comments 2        : %s\n",karyv2->comments[2]);
			printf("karyotype         : %s\n",karyv2->krtype);
			printf("Supervisior       : %s\n",karyv2->supid);
			printf("Checked date      : %s\n",karyv2->supdate);
			break;
		case 3:
			printf("Comments 1        : %s\n",kary->comments[1]);
			printf("Comments 2        : %s\n",kary->comments[2]);
			printf("Karyotype         : %s\n",kary->krtype);
			printf("Supervisior       : %s\n",kary->supid);
			printf("Checked date      : %s\n",kary->supdate);
			break;
		default:
			break;
	}
}
		
