/*
 *      W A S H L I N E .  C -- Composite karyotype program. 
 *
 *
 *  Written: Jim Piper @ MRC Edinburgh.
 *  Rewritten by Sarah E. A. Jones.
 *				
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    23 April 1984. (Rewritten 17th May 1988).
 *
 *  Modifications
 *
 *	28 Mar 1990	dcb		Change STRING to header = "Cytoscan Composite Karyotype"
 *   3 Jul 1989	SEAJ	Temporary fix for Cpgroup problem & ideogram bug;vers 1.1
 *  23 May 1989	SEAJ	Fixed disppos ordering bug.
 *   3 Mar 1989	SEAJ	Made storeobj order classes in disppos order.
 *   1 Feb 1989	SEAJ	Made all lines double thickness to stop flicker.
 *	19 Jan 1989	CAS		V1.0 In slave mode - get ddgs ptr as param, then ddgsdup
 *   9 Jan 1989 SEAJ	Started to add in ideogram stuff.
 *  12 Oct 1988	SEAJ	Added convert variable to strint to tell whether to 
 *						convert or not.
 *	26 Sep 1988	SEAJ	Changed case name slide name sequence.
 *  28 Jul 1988 SEAJ	Added subdirectory stuff in change_dir.
 *  27 Jul 1988 SEAJ	Added in change directory.  
 *  26 Jul 1988	SEAJ	Moved call to alldone to the end of startup(), so that 
 *						process dies when called from nowfork().
 */

#define INIFILE        /* define variables in this link */
#include <cmdsys.h>
#include <wstruct.h>
#include <interact.h>
#include <ddgs.h>
#include <chromanal.h>
#include <fileheaders.h>
#include <ideogram.h>

#define CELLS 10
#define CLASSES 10
#define MAXFILNAM 180
#define MAXDIRLEN 100
#define BUFSIZE 10240
#define MAXSTRING 30

#define BORDER 32
#define XGAP 6
#define YGAP 18
#define XSTART 40
#define YSTART 24
#define XFINISH 16
#define YFINISH 32
#define SCALE 4

extern int getwd();

int new_start(), add_delete(), move_grid(), getcases(), getcell();
int add_cell(), delete_cell(), add_class(), delete_class();
int next_cell(), prev_cell(), next_class(), prev_class(), restore();
int strint(), comparison(), calc_width(), calc_length();
int kickslave(), startup();
int change_dir(), checkres();

struct homolog {
	int class;
	int width;
	int length;
	struct chromosome *chrom;
	struct homolog *next;
};

struct cellheader {
	struct cyto_file cfile;
	char direct[MAXDIRLEN];
	struct cellheader *nxtcell;
	struct homolog *hom, *top;
	int col_width;
};
struct cellheader *shptr, *startptr, *storeheader();
struct ideo_list {
	int class;
	int length;
	int width;
	struct chrom_data *chptr;
	struct ideo_list *next;
};
/*
 *  Local variable defines
 */
struct ideo_list *start_ideo, *top_ideo;
int class[CLASSES], cellno[CELLS], maxlen[CLASSES];
char label[CELLS][MAXFILNAM];
char directory[MAXDIRLEN];
struct cyto_file cytfil, *cellfile;
DDGS *dg;
char header[] = "Cytoscan Composite Karyotype";

static struct pframe f;
static int ideo_on = 1, res = P300;

FILE *in, *fbopen();

/*
 *	I N T E R R U P T --  Signal catcher
 *
 */
interrupt(sig)
int		sig;
{
}
/*
 * M A I N -- 
 *
 */
main(argc,argv)
int		argc;
char	**argv;
{
	char	*swptr, **targv;
	int		targc, is_slave;
	int		store = FS_VME512, devno = 0;
	DDGS	*ddgsptr, *ddgsopen(), *ddgsdup();

#ifdef INTERRUPT
	intercept(interrupt);
#endif INTERRUPT
	targc = argc; targv = argv;

	is_slave = 0;
	argc--; argv++;			/* dump process name */
	while (argc > 0) {
		if (*argv[0] == '-') {
			swptr = *argv;
			*argv = NULL;
			while (*(++swptr)) {
				switch (*swptr) {
				  case '?':
				  	printf("Syntax:   washline <options> \n");
				  	printf("Function: \n");
				  	printf("Options:  v - Show version number\n");
				  	printf("		  s - Slave program\n");
					printf("          1 - Use seat 1 framestore\n");
					printf("          2 - Use seat 2 framestore\n");
					printf("                  also 3 and 4\n");
					printf("          I - Virtuoso framestore (Default CRS 512)\n");
					printf("		  V - Virtuoso invert framestore(Default CRS 512)\n");
				  	printf("          ? - Help\n");
				  	exit(0);
				  case 'v':
						printf("Washline - Version 1.1\n");
						exit(0);
				   case 's':
				   		is_slave = 1;
				   		ddgsptr = (DDGS *) atoi(*++argv);
				   		*argv = NULL; argc--;
				   		break;
				   case '1':
						devno = 0;
						break;
					case '2':
						devno = 1;
						break;
					case '3':
						devno = 2;
						break;
					case '4':
						devno = 3;
						break;
					case 'I':
						store = FS_VIRTUOSO;
						break;
					case 'V':
						store = FS_VIRTUOSO_INVERT;
						break;
				  default:
				 	printf("washline - unknown option\n");
				}
			}
		}
		argv++; argc--;
	}

	argc = targc; argv = targv;
	argc--; argv++;			/* dump process name */
	/*
	 * This is the start of the M A I N program
	 *
	 */
	if (is_slave == 1) {
		dg = ddgsdup(ddgsptr);
		slavemain(targv[1], startup, kickslave);
	} else {
		dg = ddgsopen(store, devno);
		muini(stdout);
		startup();
	}
}
/*
 *  K I C K S L A V E -- dummy routine
 *
 */
kickslave() {}

/*
 *  S T A R T U P -- initial setup routines
 *
 */
startup()
{
	setmemalloc(65536);
	getwd(directory, MAXDIRLEN);
	initialise();	
	doboxes(header);	
	topmenu();
	alldone(0);
}

/*
 * I N I T I A L I S E -- Initialise all variables before beginning
 *
 */
initialise()
{
	int i;

	dinit();
	tsize(80, 80);
	cellfile = &cytfil;	
	for (i = 0; i < CLASSES; i++)
		class[i] = -1;
	for (i = 0; i < CELLS; i++)
		cellno[i] = -1;
	for (i = 0; i < CELLS; i++)
		*label[i] = '\0';
	
	strcpy(cellfile->case, "UNSET");
	strcpy(cellfile->slide, "UNSET");
	shptr = NULL;
	startptr = NULL;
	start_ideo = NULL;
	top_ideo = NULL;
}
/*
 * D O B O X E S  -- Draw boxes onto screen.
 *
 */
doboxes(str)
char *str;
{
	int length;
	
	f.type = 60;
	f.scale = 8;
	f.ix = 1;
	f.iy = 1;
	clear();
	llut();
	/* boxes etc */
	lwidth(2);
	moveto((dg->minx + BORDER/4) << dg->xshift, (dg->miny + BORDER/2) <<dg->yshift);
	text("cell");
	moveto(dg->maxx << dg->xshift, (dg->miny + BORDER) << dg->yshift);
	lineto((dg->minx + BORDER) << dg->xshift, (dg->miny + BORDER) << dg->yshift);
	lineto((dg->minx + BORDER)<< dg->xshift, (dg->maxy - BORDER) << dg->yshift);
	length = strlen(str);
	length = length * 80;
	if (length > (dg->lineln << dg->xshift))
		fprintf(stderr, "The set string is too long");
	else {
		length = (dg->lineln << dg->xshift) - length;
		length /= 2;
		moveto((dg->minx << dg->xshift) + length, (dg->maxy - 7*BORDER/8) << dg->yshift);
		text(str);
	}
}

/*
 * T O P M E N U --
 *
 */
topmenu()
{
	int cmd, Excmd, Chdircmd, Ideocmd, size = 300;
	char temp[MAXDIRLEN];

	strcpy(temp, directory);
	while (1) {
		downmenu();
		newmenu("Composite Karyotype");
		cmmd("new start;%f", new_start);
		cmmd("ideograms displayed;:k40%T%-+d%f", &ideo_on, &res, checkres);Ideocmd = mmcmd;
		if (ideo_on)
			cmmd("band resolution;:k40%-+d%d%f", &ideo_on, &size, checkres);
		cmmd("add/delete class/cell;%f", add_delete);
		cmmd("move within chromosome grid;%f", move_grid);
		cmmd("change current directory;:k40%s%f", temp, change_dir); Chdircmd = mmcmd;
		cmmd("exit composite karyotype"); Excmd = mmcmd;
	
		while ((cmd = thismu()) != Excmd) {
			if (cmd == Chdircmd) {
				displin(0, Chdircmd);
				putprompt();
			}
			if (cmd == Ideocmd) {
				upmenu();
				break;
			}
		}
		if (cmd == Excmd)
			break;
	}
	upmenu();
}
/*
 * C H E C K R E S -- make resolution number correct
 *
 */
checkres(on, size)
int *on, *size;
{
	if (*size > 750) {
		*size = 900;
		res = P900;
	} else if (*size < 450) {
		*size = 300;
		res = P300;
	} else {
		*size = 600;
		res = P600;
	}
	if (start_ideo != NULL)
		delete_ideo(class);
	if (ideo_on && (startptr != NULL)) {
   		screengo(25, 20);
		fprintf(stderr, "Reading new ideograms -- Please wait.\n");
		read_ideolist(class);
		redisplay();
	}
	if (!ideo_on)
		redisplay();
}
/*
 * N E W _ S T A R T -- New information input
 *
 */
new_start()
{
	struct cellheader *ptr;
	struct ideo_list *iptr, *prev;
	int i, j, cmd;
	
	/* delete old lists before starting again */
	ptr = shptr;
	while(ptr != NULL) {
		strcpy(cellfile->case, ptr->cfile.case);
		strcpy(cellfile->slide, ptr->cfile.slide);
		cellfile->cellno = ptr->cfile.cellno;
		delete_cell(1);
		ptr = ptr->nxtcell;
	}
	if (ideo_on)
		delete_ideo(class);
	doboxes(header);
	initialise();
	cmd = getcases();
	if (cmd == 0) {
		i = 0;
		while (cellno[i] != -1 && i < CELLS) {
			buildfile(label[i], directory, cellfile->case, cellfile->slide, cellno[i]);
			i++;
		}
		screengo(25, 20);
		fprintf(stderr, "Reading file(s) -- Please wait.\n");
		for (j = 0; j < i; j++) 
			read_chrom(label[j], cellno[j], class, 1, NULL);
		if (ideo_on)
			read_ideolist(class);
		redisplay();
	}
}

/*
 * C H A N G E _ D I R -- Change the current working directory
 *
 */
change_dir(dir)
char *dir;
{
	int		newdirlen;
	char	newname[MAXDIRLEN];

	if (!dir || (*dir == '\0')) {
		strcpy(dir, directory);
		return(0);
	}
	switch (*dir) {
	case '.':
		putprompt();
		fprintf(stderr,"Illegal directory name %s\n",dir);
		return(0);
	case '/':
		if ((newdirlen = strlen(dir)) >= MAXDIRLEN) {
			putprompt();
			fprintf(stderr,"Directory name is too long\n");
			return(0);
		} else {
			strcpy(newname,dir);
		}
		break;
	default:
		if ((newdirlen = (strlen(directory) + strlen(dir))) >= MAXDIRLEN) {
			fprintf(stderr,"Directory name is too long\n");
			return(0);
		} else {
			sprintf(newname,"%s/%s",directory,dir);
		}
		break;
	}
	if (chdir(newname) == 0) {
		strcpy(directory, newname);
		strcpy(dir, newname);
	} else {
		fprintf(stderr,"Can't access directory %s\n",dir);
		strcpy(dir, directory);
		return(0);
	}
	return(1);
}
/*
 * G E T C A S E S -- get the case, slides and cell number
 *
 */
int getcases()
{
	char cellstr[MAXSTRING], classtr[MAXSTRING];
	int cmd, Quitcmd;

	cellstr[0] = '\0';
	classtr[0] = '\0';	
	
	
	newmenu("Cell Information");
	cmmd("case ID;:k40%s;t2", cellfile->case);
	cmmd("slide ID;:k40%s", cellfile->slide);
	cmmd("cells numbers;:k40%30S%-+d%-+i%-+i%f",cellstr, cellno, CELLS, 0, strint);
	cmmd("classes;:k40%30S%-+d%-+i%-+i%f", classtr, class, CLASSES, 1, strint);
	cmmd("quit"); Quitcmd = mmcmd;
	
	while ((cmd = thismu()) != Quitcmd) {
		if (cmd == 0) {
			if (strcmp(cellfile->case, "UNSET")  == 0 || *cellfile->case == '\0')
				fprintf(stderr, "Case name is incomplete\n");
			else if (strcmp(cellfile->slide, "UNSET") == 0) 
				fprintf(stderr, "Slide name is incomplete\n");
			else if (cellno[0] < 0)
				fprintf(stderr, "No cell numbers have been entered\n");
			else if (class[0] < 0)
				fprintf(stderr, "No class numbers have been entered\n");
			else
				return(0);
		}
		if (cmd == 2) {
			displin(0, 1);
			displin(0, 2);
			putprompt();
		}
	}
	return(cmd);
}

/*
 * G E T C E L L -- Get information about cell to be added or deleted.
 *
 */
int getcell()
{
	int Quitcmd, cmd;
		
	newmenu("Add/Delete Cell");
	cmmd("case ID;:k40%s;t2", cellfile->case);
	cmmd("slide ID;:k40%s;t3", cellfile->slide);
	cmmd("cell number;:k40%d",&cellfile->cellno);
	cmmd("quit"); Quitcmd = mmcmd;
	
	while ((cmd = thismu()) != Quitcmd) {
		if (cmd == 0) {
			if (strcmp(cellfile->case, "UNSET")  == 0 || *cellfile->case == '\0')
				fprintf(stderr, "Case name is incomplete\n");
			else if (strcmp(cellfile->slide, "UNSET") == 0) 
				fprintf(stderr, "Slide name is incomplete\n");
			else if (cellfile->cellno < 0)
				fprintf(stderr, "Cell number is incorrect\n");
			else
				return(0);
		}
		if (cmd == 3) {
			displin(0,1);
			displin(0,2);
			displin(0,3);
			putprompt();
		}
	}
	return(cmd);
}
	
/*
 * B U I L D F I L E -- Builds the filename from all of it's components
 *
 */
buildfile(filename, dir, casename, slide, number)
char *dir, *filename,*casename,*slide;
int number;
{
	sprintf(filename, "%s%c%s$%s$%dK", dir, '/', casename, slide, number);
}

/*
 * R E A D _ C H R O M -- Read chromosomes for file
 *
 */
int
read_chrom(name, cellno, classnos, newstart, ptr)
char *name;
int cellno, newstart, classnos[];
struct cellheader *ptr;
{
	struct chromosome *obj, *readobj();
	int j, class, flag;
	
	in = fbopen(name, "r", BUFSIZE);
	if (in == NULL) {
		fprintf(stderr, "Cannot open file %s\n", name);
		return(-1);
	} else {
		if (newstart) {
			cellfile->cellno = cellno;
			ptr = storeheader();		/* new cell */
		}
		/* read objects */
		while((obj = readobj(in)) != NULL) {
			flag = 0;
			if (obj->type == 1 && obj->plist != NULL) {
				if (obj->plist->Cotype == 1 || obj->plist->otype == 1) {
					if (obj->plist->Cpgroup)
						class = obj->plist->Cpgroup;
					else
						class = obj->plist->pgroup;
					j = 0;
					while (classnos[j] != -1 && j < CLASSES) {
						if (class == classnos[j]) {
							storeobj(obj, classnos[j], ptr);
							flag = 1;
							break;
						}
						j++;
					}
				}
			}
			if (!flag)
				freeobj(obj);
		}
	}
	fbclose(in);
	return(1);
}
/*
 * R E A D _ I D E O L I S T -- Reads all classes ideograms into list
 *
 */
read_ideolist(classno)
int classno[];
{
	int j;
	FILE *fp;
 	char tbuf[3];
	struct ideo_list *ptr, *tptr, *prev;
	struct chrom_data *chrom;
	
	fp = fopen("/dd/params/human.col", "r");
	if (fp == NULL) {
		fprintf(stderr, "Unable to open ideogram file.\n");
		return;
	}
	ptr = start_ideo;
	prev = ptr;
	j = 0;
	while (classno[j] != -1 && j < CLASSES) {
		tptr = (struct ideo_list *) Calloc(1, sizeof(struct ideo_list));
		switch(classno[j]) {
			case 23 :
				sprintf(tbuf, "X");
				break;
			case 24 :
				sprintf(tbuf, "Y");
				break;
			default :
				sprintf(tbuf, "%d", classno[j]);
				break;
		}
		if (classno[j] != 25) {
			tptr->chptr = get_ideogram(fp, tbuf, res);
			tptr->class = classno[j];
			chrom = tptr->chptr;
			tptr->length = chrom->p_length + chrom->q_length + chrom->sat_length;
			tptr->width = chrom->width;
			/* find position in list for new ideogram */
			while (ptr != NULL && ptr->class < classno[j]) {
				prev = ptr;
					ptr = ptr->next;
			}
			if (ptr == NULL) {
				if (start_ideo == NULL) { /* start of list */
					if (start_ideo == top_ideo)
						top_ideo = tptr;
					start_ideo = tptr;
					tptr->next = NULL;
					ptr = tptr;
				} else { /* end of list */
					prev->next = tptr;
					tptr->next = NULL;
					ptr = tptr;
				}
			} else {
				if (prev == ptr) {
					if (start_ideo == top_ideo)
						top_ideo = tptr;
					start_ideo = tptr;
					tptr->next = ptr;
					ptr = tptr;
				} else {
					prev->next = tptr;
					tptr->next = ptr;
				}
			}
			j++;
			prev = tptr;
		} else 
			j++;
	}  /* end of while */
	fclose(fp);
}
/* 
 * C H R O M _ S I Z E -- Calclate width and length of chromosome
 *
 */
chrom_size(ptr, object)
struct homolog *ptr;
struct chromosome *object;
{
	ptr->length = object->idom->lastln - object->idom->line1;
	if (ptr->length < 0)
		ptr->length *= -1;
	ptr->width = object->idom->lastkl - object->idom->kol1;
	if (ptr->width < 0)
		ptr->width *= -1;
}
/*
 * S T O R E H E A D E R -- Sets up header to homolog list
 *
 */
struct cellheader *storeheader()
{
	struct cellheader *ptr;
	
	ptr = shptr;
	if (ptr == NULL) {		/* new list */
		ptr = (struct cellheader *) Calloc(1, sizeof(struct cellheader));
		shptr = ptr;
		startptr = shptr;
	} else {
		while (ptr->nxtcell != NULL)
			ptr = ptr->nxtcell;
		ptr->nxtcell = (struct cellheader *) Calloc(1, sizeof(struct cellheader));
		ptr = ptr->nxtcell;
	}
	ptr->nxtcell = NULL;
	ptr->hom = NULL;
	ptr->top = NULL;
	strcpy(ptr->direct, directory);
	strcpy(ptr->cfile.case, cellfile->case);
	strcpy(ptr->cfile.slide, cellfile->slide);
	ptr->cfile.cellno = cellfile->cellno;
	return(ptr);
}	
/*
 * S T O R E O B J -- Store object information in struct homolog
 *
 */
storeobj(obj, class, ptr)
struct chromosome *obj;
int class;
struct cellheader *ptr;
{
	struct homolog *p, *prev;
	int flag;

	if (ptr->hom == NULL) {				/* empty list */
		ptr->hom = (struct homolog *) Calloc(1, sizeof(struct homolog));
		ptr->hom->next = NULL;
		ptr->hom->chrom = obj;
		ptr->hom->class = class;
		chrom_size(ptr->hom, obj);
		ptr->top = ptr->hom;
	} else {
		flag = 0;
		p = ptr->hom;
		prev = ptr->hom;
		while (p != NULL) {
			if (class <= p->class) {
				if (class == p->class) {
					while (p != NULL) {
						if ((obj->plist->disppos < p->chrom->plist->disppos) || 
							(class != p->class))
							break;
						prev = p;
						p = p->next;
					}
				}
				if (p == NULL) {
					flag = 0;
					break;
				} else if (prev == p) {			/* at top of the list */
					ptr->hom = (struct homolog *) Calloc(1, sizeof(struct homolog));
					prev = ptr->hom;
					ptr->top = ptr->hom;
				} else {
					prev->next = (struct homolog *) Calloc(1, sizeof(struct homolog));
					prev = prev->next;
				}
				prev->next = p;
				prev->chrom = obj;
				prev->class = class;
				chrom_size(prev, obj);
				flag = 1;
				break;
			}
			prev = p;
 			p = p->next;
		}
		if (!flag) {
			p = (struct homolog *) Calloc(1, sizeof(struct homolog));
			p->next = NULL;
			p->chrom = obj;
			p->class = class;
			chrom_size(p, obj);
			prev->next = p;
		}
	}
}
/*
 * A D D _ D E L E T E -- menu for adding or deleting a cell or class
 *
 */
add_delete()
{
	char class[MAXSTRING];
	
	class[0] = '\0';
	newmenu("Add/Delete Class/Cell");
	cmmd("add new cell;%f",add_cell);
	cmmd("add new class(es);:k40%30S%f", class, add_class);
	cmmd("delete cell;%-+d%f", 0, delete_cell);
	cmmd("delete class(es);:k40%30S%f", class, delete_class);
	while (thismu())
		;
}
/*
 * A D D _ C E L L -- Add a new cell to the list
 *
 */
 add_cell()
 {
 	char filename[MAXFILNAM];
 	int i, cmd;

	restore(0); 	
 	i = 0;
 	cmd = getcell();
 	if (cmd == 0) {
	 	while (cellno[i] != -1 && i < CELLS)
 			i++;
	 	if (i >= CELLS) {
 			fprintf(stderr, "%d is the maximum number of cells which are allowed.\n", CELLS);
 			fprintf(stderr, "Please delete one of the existing cells and add this one again.\n");
	 	} else {
 			buildfile(filename, directory, cellfile->case, cellfile->slide, cellfile->cellno);
 			strcpy(label[i], filename);
 			screengo(25,20);
	 		fprintf(stderr, "Reading file -- please wait\n");
			if (read_chrom(filename, cellfile->cellno, class, 1, NULL))
				cellno[i] = cellfile->cellno;
	
		}
		redisplay();
	}
 }

/*
 * A D D _ C L A S S -- Add a new clas to the list
 *
 */
add_class(string)
char *string;
{
	int i, j, k, classnos[CLASSES];
	char filename[MAXFILNAM];
	struct cellheader *ptr;

	restore(0);	
	strint(string, classnos, CLASSES, 1);
	/* check if the new classes have already been read in */
	j = 0;
	while (classnos[j] != -1 && j < CLASSES) {
		i = 0;
		while (class[i] != -1 && i < CLASSES) {
			if (classnos[j] == class[i]) {
				fprintf(stderr, "%d is already being displayed\n", class[i]);
				k = j;
				while (classnos[k] != -1 && k < CLASSES) {
					classnos[k] = classnos[k+1];
					k++;
				}
				classnos[k] = -1;
				j--;
			} else
				i++;
		}
		j++;
	}
	i = 0;
	while (class[i] != -1 && i < CLASSES)
		i++;
	j = 0;
	while (classnos[j] != -1) {
		if (i >= CLASSES) {
			screengo(25, 1);
			fprintf(stderr, "The maximum number of classes is %d.\n", CLASSES);
			while (j < CLASSES) {
				classnos[j] = -1;		/* fill rest of array with -1 */
				j++;
			}
			break;
		} else {
			class[i] = classnos[j];
			i++;
		}
		j++;
	}
	qsort(class, i, sizeof(int), comparison);
	checkclass(class);
	/* if there is anything to read */
	if (classnos[0] != -1) {
		ptr = shptr;
		screengo(25, 20);
		fprintf(stderr, "Reading file(s) -- Please wait\n");
		while( ptr != NULL) {
			buildfile(filename, ptr->direct, ptr->cfile.case, ptr->cfile.slide, ptr->cfile.cellno);
			read_chrom(filename, 0, classnos, 0, ptr);
			ptr = ptr->nxtcell;
		}
		if (ideo_on)
			read_ideolist(classnos);
		redisplay();
	}
}
/*
 * D E L E T E _ C E L L -- Remove complete cell from list, cell number from 
 *							cellno[] and filename from label[].
 */
delete_cell(setup)
int setup;
{
	struct cellheader *ptr, *prev;
	struct homolog *p, *next;
	int i, j, flag, cmd;

	restore(0);	
	i = 0;
	if (!setup)
		cmd = getcell();
	else 
		cmd = 0;
	if (cmd == 0) {
		flag = 0;
		ptr = shptr;
		prev = shptr;
		while (ptr != NULL) {
			if ((!strcmp(ptr->cfile.case, cellfile->case)) && (!strcmp(ptr->cfile.slide, cellfile->slide))
							&& (ptr->cfile.cellno == cellfile->cellno)) {
				if (ptr == shptr) {			/* at top of list */
					if (ptr->nxtcell == NULL) {
						shptr = NULL;
						startptr = NULL;
					} else {
						shptr = ptr->nxtcell;
						startptr = shptr;
					}
				}
				if (ptr->nxtcell == NULL && ptr != shptr)	/* at end of list */
					prev->nxtcell = NULL;
				else
					prev->nxtcell = ptr->nxtcell;
			
				p = ptr->top;
				if ( p != NULL ) {
					while((next = p->next) != NULL) {
						Free(p);
						p = next;
					}
					Free(p);
				}
				Free(ptr);
				for(j = i; j < CELLS; j++) {
					strcpy(label[j], label[j+1]);	/* delete filename from label */
					cellno[j] = cellno[j+1];		/* delete cell no from cellno */
				}
				cellno[CELLS - 1] = -1;
				flag = 1;
				break;
			} else {
				prev = ptr;
				ptr = ptr->nxtcell;
				i++;
			}
		}
		if (!flag)
			fprintf(stderr, "Cannot delete file %s %s %d\n", cellfile->case, cellfile->slide, cellfile->cellno);
		else if (!setup)
			redisplay();
	}
}
			
/*
 * D E L E T E _ C L A S S -- Delete class from list and classno
 *
 */
delete_class(string)
char *string;
{
	int i, j, classno[CLASSES], flag;
	struct cellheader *ptr;
	struct homolog *p, *prev;

	restore(0);
	flag = 0;
	strint(string, classno, CLASSES, 1);
	/* check all entries before deleting any classes */
	i = 0;
	while(classno[i] != -1 && i < CLASSES) {
		j = 0;
		while(class[j] != classno[i] && j < CLASSES)
			j++;
		if (j >= CLASSES) {
			fprintf(stderr, "Class %d is not in the list!!\n", classno[i]);
			flag = 1;
			*string = '\0';
			break;
		}
		i++;
	}
	if (!flag) {
		j = 0;
		while (classno[j] != -1) {
			i = 0;
			while(class[i] != classno[j] && i < CLASSES)
				i++;
			while(class[i] != -1 && i < CLASSES-1) {
				class[i] = class[i+1];
				i++;
			}
			class[i] = -1;
		
			ptr = shptr;
			while(ptr != NULL) {
				prev = ptr->hom;
				p = ptr->hom;
				while(p != NULL) {
					if (p->class == classno[j]) {
						if (p == prev) {			    /* first in list */
							ptr->hom = p->next;
							ptr->top = ptr->hom;	
							Free(p);
							prev = ptr->hom;
							p = ptr->hom;
						} else if (p->next == NULL) {	/* last in list */
							prev->next = NULL;
							Free(p);
							p = NULL;
						} else {
							prev->next = p->next;
							Free(p);
							p = prev->next;
						}
					} else {
						prev = p;
						p = p->next;
					}
				}
				ptr = ptr->nxtcell;
			}
			j++;
		}
		if (ideo_on)
			delete_ideo(classno);
		redisplay();
	}
}
/*
 * D E L E T E _ I D E O -- delete list of ideograms from ideo_list
 *
 */
delete_ideo(classno)
int classno[];
{
	struct ideo_list *iptr, *prev;
	int j;
	
	j = 0;
	iptr = start_ideo;
	prev = iptr;
	while (classno[j] != -1 && j < CLASSES) {
		while (iptr != NULL && iptr->class != classno[j]) {
			prev = iptr;
			iptr = iptr->next;
		}
		if (iptr != NULL) {
			if (iptr == start_ideo) {	/* start of list */
				iptr = iptr->next;
				free_ideogram(prev->chptr);
				Free(prev);
				prev = iptr;
				if (start_ideo == top_ideo)
					top_ideo = iptr;
				start_ideo = iptr;
			} else if (iptr->next == NULL) {	/* end of list */
				prev->next = NULL;
				free_ideogram(iptr->chptr);
				Free(iptr);
				iptr = NULL;
			} else {
				prev->next = iptr->next;
				free_ideogram(iptr->chptr);
				Free(iptr);
				iptr = prev->next;
			}
		}
		j++;
	}
}
/*
 * M O V E _ G R I D -- menu for moving within the grid
 *
 */
move_grid()
{
	newmenu("move within cell grid");
	cmmd("next cell;%f", next_cell);
	cmmd("previous cell;%f", prev_cell);
	cmmd("next class;%f", next_class);
	cmmd("previous class;%f", prev_class);
	cmmd("restore original grid;%-+d%f", 1, restore);
	
	while (thismu())
		;
}
/*
 * N E X T _ C E L L -- move to next cell on screen
 *
 */
next_cell()
{
	struct cellheader *ptr, *p;
	
	if (shptr == NULL)
		fprintf(stderr, "There is no list in!!\007\n");
	else {
		ptr = shptr;
		if (ptr->nxtcell != NULL) {
			shptr = ptr->nxtcell;
			p = shptr;
			while (p->nxtcell != NULL)
				p = p->nxtcell;
			p->nxtcell = ptr;
			ptr->nxtcell = NULL;
			redisplay();
		} else
			fprintf(stderr, "There is no next cell.\n");
	}
}
/*
 * P R E V _ C E L L -- go back to previous cell on screen
 *
 */
 prev_cell()
 {
 	struct cellheader *prev, *ptr;
 	
	if (shptr == NULL)
		fprintf(stderr, "There is no list in!!\007\n");
	else {
	 	ptr = shptr;
 		prev = ptr;
	 	while(ptr->nxtcell != NULL) {
 			prev = ptr;
 			ptr = ptr->nxtcell;
	 	}
 		if (prev != ptr) {
	 		prev->nxtcell = NULL;
	 		ptr->nxtcell = shptr;
		 	shptr = ptr;
		}
		redisplay();
	}
 }
/*
 * N E X T _ C L A S S -- show next class on screen
 *
 */
next_class()
{
	struct cellheader *ptr;
	struct homolog *p;
	int oldclass, i, cls;
	
	if (shptr == NULL)
		fprintf(stderr, "There is no list in!!\007\n");
	else {
		if (top_ideo->next != NULL)
			top_ideo = top_ideo->next;
		else
			top_ideo = start_ideo;
		ptr = shptr;
		while (ptr != NULL) {
			p = ptr->hom;
			if (p->class == class[0]) {
				oldclass = p->class;
				while (p->class == oldclass) {
					oldclass = p->class;
					p = p->next;
					if (p == NULL)
						p = ptr->top;
				}
				ptr->hom = p;
			}
			ptr = ptr->nxtcell;
		}
		i = 0;
		cls = class[0];
		while (i < CLASSES-1) {
			class[i] = class[i+1];
			i++;
		}
		class[i] = -1;
		i = 0;
		while (class[i] != -1 && i < CLASSES-1)
			i++;
		class[i] = cls;
		redisplay();
	}
}
/*
 * P R E V _ C L A S S -- show previous class on screen
 *
 */
prev_class()
{
	struct cellheader *ptr;
	struct homolog *p, *start;
	struct ideo_list *iptr;
	int i, oldclass, cls;

	if (shptr == NULL)
		fprintf(stderr, "There is no list in!!\007\n");
	else {
		iptr = start_ideo;
		if (start_ideo == top_ideo) {
			while (iptr->next != NULL)
				iptr = iptr->next;
		} else {
			while (iptr->next != top_ideo)
				iptr = iptr->next;
		}
		top_ideo = iptr;
		ptr = shptr;
		while (ptr != NULL) {
			p = ptr->top;
			start = p;
			oldclass = p->class;
			if (ptr->top == ptr->hom) {
				while (p != NULL) {
					if (p->class != oldclass)
						start = p;
					oldclass = p->class;
					p = p->next;
				}
			} else {
				while (p != ptr->hom) {
					if (p->class != oldclass)
						start = p;
					oldclass = p->class;
					p = p->next;
				}
			}
			ptr->hom = start;
			ptr = ptr->nxtcell;
		}
		i = 0;
		while (class[i] != -1 && i < CLASSES-1)
			i++;
		i--;
		cls = class[i];
		while (i > 0) {
			class[i] = class[i-1];
			i--;
		}
		class[0] = cls;
		redisplay();
	}
}
/*
 * R E S T O R E -- restore the original grid
 *
 */
restore(red)
int red;
{
	struct cellheader *ptr;
	int i;
	
	if (shptr == NULL)
		fprintf(stderr, "There is no list in!!\007\n");
	else {
		top_ideo = start_ideo;
		ptr = shptr;
		if (ptr != startptr) {
			while (ptr->nxtcell != startptr)
				ptr = ptr->nxtcell;
			ptr->nxtcell = NULL;
			ptr = startptr;
			while (ptr->nxtcell != NULL)
				ptr = ptr->nxtcell;
			ptr->nxtcell = shptr;
			shptr = startptr;
		}
		ptr = shptr;
		while (ptr != NULL) {
			ptr->hom = ptr->top;
			ptr = ptr->nxtcell;
		}
		i = 0;
		while (class[i] != -1 && i < CLASSES)
			i++;
		qsort(class, i, sizeof(int), comparison);
		if (red)
			redisplay();
	}
}
/*
 * I N I T I A L   O R I E N T A T I O N --
 * invert if area c.i. > 50 (should really be length c.i.),
 */
initialorientation(obj)
struct chromosome *obj;
{
	register struct chromplist *plist;
	plist = obj->plist;
	if (plist->dispor != 1 && plist->dispor != -1) {
		if (plist->cindexa > 50)
			plist->dispor = -1;
		else
			plist->dispor = 1;
	}
}

/*
 * S E T F R A M E --
 *
 */
setframe(obj,f)
struct chromosome *obj;
struct pframe *f;
{
	if (obj->plist->dispor == -1) {
		f->ix = -1;
		f->iy = -1;
		f->ox = obj->idom->lastkl;
		f->oy = obj->idom->lastln;
	} else {
		f->ix = 1;
		f->iy = 1;
		f->ox = obj->idom->kol1;
		f->oy = obj->idom->line1;
	}
}
/* 
 * D I S P _ C H R O M -- Display a single chromosome
 *
 */
 disp_chrom(col, line, object)
 int col, line;
 struct chromosome *object;
 {
 	col *= 8;
 	line *= 8;
 	f.dx = col;
 	f.dy = line;
 	initialorientation(object);
 	setframe(object, &f);
 	picframe(object, &f);
 }
 /*
  * D I S P _ N A M E -- display cell name at bottom of screen
  *
  */
 disp_name(ptr, xdist)
 struct cellheader *ptr;
 int xdist;
 {
 	char cellname[MAXFILNAM];
 	int len,length;
 	
	tsize(40, 40);
	sprintf(cellname, "%s %d", ptr->cfile.slide, ptr->cfile.cellno);
	length = strlen(ptr->cfile.case);
	len = strlen(cellname);
	if (length < len)
		length = len;
	length = length * 40;
	if (ptr->col_width << dg->xshift < length)
		ptr->col_width = (length >> dg->xshift) + 1;
	len = (ptr->col_width << dg->xshift) - length;
	len = (len / 2) + ((3 * BORDER) / 2);
	dispstring(ptr->cfile.case, (xdist << dg->xshift)+len, BORDER*5);
	dispstring(cellname, (xdist << dg->xshift)+len, BORDER);
	tsize(80, 80);
}
/* 
 *  R E D I S P L A Y -- Display the contents of the list onto the screen.
 *
 */
redisplay()
{
	struct cellheader *ptr;
	struct homolog *p, *start, *finish;
	int lineno, oldclass, xdist, ydist, dist, leftover, width;
	int length, len, yflag, incp;
	int ypos[CLASSES];
	char cellname[MAXFILNAM];

	doboxes(header);
	calc_width();
	calc_length();
	yflag = 0;
	ptr = shptr;
	xdist = dg->minx + XSTART;
	ydist = dg->maxy - YSTART;
	lineno = 0;
	while (class[lineno] != -1 && lineno < CLASSES) {
		ydist = ydist - YGAP - maxlen[lineno];
		ypos[lineno] = ydist;
		lineno++;
	}
	if (ideo_on)
		disp_ideo(ypos, &xdist, &yflag);
	while (ptr != NULL) {
		disp_name(ptr, xdist);
		sprintf(cellname, "%s %s %d", ptr->cfile.case, ptr->cfile.slide, ptr->cfile.cellno);
		p = ptr->hom;
		oldclass = p->class;
		lineno = 0;
		width = 0;
		start = p;
		while( start != NULL) {
			incp = 0;					/* move p to next object ? */
			dist = xdist;
			if (p == NULL || p->class == class[lineno] || p->class != oldclass) {    /* not an empty class */
				if (p == NULL || p->class != oldclass) {   	/* changed class */
					if (oldclass == class[lineno]) {
						finish = p;
						if (ptr->col_width > ((dg->maxx - XFINISH) - (dg->minx + XSTART))) {
							fprintf(stderr, "The class is too wide to fit on screen\n");
							break;
						}
						leftover = ptr->col_width - width;
						if (leftover != 0) {
							leftover /= 2;
							xdist += leftover + XGAP;
						} else
							xdist += XGAP;
						while(start != finish) {
							if (ypos[lineno] < dg->miny + YFINISH) {
								yflag = 1;
 								start = NULL;
								break;
							}
							ydist = ypos[lineno] + ((maxlen[lineno] - start->length)/2);
							disp_chrom(xdist, ydist, start->chrom);
							xdist = xdist + start->width + XGAP;
							start = start->next;
							/* loop to top if in grid movement */
							if (start == NULL && ptr->hom != ptr->top) 
								start = ptr->top;
							if (start == ptr->hom) {
								start = NULL;
								break;
							}
						}
						incp = 1;
						xdist = dist;	
						lineno++;
						width = p->width + XGAP;
					} else {
						fprintf(stderr, "%s has no chromosomes in class %d.\n", cellname, class[lineno]);
						lineno++;
						incp = 0;
						if (lineno == CLASSES || class[lineno] == -1)
							break;
					}
				} else {
					width = p->width + width + XGAP;
					incp = 1;
				}
				if (p != NULL && incp == 1) {
					oldclass = p->class;
					p = p->next;
					if (p == NULL && ptr->hom != ptr->top) {  /* loop top of list */
						p = ptr->top;			/* if in grid movement */
					}
				}
			} else {
				fprintf(stderr, "%s has no chromosomes in class %d.\n", cellname, class[lineno]);
				lineno++;
				if (lineno == CLASSES || class[lineno] == -1)
					break;
			}
		}
		xdist = xdist + ptr->col_width + XGAP;
		lwidth(2);
		moveto((xdist*8), 0);			/* vertical lines */
		lineto((xdist*8), (dg->maxy - BORDER) << dg->yshift);
		ptr = ptr->nxtcell;
		if (ptr != NULL) {
			if (xdist + ptr->col_width > dg->maxx - XFINISH) {
				fprintf(stderr, "There are more cells than will fit on the screen.\n");
				break;
			}
		}
	}
	lineno = 0;
	while (class[lineno] != -1 && lineno < CLASSES) {
		ydist = ypos[lineno] + (maxlen[lineno]/2);
		if (ypos[lineno] < dg->miny + YFINISH)
			break;
		if (class[lineno] == 23)
			dispstring("X", 104, ydist*8);
		else if (class[lineno] == 24)
			dispstring("Y", 104, ydist*8);
		else if (class[lineno] == 25)
			dispstring("A", 104, ydist*8);
		else
			dispnum(class[lineno], 104, ydist*8);
		ydist = ypos[lineno] - (YGAP/2);	/* display class nos */
		if (class[lineno+1] == -1 || ypos[lineno + 1] < dg->miny + YFINISH)
			break;
		moveto((dg->minx + BORDER) << dg->xshift, ydist*8);		/* horizontal lines */
		lineto(xdist*8, ydist*8);
		lineno++;
	}
	if (yflag)
		fprintf(stderr, "There are more classes than will fit on the screen.\n");
}
/*
 * D I S P _ I D E O -- display all ideograms
 *
 */
disp_ideo(ypos, xdist, yflag)
int ypos[], *xdist, *yflag;
{
	struct ideo_list *iptr;
	int lineno, maxwidth, x, y;
	
	lineno = 0;
	maxwidth = 0;
	iptr = top_ideo;
	x = (*xdist << dg->xshift) + ((iptr->width * SCALE) / 2);
	while (1) {
		if (ypos[lineno] < dg->miny + YFINISH) {
			*yflag = 1;
			break;
		}
		y = (ypos[lineno] << dg->yshift) + (iptr->chptr->q_length * SCALE);
		draw_ideogram(iptr->chptr, x, y, SCALE);
		if (maxwidth < iptr->width)
			maxwidth = iptr->width;
		lineno++;
		iptr = iptr->next;
		if (iptr == NULL)
			iptr = start_ideo;
		if (iptr == top_ideo)
			break;
	}
	*xdist = ((*xdist<<dg->xshift) + (maxwidth*SCALE) + (XGAP<<dg->xshift)) >> dg->xshift;
	lwidth(2);
	moveto((*xdist << dg->xshift), 0);
	lineto((*xdist << dg->xshift), (dg->maxy - BORDER) << dg->yshift);
}
/*
 * C A L C _ W I D T H -- Calculate the maximum width of each column
 *
 */
calc_width()
{
	struct cellheader *ptr;
	struct homolog *p;
	int oldclass, max_width, width;
	
	ptr = startptr;
	while( ptr != NULL) {
		p = ptr->hom;
		oldclass = p->class;
		width = 0;
		max_width = 0;
		while( p != NULL) {
			if (p->class != oldclass) { 	/* changed class */
				if (width > max_width)
					max_width = width;
				width = 0;
			}
			width = width + p->width + XGAP;
			oldclass = p->class;
			p = p->next;
			if ( p == ptr->hom) {
				p = NULL;
				break;
			}
			if (p == NULL && ptr->hom != ptr->top)
				p = ptr->top;
		}
		if (width > max_width)
			max_width = width;
		ptr->col_width = max_width;
		ptr = ptr->nxtcell;
	}
}
/*
 * C A L C _ L E N G T H -- Calculates the maximum length of each class
 *
 */
calc_length()
{
	struct cellheader *ptr;
	struct homolog *p, *parray[CELLS];
	struct ideo_list *iptr;
	int i, j, maxlength, oldclass;

	if (shptr != NULL) {
		ptr = shptr;
		i = 0;
		while(ptr != NULL) {			/* set up ptr array */
			parray[i] = ptr->hom;
			i++;
			ptr = ptr->nxtcell;
		}
		ptr = shptr;
		iptr = top_ideo;
		i = 0;
		j = 0;
		maxlength = 0;
		while (1) {
			p = parray[i];
			if (p != NULL) {
				oldclass = p->class;
				if (p->class == class[j]) {
					while (p->class == oldclass && p != NULL) {
						if (p->length > maxlength)
							maxlength = p->length;
						oldclass = p->class;
						p = p->next;
						if (p == NULL)
							p = ptr->top;
						if (p == ptr->hom) {
							p = NULL;
							break;
						}
					}
				}
			}
			parray[i] = p;
			i++;
			ptr = ptr->nxtcell;
			if (ptr == NULL) {
				i = 0;
				maxlen[j] = maxlength;
				/* if ideos on check that maxlength is longer than ideogram */
				if (ideo_on) {
					if (maxlength < ((iptr->length * SCALE) >> dg->yshift))
						maxlen[j] = ((iptr->length * SCALE) >> dg->yshift);
					iptr = iptr->next;
					if (iptr == NULL)
						iptr = start_ideo;
				}
				j++;  /* move to next class */
				maxlength = 0;
				ptr = shptr;
				if (p == NULL && class[j] == -1)
					break;
			}
		}
	}
}
/*
 * S T R I N T -- converts string input into an integer array
 *
 */
strint(str, n, max, convert)
char *str;
int n[], max, convert;
{
	int j, number;
	char character, *strbeg;
	
	strbeg = str;
	j = 0;
	while (*str != '\0' && j < max) {
		character = *str;
		if (*str == 'X' || *str == 'x') {
			n[j] = 23;
			str++;
			j++;
		} else if (*str == 'Y' || *str == 'y') {
			n[j] = 24;
			str++;
			j++;
		} else if (*str == 'A' || *str == 'a') {
			n[j] = 25;
			str++;
			j++;
		} else if (isdigit(character) != 0 || character == '-' || character == '+') {
			number = atoi(str);
			if (number >= 0) {
				n[j] = number;
				while(isdigit(*str) || *str == '-' || *str == '+')
					str++;
			} else {
				printf("%d is not a valid number!\n",number);
				break;
			}
			j++;
		} else	
			str++;
	}
	qsort(n, j, sizeof(int), comparison);
    while (j < max) {
    	n[j] = -1;
    	j++;
    }
	checkclass(n);
    str = strbeg;
    j = 0;
	while (n[j] != -1 && j < max) {
		if (convert) {
			switch (n[j]) {
				case 23:
					sprintf(str, "X ");
					break;
				case 24:
					sprintf(str, "Y ");
					break;
				case 25:
					sprintf(str, "A ");
					break;
				default:
					sprintf(str, "%d ", n[j]);
					break;
			}
		} else
			sprintf(str, "%d ", n[j]);
		str += strlen(str);
		j++;
	}
	if (j)  str--;
	*str = '\0';
}

/*
 * C O M P A R I S O N -- Comparison for qsort
 *
 */
int comparison(num1, num2)
int *num1, *num2;
{
	if (*num1 < *num2)
		return(-1);
	else if (*num1 == *num2)
		return(0);
	else
		return(1);
}
/*
 * C H E C K C L A S S
 *
 */
checkclass(n)
int n[];
{
	int i, j, k;
	
	j = 0;
	while (n[j] != -1 && j < CLASSES) {
		i = j + 1;
		while (n[i] != -1 && i < CLASSES) {
			if (n[i] == n[j]) {
				k = i;
				while (n[k] != -1 && k < CLASSES) {
					n[k] = n[k+1];
					k++;
				}
				n[k] = -1;
				j--;
			} else
				i++;
		}
		j++;
	}
}

