/*
 * anglelist.c
 * ---------
 * Ji Liang Mar 15, 1987
 *
 * get every angle along the boundary "plgd" which less 
 * than the threshould "thres"
 * 
 * Modifications
 *
 *	 4 Nov 1987		CAS		Lots of tidy + a bit of debug
 * 03-09-87	jimp	Check number of angles against MAX_ANGS as we go.
 * 27-08-87	jimp	Register variables.
 * 26-08-87	jimp	Debug messages to debug_channel
 * 4) Aug 10, 1987	Ji Liang	Use aerage angle in angle list.
 * 3) Jule 14, 1987 Ji Liang	Change float to int
 * 2) June 29, 1987 Ji Liang
 *    Change maximum number of analysible angles from 50 to 150
 * 1) June 16, 1987 Ji Liang
 * Slightly changed the way to choose the vertex position of
 *    an angle if there are more than two successive points quarlify
 *    the angle condition
 *
 */
#include <math.h>
#include <stdio.h>
#include <wstruct.h>
#include <split.h>

#ifdef DEBUG
static FILE *debug_channel = NULL;
#endif

anglelist(plgd, alist, thresh)
struct polygondomain *plgd;
register struct anglist *alist;
int thresh;
{
	register struct ivertex *wtx, *tmp;
	struct ivertex *end;
	register int i, m=0, y = -1, z = -1;
	int sign=0, k, n, x;
	/*
	 * if there are continuous small angles flag = 1
	 * (among them only choose one smallest angle)
	 * if the distance between two succissive small angles
	 * is greater than 3 pixels, interpolate one and sign = 1
	 */
	wtx = plgd->vtx;
	n = plgd->nvertices;
	if( n < 24){
#ifdef DEBUG
		if (debug_channel != NULL)
			fprintf(debug_channel,"the object is too small, cannot get list\n");
#endif
		return(0);
	}
	tmp = wtx;
	while(absdis(wtx, tmp) < ANGPIXEL)
		wtx++;
	end = wtx + n - 12;
	/*
	 * first, decide how many points should be concerned 
	 * then calculate the angle
	 * final, campare the angles, find the smallest one
	 */
	while(wtx != end){
		k = anglex(wtx, &x, ANGPIXEL);
		if(k >= 0 && cmpangl(x, thresh) < 0){
			if(sign == 0){
				tmp = wtx;
				alist->ang[m].vtx = wtx;
				y = x;
			} else {
				if(cmpangl(x, y) < 0){
					alist->ang[m].vtx = wtx;
					y = x;
				}
			}
			sign++;
		} else {
			/*
			 * recalculate the angle. it is nearly the average
			 */
			if(sign > 0){
				i = (int) (wtx - tmp);
				i >>= 1;
				if(((int)(tmp + i - alist->ang[m].vtx)) > 1)
					alist->ang[m].vtx += 1;
				else if(((int)(alist->ang[m].vtx-tmp-i)) > 1)
					alist->ang[m].vtx -= 1;
				else alist->ang[m].vtx = tmp + i;
				tmp--;
				while(absdis(alist->ang[m].vtx, tmp) < ANGPIXEL)
					tmp--;
				alist->ang[m].st = tmp;
				tmp = wtx;
				while(absdis(alist->ang[m].vtx, tmp) < ANGPIXEL)
					tmp++;
				alist->ang[m].end = tmp;
				if(angle3p(alist->ang[m].st,alist->ang[m].vtx,tmp,&x) >= 0){
					alist->ang[m].alph = x;
					alist->ang[m].prop = 1;
					/*
					 * object against fault angle caused by
					 * very narrow channel or tiny
					 * attached fragment
					 */
					if(cmpangl(x,z)<0){
						z = x;
						alist->small = m;
					}
					m++;
				}
				sign = 0;
			}
		} 
		wtx++;
		if (m >= MAX_ANGS) {
#ifdef DEBUG
			if (debug_channel != NULL)
				fprintf(debug_channel,"Anglelist: there are too many angles\n");
#endif
			return(0);
		}
	}
	alist->tanang = z;
	alist->num = m;
	return(m);
}

/*
 * assignment of iang i.e.
 * angle (iang[0], iang[1], iang[2]) equals to angle ang
 */
equalang(ang, iang)
register struct angle *ang;
register struct ivertex *iang;
{
	iang[0].vtY = ang->st->vtY;
	iang[0].vtX = ang->st->vtX;
	iang[1].vtY = ang->vtx->vtY;
	iang[1].vtX = ang->vtx->vtX;
	iang[2].vtY = ang->end->vtY;
	iang[2].vtX = ang->end->vtX;
}


/*
 * retund x/y including rounding
 */
roundivid(x, y)
register int x, y;
{
	register int i, j;
	
	if (y == 0){
#ifdef DEBUG
		if (debug_channel != NULL)
			fprintf(debug_channel,"illegal dividing\n");
#endif
		exit(-1);
	}
	j = x / y;
	i = x % y;
	i <<= 1;
	if(abs(i) > abs(y)){
		if(j > 0)
			return(j + 1);
		else if(j < 0)
			return(j - 1);
		else {
			if(x > 0 && y > 0 || x < 0 && y < 0)
				return(1);
			else return(-1);
		}
	} else return(j);
}

seeanglist(alist)
struct anglist *alist;
{
#ifdef DEBUG
	int i, j;

	if (debug_channel != NULL) {
		for(i = 0; i < alist->num; i++){
			fprintf(debug_channel,"%d\t", i);
			fprintf(debug_channel,"%d, %d \t",alist->ang[i].st->vtY,alist->ang[i].st->vtX);
			fprintf(debug_channel,"%d, %d \t",alist->ang[i].vtx->vtY,alist->ang[i].vtx->vtX);
			fprintf(debug_channel,"%d, %d \t",alist->ang[i].end->vtY,alist->ang[i].end->vtX);
			fprintf(debug_channel,"%d, %d\n ",alist->ang[i].alph,alist->ang[i].prop);
		}
		fprintf(debug_channel,"small = %d\n", alist->small);
	}
#endif
}

seepoly(plgd)
struct polygondomain *plgd;
{
#ifdef DEBUG
	register i, j;
	struct ivertex *wtx = plgd->vtx;

	if (debug_channel != NULL) {
		j = 0;
		for(i = 0; i < plgd->nvertices; i++){
			fprintf(debug_channel,"%d\t%d\t",wtx->vtY,wtx->vtX);
			if(j == 4){
				fprintf(debug_channel,"\n");
				j = 0;
			}
			wtx++;
			j++;
		}
	}
#endif
}

/*
 * calculate the angle which is formed by points
 * p0, p, p2 if the reseult is bigger than pi return -1
 * if it is equal to pi / 2 return 0 otherwise return 1
 */
angle3p(p0, p, p1, x)
register struct ivertex *p0, *p, *p1;
int *x;
{
	register int h1, d1, h2, d2;
	int i, k;
	
	h1 = p->vtY - p0->vtY;
	d1 = p->vtX - p0->vtX;
	h2 = p1->vtY - p->vtY;
	d2 = p1->vtX - p->vtX;
	i = d2 * h1 - d1 * h2;
	k = h1 * h2 + d1 * d2;
	if(i <= 0)
		return(-1);
	if(k == 0){
		*x = 0;
		return(0);
	}
	*x =  -(i * FTOI) / k;
	return(1);
}

/*
 * get the second normal smallest angle from angle list
 */
secondang(alist)
register struct anglist *alist;
{
	register int x = -1, i, j = BIGVALUE;

	for(i = 0; i < alist->num; i++){
		if(cmpangl(alist->ang[i].alph, x) < 0 && alist->ang[i].prop != 0) {
			x = alist->ang[i].alph;
			j = i;
		}
	}
	return(j);
}

/*
 * Find a point "p" which is allocated at angular bisector.
 * If the angle is less than pi ( the direction from ang[0] to
 * ang[2] is anti-clockwise ) the point is pointed out by the
 * angle. If it is bigger than pi the point is inside the angle.
 * If it is equaled to pi the point is at left hand.
 * The disstance between the point and the vertex of the angle is "d".
 */
expandpoint(ang, p, d)
register struct ivertex *ang, *p;
register int d;
{
	struct ivertex ref;
	register int i, x, y;
	int flag;

	flag = anglesspi(ang);
	reverdividp(ang, &ref);
	x = ref.vtX - ang[1].vtX;
	y = ref.vtY - ang[1].vtY;
	i = hypot8(x, y);
#ifdef DEBUG
	if (debug_channel) 
		fprintf(debug_channel,"expandpoint: x %d y %d, hyp %d\n",x,y,i);
#endif DEBUG
	if(i < 8){
	 	x = ang[1].vtX - ang[0].vtX;
		y = ang[1].vtY - ang[0].vtY;
		i = hypot8(x, y);
		flag = 0;
#ifdef DEBUG
		if (debug_channel) 
			fprintf(debug_channel,"expandpoint: x %d y %d, hyp %d\n",x,y,i);
#endif DEBUG
	} 
	if(abs(x) > abs(y)){
		y = roundivid(d * y, abs(x));
		if(x > 0)
			x = d;
		else
			x = -d;
	} else {
		x = roundivid(d * x, abs(y));
		if(y > 0)
			y = d;
		else
			y = -d;
	}
	switch(flag){
		case 1:
			p->vtY = ang[1].vtY + y;
			p->vtX = ang[1].vtX + x;
			return(0);
		case 0:
			p->vtY = ang[1].vtY + x;
			p->vtX = ang[1].vtX - y;
			return(0);
		case -1:
			p->vtY = ang[1].vtY - y;
			p->vtX = ang[1].vtX - x;
			return(0);
	}
}

/* 
 * define the angle unti-clockwise from ang[0] to ang[2].
 * if the angle < pi return 1,
 *		= 	    0,
 *		>	   -1.
 */
anglesspi(ang)
register struct ivertex *ang;
{
	register int h1, d1, h2, d2;
	int i;
	
	h1 = ang[1].vtY - ang[0].vtY;
	d1 = ang[1].vtX - ang[0].vtX;
	h2 = ang[2].vtY - ang[1].vtY;
	d2 = ang[2].vtX - ang[1].vtX;
	i = d2 * h1 - d1 * h2;
	if(i < 0)
		return(-1);
	else if(i == 0)
		return(0);
	else return(1);
}
