/*
 * averagegrey.c
 * -------------
 * Ji Liang Jun 4, 1987
 *
 * get averageand maximum grey level
 * =================================
 * Modification history
 * ------------------
 * 27-08-87	jimp	Registers and other small efficiency changes
 * 14-07-87 JL	Change double and float to int
 * 01-07-87 JL	Added function boundgrey().
 * 29-06-87 JL  Chnge totg from GREY to double for big image
 *
 */

#include <stdio.h>
#include <wstruct.h>
#include <split.h>

GREY averagegrey(obj, maxg)
struct object *obj;
GREY *maxg;
{
	struct iwspace iwsp;
    	struct gwspace gwsp;
    	register GREY *g;
    	register int max=0, total=0, i, totg=0;

    	initgreyscan(obj,&iwsp,&gwsp);
    	while (nextgreyinterval(&iwsp) == 0) {
                g = gwsp.grintptr;
		total += iwsp.colrmn;
                for (i=iwsp.colrmn; i>0; i--, g++) {
			totg += *g;
			if(*g > max)
				max = *g;
		}
	}
	*maxg = max;
	return(totg/total);
}

/*
 * measure the balance of the grey level
 * top/bottom, left/right, top left / bottom right
 * and top right / bottom
 * if balance return 1
 * otherwise return 0
 */

greybalance(obj)
struct object *obj;
{
	struct intervaldomain *idmn = obj->idom;
	struct iwspace iwsp;
    	struct gwspace gwsp;
    	register GREY *g;
    	int maxg = 0, totg = 0;
	GREY gdis[8];
	int a[8];
	register int j, k, ll, ver;
    	int i, n, hor, kol1, lastkl, rr;
	int rate[4];
	int slop, delta;

	kol1 = idmn->kol1;
	lastkl = idmn->lastkl;
	slop = (lastkl - kol1 + 1) * FTOI / (idmn->lastln - idmn->line1 + 1);
	delta = -slop;
	ver = kol1 + lastkl;
	hor = idmn->line1 + idmn->lastln;
	ver >>= 1;
	hor >>= 1;
	for(i = 0; i < 8; i++){
		gdis[i] = 0;
		a[i] = 0;
	}
    	initgreyscan(obj,&iwsp,&gwsp);
    	while (nextgreyinterval(&iwsp) == 0) {
		if(iwsp.nwlpos == 1){
			delta += slop;
			n = delta / FTOI;
			if(iwsp.linpos < hor){
				ll = kol1 + n;
				rr = lastkl - n;
			} else if (iwsp.linpos == hor){
				delta = 0;
			} else {
				ll = ver - n;
				rr = ver + n;
			}
		}
                g = gwsp.grintptr;
		j = iwsp.colpos;
                for (i=0; i<iwsp.colrmn; i++, j++, g++) {
			if(iwsp.linpos < hor){
				if(j <= ll)
					k = 0;
				else if(j > ll && j <= ver)
					k = 1;
				else if(j > ver && j <= rr)
					k = 2;
				else k = 3;
			} else {
				if(j <= ll)
					k = 4;
				else if(j > ll && j <= ver)
					k = 5;
				else if(j > ver && j <= rr)
					k = 6;
				else k = 7;
			}
			gdis[k] += *g;
			a[k] += 1;
		}
	}
	maxg = (gdis[0]+gdis[1]+gdis[2]+gdis[3]) / (a[0]+a[1]+a[2]+a[3]);
	totg = (gdis[4]+gdis[5]+gdis[6]+gdis[7]) / (a[4]+a[5]+a[6]+a[7]);
	rate[0] = bigrate(maxg, totg); 	/* top / bottom */
	maxg = (gdis[0]+gdis[1]+gdis[2]+gdis[4]) / (a[0]+a[1]+a[2]+a[4]);
	totg = (gdis[3]+gdis[5]+gdis[6]+gdis[7]) / (a[3]+a[5]+a[6]+a[7]);
	rate[1] = bigrate(maxg, totg); 	/* top left / bottom right */
	maxg = (gdis[7]+gdis[1]+gdis[2]+gdis[3]) / (a[7]+a[1]+a[2]+a[3]);
	totg = (gdis[4]+gdis[5]+gdis[6]+gdis[0]) / (a[4]+a[5]+a[6]+a[0]);
	rate[2] = bigrate(maxg, totg); 	/* top right / bottom left */
	maxg = (gdis[0]+gdis[1]+gdis[5]+gdis[4]) / (a[0]+a[1]+a[5]+a[4]);
	totg = (gdis[3]+gdis[2]+gdis[6]+gdis[7]) / (a[3]+a[2]+a[6]+a[7]);
	rate[3] = bigrate(maxg, totg); 	/* left / right */
	for(i = 0; i < 4; i++){
		if(rate[i] > 1)
			return(0);
	}
	return(1);
}

bigrate(x, y)
register int x, y;
{
	if(x == 0 || y == 0)
		return(BIGVALUE);
	if(y < x)
		return(x/y);
	else return(y/x);
}

GREY boundgrey(obj, plgd)
struct object *obj;
struct polygondomain *plgd;
{
	register struct ivertex *wtx = plgd->vtx;
	register i, n, tot=0;
	register GREY *g;
	GREY *greyval();

	n = plgd->nvertices;
	for(i = 0; i < n; i++){
		g = greyval(obj, wtx->vtY, wtx->vtX);
		tot += *g;
		wtx++;
	}
	return(tot/n);
}
