/*
 * conhull1.c 
 * ________________
 *
 * Ji Liang May 8, 1986
 *
 * construct the minimal convex cover for the object 
 * stratege: 
 * 		Three points: 'this' point, last point, and the point 
 *	before last are always used. They are pointed by 'wtx', 'wtx1' 
 *	and 'wtx0'. We deal the obj clockwise. If these three
 *	points form a right turn then 'wtx0 = wtx1' 'wtx1 = wtx'
 *	and increse 'wtx'. If failed do the following:
 *		Keep 'wtx' unchanged and search backward(decrease
 * 	'wtx0' and 'wtx1') until these three form a right turn.
 *		 When the botomn is reached do the same thing again
 *	but only concern with left end.
 * the program is a modification of convhull.c. The only difference is
 * that when the first line only has one pixel convhull.c doesn`t
 * work properly
 *
 * Modifications
 *
 *	 7 Feb 1991		CAS		remove Mizar defs -> osk, voids
 *	22 Jan 1991		CAS		Localise debug
 *	 4 Nov 1987		CAS		Debugging
 *	23 Oct 1987		CAs		Debugging, fail if vtx outwith obj line1->lastln
 *							in finddirec
 * jimp 09-09-87	handle error return from finddirec
 * jimp	08-09-87	Debugging
 * ji	25-08-87	Small change to function trace
 * jimp	24-08-87	Conditionally removed some sections (#ifndef mizar)
 */

#include <stdio.h>
#include <wstruct.h>
#include <split.h>

#ifdef DEBUG
static FILE *debug_channel = NULL;
#endif

#ifndef OSK
struct object *conhull (obj)
struct object *obj;
{
	struct object *cvh, *makemain();
	struct polygondomain *cvhdom, *makepolydmn();
	register struct ivertex *wtx, *wtx1, *wtx0;
	struct iwspace iwsp;


	cvhdom = makepolydmn(1,NULL,0,3+2*(obj->idom->lastln-obj->idom->line1),1);
	cvh = makemain(10,cvhdom,NULL,NULL,obj);
	wtx = cvhdom->vtx;

	/*
	 * procede down right hand side of object
	 */
	initrasterscan(obj,&iwsp,0);
	while (nextinterval(&iwsp) == 0) {
		/*
		 * set up first chord
		 */
		if (iwsp.linpos == obj->idom->line1) {
			if (iwsp.nwlpos == 1) {
				wtx->vtX = iwsp.lftpos;
				wtx->vtY = iwsp.linpos;
				wtx++;
			}
			if (iwsp.intrmn == 0) {
				if(iwsp.rgtpos != (wtx-1)->vtX){
					wtx->vtX = iwsp.rgtpos;
					wtx->vtY = iwsp.linpos;
					wtx++;
					cvhdom->nvertices = 2;
				} else {
					cvhdom->nvertices = 1;
				} 
				wtx1 = wtx-1;
				wtx0 = wtx-2;
			}
		} else {
		/*
		 * add extra chords, checking concavity condition
		 */
			if (iwsp.intrmn == 0) {
				wtx->vtX = iwsp.rgtpos;
				wtx->vtY = iwsp.linpos;
				cvhdom->nvertices++;
				/*
				 * concavity condition (may propagate backwards)
				 */
				while((cvhdom->nvertices > 2) && ((wtx->vtY-wtx1->vtY)*(wtx1->vtX-wtx0->vtX) <= (wtx1->vtY-wtx0->vtY)*(wtx->vtX-wtx1->vtX))){
					wtx--;
					wtx->vtX = iwsp.rgtpos;
					wtx->vtY = iwsp.linpos;
					wtx1--;
					wtx0--;
					cvhdom->nvertices--;
				}
				wtx++;
				wtx1++;
				wtx0++;
			}
		}
	}
	/*
	 * now procede up left hand side of object
	 */
	initrasterscan(obj,&iwsp,2);
	while (nextinterval(&iwsp) == 0) {
		if (iwsp.nwlpos == 1) {
			wtx->vtX = iwsp.lftpos;
			wtx->vtY = iwsp.linpos;
			cvhdom->nvertices++;
			/*
			 * concavity condition (may propagate backwards)
			 */
			while ((wtx->vtY-wtx1->vtY)*(wtx1->vtX-wtx0->vtX) <=
	    			(wtx1->vtY-wtx0->vtY)*(wtx->vtX-wtx1->vtX)) {
				wtx--;
				wtx->vtX = iwsp.lftpos;
				wtx->vtY = iwsp.linpos;
				wtx1--;
				wtx0--;
				cvhdom->nvertices--;
			}
			wtx++;
			wtx1++;
			wtx0++;
		}
	}
	/*
	 * reserve space for parameters
	 */
	cvh->vdom = (struct valuetable *) Malloc (sizeof(struct cvhdom) + (cvhdom->nvertices-1) * sizeof(struct chord));
	((struct cvhdom *)cvh->vdom)->ch = (struct chord *)((char *)cvh->vdom + sizeof(struct cvhdom));
	return(cvh);
}
#endif OSK

/* 
 * neighb():
 * ----------
 * find next interval and decide the next searching direction
 * (l, k) is the input point 
 * direc is the searching direction 
 *	0: clockwise downwards
 *	1: clockwise upwards
 *	2: unticlockwise downwards
 *	3: unticlockwise upwards
 * if the start point is in the middle of junction interval we use the following
 *	4: clockwise downwards
 *	5: clockwise upwards
 *	6: unticlockwise downwards
 *	7: unticlockwise upwards
 * (nexl, nexk) is the next point we are looking for
 */

neighb(obj, l, ak, direc, next)
struct object *obj;
register int l, ak, direc;
struct address *next;
{
	struct intervaldomain *idmn = obj->idom;
	register struct interval *intl;
	struct intervalline *itvl;
	int j, k, nitv, n, sign; 
	register int prer, left, right, fol; 	/* for this line */ 
	register int tmpl, tmpr, incr;		/* for next line */
	int kol1;

	kol1 = idmn->kol1;
	k = ak - kol1;
LOOP:
	if((direc & 01) == 0)
		incr = 1;
	else incr = -1;
	itvl = &idmn->intvlines[l - idmn->line1];
	nitv = itvl->nintvs;
	intl = itvl->intvs;
	n = nitv;
	sign = 0;
	j = 0;
	prer = fol = 1024;
	while (n-- > 0) {
		fol = intl->ileft;
		if(sign == 1)
			break;
		if (k >= intl->ileft && k <= intl->iright){
			left = intl->ileft;
			right = intl->iright;
			sign = 1;
		} else  prer = intl->iright;
		intl++;
		j++;
	}
	sign = 0;
	next->ycood = l;
	if(l == idmn->line1){
		if(direc == 1){
			next->xcood = right;
			next->order = 0;
			sign = 1;
		} else if(direc == 3){
			next->xcood = left; 
			next->order = 2;
			sign = 1;
		}
	} else if(l == idmn->lastln){
		if(direc == 0){
			next->xcood = left;
			next->order = 1;
			sign = 1;
		} else if(direc == 2){
			next->xcood = right;
			next->order = 3;
			sign = 1;
		}
	}
	if(sign == 1){
		if(l == next->ycood && k == next->xcood){
			direc = next->order;
			goto LOOP;
		} else goto END;
	}
	/*
	 * if sign = 1, there is a posibility that searching point 
	 * will be changed(from upwards to downwards or v.r)
	 * if sign = 0, we needn't bother
	 */
	sign = 0;
	if(j != nitv)
		if(direc == 0 || direc == 3)
			sign = 1;
	if (nitv != 1 && j != 1){
		if(direc == 1 || direc == 2)
			sign = 1;
	}
	/*
	 * find the number of neighbour intervals in line l - 1
	 * or l + 1 ( depends on direc)
	 */
	next->ycood = l + incr;
	itvl += incr;
	nitv = itvl->nintvs;
	intl = itvl->intvs;
	j = 0;
	n = nitv;
	while (n-- > 0) {
		if (!((right + 1) < intl->ileft || (left - 1) > intl->iright)){ 
			j++;
			if(direc == 0 || direc == 3){
				if(sign == 1 && (fol <= intl->iright + 1) && (fol >= intl->ileft - 1)){
					j = -1;
					break;
				} else {
					tmpr = intl->iright;
					tmpl = intl->ileft;
				}
			} else if(direc == 1 || direc == 2){
				if(sign == 1 && (prer <= intl->iright + 1) && (prer >= intl->ileft - 1)){
					j = -1;
					break;
				} else {
					tmpl = intl->ileft;
					break;
				}
			} else if((direc & 04) != 0){
				if((j > 1) && (k <= intl->ileft && k >= tmpr)){
					if(direc == 4 || direc == 7)
						next->xcood = tmpr;
					if(direc == 5 || direc == 6)
						next->xcood = intl->ileft;
					next->order = direc - 4;
					goto END;
				} 
				tmpr = intl->iright;
			}
		}
		intl++;
	}
	/* 
	 * if j = -1: this interval is a junction interval and searching
	 *		direction shold be changed
	 *    j = 0: that interval is a terminal interval and  searching 
	 *	     direction should be changed 
	 *    the rest: keep the direction unchanged
	 */
	switch(j){
		case 0:
			next->ycood = l;
			if(direc == 1 || direc == 2){
				next->xcood = right;
				next->order = (direc == 1) ? 0 : 3;
			} else if(direc == 0 || direc == 3){
				next->xcood = left;
				next->order = (direc == 0) ? 1 : 2; 
			}
			break;
		case -1:
			if(direc == 1 || direc == 2){
				next->xcood = (prer + left) >> 1;
				next->order = (direc == 1) ? 4 : 7; /* ji */
			} else if(direc == 0 || direc == 3){
				next->xcood = (right + fol) >> 1;
				next->order = (direc == 0) ? 5 : 6; 
			}
			break;
		default:
			if(direc == 1 || direc == 2){
				next->xcood = tmpl;
			} else if(direc == 0 || direc == 3){
				next->xcood = tmpr;
			}
			next->order = direc;
	}
	if(l == next->ycood && k == next->xcood){
		direc = next->order;
		goto LOOP;
	}
END:
	next->xcood += kol1;
	return(0);
}

/* 
 * trace():
 * --------
 * find the path between point (l1, k1) and point (l2, k2)
 * search clockwise
 */

struct polygondomain *trace(obj, l1, k1, l2, k2)
struct object *obj;
int l1, k1, l2, k2;
{
	struct intervaldomain *idmn = obj->idom;
	struct polygondomain *plgd, *makepolydmn();
	struct ivertex *ivt, *istart;
	int n, nn, line, kol, direc;
	struct address nextitv;

	n = intcount(idmn);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Trace: intcount %d l1 %d k1 %d l2 %d k2 %d\n",n,l1,k1,l2,k2);
#endif DEBUG
	n *= 3;
	nn = n;
	ivt = istart = (struct ivertex *) Malloc (sizeof(struct ivertex) * n);
	line = l1;
	kol = k1;
	if(l2 < l1)
		direc = 1;
	else if(l2 == l1){
		if(k2 < k1)
			direc = 0;
		else direc = 1;
		
	} else direc = 0;
	ivt->vtY = l1;
	ivt->vtX = k1;
	ivt++;
	n = 1;
	while(line != l2 || kol != k2){
		neighb(obj, line, kol, direc, &nextitv);
		n++;
#ifdef DEBUG
if (n > nn && debug_channel != NULL) {
fprintf(debug_channel,"Trace: ivertex array overflow: l %d k %d\n",line,kol);
break;
}
#endif DEBUG
		line = nextitv.ycood;
		kol = nextitv.xcood;
		direc = nextitv.order;
		ivt->vtY = line;
		ivt->vtX = kol;
		ivt++;
	}
	plgd = makepolydmn(1,istart, n, n + 10, 1);
	Free(istart);
	return(plgd);
}



/* 
 * tracecurve():
 * -------------
 * find the path from point (l, k) along the direction
 * searching num points
 * note:
 *	flag = 0: 	search clockwise
 *	        1: 	search unticlockwise
 *	direc: see definition in neigh()
 */

struct polygondomain *tracecurve(obj, l, k, num, flag)
struct object *obj;
int l, k, num, flag;
{
	struct polygondomain *plgd, *makepolydmn();
	struct ivertex *ivt, *istart;
	struct address nextitv;
	register int i, line, kol, n;
	int direc;

#ifdef DEBUG
	if (debug_channel) {
		fprintf(debug_channel,"tracecurve: obj 0x%x, l %d, k %d, num %d, flag %d\n",obj,l,k,num,flag);
		fflush(debug_channel);
	}
#endif DEBUG
	n = num + 1;
	direc = finddirec(obj, l, k, flag);
	if (direc < 0)
		return(NULL);
	ivt = istart = (struct ivertex *) Malloc (sizeof(struct ivertex) * n);
	line = l;
	kol = k;
	ivt->vtY = line;
	ivt->vtX = kol;
	ivt++;
	for(i = 0; i < num - 1; i++){
		neighb(obj, line, kol, direc, &nextitv);
		line = nextitv.ycood;
		kol = nextitv.xcood;
		direc = nextitv.order;
		if(line != (ivt-1)->vtY || kol != (ivt-1)->vtX){
			ivt->vtY = line;
			ivt->vtX = kol;
			ivt++;
		}
	}
	plgd = makepolydmn(1, istart, num, num + 1, 1);
	Free(istart);
	return(plgd);
}

/*
 * find serching direction for other subrotine
 * flag:
 *	0: clockwise
 *	1: unticlockwise
 */
finddirec(obj, l, kl, flag)
struct object *obj;
int l, kl, flag;
{
	struct intervaldomain *idmn;
	struct interval *intl;
	struct intervalline *itvl;
	register int n, k;
	int direc, right, sign = 0;
#ifdef DEBUG
	if (debug_channel) {
		fprintf(debug_channel,"finddirec: obj 0x%x\n",obj);
		fflush(debug_channel);
		fprintf(debug_channel,"finddirec: idom 0x%x\n",obj->idom);
		fflush(debug_channel);
		fprintf(debug_channel,"finddirec: l %d, line1 %d, lastln %d\n",l, obj->idom->line1, obj->idom->lastln);
		fflush(debug_channel);
	}
#endif DEBUG
	idmn = obj->idom;
	if ((l < idmn->line1) || (l > idmn->lastln))
		return(-1);
	/*
	 * find searching direction
	 */
	k = kl - idmn->kol1;
	itvl = &idmn->intvlines[l - idmn->line1];
	intl = itvl->intvs;
	n = itvl->nintvs;
	/*
	 * if it is an end point
	 */
	while(n-- > 0){
		if(k == intl->ileft){
			if(flag == 0)
				direc = 1;
			else direc = 2;
			sign = 1;
		} else if(k == intl->iright){
			if(flag == 0)
				direc = 0;
			else direc = 3;
			sign = 1;
		}
		if(k == intl->ileft && k == intl->iright){
#ifdef DEBUG
		if (debug_channel != NULL)
			fprintf(debug_channel,"Finddirec: deepest point ambiguous\n");
#endif DEBUG
			return(-1);
		}
		intl++;
	}
	/*
	 * that point is in a junction interval
	 */
	if(sign == 0){
		right = 1024;
		itvl++;
		intl = itvl->intvs;
		n = itvl->nintvs;
		while(n-- > 0){
			if(k >= right && k <= intl->ileft){
				if(flag == 0) 	/* ji */
					direc = 4;
				else direc = 6;
				sign = 1;
				break;
			}
			else right = intl->iright;
			intl++;
		}
		if(sign == 0){
			if(flag == 0)	/* ji */
				direc = 5;
			else direc = 7;
		}
	}
	return(direc);
}
