/*
 * contourtobj.c
 * ------------
 * Ji Liang  Aug 19, 1987
 *
 * get an object from a closed contour
 * the contour polygon shold be clockwise
 *
 * Modifications
 *
 *	 1 Jun 1989		CAS		Made reaalloc static - tempory fix.
 *							Note this realloc could cause crashes. It takes
 *							no account of overlapping mem and it IS possible
 *							for another process to grab the Free'd chunk in the
 *							middle...
 *	25 Aug 1987		jimp	(i) Unused code #ifdef-ed out,
 *							(ii) Reallc added for OS9.
 */
#include <stdio.h>
#include <wstruct.h>
#include <split.h>

#ifdef OSK
static char *realloc();
#endif OSK

#ifndef OSK
struct object *contourtobj(plgd)
struct polygondomain *plgd;
{
	struct object *obj, *uobj, *polyobj;
	struct object *unionn(), *polytoobj();
	struct object *new[5], *makemain();
	struct intervaldomain *idmn, *makedomain();
	struct interval *jp;
	struct ivertex *wtx, *ivt;
	register int i, j, k;
	int n, l1, k1, ll, lk;
	/*
	 *  need test ! put it in another file
	 */
}
#endif OSK

/*
 * get the area surrounded by the contour
 */
contourarea(plgd)
struct polygondomain *plgd;
{
	struct ivertex *v, *wtx, *ivt = plgd->vtx;
	register i, j, k, p;
	int l, m, n, area = 0;
	char *Malloc(), *realloc();

	wtx = ivt;
	n = plgd->nvertices;
	ivt += n - 1;
	if(ivt->vtY != wtx->vtY || wtx->vtX != ivt->vtX){
		fprintf(stderr,"the contour is not closed! cannot get area\n");
		return(BIGVALUE);
	}
	/*
	 * it is a line not triangle !!
	 */
	if(n < 4){
		ivt = wtx + 1;
		i = ivt->vtY - wtx->vtY;
		j = ivt->vtX - wtx->vtX;
		l = hypot8(i, j);
		l >>= 3;
		return(l);
	}
	/*
	 * interpolate if the distance between two vertices
	 * is bigger than one. Make calculate easier later on.
	 */
	m = n + n;
	v = (struct ivertex *) Malloc(sizeof(struct ivertex) * m);
	j = 0;
	ivt = wtx + 1;
	for(i = 0; i < n - 1; i++){
		v[j].vtY = wtx->vtY;
		v[j++].vtX = wtx->vtX;
		k = ivt->vtX - wtx->vtX;
		l = ivt->vtY - wtx->vtY;
		if(abs(l) + j > m){
			p = sizeof(struct ivertex) / sizeof(char);
			p *= (m + n);
			v = (struct ivertex *) realloc(v, p);
			m += n;
		}
		if(l > 1){
			for( p = wtx->vtY + 1; p < ivt->vtY; p++){
				v[j].vtY = p;
				v[j++].vtX = roundivid((p-wtx->vtY) * k, l)  + wtx->vtX; 
			}
		} else if(l < -1){
			for( p = wtx->vtY - 1; p > ivt->vtY; p--){
				v[j].vtY = p;
				v[j++].vtX = roundivid((p-wtx->vtY) * k, l)  + wtx->vtX; 
			}
		}
		ivt++;
		wtx++;
	}
	/*
	 * wrap 2 in case first two vertices in a horizotal line
	 */
	for(i = 0; i < 3; i++){
		v[j].vtY = v[i].vtY;
		v[j++].vtX = v[i].vtX;
	}
	m = j;
	/*
	 * calculate the interior area
	 */
	p = 0;
	for(i = 1, j = 2, k = 3; j < m - 1; j++, i++, k++){
		l = v[k].vtY - v[j].vtY;
		switch(l){
		case 1:
			if(v[j].vtY > v[i].vtY){
				area += v[j].vtX;
				p++;
			} else if(v[j].vtY == v[i].vtY){
				if(v[i].vtY > v[i-1].vtY){
					area += MIN(v[j].vtX, v[i].vtX);
					p++;
				} else {
					if(v[k].vtX < v[i].vtX){
						area += v[j].vtX - v[i].vtX;
						p++;
					}
				}
			} else {
				if(v[k].vtX < v[i].vtX)
					p++;
			}
			break;
		case -1:		
			if(v[j].vtY < v[i].vtY){
				area -= v[j].vtX;
			} else if(v[j].vtY == v[i].vtY){
				if(v[i].vtY < v[i-1].vtY){
					area -= MAX(v[j].vtX, v[i].vtX);
				} else {
					if(v[k].vtX > v[i].vtX){
						area -= v[j].vtX - v[i].vtX;
						p++;
					}
				}
			} else {
				if(v[k].vtX > v[i].vtX)
					p++;
			}
			break;
		default:
			break;
		}
	}
	area = abs(area);
	area += p;
	Free(v);
	return(area);
}

#ifndef OSK
contourlength(plgd)
struct polygondomain *plgd;
{
	struct ivertex *wtx, *ivt = plgd->vtx;
	register i, j, k;
	int l = 0, n = plgd->nvertices;

	wtx = ivt + 1;
	for(i = 0; i < n - 1; i++){
		j = ivt->vtY - wtx->vtY;
		k = ivt->vtX - wtx->vtX;
		l += hypot8(j, k);
		wtx++;
		ivt++;
	}
	l >>= 3;
	return(l);
}
#endif OSK

#ifdef OSK
static char *realloc(p,n)	/*compacting reallocation */
register char *p;
register unsigned int n;
{
	register char *q;
	register int	i;
	char *retval;
	register char *qtop;

	Free(p);		/* be aware that another process might take this space
						before it is realocated - wrong jim piper */
	q = Malloc(n);
	if(q == p) {
		return(q);
	}
	retval = q;
	qtop = q+n;
	while (q<qtop) {
		*q++ = *p++;
	}
	return(retval);
}
#endif OSK
