/*
 * disoverlap.c
 * -----------
 * Ji Liang Sept 3, 1986
 *
 * split overlaped object into two separate objects
 * plist gives the four intersections
 *
 * Modifications
 *
 *	24 Nov 1988		SCG/dcb	modified call to polysplit to include objlist length	
 *	22 Nov 1988		dcb		Added param. to woolz_check.. to indicate where called from
 *	16 Nov 1988		SCG		check validity of incoming object in disoverlap
 *	12 Nov 1987		SCG		poly_split can return 0 objects in which case
 *							obj in objlist freed twice.  see in-code comments
 *							for fix
 *	 5 Nov 1987		CAS		Allocate sufficient space for poly_split obj list
 *							added a split.h file..
 *	 4 Nov 1987		CAS		More freeing stuff
 *	30 oct 1987		CAS		Yes.. polysplit can return 3 objects..
 *	29 Oct 1987		CAS		+ Still more - note if poly_split returns 3 objs - 3rd will get lost..
 *	28 Oct 1987		CAS		Improve object freeing
 *							+ more still..
 * 10-09-87	jimp	Debugging.
 * 09-09-87	jimp	Handle error returns from tracecurve() and poly_split()
 *			better.
 * 26=08-87	jimp	Object free-ing changed to use linkcount.
 * 16-08-87	JL  Change return object to int
 * 11-08-87 JL	Free more space.
 * 03-08-87	JL	Change polysplit() to poly_split().
 * 14-07-87	JL	Change split() to polysplit() so as to unify
 *			the library with woolz
 *			Modified ifcross() to suit both type 1 and 10 object
 */
#include <stdio.h>
#include <ctype.h>
#include <wstruct.h>
#include <split.h>

#ifdef DEBUG
static FILE *debug_channel = NULL;
#endif

disoverlap(obj, plist, new0, new1)
struct object *obj;
struct ivertex *plist;
struct object **new0, **new1;
{
	struct object *objlist[POLYSPLITMAXOBJ], *makemain(); 
	struct object *obj1, *pobj[2], *outobj[2]; 
	struct polygondomain *trace(), *tracecurve();
	struct polygondomain *plgd[2], *getcurve(), *splitplg[2];
	int i, n, j, k, list, m;
	int num;

#ifdef DEBUG
	if (debug_channel != NULL)
	fprintf(debug_channel,"Disoverlap: entry\n");
#endif DEBUG	

	/*
	 * check whether incoming object is valid
	 */
	if(woolz_check_obj(obj, "disoverlap") != 0)
		return(0);
		
	/*
	 * each list loop gets one split object
	 * each object needs two splitting curves
	 * each curve needs two sets of points to find them
	 * k decide the searching direction
	 */
	for(list = 0; list < 2; list++){
		n = 0;
		k = 1;
		for(i = list; i < 4; i += 2){
			for(j = 0; j < 2; j++){
#ifdef DEBUG
				if (debug_channel) {
					fprintf(debug_channel,"disoverlap: before tracecurve i %d, j %d, plist 0x%x\n",i,j,plist);
					fflush(debug_channel);
					fprintf(debug_channel,"Y %d, X %d\n",plist[j+i].vtY,plist[j+i].vtX);
					fflush(debug_channel);
				}
#endif DEBUG
				plgd[j]=tracecurve(obj,plist[j+i].vtY,plist[j+i].vtX, 5, k);
				if (plgd[j] == NULL) {
					for (m=0; m < n; m++)
						freeobj(pobj[m]);
					if (j > 0)
						Free(plgd[0]);
#ifdef DEBUG
					if (debug_channel != NULL)
						fprintf(debug_channel,"Decomposition failure\n");
#endif DEBUG
					if (list == 1)
						freeobj(outobj[0]);
					return(0);
				}
				k = (k > 0) ? 0 : 1;
			}
#ifdef DEBUG
			if (debug_channel != NULL)
				fprintf(debug_channel,"Disoverlap: before getcurve (list %d, i %d) idom 0x%x\n",list,i,obj->idom);
#endif DEBUG	
			splitplg[n] = getcurve(plgd[0], plgd[1], 1);
#ifdef DEBUG
			if (debug_channel != NULL)
				fprintf(debug_channel,"Disoverlap: after getcurve idom 0x%x\n",obj->idom);
#endif DEBUG	
			Free(plgd[0]);
			Free(plgd[1]);
			pobj[n] = makemain(10, splitplg[n], NULL, NULL, NULL);
			n++;
		}
#ifdef DEBUG
		if (debug_channel != NULL)
			fprintf(debug_channel,"Disoverlap: before poly_split (0) (list %d)\n",list);
#endif DEBUG	
		poly_split(obj, pobj[0], objlist, POLYSPLITMAXOBJ, &num, 0);
#ifdef DEBUG
		if (debug_channel)
			fprintf(debug_channel,"Disoverlap - polysplit %d objects\n",num);
#endif DEBUG
		/*
		 *  It is possible for poly_split to also return with num = 0
		 *  This means that the two objects extracted from the split
		 *  were too small and label ignored them.  The code
		 *
		 *  if (num == 1) {
         *  	/ * fprintf(stderr,"Decomposition failure\n"); * /
         *		freeobj(pobj[0]);
		 *		freeobj(pobj[1]);
		 *		freeobj(objlist[0]);
		 *		if (list == 1)
		 *			freeobj(outobj[0]);
		 *		return(0);
		 *
		 *  has been replaced with the following 8 lines.  The same mod
		 *  is carried out further down in this function.  SCG
		 */
		if (num == 1 || num == 0) {
        	freeobj(pobj[0]);
			freeobj(pobj[1]);
			if(num == 1)
				freeobj(objlist[0]);
			if (list == 1)
				freeobj(outobj[0]);
			return(0);
		} else if (num > 2) {
			for (i = 2; i < num; i++) {
				freeobj(objlist[i]);
			}
		}
		if (ifcross(objlist[0], pobj[1])) {
			obj1 = objlist[0];
			freeobj(objlist[1]);
		} else {
			obj1 = objlist[1];
			freeobj(objlist[0]);
		}
#ifdef DEBUG
		if (debug_channel != NULL)
			fprintf(debug_channel,"Disoverlap: before poly_split (1) (list %d)\n",list);
#endif DEBUG	
		poly_split(obj1, pobj[1], objlist, POLYSPLITMAXOBJ, &num, 0);
		freeobj(obj1);
#ifdef DEBUG
		if (debug_channel)
			fprintf(debug_channel,"Disoverlap - 2nd polysplit %d objects\n",num);
#endif DEBUG
		if (num == 1 || num == 0) {
/*			fprintf(stderr,"Decomposition failure\n");*/
        	freeobj(pobj[0]);
			freeobj(pobj[1]);
			if(num == 1)
				freeobj(objlist[0]);
			if (list == 1)
				freeobj(outobj[0]);
			return(0);
		} else if (num > 2) {
			for (i = 2; i < num; i++) {
				freeobj(objlist[i]);
			}
		}
		if (ifcross(objlist[0], pobj[0])) {
			obj1 = objlist[0];
			freeobj(objlist[1]);
		} else {
			obj1 = objlist[1];
			freeobj(objlist[0]);
		}
		outobj[list] = obj1;
		freeobj(pobj[0]);
		freeobj(pobj[1]);
	}
	(*new0) = outobj[0];
	(*new1) = outobj[1];
#ifdef DEBUG
	if (debug_channel != NULL)
		fprintf(debug_channel,"Disoverlap: before normal exit\n");
#endif DEBUG	
	return(1);
}



/*
 * if the object and the curve have conmon point return 1
 * otherwise return 0
 */
ifcross(obj, plgobj)
struct object *obj, *plgobj;
{
	struct object *pobj, *polytoobj();
	struct intervaldomain *idmn = obj->idom;
	struct intervaldomain *pdmn;
	struct polygondomain *plgd;
	struct intervalline *itvl;
	struct interval *intl;
	int i, n, j, kl, kr;

	if(plgobj->type == 10){
		plgd = (struct polygondomain *) plgobj->idom;
		pobj = polytoobj(plgd);
	} else if (plgobj->type == 1)
		pobj = plgobj;
	else {
		fprintf(stderr,"Ifcross: wrong type of object\n");
		return(0);
	}
	pdmn = pobj->idom;
	if (idmn->line1 > pdmn->lastln || idmn->lastln < pdmn->line1 || idmn->kol1 > pdmn->lastkl || idmn->lastkl < pdmn->kol1){
		if(plgobj->type == 10)
			freeobj(pobj);
		return(0);
	}
	else {
		itvl = &pdmn->intvlines[0];
		j = pdmn->kol1 - idmn->kol1;
		for(i = pdmn->line1; i <= pdmn->lastln; i++){
			intl = itvl->intvs;
			n = itvl->nintvs;
			while(n-- > 0){
				kl = intl->ileft + j;
				kr = intl->iright + j;
				/*
				if(connect4(obj, i, intl->ileft, intl->iright)){
				*/
				if(connect(obj, i, kl, kr)){
					if(plgobj->type == 10)
						freeobj(pobj);
					return(1);
				}
				intl++;
			}
			itvl++;
		}
	}
	if (plgobj->type == 10)
		freeobj(pobj);
	return(0);
}
