/*
 * fastdilate.c
 * -----------
 * Ji Liang May 21, 1987
 * 
 * fast erosion, one pixel only
 * N.B
 * when free the erosion object please do not free
 * value table because the value table is the same
 * as original one, it can not be free unless the input
 * object is free
 */

#include <stdio.h>
#include <wstruct.h>
#include <split.h>

struct object *fastdilate(obj)
struct object *obj;
{
	struct object *dilatobj;
	struct object *makemain();
	struct intervaldomain *makedomain(), *domain;
	struct intervaldomain *idmn = obj->idom;
	struct intervalline *itvl;
	struct interval *jp, *jwp;
	register i, j, k;
	int inttot, nitv;
	int line1, lastln, line;
	int kol1,lastkl;
	int left[MAXLNITV], right[MAXLNITV], l[MAXLNITV], r[MAXLNITV];
	int odd, length;


	/*
	 * only can deal with type 1 object
	 */
	if (obj->type != 1)
		return(NULL);
	kol1 = idmn->kol1;
	lastkl = idmn->lastkl;
	line1 = idmn->line1;
	lastln = idmn->lastln;	
	/*
	 * reserve space for dilation object
	 */
	i = lastln - line1 + 3;
	inttot = intcount(idmn);
	inttot += 2 * MAXLNITV;
	jp = jwp = (struct interval *) Malloc(inttot * sizeof(struct interval));
	odd = i & 01;
	domain = makedomain(1, line1-1, lastln+1, kol1-1, lastkl+1);
	dilatobj = makemain(1, domain, NULL, NULL, NULL);
	domain->freeptr = (char *) jp;
	/*
	 * initialize
	 * ----------
	 */
	kol1 = lastkl;
	lastkl = 0;
	itvl = &idmn->intvlines[0];
	if(odd == 1)
		lastln--;
	/*
	 * get first two lines because they only depend one or two lines
	 */
	line = line1 - 1;
	nitv = set_line(itvl, jp);
	makeinterval(line++, domain, nitv, jwp);
	jwp += nitv;
	jp = jwp;
	nitv = line_int_dil(itvl, itvl+1, l, r);
	for(i = 0; i < nitv; i++){
		jp->ileft = l[i];
		jp->iright = r[i];
		jp++;
	}
	makeinterval(line++, domain, nitv, jwp);
	itvl++;
	/*
	 * get the rest lines which need combine three lines
	 */
	for(line = line1 + 1; line < lastln; ){
		length = line_int_dil(itvl, itvl+1, left, right);
		for(j = -1; j <= 2; j += 3){
			jwp = jp;
			nitv = line_arr_dil(itvl+j,left,right,length,l,r);
			k = set_arr_int(l, r, nitv, jp);
			jp += k;
			makeinterval(line++, domain, k, jwp);
		}
		itvl += 2;
	}
	line = lastln;
	if(odd == 1){
		jwp = jp;
		j = line_int_dil(itvl-1, itvl, left, right);
		nitv = line_arr_dil(itvl+1,left,right,j,l,r);
		k = set_arr_int(l, r, nitv, jp);
		jp += k;
		makeinterval(line++, domain, nitv, jwp);
	}
	/*
	 * get final two lines
	 */
	if(odd == 0)
	 	itvl--;
	jwp = jp;
	nitv = line_int_dil(itvl, itvl+1, l, r);
	for(i = 0; i < nitv; i++){
		jp->ileft = l[i];
		jp->iright = r[i];
		jp++;
	}
	makeinterval(line++, domain, nitv, jwp);
	itvl++;
	jwp = jp;
	nitv = set_line(itvl, jp);
	makeinterval(line, domain, nitv, jwp);
	return(dilatobj);
}

/*
 * find the intersection intervals between two lines
 * one set of intervals is pointed out by intva
 * the other intervals is piinted out by intvb
 * length is the result number of intersection intervals
 * in array l and r
 */

line_int_dil(inta, intb, l, r)
struct intervalline *inta;
struct intervalline *intb;
int *l, *r;
{
	struct interval *intl, *jntl;
	register i, m, n, t;
	int j, flag = 0;

	/*
	 * compare one interval by one interval
	 */
	m = inta->nintvs;
	intl = inta->intvs;
	n = intb->nintvs;
	jntl = intb->intvs;
	i = 0;
	while(m != 0 && n != 0){
		t = intv_dil(intl, jntl, &l[i], &r[i]);		
		j = i - 1;
		if(flag == 1) {
			if(if_intersec(l[j], r[j], l[i], r[i])){
				r[j] = MAX(r[j], r[i]);
				i--;
			} 
		}
		if(t == -1){
			intl++;
			m--;
			flag = 0;
		} else if (t == 1){
			jntl++;
			n--;
			flag = 0;
		} else {
			intl++;
			m--;
			jntl++;
			n--;
			flag = 1;
		}
		i++;
	}
	if(n != 0){
		m = n;
		intl = jntl;
	}
	while(m-- > 0){
		j = i - 1;
		if(r[j] + 1 < intl->ileft){
			l[i] = intl->ileft;
			r[i] = intl->iright + 2;
			i++;
		} else {
			r[j] = MAX(intl->iright + 2, r[j]);
		}
		intl++;
	}
	return(i);
}

/*
 * find the intersection intervals between tow lines
 * one set of intervals is pointed out by itvl
 * the other n intervals is in array left and right
 * length is the result number of intersection intervals
 * in array l and r
 */

line_arr_dil(itvl, left, right, n, l, r)
struct intervalline *itvl;
int *left, *right, n;
int *l, *r;
{
	struct interval *intl;
	register i, j, m, t;
	int flag = 0, k;

	/*
	 * compare one interval by one interval
	 */
	intl = itvl->intvs;
	m = itvl->nintvs;
	i = 0;
	j = 0;
	while(m != 0 && j != n){
		t = arr_dil(intl, left[j], right[j], &l[i], &r[i]);
		k = i - 1;
		if(flag == 1){
			if(if_intersec(l[k], r[k], l[i], r[i])){
				r[k] = MAX(r[k], r[i]);
				i--;
			}
		}
		if(t == -1){
			intl++;
			m--;
			flag = 0;
		} else if (t == 1){
			j++;
			flag = 0;
		} else {
			intl++;
			m--;
			j++;
			flag = 1;
		}
		i++;
	}
	while(m-- > 0){
		k = i - 1;
		if(intl->ileft > r[k] + 1){
			l[i] = intl->ileft;
			r[i] = intl->iright + 2;
			i++;
		} else {
			r[k] = MAX(intl->iright + 2, r[k]);
		}
		intl++;
	}
	while(j < n){
		k = i - 1;
		if(left[j] > r[k] + 1){
			l[i] = left[j];
			r[i] = right[j];
			i++;
		} else {
			r[k] = MAX(right[j], r[k]);
		}
		j++;
	}
	return(i);
}

/*
 * find intersection of two intervals
 * return -1: inta is in far left, there is no intersection point
 * return 2: intb is in far right, there is no intersection point
 * 	  0: there is an intersection and intb at the right of inta
 * 	  1: there is an intersection and inta at the right of intb
 * N.B the result was shifted to right by one, only for save time
 */
intv_dil(inta, intb, l, r)
struct interval *inta, *intb;
int *l, *r;
{
	if(inta->iright + 2 < intb->ileft){
		*l = inta->ileft;
		*r = inta->iright + 2;
		return(-1);
	}
	if(inta->ileft > intb->iright + 2){
		*l = intb->ileft;
		*r = intb->iright + 2;
		return(1);
	}
	*l = MIN(inta->ileft, intb->ileft);
	*r = MAX(inta->iright, intb->iright) + 2;
	return(0);
}
	
/*
 * find intersection of an interval and a section(left, right)
 * return -1: inta is in far left, there is no intersection point
 * return 2: left is in far right, there is no intersection point
 * 	  0: there is an intersection and intl is at the left of "right"
 * 	  1: there is an intersection and "right" is at the left of intl
 * N.B the result was shifted to right by one, only for save time
 */
arr_dil(intl, left, right, l, r)
struct interval *intl;
int left, right;
int *l, *r;
{
	if(intl->iright + 2 < left){
		*l = intl->ileft;
		*r = intl->iright + 2;
		return(-1);
	}
	if(intl->ileft > right){
		*l = left;
		*r = right;
		return(1);
	}
	*l = MIN(intl->ileft, left);
	*r = MAX(intl->iright + 2, right);
	return(0);
}

set_line(itvl, jntl)
struct intervalline *itvl;
struct interval *jntl;
{
	struct interval *intl, *tmp, *jp = jntl;
	register i, m;

	/*
	 * compare one interval by one interval
	 */
	intl = itvl->intvs;
	m = itvl->nintvs;
	jp->ileft = intl->ileft;
	jp->iright = intl->iright + 2;
	if(m == 1)
		return(1);
	tmp = intl;
	intl++;
	m--;
	i = 0;
	while(m-- > 0){
		if(tmp->iright + 2 < intl->ileft){
			i++;
			jp++;
			jp->ileft = intl->ileft;
			jp->iright = intl->iright + 2;
		} else {
			jp->iright = intl->iright + 2;
		}
		tmp++;
		intl++;
	}
	i++;
	return(i);
}

if_intersec(left, right, l, r, n)
int left, right, l, r, n;
{
	if(left > r + n || l > right + n)
		return(0);
	else return(1);
}

set_arr_int(l, r, n, intl)
int *l, *r, n;
struct interval *intl;
{
	register i, k;
	struct interval *jp = intl;

	jp->ileft = l[0];
	jp->iright = r[0];
	k = 1;
	for(i = 1; i < n; i++){
		if(l[i] > jp->iright + 1){
			jp++;
			k++;
			jp->ileft = l[i];
			jp->iright = r[i];
		} else { 
			jp->iright = r[i];
		}
	}
	return(k);
}
