/*
 * fasteros.c
 * -----------
 * Ji Liang May 21, 1987
 * 
 * fast erosion, one pixel only
 * N.B
 * when free the erosion object please do not free
 * value table because the value table is the same
 * as original one, it can not be free unless the input
 * object is free
 * ====================
 * MODIFICATION HISTORY
 * --------------------
 *  04-09-87	jimp		Space bug fix.  Registers.
 *  10-08-87	ji		Space freeing.
 *  05-08-87	jl/jp	Malloc, Free, etc..
 *  01-08-87	Ji Liang
 * 				reserve more space for interval
 */
#include <stdio.h>
#include <wstruct.h>
#include <split.h>

#ifdef DEBUG
static FILE *debug_channel = NULL;
#endif

struct object *fasteros(obj)
struct object *obj;
{
	struct object *erosobj;
	struct object *makemain();
	struct intervaldomain *makedomain(), *domain;
	struct intervaldomain *idmn = obj->idom;
	register struct intervalline *itvl;
	register struct interval *jp;
	struct interval *jwp;
	register i, j, kol1, m, *nitv;
	int inttot, line1, lastln, line;
	int k1, lastkl;
	int left[MAXLNITV], right[MAXLNITV], l[MAXLNITV], r[MAXLNITV];
	int n, odd;
	int length, end;

	/*
	 * only can deal with type 1 object
	 */
	if (obj->type != 1)
		return(NULL);
	kol1 = idmn->kol1;
	lastkl = idmn->lastkl;
	line1 = idmn->line1;
	lastln = idmn->lastln - 1;	
	/*
	 * if the object is too thin after erosion the object will disarppear
	 */
	i = lastln - line1;
	if(i < 0 || lastkl - kol1 < 1)
		return(NULL);
	/*
	 * reserve space for erosion object
	 */
	nitv = (int *) Malloc(sizeof(int) * (i+1));
	nitv[i] = 0;
	inttot = intcount(idmn);
	inttot += 2 * abs (inttot - i);
	jp = jwp = (struct interval *) Malloc(inttot * sizeof(struct interval));
	odd = i & 01;
	/*
	 * initialize
	 * ----------
	 */
	line1++;
	k1 = kol1;
	kol1 = lastkl;
	lastkl = 0;
	itvl = &idmn->intvlines[0];
	m = 0;
	itvl++;
	if(odd == 1)
		lastln--;
	for(line = line1; line <= lastln; line += 2){
		nitv[m] = 0;
		length = line_int_int(itvl, itvl+1, left, right);
		for(j = -1; j <= 2; j += 3){
			end = line_intv_arr(itvl+j,left,right,length,l,r);
			nitv[m] = 0;
			for(i = 0; i < end; i++){
				if(r[i] - l[i] > 2){
					jp->ileft = l[i];
					jp->iright = r[i];
					kol1 = MIN(kol1, jp->ileft);
					lastkl = MAX(lastkl, jp->iright);
					jp++;
					++nitv[m];
					inttot--;
				}
			}
			m++;
		}
		itvl += 2;
	}
	if(odd == 1){
		j = line_int_int(itvl-1, itvl, left, right);
		length = line_intv_arr(itvl+1,left,right,j,l,r);
		nitv[m] = 0;
		for(i = 0; i < length; i++){
			if(r[i] - l[i] > 2){
				jp->ileft = l[i];
				jp->iright = r[i];
				kol1 = MIN(kol1, jp->ileft);
				lastkl = MAX(lastkl, jp->iright);
				jp++;
				++nitv[m];
				inttot--;
			}
		}
		if(nitv[m] > 0)
			m++;
	}
#ifdef DEBUG
/*
 * did we allocate enough space ?
 */
if (inttot < 0 && debug_channel != NULL)
fprintf(stderr,"Fasteros: interval array (jwp) overflow\n");
#endif DEBUG
	/*
	 * get first and last line
	 */
	i = m - 1;
	while(nitv[i] == 0)
		i--;
	lastln = line1 + i;
	i = 0;
	while(nitv[i] == 0)
		i++;
	line1 += i;
	if(lastln < line1){
		Free(nitv);
		Free(jwp);
		return(NULL);
	}
	domain = makedomain(1, line1, lastln, kol1+k1+1, lastkl+k1+1);
	erosobj = makemain(1, domain, NULL, NULL, NULL);
	jp = jwp;
	domain->freeptr = (char *) jp;
	k1 = kol1;
	kol1 += 2;
	for(line = line1; line <= lastln; line++){
 		jwp = jp;
		n = nitv[i];
		while (n-- > 0){
			jp->ileft -= k1;
			jp->iright -= kol1;
			jp++;
		}
		makeinterval(line, domain, nitv[i++], jwp);
	}
	Free(nitv);
	return(erosobj);
}

/*
 * find the intersection intervals between two lines
 * one set of intervals is pointed out by intva
 * the other intervals is piinted out by intvb
 * length is the result number of intersection intervals
 * in array l and r
 */

line_int_int(inta, intb, l, r)
struct intervalline *inta;
struct intervalline *intb;
int *l, *r;
{
	register struct interval *intl, *jntl;
	register i, m, n, t;

	/*
	 * compare one interval by one interval
	 */
	m = inta->nintvs;
	intl = inta->intvs;
	n = intb->nintvs;
	jntl = intb->intvs;
	i = 0;
	while(m != 0 && n != 0){
		t = intv_intv(intl, jntl, &l[i], &r[i]);		
		if(t == -1){
			intl++;
			m--;
		} else if (t == 2){
			jntl++;
			n--;
		} else {
			if(t == 0){
				intl++;
				m--;
			} else {
				jntl++;
				n--;
			}
			i++;
		}
	}
	return(i);
}

/*
 * find the intersection intervals between tow lines
 * one set of intervals is pointed out by itvl
 * the other n intervals is in array left and right
 * length is the result number of intersection intervals
 * in array l and r
 */

line_intv_arr(itvl, left, right, n, l, r)
struct intervalline *itvl;
int *left, *right, n;
int *l, *r;
{
	register struct interval *intl;
	register i, j, m, t;

	/*
	 * compare one interval by one interval
	 */
	intl = itvl->intvs;
	m = itvl->nintvs;
	i = 0;
	j = 0;
	while(m != 0 && j != n){
		t = intv_arr(intl, left[j], right[j], &l[i], &r[i]);		
		if(t == -1){
			intl++;
			m--;
		} else if (t == 2)
			j++;
		else {
			if(t == 0){
				intl++;
				m--;
			} else j++;
			i++;
		}
	}
	return(i);
}

/*
 * find intersection of two intervals
 * return -1: inta is in far left, there is no intersection point
 * return 2: intb is in far right, there is no intersection point
 * 	  0: there is an intersection and intb at the right of inta
 * 	  1: there is an intersection and inta at the right of intb
 */
intv_intv(inta, intb, l, r)
register struct interval *inta, *intb;
register int *l, *r;
{
	if(inta->iright < intb->ileft)
		return(-1);
	if(inta->ileft > intb->iright)
		return(2);
	*l = MAX(inta->ileft, intb->ileft);
	if(inta->iright < intb->iright){
		*r = inta->iright;
		return(0);
	} else {
		*r = intb->iright;
		return(1);
	}
}
	
/*
 * find intersection of an interval and a section(left, right)
 * return -1: inta is in far left, there is no intersection point
 * return 2: left is in far right, there is no intersection point
 * 	  0: there is an intersection and intl is at the left of "right"
 * 	  1: there is an intersection and "right" is at the left of intl
 */
intv_arr(intl, left, right, l, r)
register struct interval *intl;
register int *l, *r;
{
	if(intl->iright < left)
		return(-1);
	if(intl->ileft > right)
		return(2);
	*l = MAX(intl->ileft, left);
	if(intl->iright < right){
		*r = intl->iright;
		return(0);
	} else {
		*r = right;
		return(1);
	}
}
