/*
 * gapath.c
 * --------
 * Ji Liang Dec 22, 1986
 *
 * find the gap path in a touching object
 * ( a path, which can cut the touching object
 * into separete objects)
 */

#include <stdio.h>
#include <math.h>
#include <wstruct.h>
#include <split.h>
int gfltbl[24] = {
	 1,  0, -1, -1, -1, -1, -1,  0, 
	 1,  1,  0, -1, -1, -1,  0,  1,
	 1,  1,  1,  0, -1,  0,  1,  1,
} ;
int gfktbl[24] = {
	-1,  1,  1,  0, -1, -1, -1, -1,
	 0,  0,  1,  1,  0,  0, -1, -1,
	 1,  1,  1,  1,  1, -1, -1,  0,
} ;
struct polygondomain *gapath(obj, p1, p2, maxg)
struct object *obj;
struct ivertex *p1, *p2;
GREY *maxg;
{
	struct polygondomain *makepolydmn(), *plgd;
	struct ivertex *ivt, *istart;
	struct ivertex next;
	int i, n;
	GREY g, *greyval();

	i = abs(p1->vtY - p2->vtY) + 30 + abs(p1->vtX - p2->vtX);
	ivt = (struct ivertex *) Malloc(sizeof(struct ivertex) * i);
	istart = ivt;
	next.vtY = p1->vtY;
	next.vtX = p1->vtX;
	*maxg = *greyval(obj, p1->vtY, p1->vtX);
	while(next.vtY != p2->vtY || next.vtX != p2->vtX){
		ivt->vtY = next.vtY;
		ivt->vtX = next.vtX;
		if(abs(next.vtY - p2->vtY) + abs(next.vtX - p2->vtX) < 2){
			ivt++;
			break;
		}
		g = nextgvpoint(obj, p2, ivt, &next); 	
		if(*maxg < g)
			*maxg = g;
		ivt++;
	}
	ivt->vtY = next.vtY;
	ivt->vtX = next.vtX;
	if(ivt->vtY != p2->vtY || ivt->vtX != p2->vtX){
		ivt++;
		ivt->vtY = p2->vtY;
		ivt->vtX = p2->vtX;
	}
	g = *greyval(obj, p2->vtY, p2->vtX);
	*maxg = MAX(*maxg, g);
	ivt++;
	n = (int) (ivt - istart);
	plgd = makepolydmn(1, istart, n, n + 1, 1);
	Free(istart);
	return(plgd);
}



/*
 * if the end points are in p1 and p2
 * and their grey level are g1 and g2
 * optgrey() will return the optimum grey
 * level in a middle point p
 * note: give the value d (the distance between
 * p1 and p2 is only for save calculate time 
 */
GREY optgrey(p1, p2, g1, g2, p)
struct ivertex *p1, *p2, *p;
GREY g1, g2;
{
	int x1, x2, y1, y2;
	int d1, d2, d;

	x1 = p->vtX - p1->vtX;
	x2 = p->vtX - p2->vtX;
	y1 = p->vtY - p1->vtY;
	y2 = p->vtY - p2->vtY;
	d1 = hypot8(x1, y1);
	d2 = hypot8(x2, y2);
	d = d1 + d2;
	return((int) ceil((0.0 + g1 * d2 + g2 * d1) / d));
}

/*
 * the goal point is gp
 * next point should have less grey level 
 * and neraer the goal point
 */
nextgvpoint(obj, gp, p, next)
struct object *obj;
struct ivertex *gp, *p;
struct ivertex *next;
{
	struct ivertex tmp;
	int i, j, k, l;
	int l1, l2, k1, k2;
	GREY g[4], g1;
	int d, a;
	int dl, dk, flag;

	dl = abs(gp->vtY - p->vtY);
	dk = abs(gp->vtX - p->vtX);
	g1 = 256;
	if(p->vtY == gp->vtY){
		if(p->vtX > gp->vtX)
			flag = 6;
		else flag = 2;
	} else if(p->vtY > gp->vtY){
		if(p->vtX > gp->vtX)
			flag = 5;	
		else if(p->vtX == gp->vtX)
			flag = 4;
		else flag = 3;
	} else {
		if(p->vtX > gp->vtX)
			flag = 7;
		else if(p->vtX == gp->vtX)
			flag = 0;	
		else flag = 1;	
	}
	l = p->vtY;
	k = p->vtX;
	get3grey(obj, l, k, flag, g);
	i = 0;
	for(j = 0; j < 3; j++){
		if(g1 > g[j]){
			g1 = g[j];
			i = j;
		} else if(g1 == g[j]){
			l1 = gfl(i, flag, 0);
			k1 = gfl(i, flag, 1);
			l2 = gfl(j, flag, 0);
			k2 = gfl(j, flag, 1);
			if(l1 == l2){
				if(dl > dk && k2 == 0)
					i = j;
			} else if(k1 == k2){
				if(dk > dl && k2 == 0)
					i = j;
			} else {
				tmp.vtY = l + l1;
				tmp.vtX = k + k1;
				a = squardis(gp, &tmp);
				tmp.vtY = l + l2;
				tmp.vtX = k + k2;
				d = squardis(gp, &tmp);
				if(a > d)
				i = j;
			}
		}
	}
	next->vtY = l + gfl(i, flag, 0); 
	next->vtX = k + gfl(i, flag, 1);
	return(g[i]);
}


/* 
 * num = 0, return line increasement
 *	 1, return column inceasement
 */
gfl(i,j, num)
register i,j;
int num;
{
	if(num == 0)
		return(gfltbl[i * 8 + j]);
	else return(gfktbl[i * 8 + j]);
}
