/* 
 * getcurve.c
 * -----------
 * Ji Liang Aug 19, 1986
 *
 * input two sets of points
 * output the points between them using cubic
 * curve interpolation
 * if sign = 0 only use this two sets of points
 *    sign = 1 implies that there is a point, which is just
 *		in the middle of the points
 *
 * Modifications
 *
 *	22 Jan 1991		CAS		Localise debug
 *	 5 Nov 1987		CAS		Debugging
 * jimp 25-09-87	Astart still not quite right.
 * jimp	10-09-87	Debugging space.  For some reason which I have not
 *			yet found, the array "astart" needs to be a little
 *			bigger than it was.
 * jimp	13-04-87	LSIZE,KSIZE copied from sosolae.c
 */
#include <math.h>
#include <stdio.h>
#include <wstruct.h>
#include <split.h>
#define LSIZE 4
#define KSIZE 5

#ifdef DEBUG
static FILE *debug_channel = NULL;
#endif

struct polygondomain *getcurve(p1, p2, sign)
struct polygondomain *p1, *p2;
{
	/*
	 * direc = 0, search p1, p2 clockwise
	 * 	   1, search p1, p2 anticlockwise
	 * flag = 0, x coordinate is veriable
	 *      = 1, y coordinate is veriable
	 * sign = 0, there is no reference point
	 *      = 1, there is a reference point at the middle
	 */
	struct ivertex *ivt1, *ivt2;
	struct ivertex *ivt, *istart;
	struct polygondomain *plgd, *makepolydmn();
	int m, n, tmp;
	int flag, incr;
	int *a, *astart;
	register i, k, t;
	int l1, k1, l2, k2;
	int cpl, cpk;		/* check point for polynom */
	int deltx, delty;
	double *T;
	double *s, slop;

	m = p1->nvertices;
	n = p2->nvertices;
	ivt1 = p1->vtx;
	ivt2 = p2->vtx;
	l1 = ivt1->vtY;
	k1 = ivt1->vtX;
	l2 = ivt2->vtY;
	k2 = ivt2->vtX;
	deltx = abs(k1 - k2);
	delty = abs(l1 - l2);
	if(delty > deltx)
		flag = 0;
	else flag = 1;
	i = (m + n) * 2;
	if(sign == 1){
		i+=2;
		cpl = (ivt1->vtY + ivt2->vtY) >> 1;
		cpk = (ivt1->vtX + ivt2->vtX) >> 1;
	}	

	/* 
	 * put these two sets of points in order
	 */
	i += 2;		/* jimp 10-09-87 */
	a = astart = (int *) Malloc (sizeof(int) * i);
	ivt1 += m - 1;
	t = 1;
	if(flag == 0){
		*a++ = ivt1->vtY;
		*a++ = ivt1->vtX;
		ivt1--;
		for(i = 1; i < m; i++){
			if(ivt1->vtY != *(a - 2)){
				*a++ = ivt1->vtY;
				*a++ = ivt1->vtX;
				t++;
			} else {
				*(a - 1) += ivt1->vtX;
				*(a - 1) >>= 1;
			}
			ivt1--;
		}
		if(sign == 1){
			*a++ = cpl;
			*a++ = cpk;
			t++;
		}
		for(i = 0; i < n; i++){
			if(ivt2->vtY != *(a - 2)){
				*a++ = ivt2->vtY;
				*a++ = ivt2->vtX;
				t++;
			} else {
				*(a - 1) += ivt2->vtX;
				*(a - 1) >>= 1;
			}
			ivt2++;
		}
	} else {
		*a++ = ivt1->vtX;
		*a++ = ivt1->vtY;
		ivt1--;
		for(i = 1; i < m; i++){
			if(ivt1->vtX != *(a - 2)){
				*a++ = ivt1->vtX;
				*a++ = ivt1->vtY;
				t++;
			} else {
				*(a - 1) += ivt1->vtY;
				*(a - 1) >>= 1;
			}
			ivt1--;
		}
		if(sign == 1){
			*a++ = cpk;
			*a++ = cpl;
			t++;
		}
		for(i = 0; i < n; i++){
			if(ivt2->vtX != *(a - 2)){
				*a++ = ivt2->vtX;
				*a++ = ivt2->vtY;
				t++;
			} else {
				*(a - 1) += ivt2->vtY;
				*(a - 1) >>= 1;
			}
			ivt2++;
		}
	}
#ifdef DEBUG
	if (debug_channel) fprintf(debug_channel,"astart 0x%x, i %d, t %d\n",astart,i,t);
#endif DEBUG
	a = astart;

	/* 
	 * use the least squar method to find
	 * the fitting cubic curve
	 */
	s = (double *) Malloc (sizeof(double) * LSIZE * KSIZE);
	coefmatrix(astart, t, s);
	Free(astart);
	T = (double *) Malloc (sizeof(double) * LSIZE);
	if(elemeqn(s, LSIZE, 0.001,T) == 0){
		fprintf(stderr,"wrong coefficient, can't solve the equation\n");
		return(NULL);
	}
	Free(s);
	if(flag == 0)
		ivt = (struct ivertex *) Malloc (sizeof(struct ivertex) * (delty+1));
	else ivt = (struct ivertex *) Malloc (sizeof(struct ivertex) * (deltx + 1));
	istart = ivt;

	/*
	 * dicide which is the veriable
	 * and don't allow it far from nearest straight line
	 */

	ivt->vtY = l1;
	ivt->vtX = k1;
	ivt++;
	if(flag == 0){
		n = delty - 1;
		if(l2 > l1){
			incr = 1;
		} else {
			incr = -1;
		}
		k = l1 + incr;
		slop = (double) (k2 - k1);
		slop /= (double) (l2 - l1);
		if(abs(linear(slop, cpl - l1) + k1 - polynom(T, cpl)) > 5)
			tmp = 1;
		else tmp = 0;
		for(i = 0; i < n; i++){
			if(tmp == 0)
				ivt->vtX = polynom(T, k);
			else ivt->vtX = linear(slop, k - l1) + k1;
			ivt->vtY = k;
			ivt++;
			k += incr;
		}
	} else {
		n = deltx - 1;
		if(k2 > k1)
			incr = 1;
		else {
			incr = -1;
		}
		k = k1 + incr;
		slop = (double) (l2 - l1);
		slop /= (double) (k2 - k1);
		if(abs(linear(slop, cpk - k1) + l1 - polynom(T, cpk)) > 5)
			tmp = 1;
		else tmp = 0;
		for(i = 0; i < n; i++){
			if(tmp == 0)
				ivt->vtY = polynom(T, k);
			else ivt->vtY = linear(slop, k - k1) + l1;
			ivt->vtX = k;
			ivt++;
			k += incr;
		}
	}
	Free(T);
	/*
	if(flag == 0 && l2 > l1 || flag == 1 && k2 > k1){ 
		ivt->vtY = l2;
		ivt->vtX = k2;
	}
	else if(flag == 0 && l2 < l1 || flag == 1 && k2 < k1){ 
		ivt += n + 1;
		ivt->vtY = l1;
		ivt->vtX = k1;
	}
	*/
	ivt->vtY = l2;
	ivt->vtX = k2;
	ivt = istart;
	plgd = makepolydmn(1, istart, n + 2, n+3, 1);
	Free(istart);
	return(plgd);
}

/*
 * solve linear equations using elimination method
 * A: input array in line order
 * n: the order of the set of equations
 * ep: a small value, which the accuracy depends on
 * fail: 0 ok
 *	 1 fail
 * B: the output
 */

double *algeqe(S, n, ep, fail)
double *S;
int n, fail;
double ep;
{
	register int i, j, k, l;
	double *B, *bstart, P, T, *A[LSIZE], *astart;

	/*
	 * initionlize
	m = n - 1;
	 */
	A[0] = astart = (double *) Malloc (sizeof(double) * n * (n + 1));
	for(l = 0; l < n; l++){
		A[l] = astart;
		astart += n + 1;
	}
	/*
	 * find the maximum value in the column
	 * label the line number as l
	 * if the maximum value is less than ep
	 * we can't get result otherwise change
	 * the line, which the maximum value in,
	 * to the k-th line
	 */
	for(k = 0; k < n; k++){
		P = 0.0;
		for(i = k; i < n; i++){
			if(abs(A[i][k]) > abs(P)){
				P = A[i][k];
				l = i;
			} 
		}
		if(abs(P) <= ep){
			fail = 1;
			return(NULL);
		}
		if(l != k){
			for(j = k; j <= n; j++){
				T = A[k][j];
				A[k][j] = A[l][j];
				A[l][j] = T;
			}
		}
		P = 1 /P;
		if(k != n){
			for(j = k + 1; j <= n; j++){
				A[k][j] *= P;
				for(i = k + 1; i < n; i++)
					A[i][j] -= A[i][k]*A[k][j];
			}
		}

	}
	for(i = n - 2; i >= 0; i--){
		for(j = i + 1; j < n; j++)
			A[i][n] -= A[i][j] * A[j][n];
	}
	fail = 0;
	bstart = B = (double *) Malloc (sizeof(double) * n);
	for(i = 0; i < n; i++){
		*B++ = A[i][n];
	}
	Free(astart);
	Free(bstart);
	return(bstart);
}

polynom(p, x)
double *p;
int x;
{
	register k = 1, i;
	double tmp, y = 0.0;

	for(i = 0; i < 4; i++){
		tmp = k * (*p++);	
		k *= x;
		y += tmp;
	}
	return((int) ceil(y));
}

linear(s, x)
double s;
int x;
{
	return((int) ceil(s * x));
}
