

/*
 * labelling algorithm for woolz object
 * --------------------------------------
 * ji Apr 4, 86
 * divide the input object to separate objects 
 * without using pixles. it only uses interval
 * Modification history
 * ====================
 *
 *	 7 Feb 1991		CAS			remove Mizar defs -> osk
 * jimp	24-08-87	Conditionally removed some sections (#ifndef mizar)
 *  05-08-87	jl/jp	Malloc, Free, etc..
 * 03/06/87	JL	when there is only one individual object
 *			return immeduately
 *			the valuetable use vdom of input object
 *			be careful when free sub-objects, don't
 *			free vdom
 */
#include <stdio.h>
#include <ctype.h>
#include <signal.h>
#include <wstruct.h>
#include <split.h>

#ifndef OSK
struct address termadd[SIZE];

struct object *labelling(obj)
struct object *obj;
{
	struct iwspace iwsp;
	struct object *separat();
	struct address badd, add;
	struct address juncadd[HALFSIZE];	
	struct intervaldomain *idmn = obj->idom;
	int objnum;
	int line, col, i, j, l, k, m, n;
	int firstkl;
	int colr, n1, n2, c1, c2;
	int t, w;
	/*
	 * t: the number of junction points 
	 * w: the number of terminal points 
	 */

	k = 0;
	m = 0;
	firstkl = idmn->kol1;
	initrasterscan(obj, &iwsp, 0);
	while(nextinterval(&iwsp) == 0){
		line = iwsp.linpos;
		col = iwsp.lftpos - firstkl;
		colr = iwsp.rgtpos - firstkl;

		/* 
		 * find terminal points and junction points
		 * arcording to the value of i
		 * 	0: 	terminal
		 *	1:	ordinary point
		 *	default:
		 *		junction point
		 */
		i = isconnect(obj, line, col, colr, 0);
		switch(i){
			case 0:
				termadd[m].xcood = line;
				termadd[m++].ycood = col;
				break;
			case 1:
				break;
			default:
				juncadd[k].xcood = line;
				juncadd[k].ycood = col;
				juncadd[k++].order = i;
		}
	}
	t = k;		/* the number of junction points */
	w = m;		/* the number of terminal points */
	/* label:
	 * -------
	 * if two terminal points have relation through
	 * the i-th junction point then the i-th junction
	 * point and the two terminal points have the same
	 * label
	 */

	for(i = 0; i < w; i++){
		termadd[i].order = i;
	}
	for(i = 0; i < t; i++){
		m = juncadd[i].order;
		l = juncadd[i].xcood;
		k = juncadd[i].ycood;
		base(obj, l, k, 1, &add);
		for(j = 0; j < w; j++){
			if(equal(&add, &termadd[j])){
				n1 = j;
				c1 = termadd[j].order;
				break;
			}
		}
		for(n = 2; n <= m; n++){
			base(obj, l, k, n, &badd);
			for(j = 0; j < w; j++){
				if(equal(&badd, &termadd[j])){
					n2 = j;
					c2 = termadd[j].order;
					break;
				}
			}
			if(c1 < c2){
				termadd[root(n2)].order = termadd[n1].order;
				termadd[n2].order = termadd[n1].order;
			} else {
				termadd[root(n1)].order = termadd[n2].order;
				termadd[n1].order = termadd[n2].order;
				n1 = n2;
				c1 = c2;
			}
		}
	}
	j = 0;
	for(i = 0; i < w; i++){
		if(termadd[i].order == i)
			termadd[i].order = 'A' + j++;
		else{
			n1 = termadd[i].order;
			termadd[i].order = termadd[root(n1)].order;
		}
	}
	objnum = j;
	if(j == 1){
		obj->assoc = NULL;
		return(obj);
	}
	return(separat(obj, termadd, objnum));
}


/*
 * findbase:
 * -------------------
 * find the coordinate of the left end of
 * the n-th connected interval in line l - 1
 * if it is terminal return MAXCOL
 */

 findbase(obj,l, k, n)
 struct object *obj;
 register l, k, n;
 {
	struct intervaldomain *idmn = obj->idom;
	struct interval *intl;
	struct intervalline *itvl;
	register int i, kr;

	/* find the end ponts in line l */
	if(l == idmn->line1)
		return(MAXCOL);	/* it is a terminal already */
	itvl = &idmn->intvlines[l - idmn->line1];
	i = itvl->nintvs;
	intl = itvl->intvs;
	while (i-- > 0) {
		if(k == intl->ileft){
			kr = intl->iright;
			break;
		}
		intl++;
	}
	/* find th-n connected interval in line l - 1 */
	k--;
	kr++;
	itvl--;
	i = itvl->nintvs;
	intl = itvl->intvs;
		/* find the first connected interval */
	while (i-- > 0) {
		if(!(k > intl->iright || kr < intl->ileft)){
			intl +=  n - 1;
			return(intl->ileft);
		}
		intl++;
	}
	return(MAXCOL);
}

/* find base
 * -----------
 * if point (l, k) is the left end of an interval
 * find the first connected interval in line l - 1
 * then do the same thing recursively until reachs a
 * terminal point which is base(obj, l, k, n)
 * note:
 *	n == 1: uses the first connected interval
 *	   > 1: uses the i-th connected interval
 *		make sure i != 0 (it is terminal aleady!)
 */

base(obj, l, k, n, bbadd)
struct object *obj;
register int l, k, n;
struct address *bbadd;
{
	register int j;

	j = findbase(obj, l, k, n);
	if(j != MAXCOL) 
		return(base(obj, l-1, j, 1, bbadd));
	else {
		bbadd->xcood = l;
		bbadd->ycood = k;
	}
	return(1);
}


/* equel(X, Y)
 * if points X and Y have same xcood and ycood
 * then they are the sane point
 */

equal(X, Y)
struct address *X, *Y;
{
	if(X->xcood == Y->xcood && X->ycood == Y->ycood)
		return(1);
	else 	return(0);

}




root(r)
register int r;
{
	if(termadd[r].order == r || (termadd[r].order >= 'A')){
		return(r);
	}
	else root(termadd[r].order);
}

/* 
 * isconnect:
 * if an interval (kl, kr) in line l connects
 * with any interval in line l-1, return the number of
 * conneced interval(or intervals), otherwise return 0
 * note:
 *	kr actually is not the right end of an interval
 *	it is larger than it by one. it is only for
 *	speed sake.
 *	if the fifth parameter is 0, use kr as right end
 *	otherwise find right end automaticly.
 */

isconnect(obj, l, kl, kr, j)
struct object *obj;
register int l, kl, kr;
int j;
{
	struct intervaldomain *idmn = obj->idom;
	struct interval *intl;
	struct intervalline *itvl;
	register int k1, k2, i, n; 

	if (l == idmn->line1)
		return(0);
	itvl = &idmn->intvlines[l - idmn->line1];
	n = itvl->nintvs;
	intl = itvl->intvs;
	k1 = kl - 1;
	if(j == 0)
		k2 = kr  + 1;
	else {
		while (n-- > 0) {
			if(k1 == intl->ileft){
				k2 = intl->iright + 1;
				break;
			}
			intl++;
		}
	}
	/*
	 * find the number of intervals in line l-1, which connect with 
	 * the intrval in line l
	 */
	itvl--;
	n = itvl->nintvs;
	intl = itvl->intvs;
	i = 0;
	while (n-- > 0) {
		if (!(k2 < intl->ileft || k1 > intl->iright))
			i++;
		intl++;
	}
	return(i);
}
#endif OSK

/* 
 * connect:
 * if interval(k1, k2) connects with any interval
 * in the line l, return the order of the interval
 * in that line, otherwise return 0
 */
connect(obj, l, k1, k2)
struct object *obj;
register int l, k1, k2;
{
	struct intervaldomain *idmn = obj->idom;
	struct interval *intl;
	struct intervalline *itvl;
	register int n; 

	if (l < idmn->line1 || l > idmn->lastln )
		return(0);
	itvl = &idmn->intvlines[l - idmn->line1];
	n = itvl->nintvs;
	intl = itvl->intvs;
	while (n-- > 0) {
		if (!(((k2 + 1) < intl->ileft) || (k1 > (intl->iright) + 1)))
			return(itvl->nintvs - n);
		intl++;
	}
	return(0);
}

#ifndef OSK
/* labelling the whole picture */
/* separate the object into a list of subobject */

struct object *separat(obj, termadd, w)
struct object *obj;
struct address termadd[SIZE];
int w;			/* the number of obj */
{
	struct object *objlist[OBJNUM];
	struct object *makemain();
	struct intervaldomain *makedomain(), *domain;
	struct intervaldomain *idmn = obj->idom;
	struct intervalline *itvl;
	struct interval *jp[OBJNUM], *jpstart[OBJNUM], *jpw;
	struct interval *intl;
	struct iwspace iwsp;
    	int labeltbl[2][MAXLNITV];
	int *list;
	int *subnum;
	register int labelnum;
	int limit, numitv, totintl;
	int i, j, k, l, n, line, col, colr;
	int  line1[OBJNUM], lastln[OBJNUM], col1[OBJNUM], lastkl[OBJNUM];
	/*
	int *ptr, *line1, *lastln, *col1, *lastkl;
	*/
	int order = 0, firstkl;
	/* intv[i]: pointer to the intervals of the i-th subobject */
	/* numitv: how many intervals in a line */
	/* label: the order of the subobject */

	for(i = 0; i < w; i++){
		line1[i] = MAXLINE;
		col1[i] = MAXLINE;
		lastkl[i] = 0;
	}
	totintl = intcount(obj->idom);
	list = (int *) Malloc(sizeof(int) * totintl);
	subnum = (int *) Calloc(w, sizeof(int));
	j = 0;
	order = 0;
	k = 0;
	firstkl = obj->idom->kol1;
	initrasterscan(obj, &iwsp, 0);
	while(nextinterval(&iwsp) == 0){
		line = iwsp.linpos;
		col = iwsp.lftpos - firstkl;
		colr = iwsp.rgtpos - firstkl;
		order++;
		if(iwsp.nwlpos != 0){
			for(i = 0; i < order; i++)
				labeltbl[0][i] = labeltbl[1][i];
			order = 1;
		}
		if(line != termadd[j].xcood || col != termadd[j].ycood){
			l = line - 1;
			labelnum = labeltbl[0][connect(obj, l, col, colr)];	
			labeltbl[1][order] = labelnum;
		} else {
			labelnum = (termadd[j++].order - 'A');
			labeltbl[1][order] = labelnum;
			line1[labelnum] = MIN(line1[labelnum], line);
		}
		list[k++] = labelnum;
		subnum[labelnum] += 1;
		/*
		col1[labelnum] = MIN(col1[labelnum], col);
		lastkl[labelnum] = MAX(lastkl[labelnum], colr);
		*/
		col1[labelnum] = MIN(col1[labelnum], iwsp.lftpos);
		lastkl[labelnum] = MAX(lastkl[labelnum], iwsp.rgtpos);
		lastln[labelnum] = line;
	}
	totintl += w;
	for(i = 0; i < w; i++){
		jpw = (struct interval *) Malloc((subnum[i]+1) * sizeof(struct interval));
		jp[i] = jpstart[i] = jpw;
		jpw += subnum[i] + 1;
		(jpw-1)->ileft = 0;
	}
	j = 0;
	for(line = obj->idom->line1; line <= obj->idom->lastln; line++){
		itvl = &idmn->intvlines[line - idmn->line1];
		n = itvl->nintvs;
		intl = itvl->intvs;
		while(n-- > 0){
			labelnum = list[j++];
			jp[labelnum]->ileft = intl->ileft;
			jp[labelnum]->iright = intl->iright;
			jp[labelnum]++;
			intl++;
		}
	}
	for (i = 0; i < w; i++){
		domain = makedomain(1, line1[i], lastln[i], col1[i], lastkl[i]);
		objlist[i] = makemain(1, domain, NULL, NULL, NULL);
		domain->freeptr = (char *) jp[i];
		jp[i] = jpstart[i];
		col = col1[i] - firstkl;
		for(line = 0; line <= lastln[i] - line1[i]; line++){
 			jpw = jp[i];
			numitv = 0;
			do{
				limit = jp[i]->iright;
				jp[i]->ileft -= col;
				jp[i]->iright -= col;
				jp[i]++;
				numitv++;
			} while (limit < jp[i]->ileft - 1);
			makeinterval(line + line1[i], domain, numitv, jpw);
		}
	}
	for(i = 0; i < w - 1; i++){
		objlist[i]->assoc = objlist[i + 1];
		objlist[i]->vdom = obj->vdom;
	}
	objlist[w-1]->vdom = obj->vdom;
	Free(list);
	Free(subnum);
	return(objlist[0]);
}
#endif OSK
