/*
 * mendhole.c
 * ----------
 * WARNING - OS9 COMPILER BUG - SEE COMMENT 31-08-87 BELOW
 * ----------
 * Ji Liang June 17, 1987
 *
 * mend the holes cause by wrong threshold
 * Modification history
 * ==========================
 *
 *	22 Jan 1991		CAS		Localise debug
 *	22 Nov 1988		dcb		Added param. to woolz_check.. to indicate where called from
 *	16 Nov 1988		SCG		check validity of object in mendhole
 *	12 Nov 1987		SCG		In creatholeobj malloc grabbed insufficient
 *							memory for array in certain cases.  Fixed
 *	30 Oct 1987		CAS		Debug
 *  03 Sep 1987	  jimp@IRS	MAX_HOLES used.
 *  31 Aug 1987		jimp	Debugging - note that "k" in "creatholeobj" MUST NOT
 *							be a register, else divide-by-zero error.
 *							Reduce calls to functions "abs".
 *  28 Aug 1987		jimp	Domain freeptr set in createhole().  Registers.
 *  25 Aug 1987		jimp	If no hole mended, return NULL.  More freeing.
 *							Mobj->vdom->linkcount=1 (may be redundant).
 *  19 Aug 1987		ji		More space for array in case the hole kiles very
 *							narrow rectangle.
 *   5 Aug 1987		jl/jp	Malloc, Free, etc..
 *	14 Jul 1987	  Ji Liang 	Change float to int
 *	29 Jun 1987   Ji Liang	Deleted the restriction on grey balance.
 *							Mend small holes only.
 */
#include <stdio.h>
#include <math.h>
#include <wstruct.h>
#include <split.h>

#ifdef DEBUG
static FILE *debug_channel = NULL;
#endif

struct object *
mendhole(obj, blist, thresh)
struct object *obj;
struct boundlist *blist;
int thresh;
{
	struct object *objlist[MAX_HOLES], *unionn();
	struct object *mobj, *ibound();
	struct object *creatholeobj();
	struct polygondomain *plgd[MAX_HOLES];
	register int i, j;
	int hnum;

	/*
	 * check that incoming object is valid
	 */
	if(woolz_check_obj(obj, "mendhole") != 0)
		return(NULL);
		
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Mendhole: entry\n");
#endif DEBUG	
	if(getholelist(blist->down, plgd, &hnum) == 0)
		return(NULL);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Mendhole: after getholelist, hnum %d\n",hnum);
#endif DEBUG	
	j = 0;
	for(i = 0; i < hnum; i++){
		objlist[j] = creatholeobj(obj, plgd[i]);
		if(area(objlist[j]) < thresh)
			j++;
		else
			freeobj(objlist[j]);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Mendhole: after creatholeobj, j %d\n",j);
#endif DEBUG	
	}
	if(j == 0)
		return(NULL);
	objlist[j] = obj;
	mobj = unionn(objlist, j + 1, 1);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Mendhole: after unionn\n");
#endif DEBUG	
	for(i = 0; i < j; i++)
		freeobj(objlist[i]);
#ifdef DEBUG
	if (debug_channel != NULL)
		fprintf(debug_channel,"Mendhole: mobj->vdom->linkcount was %d\n",mobj->vdom->linkcount);
#endif DEBUG	
	mobj->vdom->linkcount = 1;
	return(mobj);
}

struct object *creatholeobj(obj, plgd)
struct object *obj;
struct polygondomain *plgd;
{
	struct interval *jp, *jpw, *makeinterval();
	struct intervaldomain *domain, *makedomain();
	struct valuetable *vtb, *newvaluetb();
	struct object *mobj, *makemain();
	register struct ivertex *tmp, *wtx = plgd->vtx;
	int line1, kol1, lastln, lastkl;
	register int i, j = 0, n, w, *array;
	int f, g, linecomp(), k, kabs, siz;
	GREY *greyval();

	n = plgd->nvertices - 1;
	g = 0;
	/*
	 * Insufficient memory is grabbed here for long thin holes in
	 * the y direction.  The amount needed is the sum of the absolute
	 * differences in the y vertices.  If adjacent vertices are equal
	 * add one to the sum.  Thus upper bound = (sum + nvertices)
	 * This code
	 *
	 * j = (n << 2);
	 * array = (int *) Malloc(sizeof(int) * (j + 100));
	 *
	 * has been replaced by the following 8 lines.  SCG
	 */
	siz = 0;
	tmp = wtx;
	for(i = 0; i < n; i++) {
		siz += abs(tmp->vtY - (tmp+1)->vtY);
		tmp++;
	}
	siz += plgd->nvertices;
	array = (int *) Malloc(sizeof(int) * (2 * siz));
	
	j = 0;
	tmp = wtx;
	array[j++] = wtx->vtY;
	array[j++] = wtx->vtX;
	wtx++;
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Creatholeobj: after Malloc\n");
#endif DEBUG	
	for(i = 1; i < n; i++){
		kabs = k = wtx->vtY - tmp->vtY;
		if (kabs < 0)
			kabs = -kabs;
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Creatholeobj: for-loop: k %d\n");
#endif DEBUG	
		if(kabs < 2){
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Creatholeobj: abs(k) < 2\n");
#endif DEBUG	
			array[j++] = wtx->vtY;
			array[j++] = wtx->vtX;
		} else {
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Creatholeobj: abs(k) >=2 (%d)\n",k);
#endif DEBUG	
			/*
			 * note: FTOI = 256
			 * >>= 8 equals to divide 256
			 */
			f = (wtx->vtX - tmp->vtX) * FTOI / k;
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Creatholeobj: after divide\n");
#endif DEBUG	
			for(w = 1; w < kabs; w++){
				f *= w;
				f >>= 8;
				if(k > 0){
					array[j++] = tmp->vtY + w; 
					array[j++] = tmp->vtX + f;
				} else {
					array[j++] = tmp->vtY - w; 
					array[j++] = tmp->vtX - f;
				}
			}
			array[j++] = wtx->vtY;
			array[j++] = wtx->vtX;
		}
		g += *greyval(obj, wtx->vtY, wtx->vtX);
		tmp++;
		wtx++;
	}
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Creatholeobj: after first for-loop, g %d n %d\n",g,n);
#endif DEBUG	
	g /= n;
	w = sizeof(int);
	w <<= 1;
	n = (j >> 1);
	qsort(array, n, w, linecomp);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Creatholeobj: after qsort\n");
#endif DEBUG	
	line1 = array[0];
	lastln = array[j - 2];
	w = lastln - line1 + 1;
	jp = jpw = (struct interval *) Malloc(sizeof(struct interval) * w);
	i = 0;
	lastkl = 0;
	kol1 = 1024;
	for(j = line1; j <= lastln; j++){
		jp->ileft = array[i + 1];
		jp->iright = array[i + 1];
		while(j == array[i]){
			jp->ileft = MIN(array[i + 1], jp->ileft);
			jp->iright = MAX(array[i + 1], jp->iright);
			i += 2;
		}
		kol1 = MIN(kol1, jp->ileft);
		lastkl = MAX(lastkl, jp->iright);
		jp++;
	}
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Creatholeobj: after second for-loop\n");
#endif DEBUG	
	domain = makedomain(1, line1, lastln, kol1, lastkl);
	domain->freeptr = (char *) jpw;
	mobj = makemain(1, domain, NULL, NULL, NULL); 
	jp = jpw;
	w += line1;
	for(i = line1; i < w; i++){
		jpw = jp;
		jp->ileft -= kol1;
		jp->iright -= kol1;
		makeinterval(i, domain, 1, jpw);
		jp++;
	}
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Creatholeobj: after second for-loop\n");
#endif DEBUG	
	vtb =  newvaluetb(mobj, 1, obj->vdom->bckgrnd);
	mobj->vdom = vtb;
	setval(mobj, g);
	Free(array);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Creatholeobj: exit\n");
#endif DEBUG	
	return(mobj);
}

linecomp(i, j)
register int *i, *j;
{
	return(*i - *j);
}

#ifdef DEBUG
seearray(array, n)
int *array, n;
{
	register i;
	if (debug_channel != NULL) {
		for(i = 0; i < n; i ++)
			fprintf(debug_channel,"array[%d] = %d\t", i, array[i]);
		fprintf(debug_channel,"\n");
	}
}
#endif DEBUG
