/*
 * nearpoint.c 
 * ________________
 *
 * Ji Liang Oct 4, 1986
 * 
 * find cut points in a concative area
 *
 * Modifications
 *
 *	22 Jan 1991		CAS		Localise debug
 *	 2 Nov 1987		CAS		Bughunting - Ensure deep always returns with
 *							sensible contents
 *  11-08-87		jl		Free more space.
 */

#include <stdio.h>
#include <wstruct.h>
#include <split.h>
#define WIND 10

#ifdef DEBUG
static FILE *debug_channel = NULL;
#endif

/*
 * nearpoint()
 * ---------------
 * find deepest concave point using the central point of the shape
 * i.e calulate the distance from the points, which
 * are pointed by ptr, to the central point and find the minimum one
 */
struct address *nearpoint(cl, ck, plg)
int cl, ck;
struct polygondomain *plg;
 {
	int i, j, a, n, h;
	register int l, k;
	struct address *deep, *dstart, *tmp, *deeppoint();
	struct ivertex *ivt = plg->vtx, ctl;
	int htmp[2], sameh = 1;
	ctl.vtY = cl;
	ctl.vtX = ck;
	deep = dstart = (struct address *) Malloc (sizeof(struct address) * 2);
	n = plg->nvertices;
/*
 *	I don't understand why 64000 is used here since true order lengths of
 *	over 300K are possible - I think this is just a badly chosen fudge
 *	factor - But I may be wrong..
 */
	deep[0].order = 64000;
	deep[1].order = 64000;
/*
 *	Ensure sensible values in case we get a really strange object
 */
	deep[0].xcood = ivt->vtX;
	deep[0].ycood = ivt->vtY;
	deep[1].xcood = (ivt+n-1)->vtX;
	deep[1].ycood = (ivt+n-1)->vtY;
#ifdef DEBUG
	if (debug_channel) fprintf(debug_channel,"nearpoint: nvert %d, cl %d, ck %d\n",n,cl,ck);
#endif DEBUG
	if (n < 5) {
		return(dstart);
	}
	/*
	htmp[1] = htmp[0] = 0;
	*/
	htmp[0] = squardisi(&ctl, ivt++);
	htmp[1] = squardisi(&ctl, ivt++);
	for(i = 0; i < n - 2; i++){
		h = squardisi(&ctl, ivt);
		if(h == htmp[1])
			sameh++;
		else {
			if(htmp[1] < h && htmp[1] < htmp[0]){
				j = bigdistance(deep);
				if(htmp[1] < deep[j].order){
					if(j == 0) {
						ACOPY(deep[0],deep[1]);
					}
					deep[1].order = htmp[1];
					sameh = (sameh + 1) >> 1;
					deep[1].xcood = (ivt-sameh)->vtX;
					deep[1].ycood = (ivt-sameh)->vtY;
				}
			}
			htmp[0] = htmp[1];
			htmp[1] = h;
			sameh = 1;
		}
		ivt++;
	}
	/*
	 * if this mathod failed try
	 * use the second method i.e.
	 * find the two deepest point,which were mesureaed from
	 * the chord, then calculate how far they are from
	 * the central point
	 */
	if(deep[0].order == 64000 && deep[1].order == 64000){
		ivt = plg->vtx;
		l = ivt->vtY;
		k = ivt->vtX;
		ivt += n - 1;
		tmp = deeppoint(l, k, ivt->vtY, ivt->vtX, plg);
		for(i = 0; i < 2; i++){
			h = squardandi(&tmp[i], &ctl);
			if(h <= 0)
				h = BIGVALUE;
			deep[i].xcood = tmp[i].xcood;
			deep[i].ycood = tmp[i].ycood;
			deep[i].order = h;
		}
		Free(tmp);	
	}
	if(abs(deep[0].xcood - deep[1].xcood) < 5 && abs(deep[0].ycood - deep[1].ycood) < 5){
		if(deep[0].order <= deep[1].order)
			deep[1].order = 64000;
		else deep[0].order = 64000;
	}
	if(deep[0].order > deep[1].order){
		a = deep[1].order << 4;
		if(deep[0].order > a){
			deep[0].order = 64000;
		}
	}
	if(deep[1].order > deep[0].order){
		a = deep[0].order << 4;
		if(deep[1].order > a)
			deep[1].order = 64000;
	}
	return(dstart);
}

/*
 * bigdistance()
 *--------
 * find the order of the larger deepest points
 */
bigdistance(deep)
struct address *deep;
{
	if(deep[0].order > deep[1].order)
		return(0);
	else return(1);
}
