/*
 * objwidth.c
 * ----------
 * Jule 25, 1987 	Ji Liang
 *
 * get width of object by erosion
 * calculate after how many times
 * erosion the object will vanish
 *
 * Modification history
 * ====================
 *	22 Jan 1991		CAS		Localise debug
 *	24 Nov 1988		SCG/dcb	in goodseparate the call to poly_split is modified
 *							to take the length of the array - something bad 
 *							could have been happening here before if more than
 *							two objects returned.
 *	 3 Sep 1987		jimp	Debugging
 *  14 Aug 1987		ji		more careful about vdom.
 *	 5 Aug 1987		jl/jp	More space free-ed
 */
#include <stdio.h>
#include <wstruct.h>
#include <split.h>

#ifdef DEBUG
static FILE *debug_channel = NULL;
#endif

objwidth(obj)
struct object *obj;
{
	struct object *fasteros();
	struct valuetable *vdmn = obj->vdom;
	struct object *eobj[15];
	register i, j;

#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Objwidth: entry\n");
#endif DEBUG
	obj->vdom = NULL;
	eobj[0] = fasteros(obj);
	j = 0;
	for(i = 0; i < 15; i++){
		if(eobj[i] == NULL)
			break;
		eobj[i+1] = fasteros(eobj[i]);
		j++;
	}
	for(i = 0; i <= j; i++)
		freeobj(eobj[i]);
	obj->vdom = vdmn;
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Objwidth: exit (%d)\n",j);
#endif DEBUG
	return(j);
}

/*
 * if splitting curve plgd can separate
 * the object obj and get much thiner
 * objects, it is a good separation
 * if the length of the curve is less
 * than length it is not worth to try
 * return 1: the path is good
 *	  0: the path is not good
 *	 -1: the method is useless
 */

goodseparate(obj, plgd, length)
struct object *obj;
struct polygondomain *plgd;
int length;
{
	struct object *new[2], *polyobj, *makemain();
	struct valuetable *vdmn = obj->vdom;
	int i, j, k, n;

#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Goodseparate: entry (length %d)\n");
#endif DEBUG
	n = polylength(plgd);
	if(n < length) {
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Goodseparate: return -1 (n<length)\n");
#endif DEBUG
		return(-1);
	}
	obj->vdom = NULL;
	k = objwidth(obj);
	if(k < 5){
		obj->vdom = vdmn;
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Goodseparate: return -1 (k<5)\n");
#endif DEBUG
		return(-1);
	}
	polyobj = makemain(10, plgd, NULL, NULL, NULL);
	/*
	 * pass the length of the array new
	 */
	poly_split(obj, polyobj, new, 2, &j, 0);
	Free(polyobj);
	if(j == 1){
		obj->vdom = vdmn;
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Goodseparate: return 0 (j==1)\n");
#endif DEBUG
		return(0);
	}
	i = objwidth(new[0]);
	j = objwidth(new[1]);
	for(n = 0; n < 2; n++){
		new[n]->vdom = NULL;
		freeobj(new[n]);
	}
	obj->vdom = vdmn;
	if(i < k && j < k) {
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Goodseparate: return 1\n");
#endif DEBUG
	}
		return(1);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Goodseparate: return 0\n");
#endif DEBUG
	return(0);
}

polylength(plgd)
struct polygondomain *plgd;
{
	struct ivertex *wtx = plgd->vtx;
	int i, d, n = plgd->nvertices;

	d = 0;
	for(i = 0; i < n - 1; i++){
		d += absdis(wtx+1, wtx);
		wtx++;
	}
	return(d);
}
