/*
 * ovortouch.c
 * -----------
 * Ji Liang Sept 3, 1986
 *
 * decide if the object is a simple object
 * or an overlaped or touching object
 *
 * Modifications
 *
 *	22 Jan 1991		CAS		Localise debug
 *	22 Nov 1988		dcb		Added param. to woolz_check.. to indicate where
 *							called from
 *	16 Nov 1988		SCG		check validity of incoming object in ovortouch
 *	 3 Nov 1987		CAS		General tidy up - make some variable names
 *							meaningfull
 *	25-09-87		jimp	Handle error return from searchcent.  Make various
 *							workspaces local rather than malloc-ed.
 *	04-09-87		jimp	Debugging
 *	11-08-87		jl		More space freeing
 *							Choose four chords more careful by using rejectone()
 *							in case the central point is slightly wrong.
 *	13-04-87		jp		Structure assignments removed
 */
#include <stdio.h>
#include <ctype.h>
#include <wstruct.h>
#include <split.h>

#ifdef DEBUG
static FILE *debug_channel = NULL;
#endif

struct polygondomain *
ovortouch(obj)
struct object *obj;
{
	struct object *conobj;
	struct address ctl;
	struct object *disoverlap(), *distouching(); 
	struct object *convhull();
	struct polygondomain *ptr, *trace(), *makepolydmn(), *plgd;
	register struct ivertex *wtx, *ivt, *istart;
	struct address *m, *nearpoint(), central[2], dadd1[5], dadd2[5];
	struct address *dist[2]; 
	int i, nvert, n, k, j, l, q, num;
	int tmp, min, loop = 1;
	int l1, k1, l2, k2, deltl, deltk;
	int lab[5], dd[5], d, a[5];
	int thres, dimens, conf, ratio;

#ifdef DEBUG
	if (debug_channel != NULL)
		fprintf(debug_channel,"Ovortouch: entry\n");
#endif DEBUG

	/*
	 * check whether incoming object is valid
	 */
	if(woolz_check_obj(obj, "ovortouch") != 0)
		return(NULL);
		
	i = obj->idom->lastln - obj->idom->line1;
	j = obj->idom->lastkl - obj->idom->kol1;
	i >>= 1;
	j >>= 1;

	/*
	 * the 'scale' of the input object
	 */

	dimens = i * i + j * j;

	/* 
	 * the central point of the 'shape'
	 */
	conf = searchcent(obj, central);
	if (conf == -1)
		return(NULL);
	ctl.xcood = central[0].xcood;
	ctl.ycood = central[0].ycood;
	ctl.order = central[0].order;
#ifdef DEBUG
	if (debug_channel != NULL)
		fprintf(debug_channel,"Ovortouch: after searchcent\n");
#endif DEBUG

	/*
	 * the LOOP is actually seldom used
	 * it is used only if a wrong central point is chosen
	 */
	dist[0] = dadd1;
	dist[1] = dadd2;
	conobj = convhull(obj);
#ifdef DEBUG
	if (debug_channel != NULL)
		fprintf(debug_channel,"Ovortouch: after convhull\n");
#endif DEBUG
	plgd = (struct polygondomain *) conobj->idom;
LOOP:
#ifdef DEBUG
	if (debug_channel != NULL)
		fprintf(debug_channel,"Ovortouch: LOOP\n");
#endif DEBUG
	for(l = 0; l < 2; l++)
		for(i = 0; i < 5; i++)
			dist[l][i].order = BIGVALUE;
	for(i = 0; i < 5; i++){
		dd[i] = 0;
		a[i] = 0;
	}
	nvert = plgd->nvertices;
	wtx = plgd->vtx;
	/*
	nvert = conobj->idom->nvertices;
	wtx = conobj->idom->vtx;
	*/

	/*
	 * search the first five biggest chords in the convex hull 
	 */

	for(i = 0; i < nvert - 1; i ++){
		l1 = wtx->vtY;
		k1 = wtx->vtX;
		wtx++;
		l2 = wtx->vtY;
		k2 = wtx->vtX;
		deltl = l2 - l1;
		deltk = k2 - k1;
		d = deltl * deltl + deltk * deltk;
		tmp = dd[0];

		/* 
		 * find the smallest one in the four chords
		 */

		k = smallest(&dd[0], 5, 6);
#ifdef DEBUG
		if (debug_channel != NULL)
			fprintf(debug_channel,"Ovortouch: after smallest (LOOP)\n");
#endif DEBUG

		/*
		 * if the new one bigger than the smallest  
		 * use new one instead the old and keep
		 * the original order unchange
		 */

		if(dd[k] < d){
			for(j = k + 1; j < 5; j++){
				dd[j - 1] = dd[j];
				lab[j - 1] = lab[j];
			}
			dd[4] = d;
			lab[4] = i;
		}
	}				
	/*
	 * use the five chords to find possible cutting points
	 */
	wtx = plgd->vtx;
	d = 0;
	for(i = 0; i < nvert - 1; i ++){
		l1 = wtx->vtY;
		k1 = wtx->vtX;
		wtx++;
		if(i == lab[d]){
			l2 = wtx->vtY;
			k2 = wtx->vtX;
			ptr = trace(obj, l1, k1, l2, k2);
		  	m = nearpoint(ctl.ycood,ctl.xcood,ptr);
			ACOPY(dist[0][d],m[0]);
			ACOPY(dist[1][d],m[1]);
			if(m[0].order < dimens && m[1].order < dimens){
				a[d] = subarea(obj,m[0].ycood,m[0].xcood,m[1].ycood,m[1].xcood);
			}
			Free(m);
			Free(ptr);
			d++;
		}
	}
#ifdef DEBUG
	if (debug_channel != NULL)
		fprintf(debug_channel,"Ovortouch: after for loop\n");
#endif DEBUG
	/*
	 * chose one of the smallest of the bigger chords as condidate
	 * j: the smallest chord
	 * k: the smallest but one
	 */
	j = smallest(&dd[0], 5, 6);
	k = smallest(&dd[0], 5, j);
	if(MIN(dist[0][j].order, dist[1][j].order) < MIN(dist[0][k].order,dist[1][k].order))
		j = k;
	/*
	 * choose the final four chords
	 */
	k = j;
	j = rejectone(dist[0], dist[1], 5, &ratio);
#ifdef DEBUG
	if (debug_channel != NULL)
		fprintf(debug_channel,"Ovortouch: after rejectone\n");
#endif DEBUG
	if(j != k){
		if(ratio < 10)
			j = k;
	}
	if(j < 4){
		for(i = j; i < 4; i ++) {
			ACOPY(dist[0][i],dist[0][i + 1]);
			ACOPY(dist[1][i],dist[1][i + 1]);
			dd[i] = dd[i + 1];
			a[i] = a[i + 1];
		}
	}
	k = 0;
	q = 0;
	d = 0;
	/*
	 * find threshould to pick up strange point
	 * thres = 12 * minmun distance
	 */

	thres = BIGVALUE;
	for(i = 0; i < 2; i++)
		for(j = 0; j < 4; j++)
			thres = MIN(thres, dist[i][j].order);
	thres *= 12;
	/*
	 * q: don't care the q-th concative area
	 */
	for(i = 0; i < 4; i++){
		if(dist[0][i].order < thres && dist[1][i].order < thres){
			d++;
			l = i;
		}
		if(dist[0][i].order < thres || dist[1][i].order < thres)
			k++;
		else q = i;
	}
	if(k == 3 && d == 0){
		/*
		 * mark the first two biggest ditance, since it is
		 * touching object
		 */
		q = biggest(&dd[0], 4, 6);
		l = biggest(&dd[0], 4, q);
		k = 5;
	}
#ifdef DEBUG
	if (debug_channel != NULL)
		fprintf(debug_channel,"Ovortouch: before switch\n");
#endif DEBUG
	switch(k){
		/*
		 * case 5 came from k = 3 && d == 0 i.e.
		 * there were only three points to be found
		 */
		case 5: 
			for(i = 0; i < 4; i++){
				if(dist[0][i].order > dist[1][i].order) {
					ACOPY(dist[0][i],dist[1][i]);
				}
			} 
			if(conf == 0){
				if(q == 0) {
					ACOPY(dist[0][1],dist[0][l]);
				}
				else if(l == 0) {
					ACOPY(dist[0][1],dist[0][q]);
				}
				else{
					ACOPY(dist[0][0],dist[0][q]);
					ACOPY(dist[0][1],dist[0][l]);
				}
				ACOPY(dist[0][2],dist[0][0]);
				num = 2;
			} else {
				ctl.xcood += central[1].xcood;
				ctl.ycood += central[1].ycood;
				ctl.xcood >>= 1;
				ctl.ycood >>= 1;
				for(i = 0; i < 4; i++){
					dist[1][i].order = squardis(&ctl, &dist[0][i]);
					thres = MIN(thres, dist[1][i].order);
				}
				thres <<= 3;
				num = 4;
				for(i = 0; i < 4; i++){
					if(thres < dist[1][i].order)
						num = 2;
				}
			}
			break;
		case 4:
			for(i = 0; i < 4; i++){
				if(dist[0][i].order > dist[1][i].order) {
					ACOPY(dist[0][i],dist[1][i]);
				}
			}
			num = 4;
			break;
		case 3:
			if(d > 1){
				n = 0;
				for(i = 0; i < 4; i++){
					if(a[i] != 0 && a[i] > n){
						n = a[i];
						l = i;
					}
				}
			} else if(d == 1 && a[l] > 0)
				;
			else {
				num = 2;
				l = 0;
				for(i = 0; i < 4; i++){
					if(dist[0][i].order > dist[1][i].order) {
						ACOPY(dist[0][i],dist[1][i]);
					}
				}
				tmp = 0;
				for(i = 0; i < 4; i++){
					if(i != q){
						if(dist[0][i].order > tmp){
							tmp = dist[0][i].order;
							l = i;
						}
					}
				}
				j = 0;
				for(i = 0; i < 4; i++){
					if(i != q && i != l){
						ACOPY(dist[0][j],dist[0][i]);
						j++;
					}
				}
				ACOPY(dist[0][2],dist[0][0]);
				break;
			}
			if(l > q){
				for(i = 0; i < 4; i++){
					if(i > q && i < l){
						if(dist[0][i].order > dist[1][i].order) {
							ACOPY(dist[0][i-1],dist[1][i]);
						}
						else ACOPY(dist[0][i-1],dist[0][i]);
					} else if(i == l){
						ACOPY(dist[0][i-1],dist[0][i]);
						ACOPY(dist[0][i],dist[1][i]);
					} else if(i < q || i > l) {
						if(dist[0][i].order > dist[1][i].order) {
							ACOPY(dist[0][i],dist[1][i]);
						}
					}
				}
			} else {
				for(i = 3; i >= 0; i--){
					if(i < l || i > q){
						if(dist[0][i].order > dist[1][i].order) {
							ACOPY(dist[0][i],dist[1][i]);
						}
					} else if(i == l){
						ACOPY(dist[0][i+1],dist[1][i]);
					} else if(i > l && i < q){
						if(dist[0][i].order < dist[1][i].order) {
							ACOPY(dist[0][i+1],dist[0][i]);
						}
						else ACOPY(dist[0][i+1],dist[1][i]);	
					}
				}
			}
			num = 4;
			break;		
		case 2:
			j = 0;
			for(i = 0; i < 4; i++){
				if(dist[0][i].order < dimens || dist[1][i].order < dimens){
					if(dist[0][i].order > dist[1][i].order) {
						ACOPY(dist[0][j],dist[1][i]);
					}
					else ACOPY(dist[0][j],dist[0][i]);
					j++;
				}
			}
			ACOPY(dist[0][2],dist[0][0]);
			num = 2;
			break;
		default:
			fprintf(stderr,"can't deal with this object\n");
			freeobj(conobj);
			return(NULL);
	}
#ifdef DEBUG
	if (debug_channel != NULL)
		fprintf(debug_channel,"Ovortouch: after switch\n");
#endif DEBUG
	ACOPY(dist[0][4],dist[0][0]);
	num++;
	/*
	 * check in case a wrong central point was chosen
	 * (there is a hole in the object
	 */
	if(loop == 0){
		tmp = dist[0][0].order;
		min = tmp;
		for(i = 1; i < num; i++){
			tmp = MAX(tmp, dist[0][i].order);
			min = MIN(min, dist[0][i].order);
		}
		min <<= 3;
		if(min < tmp){
			ctl.xcood = central[1].xcood;
			ctl.ycood = central[1].ycood;
			ctl.order = central[1].order;
			loop = 1;
			goto LOOP;
		}
	}
	istart = ivt = (struct ivertex *) Malloc (sizeof(struct ivertex) * num);
	for(i = 0; i < num; i++){
		ivt->vtY = dist[0][i].ycood;
		ivt->vtX = dist[0][i].xcood;
		ivt++;
	}
	freeobj(conobj);
	plgd = makepolydmn(1, istart, num, 5, 1);
	Free(istart);
#ifdef DEBUG
	if (debug_channel != NULL)
		fprintf(debug_channel,"Ovortouch: return\n");
#endif DEBUG
	return(plgd);
}

/*
 * find the smallest element except x in
 * the n elements array 
 */
smallest(ptr, n, x)
int *ptr;
int n, x;
{
	register i, j, tmp;

	if(x != 0){
		tmp = ptr[0];
		j = 0;
	} else{
		tmp = ptr[1];
		j = 1;
	}
	for(i = 0; i < n; i++){
		if(tmp > ptr[i] && i != x){
			tmp = ptr[i];
			j = i;
		}
	}
	return(j);
}

/*
 *	B I G G E S T   --
 *
 * find the biggest element except x in
 * the n elements array 
 */
biggest(ptr, n, x)
int *ptr;
int n, x;
{
	register i, j, tmp;

	if(x != 0){
		tmp = ptr[0];
		j = 0;
	} else{
		tmp = ptr[1];
		j = 1;
	}
	for(i = 0; i < n; i++){
		if(tmp < ptr[i] && i != x){
			tmp = ptr[i];
			j = i;
		}
	}
	return(j);
}

/*
 *	R E J E C T O N E  --
 *
 * reject one chord from five candidates
 * in list ptr0 and ptr1
 * ratio is the max / min 
 */
rejectone(ptr0, ptr1, num, ratio) 
struct address *ptr0, *ptr1;
int num, *ratio;
{
	register i, v;
	int j = 0, max, min;
	
	min = MIN(ptr0[0].order, ptr1[0].order);
	max = min;
	for(i = 1; i < num; i++){
		v = MIN(ptr0[i].order, ptr1[i].order);
		if(max < v){
			max = v;
			j = i;
		}
		if(min > v) 
			min = v;
	}
	if(min != 0)
		*ratio = max / min;
	else *ratio = 100;
	return(j);
}
			
