/*
 * pathsearch.c
 * ------------
 * Ji Liang Feb 9, 1987
 *
 * find lightest path around a cut point
 * obj is the input object
 * bobj is the boundary of the object
 * p is the point we are looking for
 *
 * Modifications
 *
 *	 9 Feb 1991		CAS		Localise debug
 *  22-01-91	ih		Change a return to return(0) in non-void function
 *  22-01-88	jimp	Further change to logic round "nextfewpixels"
 *  19-01-88	jimp	Get the choice between straight ahead and sideways
 *			search right in "nextlightest".
 *			Range = 5.
 *			Only permit direction changes of +-45 degrees by
 *			function constraindirec().
 *  25-09-87	jimp	Check for search path array overflow.
 *  21-09-87	jimp	Bug-fixes, comments and code re-organisation.
 *  18-09-87	jimp	The mod of 11-09-87 causes changed behaviour,
 *			including infinite looping in "pathsearch" for no
 *			obvious reason.  Took out the "goto LOOP", at least
 *			temporarily.
 *  11-09-87	jimp	Nextfewpixels did something horrible to grey-tables
 *			(planted new values by a pointer confusion).
 *  10-09-87	jimp	Debugging.
 *  27-08-87	jimp	Registers.
 *  26-08-87	jimp	Bug-fixes in "nextlightest".
 *  05-08-87	jl/jp	Malloc, Free, etc..
 *  05-08-87	jimp
 *		Removed unnecessary "extern struct ipoint point"
 *  04-08-87	Ji Liang
 *		a) Searching range likes a cross, which along the original
 *		   directoin and the perpadicular direction. It is specially
 *		   for banded chromosome.
 *		b) Added function nextfewpixel() to do it.
 *		   Searching range looks like cross or arrow.
 *  16-06-87	Ji Liang
 *   	a) Modify the confirm direction procedure
 *		b) Reduce the search range from 7 to 5 to save time
 *		c) used more check to prevent across boundary
 *		d) simplified the check procedure
 *  05-06-87	Ji Liang
 *   	If the path is across the boundary return (0) ( fail )
 *   	otherwise return 1
 *  27-04-87	Ji Liang
 *   	Function direcsearch() has a bug when the angle between the 
 *   	boundary and the horizotal line is 45 degree or 135 degree and 
 *   	the search direction is 1 or 7 ( 3 or 5) and the path happened 
 *   	to be only 6 pixels (caused by without using nextlighest()).
 *   	Added function getnearend() to solve it.
 *  23-04-87 	Ji Liang
 *	   The function nextlightest() had a bug when search direction was 1 
 *	   or 7 and the boundary has a right angle at stop point (pathsearch 
 *	   stops since it reachs the boundary according 8 neighbour, but the 
 *	   path could split the object only when 4 neighbour was concerned.) 
 *	   Added function neighb4out() to solve the problem.
 */
 
#include <stdio.h>
#include <math.h>
#include <wstruct.h>
#include <split.h>

#ifdef DEBUG
static FILE *debug_channel = NULL;
#endif

int linc[8] = { 1, 1, 0, -1, -1, -1, 0, 1 };
int kinc[8] = { 0, 1, 1, 1, 0, -1, -1, -1 };

/*
 * Search from a start point for a low density path across object.
 * Start point is sec[1], surrounded by boundary neighbours sec[0] and sec[2].
 * The path end-point is returned in "p", the path itself in "plgd",
 * and the path mean grey-value in "g".
 * Return non-zero for success, zero for failure.
 */
pathsearch(obj, sec, p, plgd, g)
struct object *obj;
struct ivertex *sec;
struct ivertex *p;
struct polygondomain **plgd;
GREY *g;
{
	register struct intervaldomain *idmn = obj->idom;
	struct polygondomain *makepolydmn();
	struct ivertex ptr, next;
	register struct ivertex *wtx, *wtop;
	struct ivertex *wstart;
	register int direc, i, j, check = 0;
	int num = 0, range;
	int loopcount;
	GREY sigma = 0;
	GREY *greyval();

	/*
	 * decide search area
	 * direc = 0: up
 	 *	   1: up right
 	 *	   2: right
 	 *	   3: down right
 	 *	   4: down
	 *	   5: down left
	 *	   6: left
 	 *	   7: up left
	 */
#ifdef DEBUG
	if (debug_channel != NULL) {
		fprintf(debug_channel,"Pathsearch: entry\n");
		fflush(debug_channel);
	}
#endif DEBUG
	expandpoint(sec, &ptr, 3);	/* 2 */
#ifdef DEBUG
	if (debug_channel != NULL) {
		fprintf(debug_channel,"Pathsearch: after expandpoint\n");
		fflush(debug_channel);
	}
#endif DEBUG
	if(inside(obj, ptr.vtY, ptr.vtX) == 0)
		return(0);
	i = MAX(idmn->lastln - idmn->line1, idmn->lastkl - idmn->kol1);
	i += 20;	/* provide extra space for paths along chromosome */
	wstart = wtx = (struct ivertex *) Malloc(sizeof(struct ivertex) * i);
	wtop = wtx + i - 2;	/* -2 prevents need for trivial overflow check */
	/* first point on split path */
	wtx->vtY = sec[1].vtY;
	wtx->vtX = sec[1].vtX;
	wtx++;
	/* second point on split path */
	wtx->vtY = ptr.vtY;
	wtx->vtX = ptr.vtX;
	wtx++;
	direc = angpoint(sec);
#ifdef DEBUG
	if (debug_channel != NULL) {
		fprintf(debug_channel,"Pathsearch: after angpoint\n");
		fflush(debug_channel);
	}
#endif DEBUG
	/*
	 * confirm the direction or slightly change it.
	 * The loop says: check the "T"-shaped search space.  If not
	 * all points lie in the object, advance into the object in
	 * the spacified direction until condition satisfied.
	 * Obviously this can go wrong if direction is inappropriate
	 * or object is too narrow or has a noisy boundary.
	 * So prevent infinite looping by failing when search point
	 * first exits from object.
	 */
	range = 5;
	while((j = nextlightest(obj, &ptr, direc, range, &next)) == 0){
		ptr.vtY += linc[direc];
		ptr.vtX += kinc[direc];
		if (!inside(obj,ptr.vtY,ptr.vtX)) {
#ifdef DEBUG
		if (debug_channel != NULL)
			fprintf(debug_channel,"Pathsearch: break out of apparent infinite loop\n");
#endif DEBUG
			Free(wstart);
			return(0);
		}
		wtx->vtY = ptr.vtY;
		wtx->vtX = ptr.vtX;
		if (++wtx > wtop) {
			Free(wstart);
			return(0);
		}
	}
	num++;
	sigma += j - 1;
	ptr.vtY = next.vtY;
	ptr.vtX = next.vtX; 
	/*
	 * Check if we should alter direction.
	 * Force a limitation on permitted
	 * direction change to +-45 degrees
	 */
	j = whichdirec(&ptr, &sec[1]);
	if(j != direc){
		direc = constraindirec(direc,j);
		j = nextlightest(obj, &ptr, direc, range, &next);
		wtx--;
		ptr.vtY = next.vtY;
		ptr.vtX = next.vtX;
	}
	wtx->vtY = ptr.vtY;
	wtx->vtX = ptr.vtX;
	wtx++;
#ifdef DEBUG
	if (debug_channel != NULL) {
		fprintf(debug_channel,"Pathsearch: before LOOP\n");
		fflush(debug_channel);
	}
#endif DEBUG
	/*
	 * This outer loop is intended to allow a second go if the
	 * end point of the path turns out to be very close to the
	 * start point (due presumably to direction changes en route),
	 * in which case the start point is perturbed and we try
	 * again.  There is a problem here: (i) infinite looping was
	 * possible, so now only one further try is allowed, (ii) as
	 * written, we don't go back to the initial direction when
	 * trying the perturbation.
	 */
	for (loopcount=0; loopcount<=1; loopcount++) {
		/*
		 * loop, finding a sequence of lowest neighbours
		 * in the strange inverted-"T" search neighbourhood.
		 */
		while((i = nextlightest(obj, &ptr, direc, range, &next)) != 0){
#ifdef DEBUG
			if (debug_channel != NULL) {
				fprintf(debug_channel,"Pathsearch: while loop (i %d)\n",i);
				fflush(debug_channel);
			}
#endif DEBUG
			ptr.vtY = next.vtY;
			ptr.vtX = next.vtX;
			wtx->vtY = ptr.vtY;
			wtx->vtX = ptr.vtX;
			if (++wtx > wtop) {
				Free(wstart);
				return(0);
			}
			if(j < i)
				j = i;
			num++;
			sigma += i - 1;
			if(check == 1){
				/*
				 * direction can be changed more flexibly
				 */
				if((int)(wtx-wstart)%3 == 0){
					direc = constraindirec(direc,whichdirec(wtx-1, wtx-4));
				}
			} else {
				/* 
				 * it can save time 
		        	 */
				if((int)(wtx-wstart) > 6){
					range = 3;
					check = 1;
				}
			}
#ifdef DEBUG
			if (debug_channel != NULL) {
				fprintf(debug_channel,"Pathsearch: end of (back round) while loop\n");
				fflush(debug_channel);
			}
#endif DEBUG
		}
#ifdef DEBUG
		if (debug_channel != NULL) {
			fprintf(debug_channel,"Pathsearch: out of while loop\n");
			fflush(debug_channel);
		}
#endif DEBUG
		p->vtY = next.vtY;
		p->vtX = next.vtX;
		ptr.vtY = next.vtY;
		ptr.vtX = next.vtX;
		/*
		 * Check that path end-point not too close to start-point,
		 * otherwise perturb and loop.
		 */
		if(absdis(&sec[1], p) >= 5)
			break;	/* from the "loopcount" for-loop */
		else {
			ptr.vtY += linc[direc];
			ptr.vtX += kinc[direc];
		}
	}
	/*
	 * make safe for both 4-neighbour and 8-neighbour boundary points
	 */
#ifdef DEBUG
	if (debug_channel != NULL) {
		fprintf(debug_channel,"Pathsearch: before neighb4out\n");
		fflush(debug_channel);
	}
#endif DEBUG
	if(neighb4out(obj,ptr.vtY,ptr.vtX) == 0){
		if(getnearend(obj, ptr.vtY, ptr.vtX, &next) == 1){
			ptr.vtY = next.vtY;
			ptr.vtX = next.vtX;
		}
	}
	wtx->vtY = ptr.vtY;
	wtx->vtX = ptr.vtX;
	wtx++;
	*g = sigma / num;	/* ji */
	i = (int) ( wtx - wstart);
	*plgd = makepolydmn(1, wstart, i, i+2, 1);
	Free(wstart);
#ifdef DEBUG
	if (debug_channel != NULL) {
		fprintf(debug_channel,"Pathsearch: return %d\n",j-1);
		fflush(debug_channel);
	}
#endif DEBUG
	return(j - 1);
}

/*
 *	N E X T L I G H T E S T  --
 *
 */
nextlightest(obj, p, direc, range, next)
struct object *obj;
struct ivertex *p;
int direc, range;
struct ivertex *next;
{
	register struct intervaldomain *idmn = obj->idom;
	struct valuetable *vdmn = obj->vdom;
	struct intervalline *itvl;
	struct valueline *vlln;
	register struct interval *intl;
	struct ivertex tmp;
	register int i, line, kol, k1;
	int ming=255, incl, inck;
	int g, j, flag, nitv, m;

#ifdef DEBUG
	if (debug_channel != NULL) {
		fprintf(debug_channel,"Nextlightest: entry (direc %d range %d)\n",direc,range);
		fflush(debug_channel);
	}
#endif DEBUG

	/* 
	 * next point is out of object ?
	 */
	k1 = idmn->kol1;
	if(neighb8out(obj, p->vtY, p->vtX) == 1){
#ifdef DEBUG
	if (debug_channel != NULL) {
		fprintf(debug_channel,"Nextlightest: return(0) after neighb8out\n");
		fflush(debug_channel);
	}
#endif DEBUG
		return(0);
	}
#ifdef DEBUG
	if (debug_channel != NULL) {
		fprintf(debug_channel,"Nextlightest: after neighb8out\n");
		fflush(debug_channel);
	}
#endif DEBUG
	/*
	 * Advance center of the bar of the inverted-"T" search space
	 * in the specified direction.
	 */
	line = p->vtY + linc[direc];
	kol = p->vtX + kinc[direc] - k1;
	tmp.vtY = line;
	tmp.vtX = kol + k1;
	/*
	 * Compute the scan parameters for searching the bar of the
	 * inverted-"T".  If the specified search direction is vertical
	 * then the bar is horizontal and scanned from left-to-right.
	 * In all other cases the bar is scanned from bottom to top,
	 * with appropriate horizontal direction for the diagonal cases.
	 */
	j = (range >> 1);
	if(direc == 0 || direc == 4)
		incl = 0;
	else {
		incl = 1;
		line -= j;
	}
	if(direc == 1 || direc == 5){
		inck = -1;
		kol += j;
	} else if(direc == 2 || direc == 6)
		inck = 0;
	else {
		inck = 1;
		kol -= j;
	}
	/*
	 * Search the bar of the inverted-"T"
	 */
	for(i = 0; i < range; i++){
		if(line < idmn->line1 || line > idmn->lastln || kol < 0 || kol > idmn->lastkl - k1){
#ifdef DEBUG
		if (debug_channel != NULL) {
			fprintf(debug_channel,"Nextlightest: return(0) from loop (out of bounds)\n");
			fflush(debug_channel);
		}
#endif DEBUG
			return(0);
		}
		vlln = vdmn->vtblines[line - vdmn->line1];
		itvl = &idmn->intvlines[line - idmn->line1];
		nitv = itvl->nintvs;
		intl = itvl->intvs;
		m = 0;
		while(nitv-- > 0){	/* find interval containing (line,kol) */
			if(kol <= intl->iright && kol >= intl->ileft){
				m = 1;
				break;
			}
			intl++;
		}
		if(m == 1){	/* (line,kol) inside object */
			g = *(vlln->values + kol + k1 - vlln->vkol1);
			if(ming > g){
				ming = g;
				tmp .vtX = kol + k1;
				tmp.vtY = line;
				flag = i;
			} else if(ming == g){
				if(abs(flag - j) > abs(i - j)){
					tmp .vtX = kol + k1;
					tmp.vtY = line;
					flag = i;
				}
			}
		} else {	/* (line,kol) not in object - back up */
			if( i > 0 && i < range-1){
				next->vtY = line - incl;
				next->vtX = kol - inck + k1;
#ifdef DEBUG
				if (debug_channel != NULL) {
					fprintf(debug_channel,"Nextlightest: return(0) from loop (back up)\n");
					fflush(debug_channel);
				}
#endif DEBUG
				return(0);
			}
		}
		line += incl;
		kol += inck;
	}
#ifdef DEBUG
	if (debug_channel != NULL) {
		fprintf(debug_channel,"Nextlightest: before nextfewpixels\n");
		fflush(debug_channel);
	}
#endif DEBUG
	/*
	 * Now search the stalk of the inverted-"T".
	 * Bias in favour of stalk pixels by first weighting "ming".
	ming += 12;
	 */
	if(nextfewpixel(obj, p, direc, next, &g)){
		if(g < ming){
			tmp.vtY = next->vtY;
			tmp.vtX = next->vtX;
			ming = g;
		}
	}
	next->vtY = tmp.vtY;
	next->vtX = tmp.vtX;
	/*
	 * in case a grey-value of zero was found,
	 * return one plus minimum grey value to
	 * distinguish from the zero failure return
	 */
	return(ming + 1);
}

/*
 *	W A N G P O I N T  --
 *
 */
wangpoint(p)
register struct ivertex *p;
{
	register int d, q1, q2;

	q1 = getquadrant(&p[1], &p[0]);
	q2 = getquadrant(&p[1], &p[2]);
	d = q1 - q2;
	if(d < 0)
		d += 8;
	if(d > 1)
		q1 += 3;
	else q1 += 4;
	if(q1 > 7)
		q1 -= 8;
	return(q1);
}

/*
 *	G E T Q U A D R A N T  --
 *
 */
getquadrant(p0, p1)
register struct ivertex *p0, *p1;
{
	register int x, y, f;

	y = abs(p1->vtY - p0->vtY);
	x = abs(p1->vtX - p0->vtX);
	if(x < y)
		f = 0;
	else f = 1;
	if(p1->vtY > p0->vtY){
		if(p1->vtX > p0->vtX){
			if(f == 0)
				return(0);
			return(1);
		} else if(p1->vtX == p0->vtX)
			return(0);
		else {
			if(f == 0)
				return(7);
			return(6);
		}
	} else if(p1->vtY < p0->vtY){
		if(p1->vtX > p0->vtX){
			if(f == 0)
				return(3);
			return(2);
		} else if(p1->vtX == p0->vtX)
			return(4);
		else {
			if(f == 0)
				return(4);
			return(5);
		}
	} else {
		if(p1->vtX > p0->vtX)
			return(2);
		else return(6);
	}
}

/*
 *	R E V E R D I V I D P  ---
 *
 * find the point which can divid the input angle
 * if draw a line through the point and the vertex point
 * and is in the opposite direction
 */
reverdividp(ang, p)
register struct ivertex *ang, *p;
{
	register int i, j;
	int x[2], y[2];
	double k, d[2];

	j = 0;
	for(i = 0; i < 2; i++){
		x[i] = ang[1].vtX - ang[j].vtX;
		y[i] = ang[1].vtY - ang[j].vtY;
		d[i] = hypot8(x[i], y[i]);
		x[i] <<= 4;
		y[i] <<= 4;
		j += 2;
	}
/*
 *	Now make both x[0] + x[1] 'vectors' the same length
 */
	k = d[0] / d[1];
	x[1] = ceil(x[1] * k);
	y[1] = ceil(y[1] * k);
/*
 *	Reference to orig origin
 */
	for(i = 0; i < 2; i++){
		x[i] += ang[1].vtX;
		y[i] += ang[1].vtY;
	}
/*
 *	Return midpoint
 */
	p->vtY = (y[0] + y[1]) >> 1;
	p->vtX = (x[0] + x[1]) >> 1;
}

/*
 *	A N G P O I N T   ---
 *
 * from an angle get the direction, which pointed by the angle
 */
angpoint(ang)
struct ivertex *ang;
{
	struct ivertex p;

	reverdividp(ang, &p);
	return(whichdirec(&p, &ang[1]));
}


/*
 *	W H I C H D I R E C T I O N  --
 *
 * find the direction from p1 to p0
 */
whichdirec(p0, p1)
register struct ivertex *p0, *p1;
{
	register int x, y, h, d;
	int k, l, v;

	x = abs(p0->vtX - p1->vtX);
	y = abs(p0->vtY - p1->vtY);
	l = y << 1;
	k = x << 1;
	v = l + l + y;
	h = k + k + x;
	if(h <= l){
		if(p0->vtY > p1->vtY)
			d = 0;
		else d = 4;
	} else if(v <= k){
		if(p0->vtX > p1->vtX)
			d = 2;
		else d = 6;
	} else {
		if(p0->vtX > p1->vtX){
			if(p0->vtY > p1->vtY)
				d = 1;
			else d = 3;
		} else {
			if(p0->vtY > p1->vtY)
				d = 7;
			else d = 5;
		}
	}
	return(d);
}


/*
 *	D I R E C S E A R C H  --
 *
 * search the brightest path from a inside boundary ( hole )
 * to the outside boundary
 * free search start at point ref in the hole boundary
 * along the direction direc until outside boundary
 * is reached again
 */
struct polygondomain *direcsearch(obj, hp, ref, p, g)
struct object *obj;
struct ivertex *hp, *ref;
register struct ivertex *p;
GREY *g;
{
	register struct intervaldomain *idmn = obj->idom;
	struct polygondomain *plgd, *makepolydmn();
	register struct ivertex *ptr;
	struct ivertex next, *pstart;
	int direc, j, k;
	register int i, num = 0, sigma=0;
	GREY *greyval();

	i = MAX(idmn->lastln - idmn->line1, idmn->lastkl - idmn->kol1);
	pstart = ptr = (struct ivertex *) Malloc(sizeof(struct ivertex) * i);
	ptr->vtY = hp->vtY;
	ptr->vtX = hp->vtX;
	ptr++;
	ptr->vtY = ref->vtY;
	ptr->vtX = ref->vtX;
	num = 2;
	p->vtY = ref->vtY;
	p->vtX = ref->vtX;
	sigma = *greyval(obj,hp->vtY,hp->vtX) + *greyval(obj,ref->vtY,ref->vtX);
	direc = whichdirec(ref, hp);
	if(nextlightest(obj, ptr, direc, 5, &next) == 0){
		ptr->vtY = next.vtY;
		ptr->vtX = next.vtX;
	} else {
		while((i = nextlightest(obj, ptr, direc, 3, &next)) != 0){
			ptr++;
			ptr->vtY = next.vtY;
			ptr->vtX = next.vtX;
			num++;
			sigma += i - 1;
		}
	}
	if(num != 2){
		ptr++;
		ptr->vtY = next.vtY;
		ptr->vtX = next.vtX;
		p->vtY = next.vtY;
		p->vtX = next.vtX;
	} 
	if(neighb4out(obj,ptr->vtY,ptr->vtX) == 0){
		if(getnearend(obj, ptr->vtY, ptr->vtX, &next) == 1){
			ptr->vtY = next.vtY;
			ptr->vtX = next.vtX;
		}
	}
	ptr++;
	*g = sigma / num;	/* ji */
	i = (int) (ptr - pstart);
	plgd = makepolydmn(1, pstart, i, i+2, 1);
	Free(pstart);
	return(plgd);
}

/*
 *	N E I G H B 4 O U T  --
 *
 * check if one of the 4 neighbounrs is out of the obj
 * if it is, return 1 otherwise return 0
 */
neighb4out(obj, l, k)
struct object *obj;
register int l;
{
	register i, line, kol;

	line = l - 1;
	kol = k - 1;
	for(i = 0; i < 2; i++){
		if(inside(obj, line, k) == 0)
			return(1);
		line += 2;
	}
	for(i = 0; i < 2; i++){
		if(inside(obj, l, kol) == 0)
			return(1);
		kol += 2;
	}
	return(0);
}

/*
 *	G E T N E A R E N D  --
 *
 */
getnearend(obj, l, k, p)
struct object *obj;
register int l, k;
register struct ivertex *p;
{
	struct intervaldomain *idmn = obj->idom;
	register struct interval *intl;
	register struct intervalline *itvl;
	int i, j, d = BIGVALUE, n;
	register int kol, k1;

	k1 = idmn->kol1;
	if(l == idmn->line1 || l == idmn->lastln || k == k1 || k == idmn->lastkl){
		p->vtY = l;
		p->vtX = k;
		return(0);
	}
	itvl = &idmn->intvlines[l - idmn->line1 - 1];
	for(i = 0; i < 3; i++){
		n = itvl->nintvs;
		intl = itvl->intvs;
		kol = k - k1;
		while (n-- > 0) {
			if(kol >= intl->ileft && kol <= intl->iright){
				if(kol - intl->ileft <= intl->iright - kol){
					j = kol - intl->ileft;
					if(d > j){
						d = j;
						p->vtY = l - 1 + i;
						p->vtX = intl->ileft + k1;
					}
				} else {
					j = intl->iright - kol;
					if(d > j){
						d = j;
						p->vtY = l - 1 + i;
						p->vtX = intl->iright + k1;
					}
				}
			}
			intl++;
		}
		itvl++;
	}
	if(d == BIGVALUE)
		return(0);
	return(1);
}

/*
 *	N U M N E I G H B 4  --
 *
 */
numneighb4(obj, l, k)
struct object *obj;
int l, k;
{
	register i, line, kol;
	register int j = 0;

	line = l - 1;
	kol = k - 1;
	for(i = 0; i < 2; i++){
		if(inside(obj, line, k) == 1)
			j++;
		line += 2;
	}
	for(i = 0; i < 2; i++){
		if(inside(obj, l, kol) == 1)
			j++;
		kol += 2;
	}
	return(j);
}

/*
 *	N E I G H B 8 O U T --
 *
 * check if one of the 8 neighbounrs is out of the obj
 * if it is, return 1 otherwise return 0
 */
neighb8out(obj, l, k)
struct object *obj;
int l, k;
{
	register struct intervaldomain *idmn = obj->idom;
	register struct intervalline *itvl;
	register struct interval *intl;
	register i, j, line, kol;
	int n;

	if(l<=idmn->line1 || l>=idmn->lastln||k<=idmn->kol1 || k>=idmn->lastkl)
		return(1);
	itvl = &idmn->intvlines[l - idmn->line1 - 1];
	j = 0;
	for(i = 0; i < 3; i++){
		n = itvl->nintvs;
		intl = itvl->intvs;
		kol = k - idmn->kol1;
		while (n-- > 0) {
			if(kol < intl->iright && kol > intl->ileft){
				j++;
				break;
			}
			intl++;
		}
		if(j == 0)
			return(1);
		else j = 0;
		itvl++;
	}
	return(0);
}

/*
 *	N E X T F E W P I X E L  --
 *
 */
nextfewpixel(obj, p, direc, next, g)
struct object *obj;
register struct ivertex *p;
int direc;
register struct ivertex *next;
int *g;
{
	register int i, line, kol, gp;
	GREY *greyval();
	int incl, inck;

	incl = linc[direc];
	inck = kinc[direc];
	line = p->vtY + incl;
	kol = p->vtX + inck;
	*g = 256;
	for(i = 0; i < 2; i++){
		line += incl;
		kol += inck;
		if(inside(obj, line, kol)) {
			gp = *greyval(obj, line, kol);	
			if(*g > gp){
				*g = gp;
				next->vtY = line;
				next->vtX = kol;
			}
		}
	}
	if (*g < 256)
		return(1);
	else
		return(0);
}


/*
 * constraindirec - take an old and potential new direction,
 * return a direction within +-45 degrees of the old
 */
constraindirec(oldd,newd)
register int oldd,newd;
{
	register int difd = newd - oldd;
	if (difd < -3)
		difd += 8;
	if (difd == 4)
		return(oldd);	/* 180 degree turns prohibited */
	if (difd > 3)
		difd -= 8;
	if (difd > 0)
		difd = 1;
	else if (difd < 0)
		difd = -1;
	newd = oldd + difd;
	if (newd < 0)
		newd += 8;
	else if (newd > 7)
		newd -= 8;
	return(newd);
}
