/*
 *	P O L Y _ S P L I T . C  --
 *
 * July 22, 1987	Ji Liang
 *
 * This is a replacement for polysplit in woolz. It should have been put there
 * but for some reason its creator thought it would be better here...
 * The main difference is this one (poly_split) has an option to attach the
 * pixels under the split polygon to the resulting split objects.
 * flag = 0 => work like polysplit, flag = 1 => attach the bits
 *
 *	Modifications
 *
 *	21 Jan 1991		CAS			Localise debug + switch off
 *	15 Feb 1989		CAS/SEAJ	Accept status back from label and fail if label
 *								didn't have enough space (cleanly)
 *	24 Nov 1988		SCG/dcb		pass to poly_split the length of array objlist,
 *								thus allowing poly_split to create that many objects
 *	20 Jul 1988		 SEAJ		Added in freeobj(pobj) so that if striptouch returns null
 *								objects are freed.
 *	23 Sep 1987		jimp		Use intersection of the polytoobj object and input object
 *	09 Sep 1987		jimp		Debugging.
 *	03 Sep 1987		jimp@IRS	Debugging.
 *	02 Sep 1987		jimp@IRS	Free-ing glitch fix in striptouch loop.
 *								#ifdef MIZAR removed since we have not been
 *								consistent elsewhere.a
 *	24 Aug 1987		jimp		#ifdef OS9 replaced by #ifdef MIZAR,
 *								Vdom problems sorted out by detaching vdom at
 *								start and re-attaching at end.
 *								Case tmp==NULL, remobj!=NULL dealt with.
 *	 5 Aug 1987		jl/jimp		More space free-ed
 */
#include <stdio.h>
#include <wstruct.h>
#include <split.h>
#define SIZETHRESH	3
#define fewfacts	0

static FILE *debug_channel = NULL;

/*
 *	P O L Y _ S P L I T _ D E B U G
 *
 */
poly_split_debug(chan)
FILE	*chan;
{
	debug_channel = chan;
}

/*
 *	P O L Y S P L I T
 *
 * use a polygon to split a type 1 object
 * suppress objects whose length or width is less than SIZETHRESH
 */
poly_split (obj, poly, objlist, objlistlen, numobj, flag)
struct object *obj, *poly, **objlist;
register int *numobj;
int flag, objlistlen;
{
	struct object *polytoobj(), *diffdom(), *striptouch(), *union2();
	struct object *pobj, *dobj, *remobj, *tobj, *tmp, *uobj, *intersect2();
	struct valuetable *vdmn;
	register int i;
	int		stat;
	
#ifdef DEBUG
	if (debug_channel != NULL)
		fprintf(debug_channel,"Poly_split: entry\n");
#endif DEBUG
	vdmn = obj->vdom;
	obj->vdom = NULL;
	/*
	 * convert polygon to object
	 */
	pobj = polytoobj(poly->idom);
	/*
	 * subtract from input object
	 */
	dobj = diffdom(obj,pobj);
	/*
	 * segment, ignoring small objects
	 */
	if (label(dobj, numobj, objlist, objlistlen, SIZETHRESH) == 0) {
#ifdef DEBUG
		if (debug_channel != NULL)
			fprintf(debug_channel,"Poly_split: label failed (%d objects)\n",*numobj);
#endif DEBUG
		for (i=0; i < *numobj; i++) {
			freeobj(objlist[i]);
			objlist[i] = NULL;
		}
		*numobj = 0;
		stat = 0;
	} else {
#ifdef DEBUG
		if (debug_channel != NULL)
			fprintf(debug_channel,"Poly_split: after label (%d objects)\n",*numobj);
#endif DEBUG
		if (flag == 1) {
			tmp = intersect2(obj, pobj);
			freeobj(pobj);
			pobj = tmp;
			for (i = 0; i < *numobj; i++) {
				/*
				 * striptouch can return the following conditions:
				 * tmp=NULL, remobj=pobj (assignment, not copy!)
				 * pobj!=tmp!=NULL, remobj!=pobj
				 * tmp=pobj (assignment, not copy), remobj=NULL
				 */
#ifdef DEBUG
				if (debug_channel != NULL){
					fprintf(debug_channel,"Poly_split: before striptouch (i %d)\n",i);
					fflush(debug_channel);
				}
#endif DEBUG
				tmp = striptouch(objlist[i], pobj, &remobj);
#ifdef DEBUG
				if (debug_channel != NULL){
					fprintf(debug_channel,"Poly_split: after striptouch  tmp = 0x%x\n", tmp);
					fflush(debug_channel);
				}
#endif DEBUG
				if(tmp != NULL) {
					uobj = union2(objlist[i], tmp);
#ifdef DEBUG
					if (debug_channel != NULL)
						fprintf(debug_channel,"Poly_split: before freeobj(objlist[i])\n");
#endif DEBUG
					freeobj(objlist[i]);
#ifdef DEBUG
					if (debug_channel != NULL)
						fprintf(debug_channel,"Poly_split: before freeobj(tmp)\n");
#endif DEBUG
					freeobj(tmp);
					objlist[i] = uobj;
				} else {
					if (remobj == pobj && pobj != NULL) {
#ifdef DEBUG
					if (debug_channel != NULL)
						fprintf(debug_channel,"Poly_split: before freeobj(pobj)\n");
#endif DEBUG
						freeobj(pobj);
						remobj = pobj = NULL;
					}
				}
				if (remobj == NULL)
					break;	/* out of the for loop */
				else {
					if (pobj != remobj && pobj != tmp) {
#ifdef DEBUG
						if (debug_channel != NULL)
							fprintf(debug_channel,"Poly_split: before freeobj(pobj) (loop)\n");
#endif DEBUG
						freeobj(pobj);
					}
					pobj = remobj;
				}
			}
		}
		stat = 1;
	}
	/*
	 * restore value table, link to new objects.
	 */
	obj->vdom = vdmn;
	for (i=0; i < *numobj; i++) {
		objlist[i]->vdom = obj->vdom;
		if (obj->vdom)
			obj->vdom->linkcount++;
		objlist[i]->assoc = obj;
	}
	if (flag == 0) {
#ifdef DEBUG
		if (debug_channel != NULL)
			fprintf(debug_channel,"Poly_split: before freeobj(pobj) (exit)\n");
#endif DEBUG
		freeobj(pobj);
	}
#ifdef DEBUG
	if (debug_channel != NULL)
		fprintf(debug_channel,"Poly_split: before freeobj(dobj)\n");
#endif DEBUG
	freeobj(dobj);
	return(stat);
}
