/*
 *      S P L I T C H E C K . C  --  Filter to check split libary
 *
 *
 *  Written: Graham John Page
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    22nd Oct 1987
 *
 *  Modifications
 *
 *	16 Nov 1987	SCG	Make max objects returned by split_comp
 *				compatible with #define in split.h
 */
#include <stdio.h>
#include <ddgs.h>
#include <wstruct.h>
#include <split.h>

#define OVERLAY1 1

FILE *debug_channel = NULL;

/*
 *	I N T E R R U P T --  Signal catcher
 *
 */
interrupt(sig)
int		sig;
{
}

static int overlaps = 0;
static int splits = 0;
static int debug = 0;

main(argc,argv)
int	argc;
char	**argv;
{
	char	*swptr, **targv,*param;
	int	targc, doneone;
	int 	store = FS_VME512, devno = 0;
	FILE	*infile;
	struct 	pframe f;
	DDGS 	*dg,*ddgsopen();

#ifdef INTERRUPT
	intercept(interrupt);
#endif INTERRUPT
	setmemalloc(48*1024);
	targc = argc; targv = argv;
	argc--; argv++;			/* dump process name */
	while (argc > 0) {
		if (*argv[0] == '-') {
			swptr = *argv;
			*argv = NULL;
			while (*(++swptr)) {
				switch (*swptr) {
				  case '?':
				  	printf("Syntax:   split [Boptions> params <options>\n");
				  	printf("Function: \n");
				  	printf("Options:  v - Show version number\n");
					printf("          d - Debug option\n");
					printf("          o - Undo overlaps\n");
					printf("          s - Undo splits\n");
					printf("          V - Use Virtuoso framestore (Default CRS 512)\n");
					printf("          1 - Use seat 1 framestore\n");
					printf("          2 - Use seat 2 framestore\n");
					printf("          3 - Use seat 3 framestore\n");
					printf("          4 - Use seat 4 framestore\n");
														 printf("          ? - Help\n");
				  	exit(0);
				  case 'v':
					printf("Split - Version 0.1.2\n");
					exit(0);
				 case 'V':
				 	store = FS_VIRTUOSO_INVERT;
				 	break;
				 case '1':
				 	devno = 0;
				 	break;
				 case '2':
				 	devno = 1;
				 	break;
				 case '3':
				 	devno = 2;
				 	break;
				 case '4':
				 	devno = 3;
				 	break;
				 case 'o':
				 	if (splits==1){
				 		fprintf(stderr,"Only split or overlap can be selected split assumed\n");
					} else {
						overlaps = 1;
					}
				 	break;
				 case 's':
				 	if (overlaps==1){
				 		fprintf(stderr,"Only overlap or split can be selected overlap assumed\n");
					} else {
						splits = 1;
					}
					break;
				  case 'd':
					param = *++argv;
					*argv = NULL; argc--;
					debug = 1;
					break;
				  default:
				 	printf("Split - unknown option\n");
				}
			}
		}
		argv++; argc--;
	}

	argc = targc; argv = targv;
	argc--; argv++;			/* dump process name */
	doneone = 0;
	
/*
 *	Set up picture frames and frame store parameters
 */
	f.type = 60;
	f.ox = f.oy = 0;
 	f.ix = 1;
 	f.iy = 1;
 	f.scale = 8;
 	dg = ddgsopen(store,devno);
 	dinit();
 	llut();
 	f.dx=dg->xcenter;
 	f.dy=dg->ycenter;
 	clear();
  	
 
	if (debug) {
		if (strcmp("stderr",param)){
			if ((debug_channel=fopen(param,"w"))==NULL) {
				fprintf(stderr,"Can't open %s as debug file\n",param);
			} else {
				Alloc_debug(debug_channel);
				Makestruct_debug(debug_channel);
			}
		} else {
			debug_channel=stderr;
		}
	}
	
	while (argc) {
		if (*argv != NULL) {
/*
 *	Use remaining params 
 */
			if ((infile = fopen(*argv,"r")) == NULL) {
				fprintf(stderr,"Can't open file %s\n",*argv);
			} else {
				fprintf(stderr,"Split %s\n",*argv);
			}
			if (debug){
				fprintf(debug_channel,"F I L E : %s \n\n",*argv);
			}
			dofile(infile,&f);
			fclose(infile);
			doneone = 1;
		}
		argc--; argv++;
	}
	if (!doneone) {
		dofile(stdin,&f);
	}
	Show_alloc();
	Show_free();
}

/*
 * D O F I L E -- process files one at a time
 */

dofile(infile,f)
FILE *infile;
struct pframe *f;
{
struct object *obj,*objlist[POLYSPLITMAXOBJ],*readobj();
static int res;
static int n;
static int num;
static int i;

	n=0;
	while ((obj=readobj(infile))!=NULL){
		if (debug){
			fprintf(debug_channel,"Object %d\n",++n);
		}
		if (obj->type==1){
			if (overlaps==1){
				res=split_overs(obj,&objlist[0],&objlist[1],f,OVERLAY1,OVERLAY1);
				num = 2;
			}else{
				res=split_comps(obj,objlist,&num,f,OVERLAY1,OVERLAY1);
			}
			if (res) {
				for (i=0;i<num;i++){
					writeobj(stdout,objlist[i]);
					picframe(obj,f);
					freeobj(objlist[i]);
				}
			}
		}
		freeobj(obj);
	}
	clear();
}
