/*
 * searchcent.c
 * ------------
 * ji Oct 14, 86
 *
 * labelling algorithm for searching central point in skeleton
 * find two nodes in the skeleton,which have long tail
 * upnode: has two tails and one head branches
 * downode: has one tail and two heads branches
 * return 0: absolutely sure find the central point
 * 	  1: relatively sure, maybe have to try twice
 *
 * MODIFICATIONS
 * -------------
 *
 *	22 Jan 1991		CAS		Localise debug
 *	 1 Oct 1987		jimp/CAS		bug in isdownode + isupnode
 * 25-09-87	jimp	Return -1 from failures, e.g. more than 5
 *			dnodes or unodes.
 *	jp	24-09-87	Debugging a bizarre crash from overlap split
 *	jp	13-04-87	ICOPY replaces structure assignment.
 */
#include <stdio.h>
#include <ctype.h>
#include <wstruct.h>
#include <split.h>

#define ICOPY(a,b) {a.ileft=b.ileft;a.iright=b.iright;}

#ifdef DEBUG
static FILE *debug_channel = NULL;
#endif

searchcent(obj, add)
struct object *obj;
struct address *add;
{
	struct object *nobj, *skeleton();
	struct iwspace iwsp;
	struct address dnode[5], unode[5];
	struct intervaldomain *idmn;
	register int firstkl;
	register int colr;
	int line, col, i, j, k, tmp;
	int nd, nu;

#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Searchcent: entry\n");
#endif DEBUG
	dnode[0].order = unode[0].order = 0;
	nobj = skeleton(obj, 0);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Searchcent: after skeleton\n");
#endif DEBUG
	/*
	manyfacts(1);
	facts(nobj);
	*/
	idmn = nobj->idom;
	k = 0;
	j = 0;
	firstkl = idmn->kol1;
	initrasterscan(nobj, &iwsp, 0);
	while(nextinterval(&iwsp) == 0){
		line = iwsp.linpos;
		col = iwsp.lftpos - firstkl;
		colr = iwsp.rgtpos - firstkl;
		i = isdownode(nobj, line, col, colr, 0);
		if(i > 0){
			if (k > 4) {
				freeobj(nobj);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Searchcent: error return\n");
#endif DEBUG
				return(-1);
			}
			dnode[k].ycood = line;
			dnode[k].xcood = i;
			k++;
		} else if(i == 0){
			add[0].xcood = (col + colr) >> 1;
			add[0].xcood += firstkl;
			add[0].ycood = line;
			freeobj(nobj);
			return(0);
		}
		i = isupnode(nobj, line, col, colr, 0);
		if(i >= 0){
			if (j > 4) {
				freeobj(nobj);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Searchcent: error return\n");
#endif DEBUG
				return(-1);
			}
			unode[j].ycood = line;
			unode[j].xcood = i;
			j++;
		}
	}
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Searchcent: after while loop\n");
#endif DEBUG
	if(k == 1 && j == 0){
		add[0].xcood = dnode[0].xcood + firstkl;
		add[0].ycood = dnode[0].ycood;
		freeobj(nobj);
		return(0);
	} else if (k == 0 && j == 1){
		add[0].xcood = unode[0].xcood + firstkl;
		add[0].ycood = unode[0].ycood;
		freeobj(nobj);
		return(0);
	} else if (k == 1 && j == 1){
		if(abs(unode[0].ycood - dnode[0].ycood) <= 1){
			add[0].xcood = (unode[0].xcood + dnode[0].xcood) >> 1;
			add[0].xcood += firstkl;
			add[0].ycood = unode[0].ycood;
			freeobj(nobj);
			return(0);
		}
	}
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Searchcent: after various return(0) tests\n");
#endif DEBUG
	tmp = 0;
	nd = nu = 6;
	for(i = 0; i < k; i++){
		dnode[i].order = taillength(nobj, &dnode[i], -1);
		if(tmp < dnode[i].order){
			tmp = dnode[i].order;
			nd = i;
		}
	}
	tmp = 0;
	for(i = 0; i < j; i++){
		unode[i].order = taillength(nobj, &unode[i], 1);
		if(unode[i].order > tmp){
			tmp = unode[i].order;
			nu = i;
		}
	}
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Searchcent: after taillength loops\n");
#endif DEBUG
	if(nu == 6){
		add[0].xcood = dnode[nd].xcood + firstkl;
		add[0].ycood = dnode[nd].ycood;
		freeobj(nobj);
		return(0);
	} 
	if(nd == 6) {
		add[0].xcood = unode[nu].xcood + firstkl;
		add[0].ycood = unode[nu].ycood;
		freeobj(nobj);
		return(0);
	}
	if(unode[nu].order >= dnode[nd].order){
		
		add[0].xcood = unode[nu].xcood + firstkl;
		add[0].ycood = unode[nu].ycood;
		add[1].xcood = dnode[nd].xcood + firstkl;
		add[1].ycood = dnode[nd].ycood;
	} else {
		add[1].xcood = unode[nu].xcood + firstkl;
		add[1].ycood = unode[nu].ycood;
		add[0].xcood = dnode[nd].xcood + firstkl;
		add[0].ycood = dnode[nd].ycood;
	}
	freeobj(nobj);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Searchcent: return(1)\n");
#endif DEBUG
	return(1);
}


/* 
 * isdownode:
 * if an interval (kl, kr) in line l connects
 * with any interval in line l-1, return the number of
 * conneced interval(or intervals), otherwise return 0
 * note:
 *	kr actually is not the right end of an interval
 *	it is larger than it by one. it is only for
 *	speed sake.
 *	if the fifth parameter is 0, use kr as right end
 *	otherwise find right end automaticly.
 */

isdownode(obj, l, kl, kr, j)
struct object *obj;
int l, kl, kr;
int j;
{
	struct intervaldomain *idmn = obj->idom;
	register struct interval *intl;
	struct intervalline *itvl;
	int k1, k2, i, m, n, tmp[2]; 

	if ((l == idmn->line1) || (l == idmn->lastln))
		return(-1);
	itvl = &idmn->intvlines[l - idmn->line1];
	n = itvl->nintvs;
	intl = itvl->intvs;
	k1 = kl - 1;
	if(j == 0)
		k2 = kr  + 1;
	else {
		while (n-- > 0) {
			if(kl == intl->ileft){		/* ji 1 */
				k2 = intl->iright + 1;
				break;
			}
			intl++;
		}
	}
	/*
	 * find the number of intervals in line l-1, which connect with 
	 * the intrval in line l
	 */
	itvl--;
	n = itvl->nintvs;
	intl = itvl->intvs;
	i = 0;
	while (n-- > 0) {
		if (!(k2 < intl->ileft || k1 > intl->iright))
			i++;
		intl++;
	}
	if(i < 2)
		return(-1);
	/*
	 * make sure there is a branch underneath
	 */
	itvl += 2;
	n = itvl->nintvs;
	intl = itvl->intvs;
	m = 0;
	while (n-- > 0) {
		if (!(k2 < intl->ileft || k1 > intl->iright)){
/*			tmp[m] = ((k1 + k2) >> 1);*/
			m++;
		}
		intl++;
	}
	if(m == 0)
		return(-1);
	else if(m == 1)
/*		return(tmp[0]);*/
		return((k1 + k2) >> 1);
	else {
		/*
		if(tmp[0] <= kr && tmp[0] >= kl)
			return(tmp[0]);
		return(tmp[1]);
		it means that this interval has at least four branches so
		it must be the sentral interval
		*/
		return(0);
	}
}


/* 
 * isupnode:
 * if an interval (kl, kr) in line l connects
 * with any interval in line l+1, return the number of
 * conneced interval(or intervals), otherwise return 0
 * note:
 *	kr actually is not the right end of an interval
 *	it is larger than it by one. it is only for
 *	speed sake.
 *	if the fifth parameter is 0, use kr as right end
 *	otherwise find right end automaticly.
 */

isupnode(obj, l, kl, kr, j)
struct object *obj;
register int l, kl, kr;
int j;
{
	struct intervaldomain *idmn = obj->idom;
	register struct interval *intl;
	struct intervalline *itvl;
	int k1, k2, i, m, n, tmp[2]; 

	if ((l == idmn->line1) || (l == idmn->lastln))
		return(-1);
	itvl = &idmn->intvlines[l - idmn->line1];
	n = itvl->nintvs;
	intl = itvl->intvs;
	k1 = kl - 1;
	if(j == 0)
		k2 = kr  + 1;
	else {
		while (n-- > 0) {
			if(k1 == intl->ileft){
				k2 = intl->iright + 1;
				break;
			}
			intl++;
		}
	}
	/*
	 * find the number of intervals in line l+1, which connect with 
	 * the intrval in line l
	 */
	itvl++;
	n = itvl->nintvs;
	intl = itvl->intvs;
	i = 0;
	while (n-- > 0) {
		if (!(k2 < intl->ileft || k1 > intl->iright))
			i++;
		intl++;
	}
	if(i < 2)
		return(-1);
	/* 
	 * make sure there is a interval above
	 */
	itvl -= 2;
	n = itvl->nintvs;
	intl = itvl->intvs;
	m = 0;
	while (n-- > 0) {
		if (!(k2 < intl->ileft || k1 > intl->iright)){
/*			tmp[m] = ((k1 + k2) >> 1);*/
			m++;
		}
		intl++;
	}
	if(m == 0)
		return(-1);
	else if(m == 1)
/*		return(tmp[0]);*/
		return((k1 + k2) >> 1);
	else {
		/*
		if(tmp[0] <= kr && tmp[0] >= kl)
			return(tmp[0]);
		else return(tmp[1]);
		*/
		return(0);
	}
}


/*
 * a node interval has at least three neighboughs
 * upnode has at least two tails below on the contrary
 * downnode has at least two tails above
 * the length of the shorter tail is the tail length of that node.
 * note: k is relative coordinate
 */

taillength(obj, node, direc)
struct object *obj;
struct address *node;
int direc;
{
	struct intervaldomain *idmn = obj->idom;
	struct intervalline *itvl;
	struct interval *intl, *ptr;
	struct interval last, this, next, tmp;
	int l, k, incr;
	int i = 0, j, kl, kr, n;
	int tail1, tail, length = BIGVALUE, line;

	incr = direc;
	l = node->ycood;
	k = node->xcood;
	/* 
	 * find the node interval
	 */
	itvl = &idmn->intvlines[l - idmn->line1];
	n = itvl->nintvs;
	intl = itvl->intvs;
	while(n-- > 0){
		if(k >= intl->ileft && k <= intl->iright){
			kl = intl->ileft;
			kr = intl->iright; 
			last.ileft = kl;
			last.iright = kr;
			break;
		}
		intl++;
	}
	itvl += direc;
	line = l + direc;
	l = line;
	n = itvl->nintvs;
	intl = itvl->intvs;
	while(n-- > 0){
		if(!(kr + 1 < intl->ileft || kl - 1 > intl->iright)){
			if(i == 0)
				ptr = intl;
			i++;
		}
		intl++;
	}
	/*
	 * calculate the length of the tail
	 */
	intl = ptr;
	tail = ptr->iright - ptr->ileft + 1;
	tail1 = tail;
	ICOPY(tmp,last);
	for(j = 0; j < i; j++){
		this.ileft = intl->ileft;
		this.iright = intl->iright;
		while((incr = nextint(obj, &last, &this, line, incr, &next)) != 0){
			line += incr;
			ICOPY(last,this);
			ICOPY(this,next);
			tail += 1 + next.iright - next.ileft;
		}
		if(j == 0){
			length = tail;
			line = l;
			incr = direc;
			tail = tail1;
			ICOPY(last,tmp);
		}
		intl++;
	}
	if(length > tail){
	/*
		node->xcood = (intl-1)->iright;
	*/
		return(tail);
	} else {
	/*
		node->xcood = intl->ileft;
	*/
		return(length);
	}

}

/* 
 * find next neighbour interval if there is only one
 * otherwise stop and return 0
 * note: 
 *	 1: this function is only suitable for skeleton
 *  	 2:  k is relative coordinate, for simple
 *	 3: if the interval is not the end point or node return 1,
 *	    otherwise return 0
 *	 4: last, this is the points were known, next is we are
 *          looking for
 */

nextint(obj, last, this, l, direc, next)
struct object *obj;
struct interval *last, *this, *next;
int direc;
{
	struct intervaldomain *idmn = obj->idom;
	struct intervalline *itvl;
	struct interval *intl, *tmp;
	int k, kr;
	int i = 0, n, j;
	
	k = this->ileft - 1;
	kr = this->iright + 1;
	if(l == idmn->line1 && direc == -1)
		return(0);
	if(l == idmn->lastln && direc == 1)
		return(0);
	/*
	 * get this interval
	 */
	itvl = &idmn->intvlines[l - idmn->line1];
	/*
	n = itvl->nintvs;
	intl = itvl->intvs;
	while(n-- > 0){
		if(k == intl->ileft){
			kr = intl->iright;
			break;
		}
		intl++;
	}
	*/
	if(direc > 0)
		itvl++;
	else itvl--;
	n = itvl->nintvs;
	intl = itvl->intvs;
	while(n-- > 0){
		if(!(kr < intl->ileft || k > intl->iright)){
			next->ileft = intl->ileft;
			next->iright = intl->iright;
			i++;
		}
		intl++;
	}
	if(i > 1)			/* it is a node */
		return(0);
	else if(i == 0){
		if(direc < 0){
			itvl++;
			itvl++;
		} else {
			itvl--;
			itvl--;
		}
		direc = (direc > 0) ? -1 : 1;
		n = itvl->nintvs;
		intl = itvl->intvs;
		if (n == 1)
			return(0);
		j = 0;
		while(n-- > 0){
			if(last->ileft == intl->ileft){
				tmp = intl;
				j++;
			}
			intl++;
		}
		if(j == 1)
			return(0);
		intl = tmp;
		if((intl->iright == this->ileft - 1 || intl->iright == this->ileft) && ((intl+1)->ileft == this->iright || (intl+1)->ileft == this->iright + 1)){
			next->ileft = (intl+1)->ileft;
			next->iright = (intl+1)->iright;
			return(direc);
		}else if((intl->ileft == this->iright + 1 || intl->ileft == this->iright) && ((intl-1)->iright == this->ileft - 1 || (intl-1)->iright == this->ileft)){
			next->ileft = (intl-1)->ileft;
			next->iright = (intl-1)->iright;
			return(direc);
		} else return(0);
	}
	return(direc);
}
