/*
 * sosolae.c
 * -------------
 * Ji Liang Aug 18, 1986
 *
 * solve Systems of Linear Algebraic Equations 
 * 
 * Modification history
 * ====================
 *
 *  25-09-87	jimp	Put cholesky() on #ifdef NOTDEF
 *  12-08-87	jl	Use return 0 or 1 to indicate
 *					failure or success. Delete fail premiter
 */

#include <math.h>
#include <stdio.h>
#define LSIZE 4
#define KSIZE 5

/*
 * elimination method
 * A: input array in line order
 * n: the order of the set of equations
 * ep: a small value, which the accuracy depends on
 * return: 1 ok
 *	 0 fail
 * B: the output
 */

elemeqn(S, n, ep, B)
double *S;
int n;
double ep, *B;
{
	register int i, j, k, l;
	double P, T, *A[LSIZE], *astart;

	/*
	 * initionlize
	 */
	/*
	m = n - 1;
	*/
	A[0] = astart = (double *) Malloc (sizeof(double) * n * (n + 1));
	for(l = 0; l < n; l++){
		A[l] = astart;
		astart += n + 1;
		for(k = 0; k <= n; k++){
			A[l][k] = *S++;
		}
	}
	/*
	 * find the maximum value in the column
	 * label the line number as l
	 * if the maximum value is less than ep
	 * we can't get result otherwise change
	 * the line, which the maximum value in,
	 * to the k-th line
	 */
	for(k = 0; k < n; k++){
		P = 0.0;
		for(i = k; i < n; i++){
			if(abs(A[i][k]) > abs(P)){
				P = A[i][k];
				l = i;
			} 
		}
		if(abs(P) <= ep){
			return(0);
		}
		if(l != k){
			for(j = k; j <= n; j++){
				T = A[k][j];
				A[k][j] = A[l][j];
				A[l][j] = T;
			}
		}
		P = 1 /P;
		if(k != n){
			for(j = k + 1; j <= n; j++){
				A[k][j] *= P;
				for(i = k + 1; i < n; i++)
					A[i][j] -= A[i][k]*A[k][j];
			}
		}

	}
	for(i = n - 2; i >= 0; i--){
		for(j = i + 1; j < n; j++)
			A[i][n] -= A[i][j] * A[j][n];
	}
	for(i = 0; i < n; i++){
		*B++ = A[i][n];
	}
	Free(A[0]);
	return(1);
}
		
#ifdef NOTDEF
/*
 * Cholesky mathod
 * A: input array in line order
 * n: the order of the set of equations
 * ep: a small value, which the accuracy depends on
 * fail: 0 ok
 *	 1 fail
 * B: the output
 */

double *cholesky(S, n, fail)
double *S;
int n, *fail;
{
	register int i, j, k, l;
	double *B, *bstart, *A[LSIZE], *astart;
	int m;

	/*
	 * initionlize
	 */
	A[0] = astart = (double *) Malloc (sizeof(double) * n * (n + 1));
	for(l = 0; l < n; l++){
		A[l] = astart;
		astart += n + 1; 
		for(k = 0; k <= n; k++)
			A[l][k] = *S++;
	}
	A[0][0] = sqrt(A[0][0]);
	for(j = 1; j < n; j++)
		A[0][j] /= A[0][0];
	for(i = 1; i < n; i++){
		for(j = 0; j < i; j++)
			A[i][i] -= A[j][i] * A[j][i];
		if(A[i][i] < 0.0){
			*fail = 1;
			return;
		} 
		A[i][i] = sqrt(A[i][i]);
		if(i != n - 1){
			for(j = i + 1; j < n; j++){
				for(k = 0; k < i; k++)
					A[i][j] -= A[k][i] * A[k][j];
				A[i][j] /= A[i][i];
			}
		}
	}
	A[0][n] /= A[0][0];
	for(l = 1; l < n; l++){
		for(k = 0; k < l; k++)
			A[l][n] -= A[k][l] * A[k][n];
		A[l][n] /= A[l][l];
	}
	m = n - 1;
	A[m][n] /= A[m][m];
	for(l = m - 1; l >= 0; l--){
		for(k = m; k > l; k--){
			A[l][n] -= A[k][n] * A[l][k];
		}
		A[l][n] /= A[l][l];
	}
	*fail = 0;
	bstart = B = (double *) Malloc (sizeof(double) * n);
	for(i = 0; i < n; i++){
		*B++ = A[i][n];
	}
	Free(astart);
	/*
	Free(bstart);
	*/
	return(bstart);
}

#endif NOTDEF
