/*
 *	S P L I T _ C O M P S . C  --  Split touching Objects.
 *
 * Ji Liang Mar 23, 1987
 *
 *
 * Modifications
 *
 *	15 Feb 1989	CAS/SEAJ	Handle too many objects. Use status return from
 *							label + poly_split.
 *	24 Nov 1988		SCG		pass newlen to split_comps so that it knows how
 *							long the new list is.
 *	22 Nov 1988		dcb		Added param. to woolz_check.. to indicate where called from
 *	16 Nov 1988		SCG		woolz_check_obj on entry of split_comp function
 *	23-09-87		jimp	If object not connected, call label and exit.
 *	10-09-87		jimp	Timing routines inserted (temporarily).
 *	09-09-87		jimp	Cope with splithole failing.  Put in a lower size limit.
 *	03-09-87	  jimp@IRS	Check number of holes before starting
 *	26-08-87		jimp	Better control of space (linkcount, freeing).
 *	25-08-87		jimp	Mod complementing mod in mendhole.c.  Also, rely
 *							on value table linkcount when freeing.  Also, mod
 *							complementing mod in splithole.c.
 *	24-08-87		jimp	Removed the rethreshold stuff until we have thought
 *							a bit more about it.....
 *	15-08-87 jl		Added parameter big for touchpoint(), it can avoid 
 *					repeatly calculating area later on.
 *	05-08-87	jl/jp	More space free-ed
 * 04-08-87	JL  Use label() instead of labelling().
 * 30-06-87	JL	When the object is noisy and the grey level
 *			of the boundary is too low, rethreshold
 *			Can return at most 80 objects.
 * 29-06-87	JL	When hole is small, mend it.
 * 05-06-87	JL	Use new version of touchpoint()
 * 22-04-87	jimp	Use standard routine "polysplit" to split along
 *					the polygon found here.  Replace "boundary" calls
 *					by "ibound" everywhere.
 * 21-04-87	jl/jimp	Wrap=12 in call to ibound.
 * 20-04-87	jimp	Restoregrey() anyway for "interseg"
 * 25-03-87	JL	Size test at start
 * 25-03-87	jimp	Ensure freeing of working space
 */

#include <stdio.h>
#include <wstruct.h>
#include <split.h>

#ifdef DEBUG
static FILE *debug_channel;
#endif

/*
 *	S P L I T _ C O M P S  --   split touching objects
 *
 * return 0 : failed
 *	  1 : successed
 */
split_comps(comobj, new, newlen, n, fr, col0, col1)
struct object *comobj, **new;
int *n, newlen;
struct pframe *fr;
int col0, col1;
{
	struct object *polyobj, *makemain(), *obj;
	struct object *threshold(), *tobj;
	struct object *splithole(), *sobj, *mendhole(), *mobj;
	struct object *bobj, *ibound(), *smoothobj();
	struct boundlist *sublist[MAX_HOLES];
	struct boundlist *blist;
	struct polygondomain *plgd;
	struct polygondomain *tlgd;
	struct ivertex tp[4];
	int hnum, i, j, k, thresh, big = 0, stat;
	GREY g, aveg, bg, boundgrey(), averagegrey();

	/*
	 * check whether incoming object is a valid object
	 */
	if(woolz_check_obj(comobj, "split_comps.1") != 0)
		return(0);
		
	obj = comobj;
#ifdef DEBUG
	if (debug_channel != NULL)
		fprintf(debug_channel,"Split_comps entry.  Vdom linkcount %d\n",obj->vdom->linkcount);
#endif DEBUG
#ifdef TIME
	fprintf(stderr,"Split_comps entry ");
		starttimer();
#endif TIME
	tobj = sobj = mobj = NULL;
	bobj = ibound(obj, 1);
#ifdef TIME
	fprintf(stderr,"After first ibound ");
		elapsedtime();
#endif TIME
	blist = (struct boundlist *) bobj->idom;
	
/*
 * Object not connected ? - Just use label 
 */
	if (blist->next != NULL) {
		freeobj(bobj);
		if (label(obj, n, new, newlen, 4) == 0) {
			for (j = 0; j < *n; j++)				/* Label failed - dump objects + return */
				freeobj(new[j]);
			return(0);
		} else {
#ifdef DEBUG
			if (debug_channel != NULL)
				fprintf(debug_channel,"Split_comps: Disconnected object, %d components\n",*n);
#endif DEBUG
			/*
			 * If any of the returned objects is bad free the lot and fail
			 */
			for (i = 0; i < *n; i++) {
				if (woolz_check_obj(new[i], "split_comps.2")) {
					for (j = 0; j < *n; j++)
						freeobj(new[j]);
					return(0);
				}
			}
			return(1);
		}
	}
	
/*
 * Object boundary to holey ??
 */
	getholelist(blist->down, sublist, &hnum);
#ifdef TIME
	fprintf(stderr,"After getholelist ");
		elapsedtime();
#endif TIME
	if (hnum >= MAX_HOLES) {
		freeobj(bobj);
		fprintf(stderr,"Boundary too noisy for auto-split\007\n");
#ifdef DEBUG
		if (debug_channel != NULL)
			fprintf(debug_channel,"Split_comps: EXIT (too many holes)\n");
#endif DEBUG
		return(0);
	}
	
/*
 * Object large ??
 */
	i = area(comobj);
	if (i < 300) {
#ifdef DEBUG
		if (debug_channel != NULL)
			fprintf(debug_channel,"Split_comps: EXIT (too small)\n");
#endif DEBUG
		return(0);
	}
	aveg = averagegrey(obj, &g);
	if (i > 500){				/* Crazy size, this */
		if (i > 5000)
			big = 1;
#ifdef THRESHOLD
		plgd = blist->poly;
		bg = boundgrey(obj, plgd);
		j = if_noisy(obj, 3);
		if (bg == 0 || j == 1 || aveg / bg > 4){
			thresh = MIN(aveg/4, 25);
			tobj = threshold (comobj, thresh);
			obj = tobj;
			if (label(obj, n, new, newlen, 4) == 0) {
				for (j=0; j < *n; j++)
					freeobj(new[j]);
				return(0);
			} else {
				if (*n > 1) {
/*
 *					for(i = 0; i < *n; i++){
 *						picframe(new[i], fr);
 *						sublist[i] = (struct boundlist *) ibound(new[i], 1)->idom;
 *						picpoly(sublist[i]->poly, fr);
 *						intens(1);
 *					}
 *					for(i = 0; i <= j; i++)
 *						Free(sublist[i]);
 */
					freeobj(bobj);
					if(tobj != comobj){
						freedomain(tobj->idom);
						Free(tobj);
					}
					return(1);
				} else {
					obj = new[0];
					freeobj(bobj);	/* remove old boundary */
					bobj = ibound(obj, 1);
					blist = (struct boundlist *) bobj->idom;
				}
			}
		}
#endif THRESHOLD
	} 
/*
 * if there is a hole split or mend it at first
 */
	if (blist->down != NULL){
#ifdef DEBUG
		if (debug_channel != NULL)
			fprintf(debug_channel,"Mendhole called\n");
#endif DEBUG
		if(aveg > 40)
			j = 10;
		else j = 15;
		mobj = mendhole(obj, blist, j);
#ifdef TIME
		fprintf(stderr,"After mendhole ");
		elapsedtime();
#endif TIME
		if (mobj != NULL) {
#ifdef DEBUG
			if (debug_channel != NULL)
				fprintf(debug_channel,"Mendhole mended holes.  Vdom linkcount %d\n",mobj->vdom->linkcount);
#endif DEBUG
			freeobj(bobj);
			obj = mobj;
			bobj = ibound(obj,1);
			blist = (struct boundlist *) bobj->idom;
		}
	}
	if (blist->down != NULL) {
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Splithole called\n");
#endif DEBUG
		sobj = splithole(obj, blist);
#ifdef TIME
fprintf(stderr,"After splithole ");
elapsedtime();
#endif TIME
		if (sobj == NULL) {
			freeobj(bobj);
			if (mobj != NULL)
				freeobj(mobj);
			if (tobj != NULL)
				freeobj(tobj);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Split_comps: EXIT (splithole failed)\n");
#endif DEBUG
			return(0);
		}
		else {
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Splithole split holes.  Vdom linkcount %d\n",sobj->vdom->linkcount);
#endif DEBUG
			obj = sobj;
		}   
	}
	freeobj(bobj);
	bobj = ibound(obj,12);
#ifdef TIME
fprintf(stderr,"After second ibound ");
elapsedtime();
#endif TIME
	blist = (struct boundlist *) bobj->idom;
	plgd = blist->poly;
	/*
	colour(255);
	colour(col0);
	picframe(obj, fr);
	picpoly(plgd, fr);
	*/
	/*
	 * find the split curve and cut points
	 * change the curve into an object 
	 * then split the composite along this curve 
	 * finally restore the grey level
	 */
	if((touchpoint(obj, plgd, big, &tlgd)) == 0){
		fprintf(stderr,"Split rejected\n");
		freeobj(bobj);
		if (sobj != NULL)
			freeobj(sobj);
		if (mobj != NULL)
			freeobj(mobj);
		if (tobj != NULL)
			freeobj(tobj);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Split_comps: EXIT (touchpoint found no path)\n");
#endif DEBUG
		return(0);	
	} else {
#ifdef TIME
fprintf(stderr,"After touchpoint ");
elapsedtime();
#endif TIME
#ifdef DEBUGDISPLAY
		/*
		 * show the procedure of analyze:
		 * (i) show the cut points
		 */
		/*
		for(i = 0; i < 2; i++){
			point.k = tp[i].vtX;
			point.l = tp[i].vtY;
			picframe(&point, fr);
			circle(10);
		}
		*/
		/*
		 * show the split curve
		 */
		colour(col0);
		picpoly(tlgd, fr);
		intens(1);
		update();
#endif DEBUGDISPLAY
		polyobj = makemain(10, tlgd, NULL, NULL, NULL);
#ifdef DEBUG
		if (debug_channel != NULL)
			fprintf(debug_channel,"Split_comps before polysplit\n");
#endif DEBUG
		stat = poly_split(obj, polyobj, new, newlen, n, 1);
#ifdef TIME
		fprintf(stderr,"After polysplit ");
		elapsedtime();
#endif TIME
#ifdef DEBUG
		if (debug_channel != NULL)
			fprintf(debug_channel,"Split_comps after polysplit.  Vdom linkcount %d\n",obj->vdom->linkcount);
#endif DEBUG
		freeobj(bobj);
		freeobj(polyobj);
		if (sobj != NULL)
			freeobj(sobj);
		if (mobj != NULL)
			freeobj(mobj);
		if (tobj != NULL)
			freeobj(tobj);
#ifdef DEBUG
		if (debug_channel != NULL) {
			fprintf(debug_channel,"Split_comps exit.  Vdom linkcount %d\n",obj->vdom->linkcount);
			if (comobj != obj)
				fprintf(debug_channel,"(Orig obj linkcount %d)\n",comobj->vdom->linkcount);
		}
#endif DEBUG
		if (*n == 1){
			freeobj(new[0]);
			return(0);
		} else {
			/*
			 * If any of the returned objects is bad free the lot and fail
			 */
			for (i = 0; i < *n; i++) {
				if (woolz_check_obj(new[i], "split_comps.3")) {
					fprintf(stderr, "Split failed\n");
					for (j = 0; j < *n; j++)
						freeobj(new[j]);
					return(0);
				}
			}
			return(stat);
		}
	}
}

