/*
 * splithole.c
 * -----------
 * Ji Liang Dec 17, 1986
 * 
 * find the shortest distance between
 * a hole boundary and a real boundary
 * then split it along that line
 *
 * Modifications
 *
 *	22 Jan 1991		CAS		Localise debug
 *	24 Nov 1988		SCG		modified all calls to poly_split to take the length of the
 *							array objlist.
 *	22 Nov 1988		dcb		Added param. to woolz_check.. to indicate where called from
 *	16 Nov 1988		SCG		check validity of incoming object in mendhole
 *	12 Nov 1987		SCG		In getholelist ignore boundaries of holes
 *							returned by ibound with nvertices < 5
 * 10-09-87			jimp	Registers.
 * 09-09-87			jimp	Debugging.  On occasion polysplit returns daft number.
 *							LOOP: and goto LOOP removed.  Code looks dodgy here.
 *							If poly_split breaks object, return(NULL).
 * 03-09-87	jimp@IRS	MAX_HOLES used.  Getholelist always updates n.
 * 31-08-87	jimp	Free not free in two places.
 * 25-08-87	jimp	Removed some messages.  Return NULL if nothing changed.
 *			Mobj->vdom->linkcount++
 * 24-08-87	jimp	Boundilation now returns entire boundary object, so
 *			that it can be freed in envirosearch
 * 4) Aug 10, 	Ji Liang	Free more space.
 * 3) Jul 14, 1987, Ji Liang 	Change float to int.
 *				Change split() to poly_split().
 * 2) June 14 1987, Ji Liang
 *    added check on every possible split path 
 * 1) April 23, 1987, Ji Liang
 *    Rewrote splithole.c put the old one in Old.
 *    The difference between old and new is the direction from which
 *    the split begin. The old analyzed the outside boundary and found
 *    the centres of the holes then split from outside to inside. It was
 *    fast procedure but it would fail if a hole surround by lots of holes.
 *    The new one anylyzed holes and first split among the holes then split
 *    every holes from the holes to outside boundary. It is rather slow but
 *    works properly even the object consists of more than 35 touching
 *    chromosomes and has 20 holes in it.
 */
#include <math.h>
#include <stdio.h>
#include <wstruct.h>
#include <split.h>

#ifdef DEBUG
static FILE *debug_channel == NULL;
#endif


struct object *
splithole(obj, blist)
struct object *obj;
struct boundlist *blist;
{
	struct boundlist *hlist;
	struct object *tobj, *mobj, *bobj;
	struct object *objlist[2], *ibound();
	struct object *makemain();
	struct polygondomain *plgd, *llgd, *pointopoly();
	struct polygondomain *hplgd[MAX_HOLES], *path, *direcsearch();
	struct ivertex add[2];
	struct ivertex p[2];
	struct ivertex *wtx;
	int splitable[MAX_HOLES];
	int i, hnum, j, num;
	int l, k, used;
	GREY g;

#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Splithole: entry\n");
#endif DEBUG

	/*
	 * check that the incoming object is valid
	 */
	if(woolz_check_obj(obj, "splithole") != 0)
		return(NULL);
		
	mobj = obj;
	bobj = NULL;
	hnum = 0;
	if(getholelist(blist->down, hplgd, &hnum) == 0)
		return(NULL);
	hlist = blist;

#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Splithole: before join-holes LOOP (hnum %d)\n",hnum);
#endif DEBUG
	/*
	 * if there are any pair of holes nearer than 6 pixels
	 * split them at first
	 */ 
	used = -1;
	while (used != 0) {
	used = 0;
	/*
	 * if splitable[i] == 0, the i-th hole has not been tested
	 * otherwise it has been used
	 */
	for(i = 0; i < hnum; i++)
		splitable[i] = 0;
	for(i = 0; i < hnum - 1; i++){
		for( j = i + 1; j < hnum; j++){
			if(shortdis(hplgd[i], hplgd[j], add) < 37 && splitable[i] == 0 && splitable[j] == 0){
				splitable[i] = 1;
				splitable[j] = 1;
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Splithole: before pointopoly 1\n");
#endif DEBUG
				llgd = pointopoly(add);
				/*
				 * test if the path is across another
				 */
				wtx = llgd->vtx;
				k = 0;
				for(l = 0; l < llgd->nvertices; l++){
					if(inside(obj,wtx->vtY,wtx->vtX)==0){
						k = 1;
						break;
					}
					wtx++;
				}
				if(k == 0){
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Splithole: before makemain 1\n");
#endif DEBUG
					tobj = makemain(10,llgd,NULL,NULL,NULL);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Splithole: before poly_split\n 1");
#endif DEBUG
					poly_split(mobj, tobj, objlist, 2, &num, 0);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Splithole: before freeobj(tobj) 1\n");
#endif DEBUG
					freeobj(tobj);
					if(mobj != obj)
						freeobj(mobj);
					used = 1;
					mobj = objlist[0];
#ifdef DEBUG
if (debug_channel != NULL && num != 1)
fprintf(debug_channel,"Splithole: num not one (1) (%d)\n",num);
#endif DEBUG
					if(num != 1) {
						if (mobj != obj)
							freeobj(mobj);
						if (hlist != blist)
							freeobj(bobj);
						return(NULL);
					}
				}
			}
		}
	}
	if (mobj != obj && mobj->vdom == NULL) {
		mobj->vdom = obj->vdom;
		obj->vdom->linkcount++;
	}
	if(used == 1){
		if(hlist != blist)
			freeobj(bobj);
		bobj = ibound(mobj, 1);
		hlist = (struct boundlist *) bobj->idom;
		hnum = 0;
		if(getholelist(hlist->down, hplgd, &hnum) == 0){
			freeobj(bobj);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Splithole: return 1\n");
#endif DEBUG
			return(mobj);
		}	
	}
	}
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Splithole: after join-holes LOOP\n");
#endif DEBUG
	/*
	 * if distance from any point in the hole's boundary
	 * to the outside boundary is nearer than 6, split it
	 */
	plgd = hlist->poly;
	for(i = 0; i < hnum; i++){
		if(shortdis(plgd, hplgd[i], add) < 37){
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Splithole: before pointopoly 2\n");
#endif DEBUG
			llgd = pointopoly(add);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Splithole: before makemain 2\n");
#endif DEBUG
			tobj = makemain(10, llgd, NULL, NULL, NULL);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Splithole: before poly_split 2\n");
#endif DEBUG
			poly_split(mobj, tobj, objlist, 2, &num,0);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Splithole: before freeobj(tobj) 2\n");
#endif DEBUG
			freeobj(tobj);
			if(mobj != obj)
				freeobj(mobj);
			used = 1;
			mobj = objlist[0];
#ifdef DEBUG
if (debug_channel != NULL && num != 1)
fprintf(debug_channel,"Splithole: num not one (2) (%d)\n",num);
#endif DEBUG
			if(num != 1) {
				if (mobj != obj)
					freeobj(mobj);
				if (hlist != blist)
					freeobj(bobj);
				return(NULL);
			}
		}
	}
	/*
	 * split the rest of the holes, which are far
	 * from the outside boundary
	 */
	if (mobj != obj && mobj->vdom == NULL) {
		mobj->vdom = obj->vdom;
		obj->vdom->linkcount++;
	}
	if(used == 1){
		if(hlist != blist)
			freeobj(bobj);
		bobj = ibound(mobj, 1);
		hlist = (struct boundlist *) bobj->idom;
		hnum = 0;
		if(getholelist(hlist->down, hplgd, &hnum) == 0){
			freeobj(bobj);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Splithole: return 2\n");
#endif DEBUG
			return(mobj);
		}
	}
	/*
	 * if the hole is big, split direction can be found
	 * in the boundary of the hole otherwise the surrounding 
	 * area has to be searched
	 */
	for(i = 0; i < hnum; i++){
		getholeref(obj, hplgd[i], p);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Splithole: before direcsearch\n");
#endif DEBUG
		path = direcsearch(obj, &p[1], &p[0], add, &g);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Splithole: before makemain 3\n");
#endif DEBUG
		tobj = makemain(10, path, NULL, NULL, NULL);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Splithole: before poly_split 3\n");
#endif DEBUG
		poly_split(mobj, tobj, objlist, 2, &num, 0);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Splithole: before freeobj(tobj) 3\n");
#endif DEBUG
		freeobj(tobj);
		if(mobj != obj)
			freeobj(mobj);
		mobj = objlist[0];
#ifdef DEBUG
if (debug_channel != NULL && num != 1)
fprintf(debug_channel,"Splithole: num not one (3) (%d)\n",num);
#endif DEBUG
		if(num != 1) {
			if (mobj != obj)
				freeobj(mobj);
			if (hlist != blist)
				freeobj(bobj);
			return(NULL);
		}
	}
	if(hlist != blist)
		freeobj(bobj);
	if (mobj != obj && mobj->vdom == NULL) {
		mobj->vdom = obj->vdom;
		obj->vdom->linkcount++;
	}
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Splithole: return 3\n");
#endif DEBUG
	return(mobj);
}
/*
 * find the shortest distance between 
 * two polygon
 */
shortdis(plgd, ptr, add)
struct polygondomain *plgd;
register struct polygondomain *ptr;
struct ivertex *add;
{
	register struct ivertex *ivt = plgd->vtx;
	register struct ivertex *wtx = ptr->vtx;
	struct ivertex *tmp0, *tmp1;
	register int d, m, n, t = BIGVALUE;

	for(n = plgd->nvertices; n>0; n--) {
		for(m = ptr->nvertices; m>0; m--) {
			d = squardisi(ivt, wtx);
			if(t > d){
				t = d;
				tmp0 = ivt;
				tmp1 = wtx;
			}
			wtx++;
		}
		ivt++;
		wtx = ptr->vtx;
	}
	add[0].vtY = tmp0->vtY;
	add[0].vtX = tmp0->vtX;
	add[1].vtY = tmp1->vtY;
	add[1].vtX = tmp1->vtX;
	return(t);
}

/* change two points to a polygondoain
 * adds new points in new poly if these
 * two points are not connected
 */

struct polygondomain *pointopoly(ptr)
register struct ivertex *ptr;
{
	register struct ivertex *ivt, *start;
	struct polygondomain *plgd, *makepolydmn();
	register i, j, sh, sv;
	int end, k, n, m;

	n = ptr[1].vtY - ptr[0].vtY;
	m = ptr[1].vtX - ptr[0].vtX;
	/*
	 * use shift istead of float dividing
	 */
	if(n != 0){
		m <<= 8;
		k =  m / n;
	}
	if(n > 0){
		sv = ptr[0].vtY;
		sh = ptr[0].vtX;
		end = 1;
		j = n + 1;
	} else {
		sv = ptr[1].vtY;
		sh = ptr[1].vtX;
		end = 0;
		j = 1 - n;
	}
	if( j == 1){
		ivt = (struct ivertex *) Malloc(sizeof(struct ivertex) * 2);
		start = ivt;
		if(m > 0){
			ivt->vtY = ptr[0].vtY;
			ivt->vtX = ptr[0].vtX;
			ivt++;
			ivt->vtY = ptr[1].vtY;
			ivt->vtX = ptr[1].vtX;
		} else {
			ivt->vtY = ptr[1].vtY;
			ivt->vtX = ptr[1].vtX;
			ivt++;
			ivt->vtY = ptr[0].vtY;
			ivt->vtX = ptr[0].vtX;
		}
		plgd = makepolydmn(1, start, 2, 3, 1);
		Free(start);
		return(plgd);
	}
	ivt = (struct ivertex *) Malloc(sizeof(struct ivertex) * j);
	start = ivt;
	for(i = 0; i < j - 1; i++){
		ivt->vtY = sv + i;
		m = k * i;
		m >>= 8;
		ivt->vtX = m + sh;
		ivt++;
	}
	ivt->vtY = ptr[end].vtY;
	ivt->vtX = ptr[end].vtX;
	plgd = makepolydmn(1, start, j, j + 1, 1); 
	Free(start);
	return(plgd);
}

/*
 * changepolytobj()
 * ----------------
 * the differece among changepolytobj(), polytobj(),
 * and ptobj() is that: in changepolytobj, the input polygon
 * already in order, and in ptobj it is quite abitriry.
 * the output of polytobj every interval is two pixel width
 * the output of changepolytobj() or ptobj() has four neighbours
 * countineuos intervals
 */

struct object *changepolytobj(plgd)
struct polygondomain *plgd;
{
	register struct interval *jp, *jpw;
	struct intervaldomain *makedomain(), *idmn;
	struct object *makemain(), *obj;
	register struct ivertex *ivt = plgd->vtx;
	register int i, n = plgd->nvertices;
	register int line1, lastln;
	int kol1, lastkl;

	kol1 = line1 = MAXLINE;
	lastln = lastkl = 0;
	jp = (struct interval *) Malloc(sizeof(struct interval) * n);
	jpw = jp;
	if(n == 2 && ivt->vtY == (ivt + 1)->vtY){
		jp->ileft = ivt->vtX;
		kol1 = ivt->vtX;
		ivt++;
		jp->iright = ivt->vtX;
		lastkl = ivt->vtX;
		line1 = lastln = ivt->vtY;
	} else {
		line1 = ivt->vtY;
		for(i = 0; i < n; i++){
			jp->ileft = ivt->vtX;
			jp->iright = ivt->vtX;
			kol1 = MIN(ivt->vtX, kol1);
			lastkl = MAX(ivt->vtX, lastkl);
			jp++;
			ivt++;
		}
		lastln = line1 + n - 1;
	}
	idmn = makedomain(1, line1, lastln, kol1, lastkl);
	idmn->freeptr = (char *) jpw;
	obj = makemain(1, idmn, NULL, NULL, NULL);
	jp = jpw;
	for(i = line1; i <= lastln; i++){
		jpw = jp;
		jp->ileft -= kol1;
		jp->iright -= kol1;
		makeinterval(i, idmn, 1, jpw);
		jp++;
	}
	return(obj);
}

/* 
 * find the centre of a hole
 */
centre(plgd, p)
struct polygondomain *plgd;
register struct ivertex *p;
{
	register struct ivertex *wtx = plgd->vtx;
	register int i, x = 0, y = 0;
	register int n = plgd->nvertices;

	for(i = 0; i < n; i++){
		x += wtx->vtX;
		y += wtx->vtY;
		wtx++;
	}
	p->vtY = y / n;
	p->vtX = x /n;
}

/*
 * try possible path from nearer small angle along the boundary
 * to the centre point of the hole choose the lightest one as 
 * real path
 */
struct polygondomain *findpath(obj, plgd, p)
struct object *obj;
struct polygondomain *plgd;
struct ivertex *p;
{
	struct polygondomain *tlgd, *tmp[2], *gapath();
	struct polygondomain *makepolydmn();
	struct angladd anglist;
	GREY g[2];
	register int i, j, n;
	int d[2], m[2];

	smallangl(plgd, &anglist, -0.6);
	d[0] = BIGVALUE;
	d[1] = BIGVALUE;
	for(i = 3; i < anglist.num; i++){
		j = squardisi(p, anglist.angl[i]);
		if(d[0] > j){
			d[1] = d[0];
			m[1] = m[0];
			d[0] = j;
			m[0] = i;
		} else if(d[1] > j){
			d[1] = j;
			m[1] = i;
		}
	}
	for(i = 0; i < 2; i++)
		tmp[i] = gapath(obj, anglist.angl[m[i]], p, &g[i]);
	if(g[0] < g[1])
		j = 0;
	else j = 1;
	n = tmp[j]->nvertices;
	tlgd = makepolydmn(1, tmp[j]->vtx, n, n+1, 1);
	for(i = 0; i < 2; i++)
		Free(tmp[i]);
	return(tlgd);
}

/*
 * there is a bug in ibound, the holes spred in the
 * different level though they are the same. So a
 * temperary version of getholelist is used until
 * the bug to be fixed
 */
getholelist(blist, hlist, n)
struct boundlist *blist;
register struct polygondomain **hlist;
int *n;
{
	register struct boundlist *holelist;
	register struct polygondomain *plgd;
	register int i;

	holelist = blist;
	if(holelist == NULL){
		*n = 0;
		return(0);
	}
	i = 0;
	while(holelist != NULL){
		/*
		 * This code has been inserted here to ensure that any hole
		 * boundaries returned by ibound with number of vertices less
		 * than 5 are rejected.  Is there a bug in ibound??  SCG
		 */
		if(holelist->poly->nvertices >= 5) {
			plgd = holelist->poly;
			if (i >= MAX_HOLES)	
				i--;
			hlist[i++] = plgd;
		}
		holelist = holelist->next;
	}
	*n = i;
	return(1);
}

/*
 * choose two points, one of which is the brightest point p[0]
 * in the surrounding area and the other is the nearest
 * point to the brightest p[1] in the boundary
 */
getholeref(obj, plgd, p)
struct object *obj;
struct polygondomain *plgd;
register struct ivertex *p;
{
	register struct ivertex *wtx;
	register int d, i, j, n = plgd->nvertices;

	/*
	 * get a point, which is the brightest point near a hole
	 * boundary and inside the object * as a reference point
	 */
	envirosearch(obj, plgd, &p[0]);
	d = BIGVALUE;
	wtx = plgd->vtx;
	for(i = 0; i < n; i++){
		j = squardisi(&p[0], wtx);
		if(d > j){
			d = j;
			p[1].vtY = wtx->vtY;
			p[1].vtX = wtx->vtX;
		}
		wtx++;
	}
}

/*
 * search along a curve, between which and the original boundary
 * the distance is 5, find the brightest point
 * obj is a hole, plgd is the boundary, p is the brightest point
 * we are looking for
 */
envirosearch(obj, plgd, p)
struct object *obj;
struct polygondomain *plgd;
register struct ivertex *p;
{
	struct object *boundilation(), *bobj;
	struct polygondomain *dlgd;
	register struct ivertex *ivt;
	GREY *greyval();
	register int g, n, i, tmp=255;

	bobj = boundilation(plgd, 3);
	dlgd = ((struct boundlist *)(bobj->idom))->poly;
	n = dlgd->nvertices;
	ivt = dlgd->vtx;
	for(i = 0; i < n; i++){
		g = *greyval(obj, ivt->vtY, ivt->vtX);
		if(g < tmp){
			tmp = g;
			p->vtY = ivt->vtY;
			p->vtX = ivt->vtX;
		}
		ivt++;
	}
	freeobj(bobj);
}
