/*
 * touchpoint.c
 * -----------
 * Ji Liang   Jun 3, 1987
 * 
 * find cut point for a touching object
 * obj, plgd are input object, boundary
 * sepline is the result curve which can 
 * separate the touching object apart
 *
 * Modification history
 * ====================
 *	22 Jan 1991		CAS		Localise debug
 * 14-07-88 SEAJ	Free path[] before going round WORST loop for 2nd time.
 * 21-09-87	jimp	Line-bounds check added to inside().
 * 28-08-87	jimp	Registers.
 * 27-08-87	jimp	Avoid bug with small change to possiblepath().
 * 26-08-87	jimp	Debugging.  Renamed "i"=>"cur_angle", "rate"=>"ratio".
 *			Changed conditional nesting structure within LOOP so
 *			that if "possiblepath" fails, "polylength" not called.
 *	15-08-87	ji		Added parameter big for big object.
 *  10-08-87	ji		Added functon neighb8in() in getnearcut().
 *						Make sure do not reject thin chain connected 
 *						objects  (i.e. if 4-connectivity is used the object 
 *						will be separated itself).
 *  05-08-87	jl/jp	Freeing etc.
 *  05-08-87	jimp	Removed unnecessary "extern struct ipoint point"
 *  04-08-87	Ji Liang
 *				move offset 12 at the biginning
 *				initialise in getnearest more accurate
 *				Treat long path specially by using goodseparate().
 *  14-07-87	Ji Liang
 * 				Change float to int
 *	01-07-87	Ji Liang
 *				Changed ratio threshold from 10 to 8
 *	23-06-87	Ji Liang
 *				Added check at beginning of inside
 *  22-06-87    Ji Liang
 *				Replace absdis by absdis
 *  21-06-87    Ji Liang
 *				Rewrite touchpoint()
 *				a) It can analyse all angles instead of only three
 *				b) Uses segmentrate to eleminate noise
 *				c) Uses 9/16 average grey level as threshold instead 
 *				   of maximum grey level
 *				d) Uses -1.5 as angle threshold instead of -0.8 or -0.7
 *				   (first loop -1.5 is used second loop -0.75 is used)
 *				e) Added check procedure to make sure the path is 
 *					all right
 *					ANGTHRESH made a define.
 *	20-04-87	jp
 *				"thresh" in smallangl should be double (no problem
 *					on Vax, where it was a float).   Sssmallangl removed.
 *	15-04-87	jp
 *				OS9 Polysmooth does not have return value
 *					- call corrected
 *	13-04-87	jp
 *				Fix around OS9 compiler bug
 */
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <wstruct.h>
#include <split.h>

#ifdef DEBUG
static FILE *debug_channel = NULL;
#endif

touchpoint(obj, plgd, big, sepline)
struct object *obj;
struct polygondomain *plgd;
int big;
struct polygondomain **sepline;
{
	struct polygondomain *getline();
	struct polygondomain *path[MAX_ANGS];
	struct anglist alist;
	struct ivertex *tp[2], cut[3];
	int cur_angle, i, j, k, ratio;
	int worst = 0;
	GREY g[2], aveg, averagegrey();
	GREY maxg, *greyval();

#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Touchpoint: entry\n");	
#endif DEBUG
	/*
	 * Get the first cut point, where the smallest angle is.
	 * If no small angle is found the object is not composite.
	 */
	if(anglelist(plgd, &alist, ANGTHRES) == 0){
		fprintf(stderr,"Split failed\n");
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Touchpoint: not composite or needs to be rethreshold\n");
#endif DEBUG
		return(0);
	}
	/*
	 * If there is only one small angle more sensitive threshold
	 * is used to avoid a bend object.
	 */
	if(alist.num < 2)
		if(anglelist(plgd, &alist, ANGTHRES/2) == 0)
			return(0);
WORST:
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Touchpoint: WORST\n");
#endif DEBUG
	/*
	 * The worst loop is seldom used.
	 * If no possible path was found then the program will choose
	 * different threshold and do all of them once more.
	 * Make sure no chance has been missed
	 *
	 * use 1/2 aveg as threshold 
	 */
	aveg = averagegrey(obj, &maxg);
	alist.aveg = aveg;
	aveg >>= 1;
	cur_angle = alist.small;
	equalang(&alist.ang[cur_angle], cut);
	g[0] = 255;
LOOP:
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Touchpoint: LOOP: Alist.num %d, cur_angle %d\n",alist.num,cur_angle);	
#endif DEBUG
	/*
	 * This loop is for analysing every angle on the angle list.
	 * If a path was found then break the loop. If not, delete
	 * the angle from the list and analyse the next
	 */
	plgd->nvertices -= 12;		
	/*
	 * wrap 12 has been used to get plgd
	 * -12 is to avoid repeat
	 * + 12 is to restore
	 */
	j = possiblepath(obj, plgd, cut, alist.ang[cur_angle].vtx, &path[cur_angle], &g[0]);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Touchpoint: LOOP: after possiblepath (%d) path[cur_angle] = 0x%x\n",j, path[cur_angle]);
#endif DEBUG
	if (j != 0) {
		tp[0] = path[cur_angle]->vtx;
		tp[1] = tp[0] + path[cur_angle]->nvertices - 1;
		ratio = segmentrate(obj, plgd,tp[0],tp[1]);
		alist.ang[cur_angle].rate = ratio;
		alist.ang[cur_angle].g = g[0];
		k = polylength(path[cur_angle]);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Touchpoint: LOOP: after polylength (%d)\n",k);
#endif DEBUG
		/*
		 * if the touching part is very long, it is usualy dark
		 * treat it specially
		 */
		if(g[0] > aveg && k > 14 && big == 0)
			if(k > 25 || goodseparate(obj, path[cur_angle], 14) == 1)
				g[0] >>= 1;
		/*
		 * big object (area > 5000) use loose restriction
		 */
		if(big == 1)
			ratio /= 5;
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Touchpoint: LOOP: after big test\n");
#endif DEBUG
	} else {
		/*
		 * make use this angle impossible
		 */
#ifdef DEBUG
if (debug_channel != NULL) {
fprintf(debug_channel,"Touchpoint: this angle is impossible!!\n");
fprintf(debug_channel,"Touchpoint: path[%d] = NULL\n", cur_angle);
}
#endif DEBUG
		alist.ang[cur_angle].g = 255;
		alist.ang[cur_angle].rate = 100;
		path[cur_angle] = NULL;
	}
	plgd->nvertices += 12;
	if(j == 0 || g[0] >= aveg || ratio > 5){
		/*
		 * mark the failed one choose next angle
		 */
#ifdef DEBUG
if (debug_channel != NULL) {
fprintf(debug_channel,"Touchpoint: LOOP: this one failed\n");
fprintf(debug_channel,"Touchpoint: LOOP: alist.ang[%d].prop = 0\n", cur_angle);
}
#endif DEBUG
		alist.ang[cur_angle].prop = 0;
		cur_angle = secondang(&alist);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Touchpoint: LOOP: after secondang (%d)\n",cur_angle);
#endif DEBUG
		/*
		 * If no suitable candidate was found give warnning message
		 */
		if(cur_angle == BIGVALUE){
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Touchpoint: Warning!! there is no bright path, check the result\n");
#endif DEBUG
			/*
			 * first try to loose the restriction
			 */
			for(k = 0; k < alist.num; k++){
				if(alist.ang[k].g<maxg && alist.ang[k].rate<4){
					maxg = alist.ang[k].g;
					cur_angle = k;
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Touchpoint: LOOP: after loosening (%d)\n",cur_angle);
#endif DEBUG
				}
			}
			if(cur_angle != BIGVALUE && alist.ang[cur_angle].g > 16 * aveg / 9)
				cur_angle = BIGVALUE;
			/*
			 * final try to use more sensible angle analysis
			 */
			if(cur_angle == BIGVALUE){
				if(worst == 1){
					fprintf(stderr,"Split failed\n");
					return(0);
				} else worst = 1;
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Touchpoint: Freeing unused path list\n");
#endif DEBUG
				for (i=0; i < alist.num; i++) {
					if (alist.ang[i].prop == 0 && i != cur_angle) {
						if (path[i] != NULL) {
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Touchpoint: Free(path[%d])\n", i);
#endif DEBUG
							Free(path[i]);
						}
					}
				}
				if(anglelist(plgd, &alist, ANGTHRES/2) == 0){
					return(0);
				}
				goto WORST;
			}
		} else {
			equalang(&alist.ang[cur_angle], cut);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Touchpoint: LOOP: after equalang\n");
#endif DEBUG
			goto LOOP;
		}
	} 
#ifdef DEBUG
if (debug_channel != NULL) {
fprintf(debug_channel,"Touchpoint: out of LOOP\n");
fprintf(debug_channel,"Touchpoint: alist.num = %d cur_angle = %d\n", alist.num, cur_angle);
}
#endif DEBUG
	for(j = 0; j < alist.num; j++){
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Touchpoint: free path loop %d\n",j);
#endif DEBUG
		if(alist.ang[j].prop == 0 && j != cur_angle) {
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Touchpoint: 1 : path[%d] != Null\n", j);
#endif DEBUG
			if(path[j] != NULL) {
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Touchpoint: 2 : Free(path[%d]) path = 0x%x\n", j, path[j]);
#endif DEBUG
				Free(path[j]);
			}
		}
	}
	*sepline = path[cur_angle];
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Touchpoint: exit\n");
#endif DEBUG
	return(1);
}

/*
 * ref is possible cut point
 * search for another cut point cut
 * if cannot find it return BIGVALUE
 * otherwise return square distance between these two points
 */
getnearcut(obj, plgd, ref, cut)
struct object *obj;
struct polygondomain *plgd;
struct ivertex *ref, *cut;
{
	register struct ivertex *wtx, *tmp, *ptr;
	register l, k0, k, k1, d;
	int i, min = BIGVALUE, small = BIGVALUE;

	wtx = plgd->vtx;
	tmp = wtx + plgd->nvertices - 2;
	l = squardisi(tmp++, ref); 
	k = squardisi(tmp, ref);
	k1 = squardisi(wtx++, ref);
	k0 = squardisi(wtx++, ref);
	tmp = wtx;
	/*
	 * choose the nearest one but not itself
	 */
	for(i = 2; i < plgd->nvertices; i++){
		d = squardisi(wtx, ref);
		if(d > k && l > k){
			if(abs((int) (wtx - ref - 2)) > 2 ){
				if(min > k){
					small = min;
					ptr = tmp;
					min = k;
					tmp = wtx - 2;
				} else if(small > k){
					small = k;
					ptr = wtx - 2;
				}
			}
		}
		l = k0;
		k0 = k;
		k = k1;
		k1 = d;
		wtx++;
	}
	cut->vtY = tmp->vtY;
	cut->vtX = tmp->vtX;
	i = ref->vtY + cut->vtY;
	k1 = ref->vtX + cut->vtX;
	i >>= 1;
	k1 >>= 1;
	if(inside(obj, i, k1))
		return(min);
	else {
		if(neighb8in(obj, i, k1))
			return(min);
		/*
		 * check if the path which can join these two points
		 * will across the boundary. If it is use the second
		 * candidate
		 */
		cut->vtY = ptr->vtY;
		cut->vtX = ptr->vtX;
		i = ref->vtY + cut->vtY;
		k1 = ref->vtX + cut->vtX;
		i >>= 1;
		k1 >>= 1;
		if(inside(obj, i, k1))
			return(small);
		if(neighb8in(obj, i, k1))
			return(small);			
		return(BIGVALUE);
	}
}

/*
 * give an angle cut, which is at the boundary 
 * find the path, which is pointed out by the angle
 * and passes through the lowest density area
 * g is the highest density in the path
 */
possiblepath(obj, plgd, cut, ref, path, g)
struct object *obj;
struct polygondomain *plgd;
struct ivertex *cut, *ref;
struct polygondomain **path;
GREY *g;
{
	struct polygondomain *getline();
	struct ivertex tp;
	int d;
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Possiblepath: entry\n");
#endif DEBUG
	d = getnearcut(obj, plgd, ref, &tp);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Possiblepath: after getnearcut (%d)\n",d);
#endif DEBUG
	/*
	 * If there is a shorcut available use it
	 * otherwise use density analyse
	 */
	if(d <= 37){			/* was d < 37 */
		*path = getline(ref, &tp);
		*g = boundgrey(obj, *path);
		seepoly(*path);
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Possiblepath: after getline, boundgrey, seepoly\n",d);
#endif DEBUG
		return(1);
	} else {
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Possiblepath: before pathsearch\n");
#endif DEBUG
		return(pathsearch(obj, cut, &tp, path, g));
	}
}

/*
 * find a straight line which join these two
 * points and change it to polygon
 */
struct polygondomain *getline(p0, p1)
register struct ivertex *p0, *p1;
{
	struct polygondomain *plgd, *makepolydmn();
	register struct ivertex *ivt;

	ivt = (struct ivertex *) Malloc(sizeof(struct ivertex) * 2);
	ivt->vtY = p0->vtY;
	ivt->vtX = p0->vtX;
	ivt++;
	ivt->vtY = p1->vtY;
	ivt->vtX = p1->vtX;
	ivt--;
	plgd = makepolydmn(1, ivt, 2, 3, 1);
	Free(ivt);
	return(plgd);
}

/*
 * dis: the distance among the three consecutive points
 * which form the angle
 * If the angle < pi return 1
 * If it is pi/2 return 0
 * If it bigger than pi return -1
 * x is tan(angle) * 256
 */
anglex(wtx, x, dis)
register struct ivertex *wtx;
int *x;
{
	register struct ivertex *wtx1, *wtx2;
	register int h1, d1, h2, d2;
	int i, k;
	
	wtx1 = wtx - 1;
	wtx2 = wtx + 1;
	while(absdis(wtx1, wtx) < dis)
		wtx1--;
	while(absdis(wtx2, wtx) < dis)
		wtx2++;
	h1 = wtx->vtY - wtx1->vtY;
	d1 = wtx->vtX - wtx1->vtX;
	h2 = wtx2->vtY - wtx->vtY;
	d2 = wtx2->vtX - wtx->vtX;
	i = d2 * h1 - d1 * h2;
	k = h1 * h2 + d1 * d2;
	if(i <= 0)
		return(-1);
	if(k == 0){
		*x = 0;
		return(0);
	}
	*x = - roundivid(i * FTOI, k);
	return(1);
}
			
/*
 * squar distanece between X(address)
 * and Y(address)
 */

squardis(X, Y)
register struct address *X, *Y;
{
	register int i, j;

	i = X->xcood - Y->xcood;
	j = X->ycood - Y->ycood;
	return(i * i + j * j);
}

/*
 * squar distanece between X(address)
 * and Y(ivetex)
 */
squardandi(X, Y)
register struct address *X;
register struct ivertex *Y;
{
	register int i, j;

	i = X->xcood - Y->vtX;
	j = X->ycood - Y->vtY;
	return(i * i + j * j);
}

/*
 * squar distanece between X(address)
 * and Y(address)
 */
squardisi(X, Y)
register struct ivertex *X, *Y;
{
	register int i, j;

	i = X->vtY - Y->vtY;
	j = X->vtX - Y->vtX;
	return(i * i + j * j);
}

/* 
 * if angle 1 > angle 2 return 1,
 *	      =                0,
 *	      <               -1
 */
cmpangl(alph1, alph2)
register int alph1, alph2;
{
	if (alph1 == alph2) 
		return(0);
	if(alph1 == 0){
		if(alph2 > 0)
			return(1);
		else return(-1);
	} else if(alph1 > 0){
		if(alph2 > alph1 || alph2 <= 0)
			return(-1);
		else return(1);
	} else {
		if(alph2 >= 0 || alph2 < alph1)
			return(1);
		else return(-1);
	}
}

inside(obj, l, k)
struct object *obj;
register int l, k;
{
	register struct intervaldomain *idmn = obj->idom;
	register struct interval *intl;
	register struct intervalline *itvl;
	register int n, kol;

	if (l < idmn->line1 || l > idmn->lastln)
		return(0);
	itvl = &idmn->intvlines[l - idmn->line1];
	n = itvl->nintvs;
	intl = itvl->intvs;
	kol = k - idmn->kol1;
	while (n-- > 0) {
		if(kol >= intl->ileft && kol <= intl->iright){
			return(1);
		}
		intl++;
	}
	return(0);
}

/*
 * if an angle is less than thresh, it is small
 */
smallangl(plgd, anglist, thresh)
struct polygondomain *plgd;
struct angladd *anglist;
int thresh;
{
	register struct ivertex *wtx;
	register struct ivertex *ptr, *tmp;
	register int i, j, k, l;
	int n, m = 3;
	int ang = -1;
	int x, y, z;
	/*
	 * if there are continuous small angles flag = 1
	 * (among them only choose one smallest angle)
	 * if the distance between two succissive small angles
	 * is greater than 3 pixels, interpolate one and sign = 1
	 */
	int sign = 0;

	y = ang;
	z = y;
	wtx = plgd->vtx;
	n = plgd->nvertices;
	if( n > 12){
		wtx += 6;
	} else return(0);
	/*
	 * first, decide hwo many points should be concerned 
	 * then calculate the angle
	 * final, campare the angles, find the smallest one
	 */
	for(i = 0; i < n - 12; i ++){
		k = anglex(wtx, &x, ANGPIXEL);
		if(k >= 0 && cmpangl(x, thresh) < 0){
			if(cmpangl(x, z) < 0){
				z = x;
				if(sign == 0)
					tmp = wtx;
				if(cmpangl(x, y) < 0){
					y = x;
					ptr = wtx;
					j = m;
				}
			}
			sign++;
		} else {
			if(sign > 0){
				l = sign % 2;
				sign >>= 1;
				tmp += sign;
				if(l == 0){
					anglex(tmp, &x, ANGPIXEL);
					if(cmpangl(z, x) != 0)
						tmp--;
				}
				anglist->angl[m++] = tmp;
				z = ang;
				sign = 0;
			}
		} 
		wtx++;
	}
	if(sign > 0){
		l = sign % 2;
		sign >>= 1;
		tmp += sign;
		if(l == 0){
			anglex(tmp, &x, ANGPIXEL);
			if(cmpangl(z, x) != 0)
				tmp--;
		}
		anglist->angl[m++] = tmp;
	}
	if(m == 3)
		return(0);
	sign = 0;
	k = -1;
	y = -1;
	for(i = 0; i < 2; i++){
		tmp = ptr + k;
			if(anglex(tmp, &x, ANGPIXEL) >= 0 && cmpangl(x, thresh) < 0){
				if(cmpangl(x, y) < 0){
					if(squardisi(ptr, tmp) > 3){
						anglist->small[0].vtY = (tmp->vtY+ptr->vtY) / 2;
						anglist->small[0].vtX = (tmp->vtX+ptr->vtX) / 2;
					} else {
					       anglist->small[0].vtY = ptr->vtY;
					       anglist->small[0].vtX = ptr->vtX;
					}
					y = x;
					sign++;
				}
			}
		k = 1;

	}
	anglist->angl[1] = ptr;
	anglist->angl[m] = anglist->angl[j];
	for(i = j; i < m; i++)
		anglist->angl[i] = anglist->angl[i + 1];
	if(sign == 0){
		anglist->small[0].vtY = ptr->vtY;
		anglist->small[0].vtX = ptr->vtX;
	}
	j = -1;
	for(i = 0; i < 3; i += 2){
		tmp = ptr + j;
		while(absdis(&anglist->small[0], tmp) < 6)
			tmp += j;
		anglist->angl[i] = tmp;
		j = 1;
	}
	anglist->num = m;
	if(y == ang){
#ifdef DEBUG
if (debug_channel != NULL)
fprintf(debug_channel,"Smallangl: no cut point\n");
#endif DEBUG
		return(0);
	}
	return(1);
}


absdis(ptr0, ptr1)
register struct ivertex *ptr0, *ptr1;
{
	register int i, j;

	i = abs(ptr0->vtY - ptr1->vtY);
	j = abs(ptr0->vtX - ptr1->vtX);
	return(i + j);
}

/*
 * check if one of the 8 neighbounrs is in the obj
 * if it is, return 1 otherwise return 0
 */
neighb8in(obj, l, k)
struct object *obj;
int l, k;
{
	register struct intervaldomain *idmn = obj->idom;
	register struct intervalline *itvl;
	register struct interval *intl;
	register i, line, kol, n;
	int j;

	j = 0;
	line = l - 1;
	kol = k - idmn->kol1;
	for(i = 0; i < 3; i++){
		if(line < idmn->line1)
			;
		else if(line > idmn->lastln)
			break;
		else {
			itvl = &idmn->intvlines[line - idmn->line1];
			n = itvl->nintvs;
			intl = itvl->intvs;
			while (n-- > 0) {
				if(kol <= intl->iright + 1 && kol >= intl->ileft - 1){
					j++;
					break;
				}
				intl++;
			}
			if(j != 0)
				return(1);
		}
		line++;
	}
	return(0);
}

