/*
 * mftrain.c		Jim Piper			30-01-87
 *
 * C O P Y R I G H T  (C)  Medical Research Council 1987
 *
 * Train the metaphase finder classifier parameters according
 * to Andrew Carother's note from October 1986.
 *
 * Synoposis:
 *		mftrain -i inputdata  -t traindata
 * Requires input and output files, since dialogue with terminal
 * to establish the translation from the "junk - unusable - usable"
 * coding to utility values.
 */
#include <stdio.h>
#include <slides.h>
#define NPARS 6

main(argc,argv)
char **argv;
{
	double y,util[3],x[NPARS+1],xtx[NPARS+1][NPARS+1],xty[NPARS+1];
	double sqrtdet, *dtoutriang(), *dfromutriang(), *xut, *xtxi;
	struct clustdat m;
	FILE *indata,*traindata;
	register int i,j,nrec;
	int error;

	setbuf(stderr,NULL);
	indata = NULL;
	traindata = NULL;	
	while (argc > 1 && argv[1][0] == '-') {
		switch (argv[1][1]) {
			case 'i':				/* input file name */
				indata = fopen(argv[2],"r");
				if (indata == NULL) {
					fprintf(stderr,"Cannot open input file %s\n",argv[2]);
					exit(1);
				}
				argc--;
				argv++;
				break;
			case 't':				/* training data file name */
				traindata = fopen(argv[2],"w");
				if (traindata == NULL) {
					fprintf(stderr,"Cannot open training data file %s\n",argv[2]);
					exit(1);
				}
				argc--;
				argv++;
				break;
			default:
				fprintf(stderr,"unknown flag %s\n",argv[1]);
				exit(1);
		}
		argc--;
		argv++;
	}
	if (indata == NULL || traindata == NULL) {
		fprintf(stderr,"MUST specify both input and training data files\n");
		exit(1);
	}
	
	/*
	 * establish the translation from "junk - unusable - usable"
	 */
	printf("The training procedure coded metaphases as one of");
	printf(" JUNK, UNUSABLE, USABLE\n");
	printf("Please give each of these classifications a 'utility'");
	printf(" rating in the range 1 to 5\n");
	printf("What utility value should JUNK have (suggest 0)? ");
	fflush(stdout);
	scanf("%F",&util[0]);
	printf("What utility value should USABLE have (suggest 5)? ");
	fflush(stdout);
	scanf("%F",&util[2]);
	printf("What utility value should UNUSABLE have ? ");
	fflush(stdout);
	scanf("%F",&util[1]);
	printf("You chose : JUNK %3.1f, USABLE %3.1f, UNUSABLE %3.1f\n",
		util[0],util[2],util[1]);
	
	/*
	 * Clear data arrays
	 */
	for (i=0; i<=NPARS; i++) {
		xty[i] = 0.0;
		for (j=0; j<=NPARS; j++)
			xtx[i][j] = 0.0;
	}
	
	/*
	 * skip the header record
	 */
	for (i=0; i<0x90; i++)
		getc(indata);
	/*
	 * loop, reading data, filling arrays
	 */
	nrec = 0;
	while (fread(&m,1,sizeof(struct clustdat),indata) != EOF) {
		fprintf(stderr,".");
		if (nrec++ == 60) {
			nrec = 0;
			fprintf(stderr,"\n");
		}
		y = util[m.quality];
		x[0] = 1.0;
		x[1] = (double) m.nobj;
		x[2] = (double) m.sf;
		x[3] = (double) m.aobj;
		x[4] = (double) m.octa;
		x[5] = (double) m.dv;
		x[6] = (double) m.skt / m.meanod;
		
		/*
		 * fill xty (X'.Y accumulator) and xtx (X'.X accumulator)
		 */
		for (i=0; i<=NPARS; i++) {
			xty[i] += y*x[i];
			for (j=0; j<=i; j++)
				xtx[i][j] += x[i]*x[j];
		}
	}
	
	/*
	 * make xtx symetrical
	 */
	for (i=1; i<=NPARS; i++)
		for (j=0; j<i; j++)
			xtx[j][i] = xtx[i][j];

	/*
	 * diagnostic printing
	 */
	printf("X'X matrix\n");
	dmprint(xtx,NPARS+1,NPARS+1,stdout);
	printf("X'Y matrix\n");
	dmprint(xty,1,NPARS+1,stdout);

	/*
	 * invert X'X
	 */
	xut = dtoutriang(xtx,NPARS+1);
	sinv(xut,NPARS+1,0.00001,&error,&sqrtdet);
	printf("Inverted matrix (error %d) (sqrtdet %f)\n",error,sqrtdet);
	xtxi = dfromutriang(xut,NPARS+1);
	dmprint(xtxi,NPARS+1,NPARS+1,stdout);
}
