/*
 * confusion.c		Jim Piper	13/4/84
 *
 * classifier confusion matrices for multiple classifications.
 * Can have up to MAXTAB tables.  Data then consists of one correct class
 * and MAXTAB assigned classes per line.
 */

#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>

#define NCLASS 25
#define MAXTAB 5

char *split = "|  | |      |  |  | |   ||";
int cpos[NCLASS] = {0,1,2,3,4,5,6,7,8,9,10,11,22,12,13,14,15,16,17,18,19,20,21,23,24};
int dclass[NCLASS] = {0,0,0,1,1,2,2,2,2,2,2,2,3,3,3,4,4,4,5,5,6,6,2,6,7};

main(argc,argv)
char **argv;
{
	int i,j,k,n,nclass,ndim,ntab,assigned[MAXTAB], correct;
	int clasi,clasj,nc,numcorrect[MAXTAB],numwrong[MAXTAB];
	int scatter[MAXTAB][MAXCLASS][MAXCLASS],dnumcorrect[MAXTAB],total;
	float correctpc[MAXTAB],dcorrectpc[MAXTAB];
	FILE *proto;

	/* set dimensions from proto file ? */
	nclass = NCLASS;
	if (argc > 2 && argv[1][0] == '-' && argv[1][1] == 'P') {
		proto = fopen(argv[2],"r");
		if (proto == NULL) {
			fprintf(stderr,"-P can't open file %s\n",argv[2]);
			exit(1);
		}
		argc -= 2;
		argv += 2;
		fscanf(proto,"%d %d\n",&nclass,&ndim);
		fscanf(proto,"%s",split);
		split[0] = '!';
		for (i=0; i<nclass; i++)
			if (split[i] != '!')
				split[i] = ' ';
		for (i=0; i<nclass; i++) {
			cpos[i] = i;
		}
		fclose(proto);
	}
	/* Number of tables - default is one table */
	ntab = 1;
	if (argc > 1 && argv[1][0] == '-')
		ntab = atoi(&argv[1][1]);
	if (ntab > MAXTAB) {
		fprintf(stderr,"Too many tables - recompile program\n");
		exit(1);
	}
	/* read correct, assigned class pairs */
	for (i=0; i<ntab; i++) {
		numcorrect[i] = 0;
		numwrong[i] = 0;
		dnumcorrect[i] = 0;
	}
	for (i=0; i<nclass; i++)
		for (j=0; j<nclass; j++)
			for (k=0; k<ntab; k++)
				scatter[k][i][j] = 0;
	while (scanf("%d",&correct) != EOF) {
		if (correct <= 0 || correct > nclass)
			correct = nclass;
		for (i=0; i<ntab; i++) {
			scanf("%d",&assigned[i]);
			if (assigned[i] <= 0 || assigned[i] > nclass)
				assigned[i] = nclass;
			if (correct == assigned[i])
				numcorrect[i]++;
			else
				numwrong[i]++;
			if (dclass[assigned[i]-1] == dclass[correct-1])
				dnumcorrect[i]++;
			scatter[i][correct-1][assigned[i]-1]++;
		}
	}
	for (n=0; n<ntab; n++) {
		printf("\n\n\nConfusion matrix - row: correct class, column: assigned class\nclass   ");
		for (i=0; i<nclass; i++)
			printf("%2d  ",cpos[i]+1);
		printf("total\n");
		for (clasi=0; clasi<nclass; clasi++) {
			i = cpos[clasi];
			if (split[i] != ' ') {
				for (j=0; j<=nclass*4+12; j++)
					putchar('-');
				putchar('\n');
			}
			printf("%4d |",i+1);
			nc = 0;
			for (clasj=0; clasj<nclass; clasj++) {
				j = cpos[clasj];
				nc += scatter[n][i][j];
				printf("%c%3d",split[j],scatter[n][i][j]);
			}
			printf("||%5d\n",nc);
		}
		for (j=0; j<=nclass*4+12; j++)
			putchar('-');
		printf("\ntotal|");
		for (clasj=0; clasj<nclass; clasj++) {
			j = cpos[clasj];
			nc = 0;
			for (i=0; i<nclass; i++)
				nc += scatter[n][i][j];
			printf("%c%3d",split[j],nc);
		}
		total = numcorrect[n]+numwrong[n];
		printf("||%5d\n",total);

		correctpc[n] = 100.0*numcorrect[n]/total;
		dcorrectpc[n] = 100.0*dnumcorrect[n]/total;
		printf("\n\n---------------------------------------------\n");
		printf("%6d chromosomes\n",total);
		printf("%6d (%5.1f percent) correct (24 classes)\n",numcorrect[n],correctpc[n]);
		printf("%6d (%5.1f percent) correct (7 classes)\n",dnumcorrect[n],dcorrectpc[n]);
		printf("---------------------------------------------\n");
		printf("\f\n");
	}
}
