/*
 * dmatrix.c		Jim Piper		26-01-86
 *
 * C O P Y R I G H T  (C)  Medical Research Council 1987
 */
#include <stdio.h>

/*
 * transpose x[a][b] into t[b][a]
 */
double *dtranspose(x,a,b)
register int a,b;
register double *x;
{
	register int i,j;
	register double *t;
	double *tb;

	tb = t = (double *) malloc(a*b*sizeof(double));
	for (j=0; j<b; j++)
		for (i=0; i<a; i++)
			*t++ = x[i*b+j];
	return(tb);
}

/*
 * multiply x[a][b] and y[b][c] to produce m[a][c]
 */
double *dmult(x,a,b,y,c)
register double *x,*y;
register int b;
int a,c;
{
	register int i,j,k;
	register double *m;
	double *mb;

	m = mb = (double *) malloc(a*c*sizeof(double));
	for (i=0; i<a; i++)
		for (k=0; k<c; k++) {
			*m = 0.0;
			for (j=0; j<b; j++)
				*m += x[i*b+j]*y[j*c+k];
			m++;
		}
	return(mb);
}

/*
 * print matrix x[a][b] to stream
 */
dmprint(x,a,b,f)
register double *x;
register int a,b;
FILE *f;
{
	register int i,j;
	fprintf(f,"Printing %d row by %d column matrix\n\n",a,b);
	for (i=1; i<=a; i++) {
		fprintf(f,"R%4d:",i);
		for (j=1; j<=b; j++) {
			fprintf(f,"%11.5f ",*x++);
			if (j != b && j%6 == 0)
				fprintf(f,"\n      ");
		}
		fprintf(f,"\n");
	}
	fprintf(f,"\n");
}

/*
 * convert symmetric matrix x[a][a] to upper triangular matrix ut
 */
double *dtoutriang(x,a)
register double *x;
register int a;
{
	register double *ut;
	double *utb;
	register int i,j;

	ut = utb = (double *) malloc((a*(a+1)/2) * sizeof(double));
	for (i=0; i<a; i++)
		for (j=0; j<=i; j++)
			*ut++ = x[i*a+j];
	return(utb);
}

/*
 * convert upper triangular matrix ut size a to symmetric matrix x[a][a]
 */
double *dfromutriang(ut,a)
register double *ut;
register int a;
{
	register double *x;
	register int i,j;

	x = (double *) malloc(a*a * sizeof(double));
	for (i=0; i<a; i++)
		for (j=0; j<=i; j++)
			x[i*a+j] = x[j*a+i] = *ut++;
	return(x);
}
