/*
 * mftest.c		Jim Piper			30-01-87
 *
 * C O P Y R I G H T  (C)  Medical Research Council 1987
 *
 * Test the metaphase finder classifier parameters computed
 * by program "mftrain" by comparing operator classification
 * with computed utility value.
 *
 * Synoposis:
 *		mftest -i inputdata  -t traindata
 */
#include <stdio.h>
#include <slides.h>
#define NPARS 6

/*
 * The following in kyy.h, but I don't know what else I have
 * to include, and kyy.h fails to compile by itself.......
 */
#define JUNK	2
#define USABLE	4
#define UNUMET	8

main(argc,argv)
char **argv;
{
	double ahat[NPARS+1],y,x[NPARS+1];
	double util_junk, util_usable, util_unusable;
	struct clustdat m;
	FILE *indata,*traindata;
	register int i,j,nrec;
	int error, njunk, nusable, nunusable, npars;

	setbuf(stderr,NULL);
	indata = NULL;
	traindata = NULL;	
	while (argc > 1 && argv[1][0] == '-') {
		switch (argv[1][1]) {
			case 'i':				/* input file name */
				indata = fopen(argv[2],"r");
				if (indata == NULL) {
					fprintf(stderr,"Cannot open input file %s\n",argv[2]);
					exit(1);
				}
				argc--;
				argv++;
				break;
			case 't':				/* training data file name */
				traindata = fopen(argv[2],"r");
				if (traindata == NULL) {
					fprintf(stderr,"Cannot open training data file %s\n",argv[2]);
					exit(1);
				}
				argc--;
				argv++;
				break;
			default:
				fprintf(stderr,"unknown flag %s\n",argv[1]);
				exit(1);
		}
		argc--;
		argv++;
	}
	if (indata == NULL || traindata == NULL) {
		fprintf(stderr,"MUST specify both input and training data files\n");
		exit(1);
	}
	
	/*
	 * read the training parameters
	 */
	fscanf(traindata,"%F %F %F",&util_junk,&util_usable,&util_unusable);
	fscanf(traindata,"%d",&npars);	/* NPARS+1 */
	if (npars != NPARS+1) {
		fprintf(stderr,"Wrong number of parameters in training data file\n");
		exit(1);
	}
	for (i=0; i<=NPARS; i++)
		fscanf(traindata,"%F",&ahat[i]);
	
	/*
	 * skip the header record
	 */
	for (i=0; i<0x90; i++)
		getc(indata);
	/*
	 * loop, reading data, filling arrays
	 */
	nrec = njunk = nusable = nunusable = 0;
	printf("Reading data file\n");
	while (fread(&m,1,sizeof(struct clustdat),indata) == sizeof(struct clustdat)) {
		nrec++;
		x[0] = 1.0;
		x[1] = (double) m.nobj;
		x[2] = (double) m.sf;
		x[3] = (double) m.aobj;
		x[4] = (double) m.octa;
		x[5] = (double) m.dv;
		x[6] = (double) m.skt;
		x[6] /= (double) m.meanod;
		
		/*
		 * Compute the utility
		 */
		y = 0.0;
		for (i=0; i<=NPARS; i++)
			y += ahat[i]*x[i];
		printf("%5.1f ",y);
		switch (m.fclass) {
			case JUNK :
				printf("Junk\n");
				njunk++;
				break;
			case USABLE :
				printf("USABLE\n");
				nusable++;
				break;
			case UNUMET :
				printf("Unusable\n");
				nunusable++;
				break;
			default:
				printf("Bad data\n");
				continue;
		}
	}
	printf("\nData file contained %d junk, %d usable, %d unusable\n",
			njunk,nusable,nunusable);
}
