/*
 * mftrain.c		Jim Piper			30-01-87
 *
 * C O P Y R I G H T  (C)  Medical Research Council 1987
 *
 * Train the metaphase finder classifier parameters according
 * to Andrew Carother's note from October 1986.
 *
 * Synoposis:
 *		mftrain -i inputdata  -t traindata
 * Requires input and output files, since dialogue with terminal
 * to establish the translation from the "junk - unusable - usable"
 * coding to utility values.
 */
#include <stdio.h>
#include <slides.h>
#define NPARS 6

/*
 * The following in kyy.h, but I don't know what else I have
 * to include, and kyy.h fails to compile by itself.......
 */
#define JUNK	2
#define USABLE	4
#define UNUMET	8

main(argc,argv)
char **argv;
{
	double *ahat,y,x[NPARS+1],xtx[NPARS+1][NPARS+1],xty[NPARS+1];
	double util_junk, util_usable, util_unusable;
	double sqrtdet, *dtoutriang(), *dfromutriang(), *xut, *xtxi;
	double *dmult();
	struct clustdat m;
	FILE *indata,*traindata;
	register int i,j,nrec;
	int error, njunk, nusable, nunusable;

	setbuf(stderr,NULL);
	indata = NULL;
	traindata = NULL;	
	while (argc > 1 && argv[1][0] == '-') {
		switch (argv[1][1]) {
			case 'i':				/* input file name */
				indata = fopen(argv[2],"r");
				if (indata == NULL) {
					fprintf(stderr,"Cannot open input file %s\n",argv[2]);
					exit(1);
				}
				argc--;
				argv++;
				break;
			case 't':				/* training data file name */
				traindata = fopen(argv[2],"w");
				if (traindata == NULL) {
					fprintf(stderr,"Cannot open training data file %s\n",argv[2]);
					exit(1);
				}
				argc--;
				argv++;
				break;
			default:
				fprintf(stderr,"unknown flag %s\n",argv[1]);
				exit(1);
		}
		argc--;
		argv++;
	}
	if (indata == NULL || traindata == NULL) {
		fprintf(stderr,"MUST specify both input and training data files\n");
		exit(1);
	}
	
	/*
	 * establish the translation from "junk - unusable - usable"
	 */
	printf("The training procedure coded metaphases as one of");
	printf(" JUNK, UNUSABLE, USABLE\n");
	printf("Please give each of these classifications a 'utility'");
	printf(" rating in the range 1 to 5\n");
	printf("What utility value should JUNK have (suggest 0)? ");
	fflush(stdout);
	scanf("%F",&util_junk);
	printf("What utility value should USABLE have (suggest 5)? ");
	fflush(stdout);
	scanf("%F",&util_usable);
	printf("What utility value should UNUSABLE have ? ");
	fflush(stdout);
	scanf("%F",&util_unusable);
	printf("You chose : JUNK %3.1f, USABLE %3.1f, UNUSABLE %3.1f\n",
		util_junk,util_usable,util_unusable);
	
	/*
	 * Clear data arrays
	 */
	for (i=0; i<=NPARS; i++) {
		xty[i] = 0.0;
		for (j=0; j<=NPARS; j++)
			xtx[i][j] = 0.0;
	}
	
	/*
	 * skip the header record
	 */
	for (i=0; i<0x90; i++)
		getc(indata);
	/*
	 * loop, reading data, filling arrays
	 */
	nrec = njunk = nusable = nunusable = 0;
	printf("Reading data file\n");
	while (fread(&m,1,sizeof(struct clustdat),indata) == sizeof(struct clustdat)) {
		nrec++;
		/*
		 * The metaphase selection coding (junk, usable, etc..) is bizarre
		 */
		switch (m.fclass) {
			case JUNK :
				y = util_junk;
				njunk++;
				break;
			case USABLE :
				y = util_usable;
				nusable++;
				break;
			case UNUMET :
				y = util_unusable;
				nunusable++;
				break;
			default:
				fprintf(stderr,"*");
				if (nrec%60 == 0)
					fprintf(stderr,"\n");
				continue;
		}
		fprintf(stderr,".");
		if (nrec%60 == 0)
			fprintf(stderr,"\n");
		x[0] = 1.0;
		x[1] = (double) m.nobj;
		x[2] = (double) m.sf;
		x[3] = (double) m.aobj;
		x[4] = (double) m.octa;
		x[5] = (double) m.dv;
		x[6] = (double) m.skt;
		x[6] /= (double) m.meanod;
		
		/*
		 * fill xty (X'.Y accumulator) and xtx (X'.X accumulator)
		 */
		for (i=0; i<=NPARS; i++) {
			xty[i] += y*x[i];
			for (j=0; j<=i; j++)
				xtx[i][j] += x[i]*x[j];
		}
	}
	printf("\nData file contained %d junk, %d usable, %d unusable\n",
			njunk,nusable,nunusable);
	
	/*
	 * make xtx symetrical
	 */
	for (i=1; i<=NPARS; i++)
		for (j=0; j<i; j++)
			xtx[j][i] = xtx[i][j];

	/*
	 * diagnostic printing
	 */
#ifdef DEBUG
	printf("X'X matrix\n");
	dmprint(xtx,NPARS+1,NPARS+1,stdout);
	printf("X'Y matrix\n");
	dmprint(xty,1,NPARS+1,stdout);
#endif DEBUG

	/*
	 * invert X'X
	 */
	xut = dtoutriang(xtx,NPARS+1);
	sinv(xut,NPARS+1,0.00001,&error,&sqrtdet);
	xtxi = dfromutriang(xut,NPARS+1);
#ifdef DEBUG
	printf("Inverted matrix (error %d) (sqrtdet %f)\n",error,sqrtdet);
	dmprint(xtxi,NPARS+1,NPARS+1,stdout);
#endif DEBUG

	/*
	 * construct Ahat by multiplication of inverted matrix and X'Y
	 */
	 ahat = dmult(xtxi,NPARS+1,NPARS+1,xty,1);
	 printf("Ahat - least squares estimate of A\n");
	 dmprint(ahat,NPARS+1,1,stdout);
	 
	 /*
	  * write training-data file
	  */
	 fprintf(traindata,"%f %f %f\n",util_junk,util_usable,util_unusable);
	 fprintf(traindata,"%d\n",NPARS+1);
	 for (i=0; i<=NPARS; i++)
	 	fprintf(traindata,"%f\n",ahat[i]);
}
