/*
 * vcv.c	Jim Piper	23/3/84
 *
 **************************************************************************
 * OS9 VERSION HAS REDUCED FUNCTIONALITY - IN PARTICULAR, NO COVARIANCES
 **************************************************************************
 *
 * Compute variance-covariance matrix from input data vectors produced,
 * e.g., by program "chromnorm".
 * Also means, standard deviations for classes/features, and
 * class-class t statistics.
 *
 * Flags -t, -s, -S, -d turn on four modes:
 *	(1) -t classfile: classifier training, data written to classfile
 *	(2) -s: summary statistics of features and classes
 *	(3) -S:	more statistics.
 *	(4) -d: utilise data from derived objects (class coded +100)
 * Flags -G, -M, -C, -A  set five alternative methods of feature selection:
 *	(1) default - selected by hardwired program
 *	(2) -G n t	Granum's "SEPCOR" with n features, "MAXCOR" t.
 *	(3) -M n	Piper's "MODSEPCOR" with n features.
 *	(4) -C n f1 f2 ... fn Feature read from command line.
 *	(5) -A All features used.
 * Flag -P <filename> - read number of classes, features from <filename>
 */

#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <math.h>


char *split = "   ! !       !  !  ! !  !  ";
int cpos[24] = {0,1,2,3,4,5,6,7,8,9,10,11,22,12,13,14,15,16,17,18,19,20,21,23};

#ifndef MIZAR
double clvcv[MAXCLASS][MAXDIM*(MAXDIM+1)/2];
double clcp[MAXCLASS][MAXDIM][MAXDIM];
double correl2[MAXDIM][MAXDIM];
double correl3[MAXDIM][MAXDIM];
double vcv[MAXDIM*(MAXDIM+1)/2];
double clsqrtdet[MAXCLASS];
#endif MIZAR
double correl[MAXDIM][MAXDIM];
double clm[MAXCLASS][MAXDIM],clsd[MAXCLASS][MAXDIM];
double cls[MAXCLASS][MAXDIM];
double clss[MAXCLASS][MAXDIM];

main(argc,argv)
char **argv;
{
	int clasi,clasj,ndim,g,h,i,j,k,err,sinverr;
	int nsamp,class,nclass,clnsamp[MAXCLASS];
	int self[MAXDIM],nself,selfsort();
	int feature_selection_method,TRAIN,STATS,DERIV;
	double maxcor;
	double v[MAXDIM];
	double ss,t,msd,cc,cd;
	double cp[MAXDIM][MAXDIM],s[MAXDIM],m[MAXDIM],psd[MAXDIM];
	double V[MAXDIM][8];
	FILE *classfile, *proto;

	/* mode switches */
	TRAIN = 0;
	STATS = 0;
	DERIV = 0;
	proto = NULL;
	feature_selection_method = 0;
	while (argc > 1 && argv[1][0] == '-') {
		switch(argv[1][1]) {
		case 't':
			TRAIN++;
			if (argc > 2) {
				classfile = fopen(argv[2],"w");
				if (classfile == NULL) {
					fprintf(stderr,"Cannot open file %s for storing classifier\n",argv[2]);
					exit(2);
				}
				argc--;
				argv++;
			} else
				classfile = stdout;
			break;
		case 'd':
			DERIV++;
			break;
		case 's':
			STATS++;
			break;
		case 'S':
			STATS += 2;
			break;
		case 'G':
			feature_selection_method = 1;
			nself = atoi(argv[2]);
			maxcor = (double)atof(argv[3]);
			argc -= 2;
			argv += 2;
			break;
		case 'M':
			feature_selection_method = 2;
			nself = atoi(argv[2]);
			argc--;
			argv++;
			break;
		case 'C':
			feature_selection_method = 3;
			nself = atoi(argv[2]);
			argc--;
			argv++;
			for (i=0; i<nself; i++) {
				self[i] = atoi(argv[2]);
				argc--;
				argv++;
			}
			break;
		case 'A':
			feature_selection_method = 4;
			break;
		case 'P':
			proto = fopen(argv[2],"r");
			if (proto == NULL) {
				fprintf(stderr,"-P can't open file %s\n",argv[2]);
				exit(1);
			}
			argc--;
			argv++;
			fscanf(proto,"%d %d\n",&nclass,&ndim);
			fscanf(proto,"%s",split);
			for (i=0; i<nclass; i++)
				if (split[i] != '!')
					split[i] = ' ';
			for (i=0; i<nclass; i++) {
				cpos[i] = i;
			}
			fclose(proto);
			break;
		default:
			break;
		}
		argc--;
		argv++;
	}
	if (TRAIN==0 && STATS==0) {
		STATS++;
	}

	/* number of dimensions and classes */
	if (proto == NULL)
		ndim = getdim(STATS);
	if (ndim > MAXDIM) {
		printf("Too many dimensions\n");
		exit(1);
	}
	if (proto == NULL)
		nclass = getclass(STATS);
	if (nclass > MAXCLASS) {
		printf("Too many classes\n");
		exit(1);
	}

	/* clear arrays */
	nsamp = 0;
	for (i=0; i<ndim; i++) {
		s[i] = 0.0;
		for (j=0; j<ndim; j++)
			cp[j][i] = 0.0;
		for (class=0; class<nclass; class++) {
			clnsamp[class] = 0;
			cls[class][i] = 0.0;
#ifndef MIZAR
			for (j=0; j<ndim; j++)
				clcp[class][j][i] = 0.0;
#endif MIZAR
		}
	}

	/* get data vector, check class, check for derived objects */
	nsamp = 0;
	while (scanf("%d",&class) > 0) {
		class--;
		for (i=0; i<ndim; i++) {
#ifdef MIZAR
			scanf("%E",&v[i]);
#else MIZAR
			scanf("%le",&v[i]);
#endif MIZAR
		}
		if (DERIV != 0 && class >= 100)
			class -= 100;

	/* set up vector sum and matrix of sum of products */
		if (class >= 0 && class < nclass) {
			nsamp++;
			clnsamp[class]++;
			for (i=0; i<ndim; i++) {
				s[i] += v[i];
				cls[class][i] += v[i];
				clss[class][i] += v[i]*v[i];
				for (j=0; j<=i; j++) {
					cp[j][i] += v[i]*v[j];
#ifndef MIZAR
					clcp[class][j][i] += v[i]*v[j];
#endif MIZAR
				}
			}
		}
	}

	/* per feature, class-class F values and feature V-value (Granum) */
	/* F is difference of class means/mean of pair of class s.d.s */
	for (i=0; i<ndim; i++) {
		/* compute and print per class means, standard deviations */
		classmeansd(STATS,i,nclass,nsamp,clnsamp,clm,clsd,m,s,cls,clss);
		/* compute and print "f" and "V" values */
		classfandv(STATS,i,nclass,cpos,split,clnsamp,clm,clsd,V);
	}

	/* compute pooled standard deviation */
	poolsd(STATS,ndim,nsamp,m,cp,psd);
#ifndef MIZAR
	/* compute overall correlations between features */
	correlations(STATS,ndim,cp,nsamp,m,correl);
	/* and compute mean within-class correlation coefficient */
	withinclasscorrel(STATS,ndim,nclass,nsamp,clcp,clnsamp,clm,correl2);
	/* and compute weighted between-class-mean correlations coefficients */
	betweenclasscorrel(STATS,ndim,nclass,nsamp,clnsamp,clm,m,correl3);
#else MIZAR
	/* and compute weighted between-class-mean correlations coefficients */
	betweenclasscorrel(STATS,ndim,nclass,nsamp,clnsamp,clm,m,correl);
#endif MIZAR

	/* Print overall and per-Denver class V */
	if (STATS)
		Vtable(ndim,V);

	switch (feature_selection_method) {
	case 1:
		/* Granum's SEPCOR method of feature selection */
		sepcor(self,&i,V,correl,ndim,maxcor);
		if (i < nself) {
			printf("Number of features selected by SEPCOR %d is less than desired number %d\n",i,nself);
			nself = i;
		}
		break;

	case 2:
		/* Piper's modified sepcor, christened MODSEPCOR */
		modsepcor(self,nself,V,correl,ndim);
		break;

	case 3:
		/* Specified in command line */
		break;

	case 4:
		/* all features selected */
		nself = ndim;
		for (i=0; i<ndim; i++)
			self[i] = i;
		break;

	case 0:
	default:
		/* Hardwired selection of features */
		selectedfeatures(self,&nself);
		break;
	}

	/*
	 * compute variance-covariance matrices and invert for classifier.
	 * NOTE THAT "self" MUST BE IN ASCENDING ORDER SINCE
	 * "cp" AND "clcp" HAVE ONLY ONE TRIANGLE FILLED !!
	 */
	if (TRAIN) {
		qsort(self,nself,sizeof(int),selfsort);
		printf("------------------------ %2d features selected ----------------------\n",nself);
		for (i=0; i<nself; i++)
			printf("%3d",self[i]);
		printf("\n-------------------------------------------------------------------\n");
#ifndef MIZAR
		makevcv(nself,self,nclass,nsamp,clvcv,clcp,clnsamp,cp,clm,m,vcv);
		trainclass(nself,self,nclass,ndim,clvcv,clm,clsd,m,vcv,
			clsqrtdet,psd,classfile);
#else MIZAR
		trainclass(nself,self,nclass,ndim,clm,clsd,m,psd,classfile);
#endif MIZAR
	}
	exit(0);
}

selfsort(a,b)
int *a,*b;
{
	return(*a - *b);
}
