/*
 * vcvsub3.c	Jim Piper	17/9/84
 */

/* #define SQUARE		/* see "modsepcor" */
/* #define SQRT			/* see "modsepcor" */



#include <stdio.h>
#include <wstruct.h>
#include <chromanal.h>
#include <math.h>

#define abs(x) (x>=0? x: -x)

struct Vcp {
	float Vval;
	int fnum;
};


/*
 * Erik Granum's SEPCOR feature selection procedure
 */
sepcor(self,nself,V,correl,ndim,maxcor)
int *self, *nself;
double V[MAXDIM][8];
double correl[MAXDIM][MAXDIM];
double maxcor;
{
	int i,j,k;
	struct Vcp Vcopy[MAXDIM];
	int Vsort();
	for (i=0; i<ndim; i++) {
		Vcopy[i].Vval = V[i][0];
		Vcopy[i].fnum = i;
	}
	qsort(Vcopy,ndim,sizeof(struct Vcp),Vsort);
	printf("\n\nFeatures sorted by overall V value\n");
	for (i=0; i<ndim; i++)
		printf(" %2d",Vcopy[i].fnum);
	printf("\n\n\n");
	printf("SEPCOR selected features:\nMAXCOR                Features\n");
	self[0] = Vcopy[0].fnum;
	i = 1;
	for (j=1; j<ndim; j++) {
		for (k=0; k<i; k++) {
			if (abs(correl[self[k]][Vcopy[j].fnum]) >= maxcor)
				goto outsideloopend;
		}
		self[i++] = Vcopy[j].fnum;
outsideloopend:		;
	}
	*nself = i;
	printf("\n%6.2f:     ",maxcor);    
	for (j=0; j<i; j++)
		printf(" %2d",self[j]);
	printf("\n");
}



/*
 * Modified SEPCOR procedure MODSEPCOR.  Here the choice of next feature to
 * select is based on maximising measure 
 *	ADDSEP = V * (1 - max(correlation between feature and previously
 *				selected feature))
 ********************************************************************
 * #ifdef SQUARE then ADDSEP = V*(1-max*max)
 * #ifdef SQRT then ADDSEP = V*(1-sqrt(|max|))
 ********************************************************************
 */
modsepcor(self,nself,V,correl,ndim)
int *self;
double V[MAXDIM][8];
double correl[MAXDIM][MAXDIM];
{
	double corr, maxcor, sepcorval, maxsepcorval;
	int i,k,l,nextfeat,maxpos;
	struct Vcp Vcopy[MAXDIM];
	int Vsort();

	for (i=0; i<ndim; i++) {
		Vcopy[i].Vval = V[i][0];
		Vcopy[i].fnum = i;
	}
	qsort(Vcopy,ndim,sizeof(struct Vcp),Vsort);
	printf("\n\n\n\nfeatures sorted by V value:\n");
	for (i=0; i<ndim; i++)
		printf(" %2d",Vcopy[i].fnum);
	printf("\n\nMODSEPCOR selected features:\n");
	self[0] = Vcopy[0].fnum;
	printf(" %2d",self[0]);
	Vcopy[0].fnum = -1;
	for (i=1; i<nself; i++) {
		maxsepcorval = 0.0;
		for (k=1; k<ndim; k++) {
			/* feature already selected ?? */
			if (Vcopy[k].fnum < 0)
				continue;
			/*
			 * compute max correlation with
			 * previously selected feature
			 */
			maxcor = 0.0;
			for (l=0; l<i; l++) {
				corr = correl[self[l]][Vcopy[k].fnum];
				corr = abs(corr);
				if (corr > maxcor)
					maxcor = corr;
			}
			/* measure of how good this is as additional feature */
#ifdef SQRT
			sepcorval = (1.0 - sqrt(abs(maxcor))) * Vcopy[k].Vval;
#else SQRT
#ifdef SQUARE
			sepcorval = (1.0 - maxcor*maxcor) * Vcopy[k].Vval;
#else SQUARE
			sepcorval = (1.0 - maxcor) * Vcopy[k].Vval;
#endif SQUARE
#endif SQRT
			if (sepcorval > maxsepcorval) {
				maxsepcorval = sepcorval;
				nextfeat = k;
			}
		}
		self[i] = Vcopy[nextfeat].fnum;
		Vcopy[nextfeat].fnum = -1;
		printf(" %2d",self[i]);
	}
	printf("\n");
}



Vsort (a,b)
struct Vcp *a,*b;
{
	if (a->Vval > b->Vval)
		return(-1);
	else if (a->Vval == b->Vval)
		return(0);
	else
		return(1);
}


#ifdef MIZAR
getclass(print)
{
	if (print)
		printf("\n24 classes (22 autosomes, X = 23, Y = 24)\n");
	return(24);
}


/*
 * specify which features are to be used for classification.
 * Note that as usual the features are numbered from zero.
 * Program "vcv" REQUIRES SELECTED FEATURES TO BE IN ASCENDING ORDER !
 */
selectedfeatures(sfv,nsfv)
int *sfv,*nsfv;
{
	*nsfv = 3;
	sfv[0] = 0;
	sfv[1] = 9;
	sfv[2] = 15;
}

getdim(print)
{
	if (print) {
		printf("28 features (note numbering from zero) :\n\n");
		printf("0: area normalised by median object area\n");
		printf("1: length normalised by median object length\n");
		printf("2: normalised density (cell mean = 100)\n");
		printf("3: area centromeric index\n");
		printf("4: ratio of mass c.i. to area c.i.\n");
		printf("5: normalised cvdd\n");
		printf("6: normalised nssd\n");
		printf("7: normalised ratio mass to area c.i.\n");
		printf("8-13: normalised wdds\n");
		printf("14-19: normalised mwdds\n");
		printf("20-25: normalised gwdds\n");
		printf("26: length centromeric index\n");
		printf("27: hull perimeter\n");
	}
	return(28);
}

#endif MIZAR


