/*
 * interact.c	4/1/84		Jim Piper
 *
 * construct a polygon by keyboard or tablet
 * manipulation, displaying as we go along.
 *
 * Modifications
 *
 *	 7 Feb 1991		CAS		Mizar/OSK defines
 *	22 Nov 1988		dcb		Add string param to woolz_exit to say where called
 * 							from
 *	 2 Mar 1987		GJP		Woolz_exit
 *	12 Sep 1986		CAS		Includes
 *	23 Oct 1985		jp		Added extra parameter, char *term, to permit
 *							explicit naming of KEYBOARD or MOUSE port.
 */

#include <stdio.h>
#include <wstruct.h>
#ifndef OSK
#include <sgtty.h>
#else OSK
#include <sgstat.h>
#endif OSK
#include <interact.h>

#ifdef C68000
#include <dstat.h>
#endif C68000

#ifdef OSK
ggetc(in)
FILE *in;
{
	char c;
	read(fileno(in),&c,1);
	return((int)c);
}
#endif OSK

struct ivector arc = {30, 0, 0, 0, 0, 0};
struct ipoint point = {40, 0, 0, 0};


struct object *interactpoly(dev,f,sk,sl,term)
struct pframe *f;
char *term;
{
#ifdef OSK
	struct object *mousepoly();
#else OSK
	struct object *tabletpoly();
#endif OSK
	struct object *keyboardpoly();

	switch (dev) {
#ifdef OSK
	case MOUSE :
		fprintf(stderr,"interact with mouse\n");
		return(mousepoly(f,sk,sl,term));
#else OSK
	case TABLET :
		fprintf(stderr,"interact with tablet\n");
		return(tabletpoly(f,sk,sl));
#endif OSK

	default :

	case KEYBOARD :
		fprintf(stderr,"interact with keypad  -  ");
		return(keyboardpoly(f,sk,sl,term));
	}
}


/*
 * construct polygon by moving a point around with keypad keys.
 * "Fixing" the point makes a polygon vertex.
 * If polygon is abandoned, then return NULL
 * Codes:
 *	0		fix current point
 *	1-3,6,9-7,4	move in natural (as per keypad layout) direction
 *	<		"lower gear" - move less far per key stroke
 *	>		"higher gear" - move further per key stroke
 *	-		undo last polygon arc (can be iterated)
 *	E or ctrl-E	exit with currently displayed polygon
 *	ctrl-K		kill this interaction and return NULL
 */
static
struct object *keyboardpoly(f,startkol,startline,term)
struct pframe *f;
char *term;
{
	struct object *poly, *makemain();
	struct polygondomain *pdom, *makepolydmn();
	register mult;
	int i,j,k,l,kill,fix,fiy;
	char c;
#ifdef C68000
	BITS keepstatus;
	DSTAT dstat;
	FIO *in;
#else C68000
	FILE *in;
#endif C68000
#ifndef OSK
	struct sgttyb htty,ktty;
#else OSK
	struct sgbuf sgbuf;
#endif OSK

	fix = 1;
	if (f->ix < 0)
		fix = -1;
	fiy = 1;
	if (f->iy < 0)
		fiy = -1;
	kill = 0;
	pdom = makepolydmn(1,NULL,0,IPOLSIZE,1);
	poly = makemain(10,pdom,NULL,NULL,NULL);
	point.k = startkol;
	point.l = startline;
	mult = 4;
	colour(1);

#ifdef C68000
	in = &stdin;
	finit(in,STDIN,READ);
	gdev(STDIN,&dstat);
	keepstat = dstat.status;
	dstat.status = _RAW;
	sdev(STDIN,&dstat);
#else C68000
#ifdef OSK
	in = fopen(term,"r");
	setbuf(in,NULL);
	getstat(0,fileno(in),&sgbuf);
	sgbuf.sg_echo = 0;
	setstat(0,fileno(in),&sgbuf);
#else OSK
	in = fopen("/dev/tty","r");
	gtty (fileno(in),&htty);
	ktty.sg_flags = htty.sg_flags;
	htty.sg_flags |= RAW;
	htty.sg_flags &= ~ECHO;
	stty (fileno(in),&htty);
#endif OSK
#endif C68000
	fprintf(stderr,"movement scale %d\r\n",mult);
INITIALPOSITION:
	picframe(&point,f);
	/*
	 * fix start point
	 */
	while ((c = (char) (getc(in) & 127)) != '0') {
		/*
		 * delete preceding point if end point has moved
		 */
		if (c >= '1' && c <= '9' && c != '5') {
			intens(0);
			picframe(&point,f);
			intens(1);
		}
		switch (c) {
		default :
			putc('\007',stderr);
			break;

		case '?':
			helpmessage();
			break;

		case '>':
			mult *= 4;
			fprintf(stderr,"movement scale %d\r\n",mult);
			break;
		case '<':
			if (mult >= 4)
				mult /= 4;
			fprintf(stderr,"movement scale %d\r\n",mult);
			break;

		case 'E':
		case '\003':		/* ctrl C */
		case '\005':		/* ctrl E */
		case '\013':		/* ctrl K */
			kill++;
			goto EXIT;

		case '1':
			point.k -= mult * fix;
		case '2':
			point.l -= mult * fiy;
			break;

		case '3':
			point.l -= mult * fiy;
		case '6':
			point.k += mult * fix;
			break;

		case '7':
			point.l += mult * fiy;
		case '4':
			point.k -= mult * fix;
			break;

		case '9':
			point.k += mult * fix;
		case '8':
			point.l += mult * fiy;
			break;
		}
			picframe(&point,f);
	}
	/*
	 * fix subsequent points
	 */
	pdom->vtx[pdom->nvertices].vtX = point.k;
	pdom->vtx[pdom->nvertices].vtY = point.l;
	pdom->nvertices++;
	arc.k1 = arc.k2 = point.k;
	arc.l1 = arc.l2 = point.l;
	do {
		c = (char) (getc(in) & 127);
		/*
		 * delete preceding polygon if end point has moved
		 */
		if (c >= '1' && c <= '9' && c != '5') {
			intens(0);
			picframe(&arc,f);
			intens(1);
		}
		switch (c) {
		default :
			putc('\007',stderr);
			break;

		case '?':
			helpmessage();
			break;

		case '>':
			mult *= 4;
			fprintf(stderr,"movement scale %d\r\n",mult);
			break;
		case '<':
			if (mult >= 4)
				mult /= 4;
			fprintf(stderr,"movement scale %d\r\n",mult);
			break;

		case '\003':		/* ctrl C */
		case '\013':		/* ctrl K */
			intens(0);
			picframe(&arc,f);
			kill++;
			goto EXIT;

		case '-':
			intens(0);
			picframe(&arc,f);
			intens(1);
			arc.k2 = arc.k1;
			arc.l2 = arc.l1;
			if (pdom->nvertices > 1)
				pdom->nvertices--;
			else
				goto INITIALPOSITION;
			arc.k1 = pdom->vtx[pdom->nvertices-1].vtX;
			arc.l1 = pdom->vtx[pdom->nvertices-1].vtY;
			break;
			

		case '1':
			arc.k2 -= mult * fix;
		case '2':
			arc.l2 -= mult * fiy;
			break;

		case '3':
			arc.l2 -= mult * fiy;
		case '6':
			arc.k2 += mult * fix;
			break;

		case '7':
			arc.l2 += mult * fiy;
		case '4':
			arc.k2 -= mult * fix;
			break;

		case '9':
			arc.k2 += mult * fix;
		case '8':
			arc.l2 += mult * fiy;
			break;

		case '0':
		case 'E':
		case '\005':			/* ctrl E */
			if (arc.k1 != arc.k2 || arc.l1 != arc.l2) {
				arc.k1 = arc.k2;
				arc.l1 = arc.l2;
				pdom->vtx[pdom->nvertices].vtX = arc.k2;
				pdom->vtx[pdom->nvertices].vtY = arc.l2;
				pdom->nvertices++;
			}
			break;
		}
			picframe(&arc,f);
	} while (c != 'E' && c != '\005');
EXIT:
	intens(0);
	picframe(&point,f);
	picframe(poly,f);
	intens(1);
#ifdef C68000
	dstat.status = keepstat;
	sdev(STDIN,&dstat);
#else C68000
#ifndef OSK
	htty.sg_flags = ktty.sg_flags;
	stty (fileno(in),&htty);
#else OSK
	getstat(0,fileno(in),&sgbuf);
	sgbuf.sg_echo = 1;
	setstat(0,fileno(in),&sgbuf);
#endif OSK
	fclose(in);
#endif C68000
	if (kill) {
		freeobj(poly);
		poly = NULL;
	}
	return(poly);
}


helpmessage()
{
	fprintf(stderr,"? for help\r\n");
	fprintf(stderr,"> for greater movement scale, < for smaller\r\n");
	fprintf(stderr,"keypad keys 1,2,3,6,9,8,7,4 for movement\r\n");
	fprintf(stderr,"keypad key 0 to fix current vertex position\r\n");
	fprintf(stderr,"E to finish polygon and exit from this interaction\r\n");
	fprintf(stderr,"- to undo last position, + to undo an undoing (can be used\r\n      to insert a change part way along a polygon\r\n");
	fprintf(stderr,"ctrl-K to kill this interaction entirely\r\n");
}

#ifndef C68000
#ifndef OSK

#define PARITYMASK 017737677577

FILE *tblr;
struct sgttyb ttytblr = { B9600, B9600, 0, 0, RAW } ;

static
struct object *tabletpoly(f,startkol,startline)
struct pframe *f;
{
	struct object *poly, *makemain();
	struct polygondomain *pdom, *makepolydmn();
	int i,j,k,l;
	FILE *fopen();
	int button,x,y,xwas,ywas,xinc,yinc;

	if ((tblr=fopen("/dev/tty11", "r"))==NULL) {
		fprintf (stderr,"Cannot fopen() tablet. Exiting\n") ;
		woolz_exit(1, "tabletpoly") ;
	}
	stty (fileno(tblr), &ttytblr) ;

	pdom = makepolydmn(1,NULL,0,IPOLSIZE,1);
	poly = makemain(10,pdom,NULL,NULL,NULL);
	point.k = startkol;
	point.l = startline;
	colour(1);
	picframe(&point,f);
	/*
	 * fix start point
	 */
	tabletpoint(&button,&xwas,&ywas);
	do {
		while (tabletpoint(&button, &x, &y))
			;
		xinc = (x - xwas)/40;
		yinc = (y - ywas)/40;
		xwas = x;
		ywas = y;
		if (button == 1 && (xinc != 0 || yinc != 0)) {
			intens(0);
			picframe(&point,f);
			intens(1);
			point.k += xinc;
			point.l += yinc;
			picframe(&point,f);
		}
	} while (button == 1);
	/*
	 * fix subsequent points
	 */
	pdom->vtx[pdom->nvertices].vtX = point.k;
	pdom->vtx[pdom->nvertices].vtY = point.l;
	pdom->nvertices++;
	arc.k1 = arc.k2 = point.k;
	arc.l1 = arc.l2 = point.l;
	do {
		while (tabletpoint(&button, &x, &y))
			;
		xinc = (x - xwas)/40;
		yinc = (y - ywas)/40;
		xwas = x;
		ywas = y;
		switch (button) {
		case 1:
			if (xinc != 0 || yinc != 0) {
				/*
		 		* delete preceding polygon if end point has moved
		 		*/
				intens(0);
				picframe(&arc,f);
				intens(1);
				arc.k2 += xinc;
				arc.l2 += yinc;
			}
			break;
		case 2:
			arc.k1 = arc.k2;
			arc.l1 = arc.l2;
			pdom->vtx[pdom->nvertices].vtX = arc.k1;
			pdom->vtx[pdom->nvertices].vtY = arc.l1;
			pdom->nvertices++;
			/*
			 * permit coordinate reset
			 */
			while (tabletpoint(&button, &xwas, &ywas))
				break;
		default:
			break;
		}
			picframe(&arc,f);
	} while (button <4);
	intens(0);
	picframe(&point,f);
	picframe(poly,f);
	intens(1);
	fclose(tblr);
	return(poly);
}


static
tabletpoint (button, x, y)
int *button, *x, *y;
{
	union {
		char d[16];
		int D[4];
	} u;
	register i;
	int sr;
	do {
		fscanf(tblr, "%c", u.d);
	} while (u.d[0] != '@'-128);
	fscanf(tblr, "%13s", &(u.d[1]));
	for (i=0; i<4; i++)
		u.D[i] &= PARITYMASK ;
	if ((sr=sscanf(u.d, "@%d%d%d", button, x, y))!=3)
		return (1);
	else
		return(0);
}
#else OSK
#define MOUSESCALE 3
static
struct object *mousepoly(f,startkol,startline,term)
struct pframe *f;
char *term;
{
	struct object *poly, *makemain();
	register struct polygondomain *pdom;
	struct polygondomain *makepolydmn();
	struct m2mouse m;
	FILE *mouse;
	register int xwas,ywas,xinc,yinc;
	int i;

	if ((mouse=fopen(term, "r"))==NULL) {
		fprintf (stderr,"Cannot fopen() mouse. Exiting\n") ;
		woolz_exit(1, "mousepoly") ;
	}
	setbuf(mouse,NULL);

	pdom = makepolydmn(1,NULL,0,IPOLSIZE,1);
	poly = makemain(10,pdom,NULL,NULL,NULL);
	point.k = startkol;
	point.l = startline;
	colour(1);
	picframe(&point,f);
	xwas = ywas = m.X = m.Y = 0;
	/*
	 * fix start point
	 */
	do {
		n_mouse(&m,mouse);
		/* abandon this interaction ?? */
		if (m.R == 0) {
			i_clean(&point,poly,mouse,f);
			return(NULL);
		}
		xinc = (m.X - xwas) / MOUSESCALE;
		yinc = (m.Y - ywas) / MOUSESCALE;
		/*
		if (xinc > 1)
			xinc = 2;
		else if (xinc < -1)
			xinc = -2;
		if (yinc > 1)
			yinc = 2;
		else if (yinc < -1)
			yinc = -2;
		*/
		if (xinc != 0 || yinc != 0) {
			xwas += xinc * MOUSESCALE;
			ywas += yinc * MOUSESCALE;
			intens(0);
			picframe(&point,f);
			intens(1);
			point.k += xinc;
			point.l += yinc;
			picframe(&point,f);
		}
	} while (m.L == 1);
	/* wait for L button to be un-asserted */
	do {
		n_mouse(&m,mouse);
	} while (m.L == 0);
	
	/*
	 * fix subsequent points
	 */
	pdom->vtx[pdom->nvertices].vtX = point.k;
	pdom->vtx[pdom->nvertices].vtY = point.l;
	pdom->nvertices++;
	arc.k1 = arc.k2 = point.k;
	arc.l1 = arc.l2 = point.l;
	xwas = m.X;
	ywas = m.Y;
	do {
		n_mouse(&m,mouse);
		/* abandon this interaction ?? */
		if (m.R == 0) {
			i_clean(&point,poly,mouse,f);
			return(NULL);
		}
		/* delete some ? delete last 5 vertices */
		if (m.M == 0) {
			for (i=0; i<5; i++) {
				if (pdom->nvertices <= 1)
					break;	/* from this loop */
				arc.k2 = arc.k1;
				arc.l2 = arc.l1;
				pdom->nvertices--;
				arc.k1 = pdom->vtx[pdom->nvertices].vtX;
				arc.l1 = pdom->vtx[pdom->nvertices].vtY;
				intens(0);
				picframe(&arc,f);
				intens(1);
			}
			xwas = m.X;
			ywas = m.Y;
		} else {
		/* otherwise add new point */
			xinc = (m.X - xwas) / MOUSESCALE;
			yinc = (m.Y - ywas) / MOUSESCALE;
			if (xinc != 0 || yinc != 0) {
				xwas += xinc * MOUSESCALE;
				ywas += yinc * MOUSESCALE;
				arc.k2 += xinc;
				arc.l2 += yinc;
				pdom->vtx[pdom->nvertices].vtX = arc.k2;
				pdom->vtx[pdom->nvertices].vtY = arc.l2;
				if (pdom->nvertices++ > IPOLSIZE) {
					fprintf(stderr,"Polygon too big\n");
					i_clean(&point,poly,mouse,f);
					return(NULL);
				}
				picframe(&arc,f);
				arc.k1 = arc.k2;
				arc.l1 = arc.l2;
			}
		}
	} while (m.L == 1);
	
	intens(0);
	picframe(&point,f);
	picframe(poly,f);
	intens(1);
	fclose(mouse);
	return(poly);
}

i_clean(pn,pol,fi,f)
struct object *pn,*pol;
FILE *fi;
struct pframe *f;
{
	intens(0);
	picframe(pn,f);
	picframe(pol,f);
	intens(1);
	freeobj(pol);
	fclose(fi);
}
#endif OSK
#endif C68000
