/*
 * fiprect.c
 *
 * Obtain a rectangular object from CRS frame store.
 *
 * Ask for line and column bounds from terminal, invert the
 * chosen rectangle, ask if OK.  If it is, re-invert and write
 * out in Woolz rectangle format.  If it is not OK, reinvert
 * and go back to beginning.
 *
 ***********************************************************************
 * This programs uses special modified versions of some Woolz routines
 * that access the frame store and explicitly mask out the top 8 bits,
 * which are apparently invariably read as "FF".
 ***********************************************************************
 *
 *	Modifications
 *
 *	 4 Dec 1988		CAS		Convert to long woolz names + make compile
 *
 */

#include <stdio.h>
#include <wstruct.h>
#define FRAME 0x400000

/*
   greyval.c
   random access to pointer to grey value in an interval domain object
*/

main(argc,argv)
char **argv;
int argc;
{
	struct object *obj;
	struct object *makerect();
	int k1,kl,l1,ll;
	char c;
	extern int packcheck;

	invertall();
	invertall();
	packcheck = 0;
	obj = makerect(0,511,0,511,FRAME,0,NULL,obj);
	do {
		fprintf(stderr,"type l1 ll k1 kl\n");
		scanf("%d %d %d %d",&l1,&ll,&k1,&kl);
		obj->idom->line1 = l1;
		obj->idom->lastln = ll;
		obj->idom->kol1 = k1;
		obj->idom->lastkl = kl;
		invert(obj);
		fprintf(stderr,"This area OK ?\n");
		while (scanf("%c",&c), c != 'n' && c != 'y')
			;
		invert(obj);
	} while (c != 'y');
	fswriteobj(stdout,obj);
}

/*
 * invert grey-table of object.
 */
invert(obj)
struct object *obj;
{
	GREY min,max,range;
	register GREY *g;
	register int i;
	struct iwspace iwsp;
	struct gwspace gwsp;

	greyrange(obj,&min,&max);
	range = max + min;
	initgreyscan(obj,&iwsp,&gwsp);
	while (nextgreyinterval(&iwsp) == 0) {
		g = gwsp.grintptr;
		for (i=0; i<iwsp.colrmn; i++) {
			*g = range - (*g & 255);
			g++;
		}
	}
}

/*
 * compute greyrange of pixel object
 */
greyrange(obj,min,max)
struct object *obj;
GREY *min, *max;
{
	register GREY v,*g;
	register int i;
	GREY lmin, lmax;
	struct iwspace iwsp;
	struct gwspace gwsp;

	lmin = 1;
	lmax = 0;
	initgreyscan(obj,&iwsp,&gwsp);
	while (nextgreyinterval(&iwsp) == 0) {
		g = gwsp.grintptr;
		for (i=0; i<iwsp.colrmn; i++) {
			v = (*g & 255);
			g++;
			if (lmin > lmax){
				lmin = v;
				lmax = v;
			}
			else if (v > lmax)
				lmax = v;
			else if (v < lmin)
				lmin = v;
		}
	}
	*min = lmin;
	*max = lmax;
}

invertall()
{
	register int i;
	register short *fr = (short *) FRAME;
	for (i=0; i<512*512; i++) {
		*fr = 255 - (*fr&255);
		fr++;
	}
}














