/*
 * grabcell.c
 *
 * capture and threshold data from frame store.
 *
 *
 ***********************************************************************
 * This programs uses special modified versions of some Woolz routines
 * that access the frame store and explicitly mask out the top 8 bits,
 * which are apparently invariably read as "FF".
 ***********************************************************************
 *
 *	Modifications
 *
 *	 4 Dec 1988		CAS		Convert to long woolz names + make compile
 *
 */

#include <stdio.h>
#include <wstruct.h>

#define FRAME 0x400000


main(argc,argv)
char **argv;
int argc;
{
	struct object *obj, *histo, *fshisto(), *tobj, *fsthreshold();
	struct object *makerect(), *objl[100];
	struct intervaldomain *idom;
	register int i,j;
	int xs,ys,type,num,thres,numobj;
	extern int packcheck;
	extern SMALL highlow;
	char Filename[30];
	FILE *File;
	
	packcheck = 0;
	highlow = 1;
	dinit();
	fast();
	fgsetup();
	setfglut(1);
	setfgsync(0);
	clear();
	for(;;) {
		llut();
		fgdigitise();
		fprintf(stdout,"Hit CR to freeze :");
		while(getc(stdin)!='\n');
		fgfreeze();
		obj = makerect(0,511,0,511,FRAME,0,NULL,obj);
		histo = fshisto(obj,255);
		thres = compthr3(histo);
		fprintf(stderr,"Threshold %d\n",thres);
		tobj = fsthreshold(obj,thres);
		fprintf(stdout,"Threshold complete\n");
		fprintf(stderr,"Thresholded area %d\n",area(tobj));
		label(tobj,&numobj,objl,100,2);
		fprintf(stderr,"%d objects segmented\n",numobj);

		fprintf(stdout,"Filename :");
		get_line(stdin,Filename);
		if (!strlen(Filename)) continue;
		
		File = fopen(Filename,"w");
		if ( File ) {
			for (i=0; i<numobj; i++) {
				idom = objl[i]->idom;
				if (idom->line1 == 0 || idom->lastln == 511)
					continue;
				if (idom->kol1 == 0 || idom->lastkl == 511)
					continue;
				fswriteobj(File,objl[i]);
			}
		} else {
			fprintf (stdout,"Unable to open %s == Sorry\n",Filename);
		}
	}		
}



/*
 *
 *	Get a lineof text from the file pointed to by fp
 *	return the last charcter the terminated input
 *
 */
 
int get_line(fp,s)
FILE *fp;
char *s;
{
int c;
	while((c=getc(fp))!=EOF && c!= '\n') 
		{
		*s++ = c ;
		}
	*s = '\0';
	return(c);
}
/*
 * Mods due to CRS fame store being the grey table
 */
 
extern SMALL highlow;

struct object *fsthreshold (obj,thresh)
struct object *obj;
register int thresh;
{
	struct object *nobj, *makemain();
	struct intervaldomain *idom, *makedomain();
	register GREY *g;
	register colno,nints;
	register COORD over;
	COORD nl1,nll,nk1,nkl,l;
	struct iwspace iwsp;
	struct gwspace gwsp;
	register struct interval *itvl, *jtvl;

	/*
	 * first pass - find line and column bounds of thresholded
	 * object and number of intervals.
	 */
	idom = obj->idom;
	nl1 = idom->lastln;
	nll = idom->line1;
	nk1 = idom->lastkl;
	nkl = idom->kol1;
	initgreyscan(obj,&iwsp,&gwsp);
	nints = 0;
	while (nextgreyinterval(&iwsp) == 0) {
		g = gwsp.grintptr;
		over = 0;
		for (colno = iwsp.lftpos; colno <= iwsp.rgtpos; colno++) {
			if ((highlow != 0 && (*g&255) >= thresh) ||
			    (highlow == 0 && (*g&255) < thresh)) {
				if (over == 0) {
					over = 1;
					if (iwsp.linpos < nl1)
						nl1 = iwsp.linpos;
					if (iwsp.linpos > nll)
						nll = iwsp.linpos;
					if (colno < nk1)
						nk1 = colno;
				}
			} else {
				if (over == 1) {
					if (colno > nkl)
						nkl = colno;
					over = 0;
					nints++;
				}
			}
			g++;
		}
		if (over == 1) {
			if (colno > nkl)
				nkl = colno;
			over = 0;
			nints++;
		}
	}
	nkl--;	/* since we have looked at points beyond interval ends */

	/* domain structure */
	if (nints > 0) {
		idom = makedomain(1,nl1,nll,nk1,nkl);
		itvl = (struct interval *) Malloc(nints * sizeof(struct interval));
		idom->freeptr = (char *) itvl;

		/*
	 	* second pass - construct intervals
	 	*/
		initgreyscan(obj,&iwsp,&gwsp);
		nints = 0;
		jtvl = itvl;
		/* find thresholded endpoints */
		while (nextgreyinterval(&iwsp) == 0) {
			if (iwsp.linpos < nl1 || iwsp.linpos > nll)
				continue;
			g = gwsp.grintptr;
			over = 0;
			for (colno = iwsp.lftpos; colno <= iwsp.rgtpos; colno++) {
				if ((highlow != 0 && (*g&255) >= thresh) ||
			    	(highlow == 0 && (*g&255) < thresh)) {
					if (over == 0) {
						over = 1;
						itvl->ileft = colno - nk1;
					}
				} else {
					if (over == 1) {
						over = 0;
						itvl->iright = colno - nk1 - 1;
						nints++;
						itvl++;
					}
				}
				g++;
			}
			if (over == 1) {
				over = 0;
				itvl->iright = colno - nk1 - 1;
				nints++;
				itvl++;
			}
			/*
		 	* end of line ?
		 	*/
			if (iwsp.intrmn == 0) {
				makeinterval(iwsp.linpos,idom,nints,jtvl);
				jtvl = itvl;
				nints = 0;
			}
		}
	} else
		/* no thresholded points - make a dummy domain anyway */
		idom = makedomain(1,nll,nll,nkl,nkl);

	/* main object */
	nobj = makemain(1,idom,obj->vdom,obj->plist,obj);
	return(nobj);
}
/*
 * fshisto.c	Jim Piper
 * Mods since grey-table in CRS frame store
 */



/*
 * make a histogram from an object given the maximum grey value
 */
struct object *fshisto(obj,maxg)
struct object *obj;
{
	struct object *hist, *makemain();
	struct histogramdomain *hdom, *makehistodmn();
	struct iwspace iwsp;
	struct gwspace gwsp;
	register GREY *g;
	register int k, *h;
	/* make empty histogram */
	hdom = makehistodmn(1,maxg+1);
	hist = makemain(13,hdom,NULL,NULL,obj);
	/* fill histogram */
	h = hdom->hv;
	initgreyscan(obj,&iwsp,&gwsp);
	while (nextgreyinterval(&iwsp) == 0) {
		g = gwsp.grintptr;
		for (k=iwsp.lftpos; k<=iwsp.rgtpos; k++) {
			if ((*g&255) >= 0)
				h[(*g&255)] ++;
			g++;
		}
	}
	return(hist);
}

/*
 * invert grey-table of object.
 */
fsinvert(obj)
struct object *obj;
{
	GREY min,max,range;
	register GREY *g;
	register int i;
	struct iwspace iwsp;
	struct gwspace gwsp;

	fsgreyrange(obj,&min,&max);
	range = max + min;
	initgreyscan(obj,&iwsp,&gwsp);
	while (nextgreyinterval(&iwsp) == 0) {
		g = gwsp.grintptr;
		for (i=0; i<iwsp.colrmn; i++) {
			*g = range - (*g & 255);
			g++;
		}
	}
}

/*
 * compute greyrange of pixel object
 */
fsgreyrange(obj,min,max)
struct object *obj;
GREY *min, *max;
{
	register GREY v,*g;
	register int i;
	GREY lmin, lmax;
	struct iwspace iwsp;
	struct gwspace gwsp;
	lmin = 1;
	lmax = 0;
	initgreyscan(obj,&iwsp,&gwsp);
	while (nextgreyinterval(&iwsp) == 0) {
		g = gwsp.grintptr;
		for (i=0; i<iwsp.colrmn; i++) {
			v = (*g & 255);
			g++;
			if (lmin > lmax){
				lmin = v;
				lmax = v;
			}
			else if (v > lmax)
				lmax = v;
			else if (v < lmin)
				lmin = v;
		}
	}
	*min = lmin;
	*max = lmax;
}


