/*
 * fipobj.c	Jim Piper	25 June 1984
 *
 * Construct object from FIP packed input.
 * Otherwsie, leave intensity values alone - DO NOT NORMALISE HERE !
 *
 * The format of the input is as follows : all 16 bit words :
 *	(1) Line number (zero if end)
 *	(2) Column number/2 + offset
 *	(3) Pixel count
 *	(4) - (n) Pixels, packed two to a word.  Note that if the first pixel
 *		is zero then it is NOT included in the count and the column
 *		number derived from (2) must be incremented.  Also the last
 *		byte in the pixel stream may be zero and will not be included
 *		in the count.
 */

#include <stdio.h>
#include <wstruct.h>

#define MAXWIDTH 1024
#define MAXLENGTH 1280
#define FIPBUFSTART 0x1c00
#define rgetbyte(i) { i=fipbuf[bp++];}
#define rget16bits(i,j) { rgetbyte(j); rgetbyte(i); i += (j<<8); }


struct object *fipobj(fipbuf,bleng)
register unsigned char *fipbuf;
{
	register GREY *g, *gwas;
	struct object *obj, *makemain();
	struct intervaldomain *idom, *makedomain();
	struct interval *itvl, *jtvl;
	struct valuetable *makeivtable();
	int line1, lastln,kol1,lastkl,linearea,linewas;
	int col,osegleng,rsegleng,p1;
	int j,k,nints;
	register int i, line, bp, segleng;
	struct iwspace iwsp;
	struct gwspace gwsp;
	
	/*
	 * scan data, counting lines, intervals and "linearea".
	 * store data in "fipbuf"
	 */
	bp = 0;
	line1 = -1;
	linearea = 0;
	nints = 0;
	kol1 = lastkl = 0;
	linewas = -1;
	/*
	 * cycle while line number > 0
	 */
	rget16bits(line,i);
	while (line > 0 && line < MAXLENGTH) {
		nints++;
		if (line1 < 0)
			line1 = line;
		/* column number */
		rget16bits(col,i);
		col -= FIPBUFSTART;
		col *= 2;
		if (col > MAXWIDTH)
			break;
		/* true segment length */
 		rget16bits(segleng,i);
		osegleng = segleng;
		rsegleng = segleng;
/*
fprintf(stderr,"line %d col %d seglen %d col1 %d\n",line,col,segleng,kol1);
*/
		/* first byte - zero ? */
		rgetbyte(p1);
		if (p1 == 0) {
			col++;
			rsegleng++;
		} else {
			segleng--;
		}
		if (line != linewas) {
			linearea += (lastkl - kol1 + 1);
			lastln = line;
			linewas = line;
			kol1 = col;
		}
		lastkl = col + osegleng - 1;
		/* remaining bytes */
		for (; segleng>0; segleng--)
			rgetbyte(i);
		/* ensure an even number of bytes read */
		if (rsegleng&1 != 0)
			rgetbyte(i);
		rget16bits(line,i);
	}
	linearea += (lastkl - kol1 + 1);
	lastln = linewas;
/*
	fprintf(stderr,"line1 %d lastln %d linearea %d intcount %d\n",line1,lastln,linearea,nints);
*/

	/*
	 * make a suitably sized object
	 */
	idom = makedomain(1,line1,lastln,0,MAXWIDTH);
	idom->freeptr = (char *) Malloc(nints * sizeof(struct interval));
	obj = makemain(1,idom,NULL,NULL,NULL);

	/*
	 * now rescan buffer, filling intervals and grey-table
	 */
	itvl = jtvl = (struct interval *)idom->freeptr;
	bp = 0;
	nints = 0;
	linewas = -1;
	rget16bits(line,i);
	while (line > 0) {
		/* column number */
		rget16bits(col,i);
		if (col <= 0)
			break;		/* out of while loop */
		col -= FIPBUFSTART;
		col *= 2;
		/* true segment length */
		rget16bits(segleng,i);
		if (segleng <= 0)
			break;		/* out of while loop */
		osegleng = segleng;
		rsegleng = segleng;
		if (line != linewas) {
			if (linewas >= 0) {
				makeinterval(linewas,idom,nints,jtvl);
			}
			nints = 0;
			jtvl = itvl;
			linewas = line;
			kol1 = col;
		}
/*
fprintf(stderr,"line %d col %d seglen %d col1 %d\n",line,col,segleng,kol1);
*/
		/* first byte - zero ? */
		rgetbyte(p1);
		if (p1 == 0) {
			if (col == kol1)
				kol1++;
			col++;
			rsegleng++;
		} else {
			segleng--;
		}
		lastkl = col + osegleng - 1;
		itvl->ileft = col;
		itvl->iright = col + osegleng - 1;
		itvl++;
		nints++;
		/* remaining bytes */
		for (; segleng>0; segleng--)
			rgetbyte(i);
		/* ensure an even number of bytes read */
		if (rsegleng&1 != 0)
			rgetbyte(i);
		rget16bits(line,i);
	}
	makeinterval(linewas,idom,nints,jtvl);
	
	/*
	 * now rescan buffer, filling grey-table
	 */
	obj->vdom = makeivtable(21,obj,0);
	initgreyscan(obj,&iwsp,&gwsp);
	bp = 0;
	rget16bits(line,i);
	while (line > 0) {
		/* column number */
		rget16bits(col,i);
		if (col <= 0)
			break;		/* out of while loop */
		/* true segment length */
		rget16bits(segleng,i);
		if (segleng <= 0)
			break;		/* out of while loop */
		osegleng = segleng;
		rsegleng = segleng;
		nextgreyinterval(&iwsp);
		g = gwsp.grintptr;
/*
fprintf(stderr,"line %d col %d seglen %d col1 %d\n",line,col,segleng,kol1);
*/
		/* first byte - zero ? */
		rgetbyte(p1);
		if (p1 == 0) {
			rsegleng++;
		} else {
			segleng--;
			*g++ = p1;
		}
		/* remaining bytes */
		for (; segleng>0; segleng--)
			rgetbyte(*g++);
		/* ensure an even number of bytes read */
		if (rsegleng&1 != 0)
			rgetbyte(i);
		rget16bits(line,i);
	}

	/*
	 * write object away
	 */
	standardidom(idom);
	return(obj);
}
