/*
 * fsthreshold.c
 *
 * threshold data from frame store.
 *
 * DERIVED FROM threshold.c		Jim Piper		4 February 1985
 * Mods since grey-table in CRS frame store
 *
 *
 ***********************************************************************
 * This programs uses special modified versions of some Woolz routines
 * that access the frame store and explicitly mask out the top 8 bits,
 * which are apparently invariably read as "FF".
 ***********************************************************************
 */

#include <stdio.h>
#include <wstruct.h>

extern SMALL highlow;

struct object *fsthreshold (obj,thresh)
struct object *obj;
register int thresh;
{
	struct object *nobj, *makemain();
	struct intervaldomain *idom, *makedomain();
	register GREY *g;
	register colno,nints;
	register COORD over;
	COORD nl1,nll,nk1,nkl,l;
	struct iwspace iwsp;
	struct gwspace gwsp;
	register struct interval *itvl, *jtvl;

	/*
	 * first pass - find line and column bounds of thresholded
	 * object and number of intervals.
	 */
	idom = obj->idom;
	nl1 = idom->lastln;
	nll = idom->line1;
	nk1 = idom->lastkl;
	nkl = idom->kol1;
	initgreyscan(obj,&iwsp,&gwsp);
	nints = 0;
	while (nextgreyinterval(&iwsp) == 0) {
		g = gwsp.grintptr;
		over = 0;
		for (colno = iwsp.lftpos; colno <= iwsp.rgtpos; colno++) {
			if ((highlow != 0 && (*g&255) >= thresh) ||
			    (highlow == 0 && (*g&255) < thresh)) {
				if (over == 0) {
					over = 1;
					if (iwsp.linpos < nl1)
						nl1 = iwsp.linpos;
					if (iwsp.linpos > nll)
						nll = iwsp.linpos;
					if (colno < nk1)
						nk1 = colno;
				}
			} else {
				if (over == 1) {
					if (colno > nkl)
						nkl = colno;
					over = 0;
					nints++;
				}
			}
			g++;
		}
		if (over == 1) {
			if (colno > nkl)
				nkl = colno;
			over = 0;
			nints++;
		}
	}
	nkl--;	/* since we have looked at points beyond interval ends */

	/* domain structure */
	if (nints > 0) {
		idom = makedomain(1,nl1,nll,nk1,nkl);
		itvl = (struct interval *) Malloc(nints * sizeof(struct interval));
		idom->freeptr = (char *) itvl;

		/*
	 	* second pass - construct intervals
	 	*/
		initgreyscan(obj,&iwsp,&gwsp);
		nints = 0;
		jtvl = itvl;
		/* find thresholded endpoints */
		while (nextgreyinterval(&iwsp) == 0) {
			if (iwsp.linpos < nl1 || iwsp.linpos > nll)
				continue;
			g = gwsp.grintptr;
			over = 0;
			for (colno = iwsp.lftpos; colno <= iwsp.rgtpos; colno++) {
				if ((highlow != 0 && (*g&255) >= thresh) ||
			    	(highlow == 0 && (*g&255) < thresh)) {
					if (over == 0) {
						over = 1;
						itvl->ileft = colno - nk1;
					}
				} else {
					if (over == 1) {
						over = 0;
						itvl->iright = colno - nk1 - 1;
						nints++;
						itvl++;
					}
				}
				g++;
			}
			if (over == 1) {
				over = 0;
				itvl->iright = colno - nk1 - 1;
				nints++;
				itvl++;
			}
			/*
		 	* end of line ?
		 	*/
			if (iwsp.intrmn == 0) {
				makeinterval(iwsp.linpos,idom,nints,jtvl);
				jtvl = itvl;
				nints = 0;
			}
		}
	} else
		/* no thresholded points - make a dummy domain anyway */
		idom = makedomain(1,nll,nll,nkl,nkl);

	/* main object */
	nobj = makemain(1,idom,obj->vdom,obj->plist,obj);
	return(nobj);
}
