/*
 * castel.c		Jim Piper		4 July 1984
 *
 * Compute and compensate for mean density difference between odd and
 * even pixels.  This appears to be either a constant or linearly
 * dependent on pixel intensity, so for now we compute a constant
 * correction and apply it.
 *
 * If the correction is displayed sorted by pixel intensity, it appears
 * to be sloped with opposite peaks at intensities near threshold and
 * near peak values, but these are probably artefacts (i) pixels near
 * threshold are near boundary and their within-object neighbours will
 * tend to have higher values, (ii) pixels with peak intensity are on
 * peaks (!) and tend to have neighbours with lower intensity.
 *
 * Modifications
 *
 *	13 Sep 1986		CAS		Includes
 */

#include <stdio.h>
#include <wstruct.h>


main() {
	struct object *obj, *readobj(), *histo, *makemain();
	struct iwspace iwsp;
	struct gwspace gwsp;
	struct histogramdomain *makehistodmn(),*hdom;
	int i,j,k,diff,*hv,diffsum,count,m;
	GREY *g;

#ifdef HISTO
	hdom = makehistodmn(1,256);
	hv = hdom->hv;
	histo = makemain(13,hdom,NULL,NULL,NULL);
#endif HISTO

	while((obj = readobj(stdin)) != NULL) {
		if (obj->type == 1) {
			count = diffsum = 0;
#ifdef HISTO
			for (i=0; i<256; i++) {
				hcount[i] = 0;
				hv[i] = 0;
			}
#endif HISTO
			initgreyscan(obj,&iwsp,&gwsp);
			while (nextgreyinterval(&iwsp) == 0) {
				g = gwsp.grintptr;
				for (k=iwsp.lftpos; k<iwsp.rgtpos; k++) {
					if (k%2 == 0) {
						diff = *g - *(g+1);
						count++;
						diffsum += diff;
#ifdef HISTO
						hcount[*g]++;
						hv[*g] += diff;
#endif HISTO
					}
					g++;
				}
			}
			m = diffsum/count;
			fprintf(stderr,"mean difference even-odd  %d\n",m);
#ifdef HISTO
			/*
			 * histogram of mean deviations
			 */
			for (i=0; i<256; i++)
				if (hcount[i] != 0)
					hv[i] /= hcount[i];
			writeobj(stdout,histo);
			histothreshsmooth(histo);
			hdom->l = -48;
			writeobj(stdout,histo);
			hdom->l = 48;
			for (i=0; i<256; i++)
				hcount[i] /= 4;
			hdom->hv = hcount;
			writeobj(stdout,histo);
#else HISTO

			/*
			 * now correct the image
			 */
			initgreyscan(obj,&iwsp,&gwsp);
			while (nextgreyinterval(&iwsp) == 0) {
				g = gwsp.grintptr;
				for (k=iwsp.lftpos; k<=iwsp.rgtpos; k++) {
					if (k%2 == 0) {
						*g = *g - m;
						if (*g < 0)
							*g = 0;
					}
					g++;
				}
			}
#endif HISTO
		}
#ifndef HISTO
		writeobj(stdout,obj);
#endif HISTO
		freeobj(obj);
	}
}
