/*
 *      C C O M . C  --  Convert objects to chromosome common format
 *
 *
 *  Written: Clive A Stubbings
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *
 *  Date:    10th July 1987
 *
 *  Modifications
 *
 */
#include <stdio.h>
#include <wstruct.h>

#define VERSION	1		/* CCOM version number */
#define fdebugf(a,b,c,d,e,f,g,h,i,j)
#define ABS(a) (((a) > 0) ? (a) : -(a))
#define READ 1
#define WRITE 2

struct blkhead {
		int		size;		/* Block total size (including header) */
		short	name;		/* Offset to block name string */
		short	data;		/* Offset to start of data */
		short	obj_x;		/* Object x size */
		short	obj_y;		/* Object y size */
	};

extern int	_memmins;
extern char	*Malloc();

/*
 *	I N T E R R U P T --  Signal catcher
 *
 */
interrupt(sig)
int		sig;
{
}

main(argc,argv)
int		argc;
char	**argv;
{
	char	*swptr, **targv;
	int		targc, doneone, mode = 0;
	FILE	*infile;
	struct	object	*obj, *readobj(), *ccomread();

	_memmins = 32768;
#ifdef INTERRUPT
	intercept(interrupt);
#endif INTERRUPT
	targc = argc; targv = argv;
	argc--; argv++;			/* dump process name */
	while (argc > 0) {
		if (*argv[0] == '-') {
			swptr = *argv;
			*argv = NULL;
			while (*(++swptr)) {
				switch (*swptr) {
				  case '?':
				  	printf("Syntax:   ccom <options>\n");
				  	printf("Function: \n");
				  	printf("Options:  r - read ccom format\n");
				  	printf("          w - write ccom format\n");
				  	printf("          v - Show version number\n");
				  	printf("          ? - Help\n");
				  	exit(0);
				  case 'v':
					printf("Ccom - Version 0.1\n");
					exit(0);
/*				  case 'x':
					param = *++argv;
					*argv = NULL; argc--;
					break;*/
				  case 'r':
					mode = READ;
					break;
				  case 'w':
					mode = WRITE;
					break;
				  default:
				 	printf("ccom - unknown option\n");
				}
			}
		}
		argv++; argc--;
	}
#ifdef PARAMS
	argc = targc; argv = targv;
	argc--; argv++;			/* dump process name */
	doneone = 0;
	while (argc) {
		if (*argv != NULL) {
/*
 *	Use remaining params 
 */
			if ((infile = fopen(*argv,"r")) == NULL) {
				fprintf(stderr,"Can't open file %s\n",*argv);
			}
			dofile(infile);
			fclose(infile);
			doneone = 1;
		}
		argc--; argv++;
	}
	if (!doneone) {
		dofile(stdin);
	}
#endif PARAMS

	switch (mode) {
		case READ:
			ccomreadheader(stdin);
			while ((obj = ccomread(stdin)) != NULL) {
				writeobj(stdout,obj);
				freeobj(obj);
			}
			break;
		case WRITE:
			ccomwriteheader(stdout,"Cytoscan","Patient","Case ID","Slide ID","Microscope","N381",12345,54321);
			while ((obj = readobj(stdin)) != NULL) {
				if (obj->type == 1) {
					ccomwrite(stdout,obj,"F");
				}
				freeobj(obj);
			}
			break;
		default:
			break;
	}
}

/*
 *	C C O M W R I T E H E A D E R  --
 *
 */
ccomwriteheader(ch,machname,patname,caseid,slideid,scope,nefpos,xvern,yvern)
FILE	*ch;
char	*machname,*patname,*caseid,*slideid,*scope,*nefpos;
int		xvern, yvern;
{
	short	length, temp, dynmptr = 26;
	length = dynmptr + strlen(machname)+1 + strlen(patname)+1 + strlen(caseid)+1 + strlen(slideid)+1 + strlen(scope)+1;
	write(ch->_fd,&length,2);
	temp = VERSION;
	write(ch->_fd,&temp,2);
	write(ch->_fd,&dynmptr,2);		/* offset to Machine name */
	dynmptr += strlen(machname)+1;
	write(ch->_fd,&dynmptr,2);		/* offset to Pat name */
	dynmptr += strlen(patname)+1;
	write(ch->_fd,&dynmptr,2);		/* offset to Case ID */
	dynmptr += strlen(caseid)+1;
	write(ch->_fd,&dynmptr,2);		/* offset to Slide ID */
	dynmptr += strlen(slideid)+1;
	write(ch->_fd,&dynmptr,2);		/* offset to Scope Name/ID */
	dynmptr += strlen(scope)+1;
	write(ch->_fd,nefpos,4);		/* Cell location (NEF reference) */
	write(ch->_fd,&xvern,4);		/* X vernier position */
	write(ch->_fd,&yvern,4);		/* Y vernier position */
	write(ch->_fd,machname,strlen(machname)+1);
	write(ch->_fd,patname, strlen(patname)+1);
	write(ch->_fd,caseid,  strlen(caseid)+1);
	write(ch->_fd,slideid, strlen(slideid)+1);
	write(ch->_fd,scope,   strlen(scope)+1);
}
#define getint(ptr) ((16777216 * (*ptr++)) + (65536 * (*ptr++)) + (256 * (*ptr++)) + (*ptr++))
/*
 *	C O M R E A D H E A D E R  --
 *
 */
ccomreadheader(ch)
FILE	*ch;
{
	short	length,version, offset;
	unsigned char	*buf, *ptr;

	read(ch->_fd,&length,2);
	if (length > 2) {
		buf = (unsigned char *) Malloc(length);
		read(ch->_fd,buf+2,length-2);
		ptr = buf + 2;
		version = 256 * *ptr++ + *ptr++;
		fprintf(stderr,"Header version %d\n",version);
		offset = 256 * *ptr++ + *ptr++;
		fprintf(stderr,"Machine name %s\n",buf+offset);
		offset = 256 * *ptr++ + *ptr++;
		fprintf(stderr,"Patient name %s\n",buf+offset);
		offset = 256 * *ptr++ + *ptr++;
		fprintf(stderr,"Case ID %s\n",buf+offset);
		offset = 256 * *ptr++ + *ptr++;
		fprintf(stderr,"Slide ID %s\n",buf+offset);
		offset = 256 * *ptr++ + *ptr++;
		fprintf(stderr,"Scope Name/ID %s\n",buf+offset);
		fprintf(stderr,"NEF reference %4s\n",ptr); ptr += 4;
		fprintf(stderr,"Cell location %d %d\n",getint(ptr),getint(ptr));
		Free(buf);
	}
}

/*
 *	C C O M R E A D  --
 *
 */
struct object *ccomread(ch)
FILE	*ch;
{
	int		i,pos,size;
	struct	blkhead head;
	struct	object *obj, *ccomtoobj();
	char	objname[20], *wkbuf, *Malloc();

	if (read(ch->_fd,&head,sizeof(struct blkhead)) != sizeof(struct blkhead))
		return(NULL);
	pos = seekfromto(ch,sizeof(struct blkhead),head.name);
	for (i=0; i<sizeof(objname); i++) {
		read(ch->_fd,&objname[i],1);
		pos++;
		if (objname[i] == '\0')
			break;
	}
	objname[i] = '\0';
	pos = seekfromto(ch,pos,head.data);
	size = head.size-head.data;
	fdebugf(stderr,"Objname %s, size %d\n",objname,size);
	wkbuf = Malloc(size);
	read(ch->_fd,wkbuf,size);
	obj = ccomtoobj(wkbuf,size);
	Free(wkbuf);
	return(obj);
}

static int seekfromto(ch,a,b)
FILE	*ch;
int		a,b;
{
	int	len,i;
	len = b-a;
	while (len--)
		read(ch->_fd,&i,1);
	return(b);
}

/*
 */
static struct object *ccomtoobj(start,size)
char	*start;
int		size;
{
	int		greysize, nints, nlines, ncols, i;
	char	*ptr, *nextint();
	struct intervaldomain *idom, *makedomain();
	struct valuetable *vtb, *makevaluetb();
	struct intervalline *ivln;
	struct interval *itvl;
	struct object *obj, *makemain();
	GREY	*greytab;

	ccomstats(start,size,&nints,&nlines,&ncols);
	ptr = start;
	idom = makedomain(1,0,nlines-1,0,ncols-1);
	fdebugf(stderr,"nlines %d, ncols %d\n",nlines,ncols);
	ivln = idom->intvlines;
	itvl = (struct interval *) Malloc(nints * sizeof(struct interval));
	idom->freeptr = (char *) itvl;
	idom->linkcount = 0;
	for (i=0; i<nints;i++) {
		ptr = nextint(ptr,&ivln,&itvl);
	}
	ivln = idom->intvlines;
	itvl = (struct interval *) idom->freeptr;
	for (i=0; i<nlines; i++) {
		nints = ivln->nintvs;
		fdebugf(stderr,"line %d, nints %d\n",i,nints);
		makeinterval(i,idom,nints,itvl);
		itvl += nints;
		ivln++;
	}
	obj = makemain(1,idom,NULL,NULL,NULL);
	greysize = linearea(obj);
	fdebugf(stderr,"line1 %d, lastln %d, greysize %d\n",idom->line1, idom->lastln,greysize);
	vtb = makevaluetb(1,idom->line1,idom->lastln,0,obj);
	fdebugf(stderr,"Made vtb\n");
	obj->vdom = vtb;
	greytab = (GREY *)Malloc(greysize * sizeof(GREY));
	vtb->freeptr = (char *) greytab;
	vtb->linkcount = 1;
	fillgreytab(obj,start,size);
	return(obj);
}

/*
 *	F I L L G R E Y T A B  --
 *
 *	Using already constructed interval domain, extract grey table
 *	data
 */
static fillgreytab(obj,start,size)
struct	object *obj;
unsigned char	*start;
int		size;
{
	int		i, kstart;
	unsigned char *ptr;
	GREY	*greytab, *greyptr;
	struct	iwspace iwsp;

	ptr = start;
	greytab = (GREY *) obj->vdom->freeptr;
	initrasterscan(obj,&iwsp,0);
	while (nextinterval(&iwsp) == 0) {
		if (iwsp.nwlpos > 1) {
			for (i=1; i<iwsp.nwlpos;i++) {
				ptr += 4;
				fdebugf(stderr,"Empty line\n");
			}
		}
		fdebugf(stderr,"Grey line");
		ptr += 4;								/* skip length + offset */
		if (iwsp.nwlpos)
			kstart = iwsp.lftpos;				/* Get pos of 1st bit on line */
		greyptr = greytab + iwsp.lftpos - kstart;
		for (i=0; i<iwsp.colrmn;i++) {
			fdebugf(stderr," %d",*ptr);
			*greyptr++ = *ptr++;				/* Copy into grey table */
		}
		fdebugf(stderr,"     ");
		if (iwsp.intrmn == 0) {
			fdebugf(stderr,"\n");
			makevalueline(obj->vdom,iwsp.linpos,kstart,iwsp.rgtpos,greytab);
			greytab += iwsp.rgtpos - kstart + 1;
		}
	}
		
}


static char *nextint(ptr,ivln,itvl)
unsigned char	*ptr;
struct intervalline **ivln;
struct interval **itvl;
{
	short		offset, length;
	static	int curroffset = 0;
	offset = 256 * *ptr++ + *ptr++;
	length = 256 * *ptr++ + *ptr++;
	if (length == 0) {
		curroffset = 0;
		while ((length == 0) && (offset)) {
			(*ivln)->nintvs = 0;
			(*ivln)++;
			fdebugf(stderr,"interval - empty line\n");
			offset = 256 * *ptr++ + *ptr++;
			length = 256 * *ptr++ + *ptr++;
		}
		if (length == 0) return(ptr);
	}
	offset += curroffset;
	if (length > 0) {
		(*ivln)->nintvs++;
		(*ivln)++;
		curroffset = 0;
	} else if (length < 0) {
		(*ivln)->nintvs++;
		length  = -length;
		curroffset = offset + length;
	}
	ptr += length;
	(*itvl)->ileft = offset;
	(*itvl)->iright = offset+length-1;
	fdebugf(stderr,"interval left %d, right %d\n",(*itvl)->ileft,(*itvl)->iright);
	(*itvl)++;
	return(ptr);
}


/*
 *
 */
static int ccomstats(start,size,nints,nlines,ncols)
unsigned char	*start;
int		size, *nints, *nlines, *ncols;
{
	unsigned char	*ptr;
	short	offset, length;
	int		totsize, linelength, totlength;
	ptr = start;
	totlength = 0; *nints = 0;
	*nlines = 0; *ncols = 0;
	while (ptr < start+size) {
		offset = 256 * *ptr++ + *ptr++;
		length = 256 * *ptr++ + *ptr++;
		if (length > 0) {
			totlength += offset+length;
			if (totlength > *ncols)
				*ncols = totlength;
			totlength = 0;
			(*nints)++;
			(*nlines)++;
		} else if (length < 0) {
			length = -length;
			totlength += offset+length;
			(*nints)++;
		} else if (length == 0) {
			if (offset != 0) {
				(*nlines)++;
			}
		}
		ptr += length;
		fdebugf(stderr,"offset %d, length %d, intervals %d\n",offset,length,*nints);
	}
}

/*
 *	C C O M W R I T E  --
 *
 */
ccomwrite(ch,obj,objname)
FILE	*ch;
struct	object *obj;
char	*objname;
{
	struct iwspace iwsp;
	struct gwspace gwsp;
	short	indent,length;
	struct	blkhead head;
	int		i, colstart = 0;

	if (obj->vdom != NULL) {
		head.size = ccomsize(obj);
		head.size += sizeof(struct blkhead) + strlen(objname) + 1;
		head.name = sizeof(struct blkhead);
		head.obj_x = head.obj_y = 0;
		head.data = head.name + strlen(objname) + 1;
		write(ch->_fd,&head,sizeof(struct blkhead));
		write(ch->_fd,objname,strlen(objname)+1);
		initgreyscan(obj,&iwsp,&gwsp);
		while (nextgreyinterval(&iwsp) == 0) {
			if (iwsp.nwlpos > 1) {
				indent = 1;
				length = 0;
				for (i=1; i<iwsp.nwlpos; i++) {
					write(ch->_fd,&indent,2);
					write(ch->_fd,&length,2);
				}
			}
			indent = iwsp.lftpos - colstart - obj->idom->kol1;
			length = iwsp.colrmn;
			if (iwsp.intrmn > 0) {
				length = -length;
				colstart = iwsp.rgtpos+1 -obj->idom->kol1;
			} else
				colstart = 0;
			write(ch->_fd,&indent,2);
			write(ch->_fd,&length,2);
			greywrite(ch,gwsp.grintptr,ABS(length));
		}
		indent = 0; length = 0;
		write(ch->_fd,&indent,2);
		write(ch->_fd,&length,2);
	}
}

/*
 *	C C O M S I Z E  --
 *
 */
static int ccomsize(obj)
struct	object *obj;
{
	int		size = 0;
	struct iwspace iwsp;
	initrasterscan(obj,&iwsp,0);
	while (nextinterval(&iwsp) == 0) {
		if (iwsp.nwlpos > 1)
			size += 4*(iwsp.nwlpos-1);
		size += 4 + iwsp.colrmn;
	}
	size += 4;
	return(size);
}

/*
 *	G R E Y W R I T E 
 *
 */
static greywrite(ch,gp,len)
FILE	*ch;
GREY	*gp;
int		len;
{
	int		i;
	unsigned char pix;
	for (i=0; i<len; i++) {
		pix = *gp++;
		write(ch->_fd,&pix,1);
	}
}

