/*
 *
 *	D I F F D O M . C --
 *
 *	Written: B. Pinnington
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    Oct 1987
 *
 *	Purpose:		Test program for woolz function diffdom
 *
 *	Description:	Zero, one or two files may be input, if no filename is 
 *					given objects are read from stdin. The first two
 *					objects will be diff'd and the result written to stdout.
 *					When automatic testing is used all objects from one file 
 *					are diff'd with the corresponding objects from the other
 *					file; again results to stdout.
 *
 *	Modifications:
 *  05 Nov 1987		BDP		Changed object buffer size to 200 pointers 
 *	04 Nov 1987		BDP		Changed two files to work on one to one	
 *	26 Oct 1987		BDP		Option of two simultaneously interacting
 *							files added								
 *
 */

#include <stdio.h>
#include <wstruct.h>

#define OBJMAX 200
#define DEBUG


main(argc,argv)
int argc;
char **argv;
{
	FILE *fp1, *fp2;
	struct object *obj, *readobj(), *objs[OBJMAX], *obj2s[OBJMAX], *diffdom();
	int filecount,i,j,fc1,fc2,fc;
	char *str;
		
	Alloc_limits(0, 1024 * 512 * sizeof(GREY) );
	setmemalloc(65536);
	filecount = 1;
	fp1 = stdin;
	
	if (argc > 1)
	{
		if ( (fp2 = fopen(argv[1],"r") ) == NULL)
		{
			fprintf(stderr,"DIFFDOM ERROR: Unable to open file %s\n",argv[1]);
			exit(1);
		}
		else
		{
			if (argc > 2)
			{
				if ((fp1 = fopen(argv[2],"r")) == NULL)
				{
					fprintf(stderr,"DIFFDOM ERROR: Unable to open file %s\n",argv[2]);
					exit(1);
				}
			}
		}
		filecount = 2;
	}
	
	/* !!!! N.B. it is important that stdin is read before a file in the case
	where input is from both, otherwise the stdout file block gets initialised to
	unbuffered and mputc loses the first character output */
	
	fc1 = fc2 = 0;		/* initialise both array vertices */
	while((obj = readobj(fp1)) != NULL)
	{
		if (obj->type != 1)
			freeobj(obj);
		else 
			if ( fc1 >= OBJMAX ){
				fprintf(stderr,"DIFFDOM ERROR: Too many objects\n");
				exit(1);
			}
			else
				objs[fc1++] = obj;
	}
	
	if (fp1 == stdin)
		str = "stdin";
	else
		str = argv[2];

	
	if (filecount == 1) {
		obj = diffdom(objs[0],objs[1]);		/* just diff the first two */
		writeobj(stdout,obj);				/* write the diff'd object only */
	}
	else
	{
		while((obj = readobj(fp2)) !=NULL ) {
			if (obj->type != 1)
				freeobj(obj);
			else 
				if ( fc2 >= OBJMAX ){
					fprintf(stderr,"DIFFDOM ERROR: Too many objects\n");
					exit(1);
				}
				else
					obj2s[fc2++] = obj;
		}
		if ( ( fc1 == 0) || ( fc2 == 0 ) )
			fprintf(stderr,"DIFFDOM ERROR: File count zero unable to proceed\n");
		else
		{
			fc = fc1 < fc2 ? fc1 : fc2;
			for( i = 0; i < fc ; i++ )
			{
				obj = diffdom( objs[i], obj2s[i] );
				writeobj(stdout,obj);
				freeobj(obj);
			}
		}
		for ( i = 0; i < fc1; i++ )
			freeobj(objs[i]);
		for ( i = 0; i < fc2; i++ )
			freeobj(obj2s[i]);
	}
		
}
