/*
 *
 *	E X T R A C T O V . C --
 *
 *  Written: B. Pennington.
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    	   October 1987.
 *
 *	Purpose:	   Test program for woolz function extractov.
 *
 *	Description:   Allows one or two files to be in;put, there must always 
 *				   be two sources of input one of type 1 objects, the other 
 *				   of type 10 polygon objects. When both have beeen read 
 *				   fully an extract on a type 1 is performed for a basis
 *				   until the smaller file is exhausted.
 *
 *	Modifications:														  
 *																		  
 *	05 Nov 1987		BDP		Increase object buffers to 200 ponters		  
 *	29 Oct 1987		BDP		Swap order of reads, now fp2 first. Stdin	  
 *							has to be read first otherwise unbuffered	  
 *							write to stdout occurs and mputc loses the	  
 *							first character.							  
 *																		  
 */

#include <stdio.h>
#include <wstruct.h>

main(argc,argv)
int argc;
char **argv;
{
	FILE *fp1, *fp2, *fp3;
	struct object *obj, *readobj(), *extractov(), *objs[200], *obj2s[200];
	struct object **pobj, **obj1, **objsptr, **obj2sptr;
	char *str;
	int filecount,i,j,fc;
	
	Alloc_limits(0, 1024 * 512 * sizeof(GREY) );
	setmemalloc(65536);
	filecount = 1;
	fp2 = stdin;
		
	if (argc > 1)
	{
		if ( (fp1 = fopen(argv[1],"r") ) == NULL)
		{
			fprintf( stderr,"EXTRACTOV ERROR: Unable to open file %s\n",argv[1]);
			exit(1);
		}
		else
		{
			if (argc > 2)
			{
				if ((fp2 = fopen(argv[2],"r")) == NULL)
				{
					fprintf(stderr,"EXTRACTOV ERROR: Unable to open file %s\n",argv[2]);
					exit(1);
				}
			}
		}
	}
	else
	{
		fprintf(stderr,"EXTRACTOV ERROR: Too few arguements\n");
		exit(2);
	}
	
	i = j = 0;		/* initialise both array vertices */

	/* !!!! N.B. !!!!! It is important that fp2 is read first when this can
	be stdin, otherwise the stdout file block will get initialised to
	unbuffered. mputc doesn't cope with this and the first character of the 
	writeobj will get lost !! */
	
	while((obj = readobj(fp2)) !=NULL )
		obj2s[j++] = obj;
	if (fp2 == stdin)
		str = "stdin";
	else
		str = argv[2];
	fprintf(stderr,"EXTRACTOV: %d objects read from file %s\n",j,str);

	while((obj = readobj(fp1)) != NULL)
		objs[i++] = obj;
	fprintf(stderr,"EXTRACTOV: %d objects read from file %s\n",i,argv[1]);

	/* now want to work out which array contains type 1 objects. and 
	which contains type 10.  Need to bear in mind possible presence of
	header record, and possible duff array */
	
	objsptr = &objs[0];
	obj2sptr = &obj2s[0];
	while ( i > 0)
	{
		if ( ((*objsptr)->type == 10) || ((*objsptr)->type == 1) )
			break;
		else
		{
			/* reduce the count for this array and increment start pointer */
			i--;
			objsptr++;
		}
	}
	while ( j > 0)
	{
		if ( ((*obj2sptr)->type == 10) || ((*obj2sptr)->type == 1) )
			break;
		else
		{
			j--;
			obj2sptr++;
		}
	}
	fc = i < j ? i : j;			/* how many extracts are we going for */
		
	if ( fc <= 0)
	{
		fprintf(stderr,"EXTRACTOV ERROR: File count zero unable to proceed\n");
		exit(3);
	}
	
	fprintf(stderr,"EXTRACTOV: i = %d, j = %d, fc = %d\n",i,j,fc);
	
	/* at this point we know both arrays contain one or more type 1 or type 10
	objects, now check that they contain different types and identify which
	is which */
	
	if ( ((*objsptr)->type == 1) && ((*obj2sptr)->type == 10) )
	{
		obj1 = objsptr;
		pobj = obj2sptr;
	}
	else
		if ( ((*objsptr)->type == 10) && ((*obj2sptr)->type == 1) )
		{
			pobj = objsptr;
			obj1 = obj2sptr;
		}
		else
		{
			fprintf( stderr,"EXTRACTOV ERROR: Unable to distinguish a type 1 and 10 object source\n");
			exit(4);
		}
					
	while ( fc-- > 0)
	{
		if ((*obj1)->type != 1)
		{
			fprintf(stderr,"EXTRACTOV WARNING: Type %d object ignored\n",(*obj1)->type);
			obj1++;
		}
		else
			if((*pobj)->type != 10)
			{
				fprintf(stderr,"EXTRACTOV WARNING: Type %d polyobj ignored\n",(*pobj)->type);
				pobj++;
			}
			else
			{
				obj = extractov(*obj1, *pobj, 10 ); 
				writeobj(stdout,obj);
				freeobj(*obj1);
				freeobj(*pobj);
				freeobj(obj);						
				obj1++;
				pobj++;
			}
	}
			
}
