/*
 * frsamp.c		Jim Piper		08-04-86
 *
 * Re-sample a FIP digitisation at 0.05 micron using
 * convolution with cubic re-sampling function.
 *
 * Procedure:
 *	(1) make a rectangular object
 *	(2) fill its pixels using cubic convolution
 *	(3) threshold to remove background
 *
 * Would be better if an approximate interval-domain (i.e. non-rectangular)
 * object was constructed first, and used "nextgreyinterval()" at least
 * part of the time rather than "greyval()".
 *
 * Modifications
 *
 *	13 Sep 1986		CAS		Includes
 */
#include <stdio.h>
#include <wstruct.h>


main()
{
	struct object *obj, *robj, *makerect(), *readobj();
	struct object *tobj, *threshold();
	struct intervaldomain *idom;
	int i,j,k,l;
	GREY *g;
	
	while ((obj = readobj(stdin)) != NULL) {
		idom = obj->idom;
		l = (idom->lastln - idom->line1 + 3) * 2;
		k = (idom->lastkl - idom->kol1 + 4) * 8 / 3;
		g = (GREY *)calloc(l*k,sizeof(GREY));
		robj = makerect(1,l,1,k,g,NULL,NULL,obj);
		resamp(obj,robj);
		tobj = threshold(robj,1);
		writeobj(stdout,tobj);
		freeobj(obj);
		freeobj(robj);
		tobj->vdom = NULL;
		freeobj(tobj);
	}
}


/*
 * resample: new points at frequency *2 in line direction,
 * at frequency *(8/3) in column direction, to get approximately
 * 0.05 micron pixel spacing.
 */
static int lra[4] = {-1250, 6250, 6250, -1250};
static int kra[8][6] = {
	{    0,10000,    0,     0,    0,    0},
	{-1465, 7715, 4629,  -879,    0,    0},
	{ -469, 2969, 8906, -1406,    0,    0},
	{    0, -957, 9707, 1387,  -137,    0},
	{    0,-1250, 6250, 6250, -1250,    0},
	{    0, -137, 1387, 9707,  -957,    0},
	{    0,    0,-1406, 8906, 2969,  -469},
	{    0,    0, -879, 4629, 7715, -1465}
};
resamp(o1,o2)
struct object *o1,*o2;
{
	int i,j,k,l,k2,l2;
	int g;
	GREY *greyval();
	/* first pass - every-other line - very inefficient !! */
	for (l=o1->idom->line1; l<=o1->idom->lastln; l++) {
		l2 = 2*(l-o1->idom->line1+1);	/* STARTS AT LINE 2 */
		k2 = 0;
		for (k=o1->idom->kol1; k<=o1->idom->lastkl+2; k+=3) {
			for (i=0; i<8; i++) {
				g = 0;
				for (j=0; j<6; j++)
					g += (kra[i][j]* (*greyval(o1,l,k+j-3)));
				*greyval(o2,l2,k2++) = g/10000;
			}
		}
	}
	/* second pass - every-other line - very inefficient !! */
	for (l2=1; l2<=o2->idom->lastln; l2+=2) {
		for (k2=o2->idom->kol1; k2<=o2->idom->lastkl; k2++) {
			g = 0;
			for (i=0; i<4; i++)
				g += (lra[i] * (*greyval(o2,l2+2*i-3,k2)));
			*greyval(o2,l2,k2) = g/10000;
		}
	}
}
