/************************************************************************/
/*																		*/
/*						histotest.c										*/
/*																		*/
/************************************************************************/
/*																		*/
/*	Written by:		B Pinnington										*/
/*																		*/
/*	Purpose:		Test program for histogram functions.				*/
/*																		*/
/*	Description:	Performs a number of histogram interactions upon a 	*/
/*					histogram object input from stdin. This routine is 	*/
/*					not intended to exhautively test histogram routine	*/
/*					interaction, but to involve most histogram routines	*/
/*					in dealing with a range of data. It is thus intended*/
/*					to be used within a shell script generating large	*/
/*					quantities of automatic data, rather than as a		*/
/*					stand alone utility.								*/
/*																		*/
/************************************************************************/
/*																		*/
/*	Modifications:														*/
/*																		*/
/*  23 Nov 1987		BDP		Added help option							*/
/*																		*/
/************************************************************************/

#include <stdio.h>
#include <wstruct.h>

extern int histinvert(), histonormalise(), histo256normalise(), histosmooth();
extern int histothreshsmooth();

struct object *histotest(hobj,s)
struct object *hobj;
int s;
{
	struct object *hobj;

	histonormalise(hobj);
#ifdef POLYBUG
	writeobj(stdout,hobj);
#endif POLYBUG
	histo256normalise(hobj);
#ifdef POLYBUG
	writeobj(stdout,hobj);
#endif POLYBUG
	histosmooth(hobj,3);
#ifdef POLYBUG
	writeobj(stdout,hobj);
#endif POLYBUG
	histothreshsmooth(hobj);
#ifdef POLYBUG
	writeobj(stdout,hobj);
#endif POLYBUG
	histinvert(hobj);
	return(hobj);	
}

main(argc,argv)
char **argv;
int argc;
{
	struct object *obj, *readobj(), *nobj;
	int smoothings;
	
	smoothings = 3;			/* finger in the air defaults */
	while(--argc > 0)
	{
		if ((*++argv)[0] == '-') 
		{
			switch( argv[0][1] )
			{
			case '?':
				fprintf(stderr,"Syntax:     histotest [<opts>]\n");
				fprintf(stderr,"Function:   Test a number of histo functions \n");
				fprintf(stderr,"Options:\n");
				fprintf(stderr,"    -s=<smooth> Number of times to smooth histogram \n");
				exit(0);
			case 's':
				if (argv[0][2] == '=')
					smoothings = atoi( &argv[0][3] );
				break;
			default:
				fprintf(stderr,"HISTOTEST: Unrecognised parameter ignored\n");
				break;
			}
		}
		else
			fprintf(stderr,"HISTOTEST: Illegal arguement %s\n",*argv);
	}
	fprintf(stderr,"HISTOTEST DEBUG: s = %d\n",smoothings);
	setmemalloc(32 * 1024);
	Alloc_limits(0,512 * 1024 * sizeof(GREY));

	while((obj = readobj(stdin)) != NULL) {
		if (obj->type == 10)
		{
			nobj = histotest(obj,smoothings);
			writeobj(stdout,nobj);
			freeobj(nobj);
		}
		else
			writeobj(stdout,obj);
		freeobj(obj);
	}
}
