/*
 *
 *	I N T E R S E C T . C --
 *
 *  Written: B. Pennington.
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    	   October 1987.
 *
 *	Purpose:	   Testing the intersectn woolz function.
 *
 *	Description:   Takes objects from stdin and intersects them all
 *					before writing to stdout. optionally the input		  
 *					objects can be intersected in groups of 'n' and 	  
 *					written to stdout by using -c n command.			  
 *
 *	Modifications:														  
 *																		  
 *	23 Nov 1987		BDP		Added help									  
 *	20 Nov 1987		BDP		Changed arguement format to -c=n from -c n	  
 *	05 Nov 1987		BDP		Change the objects list to 200 not 100 long   
 *	27 Oct 1987		BDP		Added the -c option. Changed display message  
 *																		  
 */

#include <stdio.h>
#include <wstruct.h>

main(argc,argv)
int argc;
char **argv;
{
	struct object *obj, *readobj(), *objs[100], *intersectn();
	int i,j;
	int intercount;
	int greytabs = 1;
	
	setmemalloc(65536);
	intercount = i = 0;
	while(--argc > 0)
	{
		if ((*++argv)[0] == '-') 
		{
			switch( argv[0][1] )
			{
			case '?':
				fprintf(stderr,"Syntax:     intersect [<opts>]\n");
				fprintf(stderr,"Function:   Intersect 2 or more type one objects\n");
				fprintf(stderr,"Options:\n");
				fprintf(stderr,"    -c=<count> Max count of objects per intersection \n");
				exit(0);
			case 'c':
				if (argv[0][2] == '=')
					intercount = atoi( &argv[0][3] );
				break;
			default:
				fprintf(stderr,"INTERSECT: Unrecognised parameter ignored\n");
				break;
			}
		}
		else
			fprintf(stderr,"INTERSECT: Illegal arguement %s\n",*argv);
	}
	while((obj = readobj(stdin)) != NULL)
	{
		if (obj->type != 1)
			freeobj(obj);
		else 
		{
			objs[i++] = obj;
			if (obj->vdom == NULL)
				greytabs = 0;
		}
	}
#ifdef DEBUG
	fprintf(stderr,"INTERSECT: %d objects read \n",i);
#endif DEBUG
	
	if (intercount == 0)
	{
		obj = intersectn(objs,i,greytabs);
		writeobj(stdout,obj);
		freeobj(obj);
	}
	else
	{
		for ( j = 0; j < i; j+=intercount)
		{
			obj = intersectn(&objs[j], (i-j < intercount ? i-j : intercount),greytabs);
			writeobj(stdout,obj);
			freeobj(obj);
		}
	}
}
