/************************************************************************/
/*																		*/
/*																		*/
/*																		*/
/************************************************************************/
/*																		*/
/*	Written by:		B. Pinnington										*/
/*																		*/
/*	Purpose:		To perform NULL object handling tests on woolz		*/
/*																		*/
/*	Description:	Calls in turn, after any necessary initialisation	*/
/*					procedures, each of the woolz library routines with	*/
/*					NULL objects as parameters, and checks expected 	*/
/*					illegal status responses. All output is to stderr.	*/
/*					Routines are grouped into type related groups for	*/
/*					convenience.										*/
/*																		*/
/************************************************************************/
/*																		*/
/*	Modifications:														*/
/*																		*/
/************************************************************************/


#include <stdio.h>
#include <wstruct.h>

main(argc,argv)
char **argv;
int argc;
{
	struct object *obj,  *convhull(), *dilation(), *erosion(), *shift();
	struct object *erosion4(), *histo(), *ibound(), *minwrect(), *octagon();
	struct object *realsmoothunitpoly(), *skeleton(), *spin(), *threshold(), *unitbound();
	struct object *diffdom(), *intersectn(), *unionn();
	struct polygondomain *intrp8(), *inttorealpoly(), *iuspace(), *unitspace();
 	GREY greybuff[4];
	struct object *objarr[10];
	struct pframe picture;
	struct iwspace iwsp;
	struct gwspace gwsp;
	int i;
		
	fprintf(stderr,"testing utility functions\n");
	if ( area(NULL) == 0 )
		fprintf( stderr, "****** area ok \n");
	else
		fprintf( stderr, ">>>>>> area fails \n");

	if ( greyval(NULL,10,10) == 0 )
		fprintf( stderr, "****** greyval ok \n");
	else
		fprintf( stderr, ">>>>>> greyval fails \n");

	if ( intcount(NULL) == 0 )
		fprintf( stderr, "****** intcount ok \n");
	else
		fprintf( stderr, ">>>>>> intcount fails \n");

	if ( linearea(NULL) == 0 )
		fprintf( stderr, "****** linearea ok \n");
	else
		fprintf( stderr, ">>>>>> linearea fails \n");

	if ( seqsize(NULL) == 0 )
		fprintf( stderr, "****** seqsize ok \n");
	else
		fprintf( stderr, ">>>>>> seqsize fails \n");

	if ( compthr3(NULL) == 0 )
		fprintf( stderr, "****** compthr3 ok \n");
	else
		fprintf( stderr, ">>>>>> compthr3 fails \n");

	if ( edgegv(NULL) == 0 )
		fprintf( stderr, "****** edgegv ok \n");
	else
		fprintf( stderr, ">>>>>> edgegv fails \n");

	if ( hullarea(NULL,&i) == 0 )
		fprintf( stderr, "****** hullarea ok \n");
	else
		fprintf( stderr, ">>>>>> hullarea fails \n");

	if ( mass(NULL) == 0 )
		fprintf( stderr, "****** mass ok \n");
	else
		fprintf( stderr, ">>>>>> mass fails \n");

	if ( maxdiam(NULL) == 0 )
		fprintf( stderr, "****** maxdiam ok \n");
	else
		fprintf( stderr, ">>>>>> maxdiam fails \n");

	if ( label(NULL,&i,&objarr[0],10,0) == 0 )
		fprintf( stderr, "****** label ok \n");
	else
		fprintf( stderr, ">>>>>> label fails \n");

	/*************************************************/
	
	fprintf(stderr,"\nstatus returning procedures \n\n");
	
	if ( standardidom(NULL) > 0 )
		fprintf( stderr, "****** standardidom ok \n");
	else
		fprintf( stderr, ">>>>>> standardidom fails \n");

	if ( setval(NULL,1) > 0 )
		fprintf( stderr, "****** setval ok \n");
	else
		fprintf( stderr, ">>>>>> setval fails \n");

	if ( grey4val(NULL,10,10,&greybuff[0]) > 0 )
		fprintf( stderr, "****** grey4val ok \n");
	else
		fprintf( stderr, ">>>>>> grey4val fails \n");

	if ( greyrange(NULL,&greybuff[0],&greybuff[1]) > 0 )
		fprintf( stderr, "****** greyrange ok \n");
	else
		fprintf( stderr, ">>>>>> greyrange fails \n");

	if ( histo256normalise(NULL) > 0 )
		fprintf( stderr, "****** histo256normalise ok \n");
	else
		fprintf( stderr, ">>>>>> histo256normalise fails \n");

	if ( histinvert(NULL) > 0 )
		fprintf( stderr, "****** histinvert ok \n");
	else
		fprintf( stderr, ">>>>>> histinvert fails \n");

	if ( histonormalise(NULL) > 0 )
		fprintf( stderr, "****** histonormalise ok \n");
	else
		fprintf( stderr, ">>>>>> histonormalise fails \n");

	if ( histosmooth(NULL,1) > 0 )
		fprintf( stderr, "****** histosmooth ok \n");
	else
		fprintf( stderr, ">>>>>> histosmooth fails \n");

	if ( histothreshsmooth(NULL) > 0 )
		fprintf( stderr, "****** histothreshsmooth ok \n");
	else
		fprintf( stderr, ">>>>>> histothreshsmooth fails \n");

	if ( htnormalise(NULL) > 0 )
		fprintf( stderr, "****** htnormalise ok \n");
	else
		fprintf( stderr, ">>>>>> htnormalise fails \n");

	if ( normalise(NULL) > 0 )
		fprintf( stderr, "****** normalise ok \n");
	else
		fprintf( stderr, ">>>>>> normalise fails \n");

	if ( invert(NULL) > 0 )
		fprintf( stderr, "****** invert ok \n");
	else
		fprintf( stderr, ">>>>>> invert fails \n");

	if ( picframe(NULL,&picture) > 0 )
		fprintf( stderr, "****** picframe ok \n");
	else
		fprintf( stderr, ">>>>>> picframe fails \n");

	if ( boxobj(NULL,&picture,1) > 0 )
		fprintf( stderr, "****** boxobj ok \n");
	else
		fprintf( stderr, ">>>>>> boxobj fails \n");

	if ( backlight(NULL,&picture,2) > 0 )
		fprintf( stderr, "****** backlight ok \n");
	else
		fprintf( stderr, ">>>>>> backlight fails \n");

	if ( frameobjnum(1,NULL,&picture) > 0 )
		fprintf( stderr, "****** frameobjnum ok \n");
	else
		fprintf( stderr, ">>>>>> frameobjnum fails \n");

	if ( frameobjnnum(1,NULL,&picture) > 0 )
		fprintf( stderr, "****** frameobjnnum ok \n");
	else
		fprintf( stderr, ">>>>>> frameobjnnum fails \n");

	if ( threshlabel(NULL,1,&i,&objarr[0],10,0) > 0 )
		fprintf( stderr, "****** threshlabel ok \n");
	else
		fprintf( stderr, ">>>>>> threshlabel fails \n");

	initrasterscan(NULL,&iwsp,0);
	if ( nextintervalendpoint(&iwsp) > 0 )
		fprintf( stderr, "****** nextintervalendpoint ok \n");
	else
		fprintf( stderr, ">>>>>> nextintervalendpoint fails \n");

	if ( firstintptr(NULL,1) == NULL )
		fprintf( stderr, "****** firstintptr ok \n");
	else
		fprintf( stderr, ">>>>>> firstintptr fails \n");

	if ( lastintptr(NULL,1) == NULL )
		fprintf( stderr, "****** lastintptr ok \n");
	else
		fprintf( stderr, ">>>>>> lastintptr fails \n");

	/**************************************************/
	
	fprintf(stderr,"\nobject routines testing\n\n");

	
	if ( convhull(NULL) == NULL )
		fprintf( stderr, "****** convhull ok \n");
	else
		fprintf( stderr, ">>>>>> convhull fails \n");

	if ( dilation(NULL) == NULL )
		fprintf( stderr, "****** dilation ok \n");
	else
		fprintf( stderr, ">>>>>> dilation fails \n");

	if ( erosion(NULL) == NULL )
		fprintf( stderr, "****** erosion ok \n");
	else
		fprintf( stderr, ">>>>>> erosion fails \n");

	if ( erosion4(NULL) == NULL )
		fprintf( stderr, "****** erosion4 ok \n");
	else
		fprintf( stderr, ">>>>>> erosion4 fails \n");

	if ( histo(NULL) == NULL )
		fprintf( stderr, "****** histo ok \n");
	else
		fprintf( stderr, ">>>>>> histo fails \n");

	if ( ibound(NULL,0) == NULL )
		fprintf( stderr, "****** ibound ok \n");
	else
		fprintf( stderr, ">>>>>> ibound fails \n");

	if ( minwrect(NULL) == NULL )
		fprintf( stderr, "****** minwrect ok \n");
	else
		fprintf( stderr, ">>>>>> minwrect fails \n");

	if ( octagon(NULL,0) == NULL )
		fprintf( stderr, "****** octagon ok \n");
	else
		fprintf( stderr, ">>>>>> octagon fails \n");

	if ( realsmoothunitpoly(NULL) == NULL )
		fprintf( stderr, "****** realsmoothunitpoly ok \n");
	else
		fprintf( stderr, ">>>>>> realsmoothunitpoly fails \n");

	if ( shift(NULL,1,1) == NULL )
		fprintf( stderr, "****** shift ok \n");
	else
		fprintf( stderr, ">>>>>> shift fails \n");

	if ( skeleton(NULL,0) == NULL )
		fprintf( stderr, "****** skeleton ok \n");
	else
		fprintf( stderr, ">>>>>> skeleton fails \n");

	if ( spin(NULL,90) == NULL )
		fprintf( stderr, "****** spin ok \n");
	else
		fprintf( stderr, ">>>>>> spin fails \n");

	if ( threshold(NULL,1) == NULL )
		fprintf( stderr, "****** threshold ok \n");
	else
		fprintf( stderr, ">>>>>> threshold fails \n");

	if ( unitbound(NULL) == NULL )
		fprintf( stderr, "****** unitbound ok \n");
	else
		fprintf( stderr, ">>>>>> unitbound fails \n");

	/* for object returning routines which receive several object parameters
	more extensive comination testing is performed elsewhere, testing here is
	just for all nulls */
	
	if ( diffdom(NULL,NULL) == NULL )
		fprintf( stderr, "****** diffdom ok \n");
	else
		fprintf( stderr, ">>>>>> diffdom fails \n");

	for (i=0; i<3 ; i++)
		objarr[i] = NULL;
		
	if ( intersectn(&objarr[0],3,0) == NULL )
		fprintf( stderr, "****** intersectn ok \n");
	else
		fprintf( stderr, ">>>>>> intersectn fails \n");

	if ( unionn(&objarr[0],3,0) == NULL )
		fprintf( stderr, "****** unionn ok \n");
	else
		fprintf( stderr, ">>>>>> unionn fails \n");

	/****************************************************/
	
	fprintf(stderr,"\nscanning routines testing \n\n");
	

	initgreyscan(NULL,&iwsp,&gwsp);
	if (nextgreyinterval(&iwsp) > 0)
		fprintf( stderr, "****** greyscan ok \n");
	else
		fprintf( stderr, ">>>>>> greyscan fails \n");

	initrasterscan(NULL,&iwsp,0);
	if (nextinterval(&iwsp) > 0)
		fprintf( stderr, "****** rasterscan ok \n");
	else
		fprintf( stderr, ">>>>>> rasterscan fails \n");


	/****************************************************/
	
	fprintf(stderr,"\npolygon routines testing \n\n");
	
	if ( intrp8(NULL) == NULL )
		fprintf( stderr, "****** intrp8 ok \n");
	else
		fprintf( stderr, ">>>>>> intrp8 fails \n");

	if ( inttorealpoly(NULL) == NULL )
		fprintf( stderr, "****** inttorealpoly ok \n");
	else
		fprintf( stderr, ">>>>>> inttorealpoly fails \n");

	if ( iuspace(NULL) == NULL )
		fprintf( stderr, "****** iuspace ok \n");
	else
		fprintf( stderr, ">>>>>> iuspace fails \n");

	if ( unitspace(NULL) == NULL )
		fprintf( stderr, "****** unitspace ok \n");
	else
		fprintf( stderr, ">>>>>> unitspace fails \n");

	if ( polyinvert(NULL) > 0 )
		fprintf( stderr, "****** polyinvert ok \n");
	else
		fprintf( stderr, ">>>>>> polyinvert fails \n");

	if ( polymult(NULL,1) > 0 )
		fprintf( stderr, "****** polymult ok \n");
	else
		fprintf( stderr, ">>>>>> polymult fails \n");

	if ( polysmooth(NULL,1) > 0 )
		fprintf( stderr, "****** polysmooth ok \n");
	else
		fprintf( stderr, ">>>>>> polysmooth fails \n");

	if ( polysqueeze(NULL,1,1) > 0 )
		fprintf( stderr, "****** polysqueeze ok \n");
	else
		fprintf( stderr, ">>>>>> polysqueeze fails \n");


	fprintf(stderr,"null testing completed\n");
}
