/************************************************************************/
/*																		*/
/*					nulltest2.c											*/
/*																		*/
/************************************************************************/
/*																		*/
/*	Written by:		B. Pinnington										*/
/*																		*/
/*	Purpose:		To perform NULL object handling tests on woolz		*/
/*																		*/
/*	Description:	This routine is used to test a few specific woolz	*/
/*					routines which take more than one object parameter,	*/
/*					to test the effects of various combinations of NULL */
/*					empty, and correct objects. The file objempty is 	*/
/*					used as the source of an empty object, and legal 	*/
/*					object filename is taken from argv, thus allowing	*/
/*					several different legal objects to be tested.		*/
/*					objects are read using readobj						*/
/*																		*/
/************************************************************************/
/*																		*/
/*	Modifications:														*/
/*																		*/
/************************************************************************/


#include <stdio.h>
#include <wstruct.h>

main(argc,argv)
char **argv;
int argc;
{
	FILE *fp, *fopen();
	struct object *diffdom(), *intersectn(), *unionn(), *intersect2(), *union2();
	struct object *makemain(), *readobj(), *extractov(), *dilation(), *ibound();
	struct intervaldomain *makedomain();
	struct object *obj, *tobj, *bobj, *emptyobj, *polyobj, *objarr[10];
	struct boundlist *bound;
	struct polygondomain *polydmn;
	struct pframe picture;
	struct iwspace iwsp;
	struct gwspace gwsp;
	int i;

	if ((fp = fopen(argv[1],"r")) == NULL)
	{
		fprintf(stderr,"cannot open file %s\n",argv[1]);
		exit(1);
	}
	
	i = 0;
	while((objarr[i] = readobj(fp)) != NULL)
		i++;
		
	emptyobj = makemain(1,makedomain(1,1,0,1,0),NULL,NULL,NULL);
	
	fprintf(stderr,"diffdom testing\n");
	
	obj =  diffdom(objarr[0], NULL);
	obj =  diffdom(objarr[0],obj);
	if (wzcheckobj(obj) > 0)
		fprintf(stderr,">>>>>>>>diffdom null 1 failed\n");
	else
		if (wzemptyidom(obj->idom) == 0)
			fprintf(stderr,">>>>>>>>diffdom null 1 failed pt 2\n");
		
	obj =  diffdom(NULL,objarr[0]);
	obj =  diffdom(objarr[0],obj);
	if (wzcheckobj(obj) > 0)
		fprintf(stderr,">>>>>>>>diffdom null 2 failed\n");
	else
		if (wzemptyidom(obj->idom) == 0)
			fprintf(stderr,">>>>>>>>diffdom null 2 failed pt 2\n");

	obj =  diffdom(NULL, emptyobj);
	if (wzcheckobj(obj) > 0)
		fprintf(stderr,">>>>>>>>diffdom null 3 failed\n");
	else
		if (wzemptyidom(obj->idom) == 0)
			fprintf(stderr,">>>>>>>>diffdom null 3 failed pt 2\n");

	fprintf(stderr,"********diffdom test complete\n");

	/*	start of the intersect testing */
	
	fprintf(stderr,"start of intersect testing\n");
	
	/* the array objarr currently contains 'i' objects, so add any NULL
	of empty objects to the end of whats been read in, and after suitable
	increment of i, use this as other parameter */
	
	objarr[i++] = emptyobj;
	objarr[i] = NULL;		/* include this only on second test */
	
	obj = intersectn(&objarr[0],i,0);
	if (wzcheckobj(obj) > 0)
		fprintf(stderr,">>>>>>>>intersect empty test failed\n");
	else
		if (wzemptyidom(obj->idom) == 0)
			fprintf(stderr,">>>>>>>>intersect empty test failed pt 2\n");

	/* include NULL in this test by pre incr of i */
	
	obj = intersectn(&objarr[0],++i,0);
	if (wzcheckobj(obj) == 0)
		fprintf(stderr,">>>>>>>>intersect NULL test failed\n");

	fprintf(stderr,"********intersectn test complete\n");
	
	/* start of unionn testing */
	
	fprintf(stderr,"start of union testing\n");
	
	/* the object array has already been set up to include what ever objects
	were read from the specified file plus an empty and a NULL file. the
	index variable i currently include both these extra objects, so decr
	will remove the NULL object  */
	
	obj = unionn(&objarr[0],i,0);
	if (wzcheckobj(obj) > 0)
		fprintf(stderr,">>>>>>>>union NULL test failed\n");
	else
		if (wzemptyidom(obj->idom) > 0)
			fprintf(stderr,">>>>>>>>union NULL test failed pt 2\n");

	/* result of this tests should be same as above , neither empty nor NULL */
	
	obj = unionn(&objarr[0],--i,0);
	if (wzcheckobj(obj) > 0)
		fprintf(stderr,">>>>>>>>union NULL test failed\n");
	else
		if (wzemptyidom(obj->idom) > 0)
			fprintf(stderr,">>>>>>>>union NULL test failed pt 2\n");

	fprintf(stderr,"********unionn test complete\n");
	
	/* start of extractov testing */
	
	fprintf(stderr,"start of extractov testing\n");
	
	fprintf(stderr,"	null polygon test\n");
	obj = extractov(objarr[0],NULL,10);
	if (wzcheckobj(obj) == 0)
		fprintf(stderr,">>>>>>>>extractov NULL polygon test failed\n");

	/* make a surrounding polygon of the object to be used in two later tests */
	
	bobj = ibound(objarr[0],0);		/* not a closed polygon */
	if (wzcheckobj(bobj) > 0)
		fprintf(stderr,">>>>>>>>ibound failed\n");
	else
	{
		bound = (struct boundlist *)bobj->idom;
		fprintf(stderr,"	making polygon\n");
		polyobj = makemain(10,bound->poly,NULL,NULL,NULL);
		
		fprintf(stderr,"	null object test\n");
		obj = extractov(NULL,polyobj,10);
		if (wzcheckobj(obj) == 0)
			fprintf(stderr,">>>>>>>>extractov NULL object test failed\n");

		/* next two tests are a little more involved, they aim to test the
		functioning of extract when the polygon lies wholly outside the object,
		ie an empty object results. also the routines correct funcitoning is
		tested by deriving a polygon from the object itself, and extracting with 
		that.  */
	
		fprintf(stderr,"	normal overlapping objects test\n");
		obj = extractov(objarr[0],polyobj,10);
		if (wzcheckobj(obj) > 0)
			fprintf(stderr,">>>>>>>>extractov normal object test failed\n");

		fprintf(stderr,"	freeing objects\n");
		freeobj(polyobj);
		freeobj(bobj);

		/* for outside polygon test, generate a polygon known to be outside
		objarr[0], by dilating it 10 times, then getting the boundary of
		the dilated object, and doing an extractov for a figure of less than
		10. */

		obj = objarr[0];
		obj = makemain(1,newidomain(obj->idom),NULL,NULL,NULL);
				
		for (i=0; i < 10 ; i++ )
		{
			/* as dilation creates a new object each time , need to keep
			freeing the old one , so need to use another object pointer 
			to temporarily hold the old object.  */
			
			tobj = obj;
			obj = dilation(tobj);
			freeobj(tobj);
		}
		
		fprintf(stderr,"	making outer polygon\n");
		bobj = ibound(obj,0);		/* not a closed polygon */
		if (wzcheckobj(bobj) > 0)
			fprintf(stderr,">>>>>>>>ibound failed pt 2\n");
		else
		{
			fprintf(stderr,"	outer polygon test\n");
			bound = (struct boundlist *)bobj->idom;
			polyobj = makemain(10,bound->poly,NULL,NULL,NULL);
			
			obj = extractov(obj,polyobj,6);
			if (wzcheckobj(obj) > 0)
				fprintf(stderr,">>>>>>>>extractov outside object test failed\n");
			else
				if (wzemptyidom(obj) > 0)
					fprintf(stderr,">>>>>>>>extractov outside object test failed pt 2\n");
			
			fprintf(stderr,"	freeing objects\n");
			freeobj(polyobj);
			freeobj(bobj);
		}										
	}
	
	fprintf(stderr,"extractov testing completed\n");
		
	freeobj(emptyobj);
	fclose(fp);
	
}		
