/*
 *      P I C T U R E . C  -  output woolz object to framestore
 *
 *
 *  Written: Jim P
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    16th Aug 1983
 *
 *  Modifications
 *
 *	10 Jan 1991		CAS		Fumbled around, trying to make it possible to see
 *							single objects, finally understood what BDP was up to
 *							and realised why it caused so many problems - centering
 *							a cell object by object is hard
 *	 9 Jan 1989		CAS		Added offset option to cope with files with LUT's
 *							and non objecty things at beginning
 *	04 Dec 1987		BDP		Added -l lut reset option using llut function
 *							removed the exit(1) when screen full, now clears
 *							screen and resets line and column to origins. also
 *							restructured whole loop now needs one less tier of
 *							variables and I can understand it
 *							finally test for end against dg->extent not 4096 now
 *	01 Dec 1987		BDP		Added -i[=<num>] option. Now by default no incr occurs so
 *							cells can be output normally. (use -c to centre )
 *							Also added -f option. Scale for FIP. now means
 *							that program fippic.c is redundant
 *	22 oct 1987		CAS		_memmins set to 64k
 *	21 Sep 1987		CAS		Retro-fitted BDP's center option
 *	 2 Sep 1987		CAS		Major tidy up + Modified for new ddgs library
 *	12 Sep 1986		CAS		Includes
 *	????			Jimp	Changed def scale to 8 and km/lmstart to 
 *							a define (now 512)
 */
#include <stdio.h>
#include <ddgs.h>
#include <wstruct.h>

extern int _memmins;

#define DINC 512

/*
 *	I N T E R R U P T --  Signal catcher
 *
 */
interrupt(sig)
int		sig;
{
}

main(argc,argv)
int		argc;
char	**argv;
{
	char	*swptr, **targv;
	int		targc, doneone;
	FILE	*infile;
	int		store = FS_VME512, devno = 0;
	DDGS	*dg, *ddgsopen();
	struct object *obj, *readobj();
	struct pframe f;
	register int xmax, ymax, km,lm;
	int kmstart,lmstart,objnum,i;
	int incpos, incr;		/* incpos is a boolean, incr is increment scale */
	int fip;				/* boolean if true then object is FIP digitised */
	int center = 0;
	int scale=8;
	int initlut = 0;
	int noclear = 0;
	int	offset = 0;

	kmstart = lmstart = DINC;
	f.scale = 1;
	f.ix = f.iy = 8;
	f.ox = f.oy = 0;
	incr = DINC;
	incpos = 0;				/* default to no increment between objects */	
	fip = 0;
	_memmins = 65536;
#ifdef INTERRUPT
	intercept(interrupt);
#endif INTERRUPT
	targc = argc; targv = argv;

	argc--; argv++;			/* dump process name */
	while (argc > 0) {
		if (*argv[0] == '-') {
			swptr = *argv;
			*argv = NULL;
			while (*(++swptr)) {
				switch (*swptr) {
					case '?':
						printf("Syntax:   picture <options> scale column line <options>\n");
						printf("Function: \n");
						printf("Options:  v - Show version number\n");
						printf("          c - center image\n");
						printf("          f - Scale for FIP digitised objects\n");
						printf("          h - horizontal mirror image\n");
						printf("          i[=<inc>] - increment for next object\n");
						printf("          l - reset lut\n");
						printf("          m - vertical mirror image\n");
						printf("          n - don't clear display\n");
						printf("          o - Set offset to start of objects in file\n");
						printf("          1 - Use seat 1 framestore\n");
						printf("          2 - Use seat 2 framestore\n");
						printf("                  also 3 and 4\n");
						printf("          V - Virtuoso framestore (Default CRS 512)\n");
						printf("          ? - Help\n");
						exit(0);
					case 'v':
						printf("Picture - Version 1.0\n");
						exit(0);
					case 'n':
						noclear =1;
						break;
					case 'o':
						offset = atoi(*++argv);
						*argv = NULL; argc--;
						break;
					case 'f':
						fip = 1;
						break;
					case 'h':
						f.ix = -f.ix;
						break;
					case 'i':
						incpos = 1;		/* increment position for next object */
						if (swptr[1] == '='){
							incr = atoi(&swptr[2]);
							/* now inc swptr to one before the end to end while loop */
							while (swptr[1] != '\0' )
								swptr++;
						}
						break;
					case 'l':
						initlut = 1;
						break;
					case 'm':
						f.iy = -f.iy;
						break;
					case 'c':
						center = 1;
						break;
					case '1':
						devno = 0;
						break;
					case '2':
						devno = 1;
						break;
					case '3':
						devno = 2;
						break;
					case '4':
						devno = 3;
						break;
					case 'V':
						store = FS_VIRTUOSO_INVERT;
						break;
/*					case 'x':
 *						param = *++argv;
 *						*argv = NULL; argc--;
 *						break;
 */
					default:
						printf("picture - unknown option\n");
				}
			}
		}
		argv++; argc--;
	}

	argc = targc; argv = targv;
	argc--; argv++;			/* dump process name */
	doneone = 0;
/*
 *	Use remaining params 
 */
	while (argc) {
		if (*argv != NULL) {
			switch (doneone) {
				case 0:
					scale = atoi(*argv);
					break;
				case 1:
					kmstart = lmstart = atoi(*argv);
					break;
				case 2:
					lmstart = atoi(*argv);
					break;
			}
			doneone++;
		}
		argc--; argv++;
	}
	
	/* next sort scaling out. bit of a mess here. need to beware of mirror
	image and fip options which could have been defined in any order above */
	
	if (f.ix > 0)
		f.ix = scale;
	else
		f.ix = -scale;
	if (f.iy > 0)
		f.iy = scale;
	else
		f.iy = -scale;
		
	if (fip)
		f.iy = f.iy*3/4;	/* scale for FIP digitised cells */

	dg = ddgsopen(store,devno);
	dinit();
	if (initlut)
		llut();				/* must do this after the ddgsopen */
	if (center) {
		kmstart = dg->xcenter;
		lmstart = dg->ycenter;
	}
	if (noclear == 0) {
		fast();
		clear();
	}
	slow();
	km = kmstart;
	lm = lmstart;
	xmax = dg->lineln << dg->xshift;
	ymax = dg->lineno << dg->yshift;
	
	if (offset) {
		for (i=0; i<offset; i++)
			getc(stdin);
	}
	while ((obj = readobj(stdin)) != NULL) {
		if (obj->type == 1) {
			f.ox = (obj->idom->kol1 + obj->idom->lastkl + 1)/2;
			f.oy = (obj->idom->line1 + obj->idom->lastln + 1)/2;
			f.dx = km;
			f.dy = lm;
			picframe(obj,&f);
			if (incpos) {
				km += incr;
				if (km >= xmax) {
					km = kmstart;
					if ((lm += incr) >= ymax) {
						if (noclear == 0) {
							fast();
							clear();
							slow();
						}
						lm = lmstart;
					}
				}
			}
		}
		freeobj(obj);
	}
	finish();
}
