/************************************************************************/
/*																		*/
/*						polytest.c										*/
/*																		*/
/************************************************************************/
/*																		*/
/*	Written by:		B Pinnington										*/
/*																		*/
/*	Purpose:		Test program for polygon functions.					*/
/*																		*/
/*	Description:	Performs a number of polygon interactions upon a 	*/
/*					polygon object input from stdin. This routine is 	*/
/*					not intended to exhautively test polygon routine	*/
/*					interaction, but to involve most polygon routines	*/
/*					in dealing with a range of data. It is thus intended*/
/*					to be used within a shell script generating large	*/
/*					quantities of automatic data, rather than as a		*/
/*					stand alone utility.								*/
/*																		*/
/************************************************************************/
/*																		*/
/*	Modifications:														*/
/*																		*/
/*																		*/
/************************************************************************/

#include <stdio.h>
#include <wstruct.h>

extern int polymult(),polysqueeze(),polysmooth(),polyinvert();
extern struct object *polytoobj(), *rsupoly(), *makemain();
extern struct polygondomain *makepolydmn();

struct object *polytest(pobj,m,s)
struct object *pobj;
m,s;
{
	struct object *pobj2,*tobj;
	struct polygondomain *pdom, *pdom2;
	float sq;
		
	sq = 1.0;
	pdom = (struct polygondomain *)pobj->idom;
	pdom2 = makepolydmn(pdom->type, pdom->vtx, pdom->nvertices, pdom->maxvertices, 1);
	polymult(pdom2,m);
#ifdef POLYBUG
	pobj2 = makemain( 10, pdom2, NULL, NULL, NULL);
	writeobj(stdout,pobj2);
#endif POLYBUG
	polyinvert(pdom2);
#ifdef POLYBUG
	writeobj(stdout,pobj2);
#endif POLYBUG
	polysmooth(pdom2,s);
#ifdef POLYBUG
	writeobj(stdout,pobj2);
#endif POLYBUG
	if ( m > 0 )
		sq = 1.0/m;				/* also convert int to float */
	polysqueeze(pdom2, sq, sq );
#ifdef POLYBUG
	writeobj(stdout,pobj2);
#endif POLYBUG
	tobj = makemain( 10, iuspace(pdom2), NULL, NULL, NULL);
#ifdef POLYBUG
	freeobj(pobj2);
#else POLYBUG
	Free(pdom2);
#endif POLYBUG

	return(tobj);	
}

main(argc,argv)
char **argv;
int argc;
{
	struct object *obj, *readobj(), *nobj;
	int multfactor,smoothings;
	
	multfactor = 10;
	smoothings = 3;			/* finger in the air defaults */
	while(--argc > 0)
	{
		if ((*++argv)[0] == '-') 
		{
			switch( argv[0][1] )
			{
			case '?':
				fprintf(stderr,"Syntax:     polytest [<opts>]\n");
				fprintf(stderr,"Function:   Test a series of polygon functions\n");
				fprintf(stderr,"Options:\n");
				fprintf(stderr,"    -s=<smooths> Smooth parameter to polysmooth \n");
				fprintf(stderr,"    -m=<mult>    Multiply parameter to polymult \n");
				exit(0);
			case 'm':
				if (argv[0][2] == '=')
					multfactor = atoi( &argv[0][3] );
				break;
			case 's':
				if (argv[0][2] == '=')
					smoothings = atoi( &argv[0][3] );
				break;
			default:
				fprintf(stderr,"POLYTEST: Unrecognised parameter ignored\n");
				break;
			}
		}
		else
			fprintf(stderr,"POLYTEST: Illegal arguement %s\n",*argv);
	}
	fprintf(stderr,"POLYTEST DEBUG: m = %d, s = %d\n",multfactor,smoothings);
	setmemalloc(32 * 1024);
	Alloc_limits(0,512 * 1024 * sizeof(GREY));

	while((obj = readobj(stdin)) != NULL) {
		if (obj->type == 10)
		{
			fprintf(stderr,"POLYTEST:\n");
			nobj = polytest(obj,multfactor,smoothings);
			writeobj(stdout,nobj);
			freeobj(nobj);
		}
		else
			writeobj(stdout,obj);
		freeobj(obj);
	}
}
