/*
 *
 *	P R C H E C K . C --
 *
 *  Written: B. Pennington.
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems
 *
 *  Date:    	   October 1987.
 *
 *	Purpose:	   Main routine for the object check program
 *
 *	Description:   Takes objects from stdin and validates them using
 *				   the extended object checker.
 *
 *  Modifications
 *
 *	05 Dec 1988		SCG		woolz_check_obj instead of wtcheck and woolz_err_msg
 *							instead of prerror
 *	23 Nov 1987		BDP		Added -? help option						  
 *	06 Nov 1987		BDP		Removed prcheck function to its own module	  
 *	04 Nov 1987		BDP		Added -s option, summarise only if specified  
 *	30 Oct 1987		BDP		Added -e option, an empty object is expected  
 *																		  
 */

#include <stdio.h>
#include <wstruct.h>
#include <wtcheck.h>


main(argc,argv)
int argc;
char **argv;
{
	FILE *fp;
	struct object *obj, *readobj();
	int st, op, empty, sum;
	char *str;

	fp = stdin;
	empty = 0;					/* default to false, object shouldnt be empty */
	sum = 0;					/* default to no object summary */
	op = 1;						/* output true by default */

	while(--argc > 0)
	{
		if ((*++argv)[0] == '-') 
		{
			str = *argv + 1;
			while ( *str != '\0' )
			switch( *str++ )
			{
			case '?':
				fprintf(stderr,"Syntax:     prcheck [<opts>]\n");
				fprintf(stderr,"Function:   Perform rigorous verification of woolz objects\n");
				fprintf(stderr,"Options:\n");
				fprintf(stderr,"    -n      Disable writing of object to stdout\n");
				fprintf(stderr,"    -s      Print an object summary\n");
				fprintf(stderr,"    -e      Empty object not to be raised as an error\n\n");
				exit(0);
			case 's':
				sum = 1;		/* an object summary is required */
				break;
			case 'n':
				op = 0;			/* No output required */
				break;
			case 'e':
				empty = 1;		/* object is expected to be empty */
				break;
			default:
				fprintf(stderr,"PRCHECK: Unrecognised parameter ignored\n");
				break;
			}
		}
		else
			fprintf(stderr,"PRCHECK: Illegal arguement %s\n",*argv);
	}
	
	setmemalloc(65536);
	Alloc_limits( 0, 1024 * 512 * sizeof(GREY) );

	while((obj = readobj(stdin)) != NULL) 
	{
		st = woolz_check_obj(obj, "prcheck");
		if ( (st == CKEMPTYDMN) && empty)
			st = 0;				/* if error is empty and this expected then set to ok */
		if (( st == 0 ) && (sum == 1))
			wzsum(obj);			/* summarise a good object */
		else {
			woolz_err_msg(st);
		};
		if ( op == 1 )			/* output true ? */
			writeobj(stdout,obj);
		freeobj(obj);
	}
}
