/************************************************************************/
/*																		*/
/*							realpoly.c									*/
/*																		*/
/************************************************************************/
/*																		*/
/*	Written by:		B. Pinnington										*/
/*																		*/
/*	Purpose:		Test program for functions rsupoly and itrpoly8		*/
/*																		*/
/*	Description:	Converts an input interger vertex polygon to a real */
/*					vertex, unitspaced smoothed polygon. Takes an 		*/
/*					optional parameter for number of smoothings.		*/
/*																		*/
/************************************************************************/
/*																		*/
/*	Modifications:														*/
/*																		*/
/************************************************************************/

#include <stdio.h>
#include <wstruct.h>

main(argc,argv)
int argc;
char **argv;
{
	struct object *obj, *rpobj, *readobj(), *makemain(), *realsmoothunitpoly();
	struct polygondomain *intrp8(), *pdom;
	int i,count,itr;
	
	itr = 0;				/* by default do a real smooth unit poly */
	count = 1;				/* default smooth count */
	while(--argc > 0)
	{
		if ((*++argv)[0] == '-') 
		{
			switch( argv[0][1] )
			{
			case '?':
				fprintf(stderr,"Syntax:     realpoly [<opts>]\n");
				fprintf(stderr,"Function:   converts integer polygon to real polygon\n");
				fprintf(stderr,"Options:\n");
				fprintf(stderr,"    -i		Use itrpoly8 instead of rsupoly\n");
				fprintf(stderr,"    -s=<smooths> Number of times to smooth for rsupoly\n");
				exit(0);
			case 's':
				if (argv[0][2] == '=')
					count = atoi( &argv[0][3] );
				break;
			case 'i':
				itr = 1;		/* use alternative convertion */
				break;
			default:
				fprintf(stderr,"RSUPOLY: Unrecognised parameter ignored\n");
				break;
			}
		}
		else
			fprintf(stderr,"RSUPOLY: Illegal arguement %s\n",*argv);
	}
	while((obj = readobj(stdin)) != NULL)
	{
		if (obj->type == 10)
		{
			/* the following is all a bit artificial in terms of actually 
			providing useable output, but does provide a means of testing those
			polygon routines not covered by other test programs */
			
			if (itr)
			{
				pdom = intrp8(obj->idom);
				polysqueeze(pdom, 0.125, 0.125 );
				rpobj = makemain(10,pdom,NULL,NULL,NULL);
			}
			else
				rpobj = realsmoothunitpoly(obj,count);
			writeobj(stdout,rpobj);
			freeobj(rpobj);
		}
		else
			writeobj(stdout,obj);
		freeobj(obj);
	}
}
