/*
 *      S E G M E N T . C  --  Test program for segmentation functions
 *
 *
 *  Written: BDP
 *           Image Recognition Systems
 *           720 Birchwood Boulevard
 *           Birchwood Science Park
 *           Warrington
 *           Cheshire
 *           WA3 7PX
 *
 *	Copyright (c) and intellectual property rights Image Recognition Systems (1988)
 *
 *	Reads an object from stdin, then segments it using
 *	using the label procedure, all segmented objects
 *	are written to stdout.
 *
 *  Modifications
 *
 *	26 Nov 1987		BDP		Renamed to segment from prlabel, added
 *							options to control normalisation and to
 *							supply or generate threshold values.
 *	06 Nov 1987		BDP		Added -s , min size parameter to label
 */

#include <stdio.h>
#include <wstruct.h>

main(argc,argv)
int argc;
char **argv;
{
	char *fp, *fipbuf;
	int timer,norm,autothresh,nothresh;		/* option flags */
	int st,i,th,numobj,minsize, htperc;		/* variables    */
	struct object *obj, *hobj, *tobj, *histo(), *threshold(), *readobj(), *objlist[250];
	
	setmemalloc(64 * 1024);
	Alloc_limits(0, 512 * 1024 * sizeof(GREY) );
	
	minsize = 3;			/* default min size for segmentation */
	nothresh = 0;			/* by default do thresholding */
	timer = 0;				/* default is no timer messages */
	norm = 1;				/* default to normalise routine */
	autothresh = 1;			/* default to use comthr3 */
	while(--argc > 0) {
		if ((*++argv)[0] == '-') {
			switch( argv[0][1] ) {
			case '?':
				fprintf(stderr,"Syntax:     segment [<opts>]\n");
				fprintf(stderr,"Function:   Object thresholding segmentation and normalisation\n");
				fprintf(stderr,"Options:\n");
				fprintf(stderr,"    -d      Disable thresholding\n");
				fprintf(stderr,"    -h=<percent> Use htnormalise for top/bot percent \n");
				fprintf(stderr,"    -s=<size> Min acceptable object size for label\n");
				fprintf(stderr,"    -t=<thr> Threshold value. Use instead of calc value\n");
				fprintf(stderr,"    -x      Enable timer messages\n");
				exit(0);
			case 'd':
				nothresh = 1;			/* no threshold at all */
				break;
			case 'h':
				if (argv[0][2] == '=') {
					htperc = atoi( &argv[0][3] );
					norm = 0;			/* use htnormalise instead */
				}
				break;
			case 's':
				if (argv[0][2] == '=')
					minsize = atoi( &argv[0][3] );
				break;
			case 't':
				if (argv[0][2] == '=') {
					autothresh = 0;		/* dont do a compthr3 */
					th = atoi( &argv[0][3] );
				}
				break;
			case 'x':
				timer = 1;
				break;
			default:
				fprintf(stderr,"SEGMENT: Unrecognised parameter ignored\n");
				break;
			}
		}
		else
			fprintf(stderr,"SEGMENT: Illegal arguement %s\n",*argv);
	}


	while( (obj = readobj(stdin)) != NULL )
	{
		if (obj->type == 1){
			if (timer)
				starttimer();
		
			/* if nothresh is set then do nothing, otherwise see if threshold value
			was input or has to be calculated */
		
			if ( !nothresh) {
				if ( autothresh ) {
#ifdef DEBUG
				fprintf(stderr,"SEGMENT DEBUG: Producing histogram\n");
#endif DEBUG
					hobj = histo(obj);
#ifdef DEBUG
				fprintf(stderr,"SEGMENT DEBUG: Calculating threshold\n");
				th = 50;
#endif DEBUG
					th = compthr3(hobj);
					freeobj(hobj);
				}
#ifdef DEBUG
				fprintf(stderr,"SEGMENT DEBUG: Thresholding at %d\n",th);
#endif DEBUG
				tobj = threshold(obj, th);
				if (obj->plist != NULL)
					obj->plist = NULL;		/* its now referenced by new object */
				freeobj(obj);
				obj = tobj;
			}
			if (timer)
				elapsedtime();
			numobj = 0;
#ifdef DEBUG
				fprintf(stderr,"SEGMENT DEBUG: Labelling\n");
#endif DEBUG
			label(obj,&numobj,objlist,250,minsize);
			if (timer)
				elapsedtime();
#ifdef DEBUG
			fprintf(stderr,"SEGMENT DEBUG: %d objects segmented\n",numobj);
#endif DEBUG
			
			/* now normalise all the object, using either normalise or htnormalise
			and the input -h= value. */
		
			if (norm)
				for ( i=0; i<numobj; i++ )
					normalise(objlist[i]);
			else
				for ( i=0; i<numobj; i++ )
					htnormalise(objlist[i],(float)htperc,(float)htperc);
					
#ifdef DEBUG
			fprintf(stderr,"SEGMENT DEBUG: %d objects normalised\n",numobj);
#endif DEBUG
			if (timer)
				elapsedtime();
			for ( i=0; i<numobj; i++ ){
				writeobj(stdout,objlist[i]);
			}
			if (timer)
				elapsedtime();
			for ( i=0; i<numobj; i++ )
				freeobj(objlist[i]);
		} else {
			writeobj(stdout,obj);
		}
		freeobj(obj);
	}
}

